! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! Linear Algebra Data and Routines File
! 
! Generated by KPP-2.2.3 symbolic chemistry Kinetics PreProcessor
!       (http://www.cs.vt.edu/~asandu/Software/KPP)
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian, Villanova University, USA
!        R. Sander, Max-Planck Institute for Chemistry, Mainz, Germany
! 
! File                 : cb6_LinearAlgebra.f90
! Time                 : Mon Nov 24 11:54:36 2014
! Working directory    : /home/regcm/Code/kpp-2.2.3/Workspace/GAS_CB6r2
! Equation file        : cb6.kpp
! Output root filename : cb6
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



MODULE mod_cb6_LinearAlgebra

  USE mod_cb6_Parameters
  USE mod_cb6_JacobianSP

  IMPLICIT NONE

CONTAINS


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! SPARSE_UTIL - SPARSE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecomp( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE mod_cb6_Parameters
  USE mod_cb6_JacobianSP

      INTEGER  :: IER
      REAL(kind=dp) :: JVS(LU_NONZERO), W(NVAR_CB6), a
      INTEGER  :: k, kk, j, jj

      a = 0.0D0 ! mz_rs_20050606
      IER = 0
      DO k=1,NVAR_CB6
        ! mz_rs_20050606: don't check if real value == 0
        ! IF ( JVS( LU_DIAG(k) ) .EQ. 0. ) THEN
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(a) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecomp


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplx( JVS, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse LU factorization, complex
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE mod_cb6_Parameters
  USE mod_cb6_JacobianSP

      INTEGER        :: IER
      DOUBLE COMPLEX :: JVS(LU_NONZERO), W(NVAR_CB6), a
      REAL(kind=dp)  :: b = 0.0
      INTEGER        :: k, kk, j, jj

      IER = 0
      DO k=1,NVAR_CB6
        IF ( ABS(JVS(LU_DIAG(k))) < TINY(b) ) THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              W( LU_ICOL(kk) ) = JVS(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            a = -W(j) / JVS( LU_DIAG(j) )
            W(j) = -a
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               W( LU_ICOL(jj) ) = W( LU_ICOL(jj) ) + a*JVS(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVS(kk) = W( LU_ICOL(kk) )
         END DO
      END DO
      
END SUBROUTINE KppDecompCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppDecompCmplxR( JVSR, JVSI, IER )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!    Sparse LU factorization, complex
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE mod_cb6_Parameters
  USE mod_cb6_JacobianSP

      INTEGER       :: IER
      REAL(kind=dp) :: JVSR(LU_NONZERO), JVSI(LU_NONZERO) 
      REAL(kind=dp) :: WR(NVAR_CB6), WI(NVAR_CB6), ar, ai, den
      INTEGER       :: k, kk, j, jj

      IER = 0
      ar  = 0.0
      DO k=1,NVAR_CB6
        IF (  ( ABS(JVSR(LU_DIAG(k))) < TINY(ar) ) .AND. &
              ( ABS(JVSI(LU_DIAG(k))) < TINY(ar) ) )  THEN
            IER = k
            RETURN
        END IF
        DO kk = LU_CROW(k), LU_CROW(k+1)-1
              WR( LU_ICOL(kk) ) = JVSR(kk)
              WI( LU_ICOL(kk) ) = JVSI(kk)
        END DO
        DO kk = LU_CROW(k), LU_DIAG(k)-1
            j = LU_ICOL(kk)
            den = JVSR(LU_DIAG(j))**2 + JVSI(LU_DIAG(j))**2
            ar = -(WR(j)*JVSR(LU_DIAG(j)) + WI(j)*JVSI(LU_DIAG(j)))/den
            ai = -(WI(j)*JVSR(LU_DIAG(j)) - WR(j)*JVSI(LU_DIAG(j)))/den
            WR(j) = -ar
            WI(j) = -ai
            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
               WR( LU_ICOL(jj) ) = WR( LU_ICOL(jj) ) + ar*JVSR(jj) - ai*JVSI(jj)
               WI( LU_ICOL(jj) ) = WI( LU_ICOL(jj) ) + ar*JVSI(jj) + ai*JVSR(jj)
            END DO
         END DO
         DO kk = LU_CROW(k), LU_CROW(k+1)-1
            JVSR(kk) = WR( LU_ICOL(kk) )
            JVSI(kk) = WI( LU_ICOL(kk) )
         END DO
      END DO

END SUBROUTINE KppDecompCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE mod_cb6_Parameters
  USE mod_cb6_JacobianSP

      INTEGER  :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR_CB6), sum

      DO i=1,NVAR_CB6
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR_CB6,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRIndirect( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE mod_cb6_Parameters
  USE mod_cb6_JacobianSP

      INTEGER       :: i, j
      REAL(kind=dp) :: JVS(LU_NONZERO), X(NVAR_CB6)

      DO i=1,NVAR_CB6
        X(i) = X(i)/JVS(LU_DIAG(i))
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
	  X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
	END DO
      END DO

      DO i=NVAR_CB6, 1, -1
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
	  X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
	END DO
      END DO
      
END SUBROUTINE KppSolveTRIndirect


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE mod_cb6_Parameters
  USE mod_cb6_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR_CB6), sum

      DO i=1,NVAR_CB6
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             X(i) = X(i) - JVS(j)*X(LU_ICOL(j));
         END DO  
      END DO

      DO i=NVAR_CB6,1,-1
        sum = X(i);
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
          sum = sum - JVS(j)*X(LU_ICOL(j));
        END DO
        X(i) = sum/JVS(LU_DIAG(i));
      END DO
      
END SUBROUTINE KppSolveCmplx

! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE mod_cb6_Parameters
  USE mod_cb6_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR_CB6), XI(NVAR_CB6), sumr, sumi, den

      DO i=1,NVAR_CB6
         DO j = LU_CROW(i), LU_DIAG(i)-1 
             XR(i) = XR(i) - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
             XI(i) = XI(i) - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
         END DO  
      END DO

      DO i=NVAR_CB6,1,-1
        sumr = XR(i); sumi = XI(i)
        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
            sumr = sumr - (JVSR(j)*XR(LU_ICOL(j)) - JVSI(j)*XI(LU_ICOL(j)))
            sumi = sumi - (JVSR(j)*XI(LU_ICOL(j)) + JVSI(j)*XR(LU_ICOL(j)))
        END DO
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (sumr*JVSR(LU_DIAG(i)) + sumi*JVSI(LU_DIAG(i)))/den
        XI(i) = (sumi*JVSR(LU_DIAG(i)) - sumr*JVSI(LU_DIAG(i)))/den
      END DO
      
END SUBROUTINE KppSolveCmplxR


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplx( JVS, X )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!        Complex sparse solve transpose subroutine using indirect addressing
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE mod_cb6_Parameters
  USE mod_cb6_JacobianSP

      INTEGER        :: i, j
      DOUBLE COMPLEX :: JVS(LU_NONZERO), X(NVAR_CB6)

      DO i=1,NVAR_CB6
        X(i) = X(i)/JVS(LU_DIAG(i))
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
	  X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
	END DO
      END DO

      DO i=NVAR_CB6, 1, -1
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
	  X(LU_ICOL(j)) = X(LU_ICOL(j))-JVS(j)*X(i)
	END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplx


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUBROUTINE KppSolveTRCmplxR( JVSR, JVSI, XR, XI )
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!   Complex sparse solve transpose subroutine using indirect addressing
!   (Real and Imaginary parts are used instead of complex data type)     
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  USE mod_cb6_Parameters
  USE mod_cb6_JacobianSP

      INTEGER       ::  i, j
      REAL(kind=dp) ::  JVSR(LU_NONZERO), JVSI(LU_NONZERO), XR(NVAR_CB6), XI(NVAR_CB6), den

      DO i=1,NVAR_CB6
        den   = JVSR(LU_DIAG(i))**2 + JVSI(LU_DIAG(i))**2
        XR(i) = (XR(i)*JVSR(LU_DIAG(i)) + XI(i)*JVSI(LU_DIAG(i)))/den
        XI(i) = (XI(i)*JVSR(LU_DIAG(i)) - XR(i)*JVSI(LU_DIAG(i)))/den
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
	  XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
	  XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
	END DO
      END DO

      DO i=NVAR_CB6, 1, -1
	! subtract all nonzero elements in row i of JVS from X
        DO j=LU_CROW(i),LU_DIAG(i)-1
	  XR(LU_ICOL(j)) = XR(LU_ICOL(j))-(JVSR(j)*XR(i) - JVSI(j)*XI(i))
	  XI(LU_ICOL(j)) = XI(LU_ICOL(j))-(JVSI(j)*XR(i) + JVSR(j)*XI(i))
	END DO
      END DO
      
END SUBROUTINE KppSolveTRCmplxR


!
! Next few commented subroutines perform sparse big linear algebra
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppDecompBig( JVS, IP, IER )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse LU factorization
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE cb6_Parameters
!  USE cb6_JacobianSP
!
!      INTEGER  :: IP3(3), IER, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), W(3,3,NVAR), a(3,3), E(3,3)
!      INTEGER  :: k, kk, j, jj
!
!      a = 0.0d0
!      IER = 0
!      DO k=1,NVAR
!        DO kk = LU_CROW(k), LU_CROW(k+1)-1
!              W( 1:3,1:3,LU_ICOL(kk) ) = JVS(1:3,1:3,kk)
!        END DO
!        DO kk = LU_CROW(k), LU_DIAG(k)-1
!            j = LU_ICOL(kk)
!            E(1:3,1:3) = JVS( 1:3,1:3,LU_DIAG(j) )
!            ! CALL DGETRF(3,3,E,3,IP3,IER) 
!            CALL FAC3(E,IP3,IER)
!            IF ( IER /= 0 )  RETURN
!            ! a = W(j) / JVS( LU_DIAG(j) )
!            a(1:3,1:3) = W( 1:3,1:3,j )
!            ! CALL DGETRS ('N',3,3,E,3,IP3,a,3,IER) 
!            CALL SOL3('N',E,IP3,a(1,1))
!            CALL SOL3('N',E,IP3,a(1,2))
!            CALL SOL3('N',E,IP3,a(1,3))
!            W(1:3,1:3,j) = a(1:3,1:3)
!            DO jj = LU_DIAG(j)+1, LU_CROW(j+1)-1
!               W( 1:3,1:3,LU_ICOL(jj) ) = W( 1:3,1:3,LU_ICOL(jj) ) &
!                        - MATMUL( a(1:3,1:3) , JVS(1:3,1:3,jj) )
!            END DO
!         END DO
!         DO kk = LU_CROW(k), LU_CROW(k+1)-1
!            JVS(1:3,1:3,kk) = W( 1:3,1:3,LU_ICOL(kk) )
!         END DO
!      END DO
!
!      DO k=1,NVAR
!         ! CALL WGEFA(JVS(1,1,LU_DIAG(k)),3,3,IP(1,k),IER)
!         ! CALL DGETRF(3,3,JVS(1,1,LU_DIAG(k)),3,IP(1,k),IER)
!         CALL FAC3(JVS(1,1,LU_DIAG(k)),IP(1,k),IER)
!         IF ( IER /= 0 )  RETURN
!      END DO 
!      
!END SUBROUTINE KppDecompBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBig( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Sparse solve subroutine using indirect addressing
!!        for the Runge Kutta (3n)x(3n) linear system
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE cb6_Parameters
!  USE cb6_JacobianSP
!
!      INTEGER  :: i, j, k, m, IP3(3), IP(3,NVAR), IER
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR), sum(3)
!
!      DO i=1,NVAR
!        DO j = LU_CROW(i), LU_DIAG(i)-1 
!          !X(1:3,i) = X(1:3,i) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!	       X(k,i) = X(k,i) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO  
!      END DO
!
!      DO i=NVAR,1,-1
!        sum(1:3) = X(1:3,i);
!        DO j = LU_DIAG(i)+1, LU_CROW(i+1)-1
!          !sum(1:3) = sum(1:3) - MATMUL(JVS(1:3,1:3,j),X(1:3,LU_ICOL(j)));
!          DO k=1,3
!            DO m=1,3
!	       sum(k) = sum(k) - JVS(k,m,j)*X(m,LU_ICOL(j))
!            END DO
!          END DO
!        END DO
!        ! X(i) = sum/JVS(LU_DIAG(i));
!        ! CALL DGETRS ('N',3,1,JVS(1:3,1:3,LU_DIAG(i)),3,IP(1,i),sum,3,0) 
!        ! CALL WGESL('N',JVS(1,1,LU_DIAG(i)),3,3,IP(1,i),sum)
!        CALL SOL3('N',JVS(1,1,LU_DIAG(i)),IP(1,i),sum)
!        X(1:3,i) = sum(1:3)
!      END DO
!      
!END SUBROUTINE KppSolveBig
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE KppSolveBigTR( JVS, IP, X )
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!        Big sparse transpose solve using indirect addressing
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!  USE cb6_Parameters
!  USE cb6_JacobianSP
!
!      INTEGER       :: i, j, k, m, IP(3,NVAR)
!      REAL(kind=dp) :: JVS(3,3,LU_NONZERO), X(3,NVAR)
!
!      DO i=1,NVAR
!        ! X(i) = X(i)/JVS(LU_DIAG(i))
!        CALL SOL3('T',JVS(1,1,LU_DIAG(i)),IP(1,i),X(1,i))
!        DO j=LU_DIAG(i)+1,LU_CROW(i+1)-1
!	  !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !    - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!	       X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!	END DO
!      END DO
!
!      DO i=NVAR, 1, -1
!        DO j=LU_CROW(i),LU_DIAG(i)-1
!	  !X(1:3,LU_ICOL(j)) = X(1:3,LU_ICOL(j)) &
!          !   - MATMUL( TRANSPOSE(JVS(1:3,1:3,j)), X(1:3,i) )
!          DO k=1,3
!            DO m=1,3
!	       X(k,LU_ICOL(j)) = X(k,LU_ICOL(j)) - JVS(m,k,j)*X(m,i)
!            END DO
!          END DO
!	END DO
!      END DO
!      
!END SUBROUTINE KppSolveBigTR
!
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE FAC3(A,IPVT,INFO)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     FAC3 FACTORS THE MATRIX A (3,3) BY
!!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!!     LINPACK - LIKE 
!!
!!     Remove comments to perform pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!
!      REAL(kind=dp) :: A(3,3)
!      INTEGER       :: IPVT(3),INFO
!!      INTEGER       :: L
!!      REAL(kind=dp) :: t, dmax, da, TMP(3)
!      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0
!
!      info = 0
!!      t = TINY(da)
!!      
!!      da = ABS(A(1,1)); L = 1
!!      IF ( ABS(A(2,1))>da ) THEN
!!        da = ABS(A(2,1)); L = 2
!!        IF ( ABS(A(3,1))>da ) THEN
!!          L = 3
!!        END IF  
!!      END IF  
!!      IPVT(1)  = L
!!      IF (L /=1 ) THEN
!!         TMP(1:3) = A(L,1:3)
!!         A(L,1:3) = A(1,1:3)
!!         A(1,1:3) = TMP(1:3)
!!      END IF
!!      IF (ABS(A(1,1)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!
!      A(2,1) = A(2,1)/A(1,1)
!      A(2,2) = A(2,2) - A(2,1)*A(1,2)
!      A(2,3) = A(2,3) - A(2,1)*A(1,3)
!      A(3,1) = A(3,1)/A(1,1)
!      A(3,2) = A(3,2) - A(3,1)*A(1,2)
!      A(3,3) = A(3,3) - A(3,1)*A(1,3)
!      
!!      IPVT(2)  = 2
!!      IF (ABS(A(3,2))>ABS(A(2,2))) THEN
!!         IPVT(2)  = 3
!!         TMP(2:3) = A(3,2:3)
!!         A(3,2:3) = A(2,2:3)
!!         A(2,2:3) = TMP(2:3)
!!      END IF
!!      IF (ABS(A(2,2)) < t) THEN
!!         info = 1
!!         return
!!      END IF   
!!      
!      A(3,2)   = A(3,2)/A(2,2)
!      A(3,3)   = A(3,3) - A(3,2)*A(2,3)
!      IPVT(3)  = 3
!      
!END SUBROUTINE FAC3
!
!
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!SUBROUTINE SOL3(Trans,A,IPVT,b)
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!!     SOL3 solves the system 3x3
!!     A * x = b  or  trans(a) * x = b
!!     using the factors computed by WGEFA.
!!
!!     Trans      = 'N'   to solve  A*x = b ,
!!                = 'T'   to solve  transpose(A)*x = b
!!     LINPACK - LIKE 
!!
!!     Remove comments to use pivoting
!! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!      CHARACTER     :: Trans
!      REAL(kind=dp) :: a(3,3),b(3)
!      INTEGER       :: IPVT(3)
!!      INTEGER       :: L
!!      REAL(kind=dp) :: TMP
!      
!      SELECT CASE (Trans)
!
!      CASE ('n','N')  !  Solve  A * x = b
!
!!     Solve  L*y = b
!!         L = IPVT(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!         b(2) = b(2)-A(2,1)*b(1)
!         b(3) = b(3)-A(3,1)*b(1)
!         
!!         L = IPVT(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(3) = b(3)-A(3,2)*b(2)
!
!!     Solve  U*x = y
!         b(3) = b(3)/A(3,3)
!         b(2) = (b(2)-A(2,3)*b(3))/A(2,2)
!         b(1) = (b(1)-A(1,3)*b(3)-A(1,2)*b(2))/A(1,1)
!      
!      
!      CASE ('t','T')  !  Solve transpose(A) * x = b
!
!!      Solve transpose(U)*y = b
!         b(1) = b(1)/A(1,1)
!         b(2) = (b(2)-A(1,2)*b(1))/A(2,2)
!         b(3) = (b(3)-A(1,3)*b(1)-A(2,3)*b(2))/A(3,3)
!
!!      Solve transpose(L)*x = y
!         b(2) = b(2)-A(3,2)*b(3)
!!         L = ipvt(2)
!!         IF (L /= 2) THEN
!!            TMP = B(2); B(2) = B(L); B(L) = TMP
!!         END IF
!         b(1) = b(1)-A(3,1)*b(3)-A(2,1)*b(2)
!!         L = ipvt(1)
!!         IF (L /= 1) THEN
!!            TMP = B(1); B(1) = B(L); B(L) = TMP
!!         END IF
!   
!      END SELECT
!
!END SUBROUTINE SOL3

! End of SPARSE_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolve - sparse back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolve ( JVS, X )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR_CB6)

  X(28) = X(28)-JVS(102)*X(7)
  X(31) = X(31)-JVS(122)*X(25)
  X(34) = X(34)-JVS(146)*X(12)
  X(36) = X(36)-JVS(156)*X(9)
  X(37) = X(37)-JVS(163)*X(31)
  X(40) = X(40)-JVS(190)*X(14)
  X(42) = X(42)-JVS(202)*X(7)-JVS(203)*X(24)-JVS(204)*X(34)-JVS(205)*X(39)-JVS(206)*X(41)
  X(43) = X(43)-JVS(234)*X(15)
  X(45) = X(45)-JVS(246)*X(9)-JVS(247)*X(14)-JVS(248)*X(15)-JVS(249)*X(31)-JVS(250)*X(37)
  X(47) = X(47)-JVS(261)*X(10)-JVS(262)*X(27)
  X(48) = X(48)-JVS(270)*X(8)-JVS(271)*X(16)-JVS(272)*X(19)-JVS(273)*X(24)-JVS(274)*X(25)-JVS(275)*X(33)-JVS(276)*X(37)&
            &-JVS(277)*X(39)-JVS(278)*X(41)-JVS(279)*X(43)-JVS(280)*X(45)-JVS(281)*X(46)
  X(49) = X(49)-JVS(307)*X(11)-JVS(308)*X(35)-JVS(309)*X(41)-JVS(310)*X(47)
  X(50) = X(50)-JVS(322)*X(7)-JVS(323)*X(24)-JVS(324)*X(36)-JVS(325)*X(40)-JVS(326)*X(43)-JVS(327)*X(45)-JVS(328)*X(47)&
            &-JVS(329)*X(49)
  X(51) = X(51)-JVS(347)*X(15)-JVS(348)*X(40)-JVS(349)*X(43)
  X(52) = X(52)-JVS(358)*X(40)-JVS(359)*X(43)-JVS(360)*X(45)-JVS(361)*X(47)-JVS(362)*X(51)
  X(53) = X(53)-JVS(377)*X(9)-JVS(378)*X(14)-JVS(379)*X(36)-JVS(380)*X(37)-JVS(381)*X(40)-JVS(382)*X(43)-JVS(383)*X(45)&
            &-JVS(384)*X(51)
  X(54) = X(54)-JVS(394)*X(27)
  X(55) = X(55)-JVS(402)*X(11)-JVS(403)*X(21)-JVS(404)*X(23)-JVS(405)*X(25)-JVS(406)*X(30)-JVS(407)*X(34)-JVS(408)*X(35)&
            &-JVS(409)*X(37)-JVS(410)*X(39)-JVS(411)*X(41)-JVS(412)*X(44)-JVS(413)*X(45)-JVS(414)*X(46)-JVS(415)*X(47)&
            &-JVS(416)*X(49)-JVS(417)*X(53)-JVS(418)*X(54)
  X(56) = X(56)-JVS(434)*X(35)
  X(57) = X(57)-JVS(443)*X(6)-JVS(444)*X(11)-JVS(445)*X(13)-JVS(446)*X(29)-JVS(447)*X(39)-JVS(448)*X(51)-JVS(449)*X(53)&
            &-JVS(450)*X(54)-JVS(451)*X(56)
  X(58) = X(58)-JVS(467)*X(34)-JVS(468)*X(35)-JVS(469)*X(39)-JVS(470)*X(41)-JVS(471)*X(44)-JVS(472)*X(45)-JVS(473)*X(46)&
            &-JVS(474)*X(50)-JVS(475)*X(51)-JVS(476)*X(52)-JVS(477)*X(53)-JVS(478)*X(54)-JVS(479)*X(56)
  X(59) = X(59)-JVS(494)*X(24)-JVS(495)*X(45)-JVS(496)*X(53)
  X(60) = X(60)-JVS(507)*X(6)-JVS(508)*X(12)-JVS(509)*X(32)-JVS(510)*X(39)-JVS(511)*X(44)-JVS(512)*X(45)-JVS(513)*X(46)&
            &-JVS(514)*X(51)-JVS(515)*X(54)-JVS(516)*X(56)-JVS(517)*X(58)-JVS(518)*X(59)
  X(61) = X(61)-JVS(533)*X(27)-JVS(534)*X(46)
  X(62) = X(62)-JVS(544)*X(6)-JVS(545)*X(11)-JVS(546)*X(12)-JVS(547)*X(13)-JVS(548)*X(14)-JVS(549)*X(15)-JVS(550)*X(29)&
            &-JVS(551)*X(32)-JVS(552)*X(35)-JVS(553)*X(39)-JVS(554)*X(41)-JVS(555)*X(44)-JVS(556)*X(45)-JVS(557)*X(46)&
            &-JVS(558)*X(49)-JVS(559)*X(51)-JVS(560)*X(53)-JVS(561)*X(54)-JVS(562)*X(56)-JVS(563)*X(58)-JVS(564)*X(59)&
            &-JVS(565)*X(60)-JVS(566)*X(61)
  X(63) = X(63)-JVS(581)*X(16)-JVS(582)*X(46)-JVS(583)*X(61)
  X(64) = X(64)-JVS(593)*X(5)-JVS(594)*X(13)-JVS(595)*X(17)-JVS(596)*X(23)-JVS(597)*X(30)-JVS(598)*X(34)-JVS(599)*X(36)&
            &-JVS(600)*X(38)-JVS(601)*X(39)-JVS(602)*X(40)-JVS(603)*X(43)-JVS(604)*X(47)-JVS(605)*X(54)-JVS(606)*X(55)&
            &-JVS(607)*X(56)-JVS(608)*X(57)-JVS(609)*X(58)-JVS(610)*X(59)-JVS(611)*X(60)-JVS(612)*X(61)-JVS(613)*X(62)&
            &-JVS(614)*X(63)
  X(65) = X(65)-JVS(628)*X(12)-JVS(629)*X(27)-JVS(630)*X(35)-JVS(631)*X(39)-JVS(632)*X(44)-JVS(633)*X(46)-JVS(634)*X(54)&
            &-JVS(635)*X(56)-JVS(636)*X(59)-JVS(637)*X(61)-JVS(638)*X(63)
  X(66) = X(66)-JVS(650)*X(12)-JVS(651)*X(13)-JVS(652)*X(35)-JVS(653)*X(39)-JVS(654)*X(44)-JVS(655)*X(46)-JVS(656)*X(47)&
            &-JVS(657)*X(54)-JVS(658)*X(56)-JVS(659)*X(61)-JVS(660)*X(63)
  X(67) = X(67)-JVS(671)*X(41)-JVS(672)*X(44)-JVS(673)*X(46)-JVS(674)*X(51)-JVS(675)*X(53)-JVS(676)*X(54)-JVS(677)*X(56)&
            &-JVS(678)*X(59)-JVS(679)*X(61)-JVS(680)*X(63)
  X(68) = X(68)-JVS(691)*X(5)-JVS(692)*X(6)-JVS(693)*X(9)-JVS(694)*X(10)-JVS(695)*X(11)-JVS(696)*X(12)-JVS(697)*X(13)&
            &-JVS(698)*X(14)-JVS(699)*X(15)-JVS(700)*X(17)-JVS(701)*X(23)-JVS(702)*X(27)-JVS(703)*X(29)-JVS(704)*X(30)&
            &-JVS(705)*X(32)-JVS(706)*X(34)-JVS(707)*X(35)-JVS(708)*X(36)-JVS(709)*X(38)-JVS(710)*X(39)-JVS(711)*X(40)&
            &-JVS(712)*X(41)-JVS(713)*X(43)-JVS(714)*X(44)-JVS(715)*X(45)-JVS(716)*X(46)-JVS(717)*X(47)-JVS(718)*X(49)&
            &-JVS(719)*X(50)-JVS(720)*X(51)-JVS(721)*X(52)-JVS(722)*X(53)-JVS(723)*X(54)-JVS(724)*X(55)-JVS(725)*X(56)&
            &-JVS(726)*X(57)-JVS(727)*X(58)-JVS(728)*X(59)-JVS(729)*X(60)-JVS(730)*X(61)-JVS(731)*X(62)-JVS(732)*X(63)&
            &-JVS(733)*X(64)-JVS(734)*X(65)-JVS(735)*X(66)-JVS(736)*X(67)
  X(69) = X(69)-JVS(746)*X(3)-JVS(747)*X(4)-JVS(748)*X(5)-JVS(749)*X(6)-JVS(750)*X(7)-JVS(751)*X(9)-JVS(752)*X(10)&
            &-JVS(753)*X(11)-JVS(754)*X(12)-JVS(755)*X(14)-JVS(756)*X(15)-JVS(757)*X(16)-JVS(758)*X(18)-JVS(759)*X(20)&
            &-JVS(760)*X(21)-JVS(761)*X(22)-JVS(762)*X(23)-JVS(763)*X(24)-JVS(764)*X(25)-JVS(765)*X(26)-JVS(766)*X(27)&
            &-JVS(767)*X(28)-JVS(768)*X(29)-JVS(769)*X(30)-JVS(770)*X(32)-JVS(771)*X(33)-JVS(772)*X(34)-JVS(773)*X(35)&
            &-JVS(774)*X(37)-JVS(775)*X(38)-JVS(776)*X(39)-JVS(777)*X(41)-JVS(778)*X(42)-JVS(779)*X(44)-JVS(780)*X(45)&
            &-JVS(781)*X(46)-JVS(782)*X(47)-JVS(783)*X(48)-JVS(784)*X(49)-JVS(785)*X(50)-JVS(786)*X(51)-JVS(787)*X(52)&
            &-JVS(788)*X(53)-JVS(789)*X(54)-JVS(790)*X(55)-JVS(791)*X(56)-JVS(792)*X(57)-JVS(793)*X(58)-JVS(794)*X(59)&
            &-JVS(795)*X(60)-JVS(796)*X(61)-JVS(797)*X(62)-JVS(798)*X(63)-JVS(799)*X(64)-JVS(800)*X(65)-JVS(801)*X(66)&
            &-JVS(802)*X(67)-JVS(803)*X(68)
  X(70) = X(70)-JVS(812)*X(3)-JVS(813)*X(7)-JVS(814)*X(9)-JVS(815)*X(11)-JVS(816)*X(14)-JVS(817)*X(15)-JVS(818)*X(20)&
            &-JVS(819)*X(21)-JVS(820)*X(25)-JVS(821)*X(26)-JVS(822)*X(28)-JVS(823)*X(30)-JVS(824)*X(31)-JVS(825)*X(32)&
            &-JVS(826)*X(35)-JVS(827)*X(36)-JVS(828)*X(37)-JVS(829)*X(39)-JVS(830)*X(40)-JVS(831)*X(41)-JVS(832)*X(42)&
            &-JVS(833)*X(43)-JVS(834)*X(44)-JVS(835)*X(45)-JVS(836)*X(46)-JVS(837)*X(47)-JVS(838)*X(49)-JVS(839)*X(50)&
            &-JVS(840)*X(51)-JVS(841)*X(52)-JVS(842)*X(53)-JVS(843)*X(54)-JVS(844)*X(55)-JVS(845)*X(56)-JVS(846)*X(57)&
            &-JVS(847)*X(58)-JVS(848)*X(59)-JVS(849)*X(60)-JVS(850)*X(61)-JVS(851)*X(62)-JVS(852)*X(63)-JVS(853)*X(64)&
            &-JVS(854)*X(65)-JVS(855)*X(66)-JVS(856)*X(67)-JVS(857)*X(68)-JVS(858)*X(69)
  X(71) = X(71)-JVS(866)*X(13)-JVS(867)*X(29)-JVS(868)*X(39)-JVS(869)*X(44)-JVS(870)*X(46)-JVS(871)*X(59)-JVS(872)*X(63)&
            &-JVS(873)*X(65)-JVS(874)*X(66)-JVS(875)*X(67)-JVS(876)*X(68)-JVS(877)*X(69)-JVS(878)*X(70)
  X(72) = X(72)-JVS(885)*X(8)-JVS(886)*X(16)-JVS(887)*X(17)-JVS(888)*X(20)-JVS(889)*X(25)-JVS(890)*X(29)-JVS(891)*X(35)&
            &-JVS(892)*X(37)-JVS(893)*X(41)-JVS(894)*X(44)-JVS(895)*X(45)-JVS(896)*X(46)-JVS(897)*X(48)-JVS(898)*X(49)&
            &-JVS(899)*X(50)-JVS(900)*X(51)-JVS(901)*X(52)-JVS(902)*X(53)-JVS(903)*X(54)-JVS(904)*X(55)-JVS(905)*X(56)&
            &-JVS(906)*X(57)-JVS(907)*X(59)-JVS(908)*X(60)-JVS(909)*X(61)-JVS(910)*X(62)-JVS(911)*X(63)-JVS(912)*X(64)&
            &-JVS(913)*X(65)-JVS(914)*X(66)-JVS(915)*X(67)-JVS(916)*X(68)-JVS(917)*X(69)-JVS(918)*X(70)-JVS(919)*X(71)
  X(73) = X(73)-JVS(925)*X(13)-JVS(926)*X(17)-JVS(927)*X(18)-JVS(928)*X(34)-JVS(929)*X(36)-JVS(930)*X(39)-JVS(931)*X(40)&
            &-JVS(932)*X(43)-JVS(933)*X(47)-JVS(934)*X(49)-JVS(935)*X(51)-JVS(936)*X(52)-JVS(937)*X(53)-JVS(938)*X(54)&
            &-JVS(939)*X(56)-JVS(940)*X(57)-JVS(941)*X(58)-JVS(942)*X(59)-JVS(943)*X(60)-JVS(944)*X(61)-JVS(945)*X(62)&
            &-JVS(946)*X(63)-JVS(947)*X(64)-JVS(948)*X(65)-JVS(949)*X(66)-JVS(950)*X(67)-JVS(951)*X(68)-JVS(952)*X(69)&
            &-JVS(953)*X(70)-JVS(954)*X(71)-JVS(955)*X(72)
  X(74) = X(74)-JVS(960)*X(4)-JVS(961)*X(26)-JVS(962)*X(41)-JVS(963)*X(44)-JVS(964)*X(46)-JVS(965)*X(54)-JVS(966)*X(55)&
            &-JVS(967)*X(56)-JVS(968)*X(57)-JVS(969)*X(61)-JVS(970)*X(63)-JVS(971)*X(64)-JVS(972)*X(65)-JVS(973)*X(66)&
            &-JVS(974)*X(67)-JVS(975)*X(68)-JVS(976)*X(69)-JVS(977)*X(70)-JVS(978)*X(71)-JVS(979)*X(72)-JVS(980)*X(73)
  X(75) = X(75)-JVS(984)*X(22)-JVS(985)*X(30)-JVS(986)*X(36)-JVS(987)*X(37)-JVS(988)*X(40)-JVS(989)*X(43)-JVS(990)*X(45)&
            &-JVS(991)*X(47)-JVS(992)*X(55)-JVS(993)*X(56)-JVS(994)*X(58)-JVS(995)*X(59)-JVS(996)*X(60)-JVS(997)*X(61)&
            &-JVS(998)*X(62)-JVS(999)*X(63)-JVS(1000)*X(64)-JVS(1001)*X(65)-JVS(1002)*X(66)-JVS(1003)*X(67)-JVS(1004)*X(68)&
            &-JVS(1005)*X(69)-JVS(1006)*X(70)-JVS(1007)*X(71)-JVS(1008)*X(72)-JVS(1009)*X(73)-JVS(1010)*X(74)
  X(76) = X(76)-JVS(1013)*X(8)-JVS(1014)*X(17)-JVS(1015)*X(20)-JVS(1016)*X(22)-JVS(1017)*X(24)-JVS(1018)*X(29)-JVS(1019)&
            &*X(30)-JVS(1020)*X(31)-JVS(1021)*X(33)-JVS(1022)*X(35)-JVS(1023)*X(36)-JVS(1024)*X(37)-JVS(1025)*X(39)&
            &-JVS(1026)*X(40)-JVS(1027)*X(41)-JVS(1028)*X(43)-JVS(1029)*X(44)-JVS(1030)*X(45)-JVS(1031)*X(46)-JVS(1032)&
            &*X(47)-JVS(1033)*X(48)-JVS(1034)*X(49)-JVS(1035)*X(50)-JVS(1036)*X(51)-JVS(1037)*X(52)-JVS(1038)*X(53)&
            &-JVS(1039)*X(54)-JVS(1040)*X(55)-JVS(1041)*X(56)-JVS(1042)*X(57)-JVS(1043)*X(58)-JVS(1044)*X(59)-JVS(1045)&
            &*X(60)-JVS(1046)*X(61)-JVS(1047)*X(62)-JVS(1048)*X(63)-JVS(1049)*X(64)-JVS(1050)*X(65)-JVS(1051)*X(66)&
            &-JVS(1052)*X(67)-JVS(1053)*X(68)-JVS(1054)*X(69)-JVS(1055)*X(70)-JVS(1056)*X(71)-JVS(1057)*X(72)-JVS(1058)&
            &*X(73)-JVS(1059)*X(74)-JVS(1060)*X(75)
  X(76) = X(76)/JVS(1061)
  X(75) = (X(75)-JVS(1012)*X(76))/(JVS(1011))
  X(74) = (X(74)-JVS(982)*X(75)-JVS(983)*X(76))/(JVS(981))
  X(73) = (X(73)-JVS(957)*X(74)-JVS(958)*X(75)-JVS(959)*X(76))/(JVS(956))
  X(72) = (X(72)-JVS(921)*X(73)-JVS(922)*X(74)-JVS(923)*X(75)-JVS(924)*X(76))/(JVS(920))
  X(71) = (X(71)-JVS(880)*X(72)-JVS(881)*X(73)-JVS(882)*X(74)-JVS(883)*X(75)-JVS(884)*X(76))/(JVS(879))
  X(70) = (X(70)-JVS(860)*X(71)-JVS(861)*X(72)-JVS(862)*X(73)-JVS(863)*X(74)-JVS(864)*X(75)-JVS(865)*X(76))/(JVS(859))
  X(69) = (X(69)-JVS(805)*X(70)-JVS(806)*X(71)-JVS(807)*X(72)-JVS(808)*X(73)-JVS(809)*X(74)-JVS(810)*X(75)-JVS(811)&
            &*X(76))/(JVS(804))
  X(68) = (X(68)-JVS(738)*X(69)-JVS(739)*X(70)-JVS(740)*X(71)-JVS(741)*X(72)-JVS(742)*X(73)-JVS(743)*X(74)-JVS(744)&
            &*X(75)-JVS(745)*X(76))/(JVS(737))
  X(67) = (X(67)-JVS(682)*X(68)-JVS(683)*X(69)-JVS(684)*X(70)-JVS(685)*X(71)-JVS(686)*X(72)-JVS(687)*X(73)-JVS(688)&
            &*X(74)-JVS(689)*X(75)-JVS(690)*X(76))/(JVS(681))
  X(66) = (X(66)-JVS(662)*X(67)-JVS(663)*X(68)-JVS(664)*X(69)-JVS(665)*X(70)-JVS(666)*X(72)-JVS(667)*X(73)-JVS(668)&
            &*X(74)-JVS(669)*X(75)-JVS(670)*X(76))/(JVS(661))
  X(65) = (X(65)-JVS(640)*X(66)-JVS(641)*X(67)-JVS(642)*X(68)-JVS(643)*X(69)-JVS(644)*X(70)-JVS(645)*X(72)-JVS(646)&
            &*X(73)-JVS(647)*X(74)-JVS(648)*X(75)-JVS(649)*X(76))/(JVS(639))
  X(64) = (X(64)-JVS(616)*X(65)-JVS(617)*X(66)-JVS(618)*X(67)-JVS(619)*X(68)-JVS(620)*X(69)-JVS(621)*X(70)-JVS(622)&
            &*X(71)-JVS(623)*X(72)-JVS(624)*X(73)-JVS(625)*X(74)-JVS(626)*X(75)-JVS(627)*X(76))/(JVS(615))
  X(63) = (X(63)-JVS(585)*X(67)-JVS(586)*X(68)-JVS(587)*X(69)-JVS(588)*X(70)-JVS(589)*X(72)-JVS(590)*X(73)-JVS(591)&
            &*X(74)-JVS(592)*X(75))/(JVS(584))
  X(62) = (X(62)-JVS(568)*X(63)-JVS(569)*X(65)-JVS(570)*X(66)-JVS(571)*X(67)-JVS(572)*X(68)-JVS(573)*X(69)-JVS(574)&
            &*X(70)-JVS(575)*X(71)-JVS(576)*X(72)-JVS(577)*X(73)-JVS(578)*X(74)-JVS(579)*X(75)-JVS(580)*X(76))/(JVS(567))
  X(61) = (X(61)-JVS(536)*X(67)-JVS(537)*X(68)-JVS(538)*X(69)-JVS(539)*X(70)-JVS(540)*X(72)-JVS(541)*X(73)-JVS(542)&
            &*X(74)-JVS(543)*X(75))/(JVS(535))
  X(60) = (X(60)-JVS(520)*X(61)-JVS(521)*X(62)-JVS(522)*X(63)-JVS(523)*X(66)-JVS(524)*X(67)-JVS(525)*X(68)-JVS(526)&
            &*X(69)-JVS(527)*X(70)-JVS(528)*X(72)-JVS(529)*X(73)-JVS(530)*X(74)-JVS(531)*X(75)-JVS(532)*X(76))/(JVS(519))
  X(59) = (X(59)-JVS(498)*X(63)-JVS(499)*X(67)-JVS(500)*X(68)-JVS(501)*X(69)-JVS(502)*X(70)-JVS(503)*X(72)-JVS(504)&
            &*X(73)-JVS(505)*X(75)-JVS(506)*X(76))/(JVS(497))
  X(58) = (X(58)-JVS(481)*X(59)-JVS(482)*X(61)-JVS(483)*X(63)-JVS(484)*X(66)-JVS(485)*X(67)-JVS(486)*X(68)-JVS(487)&
            &*X(69)-JVS(488)*X(70)-JVS(489)*X(72)-JVS(490)*X(73)-JVS(491)*X(74)-JVS(492)*X(75)-JVS(493)*X(76))/(JVS(480))
  X(57) = (X(57)-JVS(453)*X(61)-JVS(454)*X(63)-JVS(455)*X(65)-JVS(456)*X(66)-JVS(457)*X(67)-JVS(458)*X(68)-JVS(459)&
            &*X(69)-JVS(460)*X(70)-JVS(461)*X(71)-JVS(462)*X(72)-JVS(463)*X(73)-JVS(464)*X(74)-JVS(465)*X(75)-JVS(466)&
            &*X(76))/(JVS(452))
  X(56) = (X(56)-JVS(436)*X(61)-JVS(437)*X(63)-JVS(438)*X(67)-JVS(439)*X(69)-JVS(440)*X(72)-JVS(441)*X(74)-JVS(442)&
            &*X(75))/(JVS(435))
  X(55) = (X(55)-JVS(420)*X(56)-JVS(421)*X(61)-JVS(422)*X(63)-JVS(423)*X(64)-JVS(424)*X(66)-JVS(425)*X(67)-JVS(426)&
            &*X(68)-JVS(427)*X(69)-JVS(428)*X(70)-JVS(429)*X(72)-JVS(430)*X(73)-JVS(431)*X(74)-JVS(432)*X(75)-JVS(433)&
            &*X(76))/(JVS(419))
  X(54) = (X(54)-JVS(396)*X(61)-JVS(397)*X(67)-JVS(398)*X(69)-JVS(399)*X(70)-JVS(400)*X(72)-JVS(401)*X(74))/(JVS(395))
  X(53) = (X(53)-JVS(386)*X(67)-JVS(387)*X(68)-JVS(388)*X(69)-JVS(389)*X(70)-JVS(390)*X(72)-JVS(391)*X(73)-JVS(392)&
            &*X(75)-JVS(393)*X(76))/(JVS(385))
  X(52) = (X(52)-JVS(364)*X(53)-JVS(365)*X(54)-JVS(366)*X(56)-JVS(367)*X(61)-JVS(368)*X(63)-JVS(369)*X(67)-JVS(370)&
            &*X(68)-JVS(371)*X(69)-JVS(372)*X(70)-JVS(373)*X(72)-JVS(374)*X(73)-JVS(375)*X(75)-JVS(376)*X(76))/(JVS(363))
  X(51) = (X(51)-JVS(351)*X(67)-JVS(352)*X(68)-JVS(353)*X(69)-JVS(354)*X(70)-JVS(355)*X(72)-JVS(356)*X(73)-JVS(357)&
            &*X(75))/(JVS(350))
  X(50) = (X(50)-JVS(331)*X(51)-JVS(332)*X(53)-JVS(333)*X(54)-JVS(334)*X(56)-JVS(335)*X(59)-JVS(336)*X(61)-JVS(337)&
            &*X(63)-JVS(338)*X(67)-JVS(339)*X(68)-JVS(340)*X(69)-JVS(341)*X(70)-JVS(342)*X(72)-JVS(343)*X(73)-JVS(344)*X(74)&
            &-JVS(345)*X(75)-JVS(346)*X(76))/(JVS(330))
  X(49) = (X(49)-JVS(312)*X(61)-JVS(313)*X(63)-JVS(314)*X(67)-JVS(315)*X(68)-JVS(316)*X(69)-JVS(317)*X(70)-JVS(318)&
            &*X(72)-JVS(319)*X(73)-JVS(320)*X(74)-JVS(321)*X(75))/(JVS(311))
  X(48) = (X(48)-JVS(283)*X(49)-JVS(284)*X(50)-JVS(285)*X(51)-JVS(286)*X(52)-JVS(287)*X(53)-JVS(288)*X(54)-JVS(289)&
            &*X(55)-JVS(290)*X(56)-JVS(291)*X(57)-JVS(292)*X(59)-JVS(293)*X(60)-JVS(294)*X(61)-JVS(295)*X(63)-JVS(296)*X(65)&
            &-JVS(297)*X(66)-JVS(298)*X(67)-JVS(299)*X(68)-JVS(300)*X(69)-JVS(301)*X(70)-JVS(302)*X(72)-JVS(303)*X(73)&
            &-JVS(304)*X(74)-JVS(305)*X(75)-JVS(306)*X(76))/(JVS(282))
  X(47) = (X(47)-JVS(264)*X(61)-JVS(265)*X(68)-JVS(266)*X(69)-JVS(267)*X(70)-JVS(268)*X(73)-JVS(269)*X(75))/(JVS(263))
  X(46) = (X(46)-JVS(257)*X(67)-JVS(258)*X(69)-JVS(259)*X(72)-JVS(260)*X(74))/(JVS(256))
  X(45) = (X(45)-JVS(252)*X(69)-JVS(253)*X(70)-JVS(254)*X(72)-JVS(255)*X(76))/(JVS(251))
  X(44) = (X(44)-JVS(242)*X(67)-JVS(243)*X(69)-JVS(244)*X(72)-JVS(245)*X(74))/(JVS(241))
  X(43) = (X(43)-JVS(236)*X(68)-JVS(237)*X(69)-JVS(238)*X(70)-JVS(239)*X(73)-JVS(240)*X(75))/(JVS(235))
  X(42) = (X(42)-JVS(208)*X(44)-JVS(209)*X(45)-JVS(210)*X(46)-JVS(211)*X(47)-JVS(212)*X(49)-JVS(213)*X(50)-JVS(214)&
            &*X(51)-JVS(215)*X(52)-JVS(216)*X(53)-JVS(217)*X(54)-JVS(218)*X(55)-JVS(219)*X(56)-JVS(220)*X(57)-JVS(221)*X(59)&
            &-JVS(222)*X(63)-JVS(223)*X(65)-JVS(224)*X(66)-JVS(225)*X(67)-JVS(226)*X(68)-JVS(227)*X(69)-JVS(228)*X(70)&
            &-JVS(229)*X(72)-JVS(230)*X(73)-JVS(231)*X(74)-JVS(232)*X(75)-JVS(233)*X(76))/(JVS(207))
  X(41) = (X(41)-JVS(198)*X(67)-JVS(199)*X(69)-JVS(200)*X(72)-JVS(201)*X(74))/(JVS(197))
  X(40) = (X(40)-JVS(192)*X(68)-JVS(193)*X(69)-JVS(194)*X(70)-JVS(195)*X(73)-JVS(196)*X(75))/(JVS(191))
  X(39) = (X(39)-JVS(187)*X(66)-JVS(188)*X(69)-JVS(189)*X(76))/(JVS(186))
  X(38) = (X(38)-JVS(171)*X(47)-JVS(172)*X(54)-JVS(173)*X(56)-JVS(174)*X(58)-JVS(175)*X(59)-JVS(176)*X(60)-JVS(177)&
            &*X(61)-JVS(178)*X(62)-JVS(179)*X(64)-JVS(180)*X(67)-JVS(181)*X(68)-JVS(182)*X(69)-JVS(183)*X(70)-JVS(184)*X(71)&
            &-JVS(185)*X(73))/(JVS(170))
  X(37) = (X(37)-JVS(165)*X(45)-JVS(166)*X(69)-JVS(167)*X(70)-JVS(168)*X(72)-JVS(169)*X(76))/(JVS(164))
  X(36) = (X(36)-JVS(158)*X(68)-JVS(159)*X(69)-JVS(160)*X(70)-JVS(161)*X(73)-JVS(162)*X(75))/(JVS(157))
  X(35) = (X(35)-JVS(153)*X(61)-JVS(154)*X(69)-JVS(155)*X(75))/(JVS(152))
  X(34) = (X(34)-JVS(148)*X(39)-JVS(149)*X(63)-JVS(150)*X(67)-JVS(151)*X(69))/(JVS(147))
  X(33) = (X(33)-JVS(137)*X(39)-JVS(138)*X(41)-JVS(139)*X(54)-JVS(140)*X(56)-JVS(141)*X(60)-JVS(142)*X(69)-JVS(143)&
            &*X(72)-JVS(144)*X(75)-JVS(145)*X(76))/(JVS(136))
  X(32) = (X(32)-JVS(131)*X(58)-JVS(132)*X(60)-JVS(133)*X(62)-JVS(134)*X(69)-JVS(135)*X(70))/(JVS(130))
  X(31) = (X(31)-JVS(124)*X(37)-JVS(125)*X(45)-JVS(126)*X(69)-JVS(127)*X(70)-JVS(128)*X(72)-JVS(129)*X(76))/(JVS(123))
  X(30) = (X(30)-JVS(119)*X(55)-JVS(120)*X(70)-JVS(121)*X(75))/(JVS(118))
  X(29) = (X(29)-JVS(115)*X(69)-JVS(116)*X(71)-JVS(117)*X(76))/(JVS(114))
  X(28) = (X(28)-JVS(104)*X(30)-JVS(105)*X(35)-JVS(106)*X(41)-JVS(107)*X(47)-JVS(108)*X(56)-JVS(109)*X(63)-JVS(110)&
            &*X(67)-JVS(111)*X(69)-JVS(112)*X(70)-JVS(113)*X(75))/(JVS(103))
  X(27) = (X(27)-JVS(99)*X(61)-JVS(100)*X(69)-JVS(101)*X(70))/(JVS(98))
  X(26) = (X(26)-JVS(92)*X(54)-JVS(93)*X(56)-JVS(94)*X(67)-JVS(95)*X(69)-JVS(96)*X(70)-JVS(97)*X(74))/(JVS(91))
  X(25) = (X(25)-JVS(88)*X(45)-JVS(89)*X(69)-JVS(90)*X(72))/(JVS(87))
  X(24) = (X(24)-JVS(84)*X(59)-JVS(85)*X(69)-JVS(86)*X(76))/(JVS(83))
  X(23) = (X(23)-JVS(79)*X(30)-JVS(80)*X(64)-JVS(81)*X(69)-JVS(82)*X(70))/(JVS(78))
  X(22) = (X(22)-JVS(74)*X(37)-JVS(75)*X(69)-JVS(76)*X(75)-JVS(77)*X(76))/(JVS(73))
  X(21) = (X(21)-JVS(69)*X(49)-JVS(70)*X(64)-JVS(71)*X(68)-JVS(72)*X(69))/(JVS(68))
  X(20) = (X(20)-JVS(65)*X(69)-JVS(66)*X(70)-JVS(67)*X(76))/(JVS(64))
  X(19) = (X(19)-JVS(53)*X(24)-JVS(54)*X(33)-JVS(55)*X(37)-JVS(56)*X(43)-JVS(57)*X(46)-JVS(58)*X(51)-JVS(59)*X(60)&
            &-JVS(60)*X(63)-JVS(61)*X(69)-JVS(62)*X(72)-JVS(63)*X(75))/(JVS(52))
  X(18) = (X(18)-JVS(47)*X(59)-JVS(48)*X(69)-JVS(49)*X(70)-JVS(50)*X(71)-JVS(51)*X(73))/(JVS(46))
  X(17) = (X(17)-JVS(44)*X(73)-JVS(45)*X(76))/(JVS(43))
  X(16) = (X(16)-JVS(41)*X(61)-JVS(42)*X(72))/(JVS(40))
  X(15) = (X(15)-JVS(39)*X(69))/(JVS(38))
  X(14) = (X(14)-JVS(37)*X(69))/(JVS(36))
  X(13) = (X(13)-JVS(35)*X(39))/(JVS(34))
  X(12) = (X(12)-JVS(33)*X(69))/(JVS(32))
  X(11) = (X(11)-JVS(31)*X(69))/(JVS(30))
  X(10) = (X(10)-JVS(28)*X(27)-JVS(29)*X(69))/(JVS(27))
  X(9) = (X(9)-JVS(26)*X(69))/(JVS(25))
  X(8) = (X(8)-JVS(23)*X(72)-JVS(24)*X(76))/(JVS(22))
  X(7) = (X(7)-JVS(21)*X(69))/(JVS(20))
  X(6) = (X(6)-JVS(19)*X(69))/(JVS(18))
  X(5) = (X(5)-JVS(17)*X(69))/(JVS(16))
  X(4) = (X(4)-JVS(15)*X(67))/(JVS(14))
  X(3) = (X(3)-JVS(13)*X(69))/(JVS(12))
  X(2) = (X(2)-JVS(10)*X(3)-JVS(11)*X(69))/(JVS(9))
  X(1) = (X(1)-JVS(2)*X(35)-JVS(3)*X(36)-JVS(4)*X(40)-JVS(5)*X(44)-JVS(6)*X(69)-JVS(7)*X(72)-JVS(8)*X(75))/(JVS(1))
      
END SUBROUTINE KppSolve

! End of KppSolve function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! KppSolveTR - sparse, transposed back substitution
!   Arguments :
!      JVS       - sparse Jacobian of variables
!      X         - Vector for variables
!      XX        - Vector for output variables
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SUBROUTINE KppSolveTR ( JVS, X, XX )

! JVS - sparse Jacobian of variables
  REAL(kind=dp) :: JVS(LU_NONZERO)
! X - Vector for variables
  REAL(kind=dp) :: X(NVAR_CB6)
! XX - Vector for output variables
  REAL(kind=dp) :: XX(NVAR_CB6)

  XX(1) = X(1)/JVS(1)
  XX(2) = X(2)/JVS(9)
  XX(3) = (X(3)-JVS(10)*XX(2))/(JVS(12))
  XX(4) = X(4)/JVS(14)
  XX(5) = X(5)/JVS(16)
  XX(6) = X(6)/JVS(18)
  XX(7) = X(7)/JVS(20)
  XX(8) = X(8)/JVS(22)
  XX(9) = X(9)/JVS(25)
  XX(10) = X(10)/JVS(27)
  XX(11) = X(11)/JVS(30)
  XX(12) = X(12)/JVS(32)
  XX(13) = X(13)/JVS(34)
  XX(14) = X(14)/JVS(36)
  XX(15) = X(15)/JVS(38)
  XX(16) = X(16)/JVS(40)
  XX(17) = X(17)/JVS(43)
  XX(18) = X(18)/JVS(46)
  XX(19) = X(19)/JVS(52)
  XX(20) = X(20)/JVS(64)
  XX(21) = X(21)/JVS(68)
  XX(22) = X(22)/JVS(73)
  XX(23) = X(23)/JVS(78)
  XX(24) = (X(24)-JVS(53)*XX(19))/(JVS(83))
  XX(25) = X(25)/JVS(87)
  XX(26) = X(26)/JVS(91)
  XX(27) = (X(27)-JVS(28)*XX(10))/(JVS(98))
  XX(28) = X(28)/JVS(103)
  XX(29) = X(29)/JVS(114)
  XX(30) = (X(30)-JVS(79)*XX(23)-JVS(104)*XX(28))/(JVS(118))
  XX(31) = X(31)/JVS(123)
  XX(32) = X(32)/JVS(130)
  XX(33) = (X(33)-JVS(54)*XX(19))/(JVS(136))
  XX(34) = X(34)/JVS(147)
  XX(35) = (X(35)-JVS(2)*XX(1)-JVS(105)*XX(28))/(JVS(152))
  XX(36) = (X(36)-JVS(3)*XX(1))/(JVS(157))
  XX(37) = (X(37)-JVS(55)*XX(19)-JVS(74)*XX(22)-JVS(124)*XX(31))/(JVS(164))
  XX(38) = X(38)/JVS(170)
  XX(39) = (X(39)-JVS(35)*XX(13)-JVS(137)*XX(33)-JVS(148)*XX(34))/(JVS(186))
  XX(40) = (X(40)-JVS(4)*XX(1))/(JVS(191))
  XX(41) = (X(41)-JVS(106)*XX(28)-JVS(138)*XX(33))/(JVS(197))
  XX(42) = X(42)/JVS(207)
  XX(43) = (X(43)-JVS(56)*XX(19))/(JVS(235))
  XX(44) = (X(44)-JVS(5)*XX(1)-JVS(208)*XX(42))/(JVS(241))
  XX(45) = (X(45)-JVS(88)*XX(25)-JVS(125)*XX(31)-JVS(165)*XX(37)-JVS(209)*XX(42))/(JVS(251))
  XX(46) = (X(46)-JVS(57)*XX(19)-JVS(210)*XX(42))/(JVS(256))
  XX(47) = (X(47)-JVS(107)*XX(28)-JVS(171)*XX(38)-JVS(211)*XX(42))/(JVS(263))
  XX(48) = X(48)/JVS(282)
  XX(49) = (X(49)-JVS(69)*XX(21)-JVS(212)*XX(42)-JVS(283)*XX(48))/(JVS(311))
  XX(50) = (X(50)-JVS(213)*XX(42)-JVS(284)*XX(48))/(JVS(330))
  XX(51) = (X(51)-JVS(58)*XX(19)-JVS(214)*XX(42)-JVS(285)*XX(48)-JVS(331)*XX(50))/(JVS(350))
  XX(52) = (X(52)-JVS(215)*XX(42)-JVS(286)*XX(48))/(JVS(363))
  XX(53) = (X(53)-JVS(216)*XX(42)-JVS(287)*XX(48)-JVS(332)*XX(50)-JVS(364)*XX(52))/(JVS(385))
  XX(54) = (X(54)-JVS(92)*XX(26)-JVS(139)*XX(33)-JVS(172)*XX(38)-JVS(217)*XX(42)-JVS(288)*XX(48)-JVS(333)*XX(50)&
             &-JVS(365)*XX(52))/(JVS(395))
  XX(55) = (X(55)-JVS(119)*XX(30)-JVS(218)*XX(42)-JVS(289)*XX(48))/(JVS(419))
  XX(56) = (X(56)-JVS(93)*XX(26)-JVS(108)*XX(28)-JVS(140)*XX(33)-JVS(173)*XX(38)-JVS(219)*XX(42)-JVS(290)*XX(48)&
             &-JVS(334)*XX(50)-JVS(366)*XX(52)-JVS(420)*XX(55))/(JVS(435))
  XX(57) = (X(57)-JVS(220)*XX(42)-JVS(291)*XX(48))/(JVS(452))
  XX(58) = (X(58)-JVS(131)*XX(32)-JVS(174)*XX(38))/(JVS(480))
  XX(59) = (X(59)-JVS(47)*XX(18)-JVS(84)*XX(24)-JVS(175)*XX(38)-JVS(221)*XX(42)-JVS(292)*XX(48)-JVS(335)*XX(50)-JVS(481)&
             &*XX(58))/(JVS(497))
  XX(60) = (X(60)-JVS(59)*XX(19)-JVS(132)*XX(32)-JVS(141)*XX(33)-JVS(176)*XX(38)-JVS(293)*XX(48))/(JVS(519))
  XX(61) = (X(61)-JVS(41)*XX(16)-JVS(99)*XX(27)-JVS(153)*XX(35)-JVS(177)*XX(38)-JVS(264)*XX(47)-JVS(294)*XX(48)-JVS(312)&
             &*XX(49)-JVS(336)*XX(50)-JVS(367)*XX(52)-JVS(396)*XX(54)-JVS(421)*XX(55)-JVS(436)*XX(56)-JVS(453)*XX(57)&
             &-JVS(482)*XX(58)-JVS(520)*XX(60))/(JVS(535))
  XX(62) = (X(62)-JVS(133)*XX(32)-JVS(178)*XX(38)-JVS(521)*XX(60))/(JVS(567))
  XX(63) = (X(63)-JVS(60)*XX(19)-JVS(109)*XX(28)-JVS(149)*XX(34)-JVS(222)*XX(42)-JVS(295)*XX(48)-JVS(313)*XX(49)&
             &-JVS(337)*XX(50)-JVS(368)*XX(52)-JVS(422)*XX(55)-JVS(437)*XX(56)-JVS(454)*XX(57)-JVS(483)*XX(58)-JVS(498)&
             &*XX(59)-JVS(522)*XX(60)-JVS(568)*XX(62))/(JVS(584))
  XX(64) = (X(64)-JVS(70)*XX(21)-JVS(80)*XX(23)-JVS(179)*XX(38)-JVS(423)*XX(55))/(JVS(615))
  XX(65) = (X(65)-JVS(223)*XX(42)-JVS(296)*XX(48)-JVS(455)*XX(57)-JVS(569)*XX(62)-JVS(616)*XX(64))/(JVS(639))
  XX(66) = (X(66)-JVS(187)*XX(39)-JVS(224)*XX(42)-JVS(297)*XX(48)-JVS(424)*XX(55)-JVS(456)*XX(57)-JVS(484)*XX(58)&
             &-JVS(523)*XX(60)-JVS(570)*XX(62)-JVS(617)*XX(64)-JVS(640)*XX(65))/(JVS(661))
  XX(67) = (X(67)-JVS(15)*XX(4)-JVS(94)*XX(26)-JVS(110)*XX(28)-JVS(150)*XX(34)-JVS(180)*XX(38)-JVS(198)*XX(41)-JVS(225)&
             &*XX(42)-JVS(242)*XX(44)-JVS(257)*XX(46)-JVS(298)*XX(48)-JVS(314)*XX(49)-JVS(338)*XX(50)-JVS(351)*XX(51)&
             &-JVS(369)*XX(52)-JVS(386)*XX(53)-JVS(397)*XX(54)-JVS(425)*XX(55)-JVS(438)*XX(56)-JVS(457)*XX(57)-JVS(485)&
             &*XX(58)-JVS(499)*XX(59)-JVS(524)*XX(60)-JVS(536)*XX(61)-JVS(571)*XX(62)-JVS(585)*XX(63)-JVS(618)*XX(64)&
             &-JVS(641)*XX(65)-JVS(662)*XX(66))/(JVS(681))
  XX(68) = (X(68)-JVS(71)*XX(21)-JVS(158)*XX(36)-JVS(181)*XX(38)-JVS(192)*XX(40)-JVS(226)*XX(42)-JVS(236)*XX(43)&
             &-JVS(265)*XX(47)-JVS(299)*XX(48)-JVS(315)*XX(49)-JVS(339)*XX(50)-JVS(352)*XX(51)-JVS(370)*XX(52)-JVS(387)&
             &*XX(53)-JVS(426)*XX(55)-JVS(458)*XX(57)-JVS(486)*XX(58)-JVS(500)*XX(59)-JVS(525)*XX(60)-JVS(537)*XX(61)&
             &-JVS(572)*XX(62)-JVS(586)*XX(63)-JVS(619)*XX(64)-JVS(642)*XX(65)-JVS(663)*XX(66)-JVS(682)*XX(67))/(JVS(737))
  XX(69) = (X(69)-JVS(6)*XX(1)-JVS(11)*XX(2)-JVS(13)*XX(3)-JVS(17)*XX(5)-JVS(19)*XX(6)-JVS(21)*XX(7)-JVS(26)*XX(9)&
             &-JVS(29)*XX(10)-JVS(31)*XX(11)-JVS(33)*XX(12)-JVS(37)*XX(14)-JVS(39)*XX(15)-JVS(48)*XX(18)-JVS(61)*XX(19)&
             &-JVS(65)*XX(20)-JVS(72)*XX(21)-JVS(75)*XX(22)-JVS(81)*XX(23)-JVS(85)*XX(24)-JVS(89)*XX(25)-JVS(95)*XX(26)&
             &-JVS(100)*XX(27)-JVS(111)*XX(28)-JVS(115)*XX(29)-JVS(126)*XX(31)-JVS(134)*XX(32)-JVS(142)*XX(33)-JVS(151)&
             &*XX(34)-JVS(154)*XX(35)-JVS(159)*XX(36)-JVS(166)*XX(37)-JVS(182)*XX(38)-JVS(188)*XX(39)-JVS(193)*XX(40)&
             &-JVS(199)*XX(41)-JVS(227)*XX(42)-JVS(237)*XX(43)-JVS(243)*XX(44)-JVS(252)*XX(45)-JVS(258)*XX(46)-JVS(266)&
             &*XX(47)-JVS(300)*XX(48)-JVS(316)*XX(49)-JVS(340)*XX(50)-JVS(353)*XX(51)-JVS(371)*XX(52)-JVS(388)*XX(53)&
             &-JVS(398)*XX(54)-JVS(427)*XX(55)-JVS(439)*XX(56)-JVS(459)*XX(57)-JVS(487)*XX(58)-JVS(501)*XX(59)-JVS(526)&
             &*XX(60)-JVS(538)*XX(61)-JVS(573)*XX(62)-JVS(587)*XX(63)-JVS(620)*XX(64)-JVS(643)*XX(65)-JVS(664)*XX(66)&
             &-JVS(683)*XX(67)-JVS(738)*XX(68))/(JVS(804))
  XX(70) = (X(70)-JVS(49)*XX(18)-JVS(66)*XX(20)-JVS(82)*XX(23)-JVS(96)*XX(26)-JVS(101)*XX(27)-JVS(112)*XX(28)-JVS(120)&
             &*XX(30)-JVS(127)*XX(31)-JVS(135)*XX(32)-JVS(160)*XX(36)-JVS(167)*XX(37)-JVS(183)*XX(38)-JVS(194)*XX(40)&
             &-JVS(228)*XX(42)-JVS(238)*XX(43)-JVS(253)*XX(45)-JVS(267)*XX(47)-JVS(301)*XX(48)-JVS(317)*XX(49)-JVS(341)&
             &*XX(50)-JVS(354)*XX(51)-JVS(372)*XX(52)-JVS(389)*XX(53)-JVS(399)*XX(54)-JVS(428)*XX(55)-JVS(460)*XX(57)&
             &-JVS(488)*XX(58)-JVS(502)*XX(59)-JVS(527)*XX(60)-JVS(539)*XX(61)-JVS(574)*XX(62)-JVS(588)*XX(63)-JVS(621)&
             &*XX(64)-JVS(644)*XX(65)-JVS(665)*XX(66)-JVS(684)*XX(67)-JVS(739)*XX(68)-JVS(805)*XX(69))/(JVS(859))
  XX(71) = (X(71)-JVS(50)*XX(18)-JVS(116)*XX(29)-JVS(184)*XX(38)-JVS(461)*XX(57)-JVS(575)*XX(62)-JVS(622)*XX(64)&
             &-JVS(685)*XX(67)-JVS(740)*XX(68)-JVS(806)*XX(69)-JVS(860)*XX(70))/(JVS(879))
  XX(72) = (X(72)-JVS(7)*XX(1)-JVS(23)*XX(8)-JVS(42)*XX(16)-JVS(62)*XX(19)-JVS(90)*XX(25)-JVS(128)*XX(31)-JVS(143)&
             &*XX(33)-JVS(168)*XX(37)-JVS(200)*XX(41)-JVS(229)*XX(42)-JVS(244)*XX(44)-JVS(254)*XX(45)-JVS(259)*XX(46)&
             &-JVS(302)*XX(48)-JVS(318)*XX(49)-JVS(342)*XX(50)-JVS(355)*XX(51)-JVS(373)*XX(52)-JVS(390)*XX(53)-JVS(400)&
             &*XX(54)-JVS(429)*XX(55)-JVS(440)*XX(56)-JVS(462)*XX(57)-JVS(489)*XX(58)-JVS(503)*XX(59)-JVS(528)*XX(60)&
             &-JVS(540)*XX(61)-JVS(576)*XX(62)-JVS(589)*XX(63)-JVS(623)*XX(64)-JVS(645)*XX(65)-JVS(666)*XX(66)-JVS(686)&
             &*XX(67)-JVS(741)*XX(68)-JVS(807)*XX(69)-JVS(861)*XX(70)-JVS(880)*XX(71))/(JVS(920))
  XX(73) = (X(73)-JVS(44)*XX(17)-JVS(51)*XX(18)-JVS(161)*XX(36)-JVS(185)*XX(38)-JVS(195)*XX(40)-JVS(230)*XX(42)-JVS(239)&
             &*XX(43)-JVS(268)*XX(47)-JVS(303)*XX(48)-JVS(319)*XX(49)-JVS(343)*XX(50)-JVS(356)*XX(51)-JVS(374)*XX(52)&
             &-JVS(391)*XX(53)-JVS(430)*XX(55)-JVS(463)*XX(57)-JVS(490)*XX(58)-JVS(504)*XX(59)-JVS(529)*XX(60)-JVS(541)&
             &*XX(61)-JVS(577)*XX(62)-JVS(590)*XX(63)-JVS(624)*XX(64)-JVS(646)*XX(65)-JVS(667)*XX(66)-JVS(687)*XX(67)&
             &-JVS(742)*XX(68)-JVS(808)*XX(69)-JVS(862)*XX(70)-JVS(881)*XX(71)-JVS(921)*XX(72))/(JVS(956))
  XX(74) = (X(74)-JVS(97)*XX(26)-JVS(201)*XX(41)-JVS(231)*XX(42)-JVS(245)*XX(44)-JVS(260)*XX(46)-JVS(304)*XX(48)&
             &-JVS(320)*XX(49)-JVS(344)*XX(50)-JVS(401)*XX(54)-JVS(431)*XX(55)-JVS(441)*XX(56)-JVS(464)*XX(57)-JVS(491)&
             &*XX(58)-JVS(530)*XX(60)-JVS(542)*XX(61)-JVS(578)*XX(62)-JVS(591)*XX(63)-JVS(625)*XX(64)-JVS(647)*XX(65)&
             &-JVS(668)*XX(66)-JVS(688)*XX(67)-JVS(743)*XX(68)-JVS(809)*XX(69)-JVS(863)*XX(70)-JVS(882)*XX(71)-JVS(922)&
             &*XX(72)-JVS(957)*XX(73))/(JVS(981))
  XX(75) = (X(75)-JVS(8)*XX(1)-JVS(63)*XX(19)-JVS(76)*XX(22)-JVS(113)*XX(28)-JVS(121)*XX(30)-JVS(144)*XX(33)-JVS(155)&
             &*XX(35)-JVS(162)*XX(36)-JVS(196)*XX(40)-JVS(232)*XX(42)-JVS(240)*XX(43)-JVS(269)*XX(47)-JVS(305)*XX(48)&
             &-JVS(321)*XX(49)-JVS(345)*XX(50)-JVS(357)*XX(51)-JVS(375)*XX(52)-JVS(392)*XX(53)-JVS(432)*XX(55)-JVS(442)&
             &*XX(56)-JVS(465)*XX(57)-JVS(492)*XX(58)-JVS(505)*XX(59)-JVS(531)*XX(60)-JVS(543)*XX(61)-JVS(579)*XX(62)&
             &-JVS(592)*XX(63)-JVS(626)*XX(64)-JVS(648)*XX(65)-JVS(669)*XX(66)-JVS(689)*XX(67)-JVS(744)*XX(68)-JVS(810)&
             &*XX(69)-JVS(864)*XX(70)-JVS(883)*XX(71)-JVS(923)*XX(72)-JVS(958)*XX(73)-JVS(982)*XX(74))/(JVS(1011))
  XX(76) = (X(76)-JVS(24)*XX(8)-JVS(45)*XX(17)-JVS(67)*XX(20)-JVS(77)*XX(22)-JVS(86)*XX(24)-JVS(117)*XX(29)-JVS(129)&
             &*XX(31)-JVS(145)*XX(33)-JVS(169)*XX(37)-JVS(189)*XX(39)-JVS(233)*XX(42)-JVS(255)*XX(45)-JVS(306)*XX(48)&
             &-JVS(346)*XX(50)-JVS(376)*XX(52)-JVS(393)*XX(53)-JVS(433)*XX(55)-JVS(466)*XX(57)-JVS(493)*XX(58)-JVS(506)&
             &*XX(59)-JVS(532)*XX(60)-JVS(580)*XX(62)-JVS(627)*XX(64)-JVS(649)*XX(65)-JVS(670)*XX(66)-JVS(690)*XX(67)&
             &-JVS(745)*XX(68)-JVS(811)*XX(69)-JVS(865)*XX(70)-JVS(884)*XX(71)-JVS(924)*XX(72)-JVS(959)*XX(73)-JVS(983)&
             &*XX(74)-JVS(1012)*XX(75))/(JVS(1061))
  XX(76) = XX(76)
  XX(75) = XX(75)-JVS(1060)*XX(76)
  XX(74) = XX(74)-JVS(1010)*XX(75)-JVS(1059)*XX(76)
  XX(73) = XX(73)-JVS(980)*XX(74)-JVS(1009)*XX(75)-JVS(1058)*XX(76)
  XX(72) = XX(72)-JVS(955)*XX(73)-JVS(979)*XX(74)-JVS(1008)*XX(75)-JVS(1057)*XX(76)
  XX(71) = XX(71)-JVS(919)*XX(72)-JVS(954)*XX(73)-JVS(978)*XX(74)-JVS(1007)*XX(75)-JVS(1056)*XX(76)
  XX(70) = XX(70)-JVS(878)*XX(71)-JVS(918)*XX(72)-JVS(953)*XX(73)-JVS(977)*XX(74)-JVS(1006)*XX(75)-JVS(1055)*XX(76)
  XX(69) = XX(69)-JVS(858)*XX(70)-JVS(877)*XX(71)-JVS(917)*XX(72)-JVS(952)*XX(73)-JVS(976)*XX(74)-JVS(1005)*XX(75)&
             &-JVS(1054)*XX(76)
  XX(68) = XX(68)-JVS(803)*XX(69)-JVS(857)*XX(70)-JVS(876)*XX(71)-JVS(916)*XX(72)-JVS(951)*XX(73)-JVS(975)*XX(74)&
             &-JVS(1004)*XX(75)-JVS(1053)*XX(76)
  XX(67) = XX(67)-JVS(736)*XX(68)-JVS(802)*XX(69)-JVS(856)*XX(70)-JVS(875)*XX(71)-JVS(915)*XX(72)-JVS(950)*XX(73)&
             &-JVS(974)*XX(74)-JVS(1003)*XX(75)-JVS(1052)*XX(76)
  XX(66) = XX(66)-JVS(735)*XX(68)-JVS(801)*XX(69)-JVS(855)*XX(70)-JVS(874)*XX(71)-JVS(914)*XX(72)-JVS(949)*XX(73)&
             &-JVS(973)*XX(74)-JVS(1002)*XX(75)-JVS(1051)*XX(76)
  XX(65) = XX(65)-JVS(734)*XX(68)-JVS(800)*XX(69)-JVS(854)*XX(70)-JVS(873)*XX(71)-JVS(913)*XX(72)-JVS(948)*XX(73)&
             &-JVS(972)*XX(74)-JVS(1001)*XX(75)-JVS(1050)*XX(76)
  XX(64) = XX(64)-JVS(733)*XX(68)-JVS(799)*XX(69)-JVS(853)*XX(70)-JVS(912)*XX(72)-JVS(947)*XX(73)-JVS(971)*XX(74)&
             &-JVS(1000)*XX(75)-JVS(1049)*XX(76)
  XX(63) = XX(63)-JVS(614)*XX(64)-JVS(638)*XX(65)-JVS(660)*XX(66)-JVS(680)*XX(67)-JVS(732)*XX(68)-JVS(798)*XX(69)&
             &-JVS(852)*XX(70)-JVS(872)*XX(71)-JVS(911)*XX(72)-JVS(946)*XX(73)-JVS(970)*XX(74)-JVS(999)*XX(75)-JVS(1048)&
             &*XX(76)
  XX(62) = XX(62)-JVS(613)*XX(64)-JVS(731)*XX(68)-JVS(797)*XX(69)-JVS(851)*XX(70)-JVS(910)*XX(72)-JVS(945)*XX(73)&
             &-JVS(998)*XX(75)-JVS(1047)*XX(76)
  XX(61) = XX(61)-JVS(566)*XX(62)-JVS(583)*XX(63)-JVS(612)*XX(64)-JVS(637)*XX(65)-JVS(659)*XX(66)-JVS(679)*XX(67)&
             &-JVS(730)*XX(68)-JVS(796)*XX(69)-JVS(850)*XX(70)-JVS(909)*XX(72)-JVS(944)*XX(73)-JVS(969)*XX(74)-JVS(997)&
             &*XX(75)-JVS(1046)*XX(76)
  XX(60) = XX(60)-JVS(565)*XX(62)-JVS(611)*XX(64)-JVS(729)*XX(68)-JVS(795)*XX(69)-JVS(849)*XX(70)-JVS(908)*XX(72)&
             &-JVS(943)*XX(73)-JVS(996)*XX(75)-JVS(1045)*XX(76)
  XX(59) = XX(59)-JVS(518)*XX(60)-JVS(564)*XX(62)-JVS(610)*XX(64)-JVS(636)*XX(65)-JVS(678)*XX(67)-JVS(728)*XX(68)&
             &-JVS(794)*XX(69)-JVS(848)*XX(70)-JVS(871)*XX(71)-JVS(907)*XX(72)-JVS(942)*XX(73)-JVS(995)*XX(75)-JVS(1044)&
             &*XX(76)
  XX(58) = XX(58)-JVS(517)*XX(60)-JVS(563)*XX(62)-JVS(609)*XX(64)-JVS(727)*XX(68)-JVS(793)*XX(69)-JVS(847)*XX(70)&
             &-JVS(941)*XX(73)-JVS(994)*XX(75)-JVS(1043)*XX(76)
  XX(57) = XX(57)-JVS(608)*XX(64)-JVS(726)*XX(68)-JVS(792)*XX(69)-JVS(846)*XX(70)-JVS(906)*XX(72)-JVS(940)*XX(73)&
             &-JVS(968)*XX(74)-JVS(1042)*XX(76)
  XX(56) = XX(56)-JVS(451)*XX(57)-JVS(479)*XX(58)-JVS(516)*XX(60)-JVS(562)*XX(62)-JVS(607)*XX(64)-JVS(635)*XX(65)&
             &-JVS(658)*XX(66)-JVS(677)*XX(67)-JVS(725)*XX(68)-JVS(791)*XX(69)-JVS(845)*XX(70)-JVS(905)*XX(72)-JVS(939)&
             &*XX(73)-JVS(967)*XX(74)-JVS(993)*XX(75)-JVS(1041)*XX(76)
  XX(55) = XX(55)-JVS(606)*XX(64)-JVS(724)*XX(68)-JVS(790)*XX(69)-JVS(844)*XX(70)-JVS(904)*XX(72)-JVS(966)*XX(74)&
             &-JVS(992)*XX(75)-JVS(1040)*XX(76)
  XX(54) = XX(54)-JVS(418)*XX(55)-JVS(450)*XX(57)-JVS(478)*XX(58)-JVS(515)*XX(60)-JVS(561)*XX(62)-JVS(605)*XX(64)&
             &-JVS(634)*XX(65)-JVS(657)*XX(66)-JVS(676)*XX(67)-JVS(723)*XX(68)-JVS(789)*XX(69)-JVS(843)*XX(70)-JVS(903)&
             &*XX(72)-JVS(938)*XX(73)-JVS(965)*XX(74)-JVS(1039)*XX(76)
  XX(53) = XX(53)-JVS(417)*XX(55)-JVS(449)*XX(57)-JVS(477)*XX(58)-JVS(496)*XX(59)-JVS(560)*XX(62)-JVS(675)*XX(67)&
             &-JVS(722)*XX(68)-JVS(788)*XX(69)-JVS(842)*XX(70)-JVS(902)*XX(72)-JVS(937)*XX(73)-JVS(1038)*XX(76)
  XX(52) = XX(52)-JVS(476)*XX(58)-JVS(721)*XX(68)-JVS(787)*XX(69)-JVS(841)*XX(70)-JVS(901)*XX(72)-JVS(936)*XX(73)&
             &-JVS(1037)*XX(76)
  XX(51) = XX(51)-JVS(362)*XX(52)-JVS(384)*XX(53)-JVS(448)*XX(57)-JVS(475)*XX(58)-JVS(514)*XX(60)-JVS(559)*XX(62)&
             &-JVS(674)*XX(67)-JVS(720)*XX(68)-JVS(786)*XX(69)-JVS(840)*XX(70)-JVS(900)*XX(72)-JVS(935)*XX(73)-JVS(1036)&
             &*XX(76)
  XX(50) = XX(50)-JVS(474)*XX(58)-JVS(719)*XX(68)-JVS(785)*XX(69)-JVS(839)*XX(70)-JVS(899)*XX(72)-JVS(1035)*XX(76)
  XX(49) = XX(49)-JVS(329)*XX(50)-JVS(416)*XX(55)-JVS(558)*XX(62)-JVS(718)*XX(68)-JVS(784)*XX(69)-JVS(838)*XX(70)&
             &-JVS(898)*XX(72)-JVS(934)*XX(73)-JVS(1034)*XX(76)
  XX(48) = XX(48)-JVS(783)*XX(69)-JVS(897)*XX(72)-JVS(1033)*XX(76)
  XX(47) = XX(47)-JVS(310)*XX(49)-JVS(328)*XX(50)-JVS(361)*XX(52)-JVS(415)*XX(55)-JVS(604)*XX(64)-JVS(656)*XX(66)&
             &-JVS(717)*XX(68)-JVS(782)*XX(69)-JVS(837)*XX(70)-JVS(933)*XX(73)-JVS(991)*XX(75)-JVS(1032)*XX(76)
  XX(46) = XX(46)-JVS(281)*XX(48)-JVS(414)*XX(55)-JVS(473)*XX(58)-JVS(513)*XX(60)-JVS(534)*XX(61)-JVS(557)*XX(62)&
             &-JVS(582)*XX(63)-JVS(633)*XX(65)-JVS(655)*XX(66)-JVS(673)*XX(67)-JVS(716)*XX(68)-JVS(781)*XX(69)-JVS(836)&
             &*XX(70)-JVS(870)*XX(71)-JVS(896)*XX(72)-JVS(964)*XX(74)-JVS(1031)*XX(76)
  XX(45) = XX(45)-JVS(280)*XX(48)-JVS(327)*XX(50)-JVS(360)*XX(52)-JVS(383)*XX(53)-JVS(413)*XX(55)-JVS(472)*XX(58)&
             &-JVS(495)*XX(59)-JVS(512)*XX(60)-JVS(556)*XX(62)-JVS(715)*XX(68)-JVS(780)*XX(69)-JVS(835)*XX(70)-JVS(895)&
             &*XX(72)-JVS(990)*XX(75)-JVS(1030)*XX(76)
  XX(44) = XX(44)-JVS(412)*XX(55)-JVS(471)*XX(58)-JVS(511)*XX(60)-JVS(555)*XX(62)-JVS(632)*XX(65)-JVS(654)*XX(66)&
             &-JVS(672)*XX(67)-JVS(714)*XX(68)-JVS(779)*XX(69)-JVS(834)*XX(70)-JVS(869)*XX(71)-JVS(894)*XX(72)-JVS(963)&
             &*XX(74)-JVS(1029)*XX(76)
  XX(43) = XX(43)-JVS(279)*XX(48)-JVS(326)*XX(50)-JVS(349)*XX(51)-JVS(359)*XX(52)-JVS(382)*XX(53)-JVS(603)*XX(64)&
             &-JVS(713)*XX(68)-JVS(833)*XX(70)-JVS(932)*XX(73)-JVS(989)*XX(75)-JVS(1028)*XX(76)
  XX(42) = XX(42)-JVS(778)*XX(69)-JVS(832)*XX(70)
  XX(41) = XX(41)-JVS(206)*XX(42)-JVS(278)*XX(48)-JVS(309)*XX(49)-JVS(411)*XX(55)-JVS(470)*XX(58)-JVS(554)*XX(62)&
             &-JVS(671)*XX(67)-JVS(712)*XX(68)-JVS(777)*XX(69)-JVS(831)*XX(70)-JVS(893)*XX(72)-JVS(962)*XX(74)-JVS(1027)&
             &*XX(76)
  XX(40) = XX(40)-JVS(325)*XX(50)-JVS(348)*XX(51)-JVS(358)*XX(52)-JVS(381)*XX(53)-JVS(602)*XX(64)-JVS(711)*XX(68)&
             &-JVS(830)*XX(70)-JVS(931)*XX(73)-JVS(988)*XX(75)-JVS(1026)*XX(76)
  XX(39) = XX(39)-JVS(205)*XX(42)-JVS(277)*XX(48)-JVS(410)*XX(55)-JVS(447)*XX(57)-JVS(469)*XX(58)-JVS(510)*XX(60)&
             &-JVS(553)*XX(62)-JVS(601)*XX(64)-JVS(631)*XX(65)-JVS(653)*XX(66)-JVS(710)*XX(68)-JVS(776)*XX(69)-JVS(829)&
             &*XX(70)-JVS(868)*XX(71)-JVS(930)*XX(73)-JVS(1025)*XX(76)
  XX(38) = XX(38)-JVS(600)*XX(64)-JVS(709)*XX(68)-JVS(775)*XX(69)
  XX(37) = XX(37)-JVS(250)*XX(45)-JVS(276)*XX(48)-JVS(380)*XX(53)-JVS(409)*XX(55)-JVS(774)*XX(69)-JVS(828)*XX(70)&
             &-JVS(892)*XX(72)-JVS(987)*XX(75)-JVS(1024)*XX(76)
  XX(36) = XX(36)-JVS(324)*XX(50)-JVS(379)*XX(53)-JVS(599)*XX(64)-JVS(708)*XX(68)-JVS(827)*XX(70)-JVS(929)*XX(73)&
             &-JVS(986)*XX(75)-JVS(1023)*XX(76)
  XX(35) = XX(35)-JVS(308)*XX(49)-JVS(408)*XX(55)-JVS(434)*XX(56)-JVS(468)*XX(58)-JVS(552)*XX(62)-JVS(630)*XX(65)&
             &-JVS(652)*XX(66)-JVS(707)*XX(68)-JVS(773)*XX(69)-JVS(826)*XX(70)-JVS(891)*XX(72)-JVS(1022)*XX(76)
  XX(34) = XX(34)-JVS(204)*XX(42)-JVS(407)*XX(55)-JVS(467)*XX(58)-JVS(598)*XX(64)-JVS(706)*XX(68)-JVS(772)*XX(69)&
             &-JVS(928)*XX(73)
  XX(33) = XX(33)-JVS(275)*XX(48)-JVS(771)*XX(69)-JVS(1021)*XX(76)
  XX(32) = XX(32)-JVS(509)*XX(60)-JVS(551)*XX(62)-JVS(705)*XX(68)-JVS(770)*XX(69)-JVS(825)*XX(70)
  XX(31) = XX(31)-JVS(163)*XX(37)-JVS(249)*XX(45)-JVS(824)*XX(70)-JVS(1020)*XX(76)
  XX(30) = XX(30)-JVS(406)*XX(55)-JVS(597)*XX(64)-JVS(704)*XX(68)-JVS(769)*XX(69)-JVS(823)*XX(70)-JVS(985)*XX(75)&
             &-JVS(1019)*XX(76)
  XX(29) = XX(29)-JVS(446)*XX(57)-JVS(550)*XX(62)-JVS(703)*XX(68)-JVS(768)*XX(69)-JVS(867)*XX(71)-JVS(890)*XX(72)&
             &-JVS(1018)*XX(76)
  XX(28) = XX(28)-JVS(767)*XX(69)-JVS(822)*XX(70)
  XX(27) = XX(27)-JVS(262)*XX(47)-JVS(394)*XX(54)-JVS(533)*XX(61)-JVS(629)*XX(65)-JVS(702)*XX(68)-JVS(766)*XX(69)
  XX(26) = XX(26)-JVS(765)*XX(69)-JVS(821)*XX(70)-JVS(961)*XX(74)
  XX(25) = XX(25)-JVS(122)*XX(31)-JVS(274)*XX(48)-JVS(405)*XX(55)-JVS(764)*XX(69)-JVS(820)*XX(70)-JVS(889)*XX(72)
  XX(24) = XX(24)-JVS(203)*XX(42)-JVS(273)*XX(48)-JVS(323)*XX(50)-JVS(494)*XX(59)-JVS(763)*XX(69)-JVS(1017)*XX(76)
  XX(23) = XX(23)-JVS(404)*XX(55)-JVS(596)*XX(64)-JVS(701)*XX(68)-JVS(762)*XX(69)
  XX(22) = XX(22)-JVS(761)*XX(69)-JVS(984)*XX(75)-JVS(1016)*XX(76)
  XX(21) = XX(21)-JVS(403)*XX(55)-JVS(760)*XX(69)-JVS(819)*XX(70)
  XX(20) = XX(20)-JVS(759)*XX(69)-JVS(818)*XX(70)-JVS(888)*XX(72)-JVS(1015)*XX(76)
  XX(19) = XX(19)-JVS(272)*XX(48)
  XX(18) = XX(18)-JVS(758)*XX(69)-JVS(927)*XX(73)
  XX(17) = XX(17)-JVS(595)*XX(64)-JVS(700)*XX(68)-JVS(887)*XX(72)-JVS(926)*XX(73)-JVS(1014)*XX(76)
  XX(16) = XX(16)-JVS(271)*XX(48)-JVS(581)*XX(63)-JVS(757)*XX(69)-JVS(886)*XX(72)
  XX(15) = XX(15)-JVS(234)*XX(43)-JVS(248)*XX(45)-JVS(347)*XX(51)-JVS(549)*XX(62)-JVS(699)*XX(68)-JVS(756)*XX(69)&
             &-JVS(817)*XX(70)
  XX(14) = XX(14)-JVS(190)*XX(40)-JVS(247)*XX(45)-JVS(378)*XX(53)-JVS(548)*XX(62)-JVS(698)*XX(68)-JVS(755)*XX(69)&
             &-JVS(816)*XX(70)
  XX(13) = XX(13)-JVS(445)*XX(57)-JVS(547)*XX(62)-JVS(594)*XX(64)-JVS(651)*XX(66)-JVS(697)*XX(68)-JVS(866)*XX(71)&
             &-JVS(925)*XX(73)
  XX(12) = XX(12)-JVS(146)*XX(34)-JVS(508)*XX(60)-JVS(546)*XX(62)-JVS(628)*XX(65)-JVS(650)*XX(66)-JVS(696)*XX(68)&
             &-JVS(754)*XX(69)
  XX(11) = XX(11)-JVS(307)*XX(49)-JVS(402)*XX(55)-JVS(444)*XX(57)-JVS(545)*XX(62)-JVS(695)*XX(68)-JVS(753)*XX(69)&
             &-JVS(815)*XX(70)
  XX(10) = XX(10)-JVS(261)*XX(47)-JVS(694)*XX(68)-JVS(752)*XX(69)
  XX(9) = XX(9)-JVS(156)*XX(36)-JVS(246)*XX(45)-JVS(377)*XX(53)-JVS(693)*XX(68)-JVS(751)*XX(69)-JVS(814)*XX(70)
  XX(8) = XX(8)-JVS(270)*XX(48)-JVS(885)*XX(72)-JVS(1013)*XX(76)
  XX(7) = XX(7)-JVS(102)*XX(28)-JVS(202)*XX(42)-JVS(322)*XX(50)-JVS(750)*XX(69)-JVS(813)*XX(70)
  XX(6) = XX(6)-JVS(443)*XX(57)-JVS(507)*XX(60)-JVS(544)*XX(62)-JVS(692)*XX(68)-JVS(749)*XX(69)
  XX(5) = XX(5)-JVS(593)*XX(64)-JVS(691)*XX(68)-JVS(748)*XX(69)
  XX(4) = XX(4)-JVS(747)*XX(69)-JVS(960)*XX(74)
  XX(3) = XX(3)-JVS(746)*XX(69)-JVS(812)*XX(70)
  XX(2) = XX(2)
  XX(1) = XX(1)
      
END SUBROUTINE KppSolveTR

! End of KppSolveTR function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! BLAS_UTIL - BLAS-LIKE utility functions
!   Arguments :
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!--------------------------------------------------------------
!
! BLAS/LAPACK-like subroutines used by the integration algorithms
! It is recommended to replace them by calls to the optimized
!      BLAS/LAPACK library for your machine
!
!  (C) Adrian Sandu, Aug. 2004
!      Virginia Polytechnic Institute and State University
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE WCOPY(N,X,incX,Y,incY)
!--------------------------------------------------------------
!     copies a vector, x, to a vector, y:  y <- x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL  SCOPY(N,X,1,Y,1)   or   CALL  DCOPY(N,X,1,Y,1)
!--------------------------------------------------------------
!     USE cb6_Precision
      
      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N)

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = X(i)
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i) = X(i)
        Y(i + 1) = X(i + 1)
        Y(i + 2) = X(i + 2)
        Y(i + 3) = X(i + 3)
        Y(i + 4) = X(i + 4)
        Y(i + 5) = X(i + 5)
        Y(i + 6) = X(i + 6)
        Y(i + 7) = X(i + 7)
      END DO

      END SUBROUTINE WCOPY


!--------------------------------------------------------------
      SUBROUTINE WAXPY(N,Alpha,X,incX,Y,incY)
!--------------------------------------------------------------
!     constant times a vector plus a vector: y <- y + Alpha*x
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SAXPY(N,Alpha,X,1,Y,1) or  CALL DAXPY(N,Alpha,X,1,Y,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,incY,M,MP1,N
      REAL(kind=dp) :: X(N),Y(N),Alpha
      REAL(kind=dp), PARAMETER :: ZERO = 0.0_dp

      IF (Alpha .EQ. ZERO) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,4)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = Y(i) + Alpha*X(i)
        END DO
        IF( N .LT. 4 ) RETURN
      END IF
      MP1 = M + 1
      DO i = MP1,N,4
        Y(i) = Y(i) + Alpha*X(i)
        Y(i + 1) = Y(i + 1) + Alpha*X(i + 1)
        Y(i + 2) = Y(i + 2) + Alpha*X(i + 2)
        Y(i + 3) = Y(i + 3) + Alpha*X(i + 3)
      END DO
      
      END SUBROUTINE WAXPY



!--------------------------------------------------------------
      SUBROUTINE WSCAL(N,Alpha,X,incX)
!--------------------------------------------------------------
!     constant times a vector: x(1:N) <- Alpha*x(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SSCAL(N,Alpha,X,1) or  CALL DSCAL(N,Alpha,X,1)
!--------------------------------------------------------------

      INTEGER  :: i,incX,M,MP1,N
      REAL(kind=dp)  :: X(N),Alpha
      REAL(kind=dp), PARAMETER  :: ZERO=0.0_dp, ONE=1.0_dp

      IF (Alpha .EQ. ONE) RETURN
      IF (N .LE. 0) RETURN

      M = MOD(N,5)
      IF( M .NE. 0 ) THEN
        IF (Alpha .EQ. (-ONE)) THEN
          DO i = 1,M
            X(i) = -X(i)
          END DO
        ELSEIF (Alpha .EQ. ZERO) THEN
          DO i = 1,M
            X(i) = ZERO
          END DO
        ELSE
          DO i = 1,M
            X(i) = Alpha*X(i)
          END DO
        END IF
        IF( N .LT. 5 ) RETURN
      END IF
      MP1 = M + 1
      IF (Alpha .EQ. (-ONE)) THEN
        DO i = MP1,N,5
          X(i)     = -X(i)
          X(i + 1) = -X(i + 1)
          X(i + 2) = -X(i + 2)
          X(i + 3) = -X(i + 3)
          X(i + 4) = -X(i + 4)
        END DO
      ELSEIF (Alpha .EQ. ZERO) THEN
        DO i = MP1,N,5
          X(i)     = ZERO
          X(i + 1) = ZERO
          X(i + 2) = ZERO
          X(i + 3) = ZERO
          X(i + 4) = ZERO
        END DO
      ELSE
        DO i = MP1,N,5
          X(i)     = Alpha*X(i)
          X(i + 1) = Alpha*X(i + 1)
          X(i + 2) = Alpha*X(i + 2)
          X(i + 3) = Alpha*X(i + 3)
          X(i + 4) = Alpha*X(i + 4)
        END DO
      END IF

      END SUBROUTINE WSCAL

!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WLAMCH( C )
!--------------------------------------------------------------
!     returns epsilon machine
!     after LAPACK
!     replace this by the function from the optimized LAPACK implementation:
!          CALL SLAMCH('E') or CALL DLAMCH('E')
!--------------------------------------------------------------
!      USE cb6_Precision

      CHARACTER ::  C
      INTEGER    :: i
      REAL(kind=dp), SAVE  ::  Eps
      REAL(kind=dp)  ::  Suma
      REAL(kind=dp), PARAMETER  ::  ONE=1.0_dp, HALF=0.5_dp
      LOGICAL, SAVE   ::  First=.TRUE.
      
      IF (First) THEN
        First = .FALSE.
        Eps = HALF**(16)
        DO i = 17, 80
          Eps = Eps*HALF
          CALL WLAMCH_ADD(ONE,Eps,Suma)
          IF (Suma.LE.ONE) GOTO 10
        END DO
        PRINT*,'ERROR IN WLAMCH. EPS < ',Eps
        RETURN
10      Eps = Eps*2
        i = i-1      
      END IF

      WLAMCH = Eps

      END FUNCTION WLAMCH
     
      SUBROUTINE WLAMCH_ADD( A, B, Suma )
!      USE cb6_Precision
      
      REAL(kind=dp) A, B, Suma
      Suma = A + B

      END SUBROUTINE WLAMCH_ADD
!--------------------------------------------------------------


!--------------------------------------------------------------
      SUBROUTINE SET2ZERO(N,Y)
!--------------------------------------------------------------
!     copies zeros into the vector y:  y <- 0
!     after BLAS
!--------------------------------------------------------------
      
      INTEGER ::  i,M,MP1,N
      REAL(kind=dp) ::  Y(N)
      REAL(kind=dp), PARAMETER :: ZERO = 0.0d0

      IF (N.LE.0) RETURN

      M = MOD(N,8)
      IF( M .NE. 0 ) THEN
        DO i = 1,M
          Y(i) = ZERO
        END DO
        IF( N .LT. 8 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,8
        Y(i)     = ZERO
        Y(i + 1) = ZERO
        Y(i + 2) = ZERO
        Y(i + 3) = ZERO
        Y(i + 4) = ZERO
        Y(i + 5) = ZERO
        Y(i + 6) = ZERO
        Y(i + 7) = ZERO
      END DO

      END SUBROUTINE SET2ZERO


!--------------------------------------------------------------
      REAL(kind=dp) FUNCTION WDOT (N, DX, incX, DY, incY) 
!--------------------------------------------------------------
!     dot produce: wdot = x(1:N)*y(1:N) 
!     only for incX=incY=1
!     after BLAS
!     replace this by the function from the optimized BLAS implementation:
!         CALL SDOT(N,X,1,Y,1) or  CALL DDOT(N,X,1,Y,1)
!--------------------------------------------------------------
!      USE messy_mecca_kpp_Precision
!--------------------------------------------------------------
      IMPLICIT NONE
      INTEGER :: N, incX, incY
      REAL(kind=dp) :: DX(N), DY(N) 

      INTEGER :: i, IX, IY, M, MP1, NS
                                 
      WDOT = 0.0D0 
      IF (N .LE. 0) RETURN 
      IF (incX .EQ. incY) IF (incX-1) 5,20,60 
!                                                                       
!     Code for unequal or nonpositive increments.                       
!                                                                       
    5 IX = 1 
      IY = 1 
      IF (incX .LT. 0) IX = (-N+1)*incX + 1 
      IF (incY .LT. 0) IY = (-N+1)*incY + 1 
      DO i = 1,N 
        WDOT = WDOT + DX(IX)*DY(IY) 
        IX = IX + incX 
        IY = IY + incY 
      END DO 
      RETURN 
!                                                                       
!     Code for both increments equal to 1.                              
!                                                                       
!     Clean-up loop so remaining vector length is a multiple of 5.      
!                                                                       
   20 M = MOD(N,5) 
      IF (M .EQ. 0) GO TO 40 
      DO i = 1,M 
         WDOT = WDOT + DX(i)*DY(i) 
      END DO 
      IF (N .LT. 5) RETURN 
   40 MP1 = M + 1 
      DO i = MP1,N,5 
          WDOT = WDOT + DX(i)*DY(i) + DX(i+1)*DY(i+1) + DX(i+2)*DY(i+2) +  &
                   DX(i+3)*DY(i+3) + DX(i+4)*DY(i+4)                   
      END DO 
      RETURN 
!                                                                       
!     Code for equal, positive, non-unit increments.                    
!                                                                       
   60 NS = N*incX 
      DO i = 1,NS,incX 
        WDOT = WDOT + DX(i)*DY(i) 
      END DO 

      END FUNCTION WDOT                                          


!--------------------------------------------------------------
      SUBROUTINE WADD(N,X,Y,Z)
!--------------------------------------------------------------
!     adds two vectors: z <- x + y
!     BLAS - like
!--------------------------------------------------------------
!     USE cb6_Precision
      
      INTEGER :: i, M, MP1, N
      REAL(kind=dp) :: X(N),Y(N),Z(N)

      IF (N.LE.0) RETURN

      M = MOD(N,5)
      IF( M /= 0 ) THEN
         DO i = 1,M
            Z(i) = X(i) + Y(i)
         END DO
         IF( N < 5 ) RETURN
      END IF    
      MP1 = M+1
      DO i = MP1,N,5
         Z(i)     = X(i)     + Y(i)
         Z(i + 1) = X(i + 1) + Y(i + 1)
         Z(i + 2) = X(i + 2) + Y(i + 2)
         Z(i + 3) = X(i + 3) + Y(i + 3)
         Z(i + 4) = X(i + 4) + Y(i + 4)
      END DO

      END SUBROUTINE WADD
      
      
      
!--------------------------------------------------------------
      SUBROUTINE WGEFA(N,A,Ipvt,info)
!--------------------------------------------------------------
!     WGEFA FACTORS THE MATRIX A (N,N) BY
!           GAUSS ELIMINATION WITH PARTIAL PIVOTING
!     LINPACK - LIKE 
!--------------------------------------------------------------
!
      INTEGER       :: N,Ipvt(N),info
      REAL(kind=dp) :: A(N,N)
      REAL(kind=dp) :: t, dmax, da
      INTEGER       :: j,k,l
      REAL(kind=dp), PARAMETER :: ZERO = 0.0, ONE = 1.0

      info = 0

size: IF (n > 1) THEN
      
col:  DO k = 1, n-1

!        find l = pivot index
!        l = idamax(n-k+1,A(k,k),1) + k - 1
         l = k; dmax = abs(A(k,k))
         DO j = k+1,n
            da = ABS(A(j,k))
            IF (da > dmax) THEN
              l = j; dmax = da
            END IF
         END DO
         Ipvt(k) = l

!        zero pivot implies this column already triangularized
         IF (ABS(A(l,k)) < TINY(ZERO)) THEN
            info = k
            return
         ELSE   
            IF (l /= k) THEN
               t = A(l,k); A(l,k) = A(k,k); A(k,k) = t
            END IF
            t = -ONE/A(k,k)
            CALL WSCAL(n-k,t,A(k+1,k),1)
            DO j = k+1, n
               t = A(l,j)
               IF (l /= k) THEN
                  A(l,j) = A(k,j); A(k,j) = t
               END IF
               CALL WAXPY(n-k,t,A(k+1,k),1,A(k+1,j),1)
            END DO         
         END IF
         
       END DO col
       
      END IF size
      
      Ipvt(N) = N
      IF (ABS(A(N,N)) == ZERO) info = N
      
      END SUBROUTINE WGEFA


!--------------------------------------------------------------
      SUBROUTINE WGESL(Trans,N,A,Ipvt,b)
!--------------------------------------------------------------
!     WGESL solves the system
!     a * x = b  or  trans(a) * x = b
!     using the factors computed by WGEFA.
!
!     Trans      = 'N'   to solve  A*x = b ,
!                = 'T'   to solve  transpose(A)*x = b
!     LINPACK - LIKE 
!--------------------------------------------------------------

      INTEGER       :: N,Ipvt(N)
      CHARACTER     :: trans
      REAL(kind=dp) :: A(N,N),b(N)
      REAL(kind=dp) :: t
      INTEGER       :: k,kb,l

      
      SELECT CASE (Trans)

      CASE ('n','N')  !  Solve  A * x = b

!        first solve  L*y = b
         IF (n >= 2) THEN
          DO k = 1, n-1
            l = Ipvt(k)
            t = b(l)
            IF (l /= k) THEN
               b(l) = b(k)
               b(k) = t
            END IF
            CALL WAXPY(n-k,t,a(k+1,k),1,b(k+1),1)
          END DO
         END IF
!        now solve  U*x = y
         DO kb = 1, n
            k = n + 1 - kb
            b(k) = b(k)/a(k,k)
            t = -b(k)
            CALL WAXPY(k-1,t,a(1,k),1,b(1),1)
         END DO
      
      CASE ('t','T')  !  Solve transpose(A) * x = b

!        first solve  trans(U)*y = b
         DO k = 1, n
            t = WDOT(k-1,a(1,k),1,b(1),1)
            b(k) = (b(k) - t)/a(k,k)
         END DO
!        now solve trans(L)*x = y
         IF (n >= 2) THEN
         DO kb = 1, n-1
            k = n - kb
            b(k) = b(k) + WDOT(n-k,a(k+1,k),1,b(k+1),1)
            l = Ipvt(k)
            IF (l /= k) THEN
               t = b(l); b(l) = b(k); b(k) = t
            END IF
         END DO
         END IF
   
      END SELECT

      END SUBROUTINE WGESL
! End of BLAS_UTIL function
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



END MODULE mod_cb6_LinearAlgebra

