! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! 
! The Reaction Rates File
! 
! Generated by KPP-2.2.3 symbolic chemistry Kinetics PreProcessor
!       (http://www.cs.vt.edu/~asandu/Software/KPP)
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian, Villanova University, USA
!        R. Sander, Max-Planck Institute for Chemistry, Mainz, Germany
! 
! File                 : cb6_Rates.f90
! Time                 : Mon Sep 15 11:56:13 2014
! Working directory    : /home/regcm/Code/kpp-2.2.3/Workspace/CB6r2
! Equation file        : cb6.kpp
! Output root filename : cb6
! 
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



MODULE mod_cb6_Rates

  USE mod_cb6_Parameters
  USE mod_cb6_Global
  IMPLICIT NONE

  CONTAINS

   !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
   ! User-defined Rate Law functions                                           !
   ! Note: the default argument type for rate laws, as read from the equations !
   ! file, is single precision but all the internal calculations are performed !
   ! in double precision                                                       !
   !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
   ! Rate functions included by J. Ciarlo` (June 2016)
   ! General Temperature Dependence (no exponent)
   real(kind=dp) function gtde0(a0,t0,p0)
     real(kind=dp) a0
     real t0,p0
     gtde0 = a0 * (tempb/dble(t0))**dble(p0)
   end function gtde0 

   ! General Temperature Dependence (no power)
   real(kind=dp) function gtdp0(a0,ea)
     real(kind=dp) a0
     real ea
     gtdp0 = a0 * exp(dble(ea)/tempb)
   end function gtdp0

   ! General Temperature Dependence 
   real(kind=dp) function gtd(a0,t0,p0,ea)
     real(kind=dp) a0
     real t0,p0,ea
     gtd = a0 * (tempb/dble(t0))**dble(p0) * exp(dble(ea)/tempb)
   end function gtd

   ! Troe-type temp and pres denendence (falloff)
   real(kind=dp) function fall(k0,ki,n,f)
     real(kind=dp) k0,ki,g,cb
     real n,f
     real(kind=dp),parameter :: av=6.02214129D23
     cb = c_mb/av
     g = log10(k0*cb/ki)/dble(n)
     g = 1 + g**2
     g = 1/g
     fall = (k0*cb/(1+(k0*cb/ki))) * dble(f)**g
   end function fall

   ! Simple pressure dependence
   real(kind=dp) function spd(k1,k2)
     real(kind=dp) k1,k2,cb
     real(kind=dp),parameter :: av=6.02214129D23
     cb = c_mb/av
     spd = k1 + k2*cb
   end function spd

   ! Lindemann - Hinshelwood
   real(kind=dp) function lihi(k1,k2,k3)
     real(kind=dp) k1,k2,k3,cb
     real(kind=dp),parameter :: av=6.02214129D23
     cb = c_mb/av
     lihi = k1 + k3*cb/(1 + k3*cb/k2)
   end function lihi

  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  !  End of User-defined Rate Law functions
  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ! 
  ! Update_SUN - update SUN light using TIME
  !   Arguments :
  ! 
  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  SUBROUTINE Update_SUN()
      !USE cb6_Parameters
      !USE cb6_Global

    IMPLICIT NONE

    REAL(kind=dp) :: SunRise, SunSet
    REAL(kind=dp) :: Thour, Tlocal, Ttmp 
    ! PI - Value of pi
    REAL(kind=dp), PARAMETER :: PI = 3.14159265358979d0
    
    SunRise = 4.5_dp 
    SunSet  = 19.5_dp 
    Thour = TIMEB/3600.0_dp 
    Tlocal = Thour - (INT(Thour)/24)*24

    IF ((Tlocal>=SunRise).AND.(Tlocal<=SunSet)) THEN
       Ttmp = (2.0*Tlocal-SunRise-SunSet)/(SunSet-SunRise)
       IF (Ttmp.GT.0) THEN
          Ttmp =  Ttmp*Ttmp
       ELSE
          Ttmp = -Ttmp*Ttmp
       END IF
       SUN = ( 1.0_dp + COS(PI*Ttmp) )/2.0_dp 
    ELSE
       SUN = 0.0_dp 
    END IF

  END SUBROUTINE Update_SUN

  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ! 
  ! Update_rconst - function to update rate constants
  !   Arguments :
  ! 
  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  SUBROUTINE Update_rconst ( )
    real(kind=dp) kk1,kk2,kk3
    integer i
    kk1 = 0.0D0
    kk2 = 0.0D0
    kk3 = 0.0D0

    ! constant rate coefficients commented
    rconst(1) = (jval_NDOX)
    rconst(2) = (gtde0(5.68D-34,300.0,-2.6))
    rconst(3) = (gtdp0(1.40D-12,-1310.0))
    rconst(4) = (gtde0(1.00D-31,300.0,-1.6))
    rconst(5) = (gtdp0(5.50D-12,188.0))
    kk1 = gtde0(1.30D-31,300.0,-1.5)
    kk2 = gtde0(2.30D-11,300.0,0.24)
    rconst(6) = (fall(kk1,kk2,1.0,0.6))
    rconst(7) = (gtdp0(8.00D-12,-2060.0))
    rconst(8) = (jval_O33P) 
    rconst(9) = (jval_O31D) 
    rconst(10) = (gtdp0(2.23D-11,115.0))
!   rconst(11) = (dble(2.14E-10))
    rconst(12) = (gtdp0(1.70D-12,-940.0))
    rconst(13) = (gtd(2.03D-16,300.0,4.57,693.0))
    rconst(14) = (gtdp0(2.40D-11,110.0))
    rconst(15) = (gtdp0(2.70D-11,224.0))
    rconst(16) = (gtd(6.20D-14,298.0,2.6,945.0))
    kk1 = gtde0(6.90D-31,300.0,-0.8)
    rconst(17) = (fall(kk1,(2.60D-11),1.13,0.5))
    rconst(18) = (gtdp0(4.80D-11,250.0))
    kk1 = gtdp0(2.20D-13,600.0)
    kk2 = gtdp0(1.90D-33,980.0)
    rconst(19) = (spd(kk1,kk2))
    kk1 = gtdp0(3.08D-34,2800.0)
    kk2 = gtdp0(2.66D-54,3180.0)
    rconst(20) = (spd(kk1,kk2))
    rconst(21) = (jval_HPOX)
    rconst(22) = (gtdp0(2.90D-12,-160.0))
    rconst(23) = (gtdp0(1.40D-12,-2000.0))
    rconst(24) = (gtdp0(3.30D-39,530.0))
    rconst(25) = (gtdp0(3.45D-12,270.0))
    rconst(26) = (gtdp0(1.40D-13,-2470.0))
    rconst(27) = (jval_NTOXb)  
    rconst(28) = (jval_NTOXa)  
    rconst(29) = (gtdp0(1.80D-11,110.0))
    rconst(30) = (gtdp0(4.50D-14,-1260.0))
!   rconst(31) = (dble(1.70E-11))
!   rconst(32) = (dble(2.00E-11))
!   rconst(33) = (dble(4.00E-12))
!   rconst(34) = (dble(1.00E-17))
    rconst(35) = (gtdp0(8.50D-13,-2450.0))
    kk1 = gtde0(3.60D-30,300.0,-4.1)
    kk2 = gtde0(1.90D-12,300.0,0.2)
    rconst(36) = (fall(kk1,kk2,1.33,0.35))
    kk1 = gtd(1.30D-3,300.0,-3.5,-11000.0)
    kk2 = gtd(9.70D+14,300.0,0.1,-11080.0)
    rconst(37) = (fall(kk1,kk2,1.33,0.35))
    rconst(38) = (jval_DNPOb)
!   rconst(39) = (dble(1.00E-22))
    kk1 = gtde0(7.40D-31,300.0,-2.4)
    kk2 = gtde0(3.30D-11,300.0,-0.3)
    rconst(40) = (fall(kk1,kk2,0.87,0.81))
!   rconst(41) = (dble(5.00E-40))
!   rconst(42) = (dble(1.00E-20))
    rconst(43) = (jval_HONO)
    rconst(44) = (gtdp0(2.50D-12,260.0))
    kk1 = gtde0(1.80D-30,300.0,-3.0)
    rconst(45) = (fall(kk1,(2.80D-11),1.0,0.6))
    kk1 = gtdp0(2.40D-14,460.0)
    kk2 = gtdp0(2.70D-17,2199.0)
    kk3 = gtdp0(6.50D-34,1335.0)
    rconst(46) = (lihi(kk1,kk2,kk3))
    rconst(47) = (jval_NTRC)
    kk1 = gtde0(1.80D-31,300.0,-3.2)
    rconst(48) = (fall(kk1,(4.70D-12),1.0,0.6))
    kk1 = gtdp0(4.10D-5,-10650.0)
    kk2 = gtdp0(4.80D+15,-11170.0)
    rconst(49) = (fall(kk1,kk2,1.0,0.6))
    rconst(50) = (jval_PNA)
    rconst(51) = (gtdp0(3.20D-13,690.0))
    kk1 = gtde0(4.50D-31,300.0,-3.9)
    kk2 = gtde0(1.30D-12,300.0,-0.7)
    rconst(52) = (fall(kk1,kk2,1.1,0.53))
    rconst(53) = (gtdp0(7.50D-12,290.0))
    kk1 = gtde0(2.70D-28,300.0,-7.1)
    kk2 = gtde0(1.20D-11,300.0,-0.9)
    rconst(54) = (fall(kk1,kk2,1.41,0.3))
    kk1 = gtdp0(4.90D-3,-12100.0)
    kk2 = gtdp0(5.40D+16,-13830.0)
    rconst(55) = (fall(kk1,kk2,1.41,0.3))
    rconst(56) = (jval_PACN)
    rconst(57) = (gtdp0(5.20D-13,980.0))
    rconst(58) = (gtdp0(8.90D-13,800.0))
    rconst(59) = (gtdp0(2.90D-12,500.0))
    rconst(60) = (gtdp0(2.90D-12,500.0))
    rconst(61) = (gtdp0(6.70D-12,340.0))
    rconst(62) = (rconst(54))
    rconst(63) = (rconst(55))
    rconst(64) = (jval_PANX)
    rconst(65) = (gtdp0(5.20D-13,980.0))
    rconst(66) = (gtdp0(8.90D-13,800.0))
    rconst(67) = (gtdp0(3.20D-12,500.0))
    rconst(68) = (gtdp0(2.40D-12,360.0))
    rconst(69) = (gtdp0(4.80D-13,800.0))
    rconst(70) = (gtdp0(6.50D-14,500.0))
    rconst(71) = (gtdp0(2.30D-12,360.0))
    rconst(72) = (gtdp0(3.80D-13,780.0))
    rconst(73) = (gtdp0(2.00D-12,500.0))
    rconst(74) = (rconst(70))
    rconst(75) = (gtdp0(2.70D-12,360.0))
    rconst(76) = (gtdp0(6.80D-13,800.0))
    rconst(77) = (rconst(58))
    rconst(78) = (rconst(70))
    rconst(79) = (rconst(75))
    rconst(80) = (rconst(76))
    rconst(81) = (rconst(58))
    rconst(82) = (rconst(70))
    rconst(83) = (rconst(75))
    rconst(84) = (rconst(76))
    rconst(85) = (rconst(58))
    rconst(86) = (rconst(70))
    rconst(87) = (gtdp0(5.30D-12,190.0))
    rconst(88) = (jval_MEPX)
    rconst(89) = (gtdp0(5.30D-12,190.0))
    rconst(90) = (jval_RPOX)
!   rconst(91) = (dble(2.00E-12))
    rconst(92) = (jval_NTR)
!   rconst(93) = (dble(4.50E-13))
    rconst(94) = (gtdp0(4.00D-14,850.0))
    rconst(95) = (gtdp0(5.30D-12,190.0))
    rconst(96) = (gtdp0(5.40D-12,135.0))
    rconst(97) = (jval_FORM)
    rconst(98) = (jval_FORM)
    rconst(99) = (gtdp0(3.40D-11,-1600.0))
!   rconst(100) = (dble(5.50E-16))
    rconst(101) = (gtdp0(9.70D-15,625.0))
    rconst(102) = (gtdp0(2.40D+12,-7000.0))
!   rconst(103) = (dble(5.60E-12))
    rconst(104) = (gtdp0(5.60D-15,2300.0))
    rconst(105) = (gtdp0(1.80D-11,-1100.0))
    rconst(106) = (gtdp0(4.70D-12,345.0))
    rconst(107) = (gtdp0(1.40D-12,-1860.0))
    rconst(108) = (jval_AALD)
    rconst(109) = (gtdp0(1.30D-11,-870.0))
    rconst(110) = (gtdp0(4.90D-12,405.0))
!   rconst(111) = (dble(6.30E-15))
    rconst(112) = (jval_ALDX)
!   rconst(113) = (dble(8.00E-12))
    rconst(114) = (jval_GLYD)
    rconst(115) = (gtdp0(1.40D-12,-1860.0))
    rconst(116) = (gtdp0(3.10D-12,340.0))
    rconst(117) = (jval_GLY)
    rconst(118) = (gtdp0(1.40D-12,-1860.0))
    rconst(119) = (jval_MEGY)
    rconst(120) = (gtdp0(1.40D-12,-1860.0))
    rconst(121) = (gtdp0(1.90D-12,575.0))
    rconst(122) = (gtdp0(7.70D-12,-2100.0))
    rconst(123) = (spd((1.44D-13),(3.43D-33)))
    rconst(124) = (gtdp0(1.85D-12,-1690.0))
    rconst(125) = (gtdp0(6.90D-12,-1000.0))
    rconst(126) = (gtdp0(2.85D-12,-345.0))
    rconst(127) = (gtdp0(3.00D-12,20.0))
    rconst(128) = (jval_KET)
    rconst(129) = (jval_ACET)
    rconst(130) = (gtdp0(1.41D-12,-620.6))
    rconst(131) = (gtdp0(7.60D-12,-585.0))
!   rconst(132) = (dble(8.10E-13))
    rconst(133) = (gtdp0(5.70D+12,-5780.0))
    rconst(134) = (gtdp0(1.50D-14,-200.0))
    rconst(135) = (gtdp0(8.60D-12,400.0))
    kk1 = gtde0(5.00D-30,300.0,-1.5)
    rconst(136) = (fall(kk1,(1.00D-12),1.3,0.37))
    rconst(137) = (gtdp0(1.04D-11,-792.0))
    kk1 = gtde0(8.60D-29,300.0,-3.1)
    kk2 = gtde0(9.00D-12,300.0,-0.85)
    rconst(138) = (fall(kk1,kk2,1.15,0.48))
    rconst(139) = (gtdp0(9.10D-15,-2580.0))
    rconst(140) = (gtdp0(3.30D-12,-2880.0))
    rconst(141) = (gtdp0(1.00D-11,-280.0))
    kk1 = gtde0(8.00D-27,300.0,-3.5)
    kk2 = gtde0(3.00D-11,300.0,-1.0)
    rconst(142) = (fall(kk1,kk2,1.13,0.5))
    rconst(143) = (gtdp0(5.50D-15,-1880.0))
    rconst(144) = (gtdp0(4.60D-13,-1155.0))
!   rconst(145) = (dble(2.30E-11))
    rconst(146) = (gtdp0(1.05D-11,519.0))
    rconst(147) = (gtdp0(4.70D-15,-1013.0))
!   rconst(148) = (dble(3.70E-13))
    rconst(149) = (gtdp0(2.70D-11,390.0))
!   rconst(150) = (dble(3.00E-11))
    rconst(151) = (gtdp0(2.39D-12,365.0))
    rconst(152) = (gtdp0(7.43D-13,700.0))
    rconst(153) = (rconst(58))
    rconst(154) = (rconst(70))
    rconst(155) = (gtdp0(3.30D+9,-8300.0))
    rconst(156) = (gtdp0(1.03D-14,-1995.0))
    rconst(157) = (gtdp0(3.03D-12,-448.0))
    rconst(158) = (gtdp0(5.58D-12,511.0))
    rconst(159) = (gtdp0(3.88D-15,-1770.0))
    rconst(160) = (gtdp0(4.10D-12,-1860.0))
    rconst(161) = (jval_ISPD)
    rconst(162) = (gtdp0(2.23D-11,372.0))
    rconst(163) = (jval_HPLD)
    rconst(164) = (gtdp0(6.00D-12,-1860.0))
    rconst(165) = (gtdp0(5.78D-11,-400.0))
    rconst(166) = (gtdp0(7.43D-13,700.0))
    rconst(167) = (gtdp0(2.39D-12,365.0))
    rconst(168) = (rconst(58))
    rconst(169) = (rconst(70))
!   rconst(170) = (dble(3.10E-11))
!   rconst(171) = (dble(3.60E-11))
    rconst(172) = (gtdp0(1.50D-11,449.0))
    rconst(173) = (gtdp0(1.20D-15,-821.0))
    rconst(174) = (gtdp0(3.70D-12,175.0))
    rconst(175) = (gtdp0(2.30D-12,-190.0))
    rconst(176) = (gtdp0(2.70D-12,360.0))
    rconst(177) = (rconst(58))
    rconst(178) = (gtdp0(1.90D-13,1300.0))
    rconst(179) = (rconst(70))
    rconst(180) = (gtdp0(1.80D-12,340.0))
    rconst(181) = (gtdp0(2.70D-12,360.0))
    rconst(182) = (rconst(58))
    rconst(183) = (gtdp0(1.90D-13,1300.0))
    rconst(184) = (rconst(70))
!   rconst(185) = (dble(1.85E-11))
    rconst(186) = (gtdp0(2.70D-12,360.0))
    rconst(187) = (gtdp0(1.90D-13,1300.0))
    rconst(188) = (rconst(58))
    rconst(189) = (rconst(70))
    rconst(190) = (gtdp0(1.70D-12,950.0))
!   rconst(191) = (dble(1.40E-11))
!   rconst(192) = (dble(2.10E-12))
!   rconst(193) = (dble(5.50E-12))
!   rconst(194) = (dble(1.53E-12))
!   rconst(195) = (dble(3.80E-12))
    rconst(196) = (jval_CRON)
    rconst(197) = (jval_XOPN)
!   rconst(198) = (dble(9.00E-11))
    rconst(199) = (gtdp0(1.08D-16,-500.0))
!   rconst(200) = (dble(3.00E-12))
    rconst(201) = (jval_ROPN)
!   rconst(202) = (dble(4.40E-11))
    rconst(203) = (gtdp0(5.40D-17,-500.0))
!   rconst(204) = (dble(3.80E-12))
!   rconst(205) = (dble(5.00E-11))
!   rconst(206) = (dble(1.70E-10))
!   rconst(207) = (dble(1.00E-11))
    rconst(208) = (rconst(54))
    rconst(209) = (rconst(55))
    rconst(210) = (rconst(57))
    rconst(211) = (rconst(59))
    rconst(212) = (rconst(58))
!   rconst(213) = (dble(3.60E-11))
!   rconst(214) = (dble(3.00E-12))
!   rconst(215) = (dble(2.30E-05))
    rconst(216) = (gtdp0(1.85D-12,-1690.0))

  END SUBROUTINE Update_rconst

  ! End of Update_rconst function
  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ! Update_PHOTO - function to update photolytical rate constants
  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  SUBROUTINE Update_PHOTO ( )

    USE mod_cb6_Global

    rconst(1) = (jval_NDOX)
    rconst(8) = (jval_O33P)
    rconst(9) = (jval_O31D)
    rconst(21) = (jval_HPOX)
    rconst(27) = (jval_NTOXa)
    rconst(28) = (jval_NTOXb)
    rconst(38) = (jval_DNPOb)
    rconst(43) = (jval_HONO)
    rconst(47) = (jval_NTRC)
    rconst(50) = (jval_PNA)
    rconst(56) = (jval_PACN)
    rconst(64) = (jval_PANX)
    rconst(88) = (jval_MEPX)
    rconst(90) = (jval_RPOX)
    rconst(92) = (jval_NTR)
    rconst(97) = (jval_FORM)
    rconst(98) = (jval_FORM)
    rconst(108) = (jval_AALD)
    rconst(112) = (jval_ALDX)
    rconst(114) = (jval_GLYD)
    rconst(117) = (jval_GLY)
    rconst(119) = (jval_MEGY)
    rconst(128) = (jval_KET)
    rconst(129) = (jval_ACET)
    rconst(161) = (jval_ISPD)
    rconst(163) = (jval_HPLD)
    rconst(196) = (jval_CRON)
    rconst(197) = (jval_XOPN)
    rconst(201) = (jval_ROPN)
      
  END SUBROUTINE Update_PHOTO

  ! End of Update_PHOTO function
  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

END MODULE mod_cb6_Rates
