! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Numerical Integrator (Time-Stepping) File
!
! Generated by KPP-2.2.3 symbolic chemistry Kinetics PreProcessor
!       (http://www.cs.vt.edu/~asandu/Software/KPP)
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian, Villanova University, USA
!        R. Sander, Max-Planck Institute for Chemistry, Mainz, Germany
!
! File                 : mod_cbmz_Integrator.f90
! Time                 : Mon Nov 25 13:41:15 2013
! Working directory    : /scratch/ashalaby/kpp-2.2.3/compare/CBMZ
! Equation file        : mod_cbmz.kpp
! Output root filename : mod_cbmz
!
!  LSODE - Stiff method based on backward differentiation formulas (BDF)  !
!  By default the code employs the KPP sparse linear algebra routines     !
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
! A. Sandu - version of July 2005

module mod_cbmz_integrator

  use mod_cbmz_precision
  use mod_cbmz_global , only : fix , rconst , time , atol , rtol
  use mod_cbmz_parameters , only : nvar , nfix , lu_nonzero
  use mod_cbmz_jacobiansp , only : lu_diag
  use mod_cbmz_linearalgebra , only : kppdecomp , kppsolve

  implicit none
  public
  save

  real(kind=dp) , private :: conit , crate , hold , rmax
  real(kind=dp) , private , dimension(13) :: el
  real(kind=dp) , private , dimension(13,12) :: elco
  real(kind=dp) , private , dimension(3,12) :: tesco
  real(kind=dp) , private :: ccmax , el0 , h , hmin , hmxi , hu , rc , tn , uround

  integer , private :: init , mxstep , mxhnil , nhnil , nslast , nyh
  integer , private :: ialth , ipup , lmax , meo , nqnyh , nslp
  integer , private :: icf, ierpj , iersl , jcur , jstart , kflag , l
  integer , private :: lyh , lewt , lacor , lsavf , lwm , liwm , meth , miter
  integer , private :: maxord , maxcor , msbp , mxncf , n , nq , nst , nfe , nje , nqu

  !~~~>  statistics on the work performed by the lsode method
  integer , private :: nfun , njac , nstp , nacc , nrej , ndec , nsol , nsng
  integer , parameter :: ifun = 1
  integer , parameter :: ijac = 2
  integer , parameter :: istp = 3
  integer , parameter :: iacc = 4
  integer , parameter :: irej = 5
  integer , parameter :: idec = 6
  integer , parameter :: isol = 7
  integer , parameter :: isng = 8

  integer , parameter :: itexit = 1
  integer , parameter :: ihexit = 2

  !  sdirk method coefficients
  real(kind=dp) , dimension(5,4) :: rkalpha
  real(kind=dp) , dimension(5,4) :: rkbeta
  real(kind=dp) , dimension(4,5) :: rkd
  real(kind=dp) , dimension(5,5) :: rka
  real(kind=dp) , dimension(5) :: rkb , rkc
  real(kind=dp) :: rkgamma

  ! mz_rs_20050717: todo: use strings of ierr_names for error messages
  ! description of the error numbers ierr
  character(len=50), parameter, dimension(-8:1) :: ierr_names = (/ &
    'matrix is repeatedly singular                     ', & ! -8
    'step size too small                               ', & ! -7
    'no of steps exceeds maximum bound                 ', & ! -6
    'improper tolerance values                         ', & ! -5
    'facmin/facmax/facrej must be positive             ', & ! -4
    'hmin/hmax/hstart must be positive                 ', & ! -3
    'improper value for maximal no of newton iterations', & ! -2
    'improper value for maximal no of steps            ', & ! -1
    '                                                  ', & !  0 (not used)
    'success                                           ' /) !  1

  contains

  subroutine reset_integrate
    implicit none
    conit = 0.0_dp
    crate = 0.0_dp
    hold = 0.0_dp
    rmax = 0.0_dp
    elco(:,:) = 0.0_dp
    tesco(:,:) = 0.0_dp
    el(:) = 0.0_dp
    ccmax = 0.0_dp
    el0 = 0.0_dp
    h = 0.0_dp
    hmin = 0.0_dp
    hmxi = 0.0_dp
    hu = 0.0_dp
    rc = 0.0_dp
    tn = 0.0_dp
    uround = 0.0_dp
    init = 0
    mxstep = 0
    mxhnil = 0
    nhnil = 0
    nslast = 0
    nyh = 0
    ialth = 0
    ipup = 0
    lmax = 0
    meo = 0
    nqnyh = 0
    nslp = 0
    icf = 0
    ierpj = 0
    iersl = 0
    jcur = 0
    jstart = 0
    kflag = 0
    l = 0
    lyh = 0
    lewt = 0
    lacor = 0
    lsavf = 0
    lwm = 0
    liwm = 0
    meth = 0
    miter = 0
    maxord = 0
    maxcor = 0
    msbp = 0
    mxncf = 0
    n = 0
    nq = 0
    nst = 0
    nfe = 0
    nje = 0
    nqu = 0
  end subroutine reset_integrate

  subroutine integrate(tin,tout,icntrl_u,rcntrl_u,istatus_u,rstatus_u,ierr_u)
    use mod_cbmz_parameters
    use mod_cbmz_global
    implicit none

    real(kind=dp), intent(in) :: tin  ! start time
    real(kind=dp), intent(in) :: tout ! end time
    ! optional input parameters and statistics
    integer,  intent(in),  optional :: icntrl_u(20)
    real(kind=dp), intent(in),  optional :: rcntrl_u(20)
    integer,  intent(out), optional :: istatus_u(20)
    real(kind=dp), intent(out), optional :: rstatus_u(20)
    integer,  intent(out), optional :: ierr_u

    real(kind=dp) :: rcntrl(20), rstatus(20)
    integer       :: icntrl(20), istatus(20), ierr

    icntrl(:)  = 0
    rcntrl(:)  = 0.0_dp
    istatus(:) = 0
    rstatus(:) = 0.0_dp

    icntrl(5) = 2 ! maximal order

    ! if optional parameters are given, and if they are >0,
    ! then they overwrite default settings.
    if (present(icntrl_u)) then
      where(icntrl_u(:) > 0) icntrl(:) = icntrl_u(:)
    end if
    if (present(rcntrl_u)) then
      where(rcntrl_u(:) > 0) rcntrl(:) = rcntrl_u(:)
    end if

    call kpplsode( tin,tout,var,rtol,atol,                &
                  rcntrl,icntrl,rstatus,istatus,ierr )

    stepmin = rstatus(ihexit) ! save last step

    ! if optional parameters are given for output they to return information
    if (present(istatus_u)) istatus_u(:) = istatus(1:20)
    if (present(rstatus_u)) rstatus_u(:) = rstatus(1:20)
    if (present(ierr_u)) then
      if (ierr==2) then ! dlsode returns "2" after successful completion
        ierr_u = 1 ! ierr_u will return "1" for successful completion
      else
        ierr_u = ierr
      endif
    endif
  end subroutine integrate
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!~~~>     input parameters:
!
!    note: for input parameters equal to zero the default values of the
!       corresponding variables are used.
!
!    note: for input parameters equal to zero the default values of the
!          corresponding variables are used.
!~~~>
!    icntrl(1) = not used
!
!    icntrl(2) = 0: abstol, reltol are nvar-dimensional vectors
!              = 1: abstol, reltol are scalars
!
!    icntrl(3) = not used
!
!    icntrl(4)  -> maximum number of integration steps
!        for icntrl(4)=0 the default value of 100000 is used
!
!    icntrl(5)  -> maximum order of the integration formula allowed
!
!~~~>  real parameters
!
!    rcntrl(1)  -> hmin, lower bound for the integration step size
!                  it is strongly recommended to keep hmin = zero
!    rcntrl(2)  -> hmax, upper bound for the integration step size
!    rcntrl(3)  -> hstart, starting value for the integration step size
!
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
!~~~>     output parameters:
!
!    note: each call to rosenbrock adds the current no. of fcn calls
!      to previous value of istatus(1), and similar for the other params.
!      set istatus(1:10) = 0 before call to avoid this accumulation.
!
!    istatus(1) = no. of function calls
!    istatus(2) = no. of jacobian calls
!    istatus(3) = no. of steps
!
!    rstatus(1)  -> texit, the time corresponding to the
!                   computed y upon return
!    rstatus(2)  -> hexit, last predicted step before exit
!    for multiple restarts, use hexit as hstart in the following run
!
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  subroutine kpplsode(tin,tout,y,reltol,abstol, &
                      rcntrl,icntrl,rstatus,istatus,ierr)
    implicit none
    real(kind=dp) :: y(nvar), abstol(nvar), reltol(nvar), tin, tout
    real(kind=dp) :: rcntrl(20), rstatus(20)
    integer       :: icntrl(20), istatus(20)
    integer, parameter :: lrw = 25 + 9*nvar+2*nvar*nvar, &
                          liw = 32 + nvar
    real(kind=dp) :: rwork(lrw)
    integer :: iwork(liw), itol, itask, ierr, iopt, mf

    !~~~> normal computation
    itask  = 1
    ierr   = 1
    iopt   = 1     ! 0=no/1=use optional input

    rwork(1:30) = 0.0d0
    iwork(1:30) = 0

    if (icntrl(2)==0) then
      itol = 4           ! abs/reltol are both vectors
    else
      itol = 1           ! abs/reltol are both scalars
    end if
    iwork(6) = icntrl(4)  ! max number of internal steps
    iwork(5) = icntrl(5)  ! maximal order

    mf = 21  !~~~> stiff case, analytic full jacobian

    rwork(5) = rcntrl(3)  ! hstart
    rwork(6) = rcntrl(2)  ! hmax
    rwork(7) = rcntrl(1)  ! hmin

    call dlsode(fun_chem, nvar, y, tin, tout, itol, reltol, abstol, itask,&
                ierr, iopt, rwork, lrw, iwork, liw, jac_chem, mf)

    istatus(1) = iwork(12) ! number of function evaluations
    istatus(2) = iwork(13) ! number of jacobian evaluations
    istatus(3) = iwork(11) ! number of steps

    rstatus(1) = tout      ! mz_rs_20050717
    rstatus(2) = rwork(11) ! mz_rs_20050717

  end subroutine kpplsode
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!***begin prologue  dlsode
!***purpose  livermore solver for ordinary differential equations.
!            dlsode solves the initial-value problem for stiff or
!            nonstiff systems of first-order ode's,
!               dy/dt = f(t,y),   or, in component form,
!               dy(i)/dt = f(i) = f(i,t,y(1),y(2),...,y(n)),  i=1,...,n.
!***category  i1a
!***type      real(kind=dp) (slsode-s, dlsode-d)
!***keywords  ordinary differential equations, initial value problem,
!             stiff, nonstiff
!***author  hindmarsh, alan c., (llnl)
!             center for applied scientific computing, l-561
!             lawrence livermore national laboratory
!             livermore, ca 94551.
!***description
!
!     note: the "usage" and "arguments" sections treat only a subset of
!           available options, in condensed fashion.  the options
!           covered and the information supplied will support most
!           standard uses of dlsode.
!
!           for more sophisticated uses, full details on all options are
!           given in the concluding section, headed "long description."
!           a synopsis of the dlsode long description is provided at the
!           beginning of that section; general topics covered are:
!           - elements of the call sequence; optional input and output
!           - optional supplemental routines in the dlsode package
!
! *usage:
!     communication between the user and the dlsode package, for normal
!     situations, is summarized here.  this summary describes a subset
!     of the available options.  see "long description" for complete
!     details, including optional communication, nonstandard options,
!     and instructions for special situations.
!
!     a sample program is given in the "examples" section.
!
!     refer to the argument descriptions for the definitions of the
!     quantities that appear in the following sample declarations.
!
!     for mf = 10,
!        parameter  (lrw = 20 + 16*neq,           liw = 20)
!     for mf = 21 or 22,
!        parameter  (lrw = 22 +  9*neq + neq**2,  liw = 20 + neq)
!     for mf = 24 or 25,
!        parameter  (lrw = 22 + 10*neq + (2*ml+mu)*neq,
!       *                                         liw = 20 + neq)
!
!        external f, jac
!        integer  neq, itol, itask, istate, iopt, lrw, iwork(liw),
!       *         liw, mf
!        real(kind=dp) y(neq), t, tout, reltol, abstol(ntol), rwork(lrw)
!
!        call dlsode (f, neq, y, t, tout, itol, reltol, abstol, itask,
!       *            istate, iopt, rwork, lrw, iwork, liw, jac, mf)
!
! *arguments:
!     f     :ext    name of subroutine for right-hand-side vector f.
!                   this name must be declared external in calling
!                   program.  the form of f must be:
!
!                   subroutine  f (neq, t, y, ydot)
!                   integer  neq
!                   real(kind=dp)  t, y(*), ydot(*)
!
!                   the inputs are neq, t, y.  f is to set
!
!                   ydot(i) = f(i,t,y(1),y(2),...,y(neq)),
!                                                     i = 1, ..., neq .
!
!     neq   :in     number of first-order ode's.
!
!     y     :inout  array of values of the y(t) vector, of length neq.
!                   input:  for the first call, y should contain the
!                           values of y(t) at t = t. (y is an input
!                           variable only if istate = 1.)
!                   output: on return, y will contain the values at the
!                           new t-value.
!
!     t     :inout  value of the independent variable.  on return it
!                   will be the current value of t (normally tout).
!
!     tout  :in     next point where output is desired (.ne. t).
!
!     itol  :in     1 or 2 according as abstol (below) is a scalar or
!                   an array.
!
!     reltol  :in     relative tolerance parameter (scalar).
!
!     abstol  :in     absolute tolerance parameter (scalar or array).
!                   if itol = 1, abstol need not be dimensioned.
!                   if itol = 2, abstol must be dimensioned at least neq.
!
!                   the estimated local error in y(i) will be controlled
!                   so as to be roughly less (in magnitude) than
!
!                   ewt(i) = reltol*abs(y(i)) + abstol     if itol = 1, or
!                   ewt(i) = reltol*abs(y(i)) + abstol(i)  if itol = 2.
!
!                   thus the local error test passes if, in each
!                   component, either the absolute error is less than
!                   abstol (or abstol(i)), or the relative error is less
!                   than reltol.
!
!                   use reltol = 0.0 for pure absolute error control, and
!                   use abstol = 0.0 (or abstol(i) = 0.0) for pure relative
!                   error control.  caution:  actual (global) errors may
!                   exceed these local tolerances, so choose them
!                   conservatively.
!
!     itask :in     flag indicating the task dlsode is to perform.
!                   use itask = 1 for normal computation of output
!                   values of y at t = tout.
!
!     istate:inout  index used for input and output to specify the state
!                   of the calculation.
!                   input:
!                    1   this is the first call for a problem.
!                    2   this is a subsequent call.
!                   output:
!                    1   nothing was done, because tout was equal to t.
!                    2   dlsode was successful (otherwise, negative).
!                        note that istate need not be modified after a
!                        successful return.
!                   -1   excess work done on this call (perhaps wrong
!                        mf).
!                   -2   excess accuracy requested (tolerances too
!                        small).
!                   -3   illegal input detected (see printed message).
!                   -4   repeated error test failures (check all
!                        inputs).
!                   -5   repeated convergence failures (perhaps bad
!                        jacobian supplied or wrong choice of mf or
!                        tolerances).
!                   -6   error weight became zero during problem
!                        (solution component i vanished, and abstol or
!                        abstol(i) = 0.).
!
!     iopt  :in     flag indicating whether optional inputs are used:
!                   0   no.
!                   1   yes.  (see "optional inputs" under "long
!                       description," part 1.)
!
!     rwork :work   real work array of length at least:
!                   20 + 16*neq                    for mf = 10,
!                   22 +  9*neq + neq**2           for mf = 21 or 22,
!                   22 + 10*neq + (2*ml + mu)*neq  for mf = 24 or 25.
!
!     lrw   :in     declared length of rwork (in user's dimension
!                   statement).
!
!     iwork :work   integer work array of length at least:
!                   20        for mf = 10,
!                   20 + neq  for mf = 21, 22, 24, or 25.
!
!                   if mf = 24 or 25, input in iwork(1),iwork(2) the
!                   lower and upper jacobian half-bandwidths ml,mu.
!
!                   on return, iwork contains information that may be
!                   of interest to the user:
!
!            name   location   meaning
!            -----  ---------  -----------------------------------------
!            nst    iwork(11)  number of steps taken for the problem so
!                              far.
!            nfe    iwork(12)  number of f evaluations for the problem
!                              so far.
!            nje    iwork(13)  number of jacobian evaluations (and of
!                              matrix lu decompositions) for the problem
!                              so far.
!            nqu    iwork(14)  method order last used (successfully).
!            lenrw  iwork(17)  length of rwork actually required.  this
!                              is defined on normal returns and on an
!                              illegal input return for insufficient
!                              storage.
!            leniw  iwork(18)  length of iwork actually required.  this
!                              is defined on normal returns and on an
!                              illegal input return for insufficient
!                              storage.
!
!     liw   :in     declared length of iwork (in user's dimension
!                   statement).
!
!     jac   :ext    name of subroutine for jacobian matrix (mf =
!                   21 or 24).  if used, this name must be declared
!                   external in calling program.  if not used, pass a
!                   dummy name.  the form of jac must be:
!
!                   subroutine jac (neq, t, y, ml, mu, pd, nrowpd)
!                   integer  neq, ml, mu, nrowpd
!                   real(kind=dp)  t, y(*), pd(nrowpd,*)
!
!                   see item c, under "description" below for more
!                   information about jac.
!
!     mf    :in     method flag.  standard values are:
!                   10  nonstiff (adams) method, no jacobian used.
!                   21  stiff (bdf) method, user-supplied full jacobian.
!                   22  stiff method, internally generated full
!                       jacobian.
!                   24  stiff method, user-supplied banded jacobian.
!                   25  stiff method, internally generated banded
!                       jacobian.
!
! *description:
!     dlsode solves the initial value problem for stiff or nonstiff
!     systems of first-order ode's,
!
!        dy/dt = f(t,y) ,
!
!     or, in component form,
!
!        dy(i)/dt = f(i) = f(i,t,y(1),y(2),...,y(neq))
!                                                  (i = 1, ..., neq) .
!
!     dlsode is a package based on the gear and gearb packages, and on
!     the october 23, 1978, version of the tentative odepack user
!     interface standard, with minor modifications.
!
!     the steps in solving such a problem are as follows.
!
!     a. first write a subroutine of the form
!
!           subroutine  f (neq, t, y, ydot)
!           integer  neq
!           real(kind=dp)  t, y(*), ydot(*)
!
!        which supplies the vector function f by loading ydot(i) with
!        f(i).
!
!     b. next determine (or guess) whether or not the problem is stiff.
!        stiffness occurs when the jacobian matrix df/dy has an
!        eigenvalue whose real part is negative and large in magnitude
!        compared to the reciprocal of the t span of interest.  if the
!        problem is nonstiff, use method flag mf = 10.  if it is stiff,
!        there are four standard choices for mf, and dlsode requires the
!        jacobian matrix in some form.  this matrix is regarded either
!        as full (mf = 21 or 22), or banded (mf = 24 or 25).  in the
!        banded case, dlsode requires two half-bandwidth parameters ml
!        and mu. these are, respectively, the widths of the lower and
!        upper parts of the band, excluding the main diagonal.  thus the
!        band consists of the locations (i,j) with
!
!           i - ml <= j <= i + mu ,
!
!        and the full bandwidth is ml + mu + 1 .
!
!     c. if the problem is stiff, you are encouraged to supply the
!        jacobian directly (mf = 21 or 24), but if this is not feasible,
!        dlsode will compute it internally by difference quotients (mf =
!        22 or 25).  if you are supplying the jacobian, write a
!        subroutine of the form
!
!           subroutine  jac (neq, t, y, ml, mu, pd, nrowpd)
!           integer  neq, ml, mu, nrwopd
!           real(kind=dp)  t, y(*), pd(nrowpd,*)
!
!        which provides df/dy by loading pd as follows:
!        - for a full jacobian (mf = 21), load pd(i,j) with df(i)/dy(j),
!          the partial derivative of f(i) with respect to y(j).  (ignore
!          the ml and mu arguments in this case.)
!        - for a banded jacobian (mf = 24), load pd(i-j+mu+1,j) with
!          df(i)/dy(j); i.e., load the diagonal lines of df/dy into the
!          rows of pd from the top down.
!        - in either case, only nonzero elements need be loaded.
!
!     d. write a main program that calls subroutine dlsode once for each
!        point at which answers are desired.  this should also provide
!        for possible use of logical unit 6 for output of error messages
!        by dlsode.
!
!        before the first call to dlsode, set istate = 1, set y and t to
!        the initial values, and set tout to the first output point.  to
!        continue the integration after a successful return, simply
!        reset tout and call dlsode again.  no other parameters need be
!        reset.
!
! *examples:
!     the following is a simple example problem, with the coding needed
!     for its solution by dlsode. the problem is from chemical kinetics,
!     and consists of the following three rate equations:
!
!        dy1/dt = -.04*y1 + 1.e4*y2*y3
!        dy2/dt = .04*y1 - 1.e4*y2*y3 - 3.e7*y2**2
!        dy3/dt = 3.e7*y2**2
!
!     on the interval from t = 0.0 to t = 4.e10, with initial conditions
!     y1 = 1.0, y2 = y3 = 0. the problem is stiff.
!
!     the following coding solves this problem with dlsode, using
!     mf = 21 and printing results at t = .4, 4., ..., 4.e10.  it uses
!     itol = 2 and abstol much smaller for y2 than for y1 or y3 because y2
!     has much smaller values.  at the end of the run, statistical
!     quantities of interest are printed.
!
!        external  fex, jex
!        integer  iopt, iout, istate, itask, itol, iwork(23), liw, lrw,
!       *         mf, neq
!        real(kind=dp)  abstol(3), reltol, rwork(58), t, tout, y(3)
!        neq = 3
!        y(1) = 1.d0
!        y(2) = 0.d0
!        y(3) = 0.d0
!        t = 0.d0
!        tout = .4d0
!        itol = 2
!        reltol = 1.d-4
!        abstol(1) = 1.d-6
!        abstol(2) = 1.d-10
!        abstol(3) = 1.d-6
!        itask = 1
!        istate = 1
!        iopt = 0
!        lrw = 58
!        liw = 23
!        mf = 21
!        do 40 iout = 1,12
!          call dlsode (fex, neq, y, t, tout, itol, reltol, abstol, itask,
!       *               istate, iopt, rwork, lrw, iwork, liw, jex, mf)
!          write(6,20)  t, y(1), y(2), y(3)
!    20    format(' at t =',d12.4,'   y =',3d14.6)
!          if (istate .lt. 0)  go to 80
!    40    tout = tout*10.d0
!        write(6,60)  iwork(11), iwork(12), iwork(13)
!    60  format(/' no. steps =',i4,',  no. f-s =',i4,',  no. j-s =',i4)
!        stop
!    80  write(6,90)  istate
!    90  format(///' error halt.. istate =',i3)
!        stop
!        end
!
!        subroutine  fex (neq, t, y, ydot)
!        integer  neq
!        real(kind=dp)  t, y(3), ydot(3)
!        ydot(1) = -.04d0*y(1) + 1.d4*y(2)*y(3)
!        ydot(3) = 3.d7*y(2)*y(2)
!        ydot(2) = -ydot(1) - ydot(3)
!        return
!        end
!
!        subroutine  jex (neq, t, y, ml, mu, pd, nrpd)
!        integer  neq, ml, mu, nrpd
!        real(kind=dp)  t, y(3), pd(nrpd,3)
!        pd(1,1) = -.04d0
!        pd(1,2) = 1.d4*y(3)
!        pd(1,3) = 1.d4*y(2)
!        pd(2,1) = .04d0
!        pd(2,3) = -pd(1,3)
!        pd(3,2) = 6.d7*y(2)
!        pd(2,2) = -pd(1,2) - pd(3,2)
!        return
!        end
!
!     the output from this program (on a cray-1 in single precision)
!     is as follows.
!
!     at t =  4.0000e-01   y =  9.851726e-01  3.386406e-05  1.479357e-02
!     at t =  4.0000e+00   y =  9.055142e-01  2.240418e-05  9.446344e-02
!     at t =  4.0000e+01   y =  7.158050e-01  9.184616e-06  2.841858e-01
!     at t =  4.0000e+02   y =  4.504846e-01  3.222434e-06  5.495122e-01
!     at t =  4.0000e+03   y =  1.831701e-01  8.940379e-07  8.168290e-01
!     at t =  4.0000e+04   y =  3.897016e-02  1.621193e-07  9.610297e-01
!     at t =  4.0000e+05   y =  4.935213e-03  1.983756e-08  9.950648e-01
!     at t =  4.0000e+06   y =  5.159269e-04  2.064759e-09  9.994841e-01
!     at t =  4.0000e+07   y =  5.306413e-05  2.122677e-10  9.999469e-01
!     at t =  4.0000e+08   y =  5.494530e-06  2.197825e-11  9.999945e-01
!     at t =  4.0000e+09   y =  5.129458e-07  2.051784e-12  9.999995e-01
!     at t =  4.0000e+10   y = -7.170603e-08 -2.868241e-13  1.000000e+00
!
!     no. steps = 330,  no. f-s = 405,  no. j-s = 69
!
! *accuracy:
!     the accuracy of the solution depends on the choice of tolerances
!     reltol and abstol.  actual (global) errors may exceed these local
!     tolerances, so choose them conservatively.
!
! *cautions:
!     the work arrays should not be altered between calls to dlsode for
!     the same problem, except possibly for the conditional and optional
!     inputs.
!
! *portability:
!     since neq is dimensioned inside dlsode, some compilers may object
!     to a call to dlsode with neq a scalar variable.  in this event,
!     use dimension neq.  similar remarks apply to reltol and abstol.
!
!     note to cray users:
!     for maximum efficiency, use the cft77 compiler.  appropriate
!     compiler optimization directives have been inserted for cft77.
!
! *reference:
!     alan c. hindmarsh, "odepack, a systematized collection of ode
!     solvers," in scientific computing, r. s. stepleman, et al., eds.
!     (north-holland, amsterdam, 1983), pp. 55-64.
!
! *long description:
!     the following complete description of the user interface to
!     dlsode consists of four parts:
!
!     1.  the call sequence to subroutine dlsode, which is a driver
!         routine for the solver.  this includes descriptions of both
!         the call sequence arguments and user-supplied routines.
!         following these descriptions is a description of optional
!         inputs available through the call sequence, and then a
!         description of optional outputs in the work arrays.
!
!     2.  descriptions of other routines in the dlsode package that may
!         be (optionally) called by the user.  these provide the ability
!         to alter error message handling, and obtain specified derivatives
!         of the solution y(t).
!
!     3.  description of two routines in the dlsode package, either of
!         which the user may replace with his own version, if desired.
!         these relate to the measurement of errors.
!
!
!                         part 1.  call sequence
!                         ----------------------
!
!     arguments
!     ---------
!     the call sequence parameters used for input only are
!
!        f, neq, tout, itol, reltol, abstol, itask, iopt, lrw, liw, jac, mf,
!
!     and those used for both input and output are
!
!        y, t, istate.
!
!     the work arrays rwork and iwork are also used for conditional and
!     optional inputs and optional outputs.  (the term output here
!     refers to the return from subroutine dlsode to the user's calling
!     program.)
!
!     the legality of input parameters will be thoroughly checked on the
!     initial call for the problem, but not checked thereafter unless a
!     change in input parameters is flagged by istate = 3 on input.
!
!     the descriptions of the call arguments are as follows.
!
!     f        the name of the user-supplied subroutine defining the ode
!              system.  the system must be put in the first-order form
!              dy/dt = f(t,y), where f is a vector-valued function of
!              the scalar t and the vector y. subroutine f is to compute
!              the function f. it is to have the form
!
!                 subroutine f (neq, t, y, ydot)
!                 real(kind=dp)  t, y(*), ydot(*)
!
!              where neq, t, and y are input, and the array ydot =
!              f(t,y) is output.  y and ydot are arrays of length neq.
!              subroutine f should not alter y(1),...,y(neq).  f must be
!              declared external in the calling program.
!
!              subroutine f may access user-defined quantities in
!              neq(2),... and/or in y(neq+1),..., if neq is an array
!              (dimensioned in f) and/or y has length exceeding neq.
!              see the descriptions of neq and y below.
!
!              if quantities computed in the f routine are needed
!              externally to dlsode, an extra call to f should be made
!              for this purpose, for consistent and accurate results.
!              if only the derivative dy/dt is needed, use dintdy
!              instead.
!
!     neq      the size of the ode system (number of first-order
!              ordinary differential equations).  used only for input.
!              neq may be decreased, but not increased, during the
!              problem.  if neq is decreased (with istate = 3 on input),
!              the remaining components of y should be left undisturbed,
!              if these are to be accessed in f and/or jac.
!
!              normally, neq is a scalar, and it is generally referred
!              to as a scalar in this user interface description.
!              however, neq may be an array, with neq set to the
!              system size.  (the dlsode package accesses only neq.)
!              in either case, this parameter is passed as the neq
!              argument in all calls to f and jac.  hence, if it is an
!              array, locations neq(2),... may be used to store other
!              integer data and pass it to f and/or jac.  subroutines
!              f and/or jac must include neq in a dimension statement
!              in that case.
!
!     y        a real array for the vector of dependent variables, of
!              length neq or more.  used for both input and output on
!              the first call (istate = 1), and only for output on
!              other calls.  on the first call, y must contain the
!              vector of initial values.  on output, y contains the
!              computed solution vector, evaluated at t. if desired,
!              the y array may be used for other purposes between
!              calls to the solver.
!
!              this array is passed as the y argument in all calls to f
!              and jac.  hence its length may exceed neq, and locations
!              y(neq+1),... may be used to store other real data and
!              pass it to f and/or jac.  (the dlsode package accesses
!              only y(1),...,y(neq).)
!
!     t        the independent variable.  on input, t is used only on
!              the first call, as the initial point of the integration.
!              on output, after each call, t is the value at which a
!              computed solution y is evaluated (usually the same as
!              tout).  on an error return, t is the farthest point
!              reached.
!
!     tout     the next value of t at which a computed solution is
!              desired.  used only for input.
!
!              when starting the problem (istate = 1), tout may be equal
!              to t for one call, then should not equal t for the next
!              call.  for the initial t, an input value of tout .ne. t
!              is used in order to determine the direction of the
!              integration (i.e., the algebraic sign of the step sizes)
!              and the rough scale of the problem.  integration in
!              either direction (forward or backward in t) is permitted.
!
!              if itask = 2 or 5 (one-step modes), tout is ignored
!              after the first call (i.e., the first call with
!              tout .ne. t).  otherwise, tout is required on every call.
!
!              if itask = 1, 3, or 4, the values of tout need not be
!              monotone, but a value of tout which backs up is limited
!              to the current internal t interval, whose endpoints are
!              tcur - hu and tcur.  (see "optional outputs" below for
!              tcur and hu.)
!
!
!     itol     an indicator for the type of error control.  see
!              description below under abstol.  used only for input.
!
!     reltol     a relative error tolerance parameter, either a scalar or
!              an array of length neq.  see description below under
!              abstol.  input only.
!
!     abstol     an absolute error tolerance parameter, either a scalar or
!              an array of length neq.  input only.
!
!              the input parameters itol, reltol, and abstol determine the
!              error control performed by the solver.  the solver will
!              control the vector e = (e(i)) of estimated local errors
!              in y, according to an inequality of the form
!
!                 rms-norm of ( e(i)/ewt(i) ) <= 1,
!
!              where
!
!                 ewt(i) = reltol(i)*abs(y(i)) + abstol(i),
!
!              and the rms-norm (root-mean-square norm) here is
!
!                 rms-norm(v) = sqrt(sum v(i)**2 / neq).
!
!              here ewt = (ewt(i)) is a vector of weights which must
!              always be positive, and the values of reltol and abstol
!              should all be nonnegative.  the following table gives the
!              types (scalar/array) of reltol and abstol, and the
!              corresponding form of ewt(i).
!
!              itol    reltol      abstol      ewt(i)
!              ----    ------    ------    -----------------------------
!              1       scalar    scalar    reltol*abs(y(i)) + abstol
!              2       scalar    array     reltol*abs(y(i)) + abstol(i)
!              3       array     scalar    reltol(i)*abs(y(i)) + abstol
!              4       array     array     reltol(i)*abs(y(i)) + abstol(i)
!
!              when either of these parameters is a scalar, it need not
!              be dimensioned in the user's calling program.
!
!              if none of the above choices (with itol, reltol, and abstol
!              fixed throughout the problem) is suitable, more general
!              error controls can be obtained by substituting
!              user-supplied routines for the setting of ewt and/or for
!              the norm calculation.  see part 4 below.
!
!              if global errors are to be estimated by making a repeated
!              run on the same problem with smaller tolerances, then all
!              components of reltol and abstol (i.e., of ewt) should be
!              scaled down uniformly.
!
!     itask    an index specifying the task to be performed.  input
!              only.  itask has the following values and meanings:
!              1   normal computation of output values of y(t) at
!                  t = tout (by overshooting and interpolating).
!              2   take one step only and return.
!              3   stop at the first internal mesh point at or beyond
!                  t = tout and return.
!              4   normal computation of output values of y(t) at
!                  t = tout but without overshooting t = tcrit.  tcrit
!                  must be input as rwork(1).  tcrit may be equal to or
!                  beyond tout, but not behind it in the direction of
!                  integration.  this option is useful if the problem
!                  has a singularity at or beyond t = tcrit.
!              5   take one step, without passing tcrit, and return.
!                  tcrit must be input as rwork(1).
!
!              note:  if itask = 4 or 5 and the solver reaches tcrit
!              (within roundoff), it will return t = tcrit (exactly) to
!              indicate this (unless itask = 4 and tout comes before
!              tcrit, in which case answers at t = tout are returned
!              first).
!
!     istate   an index used for input and output to specify the state
!              of the calculation.
!
!              on input, the values of istate are as follows:
!              1   this is the first call for the problem
!                  (initializations will be done).  see "note" below.
!              2   this is not the first call, and the calculation is to
!                  continue normally, with no change in any input
!                  parameters except possibly tout and itask.  (if itol,
!                  reltol, and/or abstol are changed between calls with
!                  istate = 2, the new values will be used but not
!                  tested for legality.)
!              3   this is not the first call, and the calculation is to
!                  continue normally, but with a change in input
!                  parameters other than tout and itask.  changes are
!                  allowed in neq, itol, reltol, abstol, iopt, lrw, liw, mf,
!                  ml, mu, and any of the optional inputs except h0.
!                  (see iwork description for ml and mu.)
!
!              note:  a preliminary call with tout = t is not counted as
!              a first call here, as no initialization or checking of
!              input is done.  (such a call is sometimes useful for the
!              purpose of outputting the initial conditions.)  thus the
!              first call for which tout .ne. t requires istate = 1 on
!              input.
!
!              on output, istate has the following values and meanings:
!               1  nothing was done, as tout was equal to t with
!                  istate = 1 on input.
!               2  the integration was performed successfully.
!              -1  an excessive amount of work (more than mxstep steps)
!                  was done on this call, before completing the
!                  requested task, but the integration was otherwise
!                  successful as far as t. (mxstep is an optional input
!                  and is normally 500.)  to continue, the user may
!                  simply reset istate to a value >1 and call again (the
!                  excess work step counter will be reset to 0).  in
!                  addition, the user may increase mxstep to avoid this
!                  error return; see "optional inputs" below.
!              -2  too much accuracy was requested for the precision of
!                  the machine being used.  this was detected before
!                  completing the requested task, but the integration
!                  was successful as far as t. to continue, the
!                  tolerance parameters must be reset, and istate must
!                  be set to 3. the optional output tolsf may be used
!                  for this purpose.  (note:  if this condition is
!                  detected before taking any steps, then an illegal
!                  input return (istate = -3) occurs instead.)
!              -3  illegal input was detected, before taking any
!                  integration steps.  see written message for details.
!                  (note:  if the solver detects an infinite loop of
!                  calls to the solver with illegal input, it will cause
!                  the run to stop.)
!              -4  there were repeated error-test failures on one
!                  attempted step, before completing the requested task,
!                  but the integration was successful as far as t.  the
!                  problem may have a singularity, or the input may be
!                  inappropriate.
!              -5  there were repeated convergence-test failures on one
!                  attempted step, before completing the requested task,
!                  but the integration was successful as far as t. this
!                  may be caused by an inaccurate jacobian matrix, if
!                  one is being used.
!              -6  ewt(i) became zero for some i during the integration.
!                  pure relative error control (abstol(i)=0.0) was
!                  requested on a variable which has now vanished.  the
!                  integration was successful as far as t.
!
!              note:  since the normal output value of istate is 2, it
!              does not need to be reset for normal continuation.  also,
!              since a negative input value of istate will be regarded
!              as illegal, a negative output value requires the user to
!              change it, and possibly other inputs, before calling the
!              solver again.
!
!     iopt     an integer flag to specify whether any optional inputs
!              are being used on this call.  input only.  the optional
!              inputs are listed under a separate heading below.
!              0   no optional inputs are being used.  default values
!                  will be used in all cases.
!              1   one or more optional inputs are being used.
!
!     rwork    a real working array (double precision).  the length of
!              rwork must be at least
!
!                 20 + nyh*(maxord + 1) + 3*neq + lwm
!
!              where
!                 nyh = the initial value of neq,
!              maxord = 12 (if meth = 1) or 5 (if meth = 2) (unless a
!                       smaller value is given as an optional input),
!                 lwm = 0           if miter = 0,
!                 lwm = neq**2 + 2  if miter = 1 or 2,
!                 lwm = neq + 2     if miter = 3, and
!                 lwm = (2*ml + mu + 1)*neq + 2
!                                   if miter = 4 or 5.
!              (see the mf description below for meth and miter.)
!
!              thus if maxord has its default value and neq is constant,
!              this length is:
!              20 + 16*neq                    for mf = 10,
!              22 + 16*neq + neq**2           for mf = 11 or 12,
!              22 + 17*neq                    for mf = 13,
!              22 + 17*neq + (2*ml + mu)*neq  for mf = 14 or 15,
!              20 +  9*neq                    for mf = 20,
!              22 +  9*neq + neq**2           for mf = 21 or 22,
!              22 + 10*neq                    for mf = 23,
!              22 + 10*neq + (2*ml + mu)*neq  for mf = 24 or 25.
!
!              the first 20 words of rwork are reserved for conditional
!              and optional inputs and optional outputs.
!
!              the following word in rwork is a conditional input:
!              rwork(1) = tcrit, the critical value of t which the
!                         solver is not to overshoot.  required if itask
!                         is 4 or 5, and ignored otherwise.  see itask.
!
!     lrw      the length of the array rwork, as declared by the user.
!              (this will be checked by the solver.)
!
!     iwork    an integer work array.  its length must be at least
!              20       if miter = 0 or 3 (mf = 10, 13, 20, 23), or
!              20 + neq otherwise (mf = 11, 12, 14, 15, 21, 22, 24, 25).
!              (see the mf description below for miter.)  the first few
!              words of iwork are used for conditional and optional
!              inputs and optional outputs.
!
!              the following two words in iwork are conditional inputs:
!              iwork(1) = ml   these are the lower and upper half-
!              iwork(2) = mu   bandwidths, respectively, of the banded
!                              jacobian, excluding the main diagonal.
!                         the band is defined by the matrix locations
!                         (i,j) with i - ml <= j <= i + mu. ml and mu
!                         must satisfy 0 <= ml,mu <= neq - 1. these are
!                         required if miter is 4 or 5, and ignored
!                         otherwise.  ml and mu may in fact be the band
!                         parameters for a matrix to which df/dy is only
!                         approximately equal.
!
!     liw      the length of the array iwork, as declared by the user.
!              (this will be checked by the solver.)
!
!     note:  the work arrays must not be altered between calls to dlsode
!     for the same problem, except possibly for the conditional and
!     optional inputs, and except for the last 3*neq words of rwork.
!     the latter space is used for internal scratch space, and so is
!     available for use by the user outside dlsode between calls, if
!     desired (but not for use by f or jac).
!
!     jac      the name of the user-supplied routine (miter = 1 or 4) to
!              compute the jacobian matrix, df/dy, as a function of the
!              scalar t and the vector y.  (see the mf description below
!              for miter.)  it is to have the form
!
!                 subroutine jac (neq, t, y, ml, mu, pd, nrowpd)
!                 real(kind=dp) t, y(*), pd(nrowpd,*)
!
!              where neq, t, y, ml, mu, and nrowpd are input and the
!              array pd is to be loaded with partial derivatives
!              (elements of the jacobian matrix) on output.  pd must be
!              given a first dimension of nrowpd.  t and y have the same
!              meaning as in subroutine f.
!
!              in the full matrix case (miter = 1), ml and mu are
!              ignored, and the jacobian is to be loaded into pd in
!              columnwise manner, with df(i)/dy(j) loaded into pd(i,j).
!
!              in the band matrix case (miter = 4), the elements within
!              the band are to be loaded into pd in columnwise manner,
!              with diagonal lines of df/dy loaded into the rows of pd.
!              thus df(i)/dy(j) is to be loaded into pd(i-j+mu+1,j).  ml
!              and mu are the half-bandwidth parameters (see iwork).
!              the locations in pd in the two triangular areas which
!              correspond to nonexistent matrix elements can be ignored
!              or loaded arbitrarily, as they are overwritten by dlsode.
!
!              jac need not provide df/dy exactly. a crude approximation
!              (possibly with a smaller bandwidth) will do.
!
!              in either case, pd is preset to zero by the solver, so
!              that only the nonzero elements need be loaded by jac.
!              each call to jac is preceded by a call to f with the same
!              arguments neq, t, and y. thus to gain some efficiency,
!              intermediate quantities shared by both calculations may
!              be saved.
!
!              subroutine jac may access user-defined quantities in
!              neq(2),... and/or in y(neq+1),... if neq is an array
!              (dimensioned in jac) and/or y has length exceeding
!              neq.  see the descriptions of neq and y above.
!
!     mf       the method flag.  used only for input.  the legal values
!              of mf are 10, 11, 12, 13, 14, 15, 20, 21, 22, 23, 24,
!              and 25.  mf has decimal digits meth and miter:
!                 mf = 10*meth + miter .
!
!              meth indicates the basic linear multistep method:
!              1   implicit adams method.
!              2   method based on backward differentiation formulas
!                  (bdf's).
!
!              miter indicates the corrector iteration method:
!              0   functional iteration (no jacobian matrix is
!                  involved).
!              1   chord iteration with a user-supplied full (neq by
!                  neq) jacobian.
!              2   chord iteration with an internally generated
!                  (difference quotient) full jacobian (using neq
!                  extra calls to f per df/dy value).
!              3   chord iteration with an internally generated
!                  diagonal jacobian approximation (using one extra call
!                  to f per df/dy evaluation).
!              4   chord iteration with a user-supplied banded jacobian.
!              5   chord iteration with an internally generated banded
!                  jacobian (using ml + mu + 1 extra calls to f per
!                  df/dy evaluation).
!
!              if miter = 1 or 4, the user must supply a subroutine jac
!              (the name is arbitrary) as described above under jac.
!              for other values of miter, a dummy argument can be used.
!
!     optional inputs
!     ---------------
!     the following is a list of the optional inputs provided for in the
!     call sequence.  (see also part 2.)  for each such input variable,
!     this table lists its name as used in this documentation, its
!     location in the call sequence, its meaning, and the default value.
!     the use of any of these inputs requires iopt = 1, and in that case
!     all of these inputs are examined.  a value of zero for any of
!     these optional inputs will cause the default value to be used.
!     thus to use a subset of the optional inputs, simply preload
!     locations 5 to 10 in rwork and iwork to 0.0 and 0 respectively,
!     and then set those of interest to nonzero values.
!
!     name    location   meaning and default value
!     ------  ---------  -----------------------------------------------
!     h0      rwork(5)   step size to be attempted on the first step.
!                        the default value is determined by the solver.
!     hmax    rwork(6)   maximum absolute step size allowed.  the
!                        default value is infinite.
!     hmin    rwork(7)   minimum absolute step size allowed.  the
!                        default value is 0.  (this lower bound is not
!                        enforced on the final step before reaching
!                        tcrit when itask = 4 or 5.)
!     maxord  iwork(5)   maximum order to be allowed.  the default value
!                        is 12 if meth = 1, and 5 if meth = 2. (see the
!                        mf description above for meth.)  if maxord
!                        exceeds the default value, it will be reduced
!                        to the default value.  if maxord is changed
!                        during the problem, it may cause the current
!                        order to be reduced.
!     mxstep  iwork(6)   maximum number of (internally defined) steps
!                        allowed during one call to the solver.  the
!                        default value is 500.
!     mxhnil  iwork(7)   maximum number of messages printed (per
!                        problem) warning that t + h = t on a step
!                        (h = step size).  this must be positive to
!                        result in a nondefault value.  the default
!                        value is 10.
!
!     optional outputs
!     ----------------
!     as optional additional output from dlsode, the variables listed
!     below are quantities related to the performance of dlsode which
!     are available to the user.  these are communicated by way of the
!     work arrays, but also have internal mnemonic names as shown.
!     except where stated otherwise, all of these outputs are defined on
!     any successful return from dlsode, and on any return with istate =
!     -1, -2, -4, -5, or -6.  on an illegal input return (istate = -3),
!     they will be unchanged from their existing values (if any), except
!     possibly for tolsf, lenrw, and leniw.  on any error return,
!     outputs relevant to the error will be defined, as noted below.
!
!     name   location   meaning
!     -----  ---------  ------------------------------------------------
!     hu     rwork(11)  step size in t last used (successfully).
!     hcur   rwork(12)  step size to be attempted on the next step.
!     tcur   rwork(13)  current value of the independent variable which
!                       the solver has actually reached, i.e., the
!                       current internal mesh point in t. on output,
!                       tcur will always be at least as far as the
!                       argument t, but may be farther (if interpolation
!                       was done).
!     tolsf  rwork(14)  tolerance scale factor, greater than 1.0,
!                       computed when a request for too much accuracy
!                       was detected (istate = -3 if detected at the
!                       start of the problem, istate = -2 otherwise).
!                       if itol is left unaltered but reltol and abstol are
!                       uniformly scaled up by a factor of tolsf for the
!                       next call, then the solver is deemed likely to
!                       succeed.  (the user may also ignore tolsf and
!                       alter the tolerance parameters in any other way
!                       appropriate.)
!     nst    iwork(11)  number of steps taken for the problem so far.
!     nfe    iwork(12)  number of f evaluations for the problem so far.
!     nje    iwork(13)  number of jacobian evaluations (and of matrix lu
!                       decompositions) for the problem so far.
!     nqu    iwork(14)  method order last used (successfully).
!     nqcur  iwork(15)  order to be attempted on the next step.
!     imxer  iwork(16)  index of the component of largest magnitude in
!                       the weighted local error vector ( e(i)/ewt(i) ),
!                       on an error return with istate = -4 or -5.
!     lenrw  iwork(17)  length of rwork actually required.  this is
!                       defined on normal returns and on an illegal
!                       input return for insufficient storage.
!     leniw  iwork(18)  length of iwork actually required.  this is
!                       defined on normal returns and on an illegal
!                       input return for insufficient storage.
!
!     the following two arrays are segments of the rwork array which may
!     also be of interest to the user as optional outputs.  for each
!     array, the table below gives its internal name, its base address
!     in rwork, and its description.
!
!     name  base address  description
!     ----  ------------  ----------------------------------------------
!     yh    21            the nordsieck history array, of size nyh by
!                         (nqcur + 1), where nyh is the initial value of
!                         neq.  for j = 0,1,...,nqcur, column j + 1 of
!                         yh contains hcur**j/factorial(j) times the jth
!                         derivative of the interpolating polynomial
!                         currently representing the solution, evaluated
!                         at t = tcur.
!     acor  lenrw-neq+1   array of size neq used for the accumulated
!                         corrections on each step, scaled on output to
!                         represent the estimated local error in y on
!                         the last step.  this is the vector e in the
!                         description of the error control.  it is
!                         defined only on successful return from dlsode.
!
!
!                    part 2.  other callable routines
!                    --------------------------------
!
!     the following are optional calls which the user may make to gain
!     additional capabilities in conjunction with dlsode.
!
!     form of call              function
!     ------------------------  ----------------------------------------
!     call xsetun(lun)          set the logical unit number, lun, for
!                               output of messages from dlsode, if the
!                               default is not desired.  the default
!                               value of lun is 6. this call may be made
!                               at any time and will take effect
!                               immediately.
!     call xsetf(mflag)         set a flag to control the printing of
!                               messages by dlsode.  mflag = 0 means do
!                               not print.  (danger:  this risks losing
!                               valuable information.)  mflag = 1 means
!                               print (the default).  this call may be
!                               made at any time and will take effect
!                               immediately.
!     call dintdy(,,,,,)        provide derivatives of y, of various
!     (see below)               orders, at a specified point t, if
!                               desired.  it may be called only after a
!                               successful return from dlsode.  detailed
!                               instructions follow.
!
!     detailed instructions for using dintdy
!     --------------------------------------
!     the form of the call is:
!
!           call dintdy (t, k, rwork(21), nyh, dky, iflag)
!
!     the input parameters are:
!
!     t          value of independent variable where answers are
!                desired (normally the same as the t last returned by
!                dlsode).  for valid results, t must lie between
!                tcur - hu and tcur.  (see "optional outputs" above
!                for tcur and hu.)
!     k          integer order of the derivative desired.  k must
!                satisfy 0 <= k <= nqcur, where nqcur is the current
!                order (see "optional outputs").  the capability
!                corresponding to k = 0, i.e., computing y(t), is
!                already provided by dlsode directly.  since
!                nqcur >= 1, the first derivative dy/dt is always
!                available with dintdy.
!     rwork(21)  the base address of the history array yh.
!     nyh        column length of yh, equal to the initial value of neq.
!
!     the output parameters are:
!
!     dky        real array of length neq containing the computed value
!                of the kth derivative of y(t).
!     iflag      integer flag, returned as 0 if k and t were legal,
!                -1 if k was illegal, and -2 if t was illegal.
!                on an error return, a message is also written.
!
!
!                          part 3.  data   blocks
!                          ----------------------
!
!     if dlsode is to be used in an overlay situation, the user must
!     declare, in the primary overlay, the variables in:
!     (1) the call sequence to dlsode
!
!              part 4.  optionally replaceable solver routines
!              -----------------------------------------------
!
!     below are descriptions of two routines in the dlsode package which
!     relate to the measurement of errors.  either routine can be
!     replaced by a user-supplied version, if desired.  however, since
!     such a replacement may have a major impact on performance, it
!     should be done only when absolutely necessary, and only with great
!     caution.  (note:  the means by which the package version of a
!     routine is superseded by the user's version may be system-
!     dependent.)
!
!     dewset
!     ------
!     the following subroutine is called just before each internal
!     integration step, and sets the array of error weights, ewt, as
!     described under itol/reltol/abstol above:
!
!           subroutine dewset (neq, itol, reltol, abstol, ycur, ewt)
!
!     where neq, itol, reltol, and abstol are as in the dlsode call
!     sequence, ycur contains the current dependent variable vector,
!     and ewt is the array of weights set by dewset.
!
!     if the user supplies this subroutine, it must return in ewt(i)
!     (i = 1,...,neq) a positive quantity suitable for comparing errors
!     in y(i) to.  the ewt array returned by dewset is passed to the
!     dvnorm routine (see below), and also used by dlsode in the
!     computation of the optional output imxer, the diagonal jacobian
!     approximation, and the increments for difference quotient
!     jacobians.
!
!     in the user-supplied version of dewset, it may be desirable to use
!     the current values of derivatives of y. derivatives up to order nq
!     are available from the history array yh, described above under
!     optional outputs.  in dewset, yh is identical to the ycur array,
!     extended to nq + 1 columns with a column length of nyh and scale
!     factors of h**j/factorial(j).  on the first call for the problem,
!     given by nst = 0, nq is 1 and h is temporarily set to 1.0.
!     nyh is the initial value of neq.  the quantities nq, h, and nst
!     can be obtained by including in sewset the statements:
!           real(kind=dp) rls
!           nq = ils(33)
!           nst = ils(34)
!           h = rls(212)
!     thus, for example, the current value of dy/dt can be obtained as
!     ycur(nyh+i)/h (i=1,...,neq) (and the division by h is unnecessary
!     when nst = 0).
!
!     dvnorm
!     ------
!     dvnorm is a real function routine which computes the weighted
!     root-mean-square norm of a vector v:
!
!        d = dvnorm (n, v, w)
!
!     where:
!     n = the length of the vector,
!     v = real array of length n containing the vector,
!     w = real array of length n containing weights,
!     d = sqrt( (1/n) * sum(v(i)*w(i))**2 ).
!
!     dvnorm is called with n = neq and with w(i) = 1.0/ewt(i), where
!     ewt is as set by subroutine dewset.
!
!     if the user supplies this function, it should return a nonnegative
!     value of dvnorm suitable for use in the error control in dlsode.
!     none of the arguments should be altered by dvnorm.  for example, a
!     user-supplied dvnorm routine might:
!     - substitute a max-norm of (v(i)*w(i)) for the rms-norm, or
!     - ignore some components of v in the norm, with the effect of
!       suppressing the error control on those components of y.
!  ---------------------------------------------------------------------
!***routines called  dewset, dintdy, dumach, dstode, dvnorm, xerrwd
!***end prologue  dlsode
!
!*internal notes:
!
! other routines in the dlsode package.
!
! in addition to subroutine dlsode, the dlsode package includes the
! following subroutines and function routines:
!  dintdy   computes an interpolated value of the y vector at t = tout.
!  dstode   is the core integrator, which does one step of the
!           integration and the associated error control.
!  dcfode   sets all method coefficients and test constants.
!  dprepj   computes and preprocesses the jacobian matrix j = df/dy
!           and the newton iteration matrix p = i - h*l0*j.
!  dsolsy   manages solution of linear system in chord iteration.
!  dewset   sets the error weight vector ewt before each step.
!  dvnorm   computes the weighted r.m.s. norm of a vector.
!  dgefa and dgesl   are routines from linpack for solving full
!           systems of linear algebraic equations.
!  dgbfa and dgbsl   are routines from linpack for solving banded
!           linear systems.
!  dumach   computes the unit roundoff in a machine-independent manner.
!  xerrwd, xsetun, xsetf, ixsav, iumach   handle the printing of all
!           error messages and warnings.  xerrwd is machine-dependent.
! note: dvnorm, dumach, ixsav, and iumach are function routines.
! all the others are subroutines.
!
!**end
  subroutine dlsode(f, neq, y, t, tout, itol, reltol, abstol, itask,   &
                    istate, iopt, rwork, lrw, iwork, liw, jac, mf)
    implicit none
    interface
      subroutine f(n,t,v,fct)
        use mod_cbmz_parameters
        integer :: n
        real(kind=dp) :: v(nvar), fct(nvar), t
      end subroutine f
    end interface
    interface
      subroutine jac(n,t,v,jf)
        use mod_cbmz_parameters
        integer :: n
        real(kind=dp) :: v(nvar), t
        real(kind=dp) :: jf(lu_nonzero)
      end subroutine jac
    end interface
    integer :: neq, itol, itask, istate, iopt, lrw, liw, iwork(liw), mf
    real(kind=dp) :: y(*), t, tout, reltol(*), abstol(*), rwork(lrw)
    integer :: i, i1, i2, iflag, imxer, kgo, lf0,                       &
        leniw, lenrw, lenwm, ml, mord(2), mu, mxhnl0, mxstp0
    real(kind=dp) :: abstoli, ayi, big, ewti, h0, hmax, hmx, rh, reltoli, &
        tcrit, tdist, tnext, tol, tolsf, tp, size, sum, w0

    logical ihit
    character(len=80) msg
    save mord, mxstp0, mxhnl0
    data  mord(1),mord(2)/12,5/, mxstp0/5000/, mxhnl0/10/
!-----------------------------------------------------------------------
! block a.
! this code block is executed on every call.
! it tests istate and itask for legality and branches appropriately.
! if istate .gt. 1 but the flag init shows that initialization has
! not yet been done, an error return occurs.
! if istate = 1 and tout = t, return immediately.
!-----------------------------------------------------------------------
!
!***first executable statement  dlsode
    if (istate .lt. 1 .or. istate .gt. 3) go to 601
    if (itask .lt. 1 .or. itask .gt. 5) go to 602
    if (istate .eq. 1) go to 10
    if (init .eq. 0) go to 603
    if (istate .eq. 2) go to 200
    go to 20
   10 init = 0
    if (tout .eq. t) return
!-----------------------------------------------------------------------
! block b.
! the next code block is executed for the initial call (istate = 1),
! or for a continuation call with parameter changes (istate = 3).
! it contains checking of all inputs and various initializations.
!
! first check legality of the non-optional inputs neq, itol, iopt,
! mf, ml, and mu.
!-----------------------------------------------------------------------
   20 if (neq .le. 0) go to 604
      if (istate .eq. 1) go to 25
      if (neq .gt. n) go to 605
   25 n = neq
      if (itol .lt. 1 .or. itol .gt. 4) go to 606
      if (iopt .lt. 0 .or. iopt .gt. 1) go to 607
      meth = mf/10
      miter = mf - 10*meth
      if (meth .lt. 1 .or. meth .gt. 2) go to 608
      if (miter .lt. 0 .or. miter .gt. 5) go to 608
      if (miter .le. 3) go to 30
      ml = iwork(1)
      mu = iwork(2)
      if (ml .lt. 0 .or. ml .ge. n) go to 609
      if (mu .lt. 0 .or. mu .ge. n) go to 610
   30 continue
! next process and check the optional inputs. --------------------------
      if (iopt .eq. 1) go to 40
      maxord = mord(meth)
      mxstep = mxstp0
      mxhnil = mxhnl0
      if (istate .eq. 1) h0 = 0.0d0
      hmxi = 0.0d0
      hmin = 0.0d0
      go to 60
   40 maxord = iwork(5)
      if (maxord .lt. 0) go to 611
      if (maxord .eq. 0) maxord = 100
      maxord = min(maxord,mord(meth))
      mxstep = iwork(6)
      if (mxstep .lt. 0) go to 612
      if (mxstep .eq. 0) mxstep = mxstp0
      mxhnil = iwork(7)
      if (mxhnil .lt. 0) go to 613
      if (mxhnil .eq. 0) mxhnil = mxhnl0
      if (istate .ne. 1) go to 50
      h0 = rwork(5)
      if ((tout - t)*h0 .lt. 0.0d0) go to 614
   50 hmax = rwork(6)
      if (hmax .lt. 0.0d0) go to 615
      hmxi = 0.0d0
      if (hmax .gt. 0.0d0) hmxi = 1.0d0/hmax
      hmin = rwork(7)
      if (hmin .lt. 0.0d0) go to 616
!-----------------------------------------------------------------------
! set work array pointers and check lengths lrw and liw.
! pointers to segments of rwork and iwork are named by prefixing l to
! the name of the segment.  e.g., the segment yh starts at rwork(lyh).
! segments of rwork (in order) are denoted  yh, wm, ewt, savf, acor.
!-----------------------------------------------------------------------
   60 lyh = 21
      if (istate .eq. 1) nyh = n
      lwm = lyh + (maxord + 1)*nyh
      if (miter .eq. 0) lenwm = 0
      if (miter .eq. 1 .or. miter .eq. 2) lenwm = n*n + 2
      if (miter .eq. 3) lenwm = n + 2
      if (miter .ge. 4) lenwm = (2*ml + mu + 1)*n + 2
      lewt = lwm + lenwm
      lsavf = lewt + n
      lacor = lsavf + n
      lenrw = lacor + n - 1
      iwork(17) = lenrw
      liwm = 1
      leniw = 20 + n
      if (miter .eq. 0 .or. miter .eq. 3) leniw = 20
      iwork(18) = leniw
      if (lenrw .gt. lrw) go to 617
      if (leniw .gt. liw) go to 618
! check reltol and abstol for legality. ------------------------------------
      reltoli = reltol(1)
      abstoli = abstol(1)
      do 70 i = 1,n
        if (itol .ge. 3) reltoli = reltol(i)
        if (itol .eq. 2 .or. itol .eq. 4) abstoli = abstol(i)
        if (reltoli .lt. 0.0d0) go to 619
        if (abstoli .lt. 0.0d0) go to 620
   70   continue
      if (istate .eq. 1) go to 100
! if istate = 3, set flag to signal parameter changes to dstode. -------
      jstart = -1
      if (nq .le. maxord) go to 90
! maxord was reduced below nq.  copy yh(*,maxord+2) into savf. ---------
      do 80 i = 1,n
        rwork(i+lsavf-1) = rwork(i+lwm-1)
   80   continue
! reload wm(1) = rwork(lwm), since lwm may have changed. ---------------
   90 if (miter .gt. 0) rwork(lwm) = sqrt(uround)
      if (n .eq. nyh) go to 200
! neq was reduced.  zero part of yh to avoid undefined references. -----
      i1 = lyh + l*nyh
      i2 = lyh + (maxord + 1)*nyh - 1
      if (i1 .gt. i2) go to 200
      do 95 i = i1,i2
        rwork(i) = 0.0d0
   95   continue
      go to 200
!-----------------------------------------------------------------------
! block c.
! the next block is for the initial call only (istate = 1).
! it contains all remaining initializations, the initial call to f,
! and the calculation of the initial step size.
! the error weights in ewt are inverted after being loaded.
!-----------------------------------------------------------------------
  100 uround = dumach()
      tn = t
      if (itask .ne. 4 .and. itask .ne. 5) go to 110
      tcrit = rwork(1)
      if ((tcrit - tout)*(tout - t) .lt. 0.0d0) go to 625
      if (h0 .ne. 0.0d0 .and. (t + h0 - tcrit)*h0 .gt. 0.0d0)           &
        h0 = tcrit - t
  110 jstart = 0
      if (miter .gt. 0) rwork(lwm) = sqrt(uround)
      nhnil = 0
      nst = 0
      nje = 0
      nslast = 0
      hu = 0.0d0
      nqu = 0
      ccmax = 0.3d0
      maxcor = 3
      msbp = 20
      mxncf = 10
! initial call to f.  (lf0 points to yh(*,2).) -------------------------
      lf0 = lyh + nyh
      call f (neq, t, y, rwork(lf0))
      nfe = 1
! load the initial value vector in yh. ---------------------------------
      do 115 i = 1,n
        rwork(i+lyh-1) = y(i)
  115   continue
! load and invert the ewt array.  (h is temporarily set to 1.0.) -------
      nq = 1
      h = 1.0d0
      call dewset (n, itol, reltol, abstol, rwork(lyh), rwork(lewt))
      do 120 i = 1,n
        if (rwork(i+lewt-1) .le. 0.0d0) go to 621
        rwork(i+lewt-1) = 1.0d0/rwork(i+lewt-1)
  120   continue
!-----------------------------------------------------------------------
! the coding below computes the step size, h0, to be attempted on the
! first step, unless the user has supplied a value for this.
! first check that tout - t differs significantly from zero.
! a scalar tolerance quantity tol is computed, as max(reltol(i))
! if this is positive, or max(abstol(i)/abs(y(i))) otherwise, adjusted
! so as to be between 100*uround and 1.0e-3.
! then the computed value h0 is given by..
!                                      neq
!   h0**2 = tol / ( w0**-2 + (1/neq) * sum ( f(i)/ywt(i) )**2  )
!                                       1
! where   w0     = max ( abs(t), abs(tout) ),
!         f(i)   = i-th component of initial value of f,
!         ywt(i) = ewt(i)/tol  (a weight for y(i)).
! the sign of h0 is inferred from the initial values of tout and t.
!-----------------------------------------------------------------------
      if (h0 .ne. 0.0d0) go to 180
      tdist = abs(tout - t)
      w0 = max(abs(t),abs(tout))
      if (tdist .lt. 2.0d0*uround*w0) go to 622
      tol = reltol(1)
      if (itol .le. 2) go to 140
      do 130 i = 1,n
        tol = max(tol,reltol(i))
  130   continue
  140 if (tol .gt. 0.0d0) go to 160
      abstoli = abstol(1)
      do 150 i = 1,n
        if (itol .eq. 2 .or. itol .eq. 4) abstoli = abstol(i)
        ayi = abs(y(i))
        if (ayi .ne. 0.0d0) tol = max(tol,abstoli/ayi)
  150   continue
  160 tol = max(tol,100.0d0*uround)
      tol = min(tol,0.001d0)
      sum = dvnorm (n, rwork(lf0), rwork(lewt))
      sum = 1.0d0/(tol*w0*w0) + tol*sum**2
      h0 = 1.0d0/sqrt(sum)
      h0 = min(h0,tdist)
      h0 = sign(h0,tout-t)
! adjust h0 if necessary to meet hmax bound. ---------------------------
  180 rh = abs(h0)*hmxi
      if (rh .gt. 1.0d0) h0 = h0/rh
! load h with h0 and scale yh(*,2) by h0. ------------------------------
      h = h0
      do 190 i = 1,n
        rwork(i+lf0-1) = h0*rwork(i+lf0-1)
  190   continue
      go to 270
!-----------------------------------------------------------------------
! block d.
! the next code block is for continuation calls only (istate = 2 or 3)
! and is to check stop conditions before taking a step.
!-----------------------------------------------------------------------
  200 nslast = nst
      select case (itask)
        case (1)
          goto 210
        case (2)
          goto 250
        case (3)
          goto 220
        case (4)
          goto 230
        case (5)
          goto 240
      end select
  210 if ((tn - tout)*h .lt. 0.0d0) go to 250
      call dintdy (tout, 0, rwork(lyh), nyh, y, iflag)
      if (iflag .ne. 0) go to 627
      t = tout
      go to 420
  220 tp = tn - hu*(1.0d0 + 100.0d0*uround)
      if ((tp - tout)*h .gt. 0.0d0) go to 623
      if ((tn - tout)*h .lt. 0.0d0) go to 250
      go to 400
  230 tcrit = rwork(1)
      if ((tn - tcrit)*h .gt. 0.0d0) go to 624
      if ((tcrit - tout)*h .lt. 0.0d0) go to 625
      if ((tn - tout)*h .lt. 0.0d0) go to 245
      call dintdy (tout, 0, rwork(lyh), nyh, y, iflag)
      if (iflag .ne. 0) go to 627
      t = tout
      go to 420
  240 tcrit = rwork(1)
      if ((tn - tcrit)*h .gt. 0.0d0) go to 624
  245 hmx = abs(tn) + abs(h)
      ihit = abs(tn - tcrit) .le. 100.0d0*uround*hmx
      if (ihit) go to 400
      tnext = tn + h*(1.0d0 + 4.0d0*uround)
      if ((tnext - tcrit)*h .le. 0.0d0) go to 250
      h = (tcrit - tn)*(1.0d0 - 4.0d0*uround)
      if (istate .eq. 2) jstart = -2
!-----------------------------------------------------------------------
! block e.
! the next block is normally executed for all calls and contains
! the call to the one-step core integrator dstode.
!
! this is a looping point for the integration steps.
!
! first check for too many steps being taken, update ewt (if not at
! start of problem), check for too much accuracy being requested, and
! check for h below the roundoff level in t.
!-----------------------------------------------------------------------
  250 continue
      if ((nst-nslast) .ge. mxstep) go to 500
      call dewset (n, itol, reltol, abstol, rwork(lyh), rwork(lewt))
      do 260 i = 1,n
        if (rwork(i+lewt-1) .le. 0.0d0) go to 510
        rwork(i+lewt-1) = 1.0d0/rwork(i+lewt-1)
  260   continue
  270 tolsf = uround*dvnorm (n, rwork(lyh), rwork(lewt))
      if (tolsf .le. 1.0d0) go to 280
      tolsf = tolsf*2.0d0
      if (nst .eq. 0) go to 626
      go to 520
  280 if ((tn + h) .ne. tn) go to 290
      nhnil = nhnil + 1
      if (nhnil .gt. mxhnil) go to 290
      msg = 'dlsode-  warning..internal t (=r1) and h (=r2) are'
      call xerrwd (msg, 50, 101, 0, 0, 0, 0, 0, 0.0d0, 0.0d0)
      msg='      such that in the machine, t + h = t on the next step  '
      call xerrwd (msg, 60, 101, 0, 0, 0, 0, 0, 0.0d0, 0.0d0)
      msg = '      (h = step size). solver will continue anyway'
      call xerrwd (msg, 50, 101, 0, 0, 0, 0, 2, tn, h)
      if (nhnil .lt. mxhnil) go to 290
      msg = 'dlsode-  above warning has been issued i1 times.  '
      call xerrwd (msg, 50, 102, 0, 0, 0, 0, 0, 0.0d0, 0.0d0)
      msg = '      it will not be issued again for this problem'
      call xerrwd (msg, 50, 102, 0, 1, mxhnil, 0, 0, 0.0d0, 0.0d0)
  290 continue
!-----------------------------------------------------------------------
!  call dstode(neq,y,yh,nyh,yh,ewt,savf,acor,wm,iwm,f,jac,dprepj,dsolsy)
!-----------------------------------------------------------------------
      call dstode (neq, y, rwork(lyh), nyh, rwork(lyh), rwork(lewt),   &
        rwork(lsavf), rwork(lacor), rwork(lwm), iwork(liwm),           &
        f, jac)
        !f, jac, dprepj, dsolsy)
      kgo = 1 - kflag
      select case (kgo)
        case (1)
          goto 300
        case (2)
          goto 530
        case (3)
          goto 540
      end select
!-----------------------------------------------------------------------
! block f.
! the following block handles the case of a successful return from the
! core integrator (kflag = 0).  test for stop conditions.
!-----------------------------------------------------------------------
  300 init = 1
      select case (itask)
        case (1)
          goto 310
        case (2)
          goto 400
        case (3)
          goto 330
        case (4)
          goto 340
        case (5)
          goto 350
      end select
! itask = 1.  if tout has been reached, interpolate. -------------------
  310 if ((tn - tout)*h .lt. 0.0d0) go to 250
      call dintdy (tout, 0, rwork(lyh), nyh, y, iflag)
      t = tout
      go to 420
! itask = 3.  jump to exit if tout was reached. ------------------------
  330 if ((tn - tout)*h .ge. 0.0d0) go to 400
      go to 250
! itask = 4.  see if tout or tcrit was reached.  adjust h if necessary.
  340 if ((tn - tout)*h .lt. 0.0d0) go to 345
      call dintdy (tout, 0, rwork(lyh), nyh, y, iflag)
      t = tout
      go to 420
  345 hmx = abs(tn) + abs(h)
      ihit = abs(tn - tcrit) .le. 100.0d0*uround*hmx
      if (ihit) go to 400
      tnext = tn + h*(1.0d0 + 4.0d0*uround)
      if ((tnext - tcrit)*h .le. 0.0d0) go to 250
      h = (tcrit - tn)*(1.0d0 - 4.0d0*uround)
      jstart = -2
      go to 250
! itask = 5.  see if tcrit was reached and jump to exit. ---------------
  350 hmx = abs(tn) + abs(h)
      ihit = abs(tn - tcrit) .le. 100.0d0*uround*hmx
!-----------------------------------------------------------------------
! block g.
! the following block handles all successful returns from dlsode.
! if itask .ne. 1, y is loaded from yh and t is set accordingly.
! istate is set to 2, and the optional outputs are loaded into the
! work arrays before returning.
!-----------------------------------------------------------------------
  400 do 410 i = 1,n
        y(i) = rwork(i+lyh-1)
  410   continue
      t = tn
      if (itask .ne. 4 .and. itask .ne. 5) go to 420
      if (ihit) t = tcrit
  420 istate = 2
      rwork(11) = hu
      rwork(12) = h
      rwork(13) = tn
      iwork(11) = nst
      iwork(12) = nfe
      iwork(13) = nje
      iwork(14) = nqu
      iwork(15) = nq
      return
!-----------------------------------------------------------------------
! block h.
! the following block handles all unsuccessful returns other than
! those for illegal input.  first the error message routine is called.
! if there was an error test or convergence test failure, imxer is set.
! then y is loaded from yh and t is set to tn.  the optional outputs
! are loaded into the work arrays before returning.
!-----------------------------------------------------------------------
! the maximum number of steps was taken before reaching tout. ----------
  500 msg = 'dlsode-  at current t (=r1), mxstep (=i1) steps   '
      call xerrwd (msg, 50, 201, 0, 0, 0, 0, 0, 0.0d0, 0.0d0)
      msg = '      taken on this call before reaching tout     '
      call xerrwd (msg, 50, 201, 0, 1, mxstep, 0, 1, tn, 0.0d0)
      istate = -1
      go to 580
! ewt(i) .le. 0.0 for some i (not at start of problem). ----------------
  510 ewti = rwork(lewt+i-1)
      msg = 'dlsode-  at t (=r1), ewt(i1) has become r2 .le. 0.'
      call xerrwd (msg, 50, 202, 0, 1, i, 0, 2, tn, ewti)
      istate = -6
      go to 580
! too much accuracy requested for machine precision. -------------------
  520 msg = 'dlsode-  at t (=r1), too much accuracy requested  '
      call xerrwd (msg, 50, 203, 0, 0, 0, 0, 0, 0.0d0, 0.0d0)
      msg = '      for precision of machine..  see tolsf (=r2) '
      call xerrwd (msg, 50, 203, 0, 0, 0, 0, 2, tn, tolsf)
      rwork(14) = tolsf
      istate = -2
      go to 580
! kflag = -1.  error test failed repeatedly or with abs(h) = hmin. -----
  530 msg = 'dlsode-  at t(=r1) and step size h(=r2), the error'
      call xerrwd (msg, 50, 204, 0, 0, 0, 0, 0, 0.0d0, 0.0d0)
      msg = '      test failed repeatedly or with abs(h) = hmin'
      call xerrwd (msg, 50, 204, 0, 0, 0, 0, 2, tn, h)
      istate = -4
      go to 560
! kflag = -2.  convergence failed repeatedly or with abs(h) = hmin. ----
  540 msg = 'dlsode-  at t (=r1) and step size h (=r2), the    '
      call xerrwd (msg, 50, 205, 0, 0, 0, 0, 0, 0.0d0, 0.0d0)
      msg = '      corrector convergence failed repeatedly     '
      call xerrwd (msg, 50, 205, 0, 0, 0, 0, 0, 0.0d0, 0.0d0)
      msg = '      or with abs(h) = hmin   '
      call xerrwd (msg, 30, 205, 0, 0, 0, 0, 2, tn, h)
      istate = -5
! compute imxer if relevant. -------------------------------------------
  560 big = 0.0d0
      imxer = 1
      do 570 i = 1,n
        size = abs(rwork(i+lacor-1)*rwork(i+lewt-1))
        if (big .ge. size) go to 570
        big = size
        imxer = i
  570   continue
      iwork(16) = imxer
! set y vector, t, and optional outputs. -------------------------------
  580 do 590 i = 1,n
        y(i) = rwork(i+lyh-1)
  590   continue
      t = tn
      rwork(11) = hu
      rwork(12) = h
      rwork(13) = tn
      iwork(11) = nst
      iwork(12) = nfe
      iwork(13) = nje
      iwork(14) = nqu
      iwork(15) = nq
      return
!-----------------------------------------------------------------------
! block i.
! the following block handles all error returns due to illegal input
! (istate = -3), as detected before calling the core integrator.
! first the error message routine is called.  if the illegal input
! is a negative istate, the run is aborted (apparent infinite loop).
!-----------------------------------------------------------------------
  601 msg = 'dlsode-  istate (=i1) illegal '
      call xerrwd (msg, 30, 1, 0, 1, istate, 0, 0, 0.0d0, 0.0d0)
      if (istate .lt. 0) go to 800
      go to 700
  602 msg = 'dlsode-  itask (=i1) illegal  '
      call xerrwd (msg, 30, 2, 0, 1, itask, 0, 0, 0.0d0, 0.0d0)
      go to 700
  603 msg = 'dlsode-  istate .gt. 1 but dlsode not initialized '
      call xerrwd (msg, 50, 3, 0, 0, 0, 0, 0, 0.0d0, 0.0d0)
      go to 700
  604 msg = 'dlsode-  neq (=i1) .lt. 1     '
      call xerrwd (msg, 30, 4, 0, 1, neq, 0, 0, 0.0d0, 0.0d0)
      go to 700
  605 msg = 'dlsode-  istate = 3 and neq increased (i1 to i2)  '
      call xerrwd (msg, 50, 5, 0, 2, n, neq, 0, 0.0d0, 0.0d0)
      go to 700
  606 msg = 'dlsode-  itol (=i1) illegal   '
      call xerrwd (msg, 30, 6, 0, 1, itol, 0, 0, 0.0d0, 0.0d0)
      go to 700
  607 msg = 'dlsode-  iopt (=i1) illegal   '
      call xerrwd (msg, 30, 7, 0, 1, iopt, 0, 0, 0.0d0, 0.0d0)
      go to 700
  608 msg = 'dlsode-  mf (=i1) illegal     '
      call xerrwd (msg, 30, 8, 0, 1, mf, 0, 0, 0.0d0, 0.0d0)
      go to 700
  609 msg = 'dlsode-  ml (=i1) illegal.. .lt.0 or .ge.neq (=i2)'
      call xerrwd (msg, 50, 9, 0, 2, ml, neq, 0, 0.0d0, 0.0d0)
      go to 700
  610 msg = 'dlsode-  mu (=i1) illegal.. .lt.0 or .ge.neq (=i2)'
      call xerrwd (msg, 50, 10, 0, 2, mu, neq, 0, 0.0d0, 0.0d0)
      go to 700
  611 msg = 'dlsode-  maxord (=i1) .lt. 0  '
      call xerrwd (msg, 30, 11, 0, 1, maxord, 0, 0, 0.0d0, 0.0d0)
      go to 700
  612 msg = 'dlsode-  mxstep (=i1) .lt. 0  '
      call xerrwd (msg, 30, 12, 0, 1, mxstep, 0, 0, 0.0d0, 0.0d0)
      go to 700
  613 msg = 'dlsode-  mxhnil (=i1) .lt. 0  '
      call xerrwd (msg, 30, 13, 0, 1, mxhnil, 0, 0, 0.0d0, 0.0d0)
      go to 700
  614 msg = 'dlsode-  tout (=r1) behind t (=r2)      '
      call xerrwd (msg, 40, 14, 0, 0, 0, 0, 2, tout, t)
      msg = '      integration direction is given by h0 (=r1)  '
      call xerrwd (msg, 50, 14, 0, 0, 0, 0, 1, h0, 0.0d0)
      go to 700
  615 msg = 'dlsode-  hmax (=r1) .lt. 0.0  '
      call xerrwd (msg, 30, 15, 0, 0, 0, 0, 1, hmax, 0.0d0)
      go to 700
  616 msg = 'dlsode-  hmin (=r1) .lt. 0.0  '
      call xerrwd (msg, 30, 16, 0, 0, 0, 0, 1, hmin, 0.0d0)
      go to 700
  617 continue
      msg='dlsode-  rwork length needed, lenrw (=i1), exceeds lrw (=i2)'
      call xerrwd (msg, 60, 17, 0, 2, lenrw, lrw, 0, 0.0d0, 0.0d0)
      go to 700
  618 continue
      msg='dlsode-  iwork length needed, leniw (=i1), exceeds liw (=i2)'
      call xerrwd (msg, 60, 18, 0, 2, leniw, liw, 0, 0.0d0, 0.0d0)
      go to 700
  619 msg = 'dlsode-  reltol(i1) is r1 .lt. 0.0        '
      call xerrwd (msg, 40, 19, 0, 1, i, 0, 1, reltoli, 0.0d0)
      go to 700
  620 msg = 'dlsode-  abstol(i1) is r1 .lt. 0.0        '
      call xerrwd (msg, 40, 20, 0, 1, i, 0, 1, abstoli, 0.0d0)
      go to 700
  621 ewti = rwork(lewt+i-1)
      msg = 'dlsode-  ewt(i1) is r1 .le. 0.0         '
      call xerrwd (msg, 40, 21, 0, 1, i, 0, 1, ewti, 0.0d0)
      go to 700
  622 continue
      msg='dlsode-  tout (=r1) too close to t(=r2) to start integration'
      call xerrwd (msg, 60, 22, 0, 0, 0, 0, 2, tout, t)
      go to 700
  623 continue
      msg='dlsode-  itask = i1 and tout (=r1) behind tcur - hu (= r2)  '
      call xerrwd (msg, 60, 23, 0, 1, itask, 0, 2, tout, tp)
      go to 700
  624 continue
      msg='dlsode-  itask = 4 or 5 and tcrit (=r1) behind tcur (=r2)   '
      call xerrwd (msg, 60, 24, 0, 0, 0, 0, 2, tcrit, tn)
      go to 700
  625 continue
      msg='dlsode-  itask = 4 or 5 and tcrit (=r1) behind tout (=r2)   '
      call xerrwd (msg, 60, 25, 0, 0, 0, 0, 2, tcrit, tout)
      go to 700
  626 msg = 'dlsode-  at start of problem, too much accuracy   '
      call xerrwd (msg, 50, 26, 0, 0, 0, 0, 0, 0.0d0, 0.0d0)
      msg='      requested for precision of machine..  see tolsf (=r1) '
      call xerrwd (msg, 60, 26, 0, 0, 0, 0, 1, tolsf, 0.0d0)
      rwork(14) = tolsf
      go to 700
  627 msg = 'dlsode-  trouble in dintdy.  itask = i1, tout = r1'
      call xerrwd (msg, 50, 27, 0, 1, itask, 0, 1, tout, 0.0d0)
!
  700 istate = -3
      return
!
  800 msg = 'dlsode-  run aborted.. apparent infinite loop     '
      call xerrwd (msg, 50, 303, 2, 0, 0, 0, 0, 0.0d0, 0.0d0)
      return
!----------------------- end of subroutine dlsode ----------------------
      !end subroutine dlsode
      contains

    real(kind=dp) function dumach ()
      implicit none
      dumach = epsilon(1.0_dp)
    end function dumach

!***begin prologue  dcfode
!***subsidiary
!***purpose  set ode integrator coefficients.
!***type      real(kind=dp) (scfode-s, dcfode-d)
!***author  hindmarsh, alan c., (llnl)
!***description
!
!  dcfode is called by the integrator routine to set coefficients
!  needed there.  the coefficients for the current method, as
!  given by the value of meth, are set for all orders and saved.
!  the maximum order assumed here is 12 if meth = 1 and 5 if meth = 2.
!  (a smaller value of the maximum order is also allowed.)
!  dcfode is called once at the beginning of the problem,
!  and is not called again unless and until meth is changed.
!
!  the elco array contains the basic method coefficients.
!  the coefficients el(i), 1 .le. i .le. nq+1, for the method of
!  order nq are stored in elco(i,nq).  they are given by a genetrating
!  polynomial, i.e.,
!      l(x) = el(1) + el(2)*x + ... + el(nq+1)*x**nq.
!  for the implicit adams methods, l(x) is given by
!      dl/dx = (x+1)*(x+2)*...*(x+nq-1)/factorial(nq-1),    l(-1) = 0.
!  for the bdf methods, l(x) is given by
!      l(x) = (x+1)*(x+2)* ... *(x+nq)/k,
!  where         k = factorial(nq)*(1 + 1/2 + ... + 1/nq).
!
!  the tesco array contains test constants used for the
!  local error test and the selection of step size and/or order.
!  at order nq, tesco(k,nq) is used for the selection of step
!  size at order nq - 1 if k = 1, at order nq if k = 2, and at order
!  nq + 1 if k = 3.
!
!***see also  dlsode
!***routines called  (none)
!***end prologue  dcfode
!**end
    subroutine dcfode
      integer :: i, ib , nq , nqm1 , nqp1
      real(kind=dp) , dimension(12) :: pc
      real(kind=dp) :: agamq , fnq , fnqm1 , pint , ragq , &
                      rqfac , rq1fac , tsign , xpin
      select case (meth)
        case (1)
          goto 100
        case (2)
          goto 200
      end select

  100 elco(1,1) = 1.0d0
      elco(2,1) = 1.0d0
      tesco(1,1) = 0.0d0
      tesco(2,1) = 2.0d0
      tesco(1,2) = 1.0d0
      tesco(3,12) = 0.0d0
      pc(1) = 1.0d0
      rqfac = 1.0d0
      do 140 nq = 2,12
!-----------------------------------------------------------------------
! the pc array will contain the coefficients of the polynomial
!     p(x) = (x+1)*(x+2)*...*(x+nq-1).
! initially, p(x) = 1.
!-----------------------------------------------------------------------
        rq1fac = rqfac
        rqfac = rqfac/nq
        nqm1 = nq - 1
        fnqm1 = nqm1
        nqp1 = nq + 1
! form coefficients of p(x)*(x+nq-1). ----------------------------------
        pc(nq) = 0.0d0
        do 110 ib = 1,nqm1
          i = nqp1 - ib
          pc(i) = pc(i-1) + fnqm1*pc(i)
  110     continue
        pc(1) = fnqm1*pc(1)
! compute integral, -1 to 0, of p(x) and x*p(x). -----------------------
        pint = pc(1)
        xpin = pc(1)/2.0d0
        tsign = 1.0d0
        do 120 i = 2,nq
          tsign = -tsign
          pint = pint + tsign*pc(i)/i
          xpin = xpin + tsign*pc(i)/(i+1)
  120     continue
! store coefficients in elco and tesco. --------------------------------
        elco(1,nq) = pint*rq1fac
        elco(2,nq) = 1.0d0
        do 130 i = 2,nq
          elco(i+1,nq) = rq1fac*pc(i)/i
  130     continue
        agamq = rqfac*xpin
        ragq = 1.0d0/agamq
        tesco(2,nq) = ragq
        if (nq .lt. 12) tesco(1,nqp1) = ragq*rqfac/nqp1
        tesco(3,nqm1) = ragq
  140   continue
      return
!
  200 pc(1) = 1.0d0
      rq1fac = 1.0d0
      do 230 nq = 1,5
!-----------------------------------------------------------------------
! the pc array will contain the coefficients of the polynomial
!     p(x) = (x+1)*(x+2)*...*(x+nq).
! initially, p(x) = 1.
!-----------------------------------------------------------------------
        fnq = nq
        nqp1 = nq + 1
! form coefficients of p(x)*(x+nq). ------------------------------------
        pc(nqp1) = 0.0d0
        do 210 ib = 1,nq
          i = nq + 2 - ib
          pc(i) = pc(i-1) + fnq*pc(i)
  210     continue
        pc(1) = fnq*pc(1)
! store coefficients in elco and tesco. --------------------------------
        do 220 i = 1,nqp1
          elco(i,nq) = pc(i)/pc(2)
  220     continue
        elco(2,nq) = 1.0d0
        tesco(1,nq) = rq1fac
        tesco(2,nq) = nqp1/elco(1,nq)
        tesco(3,nq) = (nq+2)/elco(1,nq)
        rq1fac = rq1fac/fnq
  230   continue
    end subroutine dcfode

!***begin prologue  dintdy
!***subsidiary
!***purpose  interpolate solution derivatives.
!***type      real(kind=dp) (sintdy-s, dintdy-d)
!***author  hindmarsh, alan c., (llnl)
!***description
!
!  dintdy computes interpolated values of the k-th derivative of the
!  dependent variable vector y, and stores it in dky.  this routine
!  is called within the package with k = 0 and t = tout, but may
!  also be called by the user for any k up to the current order.
!  (see detailed instructions in the usage documentation.)
!
!  the computed values in dky are gotten by interpolation using the
!  nordsieck history array yh.  this array corresponds uniquely to a
!  vector-valued polynomial of degree nqcur or less, and dky is set
!  to the k-th derivative of this polynomial at t.
!  the formula for dky is:
!               q
!   dky(i)  =  sum  c(j,k) * (t - tn)**(j-k) * h**(-j) * yh(i,j+1)
!              j=k
!  where  c(j,k) = j*(j-1)*...*(j-k+1), q = nqcur, tn = tcur, h = hcur.
!  the above sum is done in reverse order.
!  iflag is returned negative if either k or t is out of bounds.
!
!***see also  dlsode
!***routines called  xerrwd
!***end prologue  dintdy
!**end
    subroutine dintdy (t, k, yh, nyh, dky, iflag)
      integer k, nyh, iflag
      real(kind=dp) t, yh(nyh,*), dky(*)
      integer i, ic, j, jb, jb2, jj, jj1, jp1
      real(kind=dp) c, r, s, tp
      character(len=80) msg
!
!***first executable statement  dintdy
      iflag = 0
      if (k .lt. 0 .or. k .gt. nq) go to 80
      tp = tn - hu -  100.0d0*uround*sign(abs(tn) + abs(hu), hu)
      if ((t-tp)*(t-tn) .gt. 0.0d0) go to 90
!
      s = (t - tn)/h
      ic = 1
      if (k .eq. 0) go to 15
      jj1 = l - k
      do 10 jj = jj1,nq
        ic = ic*jj
   10   continue
   15 c = ic
      do 20 i = 1,n
        dky(i) = c*yh(i,l)
   20   continue
      if (k .eq. nq) go to 55
      jb2 = nq - k
      do 50 jb = 1,jb2
        j = nq - jb
        jp1 = j + 1
        ic = 1
        if (k .eq. 0) go to 35
        jj1 = jp1 - k
        do 30 jj = jj1,j
          ic = ic*jj
   30     continue
   35   c = ic
        do 40 i = 1,n
          dky(i) = c*yh(i,jp1) + s*dky(i)
   40     continue
   50   continue
      if (k .eq. 0) return
   55 r = h**(-k)
      do 60 i = 1,n
        dky(i) = r*dky(i)
   60   continue
      return
!
   80 msg = 'dintdy-  k (=i1) illegal      '
      call xerrwd (msg, 30, 51, 0, 1, k, 0, 0, 0.0d0, 0.0d0)
      iflag = -1
      return
   90 msg = 'dintdy-  t (=r1) illegal      '
      call xerrwd (msg, 30, 52, 0, 0, 0, 0, 1, t, 0.0d0)
      msg='      t not in interval tcur - hu (= r1) to tcur (=r2)      '
      call xerrwd (msg, 60, 52, 0, 0, 0, 0, 2, tp, tn)
      iflag = -2
      return
!----------------------- end of subroutine dintdy ----------------------
      end subroutine dintdy
!deck dprepj
      subroutine dprepj (neq, y, yh, nyh, ewt, ftem, savf, wm, iwm, f, jac)
!***begin prologue  dprepj
!***subsidiary
!***purpose  compute and process newton iteration matrix.
!***type      real(kind=dp) (sprepj-s, dprepj-d)
!***author  hindmarsh, alan c., (llnl)
!***description
!
!  dprepj is called by dstode to compute and process the matrix
!  p = i - h*el(1)*j , where j is an approximation to the jacobian.
!  here j is computed by the user-supplied routine jac if
!  miter = 1 or 4, or by finite differencing if miter = 2, 3, or 5.
!  if miter = 3, a diagonal approximation to j is used.
!  j is stored in wm and replaced by p.  if miter .ne. 3, p is then
!  subjected to lu decomposition in preparation for later solution
!  of linear systems with p as coefficient matrix.  this is done
!  by dgefa if miter = 1 or 2, and by dgbfa if miter = 4 or 5.
!
!  in addition to variables described in dstode and dlsode prologues,
!  communication with dprepj uses the following:
!  y     = array containing predicted values on entry.
!  ftem  = work array of length n (acor in dstode).
!  savf  = array containing f evaluated at predicted y.
!  wm    = real work space for matrices.  on output it contains the
!          inverse diagonal matrix if miter = 3 and the lu decomposition
!          of p if miter is 1, 2 , 4, or 5.
!          storage of matrix elements starts at wm(3).
!          wm also contains the following matrix-related data:
!          wm(1) = sqrt(uround), used in numerical jacobian increments.
!          wm(2) = h*el0, saved for later use if miter = 3.
!  iwm   = integer work space containing pivot information, starting at
!          iwm(21), if miter is 1, 2, 4, or 5.  iwm also contains band
!          parameters ml = iwm(1) and mu = iwm(2) if miter is 4 or 5.
!  el0   = el(1) (input).
!  ierpj = output error flag,  = 0 if no trouble, .gt. 0 if
!          p matrix found to be singular.
!  jcur  = output flag = 1 to indicate that the jacobian matrix
!          (or approximation) is now current.
!***see also  dlsode
!***routines called  dgbfa, dgefa, dvnorm
!***end prologue  dprepj
!**end
      external f, jac
      integer neq, nyh, iwm(*)
      real(kind=dp) y(*), yh(nyh,*), ewt(*), ftem(*), savf(*), wm(*)
      integer ier, i, j
      real(kind=dp) con, hl0
      !real(kind=dp)  dvnorm
!
!***first executable statement  dprepj
      nje = nje + 1
      ierpj = 0
      jcur = 1
      hl0 = h*el0
      con = -hl0

      call jac_chem (neq, tn, y, wm(3))
      do i = 1,lu_nonzero
         wm(i+2) = wm(i+2)*con
      end do
      ! add identity matrix
      do i = 1,n
        j = 2+lu_diag(i)
        wm(j) = wm(j) + 1.0d0
      end do
      ! do lu decomposition on p
      call kppdecomp(wm(3),ier)
      if (ier .ne. 0) ierpj = 1
      return
 !----------------------- end of subroutine dprepj ----------------------
      end subroutine dprepj
!deck dsolsy
      subroutine dsolsy (wm, iwm, x, tem)
!***begin prologue  dsolsy
!***subsidiary
!***purpose  odepack linear system solver.
!***type      real(kind=dp) (ssolsy-s, dsolsy-d)
!***author  hindmarsh, alan c., (llnl)
!***description
!
!  this routine manages the solution of the linear system arising from
!  a chord iteration.  it is called if miter .ne. 0.
!  if miter is 1 or 2, it calls dgesl to accomplish this.
!  if miter = 3 it updates the coefficient h*el0 in the diagonal
!  matrix, and then computes the solution.
!  if miter is 4 or 5, it calls dgbsl.
!  communication with dsolsy uses the following variables:
!  wm    = real work space containing the inverse diagonal matrix if
!          miter = 3 and the lu decomposition of the matrix otherwise.
!          storage of matrix elements starts at wm(3).
!          wm also contains the following matrix-related data:
!          wm(1) = sqrt(uround) (not used here),
!          wm(2) = hl0, the previous value of h*el0, used if miter = 3.
!  iwm   = integer work space containing pivot information, starting at
!          iwm(21), if miter is 1, 2, 4, or 5.  iwm also contains band
!          parameters ml = iwm(1) and mu = iwm(2) if miter is 4 or 5.
!  x     = the right-hand side vector on input, and the solution vector
!          on output, of length n.
!  tem   = vector of work space of length n, not used in this version.
!  iersl = output flag (in common).  iersl = 0 if no trouble occurred.
!          iersl = 1 if a singular matrix arose with miter = 3.
!
!***see also  dlsode
!***routines called  dgbsl, dgesl
!***end prologue  dsolsy
!**end
      integer iwm(*)
      real(kind=dp) wm(*), x(*), tem(*)

      iersl = 0
      call kppsolve(wm(3),x)
      return
    end subroutine dsolsy

!deck dstode
      subroutine dstode (neq, y, yh, nyh, yh1, ewt, savf, acor, &
                         wm, iwm, f, jac)
                        !wm, iwm, f, jac, pjac, slvs)
!***begin prologue  dstode
!***subsidiary
!***purpose  performs one step of an odepack integration.
!***type      real(kind=dp) (sstode-s, dstode-d)
!***author  hindmarsh, alan c., (llnl)
!***description
!
!  dstode performs one step of the integration of an initial value
!  problem for a system of ordinary differential equations.
!  note:  dstode is independent of the value of the iteration method
!  indicator miter, when this is .ne. 0, and hence is independent
!  of the type of chord method used, or the jacobian structure.
!  communication with dstode is done with the following variables:
!
!  neq    = integer array containing problem size in neq, and
!           passed as the neq argument in all calls to f and jac.
!  y      = an array of length .ge. n used as the y argument in
!           all calls to f and jac.
!  yh     = an nyh by lmax array containing the dependent variables
!           and their approximate scaled derivatives, where
!           lmax = maxord + 1.  yh(i,j+1) contains the approximate
!           j-th derivative of y(i), scaled by h**j/factorial(j)
!           (j = 0,1,...,nq).  on entry for the first step, the first
!           two columns of yh must be set from the initial values.
!  nyh    = a constant integer .ge. n, the first dimension of yh.
!  yh1    = a one-dimensional array occupying the same space as yh.
!  ewt    = an array of length n containing multiplicative weights
!           for local error measurements.  local errors in y(i) are
!           compared to 1.0/ewt(i) in various error tests.
!  savf   = an array of working storage, of length n.
!           also used for input of yh(*,maxord+2) when jstart = -1
!           and maxord .lt. the current order nq.
!  acor   = a work array of length n, used for the accumulated
!           corrections.  on a successful return, acor(i) contains
!           the estimated one-step local error in y(i).
!  wm,iwm = real and integer work arrays associated with matrix
!           operations in chord iteration (miter .ne. 0).
!  pjac   = name of routine to evaluate and preprocess jacobian matrix
!           and p = i - h*el0*jac, if a chord method is being used.
!  slvs   = name of routine to solve linear system in chord iteration.
!  ccmax  = maximum relative change in h*el0 before pjac is called.
!  h      = the step size to be attempted on the next step.
!           h is altered by the error control algorithm during the
!           problem.  h can be either positive or negative, but its
!           sign must remain constant throughout the problem.
!  hmin   = the minimum absolute value of the step size h to be used.
!  hmxi   = inverse of the maximum absolute value of h to be used.
!           hmxi = 0.0 is allowed and corresponds to an infinite hmax.
!           hmin and hmxi may be changed at any time, but will not
!           take effect until the next change of h is considered.
!  tn     = the independent variable. tn is updated on each step taken.
!  jstart = an integer used for input only, with the following
!           values and meanings:
!                0  perform the first step.
!            .gt.0  take a new step continuing from the last.
!               -1  take the next step with a new value of h, maxord,
!                     n, meth, miter, and/or matrix parameters.
!               -2  take the next step with a new value of h,
!                     but with other inputs unchanged.
!           on return, jstart is set to 1 to facilitate continuation.
!  kflag  = a completion code with the following meanings:
!                0  the step was succesful.
!               -1  the requested error could not be achieved.
!               -2  corrector convergence could not be achieved.
!               -3  fatal error in pjac or slvs.
!           a return with kflag = -1 or -2 means either
!           abs(h) = hmin or 10 consecutive failures occurred.
!           on a return with kflag negative, the values of tn and
!           the yh array are as of the beginning of the last
!           step, and h is the last step size attempted.
!  maxord = the maximum order of integration method to be allowed.
!  maxcor = the maximum number of corrector iterations allowed.
!  msbp   = maximum number of steps between pjac calls (miter .gt. 0).
!  mxncf  = maximum number of convergence failures allowed.
!  meth/miter = the method flags.  see description in driver.
!  n      = the number of first-order differential equations.
!
!***see also  dlsode
!***routines called  dcfode, dvnorm
!***end prologue  dstode
!**end
      interface
        subroutine f(n,t,v,fct)
          use mod_cbmz_parameters
          integer :: n
          real(kind=dp) :: v(nvar), fct(nvar), t
        end subroutine f
      end interface
      interface
        subroutine jac(n,t,v,jf)
          use mod_cbmz_parameters
          integer :: n
          real(kind=dp) :: v(nvar), t
          real(kind=dp) :: jf(lu_nonzero)
        end subroutine jac
      end interface
      integer neq, nyh, iwm(*)
      real(kind=dp) y(*), yh(nyh,*), yh1(*), ewt(*), savf(*),       &
                       acor(*), wm(*)
      integer i, i1, iredo, iret, j, jb, m, ncf, newq
      real(kind=dp) dcon, ddn, del, delp, dsm, dup, exdn, exsm,     &
              exup,r, rh, rhdn, rhsm, rhup, told
      !real(kind=dp) dvnorm
!
!***first executable statement  dstode
      kflag = 0
      told = tn
      ncf = 0
      ierpj = 0
      iersl = 0
      jcur = 0
      icf = 0
      delp = 0.0d0
      if (jstart .gt. 0) go to 200
      if (jstart .eq. -1) go to 100
      if (jstart .eq. -2) go to 160
!-----------------------------------------------------------------------
! on the first call, the order is set to 1, and other variables are
! initialized.  rmax is the maximum ratio by which h can be increased
! in a single step.  it is initially 1.e4 to compensate for the small
! initial h, but then is normally equal to 10.  if a failure
! occurs (in corrector convergence or error test), rmax is set to 2
! for the next increase.
!-----------------------------------------------------------------------
      lmax = maxord + 1
      nq = 1
      l = 2
      ialth = 2
      rmax = 10000.0d0
      rc = 0.0d0
      el0 = 1.0d0
      crate = 0.7d0
      hold = h
      meo = meth
      nslp = 0
      ipup = miter
      iret = 3
      go to 140
!-----------------------------------------------------------------------
! the following block handles preliminaries needed when jstart = -1.
! ipup is set to miter to force a matrix update.
! if an order increase is about to be considered (ialth = 1),
! ialth is reset to 2 to postpone consideration one more step.
! if the caller has changed meth, dcfode is called to reset
! the coefficients of the method.
! if the caller has changed maxord to a value less than the current
! order nq, nq is reduced to maxord, and a new h chosen accordingly.
! if h is to be changed, yh must be rescaled.
! if h or meth is being changed, ialth is reset to l = nq + 1
! to prevent further changes in h for that many steps.
!-----------------------------------------------------------------------
  100 ipup = miter
      lmax = maxord + 1
      if (ialth .eq. 1) ialth = 2
      if (meth .eq. meo) go to 110
      call dcfode
      meo = meth
      if (nq .gt. maxord) go to 120
      ialth = l
      iret = 1
      go to 150
  110 if (nq .le. maxord) go to 160
  120 nq = maxord
      l = lmax
      do 125 i = 1,l
        el(i) = elco(i,nq)
  125   continue
      nqnyh = nq*nyh
      rc = rc*el(1)/el0
      el0 = el(1)
      conit = 0.5d0/(nq+2)
      ddn = dvnorm (n, savf, ewt)/tesco(1,l)
      exdn = 1.0d0/l
      rhdn = 1.0d0/(1.3d0*ddn**exdn + 0.0000013d0)
      rh = min(rhdn,1.0d0)
      iredo = 3
      if (h .eq. hold) go to 170
      rh = min(rh,abs(h/hold))
      h = hold
      go to 175
!-----------------------------------------------------------------------
! dcfode is called to get all the integration coefficients for the
! current meth.  then the el vector and related constants are reset
! whenever the order nq is changed, or at the start of the problem.
!-----------------------------------------------------------------------
  140 call dcfode
  150 do 155 i = 1,l
        el(i) = elco(i,nq)
  155   continue
      nqnyh = nq*nyh
      rc = rc*el(1)/el0
      el0 = el(1)
      conit = 0.5d0/(nq+2)
      select case (iret)
        case (1)
          goto 160
        case (2)
          goto 170
        case (3)
          goto 200
      end select
!-----------------------------------------------------------------------
! if h is being changed, the h ratio rh is checked against
! rmax, hmin, and hmxi, and the yh array rescaled.  ialth is set to
! l = nq + 1 to prevent a change of h for that many steps, unless
! forced by a convergence or error test failure.
!-----------------------------------------------------------------------
  160 if (h .eq. hold) go to 200
      rh = h/hold
      h = hold
      iredo = 3
      go to 175
  170 rh = max(rh,hmin/abs(h))
  175 rh = min(rh,rmax)
      rh = rh/max(1.0d0,abs(h)*hmxi*rh)
      r = 1.0d0
      do 180 j = 2,l
        r = r*rh
        do 181 i = 1,n
          yh(i,j) = yh(i,j)*r
  181     continue
  180   continue
      h = h*rh
      rc = rc*rh
      ialth = l
      if (iredo .eq. 0) go to 690
!-----------------------------------------------------------------------
! this section computes the predicted values by effectively
! multiplying the yh array by the pascal triangle matrix.
! rc is the ratio of new to old values of the coefficient  h*el(1).
! when rc differs from 1 by more than ccmax, ipup is set to miter
! to force pjac to be called, if a jacobian is involved.
! in any case, pjac is called at least every msbp steps.
!-----------------------------------------------------------------------
  200 if (abs(rc-1.0d0) .gt. ccmax) ipup = miter
      if (nst .ge. nslp+msbp) ipup = miter
      tn = tn + h
      i1 = nqnyh + 1
      do 215 jb = 1,nq
        i1 = i1 - nyh
!dir$ ivdep
        do 210 i = i1,nqnyh
          yh1(i) = yh1(i) + yh1(i+nyh)
  210     continue
  215   continue
!-----------------------------------------------------------------------
! up to maxcor corrector iterations are taken.  a convergence test is
! made on the r.m.s. norm of each correction, weighted by the error
! weight vector ewt.  the sum of the corrections is accumulated in the
! vector acor(i).  the yh array is not altered in the corrector loop.
!-----------------------------------------------------------------------
  220 m = 0
      do 230 i = 1,n
        y(i) = yh(i,1)
  230   continue
      call f (neq, tn, y, savf)
      nfe = nfe + 1
      if (ipup .le. 0) go to 250
!-----------------------------------------------------------------------
! if indicated, the matrix p = i - h*el(1)*j is reevaluated and
! preprocessed before starting the corrector iteration.  ipup is set
! to 0 as an indicator that this has been done.
!-----------------------------------------------------------------------
      !call pjac (neq, y, yh, nyh, ewt, acor, savf, wm, iwm, f, jac)
      call dprepj(neq, y, yh, nyh, ewt, acor, savf, wm, iwm, f, jac)
      ipup = 0
      rc = 1.0d0
      nslp = nst
      crate = 0.7d0
      if (ierpj .ne. 0) go to 430
  250 do 260 i = 1,n
        acor(i) = 0.0d0
  260   continue
  270 if (miter .ne. 0) go to 350
!-----------------------------------------------------------------------
! in the case of functional iteration, update y directly from
! the result of the last function evaluation.
!-----------------------------------------------------------------------
      do 290 i = 1,n
        savf(i) = h*savf(i) - yh(i,2)
        y(i) = savf(i) - acor(i)
  290   continue
      del = dvnorm (n, y, ewt)
      do 300 i = 1,n
        y(i) = yh(i,1) + el(1)*savf(i)
        acor(i) = savf(i)
  300   continue
      go to 400
!-----------------------------------------------------------------------
! in the case of the chord method, compute the corrector error,
! and solve the linear system with that as right-hand side and
! p as coefficient matrix.
!-----------------------------------------------------------------------
  350 do 360 i = 1,n
        y(i) = h*savf(i) - (yh(i,2) + acor(i))
  360   continue
      !call slvs (wm, iwm, y, savf)
      call dsolsy(wm, iwm, y, savf)
      if (iersl .lt. 0) go to 430
      if (iersl .gt. 0) go to 410
      del = dvnorm (n, y, ewt)
      do 380 i = 1,n
        acor(i) = acor(i) + y(i)
        y(i) = yh(i,1) + el(1)*acor(i)
  380   continue
!-----------------------------------------------------------------------
! test for convergence.  if m.gt.0, an estimate of the convergence
! rate constant is stored in crate, and this is used in the test.
!-----------------------------------------------------------------------
  400 if (m .ne. 0) crate = max(0.2d0*crate,del/delp)
      dcon = del*min(1.0d0,1.5d0*crate)/(tesco(2,nq)*conit)
      if (dcon .le. 1.0d0) go to 450
      m = m + 1
      if (m .eq. maxcor) go to 410
      if (m .ge. 2 .and. del .gt. 2.0d0*delp) go to 410
      delp = del
      call f (neq, tn, y, savf)
      nfe = nfe + 1
      go to 270
!-----------------------------------------------------------------------
! the corrector iteration failed to converge.
! if miter .ne. 0 and the jacobian is out of date, pjac is called for
! the next try.  otherwise the yh array is retracted to its values
! before prediction, and h is reduced, if possible.  if h cannot be
! reduced or mxncf failures have occurred, exit with kflag = -2.
!-----------------------------------------------------------------------
  410 if (miter .eq. 0 .or. jcur .eq. 1) go to 430
      icf = 1
      ipup = miter
      go to 220
  430 icf = 2
      ncf = ncf + 1
      rmax = 2.0d0
      tn = told
      i1 = nqnyh + 1
      do 445 jb = 1,nq
        i1 = i1 - nyh
!dir$ ivdep
        do 440 i = i1,nqnyh
          yh1(i) = yh1(i) - yh1(i+nyh)
  440     continue
  445   continue
      if (ierpj .lt. 0 .or. iersl .lt. 0) go to 680
      if (abs(h) .le. hmin*1.00001d0) go to 670
      if (ncf .eq. mxncf) go to 670
      rh = 0.25d0
      ipup = miter
      iredo = 1
      go to 170
!-----------------------------------------------------------------------
! the corrector has converged.  jcur is set to 0
! to signal that the jacobian involved may need updating later.
! the local error test is made and control passes to statement 500
! if it fails.
!-----------------------------------------------------------------------
  450 jcur = 0
      if (m .eq. 0) dsm = del/tesco(2,nq)
      if (m .gt. 0) dsm = dvnorm (n, acor, ewt)/tesco(2,nq)
      if (dsm .gt. 1.0d0) go to 500
!-----------------------------------------------------------------------
! after a successful step, update the yh array.
! consider changing h if ialth = 1.  otherwise decrease ialth by 1.
! if ialth is then 1 and nq .lt. maxord, then acor is saved for
! use in a possible order increase on the next step.
! if a change in h is considered, an increase or decrease in order
! by one is considered also.  a change in h is made only if it is by a
! factor of at least 1.1.  if not, ialth is set to 3 to prevent
! testing for that many steps.
!-----------------------------------------------------------------------
      kflag = 0
      iredo = 0
      nst = nst + 1
      hu = h
      nqu = nq
      do 470 j = 1,l
        do 471 i = 1,n
          yh(i,j) = yh(i,j) + el(j)*acor(i)
  471     continue
  470   continue
      ialth = ialth - 1
      if (ialth .eq. 0) go to 520
      if (ialth .gt. 1) go to 700
      if (l .eq. lmax) go to 700
      do 490 i = 1,n
        yh(i,lmax) = acor(i)
  490   continue
      go to 700
!-----------------------------------------------------------------------
! the error test failed.  kflag keeps track of multiple failures.
! restore tn and the yh array to their previous values, and prepare
! to try the step again.  compute the optimum step size for this or
! one lower order.  after 2 or more failures, h is forced to decrease
! by a factor of 0.2 or less.
!-----------------------------------------------------------------------
  500 kflag = kflag - 1
      tn = told
      i1 = nqnyh + 1
      do 515 jb = 1,nq
        i1 = i1 - nyh
!dir$ ivdep
        do 510 i = i1,nqnyh
          yh1(i) = yh1(i) - yh1(i+nyh)
  510     continue
  515   continue
      rmax = 2.0d0
      if (abs(h) .le. hmin*1.00001d0) go to 660
      if (kflag .le. -3) go to 640
      iredo = 2
      rhup = 0.0d0
      go to 540
!-----------------------------------------------------------------------
! regardless of the success or failure of the step, factors
! rhdn, rhsm, and rhup are computed, by which h could be multiplied
! at order nq - 1, order nq, or order nq + 1, respectively.
! in the case of failure, rhup = 0.0 to avoid an order increase.
! the largest of these is determined and the new order chosen
! accordingly.  if the order is to be increased, we compute one
! additional scaled derivative.
!-----------------------------------------------------------------------
  520 rhup = 0.0d0
      if (l .eq. lmax) go to 540
      do 530 i = 1,n
        savf(i) = acor(i) - yh(i,lmax)
  530   continue
      dup = dvnorm (n, savf, ewt)/tesco(3,nq)
      exup = 1.0d0/(l+1)
      rhup = 1.0d0/(1.4d0*dup**exup + 0.0000014d0)
  540 exsm = 1.0d0/l
      rhsm = 1.0d0/(1.2d0*dsm**exsm + 0.0000012d0)
      rhdn = 0.0d0
      if (nq .eq. 1) go to 560
      ddn = dvnorm (n, yh(1,l), ewt)/tesco(1,nq)
      exdn = 1.0d0/nq
      rhdn = 1.0d0/(1.3d0*ddn**exdn + 0.0000013d0)
  560 if (rhsm .ge. rhup) go to 570
      if (rhup .gt. rhdn) go to 590
      go to 580
  570 if (rhsm .lt. rhdn) go to 580
      newq = nq
      rh = rhsm
      go to 620
  580 newq = nq - 1
      rh = rhdn
      if (kflag .lt. 0 .and. rh .gt. 1.0d0) rh = 1.0d0
      go to 620
  590 newq = l
      rh = rhup
      if (rh .lt. 1.1d0) go to 610
      r = el(l)/l
      do 600 i = 1,n
        yh(i,newq+1) = acor(i)*r
  600   continue
      go to 630
  610 ialth = 3
      go to 700
  620 if ((kflag .eq. 0) .and. (rh .lt. 1.1d0)) go to 610
      if (kflag .le. -2) rh = min(rh,0.2d0)
!-----------------------------------------------------------------------
! if there is a change of order, reset nq, l, and the coefficients.
! in any case h is reset according to rh and the yh array is rescaled.
! then exit from 690 if the step was ok, or redo the step otherwise.
!-----------------------------------------------------------------------
      if (newq .eq. nq) go to 170
  630 nq = newq
      l = nq + 1
      iret = 2
      go to 150
!-----------------------------------------------------------------------
! control reaches this section if 3 or more failures have occured.
! if 10 failures have occurred, exit with kflag = -1.
! it is assumed that the derivatives that have accumulated in the
! yh array have errors of the wrong order.  hence the first
! derivative is recomputed, and the order is set to 1.  then
! h is reduced by a factor of 10, and the step is retried,
! until it succeeds or h reaches hmin.
!-----------------------------------------------------------------------
  640 if (kflag .eq. -10) go to 660
      rh = 0.1d0
      rh = max(hmin/abs(h),rh)
      h = h*rh
      do 645 i = 1,n
        y(i) = yh(i,1)
  645   continue
      call f (neq, tn, y, savf)
      nfe = nfe + 1
      do 650 i = 1,n
        yh(i,2) = h*savf(i)
  650   continue
      ipup = miter
      ialth = 5
      if (nq .eq. 1) go to 200
      nq = 1
      l = 2
      iret = 3
      go to 150
!-----------------------------------------------------------------------
! all returns are made through this section.  h is saved in hold
! to allow the caller to change h on the next step.
!-----------------------------------------------------------------------
  660 kflag = -1
      go to 720
  670 kflag = -2
      go to 720
  680 kflag = -3
      go to 720
  690 rmax = 10.0d0
  700 r = 1.0d0/tesco(2,nqu)
      do 710 i = 1,n
        acor(i) = acor(i)*r
  710   continue
  720 hold = h
      jstart = 1
      return
!----------------------- end of subroutine dstode ----------------------
      end subroutine dstode
!deck dewset
      subroutine dewset (n, itol, reltol, abstol, ycur, ewt)
!***begin prologue  dewset
!***subsidiary
!***purpose  set error weight vector.
!***type      real(kind=dp) (sewset-s, dewset-d)
!***author  hindmarsh, alan c., (llnl)
!***description
!
!  this subroutine sets the error weight vector ewt according to
!      ewt(i) = reltol(i)*abs(ycur(i)) + abstol(i),  i = 1,...,n,
!  with the subscript on reltol and/or abstol possibly replaced by 1 above,
!  depending on the value of itol.
!
!***see also  dlsode
!***routines called  (none)
!***revision history  (yymmdd)
!   791129  date written
!   890501  modified prologue to slatec/ldoc format.  (fnf)
!   890503  minor cosmetic changes.  (fnf)
!   930809  renamed to allow single/double precision versions. (ach)
!***end prologue  dewset
!**end
      integer n, itol
      integer i
      real(kind=dp) reltol(*), abstol(*), ycur(n), ewt(n)
!
!***first executable statement  dewset
      select case (itol)
        case (1)
          goto 10
        case (2)
          goto 20
        case (3)
          goto 30
        case (4)
          goto 40
      end select
   10 continue
      do 15 i = 1,n
        ewt(i) = reltol(1)*abs(ycur(i)) + abstol(1)
   15   continue
      return
   20 continue
      do 25 i = 1,n
        ewt(i) = reltol(1)*abs(ycur(i)) + abstol(i)
   25   continue
      return
   30 continue
      do 35 i = 1,n
        ewt(i) = reltol(i)*abs(ycur(i)) + abstol(1)
   35   continue
      return
   40 continue
      do 45 i = 1,n
        ewt(i) = reltol(i)*abs(ycur(i)) + abstol(i)
   45   continue
      return
!----------------------- end of subroutine dewset ----------------------
      end subroutine dewset
!deck dvnorm
      real(kind=dp) function dvnorm (n, v, w)
!***begin prologue  dvnorm
!***subsidiary
!***purpose  weighted root-mean-square vector norm.
!***type      real(kind=dp) (svnorm-s, dvnorm-d)
!***author  hindmarsh, alan c., (llnl)
!***description
!
!  this function routine computes the weighted root-mean-square norm
!  of the vector of length n contained in the array v, with weights
!  contained in the array w of length n:
!    dvnorm = sqrt( (1/n) * sum( v(i)*w(i) )**2 )
!
!***see also  dlsode
!***routines called  (none)
!***revision history  (yymmdd)
!   791129  date written
!   890501  modified prologue to slatec/ldoc format.  (fnf)
!   890503  minor cosmetic changes.  (fnf)
!   930809  renamed to allow single/double precision versions. (ach)
!***end prologue  dvnorm
!**end
      integer n,   i
      real(kind=dp) v(n), w(n)
      real(kind=qp) :: xsum
!
!***first executable statement  dvnorm
      xsum = 0.0_qp
      do i = 1 , n
        xsum = xsum + (real(v(i),qp)*real(w(i),qp))**2
      end do
      dvnorm = real(sqrt(xsum/real(n,qp)),dp)
      return
!----------------------- end of function dvnorm ------------------------
      end function dvnorm
!deck xerrwd
      subroutine xerrwd (msg, nmes, nerr, level, ni, i1, i2, nr, r1, r2)
!***begin prologue  xerrwd
!***subsidiary
!***purpose  write error message with values.
!***category  r3c
!***type      real(kind=dp) (xerrwv-s, xerrwd-d)
!***author  hindmarsh, alan c., (llnl)
!***description
!
!  subroutines xerrwd, xsetf, xsetun, and the function routine ixsav,
!  as given here, constitute a simplified version of the slatec error
!  handling package.
!
!  all arguments are input arguments.
!
!  msg    = the message (character array).
!  nmes   = the length of msg (number of characters).
!  nerr   = the error number (not used).
!  level  = the error level..
!           0 or 1 means recoverable (control returns to caller).
!           2 means fatal (run is aborted--see note below).
!  ni     = number of integers (0, 1, or 2) to be printed with message.
!  i1,i2  = integers to be printed, depending on ni.
!  nr     = number of reals (0, 1, or 2) to be printed with message.
!  r1,r2  = reals to be printed, depending on nr.
!
!  note..  this routine is machine-dependent and specialized for use
!  in limited context, in the following ways..
!  1. the argument msg is assumed to be of type character, and
!     the message is printed with a format of (1x,a).
!  2. the message is assumed to take only one line.
!     multi-line messages are generated by repeated calls.
!  3. if level = 2, control passes to the statement   stop
!     to abort the run.  this statement may be machine-dependent.
!  4. r1 and r2 are assumed to be in double precision and are printed
!     in d21.13 format.
!
!***routines called  ixsav
!***revision history  (yymmdd)
!   920831  date written
!   921118  replaced mflgsv/lunsav by ixsav. (ach)
!   930329  modified prologue to slatec format. (fnf)
!   930407  changed msg from character*1 array to variable. (fnf)
!   930922  minor cosmetic change. (fnf)
!***end prologue  xerrwd
!
!*internal notes:
!
! for a different default logical unit number, ixsav (or a subsidiary
! routine that it calls) will need to be modified.
! for a different run-abort command, change the statement following
! statement 100 at the end.
!-----------------------------------------------------------------------
! subroutines called by xerrwd.. none
! function routine called by xerrwd.. ixsav
!-----------------------------------------------------------------------
!**end
!
!  declare arguments.
!
      real(kind=dp) r1, r2
      integer nmes, nerr, level, ni, i1, i2, nr
      character*(*) msg
!
!  declare local variables.
!
      integer lunit, mesflg !, ixsav
!
!  get logical unit number and message print flag.
!
!***first executable statement  xerrwd
      lunit = ixsav (1, 0, .false.)
      mesflg = ixsav (2, 0, .false.)
      if (mesflg .eq. 0) go to 100
!
!  write the message.
!
      write (lunit,10)  msg
   10 format(1x,a)
      if (ni .eq. 1) write (lunit, 20) i1
   20 format(6x,'in above message,  i1 =',i10)
      if (ni .eq. 2) write (lunit, 30) i1,i2
   30 format(6x,'in above message,  i1 =',i10,3x,'i2 =',i10)
      if (nr .eq. 1) write (lunit, 40) r1
   40 format(6x,'in above message,  r1 =',d21.13)
      if (nr .eq. 2) write (lunit, 50) r1,r2
   50 format(6x,'in above,  r1 =',d21.13,3x,'r2 =',d21.13)
!
!  abort the run if level = 2.
!
  100 if (level .ne. 2) return
      stop
!----------------------- end of subroutine xerrwd ----------------------
      end subroutine xerrwd
!deck xsetf
      subroutine xsetf (mflag)
!***begin prologue  xsetf
!***purpose  reset the error print control flag.
!***category  r3a
!***type      all (xsetf-a)
!***keywords  error control
!***author  hindmarsh, alan c., (llnl)
!***description
!
!   xsetf sets the error print control flag to mflag:
!      mflag=1 means print all messages (the default).
!      mflag=0 means no printing.
!
!***see also  xerrwd, xerrwv
!***references  (none)
!***routines called  ixsav
!***revision history  (yymmdd)
!   921118  date written
!   930329  added slatec format prologue. (fnf)
!   930407  corrected see also section. (fnf)
!   930922  made user-callable, and other cosmetic changes. (fnf)
!***end prologue  xsetf
!
! subroutines called by xsetf.. none
! function routine called by xsetf.. ixsav
!-----------------------------------------------------------------------
!**end
      integer mflag, junk !, ixsav
!
!***first executable statement  xsetf
      if (mflag .eq. 0 .or. mflag .eq. 1) junk = ixsav (2,mflag,.true.)
      return
!----------------------- end of subroutine xsetf -----------------------
      end subroutine xsetf
!deck xsetun
      subroutine xsetun (lun)
!***begin prologue  xsetun
!***purpose  reset the logical unit number for error messages.
!***category  r3b
!***type      all (xsetun-a)
!***keywords  error control
!***description
!
!   xsetun sets the logical unit number for error messages to lun.
!
!***author  hindmarsh, alan c., (llnl)
!***see also  xerrwd, xerrwv
!***references  (none)
!***routines called  ixsav
!***revision history  (yymmdd)
!   921118  date written
!   930329  added slatec format prologue. (fnf)
!   930407  corrected see also section. (fnf)
!   930922  made user-callable, and other cosmetic changes. (fnf)
!***end prologue  xsetun
!
! subroutines called by xsetun.. none
! function routine called by xsetun.. ixsav
!-----------------------------------------------------------------------
!**end
      integer lun, junk !, ixsav
!
!***first executable statement  xsetun
      if (lun .gt. 0) junk = ixsav (1,lun,.true.)
      return
!----------------------- end of subroutine xsetun ----------------------
      end subroutine xsetun
!deck ixsav
      integer function ixsav (ipar, ivalue, iset)
!***begin prologue  ixsav
!***subsidiary
!***purpose  save and recall error message control parameters.
!***category  r3c
!***type      all (ixsav-a)
!***author  hindmarsh, alan c., (llnl)
!***description
!
!  ixsav saves and recalls one of two error message parameters:
!    lunit, the logical unit number to which messages are printed, and
!    mesflg, the message print flag.
!  this is a modification of the slatec library routine j4save.
!
!  saved local variables..
!   lunit  = logical unit number for messages.  the default is obtained
!            by a call to iumach (may be machine-dependent).
!   mesflg = print control flag..
!            1 means print all messages (the default).
!            0 means no printing.
!
!  on input..
!    ipar   = parameter indicator (1 for lunit, 2 for mesflg).
!    ivalue = the value to be set for the parameter, if iset = .true.
!    iset   = logical flag to indicate whether to read or write.
!             if iset = .true., the parameter will be given
!             the value ivalue.  if iset = .false., the parameter
!             will be unchanged, and ivalue is a dummy argument.
!
!  on return..
!    ixsav = the (old) value of the parameter.
!
!***see also  xerrwd, xerrwv
!***routines called  iumach
!***revision history  (yymmdd)
!   921118  date written
!   930329  modified prologue to slatec format. (fnf)
!   930915  added iumach call to get default output unit.  (ach)
!   930922  minor cosmetic changes. (fnf)
!   010425  type declaration for iumach added. (ach)
!***end prologue  ixsav
!
! subroutines called by ixsav.. none
! function routine called by ixsav.. iumach
!-----------------------------------------------------------------------
!**end
      logical iset
      integer ipar, ivalue
!-----------------------------------------------------------------------
      integer lunit, mesflg!, iumach
!-----------------------------------------------------------------------
! the following fortran-77 declaration is to cause the values of the
! listed (local) variables to be saved between calls to this routine.
!-----------------------------------------------------------------------
      save lunit, mesflg
      data lunit/-1/, mesflg/1/
!
!***first executable statement  ixsav
      if (ipar .eq. 1) then
        if (lunit .eq. -1) lunit = iumach()
        ixsav = lunit
        if (iset) lunit = ivalue
        endif
!
      if (ipar .eq. 2) then
        ixsav = mesflg
        if (iset) mesflg = ivalue
        endif
!
      return
!----------------------- end of function ixsav -------------------------
      end function ixsav
!deck iumach
      integer function iumach()
!***begin prologue  iumach
!***purpose  provide standard output unit number.
!***category  r1
!***type      integer (iumach-i)
!***keywords  machine constants
!***author  hindmarsh, alan c., (llnl)
!***description
! *usage:
!        integer  lout, iumach
!        lout = iumach()
!
! *function return values:
!     lout : the standard logical unit for fortran output.
!
!***references  (none)
!***routines called  (none)
!***revision history  (yymmdd)
!   930915  date written
!   930922  made user-callable, and other cosmetic changes. (fnf)
!***end prologue  iumach
!
!*internal notes:
!  the built-in value of 6 is standard on a wide range of fortran
!  systems.  this may be machine-dependent.
!**end
!***first executable statement  iumach
      iumach = 6
!
      return
!----------------------- end of function iumach ------------------------
      end function iumach

!---- end of subroutine dlsode and its internal procedures
      end subroutine dlsode

    subroutine fun_chem(n, t, v, fct)
      use mod_cbmz_parameters
      use mod_cbmz_global
      use mod_cbmz_function, only: fun
      use mod_cbmz_rates
      implicit none
      integer :: n
      real(kind=dp) :: v(nvar), fct(nvar), t
!      told = time
!      time = t
!      call update_sun()
!      call update_rconst()
!      call update_photo()
!      time = told
      call fun(v, fix, rconst, fct)
      !nfun=nfun+1
    end subroutine fun_chem

    subroutine jac_chem (n, t, v, jf)
      use mod_cbmz_parameters
      use mod_cbmz_global
      use mod_cbmz_jacobiansp
      use mod_cbmz_jacobian, only: jac_sp
      use mod_cbmz_rates
      implicit none
      real(kind=dp) :: v(nvar), t
      integer :: n
      real(kind=dp) :: jf(lu_nonzero)
!      told = time
!      time = t
!      call update_sun()
!      call update_rconst()
!      call update_photo()
!      time = told
      call jac_sp(v, fix, rconst, jf)
      !njac=njac+1
  end subroutine jac_chem

end module mod_cbmz_integrator

! vim: tabstop=8 expandtab shiftwidth=2 softtabstop=2
