! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! The Reaction Rates File
!
! Generated by KPP-2.2.3 symbolic chemistry Kinetics PreProcessor
!       (http://www.cs.vt.edu/~asandu/Software/KPP)
! KPP is distributed under GPL, the general public licence
!       (http://www.gnu.org/copyleft/gpl.html)
! (C) 1995-1997, V. Damian & A. Sandu, CGRER, Univ. Iowa
! (C) 1997-2005, A. Sandu, Michigan Tech, Virginia Tech
!     With important contributions from:
!        M. Damian, Villanova University, USA
!        R. Sander, Max-Planck Institute for Chemistry, Mainz, Germany
!
! File                 : mod_cbmz_Rates.f90
! Time                 : Mon Nov 25 13:41:15 2013
! Working directory    : /scratch/ashalaby/kpp-2.2.3/compare/CBMZ
! Equation file        : mod_cbmz.kpp
! Output root filename : mod_cbmz
!
! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

module mod_cbmz_rates

  use mod_cbmz_parameters
  use mod_cbmz_global

  implicit none

  contains

  ! begin rate law functions from kpp_home/util/userratelaws

  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ! user-defined rate law functions
  ! note: the default argument type for rate laws, as read from the equations file,
  ! is single precision, but all the internal calculations are performed in double
  ! precision
  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  !~~~>  arrhenius
  pure real(kind=dp) function arr( a0,b0,c0 )
    implicit none
    real(kind=dp) , intent(in) :: a0 , b0 , c0
    arr = a0 * exp(-b0/temp) * (temp/300.0_dp)**c0
  end function arr

  !~~~> simplified arrhenius, with two arguments
  !~~~> note: the argument b0 has a changed sign when compared to arr
  pure real(kind=dp) function arr2( a0,b0 )
    implicit none
    real(kind=dp) , intent(in) :: a0 , b0
    arr2 = a0 * exp( b0/temp )
  end function arr2

  !~~~~~~~~~~~~~~>     added by a. shalaby june 2007 --------------------
  pure real(kind=dp) function arr3fc( a0,b0,a1,b1,fc )
    implicit none
    real(kind=dp) , intent(in) :: a0 , b0 , a1 , b1 , fc
    real(kind=dp) :: k1 , k2 , ex
    k1 = a0 * (300._dp/temp)**b0
    k2 = a1 * (300._dp/temp)**b1
    ex = 1._dp/(1._dp+(log10(k1*c_m/k2))**2)
    arr3fc = (k1*c_m / (1._dp+(k1*c_m/k2))) * fc**(ex)
  end function arr3fc

  !~~~~~~~~~~~~~~>     added by a. shalaby june 2007 --------------------
  pure real(kind=dp) function arr4( a0,b0,c0,a1,b1,c1,fc )
    implicit none
    real(kind=dp) , intent(in) :: a0 , b0 , c0 , a1 , b1 , c1 , fc
    real(kind=dp) :: k1 , k2 , ex
    k1 = a0*((300._dp/temp)**b0)*exp(c0/temp)
    k2 = a1*((300._dp/temp)**b1)*exp(c1/temp)
    ex = 1._dp/(1._dp+(log(k1*c_m/k2))**2)
    arr4 = (k2*k1*c_m)/(k2+k1*c_m) * real(fc,dp)**(ex)
  end function arr4

  pure real(kind=dp) function ep2(a0,c0,a2,c2,a3,c3)
    implicit none
    real(kind=dp) , intent(in) :: a0 , c0 , a2 , c2 , a3 , c3
    real(kind=dp) :: k0 , k2 , k3
    k0 = a0 * exp(-c0/temp)
    k2 = a2 * exp(-c2/temp)
    k3 = a3 * exp(-c3/temp)
    k3 = k3*cfactor*1.0e6_dp
    ep2 = k0 + k3/(1.0_dp+k3/k2 )
  end function ep2

  pure real(kind=dp) function ep3(a1,c1,a2,c2)
    implicit none
    real(kind=dp) , intent(in) :: a1 , c1 , a2 , c2
    real(kind=dp) :: k1 , k2
    k1 = a1 * exp(-c1/temp)
    k2 = a2 * exp(-c2/temp)
    ep3 = k1 + k2*(1.0e6_dp*cfactor)
  end function ep3

  pure real(kind=dp) function fall ( a0,b0,c0,a1,b1,c1,cf)
    implicit none
    real(kind=dp) , intent(in) :: a0 , b0 , c0 , a1 , b1 , c1 , cf
    real(kind=dp) :: k0 , k1
    k0 = a0 * exp(-b0/temp)* (temp/300.0_dp)**c0
    k1 = a1 * exp(-b1/temp)* (temp/300.0_dp)**c1
    k0 = k0*cfactor*1.0e6_dp
    k1 = k0/k1
    fall = (k0/(1.0_dp+k1))*cf**(1.0_dp/(1.0_dp+(log10(k1))**2))
  end function fall

  !---------------------------------------------------------------------------

  elemental real(kind=dp) function k_3rd(temp,cair,k0_300k,n,kinf_300k,m,fc)
    implicit none
    real(kind=dp) , intent(in) :: temp ! temperature [k]
    real(kind=dp) , intent(in) :: cair ! air concentration [molecules/cm3]
    real(kind=dp) , intent(in) :: k0_300k   ! low pressure limit at 300 k
    real(kind=dp) , intent(in) :: n         ! exponent for low pressure limit
    real(kind=dp) , intent(in) :: kinf_300k ! high pressure limit at 300 k
    real(kind=dp) , intent(in) :: m         ! exponent for high pressure limit
    real(kind=dp) , intent(in) :: fc        ! broadening factor (usually fc=0.6)
    real(kind=dp) :: zt_help , k0_t , kinf_t , k_ratio
    zt_help = 300._dp/temp
    k0_t    = k0_300k   * zt_help**(n) * cair ! k_0   at current t
    kinf_t  = kinf_300k * zt_help**(m)        ! k_inf at current t
    k_ratio = k0_t/kinf_t
    k_3rd   = k0_t/(1._dp+k_ratio)*fc**(1._dp/(1._dp+log10(k_ratio)**2))
  end function k_3rd

  elemental real(kind=dp) function k_arr (k_298,tdep,temp)
    implicit none
    ! arrhenius function
    real(kind=dp) , intent(in) :: k_298 ! k at t = 298.15k
    real(kind=dp) , intent(in) :: tdep  ! temperature dependence
    real(kind=dp) , intent(in) :: temp  ! temperature
    k_arr = k_298 * exp(tdep*(1._dp/temp-3.3540e-3_dp)) ! 1/298.15=3.3540e-3
  end function k_arr

  pure real(kind=dp) function arr3( a0,b0,a1,b1)
    implicit none
    real(kind=dp) , intent(in) :: a0 , b0 , a1 , b1
    real(kind=dp) :: k1 , k2 , ex
    k1 = a0*c_m*(300._dp/temp)**(b0)
    k2 = a1*(300._dp/temp)**(b1)
    ex = 1.0_dp/(1.0_dp+(log(k1/k2))**2)
    arr3 = (k2*k1*c_m)/(k2+k1*c_m)*0.6_dp**(ex)
  end function arr3

  pure real(kind=dp) function cbz1( temp, c_m )
    real(kind=dp) , intent(in) :: temp , c_m
    real(kind=dp) :: ka , kb , kc
    ka  = 7.2e-15_dp * exp(785._dp/temp)
    kb  = 1.9e-33_dp * exp(725._dp/temp)
    kc  = 4.1e-16_dp * exp(1440._dp/temp)
    cbz1 = (ka+c_m*kb)/(1._dp+(c_m*kb/kc))
  end function cbz1

  pure real(kind=dp) function cbz2( temp, c_m )
    real(kind=dp) , intent(in) :: temp , c_m
    real(kind=dp) :: kd , ke
    kd = 2.3e-13_dp*exp(600._dp/temp)
    ke = 1.7e-33_dp*exp(1000._dp/temp)
    cbz2 = kd + c_m*ke
  end function cbz2

  subroutine loadperoxyparameters
    implicit none
    integer :: i , j

    aperox(jch3o2,jch3o2)   = 2.5e-13_dp
    aperox(jethp,jethp)     = 6.8e-14_dp
    aperox(jc2o3,jc2o3)     = 2.9e-12_dp
    aperox(jano2,jano2)     = 8.0e-12_dp
    aperox(jnap,jnap)       = 1.0e-12_dp
    aperox(jro2,jro2)       = 5.3e-16_dp
    aperox(jisopp,jisopp)   = 3.1e-14_dp
    aperox(jisopn,jisopn)   = 3.1e-14_dp
    aperox(jisopo2,jisopo2) = 3.1e-14_dp
    aperox(jxo2,jxo2)       = 3.1e-14_dp

    bperox(jch3o2,jch3o2)   = 190._dp
    bperox(jethp,jethp)     = 0.0_dp
    bperox(jc2o3,jc2o3)     = 500._dp
    bperox(jano2,jano2)     = 0.0_dp
    bperox(jnap,jnap)       = 0.0_dp
    bperox(jro2,jro2)       = 1980._dp
    bperox(jisopp,jisopp)   = 1000._dp
    bperox(jisopn,jisopn)   = 1000._dp
    bperox(jisopo2,jisopo2) = 1000._dp
    bperox(jxo2,jxo2)       = 1000._dp

    do i = 1, nperox
      do j = 1, nperox
        if ( i /= j ) then
          aperox(i,j) = 2.0_dp*sqrt(aperox(i,i)*aperox(j,j))
          bperox(i,j) = 0.5_dp*(bperox(i,i) + bperox(j,j))
        end if
      end do
    end do
    aperox(jc2o3,jch3o2) = 1.3e-12_dp
    aperox(jch3o2,jc2o3) = 1.3e-12_dp
  end subroutine loadperoxyparameters

  subroutine peroxyrateconstants
    implicit none
    integer :: i , j
    real(kind=dp) , dimension(nperox) :: sperox(nperox)
    real(kind=dp) , dimension(nperox,nperox) :: rk_perox

    sperox(jch3o2)  =  var(indv_ch3o2)!c_ch3o2
    sperox(jethp)   =  var(indv_ethp) ! c_ethp
    sperox(jro2)    =  var(indv_ro2)  ! c_ro2
    sperox(jc2o3)   =  var(indv_c2o3) ! c_c2o3
    sperox(jano2)   =  var(indv_ano2) ! c_ano2
    sperox(jnap)    =  var(indv_nap)  ! c_nap
    sperox(jisopp)  =  var(indv_isopp)! c_isopp
    sperox(jisopn)  =  var(indv_isopn)! c_isopn
    sperox(jisopo2) =  var(indv_isopo2)! c_isopo2
    sperox(jxo2)    =  var(indv_xo2)  ! c_xo2

    do i = 1, nperox
      rk_param(i) = 0.0_dp
    end do

    do i = 1, nperox
      do j = 1, nperox
        rk_perox(i,j) = arr2(aperox(i,j), bperox(i,j))
        rk_param(i) = rk_param(i) + rk_perox(i,j)*sperox(j)
      end do
    end do
  end subroutine peroxyrateconstants

  !real(kind=dp) function react(k)
  !   integer :: k
  !   ! call loadperoxyparameters
  !   call peroxyrateconstants
  !   react = rk_param(k)
  !end function react

  pure real(kind=dp) function kdms(temp)
    implicit none
    real(kind=dp) , intent(in) :: temp
    real(kind=dp) :: k0 , k1 , k2 , k3 , k4
    k0 = temp * exp(-234._dp/temp)
    k1 = 8.46e-10_dp*exp(7230_dp/temp)
    k2 = 2.68e-10_dp*exp(7810_dp/temp)
    k3 = 1.04e11_dp*temp
    k4 = 88.1_dp*exp(7460_dp/temp)
    kdms = (k0+k1+k2)/(k3+k4)
  end function kdms

  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  !
  ! update_rconst - function to update rate constants
  !
  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  subroutine update_rconst ( )
    implicit none
    rconst(1) = (jval_no2)
    rconst(2) = (jval_no3a)
    rconst(3) = (jval_hno2)
    rconst(4) = (jval_hno3)
    rconst(5) = (jval_hno4)
    rconst(6) = (jval_o33p)
    rconst(7) = (jval_o31d)
    rconst(8) = (jval_h2o2)
    rconst(9) = (arr2(3.2e-11_dp,70._dp))
    rconst(10) = (arr2(1.8e-11_dp,110.0_dp))
    ! rconst(11) = constant rate coefficient
    rconst(12) = (c_m*6.0e-34_dp*((temp/300_dp)**(-2.3_dp)))
    rconst(13) = (arr2(8.0e-12_dp,-2060._dp))
    rconst(14) = (arr2(6.5e-12_dp,-120.0_dp))
    rconst(15) = (arr3(9.0e-32_dp,2.0_dp,2.2e-11_dp,0.0_dp))
    rconst(16) = (arr3(9.0e-32_dp,1.5_dp,3.0e-11_dp,0.0_dp))
    rconst(17) = (arr2(2.0e-12_dp,-1400._dp))
    rconst(18) = (arr2(1.2e-13_dp,-2450.0_dp))
    rconst(19) = (arr2(1.6e-12_dp,-940.0_dp))
    rconst(20) = (arr2(1.1e-14_dp,-500.0_dp))
    rconst(21) = (arr2(5.5e-12_dp,-2000._dp))
    rconst(22) = (arr3(7.0e-31_dp,2.6_dp,3.6e-11_dp,0.1_dp))
    rconst(23) = (arr3(2.5e-30_dp,4.4_dp,1.6e-11_dp,1.7_dp))
    ! rconst(24) = constant rate coefficient
    rconst(25) = (arr2(1.8e-11_dp,-390._dp))
    rconst(26) = (cbz1(temp,c_m))
    rconst(27) = (arr2(1.3e-12_dp,380.0_dp))
    rconst(28) = (arr2(4.8e-11_dp,250.0_dp))
    rconst(29) = (arr2(2.9e-12_dp,-160.0_dp))
    rconst(30) = (cbz2(temp,c_m))
    rconst(31) = (cbz2(temp,c_m)*arr2(1.4e-21_dp,2200._dp))
    rconst(32) = (arr2(3.5e-12_dp,250.0_dp))
    rconst(33) = (arr3(1.8e-31_dp,3.2_dp,4.7e-12_dp,1.4_dp))
    ! rconst(34) = constant rate coefficient
    rconst(35) = (arr3(1.8e-31_dp,3.2_dp,4.7e-12_dp,1.4_dp)*arr2(4.76e26_dp,-10900._dp))
    rconst(36) = (arr2(1.5e-11_dp,170.0_dp))
    rconst(37) = (arr2(4.5e-14_dp,-1260.0_dp))
    rconst(38) = (arr3(2.2e-30_dp,3.9_dp,1.5e-12_dp,0.7_dp))
    rconst(39) = (arr2(8.5e-13_dp,-2450._dp))
    ! rconst(40) = constant rate coefficient
    ! rconst(41) = constant rate coefficient
    rconst(42) = (arr3(2.2e-30_dp,3.9_dp,1.5e-12_dp,0.7_dp)*arr2(3.7e26_dp,-11000._dp))
    rconst(43) = (arr2(3.3e-39_dp,530.0_dp))
    rconst(44) = (1.5e-13_dp*(1._dp+2.439e-20_dp*c_m))
    rconst(45) = (arr3(3.0e-31_dp,3.3_dp,1.5e-12_dp,0.0_dp))
    rconst(46) = ((temp**0.667_dp)*arr2(2.8e-14_dp,-1575._dp))
    rconst(47) = ((temp**2)*arr2(1.5e-17_dp,-492._dp))
    ! rconst(48) = constant rate coefficient
    rconst(49) = (arr2(6.7e-12_dp,-600._dp))
    rconst(50) = (jval_ch2oa)
    rconst(51) = (jval_ch2ob)
    ! rconst(52) = constant rate coefficient
    rconst(53) = (arr2(3.4e-13_dp,-1900._dp))
    rconst(54) = (jval_ch3choa)
    rconst(55) = (arr2(5.6e-12_dp,270._dp))
    rconst(56) = (arr2(1.4e-12_dp,-1900._dp))
    rconst(57) = (jval_ch3coch3)
    rconst(58) = ((temp**2)*arr2(5.3e-18_dp,-230._dp))
    rconst(59) = (9.64_dp*jval_ch2oa)
!   rconst(60) = constant rate coefficient
    rconst(61) = (arr2(1.4e-12_dp,-1900._dp))
    rconst(62) = (arr2(1.2e-14_dp,-2630._dp))
    rconst(63) = (arr3(1.0e-28_dp,0.8_dp,8.8e-12_dp,0.0_dp))
    rconst(64) = (arr2(4.2e-15_dp,-1800._dp))
    rconst(65) = (arr2(8.9e-16_dp,-392._dp))
    rconst(66) = (arr2(5.8e-12_dp,478._dp))
    rconst(67) = (arr2(2.9e-11_dp,255._dp))
    rconst(68) = (arr2(3.1e-13_dp,-1010._dp))
!   rconst(69) = constant rate coefficient
    rconst(70) = (arr2(2.1e-12_dp,322._dp))
    rconst(71) = (arr2(1.7e-11_dp,116._dp))
!   rconst(72) = constant rate coefficient
!   rconst(73) = constant rate coefficient
!   rconst(74) = constant rate coefficient
!   rconst(75) = constant rate coefficient
!   rconst(76) = constant rate coefficient
    rconst(77) = (9.04_dp*jval_ch2oa)
    rconst(78) = (arr2(5.4e-17_dp,-500._dp))
    rconst(79) = (arr2(2.55e-11_dp,409._dp))
    rconst(80) = (arr2(1.2e-14_dp,-2013._dp))
    rconst(81) = (arr2(3.0e-12_dp,-446._dp))
!   rconst(82) = constant rate coefficient
!   rconst(83) = constant rate coefficient
    rconst(84) = (5.000e-04_dp*jval_no2)
!   rconst(85) = constant rate coefficient
    rconst(86) = (jval_ch3ooh)
    rconst(87) = (jval_ch3ooh)
    rconst(88) = (jval_ch3ooh)
    rconst(89) = (arr2(3.8e-12_dp,200._dp))
    rconst(90) = (arr2(3.8e-12_dp,200._dp))
    rconst(91) = (arr2(3.8e-12_dp,200._dp))
    rconst(92) = (arr2(1.6e-11_dp,-540._dp))
    rconst(93) = (5.000e-04_dp*jval_no2)
    rconst(94) = (arr3(9.7e-29_dp,5.6_dp,9.3e-12_dp,1.5_dp))
    rconst(95) = (arr3(9.7e-29_dp,5.6_dp,9.3e-12_dp,1.5_dp)*arr2(1.1e28_dp,-14000._dp))
    rconst(96) = (arr2(3.e-12_dp,280._dp))
    rconst(97) = (arr2(2.6e-12_dp,365._dp))
!   rconst(98) = constant rate coefficient
    rconst(99) = (arr2(5.3e-12_dp,360._dp))
!   rconst(100) = constant rate coefficient
!   rconst(101) = constant rate coefficient
!   rconst(102) = constant rate coefficient
!   rconst(103) = constant rate coefficient
!   rconst(104) = constant rate coefficient
!   rconst(105) = constant rate coefficient
!   rconst(106) = constant rate coefficient
!   rconst(107) = constant rate coefficient
!   rconst(108) = constant rate coefficient
!   rconst(109) = constant rate coefficient
!   rconst(110) = constant rate coefficient
!   rconst(111) = constant rate coefficient
!   rconst(112) = constant rate coefficient
    rconst(113) = (arr2(3.8e-13_dp,800._dp))
    rconst(114) = (arr2(7.5e-13_dp,700._dp))
    rconst(115) = (arr2(1.7e-13_dp,1300._dp))
    rconst(116) = (arr2(4.5e-13_dp,1000._dp))
    rconst(117) = (arr2(1.2e-13_dp,1300._dp))
    rconst(118) = (arr2(1.7e-13_dp,1300._dp))
    rconst(119) = (arr2(1.7e-13_dp,1300._dp))
    rconst(120) = (arr2(1.7e-13_dp,1300._dp))
    rconst(121) = (arr2(1.7e-13_dp,1300._dp))
    rconst(122) = (arr2(1.7e-13_dp,1300._dp))
    rconst(123) = (kdms(temp))
    rconst(124) = (arr2(1.0e-12_dp,500._dp))

  end subroutine update_rconst

  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  !
  ! update_photo - function to update photolytical rate constants
  !   arguments :
  !
  ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  subroutine update_photo ( )

   use mod_cbmz_global

    rconst(1) = (jval_no2)
    rconst(2) = (jval_no3a)
    rconst(3) = (jval_hno2)
    rconst(4) = (jval_hno3)
    rconst(5) = (jval_hno4)
    rconst(6) = (jval_o33p)
    rconst(7) = (jval_o31d)
    rconst(8) = (jval_h2o2)
    rconst(50) = (jval_ch2oa)
    rconst(51) = (jval_ch2ob)
    rconst(54) = (jval_ch3choa)
    rconst(57) = (jval_ch3coch3)
    rconst(59) = (9.64_dp*jval_ch2oa)
    rconst(77) = (9.04_dp*jval_ch2oa)
    rconst(84) = (5.000e-04_dp*jval_no2)
    rconst(86) = (jval_ch3ooh)
    rconst(87) = (jval_ch3ooh)
    rconst(88) = (jval_ch3ooh)
    rconst(93) = (5.000e-04_dp*jval_no2)

  end subroutine update_photo

end module mod_cbmz_rates

! vim: tabstop=8 expandtab shiftwidth=2 softtabstop=2
