!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!    This file is part of ICTP RegCM.
!
!    ICTP RegCM is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    ICTP RegCM is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with ICTP RegCM.  If not, see <http://www.gnu.org/licenses/>.
!
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

module mod_che_isorropia

  ! ------------
  ! Terms of Use
  ! ------------
  !
  ! These codes not to be included in any commercial package, or used for any
  ! commercial applications (or for profit) without prior authorization from the
  ! code authors (CF, AN, SP, CP).
  ! The code is to be used for educational or non-profit purposes only.
  ! Any other usage must first have authorization from the authors
  ! ISORROPIA/ISORROPIA-II cannot be modified in any way without the author's
  ! consent.
  ! No portion of the ISORROPIA/ISORROPIA-II source code can be used in other
  ! codes without the author's consent.
  ! The codes are provided as-is, and the authors have no liability from its
  ! usage.
  ! Usage of the model, for any purpose (educational, research, or other) must
  ! acknowledge the usage of these codes, i.e.
  ! Links to the ISORROPIA webpage must be provided
  ! (http://nenes.eas.gatech.edu/ISORROPIA) where users can download the latest
  ! version of the code, as well as manuals and other materials.
  ! The main ISORROPIA reference (Nenes et al., Aquatic Geochemistry, 1998) must
  ! be cited in all publications and documentation.
  ! The main ISORROPIA-II reference (Fountoukis and Nenes, Atmos.Chem.Phys.,
  ! 2007) must be cited in all publications and documentation.
  ! If ISORROPIA/ISORROPIA-II is to be included within another model, some
  ! kind of agreement is required (such as an e-mail confirmation) that
  ! subsequent users will abide to the terms as outlined here.
  ! Documentation and publications using ISORROPIA/ISORROPIA-II should cite the
  ! ISORROPIA webpage.
  !

  use mod_intkinds
  use mod_realkinds

  implicit none

  private

  public :: aerodriver

  contains

  subroutine aerodriver
    use mod_dynparam
    use mod_constants
    use mod_che_common
    use mod_che_indices
    use mod_che_species
    use mod_che_molwg
    implicit none
    character(len = 15) :: scasi
    integer(ik4) , parameter :: nctrl = 2
    integer(ik4) , parameter :: nother = 9
    integer(ik4) , parameter :: ncomp = 8
    integer(ik4) , parameter :: nions = 10
    integer(ik4) , parameter :: ngasaq = 3
    integer(ik4) , parameter :: nslds = 19
    real(rkx) , parameter :: w_so4 = 98.0_rkx
    real(rkx) , parameter :: w_ano3 = 62.0_rkx
    real(rkx) , parameter :: w_anh4 = 18.0_rkx
    real(rkx) , parameter :: massfactor = 1000.0_rkx
    real(rkx) , parameter :: dtaesolv = 900.0_rkx
    real(rkx) , parameter :: conmin = 1.0e-30_rkx

    real(rkx) , dimension(ncomp) :: wi , wt
    real(rkx) , dimension(ngasaq) :: gas
    real(rkx) , dimension(nslds) :: aersld
    real(rkx) , dimension(nions+ngasaq+2) :: aerliq
    real(rkx) , dimension(nctrl) :: cntrl
    real(rkx) , dimension(nother) :: other
    integer(ik4) :: i , j , k
    real(rkx) :: tempi , rhi

    do k = 1 , kz
      do i = ici1 , ici2
        do j = jci1 , jci2
          cntrl(1) = d_zero
          cntrl(2) = 1.0_rkx
          wt = d_zero
          gas = d_zero
          aerliq = d_zero
          aersld = d_zero
          scasi = ''
          other = d_zero
          tempi = ctb3d(j,i,k)
          rhi = crhb3d(j,i,k)
          if ( rhi>1.0 ) rhi = 0.99

          wi = d_zero
          wi(2) = max(chib3d(j,i,k,ih2so4)/w_so4*crhob3d(j,i,k)*massfactor,   &
                      conmin)                                     !so4
          wi(3) = max((chib3d(j,i,k,inh3)/w_nh3+chib3d(j,i,k,ianh4)/w_anh4) &
                      *crhob3d(j,i,k)*massfactor,conmin)          !nh4
          wi(4) = max((chib3d(j,i,k,ihno3)/w_hno3+chib3d(j,i,k,iano3)/ &
                       w_ano3)*crhob3d(j,i,k)*massfactor,conmin)  !no3

          call isoropia(wi,rhi,tempi,cntrl,wt,gas,aerliq,aersld,scasi,other)
!
! add to the chemical tendency the tendendy linked to thermo equilibrium
          chemten(j,i,k,ih2so4) = chemten(j,i,k,ih2so4) + (wt(2)*w_so4/massfactor/crhob3d(j,i,k)      &
                                *cpsb(j,i)-chib3d(j,i,k,ih2so4)*cpsb(j,i))    &
                                /dtaesolv                        !so4
          chemten(j,i,k,inh3) = chemten(j,i,k,inh3) + (gas(1)*w_nh3/massfactor/crhob3d(j,i,k)     &
                                *cpsb(j,i)-chib3d(j,i,k,inh3)*cpsb(j,i))    &
                                /dtaesolv                        !nh3
          chemten(j,i,k,ihno3) = chemten(j,i,k,ihno3) + (gas(2)*w_hno3/massfactor/crhob3d(j,i,k)   &
                                 *cpsb(j,i)-chib3d(j,i,k,ihno3)*cpsb(j,i))  &
                                 /dtaesolv                       !no3
          chemten(j,i,k,ianh4) = chemten(j,i,k,ianh4) + ((wt(3)-gas(1))*w_anh4/massfactor/         &
                                  crhob3d(j,i,k)*cpsb(j,i)-                 &
                                  chib3d(j,i,k,ianh4)*                      &
                                  cpsb(j,i))/dtaesolv            !anh4
          chemten(j,i,k,iano3) = chemten(j,i,k,iano3) + ((wt(4)-gas(2))*w_ano3/massfactor/            &
                                crhob3d(j,i,k)*cpsb(j,i)-chib3d(j,i,k,iano3) &
                                *cpsb(j,i))/dtaesolv             !ano3
        end do
      end do
    end do
  end subroutine aerodriver
!
! =====================================================================
!
! *** isorropia code ii
! *** subroutine isoropia
! *** this subroutine is the master routine for the isorropia
!     thermodynamic equilibrium aerosol model (version 1.1 and above)
!
! ======================= arguments / usage ===========================
!
!  inut:
!  1. [wi]
!     real(rkx) array of length [8].
!     concentrations , expressed in moles/m3. depending on the type of
!     problem solved (specified in cntrl(1)), wi contains either
!     gas+aerosol or aerosol only concentratios.
!     wi(1) - sodium
!     wi(2) - sulfate
!     wi(3) - ammonium
!     wi(4) - nitrate
!     wi(5) - chloride
!     wi(6) - calcium
!     wi(7) - potassium
!     wi(8) - magnesium
!
!  2. [rhi]
!     real(rkx) variable.
!     ambient relative humidity expressed on a (0 ,1) scale.
!
!  3. [tempi]
!     real(rkx) variable.
!     ambient temperature expressed in kelvins.
!
!  4. [cntrl]
!     real(rkx) array of length [2].
!     parameters that control the type of problem solved.
!
!     cntrl(1): defines the type of problem solved.
!     0 - forward problem is solved. in this case , array wi contains
!         gas and aerosol concentrations together.
!     1 - reverse problem is solved. in this case , array wi contains
!         aerosol concentrations only.
!
!     cntrl(2): defines the state of the aerosol
!     0 - the aerosol can have both solid+liquid phases (deliquescent)
!     1 - the aerosol is in only liquid state (metastable aerosol)
!
!  ouput:
!  1. [wt]
!     real(rkx) array of length [8].
!     total concentrations (gas+aerosol) of species , expressed in moles/m3.
!     if the foreward probelm is solved (cntrl(1) = 0), array wt is
!     identical to array wi.
!     wt(1) - total sodium
!     wt(2) - total sulfate
!     wt(3) - total ammonium
!     wt(4) - total nitrate
!     wt(5) - total chloride
!     wt(6) - total calcium
!     wt(7) - total potassium
!     wt(8) - total magnesium
!
!  2. [gas]
!     real(rkx) array of length [03].
!     gaseous species concentrations , expressed in moles/m3.
!     gas(1) - nh3
!     gas(2) - hno3
!     gas(3) - hcl
!
!  3. [aerliq]
!     real(rkx) array of length [15].
!     liquid aerosol species concentrations , expressed in moles/m3.
!     aerliq(01) - h+(aq)
!     aerliq(02) - na+(aq)
!     aerliq(03) - nh4+(aq)
!     aerliq(04) - cl-(aq)
!     aerliq(05) - so4--(aq)
!     aerliq(06) - hso4-(aq)
!     aerliq(07) - no3-(aq)
!     aerliq(08) - h2o
!     aerliq(09) - nh3(aq) (undissociated)
!     aerliq(10) - hncl(aq) (undissociated)
!     aerliq(11) - hno3(aq) (undissociated)
!     aerliq(12) - oh-(aq)
!     aerliq(13) - ca2+(aq)
!     aerliq(14) - k+(aq)
!     aerliq(15) - mg2+(aq)
!
!  4. [aersld]
!     real(rkx) array of length [19].
!     solid aerosol species concentrations , expressed in moles/m3.
!     aersld(01) - nano3(s)
!     aersld(02) - nh4no3(s)
!     aersld(03) - nacl(s)
!     aersld(04) - nh4cl(s)
!     aersld(05) - na2so4(s)
!     aersld(06) - (nh4)2so4(s)
!     aersld(07) - nahso4(s)
!     aersld(08) - nh4hso4(s)
!     aersld(09) - (nh4)4h(so4)2(s)
!     aersld(10) - caso4(s)
!     aersld(11) - ca(no3)2(s)
!     aersld(12) - cacl2(s)
!     aersld(13) - k2so4(s)
!     aersld(14) - khso4(s)
!     aersld(15) - kno3(s)
!     aersld(16) - kcl(s)
!     aersld(17) - mgso4(s)
!     aersld(18) - mg(no3)2(s)
!     aersld(19) - mgcl2(s)
!
!  5. [scasi]
!     character*15 variable.
!     returns the subcase which the input corresponds to.
!
!  6. [other]
!     real(rkx) array of length [9].
!     returns solution information.
!
!     other(1): shows if aerosol water exists.
!     0 - aerosol is wet
!     1 - aerosol is dry
!
!     other(2): aerosol sulfate ratio , defined as (in moles/m3) :
!               (total ammonia + total na) / (total sulfate)
!
!     other(3): sulfate ratio based on aerosol properties that defines
!               a sulfate poor system:
!               (aerosol ammonia + aerosol na) / (aerosol sulfate)
!
!     other(4): aerosol sodium ratio , defined as (in moles/m3) :
!               (total na) / (total sulfate)
!
!     other(5): ionic strength of the aqueous aerosol (if it exists).
!
!     other(6): total number of calls to the activity coefficient
!               calculation subroutine.
!
!     other(7): sulfate ratio with crustal species , defined as (in moles/m3) :
!               (total ammonia + total crustal species + total na) / (total sulfate)
!
!     other(8): crustal species + sodium ratio , defined as (in moles/m3) :
!               (total crustal species + total na) / (total sulfate)
!
!     other(9): crustal species ratio , defined as (in moles/m3) :
!               (total crustal species) / (total sulfate)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine isoropia(wi,rhi,tempi,cntrl,wt,gas,aerliq,aersld,scasi,    &
                        other)
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
    real(rkx) :: rhi , tempi
    integer(ik4) , parameter :: nctrl = 2
    integer(ik4) , parameter :: nother = 9
    character(len = 15) :: scasi
    real(rkx) , dimension(ncomp) :: wi , wt
    real(rkx) , dimension(ngasaq) :: gas
    real(rkx) , dimension(nslds) :: aersld
    real(rkx) , dimension(nions+ngasaq+2) :: aerliq
    real(rkx) , dimension(nctrl) :: cntrl
    real(rkx) , dimension(nother) :: other
!
! *** default values *************************************************
!
    temp = 298.0_rkx
    r = 82.0567e-6_rkx
    rh = 0.9_rkx
    eps = 1e-6_rkx
    maxit = 100
    tiny1 = epsilon(1.0_rkx)
    great = 1e10_rkx
    zero = 0.0_rkx
    one = 1.0_rkx
    nsweep = 4
    tiny2 = 1e-11_rkx
    ndiv = 5
!
    molal = 0.0_rkx
    molalr = 0.0_rkx
    gama = 0.1_rkx
    gamou = 1e10_rkx
    gamin = 1e10_rkx
    calain = .true.
    calaou = .true.
    epsact = 5e-2_rkx
    iclact = 0
    iacalc = 1
    nadj = 0
    wftyp = 2
!
    errstk = 0
    errmsg = ' '
    nofer = 0
    stkofl = .false.
!
    iprob = 0
    metstbl = 0
!
    version = '2.1 (07/19/09)'
!
! *** other parameters ***********************************************
!
    smw = (/58.5,142.,85.0,132.,80.0,53.5,98.0,98.0,115.,63.0,36.5,120.,  &
          247.,136.1,164.,111.,174.2,136.1,101.1,74.5,120.3,148.3,95.2/)
    imw = (/1.0,23.0,18.0,35.5,96.0,97.0,62.0,40.1,39.1,24.3/)
    wmw = (/23.0,98.0,17.0,63.0,36.5,40.1,39.1,24.3/)
!
    zz = (/1,2,1,2,1,1,2,1,1,1,1,1,2,4,2,2,2,1,1,1,4,2,2/)
    z = (/1,1,1,1,2,1,1,2,1,2/)
!
! *** zsr relationship parameters **************************************
!
! awa = ammonium sulfate
!
    awas = (/187.72,187.72,187.72,187.72,187.72,187.72,187.72,187.72,     &
           187.72,187.72,158.13,134.41,115.37,100.10,87.86,78.00,70.00,   &
           63.45,58.02,53.46,49.59,46.26,43.37,40.84,38.59,36.59,34.79,   &
           33.16,31.67,30.31,29.07,27.91,26.84,25.84,24.91,24.03,23.21,   &
           22.44,21.70,21.01,20.34,19.71,19.11,18.54,17.99,17.46,16.95,   &
           16.46,15.99,15.54,15.10,14.67,14.26,13.86,13.47,13.09,12.72,   &
           12.36,12.01,11.67,11.33,11.00,10.68,10.37,10.06,9.75,9.45,9.15,&
           8.86,8.57,8.29,8.01,7.73,7.45,7.18,6.91,6.64,6.37,6.10,5.83,   &
           5.56,5.29,5.02,4.74,4.47,4.19,3.91,3.63,3.34,3.05,2.75,2.45,   &
           2.14,1.83,1.51,1.19,0.87,0.56,0.26,0.1/)
!
! aws = sodium nitrate
!
    awsn = (/394.54,394.54,394.54,394.54,394.54,394.54,394.54,394.54,     &
           394.54,394.54,338.91,293.01,254.73,222.61,195.56,172.76,153.53,&
           137.32,123.65,112.08,102.26,93.88,86.68,80.45,75.02,70.24,     &
           66.02,62.26,58.89,55.85,53.09,50.57,48.26,46.14,44.17,42.35,   &
           40.65,39.06,37.57,36.17,34.85,33.60,32.42,31.29,30.22,29.20,   &
           28.22,27.28,26.39,25.52,24.69,23.89,23.12,22.37,21.65,20.94,   &
           20.26,19.60,18.96,18.33,17.72,17.12,16.53,15.96,15.40,14.85,   &
           14.31,13.78,13.26,12.75,12.25,11.75,11.26,10.77,10.29,9.82,    &
           9.35,8.88,8.42,7.97,7.52,7.07,6.62,6.18,5.75,5.32,4.89,4.47,   &
           4.05,3.64,3.24,2.84,2.45,2.07,1.70,1.34,0.99,0.65,0.31,0.1/)
!
! aws = sodium chloride
!
    awsc = (/28.16,28.16,28.16,28.16,28.16,28.16,28.16,28.16,28.16,28.16, &
           27.17,26.27,25.45,24.69,23.98,23.33,22.72,22.14,21.59,21.08,   &
           20.58,20.12,19.67,19.24,18.82,18.43,18.04,17.67,17.32,16.97,   &
           16.63,16.31,15.99,15.68,15.38,15.08,14.79,14.51,14.24,13.97,   &
           13.70,13.44,13.18,12.93,12.68,12.44,12.20,11.96,11.73,11.50,   &
           11.27,11.05,10.82,10.60,10.38,10.16,9.95,9.74,9.52,9.31,9.10,  &
           8.89,8.69,8.48,8.27,8.07,7.86,7.65,7.45,7.24,7.04,6.83,6.62,   &
           6.42,6.21,6.00,5.79,5.58,5.36,5.15,4.93,4.71,4.48,4.26,4.03,   &
           3.80,3.56,3.32,3.07,2.82,2.57,2.30,2.04,1.76,1.48,1.20,0.91,   &
           0.61,0.30,0.1/)
!
! awa = ammonium chloride
!
    awac = (/1209.00,1209.00,1209.00,1209.00,1209.00,1209.00,1209.00,     &
           1209.00,1209.00,1209.00,1067.60,949.27,848.62,761.82,686.04,   &
           619.16,559.55,505.92,457.25,412.69,371.55,333.21,297.13,262.81,&
           229.78,197.59,165.98,135.49,108.57,88.29,74.40,64.75,57.69,    &
           52.25,47.90,44.30,41.27,38.65,36.36,34.34,32.52,30.88,29.39,   &
           28.02,26.76,25.60,24.51,23.50,22.55,21.65,20.80,20.00,19.24,   &
           18.52,17.83,17.17,16.54,15.93,15.35,14.79,14.25,13.73,13.22,   &
           12.73,12.26,11.80,11.35,10.92,10.49,10.08,9.67,9.28,8.89,8.51, &
           8.14,7.77,7.42,7.06,6.72,6.37,6.03,5.70,5.37,5.05,4.72,4.40,   &
           4.08,3.77,3.45,3.14,2.82,2.51,2.20,1.89,1.57,1.26,0.94,0.62,   &
           0.31,0.1/)
!
! aws = sodium sulfate
!
    awss = (/24.10,24.10,24.10,24.10,24.10,24.10,24.10,24.10,24.10,24.10, &
           23.17,22.34,21.58,20.90,20.27,19.69,19.15,18.64,18.17,17.72,   &
           17.30,16.90,16.52,16.16,15.81,15.48,15.16,14.85,14.55,14.27,   &
           13.99,13.73,13.47,13.21,12.97,12.73,12.50,12.27,12.05,11.84,   &
           11.62,11.42,11.21,11.01,10.82,10.63,10.44,10.25,10.07,9.89,    &
           9.71,9.53,9.36,9.19,9.02,8.85,8.68,8.51,8.35,8.19,8.02,7.86,   &
           7.70,7.54,7.38,7.22,7.06,6.90,6.74,6.58,6.42,6.26,6.10,5.94,   &
           5.78,5.61,5.45,5.28,5.11,4.93,4.76,4.58,4.39,4.20,4.01,3.81,   &
           3.60,3.39,3.16,2.93,2.68,2.41,2.13,1.83,1.52,1.19,0.86,0.54,   &
           0.25,0.1/)
!
! awa = ammonium bisulfate
!
    awab = (/312.84,312.84,312.84,312.84,312.84,312.84,312.84,312.84,     &
           312.84,312.84,271.43,237.19,208.52,184.28,163.64,145.97,130.79,&
           117.72,106.42,96.64,88.16,80.77,74.33,68.67,63.70,59.30,55.39, &
           51.89,48.76,45.93,43.38,41.05,38.92,36.97,35.18,33.52,31.98,   &
           30.55,29.22,27.98,26.81,25.71,24.67,23.70,22.77,21.90,21.06,   &
           20.27,19.52,18.80,18.11,17.45,16.82,16.21,15.63,15.07,14.53,   &
           14.01,13.51,13.02,12.56,12.10,11.66,11.24,10.82,10.42,10.04,   &
           9.66,9.29,8.93,8.58,8.24,7.91,7.58,7.26,6.95,6.65,6.35,6.05,   &
           5.76,5.48,5.20,4.92,4.64,4.37,4.09,3.82,3.54,3.27,2.99,2.70,   &
           2.42,2.12,1.83,1.52,1.22,0.90,0.59,0.28,0.1/)
!
! aws = sulfuric acid
!
    awsa = (/34.00,33.56,29.22,26.55,24.61,23.11,21.89,20.87,19.99,18.45, &
           17.83,17.26,16.73,16.25,15.80,15.38,14.98,14.61,14.26,13.93,   &
           13.61,13.30,13.01,12.73,12.47,12.21,11.96,11.72,11.49,11.26,   &
           11.04,10.83,10.62,10.42,10.23,10.03,9.85,9.67,9.49,9.31,9.14,  &
           8.97,8.81,8.65,8.49,8.33,8.18,8.02,7.87,7.73,7.58,7.44,7.29,   &
           7.15,7.01,6.88,6.74,6.61,6.47,6.34,6.21,6.07,5.94,5.81,5.68,   &
           5.55,5.43,5.30,5.17,5.04,4.91,4.78,4.65,4.52,4.39,4.26,4.13,   &
           4.00,3.86,3.73,3.59,3.45,3.31,3.17,3.02,2.87,2.71,2.56,2.39,   &
           2.22,2.05,1.87,1.68,1.48,1.27,1.04,0.80,0.55,0.28,0.1/)
!
! awl = (nh4)3h(so4)2
!
    awlc = (/125.37,125.37,125.37,125.37,125.37,125.37,125.37,125.37,     &
           125.37,125.37,110.10,97.50,86.98,78.08,70.49,63.97,58.33,53.43,&
           49.14,45.36,42.03,39.07,36.44,34.08,31.97,30.06,28.33,26.76,   &
           25.32,24.01,22.81,21.70,20.67,19.71,18.83,18.00,17.23,16.50,   &
           15.82,15.18,14.58,14.01,13.46,12.95,12.46,11.99,11.55,11.13,   &
           10.72,10.33,9.96,9.60,9.26,8.93,8.61,8.30,8.00,7.72,7.44,7.17, &
           6.91,6.66,6.42,6.19,5.96,5.74,5.52,5.31,5.11,4.91,4.71,4.53,   &
           4.34,4.16,3.99,3.81,3.64,3.48,3.31,3.15,2.99,2.84,2.68,2.53,   &
           2.37,2.22,2.06,1.91,1.75,1.60,1.44,1.28,1.12,0.95,0.79,0.62,   &
           0.45,0.29,0.14,0.1/)
!
! awa = ammonium nitrate
!
    awan = (/960.19,960.19,960.19,960.19,960.19,960.19,960.19,960.19,     &
           960.19,960.19,853.15,763.85,688.20,623.27,566.92,517.54,473.91,&
           435.06,400.26,368.89,340.48,314.63,291.01,269.36,249.46,231.11,&
           214.17,198.50,184.00,170.58,158.15,146.66,136.04,126.25,117.24,&
           108.97,101.39,94.45,88.11,82.33,77.06,72.25,67.85,63.84,60.16, &
           56.78,53.68,50.81,48.17,45.71,43.43,41.31,39.32,37.46,35.71,   &
           34.06,32.50,31.03,29.63,28.30,27.03,25.82,24.67,23.56,22.49,   &
           21.47,20.48,19.53,18.61,17.72,16.86,16.02,15.20,14.41,13.64,   &
           12.89,12.15,11.43,10.73,10.05,9.38,8.73,8.09,7.47,6.86,6.27,   &
           5.70,5.15,4.61,4.09,3.60,3.12,2.66,2.23,1.81,1.41,1.03,0.67,   &
           0.32,0.1/)
!
! aws = sodium bisulfate
!
    awsb = (/55.99,55.99,55.99,55.99,55.99,55.99,55.99,55.99,55.99,55.99, &
           53.79,51.81,49.99,48.31,46.75,45.28,43.91,42.62,41.39,40.22,   &
           39.10,38.02,36.99,36.00,35.04,34.11,33.21,32.34,31.49,30.65,   &
           29.84,29.04,28.27,27.50,26.75,26.01,25.29,24.57,23.87,23.17,   &
           22.49,21.81,21.15,20.49,19.84,19.21,18.58,17.97,17.37,16.77,   &
           16.19,15.63,15.08,14.54,14.01,13.51,13.01,12.53,12.07,11.62,   &
           11.19,10.77,10.36,9.97,9.59,9.23,8.87,8.53,8.20,7.88,7.57,7.27,&
           6.97,6.69,6.41,6.14,5.88,5.62,5.36,5.11,4.87,4.63,4.39,4.15,   &
           3.92,3.68,3.45,3.21,2.98,2.74,2.49,2.24,1.98,1.72,1.44,1.16,   &
           0.87,0.57,0.28,0.1/)
!
! awp = potassium chloride
!
    awpc = (/172.62,165.75,159.10,152.67,146.46,140.45,134.64,129.03,     &
           123.61,118.38,113.34,108.48,103.79,99.27,94.93,90.74,86.71,    &
           82.84,79.11,75.53,72.09,68.79,65.63,62.59,59.68,56.90,54.23,   &
           51.68,49.24,46.91,44.68,42.56,40.53,38.60,36.76,35.00,33.33,   &
           31.75,30.24,28.81,27.45,26.16,24.94,23.78,22.68,21.64,20.66,   &
           19.74,18.86,18.03,17.25,16.51,15.82,15.16,14.54,13.96,13.41,   &
           12.89,12.40,11.94,11.50,11.08,10.69,10.32,9.96,9.62,9.30,8.99, &
           8.69,8.40,8.12,7.85,7.59,7.33,7.08,6.83,6.58,6.33,6.08,5.84,   &
           5.59,5.34,5.09,4.83,4.57,4.31,4.04,3.76,3.48,3.19,2.90,2.60,   &
           2.29,1.98,1.66,1.33,0.99,0.65,0.30,0.1/)
!
! awp = potassium sulfate
!
    awps = (/1014.82,969.72,926.16,884.11,843.54,804.41,766.68,730.32,    &
           695.30,661.58,629.14,597.93,567.92,539.09,511.41,484.83,459.34,&
           434.89,411.47,389.04,367.58,347.05,327.43,308.69,290.80,273.73,&
           257.47,241.98,227.24,213.22,199.90,187.26,175.27,163.91,153.15,&
           142.97,133.36,124.28,115.73,107.66,100.08,92.95,86.26,79.99,   &
           74.12,68.63,63.50,58.73,54.27,50.14,46.30,42.74,39.44,36.40,   &
           33.59,31.00,28.63,26.45,24.45,22.62,20.95,19.43,18.05,16.79,   &
           15.64,14.61,13.66,12.81,12.03,11.33,10.68,10.09,9.55,9.06,8.60,&
           8.17,7.76,7.38,7.02,6.66,6.32,5.98,5.65,5.31,4.98,4.64,4.31,   &
           3.96,3.62,3.27,2.92,2.57,2.22,1.87,1.53,1.20,0.87,0.57,0.28,   &
           0.1/)
!
! awp = potassium nitrate
!
    awpn = (/1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,     &
           1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,       &
           1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,       &
           1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,       &
           1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,       &
           1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,1000.00,       &
           1000.00,1000.00,953.05,881.09,813.39,749.78,690.09,634.14,     &
           581.77,532.83,487.16,444.61,405.02,368.26,334.18,302.64,273.51,&
           246.67,221.97,199.31,178.56,159.60,142.33,126.63,112.40,99.54, &
           87.96,77.55,68.24,59.92,52.53,45.98,40.2,35.11,30.65,26.75,    &
           23.35,20.40,17.85,15.63,13.72,12.06,10.61,9.35,8.24,7.25,6.37, &
           5.56,4.82,4.12,3.47,2.86,2.28,1.74,1.24,0.79,0.40,0.1/)
!
! awp = potassium bisulfate
!
    awpb = (/55.99,55.99,55.99,55.99,55.99,55.99,55.99,55.99,55.99,55.99, &
           53.79,51.81,49.99,48.31,46.75,45.28,43.91,42.62,41.39,40.22,   &
           39.10,38.02,36.99,36.00,35.04,34.11,33.21,32.34,31.49,30.65,   &
           29.84,29.04,28.27,27.50,26.75,26.01,25.29,24.57,23.87,23.17,   &
           22.49,21.81,21.15,20.49,19.84,19.21,18.58,17.97,17.37,16.77,   &
           16.19,15.63,15.08,14.54,14.01,13.51,13.01,12.53,12.07,11.62,   &
           11.19,10.77,10.36,9.97,9.59,9.23,8.87,8.53,8.20,7.88,7.57,7.27,&
           6.97,6.69,6.41,6.14,5.88,5.62,5.36,5.11,4.87,4.63,4.39,4.15,   &
           3.92,3.68,3.45,3.21,2.98,2.74,2.49,2.24,1.98,1.72,1.44,1.16,   &
           0.87,0.57,0.28,0.1/)
!
! awc = calcium chloride
!
    awcc = (/19.9,19.0,18.15,17.35,16.6,15.89,15.22,14.58,13.99,13.43,    &
           12.90,12.41,11.94,11.50,11.09,10.7,10.34,9.99,9.67,9.37,9.09,  &
           8.83,8.57,8.34,8.12,7.91,7.71,7.53,7.35,7.19,7.03,6.88,6.74,   &
           6.6,6.47,6.35,6.23,6.12,6.01,5.90,5.80,5.70,5.61,5.51,5.42,    &
           5.33,5.24,5.16,5.07,4.99,4.91,4.82,4.74,4.66,4.58,4.50,4.42,   &
           4.34,4.26,4.19,4.11,4.03,3.95,3.87,3.79,3.72,3.64,3.56,3.48,   &
           3.41,3.33,3.25,3.17,3.09,3.01,2.93,2.85,2.76,2.68,2.59,2.50,   &
           2.41,2.32,2.23,2.13,2.03,1.93,1.82,1.71,1.59,1.47,1.35,1.22,   &
           1.07,0.93,0.77,0.61,0.44,0.25,0.1/)
!
! awc = calcium nitrate
!
    awcn = (/32.89,31.46,30.12,28.84,27.64,26.51,25.44,24.44,23.49,22.59, &
           21.75,20.96,20.22,19.51,18.85,18.23,17.64,17.09,16.56,16.07,   &
           15.61,15.17,14.75,14.36,13.99,13.63,13.3,12.98,12.68,12.39,    &
           12.11,11.84,11.59,11.35,11.11,10.88,10.66,10.45,10.24,10.04,   &
           9.84,9.65,9.46,9.28,9.1,8.92,8.74,8.57,8.4,8.23,8.06,7.9,7.73, &
           7.57,7.41,7.25,7.1,6.94,6.79,6.63,6.48,6.33,6.18,6.03,5.89,    &
           5.74,5.60,5.46,5.32,5.17,5.04,4.9,4.76,4.62,4.49,4.35,4.22,    &
           4.08,3.94,3.80,3.66,3.52,3.38,3.23,3.08,2.93,2.77,2.60,2.43,   &
           2.25,2.07,1.87,1.67,1.45,1.22,0.97,0.72,0.44,0.14,0.1/)
!
! awm = magnesium chloride
!
    awmc = (/11.24,10.99,10.74,10.5,10.26,10.03,9.81,9.59,9.38,9.18,8.98, &
           8.79,8.60,8.42,8.25,8.07,7.91,7.75,7.59,7.44,7.29,7.15,7.01,   &
           6.88,6.75,6.62,6.5,6.38,6.27,6.16,6.05,5.94,5.85,5.75,5.65,    &
           5.56,5.47,5.38,5.30,5.22,5.14,5.06,4.98,4.91,4.84,4.77,4.7,    &
           4.63,4.57,4.5,4.44,4.37,4.31,4.25,4.19,4.13,4.07,4.01,3.95,    &
           3.89,3.83,3.77,3.71,3.65,3.58,3.52,3.46,3.39,3.33,3.26,3.19,   &
           3.12,3.05,2.98,2.9,2.82,2.75,2.67,2.58,2.49,2.41,2.32,2.22,    &
           2.13,2.03,1.92,1.82,1.71,1.60,1.48,1.36,1.24,1.11,0.98,0.84,   &
           0.70,0.56,0.41,0.25,0.1/)
!
! awm = magnesium nitrate
!
    awmn = (/12.00,11.84,11.68,11.52,11.36,11.2,11.04,10.88,10.72,10.56,  &
           10.40,10.25,10.09,9.93,9.78,9.63,9.47,9.32,9.17,9.02,8.87,8.72,&
           8.58,8.43,8.29,8.15,8.01,7.87,7.73,7.59,7.46,7.33,7.2,7.07,    &
           6.94,6.82,6.69,6.57,6.45,6.33,6.21,6.01,5.98,5.87,5.76,5.65,   &
           5.55,5.44,5.34,5.24,5.14,5.04,4.94,4.84,4.75,4.66,4.56,4.47,   &
           4.38,4.29,4.21,4.12,4.03,3.95,3.86,3.78,3.69,3.61,3.53,3.45,   &
           3.36,3.28,3.19,3.11,3.03,2.94,2.85,2.76,2.67,2.58,2.49,2.39,   &
           2.3,2.2,2.1,1.99,1.88,1.77,1.66,1.54,1.42,1.29,1.16,1.02,0.88, &
           0.73,0.58,0.42,0.25,0.1/)
!
! awm = magnesium sulfate
!
    awms = (/0.93,2.5,3.94,5.25,6.45,7.54,8.52,9.40,10.19,10.89,11.50,    &
           12.04,12.51,12.90,13.23,13.50,13.72,13.88,13.99,14.07,14.1,    &
           14.09,14.05,13.98,13.88,13.75,13.6,13.43,13.25,13.05,12.83,    &
           12.61,12.37,12.13,11.88,11.63,11.37,11.12,10.86,10.60,10.35,   &
           10.09,9.85,9.6,9.36,9.13,8.9,8.68,8.47,8.26,8.07,7.87,7.69,    &
           7.52,7.35,7.19,7.03,6.89,6.75,6.62,6.49,6.37,6.26,6.15,6.04,   &
           5.94,5.84,5.75,5.65,5.56,5.47,5.38,5.29,5.20,5.11,5.01,4.92,   &
           4.82,4.71,4.60,4.49,4.36,4.24,4.10,3.96,3.81,3.65,3.48,3.30,   &
           3.11,2.92,2.71,2.49,2.26,2.02,1.76,1.50,1.22,0.94,0.64/)

!
! *** problem type (0 = foreward , 1=reverse)
! ******************************
    iprob = nint(cntrl(1))
!
! *** aerosol state (0 = solid+liquid , 1=metastable)
! **********************
    metstbl = nint(cntrl(2))
!
! *** solve foreward problem
! ********************************************
    if ( iprob==0 ) then
      if ( wi(1)+wi(2)+wi(3)+wi(4)+wi(5)+wi(6)+wi(7)+wi(8)<=tiny1 ) then
                                                                     !everything = 0
        call init1(wi,rhi,tempi)
      else if ( wi(1)+wi(4)+wi(5)+wi(6)+wi(7)+wi(8)<=tiny1 ) then      !a ,k ,mg ,na ,cl ,no3 = 0
        call isrp1f(wi,rhi,tempi)
      else if ( wi(1)+wi(5)+wi(6)+wi(7)+wi(8)<=tiny1 ) then            !a ,k ,mg ,na ,cl = 0
        call isrp2f(wi,rhi,tempi)
      else if ( wi(6)+wi(7)+wi(8)<=tiny1 ) then                        !a ,k ,mg = 0
        call isrp3f(wi,rhi,tempi)
      else
        call isrp4f(wi,rhi,tempi)
      end if
!
!   *** solve reverse problem
!   *********************************************
    else if ( wi(1)+wi(2)+wi(3)+wi(4)+wi(5)+wi(6)+wi(7)+wi(8)<=tiny1 )    &
              then                                                   !everything = 0
      call init1(wi,rhi,tempi)
    else if ( wi(1)+wi(4)+wi(5)+wi(6)+wi(7)+wi(8)<=tiny1 ) then        !a ,k ,mg ,na ,cl ,no3 = 0
      call isrp1r(wi,rhi,tempi)
    else if ( wi(1)+wi(5)+wi(6)+wi(7)+wi(8)<=tiny1 ) then              !a ,k ,mg ,na ,cl = 0
      call isrp2r(wi,rhi,tempi)
    else if ( wi(6)+wi(7)+wi(8)<=tiny1 ) then                         !a ,k ,mg = 0
      call isrp3r(wi,rhi,tempi)
    else
      call isrp4r(wi,rhi,tempi)
    end if
!
! *** adjust mass balance
! ***********************************************
    if ( nadj==1 ) call adjust(wi)
!
! *** if metastable and no water - resolve as normal
! ********************
! if (water <= tiny1 .and. metstbl == 1) then
! metstbl = 0
! goto 50
! end if

!
! *** save results to arrays (units = mole/m3)
! ****************************
    gas(1) = gnh3                  ! gaseous aerosol species
    gas(2) = ghno3
    gas(3) = ghcl
!
    do i = 1 , 7               ! liquid aerosol species
      aerliq(i) = molal(i)
    end do
    do i = 1 , ngasaq
      aerliq(7+1+i) = gasaq(i)
    end do
    aerliq(7+1) = water*1.0e3_rkx/18.0_rkx
    aerliq(7+ngasaq+2) = coh
!
    do i = 8 , 10               ! liquid aerosol species
      aerliq(i+5) = molal(i)
    end do
!
    aersld(1) = cnano3              ! solid aerosol species
    aersld(2) = cnh4no3
    aersld(3) = cnacl
    aersld(4) = cnh4cl
    aersld(5) = cna2so4
    aersld(6) = cnh42s4
    aersld(7) = cnahso4
    aersld(8) = cnh4hs4
    aersld(9) = clc
    aersld(10) = ccaso4
    aersld(11) = ccano32
    aersld(12) = ccacl2
    aersld(13) = ck2so4
    aersld(14) = ckhso4
    aersld(15) = ckno3
    aersld(16) = ckcl
    aersld(17) = cmgso4
    aersld(18) = cmgno32
    aersld(19) = cmgcl2
!
    if ( water<=tiny1 ) then        ! dry flag
      other(1) = 1._rkx
    else
      other(1) = 0._rkx
    end if
!
    other(2) = sulrat              ! other stuff
    other(3) = sulratw
    other(4) = sodrat
    other(5) = ionic
    other(6) = iclact
    other(7) = so4rat
    other(8) = crnarat
    other(9) = crrat
!
    scasi = scase
!
    wt(1) = wi(1)                  ! total gas+aerosol phase
    wt(2) = wi(2)
    wt(3) = wi(3)
    wt(4) = wi(4)
    wt(5) = wi(5)
    wt(6) = wi(6)
    wt(7) = wi(7)
    wt(8) = wi(8)

    if ( iprob>0 .and. water>tiny1 ) then
      wt(3) = wt(3) + gnh3
      wt(4) = wt(4) + ghno3
      wt(5) = wt(5) + ghcl
    end if
  end subroutine isoropia

!======================================================================
!
! *** isorropia code
! *** subroutine setparm
! *** this subroutine redefines the solution parameters of isorropia
!
! ======================= arguments / usage ===========================
!
! *** note: if negative values are given for a parameter , it is
!     ignored and the current value is used instead.
!  &
!  inut:
!  1. [wftypi]
!     integer(ik4) :: variable.
!     defines the type of weighting algorithm for the solution in mutual
!     deliquescence regions (mdr's):
!     0 - mdr's are assumed dry. this is equivalent to the approach
!         used by sequilib.
!     1 - the solution is assumed "half" dry and "half" wet throughout
!         the mdr.
!     2 - the solution is a relative-humidity weighted mean of the
!         dry and wet solutions (as defined in nenes et al., 1998)
!
!  2. [iacalci]
!     integer(ik4) :: variable.
!     method of activity coefficient calculation:
!     0 - calculate coefficients during runtime
!     1 - use precalculated tables
!
!  3. [epsi]
!     double precition variable.
!     defines the convergence criterion for all iterative processes
!     in isorropia , except those for activity coefficient calculations
!     (epsacti controls that).
!
!  4. [maxiti]
!     integer(ik4) :: variable.
!     defines the maximum number of iterations for all iterative
!     processes in isorropia , except for activity coefficient calculations
!     (nsweepi controls that).
!
!  5. [nsweepi]
!     integer(ik4) :: variable.
!     defines the maximum number of iterations for activity coefficient
!     calculations.
!
!  6. [epsacti]
!     real(rkx) variable.
!     defines the convergence criterion for activity coefficient
!     calculations.
!
!  7. [ndiv]
!     integer(ik4) :: variable.
!     defines the number of subdivisions needed for the initial root
!     tracking for the bisection method. usually this parameter should
!     not be altered , but is included for completeness.
!
!  8. [nadj]
!     integer(ik4) :: variable.
!     forces the solution obtained to satisfy total mass balance
!     to machine precision
!     0 - no adjustment done (default)
!     1 - do adjustment
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine setparm(wftypi,iacalci,epsi,maxiti,nsweepi,epsacti,ndivi,  &
                       nadji)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: epsacti , epsi
    integer(ik4) :: iacalci , maxiti , nadji , ndivi , nsweepi
    integer(ik4) :: wftypi
!
! *** setup solution parameters
! *****************************************
    if ( wftypi>=0 ) wftyp = wftypi
    if ( iacalci>=0 ) iacalc = iacalci
    if ( epsi>=zero ) eps = epsi
    if ( maxiti>0 ) maxit = maxiti
    if ( nsweepi>0 ) nsweep = nsweepi
    if ( epsacti>=zero ) epsact = epsacti
    if ( ndivi>0 ) ndiv = ndivi
    if ( nadji>=0 ) nadj = nadji
  end subroutine setparm
!
!======================================================================
!
! *** isorropia code
! *** subroutine getparm
! *** this subroutine obtains the current vaules of the solution
!     parameters of isorropia
!
! ======================= arguments / usage ===========================
!
! *** the parameters are those of subroutine setparm
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine getparm(wftypi,iacalci,epsi,maxiti,nsweepi,epsacti,ndivi,  &
                       nadji)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: epsacti , epsi
    integer(ik4) :: iacalci , maxiti , nadji , ndivi , nsweepi
    integer(ik4) :: wftypi
!
! *** get solution parameters
! *******************************************
    wftypi = wftyp
    iacalci = iacalc
    epsi = eps
    maxiti = maxit
    nsweepi = nsweep
    epsacti = epsact
    ndivi = ndiv
    nadji = nadj
  end subroutine getparm
!
!======================================================================
!
! *** isorropia code
! *** subroutine init1
! *** this subroutine initializes all global variables for ammonium
!     sulfate aerosol systems (subroutine isrp1)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine init1(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: coef , rhi , t0 , t0t , tcf , tempi
    integer(ik4) :: i , irh
    real(rkx) , dimension(ncomp) :: wi
!
! *** save input variables in common block
! ******************************
    if ( iprob==0 ) then                   ! forward calculation
      do i = 1 , ncomp
        w(i) = max(wi(i),tiny1)
      end do
    else
      do i = 1 , ncomp                     ! reverse calculation
        waer(i) = max(wi(i),tiny1)
        w(i) = zero
      end do
    end if
    rh = rhi
    temp = tempi
!
! *** calculate equilibrium constants
! ***********************************
    xk1 = 1.015e-2_rkx     ! hso4(aq)         <==> h(aq)     + so4(aq)
    xk21 = 57.639_rkx    ! nh3(g)           <==> nh3(aq)
    xk22 = 1.805e-5_rkx    ! nh3(aq)          <==> nh4(aq)   + oh(aq)
    xk7 = 1.817_rkx      ! (nh4)2so4(s)     <==> 2*nh4(aq) + so4(aq)
    xk12 = 1.382e2_rkx     ! nh4hso4(s)       <==> nh4(aq)   + hso4(aq)
    xk13 = 29.268_rkx    ! (nh4)3h(so4)2(s) <==> 3*nh4(aq) + hso4(aq) + so4(aq)
    xkw = 1.010e-14_rkx    ! h2o              <==> h(aq)     + oh(aq)
!
    if ( int(temp)/=298 ) then       ! for t != 298k or 298.15k
      t0 = 298.15_rkx
      t0t = t0/temp
      coef = 1.0_rkx + log(t0t) - t0t
      xk1 = xk1*exp(8.85_rkx*(t0t-1.0_rkx)+25.140_rkx*coef)
      xk21 = xk21*exp(13.79_rkx*(t0t-1.0_rkx)-5.393_rkx*coef)
      xk22 = xk22*exp(-1.50_rkx*(t0t-1.0_rkx)+26.920_rkx*coef)
      xk7 = xk7*exp(-2.65_rkx*(t0t-1.0_rkx)+38.570_rkx*coef)
      xk12 = xk12*exp(-2.87_rkx*(t0t-1.0_rkx)+15.830_rkx*coef)
      xk13 = xk13*exp(-5.19_rkx*(t0t-1.0_rkx)+54.400_rkx*coef)
      xkw = xkw*exp(-22.52_rkx*(t0t-1.0_rkx)+26.920_rkx*coef)
    end if
    xk2 = xk21*xk22
!
! *** calculate deliquescence relative humidities (unicomponent)
! ********
    drh2so4 = 0.0000_rkx
    drnh42s4 = 0.7997_rkx
    drnh4hs4 = 0.4000_rkx
    drlc = 0.6900_rkx
    if ( int(temp)/=298 ) then
      t0 = 298.15_rkx
      tcf = 1.0_rkx/temp - 1.0_rkx/t0
      drnh42s4 = drnh42s4*exp(80._rkx*tcf)
      drnh4hs4 = drnh4hs4*exp(384._rkx*tcf)
      drlc = drlc*exp(186._rkx*tcf)
    end if
!
! *** calculate mutual deliquescence relative humidities
! ****************
    drmlcab = 0.3780_rkx                ! (nh4)3h(so4)2 & nh4hso4
    drmlcas = 0.6900_rkx                ! (nh4)3h(so4)2 & (nh4)2so4
!!!      if (int(temp) .ne. 298) then      ! for the time being.
!!!         t0       = 298.15d0
!!!         tcf      = 1.0/temp - 1.0/t0
!!!        drmlcab  = drmlcab*exp(507.506*tcf)
!!!         drmlcas  = drmlcas*exp(133.865*tcf)
!!!      end if
!
! *** liquid phase
! ******************************************************
    chno3 = zero
    chcl = zero
    ch2so4 = zero
    coh = zero
    water = tiny1
!
    do i = 1 , npair
      molalr(i) = zero
      gama(i) = 0.1_rkx
      gamin(i) = great
      gamou(i) = great
      m0(i) = 1e5_rkx
    end do
!
    do i = 1 , npair
      gama(i) = 0.1_rkx
    end do
!
    do i = 1 , nions
      molal(i) = zero
    end do
    coh = zero
!
    do i = 1 , ngasaq
      gasaq(i) = zero
    end do
!
! *** solid phase
! *******************************************************
    cnh42s4 = zero
    cnh4hs4 = zero
    cnacl = zero
    cna2so4 = zero
    cnano3 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnahso4 = zero
    clc = zero
    ccaso4 = zero
    ccano32 = zero
    ccacl2 = zero
    ck2so4 = zero
    ckhso4 = zero
    ckno3 = zero
    ckcl = zero
    cmgso4 = zero
    cmgno32 = zero
    cmgcl2 = zero
!
! *** gas phase
! *********************************************************
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
!
! *** calculate zsr parameters
! ******************************************
    irh = min(int(rh*nzsr+0.5_rkx),nzsr)        ! position in zsr arrays
    irh = max(irh,1)
!
! m0(01) = awsc(irh)      ! nacl
! if (m0(01)  <  100.0) then
! ic = m0(01)
! call kmtab(ic ,298.0 ,     gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                            xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),gii ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx) m0(01) = m0(01)*exp(ln10*(gi0-gii))
! end if
!!
! m0(02) = awss(irh)      ! (na)2so4
! if (m0(02)  <  100.0) then
! ic = 3.0*m0(02)
! call kmtab(ic ,298.0 ,     xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,gii ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(02) = m0(02)*exp(ln10*(gi0-gii))
! end if
!!
! m0(03) = awsn(irh)      ! nano3
! if (m0(03)  <  100.0) then
! ic = m0(03)
! call kmtab(ic ,298.0 ,     xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,gii ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(03) = m0(03)*exp(ln10*(gi0-gii))
! end if
!!
    m0(04) = awas(irh)        ! (nh4)2so4
!!      if (m0(04)  <  100.0) then
!!         ic = 3.0*m0(04)
!!         call kmtab(ic ,298.0 ,     xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,
!!     &                             xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx)
!!         call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,gii ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,
!!     &                             xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx)
!!         m0(04) = m0(04)*exp(ln10*(gi0-gii))
!!      end if
!
! m0(05) = awan(irh)      ! nh4no3
! if (m0(05)  <  100.0) then
! ic     = m0(05)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,gii ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(05) = m0(05)*exp(ln10*(gi0-gii))
! end if
!!
! m0(06) = awac(irh)      ! nh4cl
! if (m0(06)  <  100.0) then
! ic = m0(06)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,gii ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(06) = m0(06)*exp(ln10*(gi0-gii))
! end if
!
    m0(07) = awsa(irh)        ! 2h-so4
! if (m0(07)  <  100.0) then
! ic = 3.0*m0(07)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(07) = m0(07)*exp(ln10*(gi0-gii))
! end if
!
    m0(08) = awsa(irh)        ! h-hso4
! if (m0(08)  <  100.0) then     ! these are redundant , because m0(8)
! is not used ic = m0(08)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0
!c         call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,gii ,xx ,xx ,xx ,xx)
! ,xx ,xx ,xx ,xx) & m0(08) = m0(08)*exp(ln10*(gi0-gii))
! end if
!
    m0(09) = awab(irh)        ! nh4hso4
! if (m0(09)  <  100.0) then
! ic = m0(09)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,gii ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(09) = m0(09)*exp(ln10*(gi0-gii))
! end if
!
! m0(12) = awsb(irh)      ! nahso4
! if (m0(12)  <  100.0) then
! ic = m0(12)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,gi0 , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,gii , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) m0(12) = m0(12)*exp(ln10*(gi0-gii))
! end if
!
    m0(13) = awlc(irh)        ! (nh4)3h(so4)2
! if (m0(13)  <  100.0) then
! ic     = 4.0*m0(13)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) g130   = 0.2*(3.0*gi0+2.0*gii)
! call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) g13i   = 0.2*(3.0*gi0+2.0*gii)
! m0(13) = m0(13)*exp(ln10*real(g130-g13i,rkx))
! end if
!
! *** other initializations
! *********************************************
    iclact = 0
    calaou = .true.
    calain = .true.
    frst = .true.
    scase = '??'
    sulratw = 2._rkx
    sodrat = zero
    crnarat = zero
    crrat = zero
    nofer = 0
    stkofl = .false.
    do i = 1 , nerrmx
      errstk(i) = -999
      errmsg(i) = 'message n/a'
    end do
  end subroutine init1

!======================================================================
!
! *** isorropia code
! *** subroutine init2
! *** this subroutine initializes all global variables for ammonium ,
!     nitrate , sulfate aerosol systems (subroutine isrp2)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine init2(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: coef , rhi , t0 , t0t , tcf , tempi
    integer(ik4) :: i , irh
    real(rkx) , dimension(ncomp) :: wi
!
! *** save input variables in common block
! ******************************
    if ( iprob==0 ) then                   ! forward calculation
      do i = 1 , ncomp
        w(i) = max(wi(i),tiny1)
      end do
    else
      do i = 1 , ncomp                     ! reverse calculation
        waer(i) = max(wi(i),tiny1)
        w(i) = zero
      end do
    end if
    rh = rhi
    temp = tempi
!
! *** calculate equilibrium constants
! ***********************************
    xk1 = 1.015e-2_rkx     ! hso4(aq)         <==> h(aq)     + so4(aq)
    xk21 = 57.639_rkx    ! nh3(g)           <==> nh3(aq)
    xk22 = 1.805e-5_rkx    ! nh3(aq)          <==> nh4(aq)   + oh(aq)
    xk4 = 2.511e6_rkx      ! hno3(g)          <==> h(aq)     + no3(aq) ! isorr
! xk4  = 3.638e6_rkx   ! hno3(g)          <==> h(aq)     + no3(aq) ! sequil
    xk41 = 2.100e5_rkx     ! hno3(g)          <==> hno3(aq)
    xk7 = 1.817_rkx      ! (nh4)2so4(s)     <==> 2*nh4(aq) + so4(aq)
    xk10 = 5.746e-17_rkx   ! nh4no3(s)        <==> nh3(g)    + hno3(g) ! isorr
! xk10 = 2.985e-17_rkx ! nh4no3(s)        <==> nh3(g)    + hno3(g) ! sequil
    xk12 = 1.382e2_rkx     ! nh4hso4(s)       <==> nh4(aq)   + hso4(aq)
    xk13 = 29.268_rkx    ! (nh4)3h(so4)2(s) <==> 3*nh4(aq) + hso4(aq) + so4(aq)
    xkw = 1.010e-14_rkx    ! h2o              <==> h(aq)     + oh(aq)
!
    if ( int(temp)/=298 ) then       ! for t != 298k or 298.15k
      t0 = 298.15_rkx
      t0t = t0/temp
      coef = 1.0_rkx + log(t0t) - t0t
      xk1 = xk1*exp(8.85_rkx*(t0t-1.0_rkx)+25.140_rkx*coef)
      xk21 = xk21*exp(13.79_rkx*(t0t-1.0_rkx)-5.393_rkx*coef)
      xk22 = xk22*exp(-1.50_rkx*(t0t-1.0_rkx)+26.920_rkx*coef)
      xk4 = xk4*exp(29.17_rkx*(t0t-1.0_rkx)+16.830_rkx*coef)        !isorr
!   xk4 = xk4 *exp( 29.47_rkx*(t0t-1.0_rkx) + 16.840_rkx*coef) ! sequil
      xk41 = xk41*exp(29.17_rkx*(t0t-1.0_rkx)+16.830_rkx*coef)
      xk7 = xk7*exp(-2.65_rkx*(t0t-1.0_rkx)+38.570_rkx*coef)
      xk10 = xk10*exp(-74.38_rkx*(t0t-1.0_rkx)+6.120_rkx*coef)       ! isorr
!   xk10 = xk10*exp(-75.11_rkx*(t0t-1.0_rkx) + 13.460_rkx*coef) ! sequil
      xk12 = xk12*exp(-2.87_rkx*(t0t-1.0_rkx)+15.830_rkx*coef)
      xk13 = xk13*exp(-5.19_rkx*(t0t-1.0_rkx)+54.400_rkx*coef)
      xkw = xkw*exp(-22.52_rkx*(t0t-1.0_rkx)+26.920_rkx*coef)
    end if
    xk2 = xk21*xk22
    xk42 = xk4/xk41
!
! *** calculate deliquescence relative humidities (unicomponent)
! ********
    drh2so4 = zero
    drnh42s4 = 0.7997_rkx
    drnh4hs4 = 0.4000_rkx
    drnh4no3 = 0.6183_rkx
    drlc = 0.6900_rkx
    if ( int(temp)/=298 ) then
      t0 = 298.15_rkx
      tcf = 1.0_rkx/temp - 1.0_rkx/t0
      drnh4no3 = drnh4no3*exp(852._rkx*tcf)
      drnh42s4 = drnh42s4*exp(80._rkx*tcf)
      drnh4hs4 = drnh4hs4*exp(384._rkx*tcf)
      drlc = drlc*exp(186._rkx*tcf)
      drnh4no3 = min(drnh4no3,drnh42s4)      ! adjust for drh crossover at t<271k
    end if
!
! *** calculate mutual deliquescence relative humidities
! ****************
    drmlcab = 0.3780_rkx                ! (nh4)3h(so4)2 & nh4hso4
    drmlcas = 0.6900_rkx                ! (nh4)3h(so4)2 & (nh4)2so4
    drmasan = 0.6000_rkx                ! (nh4)2so4     & nh4no3
! if (int(temp) .ne. 298) then    ! for the time being
! t0       = 298.15d0
! tcf      = 1.0/temp - 1.0/t0
! drmlcab  = drmlcab*exp( 507.506*tcf)
! drmlcas  = drmlcas*exp( 133.865*tcf)
! drmasan  = drmasan*exp(1269.068*tcf)
! end if
!
! *** liquid phase
! ******************************************************
    chno3 = zero
    chcl = zero
    ch2so4 = zero
    coh = zero
    water = tiny1
!
    do i = 1 , npair
      molalr(i) = zero
      gama(i) = 0.1_rkx
      gamin(i) = great
      gamou(i) = great
      m0(i) = 1e5_rkx
    end do
!
    do i = 1 , npair
      gama(i) = 0.1_rkx
    end do
!
    do i = 1 , nions
      molal(i) = zero
    end do
    coh = zero
!
    do i = 1 , ngasaq
      gasaq(i) = zero
    end do
!
! *** solid phase ******************************************************
!
    cnh42s4 = zero
    cnh4hs4 = zero
    cnacl = zero
    cna2so4 = zero
    cnano3 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnahso4 = zero
    clc = zero
    ccaso4 = zero
    ccano32 = zero
    ccacl2 = zero
    ck2so4 = zero
    ckhso4 = zero
    ckno3 = zero
    ckcl = zero
    cmgso4 = zero
    cmgno32 = zero
    cmgcl2 = zero
!
! *** gas phase ********************************************************
!
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
!
! *** calculate zsr parameters *****************************************
!
    irh = min(int(rh*nzsr+0.5_rkx),nzsr)        ! position in zsr arrays
    irh = max(irh,1)
!
! m0(01) = awsc(irh)      ! nacl
! if (m0(01)  <  100.0) then
! ic = m0(01)
! call kmtab(ic ,298.0 ,     gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),gii ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(01) = m0(01)*exp(ln10*(gi0-gii))
! end if
!
! m0(02) = awss(irh)      ! (na)2so4
! if (m0(02)  <  100.0) then
! ic = 3.0*m0(02)
! call kmtab(ic ,298.0 ,     xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,gii ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(02) = m0(02)*exp(ln10*(gi0-gii))
! end if
!
! m0(03) = awsn(irh)      ! nano3
! if (m0(03)  <  100.0) then
! ic = m0(03)
! call kmtab(ic ,298.0 ,     xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,gii ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(03) = m0(03)*exp(ln10*(gi0-gii))
! end if
!
    m0(04) = awas(irh)        ! (nh4)2so4
! if (m0(04)  <  100.0) then
! ic = 3.0*m0(04)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,gii ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(04) = m0(04)*exp(ln10*(gi0-gii))
! end if
!
    m0(05) = awan(irh)        ! nh4no3
! if (m0(05)  <  100.0) then
! ic     = m0(05)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,gii ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(05) = m0(05)*exp(ln10*(gi0-gii))
! end if
!
! m0(06) = awac(irh)      ! nh4cl
! if (m0(06)  <  100.0) then
! ic = m0(06)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,gii ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(06) = m0(06)*exp(ln10*(gi0-gii))
! end if
!
    m0(07) = awsa(irh)        ! 2h-so4
! if (m0(07)  <  100.0) then
! ic = 3.0*m0(07)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(07) = m0(07)*exp(ln10*(gi0-gii))
! end if
!
    m0(08) = awsa(irh)        ! h-hso4
! if (m0(08)  <  100.0) then     ! these are redundant , because m0(8)
! is not used ic = m0(08)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0
!c         call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,gii ,xx ,xx ,xx ,xx)
! ,xx ,xx ,xx ,xx) & m0(08) = m0(08)*exp(ln10*(gi0-gii))
! end if
!
    m0(09) = awab(irh)        ! nh4hso4
! if (m0(09)  <  100.0) then
! ic = m0(09)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,gii ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(09) = m0(09)*exp(ln10*(gi0-gii))
! end if
!
! m0(12) = awsb(irh)      ! nahso4
! if (m0(12)  <  100.0) then
! ic = m0(12)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,gi0 , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,gii , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) m0(12) = m0(12)*exp(ln10*(gi0-gii))
! end if
!
    m0(13) = awlc(irh)        ! (nh4)3h(so4)2
! if (m0(13)  <  100.0) then
! ic     = 4.0*m0(13)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) g130   = 0.2*(3.0*gi0+2.0*gii)
! call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) g13i   = 0.2*(3.0*gi0+2.0*gii)
! m0(13) = m0(13)*exp(ln10*real(g130-g13i,rkx))
! end if
!
! *** other initializations
! *********************************************
    iclact = 0
    calaou = .true.
    calain = .true.
    frst = .true.
    scase = '??'
    sulratw = 2._rkx
    sodrat = zero
    crnarat = zero
    crrat = zero
    nofer = 0
    stkofl = .false.
    do i = 1 , nerrmx
      errstk(i) = -999
      errmsg(i) = 'message n/a'
    end do
  end subroutine init2
!
!======================================================================
!
! *** isorropia code
! *** subroutine isoinit3
! *** this subroutine initializes all global variables for ammonium ,
!     sodium , chloride , nitrate , sulfate aerosol systems (subroutine
!     isrp3)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine isoinit3(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: coef , rhi , t0 , t0t , tcf , tempi
    integer(ik4) :: i , irh
    real(rkx) , dimension(ncomp) :: wi
!
! *** save input variables in common block
! ******************************
    if ( iprob==0 ) then                   ! forward calculation
      do i = 1 , ncomp
        w(i) = max(wi(i),tiny1)
      end do
    else
      do i = 1 , ncomp                     ! reverse calculation
        waer(i) = max(wi(i),tiny1)
        w(i) = zero
      end do
    end if
    rh = rhi
    temp = tempi
!
! *** calculate equilibrium constants
! ***********************************
    xk1 = 1.015e-2_rkx     ! hso4(aq)         <==> h(aq)     + so4(aq)
    xk21 = 57.639_rkx    ! nh3(g)           <==> nh3(aq)
    xk22 = 1.805e-5_rkx    ! nh3(aq)          <==> nh4(aq)   + oh(aq)
    xk3 = 1.971e6_rkx      ! hcl(g)           <==> h(aq)     + cl(aq)
    xk31 = 2.500e3_rkx     ! hcl(g)           <==> hcl(aq)
    xk4 = 2.511e6_rkx      ! hno3(g)          <==> h(aq)     + no3(aq) ! isorr
! xk4  = 3.638e6   ! hno3(g)          <==> h(aq)     + no3(aq) ! sequil
    xk41 = 2.100e5_rkx     ! hno3(g)          <==> hno3(aq)
    xk5 = 0.4799_rkx     ! na2so4(s)        <==> 2*na(aq)  + so4(aq)
    xk6 = 1.086e-16_rkx    ! nh4cl(s)         <==> nh3(g)    + hcl(g)
    xk7 = 1.817_rkx      ! (nh4)2so4(s)     <==> 2*nh4(aq) + so4(aq)
    xk8 = 37.661_rkx     ! nacl(s)          <==> na(aq)    + cl(aq)
    xk10 = 5.746e-17_rkx   ! nh4no3(s)        <==> nh3(g)    + hno3(g) ! isorr
! xk10 = 2.985e-17 ! nh4no3(s)        <==> nh3(g)    + hno3(g) ! sequil
    xk11 = 2.413e4_rkx     ! nahso4(s)        <==> na(aq)    + hso4(aq)
    xk12 = 1.382e2_rkx     ! nh4hso4(s)       <==> nh4(aq)   + hso4(aq)
    xk13 = 29.268_rkx    ! (nh4)3h(so4)2(s) <==> 3*nh4(aq) + hso4(aq) + so4(aq)
    xk14 = 22.05_rkx     ! nh4cl(s)         <==> nh4(aq)   + cl(aq)
    xkw = 1.010e-14_rkx    ! h2o              <==> h(aq)     + oh(aq)
    xk9 = 11.977_rkx     ! nano3(s)         <==> na(aq)    + no3(aq)
!
    if ( int(temp)/=298 ) then       ! for t != 298k or 298.15k
      t0 = 298.15_rkx
      t0t = t0/temp
      coef = 1.0 + log(t0t) - t0t
      xk1 = xk1*exp(8.85_rkx*(t0t-1.0_rkx)+25.140_rkx*coef)
      xk21 = xk21*exp(13.79_rkx*(t0t-1.0_rkx)-5.393_rkx*coef)
      xk22 = xk22*exp(-1.50_rkx*(t0t-1.0_rkx)+26.920_rkx*coef)
      xk3 = xk3*exp(30.20_rkx*(t0t-1.0_rkx)+19.910_rkx*coef)
      xk31 = xk31*exp(30.20_rkx*(t0t-1.0_rkx)+19.910_rkx*coef)
      xk4 = xk4*exp(29.17_rkx*(t0t-1.0_rkx)+16.830_rkx*coef)        !isorr
!   xk4 = xk4 *exp( 29.47_rkx*(t0t-1.0_rkx) + 16.840_rkx*coef) ! sequil
      xk41 = xk41*exp(29.17_rkx*(t0t-1.0_rkx)+16.830_rkx*coef)
      xk5 = xk5*exp(0.98_rkx*(t0t-1.0_rkx)+39.500_rkx*coef)
      xk6 = xk6*exp(-71.00_rkx*(t0t-1.0_rkx)+2.400_rkx*coef)
      xk7 = xk7*exp(-2.65_rkx*(t0t-1.0_rkx)+38.570_rkx*coef)
      xk8 = xk8*exp(-1.56_rkx*(t0t-1.0_rkx)+16.900_rkx*coef)
      xk9 = xk9*exp(-8.22_rkx*(t0t-1.0_rkx)+16.010_rkx*coef)
      xk10 = xk10*exp(-74.38_rkx*(t0t-1.0_rkx)+6.120_rkx*coef)       ! isorr
!   xk10 = xk10*exp(-75.11_rkx*(t0t-1.0) + 13.460_rkx*coef) ! sequil
      xk11 = xk11*exp(0.79_rkx*(t0t-1.0_rkx)+14.746_rkx*coef)
      xk12 = xk12*exp(-2.87_rkx*(t0t-1.0_rkx)+15.830_rkx*coef)
      xk13 = xk13*exp(-5.19_rkx*(t0t-1.0_rkx)+54.400_rkx*coef)
      xk14 = xk14*exp(24.55_rkx*(t0t-1.0_rkx)+16.900_rkx*coef)
      xkw = xkw*exp(-22.52_rkx*(t0t-1.0_rkx)+26.920_rkx*coef)
    end if
    xk2 = xk21*xk22
    xk42 = xk4/xk41
    xk32 = xk3/xk31
!
! *** calculate deliquescence relative humidities (unicomponent)
! ********
    drh2so4 = zero
    drnh42s4 = 0.7997_rkx
    drnh4hs4 = 0.4000_rkx
    drlc = 0.6900_rkx
    drnacl = 0.7528_rkx
    drnano3 = 0.7379_rkx
    drnh4cl = 0.7710_rkx
    drnh4no3 = 0.6183_rkx
    drna2so4 = 0.9300_rkx
    drnahso4 = 0.5200_rkx
    if ( int(temp)/=298 ) then
      t0 = 298.15_rkx
      tcf = 1.0_rkx/temp - 1.0_rkx/t0
      drnacl = drnacl*exp(25._rkx*tcf)
      drnano3 = drnano3*exp(304._rkx*tcf)
      drna2so4 = drna2so4*exp(80._rkx*tcf)
      drnh4no3 = drnh4no3*exp(852._rkx*tcf)
      drnh42s4 = drnh42s4*exp(80._rkx*tcf)
      drnh4hs4 = drnh4hs4*exp(384._rkx*tcf)
      drlc = drlc*exp(186._rkx*tcf)
      drnh4cl = drnh4cl*exp(239._rkx*tcf)
      drnahso4 = drnahso4*exp(-45._rkx*tcf)
!
!   *** adjust for drh "crossover" at low temperatures
!
      drnh4no3 = min(drnh4no3,drnh4cl,drnh42s4,drnano3,drnacl)
      drnano3 = min(drnano3,drnacl)
      drnh4cl = min(drnh4cl,drnh42s4)
!
    end if
!
! *** calculate mutual deliquescence relative humidities
! ****************
    drmlcab = 0.378_rkx      ! (nh4)3h(so4)2 & nh4hso4
    drmlcas = 0.690_rkx      ! (nh4)3h(so4)2 & (nh4)2so4
    drmasan = 0.600_rkx      ! (nh4)2so4     & nh4no3
    drmg1 = 0.460_rkx        ! (nh4)2so4 , nh4no3 , na2so4 , nh4cl
    drmg2 = 0.691_rkx        ! (nh4)2so4 , na2so4 , nh4cl
    drmg3 = 0.697_rkx        ! (nh4)2so4 , na2so4
    drmh1 = 0.240_rkx        ! na2so4 , nano3 , nacl , nh4no3 , nh4cl
    drmh2 = 0.596_rkx        ! na2so4 , nano3 , nacl , nh4cl
    drmi1 = 0.240_rkx        ! lc , nahso4 , nh4hso4 , na2so4 , (nh4)2so4
    drmi2 = 0.363_rkx        ! lc , nahso4 , na2so4 , (nh4)2so4  - no data -
    drmi3 = 0.610_rkx        ! lc , na2so4 , (nh4)2so4
    drmq1 = 0.494_rkx        ! (nh4)2so4 , nh4no3 , na2so4
    drmr1 = 0.663_rkx        ! na2so4 , nano3 , nacl
    drmr2 = 0.735_rkx        ! na2so4 , nacl
    drmr3 = 0.673_rkx        ! nano3 , nacl
    drmr4 = 0.694_rkx        ! na2so4 , nacl , nh4cl
    drmr5 = 0.731_rkx        ! na2so4 , nh4cl
    drmr6 = 0.596_rkx        ! na2so4 , nano3 , nh4cl
    drmr7 = 0.380_rkx        ! na2so4 , nano3 , nacl , nh4no3
    drmr8 = 0.380_rkx        ! na2so4 , nacl , nh4no3
    drmr9 = 0.494_rkx        ! na2so4 , nh4no3
    drmr10 = 0.476_rkx       ! na2so4 , nano3 , nh4no3
    drmr11 = 0.340_rkx       ! na2so4 , nacl , nh4no3 , nh4cl
    drmr12 = 0.460_rkx       ! na2so4 , nh4no3 , nh4cl
    drmr13 = 0.438_rkx       ! na2so4 , nano3 , nh4no3 , nh4cl
! if (int(temp) .ne. 298) then
! t0       = 298.15d0
! tcf      = 1.0/temp - 1.0/t0
! drmlcab  = drmlcab*exp( 507.506*tcf)
! drmlcas  = drmlcas*exp( 133.865*tcf)
! drmasan  = drmasan*exp(1269.068*tcf)
! drmg1    = drmg1  *exp( 572.207*tcf)
! drmg2    = drmg2  *exp(  58.166*tcf)
! drmg3    = drmg3  *exp(  22.253*tcf)
! drmh1    = drmh1  *exp(2116.542*tcf)
! drmh2    = drmh2  *exp( 650.549*tcf)
! drmi1    = drmi1  *exp( 565.743*tcf)
! drmi2    = drmi2  *exp(  91.745*tcf)
! drmi3    = drmi3  *exp( 161.272*tcf)
! drmq1    = drmq1  *exp(1616.621*tcf)
! drmr1    = drmr1  *exp( 292.564*tcf)
! drmr2    = drmr2  *exp(  14.587*tcf)
! drmr3    = drmr3  *exp( 307.907*tcf)
! drmr4    = drmr4  *exp(  97.605*tcf)
! drmr5    = drmr5  *exp(  98.523*tcf)
! drmr6    = drmr6  *exp( 465.500*tcf)
! drmr7    = drmr7  *exp( 324.425*tcf)
! drmr8    = drmr8  *exp(2660.184*tcf)
! drmr9    = drmr9  *exp(1617.178*tcf)
! drmr10   = drmr10 *exp(1745.226*tcf)
! drmr11   = drmr11 *exp(3691.328*tcf)
! drmr12   = drmr12 *exp(1836.842*tcf)
! drmr13   = drmr13 *exp(1967.938*tcf)
! end if
!
! *** liquid phase
! ******************************************************
    chno3 = zero
    chcl = zero
    ch2so4 = zero
    coh = zero
    water = tiny1
!
    do i = 1 , npair
      molalr(i) = zero
      gama(i) = 0.1_rkx
      gamin(i) = great
      gamou(i) = great
      m0(i) = 1e5_rkx
    end do
!
    do i = 1 , npair
      gama(i) = 0.1_rkx
    end do
!
    do i = 1 , nions
      molal(i) = zero
    end do
    coh = zero
!
    do i = 1 , ngasaq
      gasaq(i) = zero
    end do
!
! *** solid phase
! *******************************************************
    cnh42s4 = zero
    cnh4hs4 = zero
    cnacl = zero
    cna2so4 = zero
    cnano3 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnahso4 = zero
    clc = zero
    ccaso4 = zero
    ccano32 = zero
    ccacl2 = zero
    ck2so4 = zero
    ckhso4 = zero
    ckno3 = zero
    ckcl = zero
    cmgso4 = zero
    cmgno32 = zero
    cmgcl2 = zero
!
! *** gas phase
! *********************************************************
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
!
! *** calculate zsr parameters
! ******************************************
    irh = min(int(rh*nzsr+0.5_rkx),nzsr)        ! position in zsr arrays
    irh = max(irh,1)
!
    m0(01) = awsc(irh)        ! nacl
! if (m0(01)  <  100.0) then
! ic = m0(01)
! call kmtab(ic ,298.0 ,     gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),gii ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(01) = m0(01)*exp(ln10*(gi0-gii))
! end if
!
    m0(02) = awss(irh)        ! (na)2so4
! if (m0(02)  <  100.0) then
! ic = 3.0*m0(02)
! call kmtab(ic ,298.0 ,     xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,gii ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(02) = m0(02)*exp(ln10*(gi0-gii))
! end if
!
    m0(03) = awsn(irh)        ! nano3
! if (m0(03)  <  100.0) then
! ic = m0(03)
! call kmtab(ic ,298.0 ,     xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,gii ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) c        m0(03) = m0(03)*exp(ln10*(gi0-gii))
! end if
!
    m0(04) = awas(irh)        ! (nh4)2so4
! if (m0(04)  <  100.0) then
! ic = 3.0*m0(04)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,gii ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(04) = m0(04)*exp(ln10*(gi0-gii))
! end if
!
    m0(05) = awan(irh)        ! nh4no3
! if (m0(05)  <  100.0) then
! ic     = m0(05)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,gii ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(05) = m0(05)*exp(ln10*(gi0-gii))
! end if
!
    m0(06) = awac(irh)        ! nh4cl
! if (m0(06)  <  100.0) then
! ic = m0(06)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,gii ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(06) = m0(06)*exp(ln10*(gi0-gii))
! end if
!
    m0(07) = awsa(irh)        ! 2h-so4
! if (m0(07)  <  100.0) then
! ic = 3.0*m0(07)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(07) = m0(07)*exp(ln10*(gi0-gii))
! end if
!
    m0(08) = awsa(irh)        ! h-hso4
! if (m0(08)  <  100.0) then     ! these are redundant , because m0(8)
! is not used ic = m0(08)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0
!c         call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,gii ,xx ,xx ,xx ,xx)
! ,xx ,xx ,xx ,xx) & m0(08) = m0(08)*exp(ln10*(gi0-gii))
! end if
!
    m0(09) = awab(irh)        ! nh4hso4
! if (m0(09)  <  100.0) then
! ic = m0(09)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,gii ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(09) = m0(09)*exp(ln10*(gi0-gii))
! end if
!
    m0(12) = awsb(irh)        ! nahso4
! if (m0(12)  <  100.0) then
! ic = m0(12)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,gi0 , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,gii , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) m0(12) = m0(12)*exp(ln10*(gi0-gii))
! end if
!
    m0(13) = awlc(irh)        ! (nh4)3h(so4)2
! if (m0(13)  <  100.0) then
! ic     = 4.0*m0(13)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) g130   = 0.2*(3.0*gi0+2.0*gii)
! call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) g13i   = 0.2*(3.0*gi0+2.0*gii)
! m0(13) = m0(13)*exp(ln10*real(g130-g13i,rkx))
! end if
!
! *** other initializations
! *********************************************
    iclact = 0
    calaou = .true.
    calain = .true.
    frst = .true.
    scase = '??'
    sulratw = 2._rkx
    crnarat = zero
    crrat = zero
    nofer = 0
    stkofl = .false.
    do i = 1 , nerrmx
      errstk(i) = -999
      errmsg(i) = 'message n/a'
    end do
  end subroutine isoinit3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine init4
! *** this subroutine initializes all global variables for ammonium ,
!     sodium , chloride , nitrate , sulfate , calcium , potassium , magnesium
!     aerosol systems (subroutine isrp4)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine init4(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: coef , rhi , t0 , t0t , tcf , tempi
    integer(ik4) :: i , irh
    real(rkx) , dimension(ncomp) :: wi
!
! *** save input variables in common block
! ******************************
    if ( iprob==0 ) then                   ! forward calculation
      do i = 1 , ncomp
        w(i) = max(wi(i),tiny1)
      end do
    else
      do i = 1 , ncomp                     ! reverse calculation
        waer(i) = max(wi(i),tiny1)
        w(i) = zero
      end do
    end if
    rh = rhi
    temp = tempi
!
! *** calculate equilibrium constants
! ***********************************
    xk1 = 1.015e-2_rkx     ! hso4(aq)         <==> h(aq)     + so4(aq)
    xk21 = 57.639_rkx    ! nh3(g)           <==> nh3(aq)
    xk22 = 1.805e-5_rkx    ! nh3(aq)          <==> nh4(aq)   + oh(aq)
    xk3 = 1.971e6_rkx      ! hcl(g)           <==> h(aq)     + cl(aq)
    xk31 = 2.500e3_rkx     ! hcl(g)           <==> hcl(aq)
    xk4 = 2.511e6_rkx      ! hno3(g)          <==> h(aq)     + no3(aq) ! isorr
! xk4  = 3.638e6_rkx   ! hno3(g)          <==> h(aq)     + no3(aq) ! sequil
    xk41 = 2.100e5_rkx     ! hno3(g)          <==> hno3(aq)
    xk5 = 0.4799_rkx     ! na2so4(s)        <==> 2*na(aq)  + so4(aq)
    xk6 = 1.086e-16_rkx    ! nh4cl(s)         <==> nh3(g)    + hcl(g)
    xk7 = 1.817_rkx      ! (nh4)2so4(s)     <==> 2*nh4(aq) + so4(aq)
    xk8 = 37.661_rkx     ! nacl(s)          <==> na(aq)    + cl(aq)
! xk10 = 5.746e-17_rkx ! nh4no3(s)        <==> nh3(g)    + hno3(g) ! isorr
    xk10 = 4.199e-17_rkx   ! nh4no3(s)        <==> nh3(g)    + hno3(g) ! (mozurkewich , 1993)
! xk10 = 2.985e-17_rkx ! nh4no3(s)        <==> nh3(g)    + hno3(g) ! sequil
    xk11 = 2.413e4_rkx     ! nahso4(s)        <==> na(aq)    + hso4(aq)
    xk12 = 1.382e2_rkx     ! nh4hso4(s)       <==> nh4(aq)   + hso4(aq)
    xk13 = 29.268_rkx    ! (nh4)3h(so4)2(s) <==> 3*nh4(aq) + hso4(aq) + so4(aq)
    xk14 = 22.05_rkx     ! nh4cl(s)         <==> nh4(aq)   + cl(aq)
    xkw = 1.010e-14_rkx    ! h2o              <==> h(aq)     + oh(aq)
    xk9 = 11.977_rkx     ! nano3(s)         <==> na(aq)    + no3(aq)
!
    xk15 = 6.067e5_rkx     ! ca(no3)2(s)      <==> ca(aq)    + 2no3(aq)
    xk16 = 7.974e11_rkx    ! cacl2(s)         <==> ca(aq)    + 2cl(aq)
    xk17 = 1.569e-2_rkx    ! k2so4(s)         <==> 2k(aq)    + so4(aq)
    xk18 = 24.016_rkx    ! khso4(s)         <==> k(aq)     + hso4(aq)
    xk19 = 0.872_rkx     ! kno3(s)          <==> k(aq)     + no3(aq)
    xk20 = 8.680_rkx     ! kcl(s)           <==> k(aq)     + cl(aq)
    xk23 = 1.079e5_rkx     ! mgs04(s)         <==> mg(aq)    + so4(aq)
    xk24 = 2.507e15_rkx    ! mg(no3)2(s)      <==> mg(aq)    + 2no3(aq)
    xk25 = 9.557e21_rkx    ! mgcl2(s)         <==> mg(aq)    + 2cl(aq)
! xk26 = 4.299d-7  ! co2(aq) + h2o    <==> hco3(aq)  + h(aq)
! xk27 = 4.678d-11 ! hco3(aq)         <==> co3(aq)   + h(aq)

!
    if ( int(temp)/=298 ) then       ! for t != 298k or 298.15k
      t0 = 298.15_rkx
      t0t = t0/temp
      coef = 1.0_rkx + log(t0t) - t0t
      xk1 = xk1*exp(8.85_rkx*(t0t-1.0_rkx)+25.140_rkx*coef)
      xk21 = xk21*exp(13.79_rkx*(t0t-1.0_rkx)-5.393_rkx*coef)
      xk22 = xk22*exp(-1.50_rkx*(t0t-1.0_rkx)+26.920_rkx*coef)
      xk3 = xk3*exp(30.20_rkx*(t0t-1.0_rkx)+19.910_rkx*coef)
      xk31 = xk31*exp(30.20_rkx*(t0t-1.0_rkx)+19.910_rkx*coef)
      xk4 = xk4*exp(29.17_rkx*(t0t-1.0_rkx)+16.830_rkx*coef)        !isorr
!   xk4 = xk4 *exp( 29.47_rkx*(t0t-1.0_rkx) + 16.840_rkx*coef) ! sequil
      xk41 = xk41*exp(29.17_rkx*(t0t-1.0_rkx)+16.830_rkx*coef)
      xk5 = xk5*exp(0.98_rkx*(t0t-1.0_rkx)+39.500_rkx*coef)
      xk6 = xk6*exp(-71.00_rkx*(t0t-1.0_rkx)+2.400_rkx*coef)
      xk7 = xk7*exp(-2.65_rkx*(t0t-1.0_rkx)+38.570_rkx*coef)
      xk8 = xk8*exp(-1.56_rkx*(t0t-1.0_rkx)+16.900_rkx*coef)
      xk9 = xk9*exp(-8.22_rkx*(t0t-1.0_rkx)+16.010_rkx*coef)
!   xk10 = xk10*exp(-74.38_rkx*(t0t-1.0_rkx) +  6.120_rkx*coef) ! isorr
      xk10 = xk10*exp(-74.7351_rkx*(t0t-1.0_rkx)+6.025_rkx*coef)       ! (mozurkewich , 1993)
!   xk10 = xk10*exp(-75.11_rkx*(t0t-1.0_rkx) + 13.460_rkx*coef) ! sequil
      xk11 = xk11*exp(0.79_rkx*(t0t-1.0_rkx)+14.746_rkx*coef)
      xk12 = xk12*exp(-2.87_rkx*(t0t-1.0_rkx)+15.830_rkx*coef)
      xk13 = xk13*exp(-5.19_rkx*(t0t-1.0_rkx)+54.400_rkx*coef)
      xk14 = xk14*exp(24.55_rkx*(t0t-1.0_rkx)+16.900_rkx*coef)
      xkw = xkw*exp(-22.52_rkx*(t0t-1.0_rkx)+26.920_rkx*coef)
!
!   xk15 = xk15 *exp(  .0_rkx*(t0t-1.0_rkx) + .0_rkx*coef)
!   xk16 = xk16 *exp(  .0_rkx*(t0t-1.0_rkx) + .0_rkx*coef)
      xk17 = xk17*exp(-9.585_rkx*(t0t-1.0_rkx)+45.81_rkx*coef)
      xk18 = xk18*exp(-8.423_rkx*(t0t-1.0_rkx)+17.96_rkx*coef)
      xk19 = xk19*exp(-14.08_rkx*(t0t-1.0_rkx)+19.39_rkx*coef)
      xk20 = xk20*exp(-6.902_rkx*(t0t-1.0_rkx)+19.95_rkx*coef)
!   xk23 = xk23 *exp(  .0_rkx*(t0t-1.0_rkx) + .0_rkx*coef)
!   xk24 = xk24 *exp(  .0_rkx*(t0t-1.0_rkx) + .0_rkx*coef)
!   xk25 = xk25 *exp(  .0_rkx*(t0t-1.0_rkx) + .0_rkx*coef)
!   xk26 = xk26 *exp(-3.0821_rkx*(t0t-1.0_rkx) + 31.8139_rkx*coef)
!   xk27 = xk27 *exp(-5.9908_rkx*(t0t-1.0_rkx) + 38.844_rkx*coef)

    end if
    xk2 = xk21*xk22
    xk42 = xk4/xk41
    xk32 = xk3/xk31
!
! *** calculate deliquescence relative humidities (unicomponent)
! ********
    drh2so4 = zero
    drnh42s4 = 0.7997_rkx
    drnh4hs4 = 0.4000_rkx
    drlc = 0.6900_rkx
    drnacl = 0.7528_rkx
    drnano3 = 0.7379_rkx
    drnh4cl = 0.7710_rkx
    drnh4no3 = 0.6183_rkx
    drna2so4 = 0.9300_rkx
    drnahso4 = 0.5200_rkx
    drcano32 = 0.4906_rkx
    drcacl2 = 0.2830_rkx
    drk2so4 = 0.9750_rkx
    drkhso4 = 0.8600_rkx
    drkno3 = 0.9248_rkx
    drkcl = 0.8426_rkx
    drmgso4 = 0.8613_rkx
    drmgno32 = 0.5400_rkx
    drmgcl2 = 0.3284_rkx
    if ( int(temp)/=298 ) then
      t0 = 298.15_rkx
      tcf = 1.0_rkx/temp - 1.0_rkx/t0
      drnacl = drnacl*exp(25._rkx*tcf)
      drnano3 = drnano3*exp(304._rkx*tcf)
      drna2so4 = drna2so4*exp(80._rkx*tcf)
      drnh4no3 = drnh4no3*exp(852._rkx*tcf)
      drnh42s4 = drnh42s4*exp(80._rkx*tcf)
      drnh4hs4 = drnh4hs4*exp(384._rkx*tcf)
      drlc = drlc*exp(186._rkx*tcf)
      drnh4cl = drnh4cl*exp(239._rkx*tcf)
      drnahso4 = drnahso4*exp(-45._rkx*tcf)
!   drcano32 = drcano32*exp(-430.5d0*tcf)
      drcano32 = drcano32*exp(509.4_rkx*tcf)      ! kelly & wexler (2005) for cano32.4h20
!   drcacl2  = drcacl2 *exp(-1121._rkx*tcf)
      drcacl2 = drcacl2*exp(551.1_rkx*tcf)       ! kelly & wexler (2005) for cacl2.6h20
      drk2so4 = drk2so4*exp(35.6_rkx*tcf)
!   drkhso4  = drkhso4 *exp( 0._rkx*tcf)
!   drkno3   = drkno3  *exp( 0._rkx*tcf)
      drkcl = drkcl*exp(159._rkx*tcf)
      drmgso4 = drmgso4*exp(-714.45_rkx*tcf)
      drmgno32 = drmgno32*exp(230.2_rkx*tcf)      ! kelly & wexler (2005) for mgno32.6h20
!   drmgcl2  = drmgcl2 *exp(-1860._rkx*tcf)
      drmgcl2 = drmgcl2*exp(42.23_rkx*tcf)        ! kelly & wexler (2005) for mgcl2.6h20
!
    end if
!
! *** calculate mutual deliquescence relative humidities
! ****************
    drmlcab = 0.378_rkx      ! (nh4)3h(so4)2 & nh4hso4
    drmlcas = 0.690_rkx      ! (nh4)3h(so4)2 & (nh4)2so4
    drmasan = 0.600_rkx      ! (nh4)2so4     & nh4no3
    drmg1 = 0.460_rkx        ! (nh4)2so4 , nh4no3 , na2so4 , nh4cl
    drmg2 = 0.691_rkx        ! (nh4)2so4 , na2so4 , nh4cl
    drmg3 = 0.697_rkx        ! (nh4)2so4 , na2so4
    drmh1 = 0.240_rkx        ! na2so4 , nano3 , nacl , nh4no3 , nh4cl
    drmh2 = 0.596_rkx        ! na2so4 , nano3 , nacl , nh4cl
    drmi1 = 0.240_rkx        ! lc , nahso4 , nh4hso4 , na2so4 , (nh4)2so4
    drmi2 = 0.363_rkx        ! lc , nahso4 , na2so4 , (nh4)2so4  - no data -
    drmi3 = 0.610_rkx        ! lc , na2so4 , (nh4)2so4
    drmq1 = 0.494_rkx        ! (nh4)2so4 , nh4no3 , na2so4
    drmr1 = 0.663_rkx        ! na2so4 , nano3 , nacl
    drmr2 = 0.735_rkx        ! na2so4 , nacl
    drmr3 = 0.673_rkx        ! nano3 , nacl
    drmr4 = 0.694_rkx        ! na2so4 , nacl , nh4cl
    drmr5 = 0.731_rkx        ! na2so4 , nh4cl
    drmr6 = 0.596_rkx        ! na2so4 , nano3 , nh4cl
    drmr7 = 0.380_rkx        ! na2so4 , nano3 , nacl , nh4no3
    drmr8 = 0.380_rkx        ! na2so4 , nacl , nh4no3
    drmr9 = 0.494_rkx        ! na2so4 , nh4no3
    drmr10 = 0.476_rkx       ! na2so4 , nano3 , nh4no3
    drmr11 = 0.340_rkx       ! na2so4 , nacl , nh4no3 , nh4cl
    drmr12 = 0.460_rkx       ! na2so4 , nh4no3 , nh4cl
    drmr13 = 0.438_rkx       ! na2so4 , nano3 , nh4no3 , nh4cl
!
    drmo1 = 0.460_rkx        ! (nh4)2so4 , nh4no3 , nh4cl , na2so4 , k2so4 , mgso4
    drmo2 = 0.691_rkx        ! (nh4)2so4 , nh4cl , na2so4 , k2so4 , mgso4
    drmo3 = 0.697_rkx        ! (nh4)2so4 , na2so4 , k2so4 , mgso4
    drml1 = 0.240_rkx        ! k2so4 , mgso4 , khso4 , nh4hso4 , nahso4 , (nh4)2so4 , na2so4 , lc
    drml2 = 0.363_rkx        ! k2so4 , mgso4 , khso4 , nahso4 , (nh4)2so4 , na2so4 , lc
    drml3 = 0.610_rkx        ! k2so4 , mgso4 , khso4 , (nh4)2so4 , na2so4 , lc
    drmm1 = 0.240_rkx        ! k2so4 , na2so4 , mgso4 , nh4cl , nacl , nano3 , nh4no3
    drmm2 = 0.596_rkx        ! k2so4 , na2so4 , mgso4 , nh4cl , nacl , nano3
    drmp1 = 0.200_rkx        ! ca(no3)2 , cacl2 , k2so4 , kno3 , kcl , mgso4 , mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
    drmp2 = 0.240_rkx        ! ca(no3)2 , k2so4 , kno3 , kcl , mgso4 , mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
    drmp3 = 0.240_rkx        ! ca(no3)2 , k2so4 , kno3 , kcl , mgso4 , mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl
    drmp4 = 0.240_rkx        ! k2so4 , kno3 , kcl , mgso4 , mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl
    drmp5 = 0.240_rkx        ! k2so4 , kno3 , kcl , mgso4 , nano3 , nacl , nh4no3 , nh4cl
!
    drmv1 = 0.494_rkx        ! (nh4)2so4 , nh4no3 , na2so4 , k2so4 , mgso4
!
!
! drmo1   = 0.1d0    ! (nh4)2so4 , nh4no3 , nh4cl , na2so4 , k2so4 ,
! mgso4 drmo2   = 0.1d0    ! (nh4)2so4 , nh4cl , na2so4 , k2so4 , mgso4
! drmo3   = 0.1d0    ! (nh4)2so4 , na2so4 , k2so4 , mgso4
! drml1   = 0.1d0    ! k2so4 , mgso4 , khso4 , nh4hso4 , nahso4 ,
! (nh4)2so4 , na2so4 , lc drml2   = 0.1d0    ! k2so4 , mgso4 , khso4 ,
! nahso4 , (nh4)2so4 , na2so4 , lc drml3   = 0.1d0    ! k2so4 , mgso4 ,
! khso4 , (nh4)2so4 , na2so4 , lc drmm1   = 0.1d0    ! k2so4 , na2so4 ,
! mgso4 , nh4cl , nacl , nano3 , nh4no3 drmm2   = 0.1d0    ! k2so4 ,
! na2so4 , mgso4 , nh4cl , nacl , nano3 drmp1   = 0.1d0    ! ca(no3)2 ,
! cacl2 , k2so4 , kno3 , kcl , mgso4 , mg(no3)2 , mgcl2 , nano3 , nacl
! , nh4no3 , nh4cl drmp2   = 0.1d0    ! ca(no3)2 , k2so4 , kno3 , kcl ,
! mgso4 , mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl drmp3   =
! 0.1d0    ! ca(no3)2 , k2so4 , kno3 , kcl , mgso4 , mg(no3)2 , nano3 ,
! nacl , nh4no3 , nh4cl drmp4   = 0.1d0    ! k2so4 , kno3 , kcl , mgso4
! , mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl drmp5   = 0.1d0    ! k2so4
! , kno3 , kcl , mgso4 , nano3 , nacl , nh4no3 , nh4cl drmv1   = 0.1d0
! ! (nh4)2so4 , nh4no3 , na2so4 , k2so4 , mgso4 if (int(temp) .ne. 298)
! then t0       = 298.15d0
! tcf      = 1.0/temp - 1.0/t0
! drmlcab  = drmlcab*exp( 507.506*tcf)
! drmlcas  = drmlcas*exp( 133.865*tcf)
! drmasan  = drmasan*exp(1269.068*tcf)
! drmg1    = drmg1  *exp( 572.207*tcf)
! drmg2    = drmg2  *exp(  58.166*tcf)
! drmg3    = drmg3  *exp(  22.253*tcf)
! drmh1    = drmh1  *exp(2116.542*tcf)
! drmh2    = drmh2  *exp( 650.549*tcf)
! drmi1    = drmi1  *exp( 565.743*tcf)
! drmi2    = drmi2  *exp(  91.745*tcf)
! drmi3    = drmi3  *exp( 161.272*tcf)
! drmq1    = drmq1  *exp(1616.621*tcf)
! drmr1    = drmr1  *exp( 292.564*tcf)
! drmr2    = drmr2  *exp(  14.587*tcf)
! drmr3    = drmr3  *exp( 307.907*tcf)
! drmr4    = drmr4  *exp(  97.605*tcf)
! drmr5    = drmr5  *exp(  98.523*tcf)
! drmr6    = drmr6  *exp( 465.500*tcf)
! drmr7    = drmr7  *exp( 324.425*tcf)
! drmr8    = drmr8  *exp(2660.184*tcf)
! drmr9    = drmr9  *exp(1617.178*tcf)
! drmr10   = drmr10 *exp(1745.226*tcf)
! drmr11   = drmr11 *exp(3691.328*tcf)
! drmr12   = drmr12 *exp(1836.842*tcf)
! drmr13   = drmr13 *exp(1967.938*tcf)
! end if
!
! *** liquid phase
! ******************************************************
    chno3 = zero
    chcl = zero
    ch2so4 = zero
    coh = zero
    water = tiny1
!
    do i = 1 , npair
      molalr(i) = zero
      gama(i) = 0.1_rkx
      gamin(i) = great
      gamou(i) = great
      m0(i) = 1e5_rkx
    end do
!
    do i = 1 , npair
      gama(i) = 0.1_rkx
    end do
!
    do i = 1 , nions
      molal(i) = zero
    end do
    coh = zero
!
    do i = 1 , ngasaq
      gasaq(i) = zero
    end do
!
! *** solid phase
! *******************************************************
    cnh42s4 = zero
    cnh4hs4 = zero
    cnacl = zero
    cna2so4 = zero
    cnano3 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnahso4 = zero
    clc = zero
    ccaso4 = zero
    ccano32 = zero
    ccacl2 = zero
    ck2so4 = zero
    ckhso4 = zero
    ckno3 = zero
    ckcl = zero
    cmgso4 = zero
    cmgno32 = zero
    cmgcl2 = zero
!
! *** gas phase
! *********************************************************
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
!
! *** calculate zsr parameters
! ******************************************
    irh = min(int(rh*nzsr+0.5_rkx),nzsr)        ! position in zsr arrays
    irh = max(irh,1)
!
    m0(01) = awsc(irh)        ! nacl
! if (m0(01)  <  100.0) then
! ic = m0(01)
! call kmtab(ic ,298.0 ,     gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                            xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),gii ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx , &                            xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) m0(01) = m0(01)*exp(ln10*(gi0-gii))
! end if
!
    m0(02) = awss(irh)        ! (na)2so4
! if (m0(02)  <  100.0) then
! ic = 3.0*m0(02)
! call kmtab(ic ,298.0 ,     xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,gii ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(02) = m0(02)*exp(ln10*(gi0-gii))
! end if
!
    m0(03) = awsn(irh)        ! nano3
! if (m0(03)  <  100.0) then
! ic = m0(03)
! call kmtab(ic ,298.0 ,     xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,gii ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(03) = m0(03)*exp(ln10*(gi0-gii))
! end if
!
    m0(04) = awas(irh)        ! (nh4)2so4
! if (m0(04)  <  100.0) then
! ic = 3.0*m0(04)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,gii ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(04) = m0(04)*exp(ln10*(gi0-gii))
! end if
!
    m0(05) = awan(irh)        ! nh4no3
! if (m0(05)  <  100.0) then
! ic     = m0(05)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,gii ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(05) = m0(05)*exp(ln10*(gi0-gii))
! end if
!
    m0(06) = awac(irh)        ! nh4cl
! if (m0(06)  <  100.0) then
! ic = m0(06)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,gii ,xx ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(06) = m0(06)*exp(ln10*(gi0-gii))
! end if
!
    m0(07) = awsa(irh)        ! 2h-so4
! if (m0(07)  <  100.0) then
! ic = 3.0*m0(07)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(07) = m0(07)*exp(ln10*(gi0-gii))
! end if
!
    m0(08) = awsa(irh)        ! h-hso4
! if (m0(08)  <  100.0) then     ! these are redundant , because m0(8)
! is not used ic = m0(08)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0
!c         call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,gii ,xx ,xx ,xx ,xx)
! ,xx ,xx ,xx ,xx) & m0(08) = m0(08)*exp(ln10*(gi0-gii))
! end if
!
    m0(09) = awab(irh)        ! nh4hso4
! if (m0(09)  <  100.0) then
! ic = m0(09)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,gii ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx ,xx) m0(09) = m0(09)*exp(ln10*(gi0-gii))
! end if
!
    m0(12) = awsb(irh)        ! nahso4
! if (m0(12)  <  100.0) then
! ic = m0(12)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,gi0 , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,gii , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) m0(12) = m0(12)*exp(ln10*(gi0-gii))
! end if
!
    m0(13) = awlc(irh)        ! (nh4)3h(so4)2
! if (m0(13)  <  100.0) then
! ic     = 4.0*m0(13)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) g130   = 0.2*(3.0*gi0+2.0*gii)
! call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) g13i   = 0.2*(3.0*gi0+2.0*gii)
! m0(13) = m0(13)*exp(ln10*real(g130-g13i,rkx))
! end if
!
    m0(15) = awcn(irh)        ! ca(no3)2
! if (m0(15)  <  100.0) then
! ic = m0(15)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx , &                             gi0 ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             gii ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) m0(15) = m0(15)*exp(ln10*(gi0-gii))
! end if
!
    m0(16) = awcc(irh)        ! cacl2
! if (m0(16)  <  100.0) then
! ic = m0(16)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx , &                             xx ,gi0 ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,gii ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx) m0(16) = m0(16)*exp(ln10*(gi0-gii))
! end if
!
    m0(17) = awps(irh)        ! k2so4
! if (m0(17)  <  100.0) then
! ic = m0(17)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx , &                             xx ,xx ,gi0 ,xx ,xx ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,gii ,xx ,xx ,xx ,xx
! ,xx ,xx) m0(17) = m0(17)*exp(ln10*(gi0-gii))
! end if
!
    m0(18) = awpb(irh)        ! khso4
! if (m0(18)  <  100.0) then
! ic = m0(18)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx , &                             xx ,xx ,xx ,gi0 ,xx ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,gii ,xx ,xx ,xx
! ,xx ,xx) m0(18) = m0(18)*exp(ln10*(gi0-gii))
! end if
!
    m0(19) = awpn(irh)        ! kno3
! if (m0(19)  <  100.0) then
! ic = m0(19)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx , &                             xx ,xx ,xx ,xx ,gi0 ,xx ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,gii ,xx ,xx
! ,xx ,xx) m0(19) = m0(19)*exp(ln10*(gi0-gii))
! end if
!
    m0(20) = awpc(irh)        ! kcl
! if (m0(20)  <  100.0) then
! ic = m0(20)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx , &                             xx ,xx ,xx ,xx ,xx ,gi0 ,xx ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,gii ,xx
! ,xx ,xx) m0(20) = m0(20)*exp(ln10*(gi0-gii))
! end if
!
    m0(21) = awms(irh)        ! mgso4
! if (m0(21)  <  100.0) then
! ic = m0(21)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,gi0 ,xx
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,gii
! ,xx ,xx) m0(21) = m0(21)*exp(ln10*(gi0-gii))
! end if
!
    m0(22) = awmn(irh)        ! mg(no3)2
! if (m0(22)  <  100.0) then
! ic = m0(22)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx ,gi0
! ,xx) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,gii ,xx) m0(22) = m0(22)*exp(ln10*(gi0-gii))
! end if
!
    m0(23) = awmc(irh)        ! mgcl2
! if (m0(23)  <  100.0) then
! ic = m0(23)
! call kmtab(ic ,298.0 ,     xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,gi0) call kmtab(ic ,real(temp,rkx),xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,xx , &                             xx ,xx ,xx ,xx ,xx ,xx
! ,xx ,xx ,gii) m0(23) = m0(23)*exp(ln10*(gi0-gii))
! end if
!
! *** other initializations
! *********************************************
    iclact = 0
    calaou = .true.
    calain = .true.
    frst = .true.
    scase = '??'
    sulratw = 2._rkx
    so4rat = 2._rkx
    crnarat = 2._rkx
    crrat = 2._rkx
    nofer = 0
    stkofl = .false.
    do i = 1 , nerrmx
      errstk(i) = -999
      errmsg(i) = 'message n/a'
    end do
  end subroutine init4
!
!======================================================================
!
! *** isorropia code
! *** subroutine adjust
! *** adjusts for mass balance between volatile species and sulfate
!     first calculate the excess of each precursor , and if it exists , then
!     adjust sequentialy aerosol phase species which contain the excess
!     precursor.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine adjust(wi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: excl , exnh4 , exno3 , exs4
    real(rkx) , dimension(:) :: wi
!
! *** for ammonium *****************************************************
!
    if ( iprob==0 ) then           ! calculate excess (solution - input)
      exnh4 = gnh3 + molal(3) + cnh4cl + cnh4no3 + cnh4hs4 + 2_rkx*cnh42s4 +&
              3_rkx*clc - wi(3)
    else
      exnh4 = molal(3) + cnh4cl + cnh4no3 + cnh4hs4 + 2_rkx*cnh42s4 +       &
              3_rkx*clc - wi(3)

    end if
    exnh4 = max(exnh4,zero)
    if ( exnh4>=tiny1 ) then        ! no excess nh4 , go to next precursor
!
      if ( molal(3)>exnh4 ) then   ! adjust aqueous phase nh4
        molal(3) = molal(3) - exnh4
        go to 100
      else
        exnh4 = exnh4 - molal(3)
        molal(3) = zero
      end if
!
      if ( cnh4cl>exnh4 ) then     ! adjust nh4cl(s)
        cnh4cl = cnh4cl - exnh4     ! more solid than excess
        ghcl = ghcl + exnh4         ! evaporate cl to gas phase
        go to 100
      else                          ! less solid than excess
        ghcl = ghcl + cnh4cl        ! evaporate into gas phase
        exnh4 = exnh4 - cnh4cl      ! reduce excess
        cnh4cl = zero               ! zero salt concentration
      end if
!
      if ( cnh4no3>exnh4 ) then    ! adjust nh4no3(s)
        cnh4no3 = cnh4no3 - exnh4   ! more solid than excess
        ghno3 = ghno3 + exnh4       ! evaporate no3 to gas phase
        go to 100
      else                          ! less solid than excess
        ghno3 = ghno3 + cnh4no3     ! evaporate into gas phase
        exnh4 = exnh4 - cnh4no3     ! reduce excess
        cnh4no3 = zero              ! zero salt concentration
      end if
!
      if ( clc>3_rkx*exnh4 ) then    ! adjust (nh4)3h(so4)2(s)
        clc = clc - exnh4/3_rkx       ! more solid than excess
        go to 100
      else                          ! less solid than excess
        exnh4 = exnh4 - 3_rkx*clc     ! reduce excess
        clc = zero                  ! zero salt concentration
      end if
!
      if ( cnh4hs4>exnh4 ) then    ! adjust nh4hso4(s)
        cnh4hs4 = cnh4hs4 - exnh4   ! more solid than excess
        go to 100
      else                          ! less solid than excess
        exnh4 = exnh4 - cnh4hs4     ! reduce excess
        cnh4hs4 = zero              ! zero salt concentration
      end if
!
      if ( cnh42s4>exnh4 ) then    ! adjust (nh4)2so4(s)
        cnh42s4 = cnh42s4 - exnh4   ! more solid than excess
      else                          ! less solid than excess
        exnh4 = exnh4 - cnh42s4     ! reduce excess
        cnh42s4 = zero              ! zero salt concentration
      end if
    end if
!
! *** for nitrate ******************************************************
!
 100  if ( iprob==0 ) then         ! calculate excess (solution - input)
      exno3 = ghno3 + molal(7) + cnh4no3 - wi(4)
    else
      exno3 = molal(7) + cnh4no3 - wi(4)
    end if
    exno3 = max(exno3,zero)
    if ( exno3>=tiny1 ) then        ! no excess no3 , go to next precursor
!
      if ( molal(7)>exno3 ) then   ! adjust aqueous phase no3
        molal(7) = molal(7) - exno3
        go to 200
      else
        exno3 = exno3 - molal(7)
        molal(7) = zero
      end if
!
      if ( cnh4no3>exno3 ) then    ! adjust nh4no3(s)
        cnh4no3 = cnh4no3 - exno3   ! more solid than excess
        gnh3 = gnh3 + exno3         ! evaporate no3 to gas phase
      else                          ! less solid than excess
        gnh3 = gnh3 + cnh4no3       ! evaporate into gas phase
        exno3 = exno3 - cnh4no3     ! reduce excess
        cnh4no3 = zero              ! zero salt concentration
      end if
    end if
!
! *** for chloride *****************************************************
!
 200  if ( iprob==0 ) then         ! calculate excess (solution - input)
      excl = ghcl + molal(4) + cnh4cl - wi(5)
    else
      excl = molal(4) + cnh4cl - wi(5)
    end if
    excl = max(excl,zero)
    if ( excl>=tiny1 ) then        ! no excess cl , go to next precursor
!
      if ( molal(4)>excl ) then   ! adjust aqueous phase cl
        molal(4) = molal(4) - excl
        go to 300
      else
        excl = excl - molal(4)
        molal(4) = zero
      end if
!
      if ( cnh4cl>excl ) then      ! adjust nh4cl(s)
        cnh4cl = cnh4cl - excl      ! more solid than excess
        ghcl = ghcl + excl          ! evaporate cl to gas phase
      else                          ! less solid than excess
        ghcl = ghcl + cnh4cl        ! evaporate into gas phase
        excl = excl - cnh4cl        ! reduce excess
        cnh4cl = zero               ! zero salt concentration
      end if
    end if
!
! *** for sulfate ******************************************************
!
 300  exs4 = molal(5) + molal(6) + 2._rkx*clc + cnh42s4 + cnh4hs4 +       &
               cna2so4 + cnahso4 - wi(2)
    exs4 = max(exs4,zero)           ! calculate excess (solution - input)
    if ( exs4>=tiny1 ) then        ! no excess so4 , return
!
      if ( molal(6)>exs4 ) then   ! adjust aqueous phase hso4
        molal(6) = molal(6) - exs4
        return
      else
        exs4 = exs4 - molal(6)
        molal(6) = zero
      end if
!
      if ( molal(5)>exs4 ) then   ! adjust aqueous phase so4
        molal(5) = molal(5) - exs4
        return
      else
        exs4 = exs4 - molal(5)
        molal(5) = zero
      end if
!
      if ( clc>2_rkx*exs4 ) then     ! adjust (nh4)3h(so4)2(s)
        clc = clc - exs4/2_rkx        ! more solid than excess
        gnh3 = gnh3 + 1.5_rkx*exs4    ! evaporate nh3 to gas phase
        return
      else                          ! less solid than excess
        gnh3 = gnh3 + 1.5_rkx*clc     ! evaporate nh3 to gas phase
        exs4 = exs4 - 2_rkx*clc       ! reduce excess
        clc = zero                  ! zero salt concentration
      end if
!
      if ( cnh4hs4>exs4 ) then     ! adjust nh4hso4(s)
        cnh4hs4 = cnh4hs4 - exs4    ! more solid than excess
        gnh3 = gnh3 + exs4          ! evaporate nh3 to gas phase
        return
      else                          ! less solid than excess
        gnh3 = gnh3 + cnh4hs4       ! evaporate nh3 to gas phase
        exs4 = exs4 - cnh4hs4       ! reduce excess
        cnh4hs4 = zero              ! zero salt concentration
      end if
!
      if ( cnh42s4>exs4 ) then     ! adjust (nh4)2so4(s)
        cnh42s4 = cnh42s4 - exs4    ! more solid than excess
        gnh3 = gnh3 + 2._rkx*exs4     ! evaporate nh3 to gas phase
      else                          ! less solid than excess
        gnh3 = gnh3 + 2._rkx*cnh42s4   ! evaporate nh3 to gas phase
        exs4 = exs4 - cnh42s4      ! reduce excess
        cnh42s4 = zero             ! zero salt concentration
      end if
    end if
  end subroutine adjust

!======================================================================
!
! *** isorropia code
! *** function getasr
! *** calculates the limiting nh4+/so4 ratio of a sulfate poor system
!     (i.e. sulfate ratio = 2.0) for given so4 level and rh
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function getasr(so4i,rhi)
    implicit none
    real(rkx) :: a1 , rat , wf
    integer(ik4) :: ia1 , indr , inds , indsh , indsl , iposh , iposl
    integer , parameter :: nso4s = 14
    integer , parameter :: nrhs = 20
    integer , parameter :: nasrd = nso4s*nrhs
    real(rkx) :: so4i , rhi
    real(rkx) :: asrat(nasrd) , asso4(nso4s)
!
    asso4 = (/1.0E-9,2.5E-9,5.0E-9,7.5E-9,1.0E-8,2.5E-8,5.0E-8,7.5E-8,    &
            1.0E-7,2.5E-7,5.0E-7,7.5E-7,1.0E-6,5.0E-6/)
!
    asrat(1:100) = (/1.020464,0.9998130,0.9960167,0.9984423,1.004004,     &
                   1.010885,1.018356,1.026726,1.034268,1.043846,1.052933, &
                   1.062230,1.062213,1.080050,1.088350,1.096603,1.104289, &
                   1.111745,1.094662,1.121594,1.268909,1.242444,1.233815, &
                   1.232088,1.234020,1.238068,1.243455,1.250636,1.258734, &
                   1.267543,1.276948,1.286642,1.293337,1.305592,1.314726, &
                   1.323463,1.333258,1.343604,1.344793,1.355571,1.431463, &
                   1.405204,1.395791,1.393190,1.394403,1.398107,1.403811, &
                   1.411744,1.420560,1.429990,1.439742,1.449507,1.458986, &
                   1.468403,1.477394,1.487373,1.495385,1.503854,1.512281, &
                   1.520394,1.514464,1.489699,1.480686,1.478187,1.479446, &
                   1.483310,1.489316,1.497517,1.506501,1.515816,1.524724, &
                   1.533950,1.542758,1.551730,1.559587,1.568343,1.575610, &
                   1.583140,1.590440,1.596481,1.567743,1.544426,1.535928, &
                   1.533645,1.535016,1.539003,1.545124,1.553283,1.561886, &
                   1.570530,1.579234,1.587813,1.595956,1.603901,1.611349, &
                   1.618833,1.625819,1.632543,1.639032,1.645276/)

    asrat(101:200) = (/1.707390,1.689553,1.683198,1.681810,1.683490,      &
                     1.687477,1.693148,1.700084,1.706917,1.713507,        &
                     1.719952,1.726190,1.731985,1.737544,1.742673,        &
                     1.747756,1.752431,1.756890,1.761141,1.765190,        &
                     1.785657,1.771851,1.767063,1.766229,1.767901,        &
                     1.771455,1.776223,1.781769,1.787065,1.792081,        &
                     1.796922,1.801561,1.805832,1.809896,1.813622,        &
                     1.817292,1.820651,1.823841,1.826871,1.829745,        &
                     1.822215,1.810497,1.806496,1.805898,1.807480,        &
                     1.810684,1.814860,1.819613,1.824093,1.828306,        &
                     1.832352,1.836209,1.839748,1.843105,1.846175,        &
                     1.849192,1.851948,1.854574,1.857038,1.859387,        &
                     1.844588,1.834208,1.830701,1.830233,1.831727,        &
                     1.834665,1.838429,1.842658,1.846615,1.850321,        &
                     1.853869,1.857243,1.860332,1.863257,1.865928,        &
                     1.868550,1.870942,1.873208,1.875355,1.877389,        &
                     1.899556,1.892637,1.890367,1.890165,1.891317,        &
                     1.893436,1.896036,1.898872,1.901485,1.903908,        &
                     1.906212,1.908391,1.910375,1.912248,1.913952,        &
                     1.915621,1.917140,1.918576,1.919934,1.921220/)

    asrat(201:280) = (/1.928264,1.923245,1.921625,1.921523,1.922421,      &
                     1.924016,1.925931,1.927991,1.929875,1.931614,        &
                     1.933262,1.934816,1.936229,1.937560,1.938769,        &
                     1.939951,1.941026,1.942042,1.943003,1.943911,        &
                     1.941205,1.937060,1.935734,1.935666,1.936430,        &
                     1.937769,1.939359,1.941061,1.942612,1.944041,        &
                     1.945393,1.946666,1.947823,1.948911,1.949900,        &
                     1.950866,1.951744,1.952574,1.953358,1.954099,        &
                     1.948985,1.945372,1.944221,1.944171,1.944850,        &
                     1.946027,1.947419,1.948902,1.950251,1.951494,        &
                     1.952668,1.953773,1.954776,1.955719,1.956576,        &
                     1.957413,1.958174,1.958892,1.959571,1.960213,        &
                     1.977193,1.975540,1.975023,1.975015,1.975346,        &
                     1.975903,1.976547,1.977225,1.977838,1.978401,        &
                     1.978930,1.979428,1.979879,1.980302,1.980686,        &
                     1.981060,1.981401,1.981722,1.982025,1.982312/)
!
! ** solve using full computations , not look-up tables **************
!
! w(2) = waer(2)
! w(3) = waer(2)*2.0001d0
! call calca2
! sulratw = molal(3)/waer(2)
! call init1 (wi , rhi , tempi)   ! re-initialize common block
!
! *** calculate indices ************************************************
!
    rat = so4i/1.E-9
    a1 = int(log10(rat))                         ! magnitude of rat
    ia1 = int(rat/2.5/10.0**a1)
!
    inds = int(4.0*a1 + min(ia1,4))
    inds = min(max(0,inds),nso4s-1) + 1            ! so4 component of ipos
!
    indr = int(99.0-rhi*100.0) + 1
    indr = min(max(1,indr),nrhs)                   ! rh component of ipos
!
! *** get value and return *********************************************
!
    indsl = inds
    indsh = min(indsl+1,nso4s)
    iposl = (indsl-1)*nrhs + indr                 ! low position in array
    iposh = (indsh-1)*nrhs + indr                 ! high position in array
!
    wf = (so4i-asso4(indsl))/(asso4(indsh)-asso4(indsl)+1E-7)
    wf = min(max(wf,0.0_rkx),1.0_rkx)
    getasr = wf*asrat(iposh) + (1.0-wf)*asrat(iposl)
  end function getasr
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcha
! *** calculates chlorides speciation
!
!     hydrochloric acid in the liquid phase is assumed a minor species ,
!     and does not significantly perturb the hso4-so4 equilibrium. the
!     hydrochloric acid dissolved is calculated from the
!     hcl(g) <-> (h+) + (cl-)
!     equilibrium , using the (h+) from the sulfates.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcha
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: alfa , delt , diak , x
    real(rkx) :: kapa
! character errinf*40
!
! *** calculate hcl dissolution
! *****************************************
    x = w(5)
    delt = 0.0_rkx
    if ( water>tiny1 ) then
      kapa = molal(1)
      alfa = xk3*r*temp*(water/gama(11))**2
      diak = sqrt((kapa+alfa)**2+4.0*alfa*x)
      delt = 0.5_rkx*(-(kapa+alfa)+diak)
!   if (delt/kapa > 0.1d0) then
!   write (errinf ,'(1pe10.3)') delt/kapa*100.0
!   call pusherr (0033 , errinf)
!   end if
    end if
!
! *** calculate hcl speciation in the gas phase
! *************************
    ghcl = max(x-delt,0.0_rkx)          ! gas hcl
!
! *** calculate hcl speciation in the liquid phase
! **********************
    molal(4) = delt                  ! cl-
    molal(1) = molal(1) + delt       ! h+
  end subroutine calcha
!
!======================================================================
!
! *** isorropia code
! *** subroutine calchap
! *** calculates chlorides speciation
!
!     hydrochloric acid in the liquid phase is assumed a minor species ,
!     that does not significantly perturb the hso4-so4 equilibrium.
!     the hydrochloric acid dissolved is calculated from the
!     hcl(g) -> hcl(aq)   and  hcl(aq) ->  (h+) + (cl-)
!     equilibria , using (h+) from the sulfates.
!
!     this is the version used by the inverse problem sover
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calchap
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: alfa , delt
!
! *** is there a liquid phase?
! ******************************************
    if ( water<=tiny1 ) return
!
! *** calculate hcl speciation in the gas phase
! *************************
    call calcclaq(molal(4),molal(1),delt)
    alfa = xk3*r*temp*(water/gama(11))**2
    gasaq(3) = delt
    molal(1) = molal(1) - delt
    molal(4) = molal(4) - delt
    ghcl = molal(1)*molal(4)/alfa
  end subroutine calchap

!======================================================================
!
! *** isorropia code
! *** subroutine calcna
! *** calculates nitrates speciation
!
!     nitric acid in the liquid phase is assumed a minor species , that
!     does not significantly perturb the hso4-so4 equilibrium. the nitric
!     acid dissolved is calculated from the hno3(g) -> (h+) + (no3-)
!     equilibrium , using the (h+) from the sulfates.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcna
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: alfa , delt , diak , x
    real(rkx) :: kapa
! character errinf*40
!
! *** calculate hno3 dissolution
! ****************************************
    x = w(4)
    delt = 0.0_rkx
    if ( water>tiny1 ) then
      kapa = molal(1)
      alfa = xk4*r*temp*(water/gama(10))**2
      diak = sqrt((kapa+alfa)**2+4.0*alfa*x)
      delt = 0.5_rkx*(-(kapa+alfa)+diak)
!   if (delt/kapa > 0.1d0) then
!   write (errinf ,'(1pe10.3)') delt/kapa*100.0
!   call pusherr (0019 , errinf)    ! warning error: no solution
!   end if
    end if
!
! *** calculate hno3 speciation in the gas phase
! ************************
    ghno3 = max(x-delt,0.0_rkx)         ! gas hno3
!
! *** calculate hno3 speciation in the liquid phase
! *********************
    molal(7) = delt                  ! no3-
    molal(1) = molal(1) + delt       ! h+
  end subroutine calcna

!======================================================================
!
! *** isorropia code
! *** subroutine calcnap
! *** calculates nitrates speciation
!
!     nitric acid in the liquid phase is assumed a minor species , that
!     does not significantly perturb the hso4-so4 equilibrium. the nitric
!     acid dissolved is calculated from the hno3(g) -> hno3(aq) and
!     hno3(aq) -> (h+) + (cl-) equilibria , using (h+) from the sulfates.
!
!     this is the version used by the inverse problem sover
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcnap
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: alfa , delt
!
! *** is there a liquid phase?
! ******************************************
    if ( water<=tiny1 ) return
!
! *** calculate hno3 speciation in the gas phase
! ************************
    call calcniaq(molal(7),molal(1),delt)
    alfa = xk4*r*temp*(water/gama(10))**2
    gasaq(3) = delt
    molal(1) = molal(1) - delt
    molal(7) = molal(7) - delt
    ghno3 = molal(1)*molal(7)/alfa

    write (*,*) alfa , molal(1) , molal(7) , ghno3 , delt
  end subroutine calcnap

!======================================================================
!
! *** isorropia code
! *** subroutine calcnh3
! *** calculates ammonia in gas phase
!
!     ammonia in the gas phase is assumed a minor species , that
!     does not significantly perturb the aerosol equilibrium.
!     ammonia gas is calculated from the nh3(g) + (h+)(l) <==> (nh4+)(l)
!     equilibrium , using (h+), (nh4+) from the aerosol solution.
!
!     this is the version used by the direct problem
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcnh3
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a1 , bb , cc , chi1 , chi2 , diak , psi
!
! *** is there a liquid phase?
! ******************************************
    if ( water<=tiny1 ) return
!
! *** calculate nh3 sublimation
! *****************************************
    a1 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
    chi1 = molal(3)
    chi2 = molal(1)
!
    bb = (chi2+one/a1)                ! a= 1; b!=1; c!=1
    cc = -chi1/a1
    diak = sqrt(bb*bb-4._rkx*cc)       ! always > 0
    psi = 0.5_rkx*(-bb+diak)             ! one positive root
    psi = max(tiny1,min(psi,chi1))      ! constrict in acceptible range
!
! *** calculate nh3 speciation in the gas phase
! *************************
    gnh3 = psi                       ! gas hno3
!
! *** calculate nh3 affect in the liquid phase
! **************************
    molal(3) = chi1 - psi            ! nh4+
    molal(1) = chi2 + psi            ! h+
  end subroutine calcnh3
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcnh3p
! *** calculates ammonia in gas phase
!
!     ammonia gas is calculated from the nh3(g) + (h+)(l) <==> (nh4+)(l)
!     equilibrium , using (h+), (nh4+) from the aerosol solution.
!
!     this is the version used by the inverse problem solver
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcnh3p
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a1
!
! *** is there a liquid phase?
! ******************************************
    if ( water<=tiny1 ) return
!
! *** calculate nh3 gas phase concentration
! *****************************
    a1 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
    gnh3 = molal(3)/molal(1)/a1
  end subroutine calcnh3p
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcnha
!
!     this subroutine calculates the dissolution of hcl , hno3 at
!     the presence of (h ,so4). hcl , hno3 are considered minor species ,
!     that do not significantly affect the equilibrium point.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcnha
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a3 , a4 , c1 , c2 , c3 , chi3 , chi4 , delcl , delno , omega
    integer(ik4) :: islv
    real(rkx) m1 , m2 , m3
    character(len = 40) :: errinf
!
! *** special case; water = zero
! ******************************************
    if ( water<=tiny1 ) go to 100
!
! *** special case; hcl = hno3=zero
! ***************************************
    if ( w(5)<=tiny1 .and. w(4)<=tiny1 ) return
!
! *** special case; hcl = zero
! ********************************************
    if ( w(5)<=tiny1 ) then
      call calcna                 ! call hno3 dissolution routine
      return
!
!   *** special case; hno3 = zero
!   *******************************************
    else if ( w(4)<=tiny1 ) then
      call calcha                 ! call hcl dissolution routine
      return
    end if
!
! *** calculate equilibrium constants
! ***********************************
    a3 = xk4*r*temp*(water/gama(10))**2     ! hno3
    a4 = xk3*r*temp*(water/gama(11))**2     ! hcl
!
! *** calculate cubic equation coefficients
! *****************************
    delcl = zero
    delno = zero
!
    omega = molal(1)         ! h+
    chi3 = w(4)              ! hno3
    chi4 = w(5)              ! hcl
!
    c1 = a3*chi3
    c2 = a4*chi4
    c3 = a3 - a4
!
    m1 = (c1+c2+(omega+a4)*c3)/c3
    m2 = ((omega+a4)*c2-a4*c3*chi4)/c3
    m3 = -a4*c2*chi4/c3
!
! *** calculate roots
! ***************************************************
    call poly3(m1,m2,m3,delcl,islv)            ! hcl dissolution
    if ( islv/=0 ) then
      delcl = tiny1          ! tiny1 amounts of hcl assumed when no root
      write (errinf,'(1pe10.3)') tiny1
      call pusherr(0022,errinf)          ! warning error: no solution
    end if
    delcl = min(delcl,chi4)
!
    delno = c1*delcl/(c2+c3*delcl)
    delno = min(delno,chi3)
!
    if ( delcl<zero .or. delno<zero .or. delcl>chi4 .or. delno>chi3 ) then
      delcl = tiny1     ! tiny1 amounts of hcl assumed when no root
      delno = tiny1
      write (errinf,'(1pe10.3)') tiny1
      call pusherr(0022,errinf)          ! warning error: no solution
    end if
! &
! ** compare delta to total h+ ; estimate effect to hso4 ***************
!
! if ((delcl+delno)/molal(1) > 0.1d0) then
! write (errinf ,'(1pe10.3)') (delcl+delno)/molal(1)*100.0
! call pusherr (0021 , errinf)
! end if
!
! *** effect on liquid phase
! ********************************************
    molal(1) = molal(1) + (delno+delcl)      ! h+   change
    molal(4) = molal(4) + delcl            ! cl-  change
    molal(7) = molal(7) + delno            ! no3- change
!
! *** effect on gas phase
! ***********************************************
100 continue
    ghcl = max(w(5)-molal(4),tiny1)
    ghno3 = max(w(4)-molal(7),tiny1)
  end subroutine calcnha
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcnhp
!
!     this subroutine calculates the gas phase nitric and hydrochloric
!     acid. concentrations are calculated from the dissolution
!     equilibria , using (h+), (cl-), (no3-) in the aerosol phase.
!
!     this is the version used by the inverse problem solver
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcnhp
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a3 , a4 , delt
!
! *** is there a liquid phase?
! ******************************************
    if ( water<=tiny1 ) return
!
! *** calculate equilibrium constants
! ***********************************
    a3 = xk3*r*temp*(water/gama(11))**2
    a4 = xk4*r*temp*(water/gama(10))**2
    molal(1) = molal(1) + waer(4) + waer(5)    ! h+ increases because no3 , cl are added.
!
! *** calculate concentrations
! ****************************************** *** assume that 'delt'
! from hno3 >> 'delt' from hcl
    call calcniaq(waer(4),molal(1)+molal(7)+molal(4),delt)
    molal(1) = molal(1) - delt
    molal(7) = waer(4) - delt     ! no3- = waer(4) minus any turned into (hno3aq)
    gasaq(3) = delt
!
    call calcclaq(waer(5),molal(1)+molal(7)+molal(4),delt)
    molal(1) = molal(1) - delt
    molal(4) = waer(5) - delt     ! cl- = waer(4) minus any turned into (hno3aq)
    gasaq(2) = delt
!
    ghno3 = molal(1)*molal(7)/a4
    ghcl = molal(1)*molal(4)/a3
  end subroutine calcnhp
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calchco3
! *** corrects for h+ when crustals are in excess
!
!     carbonates are in excess , hco3- is assumed a minor species ,
!     the h+ concentration is calculated from the
!     co2(aq) + h2o <-> (hco3-) + (h+)
!     hco3- <-> (h+) + (co3--) equilibrium.
!     the co3-- concentration is assumed negligible with respect to hco3-
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
!    subroutine calchco3
!      include 'isrpia.inc'
!      real(rkx) kapa
!      character errinf*40
!
! ** special case; water = zero ******************************************
!
!      if (water <= tiny1) then
!         goto 521
!      end if
!
! ** calculate co2 dissolution *****************************************
!
!      rest = 2.d0*w(2) + w(4) + w(5)
!
!      delt = 0.0d0
!      delt2 = 0.0d0
!      if (w(1)+w(6)+w(7)+w(8) > rest) then
!      kapa = molal(1)
!
! ** calculate equilibrium constants ***********************************
!
!      alfa = xk26*rh*(water/1.0)                 ! co2(aq) + h2o
!      alfa2 = xk27*(water/1.0)                    ! hco3-
!
! ** calculate cubic equation coefficients *****************************
!
!      x  = w(1)+w(6)+w(7)+w(8) - rest          ! excess of crustals equals hco3-
!
!      bb =-(kapa + x + alfa)
!      cc = kapa*x
!      dd = bb*bb - 4.d0*cc
!
!      if (dd >= zero) then
!         sqdd  = sqrt(dd)
!         delt  = 0.5d0*(-bb - sqdd)
!      else
!         delt  = zero
!      end if
!
!      end if
!
! ** calculate h+ *****************************************************
!
!      molal(1) = kapa - delt             ! h+
!
!521   return
!
! ** end of subroutine calchco3 ***************************************
!
!    end subroutine
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcamaq
! *** this subroutine calculates the nh3(aq) generated from (h ,nh4+).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcamaq(nh4i,ohi,delt)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a22 , akw , bb , cc , dd , del1 , del2 , delt
    real(rkx) :: ohi , om1 , om2
    real(rkx) :: nh4i
! character errinf*40
!
! *** equilibrium constants
!
    a22 = xk22/xkw/water*(gama(8)/gama(9))**2   ! gama(nh3) assumed 1
    akw = xkw*rh*water*water
!
! *** find root
!
    om1 = nh4i
    om2 = ohi
    bb = -(om1+om2+a22*akw)
    cc = om1*om2
    dd = sqrt(bb*bb-4._rkx*cc)

    del1 = 0.5_rkx*(-bb-dd)
    del2 = 0.5_rkx*(-bb+dd)
!
! *** get appropriate root.
!
    if ( del1>=zero ) then
      delt = del1
    else if ( del2>nh4i .or. del2>ohi ) then
      delt = zero
    else
      delt = del2
    end if
!
! ** compare delta to total nh4+ ; estimate effect *********************
!
! if (delta/hyd > 0.1d0) then
! write (errinf ,'(1pe10.3)') delta/hyd*100.0
! call pusherr (0020 , errinf)
! end if
  end subroutine calcamaq
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcamaq2
!
!     this subroutine calculates the nh3(aq) generated from (h ,nh4+).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcamaq2(ggnh3,nh4i,ohi,nh3aq)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a22 , akw , alf1 , alf2 , bb , cc , del , ggnh3 , ohi
    real(rkx) :: nh4i , nh3aq
!
! *** equilibrium constants
!
    a22 = xk22/xkw/water*(gama(8)/gama(9))**2   ! gama(nh3) assumed 1
    akw = xkw*rh*water*water
!
! *** find root
!
    alf1 = nh4i - ggnh3
    alf2 = ggnh3
    bb = alf1 + a22*akw
    cc = -a22*akw*alf2
    del = 0.5_rkx*(-bb+sqrt(bb*bb-4._rkx*cc))
!
! *** adjust concentrations
!
    nh4i = alf1 + del
    ohi = del
    if ( ohi<=tiny1 ) ohi = sqrt(akw)     ! if solution is neutral.
    nh3aq = alf2 - del
!
  end subroutine calcamaq2

!======================================================================
!
! *** isorropia code
! *** subroutine calcclaq
!
!     this subroutine calculates the hcl(aq) generated from (h+,cl-).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcclaq(cli,hi,delt)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a32 , bb , cc , dd , del1 , del2 , delt , hi , om1 , om2
    real(rkx) :: cli
!
! *** equilibrium constants
!
    a32 = xk32*water/(gama(11))**2   ! gama(hcl) assumed 1
!
! *** find root
!
    om1 = cli
    om2 = hi
    bb = -(om1+om2+a32)
    cc = om1*om2
    dd = sqrt(bb*bb-4._rkx*cc)

    del1 = 0.5_rkx*(-bb-dd)
    del2 = 0.5_rkx*(-bb+dd)
!
! *** get appropriate root.
!
    if ( del1>=zero ) then
      delt = del1
    else if ( del2<zero .or. del2>cli .or. del2>hi ) then
      delt = zero
    else
      delt = del2
    end if
  end subroutine calcclaq
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcclaq2
!
!     this subroutine calculates the hcl(aq) generated from (h+,cl-).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcclaq2(ggcl,cli,hi,claq)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a32 , akw , alf1 , alf2 , claq , coef , del1 , ggcl , hi
    real(rkx) :: cli
!
! *** equilibrium constants
!
    a32 = xk32*water/(gama(11))**2   ! gama(hcl) assumed 1
    akw = xkw*rh*water*water
!
! *** find root
!
    alf1 = cli - ggcl
    alf2 = ggcl
    coef = (alf1+a32)
    del1 = 0.5_rkx*(-coef+sqrt(coef*coef+4._rkx*a32*alf2))
!
! *** correct concentrations
!
    cli = alf1 + del1
    hi = del1
    if ( hi<=tiny1 ) hi = sqrt(akw)     ! if solution is neutral.
    claq = alf2 - del1
  end subroutine calcclaq2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcniaq
!
!     this subroutine calculates the hno3(aq) generated from (h ,no3-).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcniaq(no3i,hi,delt)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a42 , bb , cc , dd , del1 , del2 , om1 , om2
    real(rkx) :: no3i , hi , delt
!
! *** equilibrium constants
!
    a42 = xk42*water/(gama(10))**2   ! gama(hno3) assumed 1
!
! *** find root
!
    om1 = no3i
    om2 = hi
    bb = -(om1+om2+a42)
    cc = om1*om2
    dd = sqrt(bb*bb-4._rkx*cc)

    del1 = 0.5_rkx*(-bb-dd)
    del2 = 0.5_rkx*(-bb+dd)
!
! *** get appropriate root.
!
    if ( del1<zero .or. del1>hi .or. del1>no3i ) then
      !print * , delt
      delt = zero
    else
      delt = del1
      return
    end if
!
    if ( del2<zero .or. del2>no3i .or. del2>hi ) then
      delt = zero
    else
      delt = del2
    end if
  end subroutine calcniaq
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcniaq2
!
!     this subroutine calculates the undissociated hno3(aq)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcniaq2(ggno3,no3i,hi,no3aq)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a42 , akw , alf1 , alf2 , alf3 , bb , cc , del1 , ggno3 , hi
    real(rkx) :: no3i , no3aq
!
! *** equilibrium constants
!
    a42 = xk42*water/(gama(10))**2   ! gama(hno3) assumed 1
    akw = xkw*rh*water*water
!
! *** find root
!
    alf1 = no3i - ggno3
    alf2 = ggno3
    alf3 = hi
!
    bb = alf3 + alf1 + a42
    cc = alf3*alf1 - a42*alf2
    del1 = 0.5_rkx*(-bb+sqrt(bb*bb-4._rkx*cc))
!
! *** correct concentrations
!
    no3i = alf1 + del1
    hi = alf3 + del1
    if ( hi<=tiny1 ) hi = sqrt(akw)     ! if solution is neutral.
    no3aq = alf2 - del1
  end subroutine calcniaq2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcmr
! *** this subroutine calculates:
!     1. ion pair concentrations (from [molar] array)
!     2. water content of liquid aerosol phase (from zsr correlation)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcmr
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aml5 , frcl , frnh4 , frno3 , hso4i , so4i , tots4
    integer(ik4) :: i
    character(len=1) :: sc
!
! *** calculate ion pair concentrations according to specific case ****
!
    sc = scase(1:1)                     ! sulrat & sodrat case
!
! *** nh4-so4 system ; sulfate poor case
!
    if ( sc=='a' ) then
      molalr(4) = molal(5) + molal(6)  ! (nh4)2so4 - correct for so4 to hso4
!
!   *** nh4-so4 system ; sulfate rich case ; no free acid
!
    else if ( sc=='b' ) then
      so4i = molal(5) - molal(1)       ! correct for hso4 dissociation
      hso4i = molal(6) + molal(1)
      if ( so4i<hso4i ) then
        molalr(13) = so4i                       ! [lc] = [so4]
        molalr(9) = max(hso4i-so4i,zero)         ! nh4hso4
      else
        molalr(13) = hso4i                      ! [lc] = [hso4]
        molalr(4) = max(so4i-hso4i,zero)         ! (nh4)2so4
      end if
!
!   *** nh4-so4 system ; sulfate rich case ; free acid
!
    else if ( sc=='c' ) then
      molalr(4) = molal(3)                        ! nh4hso4
      molalr(7) = max(w(2)-w(3),zero)             ! h2so4
!
!   *** nh4-so4-no3 system ; sulfate poor case
!
    else if ( sc=='d' ) then
      molalr(4) = molal(5) + molal(6)             ! (nh4)2so4
      aml5 = molal(3) - 2._rkx*molalr(4)            ! "free" nh4
      molalr(5) = max(min(aml5,molal(7)),zero)     ! nh4no3 = min("free", no3)
!
!   *** nh4-so4-no3 system ; sulfate rich case ; no free acid
!
    else if ( sc=='e' ) then
      so4i = max(molal(5)-molal(1),zero)          ! from hso4 dissociation
      hso4i = molal(6) + molal(1)
      if ( so4i<hso4i ) then
        molalr(13) = so4i                         ! [lc] = [so4]
        molalr(9) = max(hso4i-so4i,zero)           ! nh4hso4
      else
        molalr(13) = hso4i                        ! [lc] = [hso4]
        molalr(4) = max(so4i-hso4i,zero)           ! (nh4)2so4
      end if
!
!   *** nh4-so4-no3 system ; sulfate rich case ; free acid
!
    else if ( sc=='f' ) then
      molalr(4) = molal(3)                                 ! nh4hso4
      molalr(7) = max(molal(5)+molal(6)-molal(3),zero)     ! h2so4
!
!   *** na-nh4-so4-no3-cl system ; sulfate poor ; sodium poor case
!
    else if ( sc=='g' ) then
      molalr(2) = 0.5_rkx*molal(2)                           ! na2so4
      tots4 = molal(5) + molal(6)                          ! total so4
      molalr(4) = max(tots4-molalr(2),zero)                ! (nh4)2so4
      frnh4 = max(molal(3)-2._rkx*molalr(4),zero)
      molalr(5) = min(molal(7),frnh4)                      ! nh4no3
      frnh4 = max(frnh4-molalr(5),zero)
      molalr(6) = min(molal(4),frnh4)                      ! nh4cl
!
!   *** na-nh4-so4-no3-cl system ; sulfate poor ; sodium rich case
!   *** retreive dissolved salts directly from common block /solut/
!
    else if ( sc=='h' ) then
      molalr(1) = psi7                                     ! nacl
      molalr(2) = psi1                                     ! na2so4
      molalr(3) = psi8                                     ! nano3
      molalr(4) = zero                                     ! (nh4)2so4
      frno3 = max(molal(7)-molalr(3),zero)                 ! "free" no3
      frcl = max(molal(4)-molalr(1),zero)                  ! "free" cl
      molalr(5) = min(molal(3),frno3)                      ! nh4no3
      frnh4 = max(molal(3)-molalr(5),zero)                 ! "free" nh3
      molalr(6) = min(frcl,frnh4)                           ! nh4cl
!
!   *** na-nh4-so4-no3-cl system ; sulfate rich case ; no free acid
!   *** retreive dissolved salts directly from common block /solut/
!
    else if ( sc=='i' ) then
      molalr(04) = psi5                                    ! (nh4)2so4
      molalr(02) = psi4                                    ! na2so4
      molalr(09) = psi1                                    ! nh4hso4
      molalr(12) = psi3                                    ! nahso4
      molalr(13) = psi2                                    ! lc
!
!   *** na-nh4-so4-no3-cl system ; sulfate rich case ; free acid
!
    else if ( sc=='j' ) then
      molalr(09) = molal(3)                                ! nh4hso4
      molalr(12) = molal(2)                                ! nahso4
      molalr(07) = molal(5) + molal(6) - molal(3) - molal(2)
                                                           ! h2so4
      molalr(07) = max(molalr(07),zero)
!
!   *** na-nh4-so4-no3-cl-ca-k-mg system ; sulfate poor ; cr+na poor
!   case
    else if ( sc=='o' ) then
      molalr(2) = 0.5_rkx*molal(2)                           ! na2so4
      tots4 = molal(5) + molal(6)                          ! total so4
      molalr(17) = 0.5_rkx*molal(9)                             ! k2so4
      molalr(21) = molal(10)                                ! mgso4
      molalr(4) = max(tots4-molalr(2)-molalr(17)-molalr(21),zero)
                                                          ! (nh4)2so4
      frnh4 = max(molal(3)-2._rkx*molalr(4),zero)
      molalr(5) = min(molal(7),frnh4)                      ! nh4no3
      frnh4 = max(frnh4-molalr(5),zero)
      molalr(6) = min(molal(4),frnh4)                      ! nh4cl
!
!   *** na-nh4-so4-no3-cl-ca-k-mg system ; sulfate poor ; cr+na rich;
!   cr poor case *** retreive dissolved salts directly from common
!   block /solut/
    else if ( sc=='m' ) then
      molalr(1) = psi7                                     ! nacl
      molalr(2) = psi1                                     ! na2so4
      molalr(3) = psi8                                     ! nano3
      molalr(4) = zero                                     ! (nh4)2so4
      frno3 = max(molal(7)-molalr(3),zero)                 ! "free" no3
      frcl = max(molal(4)-molalr(1),zero)                  ! "free" cl
      molalr(5) = min(molal(3),frno3)                      ! nh4no3
      frnh4 = max(molal(3)-molalr(5),zero)                 ! "free" nh3
      molalr(6) = min(frcl,frnh4)                           ! nh4cl
      molalr(17) = psi9                                     ! k2so4
      molalr(21) = psi10                                    ! mgso4
!
!   *** na-nh4-so4-no3-cl-ca-k-mg system ; sulfate poor ; cr+na rich;
!   cr rich case *** retreive dissolved salts directly from common
!   block /solut/
    else if ( sc=='p' ) then
      molalr(1) = psi7                                       ! nacl
      molalr(3) = psi8                                       ! nano3
      molalr(15) = psi12                                      ! cano32
      molalr(16) = psi17                                      ! cacl2
      molalr(19) = psi13                                      ! kno3
      molalr(20) = psi14                                      ! kcl
      molalr(22) = psi15                                      ! mgno32
      molalr(23) = psi16                                      ! mgcl2
      frno3 = max(molal(7)-molalr(3)-2._rkx*molalr(15)-molalr(19)           &
              -2._rkx*molalr(22),zero)                        ! "free" no3
      frcl = max(molal(4)-molalr(1)-2._rkx*molalr(16)-molalr(20)            &
             -2._rkx*molalr(23),zero)                         ! "free" cl
      molalr(5) = min(molal(3),frno3)                        ! nh4no3
      frnh4 = max(molal(3)-molalr(5),zero)                   ! "free" nh3
      molalr(6) = min(frcl,frnh4)                             ! nh4cl
      molalr(17) = psi9                                       ! k2so4
      molalr(21) = psi10                                      ! mgso4
!
!   *** na-nh4-so4-no3-cl-ca-k-mg system ; sulfate rich case ; no free
!   acid
    else if ( sc=='l' ) then
      molalr(04) = psi5                                    ! (nh4)2so4
      molalr(02) = psi4                                    ! na2so4
      molalr(09) = psi1                                    ! nh4hso4
      molalr(12) = psi3                                    ! nahso4
      molalr(13) = psi2                                    ! lc
      molalr(17) = psi6                                    ! k2so4
      molalr(21) = psi7                                    ! mgso4
      molalr(18) = psi8                                    ! khso4
!
!   *** na-nh4-so4-no3-cl-ca-k-mg system ; sulfate super rich case ;
!   free acid
    else if ( sc=='k' ) then
      molalr(09) = molal(3)                                ! nh4hso4
      molalr(12) = molal(2)                                ! nahso4
      molalr(14) = molal(8)                                ! caso4
      molalr(18) = molal(9)                                ! khso4
      molalr(21) = molal(10)                               ! mgso4
      molalr(07) = molal(5) + molal(6) - molal(3) - molal(2) - molal(8)   &
                   - molal(9) - molal(10)                  ! h2so4
      molalr(07) = max(molalr(07),zero)
!
!   ====== reverse problems ===========================================
!
!   *** nh4-so4-no3 system ; sulfate poor case
!
    else if ( sc=='n' ) then
      molalr(4) = molal(5) + molal(6)             ! (nh4)2so4
      aml5 = waer(3) - 2._rkx*molalr(4)             ! "free" nh4
      molalr(5) = max(min(aml5,waer(4)),zero)      ! nh4no3 = min("free", no3)
!
!   *** nh4-so4-no3-na-cl system ; sulfate poor , sodium poor case
!
    else if ( sc=='q' ) then
      molalr(2) = psi1                                     ! na2so4
      molalr(4) = psi6                                     ! (nh4)2so4
      molalr(5) = psi5                                     ! nh4no3
      molalr(6) = psi4                                     ! nh4cl
!
!   *** nh4-so4-no3-na-cl system ; sulfate poor , sodium rich case
!
    else if ( sc=='r' ) then
      molalr(1) = psi3                                     ! nacl
      molalr(2) = psi1                                     ! na2so4
      molalr(3) = psi2                                     ! nano3
      molalr(4) = zero                                     ! (nh4)2so4
      molalr(5) = psi5                                     ! nh4no3
      molalr(6) = psi4                                     ! nh4cl
!
!   *** nh4-so4-no3-na-cl-ca-k-mg system ; sulfate poor ,
!   crustal&sodium poor case
    else if ( sc=='v' ) then
      molalr(2) = psi1                                     ! na2so4
      molalr(4) = psi6                                     ! (nh4)2so4
      molalr(5) = psi5                                     ! nh4no3
      molalr(6) = psi4                                     ! nh4cl
      molalr(17) = psi7                                     ! k2so4
      molalr(21) = psi8                                     ! mgso4
!
!   *** nh4-so4-no3-na-cl-ca-k-mg system ; sulfate poor ,
!   crustal&sodium rich , crustal poor case
    else if ( sc=='u' ) then
      molalr(1) = psi3                                     ! nacl
      molalr(2) = psi1                                     ! na2so4
      molalr(3) = psi2                                     ! nano3
      molalr(5) = psi5                                     ! nh4no3
      molalr(6) = psi4                                     ! nh4cl
      molalr(17) = psi7                                     ! k2so4
      molalr(21) = psi8                                     ! mgso4
!
!   *** nh4-so4-no3-na-cl-ca-k-mg system ; sulfate poor ,
!   crustal&sodium rich , crustal rich case
    else if ( sc=='w' ) then
      molalr(1) = psi7                                     ! nacl
      molalr(3) = psi8                                     ! nano3
      molalr(5) = psi6                                     ! nh4no3
      molalr(6) = psi5                                     ! nh4cl
      molalr(15) = psi12                                    ! cano32
      molalr(16) = psi17                                    ! cacl2
      molalr(17) = psi9                                     ! k2so4
      molalr(19) = psi13                                    ! kno3
      molalr(20) = psi14                                    ! kcl
      molalr(21) = psi10                                    ! mgso4
      molalr(22) = psi15                                    ! mgno32
      molalr(23) = psi16                                    ! mgcl2
!
!   *** unknown case
!
!   else
!   call pusherr (1001 , ' ') ! fatal error: case not supported
    end if
!
! *** calculate water content ; zsr correlation ***********************
!
    water = zero
    do i = 1 , npair
      water = water + molalr(i)/m0(i)
    end do
    water = max(water,tiny1)
  end subroutine calcmr
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcmdrh
!
!     this is the case where the relative humidity is in the mutual
!     drh region. the solution is assumed to be the sum of two weighted
!     solutions ; the 'dry' solution (subroutine drycase) and the
!     'saturated liquid' solution (subroutine liqcase).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcmdrh(rhi,rhdry,rhliq,drycase,liqcase)
    use mod_che_common_isorropia
    implicit none
    interface
      subroutine drycase
      end subroutine drycase
    end interface
    interface
      subroutine liqcase
      end subroutine liqcase
    end interface
    real(rkx) :: clco , cna2so , cnaclo , cnahso , cnano , cnh42so , cnh4clo , &
         cnh4hso , cnh4n3o , dambis , damchl , damg , damnit , damsul ,   &
         dhag , dlc , dnag , dsobis , dsochl , dsonit
    real(rkx) :: dsosul , ghclo , ghno3o , gnh3o , onemwf , rhdry , rhi , &
         rhliq , wf
    integer(ik4) :: i
!
! *** find weight factor **********************************************
!
    if ( wftyp==0 ) then
      wf = one
    else if ( wftyp==1 ) then
      wf = 0.5_rkx
    else
      wf = (rhliq-rhi)/(rhliq-rhdry)
    end if
    onemwf = one - wf
!
! *** find first section ; dry one ************************************
!
    call drycase
    if ( abs(onemwf)>1e-5_rkx ) then         ! dry aerosol
!
      cnh42so = cnh42s4                  ! first (dry) solution
      cnh4hso = cnh4hs4
      clco = clc
      cnh4n3o = cnh4no3
      cnh4clo = cnh4cl
      cna2so = cna2so4
      cnahso = cnahso4
      cnano = cnano3
      cnaclo = cnacl
      gnh3o = gnh3
      ghno3o = ghno3
      ghclo = ghcl
!
!   *** find second section ; dry & liquid
!   ******************************
      cnh42s4 = zero
      cnh4hs4 = zero
      clc = zero
      cnh4no3 = zero
      cnh4cl = zero
      cna2so4 = zero
      cnahso4 = zero
      cnano3 = zero
      cnacl = zero
      gnh3 = zero
      ghno3 = zero
      ghcl = zero
      call liqcase                   ! second (liquid) solution
!
!   *** adjust things for the case that the liquid sub predicts dry
!   aerosol
      if ( water<=tiny1 ) then
        do i = 1 , nions
          molal(i) = zero             ! aqueous phase
        end do
        water = zero
!
        cnh42s4 = cnh42so            ! solid phase
        cna2so4 = cna2so
        cnahso4 = cnahso
        cnh4hs4 = cnh4hso
        clc = clco
        cnh4no3 = cnh4n3o
        cnano3 = cnano
        cnacl = cnaclo
        cnh4cl = cnh4clo
!
        gnh3 = gnh3o                 ! gas phase
        ghno3 = ghno3o
        ghcl = ghclo
        return
      end if
!
!   *** find salt dissolutions between dry & liquid solutions.
!
      damsul = cnh42so - cnh42s4
      dsosul = cna2so - cna2so4
      dambis = cnh4hso - cnh4hs4
      dsobis = cnahso - cnahso4
      dlc = clco - clc
      damnit = cnh4n3o - cnh4no3
      damchl = cnh4clo - cnh4cl
      dsonit = cnano - cnano3
      dsochl = cnaclo - cnacl
!
!   *** find gas dissolutions between dry & liquid solutions.
!
      damg = gnh3o - gnh3
      dhag = ghclo - ghcl
      dnag = ghno3o - ghno3
!
!   *** find solution at mdrh by weighting dry & liquid solutions.
!
!   liquid
!
      molal(1) = onemwf*molal(1)                                 ! h+
      molal(2) = onemwf*(2._rkx*dsosul+dsobis+dsonit+dsochl)       ! na+
      molal(3) = onemwf*(2._rkx*damsul+damg+dambis+damchl+3._rkx*dlc+damnit)
                                                               ! nh4+
      molal(4) = onemwf*(damchl+dsochl+dhag)                     ! cl-
      molal(5) = onemwf*(damsul+dsosul+dlc-molal(6))             ! so4-- !vb 17 sept 2001
      molal(6) = onemwf*(molal(6)+dsobis+dambis+dlc)             ! hso4-
      molal(7) = onemwf*(damnit+dsonit+dnag)                     ! no3-
      water = onemwf*water
!
!   solid
!
      cnh42s4 = wf*cnh42so + onemwf*cnh42s4
      cna2so4 = wf*cna2so + onemwf*cna2so4
      cnahso4 = wf*cnahso + onemwf*cnahso4
      cnh4hs4 = wf*cnh4hso + onemwf*cnh4hs4
      clc = wf*clco + onemwf*clc
      cnh4no3 = wf*cnh4n3o + onemwf*cnh4no3
      cnano3 = wf*cnano + onemwf*cnano3
      cnacl = wf*cnaclo + onemwf*cnacl
      cnh4cl = wf*cnh4clo + onemwf*cnh4cl
!
!   gas
!
      gnh3 = wf*gnh3o + onemwf*gnh3
      ghno3 = wf*ghno3o + onemwf*ghno3
      ghcl = wf*ghclo + onemwf*ghcl
    end if
  end subroutine calcmdrh
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcmdrh2
!
!     this is the case where the relative humidity is in the mutual
!     drh region. the solution is assumed to be the sum of two weighted
!     solutions ; the 'dry' solution (subroutine drycase) and the
!     'saturated liquid' solution (subroutine liqcase).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcmdrh2(rhi,rhdry,rhliq,drycase,liqcase)
    use mod_che_common_isorropia
    implicit none
    interface
      subroutine drycase
      end subroutine drycase
    end interface
    interface
      subroutine liqcase
      end subroutine liqcase
    end interface
    real(rkx) :: ccac2l , ccan32o , ccaso , ck2so , ckclo , ckhso , ckn3o , &
         clco , cmgc2l , cmgn32o , cmgso , cna2so , cnaclo , cnahso , cnano , &
         cnh42so , cnh4clo , cnh4hso , cnh4n3o , dambis
    real(rkx) :: damchl , damg , damnit , damsul , dcachl , dcanit , dcasul , &
         dhag , dlc , dmgchl , dmgnit , dmgsul , dnag , dpobis , dpochl , &
         dponit , dposul , dsobis , dsochl , dsonit
    real(rkx) :: dsosul , ghclo , ghno3o , gnh3o , onemwf , rhdry , rhi , &
         rhliq , wf
    integer(ik4) :: i
!
! *** find weight factor **********************************************
!
    if ( wftyp==0 ) then
      wf = one
    else if ( wftyp==1 ) then
      wf = 0.5_rkx
    else
      wf = (rhliq-rhi)/(rhliq-rhdry)
    end if
    onemwf = one - wf
!
! *** find first section ; dry one ************************************
!
    call drycase
    if ( abs(onemwf)<=1e-5_rkx ) return    ! dry aerosol
!
    cnh42so = cnh42s4                    ! first (dry) solution
    cnh4hso = cnh4hs4
    clco = clc
    cnh4n3o = cnh4no3
    cnh4clo = cnh4cl
    cna2so = cna2so4
    cnahso = cnahso4
    cnano = cnano3
    cnaclo = cnacl
    gnh3o = gnh3
    ghno3o = ghno3
    ghclo = ghcl
!
    ccaso = ccaso4
    ck2so = ck2so4
    cmgso = cmgso4
    ckhso = ckhso4
    ccan32o = ccano32
    ccac2l = ccacl2
    ckn3o = ckno3
    ckclo = ckcl
    cmgn32o = cmgno32
    cmgc2l = cmgcl2
!
! *** find second section ; dry & liquid ******************************
!
    cnh42s4 = zero
    cnh4hs4 = zero
    clc = zero
    cnh4no3 = zero
    cnh4cl = zero
    cna2so4 = zero
    cnahso4 = zero
    cnano3 = zero
    cnacl = zero
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
!
    ccaso4 = zero
    ck2so4 = zero
    cmgso4 = zero
    ckhso4 = zero
    ccano32 = zero
    ccacl2 = zero
    ckno3 = zero
    ckcl = zero
    cmgno32 = zero
    cmgcl2 = zero
!
    call liqcase                     ! second (liquid) solution
!
! *** adjust things for the case that the liquid sub predicts dry
! aerosol
    if ( water<=tiny1 ) then
      do i = 1 , nions
        molal(i) = zero               ! aqueous phase
      end do
      water = zero
!
      cnh42s4 = cnh42so              ! solid phase
      cna2so4 = cna2so
      cnahso4 = cnahso
      cnh4hs4 = cnh4hso
      clc = clco
      cnh4no3 = cnh4n3o
      cnano3 = cnano
      cnacl = cnaclo
      cnh4cl = cnh4clo
!
      gnh3 = gnh3o                   ! gas phase
      ghno3 = ghno3o
      ghcl = ghclo
!
      ccaso4 = ccaso
      ck2so4 = ck2so
      cmgso4 = cmgso
      ckhso4 = ckhso
      ccano32 = ccan32o
      ccacl2 = ccac2l
      ckno3 = ckn3o
      ckcl = ckclo
      cmgno32 = cmgn32o
      cmgcl2 = cmgc2l
!
      return
    end if
!
! *** find salt dissolutions between dry & liquid solutions.
!
    damsul = cnh42so - cnh42s4
    dsosul = cna2so - cna2so4
    dambis = cnh4hso - cnh4hs4
    dsobis = cnahso - cnahso4
    dlc = clco - clc
    damnit = cnh4n3o - cnh4no3
    damchl = cnh4clo - cnh4cl
    dsonit = cnano - cnano3
    dsochl = cnaclo - cnacl
!
    dcasul = ccaso - ccaso4
    dposul = ck2so - ck2so4
    dmgsul = cmgso - cmgso4
    dpobis = ckhso - ckhso4
    dcanit = ccan32o - ccano32
    dcachl = ccac2l - ccacl2
    dponit = ckn3o - ckno3
    dpochl = ckclo - ckcl
    dmgnit = cmgn32o - cmgno32
    dmgchl = cmgc2l - cmgcl2
!
! *** find gas dissolutions between dry & liquid solutions.
!
    damg = gnh3o - gnh3
    dhag = ghclo - ghcl
    dnag = ghno3o - ghno3
!
! *** find solution at mdrh by weighting dry & liquid solutions.
!
! liquid
!
    molal(1) = onemwf*molal(1)                                       ! h+
    molal(2) = onemwf*(2._rkx*dsosul+dsobis+dsonit+dsochl)             ! na+
    molal(3) = onemwf*(2._rkx*damsul+damg+dambis+damchl+3._rkx*dlc+damnit)
                                                                    ! nh4+
    molal(4) = onemwf*(damchl+dsochl+dhag+2._rkx*dcachl+2._rkx*dmgchl+dpochl)
                                                                     ! cl-
    molal(5) = onemwf*(damsul+dsosul+dlc-molal(6)+dcasul+dposul+dmgsul)
                                                                    ! so4-- !vb 17 sept 2001
    molal(6) = onemwf*(molal(6)+dsobis+dambis+dlc+dpobis)            ! hso4-
    molal(7) = onemwf*(damnit+dsonit+dnag+2._rkx*dcanit+2._rkx*dmgnit+dponit)
                                                                    ! no3-
    molal(8) = onemwf*(dcasul+dcanit+dcachl)                         ! ca2+
    molal(9) = onemwf*(2._rkx*dposul+dponit+dpochl+dpobis)             ! k+
    molal(10) = onemwf*(dmgsul+dmgnit+dmgchl)                         ! mg2+
    water = onemwf*water
!
! solid
!
    cnh42s4 = wf*cnh42so + onemwf*cnh42s4
    cna2so4 = wf*cna2so + onemwf*cna2so4
    cnahso4 = wf*cnahso + onemwf*cnahso4
    cnh4hs4 = wf*cnh4hso + onemwf*cnh4hs4
    clc = wf*clco + onemwf*clc
    cnh4no3 = wf*cnh4n3o + onemwf*cnh4no3
    cnano3 = wf*cnano + onemwf*cnano3
    cnacl = wf*cnaclo + onemwf*cnacl
    cnh4cl = wf*cnh4clo + onemwf*cnh4cl
!
    ccaso4 = wf*ccaso + onemwf*ccaso4
    ck2so4 = wf*ck2so + onemwf*ck2so4
    cmgso4 = wf*cmgso + onemwf*cmgso4
    ckhso4 = wf*ckhso + onemwf*ckhso4
    ccano32 = wf*ccan32o + onemwf*ccano32
    ccacl2 = wf*ccac2l + onemwf*ccacl2
    cmgno32 = wf*cmgn32o + onemwf*cmgno32
    cmgcl2 = wf*cmgc2l + onemwf*cmgcl2
    ckcl = wf*ckclo + onemwf*ckcl
!
! gas
!
    gnh3 = wf*gnh3o + onemwf*gnh3
    ghno3 = wf*ghno3o + onemwf*ghno3
    ghcl = wf*ghclo + onemwf*ghcl
!
  end subroutine calcmdrh2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcmdrp
!
!     this is the case where the relative humidity is in the mutual
!     drh region. the solution is assumed to be the sum of two weighted
!     solutions ; the 'dry' solution (subroutine drycase) and the
!     'saturated liquid' solution (subroutine liqcase).   (reverse problem)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcmdrp(rhi,rhdry,rhliq,drycase,liqcase)
    use mod_che_common_isorropia
    implicit none
    interface
      subroutine drycase
      end subroutine drycase
    end interface
    interface
      subroutine liqcase
      end subroutine liqcase
    end interface
    real(rkx) :: a2 , a3 , a4 , a8 , clco , cna2so , cnaclo , cnahso , cnano , &
         cnh42so , cnh4clo , cnh4hso , cnh4n3o , dambis , dlc , dsobis ,  &
         hien , hieq , onemwf , rhdry
    real(rkx) :: rhi , rhliq , wf
    integer(ik4) :: i
!
! *** find weight factor **********************************************
!
    if ( wftyp==0 ) then
      wf = one
    else if ( wftyp==1 ) then
      wf = 0.5_rkx
    else
      wf = (rhliq-rhi)/(rhliq-rhdry)
    end if
    onemwf = one - wf
!
! *** find first section ; dry one ************************************
!
    call drycase
    if ( abs(onemwf)>1e-5_rkx ) then         ! dry aerosol
!
      cnh42so = cnh42s4              ! first (dry) solution
      cnh4hso = cnh4hs4
      clco = clc
      cnh4n3o = cnh4no3
      cnh4clo = cnh4cl
      cna2so = cna2so4
      cnahso = cnahso4
      cnano = cnano3
      cnaclo = cnacl
!
!   *** find second section ; dry & liquid
!   ******************************
      cnh42s4 = zero
      cnh4hs4 = zero
      clc = zero
      cnh4no3 = zero
      cnh4cl = zero
      cna2so4 = zero
      cnahso4 = zero
      cnano3 = zero
      cnacl = zero
      gnh3 = zero
      ghno3 = zero
      ghcl = zero
      call liqcase                   ! second (liquid) solution
!
!   *** adjust things for the case that the liquid sub predicts dry
!   aerosol
      if ( water<=tiny1 ) then
        water = zero
        do i = 1 , nions
          molal(i) = zero
        end do
        call drycase
        return
      end if
!
!   *** find salt dissolutions between dry & liquid solutions.
!
      dambis = cnh4hso - cnh4hs4
      dsobis = cnahso - cnahso4
      dlc = clco - clc
!
!   *** find solution at mdrh by weighting dry & liquid solutions.
!
!   *** solid
!
      cnh42s4 = wf*cnh42so + onemwf*cnh42s4
      cna2so4 = wf*cna2so + onemwf*cna2so4
      cnahso4 = wf*cnahso + onemwf*cnahso4
      cnh4hs4 = wf*cnh4hso + onemwf*cnh4hs4
      clc = wf*clco + onemwf*clc
      cnh4no3 = wf*cnh4n3o + onemwf*cnh4no3
      cnano3 = wf*cnano + onemwf*cnano3
      cnacl = wf*cnaclo + onemwf*cnacl
      cnh4cl = wf*cnh4clo + onemwf*cnh4cl
!
!   *** liquid
!
      water = onemwf*water
!
      molal(2) = waer(1) - 2._rkx*cna2so4 - cnahso4 - cnano3 - cnacl
                                                               ! na+
      molal(3) = waer(3) - 2._rkx*cnh42s4 - cnh4hs4 - cnh4cl - 3._rkx*clc -   &
                 cnh4no3                                       ! nh4+
      molal(4) = waer(5) - cnacl - cnh4cl                        ! cl-
      molal(7) = waer(4) - cnano3 - cnh4no3                      ! no3-
      molal(6) = onemwf*(molal(6)+dsobis+dambis+dlc)             ! hso4-
      molal(5) = waer(2) - molal(6) - clc - cnh42s4 - cna2so4    ! so4--
!
      a8 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
      if ( molal(5)<=tiny1 ) then
        hieq = sqrt(xkw*rh*water*water)    ! neutral solution
      else
        hieq = a8*molal(6)/molal(5)
      end if
      hien = molal(4) + molal(7) + molal(6) + 2._rkx*molal(5) - molal(2)    &
             - molal(3)
      molal(1) = max(hieq,hien)                                   ! h+
!
!   *** gas (activity coefs from liquid solution)
!
      a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2     ! nh3  <==> nh4+
      a3 = xk4*r*temp*(water/gama(10))**2             ! hno3 <==> no3-
      a4 = xk3*r*temp*(water/gama(11))**2             ! hcl  <==> cl-
!
      gnh3 = molal(3)/max(molal(1),tiny1)/a2
      ghno3 = molal(1)*molal(7)/a3
      ghcl = molal(1)*molal(4)/a4
    end if
  end subroutine calcmdrp
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcmdrpii
!
!     this is the case where the relative humidity is in the mutual
!     drh region. the solution is assumed to be the sum of two weighted
!     solutions ; the 'dry' solution (subroutine drycase) and the
!     'saturated liquid' solution (subroutine liqcase).   (reverse problem)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcmdrpii(rhi,rhdry,rhliq,drycase,liqcase)
    use mod_che_common_isorropia
    implicit none
    interface
      subroutine drycase
      end subroutine drycase
    end interface
    interface
      subroutine liqcase
      end subroutine liqcase
    end interface
    real(rkx) :: a2 , a3 , a4 , a8 , ccac2l , ccan32o , ccaso , ck2so , ckclo ,&
         ckhso , ckn3o , clco , cmgc2l , cmgn32o , cmgso , cna2so ,       &
         cnaclo , cnahso , cnano , cnh42so
    real(rkx) :: cnh4clo , cnh4hso , cnh4n3o , dambis , dlc , dpobis , &
         dsobis , hien , hieq , onemwf , rhdry , rhi , rhliq , wf
    integer(ik4) :: i
!
! *** find weight factor **********************************************
!
    if ( wftyp==0 ) then
      wf = one
    else if ( wftyp==1 ) then
      wf = 0.5_rkx
    else
      wf = (rhliq-rhi)/(rhliq-rhdry)
    end if
    onemwf = one - wf
!
! *** find first section ; dry one ************************************
!
    call drycase
    if ( abs(onemwf)<=1e-5_rkx ) return   ! dry aerosol
!
    cnh42so = cnh42s4                ! first (dry) solution
    cnh4hso = cnh4hs4
    clco = clc
    cnh4n3o = cnh4no3
    cnh4clo = cnh4cl
    cna2so = cna2so4
    cnahso = cnahso4
    cnano = cnano3
    cnaclo = cnacl
!
    ccaso = ccaso4
    ck2so = ck2so4
    cmgso = cmgso4
    ckhso = ckhso4
    ccan32o = ccano32
    ccac2l = ccacl2
    ckn3o = ckno3
    ckclo = ckcl
    cmgn32o = cmgno32
    cmgc2l = cmgcl2
!
! *** find second section ; dry & liquid ******************************
!
    cnh42s4 = zero
    cnh4hs4 = zero
    clc = zero
    cnh4no3 = zero
    cnh4cl = zero
    cna2so4 = zero
    cnahso4 = zero
    cnano3 = zero
    cnacl = zero
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
!
    ccaso4 = zero
    ck2so4 = zero
    cmgso4 = zero
    ckhso4 = zero
    ccano32 = zero
    ccacl2 = zero
    ckno3 = zero
    ckcl = zero
    cmgno32 = zero
    cmgcl2 = zero
!
    call liqcase                     ! second (liquid) solution
!
! *** adjust things for the case that the liquid sub predicts dry
! aerosol
    if ( water<=tiny1 ) then
      water = zero
      do i = 1 , nions
        molal(i) = zero
      end do
      call drycase
      return
    end if
!
! *** find salt dissolutions between dry & liquid solutions.
!
    dambis = cnh4hso - cnh4hs4
    dsobis = cnahso - cnahso4
    dlc = clco - clc
    dpobis = ckhso - ckhso4
!
! *** find solution at mdrh by weighting dry & liquid solutions.
!
! *** solid
!
    cnh42s4 = wf*cnh42so + onemwf*cnh42s4
    cna2so4 = wf*cna2so + onemwf*cna2so4
    cnahso4 = wf*cnahso + onemwf*cnahso4
    cnh4hs4 = wf*cnh4hso + onemwf*cnh4hs4
    clc = wf*clco + onemwf*clc
    cnh4no3 = wf*cnh4n3o + onemwf*cnh4no3
    cnano3 = wf*cnano + onemwf*cnano3
    cnacl = wf*cnaclo + onemwf*cnacl
    cnh4cl = wf*cnh4clo + onemwf*cnh4cl
!
    ccaso4 = wf*ccaso + onemwf*ccaso4
    ck2so4 = wf*ck2so + onemwf*ck2so4
    cmgso4 = wf*cmgso + onemwf*cmgso4
    ckhso4 = wf*ckhso + onemwf*ckhso4
    ccano32 = wf*ccan32o + onemwf*ccano32
    ccacl2 = wf*ccac2l + onemwf*ccacl2
    cmgno32 = wf*cmgn32o + onemwf*cmgno32
    cmgcl2 = wf*cmgc2l + onemwf*cmgcl2
    ckcl = wf*ckclo + onemwf*ckcl
!
! *** liquid
!
    water = onemwf*water
!
    molal(2) = waer(1) - 2._rkx*cna2so4 - cnahso4 - cnano3 - cnacl     ! na+
    molal(3) = waer(3) - 2._rkx*cnh42s4 - cnh4hs4 - cnh4cl - 3._rkx*clc -     &
               cnh4no3                                               ! nh4+
    molal(4) = waer(5) - cnacl - cnh4cl - 2._rkx*ccacl2 - 2._rkx*cmgcl2 - ckcl
                                                                     ! cl-
    molal(7) = waer(4) - cnano3 - cnh4no3 - ckno3 - 2._rkx*ccano32 -        &
               2._rkx*cmgno32                                          ! no3-
    molal(6) = onemwf*(molal(6)+dsobis+dambis+dlc+dpobis)              ! hso4-
    molal(5) = waer(2) - molal(6) - clc - cnh42s4 - cna2so4 - ccaso4 -    &
               ck2so4 - cmgso4                                       ! so4--
    molal(8) = waer(6) - ccaso4 - ccano32 - ccacl2                     ! ca++
    molal(9) = waer(7) - 2._rkx*ck2so4 - ckno3 - ckcl - ckhso4           ! k+
    molal(10) = waer(8) - cmgso4 - cmgno32 - cmgcl2                     ! mg++
!
    a8 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
    if ( molal(5)<=tiny1 ) then
      hieq = sqrt(xkw*rh*water*water)      ! neutral solution
    else
      hieq = a8*molal(6)/molal(5)
    end if
    hien = molal(4) + molal(7) + molal(6) + 2._rkx*molal(5) - molal(2)      &
           - molal(3)
    molal(1) = max(hieq,hien)                                           ! h+
!
! *** gas (activity coefs from liquid solution)
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = molal(3)/max(molal(1),tiny1)/a2
    ghno3 = molal(1)*molal(7)/a3
    ghcl = molal(1)*molal(4)/a4
!
  end subroutine calcmdrpii
!
!======================================================================
!
! *** isorropia code
! *** subroutine calchs4
! *** this subroutine calculates the hso4 generated from (h ,so4).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calchs4(hi,so4i,hso4i,delta)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a8 , bb , cc , dd , delta , delta1 , delta2 , hi , &
         hso4i , so4i , sqdd
!
! *** if too little water , dont solve
!
    if ( water<=1D1*tiny1 ) then
      delta = zero
      return
    end if
!
! *** calculate hso4 speciation
! *****************************************
    a8 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
    bb = -(hi+so4i+a8)
    cc = hi*so4i - hso4i*a8
    dd = bb*bb - 4._rkx*cc
!
    if ( dd>=zero ) then
      sqdd = sqrt(dd)
      delta1 = 0.5_rkx*(-bb+sqdd)
      delta2 = 0.5_rkx*(-bb-sqdd)
      if ( hso4i<=tiny1 ) then
        delta = delta2
      else if ( hi*so4i>=a8*hso4i ) then
        delta = delta2
      else if ( hi*so4i<a8*hso4i ) then
        delta = delta1
      else
        delta = zero
      end if
    else
      delta = zero
    end if
!
! ** compare delta to total h+ ; estimate effect of hso4 ***************
!
! hyd = max(hi , molal(1))
! if (hyd > tiny1) then
! if (delta/hyd > 0.1d0) then
! write (errinf ,'(1pe10.3)') delta/hyd*100.0
! call pusherr (0020 , errinf)
! end if
! end if
!
  end subroutine calchs4

!======================================================================
!
! *** isorropia code
! *** subroutine calcph
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcph(gg,hi,ohi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: akw , bb , cc , cn , dd , gg , hi , ohi
!
    akw = xkw*rh*water*water
    cn = sqrt(akw)
!
! *** gg = (negative charge) - (positive charge)
!
    if ( gg>tiny1 ) then                          ! h+ in excess
      bb = -gg
      cc = -akw
      dd = bb*bb - 4._rkx*cc
      hi = max(0.5_rkx*(-bb+sqrt(dd)),cn)
      ohi = akw/hi
    else                                          ! oh- in excess
      bb = gg
      cc = -akw
      dd = bb*bb - 4._rkx*cc
      ohi = max(0.5_rkx*(-bb+sqrt(dd)),cn)
      hi = akw/ohi
    end if
  end subroutine calcph
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcact
! *** calculates multi-component activity coefficients from bromleys
!     method. the binary activity coefficients are calculated by
!     kusik-meisner relation (subroutine kmtab or subroutine kmful).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
!
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcact
    use mod_che_common_isorropia
    implicit none
    if ( w(1)+w(4)+w(5)+w(6)+w(7)+w(8)<=tiny1 ) then
         !a ,k ,mg ,na ,cl ,no3 = 0
      call calcact1
    else if ( w(1)+w(5)+w(6)+w(7)+w(8)<=tiny1 ) then
         !a ,k ,mg ,na ,cl = 0
      call calcact2
    else if ( w(6)+w(7)+w(8)<=tiny1 ) then
         !a ,k ,mg = 0
      call calcact3
    else
      call calcact4
    end if
  end subroutine calcact
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcact4
! *** calculates multi-component activity coefficients from bromleys
!     method for an ammonium-sulfate-nitrate-chloride-sodium-calcium-potassium-magnesium
!     aerosol system. the binary activity coefficients are calculated by
!     kusik-meisner relation (subroutine kmtab or subroutine kmful4).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
!
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcact4
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: errin , errou
    integer(ik4) :: i , j
    real(rkx) , dimension(6,4) :: g0
    real(rkx) , dimension(6) :: f1
    real(rkx) , dimension(4) :: f2a , f2b
    real(rkx) :: zpl , zmi , agama , sion , h , ch
    real(rkx) :: mpl , xij , yji
    data g0/24*0_rkx/
!
! *** save activities in old array *************************************
!
    if ( frst ) then               ! outer loop
      do i = 1 , npair
        gamou(i) = gama(i)
      end do
    end if
!
    do i = 1 , npair               ! inner loop
      gamin(i) = gama(i)
    end do
!
! *** calculate ionic activity of solution *****************************
!
    ionic = 0.0
    do i = 1 , nions
      ionic = ionic + molal(i)*z(i)*z(i)
    end do
    ionic = max(min(0.5_rkx*ionic/water,100._rkx),tiny1)
!
! *** calculate binary activity coefficients ***************************
!
! g01 ,1) = g11;g0(1 ,2)=g07;g0(1 ,3)=g08;g0(1 ,4)=g10;g0(2
! ,1)=g01;g0(2 ,2)=g02 & g02 ,3) = g12;g0(2 ,4)=g03;g0(3 ,1)=g06;g0(3
! ,2)=g04;g0(3 ,3)=g09;g0(3 ,4)=g05
    if ( iacalc==0 ) then                ! k.m.; full
      call kmful4(ionic,real(temp,rkx),g0(2,1),g0(2,2),g0(2,4),g0(3,2),g0(3,4)&
                  ,g0(3,1),g0(1,2),g0(1,3),g0(3,3),g0(1,4),g0(1,1),g0(2,3)&
                  ,g0(4,4),g0(4,1),g0(5,2),g0(5,3),g0(5,4),g0(5,1),g0(6,2)&
                  ,g0(6,4),g0(6,1))
    else                                 ! k.m.; tabulated
      call kmtab(ionic,real(temp,rkx),g0(2,1),g0(2,2),g0(2,4),g0(3,2),g0(3,4),&
                 g0(3,1),g0(1,2),g0(1,3),g0(3,3),g0(1,4),g0(1,1),g0(2,3), &
                 g0(4,4),g0(4,1),g0(5,2),g0(5,3),g0(5,4),g0(5,1),g0(6,2), &
                 g0(6,4),g0(6,1))
    end if
!
! *** calculate multicomponent activity coefficients *******************
!
    agama = 0.511*(298.0/temp)**1.5      ! debye huckel const. at t
    sion = sqrt(ionic)
    h = agama*sion/(1+sion)
!
    do i = 1 , 4
      f1(i) = 0.0
      f2a(i) = 0.0
      f2b(i) = 0.0
    end do
    f1(5) = 0.0
    f1(6) = 0.0
!
    do i = 1 , 3
      zpl = z(i)
      mpl = molal(i)/water
      do j = 1 , 4
        zmi = z(j+3)
        ch = 0.25_rkx*(zpl+zmi)*(zpl+zmi)/ionic
        xij = ch*mpl
        yji = ch*molal(j+3)/water
        f1(i) = f1(i) + real(yji*(g0(i,j)+zpl*zmi*h),rkx)
        f2a(j) = f2a(j) + real(xij*(g0(i,j)+zpl*zmi*h),rkx)
      end do
    end do
!
    do i = 4 , 6
      zpl = z(i+4)
      mpl = molal(i+4)/water
      do j = 1 , 4
        zmi = z(j+3)
        if ( j==3 ) then
          if ( i==4 .or. i==6 ) go to 100
        end if
        ch = 0.25_rkx*(zpl+zmi)*(zpl+zmi)/ionic
        xij = ch*mpl
        yji = ch*molal(j+3)/water
        f1(i) = f1(i) + real(yji*(g0(i,j)+zpl*zmi*h),rkx)
        f2b(j) = f2b(j) + real(xij*(g0(i,j)+zpl*zmi*h),rkx)
      end do
    end do
!
! *** log10 of activity coefficients ***********************************
!
100 continue
    gama(01) = ga(2,1)*zz(01)                    ! nacl
    gama(02) = ga(2,2)*zz(02)                    ! na2so4
    gama(03) = ga(2,4)*zz(03)                    ! nano3
    gama(04) = ga(3,2)*zz(04)                    ! (nh4)2so4
    gama(05) = ga(3,4)*zz(05)                    ! nh4no3
    gama(06) = ga(3,1)*zz(06)                    ! nh4cl
    gama(07) = ga(1,2)*zz(07)                    ! 2h-so4
    gama(08) = ga(1,3)*zz(08)                    ! h-hso4
    gama(09) = ga(3,3)*zz(09)                    ! nh4hso4
    gama(10) = ga(1,4)*zz(10)                    ! hno3
    gama(11) = ga(1,1)*zz(11)                    ! hcl
    gama(12) = ga(2,3)*zz(12)                    ! nahso4
    gama(13) = 0.20*(3.0*gama(04)+2.0*gama(09))  ! lc ; scape
! gama(13) = 0.50*(gama(04)+gama(09))            ! lc ; sequilib
! gama(13) = 0.25_rkx*(3.0*gama(04)+gama(07))      ! lc ; aim
    gama(14) = 0.0_rkx                             ! caso4
    gama(15) = gb(4,4)*zz(15)                    ! ca(no3)2
    gama(16) = gb(4,1)*zz(16)                    ! cacl2
    gama(17) = gb(5,2)*zz(17)                    ! k2so4
    gama(18) = gb(5,3)*zz(18)                    ! khso4
    gama(19) = gb(5,4)*zz(19)                    ! kno3
    gama(20) = gb(5,1)*zz(20)                    ! kcl
    gama(21) = gb(6,2)*zz(21)                    ! mgso4
    gama(22) = gb(6,4)*zz(22)                    ! mg(no3)2
    gama(23) = gb(6,1)*zz(23)                    ! mgcl2
!
! *** convert log (gama) coefficients to gama **************************
!
    do i = 1 , npair
      gama(i) = max(-5.0_rkx,min(gama(i),5.0_rkx))       ! f77 library routine
      gama(i) = 10.0**gama(i)
!   gama(i) = ex10(real(gama(i),rkx), 5.0)    ! cutoff set to [-5 ,5]
    end do
!
! *** setup activity calculation flags ********************************
!
! outr calculation loop ; only if frst =.true.
!
    if ( frst ) then
      errou = zero                       ! convergence criterion
      do i = 1 , npair
        errou = max(errou,abs((gamou(i)-gama(i))/gamou(i)))
      end do
      calaou = errou>=epsact             ! setup flags
      frst = .false.
    end if
!
! innr calculation loop ; always
!
    errin = zero                         ! convergence criterion
    do i = 1 , npair
      errin = max(errin,abs((gamin(i)-gama(i))/gamin(i)))
    end do
    calain = errin>=epsact
!
    iclact = iclact + 1                  ! increment activity call counter

    contains
      real(rkx) function ga(i,j)
        implicit none
        integer(ik4) , intent(in) :: i , j
        ga = (f1(i)/z(i)+f2a(j)/z(j+3))/(z(i)+z(j+3)) - h
      end function ga
      real(rkx) function gb(i,j)
        implicit none
        integer(ik4) , intent(in) :: i , j
        gb = (f1(i)/z(i+4)+f2b(j)/z(j+3))/(z(i+4)+z(j+3)) - h
      end function gb
  end subroutine calcact4
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcact3
! *** calculates multi-component activity coefficients from bromleys
!     method for an ammonium-sulfate-nitrate-chloride-sodium aerosol system.
!     the binary activity coefficients are calculated by
!     kusik-meisner relation (subroutine kmtab or subroutine kmful3).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
!
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcact3
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: errin , errou
    integer(ik4) :: i , j
    real(rkx) , dimension(6,4) :: g0
    real(rkx) , dimension(3) ::  f1
    real(rkx) , dimension(4) ::  f2
    real(rkx) :: zpl , zmi , agama , sion , h , ch
    real(rkx) :: mpl , xij , yji
!
! *** save activities in old array *************************************
!
    if ( frst ) then               ! outer loop
      do i = 1 , 13
        gamou(i) = gama(i)
      end do
    end if
!
    do i = 1 , 13                 ! inner loop
      gamin(i) = gama(i)
    end do
!
! *** calculate ionic activity of solution *****************************
!
    ionic = 0.0
    do i = 1 , 7
      ionic = ionic + molal(i)*z(i)*z(i)
    end do
    ionic = max(min(0.5_rkx*ionic/water,100._rkx),tiny1)
!
! *** calculate binary activity coefficients ***************************
!
! g01 ,1) = g11;g0(1 ,2)=g07;g0(1 ,3)=g08;g0(1 ,4)=g10;g0(2
! ,1)=g01;g0(2 ,2)=g02 & g02 ,3) = g12;g0(2 ,4)=g03;g0(3 ,1)=g06;g0(3
! ,2)=g04;g0(3 ,3)=g09;g0(3 ,4)=g05
    if ( iacalc==0 ) then                ! k.m.; full
      call kmful3(ionic,real(temp,rkx),g0(2,1),g0(2,2),g0(2,4),g0(3,2),g0(3,4)&
                  ,g0(3,1),g0(1,2),g0(1,3),g0(3,3),g0(1,4),g0(1,1),g0(2,3)&
                  )
    else                                 ! k.m.; tabulated
      call kmtab(ionic,real(temp,rkx),g0(2,1),g0(2,2),g0(2,4),g0(3,2),g0(3,4),&
                 g0(3,1),g0(1,2),g0(1,3),g0(3,3),g0(1,4),g0(1,1),g0(2,3), &
                 g0(4,4),g0(4,1),g0(5,2),g0(5,3),g0(5,4),g0(5,1),g0(6,2), &
                 g0(6,4),g0(6,1))
    end if
!
! *** calculate multicomponent activity coefficients *******************
!
    agama = 0.511*(298.0/temp)**1.5      ! debye huckel const. at t
    sion = sqrt(ionic)
    h = agama*sion/(1+sion)
!
    do i = 1 , 3
      f1(i) = 0.0
      f2(i) = 0.0
    end do
    f2(4) = 0.0
!
    do i = 1 , 3
      zpl = z(i)
      mpl = molal(i)/water
      do j = 1 , 4
        zmi = z(j+3)
        ch = 0.25_rkx*(zpl+zmi)*(zpl+zmi)/ionic
        xij = ch*mpl
        yji = ch*molal(j+3)/water
        f1(i) = f1(i) + real(yji*(g0(i,j)+zpl*zmi*h),rkx)
        f2(j) = f2(j) + real(xij*(g0(i,j)+zpl*zmi*h),rkx)
      end do
    end do
!
! *** log10 of activity coefficients ***********************************
!
    gama(01) = g(2,1)*zz(01)                        ! nacl
    gama(02) = g(2,2)*zz(02)                        ! na2so4
    gama(03) = g(2,4)*zz(03)                        ! nano3
    gama(04) = g(3,2)*zz(04)                        ! (nh4)2so4
    gama(05) = g(3,4)*zz(05)                        ! nh4no3
    gama(06) = g(3,1)*zz(06)                        ! nh4cl
    gama(07) = g(1,2)*zz(07)                        ! 2h-so4
    gama(08) = g(1,3)*zz(08)                        ! h-hso4
    gama(09) = g(3,3)*zz(09)                        ! nh4hso4
    gama(10) = g(1,4)*zz(10)                        ! hno3
    gama(11) = g(1,1)*zz(11)                        ! hcl
    gama(12) = g(2,3)*zz(12)                        ! nahso4
    gama(13) = 0.20_rkx*(3.0_rkx*gama(04)+2.0_rkx*gama(09))  ! lc ; scape
! gama(13) = 0.50_rkx*(gama(04)+gama(09))          ! lc ; sequilib
! gama(13) = 0.25_rkx*(3.0_rkx*gama(04)+gama(07))      ! lc ; aim
!
! *** convert log (gama) coefficients to gama **************************
!
    do i = 1 , 13
      gama(i) = max(-5.0_rkx,min(gama(i),5.0_rkx))       ! f77 library routine
      gama(i) = 10.0_rkx**gama(i)
!     gama(i) = exp(ln10*gama(i))
!     gama(i) = ex10(real(gama(i),rkx), 5.0_rkx)    ! cutoff set to [-5 ,5]
!     gama(i) = gamin(i)*(1.0_rkx-urf) + urf*gama(i)  ! under-relax gama's
    end do
!
! *** setup activity calculation flags *********************************
!
! outr calculation loop ; only if frst =.true.
!
    if ( frst ) then
      errou = zero                       ! convergence criterion
      do i = 1 , 13
        errou = max(errou,abs((gamou(i)-gama(i))/gamou(i)))
      end do
      calaou = errou>=epsact             ! setup flags
      frst = .false.
    end if
!
! innr calculation loop ; always
!
    errin = zero                         ! convergence criterion
    do i = 1 , 13
      errin = max(errin,abs((gamin(i)-gama(i))/gamin(i)))
    end do
    calain = errin>=epsact
!
    iclact = iclact + 1                  ! increment activity call counter
    contains
      real(rkx) function g(i,j)
        implicit none
        integer(ik4) , intent(in) :: i , j
        g = (f1(i)/z(i)+f2(j)/z(j+3))/(z(i)+z(j+3)) - h
      end function g
  end subroutine calcact3
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcact2
! *** calculates multi-component activity coefficients from bromleys
!     method for an ammonium-sulfate-nitrate aerosol system.
!     the binary activity coefficients are calculated by
!     kusik-meisner relation (subroutine kmtab or subroutine kmful2).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
!
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcact2
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: errin , errou
    integer(ik4) :: i , j
    real(rkx) , dimension(6,4) :: g0
    real(rkx) , dimension(3) :: f1
    real(rkx) , dimension(4) :: f2
    real(rkx) :: zpl , zmi , agama , sion , h , ch
    real(rkx) :: mpl , xij , yji
!
! *** save activities in old array *************************************
!
    if ( frst ) then            ! outer loop
      do i = 7 , 10
        gamou(i) = gama(i)
      end do
      gamou(4) = gama(4)
      gamou(5) = gama(5)
      gamou(13) = gama(13)
    end if
!
    do i = 7 , 10                  ! inner loop
      gamin(i) = gama(i)
    end do
    gamin(4) = gama(4)
    gamin(5) = gama(5)
    gamin(13) = gama(13)
!
! *** calculate ionic activity of solution *****************************
!
    ionic = 0.0
    molal(2) = zero
    molal(4) = zero
    do i = 1 , 7
      ionic = ionic + molal(i)*z(i)*z(i)
    end do
    ionic = max(min(0.5_rkx*ionic/water,100._rkx),tiny1)
!
! *** calculate binary activity coefficients ***************************
!
! g01 ,1) = g11;g0(1 ,2)=g07;g0(1 ,3)=g08;g0(1 ,4)=g10;g0(2
! ,1)=g01;g0(2 ,2)=g02 & g02 ,3) = g12;g0(2 ,4)=g03;g0(3 ,1)=g06;g0(3
! ,2)=g04;g0(3 ,3)=g09;g0(3 ,4)=g05
    if ( iacalc==0 ) then                ! k.m.; full
      call kmful2(ionic,real(temp,rkx),g0(3,2),g0(3,4),g0(1,2),g0(1,3),g0(3,3)&
                  ,g0(1,4))
    else                                 ! k.m.; tabulated
      call kmtab(ionic,real(temp,rkx),g0(2,1),g0(2,2),g0(2,4),g0(3,2),g0(3,4),&
                 g0(3,1),g0(1,2),g0(1,3),g0(3,3),g0(1,4),g0(1,1),g0(2,3), &
                 g0(4,4),g0(4,1),g0(5,2),g0(5,3),g0(5,4),g0(5,1),g0(6,2), &
                 g0(6,4),g0(6,1))
    end if
!
! *** calculate multicomponent activity coefficients *******************
!
    agama = 0.511*(298.0/temp)**1.5      ! debye huckel const. at t
    sion = sqrt(ionic)
    h = agama*sion/(1+sion)
!
    do i = 1 , 3
      f1(i) = 0.0
      f2(i) = 0.0
    end do
    f2(4) = 0.0
!
    do i = 1 , 3 , 2
      zpl = z(i)
      mpl = molal(i)/water
      do j = 2 , 4
        zmi = z(j+3)
        ch = 0.25_rkx*(zpl+zmi)*(zpl+zmi)/ionic
        xij = ch*mpl
        yji = ch*molal(j+3)/water
        f1(i) = f1(i) + real(yji*(g0(i,j)+zpl*zmi*h),rkx)
        f2(j) = f2(j) + real(xij*(g0(i,j)+zpl*zmi*h),rkx)
      end do
    end do
!
! *** log10 of activity coefficients ***********************************
!
! gama(01) = g(2 ,1)*zz(01)                     ! nacl
! gama(02) = g(2 ,2)*zz(02)                     ! na2so4
! gama(03) = g(2 ,4)*zz(03)                     ! nano3
    gama(04) = g(3,2)*zz(04)                        ! (nh4)2so4
    gama(05) = g(3,4)*zz(05)                        ! nh4no3
! gama(06) = g(3 ,1)*zz(06)                     ! nh4cl
    gama(07) = g(1,2)*zz(07)                        ! 2h-so4
    gama(08) = g(1,3)*zz(08)                        ! h-hso4
    gama(09) = g(3,3)*zz(09)                        ! nh4hso4
    gama(10) = g(1,4)*zz(10)                        ! hno3
! gama(11) = g(1 ,1)*zz(11)                     ! hcl
! gama(12) = g(2 ,3)*zz(12)                     ! nahso4
    gama(13) = 0.20*(3.0*gama(04)+2.0*gama(09))    ! lc ; scape
! gama(13) = 0.50*(gama(04)+gama(09))          ! lc ; sequilib
! gama(13) = 0.25_rkx*(3.0*gama(04)+gama(07))      ! lc ; aim
!
! *** convert log (gama) coefficients to gama **************************
!
    do i = 7 , 10
      gama(i) = max(-5.0_rkx,min(gama(i),5.0_rkx))       ! f77 library routine
      gama(i) = 10.0**gama(i)
!   gama(i) = exp(ln10*gama(i))
!   gama(i) = ex10(real(gama(i),rkx), 5.0)    ! cutoff set to [-5 ,5]
!   gama(i) = gamin(i)*(1.0-urf) + urf*gama(i)  ! under-relax gama's
    end do
!
    gama(4) = max(-5.0_rkx,min(gama(4),5.0_rkx))      ! f77 library routine
    gama(4) = 10.0**gama(4)
! gama(i) = exp(ln10*gama(i))
! gama(i) = ex10(real(gama(i),rkx), 5.0)    ! cutoff set to [-5 ,5]
! gama(4) = gamin(4)*(1.0-urf) + urf*gama(4)  ! under-relax gama's
!
    gama(5) = max(-5.0_rkx,min(gama(5),5.0_rkx))      ! f77 library routine
    gama(5) = 10.0**gama(5)
! gama(i) = exp(ln10*gama(i))
! gama(i) = ex10(real(gama(i),rkx), 5.0)    ! cutoff set to [-5 ,5]
! gama(5) = gamin(5)*(1.0-urf) + urf*gama(i)  ! under-relax gama's
!
    gama(13) = max(-5.0_rkx,min(gama(13),5.0_rkx))      ! f77 library routine
    gama(13) = 10.0**gama(13)
! gama(i) = exp(ln10*gama(i))
! gama(i) = ex10(real(gama(i),rkx), 5.0)    ! cutoff set to [-5 ,5]
! gama(13) = gamin(13)*(1.0-urf) + urf*gama(13)  ! under-relax gama's
!
! *** setup activity calculation flags *********************************
!
! outr calculation loop ; only if frst =.true.
!
    if ( frst ) then
      errou = zero                       ! convergence criterion
      do i = 7 , 10
        errou = max(errou,abs((gamou(i)-gama(i))/gamou(i)))
      end do
      errou = max(errou,abs((gamou(4)-gama(4))/gamou(4)))
      errou = max(errou,abs((gamou(5)-gama(5))/gamou(5)))
      errou = max(errou,abs((gamou(13)-gama(13))/gamou(13)))
!
      calaou = errou>=epsact             ! setup flags
      frst = .false.
    end if
!
! innr calculation loop ; always
!
    errin = zero                         ! convergence criterion
    do i = 7 , 10
      errin = max(errin,abs((gamin(i)-gama(i))/gamin(i)))
    end do
    errin = max(errin,abs((gamin(4)-gama(4))/gamin(4)))
    errin = max(errin,abs((gamin(5)-gama(5))/gamin(5)))
    errin = max(errin,abs((gamin(13)-gama(13))/gamin(13)))
    calain = errin>=epsact
!
    iclact = iclact + 1                  ! increment activity call counter
    contains
      real(rkx) function g(i,j)
        implicit none
        integer(ik4) , intent(in) :: i , j
        g = (f1(i)/z(i)+f2(j)/z(j+3))/(z(i)+z(j+3)) - h
      end function g
  end subroutine calcact2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcact1
! *** calculates multi-component activity coefficients from bromleys
!     method for an ammonium-sulfate aerosol system.
!     the binary activity coefficients are calculated by
!     kusik-meisner relation (subroutine kmtab or subroutine kmful1).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
!
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcact1
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: errin , errou
    integer(ik4) :: i , j
    real(rkx) , dimension(6,4) :: g0
    real(rkx) , dimension(3) :: f1
    real(rkx) , dimension(4) :: f2
    real(rkx) :: zpl , zmi , agama , sion , h , ch
    real(rkx) :: mpl , xij , yji
!
! *** save activities in old array *************************************
!
    if ( frst ) then            ! outer loop
      do i = 7 , 9
        gamou(i) = gama(i)
      end do
      gamou(4) = gama(4)
!   gamou(5) = gama(5)
      gamou(13) = gama(13)
    end if
!
    do i = 7 , 9                  ! inner loop
      gamin(i) = gama(i)
    end do
    gamin(4) = gama(4)
! gamin(5) = gama(5)
    gamin(13) = gama(13)
!
! *** calculate ionic activity of solution *****************************
!
    ionic = 0.0
    molal(2) = zero
    molal(4) = zero
    molal(7) = zero
    do i = 1 , 7
      ionic = ionic + molal(i)*z(i)*z(i)
    end do
    ionic = max(min(0.5_rkx*ionic/water,100._rkx),tiny1)
!
! *** calculate binary activity coefficients ***************************
!
! g01 ,1) = g11;g0(1 ,2)=g07;g0(1 ,3)=g08;g0(1 ,4)=g10;g0(2
! ,1)=g01;g0(2 ,2)=g02 & g02 ,3) = g12;g0(2 ,4)=g03;g0(3 ,1)=g06;g0(3
! ,2)=g04;g0(3 ,3)=g09;g0(3 ,4)=g05
    if ( iacalc==0 ) then                ! k.m.; full
      call kmful1(ionic,real(temp,rkx),g0(3,2),g0(1,2),g0(1,3))
    else                                 ! k.m.; tabulated
      call kmtab(ionic,real(temp,rkx),g0(2,1),g0(2,2),g0(2,4),g0(3,2),g0(3,4),&
                 g0(3,1),g0(1,2),g0(1,3),g0(3,3),g0(1,4),g0(1,1),g0(2,3), &
                 g0(4,4),g0(4,1),g0(5,2),g0(5,3),g0(5,4),g0(5,1),g0(6,2), &
                 g0(6,4),g0(6,1))
    end if
!
! *** calculate multicomponent activity coefficients *******************
!
    agama = 0.511*(298.0/temp)**1.5      ! debye huckel const. at t
    sion = sqrt(ionic)
    h = agama*sion/(1+sion)
!
    do i = 1 , 3
      f1(i) = 0.0
      f2(i) = 0.0
    end do
    f2(4) = 0.0
!
    do i = 1 , 3 , 2
      zpl = z(i)
      mpl = molal(i)/water
      do j = 2 , 3
        zmi = z(j+3)
        ch = 0.25_rkx*(zpl+zmi)*(zpl+zmi)/ionic
        xij = ch*mpl
        yji = ch*molal(j+3)/water
        f1(i) = f1(i) + real(yji*(g0(i,j)+zpl*zmi*h),rkx)
        f2(j) = f2(j) + real(xij*(g0(i,j)+zpl*zmi*h),rkx)
      end do
    end do
!
! *** log10 of activity coefficients ***********************************
!
! gama(01) = g(2 ,1)*zz(01)                     ! nacl
! gama(02) = g(2 ,2)*zz(02)                     ! na2so4
! gama(03) = g(2 ,4)*zz(03)                     ! nano3
    gama(04) = g(3,2)*zz(04)                        ! (nh4)2so4
! gama(05) = g(3 ,4)*zz(05)                     ! nh4no3
! gama(06) = g(3 ,1)*zz(06)                     ! nh4cl
    gama(07) = g(1,2)*zz(07)                        ! 2h-so4
    gama(08) = g(1,3)*zz(08)                        ! h-hso4
    gama(09) = 0.5_rkx*(gama(04)+gama(07))             ! nh4hso4 ; aim (wexler & seinfeld , 1991)
! gama(10) = g(1 ,4)*zz(10)                     ! hno3
! gama(11) = g(1 ,1)*zz(11)                     ! hcl
! gama(12) = g(2 ,3)*zz(12)                     ! nahso4
    gama(13) = 0.20*(3.0*gama(04)+2.0*gama(09))    ! lc ; scape
! gama(13) = 0.50*(gama(04)+gama(09))          ! lc ; sequilib
! gama(13) = 0.25_rkx*(3.0*gama(04)+gama(07))      ! lc ; aim
!
! *** convert log (gama) coefficients to gama **************************
!
    do i = 7 , 9
      gama(i) = max(-5.0_rkx,min(gama(i),5.0_rkx))       ! f77 library routine
      gama(i) = 10.0**gama(i)
!   gama(i) = exp(ln10*gama(i))
!   gama(i) = ex10(real(gama(i),rkx), 5.0)    ! cutoff set to [-5 ,5]
!   gama(i) = gamin(i)*(1.0-urf) + urf*gama(i)  ! under-relax gama's
    end do
!
    gama(4) = max(-5.0_rkx,min(gama(4),5.0_rkx))      ! f77 library routine
    gama(4) = 10.0**gama(4)
! gama(i) = exp(ln10*gama(i))
! gama(i) = ex10(real(gama(i),rkx), 5.0)    ! cutoff set to [-5 ,5]
! gama(4) = gamin(4)*(1.0-urf) + urf*gama(4)  ! under-relax gama's
!
! gama(5) = max(-5.0d0 , min(gama(5),5.0d0) ) ! f77 library routine
! gama(5) = 10.0**gama(5)
! gama(i) = exp(ln10*gama(i))
! gama(i) = ex10(real(gama(i),rkx), 5.0)    ! cutoff set to [-5 ,5]
! gama(5) = gamin(5)*(1.0-urf) + urf*gama(i)  ! under-relax gama's
!
    gama(13) = max(-5.0_rkx,min(gama(13),5.0_rkx))      ! f77 library routine
    gama(13) = 10.0**gama(13)
! gama(i) = exp(ln10*gama(i))
! gama(i) = ex10(real(gama(i),rkx), 5.0)    ! cutoff set to [-5 ,5]
! gama(13) = gamin(13)*(1.0-urf) + urf*gama(13)  ! under-relax gama's
!
! *** setup activity calculation flags *********************************
!
! outr calculation loop ; only if frst =.true.
!
    if ( frst ) then
      errou = zero                       ! convergence criterion
      do i = 7 , 9
        errou = max(errou,abs((gamou(i)-gama(i))/gamou(i)))
      end do
      errou = max(errou,abs((gamou(4)-gama(4))/gamou(4)))
!   errou = max(errou , abs((gamou(5)-gama(5))/gamou(5)))
      errou = max(errou,abs((gamou(13)-gama(13))/gamou(13)))
!
      calaou = errou>=epsact             ! setup flags
      frst = .false.
    end if
!
! innr calculation loop ; always
!
    errin = zero                         ! convergence criterion
    do i = 7 , 9
      errin = max(errin,abs((gamin(i)-gama(i))/gamin(i)))
    end do
    errin = max(errin,abs((gamin(4)-gama(4))/gamin(4)))
! errin = max (errin , abs((gamin(5)-gama(5))/gamin(5)))
    errin = max(errin,abs((gamin(13)-gama(13))/gamin(13)))
    calain = errin>=epsact
!
    iclact = iclact + 1                  ! increment activity call counter
    contains
      real(rkx) function g(i,j)
        integer(ik4) , intent(in) :: i , j
        g = (f1(i)/z(i)+f2(j)/z(j+3))/(z(i)+z(j+3)) - h
      end function g
  end subroutine calcact1
!
!======================================================================
!
! *** isorropia code
! *** subroutine rstgam
! *** resets activity coefficient arrays to default value of 0.1
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine rstgam
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
    do i = 1 , npair
      gama(i) = 0.1_rkx
    end do
  end subroutine rstgam
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine kmful4
! *** calculates binary activity coefficients by kusik-meissner method
!     for an ammonium-sulfate-nitrate-chloride-sodium-calcium-potassium-magnesium
!     aerosol system.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
!
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine kmful4(ionic,temp,g01,g02,g03,g04,g05,g06,g07,g08,g09,g10, &
                    g11,g12,g15,g16,g17,g18,g19,g20,g21,g22,g23)
    implicit none
    real(rkx) :: cf1 , cf2 , g01 , g02 , g03 , g04 , g05 , g06 , g07 , g08 , &
         g09 , g10 , g11 , g12 , g15 , g16 , g17 , g18 , g19 , g20
    real(rkx) :: g21 , g22 , g23 , sion , tc , ti , z01 , z02 , z03 , z04 , &
         z05 , z06 , z07 , z08 , z10 , z11 , z15 , z16 , z17 , z19
    real(rkx) :: z20 , z21 , z22 , z23
    real(rkx) :: ionic , temp
    data z01 , z02 , z03 , z04 , z05 , z06 , z07 , z08 , z10 , z11 , z15 ,&
         z16 , z17 , z19 , z20 , z21 , z22 , z23 &
         /1.0_rkx , 2.0_rkx , 1.0_rkx , 2.0_rkx , 1.0_rkx , 1.0_rkx , 2.0_rkx , 1.0_rkx , &
         1.0_rkx , 1.0_rkx , 2.0_rkx ,  2.0_rkx , 2.0_rkx , 1.0_rkx , 1.0_rkx , 4.0_rkx , &
         2.0_rkx , 2.0_rkx/
!
    sion = sqrt(ionic)
!
! *** coefficients at 25 oc
!
    call mkbi(2.230_rkx,ionic,sion,z01,g01)
    call mkbi(-0.19_rkx,ionic,sion,z02,g02)
    call mkbi(-0.39_rkx,ionic,sion,z03,g03)
    call mkbi(-0.25_rkx,ionic,sion,z04,g04)
    call mkbi(-1.15_rkx,ionic,sion,z05,g05)
    call mkbi(0.820_rkx,ionic,sion,z06,g06)
    call mkbi(-.100_rkx,ionic,sion,z07,g07)
    call mkbi(8.000_rkx,ionic,sion,z08,g08)
    call mkbi(2.600_rkx,ionic,sion,z10,g10)
    call mkbi(6.000_rkx,ionic,sion,z11,g11)
    call mkbi(0.930_rkx,ionic,sion,z15,g15)
    call mkbi(2.400_rkx,ionic,sion,z16,g16)
    call mkbi(-0.25_rkx,ionic,sion,z17,g17)
    call mkbi(-2.33_rkx,ionic,sion,z19,g19)
    call mkbi(0.920_rkx,ionic,sion,z20,g20)
    call mkbi(0.150_rkx,ionic,sion,z21,g21)
    call mkbi(2.320_rkx,ionic,sion,z22,g22)
    call mkbi(2.900_rkx,ionic,sion,z23,g23)
!
! *** correct for t other than 298 k
!
    ti = temp - 273.0_rkx
    tc = ti - 25.0_rkx
    if ( abs(tc)>1.0_rkx ) then
      cf1 = 1.125_rkx - 0.005_rkx*ti
      cf2 = (0.125_rkx-0.005_rkx*ti)                                          &
            *(0.039_rkx*ionic**0.92_rkx-0.41_rkx*sion/(1.0_rkx+sion))
      g01 = cf1*g01 - cf2*z01
      g02 = cf1*g02 - cf2*z02
      g03 = cf1*g03 - cf2*z03
      g04 = cf1*g04 - cf2*z04
      g05 = cf1*g05 - cf2*z05
      g06 = cf1*g06 - cf2*z06
      g07 = cf1*g07 - cf2*z07
      g08 = cf1*g08 - cf2*z08
      g10 = cf1*g10 - cf2*z10
      g11 = cf1*g11 - cf2*z11
      g15 = cf1*g15 - cf2*z15
      g16 = cf1*g16 - cf2*z16
      g17 = cf1*g17 - cf2*z17
      g19 = cf1*g19 - cf2*z19
      g20 = cf1*g20 - cf2*z20
      g21 = cf1*g21 - cf2*z21
      g22 = cf1*g22 - cf2*z22
      g23 = cf1*g23 - cf2*z23
    end if
    g09 = g06 + g08 - g11
    g12 = g01 + g08 - g11
    g18 = g08 + g20 - g11
  end subroutine kmful4
!
!======================================================================
!
! *** isorropia code
! *** subroutine kmful3
! *** calculates binary activity coefficients by kusik-meissner method
!     for an ammonium-sulfate-nitrate-chloride-sodium aerosol system.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
!
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine kmful3(ionic,temp,g01,g02,g03,g04,g05,g06,g07,g08,g09,g10, &
                      g11,g12)
    implicit none
    real(rkx) :: cf1 , cf2 , g01 , g02 , g03 , g04 , g05 , g06 , g07 , g08 , &
         g09 , g10 , g11 , g12 , sion , tc , ti , z01 , z02 , z03
    real(rkx) :: z04 , z05 , z06 , z07 , z08 , z10 , z11
    real(rkx) :: ionic , temp
    data z01 , z02 , z03 , z04 , z05 , z06 , z07 , z08 , z10 , z11 &
      /1.0_rkx , 2.0_rkx , 1.0_rkx , 2.0_rkx , 1.0_rkx , 1.0_rkx , 2.0_rkx , 1.0_rkx , &
       1.0_rkx , 1.0_rkx/
!
    sion = sqrt(ionic)
!
! *** coefficients at 25 oc
!
    call mkbi(2.230_rkx,ionic,sion,z01,g01)
    call mkbi(-0.19_rkx,ionic,sion,z02,g02)
    call mkbi(-0.39_rkx,ionic,sion,z03,g03)
    call mkbi(-0.25_rkx,ionic,sion,z04,g04)
    call mkbi(-1.15_rkx,ionic,sion,z05,g05)
    call mkbi(0.820_rkx,ionic,sion,z06,g06)
    call mkbi(-.100_rkx,ionic,sion,z07,g07)
    call mkbi(8.000_rkx,ionic,sion,z08,g08)
    call mkbi(2.600_rkx,ionic,sion,z10,g10)
    call mkbi(6.000_rkx,ionic,sion,z11,g11)
!
! *** correct for t other than 298 k
!
    ti = temp - 273.0_rkx
    tc = ti - 25.0_rkx
    if ( abs(tc)>1.0_rkx ) then
      cf1 = 1.125_rkx - 0.005_rkx*ti
      cf2 = (0.125_rkx-0.005_rkx*ti)                                          &
            *(0.039_rkx*ionic**0.92_rkx-0.41_rkx*sion/(1.0_rkx+sion))
      g01 = cf1*g01 - cf2*z01
      g02 = cf1*g02 - cf2*z02
      g03 = cf1*g03 - cf2*z03
      g04 = cf1*g04 - cf2*z04
      g05 = cf1*g05 - cf2*z05
      g06 = cf1*g06 - cf2*z06
      g07 = cf1*g07 - cf2*z07
      g08 = cf1*g08 - cf2*z08
      g10 = cf1*g10 - cf2*z10
      g11 = cf1*g11 - cf2*z11
    end if
    g09 = g06 + g08 - g11
    g12 = g01 + g08 - g11
  end subroutine kmful3
!
!======================================================================
!
! *** isorropia code
! *** subroutine kmful2
! *** calculates binary activity coefficients by kusik-meissner method
!     for an ammonium-sulfate-nitrate aerosol system.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
!
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine kmful2(ionic,temp,g04,g05,g07,g08,g09,g10)
    implicit none
    real(rkx) :: cf1 , cf2 , g04 , g05 , g07 , g08 , g09 , g10 , sion , &
         tc , ti , z01 , z02 , z03 , z04 , z05 , z06 , z07 , z08 , z10
    real(rkx) :: z11
    real(rkx) :: ionic , temp
    data z01 , z02 , z03 , z04 , z05 , z06 , z07 , z08 , z10 , z11 &
      /1.0_rkx , 2.0_rkx , 1.0_rkx , 2.0_rkx , 1.0_rkx , 1.0_rkx , 2.0_rkx , 1.0_rkx , &
       1.0_rkx , 1.0_rkx/
!
    sion = sqrt(ionic)
!
! *** coefficients at 25 oc
!
! call mkbi(2.230 , ionic , sion , z01 , g01)
! call mkbi(-0.19 , ionic , sion , z02 , g02)
! call mkbi(-0.39 , ionic , sion , z03 , g03)
    call mkbi(-0.25_rkx,ionic,sion,z04,g04)
    call mkbi(-1.15_rkx,ionic,sion,z05,g05)
! call mkbi(0.820 , ionic , sion , z06 , g06)
    call mkbi(-.100_rkx,ionic,sion,z07,g07)
    call mkbi(8.000_rkx,ionic,sion,z08,g08)
    call mkbi(2.600_rkx,ionic,sion,z10,g10)
! call mkbi(6.000 , ionic , sion , z11 , g11)
!
! *** correct for t other than 298 k
!
    ti = temp - 273.0_rkx
    tc = ti - 25.0_rkx
    if ( abs(tc)>1.0_rkx ) then
      cf1 = 1.125_rkx - 0.005_rkx*ti
      cf2 = (0.125_rkx-0.005_rkx*ti)                                          &
            *(0.039_rkx*ionic**0.92_rkx-0.41_rkx*sion/(1.0_rkx+sion))
!   g01 = cf1*g01 - cf2*z01
!   g02 = cf1*g02 - cf2*z02
!   g03 = cf1*g03 - cf2*z03
      g04 = cf1*g04 - cf2*z04
      g05 = cf1*g05 - cf2*z05
!   g06 = cf1*g06 - cf2*z06
      g07 = cf1*g07 - cf2*z07
      g08 = cf1*g08 - cf2*z08
      g10 = cf1*g10 - cf2*z10
!   g11 = cf1*g11 - cf2*z11
    end if
    g09 = g05 + g08 - g10
! g12 = g01 + g08 - g11
  end subroutine kmful2
!
!======================================================================
!
! *** isorropia code
! *** subroutine kmful1
! *** calculates binary activity coefficients by kusik-meissner method
!     for an ammonium-sulfate aerosol system.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
!
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine kmful1(ionic,temp,g04,g07,g08)
    implicit none
    real(rkx) :: cf1 , cf2 , g04 , g07 , g08 , sion , tc , ti , &
         z01 , z02 , z03 , z04 , z05 , z06 , z07 , z08 , z10 , z11
    real(rkx) :: ionic , temp
    data z01 , z02 , z03 , z04 , z05 , z06 , z07 , z08 , z10 , z11 &
      /1.0_rkx , 2.0_rkx , 1.0_rkx , 2.0_rkx , 1.0_rkx , 1.0_rkx , 2.0_rkx , 1.0_rkx , &
       1.0_rkx , 1.0_rkx/
!
    sion = sqrt(ionic)
!
! *** coefficients at 25 oc
!
! call mkbi(2.230_rkx , ionic , sion , z01 , g01)
! call mkbi(-0.19_rkx , ionic , sion , z02 , g02)
! call mkbi(-0.39_rkx , ionic , sion , z03 , g03)
    call mkbi(-0.25_rkx,ionic,sion,z04,g04)
! call mkbi(-1.15_rkx , ionic , sion , z05 , g05)
! call mkbi(0.820_rkx , ionic , sion , z06 , g06)
    call mkbi(-.100_rkx,ionic,sion,z07,g07)
    call mkbi(8.000_rkx,ionic,sion,z08,g08)
! call mkbi(2.600_rkx , ionic , sion , z10 , g10)
! call mkbi(6.000_rkx , ionic , sion , z11 , g11)
!
! *** correct for t other than 298 k
!
    ti = temp - 273.0_rkx
    tc = ti - 25.0_rkx
    if ( abs(tc)>1.0_rkx ) then
      cf1 = 1.125_rkx - 0.005_rkx*ti
      cf2 = (0.125_rkx-0.005_rkx*ti)                                          &
            *(0.039_rkx*ionic**0.92_rkx-0.41_rkx*sion/(1.0_rkx+sion))
!   g01 = cf1*g01 - cf2*z01
!   g02 = cf1*g02 - cf2*z02
!   g03 = cf1*g03 - cf2*z03
      g04 = cf1*g04 - cf2*z04
!   g05 = cf1*g05 - cf2*z05
!   g06 = cf1*g06 - cf2*z06
      g07 = cf1*g07 - cf2*z07
      g08 = cf1*g08 - cf2*z08
!   g10 = cf1*g10 - cf2*z10
!   g11 = cf1*g11 - cf2*z11
    end if
! g09 = g05 + g08 - g10   ! calculated in calcact1
! g12 = g01 + g08 - g11
  end subroutine kmful1
!
!======================================================================
!
! *** isorropia code
! *** subroutine mkbi
! *** calculates binary activity coefficients by kusik-meissner method.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine mkbi(q,ionic,sion,zip,bi)
    implicit none
    real(rkx) :: b , bi , c , q , sion , xx , zip
    real(rkx) :: ionic
    b = 0.75_rkx - .065_rkx*q
    c = 1.0_rkx
    if ( ionic<6.0_rkx ) c = 1.0_rkx +                                        &
                           .055_rkx*q*exp(-0.023_rkx*ionic*ionic*ionic)
    xx = -0.5107_rkx*sion/(1.0_rkx+c*sion)
    bi = (1.0_rkx+b*(1.0_rkx+0.1_rkx*ionic)**q-b)
    bi = zip*log10(bi) + zip*xx
  end subroutine mkbi
!======================================================================
!
! *** isorropia code
! *** subroutine kmtab
! *** calculates binary activity coefficients by kusik-meissner method.
!     the computations have been performed and the results are stored in
!     lookup tables. the ionic activity 'ionic' is input , and the array
!     'binarr' is returned with the binary coefficients.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine kmtab(ino,temp,g01,g02,g03,g04,g05,g06,g07,g08,g09,g10,g11, &
                     g12,g15,g16,g17,g18,g19,g20,g21,g22,g23)
    implicit none
    real(rkx) :: g01 , g02 , g03 , g04 , g05 , g06 , g07 , g08 , g09 , &
                 g10 , g11 , g12 , g13 , g14 , g15 , g16 , g17 , g18 , &
                 g19 , g20 , g21 , g22 , g23
    integer(ik4) :: ind
    real(rkx) :: ino , temp
    real(rkx) , dimension(23) :: binarray
!
! *** find temperature range
!
    ind = nint((temp-198.0)/25.0) + 1
    ind = min(max(ind,1),6)
!
! *** call appropriate routine
!
    if ( ind==1 ) then
      call km198(ino,binarray)
    else if ( ind==2 ) then
      call km223(ino,binarray)
    else if ( ind==3 ) then
      call km248(ino,binarray)
    else if ( ind==4 ) then
      call km273(ino,binarray)
    else if ( ind==5 ) then
      call km298(ino,binarray)
    else
      call km323(ino,binarray)
    end if
!
    g01 = binarray(01)
    g02 = binarray(02)
    g03 = binarray(03)
    g04 = binarray(04)
    g05 = binarray(05)
    g06 = binarray(06)
    g07 = binarray(07)
    g08 = binarray(08)
    g09 = binarray(09)
    g10 = binarray(10)
    g11 = binarray(11)
    g12 = binarray(12)
    g13 = binarray(13)
    g14 = binarray(14)
    g15 = binarray(15)
    g16 = binarray(16)
    g17 = binarray(17)
    g18 = binarray(18)
    g19 = binarray(19)
    g20 = binarray(20)
    g21 = binarray(21)
    g22 = binarray(22)
    g23 = binarray(23)
  end subroutine kmtab
!
!      integer(ik4) :: function ibacpos(in)
!
!     compute the index in the binary activity coefficient array
!     based on the input ionic strength.
!
!     chris nolte , 6/16/05
!
!      implicit none
!      real(rkx) in
!      if (in  <=  0.300000e+02) then
!         ibacpos = min(nint( 0.200000e+02*in) + 1 , 600)
!      else
!         ibacpos =   600+nint( 0.200000e+01*in- 0.600000e+02)
!      end if
!      ibacpos = min(ibacpos , 741)
!      return
!      end
!======================================================================
!
! *** isorropia code
! *** subroutine km198
! *** calculates binary activity coefficients by kusik-meissner method.
!     the computations have been performed and the results are stored in
!     lookup tables. the ionic activity 'in' is input , and the array
!     'binarr' is returned with the binary coefficients.
!
!     temperature is 198k
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine km198(ionic,binarr)
    implicit none
    integer(ik4) :: ipos
    real(rkx) , dimension(23) :: binarr
    real(rkx) :: ionic
    real(rkx) , dimension(561) :: bnc01m , bnc02m , bnc03m , bnc04m ,     &
                                  bnc05m , bnc06m , bnc07m , bnc08m ,     &
                                  bnc09m , bnc10m , bnc11m , bnc12m ,     &
                                  bnc13m , bnc14m , bnc15m , bnc16m ,     &
                                  bnc17m , bnc18m , bnc19m , bnc20m ,     &
                                  bnc21m , bnc22m , bnc23m
!
! *** nacl
!
    bnc01m(1:561) = (/-0.050,-0.103,-0.127,-0.142,-0.154,-0.162,-0.169,   &
                    -0.174,-0.178,-0.181,-0.184,-0.186,-0.188,-0.189,     &
                    -0.190,-0.191,-0.191,-0.192,-0.192,-0.191,-0.191,     &
                    -0.191,-0.190,-0.189,-0.188,-0.188,-0.187,-0.185,     &
                    -0.184,-0.183,-0.182,-0.181,-0.179,-0.178,-0.176,     &
                    -0.175,-0.173,-0.172,-0.170,-0.169,-0.167,-0.166,     &
                    -0.164,-0.162,-0.161,-0.159,-0.157,-0.156,-0.154,     &
                    -0.152,-0.151,-0.149,-0.147,-0.146,-0.144,-0.142,     &
                    -0.140,-0.139,-0.137,-0.135,-0.134,-0.132,-0.130,     &
                    -0.128,-0.127,-0.125,-0.123,-0.121,-0.120,-0.118,     &
                    -0.116,-0.114,-0.112,-0.111,-0.109,-0.107,-0.105,     &
                    -0.103,-0.101,-0.099,-0.098,-0.096,-0.094,-0.092,     &
                    -0.090,-0.088,-0.086,-0.084,-0.082,-0.080,-0.078,     &
                    -0.075,-0.073,-0.071,-0.069,-0.067,-0.065,-0.063,     &
                    -0.060,-0.058,-0.056,-0.054,-0.051,-0.049,-0.047,     &
                    -0.045,-0.042,-0.040,-0.038,-0.035,-0.033,-0.031,     &
                    -0.028,-0.026,-0.024,-0.021,-0.019,-0.016,-0.014,     &
                    -0.012,-0.009,-0.007,-0.004,-0.002,0.000,0.003,0.005, &
                    0.008,0.010,0.012,0.015,0.017,0.020,0.022,0.024,0.027,&
                    0.029,0.032,0.034,0.036,0.039,0.041,0.044,0.046,0.048,&
                    0.051,0.053,0.055,0.058,0.060,0.063,0.065,0.067,0.070,&
                    0.072,0.074,0.077,0.079,0.081,0.084,0.086,0.088,0.091,&
                    0.093,0.095,0.098,0.100,0.102,0.105,0.107,0.109,0.112,&
                    0.114,0.116,0.118,0.121,0.123,0.125,0.127,0.130,0.132,&
                    0.134,0.137,0.139,0.141,0.143,0.146,0.148,0.150,0.152,&
                    0.154,0.157,0.159,0.161,0.163,0.166,0.168,0.170,0.172,&
                    0.174,0.176,0.179,0.181,0.183,0.185,0.187,0.190,0.192,&
                    0.194,0.196,0.198,0.200,0.202,0.205,0.207,0.209,0.211,&
                    0.213,0.215,0.217,0.219,0.222,0.224,0.226,0.228,0.230,&
                    0.232,0.234,0.236,0.238,0.240,0.242,0.244,0.246,0.249,&
                    0.251,0.253,0.255,0.257,0.259,0.261,0.263,0.265,0.267,&
                    0.269,0.271,0.273,0.275,0.277,0.279,0.281,0.283,0.285,&
                    0.287,0.289,0.291,0.293,0.295,0.297,0.299,0.301,0.303,&
                    0.304,0.306,0.308,0.310,0.312,0.314,0.316,0.318,0.320,&
                    0.322,0.324,0.326,0.328,0.329,0.331,0.333,0.335,0.337,&
                    0.339,0.341,0.343,0.344,0.346,0.348,0.350,0.352,0.354,&
                    0.356,0.357,0.359,0.361,0.363,0.365,0.367,0.368,0.370,&
                    0.372,0.374,0.376,0.377,0.379,0.381,0.383,0.385,0.386,&
                    0.388,0.390,0.392,0.393,0.395,0.397,0.399,0.401,0.402,&
                    0.404,0.406,0.408,0.409,0.411,0.413,0.414,0.416,0.418,&
                    0.420,0.421,0.423,0.425,0.426,0.428,0.430,0.432,0.433,&
                    0.435,0.437,0.438,0.440,0.442,0.443,0.445,0.447,0.448,&
                    0.450,0.452,0.453,0.455,0.457,0.458,0.460,0.461,0.463,&
                    0.465,0.466,0.468,0.470,0.471,0.473,0.474,0.476,0.478,&
                    0.479,0.481,0.482,0.484,0.486,0.487,0.489,0.490,0.492,&
                    0.493,0.495,0.497,0.498,0.500,0.501,0.503,0.504,0.506,&
                    0.508,0.509,0.511,0.512,0.514,0.515,0.517,0.518,0.520,&
                    0.521,0.523,0.524,0.526,0.527,0.529,0.530,0.532,0.533,&
                    0.535,0.536,0.538,0.554,0.568,0.582,0.596,0.610,0.623,&
                    0.636,0.649,0.661,0.674,0.686,0.698,0.709,0.721,0.732,&
                    0.743,0.754,0.765,0.775,0.786,0.796,0.806,0.815,0.825,&
                    0.834,0.844,0.853,0.862,0.870,0.879,0.887,0.896,0.904,&
                    0.912,0.920,0.928,0.935,0.943,0.950,0.957,0.964,0.971,&
                    0.978,0.985,0.992,0.998,1.005,1.011,1.017,1.023,1.029,&
                    1.035,1.041,1.047,1.052,1.058,1.063,1.068,1.074,1.079,&
                    1.084,1.089,1.094,1.099,1.103,1.108,1.112,1.117,1.121,&
                    1.126,1.130,1.134,1.138,1.142,1.146,1.150,1.154,1.158,&
                    1.161,1.165,1.169,1.172,1.175,1.179,1.182,1.185,1.189,&
                    1.192,1.195,1.198,1.201,1.204,1.206,1.209,1.212,1.215,&
                    1.217,1.220,1.222,1.225,1.227,1.230,1.232,1.234,1.236,&
                    1.239,1.241,1.243,1.245,1.247,1.249,1.251,1.253,1.254,&
                    1.256,1.258,1.260,1.261,1.263,1.264,1.266,1.267,1.269,&
                    1.270,1.272,1.273,1.274,1.276,1.277,1.278,1.279,1.280,&
                    1.281,1.283,1.284,1.285,1.286,1.286,1.287,1.288,1.289,&
                    1.290,1.291,1.291,1.292,1.293,1.293,1.294,1.295,1.295,&
                    1.296,1.296,1.297,1.297,1.297,1.298,1.298,1.298,1.299,&
                    1.299,1.299/)
!
! *** na2so4
!
    bnc02m(1:561) = (/-0.103,-0.225,-0.288,-0.332,-0.367,-0.397,-0.422,   &
                    -0.445,-0.465,-0.484,-0.501,-0.516,-0.531,-0.545,     &
                    -0.558,-0.570,-0.582,-0.593,-0.604,-0.614,-0.624,     &
                    -0.633,-0.643,-0.651,-0.660,-0.668,-0.676,-0.684,     &
                    -0.692,-0.699,-0.707,-0.714,-0.721,-0.727,-0.734,     &
                    -0.741,-0.747,-0.753,-0.759,-0.765,-0.771,-0.777,     &
                    -0.783,-0.788,-0.794,-0.799,-0.804,-0.810,-0.815,     &
                    -0.820,-0.825,-0.830,-0.835,-0.840,-0.845,-0.849,     &
                    -0.854,-0.859,-0.863,-0.868,-0.872,-0.877,-0.881,     &
                    -0.885,-0.889,-0.894,-0.898,-0.902,-0.906,-0.910,     &
                    -0.914,-0.918,-0.922,-0.926,-0.930,-0.934,-0.938,     &
                    -0.942,-0.946,-0.949,-0.953,-0.957,-0.961,-0.964,     &
                    -0.968,-0.971,-0.975,-0.979,-0.982,-0.986,-0.989,     &
                    -0.993,-0.996,-1.000,-1.003,-1.007,-1.010,-1.014,     &
                    -1.017,-1.020,-1.024,-1.027,-1.030,-1.034,-1.037,     &
                    -1.040,-1.044,-1.047,-1.050,-1.053,-1.057,-1.060,     &
                    -1.063,-1.066,-1.069,-1.072,-1.076,-1.079,-1.082,     &
                    -1.085,-1.088,-1.091,-1.094,-1.097,-1.100,-1.103,     &
                    -1.106,-1.109,-1.112,-1.115,-1.118,-1.121,-1.124,     &
                    -1.127,-1.130,-1.133,-1.136,-1.139,-1.142,-1.145,     &
                    -1.147,-1.150,-1.153,-1.156,-1.159,-1.162,-1.165,     &
                    -1.167,-1.170,-1.173,-1.176,-1.179,-1.181,-1.184,     &
                    -1.187,-1.190,-1.192,-1.195,-1.198,-1.200,-1.203,     &
                    -1.206,-1.209,-1.211,-1.214,-1.217,-1.219,-1.222,     &
                    -1.225,-1.227,-1.230,-1.232,-1.235,-1.238,-1.240,     &
                    -1.243,-1.246,-1.248,-1.251,-1.253,-1.256,-1.258,     &
                    -1.261,-1.264,-1.266,-1.269,-1.271,-1.274,-1.276,     &
                    -1.279,-1.281,-1.284,-1.286,-1.289,-1.291,-1.294,     &
                    -1.296,-1.299,-1.301,-1.304,-1.306,-1.309,-1.311,     &
                    -1.313,-1.316,-1.318,-1.321,-1.323,-1.326,-1.328,     &
                    -1.330,-1.333,-1.335,-1.338,-1.340,-1.342,-1.345,     &
                    -1.347,-1.350,-1.352,-1.354,-1.357,-1.359,-1.361,     &
                    -1.364,-1.366,-1.368,-1.371,-1.373,-1.375,-1.378,     &
                    -1.380,-1.382,-1.385,-1.387,-1.389,-1.392,-1.394,     &
                    -1.396,-1.399,-1.401,-1.403,-1.405,-1.408,-1.410,     &
                    -1.412,-1.415,-1.417,-1.419,-1.421,-1.424,-1.426,     &
                    -1.428,-1.430,-1.433,-1.435,-1.437,-1.439,-1.442,     &
                    -1.444,-1.446,-1.448,-1.450,-1.453,-1.455,-1.457,     &
                    -1.459,-1.461,-1.464,-1.466,-1.468,-1.470,-1.472,     &
                    -1.475,-1.477,-1.479,-1.481,-1.483,-1.485,-1.488,     &
                    -1.490,-1.492,-1.494,-1.496,-1.498,-1.501,-1.503,     &
                    -1.505,-1.507,-1.509,-1.511,-1.513,-1.516,-1.518,     &
                    -1.520,-1.522,-1.524,-1.526,-1.528,-1.530,-1.533,     &
                    -1.535,-1.537,-1.539,-1.541,-1.543,-1.545,-1.547,     &
                    -1.549,-1.551,-1.554,-1.556,-1.558,-1.560,-1.562,     &
                    -1.564,-1.566,-1.568,-1.570,-1.572,-1.574,-1.576,     &
                    -1.578,-1.580,-1.583,-1.585,-1.587,-1.589,-1.591,     &
                    -1.593,-1.595,-1.597,-1.599,-1.601,-1.603,-1.605,     &
                    -1.607,-1.609,-1.611,-1.613,-1.615,-1.617,-1.619,     &
                    -1.621,-1.623,-1.625,-1.627,-1.629,-1.631,-1.633,     &
                    -1.635,-1.637,-1.639,-1.641,-1.643,-1.645,-1.647,     &
                    -1.649,-1.651,-1.653,-1.655,-1.657,-1.659,-1.661,     &
                    -1.663,-1.665,-1.667,-1.669,-1.671,-1.673,-1.675,     &
                    -1.677,-1.679,-1.681,-1.683,-1.685,-1.687,-1.689,     &
                    -1.691,-1.693,-1.695,-1.696,-1.698,-1.700,-1.702,     &
                    -1.704,-1.706,-1.708,-1.710,-1.712,-1.714,-1.716,     &
                    -1.718,-1.720,-1.722,-1.724,-1.726,-1.727,-1.729,     &
                    -1.731,-1.752,-1.771,-1.790,-1.808,-1.827,-1.845,     &
                    -1.864,-1.882,-1.900,-1.918,-1.936,-1.954,-1.972,     &
                    -1.989,-2.007,-2.024,-2.042,-2.059,-2.076,-2.093,     &
                    -2.110,-2.127,-2.144,-2.161,-2.178,-2.194,-2.211,     &
                    -2.228,-2.244,-2.261,-2.277,-2.293,-2.310,-2.326,     &
                    -2.342,-2.358,-2.374,-2.390,-2.406,-2.422,-2.438,     &
                    -2.454,-2.470,-2.486,-2.501,-2.517,-2.533,-2.548,     &
                    -2.564,-2.579,-2.595,-2.610,-2.626,-2.641,-2.657,     &
                    -2.672,-2.687,-2.702,-2.718,-2.733,-2.748,-2.763,     &
                    -2.778,-2.793,-2.808,-2.823,-2.838,-2.853,-2.868,     &
                    -2.883,-2.898,-2.913,-2.927,-2.942,-2.957,-2.972,     &
                    -2.986,-3.001,-3.016,-3.030,-3.045,-3.059,-3.074,     &
                    -3.089,-3.103,-3.118,-3.132,-3.147,-3.161,-3.175,     &
                    -3.190,-3.204,-3.219,-3.233,-3.247,-3.261,-3.276,     &
                    -3.290,-3.304,-3.318,-3.333,-3.347,-3.361,-3.375,     &
                    -3.389,-3.403,-3.417,-3.432,-3.446,-3.460,-3.474,     &
                    -3.488,-3.502,-3.516,-3.530,-3.544,-3.558,-3.571,     &
                    -3.585,-3.599,-3.613,-3.627,-3.641,-3.655,-3.669,     &
                    -3.682,-3.696,-3.710,-3.724,-3.737,-3.751,-3.765,     &
                    -3.779,-3.792,-3.806,-3.820,-3.833,-3.847,-3.861,     &
                    -3.874,-3.888,-3.901,-3.915,-3.929,-3.942,-3.956,     &
                    -3.969,-3.983,-3.996,-4.010,-4.023,-4.037,-4.050,     &
                    -4.064,-4.077,-4.091,-4.104,-4.118,-4.131,-4.144,     &
                    -4.158/)
!
! *** nano3
!
    bnc03m(1:561) = (/-0.052,-0.114,-0.145,-0.168,-0.187,-0.202,-0.215,   &
                    -0.227,-0.238,-0.248,-0.257,-0.265,-0.273,-0.281,     &
                    -0.288,-0.294,-0.301,-0.307,-0.313,-0.318,-0.324,     &
                    -0.329,-0.334,-0.339,-0.344,-0.349,-0.353,-0.357,     &
                    -0.362,-0.366,-0.370,-0.374,-0.378,-0.382,-0.386,     &
                    -0.389,-0.393,-0.396,-0.400,-0.403,-0.407,-0.410,     &
                    -0.413,-0.416,-0.419,-0.423,-0.426,-0.429,-0.432,     &
                    -0.435,-0.437,-0.440,-0.443,-0.446,-0.449,-0.451,     &
                    -0.454,-0.457,-0.459,-0.462,-0.464,-0.467,-0.469,     &
                    -0.472,-0.474,-0.477,-0.479,-0.482,-0.484,-0.486,     &
                    -0.489,-0.491,-0.493,-0.496,-0.498,-0.500,-0.502,     &
                    -0.505,-0.507,-0.509,-0.511,-0.513,-0.516,-0.518,     &
                    -0.520,-0.522,-0.524,-0.526,-0.528,-0.530,-0.532,     &
                    -0.534,-0.537,-0.539,-0.541,-0.543,-0.545,-0.547,     &
                    -0.549,-0.551,-0.553,-0.555,-0.557,-0.559,-0.561,     &
                    -0.562,-0.564,-0.566,-0.568,-0.570,-0.572,-0.574,     &
                    -0.576,-0.578,-0.580,-0.582,-0.583,-0.585,-0.587,     &
                    -0.589,-0.591,-0.593,-0.594,-0.596,-0.598,-0.600,     &
                    -0.602,-0.604,-0.605,-0.607,-0.609,-0.611,-0.612,     &
                    -0.614,-0.616,-0.618,-0.619,-0.621,-0.623,-0.625,     &
                    -0.626,-0.628,-0.630,-0.631,-0.633,-0.635,-0.637,     &
                    -0.638,-0.640,-0.642,-0.643,-0.645,-0.647,-0.648,     &
                    -0.650,-0.651,-0.653,-0.655,-0.656,-0.658,-0.660,     &
                    -0.661,-0.663,-0.664,-0.666,-0.668,-0.669,-0.671,     &
                    -0.672,-0.674,-0.676,-0.677,-0.679,-0.680,-0.682,     &
                    -0.683,-0.685,-0.686,-0.688,-0.689,-0.691,-0.693,     &
                    -0.694,-0.696,-0.697,-0.699,-0.700,-0.702,-0.703,     &
                    -0.705,-0.706,-0.708,-0.709,-0.711,-0.712,-0.714,     &
                    -0.715,-0.717,-0.718,-0.719,-0.721,-0.722,-0.724,     &
                    -0.725,-0.727,-0.728,-0.730,-0.731,-0.733,-0.734,     &
                    -0.735,-0.737,-0.738,-0.740,-0.741,-0.743,-0.744,     &
                    -0.745,-0.747,-0.748,-0.750,-0.751,-0.752,-0.754,     &
                    -0.755,-0.757,-0.758,-0.759,-0.761,-0.762,-0.763,     &
                    -0.765,-0.766,-0.768,-0.769,-0.770,-0.772,-0.773,     &
                    -0.774,-0.776,-0.777,-0.778,-0.780,-0.781,-0.782,     &
                    -0.784,-0.785,-0.787,-0.788,-0.789,-0.791,-0.792,     &
                    -0.793,-0.794,-0.796,-0.797,-0.798,-0.800,-0.801,     &
                    -0.802,-0.804,-0.805,-0.806,-0.808,-0.809,-0.810,     &
                    -0.812,-0.813,-0.814,-0.815,-0.817,-0.818,-0.819,     &
                    -0.821,-0.822,-0.823,-0.824,-0.826,-0.827,-0.828,     &
                    -0.829,-0.831,-0.832,-0.833,-0.835,-0.836,-0.837,     &
                    -0.838,-0.840,-0.841,-0.842,-0.843,-0.845,-0.846,     &
                    -0.847,-0.848,-0.850,-0.851,-0.852,-0.853,-0.854,     &
                    -0.856,-0.857,-0.858,-0.859,-0.861,-0.862,-0.863,     &
                    -0.864,-0.866,-0.867,-0.868,-0.869,-0.870,-0.872,     &
                    -0.873,-0.874,-0.875,-0.876,-0.878,-0.879,-0.880,     &
                    -0.881,-0.882,-0.884,-0.885,-0.886,-0.887,-0.888,     &
                    -0.890,-0.891,-0.892,-0.893,-0.894,-0.896,-0.897,     &
                    -0.898,-0.899,-0.900,-0.901,-0.903,-0.904,-0.905,     &
                    -0.906,-0.907,-0.908,-0.910,-0.911,-0.912,-0.913,     &
                    -0.914,-0.915,-0.917,-0.918,-0.919,-0.920,-0.921,     &
                    -0.922,-0.924,-0.925,-0.926,-0.927,-0.928,-0.929,     &
                    -0.930,-0.932,-0.933,-0.934,-0.935,-0.936,-0.937,     &
                    -0.938,-0.940,-0.941,-0.942,-0.943,-0.944,-0.945,     &
                    -0.946,-0.947,-0.949,-0.950,-0.951,-0.952,-0.953,     &
                    -0.954,-0.955,-0.956,-0.958,-0.959,-0.960,-0.961,     &
                    -0.962,-0.963,-0.964,-0.965,-0.966,-0.968,-0.969,     &
                    -0.970,-0.982,-0.992,-1.003,-1.014,-1.025,-1.035,     &
                    -1.046,-1.056,-1.066,-1.076,-1.087,-1.097,-1.107,     &
                    -1.117,-1.126,-1.136,-1.146,-1.156,-1.165,-1.175,     &
                    -1.185,-1.194,-1.203,-1.213,-1.222,-1.231,-1.241,     &
                    -1.250,-1.259,-1.268,-1.277,-1.286,-1.295,-1.304,     &
                    -1.313,-1.322,-1.331,-1.340,-1.349,-1.357,-1.366,     &
                    -1.375,-1.383,-1.392,-1.401,-1.409,-1.418,-1.426,     &
                    -1.435,-1.443,-1.452,-1.460,-1.468,-1.477,-1.485,     &
                    -1.493,-1.502,-1.510,-1.518,-1.526,-1.535,-1.543,     &
                    -1.551,-1.559,-1.567,-1.575,-1.583,-1.591,-1.599,     &
                    -1.607,-1.615,-1.623,-1.631,-1.639,-1.647,-1.655,     &
                    -1.663,-1.671,-1.679,-1.686,-1.694,-1.702,-1.710,     &
                    -1.718,-1.725,-1.733,-1.741,-1.749,-1.756,-1.764,     &
                    -1.772,-1.779,-1.787,-1.794,-1.802,-1.810,-1.817,     &
                    -1.825,-1.832,-1.840,-1.847,-1.855,-1.862,-1.870,     &
                    -1.877,-1.885,-1.892,-1.900,-1.907,-1.915,-1.922,     &
                    -1.929,-1.937,-1.944,-1.952,-1.959,-1.966,-1.974,     &
                    -1.981,-1.988,-1.996,-2.003,-2.010,-2.018,-2.025,     &
                    -2.032,-2.039,-2.047,-2.054,-2.061,-2.068,-2.075,     &
                    -2.083,-2.090,-2.097,-2.104,-2.111,-2.118,-2.126,     &
                    -2.133,-2.140,-2.147,-2.154,-2.161,-2.168,-2.175,     &
                    -2.183,-2.190,-2.197,-2.204,-2.211,-2.218,-2.225,     &
                    -2.232,-2.239,-2.246,-2.253,-2.260,-2.267,-2.274,     &
                    -2.281/)
!
! *** (nh4)2so4
!
    bnc04m(1:561) = (/-0.103,-0.226,-0.289,-0.334,-0.369,-0.399,-0.425,   &
                    -0.448,-0.468,-0.487,-0.505,-0.521,-0.536,-0.550,     &
                    -0.563,-0.576,-0.588,-0.599,-0.610,-0.621,-0.631,     &
                    -0.641,-0.650,-0.659,-0.668,-0.677,-0.685,-0.693,     &
                    -0.701,-0.709,-0.717,-0.724,-0.731,-0.738,-0.745,     &
                    -0.752,-0.758,-0.765,-0.771,-0.777,-0.784,-0.790,     &
                    -0.796,-0.801,-0.807,-0.813,-0.818,-0.824,-0.829,     &
                    -0.835,-0.840,-0.845,-0.850,-0.855,-0.860,-0.865,     &
                    -0.870,-0.875,-0.880,-0.884,-0.889,-0.894,-0.898,     &
                    -0.903,-0.907,-0.912,-0.916,-0.920,-0.925,-0.929,     &
                    -0.933,-0.937,-0.942,-0.946,-0.950,-0.954,-0.958,     &
                    -0.962,-0.966,-0.970,-0.974,-0.978,-0.982,-0.985,     &
                    -0.989,-0.993,-0.997,-1.001,-1.004,-1.008,-1.012,     &
                    -1.016,-1.019,-1.023,-1.027,-1.030,-1.034,-1.037,     &
                    -1.041,-1.045,-1.048,-1.052,-1.055,-1.059,-1.062,     &
                    -1.066,-1.069,-1.072,-1.076,-1.079,-1.083,-1.086,     &
                    -1.089,-1.093,-1.096,-1.099,-1.103,-1.106,-1.109,     &
                    -1.113,-1.116,-1.119,-1.122,-1.126,-1.129,-1.132,     &
                    -1.135,-1.138,-1.142,-1.145,-1.148,-1.151,-1.154,     &
                    -1.157,-1.160,-1.164,-1.167,-1.170,-1.173,-1.176,     &
                    -1.179,-1.182,-1.185,-1.188,-1.191,-1.194,-1.197,     &
                    -1.200,-1.203,-1.206,-1.209,-1.212,-1.215,-1.218,     &
                    -1.221,-1.223,-1.226,-1.229,-1.232,-1.235,-1.238,     &
                    -1.241,-1.244,-1.246,-1.249,-1.252,-1.255,-1.258,     &
                    -1.261,-1.263,-1.266,-1.269,-1.272,-1.274,-1.277,     &
                    -1.280,-1.283,-1.285,-1.288,-1.291,-1.294,-1.296,     &
                    -1.299,-1.302,-1.304,-1.307,-1.310,-1.313,-1.315,     &
                    -1.318,-1.321,-1.323,-1.326,-1.328,-1.331,-1.334,     &
                    -1.336,-1.339,-1.342,-1.344,-1.347,-1.349,-1.352,     &
                    -1.355,-1.357,-1.360,-1.362,-1.365,-1.367,-1.370,     &
                    -1.373,-1.375,-1.378,-1.380,-1.383,-1.385,-1.388,     &
                    -1.390,-1.393,-1.395,-1.398,-1.400,-1.403,-1.405,     &
                    -1.408,-1.410,-1.413,-1.415,-1.418,-1.420,-1.423,     &
                    -1.425,-1.427,-1.430,-1.432,-1.435,-1.437,-1.440,     &
                    -1.442,-1.445,-1.447,-1.449,-1.452,-1.454,-1.457,     &
                    -1.459,-1.461,-1.464,-1.466,-1.469,-1.471,-1.473,     &
                    -1.476,-1.478,-1.480,-1.483,-1.485,-1.487,-1.490,     &
                    -1.492,-1.495,-1.497,-1.499,-1.502,-1.504,-1.506,     &
                    -1.509,-1.511,-1.513,-1.515,-1.518,-1.520,-1.522,     &
                    -1.525,-1.527,-1.529,-1.532,-1.534,-1.536,-1.538,     &
                    -1.541,-1.543,-1.545,-1.548,-1.550,-1.552,-1.554,     &
                    -1.557,-1.559,-1.561,-1.563,-1.566,-1.568,-1.570,     &
                    -1.572,-1.575,-1.577,-1.579,-1.581,-1.583,-1.586,     &
                    -1.588,-1.590,-1.592,-1.595,-1.597,-1.599,-1.601,     &
                    -1.603,-1.606,-1.608,-1.610,-1.612,-1.614,-1.616,     &
                    -1.619,-1.621,-1.623,-1.625,-1.627,-1.630,-1.632,     &
                    -1.634,-1.636,-1.638,-1.640,-1.642,-1.645,-1.647,     &
                    -1.649,-1.651,-1.653,-1.655,-1.658,-1.660,-1.662,     &
                    -1.664,-1.666,-1.668,-1.670,-1.672,-1.675,-1.677,     &
                    -1.679,-1.681,-1.683,-1.685,-1.687,-1.689,-1.691,     &
                    -1.694,-1.696,-1.698,-1.700,-1.702,-1.704,-1.706,     &
                    -1.708,-1.710,-1.712,-1.714,-1.717,-1.719,-1.721,     &
                    -1.723,-1.725,-1.727,-1.729,-1.731,-1.733,-1.735,     &
                    -1.737,-1.739,-1.741,-1.743,-1.745,-1.748,-1.750,     &
                    -1.752,-1.754,-1.756,-1.758,-1.760,-1.762,-1.764,     &
                    -1.766,-1.768,-1.770,-1.772,-1.774,-1.776,-1.778,     &
                    -1.780,-1.782,-1.784,-1.786,-1.788,-1.790,-1.792,     &
                    -1.794,-1.816,-1.836,-1.855,-1.875,-1.894,-1.914,     &
                    -1.933,-1.952,-1.971,-1.989,-2.008,-2.027,-2.045,     &
                    -2.063,-2.082,-2.100,-2.118,-2.136,-2.154,-2.171,     &
                    -2.189,-2.207,-2.224,-2.242,-2.259,-2.276,-2.293,     &
                    -2.311,-2.328,-2.345,-2.362,-2.379,-2.395,-2.412,     &
                    -2.429,-2.446,-2.462,-2.479,-2.495,-2.512,-2.528,     &
                    -2.544,-2.561,-2.577,-2.593,-2.609,-2.625,-2.642,     &
                    -2.658,-2.674,-2.689,-2.705,-2.721,-2.737,-2.753,     &
                    -2.768,-2.784,-2.800,-2.815,-2.831,-2.847,-2.862,     &
                    -2.878,-2.893,-2.908,-2.924,-2.939,-2.954,-2.970,     &
                    -2.985,-3.000,-3.015,-3.031,-3.046,-3.061,-3.076,     &
                    -3.091,-3.106,-3.121,-3.136,-3.151,-3.166,-3.181,     &
                    -3.196,-3.210,-3.225,-3.240,-3.255,-3.270,-3.284,     &
                    -3.299,-3.314,-3.328,-3.343,-3.358,-3.372,-3.387,     &
                    -3.401,-3.416,-3.430,-3.445,-3.459,-3.474,-3.488,     &
                    -3.503,-3.517,-3.531,-3.546,-3.560,-3.574,-3.589,     &
                    -3.603,-3.617,-3.631,-3.646,-3.660,-3.674,-3.688,     &
                    -3.702,-3.717,-3.731,-3.745,-3.759,-3.773,-3.787,     &
                    -3.801,-3.815,-3.829,-3.843,-3.857,-3.871,-3.885,     &
                    -3.899,-3.913,-3.927,-3.941,-3.955,-3.969,-3.982,     &
                    -3.996,-4.010,-4.024,-4.038,-4.051,-4.065,-4.079,     &
                    -4.093,-4.107,-4.120,-4.134,-4.148,-4.161,-4.175,     &
                    -4.189,-4.202,-4.216,-4.230,-4.243,-4.257,-4.271,     &
                    -4.284/)
!
! *** nh4no3
!
    bnc05m(1:561) = (/-0.052,-0.117,-0.152,-0.178,-0.199,-0.217,-0.232,   &
                    -0.247,-0.260,-0.272,-0.283,-0.294,-0.304,-0.314,     &
                    -0.323,-0.332,-0.341,-0.349,-0.357,-0.365,-0.372,     &
                    -0.379,-0.386,-0.393,-0.400,-0.407,-0.413,-0.419,     &
                    -0.426,-0.432,-0.437,-0.443,-0.449,-0.455,-0.460,     &
                    -0.466,-0.471,-0.476,-0.481,-0.486,-0.491,-0.496,     &
                    -0.501,-0.506,-0.511,-0.515,-0.520,-0.524,-0.529,     &
                    -0.533,-0.537,-0.542,-0.546,-0.550,-0.554,-0.558,     &
                    -0.563,-0.567,-0.570,-0.574,-0.578,-0.582,-0.586,     &
                    -0.590,-0.593,-0.597,-0.601,-0.604,-0.608,-0.612,     &
                    -0.615,-0.619,-0.622,-0.626,-0.629,-0.633,-0.636,     &
                    -0.640,-0.643,-0.646,-0.650,-0.653,-0.656,-0.660,     &
                    -0.663,-0.666,-0.670,-0.673,-0.676,-0.679,-0.683,     &
                    -0.686,-0.689,-0.692,-0.696,-0.699,-0.702,-0.705,     &
                    -0.708,-0.711,-0.715,-0.718,-0.721,-0.724,-0.727,     &
                    -0.730,-0.733,-0.736,-0.739,-0.742,-0.746,-0.749,     &
                    -0.752,-0.755,-0.758,-0.761,-0.764,-0.767,-0.770,     &
                    -0.773,-0.776,-0.778,-0.781,-0.784,-0.787,-0.790,     &
                    -0.793,-0.796,-0.799,-0.802,-0.805,-0.807,-0.810,     &
                    -0.813,-0.816,-0.819,-0.821,-0.824,-0.827,-0.830,     &
                    -0.833,-0.835,-0.838,-0.841,-0.843,-0.846,-0.849,     &
                    -0.852,-0.854,-0.857,-0.860,-0.862,-0.865,-0.867,     &
                    -0.870,-0.873,-0.875,-0.878,-0.880,-0.883,-0.886,     &
                    -0.888,-0.891,-0.893,-0.896,-0.898,-0.901,-0.903,     &
                    -0.906,-0.908,-0.911,-0.913,-0.916,-0.918,-0.921,     &
                    -0.923,-0.926,-0.928,-0.930,-0.933,-0.935,-0.938,     &
                    -0.940,-0.942,-0.945,-0.947,-0.950,-0.952,-0.954,     &
                    -0.957,-0.959,-0.961,-0.964,-0.966,-0.968,-0.971,     &
                    -0.973,-0.975,-0.977,-0.980,-0.982,-0.984,-0.987,     &
                    -0.989,-0.991,-0.993,-0.996,-0.998,-1.000,-1.002,     &
                    -1.004,-1.007,-1.009,-1.011,-1.013,-1.015,-1.018,     &
                    -1.020,-1.022,-1.024,-1.026,-1.028,-1.031,-1.033,     &
                    -1.035,-1.037,-1.039,-1.041,-1.043,-1.046,-1.048,     &
                    -1.050,-1.052,-1.054,-1.056,-1.058,-1.060,-1.062,     &
                    -1.064,-1.066,-1.068,-1.070,-1.072,-1.075,-1.077,     &
                    -1.079,-1.081,-1.083,-1.085,-1.087,-1.089,-1.091,     &
                    -1.093,-1.095,-1.097,-1.099,-1.101,-1.103,-1.105,     &
                    -1.107,-1.109,-1.110,-1.112,-1.114,-1.116,-1.118,     &
                    -1.120,-1.122,-1.124,-1.126,-1.128,-1.130,-1.132,     &
                    -1.134,-1.136,-1.137,-1.139,-1.141,-1.143,-1.145,     &
                    -1.147,-1.149,-1.151,-1.153,-1.154,-1.156,-1.158,     &
                    -1.160,-1.162,-1.164,-1.166,-1.167,-1.169,-1.171,     &
                    -1.173,-1.175,-1.176,-1.178,-1.180,-1.182,-1.184,     &
                    -1.186,-1.187,-1.189,-1.191,-1.193,-1.195,-1.196,     &
                    -1.198,-1.200,-1.202,-1.203,-1.205,-1.207,-1.209,     &
                    -1.210,-1.212,-1.214,-1.216,-1.217,-1.219,-1.221,     &
                    -1.223,-1.224,-1.226,-1.228,-1.230,-1.231,-1.233,     &
                    -1.235,-1.236,-1.238,-1.240,-1.241,-1.243,-1.245,     &
                    -1.247,-1.248,-1.250,-1.252,-1.253,-1.255,-1.257,     &
                    -1.258,-1.260,-1.262,-1.263,-1.265,-1.267,-1.268,     &
                    -1.270,-1.272,-1.273,-1.275,-1.276,-1.278,-1.280,     &
                    -1.281,-1.283,-1.285,-1.286,-1.288,-1.289,-1.291,     &
                    -1.293,-1.294,-1.296,-1.298,-1.299,-1.301,-1.302,     &
                    -1.304,-1.306,-1.307,-1.309,-1.310,-1.312,-1.313,     &
                    -1.315,-1.317,-1.318,-1.320,-1.321,-1.323,-1.324,     &
                    -1.326,-1.328,-1.329,-1.331,-1.332,-1.334,-1.335,     &
                    -1.337,-1.338,-1.340,-1.341,-1.343,-1.345,-1.346,     &
                    -1.348,-1.364,-1.379,-1.394,-1.408,-1.422,-1.436,     &
                    -1.450,-1.464,-1.478,-1.491,-1.504,-1.518,-1.531,     &
                    -1.543,-1.556,-1.569,-1.581,-1.593,-1.606,-1.618,     &
                    -1.630,-1.641,-1.653,-1.665,-1.676,-1.688,-1.699,     &
                    -1.710,-1.722,-1.733,-1.744,-1.755,-1.765,-1.776,     &
                    -1.787,-1.797,-1.808,-1.818,-1.829,-1.839,-1.849,     &
                    -1.859,-1.870,-1.880,-1.890,-1.900,-1.909,-1.919,     &
                    -1.929,-1.939,-1.948,-1.958,-1.968,-1.977,-1.986,     &
                    -1.996,-2.005,-2.015,-2.024,-2.033,-2.042,-2.051,     &
                    -2.060,-2.070,-2.079,-2.088,-2.096,-2.105,-2.114,     &
                    -2.123,-2.132,-2.141,-2.149,-2.158,-2.167,-2.175,     &
                    -2.184,-2.193,-2.201,-2.210,-2.218,-2.226,-2.235,     &
                    -2.243,-2.252,-2.260,-2.268,-2.277,-2.285,-2.293,     &
                    -2.301,-2.309,-2.318,-2.326,-2.334,-2.342,-2.350,     &
                    -2.358,-2.366,-2.374,-2.382,-2.390,-2.398,-2.406,     &
                    -2.414,-2.422,-2.429,-2.437,-2.445,-2.453,-2.461,     &
                    -2.468,-2.476,-2.484,-2.492,-2.499,-2.507,-2.515,     &
                    -2.522,-2.530,-2.537,-2.545,-2.553,-2.560,-2.568,     &
                    -2.575,-2.583,-2.590,-2.598,-2.605,-2.613,-2.620,     &
                    -2.628,-2.635,-2.642,-2.650,-2.657,-2.664,-2.672,     &
                    -2.679,-2.686,-2.694,-2.701,-2.708,-2.716,-2.723,     &
                    -2.730,-2.737,-2.745,-2.752,-2.759,-2.766,-2.773,     &
                    -2.781,-2.788,-2.795,-2.802,-2.809,-2.816,-2.823,     &
                    -2.831/)
!
! *** nh4cl
!
    bnc06m(1:561) = (/-0.051,-0.108,-0.136,-0.155,-0.170,-0.182,-0.192,   &
                    -0.200,-0.207,-0.214,-0.220,-0.225,-0.230,-0.234,     &
                    -0.238,-0.242,-0.245,-0.248,-0.251,-0.254,-0.256,     &
                    -0.259,-0.261,-0.263,-0.265,-0.267,-0.269,-0.270,     &
                    -0.272,-0.274,-0.275,-0.276,-0.278,-0.279,-0.280,     &
                    -0.281,-0.283,-0.284,-0.285,-0.286,-0.287,-0.288,     &
                    -0.289,-0.289,-0.290,-0.291,-0.292,-0.293,-0.293,     &
                    -0.294,-0.295,-0.296,-0.296,-0.297,-0.298,-0.298,     &
                    -0.299,-0.300,-0.300,-0.301,-0.301,-0.302,-0.302,     &
                    -0.303,-0.303,-0.304,-0.304,-0.305,-0.305,-0.306,     &
                    -0.306,-0.307,-0.307,-0.308,-0.308,-0.308,-0.309,     &
                    -0.309,-0.309,-0.310,-0.310,-0.310,-0.311,-0.311,     &
                    -0.311,-0.312,-0.312,-0.312,-0.312,-0.313,-0.313,     &
                    -0.313,-0.313,-0.313,-0.314,-0.314,-0.314,-0.314,     &
                    -0.314,-0.314,-0.314,-0.314,-0.315,-0.315,-0.315,     &
                    -0.315,-0.315,-0.315,-0.315,-0.315,-0.315,-0.315,     &
                    -0.315,-0.315,-0.315,-0.315,-0.315,-0.315,-0.315,     &
                    -0.315,-0.315,-0.315,-0.315,-0.315,-0.315,-0.315,     &
                    -0.315,-0.315,-0.315,-0.315,-0.315,-0.315,-0.315,     &
                    -0.315,-0.315,-0.315,-0.314,-0.314,-0.314,-0.314,     &
                    -0.314,-0.314,-0.314,-0.314,-0.314,-0.314,-0.314,     &
                    -0.314,-0.314,-0.314,-0.314,-0.314,-0.313,-0.313,     &
                    -0.313,-0.313,-0.313,-0.313,-0.313,-0.313,-0.313,     &
                    -0.313,-0.313,-0.313,-0.313,-0.313,-0.312,-0.312,     &
                    -0.312,-0.312,-0.312,-0.312,-0.312,-0.312,-0.312,     &
                    -0.312,-0.312,-0.312,-0.312,-0.312,-0.311,-0.311,     &
                    -0.311,-0.311,-0.311,-0.311,-0.311,-0.311,-0.311,     &
                    -0.311,-0.311,-0.311,-0.311,-0.311,-0.310,-0.310,     &
                    -0.310,-0.310,-0.310,-0.310,-0.310,-0.310,-0.310,     &
                    -0.310,-0.310,-0.310,-0.310,-0.310,-0.309,-0.309,     &
                    -0.309,-0.309,-0.309,-0.309,-0.309,-0.309,-0.309,     &
                    -0.309,-0.309,-0.309,-0.309,-0.309,-0.309,-0.309,     &
                    -0.308,-0.308,-0.308,-0.308,-0.308,-0.308,-0.308,     &
                    -0.308,-0.308,-0.308,-0.308,-0.308,-0.308,-0.308,     &
                    -0.308,-0.308,-0.308,-0.308,-0.307,-0.307,-0.307,     &
                    -0.307,-0.307,-0.307,-0.307,-0.307,-0.307,-0.307,     &
                    -0.307,-0.307,-0.307,-0.307,-0.307,-0.307,-0.307,     &
                    -0.307,-0.307,-0.307,-0.307,-0.307,-0.307,-0.307,     &
                    -0.307,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.306,-0.306,-0.306,     &
                    -0.306,-0.306,-0.306,-0.306,-0.307,-0.307,-0.307,     &
                    -0.307,-0.307,-0.307,-0.307,-0.307,-0.307,-0.307,     &
                    -0.307,-0.307,-0.307,-0.307,-0.307,-0.307,-0.307,     &
                    -0.307,-0.307,-0.307,-0.307,-0.307,-0.307,-0.307,     &
                    -0.308,-0.308,-0.308,-0.308,-0.308,-0.308,-0.308,     &
                    -0.308,-0.308,-0.309,-0.310,-0.311,-0.311,-0.312,     &
                    -0.313,-0.314,-0.315,-0.316,-0.317,-0.319,-0.320,     &
                    -0.321,-0.322,-0.324,-0.325,-0.327,-0.328,-0.330,     &
                    -0.331,-0.333,-0.335,-0.336,-0.338,-0.340,-0.342,     &
                    -0.344,-0.346,-0.348,-0.350,-0.352,-0.354,-0.356,     &
                    -0.358,-0.360,-0.362,-0.365,-0.367,-0.369,-0.371,     &
                    -0.374,-0.376,-0.379,-0.381,-0.383,-0.386,-0.389,     &
                    -0.391,-0.394,-0.396,-0.399,-0.401,-0.404,-0.407,     &
                    -0.410,-0.412,-0.415,-0.418,-0.421,-0.423,-0.426,     &
                    -0.429,-0.432,-0.435,-0.438,-0.441,-0.444,-0.447,     &
                    -0.450,-0.453,-0.456,-0.459,-0.462,-0.465,-0.468,     &
                    -0.471,-0.475,-0.478,-0.481,-0.484,-0.487,-0.491,     &
                    -0.494,-0.497,-0.500,-0.504,-0.507,-0.510,-0.514,     &
                    -0.517,-0.520,-0.524,-0.527,-0.531,-0.534,-0.537,     &
                    -0.541,-0.544,-0.548,-0.551,-0.555,-0.558,-0.562,     &
                    -0.565,-0.569,-0.573,-0.576,-0.580,-0.583,-0.587,     &
                    -0.591,-0.594,-0.598,-0.601,-0.605,-0.609,-0.612,     &
                    -0.616,-0.620,-0.624,-0.627,-0.631,-0.635,-0.639,     &
                    -0.642,-0.646,-0.650,-0.654,-0.657,-0.661,-0.665,     &
                    -0.669,-0.673,-0.677,-0.680,-0.684,-0.688,-0.692,     &
                    -0.696,-0.700,-0.704,-0.708,-0.711,-0.715,-0.719,     &
                    -0.723,-0.727,-0.731,-0.735,-0.739,-0.743,-0.747,     &
                    -0.751,-0.755,-0.759,-0.763,-0.767,-0.771,-0.775,     &
                    -0.779/)
!
! *** (2h ,so4)
!
    bnc07m(1:561) = (/-0.103,-0.225,-0.286,-0.330,-0.365,-0.394,-0.419,   &
                    -0.441,-0.460,-0.478,-0.495,-0.510,-0.524,-0.538,     &
                    -0.550,-0.562,-0.573,-0.584,-0.594,-0.604,-0.613,     &
                    -0.622,-0.631,-0.640,-0.648,-0.656,-0.663,-0.671,     &
                    -0.678,-0.685,-0.692,-0.699,-0.705,-0.711,-0.718,     &
                    -0.724,-0.730,-0.736,-0.741,-0.747,-0.753,-0.758,     &
                    -0.763,-0.769,-0.774,-0.779,-0.784,-0.789,-0.794,     &
                    -0.798,-0.803,-0.808,-0.812,-0.817,-0.821,-0.826,     &
                    -0.830,-0.834,-0.839,-0.843,-0.847,-0.851,-0.855,     &
                    -0.859,-0.863,-0.867,-0.871,-0.875,-0.879,-0.883,     &
                    -0.886,-0.890,-0.894,-0.897,-0.901,-0.905,-0.908,     &
                    -0.912,-0.915,-0.919,-0.922,-0.926,-0.929,-0.933,     &
                    -0.936,-0.939,-0.943,-0.946,-0.949,-0.953,-0.956,     &
                    -0.959,-0.962,-0.965,-0.969,-0.972,-0.975,-0.978,     &
                    -0.981,-0.984,-0.987,-0.990,-0.994,-0.997,-1.000,     &
                    -1.003,-1.006,-1.009,-1.012,-1.015,-1.017,-1.020,     &
                    -1.023,-1.026,-1.029,-1.032,-1.035,-1.038,-1.041,     &
                    -1.043,-1.046,-1.049,-1.052,-1.055,-1.057,-1.060,     &
                    -1.063,-1.066,-1.068,-1.071,-1.074,-1.077,-1.079,     &
                    -1.082,-1.085,-1.087,-1.090,-1.093,-1.095,-1.098,     &
                    -1.100,-1.103,-1.106,-1.108,-1.111,-1.113,-1.116,     &
                    -1.119,-1.121,-1.124,-1.126,-1.129,-1.131,-1.134,     &
                    -1.136,-1.139,-1.141,-1.144,-1.146,-1.149,-1.151,     &
                    -1.154,-1.156,-1.159,-1.161,-1.163,-1.166,-1.168,     &
                    -1.171,-1.173,-1.176,-1.178,-1.180,-1.183,-1.185,     &
                    -1.187,-1.190,-1.192,-1.195,-1.197,-1.199,-1.202,     &
                    -1.204,-1.206,-1.209,-1.211,-1.213,-1.216,-1.218,     &
                    -1.220,-1.222,-1.225,-1.227,-1.229,-1.232,-1.234,     &
                    -1.236,-1.238,-1.241,-1.243,-1.245,-1.247,-1.250,     &
                    -1.252,-1.254,-1.256,-1.258,-1.261,-1.263,-1.265,     &
                    -1.267,-1.270,-1.272,-1.274,-1.276,-1.278,-1.280,     &
                    -1.283,-1.285,-1.287,-1.289,-1.291,-1.293,-1.296,     &
                    -1.298,-1.300,-1.302,-1.304,-1.306,-1.308,-1.311,     &
                    -1.313,-1.315,-1.317,-1.319,-1.321,-1.323,-1.325,     &
                    -1.328,-1.330,-1.332,-1.334,-1.336,-1.338,-1.340,     &
                    -1.342,-1.344,-1.346,-1.348,-1.350,-1.353,-1.355,     &
                    -1.357,-1.359,-1.361,-1.363,-1.365,-1.367,-1.369,     &
                    -1.371,-1.373,-1.375,-1.377,-1.379,-1.381,-1.383,     &
                    -1.385,-1.387,-1.389,-1.391,-1.393,-1.395,-1.397,     &
                    -1.399,-1.401,-1.403,-1.405,-1.407,-1.409,-1.411,     &
                    -1.413,-1.415,-1.417,-1.419,-1.421,-1.423,-1.425,     &
                    -1.427,-1.429,-1.431,-1.433,-1.435,-1.437,-1.439,     &
                    -1.441,-1.443,-1.445,-1.447,-1.449,-1.451,-1.453,     &
                    -1.455,-1.456,-1.458,-1.460,-1.462,-1.464,-1.466,     &
                    -1.468,-1.470,-1.472,-1.474,-1.476,-1.478,-1.480,     &
                    -1.481,-1.483,-1.485,-1.487,-1.489,-1.491,-1.493,     &
                    -1.495,-1.497,-1.499,-1.500,-1.502,-1.504,-1.506,     &
                    -1.508,-1.510,-1.512,-1.514,-1.516,-1.517,-1.519,     &
                    -1.521,-1.523,-1.525,-1.527,-1.529,-1.531,-1.532,     &
                    -1.534,-1.536,-1.538,-1.540,-1.542,-1.544,-1.545,     &
                    -1.547,-1.549,-1.551,-1.553,-1.555,-1.557,-1.558,     &
                    -1.560,-1.562,-1.564,-1.566,-1.568,-1.569,-1.571,     &
                    -1.573,-1.575,-1.577,-1.579,-1.580,-1.582,-1.584,     &
                    -1.586,-1.588,-1.589,-1.591,-1.593,-1.595,-1.597,     &
                    -1.598,-1.600,-1.602,-1.604,-1.606,-1.608,-1.609,     &
                    -1.611,-1.613,-1.615,-1.617,-1.618,-1.620,-1.622,     &
                    -1.624,-1.625,-1.627,-1.629,-1.631,-1.633,-1.634,     &
                    -1.636,-1.655,-1.673,-1.691,-1.708,-1.725,-1.742,     &
                    -1.760,-1.777,-1.793,-1.810,-1.827,-1.844,-1.860,     &
                    -1.877,-1.893,-1.910,-1.926,-1.942,-1.958,-1.974,     &
                    -1.990,-2.006,-2.022,-2.038,-2.054,-2.070,-2.086,     &
                    -2.101,-2.117,-2.132,-2.148,-2.163,-2.179,-2.194,     &
                    -2.210,-2.225,-2.240,-2.255,-2.271,-2.286,-2.301,     &
                    -2.316,-2.331,-2.346,-2.361,-2.376,-2.391,-2.406,     &
                    -2.420,-2.435,-2.450,-2.465,-2.479,-2.494,-2.509,     &
                    -2.523,-2.538,-2.553,-2.567,-2.582,-2.596,-2.611,     &
                    -2.625,-2.640,-2.654,-2.668,-2.683,-2.697,-2.711,     &
                    -2.726,-2.740,-2.754,-2.768,-2.783,-2.797,-2.811,     &
                    -2.825,-2.839,-2.853,-2.867,-2.881,-2.895,-2.909,     &
                    -2.923,-2.937,-2.951,-2.965,-2.979,-2.993,-3.007,     &
                    -3.021,-3.035,-3.049,-3.063,-3.076,-3.090,-3.104,     &
                    -3.118,-3.132,-3.145,-3.159,-3.173,-3.186,-3.200,     &
                    -3.214,-3.227,-3.241,-3.255,-3.268,-3.282,-3.295,     &
                    -3.309,-3.323,-3.336,-3.350,-3.363,-3.377,-3.390,     &
                    -3.404,-3.417,-3.431,-3.444,-3.458,-3.471,-3.484,     &
                    -3.498,-3.511,-3.525,-3.538,-3.551,-3.565,-3.578,     &
                    -3.591,-3.605,-3.618,-3.631,-3.645,-3.658,-3.671,     &
                    -3.684,-3.698,-3.711,-3.724,-3.737,-3.750,-3.764,     &
                    -3.777,-3.790,-3.803,-3.816,-3.829,-3.843,-3.856,     &
                    -3.869,-3.882,-3.895,-3.908,-3.921,-3.934,-3.947,     &
                    -3.960/)
!
! *** (h ,hso4)
!
    bnc08m(1:561) = (/-0.047,-0.093,-0.110,-0.119,-0.125,-0.128,-0.130,   &
                    -0.130,-0.129,-0.128,-0.126,-0.123,-0.119,-0.116,     &
                    -0.111,-0.107,-0.102,-0.096,-0.091,-0.085,-0.079,     &
                    -0.073,-0.066,-0.059,-0.052,-0.045,-0.037,-0.030,     &
                    -0.022,-0.014,-0.006,0.003,0.011,0.020,0.028,0.037,   &
                    0.046,0.055,0.065,0.074,0.083,0.093,0.103,0.113,0.122,&
                    0.132,0.142,0.152,0.163,0.173,0.183,0.194,0.204,0.215,&
                    0.225,0.236,0.247,0.258,0.268,0.279,0.290,0.301,0.312,&
                    0.323,0.334,0.346,0.357,0.368,0.380,0.391,0.402,0.414,&
                    0.425,0.437,0.449,0.461,0.472,0.484,0.496,0.508,0.520,&
                    0.532,0.544,0.557,0.569,0.581,0.594,0.606,0.619,0.631,&
                    0.644,0.657,0.669,0.682,0.695,0.708,0.721,0.734,0.747,&
                    0.761,0.774,0.787,0.800,0.814,0.827,0.841,0.854,0.868,&
                    0.881,0.895,0.908,0.922,0.936,0.949,0.963,0.977,0.990,&
                    1.004,1.018,1.031,1.045,1.059,1.072,1.086,1.100,1.114,&
                    1.127,1.141,1.155,1.168,1.182,1.195,1.209,1.223,1.236,&
                    1.250,1.263,1.277,1.290,1.304,1.317,1.331,1.344,1.358,&
                    1.371,1.384,1.398,1.411,1.424,1.437,1.451,1.464,1.477,&
                    1.490,1.503,1.516,1.529,1.542,1.555,1.568,1.581,1.594,&
                    1.607,1.620,1.633,1.646,1.659,1.671,1.684,1.697,1.709,&
                    1.722,1.735,1.747,1.760,1.772,1.785,1.798,1.810,1.822,&
                    1.835,1.847,1.860,1.872,1.884,1.896,1.909,1.921,1.933,&
                    1.945,1.957,1.970,1.982,1.994,2.006,2.018,2.030,2.042,&
                    2.054,2.065,2.077,2.089,2.101,2.113,2.125,2.136,2.148,&
                    2.160,2.171,2.183,2.195,2.206,2.218,2.229,2.241,2.252,&
                    2.264,2.275,2.286,2.298,2.309,2.320,2.332,2.343,2.354,&
                    2.365,2.377,2.388,2.399,2.410,2.421,2.432,2.443,2.454,&
                    2.465,2.476,2.487,2.498,2.509,2.520,2.531,2.541,2.552,&
                    2.563,2.574,2.584,2.595,2.606,2.616,2.627,2.638,2.648,&
                    2.659,2.669,2.680,2.690,2.701,2.711,2.722,2.732,2.742,&
                    2.753,2.763,2.773,2.784,2.794,2.804,2.814,2.825,2.835,&
                    2.845,2.855,2.865,2.875,2.885,2.895,2.905,2.915,2.925,&
                    2.935,2.945,2.955,2.965,2.975,2.985,2.995,3.005,3.014,&
                    3.024,3.034,3.044,3.053,3.063,3.073,3.082,3.092,3.102,&
                    3.111,3.121,3.130,3.140,3.149,3.159,3.168,3.178,3.187,&
                    3.197,3.206,3.215,3.225,3.234,3.243,3.253,3.262,3.271,&
                    3.280,3.290,3.299,3.308,3.317,3.326,3.336,3.345,3.354,&
                    3.363,3.372,3.381,3.390,3.399,3.408,3.417,3.426,3.435,&
                    3.444,3.453,3.462,3.470,3.479,3.488,3.497,3.506,3.514,&
                    3.523,3.532,3.541,3.549,3.558,3.567,3.575,3.584,3.593,&
                    3.601,3.610,3.619,3.627,3.636,3.644,3.653,3.661,3.670,&
                    3.678,3.687,3.695,3.704,3.712,3.720,3.729,3.737,3.745,&
                    3.754,3.762,3.770,3.779,3.787,3.795,3.803,3.812,3.820,&
                    3.828,3.836,3.844,3.852,3.861,3.869,3.877,3.885,3.893,&
                    3.901,3.909,3.917,3.925,3.933,3.941,3.949,3.957,3.965,&
                    3.973,3.981,3.989,3.997,4.081,4.158,4.233,4.307,4.380,&
                    4.452,4.522,4.592,4.660,4.727,4.793,4.859,4.923,4.986,&
                    5.048,5.110,5.171,5.230,5.289,5.347,5.405,5.461,5.517,&
                    5.572,5.626,5.680,5.733,5.785,5.837,5.888,5.938,5.988,&
                    6.037,6.085,6.133,6.181,6.228,6.274,6.320,6.365,6.410,&
                    6.454,6.498,6.541,6.584,6.626,6.668,6.710,6.751,6.792,&
                    6.832,6.872,6.911,6.950,6.989,7.027,7.065,7.102,7.139,&
                    7.176,7.212,7.248,7.284,7.319,7.354,7.389,7.424,7.458,&
                    7.491,7.525,7.558,7.591,7.623,7.656,7.688,7.719,7.751,&
                    7.782,7.813,7.844,7.874,7.904,7.934,7.964,7.993,8.022,&
                    8.051,8.080,8.108,8.137,8.165,8.192,8.220,8.247,8.274,&
                    8.301,8.328,8.354,8.381,8.407,8.433,8.458,8.484,8.509,&
                    8.534,8.559,8.584,8.609,8.633,8.657,8.681,8.705,8.729,&
                    8.752,8.776,8.799,8.822,8.845,8.868,8.890,8.913,8.935,&
                    8.957,8.979,9.001,9.022,9.044,9.065,9.086,9.107,9.128,&
                    9.149,9.170,9.190,9.211,9.231,9.251,9.271,9.291,9.311,&
                    9.330,9.350,9.369,9.388,9.407,9.426,9.445,9.464,9.483,&
                    9.501,9.520,9.538,9.556,9.574,9.592,9.610,9.628,9.645,&
                    9.663,9.680,9.698/)
!
! *** nh4hso4
!
    bnc09m(1:561) = (/-0.050,-0.107,-0.134,-0.153,-0.167,-0.179,-0.189,   &
                    -0.197,-0.204,-0.211,-0.216,-0.221,-0.226,-0.230,     &
                    -0.234,-0.237,-0.240,-0.243,-0.245,-0.247,-0.249,     &
                    -0.251,-0.252,-0.254,-0.255,-0.256,-0.256,-0.257,     &
                    -0.257,-0.258,-0.258,-0.258,-0.258,-0.258,-0.257,     &
                    -0.257,-0.256,-0.255,-0.255,-0.254,-0.253,-0.252,     &
                    -0.250,-0.249,-0.248,-0.246,-0.245,-0.243,-0.242,     &
                    -0.240,-0.238,-0.236,-0.234,-0.232,-0.230,-0.228,     &
                    -0.226,-0.224,-0.221,-0.219,-0.217,-0.214,-0.212,     &
                    -0.209,-0.207,-0.204,-0.201,-0.199,-0.196,-0.193,     &
                    -0.190,-0.188,-0.185,-0.182,-0.179,-0.176,-0.173,     &
                    -0.170,-0.167,-0.163,-0.160,-0.157,-0.154,-0.150,     &
                    -0.147,-0.144,-0.140,-0.137,-0.133,-0.130,-0.126,     &
                    -0.123,-0.119,-0.116,-0.112,-0.108,-0.104,-0.101,     &
                    -0.097,-0.093,-0.089,-0.086,-0.082,-0.078,-0.074,     &
                    -0.070,-0.066,-0.062,-0.058,-0.054,-0.050,-0.046,     &
                    -0.042,-0.038,-0.034,-0.030,-0.026,-0.022,-0.018,     &
                    -0.014,-0.010,-0.006,-0.002,0.002,0.006,0.010,0.014,  &
                    0.018,0.022,0.026,0.030,0.034,0.038,0.042,0.046,0.050,&
                    0.054,0.058,0.062,0.066,0.070,0.074,0.078,0.082,0.086,&
                    0.090,0.094,0.098,0.102,0.106,0.110,0.114,0.117,0.121,&
                    0.125,0.129,0.133,0.137,0.140,0.144,0.148,0.152,0.156,&
                    0.159,0.163,0.167,0.171,0.174,0.178,0.182,0.186,0.189,&
                    0.193,0.197,0.200,0.204,0.208,0.211,0.215,0.219,0.222,&
                    0.226,0.229,0.233,0.237,0.240,0.244,0.247,0.251,0.254,&
                    0.258,0.261,0.265,0.268,0.272,0.275,0.279,0.282,0.286,&
                    0.289,0.293,0.296,0.299,0.303,0.306,0.309,0.313,0.316,&
                    0.320,0.323,0.326,0.330,0.333,0.336,0.339,0.343,0.346,&
                    0.349,0.353,0.356,0.359,0.362,0.366,0.369,0.372,0.375,&
                    0.378,0.382,0.385,0.388,0.391,0.394,0.397,0.401,0.404,&
                    0.407,0.410,0.413,0.416,0.419,0.422,0.425,0.428,0.431,&
                    0.434,0.438,0.441,0.444,0.447,0.450,0.453,0.456,0.459,&
                    0.462,0.465,0.467,0.470,0.473,0.476,0.479,0.482,0.485,&
                    0.488,0.491,0.494,0.497,0.500,0.502,0.505,0.508,0.511,&
                    0.514,0.517,0.520,0.522,0.525,0.528,0.531,0.534,0.536,&
                    0.539,0.542,0.545,0.547,0.550,0.553,0.556,0.558,0.561,&
                    0.564,0.567,0.569,0.572,0.575,0.577,0.580,0.583,0.585,&
                    0.588,0.591,0.593,0.596,0.599,0.601,0.604,0.607,0.609,&
                    0.612,0.614,0.617,0.620,0.622,0.625,0.627,0.630,0.633,&
                    0.635,0.638,0.640,0.643,0.645,0.648,0.650,0.653,0.655,&
                    0.658,0.660,0.663,0.665,0.668,0.670,0.673,0.675,0.678,&
                    0.680,0.683,0.685,0.687,0.690,0.692,0.695,0.697,0.700,&
                    0.702,0.704,0.707,0.709,0.711,0.714,0.716,0.719,0.721,&
                    0.723,0.726,0.728,0.730,0.733,0.735,0.737,0.740,0.742,&
                    0.744,0.747,0.749,0.751,0.754,0.756,0.758,0.760,0.763,&
                    0.765,0.767,0.769,0.772,0.774,0.776,0.778,0.781,0.783,&
                    0.785,0.787,0.790,0.792,0.794,0.796,0.798,0.801,0.803,&
                    0.805,0.807,0.809,0.811,0.814,0.816,0.818,0.820,0.822,&
                    0.824,0.827,0.829,0.851,0.872,0.892,0.912,0.932,0.951,&
                    0.969,0.988,1.006,1.024,1.041,1.058,1.075,1.091,1.107,&
                    1.123,1.139,1.154,1.169,1.184,1.199,1.213,1.227,1.241,&
                    1.255,1.268,1.282,1.295,1.307,1.320,1.332,1.345,1.357,&
                    1.368,1.380,1.392,1.403,1.414,1.425,1.436,1.446,1.457,&
                    1.467,1.478,1.488,1.497,1.507,1.517,1.526,1.536,1.545,&
                    1.554,1.563,1.572,1.580,1.589,1.597,1.606,1.614,1.622,&
                    1.630,1.638,1.646,1.653,1.661,1.668,1.676,1.683,1.690,&
                    1.697,1.704,1.711,1.718,1.724,1.731,1.737,1.744,1.750,&
                    1.756,1.762,1.769,1.775,1.780,1.786,1.792,1.798,1.803,&
                    1.809,1.814,1.820,1.825,1.830,1.835,1.840,1.845,1.850,&
                    1.855,1.860,1.865,1.870,1.874,1.879,1.883,1.888,1.892,&
                    1.896,1.901,1.905,1.909,1.913,1.917,1.921,1.925,1.929,&
                    1.933,1.936,1.940,1.944,1.947,1.951,1.955,1.958,1.961,&
                    1.965,1.968,1.971,1.975,1.978,1.981,1.984,1.987,1.990,&
                    1.993,1.996,1.999,2.001,2.004,2.007,2.010,2.012,2.015,&
                    2.017,2.020,2.022,2.025,2.027,2.030,2.032,2.034,2.037,&
                    2.039,2.041,2.043,2.045,2.047,2.049,2.051,2.053,2.055,&
                    2.057,2.059/)
!
! *** (h ,no3)
!
    bnc10m(1:561) = (/-0.049,-0.102,-0.125,-0.140,-0.150,-0.158,-0.163,   &
                    -0.168,-0.171,-0.174,-0.176,-0.178,-0.179,-0.179,     &
                    -0.179,-0.179,-0.179,-0.179,-0.178,-0.177,-0.176,     &
                    -0.175,-0.174,-0.172,-0.171,-0.169,-0.168,-0.166,     &
                    -0.164,-0.162,-0.160,-0.158,-0.156,-0.154,-0.152,     &
                    -0.150,-0.148,-0.146,-0.144,-0.141,-0.139,-0.137,     &
                    -0.135,-0.132,-0.130,-0.128,-0.126,-0.123,-0.121,     &
                    -0.119,-0.116,-0.114,-0.112,-0.109,-0.107,-0.105,     &
                    -0.102,-0.100,-0.098,-0.095,-0.093,-0.091,-0.088,     &
                    -0.086,-0.084,-0.081,-0.079,-0.077,-0.074,-0.072,     &
                    -0.070,-0.067,-0.065,-0.062,-0.060,-0.057,-0.055,     &
                    -0.052,-0.050,-0.048,-0.045,-0.042,-0.040,-0.037,     &
                    -0.035,-0.032,-0.030,-0.027,-0.024,-0.022,-0.019,     &
                    -0.016,-0.013,-0.011,-0.008,-0.005,-0.002,0.001,0.003,&
                    0.006,0.009,0.012,0.015,0.018,0.021,0.024,0.027,0.030,&
                    0.033,0.036,0.039,0.042,0.045,0.048,0.051,0.054,0.057,&
                    0.060,0.063,0.066,0.069,0.072,0.075,0.078,0.081,0.084,&
                    0.087,0.091,0.094,0.097,0.100,0.103,0.106,0.109,0.112,&
                    0.115,0.118,0.121,0.124,0.127,0.130,0.133,0.136,0.139,&
                    0.142,0.146,0.149,0.152,0.155,0.158,0.161,0.164,0.167,&
                    0.170,0.173,0.176,0.179,0.182,0.185,0.188,0.191,0.193,&
                    0.196,0.199,0.202,0.205,0.208,0.211,0.214,0.217,0.220,&
                    0.223,0.226,0.229,0.232,0.235,0.237,0.240,0.243,0.246,&
                    0.249,0.252,0.255,0.258,0.260,0.263,0.266,0.269,0.272,&
                    0.275,0.278,0.280,0.283,0.286,0.289,0.292,0.294,0.297,&
                    0.300,0.303,0.306,0.308,0.311,0.314,0.317,0.319,0.322,&
                    0.325,0.328,0.330,0.333,0.336,0.339,0.341,0.344,0.347,&
                    0.349,0.352,0.355,0.358,0.360,0.363,0.366,0.368,0.371,&
                    0.374,0.376,0.379,0.382,0.384,0.387,0.389,0.392,0.395,&
                    0.397,0.400,0.403,0.405,0.408,0.410,0.413,0.415,0.418,&
                    0.421,0.423,0.426,0.428,0.431,0.433,0.436,0.438,0.441,&
                    0.444,0.446,0.449,0.451,0.454,0.456,0.459,0.461,0.464,&
                    0.466,0.469,0.471,0.473,0.476,0.478,0.481,0.483,0.486,&
                    0.488,0.491,0.493,0.495,0.498,0.500,0.503,0.505,0.508,&
                    0.510,0.512,0.515,0.517,0.519,0.522,0.524,0.527,0.529,&
                    0.531,0.534,0.536,0.538,0.541,0.543,0.545,0.548,0.550,&
                    0.552,0.555,0.557,0.559,0.562,0.564,0.566,0.568,0.571,&
                    0.573,0.575,0.578,0.580,0.582,0.584,0.587,0.589,0.591,&
                    0.593,0.595,0.598,0.600,0.602,0.604,0.607,0.609,0.611,&
                    0.613,0.615,0.618,0.620,0.622,0.624,0.626,0.628,0.631,&
                    0.633,0.635,0.637,0.639,0.641,0.644,0.646,0.648,0.650,&
                    0.652,0.654,0.656,0.658,0.660,0.663,0.665,0.667,0.669,&
                    0.671,0.673,0.675,0.677,0.679,0.681,0.683,0.685,0.688,&
                    0.690,0.692,0.694,0.696,0.698,0.700,0.702,0.704,0.706,&
                    0.708,0.710,0.712,0.714,0.716,0.718,0.720,0.722,0.724,&
                    0.726,0.728,0.730,0.732,0.734,0.736,0.738,0.740,0.742,&
                    0.743,0.745,0.747,0.749,0.751,0.753,0.755,0.757,0.759,&
                    0.761,0.763,0.765,0.767,0.787,0.805,0.824,0.841,0.859,&
                    0.876,0.893,0.909,0.926,0.942,0.957,0.973,0.988,1.003,&
                    1.017,1.032,1.046,1.060,1.073,1.087,1.100,1.113,1.126,&
                    1.138,1.151,1.163,1.175,1.187,1.198,1.210,1.221,1.232,&
                    1.243,1.253,1.264,1.274,1.285,1.295,1.305,1.314,1.324,&
                    1.333,1.343,1.352,1.361,1.370,1.378,1.387,1.395,1.404,&
                    1.412,1.420,1.428,1.436,1.444,1.451,1.459,1.466,1.474,&
                    1.481,1.488,1.495,1.502,1.509,1.515,1.522,1.529,1.535,&
                    1.541,1.548,1.554,1.560,1.566,1.572,1.577,1.583,1.589,&
                    1.594,1.600,1.605,1.610,1.616,1.621,1.626,1.631,1.636,&
                    1.641,1.645,1.650,1.655,1.659,1.664,1.668,1.673,1.677,&
                    1.681,1.685,1.690,1.694,1.698,1.702,1.705,1.709,1.713,&
                    1.717,1.720,1.724,1.728,1.731,1.734,1.738,1.741,1.744,&
                    1.748,1.751,1.754,1.757,1.760,1.763,1.766,1.769,1.772,&
                    1.775,1.777,1.780,1.783,1.785,1.788,1.790,1.793,1.795,&
                    1.798,1.800,1.802,1.805,1.807,1.809,1.811,1.813,1.815,&
                    1.817,1.819,1.821,1.823,1.825,1.827,1.829,1.831,1.832,&
                    1.834,1.836,1.838,1.839,1.841,1.842,1.844,1.845,1.847,&
                    1.848,1.849,1.851/)
!
! *** (h ,cl)
!
    bnc11m(1:561) = (/-0.048,-0.094,-0.112,-0.122,-0.128,-0.131,-0.133,   &
                    -0.133,-0.133,-0.131,-0.129,-0.126,-0.123,-0.119,     &
                    -0.116,-0.111,-0.107,-0.102,-0.097,-0.092,-0.086,     &
                    -0.080,-0.075,-0.069,-0.062,-0.056,-0.050,-0.043,     &
                    -0.037,-0.030,-0.023,-0.016,-0.009,-0.002,0.005,0.012,&
                    0.020,0.027,0.035,0.042,0.050,0.057,0.065,0.072,0.080,&
                    0.088,0.095,0.103,0.111,0.119,0.126,0.134,0.142,0.150,&
                    0.158,0.166,0.174,0.182,0.190,0.198,0.206,0.214,0.222,&
                    0.230,0.238,0.246,0.254,0.262,0.270,0.278,0.287,0.295,&
                    0.303,0.311,0.320,0.328,0.336,0.345,0.353,0.362,0.370,&
                    0.379,0.387,0.396,0.405,0.413,0.422,0.431,0.440,0.449,&
                    0.457,0.466,0.475,0.484,0.494,0.503,0.512,0.521,0.530,&
                    0.540,0.549,0.558,0.568,0.577,0.587,0.596,0.606,0.615,&
                    0.625,0.634,0.644,0.653,0.663,0.673,0.682,0.692,0.702,&
                    0.711,0.721,0.731,0.740,0.750,0.760,0.769,0.779,0.789,&
                    0.798,0.808,0.818,0.827,0.837,0.847,0.856,0.866,0.875,&
                    0.885,0.895,0.904,0.914,0.923,0.933,0.942,0.952,0.961,&
                    0.971,0.980,0.990,0.999,1.009,1.018,1.027,1.037,1.046,&
                    1.055,1.065,1.074,1.083,1.093,1.102,1.111,1.120,1.130,&
                    1.139,1.148,1.157,1.166,1.175,1.184,1.194,1.203,1.212,&
                    1.221,1.230,1.239,1.248,1.257,1.266,1.275,1.283,1.292,&
                    1.301,1.310,1.319,1.328,1.336,1.345,1.354,1.363,1.372,&
                    1.380,1.389,1.398,1.406,1.415,1.423,1.432,1.441,1.449,&
                    1.458,1.466,1.475,1.483,1.492,1.500,1.509,1.517,1.526,&
                    1.534,1.542,1.551,1.559,1.567,1.576,1.584,1.592,1.600,&
                    1.609,1.617,1.625,1.633,1.641,1.649,1.658,1.666,1.674,&
                    1.682,1.690,1.698,1.706,1.714,1.722,1.730,1.738,1.746,&
                    1.754,1.762,1.769,1.777,1.785,1.793,1.801,1.809,1.816,&
                    1.824,1.832,1.840,1.847,1.855,1.863,1.870,1.878,1.886,&
                    1.893,1.901,1.908,1.916,1.924,1.931,1.939,1.946,1.954,&
                    1.961,1.969,1.976,1.983,1.991,1.998,2.006,2.013,2.020,&
                    2.028,2.035,2.042,2.049,2.057,2.064,2.071,2.078,2.086,&
                    2.093,2.100,2.107,2.114,2.122,2.129,2.136,2.143,2.150,&
                    2.157,2.164,2.171,2.178,2.185,2.192,2.199,2.206,2.213,&
                    2.220,2.227,2.234,2.241,2.247,2.254,2.261,2.268,2.275,&
                    2.282,2.288,2.295,2.302,2.309,2.315,2.322,2.329,2.336,&
                    2.342,2.349,2.356,2.362,2.369,2.376,2.382,2.389,2.395,&
                    2.402,2.408,2.415,2.421,2.428,2.434,2.441,2.447,2.454,&
                    2.460,2.467,2.473,2.480,2.486,2.492,2.499,2.505,2.512,&
                    2.518,2.524,2.531,2.537,2.543,2.549,2.556,2.562,2.568,&
                    2.574,2.581,2.587,2.593,2.599,2.605,2.612,2.618,2.624,&
                    2.630,2.636,2.642,2.648,2.654,2.660,2.666,2.672,2.679,&
                    2.685,2.691,2.697,2.703,2.709,2.714,2.720,2.726,2.732,&
                    2.738,2.744,2.750,2.756,2.762,2.768,2.774,2.779,2.785,&
                    2.791,2.797,2.803,2.808,2.814,2.820,2.826,2.832,2.837,&
                    2.843,2.849,2.854,2.860,2.921,2.976,3.031,3.084,3.137,&
                    3.189,3.240,3.290,3.339,3.387,3.435,3.482,3.528,3.574,&
                    3.619,3.663,3.706,3.749,3.792,3.833,3.874,3.915,3.955,&
                    3.994,4.033,4.072,4.109,4.147,4.184,4.220,4.256,4.291,&
                    4.327,4.361,4.395,4.429,4.462,4.495,4.528,4.560,4.592,&
                    4.623,4.654,4.685,4.716,4.746,4.775,4.805,4.834,4.862,&
                    4.891,4.919,4.947,4.974,5.001,5.028,5.055,5.081,5.107,&
                    5.133,5.159,5.184,5.209,5.234,5.259,5.283,5.307,5.331,&
                    5.355,5.378,5.401,5.424,5.447,5.469,5.492,5.514,5.536,&
                    5.557,5.579,5.600,5.621,5.642,5.663,5.684,5.704,5.724,&
                    5.744,5.764,5.784,5.803,5.823,5.842,5.861,5.880,5.898,&
                    5.917,5.935,5.953,5.972,5.989,6.007,6.025,6.042,6.060,&
                    6.077,6.094,6.111,6.128,6.144,6.161,6.177,6.194,6.210,&
                    6.226,6.242,6.257,6.273,6.289,6.304,6.319,6.334,6.350,&
                    6.365,6.379,6.394,6.409,6.423,6.438,6.452,6.466,6.480,&
                    6.494,6.508,6.522,6.536,6.549,6.563,6.576,6.589,6.602,&
                    6.616,6.629,6.641,6.654,6.667,6.680,6.692,6.705,6.717,&
                    6.729,6.742,6.754,6.766,6.778,6.790,6.801,6.813,6.825,&
                    6.836,6.848,6.859/)
!
! *** nahso4
!
    bnc12m(1:561) = (/-0.049,-0.101,-0.125,-0.140,-0.151,-0.159,-0.166,   &
                    -0.171,-0.175,-0.178,-0.181,-0.183,-0.184,-0.185,     &
                    -0.186,-0.186,-0.186,-0.186,-0.186,-0.185,-0.184,     &
                    -0.183,-0.181,-0.180,-0.178,-0.176,-0.174,-0.172,     &
                    -0.170,-0.167,-0.165,-0.162,-0.159,-0.156,-0.153,     &
                    -0.150,-0.147,-0.144,-0.140,-0.137,-0.133,-0.130,     &
                    -0.126,-0.122,-0.118,-0.114,-0.110,-0.106,-0.102,     &
                    -0.098,-0.094,-0.090,-0.085,-0.081,-0.076,-0.072,     &
                    -0.068,-0.063,-0.058,-0.054,-0.049,-0.044,-0.040,     &
                    -0.035,-0.030,-0.025,-0.020,-0.015,-0.010,-0.005,     &
                    0.000,0.005,0.010,0.015,0.020,0.026,0.031,0.036,0.042,&
                    0.047,0.052,0.058,0.063,0.069,0.075,0.080,0.086,0.092,&
                    0.097,0.103,0.109,0.115,0.121,0.127,0.133,0.139,0.145,&
                    0.151,0.157,0.163,0.169,0.175,0.181,0.188,0.194,0.200,&
                    0.206,0.213,0.219,0.225,0.232,0.238,0.244,0.251,0.257,&
                    0.264,0.270,0.276,0.283,0.289,0.296,0.302,0.308,0.315,&
                    0.321,0.328,0.334,0.341,0.347,0.353,0.360,0.366,0.373,&
                    0.379,0.385,0.392,0.398,0.404,0.411,0.417,0.423,0.430,&
                    0.436,0.442,0.448,0.455,0.461,0.467,0.473,0.480,0.486,&
                    0.492,0.498,0.504,0.510,0.517,0.523,0.529,0.535,0.541,&
                    0.547,0.553,0.559,0.565,0.571,0.577,0.583,0.589,0.595,&
                    0.601,0.607,0.613,0.619,0.625,0.631,0.637,0.642,0.648,&
                    0.654,0.660,0.666,0.671,0.677,0.683,0.689,0.694,0.700,&
                    0.706,0.712,0.717,0.723,0.729,0.734,0.740,0.746,0.751,&
                    0.757,0.762,0.768,0.773,0.779,0.785,0.790,0.796,0.801,&
                    0.807,0.812,0.817,0.823,0.828,0.834,0.839,0.845,0.850,&
                    0.855,0.861,0.866,0.871,0.877,0.882,0.887,0.893,0.898,&
                    0.903,0.908,0.914,0.919,0.924,0.929,0.934,0.939,0.945,&
                    0.950,0.955,0.960,0.965,0.970,0.975,0.980,0.986,0.991,&
                    0.996,1.001,1.006,1.011,1.016,1.021,1.026,1.031,1.036,&
                    1.041,1.046,1.050,1.055,1.060,1.065,1.070,1.075,1.080,&
                    1.085,1.089,1.094,1.099,1.104,1.109,1.114,1.118,1.123,&
                    1.128,1.133,1.137,1.142,1.147,1.151,1.156,1.161,1.165,&
                    1.170,1.175,1.179,1.184,1.189,1.193,1.198,1.203,1.207,&
                    1.212,1.216,1.221,1.225,1.230,1.234,1.239,1.243,1.248,&
                    1.252,1.257,1.261,1.266,1.270,1.275,1.279,1.284,1.288,&
                    1.293,1.297,1.301,1.306,1.310,1.314,1.319,1.323,1.327,&
                    1.332,1.336,1.340,1.345,1.349,1.353,1.358,1.362,1.366,&
                    1.370,1.375,1.379,1.383,1.387,1.391,1.396,1.400,1.404,&
                    1.408,1.412,1.417,1.421,1.425,1.429,1.433,1.437,1.441,&
                    1.445,1.450,1.454,1.458,1.462,1.466,1.470,1.474,1.478,&
                    1.482,1.486,1.490,1.494,1.498,1.502,1.506,1.510,1.514,&
                    1.518,1.522,1.526,1.530,1.534,1.538,1.542,1.546,1.549,&
                    1.553,1.557,1.561,1.565,1.569,1.573,1.577,1.580,1.584,&
                    1.588,1.592,1.596,1.600,1.603,1.607,1.611,1.615,1.618,&
                    1.622,1.626,1.630,1.633,1.637,1.641,1.645,1.648,1.652,&
                    1.656,1.660,1.663,1.667,1.671,1.674,1.714,1.749,1.784,&
                    1.819,1.853,1.886,1.919,1.951,1.982,2.014,2.044,2.074,&
                    2.104,2.133,2.162,2.190,2.218,2.246,2.273,2.300,2.326,&
                    2.352,2.377,2.403,2.427,2.452,2.476,2.500,2.523,2.547,&
                    2.569,2.592,2.614,2.636,2.658,2.679,2.700,2.721,2.742,&
                    2.762,2.782,2.802,2.822,2.841,2.860,2.879,2.898,2.916,&
                    2.934,2.952,2.970,2.988,3.005,3.022,3.039,3.056,3.073,&
                    3.089,3.105,3.121,3.137,3.153,3.168,3.184,3.199,3.214,&
                    3.229,3.244,3.258,3.273,3.287,3.301,3.315,3.329,3.342,&
                    3.356,3.369,3.382,3.395,3.408,3.421,3.434,3.446,3.459,&
                    3.471,3.483,3.496,3.507,3.519,3.531,3.543,3.554,3.566,&
                    3.577,3.588,3.599,3.610,3.621,3.632,3.642,3.653,3.663,&
                    3.674,3.684,3.694,3.704,3.714,3.724,3.734,3.743,3.753,&
                    3.762,3.772,3.781,3.790,3.800,3.809,3.818,3.826,3.835,&
                    3.844,3.853,3.861,3.870,3.878,3.887,3.895,3.903,3.911,&
                    3.919,3.927,3.935,3.943,3.951,3.959,3.966,3.974,3.981,&
                    3.989,3.996,4.004,4.011,4.018,4.025,4.032,4.039,4.046,&
                    4.053,4.060,4.067,4.073,4.080,4.087,4.093,4.100,4.106,&
                    4.113,4.119,4.125,4.131,4.138/)
!
! *** (nh4)3h(so4)2
!
    bnc13m(1:561) = (/-0.082,-0.178,-0.227,-0.261,-0.288,-0.311,-0.330,   &
                    -0.347,-0.363,-0.377,-0.389,-0.401,-0.412,-0.422,     &
                    -0.431,-0.440,-0.449,-0.457,-0.464,-0.471,-0.478,     &
                    -0.485,-0.491,-0.497,-0.503,-0.508,-0.514,-0.519,     &
                    -0.524,-0.529,-0.533,-0.538,-0.542,-0.546,-0.550,     &
                    -0.554,-0.557,-0.561,-0.565,-0.568,-0.571,-0.574,     &
                    -0.578,-0.581,-0.583,-0.586,-0.589,-0.592,-0.594,     &
                    -0.597,-0.599,-0.602,-0.604,-0.606,-0.608,-0.610,     &
                    -0.612,-0.614,-0.616,-0.618,-0.620,-0.622,-0.624,     &
                    -0.625,-0.627,-0.629,-0.630,-0.632,-0.633,-0.635,     &
                    -0.636,-0.637,-0.639,-0.640,-0.641,-0.643,-0.644,     &
                    -0.645,-0.646,-0.647,-0.648,-0.649,-0.650,-0.651,     &
                    -0.652,-0.653,-0.654,-0.655,-0.656,-0.657,-0.658,     &
                    -0.658,-0.659,-0.660,-0.661,-0.661,-0.662,-0.663,     &
                    -0.663,-0.664,-0.665,-0.665,-0.666,-0.666,-0.667,     &
                    -0.667,-0.668,-0.668,-0.669,-0.669,-0.670,-0.670,     &
                    -0.671,-0.671,-0.671,-0.672,-0.672,-0.673,-0.673,     &
                    -0.673,-0.674,-0.674,-0.674,-0.675,-0.675,-0.675,     &
                    -0.676,-0.676,-0.676,-0.676,-0.677,-0.677,-0.677,     &
                    -0.677,-0.678,-0.678,-0.678,-0.679,-0.679,-0.679,     &
                    -0.679,-0.679,-0.680,-0.680,-0.680,-0.680,-0.681,     &
                    -0.681,-0.681,-0.681,-0.681,-0.682,-0.682,-0.682,     &
                    -0.682,-0.682,-0.683,-0.683,-0.683,-0.683,-0.683,     &
                    -0.684,-0.684,-0.684,-0.684,-0.684,-0.685,-0.685,     &
                    -0.685,-0.685,-0.685,-0.686,-0.686,-0.686,-0.686,     &
                    -0.686,-0.687,-0.687,-0.687,-0.687,-0.687,-0.687,     &
                    -0.688,-0.688,-0.688,-0.688,-0.688,-0.689,-0.689,     &
                    -0.689,-0.689,-0.689,-0.690,-0.690,-0.690,-0.690,     &
                    -0.690,-0.691,-0.691,-0.691,-0.691,-0.691,-0.691,     &
                    -0.692,-0.692,-0.692,-0.692,-0.692,-0.693,-0.693,     &
                    -0.693,-0.693,-0.693,-0.694,-0.694,-0.694,-0.694,     &
                    -0.694,-0.695,-0.695,-0.695,-0.695,-0.695,-0.696,     &
                    -0.696,-0.696,-0.696,-0.696,-0.697,-0.697,-0.697,     &
                    -0.697,-0.698,-0.698,-0.698,-0.698,-0.698,-0.699,     &
                    -0.699,-0.699,-0.699,-0.699,-0.700,-0.700,-0.700,     &
                    -0.700,-0.701,-0.701,-0.701,-0.701,-0.702,-0.702,     &
                    -0.702,-0.702,-0.702,-0.703,-0.703,-0.703,-0.703,     &
                    -0.704,-0.704,-0.704,-0.704,-0.705,-0.705,-0.705,     &
                    -0.705,-0.706,-0.706,-0.706,-0.706,-0.706,-0.707,     &
                    -0.707,-0.707,-0.707,-0.708,-0.708,-0.708,-0.708,     &
                    -0.709,-0.709,-0.709,-0.710,-0.710,-0.710,-0.710,     &
                    -0.711,-0.711,-0.711,-0.711,-0.712,-0.712,-0.712,     &
                    -0.712,-0.713,-0.713,-0.713,-0.713,-0.714,-0.714,     &
                    -0.714,-0.715,-0.715,-0.715,-0.715,-0.716,-0.716,     &
                    -0.716,-0.716,-0.717,-0.717,-0.717,-0.718,-0.718,     &
                    -0.718,-0.718,-0.719,-0.719,-0.719,-0.720,-0.720,     &
                    -0.720,-0.720,-0.721,-0.721,-0.721,-0.722,-0.722,     &
                    -0.722,-0.723,-0.723,-0.723,-0.723,-0.724,-0.724,     &
                    -0.724,-0.725,-0.725,-0.725,-0.726,-0.726,-0.726,     &
                    -0.727,-0.727,-0.727,-0.727,-0.728,-0.728,-0.728,     &
                    -0.729,-0.729,-0.729,-0.730,-0.730,-0.730,-0.731,     &
                    -0.731,-0.731,-0.732,-0.732,-0.732,-0.733,-0.733,     &
                    -0.733,-0.733,-0.734,-0.734,-0.734,-0.735,-0.735,     &
                    -0.735,-0.736,-0.736,-0.736,-0.737,-0.737,-0.737,     &
                    -0.738,-0.738,-0.738,-0.739,-0.739,-0.740,-0.740,     &
                    -0.740,-0.741,-0.741,-0.741,-0.742,-0.742,-0.742,     &
                    -0.743,-0.743,-0.743,-0.744,-0.744,-0.744,-0.745,     &
                    -0.745,-0.749,-0.753,-0.756,-0.760,-0.764,-0.768,     &
                    -0.772,-0.776,-0.780,-0.784,-0.788,-0.793,-0.797,     &
                    -0.801,-0.806,-0.810,-0.815,-0.820,-0.824,-0.829,     &
                    -0.834,-0.839,-0.844,-0.848,-0.853,-0.858,-0.863,     &
                    -0.869,-0.874,-0.879,-0.884,-0.889,-0.895,-0.900,     &
                    -0.905,-0.911,-0.916,-0.922,-0.927,-0.933,-0.938,     &
                    -0.944,-0.950,-0.955,-0.961,-0.967,-0.972,-0.978,     &
                    -0.984,-0.990,-0.996,-1.002,-1.008,-1.014,-1.020,     &
                    -1.026,-1.032,-1.038,-1.044,-1.050,-1.056,-1.062,     &
                    -1.068,-1.075,-1.081,-1.087,-1.093,-1.100,-1.106,     &
                    -1.112,-1.119,-1.125,-1.131,-1.138,-1.144,-1.151,     &
                    -1.157,-1.164,-1.170,-1.177,-1.183,-1.190,-1.196,     &
                    -1.203,-1.209,-1.216,-1.223,-1.229,-1.236,-1.243,     &
                    -1.249,-1.256,-1.263,-1.270,-1.276,-1.283,-1.290,     &
                    -1.297,-1.304,-1.310,-1.317,-1.324,-1.331,-1.338,     &
                    -1.345,-1.352,-1.359,-1.365,-1.372,-1.379,-1.386,     &
                    -1.393,-1.400,-1.407,-1.414,-1.421,-1.428,-1.435,     &
                    -1.442,-1.450,-1.457,-1.464,-1.471,-1.478,-1.485,     &
                    -1.492,-1.499,-1.506,-1.514,-1.521,-1.528,-1.535,     &
                    -1.542,-1.549,-1.557,-1.564,-1.571,-1.578,-1.586,     &
                    -1.593,-1.600,-1.607,-1.615,-1.622,-1.629,-1.636,     &
                    -1.644,-1.651,-1.658,-1.666,-1.673,-1.680,-1.688,     &
                    -1.695,-1.702,-1.710,-1.717,-1.725,-1.732,-1.739,     &
                    -1.747/)
!
! *** caso4
!
    bnc14m(1:561) = (/0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,    &
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000/)
!
! *** cano32
!
    bnc15m(1:561) = (/-0.101,-0.216,-0.271,-0.308,-0.337,-0.360,-0.379,   &
                    -0.396,-0.410,-0.422,-0.433,-0.443,-0.452,-0.460,     &
                    -0.468,-0.475,-0.481,-0.487,-0.492,-0.497,-0.502,     &
                    -0.506,-0.510,-0.514,-0.517,-0.520,-0.524,-0.526,     &
                    -0.529,-0.532,-0.534,-0.537,-0.539,-0.541,-0.543,     &
                    -0.545,-0.547,-0.549,-0.550,-0.552,-0.553,-0.555,     &
                    -0.556,-0.558,-0.559,-0.560,-0.561,-0.563,-0.564,     &
                    -0.565,-0.566,-0.567,-0.568,-0.569,-0.570,-0.571,     &
                    -0.572,-0.572,-0.573,-0.574,-0.575,-0.576,-0.576,     &
                    -0.577,-0.578,-0.578,-0.579,-0.580,-0.580,-0.581,     &
                    -0.581,-0.582,-0.582,-0.583,-0.583,-0.584,-0.584,     &
                    -0.585,-0.585,-0.585,-0.586,-0.586,-0.586,-0.586,     &
                    -0.587,-0.587,-0.587,-0.587,-0.587,-0.587,-0.587,     &
                    -0.588,-0.588,-0.588,-0.588,-0.588,-0.587,-0.587,     &
                    -0.587,-0.587,-0.587,-0.587,-0.587,-0.587,-0.586,     &
                    -0.586,-0.586,-0.586,-0.585,-0.585,-0.585,-0.585,     &
                    -0.584,-0.584,-0.584,-0.583,-0.583,-0.583,-0.582,     &
                    -0.582,-0.581,-0.581,-0.581,-0.580,-0.580,-0.579,     &
                    -0.579,-0.579,-0.578,-0.578,-0.577,-0.577,-0.576,     &
                    -0.576,-0.576,-0.575,-0.575,-0.574,-0.574,-0.573,     &
                    -0.573,-0.572,-0.572,-0.571,-0.571,-0.570,-0.570,     &
                    -0.570,-0.569,-0.569,-0.568,-0.568,-0.567,-0.567,     &
                    -0.566,-0.566,-0.565,-0.565,-0.564,-0.564,-0.563,     &
                    -0.563,-0.562,-0.562,-0.562,-0.561,-0.561,-0.560,     &
                    -0.560,-0.559,-0.559,-0.558,-0.558,-0.557,-0.557,     &
                    -0.556,-0.556,-0.555,-0.555,-0.554,-0.554,-0.554,     &
                    -0.553,-0.553,-0.552,-0.552,-0.551,-0.551,-0.550,     &
                    -0.550,-0.549,-0.549,-0.548,-0.548,-0.548,-0.547,     &
                    -0.547,-0.546,-0.546,-0.545,-0.545,-0.544,-0.544,     &
                    -0.544,-0.543,-0.543,-0.542,-0.542,-0.541,-0.541,     &
                    -0.540,-0.540,-0.540,-0.539,-0.539,-0.538,-0.538,     &
                    -0.537,-0.537,-0.537,-0.536,-0.536,-0.535,-0.535,     &
                    -0.535,-0.534,-0.534,-0.533,-0.533,-0.533,-0.532,     &
                    -0.532,-0.531,-0.531,-0.531,-0.530,-0.530,-0.529,     &
                    -0.529,-0.529,-0.528,-0.528,-0.527,-0.527,-0.527,     &
                    -0.526,-0.526,-0.525,-0.525,-0.525,-0.524,-0.524,     &
                    -0.524,-0.523,-0.523,-0.523,-0.522,-0.522,-0.521,     &
                    -0.521,-0.521,-0.520,-0.520,-0.520,-0.519,-0.519,     &
                    -0.519,-0.518,-0.518,-0.518,-0.517,-0.517,-0.517,     &
                    -0.516,-0.516,-0.516,-0.515,-0.515,-0.515,-0.514,     &
                    -0.514,-0.514,-0.513,-0.513,-0.513,-0.512,-0.512,     &
                    -0.512,-0.512,-0.511,-0.511,-0.511,-0.510,-0.510,     &
                    -0.510,-0.509,-0.509,-0.509,-0.509,-0.508,-0.508,     &
                    -0.508,-0.507,-0.507,-0.507,-0.507,-0.506,-0.506,     &
                    -0.506,-0.506,-0.505,-0.505,-0.505,-0.504,-0.504,     &
                    -0.504,-0.504,-0.503,-0.503,-0.503,-0.503,-0.502,     &
                    -0.502,-0.502,-0.502,-0.501,-0.501,-0.501,-0.501,     &
                    -0.501,-0.500,-0.500,-0.500,-0.500,-0.499,-0.499,     &
                    -0.499,-0.499,-0.499,-0.498,-0.498,-0.498,-0.498,     &
                    -0.497,-0.497,-0.497,-0.497,-0.497,-0.496,-0.496,     &
                    -0.496,-0.496,-0.496,-0.495,-0.495,-0.495,-0.495,     &
                    -0.495,-0.495,-0.494,-0.494,-0.494,-0.494,-0.494,     &
                    -0.493,-0.493,-0.493,-0.493,-0.493,-0.493,-0.492,     &
                    -0.492,-0.492,-0.492,-0.492,-0.492,-0.491,-0.491,     &
                    -0.491,-0.491,-0.491,-0.491,-0.491,-0.490,-0.490,     &
                    -0.490,-0.490,-0.490,-0.490,-0.490,-0.489,-0.489,     &
                    -0.489,-0.489,-0.489,-0.489,-0.489,-0.489,-0.488,     &
                    -0.488,-0.487,-0.486,-0.485,-0.485,-0.484,-0.484,     &
                    -0.484,-0.484,-0.484,-0.484,-0.484,-0.485,-0.485,     &
                    -0.486,-0.487,-0.488,-0.489,-0.490,-0.491,-0.492,     &
                    -0.494,-0.495,-0.497,-0.499,-0.501,-0.503,-0.505,     &
                    -0.507,-0.509,-0.511,-0.514,-0.516,-0.519,-0.522,     &
                    -0.524,-0.527,-0.530,-0.533,-0.536,-0.539,-0.543,     &
                    -0.546,-0.549,-0.553,-0.556,-0.560,-0.563,-0.567,     &
                    -0.571,-0.575,-0.578,-0.582,-0.586,-0.590,-0.595,     &
                    -0.599,-0.603,-0.607,-0.612,-0.616,-0.620,-0.625,     &
                    -0.629,-0.634,-0.639,-0.643,-0.648,-0.653,-0.658,     &
                    -0.663,-0.668,-0.673,-0.678,-0.683,-0.688,-0.693,     &
                    -0.698,-0.703,-0.709,-0.714,-0.719,-0.725,-0.730,     &
                    -0.736,-0.741,-0.747,-0.752,-0.758,-0.764,-0.769,     &
                    -0.775,-0.781,-0.787,-0.792,-0.798,-0.804,-0.810,     &
                    -0.816,-0.822,-0.828,-0.834,-0.840,-0.846,-0.853,     &
                    -0.859,-0.865,-0.871,-0.877,-0.884,-0.890,-0.896,     &
                    -0.903,-0.909,-0.916,-0.922,-0.929,-0.935,-0.942,     &
                    -0.948,-0.955,-0.961,-0.968,-0.975,-0.981,-0.988,     &
                    -0.995,-1.001,-1.008,-1.015,-1.022,-1.029,-1.036,     &
                    -1.042,-1.049,-1.056,-1.063,-1.070,-1.077,-1.084,     &
                    -1.091,-1.098,-1.105,-1.112,-1.119,-1.127,-1.134,     &
                    -1.141,-1.148,-1.155,-1.162,-1.170,-1.177,-1.184,     &
                    -1.191,-1.199,-1.206,-1.213,-1.221,-1.228,-1.235,     &
                    -1.243/)
!
! *** cacl2
!
    bnc16m(1:561) = (/-0.099,-0.205,-0.252,-0.282,-0.304,-0.320,-0.332,   &
                    -0.342,-0.350,-0.356,-0.361,-0.364,-0.367,-0.369,     &
                    -0.370,-0.371,-0.371,-0.371,-0.370,-0.370,-0.368,     &
                    -0.367,-0.365,-0.363,-0.361,-0.358,-0.356,-0.353,     &
                    -0.350,-0.347,-0.344,-0.341,-0.337,-0.334,-0.330,     &
                    -0.327,-0.323,-0.320,-0.316,-0.312,-0.308,-0.305,     &
                    -0.301,-0.297,-0.293,-0.289,-0.285,-0.281,-0.277,     &
                    -0.273,-0.269,-0.265,-0.262,-0.258,-0.254,-0.250,     &
                    -0.246,-0.242,-0.238,-0.234,-0.230,-0.226,-0.222,     &
                    -0.218,-0.213,-0.209,-0.205,-0.201,-0.197,-0.193,     &
                    -0.189,-0.185,-0.181,-0.176,-0.172,-0.168,-0.164,     &
                    -0.160,-0.155,-0.151,-0.146,-0.142,-0.138,-0.133,     &
                    -0.129,-0.124,-0.120,-0.115,-0.110,-0.106,-0.101,     &
                    -0.096,-0.091,-0.086,-0.082,-0.077,-0.072,-0.067,     &
                    -0.062,-0.057,-0.052,-0.047,-0.042,-0.036,-0.031,     &
                    -0.026,-0.021,-0.016,-0.010,-0.005,0.000,0.005,0.011, &
                    0.016,0.021,0.027,0.032,0.037,0.043,0.048,0.054,0.059,&
                    0.064,0.070,0.075,0.081,0.086,0.092,0.097,0.102,0.108,&
                    0.113,0.119,0.124,0.129,0.135,0.140,0.146,0.151,0.156,&
                    0.162,0.167,0.173,0.178,0.183,0.189,0.194,0.199,0.205,&
                    0.210,0.215,0.221,0.226,0.231,0.237,0.242,0.247,0.252,&
                    0.258,0.263,0.268,0.273,0.279,0.284,0.289,0.294,0.299,&
                    0.305,0.310,0.315,0.320,0.325,0.331,0.336,0.341,0.346,&
                    0.351,0.356,0.361,0.366,0.371,0.377,0.382,0.387,0.392,&
                    0.397,0.402,0.407,0.412,0.417,0.422,0.427,0.432,0.437,&
                    0.442,0.447,0.452,0.457,0.462,0.467,0.472,0.476,0.481,&
                    0.486,0.491,0.496,0.501,0.506,0.511,0.515,0.520,0.525,&
                    0.530,0.535,0.539,0.544,0.549,0.554,0.559,0.563,0.568,&
                    0.573,0.578,0.582,0.587,0.592,0.596,0.601,0.606,0.610,&
                    0.615,0.620,0.624,0.629,0.634,0.638,0.643,0.647,0.652,&
                    0.657,0.661,0.666,0.670,0.675,0.679,0.684,0.688,0.693,&
                    0.697,0.702,0.706,0.711,0.715,0.720,0.724,0.729,0.733,&
                    0.737,0.742,0.746,0.751,0.755,0.759,0.764,0.768,0.773,&
                    0.777,0.781,0.786,0.790,0.794,0.798,0.803,0.807,0.811,&
                    0.816,0.820,0.824,0.828,0.833,0.837,0.841,0.845,0.849,&
                    0.854,0.858,0.862,0.866,0.870,0.874,0.879,0.883,0.887,&
                    0.891,0.895,0.899,0.903,0.907,0.912,0.916,0.920,0.924,&
                    0.928,0.932,0.936,0.940,0.944,0.948,0.952,0.956,0.960,&
                    0.964,0.968,0.972,0.976,0.980,0.984,0.988,0.992,0.995,&
                    0.999,1.003,1.007,1.011,1.015,1.019,1.023,1.026,1.030,&
                    1.034,1.038,1.042,1.046,1.049,1.053,1.057,1.061,1.065,&
                    1.068,1.072,1.076,1.080,1.083,1.087,1.091,1.095,1.098,&
                    1.102,1.106,1.109,1.113,1.117,1.120,1.124,1.128,1.131,&
                    1.135,1.139,1.142,1.146,1.150,1.153,1.157,1.160,1.164,&
                    1.168,1.171,1.175,1.178,1.182,1.185,1.189,1.192,1.196,&
                    1.200,1.203,1.207,1.210,1.214,1.217,1.221,1.224,1.227,&
                    1.231,1.234,1.238,1.241,1.245,1.248,1.252,1.255,1.258,&
                    1.262,1.265,1.269,1.272,1.275,1.279,1.282,1.285,1.321,&
                    1.354,1.386,1.417,1.448,1.478,1.508,1.537,1.565,1.593,&
                    1.621,1.648,1.674,1.700,1.726,1.751,1.776,1.800,1.824,&
                    1.848,1.871,1.893,1.916,1.938,1.959,1.980,2.001,2.022,&
                    2.042,2.061,2.081,2.100,2.119,2.137,2.156,2.173,2.191,&
                    2.208,2.225,2.242,2.259,2.275,2.291,2.307,2.322,2.337,&
                    2.352,2.367,2.382,2.396,2.410,2.424,2.438,2.451,2.464,&
                    2.477,2.490,2.502,2.515,2.527,2.539,2.551,2.562,2.574,&
                    2.585,2.596,2.607,2.618,2.628,2.639,2.649,2.659,2.669,&
                    2.679,2.688,2.698,2.707,2.716,2.725,2.734,2.743,2.751,&
                    2.760,2.768,2.776,2.784,2.792,2.800,2.808,2.815,2.823,&
                    2.830,2.837,2.844,2.851,2.858,2.865,2.871,2.878,2.884,&
                    2.890,2.896,2.902,2.908,2.914,2.920,2.925,2.931,2.936,&
                    2.942,2.947,2.952,2.957,2.962,2.967,2.972,2.976,2.981,&
                    2.985,2.990,2.994,2.998,3.003,3.007,3.011,3.014,3.018,&
                    3.022,3.026,3.029,3.033,3.036,3.040,3.043,3.046,3.049,&
                    3.052,3.055,3.058,3.061,3.064,3.066,3.069,3.072,3.074,&
                    3.077,3.079,3.081,3.083,3.086,3.088,3.090,3.092,3.094,&
                    3.096,3.097,3.099,3.101,3.102,3.104,3.105/)
!
! *** k2so4
!
    bnc17m(1:561) = (/-0.103,-0.226,-0.289,-0.334,-0.369,-0.399,-0.425,   &
                    -0.448,-0.468,-0.487,-0.505,-0.521,-0.536,-0.550,     &
                    -0.563,-0.576,-0.588,-0.599,-0.610,-0.621,-0.631,     &
                    -0.641,-0.650,-0.659,-0.668,-0.677,-0.685,-0.693,     &
                    -0.701,-0.709,-0.717,-0.724,-0.731,-0.738,-0.745,     &
                    -0.752,-0.758,-0.765,-0.771,-0.777,-0.784,-0.790,     &
                    -0.796,-0.801,-0.807,-0.813,-0.818,-0.824,-0.829,     &
                    -0.835,-0.840,-0.845,-0.850,-0.855,-0.860,-0.865,     &
                    -0.870,-0.875,-0.880,-0.884,-0.889,-0.894,-0.898,     &
                    -0.903,-0.907,-0.912,-0.916,-0.920,-0.925,-0.929,     &
                    -0.933,-0.937,-0.942,-0.946,-0.950,-0.954,-0.958,     &
                    -0.962,-0.966,-0.970,-0.974,-0.978,-0.982,-0.985,     &
                    -0.989,-0.993,-0.997,-1.001,-1.004,-1.008,-1.012,     &
                    -1.016,-1.019,-1.023,-1.027,-1.030,-1.034,-1.037,     &
                    -1.041,-1.045,-1.048,-1.052,-1.055,-1.059,-1.062,     &
                    -1.066,-1.069,-1.072,-1.076,-1.079,-1.083,-1.086,     &
                    -1.089,-1.093,-1.096,-1.099,-1.103,-1.106,-1.109,     &
                    -1.113,-1.116,-1.119,-1.122,-1.126,-1.129,-1.132,     &
                    -1.135,-1.138,-1.142,-1.145,-1.148,-1.151,-1.154,     &
                    -1.157,-1.160,-1.164,-1.167,-1.170,-1.173,-1.176,     &
                    -1.179,-1.182,-1.185,-1.188,-1.191,-1.194,-1.197,     &
                    -1.200,-1.203,-1.206,-1.209,-1.212,-1.215,-1.218,     &
                    -1.221,-1.223,-1.226,-1.229,-1.232,-1.235,-1.238,     &
                    -1.241,-1.244,-1.246,-1.249,-1.252,-1.255,-1.258,     &
                    -1.261,-1.263,-1.266,-1.269,-1.272,-1.274,-1.277,     &
                    -1.280,-1.283,-1.285,-1.288,-1.291,-1.294,-1.296,     &
                    -1.299,-1.302,-1.304,-1.307,-1.310,-1.313,-1.315,     &
                    -1.318,-1.321,-1.323,-1.326,-1.328,-1.331,-1.334,     &
                    -1.336,-1.339,-1.342,-1.344,-1.347,-1.349,-1.352,     &
                    -1.355,-1.357,-1.360,-1.362,-1.365,-1.367,-1.370,     &
                    -1.373,-1.375,-1.378,-1.380,-1.383,-1.385,-1.388,     &
                    -1.390,-1.393,-1.395,-1.398,-1.400,-1.403,-1.405,     &
                    -1.408,-1.410,-1.413,-1.415,-1.418,-1.420,-1.423,     &
                    -1.425,-1.427,-1.430,-1.432,-1.435,-1.437,-1.440,     &
                    -1.442,-1.445,-1.447,-1.449,-1.452,-1.454,-1.457,     &
                    -1.459,-1.461,-1.464,-1.466,-1.469,-1.471,-1.473,     &
                    -1.476,-1.478,-1.480,-1.483,-1.485,-1.487,-1.490,     &
                    -1.492,-1.495,-1.497,-1.499,-1.502,-1.504,-1.506,     &
                    -1.509,-1.511,-1.513,-1.515,-1.518,-1.520,-1.522,     &
                    -1.525,-1.527,-1.529,-1.532,-1.534,-1.536,-1.538,     &
                    -1.541,-1.543,-1.545,-1.548,-1.550,-1.552,-1.554,     &
                    -1.557,-1.559,-1.561,-1.563,-1.566,-1.568,-1.570,     &
                    -1.572,-1.575,-1.577,-1.579,-1.581,-1.583,-1.586,     &
                    -1.588,-1.590,-1.592,-1.595,-1.597,-1.599,-1.601,     &
                    -1.603,-1.606,-1.608,-1.610,-1.612,-1.614,-1.616,     &
                    -1.619,-1.621,-1.623,-1.625,-1.627,-1.630,-1.632,     &
                    -1.634,-1.636,-1.638,-1.640,-1.642,-1.645,-1.647,     &
                    -1.649,-1.651,-1.653,-1.655,-1.658,-1.660,-1.662,     &
                    -1.664,-1.666,-1.668,-1.670,-1.672,-1.675,-1.677,     &
                    -1.679,-1.681,-1.683,-1.685,-1.687,-1.689,-1.691,     &
                    -1.694,-1.696,-1.698,-1.700,-1.702,-1.704,-1.706,     &
                    -1.708,-1.710,-1.712,-1.714,-1.717,-1.719,-1.721,     &
                    -1.723,-1.725,-1.727,-1.729,-1.731,-1.733,-1.735,     &
                    -1.737,-1.739,-1.741,-1.743,-1.745,-1.748,-1.750,     &
                    -1.752,-1.754,-1.756,-1.758,-1.760,-1.762,-1.764,     &
                    -1.766,-1.768,-1.770,-1.772,-1.774,-1.776,-1.778,     &
                    -1.780,-1.782,-1.784,-1.786,-1.788,-1.790,-1.792,     &
                    -1.794,-1.816,-1.836,-1.855,-1.875,-1.894,-1.914,     &
                    -1.933,-1.952,-1.971,-1.989,-2.008,-2.027,-2.045,     &
                    -2.063,-2.082,-2.100,-2.118,-2.136,-2.154,-2.171,     &
                    -2.189,-2.207,-2.224,-2.242,-2.259,-2.276,-2.293,     &
                    -2.311,-2.328,-2.345,-2.362,-2.379,-2.395,-2.412,     &
                    -2.429,-2.446,-2.462,-2.479,-2.495,-2.512,-2.528,     &
                    -2.544,-2.561,-2.577,-2.593,-2.609,-2.625,-2.642,     &
                    -2.658,-2.674,-2.689,-2.705,-2.721,-2.737,-2.753,     &
                    -2.768,-2.784,-2.800,-2.815,-2.831,-2.847,-2.862,     &
                    -2.878,-2.893,-2.908,-2.924,-2.939,-2.954,-2.970,     &
                    -2.985,-3.000,-3.015,-3.031,-3.046,-3.061,-3.076,     &
                    -3.091,-3.106,-3.121,-3.136,-3.151,-3.166,-3.181,     &
                    -3.196,-3.210,-3.225,-3.240,-3.255,-3.270,-3.284,     &
                    -3.299,-3.314,-3.328,-3.343,-3.358,-3.372,-3.387,     &
                    -3.401,-3.416,-3.430,-3.445,-3.459,-3.474,-3.488,     &
                    -3.503,-3.517,-3.531,-3.546,-3.560,-3.574,-3.589,     &
                    -3.603,-3.617,-3.631,-3.646,-3.660,-3.674,-3.688,     &
                    -3.702,-3.717,-3.731,-3.745,-3.759,-3.773,-3.787,     &
                    -3.801,-3.815,-3.829,-3.843,-3.857,-3.871,-3.885,     &
                    -3.899,-3.913,-3.927,-3.941,-3.955,-3.969,-3.982,     &
                    -3.996,-4.010,-4.024,-4.038,-4.051,-4.065,-4.079,     &
                    -4.093,-4.107,-4.120,-4.134,-4.148,-4.161,-4.175,     &
                    -4.189,-4.202,-4.216,-4.230,-4.243,-4.257,-4.271,     &
                    -4.284/)
!
! *** khso4
!
    bnc18m(1:561) = (/-0.050,-0.106,-0.133,-0.152,-0.166,-0.177,-0.187,   &
                    -0.195,-0.202,-0.208,-0.214,-0.218,-0.223,-0.227,     &
                    -0.230,-0.233,-0.236,-0.238,-0.240,-0.242,-0.244,     &
                    -0.246,-0.247,-0.248,-0.249,-0.250,-0.250,-0.250,     &
                    -0.251,-0.251,-0.251,-0.251,-0.250,-0.250,-0.249,     &
                    -0.249,-0.248,-0.247,-0.246,-0.245,-0.244,-0.242,     &
                    -0.241,-0.240,-0.238,-0.236,-0.235,-0.233,-0.231,     &
                    -0.229,-0.227,-0.225,-0.223,-0.221,-0.219,-0.216,     &
                    -0.214,-0.212,-0.209,-0.207,-0.204,-0.202,-0.199,     &
                    -0.196,-0.193,-0.191,-0.188,-0.185,-0.182,-0.179,     &
                    -0.176,-0.173,-0.170,-0.167,-0.164,-0.161,-0.158,     &
                    -0.154,-0.151,-0.148,-0.144,-0.141,-0.138,-0.134,     &
                    -0.131,-0.127,-0.124,-0.120,-0.116,-0.113,-0.109,     &
                    -0.105,-0.102,-0.098,-0.094,-0.090,-0.086,-0.082,     &
                    -0.078,-0.074,-0.071,-0.067,-0.063,-0.059,-0.054,     &
                    -0.050,-0.046,-0.042,-0.038,-0.034,-0.030,-0.026,     &
                    -0.022,-0.017,-0.013,-0.009,-0.005,-0.001,0.004,0.008,&
                    0.012,0.016,0.020,0.025,0.029,0.033,0.037,0.041,0.046,&
                    0.050,0.054,0.058,0.062,0.067,0.071,0.075,0.079,0.083,&
                    0.087,0.091,0.096,0.100,0.104,0.108,0.112,0.116,0.120,&
                    0.124,0.128,0.132,0.136,0.141,0.145,0.149,0.153,0.157,&
                    0.161,0.165,0.169,0.173,0.176,0.180,0.184,0.188,0.192,&
                    0.196,0.200,0.204,0.208,0.212,0.215,0.219,0.223,0.227,&
                    0.231,0.235,0.238,0.242,0.246,0.250,0.254,0.257,0.261,&
                    0.265,0.268,0.272,0.276,0.280,0.283,0.287,0.291,0.294,&
                    0.298,0.302,0.305,0.309,0.312,0.316,0.320,0.323,0.327,&
                    0.330,0.334,0.337,0.341,0.344,0.348,0.351,0.355,0.358,&
                    0.362,0.365,0.369,0.372,0.376,0.379,0.382,0.386,0.389,&
                    0.393,0.396,0.399,0.403,0.406,0.409,0.413,0.416,0.419,&
                    0.423,0.426,0.429,0.433,0.436,0.439,0.442,0.446,0.449,&
                    0.452,0.455,0.459,0.462,0.465,0.468,0.471,0.475,0.478,&
                    0.481,0.484,0.487,0.490,0.493,0.497,0.500,0.503,0.506,&
                    0.509,0.512,0.515,0.518,0.521,0.524,0.527,0.530,0.533,&
                    0.536,0.539,0.542,0.545,0.548,0.551,0.554,0.557,0.560,&
                    0.563,0.566,0.569,0.572,0.575,0.578,0.581,0.584,0.587,&
                    0.590,0.592,0.595,0.598,0.601,0.604,0.607,0.610,0.612,&
                    0.615,0.618,0.621,0.624,0.627,0.629,0.632,0.635,0.638,&
                    0.640,0.643,0.646,0.649,0.651,0.654,0.657,0.660,0.662,&
                    0.665,0.668,0.671,0.673,0.676,0.679,0.681,0.684,0.687,&
                    0.689,0.692,0.695,0.697,0.700,0.702,0.705,0.708,0.710,&
                    0.713,0.716,0.718,0.721,0.723,0.726,0.728,0.731,0.734,&
                    0.736,0.739,0.741,0.744,0.746,0.749,0.751,0.754,0.756,&
                    0.759,0.761,0.764,0.766,0.769,0.771,0.774,0.776,0.779,&
                    0.781,0.784,0.786,0.788,0.791,0.793,0.796,0.798,0.801,&
                    0.803,0.805,0.808,0.810,0.813,0.815,0.817,0.820,0.822,&
                    0.824,0.827,0.829,0.832,0.834,0.836,0.839,0.841,0.843,&
                    0.846,0.848,0.850,0.852,0.855,0.857,0.859,0.862,0.864,&
                    0.866,0.868,0.871,0.873,0.875,0.878,0.880,0.882,0.884,&
                    0.887,0.910,0.932,0.953,0.974,0.995,1.015,1.034,1.054,&
                    1.072,1.091,1.109,1.127,1.145,1.162,1.179,1.196,1.213,&
                    1.229,1.245,1.260,1.276,1.291,1.306,1.320,1.335,1.349,&
                    1.363,1.377,1.390,1.404,1.417,1.430,1.442,1.455,1.467,&
                    1.479,1.491,1.503,1.515,1.526,1.538,1.549,1.560,1.570,&
                    1.581,1.592,1.602,1.612,1.622,1.632,1.642,1.652,1.661,&
                    1.671,1.680,1.689,1.698,1.707,1.716,1.724,1.733,1.741,&
                    1.750,1.758,1.766,1.774,1.782,1.789,1.797,1.805,1.812,&
                    1.820,1.827,1.834,1.841,1.848,1.855,1.862,1.868,1.875,&
                    1.882,1.888,1.894,1.901,1.907,1.913,1.919,1.925,1.931,&
                    1.937,1.943,1.948,1.954,1.959,1.965,1.970,1.976,1.981,&
                    1.986,1.991,1.996,2.001,2.006,2.011,2.016,2.020,2.025,&
                    2.030,2.034,2.039,2.043,2.048,2.052,2.056,2.060,2.064,&
                    2.069,2.073,2.077,2.080,2.084,2.088,2.092,2.096,2.099,&
                    2.103,2.107,2.110,2.114,2.117,2.120,2.124,2.127,2.130,&
                    2.133,2.137,2.140,2.143,2.146,2.149,2.152,2.155,2.158,&
                    2.160,2.163,2.166,2.169,2.171,2.174,2.176,2.179,2.181,&
                    2.184,2.186,2.189,2.191,2.193,2.196,2.198,2.200,      &
                    2.202/)
!
! *** kno3
!
    bnc19m(1:561) = (/-0.053,-0.124,-0.164,-0.194,-0.219,-0.242,-0.262,   &
                    -0.281,-0.298,-0.314,-0.330,-0.345,-0.359,-0.372,     &
                    -0.385,-0.398,-0.411,-0.423,-0.434,-0.446,-0.457,     &
                    -0.468,-0.478,-0.489,-0.499,-0.509,-0.519,-0.528,     &
                    -0.538,-0.547,-0.556,-0.565,-0.574,-0.583,-0.592,     &
                    -0.600,-0.608,-0.617,-0.625,-0.633,-0.641,-0.648,     &
                    -0.656,-0.664,-0.671,-0.678,-0.686,-0.693,-0.700,     &
                    -0.707,-0.714,-0.721,-0.727,-0.734,-0.741,-0.747,     &
                    -0.754,-0.760,-0.766,-0.773,-0.779,-0.785,-0.791,     &
                    -0.797,-0.803,-0.809,-0.815,-0.821,-0.827,-0.832,     &
                    -0.838,-0.844,-0.849,-0.855,-0.861,-0.866,-0.872,     &
                    -0.877,-0.883,-0.888,-0.894,-0.899,-0.904,-0.910,     &
                    -0.915,-0.920,-0.926,-0.931,-0.936,-0.942,-0.947,     &
                    -0.952,-0.957,-0.963,-0.968,-0.973,-0.978,-0.983,     &
                    -0.988,-0.994,-0.999,-1.004,-1.009,-1.014,-1.019,     &
                    -1.024,-1.029,-1.034,-1.039,-1.044,-1.049,-1.054,     &
                    -1.059,-1.064,-1.069,-1.074,-1.079,-1.083,-1.088,     &
                    -1.093,-1.098,-1.103,-1.107,-1.112,-1.117,-1.122,     &
                    -1.126,-1.131,-1.136,-1.140,-1.145,-1.149,-1.154,     &
                    -1.158,-1.163,-1.168,-1.172,-1.176,-1.181,-1.185,     &
                    -1.190,-1.194,-1.199,-1.203,-1.207,-1.212,-1.216,     &
                    -1.220,-1.224,-1.229,-1.233,-1.237,-1.241,-1.245,     &
                    -1.250,-1.254,-1.258,-1.262,-1.266,-1.270,-1.274,     &
                    -1.278,-1.282,-1.286,-1.290,-1.294,-1.298,-1.302,     &
                    -1.306,-1.310,-1.314,-1.318,-1.321,-1.325,-1.329,     &
                    -1.333,-1.337,-1.341,-1.344,-1.348,-1.352,-1.355,     &
                    -1.359,-1.363,-1.367,-1.370,-1.374,-1.378,-1.381,     &
                    -1.385,-1.388,-1.392,-1.395,-1.399,-1.403,-1.406,     &
                    -1.410,-1.413,-1.417,-1.420,-1.423,-1.427,-1.430,     &
                    -1.434,-1.437,-1.441,-1.444,-1.447,-1.451,-1.454,     &
                    -1.457,-1.461,-1.464,-1.467,-1.470,-1.474,-1.477,     &
                    -1.480,-1.483,-1.487,-1.490,-1.493,-1.496,-1.499,     &
                    -1.503,-1.506,-1.509,-1.512,-1.515,-1.518,-1.521,     &
                    -1.524,-1.528,-1.531,-1.534,-1.537,-1.540,-1.543,     &
                    -1.546,-1.549,-1.552,-1.555,-1.558,-1.561,-1.564,     &
                    -1.566,-1.569,-1.572,-1.575,-1.578,-1.581,-1.584,     &
                    -1.587,-1.590,-1.592,-1.595,-1.598,-1.601,-1.604,     &
                    -1.607,-1.609,-1.612,-1.615,-1.618,-1.620,-1.623,     &
                    -1.626,-1.629,-1.631,-1.634,-1.637,-1.639,-1.642,     &
                    -1.645,-1.647,-1.650,-1.653,-1.655,-1.658,-1.661,     &
                    -1.663,-1.666,-1.668,-1.671,-1.674,-1.676,-1.679,     &
                    -1.681,-1.684,-1.686,-1.689,-1.691,-1.694,-1.696,     &
                    -1.699,-1.701,-1.704,-1.706,-1.709,-1.711,-1.714,     &
                    -1.716,-1.719,-1.721,-1.724,-1.726,-1.728,-1.731,     &
                    -1.733,-1.736,-1.738,-1.740,-1.743,-1.745,-1.747,     &
                    -1.750,-1.752,-1.754,-1.757,-1.759,-1.761,-1.764,     &
                    -1.766,-1.768,-1.771,-1.773,-1.775,-1.777,-1.780,     &
                    -1.782,-1.784,-1.786,-1.789,-1.791,-1.793,-1.795,     &
                    -1.797,-1.800,-1.802,-1.804,-1.806,-1.808,-1.811,     &
                    -1.813,-1.815,-1.817,-1.819,-1.821,-1.823,-1.826,     &
                    -1.828,-1.830,-1.832,-1.834,-1.836,-1.838,-1.840,     &
                    -1.842,-1.844,-1.847,-1.849,-1.851,-1.853,-1.855,     &
                    -1.857,-1.859,-1.861,-1.863,-1.865,-1.867,-1.869,     &
                    -1.871,-1.873,-1.875,-1.877,-1.879,-1.881,-1.883,     &
                    -1.885,-1.887,-1.889,-1.891,-1.893,-1.895,-1.897,     &
                    -1.898,-1.900,-1.902,-1.904,-1.906,-1.908,-1.910,     &
                    -1.912,-1.914,-1.916,-1.917,-1.919,-1.921,-1.923,     &
                    -1.925,-1.945,-1.963,-1.980,-1.997,-2.014,-2.030,     &
                    -2.046,-2.062,-2.077,-2.092,-2.107,-2.122,-2.136,     &
                    -2.150,-2.163,-2.177,-2.190,-2.203,-2.216,-2.228,     &
                    -2.241,-2.253,-2.265,-2.277,-2.289,-2.300,-2.311,     &
                    -2.323,-2.334,-2.345,-2.356,-2.366,-2.377,-2.387,     &
                    -2.398,-2.408,-2.418,-2.428,-2.438,-2.448,-2.457,     &
                    -2.467,-2.477,-2.486,-2.496,-2.505,-2.514,-2.523,     &
                    -2.532,-2.541,-2.550,-2.559,-2.568,-2.577,-2.586,     &
                    -2.594,-2.603,-2.611,-2.620,-2.628,-2.637,-2.645,     &
                    -2.653,-2.662,-2.670,-2.678,-2.686,-2.694,-2.702,     &
                    -2.710,-2.718,-2.726,-2.734,-2.742,-2.750,-2.758,     &
                    -2.765,-2.773,-2.781,-2.789,-2.796,-2.804,-2.811,     &
                    -2.819,-2.827,-2.834,-2.842,-2.849,-2.857,-2.864,     &
                    -2.871,-2.879,-2.886,-2.893,-2.901,-2.908,-2.915,     &
                    -2.923,-2.930,-2.937,-2.944,-2.951,-2.959,-2.966,     &
                    -2.973,-2.980,-2.987,-2.994,-3.001,-3.008,-3.015,     &
                    -3.022,-3.029,-3.036,-3.043,-3.050,-3.057,-3.064,     &
                    -3.071,-3.078,-3.085,-3.092,-3.099,-3.106,-3.113,     &
                    -3.119,-3.126,-3.133,-3.140,-3.147,-3.154,-3.160,     &
                    -3.167,-3.174,-3.181,-3.187,-3.194,-3.201,-3.208,     &
                    -3.214,-3.221,-3.228,-3.234,-3.241,-3.248,-3.254,     &
                    -3.261,-3.268,-3.274,-3.281,-3.287,-3.294,-3.301,     &
                    -3.307,-3.314,-3.320,-3.327,-3.334,-3.340,-3.347,     &
                    -3.353/)
!
! *** kcl
!
    bnc20m(1:561) = (/-0.051,-0.108,-0.135,-0.154,-0.169,-0.180,-0.190,   &
                    -0.198,-0.205,-0.211,-0.217,-0.222,-0.226,-0.231,     &
                    -0.234,-0.238,-0.241,-0.244,-0.246,-0.249,-0.251,     &
                    -0.253,-0.255,-0.257,-0.259,-0.261,-0.262,-0.264,     &
                    -0.265,-0.267,-0.268,-0.269,-0.270,-0.271,-0.272,     &
                    -0.273,-0.274,-0.275,-0.276,-0.277,-0.278,-0.278,     &
                    -0.279,-0.280,-0.280,-0.281,-0.282,-0.282,-0.283,     &
                    -0.283,-0.284,-0.285,-0.285,-0.286,-0.286,-0.287,     &
                    -0.287,-0.287,-0.288,-0.288,-0.289,-0.289,-0.289,     &
                    -0.290,-0.290,-0.291,-0.291,-0.291,-0.292,-0.292,     &
                    -0.292,-0.292,-0.293,-0.293,-0.293,-0.293,-0.294,     &
                    -0.294,-0.294,-0.294,-0.294,-0.295,-0.295,-0.295,     &
                    -0.295,-0.295,-0.295,-0.295,-0.295,-0.295,-0.295,     &
                    -0.296,-0.296,-0.296,-0.296,-0.296,-0.296,-0.296,     &
                    -0.296,-0.295,-0.295,-0.295,-0.295,-0.295,-0.295,     &
                    -0.295,-0.295,-0.295,-0.295,-0.295,-0.294,-0.294,     &
                    -0.294,-0.294,-0.294,-0.294,-0.294,-0.293,-0.293,     &
                    -0.293,-0.293,-0.293,-0.293,-0.292,-0.292,-0.292,     &
                    -0.292,-0.292,-0.291,-0.291,-0.291,-0.291,-0.291,     &
                    -0.290,-0.290,-0.290,-0.290,-0.290,-0.289,-0.289,     &
                    -0.289,-0.289,-0.289,-0.288,-0.288,-0.288,-0.288,     &
                    -0.287,-0.287,-0.287,-0.287,-0.287,-0.286,-0.286,     &
                    -0.286,-0.286,-0.285,-0.285,-0.285,-0.285,-0.285,     &
                    -0.284,-0.284,-0.284,-0.284,-0.283,-0.283,-0.283,     &
                    -0.283,-0.283,-0.282,-0.282,-0.282,-0.282,-0.281,     &
                    -0.281,-0.281,-0.281,-0.281,-0.280,-0.280,-0.280,     &
                    -0.280,-0.279,-0.279,-0.279,-0.279,-0.279,-0.278,     &
                    -0.278,-0.278,-0.278,-0.278,-0.277,-0.277,-0.277,     &
                    -0.277,-0.276,-0.276,-0.276,-0.276,-0.276,-0.275,     &
                    -0.275,-0.275,-0.275,-0.275,-0.274,-0.274,-0.274,     &
                    -0.274,-0.274,-0.273,-0.273,-0.273,-0.273,-0.273,     &
                    -0.272,-0.272,-0.272,-0.272,-0.272,-0.271,-0.271,     &
                    -0.271,-0.271,-0.271,-0.270,-0.270,-0.270,-0.270,     &
                    -0.270,-0.270,-0.269,-0.269,-0.269,-0.269,-0.269,     &
                    -0.268,-0.268,-0.268,-0.268,-0.268,-0.268,-0.267,     &
                    -0.267,-0.267,-0.267,-0.267,-0.266,-0.266,-0.266,     &
                    -0.266,-0.266,-0.266,-0.265,-0.265,-0.265,-0.265,     &
                    -0.265,-0.265,-0.264,-0.264,-0.264,-0.264,-0.264,     &
                    -0.264,-0.263,-0.263,-0.263,-0.263,-0.263,-0.263,     &
                    -0.263,-0.262,-0.262,-0.262,-0.262,-0.262,-0.262,     &
                    -0.261,-0.261,-0.261,-0.261,-0.261,-0.261,-0.261,     &
                    -0.260,-0.260,-0.260,-0.260,-0.260,-0.260,-0.260,     &
                    -0.260,-0.259,-0.259,-0.259,-0.259,-0.259,-0.259,     &
                    -0.259,-0.258,-0.258,-0.258,-0.258,-0.258,-0.258,     &
                    -0.258,-0.258,-0.257,-0.257,-0.257,-0.257,-0.257,     &
                    -0.257,-0.257,-0.257,-0.257,-0.256,-0.256,-0.256,     &
                    -0.256,-0.256,-0.256,-0.256,-0.256,-0.256,-0.255,     &
                    -0.255,-0.255,-0.255,-0.255,-0.255,-0.255,-0.255,     &
                    -0.255,-0.255,-0.254,-0.254,-0.254,-0.254,-0.254,     &
                    -0.254,-0.254,-0.254,-0.254,-0.254,-0.253,-0.253,     &
                    -0.253,-0.253,-0.253,-0.253,-0.253,-0.253,-0.253,     &
                    -0.253,-0.253,-0.253,-0.252,-0.252,-0.252,-0.252,     &
                    -0.252,-0.252,-0.252,-0.252,-0.252,-0.252,-0.252,     &
                    -0.252,-0.252,-0.252,-0.251,-0.251,-0.251,-0.251,     &
                    -0.251,-0.251,-0.251,-0.251,-0.251,-0.251,-0.251,     &
                    -0.251,-0.251,-0.251,-0.251,-0.251,-0.250,-0.250,     &
                    -0.250,-0.250,-0.250,-0.250,-0.250,-0.250,-0.250,     &
                    -0.250,-0.250,-0.249,-0.249,-0.249,-0.248,-0.248,     &
                    -0.248,-0.248,-0.249,-0.249,-0.249,-0.249,-0.250,     &
                    -0.250,-0.251,-0.251,-0.252,-0.252,-0.253,-0.254,     &
                    -0.255,-0.255,-0.256,-0.257,-0.258,-0.259,-0.261,     &
                    -0.262,-0.263,-0.264,-0.265,-0.267,-0.268,-0.269,     &
                    -0.271,-0.272,-0.274,-0.276,-0.277,-0.279,-0.280,     &
                    -0.282,-0.284,-0.286,-0.287,-0.289,-0.291,-0.293,     &
                    -0.295,-0.297,-0.299,-0.301,-0.303,-0.305,-0.307,     &
                    -0.309,-0.312,-0.314,-0.316,-0.318,-0.321,-0.323,     &
                    -0.325,-0.328,-0.330,-0.332,-0.335,-0.337,-0.340,     &
                    -0.342,-0.345,-0.347,-0.350,-0.352,-0.355,-0.358,     &
                    -0.360,-0.363,-0.366,-0.368,-0.371,-0.374,-0.377,     &
                    -0.379,-0.382,-0.385,-0.388,-0.391,-0.394,-0.396,     &
                    -0.399,-0.402,-0.405,-0.408,-0.411,-0.414,-0.417,     &
                    -0.420,-0.423,-0.426,-0.429,-0.432,-0.436,-0.439,     &
                    -0.442,-0.445,-0.448,-0.451,-0.454,-0.458,-0.461,     &
                    -0.464,-0.467,-0.471,-0.474,-0.477,-0.480,-0.484,     &
                    -0.487,-0.490,-0.494,-0.497,-0.500,-0.504,-0.507,     &
                    -0.511,-0.514,-0.517,-0.521,-0.524,-0.528,-0.531,     &
                    -0.535,-0.538,-0.542,-0.545,-0.549,-0.552,-0.556,     &
                    -0.559,-0.563,-0.566,-0.570,-0.574,-0.577,-0.581,     &
                    -0.584,-0.588,-0.592,-0.595,-0.599,-0.603,-0.606,     &
                    -0.610,-0.614,-0.617,-0.621,-0.625,-0.628,-0.632,     &
                    -0.636/)
!
! *** mgso4
!
    bnc21m(1:561) = (/-0.205,-0.445,-0.564,-0.649,-0.715,-0.770,-0.817,   &
                    -0.858,-0.895,-0.928,-0.958,-0.986,-1.012,-1.036,     &
                    -1.058,-1.079,-1.099,-1.118,-1.136,-1.153,-1.170,     &
                    -1.185,-1.200,-1.215,-1.229,-1.242,-1.255,-1.268,     &
                    -1.280,-1.292,-1.303,-1.315,-1.325,-1.336,-1.346,     &
                    -1.357,-1.366,-1.376,-1.386,-1.395,-1.404,-1.413,     &
                    -1.421,-1.430,-1.438,-1.447,-1.455,-1.463,-1.471,     &
                    -1.478,-1.486,-1.494,-1.501,-1.508,-1.515,-1.523,     &
                    -1.530,-1.536,-1.543,-1.550,-1.557,-1.563,-1.570,     &
                    -1.576,-1.583,-1.589,-1.595,-1.601,-1.607,-1.613,     &
                    -1.619,-1.625,-1.631,-1.637,-1.643,-1.648,-1.654,     &
                    -1.659,-1.665,-1.670,-1.676,-1.681,-1.687,-1.692,     &
                    -1.697,-1.702,-1.707,-1.713,-1.718,-1.723,-1.728,     &
                    -1.733,-1.738,-1.742,-1.747,-1.752,-1.757,-1.762,     &
                    -1.766,-1.771,-1.776,-1.780,-1.785,-1.789,-1.794,     &
                    -1.798,-1.803,-1.807,-1.812,-1.816,-1.820,-1.825,     &
                    -1.829,-1.833,-1.838,-1.842,-1.846,-1.850,-1.854,     &
                    -1.858,-1.863,-1.867,-1.871,-1.875,-1.879,-1.883,     &
                    -1.887,-1.891,-1.895,-1.899,-1.903,-1.907,-1.911,     &
                    -1.915,-1.919,-1.922,-1.926,-1.930,-1.934,-1.938,     &
                    -1.942,-1.945,-1.949,-1.953,-1.957,-1.960,-1.964,     &
                    -1.968,-1.972,-1.975,-1.979,-1.983,-1.986,-1.990,     &
                    -1.994,-1.997,-2.001,-2.004,-2.008,-2.012,-2.015,     &
                    -2.019,-2.022,-2.026,-2.029,-2.033,-2.036,-2.040,     &
                    -2.043,-2.047,-2.050,-2.054,-2.057,-2.061,-2.064,     &
                    -2.068,-2.071,-2.074,-2.078,-2.081,-2.085,-2.088,     &
                    -2.091,-2.095,-2.098,-2.102,-2.105,-2.108,-2.112,     &
                    -2.115,-2.118,-2.122,-2.125,-2.128,-2.132,-2.135,     &
                    -2.138,-2.141,-2.145,-2.148,-2.151,-2.154,-2.158,     &
                    -2.161,-2.164,-2.167,-2.171,-2.174,-2.177,-2.180,     &
                    -2.184,-2.187,-2.190,-2.193,-2.196,-2.200,-2.203,     &
                    -2.206,-2.209,-2.212,-2.215,-2.219,-2.222,-2.225,     &
                    -2.228,-2.231,-2.234,-2.237,-2.241,-2.244,-2.247,     &
                    -2.250,-2.253,-2.256,-2.259,-2.262,-2.265,-2.269,     &
                    -2.272,-2.275,-2.278,-2.281,-2.284,-2.287,-2.290,     &
                    -2.293,-2.296,-2.299,-2.302,-2.305,-2.308,-2.311,     &
                    -2.314,-2.317,-2.320,-2.324,-2.327,-2.330,-2.333,     &
                    -2.336,-2.339,-2.342,-2.345,-2.348,-2.351,-2.354,     &
                    -2.357,-2.360,-2.363,-2.366,-2.369,-2.371,-2.374,     &
                    -2.377,-2.380,-2.383,-2.386,-2.389,-2.392,-2.395,     &
                    -2.398,-2.401,-2.404,-2.407,-2.410,-2.413,-2.416,     &
                    -2.419,-2.422,-2.425,-2.427,-2.430,-2.433,-2.436,     &
                    -2.439,-2.442,-2.445,-2.448,-2.451,-2.454,-2.457,     &
                    -2.460,-2.462,-2.465,-2.468,-2.471,-2.474,-2.477,     &
                    -2.480,-2.483,-2.485,-2.488,-2.491,-2.494,-2.497,     &
                    -2.500,-2.503,-2.506,-2.508,-2.511,-2.514,-2.517,     &
                    -2.520,-2.523,-2.526,-2.528,-2.531,-2.534,-2.537,     &
                    -2.540,-2.543,-2.546,-2.548,-2.551,-2.554,-2.557,     &
                    -2.560,-2.563,-2.565,-2.568,-2.571,-2.574,-2.577,     &
                    -2.579,-2.582,-2.585,-2.588,-2.591,-2.594,-2.596,     &
                    -2.599,-2.602,-2.605,-2.608,-2.610,-2.613,-2.616,     &
                    -2.619,-2.622,-2.624,-2.627,-2.630,-2.633,-2.635,     &
                    -2.638,-2.641,-2.644,-2.647,-2.649,-2.652,-2.655,     &
                    -2.658,-2.661,-2.663,-2.666,-2.669,-2.672,-2.674,     &
                    -2.677,-2.680,-2.683,-2.685,-2.688,-2.691,-2.694,     &
                    -2.696,-2.699,-2.702,-2.705,-2.707,-2.710,-2.713,     &
                    -2.716,-2.718,-2.721,-2.724,-2.727,-2.729,-2.732,     &
                    -2.735,-2.765,-2.792,-2.819,-2.846,-2.873,-2.900,     &
                    -2.927,-2.953,-2.980,-3.006,-3.033,-3.059,-3.086,     &
                    -3.112,-3.138,-3.164,-3.191,-3.217,-3.243,-3.269,     &
                    -3.295,-3.321,-3.346,-3.372,-3.398,-3.424,-3.449,     &
                    -3.475,-3.501,-3.526,-3.552,-3.578,-3.603,-3.629,     &
                    -3.654,-3.679,-3.705,-3.730,-3.755,-3.781,-3.806,     &
                    -3.831,-3.857,-3.882,-3.907,-3.932,-3.957,-3.982,     &
                    -4.008,-4.033,-4.058,-4.083,-4.108,-4.133,-4.158,     &
                    -4.183,-4.208,-4.233,-4.258,-4.282,-4.307,-4.332,     &
                    -4.357,-4.382,-4.407,-4.431,-4.456,-4.481,-4.506,     &
                    -4.530,-4.555,-4.580,-4.605,-4.629,-4.654,-4.679,     &
                    -4.703,-4.728,-4.752,-4.777,-4.802,-4.826,-4.851,     &
                    -4.875,-4.900,-4.924,-4.949,-4.973,-4.998,-5.022,     &
                    -5.047,-5.071,-5.096,-5.120,-5.145,-5.169,-5.193,     &
                    -5.218,-5.242,-5.267,-5.291,-5.315,-5.340,-5.364,     &
                    -5.388,-5.413,-5.437,-5.461,-5.485,-5.510,-5.534,     &
                    -5.558,-5.582,-5.607,-5.631,-5.655,-5.679,-5.703,     &
                    -5.728,-5.752,-5.776,-5.800,-5.824,-5.848,-5.873,     &
                    -5.897,-5.921,-5.945,-5.969,-5.993,-6.017,-6.041,     &
                    -6.065,-6.089,-6.113,-6.137,-6.161,-6.186,-6.210,     &
                    -6.234,-6.258,-6.282,-6.306,-6.329,-6.353,-6.377,     &
                    -6.401,-6.425,-6.449,-6.473,-6.497,-6.521,-6.545,     &
                    -6.569,-6.593,-6.617,-6.641,-6.664,-6.688,-6.712,     &
                    -6.736/)
!
! *** mgno32
!
    bnc22m(1:561) = (/-0.099,-0.205,-0.253,-0.283,-0.305,-0.322,-0.335,   &
                    -0.345,-0.353,-0.359,-0.364,-0.368,-0.371,-0.374,     &
                    -0.375,-0.376,-0.377,-0.377,-0.376,-0.376,-0.375,     &
                    -0.373,-0.372,-0.370,-0.368,-0.366,-0.364,-0.361,     &
                    -0.359,-0.356,-0.353,-0.350,-0.347,-0.344,-0.341,     &
                    -0.338,-0.334,-0.331,-0.328,-0.324,-0.321,-0.317,     &
                    -0.314,-0.310,-0.306,-0.303,-0.299,-0.295,-0.292,     &
                    -0.288,-0.284,-0.281,-0.277,-0.273,-0.270,-0.266,     &
                    -0.262,-0.258,-0.255,-0.251,-0.247,-0.243,-0.240,     &
                    -0.236,-0.232,-0.228,-0.225,-0.221,-0.217,-0.213,     &
                    -0.209,-0.205,-0.201,-0.197,-0.194,-0.190,-0.186,     &
                    -0.182,-0.177,-0.173,-0.169,-0.165,-0.161,-0.157,     &
                    -0.153,-0.148,-0.144,-0.140,-0.135,-0.131,-0.126,     &
                    -0.122,-0.117,-0.113,-0.108,-0.104,-0.099,-0.094,     &
                    -0.090,-0.085,-0.080,-0.075,-0.070,-0.066,-0.061,     &
                    -0.056,-0.051,-0.046,-0.041,-0.036,-0.031,-0.026,     &
                    -0.021,-0.016,-0.011,-0.006,-0.001,0.004,0.010,0.015, &
                    0.020,0.025,0.030,0.035,0.040,0.045,0.050,0.056,0.061,&
                    0.066,0.071,0.076,0.081,0.086,0.092,0.097,0.102,0.107,&
                    0.112,0.117,0.122,0.127,0.132,0.137,0.143,0.148,0.153,&
                    0.158,0.163,0.168,0.173,0.178,0.183,0.188,0.193,0.198,&
                    0.203,0.208,0.213,0.218,0.223,0.228,0.233,0.238,0.243,&
                    0.248,0.253,0.258,0.262,0.267,0.272,0.277,0.282,0.287,&
                    0.292,0.297,0.302,0.306,0.311,0.316,0.321,0.326,0.331,&
                    0.335,0.340,0.345,0.350,0.354,0.359,0.364,0.369,0.373,&
                    0.378,0.383,0.388,0.392,0.397,0.402,0.406,0.411,0.416,&
                    0.420,0.425,0.430,0.434,0.439,0.444,0.448,0.453,0.457,&
                    0.462,0.466,0.471,0.476,0.480,0.485,0.489,0.494,0.498,&
                    0.503,0.507,0.512,0.516,0.521,0.525,0.530,0.534,0.538,&
                    0.543,0.547,0.552,0.556,0.561,0.565,0.569,0.574,0.578,&
                    0.582,0.587,0.591,0.595,0.600,0.604,0.608,0.613,0.617,&
                    0.621,0.625,0.630,0.634,0.638,0.642,0.647,0.651,0.655,&
                    0.659,0.664,0.668,0.672,0.676,0.680,0.684,0.689,0.693,&
                    0.697,0.701,0.705,0.709,0.713,0.717,0.721,0.726,0.730,&
                    0.734,0.738,0.742,0.746,0.750,0.754,0.758,0.762,0.766,&
                    0.770,0.774,0.778,0.782,0.786,0.790,0.794,0.798,0.801,&
                    0.805,0.809,0.813,0.817,0.821,0.825,0.829,0.833,0.836,&
                    0.840,0.844,0.848,0.852,0.856,0.859,0.863,0.867,0.871,&
                    0.875,0.878,0.882,0.886,0.890,0.893,0.897,0.901,0.905,&
                    0.908,0.912,0.916,0.919,0.923,0.927,0.931,0.934,0.938,&
                    0.941,0.945,0.949,0.952,0.956,0.960,0.963,0.967,0.970,&
                    0.974,0.978,0.981,0.985,0.988,0.992,0.995,0.999,1.002,&
                    1.006,1.010,1.013,1.017,1.020,1.024,1.027,1.031,1.034,&
                    1.037,1.041,1.044,1.048,1.051,1.055,1.058,1.061,1.065,&
                    1.068,1.072,1.075,1.078,1.082,1.085,1.089,1.092,1.095,&
                    1.099,1.102,1.105,1.109,1.112,1.115,1.119,1.122,1.125,&
                    1.128,1.132,1.135,1.138,1.142,1.145,1.148,1.151,1.155,&
                    1.158,1.161,1.164,1.167,1.171,1.174,1.177,1.180,1.183,&
                    1.187,1.220,1.251,1.281,1.311,1.340,1.369,1.397,1.424,&
                    1.451,1.477,1.503,1.529,1.554,1.579,1.603,1.627,1.650,&
                    1.673,1.695,1.717,1.739,1.760,1.781,1.802,1.822,1.842,&
                    1.862,1.881,1.900,1.918,1.937,1.955,1.972,1.990,2.007,&
                    2.024,2.040,2.056,2.072,2.088,2.103,2.119,2.134,2.148,&
                    2.163,2.177,2.191,2.205,2.218,2.231,2.245,2.257,2.270,&
                    2.283,2.295,2.307,2.319,2.330,2.342,2.353,2.364,2.375,&
                    2.386,2.397,2.407,2.417,2.427,2.437,2.447,2.456,2.466,&
                    2.475,2.484,2.493,2.502,2.511,2.519,2.528,2.536,2.544,&
                    2.552,2.560,2.567,2.575,2.582,2.590,2.597,2.604,2.611,&
                    2.618,2.624,2.631,2.637,2.644,2.650,2.656,2.662,2.668,&
                    2.674,2.680,2.685,2.691,2.696,2.701,2.706,2.712,2.717,&
                    2.721,2.726,2.731,2.735,2.740,2.744,2.749,2.753,2.757,&
                    2.761,2.765,2.769,2.773,2.777,2.780,2.784,2.787,2.791,&
                    2.794,2.797,2.801,2.804,2.807,2.810,2.812,2.815,2.818,&
                    2.821,2.823,2.826,2.828,2.831,2.833,2.835,2.837,2.839,&
                    2.841,2.843,2.845,2.847,2.849,2.851,2.852,2.854,2.856,&
                    2.857,2.859,2.860,2.861,2.862,2.864,2.865,2.866,      &
                    2.867/)
!
! *** mgcl2
!
    bnc23m(1:561) = (/-0.098,-0.202,-0.247,-0.275,-0.294,-0.308,-0.319,   &
                    -0.327,-0.333,-0.337,-0.340,-0.342,-0.343,-0.343,     &
                    -0.343,-0.341,-0.340,-0.338,-0.336,-0.333,-0.330,     &
                    -0.326,-0.323,-0.319,-0.315,-0.311,-0.306,-0.302,     &
                    -0.297,-0.293,-0.288,-0.283,-0.278,-0.273,-0.267,     &
                    -0.262,-0.257,-0.251,-0.246,-0.241,-0.235,-0.230,     &
                    -0.224,-0.218,-0.213,-0.207,-0.202,-0.196,-0.190,     &
                    -0.185,-0.179,-0.174,-0.168,-0.162,-0.157,-0.151,     &
                    -0.145,-0.140,-0.134,-0.128,-0.123,-0.117,-0.111,     &
                    -0.106,-0.100,-0.094,-0.089,-0.083,-0.077,-0.071,     &
                    -0.066,-0.060,-0.054,-0.048,-0.042,-0.037,-0.031,     &
                    -0.025,-0.019,-0.013,-0.007,-0.001,0.006,0.012,0.018, &
                    0.024,0.030,0.037,0.043,0.049,0.056,0.062,0.069,0.075,&
                    0.082,0.089,0.095,0.102,0.109,0.116,0.122,0.129,0.136,&
                    0.143,0.150,0.157,0.164,0.171,0.178,0.185,0.192,0.199,&
                    0.207,0.214,0.221,0.228,0.235,0.242,0.250,0.257,0.264,&
                    0.271,0.278,0.286,0.293,0.300,0.307,0.315,0.322,0.329,&
                    0.336,0.344,0.351,0.358,0.365,0.372,0.380,0.387,0.394,&
                    0.401,0.408,0.416,0.423,0.430,0.437,0.444,0.451,0.458,&
                    0.466,0.473,0.480,0.487,0.494,0.501,0.508,0.515,0.522,&
                    0.529,0.536,0.543,0.550,0.557,0.564,0.571,0.578,0.585,&
                    0.592,0.599,0.606,0.613,0.620,0.626,0.633,0.640,0.647,&
                    0.654,0.661,0.667,0.674,0.681,0.688,0.695,0.701,0.708,&
                    0.715,0.722,0.728,0.735,0.742,0.748,0.755,0.762,0.768,&
                    0.775,0.782,0.788,0.795,0.801,0.808,0.815,0.821,0.828,&
                    0.834,0.841,0.847,0.854,0.860,0.867,0.873,0.879,0.886,&
                    0.892,0.899,0.905,0.912,0.918,0.924,0.931,0.937,0.943,&
                    0.950,0.956,0.962,0.968,0.975,0.981,0.987,0.993,1.000,&
                    1.006,1.012,1.018,1.024,1.031,1.037,1.043,1.049,1.055,&
                    1.061,1.067,1.073,1.080,1.086,1.092,1.098,1.104,1.110,&
                    1.116,1.122,1.128,1.134,1.140,1.146,1.151,1.157,1.163,&
                    1.169,1.175,1.181,1.187,1.193,1.199,1.204,1.210,1.216,&
                    1.222,1.228,1.233,1.239,1.245,1.251,1.256,1.262,1.268,&
                    1.273,1.279,1.285,1.291,1.296,1.302,1.307,1.313,1.319,&
                    1.324,1.330,1.335,1.341,1.347,1.352,1.358,1.363,1.369,&
                    1.374,1.380,1.385,1.391,1.396,1.402,1.407,1.412,1.418,&
                    1.423,1.429,1.434,1.439,1.445,1.450,1.455,1.461,1.466,&
                    1.471,1.477,1.482,1.487,1.493,1.498,1.503,1.508,1.514,&
                    1.519,1.524,1.529,1.534,1.540,1.545,1.550,1.555,1.560,&
                    1.565,1.570,1.576,1.581,1.586,1.591,1.596,1.601,1.606,&
                    1.611,1.616,1.621,1.626,1.631,1.636,1.641,1.646,1.651,&
                    1.656,1.661,1.666,1.671,1.676,1.681,1.686,1.691,1.695,&
                    1.700,1.705,1.710,1.715,1.720,1.725,1.729,1.734,1.739,&
                    1.744,1.749,1.753,1.758,1.763,1.768,1.772,1.777,1.782,&
                    1.786,1.791,1.796,1.801,1.805,1.810,1.815,1.819,1.824,&
                    1.829,1.833,1.838,1.842,1.847,1.852,1.856,1.861,1.865,&
                    1.870,1.874,1.879,1.883,1.888,1.892,1.897,1.902,1.906,&
                    1.954,1.998,2.040,2.082,2.124,2.164,2.204,2.243,2.282,&
                    2.319,2.356,2.393,2.429,2.464,2.499,2.533,2.567,2.600,&
                    2.632,2.664,2.695,2.726,2.757,2.787,2.816,2.845,2.874,&
                    2.902,2.930,2.957,2.984,3.010,3.036,3.062,3.087,3.112,&
                    3.137,3.161,3.185,3.208,3.232,3.254,3.277,3.299,3.321,&
                    3.342,3.364,3.385,3.405,3.426,3.446,3.466,3.485,3.504,&
                    3.523,3.542,3.561,3.579,3.597,3.615,3.632,3.649,3.666,&
                    3.683,3.700,3.716,3.732,3.748,3.764,3.779,3.795,3.810,&
                    3.825,3.839,3.854,3.868,3.882,3.896,3.910,3.924,3.937,&
                    3.950,3.964,3.976,3.989,4.002,4.014,4.026,4.039,4.050,&
                    4.062,4.074,4.085,4.097,4.108,4.119,4.130,4.141,4.151,&
                    4.162,4.172,4.182,4.192,4.202,4.212,4.222,4.231,4.241,&
                    4.250,4.259,4.268,4.277,4.286,4.295,4.304,4.312,4.321,&
                    4.329,4.337,4.345,4.353,4.361,4.369,4.376,4.384,4.391,&
                    4.399,4.406,4.413,4.420,4.427,4.434,4.441,4.447,4.454,&
                    4.460,4.467,4.473,4.479,4.485,4.491,4.497,4.503,4.509,&
                    4.515,4.520,4.526,4.531,4.537,4.542,4.547,4.552,4.557,&
                    4.562,4.567,4.572,4.577,4.582,4.586,4.591,4.595/)
!
! *** find position in arrays for bincoef
!
    if ( ionic<=0.200000D+02 ) then
      ipos = min(nint(0.200000D+02*ionic)+1,400)
    else
      ipos = 400 + nint(0.200000D+01*ionic-0.400000D+02)
    end if
    ipos = min(ipos,561)
!
! *** assign values to return array
!
    binarr(01) = bnc01m(ipos)
    binarr(02) = bnc02m(ipos)
    binarr(03) = bnc03m(ipos)
    binarr(04) = bnc04m(ipos)
    binarr(05) = bnc05m(ipos)
    binarr(06) = bnc06m(ipos)
    binarr(07) = bnc07m(ipos)
    binarr(08) = bnc08m(ipos)
    binarr(09) = bnc09m(ipos)
    binarr(10) = bnc10m(ipos)
    binarr(11) = bnc11m(ipos)
    binarr(12) = bnc12m(ipos)
    binarr(13) = bnc13m(ipos)
    binarr(14) = bnc14m(ipos)
    binarr(15) = bnc15m(ipos)
    binarr(16) = bnc16m(ipos)
    binarr(17) = bnc17m(ipos)
    binarr(18) = bnc18m(ipos)
    binarr(19) = bnc19m(ipos)
    binarr(20) = bnc20m(ipos)
    binarr(21) = bnc21m(ipos)
    binarr(22) = bnc22m(ipos)
    binarr(23) = bnc23m(ipos)
  end subroutine km198
!
!======================================================================
!
! *** isorropia code
! *** subroutine km223
! *** calculates binary activity coefficients by kusik-meissner method.
!     the computations have been performed and the results are stored in
!     lookup tables. the ionic activity 'in' is input , and the array
!     'binarr' is returned with the binary coefficients.
!
!     temperature is 223k
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine km223(ionic,binarr)
    implicit none
    integer(ik4) :: ipos
    real(rkx) , dimension(23) :: binarr
    real(rkx) :: ionic
    real(rkx) , dimension(561) :: bnc01m , bnc02m , bnc03m , bnc04m ,     &
                                  bnc05m , bnc06m , bnc07m , bnc08m ,     &
                                  bnc09m , bnc10m , bnc11m , bnc12m ,     &
                                  bnc13m , bnc14m , bnc15m , bnc16m ,     &
                                  bnc17m , bnc18m , bnc19m , bnc20m ,     &
                                  bnc21m , bnc22m , bnc23m
!
! *** nacl
!
    bnc01m(1:561) = (/-0.049,-0.101,-0.124,-0.140,-0.151,-0.159,-0.166,   &
                    -0.171,-0.175,-0.178,-0.181,-0.183,-0.185,-0.187,     &
                    -0.188,-0.188,-0.189,-0.189,-0.189,-0.189,-0.189,     &
                    -0.189,-0.188,-0.187,-0.187,-0.186,-0.185,-0.184,     &
                    -0.183,-0.182,-0.181,-0.180,-0.178,-0.177,-0.176,     &
                    -0.174,-0.173,-0.172,-0.170,-0.169,-0.167,-0.166,     &
                    -0.164,-0.163,-0.161,-0.159,-0.158,-0.156,-0.155,     &
                    -0.153,-0.151,-0.150,-0.148,-0.147,-0.145,-0.143,     &
                    -0.142,-0.140,-0.138,-0.137,-0.135,-0.134,-0.132,     &
                    -0.130,-0.128,-0.127,-0.125,-0.123,-0.122,-0.120,     &
                    -0.118,-0.117,-0.115,-0.113,-0.111,-0.110,-0.108,     &
                    -0.106,-0.104,-0.102,-0.101,-0.099,-0.097,-0.095,     &
                    -0.093,-0.091,-0.089,-0.087,-0.085,-0.083,-0.081,     &
                    -0.079,-0.077,-0.075,-0.073,-0.071,-0.069,-0.067,     &
                    -0.065,-0.063,-0.061,-0.058,-0.056,-0.054,-0.052,     &
                    -0.050,-0.048,-0.045,-0.043,-0.041,-0.039,-0.036,     &
                    -0.034,-0.032,-0.030,-0.027,-0.025,-0.023,-0.020,     &
                    -0.018,-0.016,-0.014,-0.011,-0.009,-0.007,-0.004,     &
                    -0.002,0.000,0.003,0.005,0.007,0.010,0.012,0.014,     &
                    0.016,0.019,0.021,0.023,0.026,0.028,0.030,0.033,0.035,&
                    0.037,0.039,0.042,0.044,0.046,0.049,0.051,0.053,0.055,&
                    0.058,0.060,0.062,0.064,0.067,0.069,0.071,0.073,0.076,&
                    0.078,0.080,0.082,0.085,0.087,0.089,0.091,0.094,0.096,&
                    0.098,0.100,0.102,0.105,0.107,0.109,0.111,0.113,0.116,&
                    0.118,0.120,0.122,0.124,0.126,0.129,0.131,0.133,0.135,&
                    0.137,0.139,0.142,0.144,0.146,0.148,0.150,0.152,0.154,&
                    0.157,0.159,0.161,0.163,0.165,0.167,0.169,0.171,0.173,&
                    0.175,0.178,0.180,0.182,0.184,0.186,0.188,0.190,0.192,&
                    0.194,0.196,0.198,0.200,0.202,0.204,0.206,0.208,0.211,&
                    0.213,0.215,0.217,0.219,0.221,0.223,0.225,0.227,0.229,&
                    0.231,0.233,0.235,0.237,0.239,0.241,0.243,0.245,0.246,&
                    0.248,0.250,0.252,0.254,0.256,0.258,0.260,0.262,0.264,&
                    0.266,0.268,0.270,0.272,0.274,0.276,0.277,0.279,0.281,&
                    0.283,0.285,0.287,0.289,0.291,0.293,0.295,0.296,0.298,&
                    0.300,0.302,0.304,0.306,0.308,0.309,0.311,0.313,0.315,&
                    0.317,0.319,0.320,0.322,0.324,0.326,0.328,0.330,0.331,&
                    0.333,0.335,0.337,0.339,0.340,0.342,0.344,0.346,0.348,&
                    0.349,0.351,0.353,0.355,0.356,0.358,0.360,0.362,0.363,&
                    0.365,0.367,0.369,0.370,0.372,0.374,0.376,0.377,0.379,&
                    0.381,0.383,0.384,0.386,0.388,0.389,0.391,0.393,0.394,&
                    0.396,0.398,0.400,0.401,0.403,0.405,0.406,0.408,0.410,&
                    0.411,0.413,0.415,0.416,0.418,0.420,0.421,0.423,0.424,&
                    0.426,0.428,0.429,0.431,0.433,0.434,0.436,0.437,0.439,&
                    0.441,0.442,0.444,0.446,0.447,0.449,0.450,0.452,0.453,&
                    0.455,0.457,0.458,0.460,0.461,0.463,0.464,0.466,0.468,&
                    0.469,0.471,0.472,0.474,0.475,0.477,0.478,0.480,0.482,&
                    0.483,0.485,0.486,0.488,0.489,0.491,0.492,0.494,0.495,&
                    0.497,0.498,0.500,0.501,0.503,0.504,0.506,0.507,0.509,&
                    0.510,0.512,0.513,0.515,0.516,0.532,0.546,0.560,0.574,&
                    0.588,0.601,0.614,0.627,0.639,0.652,0.664,0.676,0.688,&
                    0.700,0.711,0.722,0.733,0.744,0.755,0.765,0.776,0.786,&
                    0.796,0.806,0.816,0.825,0.835,0.844,0.853,0.862,0.871,&
                    0.880,0.888,0.897,0.905,0.913,0.921,0.929,0.937,0.945,&
                    0.952,0.960,0.967,0.975,0.982,0.989,0.996,1.003,1.009,&
                    1.016,1.023,1.029,1.036,1.042,1.048,1.054,1.060,1.066,&
                    1.072,1.078,1.084,1.089,1.095,1.100,1.106,1.111,1.116,&
                    1.121,1.126,1.131,1.136,1.141,1.146,1.151,1.156,1.160,&
                    1.165,1.169,1.174,1.178,1.182,1.187,1.191,1.195,1.199,&
                    1.203,1.207,1.211,1.215,1.219,1.223,1.226,1.230,1.234,&
                    1.237,1.241,1.244,1.247,1.251,1.254,1.257,1.261,1.264,&
                    1.267,1.270,1.273,1.276,1.279,1.282,1.285,1.288,1.291,&
                    1.293,1.296,1.299,1.301,1.304,1.306,1.309,1.311,1.314,&
                    1.316,1.319,1.321,1.323,1.326,1.328,1.330,1.332,1.334,&
                    1.336,1.339,1.341,1.343,1.345,1.347,1.348,1.350,1.352,&
                    1.354,1.356,1.358,1.359,1.361,1.363,1.364,1.366,1.368,&
                    1.369,1.371,1.372,1.374,1.375,1.377,1.378,1.379,1.381,&
                    1.382,1.383,1.385,1.386/)
!
! *** na2so4
!
    bnc02m(1:561) = (/-0.100,-0.220,-0.280,-0.323,-0.357,-0.385,-0.409,   &
                    -0.431,-0.450,-0.468,-0.484,-0.499,-0.513,-0.526,     &
                    -0.538,-0.549,-0.560,-0.571,-0.581,-0.590,-0.600,     &
                    -0.608,-0.617,-0.625,-0.633,-0.641,-0.648,-0.655,     &
                    -0.662,-0.669,-0.676,-0.682,-0.689,-0.695,-0.701,     &
                    -0.707,-0.713,-0.718,-0.724,-0.729,-0.735,-0.740,     &
                    -0.745,-0.750,-0.755,-0.760,-0.765,-0.769,-0.774,     &
                    -0.779,-0.783,-0.788,-0.792,-0.796,-0.800,-0.805,     &
                    -0.809,-0.813,-0.817,-0.821,-0.825,-0.829,-0.833,     &
                    -0.836,-0.840,-0.844,-0.848,-0.851,-0.855,-0.858,     &
                    -0.862,-0.866,-0.869,-0.873,-0.876,-0.879,-0.883,     &
                    -0.886,-0.889,-0.893,-0.896,-0.899,-0.902,-0.906,     &
                    -0.909,-0.912,-0.915,-0.918,-0.921,-0.924,-0.927,     &
                    -0.930,-0.933,-0.936,-0.939,-0.942,-0.945,-0.948,     &
                    -0.951,-0.954,-0.957,-0.960,-0.963,-0.966,-0.968,     &
                    -0.971,-0.974,-0.977,-0.980,-0.982,-0.985,-0.988,     &
                    -0.991,-0.993,-0.996,-0.999,-1.001,-1.004,-1.007,     &
                    -1.009,-1.012,-1.015,-1.017,-1.020,-1.022,-1.025,     &
                    -1.028,-1.030,-1.033,-1.035,-1.038,-1.040,-1.043,     &
                    -1.045,-1.048,-1.050,-1.053,-1.055,-1.058,-1.060,     &
                    -1.062,-1.065,-1.067,-1.070,-1.072,-1.074,-1.077,     &
                    -1.079,-1.082,-1.084,-1.086,-1.089,-1.091,-1.093,     &
                    -1.096,-1.098,-1.100,-1.102,-1.105,-1.107,-1.109,     &
                    -1.112,-1.114,-1.116,-1.118,-1.121,-1.123,-1.125,     &
                    -1.127,-1.129,-1.132,-1.134,-1.136,-1.138,-1.140,     &
                    -1.143,-1.145,-1.147,-1.149,-1.151,-1.153,-1.155,     &
                    -1.158,-1.160,-1.162,-1.164,-1.166,-1.168,-1.170,     &
                    -1.172,-1.174,-1.177,-1.179,-1.181,-1.183,-1.185,     &
                    -1.187,-1.189,-1.191,-1.193,-1.195,-1.197,-1.199,     &
                    -1.201,-1.203,-1.205,-1.207,-1.209,-1.211,-1.213,     &
                    -1.215,-1.217,-1.219,-1.221,-1.223,-1.225,-1.227,     &
                    -1.229,-1.231,-1.233,-1.235,-1.237,-1.239,-1.241,     &
                    -1.243,-1.245,-1.247,-1.248,-1.250,-1.252,-1.254,     &
                    -1.256,-1.258,-1.260,-1.262,-1.264,-1.266,-1.268,     &
                    -1.269,-1.271,-1.273,-1.275,-1.277,-1.279,-1.281,     &
                    -1.283,-1.284,-1.286,-1.288,-1.290,-1.292,-1.294,     &
                    -1.296,-1.297,-1.299,-1.301,-1.303,-1.305,-1.306,     &
                    -1.308,-1.310,-1.312,-1.314,-1.316,-1.317,-1.319,     &
                    -1.321,-1.323,-1.325,-1.326,-1.328,-1.330,-1.332,     &
                    -1.333,-1.335,-1.337,-1.339,-1.341,-1.342,-1.344,     &
                    -1.346,-1.348,-1.349,-1.351,-1.353,-1.355,-1.356,     &
                    -1.358,-1.360,-1.362,-1.363,-1.365,-1.367,-1.369,     &
                    -1.370,-1.372,-1.374,-1.375,-1.377,-1.379,-1.381,     &
                    -1.382,-1.384,-1.386,-1.387,-1.389,-1.391,-1.393,     &
                    -1.394,-1.396,-1.398,-1.399,-1.401,-1.403,-1.404,     &
                    -1.406,-1.408,-1.409,-1.411,-1.413,-1.414,-1.416,     &
                    -1.418,-1.419,-1.421,-1.423,-1.424,-1.426,-1.428,     &
                    -1.429,-1.431,-1.433,-1.434,-1.436,-1.438,-1.439,     &
                    -1.441,-1.442,-1.444,-1.446,-1.447,-1.449,-1.451,     &
                    -1.452,-1.454,-1.456,-1.457,-1.459,-1.460,-1.462,     &
                    -1.464,-1.465,-1.467,-1.468,-1.470,-1.472,-1.473,     &
                    -1.475,-1.476,-1.478,-1.480,-1.481,-1.483,-1.484,     &
                    -1.486,-1.488,-1.489,-1.491,-1.492,-1.494,-1.496,     &
                    -1.497,-1.499,-1.500,-1.502,-1.503,-1.505,-1.507,     &
                    -1.508,-1.510,-1.511,-1.513,-1.514,-1.516,-1.518,     &
                    -1.519,-1.521,-1.522,-1.524,-1.525,-1.527,-1.528,     &
                    -1.530,-1.531,-1.533,-1.535,-1.536,-1.538,-1.539,     &
                    -1.541,-1.557,-1.572,-1.587,-1.602,-1.617,-1.632,     &
                    -1.647,-1.661,-1.675,-1.690,-1.704,-1.718,-1.732,     &
                    -1.746,-1.760,-1.774,-1.787,-1.801,-1.815,-1.828,     &
                    -1.841,-1.855,-1.868,-1.881,-1.895,-1.908,-1.921,     &
                    -1.934,-1.947,-1.960,-1.972,-1.985,-1.998,-2.011,     &
                    -2.023,-2.036,-2.048,-2.061,-2.074,-2.086,-2.098,     &
                    -2.111,-2.123,-2.135,-2.148,-2.160,-2.172,-2.184,     &
                    -2.196,-2.208,-2.220,-2.232,-2.244,-2.256,-2.268,     &
                    -2.280,-2.292,-2.304,-2.315,-2.327,-2.339,-2.351,     &
                    -2.362,-2.374,-2.386,-2.397,-2.409,-2.420,-2.432,     &
                    -2.443,-2.455,-2.466,-2.478,-2.489,-2.501,-2.512,     &
                    -2.523,-2.535,-2.546,-2.557,-2.568,-2.580,-2.591,     &
                    -2.602,-2.613,-2.625,-2.636,-2.647,-2.658,-2.669,     &
                    -2.680,-2.691,-2.702,-2.713,-2.724,-2.735,-2.746,     &
                    -2.757,-2.768,-2.779,-2.790,-2.801,-2.812,-2.823,     &
                    -2.833,-2.844,-2.855,-2.866,-2.877,-2.888,-2.898,     &
                    -2.909,-2.920,-2.930,-2.941,-2.952,-2.963,-2.973,     &
                    -2.984,-2.995,-3.005,-3.016,-3.026,-3.037,-3.048,     &
                    -3.058,-3.069,-3.079,-3.090,-3.100,-3.111,-3.121,     &
                    -3.132,-3.142,-3.153,-3.163,-3.174,-3.184,-3.195,     &
                    -3.205,-3.215,-3.226,-3.236,-3.247,-3.257,-3.267,     &
                    -3.278,-3.288,-3.298,-3.309,-3.319,-3.329,-3.340,     &
                    -3.350,-3.360,-3.370,-3.381,-3.391,-3.401,-3.411,     &
                    -3.422/)
!
! *** nano3
!
    bnc03m(1:561) = (/-0.050,-0.111,-0.141,-0.164,-0.181,-0.196,-0.209,   &
                    -0.220,-0.230,-0.239,-0.248,-0.256,-0.263,-0.270,     &
                    -0.277,-0.283,-0.289,-0.295,-0.300,-0.306,-0.311,     &
                    -0.316,-0.320,-0.325,-0.329,-0.334,-0.338,-0.342,     &
                    -0.346,-0.350,-0.353,-0.357,-0.360,-0.364,-0.367,     &
                    -0.371,-0.374,-0.377,-0.380,-0.384,-0.387,-0.390,     &
                    -0.392,-0.395,-0.398,-0.401,-0.404,-0.406,-0.409,     &
                    -0.412,-0.414,-0.417,-0.419,-0.422,-0.424,-0.427,     &
                    -0.429,-0.432,-0.434,-0.436,-0.438,-0.441,-0.443,     &
                    -0.445,-0.447,-0.449,-0.452,-0.454,-0.456,-0.458,     &
                    -0.460,-0.462,-0.464,-0.466,-0.468,-0.470,-0.472,     &
                    -0.474,-0.476,-0.478,-0.480,-0.482,-0.484,-0.485,     &
                    -0.487,-0.489,-0.491,-0.493,-0.495,-0.496,-0.498,     &
                    -0.500,-0.502,-0.504,-0.505,-0.507,-0.509,-0.511,     &
                    -0.512,-0.514,-0.516,-0.518,-0.519,-0.521,-0.523,     &
                    -0.524,-0.526,-0.528,-0.529,-0.531,-0.533,-0.534,     &
                    -0.536,-0.538,-0.539,-0.541,-0.543,-0.544,-0.546,     &
                    -0.547,-0.549,-0.551,-0.552,-0.554,-0.555,-0.557,     &
                    -0.558,-0.560,-0.561,-0.563,-0.564,-0.566,-0.568,     &
                    -0.569,-0.571,-0.572,-0.574,-0.575,-0.576,-0.578,     &
                    -0.579,-0.581,-0.582,-0.584,-0.585,-0.587,-0.588,     &
                    -0.590,-0.591,-0.592,-0.594,-0.595,-0.597,-0.598,     &
                    -0.600,-0.601,-0.602,-0.604,-0.605,-0.606,-0.608,     &
                    -0.609,-0.611,-0.612,-0.613,-0.615,-0.616,-0.617,     &
                    -0.619,-0.620,-0.621,-0.623,-0.624,-0.625,-0.627,     &
                    -0.628,-0.629,-0.631,-0.632,-0.633,-0.635,-0.636,     &
                    -0.637,-0.638,-0.640,-0.641,-0.642,-0.643,-0.645,     &
                    -0.646,-0.647,-0.649,-0.650,-0.651,-0.652,-0.654,     &
                    -0.655,-0.656,-0.657,-0.659,-0.660,-0.661,-0.662,     &
                    -0.663,-0.665,-0.666,-0.667,-0.668,-0.670,-0.671,     &
                    -0.672,-0.673,-0.674,-0.676,-0.677,-0.678,-0.679,     &
                    -0.680,-0.681,-0.683,-0.684,-0.685,-0.686,-0.687,     &
                    -0.689,-0.690,-0.691,-0.692,-0.693,-0.694,-0.695,     &
                    -0.697,-0.698,-0.699,-0.700,-0.701,-0.702,-0.703,     &
                    -0.705,-0.706,-0.707,-0.708,-0.709,-0.710,-0.711,     &
                    -0.712,-0.714,-0.715,-0.716,-0.717,-0.718,-0.719,     &
                    -0.720,-0.721,-0.722,-0.724,-0.725,-0.726,-0.727,     &
                    -0.728,-0.729,-0.730,-0.731,-0.732,-0.733,-0.734,     &
                    -0.736,-0.737,-0.738,-0.739,-0.740,-0.741,-0.742,     &
                    -0.743,-0.744,-0.745,-0.746,-0.747,-0.748,-0.749,     &
                    -0.750,-0.751,-0.753,-0.754,-0.755,-0.756,-0.757,     &
                    -0.758,-0.759,-0.760,-0.761,-0.762,-0.763,-0.764,     &
                    -0.765,-0.766,-0.767,-0.768,-0.769,-0.770,-0.771,     &
                    -0.772,-0.773,-0.774,-0.775,-0.776,-0.777,-0.778,     &
                    -0.779,-0.780,-0.781,-0.782,-0.783,-0.784,-0.785,     &
                    -0.786,-0.787,-0.788,-0.789,-0.790,-0.791,-0.792,     &
                    -0.793,-0.794,-0.795,-0.796,-0.797,-0.798,-0.799,     &
                    -0.800,-0.801,-0.802,-0.803,-0.804,-0.805,-0.806,     &
                    -0.807,-0.808,-0.809,-0.810,-0.811,-0.812,-0.813,     &
                    -0.814,-0.815,-0.816,-0.817,-0.818,-0.818,-0.819,     &
                    -0.820,-0.821,-0.822,-0.823,-0.824,-0.825,-0.826,     &
                    -0.827,-0.828,-0.829,-0.830,-0.831,-0.832,-0.833,     &
                    -0.834,-0.835,-0.835,-0.836,-0.837,-0.838,-0.839,     &
                    -0.840,-0.841,-0.842,-0.843,-0.844,-0.845,-0.846,     &
                    -0.847,-0.848,-0.848,-0.849,-0.850,-0.851,-0.852,     &
                    -0.853,-0.854,-0.855,-0.856,-0.857,-0.858,-0.859,     &
                    -0.859,-0.860,-0.861,-0.862,-0.863,-0.864,-0.865,     &
                    -0.866,-0.875,-0.884,-0.893,-0.902,-0.910,-0.919,     &
                    -0.927,-0.936,-0.944,-0.952,-0.961,-0.969,-0.977,     &
                    -0.985,-0.993,-1.001,-1.008,-1.016,-1.024,-1.032,     &
                    -1.039,-1.047,-1.054,-1.062,-1.069,-1.077,-1.084,     &
                    -1.092,-1.099,-1.106,-1.113,-1.121,-1.128,-1.135,     &
                    -1.142,-1.149,-1.156,-1.163,-1.170,-1.177,-1.184,     &
                    -1.191,-1.198,-1.204,-1.211,-1.218,-1.225,-1.231,     &
                    -1.238,-1.245,-1.251,-1.258,-1.265,-1.271,-1.278,     &
                    -1.284,-1.291,-1.297,-1.304,-1.310,-1.317,-1.323,     &
                    -1.330,-1.336,-1.342,-1.349,-1.355,-1.361,-1.368,     &
                    -1.374,-1.380,-1.386,-1.393,-1.399,-1.405,-1.411,     &
                    -1.417,-1.423,-1.430,-1.436,-1.442,-1.448,-1.454,     &
                    -1.460,-1.466,-1.472,-1.478,-1.484,-1.490,-1.496,     &
                    -1.502,-1.508,-1.514,-1.520,-1.526,-1.532,-1.538,     &
                    -1.543,-1.549,-1.555,-1.561,-1.567,-1.573,-1.578,     &
                    -1.584,-1.590,-1.596,-1.602,-1.607,-1.613,-1.619,     &
                    -1.625,-1.630,-1.636,-1.642,-1.648,-1.653,-1.659,     &
                    -1.665,-1.670,-1.676,-1.682,-1.687,-1.693,-1.698,     &
                    -1.704,-1.710,-1.715,-1.721,-1.726,-1.732,-1.738,     &
                    -1.743,-1.749,-1.754,-1.760,-1.765,-1.771,-1.776,     &
                    -1.782,-1.787,-1.793,-1.798,-1.804,-1.809,-1.815,     &
                    -1.820,-1.826,-1.831,-1.837,-1.842,-1.847,-1.853,     &
                    -1.858,-1.864,-1.869,-1.875,-1.880,-1.885,-1.891,     &
                    -1.896/)
!
! *** (nh4)2so4
!
    bnc04m(1:561) = (/-0.101,-0.220,-0.281,-0.324,-0.358,-0.387,-0.412,   &
                    -0.434,-0.453,-0.471,-0.487,-0.503,-0.517,-0.530,     &
                    -0.543,-0.555,-0.566,-0.577,-0.587,-0.597,-0.606,     &
                    -0.615,-0.624,-0.632,-0.641,-0.649,-0.656,-0.664,     &
                    -0.671,-0.678,-0.685,-0.692,-0.698,-0.705,-0.711,     &
                    -0.717,-0.723,-0.729,-0.735,-0.740,-0.746,-0.752,     &
                    -0.757,-0.762,-0.767,-0.772,-0.777,-0.782,-0.787,     &
                    -0.792,-0.797,-0.801,-0.806,-0.810,-0.815,-0.819,     &
                    -0.824,-0.828,-0.832,-0.836,-0.840,-0.844,-0.848,     &
                    -0.852,-0.856,-0.860,-0.864,-0.868,-0.872,-0.876,     &
                    -0.879,-0.883,-0.887,-0.890,-0.894,-0.897,-0.901,     &
                    -0.904,-0.908,-0.911,-0.915,-0.918,-0.922,-0.925,     &
                    -0.928,-0.932,-0.935,-0.938,-0.942,-0.945,-0.948,     &
                    -0.951,-0.954,-0.958,-0.961,-0.964,-0.967,-0.970,     &
                    -0.973,-0.976,-0.979,-0.982,-0.985,-0.988,-0.991,     &
                    -0.994,-0.997,-1.000,-1.003,-1.006,-1.009,-1.012,     &
                    -1.015,-1.018,-1.021,-1.024,-1.026,-1.029,-1.032,     &
                    -1.035,-1.038,-1.040,-1.043,-1.046,-1.049,-1.051,     &
                    -1.054,-1.057,-1.060,-1.062,-1.065,-1.068,-1.070,     &
                    -1.073,-1.076,-1.078,-1.081,-1.083,-1.086,-1.089,     &
                    -1.091,-1.094,-1.096,-1.099,-1.102,-1.104,-1.107,     &
                    -1.109,-1.112,-1.114,-1.117,-1.119,-1.122,-1.124,     &
                    -1.127,-1.129,-1.131,-1.134,-1.136,-1.139,-1.141,     &
                    -1.144,-1.146,-1.148,-1.151,-1.153,-1.155,-1.158,     &
                    -1.160,-1.163,-1.165,-1.167,-1.170,-1.172,-1.174,     &
                    -1.177,-1.179,-1.181,-1.183,-1.186,-1.188,-1.190,     &
                    -1.193,-1.195,-1.197,-1.199,-1.202,-1.204,-1.206,     &
                    -1.208,-1.210,-1.213,-1.215,-1.217,-1.219,-1.221,     &
                    -1.224,-1.226,-1.228,-1.230,-1.232,-1.235,-1.237,     &
                    -1.239,-1.241,-1.243,-1.245,-1.247,-1.250,-1.252,     &
                    -1.254,-1.256,-1.258,-1.260,-1.262,-1.264,-1.266,     &
                    -1.269,-1.271,-1.273,-1.275,-1.277,-1.279,-1.281,     &
                    -1.283,-1.285,-1.287,-1.289,-1.291,-1.293,-1.295,     &
                    -1.297,-1.299,-1.301,-1.303,-1.305,-1.307,-1.309,     &
                    -1.311,-1.313,-1.315,-1.317,-1.319,-1.321,-1.323,     &
                    -1.325,-1.327,-1.329,-1.331,-1.333,-1.335,-1.337,     &
                    -1.339,-1.341,-1.343,-1.345,-1.347,-1.349,-1.351,     &
                    -1.353,-1.355,-1.357,-1.358,-1.360,-1.362,-1.364,     &
                    -1.366,-1.368,-1.370,-1.372,-1.374,-1.376,-1.377,     &
                    -1.379,-1.381,-1.383,-1.385,-1.387,-1.389,-1.391,     &
                    -1.392,-1.394,-1.396,-1.398,-1.400,-1.402,-1.404,     &
                    -1.405,-1.407,-1.409,-1.411,-1.413,-1.415,-1.417,     &
                    -1.418,-1.420,-1.422,-1.424,-1.426,-1.427,-1.429,     &
                    -1.431,-1.433,-1.435,-1.437,-1.438,-1.440,-1.442,     &
                    -1.444,-1.445,-1.447,-1.449,-1.451,-1.453,-1.454,     &
                    -1.456,-1.458,-1.460,-1.462,-1.463,-1.465,-1.467,     &
                    -1.469,-1.470,-1.472,-1.474,-1.476,-1.477,-1.479,     &
                    -1.481,-1.483,-1.484,-1.486,-1.488,-1.490,-1.491,     &
                    -1.493,-1.495,-1.496,-1.498,-1.500,-1.502,-1.503,     &
                    -1.505,-1.507,-1.509,-1.510,-1.512,-1.514,-1.515,     &
                    -1.517,-1.519,-1.520,-1.522,-1.524,-1.526,-1.527,     &
                    -1.529,-1.531,-1.532,-1.534,-1.536,-1.537,-1.539,     &
                    -1.541,-1.542,-1.544,-1.546,-1.547,-1.549,-1.551,     &
                    -1.552,-1.554,-1.556,-1.557,-1.559,-1.561,-1.562,     &
                    -1.564,-1.566,-1.567,-1.569,-1.571,-1.572,-1.574,     &
                    -1.576,-1.577,-1.579,-1.580,-1.582,-1.584,-1.585,     &
                    -1.587,-1.589,-1.590,-1.592,-1.594,-1.595,-1.597,     &
                    -1.598,-1.616,-1.632,-1.648,-1.663,-1.679,-1.694,     &
                    -1.710,-1.725,-1.740,-1.755,-1.770,-1.785,-1.799,     &
                    -1.814,-1.828,-1.843,-1.857,-1.871,-1.885,-1.900,     &
                    -1.914,-1.928,-1.941,-1.955,-1.969,-1.983,-1.996,     &
                    -2.010,-2.023,-2.037,-2.050,-2.063,-2.077,-2.090,     &
                    -2.103,-2.116,-2.129,-2.142,-2.155,-2.168,-2.181,     &
                    -2.194,-2.206,-2.219,-2.232,-2.244,-2.257,-2.269,     &
                    -2.282,-2.294,-2.307,-2.319,-2.332,-2.344,-2.356,     &
                    -2.369,-2.381,-2.393,-2.405,-2.417,-2.429,-2.441,     &
                    -2.454,-2.466,-2.478,-2.489,-2.501,-2.513,-2.525,     &
                    -2.537,-2.549,-2.561,-2.572,-2.584,-2.596,-2.608,     &
                    -2.619,-2.631,-2.642,-2.654,-2.666,-2.677,-2.689,     &
                    -2.700,-2.712,-2.723,-2.735,-2.746,-2.757,-2.769,     &
                    -2.780,-2.792,-2.803,-2.814,-2.825,-2.837,-2.848,     &
                    -2.859,-2.870,-2.882,-2.893,-2.904,-2.915,-2.926,     &
                    -2.937,-2.948,-2.959,-2.971,-2.982,-2.993,-3.004,     &
                    -3.015,-3.026,-3.037,-3.048,-3.058,-3.069,-3.080,     &
                    -3.091,-3.102,-3.113,-3.124,-3.135,-3.145,-3.156,     &
                    -3.167,-3.178,-3.189,-3.199,-3.210,-3.221,-3.232,     &
                    -3.242,-3.253,-3.264,-3.274,-3.285,-3.296,-3.306,     &
                    -3.317,-3.327,-3.338,-3.349,-3.359,-3.370,-3.380,     &
                    -3.391,-3.401,-3.412,-3.422,-3.433,-3.443,-3.454,     &
                    -3.464,-3.475,-3.485,-3.496,-3.506,-3.517,-3.527,     &
                    -3.537/)
!
! *** nh4no3
!
    bnc05m(1:561) = (/-0.051,-0.114,-0.148,-0.172,-0.192,-0.209,-0.224,   &
                    -0.238,-0.250,-0.262,-0.272,-0.282,-0.292,-0.301,     &
                    -0.310,-0.318,-0.326,-0.333,-0.341,-0.348,-0.355,     &
                    -0.362,-0.368,-0.375,-0.381,-0.387,-0.393,-0.398,     &
                    -0.404,-0.410,-0.415,-0.420,-0.426,-0.431,-0.436,     &
                    -0.441,-0.446,-0.450,-0.455,-0.460,-0.464,-0.469,     &
                    -0.473,-0.477,-0.482,-0.486,-0.490,-0.494,-0.498,     &
                    -0.502,-0.506,-0.510,-0.514,-0.518,-0.521,-0.525,     &
                    -0.529,-0.532,-0.536,-0.539,-0.543,-0.546,-0.550,     &
                    -0.553,-0.556,-0.560,-0.563,-0.566,-0.570,-0.573,     &
                    -0.576,-0.579,-0.582,-0.585,-0.588,-0.592,-0.595,     &
                    -0.598,-0.601,-0.604,-0.607,-0.610,-0.613,-0.616,     &
                    -0.619,-0.622,-0.624,-0.627,-0.630,-0.633,-0.636,     &
                    -0.639,-0.642,-0.645,-0.647,-0.650,-0.653,-0.656,     &
                    -0.659,-0.662,-0.664,-0.667,-0.670,-0.673,-0.675,     &
                    -0.678,-0.681,-0.684,-0.686,-0.689,-0.692,-0.694,     &
                    -0.697,-0.700,-0.702,-0.705,-0.708,-0.710,-0.713,     &
                    -0.716,-0.718,-0.721,-0.723,-0.726,-0.729,-0.731,     &
                    -0.734,-0.736,-0.739,-0.741,-0.744,-0.746,-0.749,     &
                    -0.751,-0.754,-0.756,-0.759,-0.761,-0.764,-0.766,     &
                    -0.768,-0.771,-0.773,-0.776,-0.778,-0.780,-0.783,     &
                    -0.785,-0.787,-0.790,-0.792,-0.794,-0.797,-0.799,     &
                    -0.801,-0.804,-0.806,-0.808,-0.810,-0.813,-0.815,     &
                    -0.817,-0.819,-0.822,-0.824,-0.826,-0.828,-0.831,     &
                    -0.833,-0.835,-0.837,-0.839,-0.841,-0.844,-0.846,     &
                    -0.848,-0.850,-0.852,-0.854,-0.856,-0.858,-0.860,     &
                    -0.863,-0.865,-0.867,-0.869,-0.871,-0.873,-0.875,     &
                    -0.877,-0.879,-0.881,-0.883,-0.885,-0.887,-0.889,     &
                    -0.891,-0.893,-0.895,-0.897,-0.899,-0.901,-0.903,     &
                    -0.905,-0.907,-0.909,-0.911,-0.913,-0.915,-0.917,     &
                    -0.919,-0.920,-0.922,-0.924,-0.926,-0.928,-0.930,     &
                    -0.932,-0.934,-0.936,-0.938,-0.939,-0.941,-0.943,     &
                    -0.945,-0.947,-0.949,-0.950,-0.952,-0.954,-0.956,     &
                    -0.958,-0.960,-0.961,-0.963,-0.965,-0.967,-0.969,     &
                    -0.970,-0.972,-0.974,-0.976,-0.977,-0.979,-0.981,     &
                    -0.983,-0.984,-0.986,-0.988,-0.990,-0.991,-0.993,     &
                    -0.995,-0.997,-0.998,-1.000,-1.002,-1.003,-1.005,     &
                    -1.007,-1.008,-1.010,-1.012,-1.013,-1.015,-1.017,     &
                    -1.019,-1.020,-1.022,-1.023,-1.025,-1.027,-1.028,     &
                    -1.030,-1.032,-1.033,-1.035,-1.037,-1.038,-1.040,     &
                    -1.041,-1.043,-1.045,-1.046,-1.048,-1.049,-1.051,     &
                    -1.053,-1.054,-1.056,-1.057,-1.059,-1.061,-1.062,     &
                    -1.064,-1.065,-1.067,-1.068,-1.070,-1.071,-1.073,     &
                    -1.075,-1.076,-1.078,-1.079,-1.081,-1.082,-1.084,     &
                    -1.085,-1.087,-1.088,-1.090,-1.091,-1.093,-1.094,     &
                    -1.096,-1.097,-1.099,-1.100,-1.102,-1.103,-1.105,     &
                    -1.106,-1.108,-1.109,-1.111,-1.112,-1.114,-1.115,     &
                    -1.116,-1.118,-1.119,-1.121,-1.122,-1.124,-1.125,     &
                    -1.127,-1.128,-1.129,-1.131,-1.132,-1.134,-1.135,     &
                    -1.137,-1.138,-1.139,-1.141,-1.142,-1.144,-1.145,     &
                    -1.146,-1.148,-1.149,-1.151,-1.152,-1.153,-1.155,     &
                    -1.156,-1.158,-1.159,-1.160,-1.162,-1.163,-1.164,     &
                    -1.166,-1.167,-1.168,-1.170,-1.171,-1.173,-1.174,     &
                    -1.175,-1.177,-1.178,-1.179,-1.181,-1.182,-1.183,     &
                    -1.185,-1.186,-1.187,-1.189,-1.190,-1.191,-1.193,     &
                    -1.194,-1.195,-1.197,-1.198,-1.199,-1.200,-1.202,     &
                    -1.203,-1.204,-1.206,-1.207,-1.208,-1.210,-1.211,     &
                    -1.212,-1.226,-1.238,-1.251,-1.263,-1.275,-1.287,     &
                    -1.298,-1.310,-1.321,-1.333,-1.344,-1.355,-1.365,     &
                    -1.376,-1.387,-1.397,-1.407,-1.417,-1.427,-1.437,     &
                    -1.447,-1.457,-1.467,-1.476,-1.486,-1.495,-1.505,     &
                    -1.514,-1.523,-1.532,-1.541,-1.550,-1.559,-1.567,     &
                    -1.576,-1.585,-1.593,-1.602,-1.610,-1.619,-1.627,     &
                    -1.635,-1.643,-1.651,-1.659,-1.667,-1.675,-1.683,     &
                    -1.691,-1.699,-1.707,-1.715,-1.722,-1.730,-1.737,     &
                    -1.745,-1.752,-1.760,-1.767,-1.775,-1.782,-1.789,     &
                    -1.797,-1.804,-1.811,-1.818,-1.825,-1.832,-1.840,     &
                    -1.847,-1.854,-1.861,-1.867,-1.874,-1.881,-1.888,     &
                    -1.895,-1.902,-1.908,-1.915,-1.922,-1.929,-1.935,     &
                    -1.942,-1.948,-1.955,-1.962,-1.968,-1.975,-1.981,     &
                    -1.988,-1.994,-2.000,-2.007,-2.013,-2.020,-2.026,     &
                    -2.032,-2.039,-2.045,-2.051,-2.057,-2.064,-2.070,     &
                    -2.076,-2.082,-2.088,-2.094,-2.100,-2.107,-2.113,     &
                    -2.119,-2.125,-2.131,-2.137,-2.143,-2.149,-2.155,     &
                    -2.161,-2.167,-2.173,-2.179,-2.184,-2.190,-2.196,     &
                    -2.202,-2.208,-2.214,-2.220,-2.225,-2.231,-2.237,     &
                    -2.243,-2.248,-2.254,-2.260,-2.266,-2.271,-2.277,     &
                    -2.283,-2.288,-2.294,-2.300,-2.305,-2.311,-2.317,     &
                    -2.322,-2.328,-2.333,-2.339,-2.345,-2.350,-2.356,     &
                    -2.361,-2.367,-2.372,-2.378,-2.383,-2.389,-2.394,     &
                    -2.400/)
!
! *** nh4cl
!
    bnc06m(1:561) = (/-0.049,-0.106,-0.133,-0.151,-0.166,-0.177,-0.187,   &
                    -0.195,-0.202,-0.208,-0.214,-0.219,-0.224,-0.228,     &
                    -0.231,-0.235,-0.238,-0.241,-0.244,-0.246,-0.249,     &
                    -0.251,-0.253,-0.255,-0.257,-0.259,-0.260,-0.262,     &
                    -0.264,-0.265,-0.266,-0.268,-0.269,-0.270,-0.271,     &
                    -0.272,-0.273,-0.274,-0.275,-0.276,-0.277,-0.278,     &
                    -0.278,-0.279,-0.280,-0.281,-0.281,-0.282,-0.283,     &
                    -0.283,-0.284,-0.284,-0.285,-0.285,-0.286,-0.287,     &
                    -0.287,-0.288,-0.288,-0.288,-0.289,-0.289,-0.290,     &
                    -0.290,-0.291,-0.291,-0.291,-0.292,-0.292,-0.292,     &
                    -0.293,-0.293,-0.293,-0.294,-0.294,-0.294,-0.294,     &
                    -0.295,-0.295,-0.295,-0.295,-0.296,-0.296,-0.296,     &
                    -0.296,-0.296,-0.296,-0.297,-0.297,-0.297,-0.297,     &
                    -0.297,-0.297,-0.297,-0.297,-0.297,-0.297,-0.297,     &
                    -0.297,-0.297,-0.297,-0.297,-0.297,-0.297,-0.297,     &
                    -0.297,-0.297,-0.297,-0.297,-0.297,-0.297,-0.297,     &
                    -0.297,-0.297,-0.297,-0.297,-0.297,-0.296,-0.296,     &
                    -0.296,-0.296,-0.296,-0.296,-0.296,-0.296,-0.296,     &
                    -0.295,-0.295,-0.295,-0.295,-0.295,-0.295,-0.295,     &
                    -0.294,-0.294,-0.294,-0.294,-0.294,-0.294,-0.294,     &
                    -0.293,-0.293,-0.293,-0.293,-0.293,-0.293,-0.292,     &
                    -0.292,-0.292,-0.292,-0.292,-0.292,-0.291,-0.291,     &
                    -0.291,-0.291,-0.291,-0.291,-0.290,-0.290,-0.290,     &
                    -0.290,-0.290,-0.290,-0.289,-0.289,-0.289,-0.289,     &
                    -0.289,-0.288,-0.288,-0.288,-0.288,-0.288,-0.288,     &
                    -0.287,-0.287,-0.287,-0.287,-0.287,-0.287,-0.286,     &
                    -0.286,-0.286,-0.286,-0.286,-0.286,-0.285,-0.285,     &
                    -0.285,-0.285,-0.285,-0.284,-0.284,-0.284,-0.284,     &
                    -0.284,-0.284,-0.283,-0.283,-0.283,-0.283,-0.283,     &
                    -0.283,-0.282,-0.282,-0.282,-0.282,-0.282,-0.282,     &
                    -0.281,-0.281,-0.281,-0.281,-0.281,-0.281,-0.280,     &
                    -0.280,-0.280,-0.280,-0.280,-0.280,-0.279,-0.279,     &
                    -0.279,-0.279,-0.279,-0.279,-0.278,-0.278,-0.278,     &
                    -0.278,-0.278,-0.278,-0.277,-0.277,-0.277,-0.277,     &
                    -0.277,-0.277,-0.276,-0.276,-0.276,-0.276,-0.276,     &
                    -0.276,-0.276,-0.275,-0.275,-0.275,-0.275,-0.275,     &
                    -0.275,-0.275,-0.274,-0.274,-0.274,-0.274,-0.274,     &
                    -0.274,-0.273,-0.273,-0.273,-0.273,-0.273,-0.273,     &
                    -0.273,-0.272,-0.272,-0.272,-0.272,-0.272,-0.272,     &
                    -0.272,-0.271,-0.271,-0.271,-0.271,-0.271,-0.271,     &
                    -0.271,-0.271,-0.270,-0.270,-0.270,-0.270,-0.270,     &
                    -0.270,-0.270,-0.270,-0.269,-0.269,-0.269,-0.269,     &
                    -0.269,-0.269,-0.269,-0.268,-0.268,-0.268,-0.268,     &
                    -0.268,-0.268,-0.268,-0.268,-0.268,-0.267,-0.267,     &
                    -0.267,-0.267,-0.267,-0.267,-0.267,-0.267,-0.266,     &
                    -0.266,-0.266,-0.266,-0.266,-0.266,-0.266,-0.266,     &
                    -0.266,-0.265,-0.265,-0.265,-0.265,-0.265,-0.265,     &
                    -0.265,-0.265,-0.265,-0.265,-0.264,-0.264,-0.264,     &
                    -0.264,-0.264,-0.264,-0.264,-0.264,-0.264,-0.264,     &
                    -0.263,-0.263,-0.263,-0.263,-0.263,-0.263,-0.263,     &
                    -0.263,-0.263,-0.263,-0.263,-0.262,-0.262,-0.262,     &
                    -0.262,-0.262,-0.262,-0.262,-0.262,-0.262,-0.262,     &
                    -0.262,-0.262,-0.261,-0.261,-0.261,-0.261,-0.261,     &
                    -0.261,-0.261,-0.261,-0.261,-0.261,-0.261,-0.261,     &
                    -0.261,-0.260,-0.260,-0.260,-0.260,-0.260,-0.260,     &
                    -0.260,-0.260,-0.260,-0.260,-0.260,-0.260,-0.260,     &
                    -0.260,-0.259,-0.259,-0.259,-0.259,-0.259,-0.259,     &
                    -0.259,-0.258,-0.258,-0.257,-0.257,-0.257,-0.256,     &
                    -0.256,-0.256,-0.256,-0.256,-0.255,-0.255,-0.256,     &
                    -0.256,-0.256,-0.256,-0.256,-0.256,-0.257,-0.257,     &
                    -0.257,-0.258,-0.258,-0.259,-0.259,-0.260,-0.260,     &
                    -0.261,-0.262,-0.262,-0.263,-0.264,-0.265,-0.265,     &
                    -0.266,-0.267,-0.268,-0.269,-0.270,-0.271,-0.272,     &
                    -0.273,-0.274,-0.275,-0.277,-0.278,-0.279,-0.280,     &
                    -0.281,-0.283,-0.284,-0.285,-0.287,-0.288,-0.289,     &
                    -0.291,-0.292,-0.294,-0.295,-0.297,-0.298,-0.300,     &
                    -0.301,-0.303,-0.304,-0.306,-0.308,-0.309,-0.311,     &
                    -0.313,-0.314,-0.316,-0.318,-0.320,-0.321,-0.323,     &
                    -0.325,-0.327,-0.329,-0.331,-0.332,-0.334,-0.336,     &
                    -0.338,-0.340,-0.342,-0.344,-0.346,-0.348,-0.350,     &
                    -0.352,-0.354,-0.356,-0.358,-0.360,-0.362,-0.364,     &
                    -0.367,-0.369,-0.371,-0.373,-0.375,-0.377,-0.379,     &
                    -0.382,-0.384,-0.386,-0.388,-0.391,-0.393,-0.395,     &
                    -0.397,-0.400,-0.402,-0.404,-0.406,-0.409,-0.411,     &
                    -0.413,-0.416,-0.418,-0.421,-0.423,-0.425,-0.428,     &
                    -0.430,-0.433,-0.435,-0.437,-0.440,-0.442,-0.445,     &
                    -0.447,-0.450,-0.452,-0.455,-0.457,-0.460,-0.462,     &
                    -0.465,-0.467,-0.470,-0.472,-0.475,-0.477,-0.480,     &
                    -0.483,-0.485,-0.488,-0.490,-0.493,-0.496,-0.498,     &
                    -0.501,-0.503,-0.506,-0.509,-0.511,-0.514,-0.517,     &
                    -0.519/)
!
! *** (2h ,so4)
!
    bnc07m(1:561) = (/-0.100,-0.219,-0.279,-0.321,-0.354,-0.382,-0.406,   &
                    -0.427,-0.446,-0.463,-0.479,-0.493,-0.506,-0.519,     &
                    -0.531,-0.542,-0.552,-0.562,-0.572,-0.581,-0.590,     &
                    -0.598,-0.606,-0.614,-0.622,-0.629,-0.636,-0.643,     &
                    -0.650,-0.656,-0.662,-0.668,-0.674,-0.680,-0.686,     &
                    -0.692,-0.697,-0.702,-0.707,-0.713,-0.718,-0.722,     &
                    -0.727,-0.732,-0.737,-0.741,-0.746,-0.750,-0.754,     &
                    -0.759,-0.763,-0.767,-0.771,-0.775,-0.779,-0.783,     &
                    -0.787,-0.791,-0.794,-0.798,-0.802,-0.805,-0.809,     &
                    -0.813,-0.816,-0.820,-0.823,-0.826,-0.830,-0.833,     &
                    -0.836,-0.840,-0.843,-0.846,-0.849,-0.852,-0.855,     &
                    -0.859,-0.862,-0.865,-0.868,-0.871,-0.874,-0.877,     &
                    -0.879,-0.882,-0.885,-0.888,-0.891,-0.894,-0.897,     &
                    -0.899,-0.902,-0.905,-0.908,-0.910,-0.913,-0.916,     &
                    -0.918,-0.921,-0.924,-0.926,-0.929,-0.932,-0.934,     &
                    -0.937,-0.939,-0.942,-0.944,-0.947,-0.949,-0.952,     &
                    -0.954,-0.957,-0.959,-0.962,-0.964,-0.967,-0.969,     &
                    -0.971,-0.974,-0.976,-0.978,-0.981,-0.983,-0.985,     &
                    -0.988,-0.990,-0.992,-0.995,-0.997,-0.999,-1.002,     &
                    -1.004,-1.006,-1.008,-1.011,-1.013,-1.015,-1.017,     &
                    -1.019,-1.022,-1.024,-1.026,-1.028,-1.030,-1.032,     &
                    -1.034,-1.037,-1.039,-1.041,-1.043,-1.045,-1.047,     &
                    -1.049,-1.051,-1.053,-1.056,-1.058,-1.060,-1.062,     &
                    -1.064,-1.066,-1.068,-1.070,-1.072,-1.074,-1.076,     &
                    -1.078,-1.080,-1.082,-1.084,-1.086,-1.088,-1.090,     &
                    -1.092,-1.094,-1.096,-1.098,-1.099,-1.101,-1.103,     &
                    -1.105,-1.107,-1.109,-1.111,-1.113,-1.115,-1.117,     &
                    -1.119,-1.121,-1.122,-1.124,-1.126,-1.128,-1.130,     &
                    -1.132,-1.134,-1.135,-1.137,-1.139,-1.141,-1.143,     &
                    -1.145,-1.146,-1.148,-1.150,-1.152,-1.154,-1.156,     &
                    -1.157,-1.159,-1.161,-1.163,-1.165,-1.166,-1.168,     &
                    -1.170,-1.172,-1.173,-1.175,-1.177,-1.179,-1.180,     &
                    -1.182,-1.184,-1.186,-1.187,-1.189,-1.191,-1.193,     &
                    -1.194,-1.196,-1.198,-1.200,-1.201,-1.203,-1.205,     &
                    -1.206,-1.208,-1.210,-1.212,-1.213,-1.215,-1.217,     &
                    -1.218,-1.220,-1.222,-1.223,-1.225,-1.227,-1.228,     &
                    -1.230,-1.232,-1.233,-1.235,-1.237,-1.238,-1.240,     &
                    -1.242,-1.243,-1.245,-1.247,-1.248,-1.250,-1.251,     &
                    -1.253,-1.255,-1.256,-1.258,-1.260,-1.261,-1.263,     &
                    -1.264,-1.266,-1.268,-1.269,-1.271,-1.273,-1.274,     &
                    -1.276,-1.277,-1.279,-1.280,-1.282,-1.284,-1.285,     &
                    -1.287,-1.288,-1.290,-1.292,-1.293,-1.295,-1.296,     &
                    -1.298,-1.299,-1.301,-1.303,-1.304,-1.306,-1.307,     &
                    -1.309,-1.310,-1.312,-1.313,-1.315,-1.317,-1.318,     &
                    -1.320,-1.321,-1.323,-1.324,-1.326,-1.327,-1.329,     &
                    -1.330,-1.332,-1.333,-1.335,-1.337,-1.338,-1.340,     &
                    -1.341,-1.343,-1.344,-1.346,-1.347,-1.349,-1.350,     &
                    -1.352,-1.353,-1.355,-1.356,-1.358,-1.359,-1.361,     &
                    -1.362,-1.364,-1.365,-1.367,-1.368,-1.370,-1.371,     &
                    -1.373,-1.374,-1.376,-1.377,-1.379,-1.380,-1.381,     &
                    -1.383,-1.384,-1.386,-1.387,-1.389,-1.390,-1.392,     &
                    -1.393,-1.395,-1.396,-1.398,-1.399,-1.401,-1.402,     &
                    -1.403,-1.405,-1.406,-1.408,-1.409,-1.411,-1.412,     &
                    -1.414,-1.415,-1.416,-1.418,-1.419,-1.421,-1.422,     &
                    -1.424,-1.425,-1.427,-1.428,-1.429,-1.431,-1.432,     &
                    -1.434,-1.435,-1.437,-1.438,-1.439,-1.441,-1.442,     &
                    -1.444,-1.445,-1.447,-1.448,-1.449,-1.451,-1.452,     &
                    -1.454,-1.469,-1.483,-1.497,-1.510,-1.524,-1.537,     &
                    -1.551,-1.564,-1.578,-1.591,-1.604,-1.617,-1.630,     &
                    -1.643,-1.656,-1.669,-1.681,-1.694,-1.707,-1.719,     &
                    -1.732,-1.744,-1.756,-1.769,-1.781,-1.793,-1.806,     &
                    -1.818,-1.830,-1.842,-1.854,-1.866,-1.878,-1.890,     &
                    -1.902,-1.914,-1.925,-1.937,-1.949,-1.961,-1.972,     &
                    -1.984,-1.996,-2.007,-2.019,-2.030,-2.042,-2.053,     &
                    -2.065,-2.076,-2.087,-2.099,-2.110,-2.121,-2.133,     &
                    -2.144,-2.155,-2.166,-2.178,-2.189,-2.200,-2.211,     &
                    -2.222,-2.233,-2.244,-2.255,-2.266,-2.277,-2.288,     &
                    -2.299,-2.310,-2.321,-2.332,-2.343,-2.354,-2.365,     &
                    -2.375,-2.386,-2.397,-2.408,-2.419,-2.429,-2.440,     &
                    -2.451,-2.461,-2.472,-2.483,-2.494,-2.504,-2.515,     &
                    -2.525,-2.536,-2.547,-2.557,-2.568,-2.578,-2.589,     &
                    -2.599,-2.610,-2.620,-2.631,-2.641,-2.652,-2.662,     &
                    -2.673,-2.683,-2.693,-2.704,-2.714,-2.725,-2.735,     &
                    -2.745,-2.756,-2.766,-2.776,-2.787,-2.797,-2.807,     &
                    -2.817,-2.828,-2.838,-2.848,-2.858,-2.869,-2.879,     &
                    -2.889,-2.899,-2.909,-2.920,-2.930,-2.940,-2.950,     &
                    -2.960,-2.970,-2.980,-2.991,-3.001,-3.011,-3.021,     &
                    -3.031,-3.041,-3.051,-3.061,-3.071,-3.081,-3.091,     &
                    -3.101,-3.111,-3.121,-3.131,-3.141,-3.151,-3.161,     &
                    -3.171,-3.181,-3.191,-3.201,-3.211,-3.221,-3.231,     &
                    -3.241/)
!
! *** (h ,hso4)
!
    bnc08m(1:561) = (/-0.047,-0.091,-0.109,-0.119,-0.125,-0.128,-0.130,   &
                    -0.131,-0.130,-0.129,-0.128,-0.125,-0.122,-0.119,     &
                    -0.115,-0.111,-0.107,-0.102,-0.097,-0.092,-0.086,     &
                    -0.080,-0.074,-0.068,-0.062,-0.055,-0.048,-0.041,     &
                    -0.034,-0.027,-0.019,-0.012,-0.004,0.004,0.012,0.020, &
                    0.028,0.037,0.045,0.054,0.063,0.071,0.080,0.089,0.098,&
                    0.108,0.117,0.126,0.136,0.145,0.155,0.164,0.174,0.184,&
                    0.193,0.203,0.213,0.223,0.233,0.243,0.253,0.263,0.274,&
                    0.284,0.294,0.304,0.315,0.325,0.336,0.346,0.357,0.368,&
                    0.378,0.389,0.400,0.411,0.421,0.432,0.443,0.455,0.466,&
                    0.477,0.488,0.499,0.511,0.522,0.534,0.545,0.557,0.568,&
                    0.580,0.592,0.604,0.615,0.627,0.639,0.651,0.664,0.676,&
                    0.688,0.700,0.712,0.725,0.737,0.749,0.762,0.774,0.787,&
                    0.799,0.812,0.824,0.837,0.849,0.862,0.875,0.887,0.900,&
                    0.913,0.925,0.938,0.951,0.963,0.976,0.989,1.001,1.014,&
                    1.027,1.039,1.052,1.064,1.077,1.090,1.102,1.115,1.127,&
                    1.140,1.152,1.165,1.177,1.190,1.202,1.215,1.227,1.239,&
                    1.252,1.264,1.276,1.289,1.301,1.313,1.325,1.338,1.350,&
                    1.362,1.374,1.386,1.398,1.410,1.422,1.434,1.446,1.458,&
                    1.470,1.482,1.494,1.506,1.518,1.530,1.541,1.553,1.565,&
                    1.577,1.588,1.600,1.612,1.623,1.635,1.646,1.658,1.669,&
                    1.681,1.692,1.704,1.715,1.727,1.738,1.749,1.761,1.772,&
                    1.783,1.794,1.806,1.817,1.828,1.839,1.850,1.861,1.872,&
                    1.883,1.894,1.905,1.916,1.927,1.938,1.949,1.960,1.971,&
                    1.982,1.992,2.003,2.014,2.025,2.035,2.046,2.057,2.067,&
                    2.078,2.088,2.099,2.109,2.120,2.130,2.141,2.151,2.162,&
                    2.172,2.182,2.193,2.203,2.213,2.224,2.234,2.244,2.254,&
                    2.265,2.275,2.285,2.295,2.305,2.315,2.325,2.335,2.345,&
                    2.355,2.365,2.375,2.385,2.395,2.405,2.415,2.424,2.434,&
                    2.444,2.454,2.464,2.473,2.483,2.493,2.502,2.512,2.522,&
                    2.531,2.541,2.550,2.560,2.569,2.579,2.588,2.598,2.607,&
                    2.617,2.626,2.635,2.645,2.654,2.663,2.673,2.682,2.691,&
                    2.701,2.710,2.719,2.728,2.737,2.746,2.756,2.765,2.774,&
                    2.783,2.792,2.801,2.810,2.819,2.828,2.837,2.846,2.855,&
                    2.864,2.873,2.881,2.890,2.899,2.908,2.917,2.925,2.934,&
                    2.943,2.952,2.960,2.969,2.978,2.986,2.995,3.004,3.012,&
                    3.021,3.029,3.038,3.047,3.055,3.064,3.072,3.080,3.089,&
                    3.097,3.106,3.114,3.123,3.131,3.139,3.148,3.156,3.164,&
                    3.173,3.181,3.189,3.197,3.206,3.214,3.222,3.230,3.238,&
                    3.246,3.255,3.263,3.271,3.279,3.287,3.295,3.303,3.311,&
                    3.319,3.327,3.335,3.343,3.351,3.359,3.367,3.375,3.383,&
                    3.390,3.398,3.406,3.414,3.422,3.430,3.437,3.445,3.453,&
                    3.461,3.468,3.476,3.484,3.492,3.499,3.507,3.515,3.522,&
                    3.530,3.537,3.545,3.553,3.560,3.568,3.575,3.583,3.590,&
                    3.598,3.605,3.613,3.620,3.628,3.635,3.642,3.650,3.657,&
                    3.665,3.672,3.679,3.687,3.765,3.837,3.907,3.976,4.044,&
                    4.111,4.176,4.241,4.305,4.368,4.429,4.490,4.550,4.609,&
                    4.668,4.725,4.782,4.838,4.893,4.947,5.001,5.054,5.106,&
                    5.157,5.208,5.259,5.308,5.357,5.406,5.453,5.501,5.547,&
                    5.593,5.639,5.684,5.729,5.773,5.816,5.859,5.902,5.944,&
                    5.986,6.027,6.068,6.108,6.148,6.188,6.227,6.265,6.304,&
                    6.342,6.379,6.416,6.453,6.490,6.526,6.562,6.597,6.632,&
                    6.667,6.701,6.735,6.769,6.803,6.836,6.869,6.901,6.934,&
                    6.966,6.997,7.029,7.060,7.091,7.122,7.152,7.182,7.212,&
                    7.242,7.271,7.300,7.329,7.358,7.386,7.415,7.443,7.470,&
                    7.498,7.525,7.552,7.579,7.606,7.633,7.659,7.685,7.711,&
                    7.737,7.762,7.788,7.813,7.838,7.862,7.887,7.912,7.936,&
                    7.960,7.984,8.008,8.031,8.055,8.078,8.101,8.124,8.147,&
                    8.169,8.192,8.214,8.236,8.258,8.280,8.302,8.323,8.345,&
                    8.366,8.387,8.408,8.429,8.450,8.470,8.491,8.511,8.531,&
                    8.552,8.572,8.591,8.611,8.631,8.650,8.670,8.689,8.708,&
                    8.727,8.746,8.765,8.783,8.802,8.820,8.839,8.857,8.875,&
                    8.893,8.911,8.929,8.946,8.964,8.981,8.999,9.016,9.033,&
                    9.050,9.067,9.084/)
!
! *** nh4hso4
!
    bnc09m(1:561) = (/-0.049,-0.104,-0.131,-0.149,-0.163,-0.174,-0.184,   &
                    -0.192,-0.199,-0.205,-0.211,-0.216,-0.220,-0.224,     &
                    -0.228,-0.231,-0.234,-0.236,-0.238,-0.240,-0.242,     &
                    -0.244,-0.245,-0.247,-0.248,-0.248,-0.249,-0.250,     &
                    -0.250,-0.250,-0.251,-0.251,-0.250,-0.250,-0.250,     &
                    -0.249,-0.249,-0.248,-0.247,-0.247,-0.246,-0.245,     &
                    -0.243,-0.242,-0.241,-0.240,-0.238,-0.237,-0.235,     &
                    -0.233,-0.232,-0.230,-0.228,-0.226,-0.224,-0.222,     &
                    -0.220,-0.218,-0.216,-0.214,-0.211,-0.209,-0.207,     &
                    -0.204,-0.202,-0.199,-0.197,-0.194,-0.192,-0.189,     &
                    -0.187,-0.184,-0.181,-0.178,-0.176,-0.173,-0.170,     &
                    -0.167,-0.164,-0.161,-0.158,-0.155,-0.152,-0.149,     &
                    -0.146,-0.142,-0.139,-0.136,-0.133,-0.129,-0.126,     &
                    -0.123,-0.119,-0.116,-0.112,-0.109,-0.105,-0.102,     &
                    -0.098,-0.095,-0.091,-0.088,-0.084,-0.080,-0.077,     &
                    -0.073,-0.069,-0.066,-0.062,-0.058,-0.055,-0.051,     &
                    -0.047,-0.043,-0.039,-0.036,-0.032,-0.028,-0.024,     &
                    -0.020,-0.017,-0.013,-0.009,-0.005,-0.001,0.002,0.006,&
                    0.010,0.014,0.018,0.021,0.025,0.029,0.033,0.037,0.040,&
                    0.044,0.048,0.052,0.055,0.059,0.063,0.067,0.070,0.074,&
                    0.078,0.081,0.085,0.089,0.093,0.096,0.100,0.104,0.107,&
                    0.111,0.114,0.118,0.122,0.125,0.129,0.133,0.136,0.140,&
                    0.143,0.147,0.150,0.154,0.157,0.161,0.165,0.168,0.172,&
                    0.175,0.178,0.182,0.185,0.189,0.192,0.196,0.199,0.203,&
                    0.206,0.209,0.213,0.216,0.220,0.223,0.226,0.230,0.233,&
                    0.236,0.240,0.243,0.246,0.250,0.253,0.256,0.259,0.263,&
                    0.266,0.269,0.272,0.276,0.279,0.282,0.285,0.289,0.292,&
                    0.295,0.298,0.301,0.304,0.308,0.311,0.314,0.317,0.320,&
                    0.323,0.326,0.329,0.332,0.336,0.339,0.342,0.345,0.348,&
                    0.351,0.354,0.357,0.360,0.363,0.366,0.369,0.372,0.375,&
                    0.378,0.381,0.384,0.387,0.390,0.393,0.395,0.398,0.401,&
                    0.404,0.407,0.410,0.413,0.416,0.419,0.421,0.424,0.427,&
                    0.430,0.433,0.436,0.438,0.441,0.444,0.447,0.450,0.452,&
                    0.455,0.458,0.461,0.464,0.466,0.469,0.472,0.475,0.477,&
                    0.480,0.483,0.485,0.488,0.491,0.493,0.496,0.499,0.501,&
                    0.504,0.507,0.509,0.512,0.515,0.517,0.520,0.523,0.525,&
                    0.528,0.530,0.533,0.536,0.538,0.541,0.543,0.546,0.548,&
                    0.551,0.554,0.556,0.559,0.561,0.564,0.566,0.569,0.571,&
                    0.574,0.576,0.579,0.581,0.584,0.586,0.589,0.591,0.594,&
                    0.596,0.598,0.601,0.603,0.606,0.608,0.611,0.613,0.615,&
                    0.618,0.620,0.623,0.625,0.627,0.630,0.632,0.635,0.637,&
                    0.639,0.642,0.644,0.646,0.649,0.651,0.653,0.656,0.658,&
                    0.660,0.663,0.665,0.667,0.670,0.672,0.674,0.676,0.679,&
                    0.681,0.683,0.685,0.688,0.690,0.692,0.694,0.697,0.699,&
                    0.701,0.703,0.706,0.708,0.710,0.712,0.714,0.717,0.719,&
                    0.721,0.723,0.725,0.728,0.730,0.732,0.734,0.736,0.738,&
                    0.741,0.743,0.745,0.747,0.749,0.751,0.753,0.756,0.758,&
                    0.760,0.762,0.764,0.766,0.768,0.770,0.772,0.774,0.777,&
                    0.779,0.781,0.783,0.805,0.825,0.845,0.864,0.883,0.901,&
                    0.920,0.938,0.955,0.973,0.990,1.006,1.023,1.039,1.055,&
                    1.071,1.086,1.101,1.116,1.131,1.145,1.160,1.174,1.188,&
                    1.201,1.215,1.228,1.241,1.254,1.266,1.279,1.291,1.303,&
                    1.315,1.327,1.339,1.350,1.361,1.372,1.383,1.394,1.405,&
                    1.416,1.426,1.436,1.446,1.456,1.466,1.476,1.486,1.495,&
                    1.505,1.514,1.523,1.532,1.541,1.550,1.559,1.567,1.576,&
                    1.584,1.592,1.601,1.609,1.617,1.625,1.632,1.640,1.648,&
                    1.655,1.663,1.670,1.677,1.684,1.692,1.699,1.706,1.712,&
                    1.719,1.726,1.732,1.739,1.746,1.752,1.758,1.765,1.771,&
                    1.777,1.783,1.789,1.795,1.801,1.806,1.812,1.818,1.823,&
                    1.829,1.834,1.840,1.845,1.850,1.856,1.861,1.866,1.871,&
                    1.876,1.881,1.886,1.891,1.896,1.900,1.905,1.910,1.914,&
                    1.919,1.923,1.928,1.932,1.937,1.941,1.945,1.949,1.953,&
                    1.958,1.962,1.966,1.970,1.974,1.978,1.981,1.985,1.989,&
                    1.993,1.996,2.000,2.004,2.007,2.011,2.014,2.018,2.021,&
                    2.025,2.028,2.031,2.035,2.038,2.041,2.044,2.047,2.050,&
                    2.053,2.056,2.059,2.062,2.065,2.068,2.071,2.074,2.077,&
                    2.080,2.082/)
!
! *** (h ,no3)
!
    bnc10m(1:561) = (/-0.048,-0.100,-0.123,-0.137,-0.147,-0.155,-0.161,   &
                    -0.165,-0.169,-0.172,-0.174,-0.175,-0.177,-0.177,     &
                    -0.178,-0.178,-0.178,-0.177,-0.177,-0.176,-0.175,     &
                    -0.174,-0.173,-0.172,-0.171,-0.169,-0.168,-0.166,     &
                    -0.165,-0.163,-0.161,-0.159,-0.157,-0.156,-0.154,     &
                    -0.152,-0.150,-0.148,-0.146,-0.144,-0.141,-0.139,     &
                    -0.137,-0.135,-0.133,-0.131,-0.129,-0.127,-0.124,     &
                    -0.122,-0.120,-0.118,-0.116,-0.113,-0.111,-0.109,     &
                    -0.107,-0.105,-0.102,-0.100,-0.098,-0.096,-0.094,     &
                    -0.091,-0.089,-0.087,-0.085,-0.082,-0.080,-0.078,     &
                    -0.076,-0.073,-0.071,-0.069,-0.067,-0.064,-0.062,     &
                    -0.060,-0.057,-0.055,-0.052,-0.050,-0.048,-0.045,     &
                    -0.043,-0.040,-0.038,-0.035,-0.033,-0.030,-0.027,     &
                    -0.025,-0.022,-0.020,-0.017,-0.014,-0.012,-0.009,     &
                    -0.006,-0.004,-0.001,0.002,0.005,0.007,0.010,0.013,   &
                    0.016,0.019,0.021,0.024,0.027,0.030,0.033,0.036,0.039,&
                    0.041,0.044,0.047,0.050,0.053,0.056,0.059,0.062,0.065,&
                    0.068,0.071,0.073,0.076,0.079,0.082,0.085,0.088,0.091,&
                    0.094,0.097,0.100,0.103,0.105,0.108,0.111,0.114,0.117,&
                    0.120,0.123,0.126,0.129,0.131,0.134,0.137,0.140,0.143,&
                    0.146,0.149,0.152,0.154,0.157,0.160,0.163,0.166,0.169,&
                    0.171,0.174,0.177,0.180,0.183,0.186,0.188,0.191,0.194,&
                    0.197,0.200,0.202,0.205,0.208,0.211,0.213,0.216,0.219,&
                    0.222,0.224,0.227,0.230,0.233,0.235,0.238,0.241,0.244,&
                    0.246,0.249,0.252,0.254,0.257,0.260,0.263,0.265,0.268,&
                    0.271,0.273,0.276,0.279,0.281,0.284,0.287,0.289,0.292,&
                    0.294,0.297,0.300,0.302,0.305,0.308,0.310,0.313,0.315,&
                    0.318,0.321,0.323,0.326,0.328,0.331,0.333,0.336,0.339,&
                    0.341,0.344,0.346,0.349,0.351,0.354,0.356,0.359,0.361,&
                    0.364,0.366,0.369,0.372,0.374,0.376,0.379,0.381,0.384,&
                    0.386,0.389,0.391,0.394,0.396,0.399,0.401,0.404,0.406,&
                    0.409,0.411,0.413,0.416,0.418,0.421,0.423,0.425,0.428,&
                    0.430,0.433,0.435,0.437,0.440,0.442,0.445,0.447,0.449,&
                    0.452,0.454,0.456,0.459,0.461,0.463,0.466,0.468,0.470,&
                    0.473,0.475,0.477,0.480,0.482,0.484,0.486,0.489,0.491,&
                    0.493,0.496,0.498,0.500,0.502,0.505,0.507,0.509,0.511,&
                    0.514,0.516,0.518,0.520,0.523,0.525,0.527,0.529,0.531,&
                    0.534,0.536,0.538,0.540,0.542,0.545,0.547,0.549,0.551,&
                    0.553,0.555,0.558,0.560,0.562,0.564,0.566,0.568,0.570,&
                    0.573,0.575,0.577,0.579,0.581,0.583,0.585,0.587,0.590,&
                    0.592,0.594,0.596,0.598,0.600,0.602,0.604,0.606,0.608,&
                    0.610,0.612,0.614,0.617,0.619,0.621,0.623,0.625,0.627,&
                    0.629,0.631,0.633,0.635,0.637,0.639,0.641,0.643,0.645,&
                    0.647,0.649,0.651,0.653,0.655,0.657,0.659,0.661,0.663,&
                    0.665,0.667,0.669,0.671,0.673,0.674,0.676,0.678,0.680,&
                    0.682,0.684,0.686,0.688,0.690,0.692,0.694,0.696,0.698,&
                    0.700,0.701,0.703,0.705,0.707,0.709,0.711,0.713,0.715,&
                    0.716,0.718,0.720,0.722,0.724,0.726,0.746,0.764,0.781,&
                    0.799,0.816,0.833,0.849,0.866,0.882,0.897,0.913,0.928,&
                    0.943,0.958,0.972,0.987,1.001,1.015,1.028,1.042,1.055,&
                    1.068,1.081,1.093,1.106,1.118,1.130,1.142,1.154,1.165,&
                    1.177,1.188,1.199,1.210,1.221,1.231,1.242,1.252,1.262,&
                    1.272,1.282,1.292,1.301,1.311,1.320,1.329,1.338,1.347,&
                    1.356,1.365,1.374,1.382,1.391,1.399,1.407,1.415,1.423,&
                    1.431,1.439,1.446,1.454,1.462,1.469,1.476,1.483,1.491,&
                    1.498,1.505,1.511,1.518,1.525,1.531,1.538,1.544,1.551,&
                    1.557,1.563,1.569,1.576,1.582,1.587,1.593,1.599,1.605,&
                    1.610,1.616,1.622,1.627,1.632,1.638,1.643,1.648,1.653,&
                    1.658,1.663,1.668,1.673,1.678,1.683,1.688,1.692,1.697,&
                    1.701,1.706,1.710,1.715,1.719,1.723,1.728,1.732,1.736,&
                    1.740,1.744,1.748,1.752,1.756,1.760,1.764,1.768,1.771,&
                    1.775,1.779,1.782,1.786,1.789,1.793,1.796,1.800,1.803,&
                    1.806,1.810,1.813,1.816,1.819,1.822,1.825,1.828,1.831,&
                    1.834,1.837,1.840,1.843,1.846,1.849,1.852,1.854,1.857,&
                    1.860,1.862,1.865,1.867,1.870,1.872,1.875,1.877,1.880,&
                    1.882,1.885,1.887,1.889,1.891/)
!
! *** (h ,cl)
!
    bnc11m(1:561) = (/-0.047,-0.093,-0.111,-0.121,-0.127,-0.131,-0.133,   &
                    -0.134,-0.133,-0.132,-0.131,-0.128,-0.126,-0.123,     &
                    -0.119,-0.115,-0.111,-0.107,-0.102,-0.098,-0.093,     &
                    -0.088,-0.082,-0.077,-0.071,-0.065,-0.060,-0.054,     &
                    -0.048,-0.041,-0.035,-0.029,-0.022,-0.016,-0.009,     &
                    -0.003,0.004,0.011,0.018,0.025,0.032,0.038,0.045,     &
                    0.053,0.060,0.067,0.074,0.081,0.088,0.095,0.103,0.110,&
                    0.117,0.124,0.132,0.139,0.146,0.154,0.161,0.168,0.176,&
                    0.183,0.191,0.198,0.206,0.213,0.221,0.228,0.236,0.243,&
                    0.251,0.258,0.266,0.274,0.281,0.289,0.297,0.305,0.312,&
                    0.320,0.328,0.336,0.344,0.352,0.360,0.368,0.376,0.384,&
                    0.393,0.401,0.409,0.417,0.426,0.434,0.443,0.451,0.460,&
                    0.468,0.477,0.485,0.494,0.503,0.511,0.520,0.529,0.538,&
                    0.546,0.555,0.564,0.573,0.582,0.591,0.600,0.608,0.617,&
                    0.626,0.635,0.644,0.653,0.662,0.671,0.680,0.689,0.698,&
                    0.707,0.716,0.725,0.734,0.743,0.752,0.761,0.770,0.779,&
                    0.788,0.796,0.805,0.814,0.823,0.832,0.841,0.850,0.859,&
                    0.867,0.876,0.885,0.894,0.903,0.911,0.920,0.929,0.937,&
                    0.946,0.955,0.964,0.972,0.981,0.989,0.998,1.007,1.015,&
                    1.024,1.032,1.041,1.049,1.058,1.066,1.075,1.083,1.092,&
                    1.100,1.109,1.117,1.125,1.134,1.142,1.150,1.159,1.167,&
                    1.175,1.183,1.192,1.200,1.208,1.216,1.224,1.233,1.241,&
                    1.249,1.257,1.265,1.273,1.281,1.289,1.297,1.305,1.313,&
                    1.321,1.329,1.337,1.345,1.353,1.361,1.369,1.377,1.384,&
                    1.392,1.400,1.408,1.416,1.423,1.431,1.439,1.447,1.454,&
                    1.462,1.470,1.477,1.485,1.493,1.500,1.508,1.515,1.523,&
                    1.530,1.538,1.545,1.553,1.560,1.568,1.575,1.583,1.590,&
                    1.598,1.605,1.612,1.620,1.627,1.634,1.642,1.649,1.656,&
                    1.663,1.671,1.678,1.685,1.692,1.699,1.707,1.714,1.721,&
                    1.728,1.735,1.742,1.749,1.756,1.763,1.770,1.778,1.785,&
                    1.791,1.798,1.805,1.812,1.819,1.826,1.833,1.840,1.847,&
                    1.854,1.861,1.867,1.874,1.881,1.888,1.895,1.901,1.908,&
                    1.915,1.922,1.928,1.935,1.942,1.948,1.955,1.962,1.968,&
                    1.975,1.981,1.988,1.995,2.001,2.008,2.014,2.021,2.027,&
                    2.034,2.040,2.047,2.053,2.060,2.066,2.072,2.079,2.085,&
                    2.092,2.098,2.104,2.111,2.117,2.123,2.130,2.136,2.142,&
                    2.148,2.155,2.161,2.167,2.173,2.180,2.186,2.192,2.198,&
                    2.204,2.210,2.216,2.223,2.229,2.235,2.241,2.247,2.253,&
                    2.259,2.265,2.271,2.277,2.283,2.289,2.295,2.301,2.307,&
                    2.313,2.319,2.325,2.331,2.337,2.343,2.348,2.354,2.360,&
                    2.366,2.372,2.378,2.383,2.389,2.395,2.401,2.407,2.412,&
                    2.418,2.424,2.430,2.435,2.441,2.447,2.452,2.458,2.464,&
                    2.469,2.475,2.481,2.486,2.492,2.497,2.503,2.509,2.514,&
                    2.520,2.525,2.531,2.536,2.542,2.547,2.553,2.558,2.564,&
                    2.569,2.575,2.580,2.586,2.591,2.597,2.602,2.607,2.613,&
                    2.618,2.624,2.629,2.634,2.640,2.645,2.702,2.754,2.805,&
                    2.855,2.904,2.953,3.001,3.048,3.094,3.139,3.184,3.228,&
                    3.272,3.315,3.357,3.399,3.440,3.480,3.520,3.559,3.598,&
                    3.636,3.674,3.711,3.748,3.784,3.820,3.855,3.890,3.925,&
                    3.959,3.992,4.026,4.058,4.091,4.123,4.155,4.186,4.217,&
                    4.247,4.278,4.308,4.337,4.366,4.395,4.424,4.452,4.480,&
                    4.508,4.535,4.562,4.589,4.616,4.642,4.668,4.694,4.719,&
                    4.745,4.770,4.794,4.819,4.843,4.867,4.891,4.915,4.938,&
                    4.961,4.984,5.007,5.029,5.052,5.074,5.096,5.117,5.139,&
                    5.160,5.181,5.202,5.223,5.244,5.264,5.285,5.305,5.324,&
                    5.344,5.364,5.383,5.402,5.422,5.441,5.459,5.478,5.496,&
                    5.515,5.533,5.551,5.569,5.587,5.604,5.622,5.639,5.656,&
                    5.673,5.690,5.707,5.724,5.740,5.757,5.773,5.789,5.806,&
                    5.821,5.837,5.853,5.869,5.884,5.900,5.915,5.930,5.945,&
                    5.960,5.975,5.990,6.004,6.019,6.033,6.048,6.062,6.076,&
                    6.090,6.104,6.118,6.132,6.145,6.159,6.172,6.186,6.199,&
                    6.212,6.225,6.238,6.251,6.264,6.277,6.290,6.302,6.315,&
                    6.327,6.340,6.352,6.364,6.377,6.389,6.401,6.413,6.424,&
                    6.436,6.448,6.459,6.471,6.482/)
!
! *** nahso4
!
    bnc12m(1:561) = (/-0.048,-0.099,-0.122,-0.137,-0.148,-0.156,-0.163,   &
                    -0.168,-0.172,-0.175,-0.178,-0.180,-0.182,-0.183,     &
                    -0.184,-0.184,-0.184,-0.184,-0.184,-0.183,-0.182,     &
                    -0.181,-0.180,-0.179,-0.177,-0.176,-0.174,-0.172,     &
                    -0.170,-0.167,-0.165,-0.163,-0.160,-0.157,-0.155,     &
                    -0.152,-0.149,-0.146,-0.143,-0.139,-0.136,-0.133,     &
                    -0.129,-0.126,-0.122,-0.118,-0.115,-0.111,-0.107,     &
                    -0.103,-0.099,-0.095,-0.091,-0.087,-0.083,-0.079,     &
                    -0.075,-0.071,-0.066,-0.062,-0.058,-0.053,-0.049,     &
                    -0.044,-0.040,-0.035,-0.031,-0.026,-0.022,-0.017,     &
                    -0.012,-0.007,-0.003,0.002,0.007,0.012,0.017,0.022,   &
                    0.027,0.032,0.037,0.042,0.047,0.052,0.058,0.063,0.068,&
                    0.073,0.079,0.084,0.090,0.095,0.101,0.106,0.112,0.117,&
                    0.123,0.128,0.134,0.140,0.146,0.151,0.157,0.163,0.169,&
                    0.175,0.180,0.186,0.192,0.198,0.204,0.210,0.216,0.222,&
                    0.228,0.234,0.240,0.246,0.252,0.258,0.264,0.270,0.276,&
                    0.282,0.288,0.294,0.300,0.306,0.312,0.318,0.323,0.329,&
                    0.335,0.341,0.347,0.353,0.359,0.365,0.371,0.377,0.383,&
                    0.389,0.394,0.400,0.406,0.412,0.418,0.424,0.429,0.435,&
                    0.441,0.447,0.453,0.458,0.464,0.470,0.476,0.481,0.487,&
                    0.493,0.498,0.504,0.510,0.515,0.521,0.526,0.532,0.538,&
                    0.543,0.549,0.554,0.560,0.565,0.571,0.576,0.582,0.587,&
                    0.593,0.598,0.604,0.609,0.615,0.620,0.625,0.631,0.636,&
                    0.641,0.647,0.652,0.657,0.663,0.668,0.673,0.679,0.684,&
                    0.689,0.694,0.700,0.705,0.710,0.715,0.720,0.726,0.731,&
                    0.736,0.741,0.746,0.751,0.756,0.761,0.766,0.772,0.777,&
                    0.782,0.787,0.792,0.797,0.802,0.807,0.812,0.817,0.822,&
                    0.826,0.831,0.836,0.841,0.846,0.851,0.856,0.861,0.866,&
                    0.870,0.875,0.880,0.885,0.890,0.894,0.899,0.904,0.909,&
                    0.914,0.918,0.923,0.928,0.932,0.937,0.942,0.946,0.951,&
                    0.956,0.960,0.965,0.970,0.974,0.979,0.984,0.988,0.993,&
                    0.997,1.002,1.006,1.011,1.015,1.020,1.024,1.029,1.033,&
                    1.038,1.042,1.047,1.051,1.056,1.060,1.065,1.069,1.073,&
                    1.078,1.082,1.087,1.091,1.095,1.100,1.104,1.108,1.113,&
                    1.117,1.121,1.126,1.130,1.134,1.138,1.143,1.147,1.151,&
                    1.155,1.160,1.164,1.168,1.172,1.177,1.181,1.185,1.189,&
                    1.193,1.197,1.202,1.206,1.210,1.214,1.218,1.222,1.226,&
                    1.230,1.234,1.238,1.243,1.247,1.251,1.255,1.259,1.263,&
                    1.267,1.271,1.275,1.279,1.283,1.287,1.291,1.295,1.299,&
                    1.303,1.306,1.310,1.314,1.318,1.322,1.326,1.330,1.334,&
                    1.338,1.342,1.345,1.349,1.353,1.357,1.361,1.365,1.368,&
                    1.372,1.376,1.380,1.384,1.387,1.391,1.395,1.399,1.403,&
                    1.406,1.410,1.414,1.418,1.421,1.425,1.429,1.432,1.436,&
                    1.440,1.443,1.447,1.451,1.454,1.458,1.462,1.465,1.469,&
                    1.473,1.476,1.480,1.484,1.487,1.491,1.494,1.498,1.501,&
                    1.505,1.509,1.512,1.516,1.519,1.523,1.526,1.530,1.533,&
                    1.537,1.540,1.544,1.547,1.551,1.554,1.558,1.595,1.629,&
                    1.662,1.695,1.727,1.759,1.790,1.820,1.850,1.880,1.909,&
                    1.938,1.966,1.994,2.022,2.049,2.076,2.102,2.128,2.153,&
                    2.179,2.203,2.228,2.252,2.276,2.300,2.323,2.346,2.368,&
                    2.391,2.413,2.435,2.456,2.477,2.498,2.519,2.539,2.560,&
                    2.580,2.599,2.619,2.638,2.657,2.676,2.695,2.713,2.731,&
                    2.749,2.767,2.785,2.802,2.819,2.836,2.853,2.870,2.886,&
                    2.902,2.918,2.934,2.950,2.966,2.981,2.997,3.012,3.027,&
                    3.042,3.056,3.071,3.085,3.099,3.114,3.128,3.141,3.155,&
                    3.169,3.182,3.195,3.209,3.222,3.235,3.247,3.260,3.273,&
                    3.285,3.298,3.310,3.322,3.334,3.346,3.358,3.369,3.381,&
                    3.392,3.404,3.415,3.426,3.437,3.448,3.459,3.470,3.481,&
                    3.491,3.502,3.512,3.523,3.533,3.543,3.553,3.563,3.573,&
                    3.583,3.593,3.603,3.612,3.622,3.631,3.640,3.650,3.659,&
                    3.668,3.677,3.686,3.695,3.704,3.713,3.721,3.730,3.739,&
                    3.747,3.756,3.764,3.772,3.781,3.789,3.797,3.805,3.813,&
                    3.821,3.829,3.836,3.844,3.852,3.860,3.867,3.875,3.882,&
                    3.889,3.897,3.904,3.911,3.919,3.926,3.933,3.940,3.947,&
                    3.954,3.961,3.967,3.974,3.981,3.988/)
!
! *** (nh4)3h(so4)2
!
    bnc13m(1:561) = (/-0.080,-0.174,-0.221,-0.254,-0.280,-0.302,-0.321,   &
                    -0.337,-0.352,-0.365,-0.377,-0.388,-0.398,-0.408,     &
                    -0.417,-0.425,-0.433,-0.440,-0.447,-0.454,-0.461,     &
                    -0.467,-0.472,-0.478,-0.483,-0.488,-0.493,-0.498,     &
                    -0.503,-0.507,-0.511,-0.515,-0.519,-0.523,-0.527,     &
                    -0.530,-0.533,-0.537,-0.540,-0.543,-0.546,-0.549,     &
                    -0.551,-0.554,-0.557,-0.559,-0.562,-0.564,-0.566,     &
                    -0.569,-0.571,-0.573,-0.575,-0.577,-0.579,-0.580,     &
                    -0.582,-0.584,-0.586,-0.587,-0.589,-0.590,-0.592,     &
                    -0.593,-0.595,-0.596,-0.597,-0.599,-0.600,-0.601,     &
                    -0.602,-0.603,-0.604,-0.605,-0.607,-0.608,-0.609,     &
                    -0.609,-0.610,-0.611,-0.612,-0.613,-0.614,-0.614,     &
                    -0.615,-0.616,-0.617,-0.617,-0.618,-0.619,-0.619,     &
                    -0.620,-0.620,-0.621,-0.621,-0.622,-0.622,-0.623,     &
                    -0.623,-0.624,-0.624,-0.624,-0.625,-0.625,-0.626,     &
                    -0.626,-0.626,-0.626,-0.627,-0.627,-0.627,-0.628,     &
                    -0.628,-0.628,-0.628,-0.628,-0.629,-0.629,-0.629,     &
                    -0.629,-0.629,-0.629,-0.630,-0.630,-0.630,-0.630,     &
                    -0.630,-0.630,-0.630,-0.630,-0.630,-0.631,-0.631,     &
                    -0.631,-0.631,-0.631,-0.631,-0.631,-0.631,-0.631,     &
                    -0.631,-0.631,-0.631,-0.631,-0.631,-0.631,-0.631,     &
                    -0.631,-0.631,-0.631,-0.631,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.632,-0.632,     &
                    -0.632,-0.632,-0.632,-0.632,-0.632,-0.633,-0.633,     &
                    -0.633,-0.633,-0.633,-0.633,-0.633,-0.633,-0.633,     &
                    -0.633,-0.633,-0.633,-0.633,-0.633,-0.633,-0.633,     &
                    -0.633,-0.633,-0.633,-0.633,-0.633,-0.633,-0.633,     &
                    -0.633,-0.634,-0.634,-0.634,-0.634,-0.634,-0.634,     &
                    -0.634,-0.634,-0.634,-0.634,-0.634,-0.634,-0.634,     &
                    -0.634,-0.634,-0.634,-0.634,-0.634,-0.635,-0.635,     &
                    -0.635,-0.635,-0.635,-0.635,-0.635,-0.635,-0.635,     &
                    -0.635,-0.635,-0.635,-0.635,-0.635,-0.636,-0.636,     &
                    -0.636,-0.636,-0.636,-0.636,-0.636,-0.636,-0.636,     &
                    -0.636,-0.636,-0.636,-0.637,-0.637,-0.637,-0.637,     &
                    -0.637,-0.637,-0.637,-0.637,-0.637,-0.637,-0.637,     &
                    -0.638,-0.638,-0.638,-0.638,-0.638,-0.638,-0.638,     &
                    -0.638,-0.638,-0.638,-0.639,-0.639,-0.639,-0.639,     &
                    -0.639,-0.639,-0.639,-0.639,-0.639,-0.639,-0.640,     &
                    -0.640,-0.640,-0.640,-0.640,-0.640,-0.640,-0.640,     &
                    -0.640,-0.641,-0.641,-0.641,-0.641,-0.641,-0.641,     &
                    -0.641,-0.641,-0.642,-0.642,-0.642,-0.642,-0.642,     &
                    -0.642,-0.642,-0.642,-0.643,-0.643,-0.643,-0.643,     &
                    -0.643,-0.643,-0.643,-0.643,-0.644,-0.644,-0.644,     &
                    -0.644,-0.644,-0.644,-0.644,-0.645,-0.645,-0.645,     &
                    -0.645,-0.645,-0.645,-0.645,-0.646,-0.646,-0.646,     &
                    -0.646,-0.648,-0.649,-0.651,-0.652,-0.654,-0.656,     &
                    -0.658,-0.660,-0.662,-0.664,-0.666,-0.668,-0.670,     &
                    -0.673,-0.675,-0.677,-0.680,-0.682,-0.685,-0.687,     &
                    -0.690,-0.693,-0.695,-0.698,-0.701,-0.704,-0.707,     &
                    -0.709,-0.712,-0.715,-0.718,-0.722,-0.725,-0.728,     &
                    -0.731,-0.734,-0.737,-0.741,-0.744,-0.747,-0.751,     &
                    -0.754,-0.758,-0.761,-0.764,-0.768,-0.772,-0.775,     &
                    -0.779,-0.782,-0.786,-0.790,-0.793,-0.797,-0.801,     &
                    -0.805,-0.809,-0.812,-0.816,-0.820,-0.824,-0.828,     &
                    -0.832,-0.836,-0.840,-0.844,-0.848,-0.852,-0.856,     &
                    -0.860,-0.864,-0.868,-0.873,-0.877,-0.881,-0.885,     &
                    -0.889,-0.894,-0.898,-0.902,-0.906,-0.911,-0.915,     &
                    -0.919,-0.924,-0.928,-0.932,-0.937,-0.941,-0.946,     &
                    -0.950,-0.955,-0.959,-0.964,-0.968,-0.973,-0.977,     &
                    -0.982,-0.986,-0.991,-0.995,-1.000,-1.005,-1.009,     &
                    -1.014,-1.019,-1.023,-1.028,-1.033,-1.037,-1.042,     &
                    -1.047,-1.051,-1.056,-1.061,-1.066,-1.071,-1.075,     &
                    -1.080,-1.085,-1.090,-1.095,-1.099,-1.104,-1.109,     &
                    -1.114,-1.119,-1.124,-1.129,-1.134,-1.138,-1.143,     &
                    -1.148,-1.153,-1.158,-1.163,-1.168,-1.173,-1.178,     &
                    -1.183,-1.188,-1.193,-1.198,-1.203,-1.208,-1.213,     &
                    -1.218,-1.223,-1.228,-1.233,-1.238,-1.244,-1.249,     &
                    -1.254,-1.259,-1.264,-1.269,-1.274,-1.279,-1.284,     &
                    -1.290/)
!
! *** caso4
!
    bnc14m(1:561) = (/0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,    &
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000/)
!
! *** cano32
!
    bnc15m(1:561) = (/-0.099,-0.211,-0.264,-0.301,-0.329,-0.351,-0.370,   &
                    -0.386,-0.399,-0.411,-0.422,-0.432,-0.440,-0.448,     &
                    -0.455,-0.462,-0.468,-0.473,-0.478,-0.483,-0.487,     &
                    -0.491,-0.495,-0.499,-0.502,-0.505,-0.508,-0.511,     &
                    -0.513,-0.516,-0.518,-0.520,-0.522,-0.524,-0.526,     &
                    -0.528,-0.529,-0.531,-0.532,-0.534,-0.535,-0.536,     &
                    -0.537,-0.539,-0.540,-0.541,-0.542,-0.543,-0.544,     &
                    -0.545,-0.545,-0.546,-0.547,-0.548,-0.549,-0.549,     &
                    -0.550,-0.551,-0.551,-0.552,-0.552,-0.553,-0.553,     &
                    -0.554,-0.554,-0.555,-0.555,-0.556,-0.556,-0.556,     &
                    -0.557,-0.557,-0.557,-0.558,-0.558,-0.558,-0.558,     &
                    -0.559,-0.559,-0.559,-0.559,-0.559,-0.559,-0.559,     &
                    -0.559,-0.559,-0.559,-0.559,-0.559,-0.559,-0.559,     &
                    -0.559,-0.559,-0.559,-0.558,-0.558,-0.558,-0.558,     &
                    -0.557,-0.557,-0.557,-0.556,-0.556,-0.556,-0.555,     &
                    -0.555,-0.555,-0.554,-0.554,-0.553,-0.553,-0.552,     &
                    -0.552,-0.551,-0.551,-0.550,-0.550,-0.549,-0.549,     &
                    -0.548,-0.548,-0.547,-0.547,-0.546,-0.545,-0.545,     &
                    -0.544,-0.544,-0.543,-0.542,-0.542,-0.541,-0.541,     &
                    -0.540,-0.539,-0.539,-0.538,-0.538,-0.537,-0.536,     &
                    -0.536,-0.535,-0.534,-0.534,-0.533,-0.533,-0.532,     &
                    -0.531,-0.531,-0.530,-0.529,-0.529,-0.528,-0.527,     &
                    -0.527,-0.526,-0.525,-0.525,-0.524,-0.524,-0.523,     &
                    -0.522,-0.522,-0.521,-0.520,-0.520,-0.519,-0.518,     &
                    -0.518,-0.517,-0.516,-0.516,-0.515,-0.514,-0.514,     &
                    -0.513,-0.513,-0.512,-0.511,-0.511,-0.510,-0.509,     &
                    -0.509,-0.508,-0.507,-0.507,-0.506,-0.505,-0.505,     &
                    -0.504,-0.504,-0.503,-0.502,-0.502,-0.501,-0.500,     &
                    -0.500,-0.499,-0.499,-0.498,-0.497,-0.497,-0.496,     &
                    -0.495,-0.495,-0.494,-0.494,-0.493,-0.492,-0.492,     &
                    -0.491,-0.490,-0.490,-0.489,-0.489,-0.488,-0.487,     &
                    -0.487,-0.486,-0.486,-0.485,-0.484,-0.484,-0.483,     &
                    -0.483,-0.482,-0.481,-0.481,-0.480,-0.480,-0.479,     &
                    -0.478,-0.478,-0.477,-0.477,-0.476,-0.475,-0.475,     &
                    -0.474,-0.474,-0.473,-0.473,-0.472,-0.471,-0.471,     &
                    -0.470,-0.470,-0.469,-0.469,-0.468,-0.468,-0.467,     &
                    -0.466,-0.466,-0.465,-0.465,-0.464,-0.464,-0.463,     &
                    -0.463,-0.462,-0.461,-0.461,-0.460,-0.460,-0.459,     &
                    -0.459,-0.458,-0.458,-0.457,-0.457,-0.456,-0.456,     &
                    -0.455,-0.455,-0.454,-0.453,-0.453,-0.452,-0.452,     &
                    -0.451,-0.451,-0.450,-0.450,-0.449,-0.449,-0.448,     &
                    -0.448,-0.447,-0.447,-0.446,-0.446,-0.445,-0.445,     &
                    -0.444,-0.444,-0.443,-0.443,-0.442,-0.442,-0.441,     &
                    -0.441,-0.440,-0.440,-0.440,-0.439,-0.439,-0.438,     &
                    -0.438,-0.437,-0.437,-0.436,-0.436,-0.435,-0.435,     &
                    -0.434,-0.434,-0.433,-0.433,-0.433,-0.432,-0.432,     &
                    -0.431,-0.431,-0.430,-0.430,-0.429,-0.429,-0.429,     &
                    -0.428,-0.428,-0.427,-0.427,-0.426,-0.426,-0.426,     &
                    -0.425,-0.425,-0.424,-0.424,-0.423,-0.423,-0.423,     &
                    -0.422,-0.422,-0.421,-0.421,-0.421,-0.420,-0.420,     &
                    -0.419,-0.419,-0.419,-0.418,-0.418,-0.417,-0.417,     &
                    -0.417,-0.416,-0.416,-0.415,-0.415,-0.415,-0.414,     &
                    -0.414,-0.413,-0.413,-0.413,-0.412,-0.412,-0.412,     &
                    -0.411,-0.411,-0.410,-0.410,-0.410,-0.409,-0.409,     &
                    -0.409,-0.408,-0.408,-0.408,-0.407,-0.407,-0.407,     &
                    -0.406,-0.406,-0.405,-0.405,-0.405,-0.404,-0.404,     &
                    -0.404,-0.403,-0.403,-0.403,-0.402,-0.402,-0.402,     &
                    -0.401,-0.398,-0.395,-0.392,-0.389,-0.386,-0.384,     &
                    -0.381,-0.379,-0.377,-0.375,-0.373,-0.371,-0.369,     &
                    -0.368,-0.366,-0.365,-0.364,-0.363,-0.362,-0.361,     &
                    -0.360,-0.359,-0.358,-0.358,-0.357,-0.357,-0.357,     &
                    -0.356,-0.356,-0.356,-0.356,-0.356,-0.356,-0.357,     &
                    -0.357,-0.357,-0.358,-0.358,-0.359,-0.360,-0.361,     &
                    -0.361,-0.362,-0.363,-0.364,-0.365,-0.367,-0.368,     &
                    -0.369,-0.370,-0.372,-0.373,-0.375,-0.376,-0.378,     &
                    -0.380,-0.381,-0.383,-0.385,-0.387,-0.389,-0.391,     &
                    -0.393,-0.395,-0.397,-0.399,-0.401,-0.404,-0.406,     &
                    -0.408,-0.411,-0.413,-0.416,-0.418,-0.421,-0.423,     &
                    -0.426,-0.429,-0.431,-0.434,-0.437,-0.440,-0.442,     &
                    -0.445,-0.448,-0.451,-0.454,-0.457,-0.460,-0.463,     &
                    -0.467,-0.470,-0.473,-0.476,-0.480,-0.483,-0.486,     &
                    -0.490,-0.493,-0.496,-0.500,-0.503,-0.507,-0.510,     &
                    -0.514,-0.517,-0.521,-0.525,-0.528,-0.532,-0.536,     &
                    -0.540,-0.543,-0.547,-0.551,-0.555,-0.559,-0.563,     &
                    -0.566,-0.570,-0.574,-0.578,-0.582,-0.586,-0.590,     &
                    -0.595,-0.599,-0.603,-0.607,-0.611,-0.615,-0.619,     &
                    -0.624,-0.628,-0.632,-0.637,-0.641,-0.645,-0.650,     &
                    -0.654,-0.658,-0.663,-0.667,-0.672,-0.676,-0.680,     &
                    -0.685,-0.689,-0.694,-0.699,-0.703,-0.708,-0.712,     &
                    -0.717,-0.722,-0.726,-0.731,-0.736,-0.740,-0.745,     &
                    -0.750/)
!
! *** cacl2
!
    bnc16m(1:561) = (/-0.097,-0.201,-0.247,-0.277,-0.298,-0.314,-0.327,   &
                    -0.337,-0.345,-0.351,-0.356,-0.359,-0.362,-0.365,     &
                    -0.366,-0.367,-0.367,-0.367,-0.367,-0.366,-0.365,     &
                    -0.364,-0.362,-0.361,-0.359,-0.356,-0.354,-0.352,     &
                    -0.349,-0.346,-0.343,-0.340,-0.337,-0.334,-0.331,     &
                    -0.328,-0.324,-0.321,-0.317,-0.314,-0.310,-0.307,     &
                    -0.303,-0.300,-0.296,-0.292,-0.289,-0.285,-0.281,     &
                    -0.277,-0.274,-0.270,-0.266,-0.262,-0.259,-0.255,     &
                    -0.251,-0.247,-0.244,-0.240,-0.236,-0.232,-0.228,     &
                    -0.224,-0.221,-0.217,-0.213,-0.209,-0.205,-0.201,     &
                    -0.197,-0.193,-0.189,-0.185,-0.181,-0.177,-0.173,     &
                    -0.169,-0.165,-0.161,-0.157,-0.152,-0.148,-0.144,     &
                    -0.139,-0.135,-0.131,-0.126,-0.122,-0.117,-0.113,     &
                    -0.108,-0.104,-0.099,-0.095,-0.090,-0.085,-0.080,     &
                    -0.076,-0.071,-0.066,-0.061,-0.056,-0.051,-0.047,     &
                    -0.042,-0.037,-0.032,-0.027,-0.022,-0.017,-0.012,     &
                    -0.006,-0.001,0.004,0.009,0.014,0.019,0.024,0.029,    &
                    0.034,0.040,0.045,0.050,0.055,0.060,0.065,0.071,0.076,&
                    0.081,0.086,0.091,0.096,0.102,0.107,0.112,0.117,0.122,&
                    0.127,0.133,0.138,0.143,0.148,0.153,0.158,0.163,0.168,&
                    0.174,0.179,0.184,0.189,0.194,0.199,0.204,0.209,0.214,&
                    0.219,0.224,0.229,0.234,0.239,0.244,0.249,0.254,0.259,&
                    0.264,0.269,0.274,0.279,0.284,0.289,0.294,0.299,0.304,&
                    0.309,0.314,0.319,0.324,0.329,0.334,0.338,0.343,0.348,&
                    0.353,0.358,0.363,0.368,0.372,0.377,0.382,0.387,0.392,&
                    0.396,0.401,0.406,0.411,0.415,0.420,0.425,0.430,0.434,&
                    0.439,0.444,0.449,0.453,0.458,0.463,0.467,0.472,0.477,&
                    0.481,0.486,0.491,0.495,0.500,0.504,0.509,0.514,0.518,&
                    0.523,0.527,0.532,0.536,0.541,0.545,0.550,0.555,0.559,&
                    0.564,0.568,0.573,0.577,0.581,0.586,0.590,0.595,0.599,&
                    0.604,0.608,0.613,0.617,0.621,0.626,0.630,0.635,0.639,&
                    0.643,0.648,0.652,0.656,0.661,0.665,0.669,0.674,0.678,&
                    0.682,0.686,0.691,0.695,0.699,0.703,0.708,0.712,0.716,&
                    0.720,0.725,0.729,0.733,0.737,0.741,0.746,0.750,0.754,&
                    0.758,0.762,0.766,0.770,0.775,0.779,0.783,0.787,0.791,&
                    0.795,0.799,0.803,0.807,0.811,0.815,0.819,0.823,0.827,&
                    0.831,0.835,0.839,0.843,0.847,0.851,0.855,0.859,0.863,&
                    0.867,0.871,0.875,0.879,0.883,0.887,0.891,0.895,0.899,&
                    0.902,0.906,0.910,0.914,0.918,0.922,0.926,0.929,0.933,&
                    0.937,0.941,0.945,0.949,0.952,0.956,0.960,0.964,0.967,&
                    0.971,0.975,0.979,0.982,0.986,0.990,0.994,0.997,1.001,&
                    1.005,1.008,1.012,1.016,1.019,1.023,1.027,1.030,1.034,&
                    1.038,1.041,1.045,1.049,1.052,1.056,1.059,1.063,1.067,&
                    1.070,1.074,1.077,1.081,1.084,1.088,1.091,1.095,1.098,&
                    1.102,1.106,1.109,1.113,1.116,1.119,1.123,1.126,1.130,&
                    1.133,1.137,1.140,1.144,1.147,1.151,1.154,1.157,1.161,&
                    1.164,1.168,1.171,1.174,1.178,1.181,1.185,1.188,1.191,&
                    1.195,1.198,1.201,1.205,1.208,1.211,1.215,1.218,1.221,&
                    1.225,1.260,1.292,1.323,1.354,1.385,1.415,1.444,1.473,&
                    1.501,1.529,1.557,1.584,1.610,1.636,1.662,1.687,1.712,&
                    1.737,1.761,1.784,1.808,1.831,1.853,1.876,1.898,1.919,&
                    1.940,1.961,1.982,2.002,2.022,2.042,2.062,2.081,2.100,&
                    2.118,2.137,2.155,2.172,2.190,2.207,2.224,2.241,2.258,&
                    2.274,2.290,2.306,2.322,2.337,2.353,2.368,2.382,2.397,&
                    2.412,2.426,2.440,2.454,2.467,2.481,2.494,2.507,2.520,&
                    2.533,2.546,2.558,2.571,2.583,2.595,2.606,2.618,2.630,&
                    2.641,2.652,2.663,2.674,2.685,2.696,2.706,2.717,2.727,&
                    2.737,2.747,2.757,2.767,2.776,2.786,2.795,2.804,2.813,&
                    2.822,2.831,2.840,2.849,2.857,2.866,2.874,2.882,2.891,&
                    2.899,2.907,2.914,2.922,2.930,2.937,2.945,2.952,2.959,&
                    2.966,2.973,2.980,2.987,2.994,3.001,3.007,3.014,3.020,&
                    3.027,3.033,3.039,3.045,3.051,3.057,3.063,3.069,3.075,&
                    3.081,3.086,3.092,3.097,3.102,3.108,3.113,3.118,3.123,&
                    3.128,3.133,3.138,3.143,3.147,3.152,3.157,3.161,3.166,&
                    3.170,3.175,3.179,3.183,3.187,3.191,3.195,3.199,3.203,&
                    3.207,3.211,3.215,3.219,3.222,3.226,3.229,3.233,      &
                    3.236/)
!
! *** k2so4
!
    bnc17m(1:561) = (/-0.101,-0.220,-0.281,-0.324,-0.358,-0.387,-0.412,   &
                    -0.434,-0.453,-0.471,-0.487,-0.503,-0.517,-0.530,     &
                    -0.543,-0.555,-0.566,-0.577,-0.587,-0.597,-0.606,     &
                    -0.615,-0.624,-0.632,-0.641,-0.649,-0.656,-0.664,     &
                    -0.671,-0.678,-0.685,-0.692,-0.698,-0.705,-0.711,     &
                    -0.717,-0.723,-0.729,-0.735,-0.740,-0.746,-0.752,     &
                    -0.757,-0.762,-0.767,-0.772,-0.777,-0.782,-0.787,     &
                    -0.792,-0.797,-0.801,-0.806,-0.810,-0.815,-0.819,     &
                    -0.824,-0.828,-0.832,-0.836,-0.840,-0.844,-0.848,     &
                    -0.852,-0.856,-0.860,-0.864,-0.868,-0.872,-0.876,     &
                    -0.879,-0.883,-0.887,-0.890,-0.894,-0.897,-0.901,     &
                    -0.904,-0.908,-0.911,-0.915,-0.918,-0.922,-0.925,     &
                    -0.928,-0.932,-0.935,-0.938,-0.942,-0.945,-0.948,     &
                    -0.951,-0.954,-0.958,-0.961,-0.964,-0.967,-0.970,     &
                    -0.973,-0.976,-0.979,-0.982,-0.985,-0.988,-0.991,     &
                    -0.994,-0.997,-1.000,-1.003,-1.006,-1.009,-1.012,     &
                    -1.015,-1.018,-1.021,-1.024,-1.026,-1.029,-1.032,     &
                    -1.035,-1.038,-1.040,-1.043,-1.046,-1.049,-1.051,     &
                    -1.054,-1.057,-1.060,-1.062,-1.065,-1.068,-1.070,     &
                    -1.073,-1.076,-1.078,-1.081,-1.083,-1.086,-1.089,     &
                    -1.091,-1.094,-1.096,-1.099,-1.102,-1.104,-1.107,     &
                    -1.109,-1.112,-1.114,-1.117,-1.119,-1.122,-1.124,     &
                    -1.127,-1.129,-1.131,-1.134,-1.136,-1.139,-1.141,     &
                    -1.144,-1.146,-1.148,-1.151,-1.153,-1.155,-1.158,     &
                    -1.160,-1.163,-1.165,-1.167,-1.170,-1.172,-1.174,     &
                    -1.177,-1.179,-1.181,-1.183,-1.186,-1.188,-1.190,     &
                    -1.193,-1.195,-1.197,-1.199,-1.202,-1.204,-1.206,     &
                    -1.208,-1.210,-1.213,-1.215,-1.217,-1.219,-1.221,     &
                    -1.224,-1.226,-1.228,-1.230,-1.232,-1.235,-1.237,     &
                    -1.239,-1.241,-1.243,-1.245,-1.247,-1.250,-1.252,     &
                    -1.254,-1.256,-1.258,-1.260,-1.262,-1.264,-1.266,     &
                    -1.269,-1.271,-1.273,-1.275,-1.277,-1.279,-1.281,     &
                    -1.283,-1.285,-1.287,-1.289,-1.291,-1.293,-1.295,     &
                    -1.297,-1.299,-1.301,-1.303,-1.305,-1.307,-1.309,     &
                    -1.311,-1.313,-1.315,-1.317,-1.319,-1.321,-1.323,     &
                    -1.325,-1.327,-1.329,-1.331,-1.333,-1.335,-1.337,     &
                    -1.339,-1.341,-1.343,-1.345,-1.347,-1.349,-1.351,     &
                    -1.353,-1.355,-1.357,-1.358,-1.360,-1.362,-1.364,     &
                    -1.366,-1.368,-1.370,-1.372,-1.374,-1.376,-1.377,     &
                    -1.379,-1.381,-1.383,-1.385,-1.387,-1.389,-1.391,     &
                    -1.392,-1.394,-1.396,-1.398,-1.400,-1.402,-1.404,     &
                    -1.405,-1.407,-1.409,-1.411,-1.413,-1.415,-1.417,     &
                    -1.418,-1.420,-1.422,-1.424,-1.426,-1.427,-1.429,     &
                    -1.431,-1.433,-1.435,-1.437,-1.438,-1.440,-1.442,     &
                    -1.444,-1.445,-1.447,-1.449,-1.451,-1.453,-1.454,     &
                    -1.456,-1.458,-1.460,-1.462,-1.463,-1.465,-1.467,     &
                    -1.469,-1.470,-1.472,-1.474,-1.476,-1.477,-1.479,     &
                    -1.481,-1.483,-1.484,-1.486,-1.488,-1.490,-1.491,     &
                    -1.493,-1.495,-1.496,-1.498,-1.500,-1.502,-1.503,     &
                    -1.505,-1.507,-1.509,-1.510,-1.512,-1.514,-1.515,     &
                    -1.517,-1.519,-1.520,-1.522,-1.524,-1.526,-1.527,     &
                    -1.529,-1.531,-1.532,-1.534,-1.536,-1.537,-1.539,     &
                    -1.541,-1.542,-1.544,-1.546,-1.547,-1.549,-1.551,     &
                    -1.552,-1.554,-1.556,-1.557,-1.559,-1.561,-1.562,     &
                    -1.564,-1.566,-1.567,-1.569,-1.571,-1.572,-1.574,     &
                    -1.576,-1.577,-1.579,-1.580,-1.582,-1.584,-1.585,     &
                    -1.587,-1.589,-1.590,-1.592,-1.594,-1.595,-1.597,     &
                    -1.598,-1.616,-1.632,-1.648,-1.663,-1.679,-1.694,     &
                    -1.710,-1.725,-1.740,-1.755,-1.770,-1.785,-1.799,     &
                    -1.814,-1.828,-1.843,-1.857,-1.871,-1.885,-1.900,     &
                    -1.914,-1.928,-1.941,-1.955,-1.969,-1.983,-1.996,     &
                    -2.010,-2.023,-2.037,-2.050,-2.063,-2.077,-2.090,     &
                    -2.103,-2.116,-2.129,-2.142,-2.155,-2.168,-2.181,     &
                    -2.194,-2.206,-2.219,-2.232,-2.244,-2.257,-2.269,     &
                    -2.282,-2.294,-2.307,-2.319,-2.332,-2.344,-2.356,     &
                    -2.369,-2.381,-2.393,-2.405,-2.417,-2.429,-2.441,     &
                    -2.454,-2.466,-2.478,-2.489,-2.501,-2.513,-2.525,     &
                    -2.537,-2.549,-2.561,-2.572,-2.584,-2.596,-2.608,     &
                    -2.619,-2.631,-2.642,-2.654,-2.666,-2.677,-2.689,     &
                    -2.700,-2.712,-2.723,-2.735,-2.746,-2.757,-2.769,     &
                    -2.780,-2.792,-2.803,-2.814,-2.825,-2.837,-2.848,     &
                    -2.859,-2.870,-2.882,-2.893,-2.904,-2.915,-2.926,     &
                    -2.937,-2.948,-2.959,-2.971,-2.982,-2.993,-3.004,     &
                    -3.015,-3.026,-3.037,-3.048,-3.058,-3.069,-3.080,     &
                    -3.091,-3.102,-3.113,-3.124,-3.135,-3.145,-3.156,     &
                    -3.167,-3.178,-3.189,-3.199,-3.210,-3.221,-3.232,     &
                    -3.242,-3.253,-3.264,-3.274,-3.285,-3.296,-3.306,     &
                    -3.317,-3.327,-3.338,-3.349,-3.359,-3.370,-3.380,     &
                    -3.391,-3.401,-3.412,-3.422,-3.433,-3.443,-3.454,     &
                    -3.464,-3.475,-3.485,-3.496,-3.506,-3.517,-3.527,     &
                    -3.537/)
!
! *** khso4
!
    bnc18m(1:561) = (/-0.049,-0.104,-0.130,-0.148,-0.162,-0.173,-0.182,   &
                    -0.190,-0.197,-0.203,-0.208,-0.213,-0.217,-0.221,     &
                    -0.224,-0.227,-0.230,-0.232,-0.234,-0.236,-0.238,     &
                    -0.239,-0.240,-0.241,-0.242,-0.243,-0.243,-0.244,     &
                    -0.244,-0.244,-0.244,-0.244,-0.243,-0.243,-0.242,     &
                    -0.242,-0.241,-0.240,-0.239,-0.238,-0.237,-0.236,     &
                    -0.235,-0.233,-0.232,-0.230,-0.229,-0.227,-0.225,     &
                    -0.223,-0.222,-0.220,-0.218,-0.216,-0.214,-0.211,     &
                    -0.209,-0.207,-0.205,-0.202,-0.200,-0.197,-0.195,     &
                    -0.192,-0.190,-0.187,-0.185,-0.182,-0.179,-0.176,     &
                    -0.174,-0.171,-0.168,-0.165,-0.162,-0.159,-0.156,     &
                    -0.153,-0.150,-0.147,-0.143,-0.140,-0.137,-0.134,     &
                    -0.131,-0.127,-0.124,-0.120,-0.117,-0.114,-0.110,     &
                    -0.107,-0.103,-0.100,-0.096,-0.092,-0.089,-0.085,     &
                    -0.081,-0.078,-0.074,-0.070,-0.066,-0.063,-0.059,     &
                    -0.055,-0.051,-0.047,-0.043,-0.040,-0.036,-0.032,     &
                    -0.028,-0.024,-0.020,-0.016,-0.012,-0.008,-0.004,     &
                    0.000,0.004,0.008,0.012,0.015,0.019,0.023,0.027,0.031,&
                    0.035,0.039,0.043,0.047,0.051,0.055,0.059,0.063,0.067,&
                    0.071,0.075,0.078,0.082,0.086,0.090,0.094,0.098,0.102,&
                    0.106,0.109,0.113,0.117,0.121,0.125,0.128,0.132,0.136,&
                    0.140,0.144,0.147,0.151,0.155,0.159,0.162,0.166,0.170,&
                    0.173,0.177,0.181,0.184,0.188,0.192,0.195,0.199,0.203,&
                    0.206,0.210,0.213,0.217,0.221,0.224,0.228,0.231,0.235,&
                    0.238,0.242,0.245,0.249,0.252,0.256,0.259,0.263,0.266,&
                    0.270,0.273,0.277,0.280,0.284,0.287,0.290,0.294,0.297,&
                    0.301,0.304,0.307,0.311,0.314,0.317,0.321,0.324,0.327,&
                    0.331,0.334,0.337,0.340,0.344,0.347,0.350,0.353,0.357,&
                    0.360,0.363,0.366,0.369,0.373,0.376,0.379,0.382,0.385,&
                    0.389,0.392,0.395,0.398,0.401,0.404,0.407,0.410,0.413,&
                    0.417,0.420,0.423,0.426,0.429,0.432,0.435,0.438,0.441,&
                    0.444,0.447,0.450,0.453,0.456,0.459,0.462,0.465,0.468,&
                    0.471,0.474,0.477,0.480,0.482,0.485,0.488,0.491,0.494,&
                    0.497,0.500,0.503,0.506,0.508,0.511,0.514,0.517,0.520,&
                    0.523,0.525,0.528,0.531,0.534,0.537,0.539,0.542,0.545,&
                    0.548,0.551,0.553,0.556,0.559,0.562,0.564,0.567,0.570,&
                    0.572,0.575,0.578,0.581,0.583,0.586,0.589,0.591,0.594,&
                    0.597,0.599,0.602,0.604,0.607,0.610,0.612,0.615,0.618,&
                    0.620,0.623,0.625,0.628,0.631,0.633,0.636,0.638,0.641,&
                    0.643,0.646,0.648,0.651,0.654,0.656,0.659,0.661,0.664,&
                    0.666,0.669,0.671,0.674,0.676,0.679,0.681,0.684,0.686,&
                    0.688,0.691,0.693,0.696,0.698,0.701,0.703,0.706,0.708,&
                    0.710,0.713,0.715,0.718,0.720,0.722,0.725,0.727,0.729,&
                    0.732,0.734,0.737,0.739,0.741,0.744,0.746,0.748,0.751,&
                    0.753,0.755,0.758,0.760,0.762,0.764,0.767,0.769,0.771,&
                    0.774,0.776,0.778,0.780,0.783,0.785,0.787,0.789,0.792,&
                    0.794,0.796,0.798,0.801,0.803,0.805,0.807,0.810,0.812,&
                    0.814,0.816,0.818,0.821,0.823,0.825,0.827,0.829,0.831,&
                    0.834,0.836,0.859,0.880,0.900,0.921,0.940,0.960,0.979,&
                    0.998,1.016,1.034,1.052,1.070,1.087,1.104,1.121,1.137,&
                    1.154,1.169,1.185,1.201,1.216,1.231,1.246,1.260,1.274,&
                    1.289,1.302,1.316,1.330,1.343,1.356,1.369,1.382,1.394,&
                    1.407,1.419,1.431,1.443,1.455,1.466,1.478,1.489,1.500,&
                    1.511,1.522,1.533,1.543,1.554,1.564,1.574,1.584,1.594,&
                    1.604,1.614,1.623,1.633,1.642,1.651,1.660,1.670,1.678,&
                    1.687,1.696,1.704,1.713,1.721,1.730,1.738,1.746,1.754,&
                    1.762,1.770,1.777,1.785,1.793,1.800,1.807,1.815,1.822,&
                    1.829,1.836,1.843,1.850,1.857,1.864,1.870,1.877,1.883,&
                    1.890,1.896,1.903,1.909,1.915,1.921,1.927,1.933,1.939,&
                    1.945,1.951,1.957,1.962,1.968,1.973,1.979,1.984,1.990,&
                    1.995,2.000,2.006,2.011,2.016,2.021,2.026,2.031,2.036,&
                    2.041,2.045,2.050,2.055,2.059,2.064,2.069,2.073,2.078,&
                    2.082,2.086,2.091,2.095,2.099,2.103,2.108,2.112,2.116,&
                    2.120,2.124,2.128,2.132,2.135,2.139,2.143,2.147,2.150,&
                    2.154,2.158,2.161,2.165,2.168,2.172,2.175,2.179,2.182,&
                    2.185,2.189,2.192,2.195,2.198,2.201,2.205,2.208,2.211,&
                    2.214/)
!
! *** kno3
!
    bnc19m(1:561) = (/-0.052,-0.120,-0.158,-0.187,-0.211,-0.232,-0.251,   &
                    -0.269,-0.285,-0.300,-0.315,-0.329,-0.342,-0.354,     &
                    -0.367,-0.378,-0.390,-0.401,-0.412,-0.422,-0.432,     &
                    -0.443,-0.452,-0.462,-0.471,-0.481,-0.490,-0.498,     &
                    -0.507,-0.516,-0.524,-0.532,-0.540,-0.548,-0.556,     &
                    -0.564,-0.572,-0.579,-0.587,-0.594,-0.601,-0.608,     &
                    -0.615,-0.622,-0.629,-0.636,-0.642,-0.649,-0.655,     &
                    -0.661,-0.668,-0.674,-0.680,-0.686,-0.692,-0.698,     &
                    -0.704,-0.710,-0.715,-0.721,-0.727,-0.732,-0.738,     &
                    -0.743,-0.749,-0.754,-0.759,-0.765,-0.770,-0.775,     &
                    -0.780,-0.785,-0.790,-0.796,-0.801,-0.806,-0.811,     &
                    -0.816,-0.820,-0.825,-0.830,-0.835,-0.840,-0.845,     &
                    -0.850,-0.854,-0.859,-0.864,-0.869,-0.873,-0.878,     &
                    -0.883,-0.888,-0.892,-0.897,-0.902,-0.906,-0.911,     &
                    -0.916,-0.920,-0.925,-0.929,-0.934,-0.938,-0.943,     &
                    -0.948,-0.952,-0.957,-0.961,-0.966,-0.970,-0.974,     &
                    -0.979,-0.983,-0.988,-0.992,-0.996,-1.001,-1.005,     &
                    -1.009,-1.014,-1.018,-1.022,-1.026,-1.031,-1.035,     &
                    -1.039,-1.043,-1.047,-1.052,-1.056,-1.060,-1.064,     &
                    -1.068,-1.072,-1.076,-1.080,-1.084,-1.088,-1.092,     &
                    -1.096,-1.100,-1.104,-1.108,-1.112,-1.115,-1.119,     &
                    -1.123,-1.127,-1.131,-1.134,-1.138,-1.142,-1.146,     &
                    -1.149,-1.153,-1.157,-1.160,-1.164,-1.168,-1.171,     &
                    -1.175,-1.178,-1.182,-1.185,-1.189,-1.192,-1.196,     &
                    -1.199,-1.203,-1.206,-1.210,-1.213,-1.217,-1.220,     &
                    -1.223,-1.227,-1.230,-1.234,-1.237,-1.240,-1.243,     &
                    -1.247,-1.250,-1.253,-1.257,-1.260,-1.263,-1.266,     &
                    -1.269,-1.273,-1.276,-1.279,-1.282,-1.285,-1.288,     &
                    -1.291,-1.295,-1.298,-1.301,-1.304,-1.307,-1.310,     &
                    -1.313,-1.316,-1.319,-1.322,-1.325,-1.328,-1.331,     &
                    -1.334,-1.337,-1.340,-1.342,-1.345,-1.348,-1.351,     &
                    -1.354,-1.357,-1.360,-1.362,-1.365,-1.368,-1.371,     &
                    -1.374,-1.376,-1.379,-1.382,-1.385,-1.387,-1.390,     &
                    -1.393,-1.396,-1.398,-1.401,-1.404,-1.406,-1.409,     &
                    -1.412,-1.414,-1.417,-1.420,-1.422,-1.425,-1.427,     &
                    -1.430,-1.432,-1.435,-1.438,-1.440,-1.443,-1.445,     &
                    -1.448,-1.450,-1.453,-1.455,-1.458,-1.460,-1.463,     &
                    -1.465,-1.468,-1.470,-1.472,-1.475,-1.477,-1.480,     &
                    -1.482,-1.484,-1.487,-1.489,-1.492,-1.494,-1.496,     &
                    -1.499,-1.501,-1.503,-1.506,-1.508,-1.510,-1.512,     &
                    -1.515,-1.517,-1.519,-1.522,-1.524,-1.526,-1.528,     &
                    -1.531,-1.533,-1.535,-1.537,-1.539,-1.542,-1.544,     &
                    -1.546,-1.548,-1.550,-1.552,-1.555,-1.557,-1.559,     &
                    -1.561,-1.563,-1.565,-1.567,-1.569,-1.572,-1.574,     &
                    -1.576,-1.578,-1.580,-1.582,-1.584,-1.586,-1.588,     &
                    -1.590,-1.592,-1.594,-1.596,-1.598,-1.600,-1.602,     &
                    -1.604,-1.606,-1.608,-1.610,-1.612,-1.614,-1.616,     &
                    -1.618,-1.620,-1.622,-1.624,-1.626,-1.628,-1.630,     &
                    -1.632,-1.633,-1.635,-1.637,-1.639,-1.641,-1.643,     &
                    -1.645,-1.647,-1.648,-1.650,-1.652,-1.654,-1.656,     &
                    -1.658,-1.660,-1.661,-1.663,-1.665,-1.667,-1.669,     &
                    -1.670,-1.672,-1.674,-1.676,-1.678,-1.679,-1.681,     &
                    -1.683,-1.685,-1.686,-1.688,-1.690,-1.692,-1.693,     &
                    -1.695,-1.697,-1.698,-1.700,-1.702,-1.704,-1.705,     &
                    -1.707,-1.709,-1.710,-1.712,-1.714,-1.715,-1.717,     &
                    -1.719,-1.720,-1.722,-1.724,-1.725,-1.727,-1.729,     &
                    -1.730,-1.732,-1.733,-1.735,-1.737,-1.738,-1.740,     &
                    -1.741,-1.758,-1.774,-1.789,-1.803,-1.817,-1.831,     &
                    -1.845,-1.858,-1.871,-1.884,-1.896,-1.908,-1.920,     &
                    -1.932,-1.943,-1.954,-1.965,-1.976,-1.987,-1.997,     &
                    -2.008,-2.018,-2.028,-2.037,-2.047,-2.056,-2.066,     &
                    -2.075,-2.084,-2.093,-2.102,-2.111,-2.119,-2.128,     &
                    -2.136,-2.144,-2.152,-2.161,-2.169,-2.176,-2.184,     &
                    -2.192,-2.200,-2.207,-2.215,-2.222,-2.230,-2.237,     &
                    -2.244,-2.251,-2.259,-2.266,-2.273,-2.280,-2.287,     &
                    -2.293,-2.300,-2.307,-2.314,-2.320,-2.327,-2.334,     &
                    -2.340,-2.347,-2.353,-2.360,-2.366,-2.372,-2.379,     &
                    -2.385,-2.391,-2.397,-2.403,-2.410,-2.416,-2.422,     &
                    -2.428,-2.434,-2.440,-2.446,-2.452,-2.458,-2.464,     &
                    -2.470,-2.475,-2.481,-2.487,-2.493,-2.499,-2.504,     &
                    -2.510,-2.516,-2.522,-2.527,-2.533,-2.539,-2.544,     &
                    -2.550,-2.555,-2.561,-2.566,-2.572,-2.578,-2.583,     &
                    -2.588,-2.594,-2.599,-2.605,-2.610,-2.616,-2.621,     &
                    -2.627,-2.632,-2.637,-2.643,-2.648,-2.653,-2.659,     &
                    -2.664,-2.669,-2.675,-2.680,-2.685,-2.690,-2.696,     &
                    -2.701,-2.706,-2.711,-2.716,-2.722,-2.727,-2.732,     &
                    -2.737,-2.742,-2.748,-2.753,-2.758,-2.763,-2.768,     &
                    -2.773,-2.778,-2.783,-2.788,-2.794,-2.799,-2.804,     &
                    -2.809,-2.814,-2.819,-2.824,-2.829,-2.834,-2.839,     &
                    -2.844,-2.849,-2.854,-2.859,-2.864,-2.869,-2.874,     &
                    -2.879/)
!
! *** kcl
!
    bnc20m(1:561) = (/-0.049,-0.105,-0.132,-0.151,-0.164,-0.176,-0.185,   &
                    -0.193,-0.200,-0.206,-0.211,-0.216,-0.221,-0.224,     &
                    -0.228,-0.231,-0.234,-0.237,-0.240,-0.242,-0.244,     &
                    -0.246,-0.248,-0.250,-0.252,-0.253,-0.255,-0.256,     &
                    -0.257,-0.258,-0.260,-0.261,-0.262,-0.263,-0.264,     &
                    -0.265,-0.265,-0.266,-0.267,-0.268,-0.268,-0.269,     &
                    -0.270,-0.270,-0.271,-0.271,-0.272,-0.272,-0.273,     &
                    -0.273,-0.274,-0.274,-0.275,-0.275,-0.275,-0.276,     &
                    -0.276,-0.276,-0.277,-0.277,-0.277,-0.278,-0.278,     &
                    -0.278,-0.278,-0.279,-0.279,-0.279,-0.279,-0.280,     &
                    -0.280,-0.280,-0.280,-0.280,-0.280,-0.280,-0.281,     &
                    -0.281,-0.281,-0.281,-0.281,-0.281,-0.281,-0.281,     &
                    -0.281,-0.281,-0.281,-0.281,-0.281,-0.281,-0.281,     &
                    -0.281,-0.281,-0.281,-0.281,-0.281,-0.281,-0.280,     &
                    -0.280,-0.280,-0.280,-0.280,-0.280,-0.280,-0.279,     &
                    -0.279,-0.279,-0.279,-0.279,-0.278,-0.278,-0.278,     &
                    -0.278,-0.278,-0.277,-0.277,-0.277,-0.277,-0.276,     &
                    -0.276,-0.276,-0.276,-0.275,-0.275,-0.275,-0.275,     &
                    -0.274,-0.274,-0.274,-0.273,-0.273,-0.273,-0.273,     &
                    -0.272,-0.272,-0.272,-0.271,-0.271,-0.271,-0.270,     &
                    -0.270,-0.270,-0.270,-0.269,-0.269,-0.269,-0.268,     &
                    -0.268,-0.268,-0.267,-0.267,-0.267,-0.267,-0.266,     &
                    -0.266,-0.266,-0.265,-0.265,-0.265,-0.264,-0.264,     &
                    -0.264,-0.263,-0.263,-0.263,-0.262,-0.262,-0.262,     &
                    -0.262,-0.261,-0.261,-0.261,-0.260,-0.260,-0.260,     &
                    -0.259,-0.259,-0.259,-0.258,-0.258,-0.258,-0.258,     &
                    -0.257,-0.257,-0.257,-0.256,-0.256,-0.256,-0.255,     &
                    -0.255,-0.255,-0.254,-0.254,-0.254,-0.254,-0.253,     &
                    -0.253,-0.253,-0.252,-0.252,-0.252,-0.251,-0.251,     &
                    -0.251,-0.251,-0.250,-0.250,-0.250,-0.249,-0.249,     &
                    -0.249,-0.248,-0.248,-0.248,-0.248,-0.247,-0.247,     &
                    -0.247,-0.246,-0.246,-0.246,-0.246,-0.245,-0.245,     &
                    -0.245,-0.244,-0.244,-0.244,-0.244,-0.243,-0.243,     &
                    -0.243,-0.242,-0.242,-0.242,-0.242,-0.241,-0.241,     &
                    -0.241,-0.241,-0.240,-0.240,-0.240,-0.239,-0.239,     &
                    -0.239,-0.239,-0.238,-0.238,-0.238,-0.238,-0.237,     &
                    -0.237,-0.237,-0.236,-0.236,-0.236,-0.236,-0.235,     &
                    -0.235,-0.235,-0.235,-0.234,-0.234,-0.234,-0.234,     &
                    -0.233,-0.233,-0.233,-0.233,-0.232,-0.232,-0.232,     &
                    -0.232,-0.231,-0.231,-0.231,-0.231,-0.230,-0.230,     &
                    -0.230,-0.230,-0.229,-0.229,-0.229,-0.229,-0.228,     &
                    -0.228,-0.228,-0.228,-0.227,-0.227,-0.227,-0.227,     &
                    -0.226,-0.226,-0.226,-0.226,-0.226,-0.225,-0.225,     &
                    -0.225,-0.225,-0.224,-0.224,-0.224,-0.224,-0.223,     &
                    -0.223,-0.223,-0.223,-0.223,-0.222,-0.222,-0.222,     &
                    -0.222,-0.221,-0.221,-0.221,-0.221,-0.221,-0.220,     &
                    -0.220,-0.220,-0.220,-0.220,-0.219,-0.219,-0.219,     &
                    -0.219,-0.218,-0.218,-0.218,-0.218,-0.218,-0.217,     &
                    -0.217,-0.217,-0.217,-0.217,-0.216,-0.216,-0.216,     &
                    -0.216,-0.216,-0.215,-0.215,-0.215,-0.215,-0.215,     &
                    -0.215,-0.214,-0.214,-0.214,-0.214,-0.214,-0.213,     &
                    -0.213,-0.213,-0.213,-0.213,-0.212,-0.212,-0.212,     &
                    -0.212,-0.212,-0.212,-0.211,-0.211,-0.211,-0.211,     &
                    -0.211,-0.210,-0.210,-0.210,-0.210,-0.210,-0.210,     &
                    -0.209,-0.209,-0.209,-0.209,-0.209,-0.209,-0.208,     &
                    -0.208,-0.208,-0.208,-0.208,-0.208,-0.207,-0.207,     &
                    -0.207,-0.207,-0.207,-0.207,-0.206,-0.206,-0.206,     &
                    -0.206,-0.204,-0.203,-0.202,-0.200,-0.199,-0.198,     &
                    -0.197,-0.196,-0.195,-0.194,-0.193,-0.192,-0.191,     &
                    -0.190,-0.190,-0.189,-0.189,-0.188,-0.188,-0.187,     &
                    -0.187,-0.187,-0.186,-0.186,-0.186,-0.186,-0.186,     &
                    -0.186,-0.186,-0.186,-0.186,-0.186,-0.186,-0.186,     &
                    -0.187,-0.187,-0.187,-0.187,-0.188,-0.188,-0.189,     &
                    -0.189,-0.190,-0.190,-0.191,-0.191,-0.192,-0.193,     &
                    -0.193,-0.194,-0.195,-0.196,-0.196,-0.197,-0.198,     &
                    -0.199,-0.200,-0.201,-0.202,-0.203,-0.204,-0.205,     &
                    -0.206,-0.207,-0.208,-0.209,-0.210,-0.212,-0.213,     &
                    -0.214,-0.215,-0.217,-0.218,-0.219,-0.220,-0.222,     &
                    -0.223,-0.225,-0.226,-0.227,-0.229,-0.230,-0.232,     &
                    -0.233,-0.235,-0.236,-0.238,-0.239,-0.241,-0.243,     &
                    -0.244,-0.246,-0.247,-0.249,-0.251,-0.252,-0.254,     &
                    -0.256,-0.258,-0.259,-0.261,-0.263,-0.265,-0.266,     &
                    -0.268,-0.270,-0.272,-0.274,-0.276,-0.278,-0.280,     &
                    -0.281,-0.283,-0.285,-0.287,-0.289,-0.291,-0.293,     &
                    -0.295,-0.297,-0.299,-0.301,-0.303,-0.305,-0.307,     &
                    -0.309,-0.311,-0.314,-0.316,-0.318,-0.320,-0.322,     &
                    -0.324,-0.326,-0.329,-0.331,-0.333,-0.335,-0.337,     &
                    -0.340,-0.342,-0.344,-0.346,-0.348,-0.351,-0.353,     &
                    -0.355,-0.358,-0.360,-0.362,-0.364,-0.367,-0.369,     &
                    -0.371,-0.374,-0.376,-0.379,-0.381,-0.383,-0.386,     &
                    -0.388/)
!
! *** mgso4
!
    bnc21m(1:561) = (/-0.200,-0.434,-0.550,-0.632,-0.696,-0.748,-0.793,   &
                    -0.833,-0.868,-0.900,-0.928,-0.955,-0.979,-1.002,     &
                    -1.023,-1.043,-1.062,-1.079,-1.096,-1.112,-1.127,     &
                    -1.142,-1.156,-1.170,-1.182,-1.195,-1.207,-1.218,     &
                    -1.230,-1.241,-1.251,-1.261,-1.271,-1.281,-1.290,     &
                    -1.300,-1.309,-1.317,-1.326,-1.334,-1.342,-1.350,     &
                    -1.358,-1.366,-1.373,-1.381,-1.388,-1.395,-1.402,     &
                    -1.409,-1.416,-1.422,-1.429,-1.435,-1.442,-1.448,     &
                    -1.454,-1.460,-1.466,-1.472,-1.478,-1.484,-1.489,     &
                    -1.495,-1.500,-1.506,-1.511,-1.517,-1.522,-1.527,     &
                    -1.532,-1.537,-1.542,-1.547,-1.552,-1.557,-1.562,     &
                    -1.567,-1.571,-1.576,-1.581,-1.585,-1.590,-1.594,     &
                    -1.599,-1.603,-1.608,-1.612,-1.616,-1.620,-1.625,     &
                    -1.629,-1.633,-1.637,-1.641,-1.645,-1.649,-1.653,     &
                    -1.657,-1.661,-1.665,-1.669,-1.672,-1.676,-1.680,     &
                    -1.684,-1.687,-1.691,-1.695,-1.698,-1.702,-1.706,     &
                    -1.709,-1.713,-1.716,-1.720,-1.723,-1.727,-1.730,     &
                    -1.733,-1.737,-1.740,-1.744,-1.747,-1.750,-1.753,     &
                    -1.757,-1.760,-1.763,-1.766,-1.770,-1.773,-1.776,     &
                    -1.779,-1.782,-1.786,-1.789,-1.792,-1.795,-1.798,     &
                    -1.801,-1.804,-1.807,-1.810,-1.813,-1.816,-1.819,     &
                    -1.822,-1.825,-1.828,-1.831,-1.834,-1.837,-1.840,     &
                    -1.843,-1.846,-1.849,-1.851,-1.854,-1.857,-1.860,     &
                    -1.863,-1.866,-1.868,-1.871,-1.874,-1.877,-1.880,     &
                    -1.882,-1.885,-1.888,-1.891,-1.894,-1.896,-1.899,     &
                    -1.902,-1.904,-1.907,-1.910,-1.913,-1.915,-1.918,     &
                    -1.921,-1.923,-1.926,-1.929,-1.931,-1.934,-1.936,     &
                    -1.939,-1.942,-1.944,-1.947,-1.950,-1.952,-1.955,     &
                    -1.957,-1.960,-1.962,-1.965,-1.968,-1.970,-1.973,     &
                    -1.975,-1.978,-1.980,-1.983,-1.985,-1.988,-1.990,     &
                    -1.993,-1.995,-1.998,-2.000,-2.003,-2.005,-2.008,     &
                    -2.010,-2.013,-2.015,-2.018,-2.020,-2.023,-2.025,     &
                    -2.028,-2.030,-2.032,-2.035,-2.037,-2.040,-2.042,     &
                    -2.044,-2.047,-2.049,-2.052,-2.054,-2.057,-2.059,     &
                    -2.061,-2.064,-2.066,-2.068,-2.071,-2.073,-2.076,     &
                    -2.078,-2.080,-2.083,-2.085,-2.087,-2.090,-2.092,     &
                    -2.094,-2.097,-2.099,-2.101,-2.104,-2.106,-2.108,     &
                    -2.111,-2.113,-2.115,-2.118,-2.120,-2.122,-2.124,     &
                    -2.127,-2.129,-2.131,-2.134,-2.136,-2.138,-2.141,     &
                    -2.143,-2.145,-2.147,-2.150,-2.152,-2.154,-2.156,     &
                    -2.159,-2.161,-2.163,-2.165,-2.168,-2.170,-2.172,     &
                    -2.174,-2.177,-2.179,-2.181,-2.183,-2.186,-2.188,     &
                    -2.190,-2.192,-2.195,-2.197,-2.199,-2.201,-2.203,     &
                    -2.206,-2.208,-2.210,-2.212,-2.214,-2.217,-2.219,     &
                    -2.221,-2.223,-2.225,-2.228,-2.230,-2.232,-2.234,     &
                    -2.236,-2.239,-2.241,-2.243,-2.245,-2.247,-2.249,     &
                    -2.252,-2.254,-2.256,-2.258,-2.260,-2.262,-2.265,     &
                    -2.267,-2.269,-2.271,-2.273,-2.275,-2.278,-2.280,     &
                    -2.282,-2.284,-2.286,-2.288,-2.290,-2.293,-2.295,     &
                    -2.297,-2.299,-2.301,-2.303,-2.305,-2.308,-2.310,     &
                    -2.312,-2.314,-2.316,-2.318,-2.320,-2.322,-2.325,     &
                    -2.327,-2.329,-2.331,-2.333,-2.335,-2.337,-2.339,     &
                    -2.341,-2.344,-2.346,-2.348,-2.350,-2.352,-2.354,     &
                    -2.356,-2.358,-2.360,-2.362,-2.365,-2.367,-2.369,     &
                    -2.371,-2.373,-2.375,-2.377,-2.379,-2.381,-2.383,     &
                    -2.385,-2.388,-2.390,-2.392,-2.394,-2.396,-2.398,     &
                    -2.400,-2.402,-2.404,-2.406,-2.408,-2.410,-2.412,     &
                    -2.415,-2.437,-2.457,-2.478,-2.498,-2.518,-2.539,     &
                    -2.559,-2.579,-2.599,-2.619,-2.638,-2.658,-2.678,     &
                    -2.698,-2.717,-2.737,-2.756,-2.776,-2.795,-2.815,     &
                    -2.834,-2.854,-2.873,-2.892,-2.911,-2.931,-2.950,     &
                    -2.969,-2.988,-3.007,-3.026,-3.045,-3.064,-3.083,     &
                    -3.102,-3.121,-3.140,-3.159,-3.178,-3.197,-3.215,     &
                    -3.234,-3.253,-3.272,-3.291,-3.309,-3.328,-3.347,     &
                    -3.365,-3.384,-3.403,-3.421,-3.440,-3.459,-3.477,     &
                    -3.496,-3.514,-3.533,-3.551,-3.570,-3.588,-3.607,     &
                    -3.625,-3.644,-3.662,-3.681,-3.699,-3.718,-3.736,     &
                    -3.754,-3.773,-3.791,-3.810,-3.828,-3.846,-3.865,     &
                    -3.883,-3.901,-3.920,-3.938,-3.956,-3.974,-3.993,     &
                    -4.011,-4.029,-4.047,-4.066,-4.084,-4.102,-4.120,     &
                    -4.139,-4.157,-4.175,-4.193,-4.211,-4.229,-4.248,     &
                    -4.266,-4.284,-4.302,-4.320,-4.338,-4.356,-4.374,     &
                    -4.392,-4.411,-4.429,-4.447,-4.465,-4.483,-4.501,     &
                    -4.519,-4.537,-4.555,-4.573,-4.591,-4.609,-4.627,     &
                    -4.645,-4.663,-4.681,-4.699,-4.717,-4.735,-4.753,     &
                    -4.771,-4.789,-4.807,-4.825,-4.842,-4.860,-4.878,     &
                    -4.896,-4.914,-4.932,-4.950,-4.968,-4.986,-5.004,     &
                    -5.021,-5.039,-5.057,-5.075,-5.093,-5.111,-5.129,     &
                    -5.146,-5.164,-5.182,-5.200,-5.218,-5.235,-5.253,     &
                    -5.271,-5.289,-5.307,-5.324,-5.342,-5.360,-5.378,     &
                    -5.395/)
!
! *** mgno32
!
    bnc22m(1:561) = (/-0.097,-0.201,-0.248,-0.278,-0.300,-0.316,-0.329,   &
                    -0.339,-0.347,-0.354,-0.359,-0.363,-0.366,-0.369,     &
                    -0.370,-0.372,-0.372,-0.373,-0.372,-0.372,-0.371,     &
                    -0.370,-0.369,-0.367,-0.366,-0.364,-0.362,-0.359,     &
                    -0.357,-0.354,-0.352,-0.349,-0.346,-0.344,-0.341,     &
                    -0.338,-0.334,-0.331,-0.328,-0.325,-0.322,-0.318,     &
                    -0.315,-0.312,-0.308,-0.305,-0.301,-0.298,-0.294,     &
                    -0.291,-0.287,-0.284,-0.280,-0.277,-0.273,-0.270,     &
                    -0.266,-0.263,-0.259,-0.256,-0.252,-0.248,-0.245,     &
                    -0.241,-0.238,-0.234,-0.230,-0.227,-0.223,-0.219,     &
                    -0.216,-0.212,-0.208,-0.204,-0.201,-0.197,-0.193,     &
                    -0.189,-0.185,-0.181,-0.177,-0.173,-0.169,-0.165,     &
                    -0.161,-0.157,-0.153,-0.149,-0.145,-0.141,-0.136,     &
                    -0.132,-0.128,-0.123,-0.119,-0.115,-0.110,-0.106,     &
                    -0.101,-0.097,-0.092,-0.087,-0.083,-0.078,-0.073,     &
                    -0.069,-0.064,-0.059,-0.055,-0.050,-0.045,-0.040,     &
                    -0.035,-0.031,-0.026,-0.021,-0.016,-0.011,-0.006,     &
                    -0.001,0.003,0.008,0.013,0.018,0.023,0.028,0.033,     &
                    0.038,0.043,0.048,0.052,0.057,0.062,0.067,0.072,0.077,&
                    0.082,0.087,0.092,0.097,0.101,0.106,0.111,0.116,0.121,&
                    0.126,0.131,0.135,0.140,0.145,0.150,0.155,0.160,0.164,&
                    0.169,0.174,0.179,0.184,0.188,0.193,0.198,0.203,0.207,&
                    0.212,0.217,0.222,0.226,0.231,0.236,0.241,0.245,0.250,&
                    0.255,0.259,0.264,0.269,0.273,0.278,0.283,0.287,0.292,&
                    0.297,0.301,0.306,0.311,0.315,0.320,0.324,0.329,0.333,&
                    0.338,0.343,0.347,0.352,0.356,0.361,0.365,0.370,0.374,&
                    0.379,0.383,0.388,0.392,0.397,0.401,0.406,0.410,0.414,&
                    0.419,0.423,0.428,0.432,0.437,0.441,0.445,0.450,0.454,&
                    0.458,0.463,0.467,0.472,0.476,0.480,0.484,0.489,0.493,&
                    0.497,0.502,0.506,0.510,0.514,0.519,0.523,0.527,0.531,&
                    0.536,0.540,0.544,0.548,0.553,0.557,0.561,0.565,0.569,&
                    0.573,0.578,0.582,0.586,0.590,0.594,0.598,0.602,0.606,&
                    0.610,0.615,0.619,0.623,0.627,0.631,0.635,0.639,0.643,&
                    0.647,0.651,0.655,0.659,0.663,0.667,0.671,0.675,0.679,&
                    0.683,0.687,0.691,0.695,0.699,0.703,0.706,0.710,0.714,&
                    0.718,0.722,0.726,0.730,0.734,0.738,0.741,0.745,0.749,&
                    0.753,0.757,0.761,0.764,0.768,0.772,0.776,0.779,0.783,&
                    0.787,0.791,0.795,0.798,0.802,0.806,0.809,0.813,0.817,&
                    0.821,0.824,0.828,0.832,0.835,0.839,0.843,0.846,0.850,&
                    0.854,0.857,0.861,0.865,0.868,0.872,0.875,0.879,0.883,&
                    0.886,0.890,0.893,0.897,0.900,0.904,0.907,0.911,0.915,&
                    0.918,0.922,0.925,0.929,0.932,0.936,0.939,0.943,0.946,&
                    0.950,0.953,0.956,0.960,0.963,0.967,0.970,0.974,0.977,&
                    0.980,0.984,0.987,0.991,0.994,0.997,1.001,1.004,1.007,&
                    1.011,1.014,1.018,1.021,1.024,1.028,1.031,1.034,1.037,&
                    1.041,1.044,1.047,1.051,1.054,1.057,1.060,1.064,1.067,&
                    1.070,1.073,1.077,1.080,1.083,1.086,1.090,1.093,1.096,&
                    1.099,1.102,1.106,1.109,1.112,1.115,1.118,1.121,1.125,&
                    1.128,1.131,1.134,1.167,1.198,1.228,1.257,1.286,1.314,&
                    1.342,1.370,1.397,1.423,1.449,1.475,1.500,1.525,1.549,&
                    1.573,1.597,1.620,1.643,1.665,1.687,1.709,1.730,1.751,&
                    1.772,1.793,1.813,1.833,1.852,1.871,1.890,1.909,1.927,&
                    1.945,1.963,1.981,1.998,2.015,2.032,2.049,2.065,2.081,&
                    2.097,2.112,2.128,2.143,2.158,2.173,2.187,2.202,2.216,&
                    2.230,2.244,2.257,2.271,2.284,2.297,2.310,2.322,2.335,&
                    2.347,2.359,2.371,2.383,2.395,2.406,2.418,2.429,2.440,&
                    2.451,2.462,2.472,2.483,2.493,2.503,2.513,2.523,2.533,&
                    2.543,2.552,2.562,2.571,2.580,2.589,2.598,2.607,2.616,&
                    2.624,2.633,2.641,2.650,2.658,2.666,2.674,2.681,2.689,&
                    2.697,2.704,2.712,2.719,2.726,2.733,2.740,2.747,2.754,&
                    2.761,2.768,2.774,2.781,2.787,2.793,2.800,2.806,2.812,&
                    2.818,2.824,2.830,2.835,2.841,2.847,2.852,2.858,2.863,&
                    2.868,2.873,2.879,2.884,2.889,2.893,2.898,2.903,2.908,&
                    2.912,2.917,2.922,2.926,2.930,2.935,2.939,2.943,2.947,&
                    2.951,2.955,2.959,2.963,2.967,2.971,2.974,2.978,2.982,&
                    2.985,2.989,2.992,2.996,2.999,3.002,3.005,3.009,3.012,&
                    3.015,3.018/)
!
! *** mgcl2
!
    bnc23m(1:561) = (/-0.096,-0.198,-0.242,-0.270,-0.290,-0.304,-0.315,   &
                    -0.323,-0.329,-0.333,-0.337,-0.339,-0.340,-0.341,     &
                    -0.340,-0.340,-0.339,-0.337,-0.335,-0.333,-0.330,     &
                    -0.327,-0.324,-0.320,-0.317,-0.313,-0.309,-0.305,     &
                    -0.301,-0.296,-0.292,-0.287,-0.283,-0.278,-0.273,     &
                    -0.268,-0.263,-0.258,-0.253,-0.248,-0.243,-0.238,     &
                    -0.233,-0.228,-0.222,-0.217,-0.212,-0.207,-0.202,     &
                    -0.196,-0.191,-0.186,-0.180,-0.175,-0.170,-0.164,     &
                    -0.159,-0.154,-0.149,-0.143,-0.138,-0.133,-0.127,     &
                    -0.122,-0.117,-0.111,-0.106,-0.100,-0.095,-0.090,     &
                    -0.084,-0.079,-0.073,-0.068,-0.062,-0.057,-0.051,     &
                    -0.045,-0.040,-0.034,-0.028,-0.023,-0.017,-0.011,     &
                    -0.005,0.001,0.007,0.013,0.019,0.025,0.031,0.037,     &
                    0.043,0.049,0.055,0.062,0.068,0.074,0.081,0.087,0.094,&
                    0.100,0.107,0.113,0.120,0.126,0.133,0.140,0.146,0.153,&
                    0.160,0.166,0.173,0.180,0.187,0.193,0.200,0.207,0.214,&
                    0.220,0.227,0.234,0.241,0.248,0.255,0.261,0.268,0.275,&
                    0.282,0.289,0.296,0.302,0.309,0.316,0.323,0.330,0.337,&
                    0.343,0.350,0.357,0.364,0.371,0.377,0.384,0.391,0.398,&
                    0.404,0.411,0.418,0.425,0.431,0.438,0.445,0.451,0.458,&
                    0.465,0.471,0.478,0.485,0.491,0.498,0.505,0.511,0.518,&
                    0.524,0.531,0.537,0.544,0.551,0.557,0.564,0.570,0.577,&
                    0.583,0.590,0.596,0.603,0.609,0.616,0.622,0.628,0.635,&
                    0.641,0.648,0.654,0.660,0.667,0.673,0.679,0.686,0.692,&
                    0.698,0.705,0.711,0.717,0.724,0.730,0.736,0.742,0.749,&
                    0.755,0.761,0.767,0.773,0.780,0.786,0.792,0.798,0.804,&
                    0.810,0.816,0.823,0.829,0.835,0.841,0.847,0.853,0.859,&
                    0.865,0.871,0.877,0.883,0.889,0.895,0.901,0.907,0.913,&
                    0.919,0.925,0.931,0.937,0.942,0.948,0.954,0.960,0.966,&
                    0.972,0.978,0.983,0.989,0.995,1.001,1.007,1.012,1.018,&
                    1.024,1.030,1.035,1.041,1.047,1.052,1.058,1.064,1.069,&
                    1.075,1.081,1.086,1.092,1.098,1.103,1.109,1.114,1.120,&
                    1.125,1.131,1.136,1.142,1.148,1.153,1.159,1.164,1.169,&
                    1.175,1.180,1.186,1.191,1.197,1.202,1.208,1.213,1.218,&
                    1.224,1.229,1.234,1.240,1.245,1.250,1.256,1.261,1.266,&
                    1.272,1.277,1.282,1.287,1.293,1.298,1.303,1.308,1.314,&
                    1.319,1.324,1.329,1.334,1.339,1.345,1.350,1.355,1.360,&
                    1.365,1.370,1.375,1.380,1.385,1.390,1.396,1.401,1.406,&
                    1.411,1.416,1.421,1.426,1.431,1.436,1.441,1.446,1.451,&
                    1.456,1.460,1.465,1.470,1.475,1.480,1.485,1.490,1.495,&
                    1.500,1.504,1.509,1.514,1.519,1.524,1.529,1.533,1.538,&
                    1.543,1.548,1.553,1.557,1.562,1.567,1.572,1.576,1.581,&
                    1.586,1.590,1.595,1.600,1.604,1.609,1.614,1.618,1.623,&
                    1.628,1.632,1.637,1.642,1.646,1.651,1.655,1.660,1.664,&
                    1.669,1.674,1.678,1.683,1.687,1.692,1.696,1.701,1.705,&
                    1.710,1.714,1.719,1.723,1.728,1.732,1.736,1.741,1.745,&
                    1.750,1.754,1.759,1.763,1.767,1.772,1.776,1.780,1.785,&
                    1.789,1.793,1.840,1.882,1.923,1.964,2.004,2.044,2.082,&
                    2.120,2.158,2.195,2.231,2.267,2.302,2.336,2.370,2.404,&
                    2.437,2.469,2.501,2.533,2.564,2.594,2.624,2.654,2.683,&
                    2.712,2.741,2.768,2.796,2.823,2.850,2.877,2.903,2.928,&
                    2.954,2.979,3.003,3.028,3.052,3.076,3.099,3.122,3.145,&
                    3.167,3.190,3.211,3.233,3.254,3.276,3.296,3.317,3.337,&
                    3.357,3.377,3.397,3.416,3.435,3.454,3.473,3.491,3.509,&
                    3.527,3.545,3.563,3.580,3.597,3.614,3.631,3.647,3.664,&
                    3.680,3.696,3.712,3.727,3.743,3.758,3.773,3.788,3.803,&
                    3.817,3.832,3.846,3.860,3.874,3.888,3.902,3.915,3.928,&
                    3.942,3.955,3.968,3.980,3.993,4.006,4.018,4.030,4.042,&
                    4.054,4.066,4.078,4.089,4.101,4.112,4.123,4.134,4.145,&
                    4.156,4.167,4.178,4.188,4.199,4.209,4.219,4.229,4.239,&
                    4.249,4.259,4.269,4.278,4.288,4.297,4.306,4.316,4.325,&
                    4.334,4.343,4.351,4.360,4.369,4.377,4.386,4.394,4.402,&
                    4.411,4.419,4.427,4.435,4.442,4.450,4.458,4.466,4.473,&
                    4.480,4.488,4.495,4.502,4.510,4.517,4.524,4.531,4.537,&
                    4.544,4.551,4.558,4.564,4.571,4.577,4.583,4.590,4.596,&
                    4.602/)
!
! *** find position in arrays for bincoef
!
    if ( ionic<=0.200000E+02 ) then
      ipos = min(nint(0.200000D+02*ionic)+1,400)
    else
      ipos = 400 + nint(0.200000D+01*ionic-0.400000D+02)
    end if
    ipos = min(ipos,561)
!
! *** assign values to return array
!
    binarr(01) = bnc01m(ipos)
    binarr(02) = bnc02m(ipos)
    binarr(03) = bnc03m(ipos)
    binarr(04) = bnc04m(ipos)
    binarr(05) = bnc05m(ipos)
    binarr(06) = bnc06m(ipos)
    binarr(07) = bnc07m(ipos)
    binarr(08) = bnc08m(ipos)
    binarr(09) = bnc09m(ipos)
    binarr(10) = bnc10m(ipos)
    binarr(11) = bnc11m(ipos)
    binarr(12) = bnc12m(ipos)
    binarr(13) = bnc13m(ipos)
    binarr(14) = bnc14m(ipos)
    binarr(15) = bnc15m(ipos)
    binarr(16) = bnc16m(ipos)
    binarr(17) = bnc17m(ipos)
    binarr(18) = bnc18m(ipos)
    binarr(19) = bnc19m(ipos)
    binarr(20) = bnc20m(ipos)
    binarr(21) = bnc21m(ipos)
    binarr(22) = bnc22m(ipos)
    binarr(23) = bnc23m(ipos)
!
! *** return point ; end of subroutine
!
  end subroutine km223
!
!======================================================================
!
! *** isorropia code
! *** subroutine km248
! *** calculates binary activity coefficients by kusik-meissner method.
!     the computations have been performed and the results are stored in
!     lookup tables. the ionic activity 'in' is input , and the array
!     'binarr' is returned with the binary coefficients.
!
!     temperature is 248k
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine km248(ionic,binarr)
    implicit none
    integer(ik4) :: ipos
    real(rkx) , dimension(23) :: binarr
    real(rkx) :: ionic
    real(rkx) , dimension(561) :: bnc01m , bnc02m , bnc03m , bnc04m ,     &
                                  bnc05m , bnc06m , bnc07m , bnc08m ,     &
                                  bnc09m , bnc10m , bnc11m , bnc12m ,     &
                                  bnc13m , bnc14m , bnc15m , bnc16m ,     &
                                  bnc17m , bnc18m , bnc19m , bnc20m ,     &
                                  bnc21m , bnc22m , bnc23m
!
! *** nacl
!
    bnc01m(1:561) = (/-0.047,-0.099,-0.122,-0.137,-0.148,-0.156,-0.163,   &
                    -0.168,-0.172,-0.176,-0.178,-0.181,-0.182,-0.184,     &
                    -0.185,-0.186,-0.186,-0.187,-0.187,-0.187,-0.187,     &
                    -0.187,-0.186,-0.186,-0.185,-0.184,-0.184,-0.183,     &
                    -0.182,-0.181,-0.180,-0.179,-0.178,-0.176,-0.175,     &
                    -0.174,-0.173,-0.171,-0.170,-0.168,-0.167,-0.166,     &
                    -0.164,-0.163,-0.161,-0.160,-0.158,-0.157,-0.155,     &
                    -0.154,-0.152,-0.151,-0.149,-0.148,-0.146,-0.145,     &
                    -0.143,-0.141,-0.140,-0.138,-0.137,-0.135,-0.134,     &
                    -0.132,-0.130,-0.129,-0.127,-0.126,-0.124,-0.122,     &
                    -0.121,-0.119,-0.117,-0.116,-0.114,-0.112,-0.111,     &
                    -0.109,-0.107,-0.105,-0.104,-0.102,-0.100,-0.098,     &
                    -0.096,-0.095,-0.093,-0.091,-0.089,-0.087,-0.085,     &
                    -0.083,-0.081,-0.079,-0.077,-0.075,-0.073,-0.071,     &
                    -0.069,-0.067,-0.065,-0.063,-0.061,-0.059,-0.057,     &
                    -0.055,-0.053,-0.051,-0.048,-0.046,-0.044,-0.042,     &
                    -0.040,-0.038,-0.036,-0.033,-0.031,-0.029,-0.027,     &
                    -0.025,-0.022,-0.020,-0.018,-0.016,-0.014,-0.011,     &
                    -0.009,-0.007,-0.005,-0.003,0.000,0.002,0.004,0.006,  &
                    0.008,0.011,0.013,0.015,0.017,0.020,0.022,0.024,0.026,&
                    0.028,0.030,0.033,0.035,0.037,0.039,0.041,0.044,0.046,&
                    0.048,0.050,0.052,0.054,0.057,0.059,0.061,0.063,0.065,&
                    0.067,0.070,0.072,0.074,0.076,0.078,0.080,0.082,0.085,&
                    0.087,0.089,0.091,0.093,0.095,0.097,0.099,0.102,0.104,&
                    0.106,0.108,0.110,0.112,0.114,0.116,0.118,0.120,0.122,&
                    0.125,0.127,0.129,0.131,0.133,0.135,0.137,0.139,0.141,&
                    0.143,0.145,0.147,0.149,0.151,0.153,0.155,0.157,0.159,&
                    0.161,0.163,0.165,0.167,0.169,0.171,0.173,0.175,0.177,&
                    0.179,0.181,0.183,0.185,0.187,0.189,0.191,0.193,0.195,&
                    0.197,0.199,0.201,0.203,0.205,0.207,0.209,0.211,0.213,&
                    0.215,0.217,0.219,0.221,0.223,0.224,0.226,0.228,0.230,&
                    0.232,0.234,0.236,0.238,0.240,0.242,0.243,0.245,0.247,&
                    0.249,0.251,0.253,0.255,0.257,0.258,0.260,0.262,0.264,&
                    0.266,0.268,0.270,0.271,0.273,0.275,0.277,0.279,0.281,&
                    0.282,0.284,0.286,0.288,0.290,0.291,0.293,0.295,0.297,&
                    0.299,0.300,0.302,0.304,0.306,0.308,0.309,0.311,0.313,&
                    0.315,0.316,0.318,0.320,0.322,0.323,0.325,0.327,0.329,&
                    0.330,0.332,0.334,0.336,0.337,0.339,0.341,0.342,0.344,&
                    0.346,0.348,0.349,0.351,0.353,0.354,0.356,0.358,0.359,&
                    0.361,0.363,0.364,0.366,0.368,0.370,0.371,0.373,0.374,&
                    0.376,0.378,0.379,0.381,0.383,0.384,0.386,0.388,0.389,&
                    0.391,0.393,0.394,0.396,0.397,0.399,0.401,0.402,0.404,&
                    0.406,0.407,0.409,0.410,0.412,0.414,0.415,0.417,0.418,&
                    0.420,0.421,0.423,0.425,0.426,0.428,0.429,0.431,0.433,&
                    0.434,0.436,0.437,0.439,0.440,0.442,0.443,0.445,0.446,&
                    0.448,0.450,0.451,0.453,0.454,0.456,0.457,0.459,0.460,&
                    0.462,0.463,0.465,0.466,0.468,0.469,0.471,0.472,0.474,&
                    0.475,0.477,0.478,0.480,0.481,0.483,0.484,0.486,0.487,&
                    0.489,0.490,0.491,0.493,0.494,0.510,0.524,0.538,0.552,&
                    0.566,0.579,0.592,0.605,0.618,0.630,0.642,0.655,0.667,&
                    0.678,0.690,0.701,0.713,0.724,0.735,0.745,0.756,0.766,&
                    0.777,0.787,0.797,0.807,0.817,0.826,0.836,0.845,0.854,&
                    0.864,0.873,0.881,0.890,0.899,0.907,0.916,0.924,0.932,&
                    0.940,0.948,0.956,0.964,0.972,0.979,0.987,0.994,1.002,&
                    1.009,1.016,1.023,1.030,1.037,1.044,1.051,1.057,1.064,&
                    1.070,1.077,1.083,1.089,1.096,1.102,1.108,1.114,1.120,&
                    1.126,1.132,1.137,1.143,1.149,1.154,1.160,1.165,1.170,&
                    1.176,1.181,1.186,1.191,1.196,1.201,1.206,1.211,1.216,&
                    1.221,1.226,1.231,1.235,1.240,1.244,1.249,1.253,1.258,&
                    1.262,1.267,1.271,1.275,1.279,1.284,1.288,1.292,1.296,&
                    1.300,1.304,1.308,1.312,1.315,1.319,1.323,1.327,1.330,&
                    1.334,1.338,1.341,1.345,1.348,1.352,1.355,1.358,1.362,&
                    1.365,1.368,1.372,1.375,1.378,1.381,1.384,1.388,1.391,&
                    1.394,1.397,1.400,1.403,1.406,1.409,1.411,1.414,1.417,&
                    1.420,1.423,1.425,1.428,1.431,1.433,1.436,1.439,1.441,&
                    1.444,1.446,1.449,1.451,1.454,1.456,1.458,1.461,1.463,&
                    1.466,1.468,1.470,1.472/)
!
! *** na2so4
!
    bnc02m(1:561) = (/-0.098,-0.214,-0.272,-0.313,-0.346,-0.373,-0.396,   &
                    -0.417,-0.435,-0.452,-0.467,-0.481,-0.494,-0.506,     &
                    -0.518,-0.529,-0.539,-0.549,-0.558,-0.567,-0.575,     &
                    -0.583,-0.591,-0.599,-0.606,-0.613,-0.620,-0.627,     &
                    -0.633,-0.639,-0.645,-0.651,-0.657,-0.662,-0.668,     &
                    -0.673,-0.678,-0.683,-0.688,-0.693,-0.698,-0.703,     &
                    -0.707,-0.712,-0.716,-0.721,-0.725,-0.729,-0.733,     &
                    -0.737,-0.741,-0.745,-0.749,-0.753,-0.756,-0.760,     &
                    -0.764,-0.767,-0.771,-0.774,-0.778,-0.781,-0.784,     &
                    -0.788,-0.791,-0.794,-0.797,-0.801,-0.804,-0.807,     &
                    -0.810,-0.813,-0.816,-0.819,-0.822,-0.825,-0.827,     &
                    -0.830,-0.833,-0.836,-0.839,-0.841,-0.844,-0.847,     &
                    -0.850,-0.852,-0.855,-0.858,-0.860,-0.863,-0.865,     &
                    -0.868,-0.871,-0.873,-0.876,-0.878,-0.881,-0.883,     &
                    -0.885,-0.888,-0.890,-0.893,-0.895,-0.897,-0.900,     &
                    -0.902,-0.905,-0.907,-0.909,-0.911,-0.914,-0.916,     &
                    -0.918,-0.921,-0.923,-0.925,-0.927,-0.929,-0.932,     &
                    -0.934,-0.936,-0.938,-0.940,-0.942,-0.945,-0.947,     &
                    -0.949,-0.951,-0.953,-0.955,-0.957,-0.959,-0.961,     &
                    -0.963,-0.965,-0.967,-0.969,-0.971,-0.973,-0.975,     &
                    -0.977,-0.979,-0.981,-0.983,-0.985,-0.987,-0.989,     &
                    -0.991,-0.993,-0.995,-0.997,-0.999,-1.001,-1.002,     &
                    -1.004,-1.006,-1.008,-1.010,-1.012,-1.014,-1.015,     &
                    -1.017,-1.019,-1.021,-1.023,-1.025,-1.026,-1.028,     &
                    -1.030,-1.032,-1.033,-1.035,-1.037,-1.039,-1.040,     &
                    -1.042,-1.044,-1.046,-1.047,-1.049,-1.051,-1.053,     &
                    -1.054,-1.056,-1.058,-1.059,-1.061,-1.063,-1.064,     &
                    -1.066,-1.068,-1.069,-1.071,-1.073,-1.074,-1.076,     &
                    -1.078,-1.079,-1.081,-1.082,-1.084,-1.086,-1.087,     &
                    -1.089,-1.090,-1.092,-1.094,-1.095,-1.097,-1.098,     &
                    -1.100,-1.102,-1.103,-1.105,-1.106,-1.108,-1.109,     &
                    -1.111,-1.112,-1.114,-1.116,-1.117,-1.119,-1.120,     &
                    -1.122,-1.123,-1.125,-1.126,-1.128,-1.129,-1.131,     &
                    -1.132,-1.134,-1.135,-1.137,-1.138,-1.140,-1.141,     &
                    -1.143,-1.144,-1.146,-1.147,-1.148,-1.150,-1.151,     &
                    -1.153,-1.154,-1.156,-1.157,-1.159,-1.160,-1.161,     &
                    -1.163,-1.164,-1.166,-1.167,-1.169,-1.170,-1.171,     &
                    -1.173,-1.174,-1.176,-1.177,-1.178,-1.180,-1.181,     &
                    -1.183,-1.184,-1.185,-1.187,-1.188,-1.190,-1.191,     &
                    -1.192,-1.194,-1.195,-1.196,-1.198,-1.199,-1.201,     &
                    -1.202,-1.203,-1.205,-1.206,-1.207,-1.209,-1.210,     &
                    -1.211,-1.213,-1.214,-1.215,-1.217,-1.218,-1.219,     &
                    -1.221,-1.222,-1.223,-1.225,-1.226,-1.227,-1.229,     &
                    -1.230,-1.231,-1.233,-1.234,-1.235,-1.236,-1.238,     &
                    -1.239,-1.240,-1.242,-1.243,-1.244,-1.246,-1.247,     &
                    -1.248,-1.249,-1.251,-1.252,-1.253,-1.255,-1.256,     &
                    -1.257,-1.258,-1.260,-1.261,-1.262,-1.263,-1.265,     &
                    -1.266,-1.267,-1.268,-1.270,-1.271,-1.272,-1.273,     &
                    -1.275,-1.276,-1.277,-1.278,-1.280,-1.281,-1.282,     &
                    -1.283,-1.285,-1.286,-1.287,-1.288,-1.290,-1.291,     &
                    -1.292,-1.293,-1.294,-1.296,-1.297,-1.298,-1.299,     &
                    -1.301,-1.302,-1.303,-1.304,-1.305,-1.307,-1.308,     &
                    -1.309,-1.310,-1.311,-1.313,-1.314,-1.315,-1.316,     &
                    -1.317,-1.319,-1.320,-1.321,-1.322,-1.323,-1.324,     &
                    -1.326,-1.327,-1.328,-1.329,-1.330,-1.332,-1.333,     &
                    -1.334,-1.335,-1.336,-1.337,-1.339,-1.340,-1.341,     &
                    -1.342,-1.343,-1.344,-1.346,-1.347,-1.348,-1.349,     &
                    -1.350,-1.363,-1.374,-1.385,-1.396,-1.407,-1.418,     &
                    -1.429,-1.440,-1.451,-1.461,-1.472,-1.482,-1.492,     &
                    -1.503,-1.513,-1.523,-1.533,-1.543,-1.553,-1.563,     &
                    -1.573,-1.582,-1.592,-1.602,-1.611,-1.621,-1.630,     &
                    -1.640,-1.649,-1.658,-1.668,-1.677,-1.686,-1.695,     &
                    -1.704,-1.714,-1.723,-1.732,-1.741,-1.750,-1.758,     &
                    -1.767,-1.776,-1.785,-1.794,-1.802,-1.811,-1.820,     &
                    -1.828,-1.837,-1.846,-1.854,-1.863,-1.871,-1.880,     &
                    -1.888,-1.897,-1.905,-1.913,-1.922,-1.930,-1.938,     &
                    -1.947,-1.955,-1.963,-1.971,-1.979,-1.988,-1.996,     &
                    -2.004,-2.012,-2.020,-2.028,-2.036,-2.044,-2.052,     &
                    -2.060,-2.068,-2.076,-2.084,-2.092,-2.100,-2.108,     &
                    -2.116,-2.124,-2.131,-2.139,-2.147,-2.155,-2.163,     &
                    -2.170,-2.178,-2.186,-2.194,-2.201,-2.209,-2.217,     &
                    -2.224,-2.232,-2.240,-2.247,-2.255,-2.262,-2.270,     &
                    -2.278,-2.285,-2.293,-2.300,-2.308,-2.315,-2.323,     &
                    -2.330,-2.338,-2.345,-2.353,-2.360,-2.368,-2.375,     &
                    -2.382,-2.390,-2.397,-2.405,-2.412,-2.419,-2.427,     &
                    -2.434,-2.441,-2.449,-2.456,-2.463,-2.471,-2.478,     &
                    -2.485,-2.492,-2.500,-2.507,-2.514,-2.521,-2.529,     &
                    -2.536,-2.543,-2.550,-2.557,-2.565,-2.572,-2.579,     &
                    -2.586,-2.593,-2.600,-2.608,-2.615,-2.622,-2.629,     &
                    -2.636,-2.643,-2.650,-2.657,-2.664,-2.671,-2.679,     &
                    -2.686/)
!
! *** nano3
!
    bnc03m(1:561) = (/-0.049,-0.108,-0.137,-0.159,-0.175,-0.190,-0.202,   &
                    -0.212,-0.222,-0.231,-0.239,-0.247,-0.254,-0.260,     &
                    -0.266,-0.272,-0.278,-0.283,-0.288,-0.293,-0.298,     &
                    -0.302,-0.306,-0.311,-0.315,-0.319,-0.322,-0.326,     &
                    -0.330,-0.333,-0.337,-0.340,-0.343,-0.346,-0.349,     &
                    -0.352,-0.355,-0.358,-0.361,-0.364,-0.367,-0.369,     &
                    -0.372,-0.374,-0.377,-0.379,-0.382,-0.384,-0.387,     &
                    -0.389,-0.391,-0.394,-0.396,-0.398,-0.400,-0.402,     &
                    -0.404,-0.406,-0.408,-0.410,-0.412,-0.414,-0.416,     &
                    -0.418,-0.420,-0.422,-0.424,-0.426,-0.428,-0.429,     &
                    -0.431,-0.433,-0.435,-0.436,-0.438,-0.440,-0.442,     &
                    -0.443,-0.445,-0.447,-0.448,-0.450,-0.452,-0.453,     &
                    -0.455,-0.456,-0.458,-0.459,-0.461,-0.463,-0.464,     &
                    -0.466,-0.467,-0.469,-0.470,-0.472,-0.473,-0.475,     &
                    -0.476,-0.478,-0.479,-0.481,-0.482,-0.483,-0.485,     &
                    -0.486,-0.488,-0.489,-0.491,-0.492,-0.493,-0.495,     &
                    -0.496,-0.498,-0.499,-0.500,-0.502,-0.503,-0.504,     &
                    -0.506,-0.507,-0.508,-0.510,-0.511,-0.512,-0.514,     &
                    -0.515,-0.516,-0.518,-0.519,-0.520,-0.521,-0.523,     &
                    -0.524,-0.525,-0.526,-0.528,-0.529,-0.530,-0.531,     &
                    -0.533,-0.534,-0.535,-0.536,-0.537,-0.539,-0.540,     &
                    -0.541,-0.542,-0.543,-0.545,-0.546,-0.547,-0.548,     &
                    -0.549,-0.550,-0.552,-0.553,-0.554,-0.555,-0.556,     &
                    -0.557,-0.558,-0.559,-0.561,-0.562,-0.563,-0.564,     &
                    -0.565,-0.566,-0.567,-0.568,-0.569,-0.570,-0.572,     &
                    -0.573,-0.574,-0.575,-0.576,-0.577,-0.578,-0.579,     &
                    -0.580,-0.581,-0.582,-0.583,-0.584,-0.585,-0.586,     &
                    -0.587,-0.588,-0.589,-0.590,-0.592,-0.593,-0.594,     &
                    -0.595,-0.596,-0.597,-0.598,-0.599,-0.600,-0.601,     &
                    -0.602,-0.603,-0.604,-0.605,-0.606,-0.607,-0.607,     &
                    -0.608,-0.609,-0.610,-0.611,-0.612,-0.613,-0.614,     &
                    -0.615,-0.616,-0.617,-0.618,-0.619,-0.620,-0.621,     &
                    -0.622,-0.623,-0.624,-0.625,-0.626,-0.627,-0.627,     &
                    -0.628,-0.629,-0.630,-0.631,-0.632,-0.633,-0.634,     &
                    -0.635,-0.636,-0.637,-0.638,-0.638,-0.639,-0.640,     &
                    -0.641,-0.642,-0.643,-0.644,-0.645,-0.646,-0.646,     &
                    -0.647,-0.648,-0.649,-0.650,-0.651,-0.652,-0.653,     &
                    -0.654,-0.654,-0.655,-0.656,-0.657,-0.658,-0.659,     &
                    -0.660,-0.660,-0.661,-0.662,-0.663,-0.664,-0.665,     &
                    -0.666,-0.666,-0.667,-0.668,-0.669,-0.670,-0.671,     &
                    -0.671,-0.672,-0.673,-0.674,-0.675,-0.676,-0.676,     &
                    -0.677,-0.678,-0.679,-0.680,-0.681,-0.681,-0.682,     &
                    -0.683,-0.684,-0.685,-0.685,-0.686,-0.687,-0.688,     &
                    -0.689,-0.689,-0.690,-0.691,-0.692,-0.693,-0.693,     &
                    -0.694,-0.695,-0.696,-0.697,-0.697,-0.698,-0.699,     &
                    -0.700,-0.701,-0.701,-0.702,-0.703,-0.704,-0.704,     &
                    -0.705,-0.706,-0.707,-0.708,-0.708,-0.709,-0.710,     &
                    -0.711,-0.711,-0.712,-0.713,-0.714,-0.714,-0.715,     &
                    -0.716,-0.717,-0.718,-0.718,-0.719,-0.720,-0.721,     &
                    -0.721,-0.722,-0.723,-0.724,-0.724,-0.725,-0.726,     &
                    -0.727,-0.727,-0.728,-0.729,-0.729,-0.730,-0.731,     &
                    -0.732,-0.732,-0.733,-0.734,-0.735,-0.735,-0.736,     &
                    -0.737,-0.738,-0.738,-0.739,-0.740,-0.740,-0.741,     &
                    -0.742,-0.743,-0.743,-0.744,-0.745,-0.746,-0.746,     &
                    -0.747,-0.748,-0.748,-0.749,-0.750,-0.751,-0.751,     &
                    -0.752,-0.753,-0.753,-0.754,-0.755,-0.756,-0.756,     &
                    -0.757,-0.758,-0.758,-0.759,-0.760,-0.760,-0.761,     &
                    -0.762,-0.769,-0.776,-0.783,-0.790,-0.796,-0.803,     &
                    -0.809,-0.816,-0.822,-0.828,-0.835,-0.841,-0.847,     &
                    -0.853,-0.859,-0.865,-0.871,-0.877,-0.883,-0.888,     &
                    -0.894,-0.900,-0.905,-0.911,-0.917,-0.922,-0.928,     &
                    -0.933,-0.939,-0.944,-0.949,-0.955,-0.960,-0.965,     &
                    -0.971,-0.976,-0.981,-0.986,-0.991,-0.997,-1.002,     &
                    -1.007,-1.012,-1.017,-1.022,-1.027,-1.032,-1.037,     &
                    -1.042,-1.046,-1.051,-1.056,-1.061,-1.066,-1.071,     &
                    -1.075,-1.080,-1.085,-1.089,-1.094,-1.099,-1.104,     &
                    -1.108,-1.113,-1.117,-1.122,-1.127,-1.131,-1.136,     &
                    -1.140,-1.145,-1.149,-1.154,-1.158,-1.163,-1.167,     &
                    -1.172,-1.176,-1.180,-1.185,-1.189,-1.194,-1.198,     &
                    -1.202,-1.207,-1.211,-1.215,-1.220,-1.224,-1.228,     &
                    -1.232,-1.237,-1.241,-1.245,-1.249,-1.254,-1.258,     &
                    -1.262,-1.266,-1.270,-1.275,-1.279,-1.283,-1.287,     &
                    -1.291,-1.295,-1.299,-1.304,-1.308,-1.312,-1.316,     &
                    -1.320,-1.324,-1.328,-1.332,-1.336,-1.340,-1.344,     &
                    -1.348,-1.352,-1.356,-1.360,-1.364,-1.368,-1.372,     &
                    -1.376,-1.380,-1.384,-1.388,-1.392,-1.396,-1.400,     &
                    -1.404,-1.408,-1.412,-1.415,-1.419,-1.423,-1.427,     &
                    -1.431,-1.435,-1.439,-1.443,-1.446,-1.450,-1.454,     &
                    -1.458,-1.462,-1.466,-1.469,-1.473,-1.477,-1.481,     &
                    -1.485,-1.488,-1.492,-1.496,-1.500,-1.504,-1.507,     &
                    -1.511/)
!
! *** (nh4)2so4
!
    bnc04m(1:561) = (/-0.098,-0.214,-0.273,-0.315,-0.347,-0.375,-0.399,   &
                    -0.419,-0.438,-0.455,-0.470,-0.485,-0.498,-0.511,     &
                    -0.522,-0.533,-0.544,-0.554,-0.563,-0.572,-0.581,     &
                    -0.590,-0.598,-0.605,-0.613,-0.620,-0.627,-0.634,     &
                    -0.641,-0.647,-0.653,-0.660,-0.666,-0.671,-0.677,     &
                    -0.683,-0.688,-0.693,-0.698,-0.703,-0.708,-0.713,     &
                    -0.718,-0.723,-0.727,-0.732,-0.736,-0.741,-0.745,     &
                    -0.749,-0.753,-0.757,-0.762,-0.765,-0.769,-0.773,     &
                    -0.777,-0.781,-0.784,-0.788,-0.792,-0.795,-0.799,     &
                    -0.802,-0.806,-0.809,-0.812,-0.816,-0.819,-0.822,     &
                    -0.825,-0.829,-0.832,-0.835,-0.838,-0.841,-0.844,     &
                    -0.847,-0.850,-0.853,-0.856,-0.859,-0.862,-0.865,     &
                    -0.867,-0.870,-0.873,-0.876,-0.879,-0.881,-0.884,     &
                    -0.887,-0.890,-0.892,-0.895,-0.898,-0.900,-0.903,     &
                    -0.905,-0.908,-0.911,-0.913,-0.916,-0.918,-0.921,     &
                    -0.923,-0.926,-0.928,-0.931,-0.933,-0.936,-0.938,     &
                    -0.940,-0.943,-0.945,-0.948,-0.950,-0.952,-0.955,     &
                    -0.957,-0.959,-0.962,-0.964,-0.966,-0.969,-0.971,     &
                    -0.973,-0.975,-0.978,-0.980,-0.982,-0.984,-0.986,     &
                    -0.989,-0.991,-0.993,-0.995,-0.997,-0.999,-1.002,     &
                    -1.004,-1.006,-1.008,-1.010,-1.012,-1.014,-1.016,     &
                    -1.018,-1.020,-1.022,-1.024,-1.026,-1.028,-1.030,     &
                    -1.032,-1.034,-1.036,-1.038,-1.040,-1.042,-1.044,     &
                    -1.046,-1.048,-1.050,-1.052,-1.054,-1.056,-1.058,     &
                    -1.060,-1.062,-1.064,-1.066,-1.067,-1.069,-1.071,     &
                    -1.073,-1.075,-1.077,-1.079,-1.080,-1.082,-1.084,     &
                    -1.086,-1.088,-1.090,-1.091,-1.093,-1.095,-1.097,     &
                    -1.099,-1.100,-1.102,-1.104,-1.106,-1.107,-1.109,     &
                    -1.111,-1.113,-1.114,-1.116,-1.118,-1.120,-1.121,     &
                    -1.123,-1.125,-1.127,-1.128,-1.130,-1.132,-1.133,     &
                    -1.135,-1.137,-1.138,-1.140,-1.142,-1.143,-1.145,     &
                    -1.147,-1.148,-1.150,-1.152,-1.153,-1.155,-1.157,     &
                    -1.158,-1.160,-1.162,-1.163,-1.165,-1.166,-1.168,     &
                    -1.170,-1.171,-1.173,-1.174,-1.176,-1.178,-1.179,     &
                    -1.181,-1.182,-1.184,-1.185,-1.187,-1.189,-1.190,     &
                    -1.192,-1.193,-1.195,-1.196,-1.198,-1.199,-1.201,     &
                    -1.203,-1.204,-1.206,-1.207,-1.209,-1.210,-1.212,     &
                    -1.213,-1.215,-1.216,-1.218,-1.219,-1.221,-1.222,     &
                    -1.224,-1.225,-1.227,-1.228,-1.230,-1.231,-1.233,     &
                    -1.234,-1.236,-1.237,-1.238,-1.240,-1.241,-1.243,     &
                    -1.244,-1.246,-1.247,-1.249,-1.250,-1.252,-1.253,     &
                    -1.254,-1.256,-1.257,-1.259,-1.260,-1.262,-1.263,     &
                    -1.264,-1.266,-1.267,-1.269,-1.270,-1.271,-1.273,     &
                    -1.274,-1.276,-1.277,-1.278,-1.280,-1.281,-1.283,     &
                    -1.284,-1.285,-1.287,-1.288,-1.290,-1.291,-1.292,     &
                    -1.294,-1.295,-1.296,-1.298,-1.299,-1.301,-1.302,     &
                    -1.303,-1.305,-1.306,-1.307,-1.309,-1.310,-1.311,     &
                    -1.313,-1.314,-1.315,-1.317,-1.318,-1.319,-1.321,     &
                    -1.322,-1.323,-1.325,-1.326,-1.327,-1.329,-1.330,     &
                    -1.331,-1.333,-1.334,-1.335,-1.337,-1.338,-1.339,     &
                    -1.341,-1.342,-1.343,-1.344,-1.346,-1.347,-1.348,     &
                    -1.350,-1.351,-1.352,-1.354,-1.355,-1.356,-1.357,     &
                    -1.359,-1.360,-1.361,-1.363,-1.364,-1.365,-1.366,     &
                    -1.368,-1.369,-1.370,-1.371,-1.373,-1.374,-1.375,     &
                    -1.376,-1.378,-1.379,-1.380,-1.381,-1.383,-1.384,     &
                    -1.385,-1.387,-1.388,-1.389,-1.390,-1.391,-1.393,     &
                    -1.394,-1.395,-1.396,-1.398,-1.399,-1.400,-1.401,     &
                    -1.403,-1.416,-1.428,-1.440,-1.452,-1.463,-1.475,     &
                    -1.487,-1.498,-1.509,-1.520,-1.532,-1.543,-1.554,     &
                    -1.564,-1.575,-1.586,-1.596,-1.607,-1.617,-1.628,     &
                    -1.638,-1.648,-1.659,-1.669,-1.679,-1.689,-1.699,     &
                    -1.709,-1.719,-1.729,-1.738,-1.748,-1.758,-1.767,     &
                    -1.777,-1.786,-1.796,-1.805,-1.815,-1.824,-1.833,     &
                    -1.843,-1.852,-1.861,-1.870,-1.879,-1.888,-1.897,     &
                    -1.906,-1.915,-1.924,-1.933,-1.942,-1.951,-1.960,     &
                    -1.969,-1.977,-1.986,-1.995,-2.004,-2.012,-2.021,     &
                    -2.029,-2.038,-2.047,-2.055,-2.064,-2.072,-2.081,     &
                    -2.089,-2.097,-2.106,-2.114,-2.123,-2.131,-2.139,     &
                    -2.147,-2.156,-2.164,-2.172,-2.180,-2.189,-2.197,     &
                    -2.205,-2.213,-2.221,-2.229,-2.237,-2.245,-2.253,     &
                    -2.261,-2.269,-2.277,-2.285,-2.293,-2.301,-2.309,     &
                    -2.317,-2.325,-2.333,-2.341,-2.349,-2.356,-2.364,     &
                    -2.372,-2.380,-2.388,-2.395,-2.403,-2.411,-2.419,     &
                    -2.426,-2.434,-2.442,-2.449,-2.457,-2.465,-2.472,     &
                    -2.480,-2.488,-2.495,-2.503,-2.510,-2.518,-2.525,     &
                    -2.533,-2.541,-2.548,-2.556,-2.563,-2.571,-2.578,     &
                    -2.586,-2.593,-2.600,-2.608,-2.615,-2.623,-2.630,     &
                    -2.638,-2.645,-2.652,-2.660,-2.667,-2.674,-2.682,     &
                    -2.689,-2.696,-2.704,-2.711,-2.718,-2.726,-2.733,     &
                    -2.740,-2.747,-2.755,-2.762,-2.769,-2.776,-2.784,     &
                    -2.791/)
!
! *** nh4no3
!
    bnc05m(1:561) = (/-0.050,-0.111,-0.143,-0.166,-0.185,-0.202,-0.216,   &
                    -0.229,-0.240,-0.251,-0.261,-0.271,-0.279,-0.288,     &
                    -0.296,-0.304,-0.311,-0.318,-0.325,-0.331,-0.338,     &
                    -0.344,-0.350,-0.356,-0.361,-0.367,-0.372,-0.378,     &
                    -0.383,-0.388,-0.393,-0.398,-0.402,-0.407,-0.412,     &
                    -0.416,-0.420,-0.425,-0.429,-0.433,-0.437,-0.441,     &
                    -0.445,-0.449,-0.453,-0.457,-0.460,-0.464,-0.468,     &
                    -0.471,-0.475,-0.478,-0.482,-0.485,-0.488,-0.491,     &
                    -0.495,-0.498,-0.501,-0.504,-0.507,-0.510,-0.513,     &
                    -0.516,-0.519,-0.522,-0.525,-0.528,-0.531,-0.534,     &
                    -0.537,-0.539,-0.542,-0.545,-0.548,-0.550,-0.553,     &
                    -0.556,-0.558,-0.561,-0.564,-0.566,-0.569,-0.572,     &
                    -0.574,-0.577,-0.579,-0.582,-0.584,-0.587,-0.589,     &
                    -0.592,-0.594,-0.597,-0.599,-0.602,-0.604,-0.607,     &
                    -0.609,-0.612,-0.614,-0.617,-0.619,-0.621,-0.624,     &
                    -0.626,-0.629,-0.631,-0.633,-0.636,-0.638,-0.640,     &
                    -0.643,-0.645,-0.647,-0.650,-0.652,-0.654,-0.656,     &
                    -0.659,-0.661,-0.663,-0.665,-0.668,-0.670,-0.672,     &
                    -0.674,-0.677,-0.679,-0.681,-0.683,-0.685,-0.687,     &
                    -0.690,-0.692,-0.694,-0.696,-0.698,-0.700,-0.702,     &
                    -0.704,-0.706,-0.709,-0.711,-0.713,-0.715,-0.717,     &
                    -0.719,-0.721,-0.723,-0.725,-0.727,-0.729,-0.731,     &
                    -0.733,-0.735,-0.737,-0.739,-0.741,-0.742,-0.744,     &
                    -0.746,-0.748,-0.750,-0.752,-0.754,-0.756,-0.758,     &
                    -0.760,-0.761,-0.763,-0.765,-0.767,-0.769,-0.771,     &
                    -0.772,-0.774,-0.776,-0.778,-0.780,-0.782,-0.783,     &
                    -0.785,-0.787,-0.789,-0.790,-0.792,-0.794,-0.796,     &
                    -0.797,-0.799,-0.801,-0.803,-0.804,-0.806,-0.808,     &
                    -0.809,-0.811,-0.813,-0.814,-0.816,-0.818,-0.820,     &
                    -0.821,-0.823,-0.824,-0.826,-0.828,-0.829,-0.831,     &
                    -0.833,-0.834,-0.836,-0.838,-0.839,-0.841,-0.842,     &
                    -0.844,-0.846,-0.847,-0.849,-0.850,-0.852,-0.853,     &
                    -0.855,-0.857,-0.858,-0.860,-0.861,-0.863,-0.864,     &
                    -0.866,-0.867,-0.869,-0.870,-0.872,-0.873,-0.875,     &
                    -0.876,-0.878,-0.879,-0.881,-0.882,-0.884,-0.885,     &
                    -0.887,-0.888,-0.890,-0.891,-0.893,-0.894,-0.896,     &
                    -0.897,-0.898,-0.900,-0.901,-0.903,-0.904,-0.906,     &
                    -0.907,-0.908,-0.910,-0.911,-0.913,-0.914,-0.915,     &
                    -0.917,-0.918,-0.920,-0.921,-0.922,-0.924,-0.925,     &
                    -0.926,-0.928,-0.929,-0.931,-0.932,-0.933,-0.935,     &
                    -0.936,-0.937,-0.939,-0.940,-0.941,-0.943,-0.944,     &
                    -0.945,-0.947,-0.948,-0.949,-0.951,-0.952,-0.953,     &
                    -0.954,-0.956,-0.957,-0.958,-0.960,-0.961,-0.962,     &
                    -0.963,-0.965,-0.966,-0.967,-0.969,-0.970,-0.971,     &
                    -0.972,-0.974,-0.975,-0.976,-0.977,-0.979,-0.980,     &
                    -0.981,-0.982,-0.984,-0.985,-0.986,-0.987,-0.988,     &
                    -0.990,-0.991,-0.992,-0.993,-0.995,-0.996,-0.997,     &
                    -0.998,-0.999,-1.001,-1.002,-1.003,-1.004,-1.005,     &
                    -1.007,-1.008,-1.009,-1.010,-1.011,-1.012,-1.014,     &
                    -1.015,-1.016,-1.017,-1.018,-1.019,-1.021,-1.022,     &
                    -1.023,-1.024,-1.025,-1.026,-1.027,-1.029,-1.030,     &
                    -1.031,-1.032,-1.033,-1.034,-1.035,-1.037,-1.038,     &
                    -1.039,-1.040,-1.041,-1.042,-1.043,-1.044,-1.045,     &
                    -1.047,-1.048,-1.049,-1.050,-1.051,-1.052,-1.053,     &
                    -1.054,-1.055,-1.056,-1.058,-1.059,-1.060,-1.061,     &
                    -1.062,-1.063,-1.064,-1.065,-1.066,-1.067,-1.068,     &
                    -1.069,-1.070,-1.071,-1.072,-1.074,-1.075,-1.076,     &
                    -1.077,-1.088,-1.098,-1.108,-1.118,-1.128,-1.137,     &
                    -1.147,-1.156,-1.165,-1.174,-1.183,-1.192,-1.200,     &
                    -1.209,-1.217,-1.225,-1.233,-1.241,-1.249,-1.257,     &
                    -1.265,-1.273,-1.280,-1.288,-1.295,-1.303,-1.310,     &
                    -1.317,-1.324,-1.331,-1.338,-1.345,-1.352,-1.359,     &
                    -1.365,-1.372,-1.379,-1.385,-1.392,-1.398,-1.404,     &
                    -1.411,-1.417,-1.423,-1.429,-1.435,-1.441,-1.447,     &
                    -1.453,-1.459,-1.465,-1.471,-1.477,-1.483,-1.488,     &
                    -1.494,-1.500,-1.505,-1.511,-1.516,-1.522,-1.527,     &
                    -1.533,-1.538,-1.544,-1.549,-1.554,-1.560,-1.565,     &
                    -1.570,-1.575,-1.580,-1.586,-1.591,-1.596,-1.601,     &
                    -1.606,-1.611,-1.616,-1.621,-1.626,-1.631,-1.635,     &
                    -1.640,-1.645,-1.650,-1.655,-1.660,-1.664,-1.669,     &
                    -1.674,-1.679,-1.683,-1.688,-1.693,-1.697,-1.702,     &
                    -1.706,-1.711,-1.716,-1.720,-1.725,-1.729,-1.734,     &
                    -1.738,-1.743,-1.747,-1.751,-1.756,-1.760,-1.765,     &
                    -1.769,-1.773,-1.778,-1.782,-1.786,-1.791,-1.795,     &
                    -1.799,-1.803,-1.808,-1.812,-1.816,-1.820,-1.825,     &
                    -1.829,-1.833,-1.837,-1.841,-1.845,-1.850,-1.854,     &
                    -1.858,-1.862,-1.866,-1.870,-1.874,-1.878,-1.882,     &
                    -1.886,-1.890,-1.894,-1.898,-1.902,-1.906,-1.910,     &
                    -1.914,-1.918,-1.922,-1.926,-1.930,-1.934,-1.938,     &
                    -1.942,-1.946,-1.950,-1.954,-1.958,-1.961,-1.965,     &
                    -1.969/)
!
! *** nh4cl
!
    bnc06m(1:561) = (/-0.048,-0.103,-0.130,-0.148,-0.161,-0.173,-0.182,   &
                    -0.190,-0.197,-0.203,-0.208,-0.213,-0.217,-0.221,     &
                    -0.225,-0.228,-0.231,-0.234,-0.237,-0.239,-0.241,     &
                    -0.243,-0.245,-0.247,-0.249,-0.251,-0.252,-0.254,     &
                    -0.255,-0.256,-0.257,-0.259,-0.260,-0.261,-0.262,     &
                    -0.263,-0.263,-0.264,-0.265,-0.266,-0.267,-0.267,     &
                    -0.268,-0.269,-0.269,-0.270,-0.270,-0.271,-0.272,     &
                    -0.272,-0.273,-0.273,-0.273,-0.274,-0.274,-0.275,     &
                    -0.275,-0.275,-0.276,-0.276,-0.277,-0.277,-0.277,     &
                    -0.277,-0.278,-0.278,-0.278,-0.279,-0.279,-0.279,     &
                    -0.279,-0.279,-0.280,-0.280,-0.280,-0.280,-0.280,     &
                    -0.280,-0.280,-0.281,-0.281,-0.281,-0.281,-0.281,     &
                    -0.281,-0.281,-0.281,-0.281,-0.281,-0.281,-0.281,     &
                    -0.281,-0.281,-0.281,-0.281,-0.281,-0.281,-0.281,     &
                    -0.281,-0.281,-0.281,-0.280,-0.280,-0.280,-0.280,     &
                    -0.280,-0.280,-0.280,-0.279,-0.279,-0.279,-0.279,     &
                    -0.279,-0.279,-0.278,-0.278,-0.278,-0.278,-0.278,     &
                    -0.277,-0.277,-0.277,-0.277,-0.277,-0.276,-0.276,     &
                    -0.276,-0.276,-0.275,-0.275,-0.275,-0.275,-0.274,     &
                    -0.274,-0.274,-0.274,-0.273,-0.273,-0.273,-0.273,     &
                    -0.272,-0.272,-0.272,-0.272,-0.271,-0.271,-0.271,     &
                    -0.271,-0.270,-0.270,-0.270,-0.270,-0.269,-0.269,     &
                    -0.269,-0.269,-0.268,-0.268,-0.268,-0.267,-0.267,     &
                    -0.267,-0.267,-0.266,-0.266,-0.266,-0.266,-0.265,     &
                    -0.265,-0.265,-0.264,-0.264,-0.264,-0.264,-0.263,     &
                    -0.263,-0.263,-0.263,-0.262,-0.262,-0.262,-0.261,     &
                    -0.261,-0.261,-0.261,-0.260,-0.260,-0.260,-0.259,     &
                    -0.259,-0.259,-0.259,-0.258,-0.258,-0.258,-0.258,     &
                    -0.257,-0.257,-0.257,-0.256,-0.256,-0.256,-0.256,     &
                    -0.255,-0.255,-0.255,-0.255,-0.254,-0.254,-0.254,     &
                    -0.253,-0.253,-0.253,-0.253,-0.252,-0.252,-0.252,     &
                    -0.252,-0.251,-0.251,-0.251,-0.250,-0.250,-0.250,     &
                    -0.250,-0.249,-0.249,-0.249,-0.249,-0.248,-0.248,     &
                    -0.248,-0.247,-0.247,-0.247,-0.247,-0.246,-0.246,     &
                    -0.246,-0.246,-0.245,-0.245,-0.245,-0.245,-0.244,     &
                    -0.244,-0.244,-0.244,-0.243,-0.243,-0.243,-0.243,     &
                    -0.242,-0.242,-0.242,-0.241,-0.241,-0.241,-0.241,     &
                    -0.240,-0.240,-0.240,-0.240,-0.239,-0.239,-0.239,     &
                    -0.239,-0.238,-0.238,-0.238,-0.238,-0.237,-0.237,     &
                    -0.237,-0.237,-0.236,-0.236,-0.236,-0.236,-0.236,     &
                    -0.235,-0.235,-0.235,-0.235,-0.234,-0.234,-0.234,     &
                    -0.234,-0.233,-0.233,-0.233,-0.233,-0.232,-0.232,     &
                    -0.232,-0.232,-0.231,-0.231,-0.231,-0.231,-0.231,     &
                    -0.230,-0.230,-0.230,-0.230,-0.229,-0.229,-0.229,     &
                    -0.229,-0.228,-0.228,-0.228,-0.228,-0.228,-0.227,     &
                    -0.227,-0.227,-0.227,-0.226,-0.226,-0.226,-0.226,     &
                    -0.226,-0.225,-0.225,-0.225,-0.225,-0.224,-0.224,     &
                    -0.224,-0.224,-0.224,-0.223,-0.223,-0.223,-0.223,     &
                    -0.223,-0.222,-0.222,-0.222,-0.222,-0.221,-0.221,     &
                    -0.221,-0.221,-0.221,-0.220,-0.220,-0.220,-0.220,     &
                    -0.220,-0.219,-0.219,-0.219,-0.219,-0.219,-0.218,     &
                    -0.218,-0.218,-0.218,-0.218,-0.217,-0.217,-0.217,     &
                    -0.217,-0.217,-0.216,-0.216,-0.216,-0.216,-0.216,     &
                    -0.215,-0.215,-0.215,-0.215,-0.215,-0.215,-0.214,     &
                    -0.214,-0.214,-0.214,-0.214,-0.213,-0.213,-0.213,     &
                    -0.213,-0.213,-0.212,-0.212,-0.212,-0.212,-0.212,     &
                    -0.212,-0.211,-0.211,-0.211,-0.211,-0.211,-0.210,     &
                    -0.210,-0.208,-0.207,-0.205,-0.203,-0.202,-0.200,     &
                    -0.199,-0.198,-0.196,-0.195,-0.194,-0.192,-0.191,     &
                    -0.190,-0.189,-0.188,-0.187,-0.186,-0.185,-0.184,     &
                    -0.183,-0.182,-0.182,-0.181,-0.180,-0.179,-0.179,     &
                    -0.178,-0.178,-0.177,-0.176,-0.176,-0.176,-0.175,     &
                    -0.175,-0.174,-0.174,-0.174,-0.173,-0.173,-0.173,     &
                    -0.173,-0.172,-0.172,-0.172,-0.172,-0.172,-0.172,     &
                    -0.172,-0.172,-0.172,-0.172,-0.172,-0.172,-0.172,     &
                    -0.172,-0.172,-0.172,-0.173,-0.173,-0.173,-0.173,     &
                    -0.173,-0.174,-0.174,-0.174,-0.175,-0.175,-0.175,     &
                    -0.176,-0.176,-0.176,-0.177,-0.177,-0.178,-0.178,     &
                    -0.179,-0.179,-0.180,-0.180,-0.181,-0.181,-0.182,     &
                    -0.183,-0.183,-0.184,-0.184,-0.185,-0.186,-0.186,     &
                    -0.187,-0.188,-0.188,-0.189,-0.190,-0.191,-0.191,     &
                    -0.192,-0.193,-0.194,-0.194,-0.195,-0.196,-0.197,     &
                    -0.198,-0.199,-0.200,-0.200,-0.201,-0.202,-0.203,     &
                    -0.204,-0.205,-0.206,-0.207,-0.208,-0.209,-0.210,     &
                    -0.211,-0.212,-0.213,-0.214,-0.215,-0.216,-0.217,     &
                    -0.218,-0.219,-0.220,-0.221,-0.222,-0.223,-0.224,     &
                    -0.226,-0.227,-0.228,-0.229,-0.230,-0.231,-0.232,     &
                    -0.234,-0.235,-0.236,-0.237,-0.238,-0.239,-0.241,     &
                    -0.242,-0.243,-0.244,-0.246,-0.247,-0.248,-0.249,     &
                    -0.251,-0.252,-0.253,-0.254,-0.256,-0.257,-0.258,     &
                    -0.260/)
!
! *** (2h ,so4)
!
    bnc07m(1:561) = (/-0.098,-0.213,-0.271,-0.312,-0.344,-0.370,-0.393,   &
                    -0.413,-0.431,-0.448,-0.462,-0.476,-0.489,-0.500,     &
                    -0.511,-0.522,-0.532,-0.541,-0.550,-0.558,-0.567,     &
                    -0.574,-0.582,-0.589,-0.596,-0.603,-0.609,-0.615,     &
                    -0.621,-0.627,-0.633,-0.638,-0.644,-0.649,-0.654,     &
                    -0.659,-0.664,-0.669,-0.674,-0.678,-0.683,-0.687,     &
                    -0.691,-0.695,-0.700,-0.704,-0.708,-0.711,-0.715,     &
                    -0.719,-0.723,-0.726,-0.730,-0.733,-0.737,-0.740,     &
                    -0.744,-0.747,-0.750,-0.753,-0.757,-0.760,-0.763,     &
                    -0.766,-0.769,-0.772,-0.775,-0.778,-0.781,-0.784,     &
                    -0.786,-0.789,-0.792,-0.795,-0.797,-0.800,-0.803,     &
                    -0.805,-0.808,-0.811,-0.813,-0.816,-0.818,-0.821,     &
                    -0.823,-0.825,-0.828,-0.830,-0.833,-0.835,-0.837,     &
                    -0.840,-0.842,-0.844,-0.847,-0.849,-0.851,-0.853,     &
                    -0.856,-0.858,-0.860,-0.862,-0.864,-0.867,-0.869,     &
                    -0.871,-0.873,-0.875,-0.877,-0.879,-0.881,-0.883,     &
                    -0.885,-0.887,-0.889,-0.891,-0.893,-0.895,-0.897,     &
                    -0.899,-0.901,-0.903,-0.905,-0.907,-0.909,-0.911,     &
                    -0.913,-0.915,-0.916,-0.918,-0.920,-0.922,-0.924,     &
                    -0.926,-0.928,-0.929,-0.931,-0.933,-0.935,-0.936,     &
                    -0.938,-0.940,-0.942,-0.944,-0.945,-0.947,-0.949,     &
                    -0.950,-0.952,-0.954,-0.956,-0.957,-0.959,-0.961,     &
                    -0.962,-0.964,-0.966,-0.967,-0.969,-0.971,-0.972,     &
                    -0.974,-0.975,-0.977,-0.979,-0.980,-0.982,-0.983,     &
                    -0.985,-0.987,-0.988,-0.990,-0.991,-0.993,-0.994,     &
                    -0.996,-0.998,-0.999,-1.001,-1.002,-1.004,-1.005,     &
                    -1.007,-1.008,-1.010,-1.011,-1.013,-1.014,-1.016,     &
                    -1.017,-1.019,-1.020,-1.022,-1.023,-1.024,-1.026,     &
                    -1.027,-1.029,-1.030,-1.032,-1.033,-1.035,-1.036,     &
                    -1.037,-1.039,-1.040,-1.042,-1.043,-1.045,-1.046,     &
                    -1.047,-1.049,-1.050,-1.052,-1.053,-1.054,-1.056,     &
                    -1.057,-1.058,-1.060,-1.061,-1.063,-1.064,-1.065,     &
                    -1.067,-1.068,-1.069,-1.071,-1.072,-1.073,-1.075,     &
                    -1.076,-1.077,-1.079,-1.080,-1.081,-1.083,-1.084,     &
                    -1.085,-1.087,-1.088,-1.089,-1.090,-1.092,-1.093,     &
                    -1.094,-1.096,-1.097,-1.098,-1.100,-1.101,-1.102,     &
                    -1.103,-1.105,-1.106,-1.107,-1.108,-1.110,-1.111,     &
                    -1.112,-1.113,-1.115,-1.116,-1.117,-1.118,-1.120,     &
                    -1.121,-1.122,-1.123,-1.125,-1.126,-1.127,-1.128,     &
                    -1.130,-1.131,-1.132,-1.133,-1.134,-1.136,-1.137,     &
                    -1.138,-1.139,-1.141,-1.142,-1.143,-1.144,-1.145,     &
                    -1.147,-1.148,-1.149,-1.150,-1.151,-1.153,-1.154,     &
                    -1.155,-1.156,-1.157,-1.158,-1.160,-1.161,-1.162,     &
                    -1.163,-1.164,-1.165,-1.167,-1.168,-1.169,-1.170,     &
                    -1.171,-1.172,-1.174,-1.175,-1.176,-1.177,-1.178,     &
                    -1.179,-1.181,-1.182,-1.183,-1.184,-1.185,-1.186,     &
                    -1.187,-1.189,-1.190,-1.191,-1.192,-1.193,-1.194,     &
                    -1.195,-1.196,-1.198,-1.199,-1.200,-1.201,-1.202,     &
                    -1.203,-1.204,-1.205,-1.207,-1.208,-1.209,-1.210,     &
                    -1.211,-1.212,-1.213,-1.214,-1.215,-1.216,-1.218,     &
                    -1.219,-1.220,-1.221,-1.222,-1.223,-1.224,-1.225,     &
                    -1.226,-1.227,-1.228,-1.230,-1.231,-1.232,-1.233,     &
                    -1.234,-1.235,-1.236,-1.237,-1.238,-1.239,-1.240,     &
                    -1.241,-1.242,-1.244,-1.245,-1.246,-1.247,-1.248,     &
                    -1.249,-1.250,-1.251,-1.252,-1.253,-1.254,-1.255,     &
                    -1.256,-1.257,-1.258,-1.259,-1.261,-1.262,-1.263,     &
                    -1.264,-1.265,-1.266,-1.267,-1.268,-1.269,-1.270,     &
                    -1.271,-1.282,-1.292,-1.303,-1.313,-1.323,-1.332,     &
                    -1.342,-1.352,-1.362,-1.371,-1.381,-1.390,-1.400,     &
                    -1.409,-1.418,-1.427,-1.437,-1.446,-1.455,-1.464,     &
                    -1.473,-1.482,-1.491,-1.499,-1.508,-1.517,-1.526,     &
                    -1.534,-1.543,-1.552,-1.560,-1.569,-1.577,-1.586,     &
                    -1.594,-1.602,-1.611,-1.619,-1.627,-1.636,-1.644,     &
                    -1.652,-1.660,-1.668,-1.677,-1.685,-1.693,-1.701,     &
                    -1.709,-1.717,-1.725,-1.733,-1.741,-1.749,-1.757,     &
                    -1.764,-1.772,-1.780,-1.788,-1.796,-1.804,-1.811,     &
                    -1.819,-1.827,-1.835,-1.842,-1.850,-1.858,-1.865,     &
                    -1.873,-1.880,-1.888,-1.896,-1.903,-1.911,-1.918,     &
                    -1.926,-1.933,-1.941,-1.948,-1.956,-1.963,-1.971,     &
                    -1.978,-1.986,-1.993,-2.000,-2.008,-2.015,-2.022,     &
                    -2.030,-2.037,-2.044,-2.052,-2.059,-2.066,-2.074,     &
                    -2.081,-2.088,-2.095,-2.103,-2.110,-2.117,-2.124,     &
                    -2.131,-2.139,-2.146,-2.153,-2.160,-2.167,-2.174,     &
                    -2.181,-2.189,-2.196,-2.203,-2.210,-2.217,-2.224,     &
                    -2.231,-2.238,-2.245,-2.252,-2.259,-2.266,-2.273,     &
                    -2.280,-2.287,-2.294,-2.301,-2.308,-2.315,-2.322,     &
                    -2.329,-2.336,-2.343,-2.350,-2.357,-2.364,-2.371,     &
                    -2.378,-2.385,-2.391,-2.398,-2.405,-2.412,-2.419,     &
                    -2.426,-2.433,-2.439,-2.446,-2.453,-2.460,-2.467,     &
                    -2.474,-2.480,-2.487,-2.494,-2.501,-2.508,-2.514,     &
                    -2.521/)
!
! *** (h ,hso4)
!
    bnc08m(1:561) = (/-0.046,-0.090,-0.108,-0.118,-0.124,-0.128,-0.130,   &
                    -0.132,-0.132,-0.131,-0.130,-0.128,-0.125,-0.122,     &
                    -0.119,-0.116,-0.112,-0.108,-0.103,-0.098,-0.093,     &
                    -0.088,-0.083,-0.077,-0.071,-0.065,-0.059,-0.053,     &
                    -0.046,-0.040,-0.033,-0.026,-0.019,-0.012,-0.004,     &
                    0.003,0.011,0.018,0.026,0.034,0.042,0.050,0.058,0.066,&
                    0.075,0.083,0.091,0.100,0.109,0.117,0.126,0.135,0.144,&
                    0.153,0.162,0.171,0.180,0.189,0.198,0.207,0.216,0.226,&
                    0.235,0.244,0.254,0.263,0.273,0.282,0.292,0.302,0.311,&
                    0.321,0.331,0.341,0.351,0.361,0.371,0.381,0.391,0.401,&
                    0.411,0.421,0.432,0.442,0.452,0.463,0.473,0.484,0.495,&
                    0.505,0.516,0.527,0.538,0.549,0.560,0.571,0.582,0.593,&
                    0.604,0.615,0.626,0.638,0.649,0.660,0.672,0.683,0.694,&
                    0.706,0.717,0.729,0.740,0.752,0.763,0.775,0.787,0.798,&
                    0.810,0.821,0.833,0.845,0.856,0.868,0.879,0.891,0.903,&
                    0.914,0.926,0.937,0.949,0.961,0.972,0.984,0.995,1.007,&
                    1.018,1.030,1.041,1.053,1.064,1.076,1.087,1.099,1.110,&
                    1.121,1.133,1.144,1.155,1.167,1.178,1.189,1.200,1.212,&
                    1.223,1.234,1.245,1.256,1.267,1.278,1.289,1.300,1.311,&
                    1.322,1.333,1.344,1.355,1.366,1.377,1.388,1.399,1.409,&
                    1.420,1.431,1.442,1.452,1.463,1.474,1.485,1.495,1.506,&
                    1.516,1.527,1.537,1.548,1.558,1.569,1.579,1.590,1.600,&
                    1.611,1.621,1.631,1.642,1.652,1.662,1.672,1.683,1.693,&
                    1.703,1.713,1.723,1.733,1.743,1.753,1.763,1.774,1.784,&
                    1.793,1.803,1.813,1.823,1.833,1.843,1.853,1.863,1.872,&
                    1.882,1.892,1.902,1.911,1.921,1.931,1.940,1.950,1.960,&
                    1.969,1.979,1.988,1.998,2.007,2.017,2.026,2.036,2.045,&
                    2.055,2.064,2.073,2.083,2.092,2.101,2.111,2.120,2.129,&
                    2.138,2.147,2.157,2.166,2.175,2.184,2.193,2.202,2.211,&
                    2.220,2.229,2.238,2.247,2.256,2.265,2.274,2.283,2.292,&
                    2.301,2.310,2.318,2.327,2.336,2.345,2.354,2.362,2.371,&
                    2.380,2.388,2.397,2.406,2.414,2.423,2.432,2.440,2.449,&
                    2.457,2.466,2.474,2.483,2.491,2.500,2.508,2.517,2.525,&
                    2.533,2.542,2.550,2.558,2.567,2.575,2.583,2.591,2.600,&
                    2.608,2.616,2.624,2.633,2.641,2.649,2.657,2.665,2.673,&
                    2.681,2.689,2.697,2.705,2.713,2.721,2.729,2.737,2.745,&
                    2.753,2.761,2.769,2.777,2.785,2.793,2.801,2.809,2.816,&
                    2.824,2.832,2.840,2.847,2.855,2.863,2.871,2.878,2.886,&
                    2.894,2.901,2.909,2.917,2.924,2.932,2.939,2.947,2.955,&
                    2.962,2.970,2.977,2.985,2.992,3.000,3.007,3.014,3.022,&
                    3.029,3.037,3.044,3.051,3.059,3.066,3.074,3.081,3.088,&
                    3.095,3.103,3.110,3.117,3.125,3.132,3.139,3.146,3.153,&
                    3.161,3.168,3.175,3.182,3.189,3.196,3.203,3.210,3.217,&
                    3.225,3.232,3.239,3.246,3.253,3.260,3.267,3.274,3.281,&
                    3.288,3.294,3.301,3.308,3.315,3.322,3.329,3.336,3.343,&
                    3.350,3.356,3.363,3.370,3.377,3.450,3.516,3.581,3.645,&
                    3.708,3.770,3.831,3.891,3.950,4.008,4.065,4.122,4.178,&
                    4.233,4.287,4.340,4.393,4.445,4.496,4.547,4.597,4.646,&
                    4.695,4.743,4.790,4.837,4.884,4.929,4.974,5.019,5.063,&
                    5.107,5.150,5.193,5.235,5.277,5.318,5.359,5.399,5.439,&
                    5.478,5.517,5.556,5.594,5.632,5.670,5.707,5.743,5.780,&
                    5.816,5.852,5.887,5.922,5.956,5.991,6.025,6.058,6.092,&
                    6.125,6.158,6.190,6.222,6.254,6.286,6.317,6.348,6.379,&
                    6.410,6.440,6.470,6.500,6.529,6.559,6.588,6.616,6.645,&
                    6.673,6.701,6.729,6.757,6.784,6.812,6.839,6.865,6.892,&
                    6.919,6.945,6.971,6.997,7.022,7.048,7.073,7.098,7.123,&
                    7.148,7.172,7.196,7.221,7.245,7.269,7.292,7.316,7.339,&
                    7.362,7.385,7.408,7.431,7.454,7.476,7.498,7.520,7.542,&
                    7.564,7.586,7.607,7.629,7.650,7.671,7.692,7.713,7.734,&
                    7.755,7.775,7.795,7.816,7.836,7.856,7.876,7.895,7.915,&
                    7.935,7.954,7.973,7.992,8.012,8.030,8.049,8.068,8.087,&
                    8.105,8.124,8.142,8.160,8.178,8.196,8.214,8.232,8.250,&
                    8.267,8.285,8.302,8.319,8.337,8.354,8.371,8.388,8.404,&
                    8.421,8.438,8.454,8.471/)
!
! *** nh4hso4
!
    bnc09m(1:561) = (/-0.048,-0.102,-0.128,-0.146,-0.159,-0.170,-0.179,   &
                    -0.187,-0.194,-0.200,-0.205,-0.210,-0.214,-0.218,     &
                    -0.221,-0.224,-0.227,-0.230,-0.232,-0.234,-0.235,     &
                    -0.237,-0.238,-0.239,-0.240,-0.241,-0.242,-0.242,     &
                    -0.243,-0.243,-0.243,-0.243,-0.243,-0.243,-0.242,     &
                    -0.242,-0.241,-0.241,-0.240,-0.239,-0.238,-0.237,     &
                    -0.236,-0.235,-0.234,-0.233,-0.231,-0.230,-0.228,     &
                    -0.227,-0.225,-0.224,-0.222,-0.220,-0.218,-0.216,     &
                    -0.214,-0.212,-0.210,-0.208,-0.206,-0.204,-0.202,     &
                    -0.199,-0.197,-0.195,-0.192,-0.190,-0.188,-0.185,     &
                    -0.183,-0.180,-0.178,-0.175,-0.172,-0.170,-0.167,     &
                    -0.164,-0.161,-0.159,-0.156,-0.153,-0.150,-0.147,     &
                    -0.144,-0.141,-0.138,-0.135,-0.132,-0.129,-0.126,     &
                    -0.123,-0.119,-0.116,-0.113,-0.110,-0.106,-0.103,     &
                    -0.100,-0.096,-0.093,-0.090,-0.086,-0.083,-0.080,     &
                    -0.076,-0.073,-0.069,-0.066,-0.062,-0.059,-0.055,     &
                    -0.052,-0.048,-0.045,-0.041,-0.037,-0.034,-0.030,     &
                    -0.027,-0.023,-0.020,-0.016,-0.012,-0.009,-0.005,     &
                    -0.002,0.002,0.005,0.009,0.013,0.016,0.020,0.023,     &
                    0.027,0.030,0.034,0.037,0.041,0.044,0.048,0.051,0.055,&
                    0.058,0.062,0.065,0.069,0.072,0.076,0.079,0.083,0.086,&
                    0.090,0.093,0.097,0.100,0.103,0.107,0.110,0.114,0.117,&
                    0.120,0.124,0.127,0.130,0.134,0.137,0.140,0.144,0.147,&
                    0.150,0.154,0.157,0.160,0.164,0.167,0.170,0.173,0.177,&
                    0.180,0.183,0.186,0.189,0.193,0.196,0.199,0.202,0.205,&
                    0.209,0.212,0.215,0.218,0.221,0.224,0.227,0.231,0.234,&
                    0.237,0.240,0.243,0.246,0.249,0.252,0.255,0.258,0.261,&
                    0.264,0.267,0.270,0.273,0.276,0.279,0.282,0.285,0.288,&
                    0.291,0.294,0.297,0.300,0.303,0.306,0.309,0.312,0.315,&
                    0.317,0.320,0.323,0.326,0.329,0.332,0.335,0.338,0.340,&
                    0.343,0.346,0.349,0.352,0.354,0.357,0.360,0.363,0.366,&
                    0.368,0.371,0.374,0.377,0.379,0.382,0.385,0.388,0.390,&
                    0.393,0.396,0.398,0.401,0.404,0.407,0.409,0.412,0.415,&
                    0.417,0.420,0.422,0.425,0.428,0.430,0.433,0.436,0.438,&
                    0.441,0.443,0.446,0.449,0.451,0.454,0.456,0.459,0.461,&
                    0.464,0.467,0.469,0.472,0.474,0.477,0.479,0.482,0.484,&
                    0.487,0.489,0.492,0.494,0.497,0.499,0.502,0.504,0.507,&
                    0.509,0.511,0.514,0.516,0.519,0.521,0.524,0.526,0.528,&
                    0.531,0.533,0.536,0.538,0.540,0.543,0.545,0.548,0.550,&
                    0.552,0.555,0.557,0.559,0.562,0.564,0.566,0.569,0.571,&
                    0.573,0.576,0.578,0.580,0.583,0.585,0.587,0.589,0.592,&
                    0.594,0.596,0.599,0.601,0.603,0.605,0.608,0.610,0.612,&
                    0.614,0.617,0.619,0.621,0.623,0.625,0.628,0.630,0.632,&
                    0.634,0.636,0.639,0.641,0.643,0.645,0.647,0.649,0.652,&
                    0.654,0.656,0.658,0.660,0.662,0.665,0.667,0.669,0.671,&
                    0.673,0.675,0.677,0.679,0.681,0.684,0.686,0.688,0.690,&
                    0.692,0.694,0.696,0.698,0.700,0.702,0.704,0.706,0.708,&
                    0.710,0.713,0.715,0.717,0.719,0.721,0.723,0.725,0.727,&
                    0.729,0.731,0.733,0.735,0.737,0.758,0.778,0.797,0.815,&
                    0.834,0.852,0.870,0.887,0.905,0.922,0.938,0.955,0.971,&
                    0.987,1.003,1.018,1.033,1.048,1.063,1.078,1.092,1.106,&
                    1.120,1.134,1.148,1.161,1.174,1.187,1.200,1.213,1.225,&
                    1.238,1.250,1.262,1.274,1.286,1.297,1.309,1.320,1.331,&
                    1.342,1.353,1.364,1.374,1.385,1.395,1.406,1.416,1.426,&
                    1.436,1.446,1.455,1.465,1.475,1.484,1.493,1.502,1.512,&
                    1.521,1.529,1.538,1.547,1.556,1.564,1.572,1.581,1.589,&
                    1.597,1.605,1.613,1.621,1.629,1.637,1.645,1.652,1.660,&
                    1.667,1.675,1.682,1.689,1.696,1.704,1.711,1.718,1.725,&
                    1.731,1.738,1.745,1.752,1.758,1.765,1.771,1.778,1.784,&
                    1.790,1.796,1.803,1.809,1.815,1.821,1.827,1.833,1.839,&
                    1.844,1.850,1.856,1.861,1.867,1.873,1.878,1.884,1.889,&
                    1.894,1.900,1.905,1.910,1.915,1.920,1.926,1.931,1.936,&
                    1.941,1.946,1.950,1.955,1.960,1.965,1.970,1.974,1.979,&
                    1.983,1.988,1.993,1.997,2.001,2.006,2.010,2.015,2.019,&
                    2.023,2.027,2.032,2.036,2.040,2.044,2.048,2.052,2.056,&
                    2.060,2.064,2.068,2.072,2.076,2.080,2.083,2.087,2.091,&
                    2.095,2.098,2.102,2.106/)
!
! *** (h ,no3)
!
    bnc10m(1:561) = (/-0.047,-0.098,-0.120,-0.135,-0.145,-0.153,-0.158,   &
                    -0.163,-0.167,-0.169,-0.172,-0.173,-0.175,-0.176,     &
                    -0.176,-0.176,-0.176,-0.176,-0.176,-0.175,-0.175,     &
                    -0.174,-0.173,-0.172,-0.170,-0.169,-0.168,-0.166,     &
                    -0.165,-0.163,-0.162,-0.160,-0.159,-0.157,-0.155,     &
                    -0.153,-0.151,-0.149,-0.148,-0.146,-0.144,-0.142,     &
                    -0.140,-0.138,-0.136,-0.134,-0.132,-0.130,-0.128,     &
                    -0.126,-0.124,-0.122,-0.120,-0.118,-0.115,-0.113,     &
                    -0.111,-0.109,-0.107,-0.105,-0.103,-0.101,-0.099,     &
                    -0.097,-0.095,-0.093,-0.090,-0.088,-0.086,-0.084,     &
                    -0.082,-0.080,-0.078,-0.075,-0.073,-0.071,-0.069,     &
                    -0.067,-0.064,-0.062,-0.060,-0.057,-0.055,-0.053,     &
                    -0.051,-0.048,-0.046,-0.043,-0.041,-0.039,-0.036,     &
                    -0.034,-0.031,-0.029,-0.026,-0.024,-0.021,-0.019,     &
                    -0.016,-0.014,-0.011,-0.008,-0.006,-0.003,0.000,0.002,&
                    0.005,0.007,0.010,0.013,0.016,0.018,0.021,0.024,0.026,&
                    0.029,0.032,0.035,0.037,0.040,0.043,0.046,0.048,0.051,&
                    0.054,0.057,0.059,0.062,0.065,0.068,0.070,0.073,0.076,&
                    0.079,0.081,0.084,0.087,0.090,0.092,0.095,0.098,0.101,&
                    0.103,0.106,0.109,0.112,0.114,0.117,0.120,0.123,0.125,&
                    0.128,0.131,0.133,0.136,0.139,0.142,0.144,0.147,0.150,&
                    0.152,0.155,0.158,0.160,0.163,0.166,0.168,0.171,0.174,&
                    0.176,0.179,0.182,0.184,0.187,0.190,0.192,0.195,0.198,&
                    0.200,0.203,0.205,0.208,0.211,0.213,0.216,0.218,0.221,&
                    0.224,0.226,0.229,0.231,0.234,0.236,0.239,0.242,0.244,&
                    0.247,0.249,0.252,0.254,0.257,0.259,0.262,0.264,0.267,&
                    0.269,0.272,0.275,0.277,0.280,0.282,0.285,0.287,0.289,&
                    0.292,0.294,0.297,0.299,0.302,0.304,0.307,0.309,0.312,&
                    0.314,0.317,0.319,0.321,0.324,0.326,0.329,0.331,0.334,&
                    0.336,0.338,0.341,0.343,0.346,0.348,0.350,0.353,0.355,&
                    0.357,0.360,0.362,0.364,0.367,0.369,0.372,0.374,0.376,&
                    0.379,0.381,0.383,0.386,0.388,0.390,0.392,0.395,0.397,&
                    0.399,0.402,0.404,0.406,0.409,0.411,0.413,0.415,0.418,&
                    0.420,0.422,0.424,0.427,0.429,0.431,0.433,0.436,0.438,&
                    0.440,0.442,0.445,0.447,0.449,0.451,0.453,0.456,0.458,&
                    0.460,0.462,0.464,0.467,0.469,0.471,0.473,0.475,0.477,&
                    0.480,0.482,0.484,0.486,0.488,0.490,0.492,0.495,0.497,&
                    0.499,0.501,0.503,0.505,0.507,0.509,0.511,0.514,0.516,&
                    0.518,0.520,0.522,0.524,0.526,0.528,0.530,0.532,0.534,&
                    0.536,0.539,0.541,0.543,0.545,0.547,0.549,0.551,0.553,&
                    0.555,0.557,0.559,0.561,0.563,0.565,0.567,0.569,0.571,&
                    0.573,0.575,0.577,0.579,0.581,0.583,0.585,0.587,0.589,&
                    0.591,0.593,0.595,0.597,0.599,0.601,0.603,0.604,0.606,&
                    0.608,0.610,0.612,0.614,0.616,0.618,0.620,0.622,0.624,&
                    0.626,0.628,0.629,0.631,0.633,0.635,0.637,0.639,0.641,&
                    0.643,0.645,0.646,0.648,0.650,0.652,0.654,0.656,0.658,&
                    0.659,0.661,0.663,0.665,0.667,0.669,0.671,0.672,0.674,&
                    0.676,0.678,0.680,0.681,0.683,0.685,0.704,0.722,0.739,&
                    0.756,0.773,0.790,0.806,0.822,0.838,0.853,0.869,0.884,&
                    0.899,0.913,0.928,0.942,0.956,0.970,0.983,0.996,1.010,&
                    1.023,1.036,1.048,1.061,1.073,1.085,1.097,1.109,1.121,&
                    1.132,1.144,1.155,1.166,1.177,1.188,1.198,1.209,1.219,&
                    1.230,1.240,1.250,1.260,1.270,1.279,1.289,1.298,1.308,&
                    1.317,1.326,1.335,1.344,1.353,1.362,1.370,1.379,1.387,&
                    1.396,1.404,1.412,1.420,1.428,1.436,1.444,1.451,1.459,&
                    1.467,1.474,1.481,1.489,1.496,1.503,1.510,1.517,1.524,&
                    1.531,1.538,1.545,1.551,1.558,1.565,1.571,1.578,1.584,&
                    1.590,1.596,1.603,1.609,1.615,1.621,1.627,1.632,1.638,&
                    1.644,1.650,1.655,1.661,1.667,1.672,1.678,1.683,1.688,&
                    1.694,1.699,1.704,1.709,1.714,1.719,1.724,1.729,1.734,&
                    1.739,1.744,1.749,1.753,1.758,1.763,1.767,1.772,1.776,&
                    1.781,1.785,1.790,1.794,1.799,1.803,1.807,1.811,1.815,&
                    1.820,1.824,1.828,1.832,1.836,1.840,1.844,1.848,1.852,&
                    1.855,1.859,1.863,1.867,1.870,1.874,1.878,1.881,1.885,&
                    1.888,1.892,1.895,1.899,1.902,1.906,1.909,1.913,1.916,&
                    1.919,1.922,1.926,1.929,1.932/)
!
! *** (h ,cl)
!
    bnc11m(1:561) = (/-0.046,-0.091,-0.110,-0.120,-0.127,-0.131,-0.133,   &
                    -0.134,-0.134,-0.134,-0.132,-0.131,-0.128,-0.126,     &
                    -0.123,-0.120,-0.116,-0.112,-0.108,-0.104,-0.099,     &
                    -0.095,-0.090,-0.085,-0.080,-0.075,-0.070,-0.064,     &
                    -0.059,-0.053,-0.047,-0.042,-0.036,-0.030,-0.024,     &
                    -0.018,-0.011,-0.005,0.001,0.007,0.014,0.020,0.026,   &
                    0.033,0.039,0.046,0.052,0.059,0.065,0.072,0.079,0.085,&
                    0.092,0.099,0.105,0.112,0.119,0.126,0.132,0.139,0.146,&
                    0.153,0.160,0.166,0.173,0.180,0.187,0.194,0.201,0.208,&
                    0.215,0.222,0.229,0.236,0.243,0.250,0.257,0.264,0.272,&
                    0.279,0.286,0.293,0.301,0.308,0.316,0.323,0.330,0.338,&
                    0.346,0.353,0.361,0.368,0.376,0.384,0.392,0.399,0.407,&
                    0.415,0.423,0.431,0.439,0.447,0.455,0.463,0.471,0.479,&
                    0.487,0.495,0.503,0.512,0.520,0.528,0.536,0.544,0.553,&
                    0.561,0.569,0.577,0.586,0.594,0.602,0.610,0.619,0.627,&
                    0.635,0.643,0.652,0.660,0.668,0.676,0.685,0.693,0.701,&
                    0.709,0.718,0.726,0.734,0.742,0.750,0.759,0.767,0.775,&
                    0.783,0.791,0.799,0.807,0.815,0.823,0.832,0.840,0.848,&
                    0.856,0.864,0.872,0.880,0.888,0.896,0.903,0.911,0.919,&
                    0.927,0.935,0.943,0.951,0.959,0.966,0.974,0.982,0.990,&
                    0.998,1.005,1.013,1.021,1.029,1.036,1.044,1.052,1.059,&
                    1.067,1.075,1.082,1.090,1.097,1.105,1.112,1.120,1.128,&
                    1.135,1.143,1.150,1.157,1.165,1.172,1.180,1.187,1.195,&
                    1.202,1.209,1.217,1.224,1.231,1.239,1.246,1.253,1.260,&
                    1.268,1.275,1.282,1.289,1.296,1.303,1.311,1.318,1.325,&
                    1.332,1.339,1.346,1.353,1.360,1.367,1.374,1.381,1.388,&
                    1.395,1.402,1.409,1.416,1.423,1.430,1.437,1.444,1.451,&
                    1.457,1.464,1.471,1.478,1.485,1.491,1.498,1.505,1.512,&
                    1.518,1.525,1.532,1.538,1.545,1.552,1.558,1.565,1.572,&
                    1.578,1.585,1.591,1.598,1.604,1.611,1.617,1.624,1.630,&
                    1.637,1.643,1.650,1.656,1.663,1.669,1.675,1.682,1.688,&
                    1.695,1.701,1.707,1.714,1.720,1.726,1.732,1.739,1.745,&
                    1.751,1.757,1.764,1.770,1.776,1.782,1.788,1.795,1.801,&
                    1.807,1.813,1.819,1.825,1.831,1.837,1.843,1.849,1.855,&
                    1.861,1.867,1.873,1.879,1.885,1.891,1.897,1.903,1.909,&
                    1.915,1.921,1.927,1.933,1.939,1.945,1.950,1.956,1.962,&
                    1.968,1.974,1.979,1.985,1.991,1.997,2.003,2.008,2.014,&
                    2.020,2.025,2.031,2.037,2.042,2.048,2.054,2.059,2.065,&
                    2.071,2.076,2.082,2.087,2.093,2.099,2.104,2.110,2.115,&
                    2.121,2.126,2.132,2.137,2.143,2.148,2.154,2.159,2.165,&
                    2.170,2.175,2.181,2.186,2.192,2.197,2.202,2.208,2.213,&
                    2.219,2.224,2.229,2.235,2.240,2.245,2.250,2.256,2.261,&
                    2.266,2.271,2.277,2.282,2.287,2.292,2.298,2.303,2.308,&
                    2.313,2.318,2.323,2.329,2.334,2.339,2.344,2.349,2.354,&
                    2.359,2.364,2.369,2.375,2.380,2.385,2.390,2.395,2.400,&
                    2.405,2.410,2.415,2.420,2.425,2.430,2.483,2.531,2.579,&
                    2.626,2.672,2.717,2.762,2.806,2.849,2.891,2.933,2.975,&
                    3.015,3.056,3.095,3.134,3.173,3.211,3.248,3.285,3.322,&
                    3.358,3.393,3.428,3.463,3.497,3.531,3.564,3.597,3.629,&
                    3.662,3.693,3.725,3.756,3.786,3.817,3.847,3.876,3.906,&
                    3.935,3.963,3.992,4.020,4.048,4.075,4.102,4.129,4.156,&
                    4.182,4.208,4.234,4.260,4.285,4.310,4.335,4.359,4.384,&
                    4.408,4.432,4.456,4.479,4.502,4.525,4.548,4.571,4.593,&
                    4.615,4.637,4.659,4.681,4.702,4.724,4.745,4.766,4.786,&
                    4.807,4.827,4.847,4.868,4.887,4.907,4.927,4.946,4.965,&
                    4.984,5.003,5.022,5.041,5.059,5.078,5.096,5.114,5.132,&
                    5.150,5.168,5.185,5.203,5.220,5.237,5.254,5.271,5.288,&
                    5.304,5.321,5.337,5.354,5.370,5.386,5.402,5.418,5.434,&
                    5.449,5.465,5.480,5.496,5.511,5.526,5.541,5.556,5.571,&
                    5.586,5.600,5.615,5.629,5.644,5.658,5.672,5.686,5.700,&
                    5.714,5.728,5.742,5.755,5.769,5.782,5.796,5.809,5.822,&
                    5.835,5.848,5.861,5.874,5.887,5.900,5.913,5.925,5.938,&
                    5.950,5.963,5.975,5.987,5.999,6.011,6.023,6.035,6.047,&
                    6.059,6.071,6.083,6.094,6.106/)
!
! *** nahso4
!
    bnc12m(1:561) = (/-0.047,-0.097,-0.120,-0.135,-0.146,-0.154,-0.160,   &
                    -0.165,-0.170,-0.173,-0.176,-0.178,-0.179,-0.181,     &
                    -0.181,-0.182,-0.182,-0.182,-0.182,-0.182,-0.181,     &
                    -0.180,-0.179,-0.178,-0.176,-0.175,-0.173,-0.172,     &
                    -0.170,-0.168,-0.165,-0.163,-0.161,-0.158,-0.156,     &
                    -0.153,-0.150,-0.148,-0.145,-0.142,-0.139,-0.136,     &
                    -0.132,-0.129,-0.126,-0.123,-0.119,-0.116,-0.112,     &
                    -0.109,-0.105,-0.101,-0.098,-0.094,-0.090,-0.086,     &
                    -0.082,-0.078,-0.074,-0.070,-0.066,-0.062,-0.058,     &
                    -0.054,-0.050,-0.046,-0.041,-0.037,-0.033,-0.028,     &
                    -0.024,-0.020,-0.015,-0.011,-0.006,-0.002,0.003,0.007,&
                    0.012,0.017,0.021,0.026,0.031,0.036,0.041,0.045,0.050,&
                    0.055,0.060,0.065,0.070,0.075,0.080,0.086,0.091,0.096,&
                    0.101,0.106,0.112,0.117,0.122,0.128,0.133,0.138,0.144,&
                    0.149,0.154,0.160,0.165,0.171,0.176,0.182,0.187,0.193,&
                    0.198,0.204,0.209,0.215,0.221,0.226,0.232,0.237,0.243,&
                    0.248,0.254,0.259,0.265,0.271,0.276,0.282,0.287,0.293,&
                    0.298,0.304,0.309,0.315,0.320,0.326,0.331,0.337,0.342,&
                    0.348,0.353,0.358,0.364,0.369,0.375,0.380,0.386,0.391,&
                    0.396,0.402,0.407,0.412,0.418,0.423,0.428,0.434,0.439,&
                    0.444,0.449,0.455,0.460,0.465,0.470,0.476,0.481,0.486,&
                    0.491,0.496,0.502,0.507,0.512,0.517,0.522,0.527,0.532,&
                    0.537,0.542,0.548,0.553,0.558,0.563,0.568,0.573,0.578,&
                    0.583,0.588,0.593,0.598,0.603,0.607,0.612,0.617,0.622,&
                    0.627,0.632,0.637,0.642,0.647,0.651,0.656,0.661,0.666,&
                    0.671,0.675,0.680,0.685,0.690,0.694,0.699,0.704,0.709,&
                    0.713,0.718,0.723,0.727,0.732,0.737,0.741,0.746,0.751,&
                    0.755,0.760,0.764,0.769,0.773,0.778,0.783,0.787,0.792,&
                    0.796,0.801,0.805,0.810,0.814,0.819,0.823,0.828,0.832,&
                    0.836,0.841,0.845,0.850,0.854,0.858,0.863,0.867,0.872,&
                    0.876,0.880,0.885,0.889,0.893,0.898,0.902,0.906,0.910,&
                    0.915,0.919,0.923,0.928,0.932,0.936,0.940,0.944,0.949,&
                    0.953,0.957,0.961,0.965,0.969,0.974,0.978,0.982,0.986,&
                    0.990,0.994,0.998,1.002,1.007,1.011,1.015,1.019,1.023,&
                    1.027,1.031,1.035,1.039,1.043,1.047,1.051,1.055,1.059,&
                    1.063,1.067,1.071,1.075,1.079,1.083,1.087,1.091,1.094,&
                    1.098,1.102,1.106,1.110,1.114,1.118,1.122,1.125,1.129,&
                    1.133,1.137,1.141,1.145,1.148,1.152,1.156,1.160,1.164,&
                    1.167,1.171,1.175,1.179,1.182,1.186,1.190,1.194,1.197,&
                    1.201,1.205,1.208,1.212,1.216,1.219,1.223,1.227,1.230,&
                    1.234,1.238,1.241,1.245,1.249,1.252,1.256,1.259,1.263,&
                    1.267,1.270,1.274,1.277,1.281,1.284,1.288,1.292,1.295,&
                    1.299,1.302,1.306,1.309,1.313,1.316,1.320,1.323,1.327,&
                    1.330,1.334,1.337,1.341,1.344,1.347,1.351,1.354,1.358,&
                    1.361,1.365,1.368,1.371,1.375,1.378,1.382,1.385,1.388,&
                    1.392,1.395,1.398,1.402,1.405,1.408,1.412,1.415,1.418,&
                    1.422,1.425,1.428,1.432,1.435,1.438,1.441,1.477,1.509,&
                    1.540,1.571,1.601,1.631,1.661,1.690,1.718,1.747,1.774,&
                    1.802,1.829,1.855,1.881,1.907,1.933,1.958,1.983,2.007,&
                    2.031,2.055,2.079,2.102,2.125,2.147,2.170,2.192,2.213,&
                    2.235,2.256,2.277,2.298,2.318,2.339,2.359,2.378,2.398,&
                    2.417,2.436,2.455,2.474,2.493,2.511,2.529,2.547,2.565,&
                    2.582,2.599,2.617,2.634,2.650,2.667,2.684,2.700,2.716,&
                    2.732,2.748,2.763,2.779,2.794,2.810,2.825,2.840,2.854,&
                    2.869,2.884,2.898,2.912,2.926,2.940,2.954,2.968,2.982,&
                    2.995,3.008,3.022,3.035,3.048,3.061,3.074,3.086,3.099,&
                    3.111,3.124,3.136,3.148,3.160,3.172,3.184,3.196,3.208,&
                    3.219,3.231,3.242,3.254,3.265,3.276,3.287,3.298,3.309,&
                    3.320,3.330,3.341,3.352,3.362,3.373,3.383,3.393,3.403,&
                    3.413,3.423,3.433,3.443,3.453,3.463,3.472,3.482,3.491,&
                    3.501,3.510,3.520,3.529,3.538,3.547,3.556,3.565,3.574,&
                    3.583,3.592,3.601,3.609,3.618,3.626,3.635,3.643,3.652,&
                    3.660,3.668,3.677,3.685,3.693,3.701,3.709,3.717,3.725,&
                    3.733,3.741,3.748,3.756,3.764,3.771,3.779,3.786,3.794,&
                    3.801,3.809,3.816,3.823,3.830,3.838/)
!
! *** (nh4)3h(so4)2
!
    bnc13m(1:561) = (/-0.078,-0.169,-0.215,-0.247,-0.272,-0.293,-0.311,   &
                    -0.326,-0.340,-0.353,-0.364,-0.375,-0.384,-0.393,     &
                    -0.402,-0.410,-0.417,-0.424,-0.431,-0.437,-0.443,     &
                    -0.448,-0.454,-0.459,-0.464,-0.469,-0.473,-0.477,     &
                    -0.482,-0.486,-0.489,-0.493,-0.497,-0.500,-0.503,     &
                    -0.506,-0.509,-0.512,-0.515,-0.518,-0.520,-0.523,     &
                    -0.525,-0.528,-0.530,-0.532,-0.534,-0.536,-0.538,     &
                    -0.540,-0.542,-0.544,-0.546,-0.547,-0.549,-0.550,     &
                    -0.552,-0.553,-0.555,-0.556,-0.557,-0.559,-0.560,     &
                    -0.561,-0.562,-0.563,-0.564,-0.565,-0.566,-0.567,     &
                    -0.568,-0.569,-0.570,-0.571,-0.572,-0.572,-0.573,     &
                    -0.574,-0.575,-0.575,-0.576,-0.576,-0.577,-0.578,     &
                    -0.578,-0.579,-0.579,-0.580,-0.580,-0.580,-0.581,     &
                    -0.581,-0.582,-0.582,-0.582,-0.582,-0.583,-0.583,     &
                    -0.583,-0.583,-0.584,-0.584,-0.584,-0.584,-0.584,     &
                    -0.584,-0.584,-0.585,-0.585,-0.585,-0.585,-0.585,     &
                    -0.585,-0.585,-0.585,-0.585,-0.585,-0.585,-0.585,     &
                    -0.585,-0.585,-0.585,-0.585,-0.585,-0.585,-0.585,     &
                    -0.585,-0.584,-0.584,-0.584,-0.584,-0.584,-0.584,     &
                    -0.584,-0.584,-0.584,-0.584,-0.583,-0.583,-0.583,     &
                    -0.583,-0.583,-0.583,-0.583,-0.582,-0.582,-0.582,     &
                    -0.582,-0.582,-0.582,-0.582,-0.581,-0.581,-0.581,     &
                    -0.581,-0.581,-0.581,-0.580,-0.580,-0.580,-0.580,     &
                    -0.580,-0.579,-0.579,-0.579,-0.579,-0.579,-0.579,     &
                    -0.578,-0.578,-0.578,-0.578,-0.578,-0.577,-0.577,     &
                    -0.577,-0.577,-0.577,-0.577,-0.576,-0.576,-0.576,     &
                    -0.576,-0.576,-0.575,-0.575,-0.575,-0.575,-0.575,     &
                    -0.574,-0.574,-0.574,-0.574,-0.574,-0.574,-0.573,     &
                    -0.573,-0.573,-0.573,-0.573,-0.572,-0.572,-0.572,     &
                    -0.572,-0.572,-0.571,-0.571,-0.571,-0.571,-0.571,     &
                    -0.571,-0.570,-0.570,-0.570,-0.570,-0.570,-0.569,     &
                    -0.569,-0.569,-0.569,-0.569,-0.569,-0.568,-0.568,     &
                    -0.568,-0.568,-0.568,-0.567,-0.567,-0.567,-0.567,     &
                    -0.567,-0.567,-0.566,-0.566,-0.566,-0.566,-0.566,     &
                    -0.566,-0.565,-0.565,-0.565,-0.565,-0.565,-0.565,     &
                    -0.564,-0.564,-0.564,-0.564,-0.564,-0.564,-0.563,     &
                    -0.563,-0.563,-0.563,-0.563,-0.563,-0.562,-0.562,     &
                    -0.562,-0.562,-0.562,-0.562,-0.561,-0.561,-0.561,     &
                    -0.561,-0.561,-0.561,-0.561,-0.560,-0.560,-0.560,     &
                    -0.560,-0.560,-0.560,-0.560,-0.559,-0.559,-0.559,     &
                    -0.559,-0.559,-0.559,-0.559,-0.558,-0.558,-0.558,     &
                    -0.558,-0.558,-0.558,-0.558,-0.557,-0.557,-0.557,     &
                    -0.557,-0.557,-0.557,-0.557,-0.556,-0.556,-0.556,     &
                    -0.556,-0.556,-0.556,-0.556,-0.556,-0.555,-0.555,     &
                    -0.555,-0.555,-0.555,-0.555,-0.555,-0.555,-0.554,     &
                    -0.554,-0.554,-0.554,-0.554,-0.554,-0.554,-0.554,     &
                    -0.554,-0.553,-0.553,-0.553,-0.553,-0.553,-0.553,     &
                    -0.553,-0.553,-0.553,-0.552,-0.552,-0.552,-0.552,     &
                    -0.552,-0.552,-0.552,-0.552,-0.552,-0.552,-0.551,     &
                    -0.551,-0.551,-0.551,-0.551,-0.551,-0.551,-0.551,     &
                    -0.551,-0.551,-0.550,-0.550,-0.550,-0.550,-0.550,     &
                    -0.550,-0.550,-0.550,-0.550,-0.550,-0.550,-0.549,     &
                    -0.549,-0.549,-0.549,-0.549,-0.549,-0.549,-0.549,     &
                    -0.549,-0.549,-0.549,-0.549,-0.549,-0.548,-0.548,     &
                    -0.548,-0.548,-0.548,-0.548,-0.548,-0.548,-0.548,     &
                    -0.548,-0.548,-0.548,-0.548,-0.547,-0.547,-0.547,     &
                    -0.547,-0.547,-0.547,-0.547,-0.547,-0.547,-0.547,     &
                    -0.547,-0.546,-0.546,-0.545,-0.545,-0.544,-0.544,     &
                    -0.544,-0.544,-0.544,-0.544,-0.544,-0.544,-0.544,     &
                    -0.544,-0.544,-0.544,-0.545,-0.545,-0.545,-0.546,     &
                    -0.546,-0.547,-0.547,-0.548,-0.548,-0.549,-0.550,     &
                    -0.550,-0.551,-0.552,-0.553,-0.554,-0.555,-0.556,     &
                    -0.557,-0.558,-0.559,-0.560,-0.561,-0.562,-0.563,     &
                    -0.564,-0.566,-0.567,-0.568,-0.569,-0.571,-0.572,     &
                    -0.573,-0.575,-0.576,-0.578,-0.579,-0.581,-0.582,     &
                    -0.584,-0.586,-0.587,-0.589,-0.590,-0.592,-0.594,     &
                    -0.595,-0.597,-0.599,-0.601,-0.603,-0.604,-0.606,     &
                    -0.608,-0.610,-0.612,-0.614,-0.616,-0.618,-0.620,     &
                    -0.622,-0.624,-0.626,-0.628,-0.630,-0.632,-0.634,     &
                    -0.636,-0.638,-0.640,-0.642,-0.644,-0.647,-0.649,     &
                    -0.651,-0.653,-0.655,-0.658,-0.660,-0.662,-0.664,     &
                    -0.667,-0.669,-0.671,-0.674,-0.676,-0.678,-0.681,     &
                    -0.683,-0.686,-0.688,-0.690,-0.693,-0.695,-0.698,     &
                    -0.700,-0.703,-0.705,-0.708,-0.710,-0.713,-0.715,     &
                    -0.718,-0.720,-0.723,-0.725,-0.728,-0.731,-0.733,     &
                    -0.736,-0.738,-0.741,-0.744,-0.746,-0.749,-0.752,     &
                    -0.754,-0.757,-0.760,-0.762,-0.765,-0.768,-0.770,     &
                    -0.773,-0.776,-0.779,-0.781,-0.784,-0.787,-0.790,     &
                    -0.793,-0.795,-0.798,-0.801,-0.804,-0.807,-0.809,     &
                    -0.812,-0.815,-0.818,-0.821,-0.824,-0.827,-0.829,     &
                    -0.832/)
!
! *** caso4
!
    bnc14m(1:561) = (/0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,    &
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000/)
!
! *** cano32
!
    bnc15m(1:561) = (/-0.096,-0.206,-0.258,-0.293,-0.321,-0.342,-0.360,   &
                    -0.376,-0.389,-0.401,-0.411,-0.420,-0.429,-0.436,     &
                    -0.443,-0.449,-0.455,-0.460,-0.465,-0.469,-0.473,     &
                    -0.477,-0.481,-0.484,-0.487,-0.490,-0.493,-0.495,     &
                    -0.497,-0.500,-0.502,-0.504,-0.505,-0.507,-0.509,     &
                    -0.510,-0.512,-0.513,-0.514,-0.515,-0.517,-0.518,     &
                    -0.519,-0.520,-0.520,-0.521,-0.522,-0.523,-0.524,     &
                    -0.524,-0.525,-0.526,-0.526,-0.527,-0.527,-0.528,     &
                    -0.528,-0.529,-0.529,-0.530,-0.530,-0.530,-0.531,     &
                    -0.531,-0.531,-0.531,-0.532,-0.532,-0.532,-0.532,     &
                    -0.532,-0.532,-0.533,-0.533,-0.533,-0.533,-0.533,     &
                    -0.533,-0.533,-0.533,-0.533,-0.532,-0.532,-0.532,     &
                    -0.532,-0.532,-0.532,-0.531,-0.531,-0.531,-0.530,     &
                    -0.530,-0.530,-0.529,-0.529,-0.529,-0.528,-0.528,     &
                    -0.527,-0.527,-0.526,-0.526,-0.525,-0.525,-0.524,     &
                    -0.524,-0.523,-0.523,-0.522,-0.521,-0.521,-0.520,     &
                    -0.519,-0.519,-0.518,-0.517,-0.517,-0.516,-0.515,     &
                    -0.515,-0.514,-0.513,-0.512,-0.512,-0.511,-0.510,     &
                    -0.509,-0.509,-0.508,-0.507,-0.506,-0.506,-0.505,     &
                    -0.504,-0.503,-0.503,-0.502,-0.501,-0.500,-0.499,     &
                    -0.499,-0.498,-0.497,-0.496,-0.495,-0.495,-0.494,     &
                    -0.493,-0.492,-0.491,-0.491,-0.490,-0.489,-0.488,     &
                    -0.487,-0.486,-0.486,-0.485,-0.484,-0.483,-0.482,     &
                    -0.482,-0.481,-0.480,-0.479,-0.478,-0.477,-0.477,     &
                    -0.476,-0.475,-0.474,-0.473,-0.472,-0.472,-0.471,     &
                    -0.470,-0.469,-0.468,-0.468,-0.467,-0.466,-0.465,     &
                    -0.464,-0.463,-0.463,-0.462,-0.461,-0.460,-0.459,     &
                    -0.459,-0.458,-0.457,-0.456,-0.455,-0.455,-0.454,     &
                    -0.453,-0.452,-0.451,-0.450,-0.450,-0.449,-0.448,     &
                    -0.447,-0.446,-0.446,-0.445,-0.444,-0.443,-0.442,     &
                    -0.442,-0.441,-0.440,-0.439,-0.438,-0.438,-0.437,     &
                    -0.436,-0.435,-0.435,-0.434,-0.433,-0.432,-0.431,     &
                    -0.431,-0.430,-0.429,-0.428,-0.427,-0.427,-0.426,     &
                    -0.425,-0.424,-0.424,-0.423,-0.422,-0.421,-0.421,     &
                    -0.420,-0.419,-0.418,-0.417,-0.417,-0.416,-0.415,     &
                    -0.414,-0.414,-0.413,-0.412,-0.411,-0.411,-0.410,     &
                    -0.409,-0.408,-0.408,-0.407,-0.406,-0.405,-0.405,     &
                    -0.404,-0.403,-0.402,-0.402,-0.401,-0.400,-0.400,     &
                    -0.399,-0.398,-0.397,-0.397,-0.396,-0.395,-0.394,     &
                    -0.394,-0.393,-0.392,-0.392,-0.391,-0.390,-0.389,     &
                    -0.389,-0.388,-0.387,-0.387,-0.386,-0.385,-0.385,     &
                    -0.384,-0.383,-0.382,-0.382,-0.381,-0.380,-0.380,     &
                    -0.379,-0.378,-0.378,-0.377,-0.376,-0.376,-0.375,     &
                    -0.374,-0.374,-0.373,-0.372,-0.371,-0.371,-0.370,     &
                    -0.369,-0.369,-0.368,-0.367,-0.367,-0.366,-0.365,     &
                    -0.365,-0.364,-0.364,-0.363,-0.362,-0.362,-0.361,     &
                    -0.360,-0.360,-0.359,-0.358,-0.358,-0.357,-0.356,     &
                    -0.356,-0.355,-0.354,-0.354,-0.353,-0.353,-0.352,     &
                    -0.351,-0.351,-0.350,-0.349,-0.349,-0.348,-0.348,     &
                    -0.347,-0.346,-0.346,-0.345,-0.344,-0.344,-0.343,     &
                    -0.343,-0.342,-0.341,-0.341,-0.340,-0.340,-0.339,     &
                    -0.338,-0.338,-0.337,-0.337,-0.336,-0.335,-0.335,     &
                    -0.334,-0.334,-0.333,-0.332,-0.332,-0.331,-0.331,     &
                    -0.330,-0.330,-0.329,-0.328,-0.328,-0.327,-0.327,     &
                    -0.326,-0.326,-0.325,-0.324,-0.324,-0.323,-0.323,     &
                    -0.322,-0.322,-0.321,-0.320,-0.320,-0.319,-0.319,     &
                    -0.318,-0.318,-0.317,-0.317,-0.316,-0.316,-0.315,     &
                    -0.314,-0.309,-0.303,-0.298,-0.293,-0.288,-0.284,     &
                    -0.279,-0.275,-0.270,-0.266,-0.262,-0.258,-0.254,     &
                    -0.250,-0.246,-0.242,-0.239,-0.235,-0.232,-0.229,     &
                    -0.226,-0.223,-0.219,-0.217,-0.214,-0.211,-0.208,     &
                    -0.206,-0.203,-0.201,-0.198,-0.196,-0.194,-0.192,     &
                    -0.190,-0.188,-0.186,-0.184,-0.182,-0.180,-0.179,     &
                    -0.177,-0.175,-0.174,-0.173,-0.171,-0.170,-0.169,     &
                    -0.167,-0.166,-0.165,-0.164,-0.163,-0.162,-0.161,     &
                    -0.161,-0.160,-0.159,-0.158,-0.158,-0.157,-0.157,     &
                    -0.156,-0.156,-0.155,-0.155,-0.155,-0.154,-0.154,     &
                    -0.154,-0.154,-0.154,-0.153,-0.153,-0.153,-0.153,     &
                    -0.153,-0.154,-0.154,-0.154,-0.154,-0.154,-0.155,     &
                    -0.155,-0.155,-0.156,-0.156,-0.157,-0.157,-0.158,     &
                    -0.158,-0.159,-0.159,-0.160,-0.161,-0.161,-0.162,     &
                    -0.163,-0.164,-0.164,-0.165,-0.166,-0.167,-0.168,     &
                    -0.169,-0.170,-0.171,-0.172,-0.173,-0.174,-0.175,     &
                    -0.176,-0.177,-0.179,-0.180,-0.181,-0.182,-0.183,     &
                    -0.185,-0.186,-0.187,-0.189,-0.190,-0.192,-0.193,     &
                    -0.194,-0.196,-0.197,-0.199,-0.200,-0.202,-0.203,     &
                    -0.205,-0.207,-0.208,-0.210,-0.212,-0.213,-0.215,     &
                    -0.217,-0.218,-0.220,-0.222,-0.224,-0.225,-0.227,     &
                    -0.229,-0.231,-0.233,-0.235,-0.237,-0.239,-0.240,     &
                    -0.242,-0.244,-0.246,-0.248,-0.250,-0.252,-0.254,     &
                    -0.256/)
!
! *** cacl2
!
    bnc16m(1:561) = (/-0.095,-0.197,-0.242,-0.272,-0.293,-0.309,-0.321,   &
                    -0.331,-0.339,-0.345,-0.351,-0.355,-0.358,-0.360,     &
                    -0.362,-0.363,-0.363,-0.364,-0.364,-0.363,-0.362,     &
                    -0.361,-0.360,-0.358,-0.357,-0.355,-0.353,-0.350,     &
                    -0.348,-0.345,-0.343,-0.340,-0.337,-0.334,-0.331,     &
                    -0.328,-0.325,-0.322,-0.319,-0.316,-0.312,-0.309,     &
                    -0.306,-0.302,-0.299,-0.295,-0.292,-0.289,-0.285,     &
                    -0.282,-0.278,-0.274,-0.271,-0.267,-0.264,-0.260,     &
                    -0.257,-0.253,-0.249,-0.246,-0.242,-0.239,-0.235,     &
                    -0.231,-0.228,-0.224,-0.220,-0.217,-0.213,-0.209,     &
                    -0.205,-0.202,-0.198,-0.194,-0.190,-0.186,-0.182,     &
                    -0.178,-0.175,-0.171,-0.167,-0.163,-0.158,-0.154,     &
                    -0.150,-0.146,-0.142,-0.138,-0.134,-0.129,-0.125,     &
                    -0.121,-0.116,-0.112,-0.107,-0.103,-0.099,-0.094,     &
                    -0.090,-0.085,-0.080,-0.076,-0.071,-0.066,-0.062,     &
                    -0.057,-0.052,-0.048,-0.043,-0.038,-0.033,-0.028,     &
                    -0.024,-0.019,-0.014,-0.009,-0.004,0.001,0.006,0.010, &
                    0.015,0.020,0.025,0.030,0.035,0.040,0.045,0.050,0.055,&
                    0.060,0.064,0.069,0.074,0.079,0.084,0.089,0.094,0.099,&
                    0.104,0.109,0.114,0.119,0.123,0.128,0.133,0.138,0.143,&
                    0.148,0.153,0.158,0.162,0.167,0.172,0.177,0.182,0.187,&
                    0.191,0.196,0.201,0.206,0.211,0.215,0.220,0.225,0.230,&
                    0.235,0.239,0.244,0.249,0.254,0.258,0.263,0.268,0.272,&
                    0.277,0.282,0.287,0.291,0.296,0.301,0.305,0.310,0.315,&
                    0.319,0.324,0.329,0.333,0.338,0.342,0.347,0.352,0.356,&
                    0.361,0.365,0.370,0.375,0.379,0.384,0.388,0.393,0.397,&
                    0.402,0.406,0.411,0.415,0.420,0.424,0.429,0.433,0.438,&
                    0.442,0.447,0.451,0.456,0.460,0.464,0.469,0.473,0.478,&
                    0.482,0.487,0.491,0.495,0.500,0.504,0.508,0.513,0.517,&
                    0.521,0.526,0.530,0.534,0.539,0.543,0.547,0.552,0.556,&
                    0.560,0.564,0.569,0.573,0.577,0.581,0.586,0.590,0.594,&
                    0.598,0.602,0.607,0.611,0.615,0.619,0.623,0.627,0.632,&
                    0.636,0.640,0.644,0.648,0.652,0.656,0.660,0.664,0.669,&
                    0.673,0.677,0.681,0.685,0.689,0.693,0.697,0.701,0.705,&
                    0.709,0.713,0.717,0.721,0.725,0.729,0.733,0.737,0.741,&
                    0.745,0.749,0.753,0.757,0.761,0.765,0.768,0.772,0.776,&
                    0.780,0.784,0.788,0.792,0.796,0.799,0.803,0.807,0.811,&
                    0.815,0.819,0.823,0.826,0.830,0.834,0.838,0.842,0.845,&
                    0.849,0.853,0.857,0.860,0.864,0.868,0.872,0.875,0.879,&
                    0.883,0.886,0.890,0.894,0.898,0.901,0.905,0.909,0.912,&
                    0.916,0.920,0.923,0.927,0.930,0.934,0.938,0.941,0.945,&
                    0.949,0.952,0.956,0.959,0.963,0.966,0.970,0.974,0.977,&
                    0.981,0.984,0.988,0.991,0.995,0.998,1.002,1.005,1.009,&
                    1.012,1.016,1.019,1.023,1.026,1.030,1.033,1.037,1.040,&
                    1.044,1.047,1.050,1.054,1.057,1.061,1.064,1.067,1.071,&
                    1.074,1.078,1.081,1.084,1.088,1.091,1.094,1.098,1.101,&
                    1.105,1.108,1.111,1.115,1.118,1.121,1.124,1.128,1.131,&
                    1.134,1.138,1.141,1.144,1.147,1.151,1.154,1.157,1.160,&
                    1.164,1.198,1.230,1.261,1.292,1.322,1.351,1.380,1.409,&
                    1.437,1.465,1.492,1.519,1.546,1.572,1.598,1.623,1.648,&
                    1.673,1.697,1.721,1.745,1.768,1.791,1.814,1.836,1.858,&
                    1.880,1.901,1.922,1.943,1.964,1.984,2.004,2.024,2.044,&
                    2.063,2.082,2.101,2.119,2.138,2.156,2.174,2.191,2.209,&
                    2.226,2.243,2.260,2.276,2.293,2.309,2.325,2.341,2.357,&
                    2.372,2.387,2.403,2.418,2.432,2.447,2.461,2.476,2.490,&
                    2.504,2.518,2.531,2.545,2.558,2.571,2.585,2.597,2.610,&
                    2.623,2.635,2.648,2.660,2.672,2.684,2.696,2.708,2.720,&
                    2.731,2.742,2.754,2.765,2.776,2.787,2.798,2.808,2.819,&
                    2.830,2.840,2.850,2.860,2.871,2.881,2.890,2.900,2.910,&
                    2.920,2.929,2.938,2.948,2.957,2.966,2.975,2.984,2.993,&
                    3.002,3.011,3.019,3.028,3.036,3.045,3.053,3.061,3.069,&
                    3.077,3.085,3.093,3.101,3.109,3.117,3.124,3.132,3.139,&
                    3.147,3.154,3.161,3.168,3.176,3.183,3.190,3.197,3.203,&
                    3.210,3.217,3.224,3.230,3.237,3.243,3.250,3.256,3.263,&
                    3.269,3.275,3.281,3.287,3.293,3.299,3.305,3.311,3.317,&
                    3.323,3.329,3.334,3.340,3.345,3.351,3.356,3.362,      &
                    3.367/)
!
! *** k2so4
!
    bnc17m(1:561) = (/-0.098,-0.214,-0.273,-0.315,-0.347,-0.375,-0.399,   &
                    -0.419,-0.438,-0.455,-0.470,-0.485,-0.498,-0.511,     &
                    -0.522,-0.533,-0.544,-0.554,-0.563,-0.572,-0.581,     &
                    -0.590,-0.598,-0.605,-0.613,-0.620,-0.627,-0.634,     &
                    -0.641,-0.647,-0.653,-0.660,-0.666,-0.671,-0.677,     &
                    -0.683,-0.688,-0.693,-0.698,-0.703,-0.708,-0.713,     &
                    -0.718,-0.723,-0.727,-0.732,-0.736,-0.741,-0.745,     &
                    -0.749,-0.753,-0.757,-0.762,-0.765,-0.769,-0.773,     &
                    -0.777,-0.781,-0.784,-0.788,-0.792,-0.795,-0.799,     &
                    -0.802,-0.806,-0.809,-0.812,-0.816,-0.819,-0.822,     &
                    -0.825,-0.829,-0.832,-0.835,-0.838,-0.841,-0.844,     &
                    -0.847,-0.850,-0.853,-0.856,-0.859,-0.862,-0.865,     &
                    -0.867,-0.870,-0.873,-0.876,-0.879,-0.881,-0.884,     &
                    -0.887,-0.890,-0.892,-0.895,-0.898,-0.900,-0.903,     &
                    -0.905,-0.908,-0.911,-0.913,-0.916,-0.918,-0.921,     &
                    -0.923,-0.926,-0.928,-0.931,-0.933,-0.936,-0.938,     &
                    -0.940,-0.943,-0.945,-0.948,-0.950,-0.952,-0.955,     &
                    -0.957,-0.959,-0.962,-0.964,-0.966,-0.969,-0.971,     &
                    -0.973,-0.975,-0.978,-0.980,-0.982,-0.984,-0.986,     &
                    -0.989,-0.991,-0.993,-0.995,-0.997,-0.999,-1.002,     &
                    -1.004,-1.006,-1.008,-1.010,-1.012,-1.014,-1.016,     &
                    -1.018,-1.020,-1.022,-1.024,-1.026,-1.028,-1.030,     &
                    -1.032,-1.034,-1.036,-1.038,-1.040,-1.042,-1.044,     &
                    -1.046,-1.048,-1.050,-1.052,-1.054,-1.056,-1.058,     &
                    -1.060,-1.062,-1.064,-1.066,-1.067,-1.069,-1.071,     &
                    -1.073,-1.075,-1.077,-1.079,-1.080,-1.082,-1.084,     &
                    -1.086,-1.088,-1.090,-1.091,-1.093,-1.095,-1.097,     &
                    -1.099,-1.100,-1.102,-1.104,-1.106,-1.107,-1.109,     &
                    -1.111,-1.113,-1.114,-1.116,-1.118,-1.120,-1.121,     &
                    -1.123,-1.125,-1.127,-1.128,-1.130,-1.132,-1.133,     &
                    -1.135,-1.137,-1.138,-1.140,-1.142,-1.143,-1.145,     &
                    -1.147,-1.148,-1.150,-1.152,-1.153,-1.155,-1.157,     &
                    -1.158,-1.160,-1.162,-1.163,-1.165,-1.166,-1.168,     &
                    -1.170,-1.171,-1.173,-1.174,-1.176,-1.178,-1.179,     &
                    -1.181,-1.182,-1.184,-1.185,-1.187,-1.189,-1.190,     &
                    -1.192,-1.193,-1.195,-1.196,-1.198,-1.199,-1.201,     &
                    -1.203,-1.204,-1.206,-1.207,-1.209,-1.210,-1.212,     &
                    -1.213,-1.215,-1.216,-1.218,-1.219,-1.221,-1.222,     &
                    -1.224,-1.225,-1.227,-1.228,-1.230,-1.231,-1.233,     &
                    -1.234,-1.236,-1.237,-1.238,-1.240,-1.241,-1.243,     &
                    -1.244,-1.246,-1.247,-1.249,-1.250,-1.252,-1.253,     &
                    -1.254,-1.256,-1.257,-1.259,-1.260,-1.262,-1.263,     &
                    -1.264,-1.266,-1.267,-1.269,-1.270,-1.271,-1.273,     &
                    -1.274,-1.276,-1.277,-1.278,-1.280,-1.281,-1.283,     &
                    -1.284,-1.285,-1.287,-1.288,-1.290,-1.291,-1.292,     &
                    -1.294,-1.295,-1.296,-1.298,-1.299,-1.301,-1.302,     &
                    -1.303,-1.305,-1.306,-1.307,-1.309,-1.310,-1.311,     &
                    -1.313,-1.314,-1.315,-1.317,-1.318,-1.319,-1.321,     &
                    -1.322,-1.323,-1.325,-1.326,-1.327,-1.329,-1.330,     &
                    -1.331,-1.333,-1.334,-1.335,-1.337,-1.338,-1.339,     &
                    -1.341,-1.342,-1.343,-1.344,-1.346,-1.347,-1.348,     &
                    -1.350,-1.351,-1.352,-1.354,-1.355,-1.356,-1.357,     &
                    -1.359,-1.360,-1.361,-1.363,-1.364,-1.365,-1.366,     &
                    -1.368,-1.369,-1.370,-1.371,-1.373,-1.374,-1.375,     &
                    -1.376,-1.378,-1.379,-1.380,-1.381,-1.383,-1.384,     &
                    -1.385,-1.387,-1.388,-1.389,-1.390,-1.391,-1.393,     &
                    -1.394,-1.395,-1.396,-1.398,-1.399,-1.400,-1.401,     &
                    -1.403,-1.416,-1.428,-1.440,-1.452,-1.463,-1.475,     &
                    -1.487,-1.498,-1.509,-1.520,-1.532,-1.543,-1.554,     &
                    -1.564,-1.575,-1.586,-1.596,-1.607,-1.617,-1.628,     &
                    -1.638,-1.648,-1.659,-1.669,-1.679,-1.689,-1.699,     &
                    -1.709,-1.719,-1.729,-1.738,-1.748,-1.758,-1.767,     &
                    -1.777,-1.786,-1.796,-1.805,-1.815,-1.824,-1.833,     &
                    -1.843,-1.852,-1.861,-1.870,-1.879,-1.888,-1.897,     &
                    -1.906,-1.915,-1.924,-1.933,-1.942,-1.951,-1.960,     &
                    -1.969,-1.977,-1.986,-1.995,-2.004,-2.012,-2.021,     &
                    -2.029,-2.038,-2.047,-2.055,-2.064,-2.072,-2.081,     &
                    -2.089,-2.097,-2.106,-2.114,-2.123,-2.131,-2.139,     &
                    -2.147,-2.156,-2.164,-2.172,-2.180,-2.189,-2.197,     &
                    -2.205,-2.213,-2.221,-2.229,-2.237,-2.245,-2.253,     &
                    -2.261,-2.269,-2.277,-2.285,-2.293,-2.301,-2.309,     &
                    -2.317,-2.325,-2.333,-2.341,-2.349,-2.356,-2.364,     &
                    -2.372,-2.380,-2.388,-2.395,-2.403,-2.411,-2.419,     &
                    -2.426,-2.434,-2.442,-2.449,-2.457,-2.465,-2.472,     &
                    -2.480,-2.488,-2.495,-2.503,-2.510,-2.518,-2.525,     &
                    -2.533,-2.541,-2.548,-2.556,-2.563,-2.571,-2.578,     &
                    -2.586,-2.593,-2.600,-2.608,-2.615,-2.623,-2.630,     &
                    -2.638,-2.645,-2.652,-2.660,-2.667,-2.674,-2.682,     &
                    -2.689,-2.696,-2.704,-2.711,-2.718,-2.726,-2.733,     &
                    -2.740,-2.747,-2.755,-2.762,-2.769,-2.776,-2.784,     &
                    -2.791/)
!
! *** khso4
!
    bnc18m(1:561) = (/-0.048,-0.102,-0.127,-0.145,-0.158,-0.169,-0.178,   &
                    -0.185,-0.192,-0.198,-0.203,-0.207,-0.211,-0.215,     &
                    -0.218,-0.221,-0.224,-0.226,-0.228,-0.230,-0.231,     &
                    -0.232,-0.234,-0.235,-0.235,-0.236,-0.237,-0.237,     &
                    -0.237,-0.237,-0.237,-0.237,-0.237,-0.236,-0.236,     &
                    -0.235,-0.234,-0.234,-0.233,-0.232,-0.231,-0.230,     &
                    -0.228,-0.227,-0.226,-0.224,-0.223,-0.221,-0.220,     &
                    -0.218,-0.216,-0.214,-0.212,-0.210,-0.208,-0.206,     &
                    -0.204,-0.202,-0.200,-0.198,-0.196,-0.193,-0.191,     &
                    -0.189,-0.186,-0.184,-0.181,-0.179,-0.176,-0.173,     &
                    -0.171,-0.168,-0.165,-0.163,-0.160,-0.157,-0.154,     &
                    -0.151,-0.148,-0.146,-0.143,-0.140,-0.137,-0.133,     &
                    -0.130,-0.127,-0.124,-0.121,-0.118,-0.115,-0.111,     &
                    -0.108,-0.105,-0.101,-0.098,-0.095,-0.091,-0.088,     &
                    -0.084,-0.081,-0.077,-0.074,-0.070,-0.067,-0.063,     &
                    -0.060,-0.056,-0.052,-0.049,-0.045,-0.042,-0.038,     &
                    -0.034,-0.031,-0.027,-0.023,-0.020,-0.016,-0.012,     &
                    -0.008,-0.005,-0.001,0.003,0.006,0.010,0.014,0.018,   &
                    0.021,0.025,0.029,0.032,0.036,0.040,0.043,0.047,0.051,&
                    0.054,0.058,0.062,0.065,0.069,0.073,0.076,0.080,0.084,&
                    0.087,0.091,0.094,0.098,0.102,0.105,0.109,0.112,0.116,&
                    0.119,0.123,0.127,0.130,0.134,0.137,0.141,0.144,0.148,&
                    0.151,0.155,0.158,0.162,0.165,0.168,0.172,0.175,0.179,&
                    0.182,0.186,0.189,0.192,0.196,0.199,0.202,0.206,0.209,&
                    0.212,0.216,0.219,0.222,0.226,0.229,0.232,0.236,0.239,&
                    0.242,0.245,0.249,0.252,0.255,0.258,0.262,0.265,0.268,&
                    0.271,0.274,0.278,0.281,0.284,0.287,0.290,0.293,0.297,&
                    0.300,0.303,0.306,0.309,0.312,0.315,0.318,0.321,0.324,&
                    0.327,0.330,0.334,0.337,0.340,0.343,0.346,0.349,0.352,&
                    0.355,0.358,0.361,0.364,0.367,0.369,0.372,0.375,0.378,&
                    0.381,0.384,0.387,0.390,0.393,0.396,0.399,0.402,0.404,&
                    0.407,0.410,0.413,0.416,0.419,0.422,0.424,0.427,0.430,&
                    0.433,0.436,0.438,0.441,0.444,0.447,0.450,0.452,0.455,&
                    0.458,0.461,0.463,0.466,0.469,0.471,0.474,0.477,0.480,&
                    0.482,0.485,0.488,0.490,0.493,0.496,0.498,0.501,0.504,&
                    0.506,0.509,0.512,0.514,0.517,0.519,0.522,0.525,0.527,&
                    0.530,0.532,0.535,0.538,0.540,0.543,0.545,0.548,0.550,&
                    0.553,0.555,0.558,0.560,0.563,0.566,0.568,0.571,0.573,&
                    0.576,0.578,0.581,0.583,0.585,0.588,0.590,0.593,0.595,&
                    0.598,0.600,0.603,0.605,0.607,0.610,0.612,0.615,0.617,&
                    0.620,0.622,0.624,0.627,0.629,0.632,0.634,0.636,0.639,&
                    0.641,0.643,0.646,0.648,0.650,0.653,0.655,0.657,0.660,&
                    0.662,0.664,0.667,0.669,0.671,0.674,0.676,0.678,0.680,&
                    0.683,0.685,0.687,0.689,0.692,0.694,0.696,0.699,0.701,&
                    0.703,0.705,0.707,0.710,0.712,0.714,0.716,0.719,0.721,&
                    0.723,0.725,0.727,0.730,0.732,0.734,0.736,0.738,0.740,&
                    0.743,0.745,0.747,0.749,0.751,0.753,0.755,0.758,0.760,&
                    0.762,0.764,0.766,0.768,0.770,0.772,0.775,0.777,0.779,&
                    0.781,0.783,0.785,0.807,0.828,0.848,0.867,0.886,0.905,&
                    0.924,0.942,0.960,0.978,0.995,1.013,1.029,1.046,1.062,&
                    1.079,1.095,1.110,1.126,1.141,1.156,1.171,1.185,1.200,&
                    1.214,1.228,1.242,1.256,1.269,1.282,1.295,1.308,1.321,&
                    1.334,1.346,1.359,1.371,1.383,1.395,1.406,1.418,1.429,&
                    1.441,1.452,1.463,1.474,1.485,1.495,1.506,1.516,1.527,&
                    1.537,1.547,1.557,1.567,1.577,1.586,1.596,1.605,1.615,&
                    1.624,1.633,1.642,1.651,1.660,1.669,1.678,1.686,1.695,&
                    1.703,1.711,1.720,1.728,1.736,1.744,1.752,1.760,1.768,&
                    1.775,1.783,1.791,1.798,1.806,1.813,1.820,1.828,1.835,&
                    1.842,1.849,1.856,1.863,1.870,1.876,1.883,1.890,1.896,&
                    1.903,1.909,1.916,1.922,1.928,1.935,1.941,1.947,1.953,&
                    1.959,1.965,1.971,1.977,1.983,1.989,1.994,2.000,2.006,&
                    2.011,2.017,2.022,2.028,2.033,2.039,2.044,2.049,2.054,&
                    2.060,2.065,2.070,2.075,2.080,2.085,2.090,2.095,2.100,&
                    2.104,2.109,2.114,2.119,2.123,2.128,2.132,2.137,2.142,&
                    2.146,2.150,2.155,2.159,2.164,2.168,2.172,2.176,2.181,&
                    2.185,2.189,2.193,2.197,2.201,2.205,2.209,2.213,2.217,&
                    2.221,2.225/)
!
! *** kno3
!
    bnc19m(1:561) = (/-0.051,-0.116,-0.152,-0.180,-0.203,-0.223,-0.241,   &
                    -0.257,-0.272,-0.286,-0.300,-0.313,-0.325,-0.336,     &
                    -0.348,-0.359,-0.369,-0.379,-0.389,-0.399,-0.408,     &
                    -0.417,-0.426,-0.435,-0.444,-0.452,-0.460,-0.468,     &
                    -0.476,-0.484,-0.492,-0.499,-0.507,-0.514,-0.521,     &
                    -0.528,-0.535,-0.542,-0.548,-0.555,-0.562,-0.568,     &
                    -0.574,-0.580,-0.587,-0.593,-0.599,-0.604,-0.610,     &
                    -0.616,-0.622,-0.627,-0.633,-0.638,-0.644,-0.649,     &
                    -0.654,-0.659,-0.664,-0.670,-0.675,-0.680,-0.684,     &
                    -0.689,-0.694,-0.699,-0.704,-0.708,-0.713,-0.718,     &
                    -0.722,-0.727,-0.731,-0.736,-0.740,-0.745,-0.749,     &
                    -0.754,-0.758,-0.763,-0.767,-0.771,-0.776,-0.780,     &
                    -0.784,-0.788,-0.793,-0.797,-0.801,-0.805,-0.810,     &
                    -0.814,-0.818,-0.822,-0.826,-0.830,-0.834,-0.839,     &
                    -0.843,-0.847,-0.851,-0.855,-0.859,-0.863,-0.867,     &
                    -0.871,-0.875,-0.879,-0.883,-0.887,-0.891,-0.895,     &
                    -0.899,-0.903,-0.907,-0.910,-0.914,-0.918,-0.922,     &
                    -0.926,-0.930,-0.933,-0.937,-0.941,-0.945,-0.948,     &
                    -0.952,-0.956,-0.959,-0.963,-0.967,-0.970,-0.974,     &
                    -0.977,-0.981,-0.985,-0.988,-0.992,-0.995,-0.999,     &
                    -1.002,-1.006,-1.009,-1.012,-1.016,-1.019,-1.023,     &
                    -1.026,-1.029,-1.033,-1.036,-1.039,-1.042,-1.046,     &
                    -1.049,-1.052,-1.055,-1.059,-1.062,-1.065,-1.068,     &
                    -1.071,-1.074,-1.078,-1.081,-1.084,-1.087,-1.090,     &
                    -1.093,-1.096,-1.099,-1.102,-1.105,-1.108,-1.111,     &
                    -1.114,-1.117,-1.120,-1.123,-1.126,-1.129,-1.131,     &
                    -1.134,-1.137,-1.140,-1.143,-1.146,-1.149,-1.151,     &
                    -1.154,-1.157,-1.160,-1.162,-1.165,-1.168,-1.171,     &
                    -1.173,-1.176,-1.179,-1.181,-1.184,-1.187,-1.189,     &
                    -1.192,-1.195,-1.197,-1.200,-1.202,-1.205,-1.207,     &
                    -1.210,-1.213,-1.215,-1.218,-1.220,-1.223,-1.225,     &
                    -1.228,-1.230,-1.233,-1.235,-1.237,-1.240,-1.242,     &
                    -1.245,-1.247,-1.250,-1.252,-1.254,-1.257,-1.259,     &
                    -1.261,-1.264,-1.266,-1.268,-1.271,-1.273,-1.275,     &
                    -1.278,-1.280,-1.282,-1.284,-1.287,-1.289,-1.291,     &
                    -1.293,-1.296,-1.298,-1.300,-1.302,-1.304,-1.307,     &
                    -1.309,-1.311,-1.313,-1.315,-1.317,-1.319,-1.322,     &
                    -1.324,-1.326,-1.328,-1.330,-1.332,-1.334,-1.336,     &
                    -1.338,-1.340,-1.342,-1.344,-1.346,-1.348,-1.350,     &
                    -1.352,-1.354,-1.356,-1.358,-1.360,-1.362,-1.364,     &
                    -1.366,-1.368,-1.370,-1.372,-1.374,-1.376,-1.378,     &
                    -1.380,-1.382,-1.384,-1.385,-1.387,-1.389,-1.391,     &
                    -1.393,-1.395,-1.397,-1.398,-1.400,-1.402,-1.404,     &
                    -1.406,-1.408,-1.409,-1.411,-1.413,-1.415,-1.417,     &
                    -1.418,-1.420,-1.422,-1.424,-1.425,-1.427,-1.429,     &
                    -1.431,-1.432,-1.434,-1.436,-1.437,-1.439,-1.441,     &
                    -1.443,-1.444,-1.446,-1.448,-1.449,-1.451,-1.453,     &
                    -1.454,-1.456,-1.458,-1.459,-1.461,-1.462,-1.464,     &
                    -1.466,-1.467,-1.469,-1.470,-1.472,-1.474,-1.475,     &
                    -1.477,-1.478,-1.480,-1.482,-1.483,-1.485,-1.486,     &
                    -1.488,-1.489,-1.491,-1.492,-1.494,-1.495,-1.497,     &
                    -1.498,-1.500,-1.501,-1.503,-1.504,-1.506,-1.507,     &
                    -1.509,-1.510,-1.512,-1.513,-1.515,-1.516,-1.518,     &
                    -1.519,-1.521,-1.522,-1.523,-1.525,-1.526,-1.528,     &
                    -1.529,-1.530,-1.532,-1.533,-1.535,-1.536,-1.537,     &
                    -1.539,-1.540,-1.542,-1.543,-1.544,-1.546,-1.547,     &
                    -1.548,-1.550,-1.551,-1.553,-1.554,-1.555,-1.557,     &
                    -1.558,-1.572,-1.585,-1.597,-1.609,-1.621,-1.632,     &
                    -1.643,-1.654,-1.665,-1.675,-1.685,-1.695,-1.705,     &
                    -1.714,-1.723,-1.732,-1.741,-1.750,-1.758,-1.766,     &
                    -1.774,-1.782,-1.790,-1.798,-1.805,-1.813,-1.820,     &
                    -1.827,-1.834,-1.841,-1.848,-1.855,-1.861,-1.868,     &
                    -1.874,-1.881,-1.887,-1.893,-1.899,-1.905,-1.911,     &
                    -1.917,-1.923,-1.929,-1.934,-1.940,-1.945,-1.951,     &
                    -1.956,-1.962,-1.967,-1.972,-1.977,-1.982,-1.988,     &
                    -1.993,-1.998,-2.003,-2.008,-2.012,-2.017,-2.022,     &
                    -2.027,-2.032,-2.036,-2.041,-2.046,-2.050,-2.055,     &
                    -2.059,-2.064,-2.068,-2.073,-2.077,-2.082,-2.086,     &
                    -2.090,-2.095,-2.099,-2.103,-2.108,-2.112,-2.116,     &
                    -2.120,-2.124,-2.128,-2.133,-2.137,-2.141,-2.145,     &
                    -2.149,-2.153,-2.157,-2.161,-2.165,-2.169,-2.173,     &
                    -2.177,-2.181,-2.185,-2.189,-2.192,-2.196,-2.200,     &
                    -2.204,-2.208,-2.212,-2.216,-2.219,-2.223,-2.227,     &
                    -2.231,-2.234,-2.238,-2.242,-2.246,-2.249,-2.253,     &
                    -2.257,-2.260,-2.264,-2.268,-2.271,-2.275,-2.279,     &
                    -2.282,-2.286,-2.289,-2.293,-2.297,-2.300,-2.304,     &
                    -2.307,-2.311,-2.315,-2.318,-2.322,-2.325,-2.329,     &
                    -2.332,-2.336,-2.339,-2.343,-2.346,-2.350,-2.353,     &
                    -2.357,-2.360,-2.364,-2.367,-2.370,-2.374,-2.377,     &
                    -2.381,-2.384,-2.388,-2.391,-2.395,-2.398,-2.401,     &
                    -2.405/)
!
! *** kcl
!
    bnc20m(1:561) = (/-0.048,-0.103,-0.129,-0.147,-0.160,-0.171,-0.180,   &
                    -0.188,-0.195,-0.201,-0.206,-0.210,-0.215,-0.218,     &
                    -0.222,-0.225,-0.228,-0.230,-0.233,-0.235,-0.237,     &
                    -0.239,-0.241,-0.242,-0.244,-0.245,-0.247,-0.248,     &
                    -0.249,-0.250,-0.251,-0.252,-0.253,-0.254,-0.255,     &
                    -0.256,-0.256,-0.257,-0.258,-0.258,-0.259,-0.260,     &
                    -0.260,-0.261,-0.261,-0.261,-0.262,-0.262,-0.263,     &
                    -0.263,-0.263,-0.264,-0.264,-0.264,-0.265,-0.265,     &
                    -0.265,-0.265,-0.266,-0.266,-0.266,-0.266,-0.266,     &
                    -0.267,-0.267,-0.267,-0.267,-0.267,-0.267,-0.267,     &
                    -0.267,-0.267,-0.267,-0.268,-0.268,-0.268,-0.268,     &
                    -0.268,-0.268,-0.268,-0.268,-0.268,-0.267,-0.267,     &
                    -0.267,-0.267,-0.267,-0.267,-0.267,-0.267,-0.267,     &
                    -0.267,-0.266,-0.266,-0.266,-0.266,-0.266,-0.265,     &
                    -0.265,-0.265,-0.265,-0.265,-0.264,-0.264,-0.264,     &
                    -0.264,-0.263,-0.263,-0.263,-0.262,-0.262,-0.262,     &
                    -0.261,-0.261,-0.261,-0.260,-0.260,-0.260,-0.259,     &
                    -0.259,-0.259,-0.258,-0.258,-0.258,-0.257,-0.257,     &
                    -0.257,-0.256,-0.256,-0.256,-0.255,-0.255,-0.254,     &
                    -0.254,-0.254,-0.253,-0.253,-0.253,-0.252,-0.252,     &
                    -0.251,-0.251,-0.251,-0.250,-0.250,-0.249,-0.249,     &
                    -0.249,-0.248,-0.248,-0.247,-0.247,-0.247,-0.246,     &
                    -0.246,-0.246,-0.245,-0.245,-0.244,-0.244,-0.244,     &
                    -0.243,-0.243,-0.242,-0.242,-0.242,-0.241,-0.241,     &
                    -0.240,-0.240,-0.240,-0.239,-0.239,-0.238,-0.238,     &
                    -0.238,-0.237,-0.237,-0.236,-0.236,-0.236,-0.235,     &
                    -0.235,-0.234,-0.234,-0.234,-0.233,-0.233,-0.232,     &
                    -0.232,-0.232,-0.231,-0.231,-0.230,-0.230,-0.230,     &
                    -0.229,-0.229,-0.228,-0.228,-0.228,-0.227,-0.227,     &
                    -0.227,-0.226,-0.226,-0.225,-0.225,-0.225,-0.224,     &
                    -0.224,-0.223,-0.223,-0.223,-0.222,-0.222,-0.221,     &
                    -0.221,-0.221,-0.220,-0.220,-0.220,-0.219,-0.219,     &
                    -0.218,-0.218,-0.218,-0.217,-0.217,-0.217,-0.216,     &
                    -0.216,-0.215,-0.215,-0.215,-0.214,-0.214,-0.214,     &
                    -0.213,-0.213,-0.212,-0.212,-0.212,-0.211,-0.211,     &
                    -0.211,-0.210,-0.210,-0.209,-0.209,-0.209,-0.208,     &
                    -0.208,-0.208,-0.207,-0.207,-0.207,-0.206,-0.206,     &
                    -0.205,-0.205,-0.205,-0.204,-0.204,-0.204,-0.203,     &
                    -0.203,-0.203,-0.202,-0.202,-0.202,-0.201,-0.201,     &
                    -0.201,-0.200,-0.200,-0.199,-0.199,-0.199,-0.198,     &
                    -0.198,-0.198,-0.197,-0.197,-0.197,-0.196,-0.196,     &
                    -0.196,-0.195,-0.195,-0.195,-0.194,-0.194,-0.194,     &
                    -0.193,-0.193,-0.193,-0.192,-0.192,-0.192,-0.191,     &
                    -0.191,-0.191,-0.190,-0.190,-0.190,-0.189,-0.189,     &
                    -0.189,-0.188,-0.188,-0.188,-0.187,-0.187,-0.187,     &
                    -0.186,-0.186,-0.186,-0.186,-0.185,-0.185,-0.185,     &
                    -0.184,-0.184,-0.184,-0.183,-0.183,-0.183,-0.182,     &
                    -0.182,-0.182,-0.181,-0.181,-0.181,-0.181,-0.180,     &
                    -0.180,-0.180,-0.179,-0.179,-0.179,-0.178,-0.178,     &
                    -0.178,-0.178,-0.177,-0.177,-0.177,-0.176,-0.176,     &
                    -0.176,-0.175,-0.175,-0.175,-0.175,-0.174,-0.174,     &
                    -0.174,-0.173,-0.173,-0.173,-0.173,-0.172,-0.172,     &
                    -0.172,-0.171,-0.171,-0.171,-0.171,-0.170,-0.170,     &
                    -0.170,-0.169,-0.169,-0.169,-0.169,-0.168,-0.168,     &
                    -0.168,-0.167,-0.167,-0.167,-0.167,-0.166,-0.166,     &
                    -0.166,-0.166,-0.165,-0.165,-0.165,-0.164,-0.164,     &
                    -0.164,-0.164,-0.163,-0.163,-0.163,-0.163,-0.162,     &
                    -0.162,-0.159,-0.157,-0.154,-0.152,-0.149,-0.147,     &
                    -0.145,-0.143,-0.141,-0.139,-0.137,-0.135,-0.133,     &
                    -0.131,-0.129,-0.127,-0.126,-0.124,-0.122,-0.121,     &
                    -0.119,-0.118,-0.116,-0.115,-0.114,-0.112,-0.111,     &
                    -0.110,-0.109,-0.107,-0.106,-0.105,-0.104,-0.103,     &
                    -0.102,-0.101,-0.100,-0.099,-0.099,-0.098,-0.097,     &
                    -0.096,-0.095,-0.095,-0.094,-0.093,-0.093,-0.092,     &
                    -0.092,-0.091,-0.091,-0.090,-0.090,-0.089,-0.089,     &
                    -0.089,-0.088,-0.088,-0.088,-0.087,-0.087,-0.087,     &
                    -0.087,-0.087,-0.086,-0.086,-0.086,-0.086,-0.086,     &
                    -0.086,-0.086,-0.086,-0.086,-0.086,-0.086,-0.086,     &
                    -0.086,-0.086,-0.086,-0.086,-0.087,-0.087,-0.087,     &
                    -0.087,-0.087,-0.088,-0.088,-0.088,-0.088,-0.089,     &
                    -0.089,-0.089,-0.090,-0.090,-0.090,-0.091,-0.091,     &
                    -0.092,-0.092,-0.092,-0.093,-0.093,-0.094,-0.094,     &
                    -0.095,-0.095,-0.096,-0.096,-0.097,-0.098,-0.098,     &
                    -0.099,-0.099,-0.100,-0.101,-0.101,-0.102,-0.103,     &
                    -0.103,-0.104,-0.105,-0.105,-0.106,-0.107,-0.107,     &
                    -0.108,-0.109,-0.110,-0.111,-0.111,-0.112,-0.113,     &
                    -0.114,-0.115,-0.115,-0.116,-0.117,-0.118,-0.119,     &
                    -0.120,-0.121,-0.122,-0.122,-0.123,-0.124,-0.125,     &
                    -0.126,-0.127,-0.128,-0.129,-0.130,-0.131,-0.132,     &
                    -0.133,-0.134,-0.135,-0.136,-0.137,-0.138,-0.139,     &
                    -0.140/)
!
! *** mgso4
!
    bnc21m(1:561) = (/-0.195,-0.422,-0.535,-0.614,-0.676,-0.727,-0.770,   &
                    -0.808,-0.841,-0.871,-0.898,-0.923,-0.946,-0.968,     &
                    -0.988,-1.006,-1.024,-1.041,-1.056,-1.071,-1.085,     &
                    -1.099,-1.112,-1.124,-1.136,-1.148,-1.159,-1.169,     &
                    -1.179,-1.189,-1.199,-1.208,-1.217,-1.226,-1.234,     &
                    -1.243,-1.251,-1.258,-1.266,-1.273,-1.281,-1.288,     &
                    -1.295,-1.302,-1.308,-1.315,-1.321,-1.327,-1.333,     &
                    -1.339,-1.345,-1.351,-1.357,-1.362,-1.368,-1.373,     &
                    -1.379,-1.384,-1.389,-1.394,-1.399,-1.404,-1.409,     &
                    -1.413,-1.418,-1.423,-1.427,-1.432,-1.436,-1.441,     &
                    -1.445,-1.449,-1.454,-1.458,-1.462,-1.466,-1.470,     &
                    -1.474,-1.478,-1.482,-1.486,-1.489,-1.493,-1.497,     &
                    -1.500,-1.504,-1.508,-1.511,-1.515,-1.518,-1.522,     &
                    -1.525,-1.528,-1.532,-1.535,-1.538,-1.541,-1.545,     &
                    -1.548,-1.551,-1.554,-1.557,-1.560,-1.563,-1.566,     &
                    -1.569,-1.572,-1.575,-1.578,-1.581,-1.584,-1.586,     &
                    -1.589,-1.592,-1.595,-1.598,-1.600,-1.603,-1.606,     &
                    -1.608,-1.611,-1.614,-1.616,-1.619,-1.621,-1.624,     &
                    -1.626,-1.629,-1.631,-1.634,-1.636,-1.639,-1.641,     &
                    -1.644,-1.646,-1.649,-1.651,-1.653,-1.656,-1.658,     &
                    -1.660,-1.663,-1.665,-1.667,-1.670,-1.672,-1.674,     &
                    -1.676,-1.679,-1.681,-1.683,-1.685,-1.688,-1.690,     &
                    -1.692,-1.694,-1.696,-1.698,-1.701,-1.703,-1.705,     &
                    -1.707,-1.709,-1.711,-1.713,-1.715,-1.717,-1.720,     &
                    -1.722,-1.724,-1.726,-1.728,-1.730,-1.732,-1.734,     &
                    -1.736,-1.738,-1.740,-1.742,-1.744,-1.746,-1.748,     &
                    -1.750,-1.752,-1.754,-1.756,-1.757,-1.759,-1.761,     &
                    -1.763,-1.765,-1.767,-1.769,-1.771,-1.773,-1.775,     &
                    -1.776,-1.778,-1.780,-1.782,-1.784,-1.786,-1.788,     &
                    -1.789,-1.791,-1.793,-1.795,-1.797,-1.799,-1.800,     &
                    -1.802,-1.804,-1.806,-1.808,-1.809,-1.811,-1.813,     &
                    -1.815,-1.816,-1.818,-1.820,-1.822,-1.823,-1.825,     &
                    -1.827,-1.829,-1.830,-1.832,-1.834,-1.836,-1.837,     &
                    -1.839,-1.841,-1.842,-1.844,-1.846,-1.848,-1.849,     &
                    -1.851,-1.853,-1.854,-1.856,-1.858,-1.859,-1.861,     &
                    -1.863,-1.864,-1.866,-1.868,-1.869,-1.871,-1.873,     &
                    -1.874,-1.876,-1.878,-1.879,-1.881,-1.882,-1.884,     &
                    -1.886,-1.887,-1.889,-1.891,-1.892,-1.894,-1.895,     &
                    -1.897,-1.899,-1.900,-1.902,-1.903,-1.905,-1.907,     &
                    -1.908,-1.910,-1.911,-1.913,-1.915,-1.916,-1.918,     &
                    -1.919,-1.921,-1.922,-1.924,-1.925,-1.927,-1.929,     &
                    -1.930,-1.932,-1.933,-1.935,-1.936,-1.938,-1.939,     &
                    -1.941,-1.943,-1.944,-1.946,-1.947,-1.949,-1.950,     &
                    -1.952,-1.953,-1.955,-1.956,-1.958,-1.959,-1.961,     &
                    -1.962,-1.964,-1.965,-1.967,-1.968,-1.970,-1.971,     &
                    -1.973,-1.974,-1.976,-1.977,-1.979,-1.980,-1.982,     &
                    -1.983,-1.985,-1.986,-1.988,-1.989,-1.991,-1.992,     &
                    -1.994,-1.995,-1.997,-1.998,-2.000,-2.001,-2.003,     &
                    -2.004,-2.006,-2.007,-2.008,-2.010,-2.011,-2.013,     &
                    -2.014,-2.016,-2.017,-2.019,-2.020,-2.022,-2.023,     &
                    -2.024,-2.026,-2.027,-2.029,-2.030,-2.032,-2.033,     &
                    -2.035,-2.036,-2.037,-2.039,-2.040,-2.042,-2.043,     &
                    -2.045,-2.046,-2.047,-2.049,-2.050,-2.052,-2.053,     &
                    -2.055,-2.056,-2.057,-2.059,-2.060,-2.062,-2.063,     &
                    -2.065,-2.066,-2.067,-2.069,-2.070,-2.072,-2.073,     &
                    -2.074,-2.076,-2.077,-2.079,-2.080,-2.081,-2.083,     &
                    -2.084,-2.086,-2.087,-2.088,-2.090,-2.091,-2.093,     &
                    -2.094,-2.109,-2.123,-2.137,-2.150,-2.164,-2.177,     &
                    -2.191,-2.204,-2.217,-2.231,-2.244,-2.257,-2.270,     &
                    -2.283,-2.296,-2.309,-2.322,-2.335,-2.348,-2.361,     &
                    -2.374,-2.386,-2.399,-2.412,-2.425,-2.437,-2.450,     &
                    -2.463,-2.475,-2.488,-2.500,-2.513,-2.525,-2.538,     &
                    -2.550,-2.563,-2.575,-2.588,-2.600,-2.612,-2.625,     &
                    -2.637,-2.650,-2.662,-2.674,-2.686,-2.699,-2.711,     &
                    -2.723,-2.736,-2.748,-2.760,-2.772,-2.784,-2.797,     &
                    -2.809,-2.821,-2.833,-2.845,-2.857,-2.870,-2.882,     &
                    -2.894,-2.906,-2.918,-2.930,-2.942,-2.954,-2.966,     &
                    -2.978,-2.991,-3.003,-3.015,-3.027,-3.039,-3.051,     &
                    -3.063,-3.075,-3.087,-3.099,-3.111,-3.123,-3.135,     &
                    -3.147,-3.159,-3.171,-3.182,-3.194,-3.206,-3.218,     &
                    -3.230,-3.242,-3.254,-3.266,-3.278,-3.290,-3.302,     &
                    -3.314,-3.325,-3.337,-3.349,-3.361,-3.373,-3.385,     &
                    -3.397,-3.409,-3.420,-3.432,-3.444,-3.456,-3.468,     &
                    -3.480,-3.491,-3.503,-3.515,-3.527,-3.539,-3.551,     &
                    -3.562,-3.574,-3.586,-3.598,-3.609,-3.621,-3.633,     &
                    -3.645,-3.657,-3.668,-3.680,-3.692,-3.704,-3.715,     &
                    -3.727,-3.739,-3.751,-3.762,-3.774,-3.786,-3.798,     &
                    -3.809,-3.821,-3.833,-3.844,-3.856,-3.868,-3.880,     &
                    -3.891,-3.903,-3.915,-3.926,-3.938,-3.950,-3.961,     &
                    -3.973,-3.985,-3.997,-4.008,-4.020,-4.032,-4.043,     &
                    -4.055/)
!
! *** mgno32
!
    bnc22m(1:561) = (/-0.095,-0.197,-0.243,-0.273,-0.294,-0.311,-0.323,   &
                    -0.333,-0.342,-0.348,-0.353,-0.358,-0.361,-0.364,     &
                    -0.366,-0.367,-0.368,-0.368,-0.368,-0.368,-0.368,     &
                    -0.367,-0.366,-0.364,-0.363,-0.361,-0.359,-0.357,     &
                    -0.355,-0.353,-0.351,-0.348,-0.346,-0.343,-0.340,     &
                    -0.337,-0.335,-0.332,-0.329,-0.326,-0.323,-0.320,     &
                    -0.316,-0.313,-0.310,-0.307,-0.304,-0.300,-0.297,     &
                    -0.294,-0.290,-0.287,-0.284,-0.280,-0.277,-0.274,     &
                    -0.270,-0.267,-0.264,-0.260,-0.257,-0.253,-0.250,     &
                    -0.247,-0.243,-0.240,-0.236,-0.233,-0.229,-0.226,     &
                    -0.222,-0.219,-0.215,-0.211,-0.208,-0.204,-0.200,     &
                    -0.197,-0.193,-0.189,-0.186,-0.182,-0.178,-0.174,     &
                    -0.170,-0.166,-0.162,-0.158,-0.154,-0.150,-0.146,     &
                    -0.142,-0.138,-0.134,-0.130,-0.125,-0.121,-0.117,     &
                    -0.113,-0.108,-0.104,-0.100,-0.095,-0.091,-0.086,     &
                    -0.082,-0.077,-0.073,-0.068,-0.064,-0.059,-0.055,     &
                    -0.050,-0.045,-0.041,-0.036,-0.032,-0.027,-0.022,     &
                    -0.018,-0.013,-0.008,-0.004,0.001,0.006,0.010,0.015,  &
                    0.020,0.024,0.029,0.034,0.039,0.043,0.048,0.053,0.057,&
                    0.062,0.067,0.071,0.076,0.081,0.085,0.090,0.095,0.099,&
                    0.104,0.109,0.113,0.118,0.122,0.127,0.132,0.136,0.141,&
                    0.145,0.150,0.155,0.159,0.164,0.168,0.173,0.178,0.182,&
                    0.187,0.191,0.196,0.200,0.205,0.209,0.214,0.218,0.223,&
                    0.227,0.232,0.236,0.241,0.245,0.250,0.254,0.259,0.263,&
                    0.268,0.272,0.276,0.281,0.285,0.290,0.294,0.299,0.303,&
                    0.307,0.312,0.316,0.320,0.325,0.329,0.333,0.338,0.342,&
                    0.346,0.351,0.355,0.359,0.364,0.368,0.372,0.377,0.381,&
                    0.385,0.389,0.394,0.398,0.402,0.406,0.411,0.415,0.419,&
                    0.423,0.427,0.432,0.436,0.440,0.444,0.448,0.452,0.457,&
                    0.461,0.465,0.469,0.473,0.477,0.481,0.485,0.490,0.494,&
                    0.498,0.502,0.506,0.510,0.514,0.518,0.522,0.526,0.530,&
                    0.534,0.538,0.542,0.546,0.550,0.554,0.558,0.562,0.566,&
                    0.570,0.574,0.578,0.582,0.586,0.590,0.594,0.598,0.602,&
                    0.605,0.609,0.613,0.617,0.621,0.625,0.629,0.633,0.636,&
                    0.640,0.644,0.648,0.652,0.656,0.659,0.663,0.667,0.671,&
                    0.674,0.678,0.682,0.686,0.690,0.693,0.697,0.701,0.704,&
                    0.708,0.712,0.716,0.719,0.723,0.727,0.730,0.734,0.738,&
                    0.741,0.745,0.749,0.752,0.756,0.760,0.763,0.767,0.771,&
                    0.774,0.778,0.781,0.785,0.789,0.792,0.796,0.799,0.803,&
                    0.806,0.810,0.813,0.817,0.821,0.824,0.828,0.831,0.835,&
                    0.838,0.842,0.845,0.849,0.852,0.856,0.859,0.862,0.866,&
                    0.869,0.873,0.876,0.880,0.883,0.886,0.890,0.893,0.897,&
                    0.900,0.903,0.907,0.910,0.914,0.917,0.920,0.924,0.927,&
                    0.930,0.934,0.937,0.940,0.944,0.947,0.950,0.954,0.957,&
                    0.960,0.963,0.967,0.970,0.973,0.977,0.980,0.983,0.986,&
                    0.990,0.993,0.996,0.999,1.003,1.006,1.009,1.012,1.015,&
                    1.019,1.022,1.025,1.028,1.031,1.034,1.038,1.041,1.044,&
                    1.047,1.050,1.053,1.056,1.060,1.063,1.066,1.069,1.072,&
                    1.075,1.078,1.081,1.114,1.144,1.174,1.203,1.232,1.260,&
                    1.288,1.315,1.342,1.369,1.395,1.420,1.446,1.471,1.495,&
                    1.519,1.543,1.567,1.590,1.613,1.635,1.657,1.679,1.701,&
                    1.722,1.743,1.764,1.784,1.804,1.824,1.844,1.863,1.882,&
                    1.901,1.920,1.938,1.956,1.974,1.992,2.009,2.026,2.043,&
                    2.060,2.077,2.093,2.109,2.125,2.141,2.157,2.172,2.187,&
                    2.202,2.217,2.232,2.246,2.261,2.275,2.289,2.303,2.317,&
                    2.330,2.344,2.357,2.370,2.383,2.396,2.408,2.421,2.433,&
                    2.445,2.458,2.470,2.481,2.493,2.505,2.516,2.528,2.539,&
                    2.550,2.561,2.572,2.583,2.593,2.604,2.614,2.625,2.635,&
                    2.645,2.655,2.665,2.675,2.684,2.694,2.704,2.713,2.722,&
                    2.732,2.741,2.750,2.759,2.768,2.776,2.785,2.794,2.802,&
                    2.811,2.819,2.827,2.835,2.843,2.851,2.859,2.867,2.875,&
                    2.883,2.890,2.898,2.906,2.913,2.920,2.928,2.935,2.942,&
                    2.949,2.956,2.963,2.970,2.977,2.983,2.990,2.997,3.003,&
                    3.010,3.016,3.022,3.029,3.035,3.041,3.047,3.053,3.059,&
                    3.065,3.071,3.077,3.083,3.089,3.094,3.100,3.105,3.111,&
                    3.116,3.122,3.127,3.133,3.138,3.143,3.148,3.153,3.158,&
                    3.163,3.168/)
!
! *** mgcl2
!
    bnc23m(1:561) = (/-0.094,-0.194,-0.238,-0.266,-0.285,-0.299,-0.310,   &
                    -0.319,-0.325,-0.330,-0.333,-0.336,-0.337,-0.338,     &
                    -0.338,-0.338,-0.337,-0.336,-0.334,-0.332,-0.330,     &
                    -0.328,-0.325,-0.322,-0.319,-0.315,-0.312,-0.308,     &
                    -0.304,-0.300,-0.296,-0.292,-0.288,-0.283,-0.279,     &
                    -0.274,-0.270,-0.265,-0.261,-0.256,-0.251,-0.247,     &
                    -0.242,-0.237,-0.232,-0.227,-0.222,-0.218,-0.213,     &
                    -0.208,-0.203,-0.198,-0.193,-0.188,-0.183,-0.178,     &
                    -0.173,-0.168,-0.163,-0.158,-0.153,-0.148,-0.143,     &
                    -0.138,-0.133,-0.128,-0.123,-0.118,-0.113,-0.108,     &
                    -0.103,-0.097,-0.092,-0.087,-0.082,-0.077,-0.071,     &
                    -0.066,-0.061,-0.055,-0.050,-0.045,-0.039,-0.034,     &
                    -0.028,-0.023,-0.017,-0.011,-0.006,0.000,0.006,0.011, &
                    0.017,0.023,0.029,0.035,0.041,0.047,0.053,0.059,0.065,&
                    0.071,0.077,0.083,0.089,0.096,0.102,0.108,0.114,0.121,&
                    0.127,0.133,0.140,0.146,0.152,0.159,0.165,0.171,0.178,&
                    0.184,0.191,0.197,0.203,0.210,0.216,0.223,0.229,0.236,&
                    0.242,0.249,0.255,0.261,0.268,0.274,0.281,0.287,0.293,&
                    0.300,0.306,0.313,0.319,0.325,0.332,0.338,0.345,0.351,&
                    0.357,0.364,0.370,0.376,0.383,0.389,0.395,0.402,0.408,&
                    0.414,0.421,0.427,0.433,0.439,0.446,0.452,0.458,0.464,&
                    0.471,0.477,0.483,0.489,0.495,0.502,0.508,0.514,0.520,&
                    0.526,0.532,0.538,0.545,0.551,0.557,0.563,0.569,0.575,&
                    0.581,0.587,0.593,0.599,0.605,0.611,0.617,0.623,0.629,&
                    0.635,0.641,0.647,0.653,0.659,0.665,0.671,0.677,0.683,&
                    0.689,0.694,0.700,0.706,0.712,0.718,0.724,0.730,0.735,&
                    0.741,0.747,0.753,0.759,0.764,0.770,0.776,0.782,0.787,&
                    0.793,0.799,0.804,0.810,0.816,0.822,0.827,0.833,0.838,&
                    0.844,0.850,0.855,0.861,0.867,0.872,0.878,0.883,0.889,&
                    0.894,0.900,0.905,0.911,0.916,0.922,0.927,0.933,0.938,&
                    0.944,0.949,0.955,0.960,0.966,0.971,0.976,0.982,0.987,&
                    0.993,0.998,1.003,1.009,1.014,1.019,1.025,1.030,1.035,&
                    1.041,1.046,1.051,1.056,1.062,1.067,1.072,1.077,1.083,&
                    1.088,1.093,1.098,1.103,1.109,1.114,1.119,1.124,1.129,&
                    1.134,1.139,1.144,1.150,1.155,1.160,1.165,1.170,1.175,&
                    1.180,1.185,1.190,1.195,1.200,1.205,1.210,1.215,1.220,&
                    1.225,1.230,1.235,1.240,1.245,1.250,1.255,1.260,1.264,&
                    1.269,1.274,1.279,1.284,1.289,1.294,1.299,1.303,1.308,&
                    1.313,1.318,1.323,1.327,1.332,1.337,1.342,1.346,1.351,&
                    1.356,1.361,1.365,1.370,1.375,1.380,1.384,1.389,1.394,&
                    1.398,1.403,1.408,1.412,1.417,1.421,1.426,1.431,1.435,&
                    1.440,1.444,1.449,1.454,1.458,1.463,1.467,1.472,1.476,&
                    1.481,1.485,1.490,1.494,1.499,1.503,1.508,1.512,1.517,&
                    1.521,1.526,1.530,1.535,1.539,1.543,1.548,1.552,1.557,&
                    1.561,1.565,1.570,1.574,1.578,1.583,1.587,1.591,1.596,&
                    1.600,1.604,1.609,1.613,1.617,1.622,1.626,1.630,1.634,&
                    1.639,1.643,1.647,1.651,1.656,1.660,1.664,1.668,1.672,&
                    1.677,1.681,1.726,1.766,1.807,1.846,1.885,1.923,1.961,&
                    1.998,2.034,2.070,2.105,2.140,2.175,2.208,2.242,2.275,&
                    2.307,2.339,2.370,2.401,2.432,2.462,2.492,2.521,2.550,&
                    2.579,2.607,2.635,2.662,2.690,2.716,2.743,2.769,2.795,&
                    2.820,2.845,2.870,2.895,2.919,2.943,2.966,2.990,3.013,&
                    3.036,3.058,3.081,3.103,3.124,3.146,3.167,3.188,3.209,&
                    3.230,3.250,3.270,3.290,3.310,3.329,3.349,3.368,3.387,&
                    3.405,3.424,3.442,3.460,3.478,3.496,3.513,3.531,3.548,&
                    3.565,3.582,3.599,3.615,3.631,3.648,3.664,3.680,3.695,&
                    3.711,3.726,3.742,3.757,3.772,3.787,3.801,3.816,3.830,&
                    3.845,3.859,3.873,3.887,3.901,3.914,3.928,3.941,3.955,&
                    3.968,3.981,3.994,4.007,4.019,4.032,4.044,4.057,4.069,&
                    4.081,4.093,4.105,4.117,4.129,4.141,4.152,4.164,4.175,&
                    4.186,4.197,4.209,4.220,4.230,4.241,4.252,4.263,4.273,&
                    4.284,4.294,4.304,4.314,4.324,4.335,4.344,4.354,4.364,&
                    4.374,4.383,4.393,4.402,4.412,4.421,4.430,4.440,4.449,&
                    4.458,4.467,4.476,4.484,4.493,4.502,4.510,4.519,4.527,&
                    4.536,4.544,4.553,4.561,4.569,4.577,4.585,4.593,4.601,&
                    4.609/)
!
! *** find position in arrays for bincoef
!
    if ( ionic<=0.200000D+02 ) then
      ipos = min(nint(0.200000D+02*ionic)+1,400)
    else
      ipos = 400 + nint(0.200000D+01*ionic-0.400000D+02)
    end if
    ipos = min(ipos,561)
!
! *** assign values to return array
!
    binarr(01) = bnc01m(ipos)
    binarr(02) = bnc02m(ipos)
    binarr(03) = bnc03m(ipos)
    binarr(04) = bnc04m(ipos)
    binarr(05) = bnc05m(ipos)
    binarr(06) = bnc06m(ipos)
    binarr(07) = bnc07m(ipos)
    binarr(08) = bnc08m(ipos)
    binarr(09) = bnc09m(ipos)
    binarr(10) = bnc10m(ipos)
    binarr(11) = bnc11m(ipos)
    binarr(12) = bnc12m(ipos)
    binarr(13) = bnc13m(ipos)
    binarr(14) = bnc14m(ipos)
    binarr(15) = bnc15m(ipos)
    binarr(16) = bnc16m(ipos)
    binarr(17) = bnc17m(ipos)
    binarr(18) = bnc18m(ipos)
    binarr(19) = bnc19m(ipos)
    binarr(20) = bnc20m(ipos)
    binarr(21) = bnc21m(ipos)
    binarr(22) = bnc22m(ipos)
    binarr(23) = bnc23m(ipos)
  end subroutine km248
!
!======================================================================
!
! *** isorropia code
! *** subroutine km273
! *** calculates binary activity coefficients by kusik-meissner method.
!     the computations have been performed and the results are stored in
!     lookup tables. the ionic activity 'in' is input , and the array
!     'binarr' is returned with the binary coefficients.
!
!     temperature is 273k
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine km273(ionic,binarr)
    implicit none
    integer(ik4) :: ipos
    real(rkx) , dimension(23) :: binarr
    real(rkx) :: ionic
    real(rkx) , dimension(561) :: bnc01m , bnc02m , bnc03m , bnc04m ,     &
                                  bnc05m , bnc06m , bnc07m , bnc08m ,     &
                                  bnc09m , bnc10m , bnc11m , bnc12m ,     &
                                  bnc13m , bnc14m , bnc15m , bnc16m ,     &
                                  bnc17m , bnc18m , bnc19m , bnc20m ,     &
                                  bnc21m , bnc22m , bnc23m
!
! *** nacl
!
    bnc01m(1:561) = (/-0.046,-0.097,-0.119,-0.134,-0.145,-0.153,-0.160,   &
                    -0.165,-0.169,-0.173,-0.176,-0.178,-0.180,-0.181,     &
                    -0.182,-0.183,-0.184,-0.184,-0.185,-0.185,-0.185,     &
                    -0.185,-0.184,-0.184,-0.183,-0.183,-0.182,-0.181,     &
                    -0.181,-0.180,-0.179,-0.178,-0.177,-0.176,-0.174,     &
                    -0.173,-0.172,-0.171,-0.170,-0.168,-0.167,-0.166,     &
                    -0.164,-0.163,-0.162,-0.160,-0.159,-0.157,-0.156,     &
                    -0.155,-0.153,-0.152,-0.150,-0.149,-0.147,-0.146,     &
                    -0.144,-0.143,-0.141,-0.140,-0.138,-0.137,-0.135,     &
                    -0.134,-0.132,-0.131,-0.129,-0.128,-0.126,-0.124,     &
                    -0.123,-0.121,-0.120,-0.118,-0.116,-0.115,-0.113,     &
                    -0.112,-0.110,-0.108,-0.107,-0.105,-0.103,-0.101,     &
                    -0.100,-0.098,-0.096,-0.094,-0.093,-0.091,-0.089,     &
                    -0.087,-0.085,-0.083,-0.081,-0.080,-0.078,-0.076,     &
                    -0.074,-0.072,-0.070,-0.068,-0.066,-0.064,-0.062,     &
                    -0.060,-0.058,-0.056,-0.054,-0.052,-0.050,-0.048,     &
                    -0.046,-0.044,-0.042,-0.039,-0.037,-0.035,-0.033,     &
                    -0.031,-0.029,-0.027,-0.025,-0.023,-0.021,-0.018,     &
                    -0.016,-0.014,-0.012,-0.010,-0.008,-0.006,-0.004,     &
                    -0.002,0.001,0.003,0.005,0.007,0.009,0.011,0.013,     &
                    0.015,0.017,0.019,0.022,0.024,0.026,0.028,0.030,0.032,&
                    0.034,0.036,0.038,0.040,0.042,0.045,0.047,0.049,0.051,&
                    0.053,0.055,0.057,0.059,0.061,0.063,0.065,0.067,0.069,&
                    0.071,0.073,0.075,0.078,0.080,0.082,0.084,0.086,0.088,&
                    0.090,0.092,0.094,0.096,0.098,0.100,0.102,0.104,0.106,&
                    0.108,0.110,0.112,0.114,0.116,0.118,0.120,0.122,0.124,&
                    0.126,0.128,0.130,0.132,0.134,0.136,0.138,0.140,0.142,&
                    0.143,0.145,0.147,0.149,0.151,0.153,0.155,0.157,0.159,&
                    0.161,0.163,0.165,0.167,0.169,0.171,0.172,0.174,0.176,&
                    0.178,0.180,0.182,0.184,0.186,0.188,0.190,0.191,0.193,&
                    0.195,0.197,0.199,0.201,0.203,0.205,0.206,0.208,0.210,&
                    0.212,0.214,0.216,0.218,0.219,0.221,0.223,0.225,0.227,&
                    0.229,0.230,0.232,0.234,0.236,0.238,0.239,0.241,0.243,&
                    0.245,0.247,0.248,0.250,0.252,0.254,0.256,0.257,0.259,&
                    0.261,0.263,0.265,0.266,0.268,0.270,0.272,0.273,0.275,&
                    0.277,0.279,0.280,0.282,0.284,0.286,0.287,0.289,0.291,&
                    0.292,0.294,0.296,0.298,0.299,0.301,0.303,0.305,0.306,&
                    0.308,0.310,0.311,0.313,0.315,0.316,0.318,0.320,0.321,&
                    0.323,0.325,0.326,0.328,0.330,0.331,0.333,0.335,0.336,&
                    0.338,0.340,0.341,0.343,0.345,0.346,0.348,0.350,0.351,&
                    0.353,0.355,0.356,0.358,0.359,0.361,0.363,0.364,0.366,&
                    0.367,0.369,0.371,0.372,0.374,0.375,0.377,0.379,0.380,&
                    0.382,0.383,0.385,0.387,0.388,0.390,0.391,0.393,0.394,&
                    0.396,0.398,0.399,0.401,0.402,0.404,0.405,0.407,0.408,&
                    0.410,0.412,0.413,0.415,0.416,0.418,0.419,0.421,0.422,&
                    0.424,0.425,0.427,0.428,0.430,0.431,0.433,0.434,0.436,&
                    0.437,0.439,0.440,0.442,0.443,0.445,0.446,0.448,0.449,&
                    0.451,0.452,0.454,0.455,0.457,0.458,0.460,0.461,0.463,&
                    0.464,0.465,0.467,0.468,0.470,0.471,0.473,0.488,0.502,&
                    0.516,0.530,0.544,0.557,0.570,0.583,0.596,0.608,0.621,&
                    0.633,0.645,0.657,0.669,0.680,0.692,0.703,0.714,0.725,&
                    0.736,0.747,0.758,0.768,0.778,0.789,0.799,0.809,0.819,&
                    0.828,0.838,0.847,0.857,0.866,0.875,0.884,0.893,0.902,&
                    0.911,0.920,0.928,0.937,0.945,0.954,0.962,0.970,0.978,&
                    0.986,0.994,1.002,1.010,1.017,1.025,1.032,1.040,1.047,&
                    1.054,1.062,1.069,1.076,1.083,1.090,1.097,1.103,1.110,&
                    1.117,1.124,1.130,1.137,1.143,1.149,1.156,1.162,1.168,&
                    1.174,1.181,1.187,1.193,1.199,1.205,1.210,1.216,1.222,&
                    1.228,1.233,1.239,1.245,1.250,1.255,1.261,1.266,1.272,&
                    1.277,1.282,1.287,1.293,1.298,1.303,1.308,1.313,1.318,&
                    1.323,1.328,1.333,1.337,1.342,1.347,1.352,1.356,1.361,&
                    1.366,1.370,1.375,1.379,1.384,1.388,1.392,1.397,1.401,&
                    1.406,1.410,1.414,1.418,1.422,1.427,1.431,1.435,1.439,&
                    1.443,1.447,1.451,1.455,1.459,1.463,1.467,1.470,1.474,&
                    1.478,1.482,1.486,1.489,1.493,1.497,1.500,1.504,1.508,&
                    1.511,1.515,1.518,1.522,1.525,1.529,1.532,1.536,1.539,&
                    1.542,1.546,1.549,1.552,1.556,1.559/)
!
! *** na2so4
!
    bnc02m(1:561) = (/-0.096,-0.208,-0.264,-0.304,-0.335,-0.361,-0.383,   &
                    -0.403,-0.420,-0.436,-0.450,-0.464,-0.476,-0.487,     &
                    -0.498,-0.508,-0.517,-0.526,-0.535,-0.543,-0.551,     &
                    -0.558,-0.566,-0.572,-0.579,-0.586,-0.592,-0.598,     &
                    -0.603,-0.609,-0.614,-0.620,-0.625,-0.630,-0.635,     &
                    -0.639,-0.644,-0.649,-0.653,-0.657,-0.661,-0.666,     &
                    -0.670,-0.674,-0.677,-0.681,-0.685,-0.689,-0.692,     &
                    -0.696,-0.699,-0.702,-0.706,-0.709,-0.712,-0.715,     &
                    -0.718,-0.721,-0.725,-0.727,-0.730,-0.733,-0.736,     &
                    -0.739,-0.742,-0.744,-0.747,-0.750,-0.752,-0.755,     &
                    -0.757,-0.760,-0.763,-0.765,-0.767,-0.770,-0.772,     &
                    -0.775,-0.777,-0.779,-0.782,-0.784,-0.786,-0.788,     &
                    -0.791,-0.793,-0.795,-0.797,-0.799,-0.801,-0.803,     &
                    -0.806,-0.808,-0.810,-0.812,-0.814,-0.816,-0.818,     &
                    -0.820,-0.822,-0.824,-0.826,-0.827,-0.829,-0.831,     &
                    -0.833,-0.835,-0.837,-0.839,-0.841,-0.842,-0.844,     &
                    -0.846,-0.848,-0.850,-0.851,-0.853,-0.855,-0.857,     &
                    -0.858,-0.860,-0.862,-0.863,-0.865,-0.867,-0.868,     &
                    -0.870,-0.872,-0.873,-0.875,-0.877,-0.878,-0.880,     &
                    -0.881,-0.883,-0.885,-0.886,-0.888,-0.889,-0.891,     &
                    -0.892,-0.894,-0.895,-0.897,-0.898,-0.900,-0.901,     &
                    -0.903,-0.904,-0.906,-0.907,-0.909,-0.910,-0.912,     &
                    -0.913,-0.915,-0.916,-0.917,-0.919,-0.920,-0.922,     &
                    -0.923,-0.924,-0.926,-0.927,-0.928,-0.930,-0.931,     &
                    -0.933,-0.934,-0.935,-0.937,-0.938,-0.939,-0.941,     &
                    -0.942,-0.943,-0.944,-0.946,-0.947,-0.948,-0.950,     &
                    -0.951,-0.952,-0.953,-0.955,-0.956,-0.957,-0.958,     &
                    -0.960,-0.961,-0.962,-0.963,-0.965,-0.966,-0.967,     &
                    -0.968,-0.969,-0.971,-0.972,-0.973,-0.974,-0.975,     &
                    -0.977,-0.978,-0.979,-0.980,-0.981,-0.982,-0.984,     &
                    -0.985,-0.986,-0.987,-0.988,-0.989,-0.990,-0.992,     &
                    -0.993,-0.994,-0.995,-0.996,-0.997,-0.998,-0.999,     &
                    -1.001,-1.002,-1.003,-1.004,-1.005,-1.006,-1.007,     &
                    -1.008,-1.009,-1.010,-1.011,-1.012,-1.014,-1.015,     &
                    -1.016,-1.017,-1.018,-1.019,-1.020,-1.021,-1.022,     &
                    -1.023,-1.024,-1.025,-1.026,-1.027,-1.028,-1.029,     &
                    -1.030,-1.031,-1.032,-1.033,-1.034,-1.035,-1.036,     &
                    -1.037,-1.038,-1.039,-1.040,-1.041,-1.042,-1.043,     &
                    -1.044,-1.045,-1.046,-1.047,-1.048,-1.049,-1.050,     &
                    -1.051,-1.052,-1.053,-1.054,-1.055,-1.056,-1.057,     &
                    -1.058,-1.059,-1.060,-1.061,-1.062,-1.063,-1.064,     &
                    -1.065,-1.066,-1.066,-1.067,-1.068,-1.069,-1.070,     &
                    -1.071,-1.072,-1.073,-1.074,-1.075,-1.076,-1.077,     &
                    -1.078,-1.079,-1.079,-1.080,-1.081,-1.082,-1.083,     &
                    -1.084,-1.085,-1.086,-1.087,-1.088,-1.088,-1.089,     &
                    -1.090,-1.091,-1.092,-1.093,-1.094,-1.095,-1.096,     &
                    -1.096,-1.097,-1.098,-1.099,-1.100,-1.101,-1.102,     &
                    -1.102,-1.103,-1.104,-1.105,-1.106,-1.107,-1.108,     &
                    -1.109,-1.109,-1.110,-1.111,-1.112,-1.113,-1.114,     &
                    -1.114,-1.115,-1.116,-1.117,-1.118,-1.119,-1.120,     &
                    -1.120,-1.121,-1.122,-1.123,-1.124,-1.124,-1.125,     &
                    -1.126,-1.127,-1.128,-1.129,-1.129,-1.130,-1.131,     &
                    -1.132,-1.133,-1.134,-1.134,-1.135,-1.136,-1.137,     &
                    -1.138,-1.138,-1.139,-1.140,-1.141,-1.142,-1.142,     &
                    -1.143,-1.144,-1.145,-1.146,-1.146,-1.147,-1.148,     &
                    -1.149,-1.150,-1.150,-1.151,-1.152,-1.153,-1.153,     &
                    -1.154,-1.155,-1.156,-1.157,-1.157,-1.158,-1.159,     &
                    -1.160,-1.168,-1.175,-1.183,-1.190,-1.198,-1.205,     &
                    -1.212,-1.219,-1.226,-1.233,-1.239,-1.246,-1.253,     &
                    -1.259,-1.266,-1.272,-1.279,-1.285,-1.291,-1.298,     &
                    -1.304,-1.310,-1.316,-1.322,-1.328,-1.334,-1.340,     &
                    -1.346,-1.352,-1.357,-1.363,-1.369,-1.374,-1.380,     &
                    -1.386,-1.391,-1.397,-1.402,-1.408,-1.413,-1.419,     &
                    -1.424,-1.429,-1.435,-1.440,-1.445,-1.450,-1.456,     &
                    -1.461,-1.466,-1.471,-1.476,-1.481,-1.486,-1.491,     &
                    -1.496,-1.501,-1.506,-1.511,-1.516,-1.521,-1.526,     &
                    -1.531,-1.536,-1.541,-1.545,-1.550,-1.555,-1.560,     &
                    -1.564,-1.569,-1.574,-1.579,-1.583,-1.588,-1.593,     &
                    -1.597,-1.602,-1.606,-1.611,-1.616,-1.620,-1.625,     &
                    -1.629,-1.634,-1.638,-1.643,-1.647,-1.652,-1.656,     &
                    -1.661,-1.665,-1.670,-1.674,-1.678,-1.683,-1.687,     &
                    -1.692,-1.696,-1.700,-1.705,-1.709,-1.713,-1.718,     &
                    -1.722,-1.726,-1.730,-1.735,-1.739,-1.743,-1.747,     &
                    -1.752,-1.756,-1.760,-1.764,-1.768,-1.773,-1.777,     &
                    -1.781,-1.785,-1.789,-1.793,-1.798,-1.802,-1.806,     &
                    -1.810,-1.814,-1.818,-1.822,-1.826,-1.830,-1.834,     &
                    -1.838,-1.843,-1.847,-1.851,-1.855,-1.859,-1.863,     &
                    -1.867,-1.871,-1.875,-1.879,-1.883,-1.887,-1.891,     &
                    -1.895,-1.899,-1.902,-1.906,-1.910,-1.914,-1.918,     &
                    -1.922,-1.926,-1.930,-1.934,-1.938,-1.942,-1.946,     &
                    -1.949/)
!
! *** nano3
!
    bnc03m(1:561) = (/-0.048,-0.105,-0.133,-0.154,-0.170,-0.183,-0.195,   &
                    -0.205,-0.214,-0.223,-0.230,-0.237,-0.244,-0.250,     &
                    -0.256,-0.261,-0.266,-0.271,-0.276,-0.280,-0.284,     &
                    -0.289,-0.292,-0.296,-0.300,-0.304,-0.307,-0.310,     &
                    -0.314,-0.317,-0.320,-0.323,-0.326,-0.328,-0.331,     &
                    -0.334,-0.337,-0.339,-0.342,-0.344,-0.347,-0.349,     &
                    -0.351,-0.353,-0.356,-0.358,-0.360,-0.362,-0.364,     &
                    -0.366,-0.368,-0.370,-0.372,-0.374,-0.376,-0.378,     &
                    -0.379,-0.381,-0.383,-0.385,-0.386,-0.388,-0.390,     &
                    -0.391,-0.393,-0.395,-0.396,-0.398,-0.399,-0.401,     &
                    -0.402,-0.404,-0.405,-0.407,-0.408,-0.410,-0.411,     &
                    -0.413,-0.414,-0.415,-0.417,-0.418,-0.420,-0.421,     &
                    -0.422,-0.424,-0.425,-0.426,-0.427,-0.429,-0.430,     &
                    -0.431,-0.433,-0.434,-0.435,-0.436,-0.438,-0.439,     &
                    -0.440,-0.441,-0.442,-0.444,-0.445,-0.446,-0.447,     &
                    -0.448,-0.449,-0.451,-0.452,-0.453,-0.454,-0.455,     &
                    -0.456,-0.457,-0.459,-0.460,-0.461,-0.462,-0.463,     &
                    -0.464,-0.465,-0.466,-0.467,-0.468,-0.469,-0.470,     &
                    -0.472,-0.473,-0.474,-0.475,-0.476,-0.477,-0.478,     &
                    -0.479,-0.480,-0.481,-0.482,-0.483,-0.484,-0.485,     &
                    -0.486,-0.487,-0.488,-0.489,-0.490,-0.490,-0.491,     &
                    -0.492,-0.493,-0.494,-0.495,-0.496,-0.497,-0.498,     &
                    -0.499,-0.500,-0.501,-0.502,-0.503,-0.503,-0.504,     &
                    -0.505,-0.506,-0.507,-0.508,-0.509,-0.510,-0.510,     &
                    -0.511,-0.512,-0.513,-0.514,-0.515,-0.516,-0.516,     &
                    -0.517,-0.518,-0.519,-0.520,-0.521,-0.521,-0.522,     &
                    -0.523,-0.524,-0.525,-0.526,-0.526,-0.527,-0.528,     &
                    -0.529,-0.530,-0.530,-0.531,-0.532,-0.533,-0.534,     &
                    -0.534,-0.535,-0.536,-0.537,-0.537,-0.538,-0.539,     &
                    -0.540,-0.540,-0.541,-0.542,-0.543,-0.543,-0.544,     &
                    -0.545,-0.546,-0.546,-0.547,-0.548,-0.549,-0.549,     &
                    -0.550,-0.551,-0.552,-0.552,-0.553,-0.554,-0.554,     &
                    -0.555,-0.556,-0.557,-0.557,-0.558,-0.559,-0.559,     &
                    -0.560,-0.561,-0.562,-0.562,-0.563,-0.564,-0.564,     &
                    -0.565,-0.566,-0.566,-0.567,-0.568,-0.568,-0.569,     &
                    -0.570,-0.570,-0.571,-0.572,-0.572,-0.573,-0.574,     &
                    -0.574,-0.575,-0.576,-0.576,-0.577,-0.578,-0.578,     &
                    -0.579,-0.580,-0.580,-0.581,-0.582,-0.582,-0.583,     &
                    -0.584,-0.584,-0.585,-0.585,-0.586,-0.587,-0.587,     &
                    -0.588,-0.589,-0.589,-0.590,-0.591,-0.591,-0.592,     &
                    -0.592,-0.593,-0.594,-0.594,-0.595,-0.595,-0.596,     &
                    -0.597,-0.597,-0.598,-0.599,-0.599,-0.600,-0.600,     &
                    -0.601,-0.602,-0.602,-0.603,-0.603,-0.604,-0.604,     &
                    -0.605,-0.606,-0.606,-0.607,-0.607,-0.608,-0.609,     &
                    -0.609,-0.610,-0.610,-0.611,-0.612,-0.612,-0.613,     &
                    -0.613,-0.614,-0.614,-0.615,-0.616,-0.616,-0.617,     &
                    -0.617,-0.618,-0.618,-0.619,-0.619,-0.620,-0.621,     &
                    -0.621,-0.622,-0.622,-0.623,-0.623,-0.624,-0.624,     &
                    -0.625,-0.626,-0.626,-0.627,-0.627,-0.628,-0.628,     &
                    -0.629,-0.629,-0.630,-0.630,-0.631,-0.632,-0.632,     &
                    -0.633,-0.633,-0.634,-0.634,-0.635,-0.635,-0.636,     &
                    -0.636,-0.637,-0.637,-0.638,-0.638,-0.639,-0.640,     &
                    -0.640,-0.641,-0.641,-0.642,-0.642,-0.643,-0.643,     &
                    -0.644,-0.644,-0.645,-0.645,-0.646,-0.646,-0.647,     &
                    -0.647,-0.648,-0.648,-0.649,-0.649,-0.650,-0.650,     &
                    -0.651,-0.651,-0.652,-0.652,-0.653,-0.653,-0.654,     &
                    -0.654,-0.655,-0.655,-0.656,-0.656,-0.657,-0.657,     &
                    -0.658,-0.663,-0.668,-0.673,-0.677,-0.682,-0.687,     &
                    -0.691,-0.696,-0.700,-0.704,-0.709,-0.713,-0.717,     &
                    -0.721,-0.725,-0.729,-0.733,-0.737,-0.741,-0.745,     &
                    -0.749,-0.753,-0.757,-0.760,-0.764,-0.768,-0.771,     &
                    -0.775,-0.779,-0.782,-0.786,-0.789,-0.793,-0.796,     &
                    -0.799,-0.803,-0.806,-0.810,-0.813,-0.816,-0.819,     &
                    -0.823,-0.826,-0.829,-0.832,-0.836,-0.839,-0.842,     &
                    -0.845,-0.848,-0.851,-0.854,-0.857,-0.860,-0.863,     &
                    -0.866,-0.869,-0.872,-0.875,-0.878,-0.881,-0.884,     &
                    -0.887,-0.890,-0.893,-0.895,-0.898,-0.901,-0.904,     &
                    -0.907,-0.909,-0.912,-0.915,-0.918,-0.920,-0.923,     &
                    -0.926,-0.929,-0.931,-0.934,-0.937,-0.939,-0.942,     &
                    -0.945,-0.947,-0.950,-0.952,-0.955,-0.958,-0.960,     &
                    -0.963,-0.965,-0.968,-0.971,-0.973,-0.976,-0.978,     &
                    -0.981,-0.983,-0.986,-0.988,-0.991,-0.993,-0.996,     &
                    -0.998,-1.000,-1.003,-1.005,-1.008,-1.010,-1.013,     &
                    -1.015,-1.017,-1.020,-1.022,-1.025,-1.027,-1.029,     &
                    -1.032,-1.034,-1.036,-1.039,-1.041,-1.043,-1.046,     &
                    -1.048,-1.050,-1.053,-1.055,-1.057,-1.060,-1.062,     &
                    -1.064,-1.067,-1.069,-1.071,-1.073,-1.076,-1.078,     &
                    -1.080,-1.082,-1.085,-1.087,-1.089,-1.091,-1.093,     &
                    -1.096,-1.098,-1.100,-1.102,-1.105,-1.107,-1.109,     &
                    -1.111,-1.113,-1.115,-1.118,-1.120,-1.122,-1.124,     &
                    -1.126/)
!
! *** (nh4)2so4
!
    bnc04m(1:561) = (/-0.096,-0.208,-0.265,-0.305,-0.337,-0.363,-0.385,   &
                    -0.405,-0.423,-0.439,-0.453,-0.467,-0.479,-0.491,     &
                    -0.502,-0.512,-0.522,-0.531,-0.540,-0.548,-0.556,     &
                    -0.564,-0.571,-0.578,-0.585,-0.592,-0.598,-0.605,     &
                    -0.610,-0.616,-0.622,-0.627,-0.633,-0.638,-0.643,     &
                    -0.648,-0.653,-0.657,-0.662,-0.666,-0.671,-0.675,     &
                    -0.679,-0.683,-0.688,-0.691,-0.695,-0.699,-0.703,     &
                    -0.707,-0.710,-0.714,-0.717,-0.721,-0.724,-0.727,     &
                    -0.731,-0.734,-0.737,-0.740,-0.743,-0.746,-0.749,     &
                    -0.752,-0.755,-0.758,-0.761,-0.763,-0.766,-0.769,     &
                    -0.772,-0.774,-0.777,-0.780,-0.782,-0.785,-0.787,     &
                    -0.790,-0.792,-0.795,-0.797,-0.799,-0.802,-0.804,     &
                    -0.807,-0.809,-0.811,-0.814,-0.816,-0.818,-0.820,     &
                    -0.823,-0.825,-0.827,-0.829,-0.831,-0.833,-0.836,     &
                    -0.838,-0.840,-0.842,-0.844,-0.846,-0.848,-0.850,     &
                    -0.852,-0.854,-0.856,-0.858,-0.860,-0.862,-0.864,     &
                    -0.866,-0.868,-0.870,-0.872,-0.874,-0.875,-0.877,     &
                    -0.879,-0.881,-0.883,-0.885,-0.887,-0.888,-0.890,     &
                    -0.892,-0.894,-0.895,-0.897,-0.899,-0.901,-0.902,     &
                    -0.904,-0.906,-0.908,-0.909,-0.911,-0.913,-0.914,     &
                    -0.916,-0.918,-0.919,-0.921,-0.923,-0.924,-0.926,     &
                    -0.927,-0.929,-0.931,-0.932,-0.934,-0.935,-0.937,     &
                    -0.938,-0.940,-0.942,-0.943,-0.945,-0.946,-0.948,     &
                    -0.949,-0.951,-0.952,-0.954,-0.955,-0.957,-0.958,     &
                    -0.960,-0.961,-0.962,-0.964,-0.965,-0.967,-0.968,     &
                    -0.970,-0.971,-0.972,-0.974,-0.975,-0.977,-0.978,     &
                    -0.979,-0.981,-0.982,-0.984,-0.985,-0.986,-0.988,     &
                    -0.989,-0.990,-0.992,-0.993,-0.994,-0.996,-0.997,     &
                    -0.998,-1.000,-1.001,-1.002,-1.004,-1.005,-1.006,     &
                    -1.007,-1.009,-1.010,-1.011,-1.013,-1.014,-1.015,     &
                    -1.016,-1.018,-1.019,-1.020,-1.021,-1.023,-1.024,     &
                    -1.025,-1.026,-1.027,-1.029,-1.030,-1.031,-1.032,     &
                    -1.034,-1.035,-1.036,-1.037,-1.038,-1.039,-1.041,     &
                    -1.042,-1.043,-1.044,-1.045,-1.047,-1.048,-1.049,     &
                    -1.050,-1.051,-1.052,-1.053,-1.055,-1.056,-1.057,     &
                    -1.058,-1.059,-1.060,-1.061,-1.063,-1.064,-1.065,     &
                    -1.066,-1.067,-1.068,-1.069,-1.070,-1.071,-1.073,     &
                    -1.074,-1.075,-1.076,-1.077,-1.078,-1.079,-1.080,     &
                    -1.081,-1.082,-1.083,-1.084,-1.086,-1.087,-1.088,     &
                    -1.089,-1.090,-1.091,-1.092,-1.093,-1.094,-1.095,     &
                    -1.096,-1.097,-1.098,-1.099,-1.100,-1.101,-1.102,     &
                    -1.103,-1.104,-1.105,-1.106,-1.107,-1.108,-1.109,     &
                    -1.110,-1.111,-1.112,-1.114,-1.115,-1.116,-1.117,     &
                    -1.118,-1.119,-1.119,-1.120,-1.121,-1.122,-1.123,     &
                    -1.124,-1.125,-1.126,-1.127,-1.128,-1.129,-1.130,     &
                    -1.131,-1.132,-1.133,-1.134,-1.135,-1.136,-1.137,     &
                    -1.138,-1.139,-1.140,-1.141,-1.142,-1.143,-1.144,     &
                    -1.145,-1.146,-1.147,-1.147,-1.148,-1.149,-1.150,     &
                    -1.151,-1.152,-1.153,-1.154,-1.155,-1.156,-1.157,     &
                    -1.158,-1.159,-1.160,-1.160,-1.161,-1.162,-1.163,     &
                    -1.164,-1.165,-1.166,-1.167,-1.168,-1.169,-1.169,     &
                    -1.170,-1.171,-1.172,-1.173,-1.174,-1.175,-1.176,     &
                    -1.177,-1.178,-1.178,-1.179,-1.180,-1.181,-1.182,     &
                    -1.183,-1.184,-1.185,-1.185,-1.186,-1.187,-1.188,     &
                    -1.189,-1.190,-1.191,-1.191,-1.192,-1.193,-1.194,     &
                    -1.195,-1.196,-1.197,-1.198,-1.198,-1.199,-1.200,     &
                    -1.201,-1.202,-1.203,-1.203,-1.204,-1.205,-1.206,     &
                    -1.207,-1.216,-1.224,-1.232,-1.240,-1.248,-1.256,     &
                    -1.263,-1.271,-1.279,-1.286,-1.293,-1.301,-1.308,     &
                    -1.315,-1.322,-1.329,-1.336,-1.343,-1.349,-1.356,     &
                    -1.363,-1.369,-1.376,-1.382,-1.389,-1.395,-1.402,     &
                    -1.408,-1.414,-1.420,-1.427,-1.433,-1.439,-1.445,     &
                    -1.451,-1.457,-1.463,-1.468,-1.474,-1.480,-1.486,     &
                    -1.492,-1.497,-1.503,-1.509,-1.514,-1.520,-1.525,     &
                    -1.531,-1.536,-1.542,-1.547,-1.553,-1.558,-1.563,     &
                    -1.569,-1.574,-1.579,-1.585,-1.590,-1.595,-1.600,     &
                    -1.605,-1.611,-1.616,-1.621,-1.626,-1.631,-1.636,     &
                    -1.641,-1.646,-1.651,-1.656,-1.661,-1.666,-1.671,     &
                    -1.676,-1.681,-1.685,-1.690,-1.695,-1.700,-1.705,     &
                    -1.710,-1.714,-1.719,-1.724,-1.729,-1.733,-1.738,     &
                    -1.743,-1.747,-1.752,-1.757,-1.761,-1.766,-1.770,     &
                    -1.775,-1.780,-1.784,-1.789,-1.793,-1.798,-1.802,     &
                    -1.807,-1.811,-1.816,-1.820,-1.825,-1.829,-1.834,     &
                    -1.838,-1.842,-1.847,-1.851,-1.856,-1.860,-1.864,     &
                    -1.869,-1.873,-1.877,-1.882,-1.886,-1.890,-1.895,     &
                    -1.899,-1.903,-1.908,-1.912,-1.916,-1.920,-1.925,     &
                    -1.929,-1.933,-1.937,-1.941,-1.946,-1.950,-1.954,     &
                    -1.958,-1.962,-1.966,-1.971,-1.975,-1.979,-1.983,     &
                    -1.987,-1.991,-1.995,-2.000,-2.004,-2.008,-2.012,     &
                    -2.016,-2.020,-2.024,-2.028,-2.032,-2.036,-2.040,     &
                    -2.044/)
!
! *** nh4no3
!
    bnc05m(1:561) = (/-0.048,-0.108,-0.138,-0.161,-0.179,-0.194,-0.208,   &
                    -0.220,-0.231,-0.241,-0.250,-0.259,-0.267,-0.275,     &
                    -0.282,-0.289,-0.296,-0.302,-0.309,-0.315,-0.321,     &
                    -0.326,-0.332,-0.337,-0.342,-0.347,-0.352,-0.357,     &
                    -0.361,-0.366,-0.370,-0.375,-0.379,-0.383,-0.387,     &
                    -0.391,-0.395,-0.399,-0.403,-0.406,-0.410,-0.414,     &
                    -0.417,-0.421,-0.424,-0.427,-0.431,-0.434,-0.437,     &
                    -0.440,-0.443,-0.446,-0.449,-0.452,-0.455,-0.458,     &
                    -0.461,-0.464,-0.466,-0.469,-0.472,-0.475,-0.477,     &
                    -0.480,-0.482,-0.485,-0.487,-0.490,-0.492,-0.495,     &
                    -0.497,-0.500,-0.502,-0.505,-0.507,-0.509,-0.512,     &
                    -0.514,-0.516,-0.518,-0.521,-0.523,-0.525,-0.527,     &
                    -0.530,-0.532,-0.534,-0.536,-0.538,-0.541,-0.543,     &
                    -0.545,-0.547,-0.549,-0.551,-0.553,-0.555,-0.558,     &
                    -0.560,-0.562,-0.564,-0.566,-0.568,-0.570,-0.572,     &
                    -0.574,-0.576,-0.578,-0.580,-0.582,-0.584,-0.586,     &
                    -0.588,-0.590,-0.592,-0.594,-0.596,-0.598,-0.600,     &
                    -0.602,-0.604,-0.606,-0.608,-0.609,-0.611,-0.613,     &
                    -0.615,-0.617,-0.619,-0.621,-0.622,-0.624,-0.626,     &
                    -0.628,-0.630,-0.631,-0.633,-0.635,-0.637,-0.639,     &
                    -0.640,-0.642,-0.644,-0.646,-0.647,-0.649,-0.651,     &
                    -0.652,-0.654,-0.656,-0.657,-0.659,-0.661,-0.662,     &
                    -0.664,-0.666,-0.667,-0.669,-0.671,-0.672,-0.674,     &
                    -0.675,-0.677,-0.679,-0.680,-0.682,-0.683,-0.685,     &
                    -0.686,-0.688,-0.690,-0.691,-0.693,-0.694,-0.696,     &
                    -0.697,-0.699,-0.700,-0.702,-0.703,-0.705,-0.706,     &
                    -0.708,-0.709,-0.711,-0.712,-0.713,-0.715,-0.716,     &
                    -0.718,-0.719,-0.721,-0.722,-0.723,-0.725,-0.726,     &
                    -0.728,-0.729,-0.730,-0.732,-0.733,-0.735,-0.736,     &
                    -0.737,-0.739,-0.740,-0.741,-0.743,-0.744,-0.745,     &
                    -0.747,-0.748,-0.749,-0.751,-0.752,-0.753,-0.755,     &
                    -0.756,-0.757,-0.759,-0.760,-0.761,-0.762,-0.764,     &
                    -0.765,-0.766,-0.768,-0.769,-0.770,-0.771,-0.773,     &
                    -0.774,-0.775,-0.776,-0.778,-0.779,-0.780,-0.781,     &
                    -0.782,-0.784,-0.785,-0.786,-0.787,-0.788,-0.790,     &
                    -0.791,-0.792,-0.793,-0.794,-0.796,-0.797,-0.798,     &
                    -0.799,-0.800,-0.801,-0.803,-0.804,-0.805,-0.806,     &
                    -0.807,-0.808,-0.809,-0.811,-0.812,-0.813,-0.814,     &
                    -0.815,-0.816,-0.817,-0.818,-0.820,-0.821,-0.822,     &
                    -0.823,-0.824,-0.825,-0.826,-0.827,-0.828,-0.829,     &
                    -0.830,-0.832,-0.833,-0.834,-0.835,-0.836,-0.837,     &
                    -0.838,-0.839,-0.840,-0.841,-0.842,-0.843,-0.844,     &
                    -0.845,-0.846,-0.847,-0.848,-0.849,-0.850,-0.851,     &
                    -0.852,-0.853,-0.854,-0.855,-0.857,-0.858,-0.859,     &
                    -0.860,-0.861,-0.862,-0.863,-0.863,-0.864,-0.865,     &
                    -0.866,-0.867,-0.868,-0.869,-0.870,-0.871,-0.872,     &
                    -0.873,-0.874,-0.875,-0.876,-0.877,-0.878,-0.879,     &
                    -0.880,-0.881,-0.882,-0.883,-0.884,-0.885,-0.886,     &
                    -0.887,-0.887,-0.888,-0.889,-0.890,-0.891,-0.892,     &
                    -0.893,-0.894,-0.895,-0.896,-0.897,-0.898,-0.898,     &
                    -0.899,-0.900,-0.901,-0.902,-0.903,-0.904,-0.905,     &
                    -0.906,-0.907,-0.907,-0.908,-0.909,-0.910,-0.911,     &
                    -0.912,-0.913,-0.914,-0.914,-0.915,-0.916,-0.917,     &
                    -0.918,-0.919,-0.920,-0.920,-0.921,-0.922,-0.923,     &
                    -0.924,-0.925,-0.926,-0.926,-0.927,-0.928,-0.929,     &
                    -0.930,-0.931,-0.931,-0.932,-0.933,-0.934,-0.935,     &
                    -0.936,-0.936,-0.937,-0.938,-0.939,-0.940,-0.940,     &
                    -0.941,-0.950,-0.958,-0.965,-0.973,-0.980,-0.988,     &
                    -0.995,-1.002,-1.009,-1.015,-1.022,-1.029,-1.035,     &
                    -1.041,-1.047,-1.054,-1.060,-1.066,-1.071,-1.077,     &
                    -1.083,-1.088,-1.094,-1.099,-1.105,-1.110,-1.115,     &
                    -1.120,-1.125,-1.130,-1.135,-1.140,-1.145,-1.150,     &
                    -1.155,-1.159,-1.164,-1.168,-1.173,-1.177,-1.182,     &
                    -1.186,-1.191,-1.195,-1.199,-1.203,-1.207,-1.212,     &
                    -1.216,-1.220,-1.224,-1.228,-1.232,-1.235,-1.239,     &
                    -1.243,-1.247,-1.251,-1.254,-1.258,-1.262,-1.265,     &
                    -1.269,-1.273,-1.276,-1.280,-1.283,-1.287,-1.290,     &
                    -1.293,-1.297,-1.300,-1.304,-1.307,-1.310,-1.313,     &
                    -1.317,-1.320,-1.323,-1.326,-1.330,-1.333,-1.336,     &
                    -1.339,-1.342,-1.345,-1.348,-1.351,-1.354,-1.357,     &
                    -1.360,-1.363,-1.366,-1.369,-1.372,-1.375,-1.378,     &
                    -1.381,-1.383,-1.386,-1.389,-1.392,-1.395,-1.397,     &
                    -1.400,-1.403,-1.406,-1.408,-1.411,-1.414,-1.417,     &
                    -1.419,-1.422,-1.425,-1.427,-1.430,-1.432,-1.435,     &
                    -1.438,-1.440,-1.443,-1.445,-1.448,-1.450,-1.453,     &
                    -1.456,-1.458,-1.461,-1.463,-1.466,-1.468,-1.470,     &
                    -1.473,-1.475,-1.478,-1.480,-1.483,-1.485,-1.487,     &
                    -1.490,-1.492,-1.495,-1.497,-1.499,-1.502,-1.504,     &
                    -1.506,-1.509,-1.511,-1.513,-1.516,-1.518,-1.520,     &
                    -1.523,-1.525,-1.527,-1.529,-1.532,-1.534,-1.536,     &
                    -1.538/)
!
! *** nh4cl
!
    bnc06m(1:561) = (/-0.047,-0.101,-0.126,-0.144,-0.157,-0.168,-0.177,   &
                    -0.185,-0.191,-0.197,-0.202,-0.207,-0.211,-0.215,     &
                    -0.218,-0.221,-0.224,-0.227,-0.229,-0.232,-0.234,     &
                    -0.236,-0.238,-0.239,-0.241,-0.242,-0.244,-0.245,     &
                    -0.246,-0.247,-0.249,-0.250,-0.251,-0.251,-0.252,     &
                    -0.253,-0.254,-0.255,-0.255,-0.256,-0.257,-0.257,     &
                    -0.258,-0.258,-0.259,-0.259,-0.260,-0.260,-0.261,     &
                    -0.261,-0.261,-0.262,-0.262,-0.262,-0.263,-0.263,     &
                    -0.263,-0.263,-0.264,-0.264,-0.264,-0.264,-0.265,     &
                    -0.265,-0.265,-0.265,-0.265,-0.265,-0.265,-0.266,     &
                    -0.266,-0.266,-0.266,-0.266,-0.266,-0.266,-0.266,     &
                    -0.266,-0.266,-0.266,-0.266,-0.266,-0.266,-0.266,     &
                    -0.266,-0.266,-0.266,-0.266,-0.266,-0.265,-0.265,     &
                    -0.265,-0.265,-0.265,-0.265,-0.265,-0.264,-0.264,     &
                    -0.264,-0.264,-0.264,-0.263,-0.263,-0.263,-0.263,     &
                    -0.263,-0.262,-0.262,-0.262,-0.262,-0.261,-0.261,     &
                    -0.261,-0.260,-0.260,-0.260,-0.260,-0.259,-0.259,     &
                    -0.259,-0.258,-0.258,-0.258,-0.257,-0.257,-0.257,     &
                    -0.256,-0.256,-0.256,-0.255,-0.255,-0.255,-0.254,     &
                    -0.254,-0.254,-0.253,-0.253,-0.253,-0.252,-0.252,     &
                    -0.252,-0.251,-0.251,-0.251,-0.250,-0.250,-0.249,     &
                    -0.249,-0.249,-0.248,-0.248,-0.248,-0.247,-0.247,     &
                    -0.247,-0.246,-0.246,-0.245,-0.245,-0.245,-0.244,     &
                    -0.244,-0.244,-0.243,-0.243,-0.242,-0.242,-0.242,     &
                    -0.241,-0.241,-0.241,-0.240,-0.240,-0.239,-0.239,     &
                    -0.239,-0.238,-0.238,-0.238,-0.237,-0.237,-0.236,     &
                    -0.236,-0.236,-0.235,-0.235,-0.235,-0.234,-0.234,     &
                    -0.233,-0.233,-0.233,-0.232,-0.232,-0.231,-0.231,     &
                    -0.231,-0.230,-0.230,-0.230,-0.229,-0.229,-0.228,     &
                    -0.228,-0.228,-0.227,-0.227,-0.227,-0.226,-0.226,     &
                    -0.225,-0.225,-0.225,-0.224,-0.224,-0.224,-0.223,     &
                    -0.223,-0.222,-0.222,-0.222,-0.221,-0.221,-0.221,     &
                    -0.220,-0.220,-0.219,-0.219,-0.219,-0.218,-0.218,     &
                    -0.218,-0.217,-0.217,-0.216,-0.216,-0.216,-0.215,     &
                    -0.215,-0.215,-0.214,-0.214,-0.214,-0.213,-0.213,     &
                    -0.212,-0.212,-0.212,-0.211,-0.211,-0.211,-0.210,     &
                    -0.210,-0.210,-0.209,-0.209,-0.208,-0.208,-0.208,     &
                    -0.207,-0.207,-0.207,-0.206,-0.206,-0.206,-0.205,     &
                    -0.205,-0.204,-0.204,-0.204,-0.203,-0.203,-0.203,     &
                    -0.202,-0.202,-0.202,-0.201,-0.201,-0.201,-0.200,     &
                    -0.200,-0.200,-0.199,-0.199,-0.198,-0.198,-0.198,     &
                    -0.197,-0.197,-0.197,-0.196,-0.196,-0.196,-0.195,     &
                    -0.195,-0.195,-0.194,-0.194,-0.194,-0.193,-0.193,     &
                    -0.193,-0.192,-0.192,-0.192,-0.191,-0.191,-0.191,     &
                    -0.190,-0.190,-0.190,-0.189,-0.189,-0.189,-0.188,     &
                    -0.188,-0.188,-0.187,-0.187,-0.187,-0.186,-0.186,     &
                    -0.186,-0.185,-0.185,-0.185,-0.184,-0.184,-0.184,     &
                    -0.183,-0.183,-0.183,-0.182,-0.182,-0.182,-0.181,     &
                    -0.181,-0.181,-0.180,-0.180,-0.180,-0.179,-0.179,     &
                    -0.179,-0.178,-0.178,-0.178,-0.177,-0.177,-0.177,     &
                    -0.176,-0.176,-0.176,-0.176,-0.175,-0.175,-0.175,     &
                    -0.174,-0.174,-0.174,-0.173,-0.173,-0.173,-0.172,     &
                    -0.172,-0.172,-0.171,-0.171,-0.171,-0.171,-0.170,     &
                    -0.170,-0.170,-0.169,-0.169,-0.169,-0.168,-0.168,     &
                    -0.168,-0.167,-0.167,-0.167,-0.167,-0.166,-0.166,     &
                    -0.166,-0.165,-0.165,-0.165,-0.164,-0.164,-0.164,     &
                    -0.164,-0.163,-0.163,-0.163,-0.162,-0.162,-0.162,     &
                    -0.161,-0.158,-0.155,-0.153,-0.150,-0.147,-0.144,     &
                    -0.142,-0.139,-0.137,-0.134,-0.132,-0.129,-0.127,     &
                    -0.125,-0.122,-0.120,-0.118,-0.116,-0.113,-0.111,     &
                    -0.109,-0.107,-0.105,-0.103,-0.101,-0.099,-0.097,     &
                    -0.095,-0.093,-0.092,-0.090,-0.088,-0.086,-0.085,     &
                    -0.083,-0.081,-0.080,-0.078,-0.077,-0.075,-0.073,     &
                    -0.072,-0.071,-0.069,-0.068,-0.066,-0.065,-0.064,     &
                    -0.062,-0.061,-0.060,-0.058,-0.057,-0.056,-0.055,     &
                    -0.053,-0.052,-0.051,-0.050,-0.049,-0.048,-0.047,     &
                    -0.046,-0.044,-0.043,-0.042,-0.041,-0.040,-0.039,     &
                    -0.039,-0.038,-0.037,-0.036,-0.035,-0.034,-0.033,     &
                    -0.032,-0.032,-0.031,-0.030,-0.029,-0.028,-0.028,     &
                    -0.027,-0.026,-0.025,-0.025,-0.024,-0.023,-0.023,     &
                    -0.022,-0.021,-0.021,-0.020,-0.019,-0.019,-0.018,     &
                    -0.018,-0.017,-0.017,-0.016,-0.016,-0.015,-0.015,     &
                    -0.014,-0.014,-0.013,-0.013,-0.012,-0.012,-0.011,     &
                    -0.011,-0.010,-0.010,-0.010,-0.009,-0.009,-0.008,     &
                    -0.008,-0.008,-0.007,-0.007,-0.007,-0.006,-0.006,     &
                    -0.006,-0.005,-0.005,-0.005,-0.005,-0.004,-0.004,     &
                    -0.004,-0.004,-0.003,-0.003,-0.003,-0.003,-0.003,     &
                    -0.002,-0.002,-0.002,-0.002,-0.002,-0.002,-0.001,     &
                    -0.001,-0.001,-0.001,-0.001,-0.001,-0.001,-0.001,     &
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000/)
!
! *** (2h ,so4)
!
    bnc07m(1:561) = (/-0.095,-0.207,-0.263,-0.303,-0.333,-0.359,-0.381,   &
                    -0.400,-0.417,-0.432,-0.446,-0.459,-0.471,-0.482,     &
                    -0.492,-0.502,-0.511,-0.520,-0.528,-0.536,-0.543,     &
                    -0.550,-0.557,-0.564,-0.570,-0.576,-0.582,-0.588,     &
                    -0.593,-0.598,-0.603,-0.608,-0.613,-0.618,-0.622,     &
                    -0.627,-0.631,-0.635,-0.640,-0.644,-0.648,-0.651,     &
                    -0.655,-0.659,-0.662,-0.666,-0.669,-0.673,-0.676,     &
                    -0.679,-0.682,-0.686,-0.689,-0.692,-0.695,-0.698,     &
                    -0.700,-0.703,-0.706,-0.709,-0.712,-0.714,-0.717,     &
                    -0.719,-0.722,-0.724,-0.727,-0.729,-0.732,-0.734,     &
                    -0.736,-0.739,-0.741,-0.743,-0.746,-0.748,-0.750,     &
                    -0.752,-0.754,-0.756,-0.758,-0.761,-0.763,-0.765,     &
                    -0.767,-0.769,-0.771,-0.773,-0.774,-0.776,-0.778,     &
                    -0.780,-0.782,-0.784,-0.786,-0.787,-0.789,-0.791,     &
                    -0.793,-0.795,-0.796,-0.798,-0.800,-0.802,-0.803,     &
                    -0.805,-0.807,-0.808,-0.810,-0.811,-0.813,-0.815,     &
                    -0.816,-0.818,-0.819,-0.821,-0.823,-0.824,-0.826,     &
                    -0.827,-0.829,-0.830,-0.832,-0.833,-0.835,-0.836,     &
                    -0.838,-0.839,-0.841,-0.842,-0.843,-0.845,-0.846,     &
                    -0.848,-0.849,-0.850,-0.852,-0.853,-0.854,-0.856,     &
                    -0.857,-0.858,-0.860,-0.861,-0.862,-0.864,-0.865,     &
                    -0.866,-0.868,-0.869,-0.870,-0.871,-0.873,-0.874,     &
                    -0.875,-0.876,-0.878,-0.879,-0.880,-0.881,-0.883,     &
                    -0.884,-0.885,-0.886,-0.887,-0.889,-0.890,-0.891,     &
                    -0.892,-0.893,-0.894,-0.896,-0.897,-0.898,-0.899,     &
                    -0.900,-0.901,-0.902,-0.904,-0.905,-0.906,-0.907,     &
                    -0.908,-0.909,-0.910,-0.911,-0.912,-0.914,-0.915,     &
                    -0.916,-0.917,-0.918,-0.919,-0.920,-0.921,-0.922,     &
                    -0.923,-0.924,-0.925,-0.926,-0.927,-0.928,-0.929,     &
                    -0.930,-0.931,-0.932,-0.933,-0.934,-0.935,-0.936,     &
                    -0.937,-0.938,-0.939,-0.940,-0.941,-0.942,-0.943,     &
                    -0.944,-0.945,-0.946,-0.947,-0.948,-0.949,-0.950,     &
                    -0.951,-0.952,-0.953,-0.954,-0.955,-0.956,-0.957,     &
                    -0.958,-0.959,-0.960,-0.960,-0.961,-0.962,-0.963,     &
                    -0.964,-0.965,-0.966,-0.967,-0.968,-0.969,-0.970,     &
                    -0.970,-0.971,-0.972,-0.973,-0.974,-0.975,-0.976,     &
                    -0.977,-0.978,-0.978,-0.979,-0.980,-0.981,-0.982,     &
                    -0.983,-0.984,-0.985,-0.985,-0.986,-0.987,-0.988,     &
                    -0.989,-0.990,-0.991,-0.991,-0.992,-0.993,-0.994,     &
                    -0.995,-0.996,-0.996,-0.997,-0.998,-0.999,-1.000,     &
                    -1.001,-1.001,-1.002,-1.003,-1.004,-1.005,-1.005,     &
                    -1.006,-1.007,-1.008,-1.009,-1.010,-1.010,-1.011,     &
                    -1.012,-1.013,-1.014,-1.014,-1.015,-1.016,-1.017,     &
                    -1.017,-1.018,-1.019,-1.020,-1.021,-1.021,-1.022,     &
                    -1.023,-1.024,-1.025,-1.025,-1.026,-1.027,-1.028,     &
                    -1.028,-1.029,-1.030,-1.031,-1.031,-1.032,-1.033,     &
                    -1.034,-1.034,-1.035,-1.036,-1.037,-1.037,-1.038,     &
                    -1.039,-1.040,-1.040,-1.041,-1.042,-1.043,-1.043,     &
                    -1.044,-1.045,-1.046,-1.046,-1.047,-1.048,-1.049,     &
                    -1.049,-1.050,-1.051,-1.051,-1.052,-1.053,-1.054,     &
                    -1.054,-1.055,-1.056,-1.057,-1.057,-1.058,-1.059,     &
                    -1.059,-1.060,-1.061,-1.061,-1.062,-1.063,-1.064,     &
                    -1.064,-1.065,-1.066,-1.066,-1.067,-1.068,-1.069,     &
                    -1.069,-1.070,-1.071,-1.071,-1.072,-1.073,-1.073,     &
                    -1.074,-1.075,-1.075,-1.076,-1.077,-1.078,-1.078,     &
                    -1.079,-1.080,-1.080,-1.081,-1.082,-1.082,-1.083,     &
                    -1.084,-1.084,-1.085,-1.086,-1.086,-1.087,-1.088,     &
                    -1.088,-1.096,-1.102,-1.109,-1.115,-1.121,-1.127,     &
                    -1.134,-1.140,-1.146,-1.152,-1.158,-1.163,-1.169,     &
                    -1.175,-1.181,-1.186,-1.192,-1.198,-1.203,-1.209,     &
                    -1.214,-1.219,-1.225,-1.230,-1.235,-1.241,-1.246,     &
                    -1.251,-1.256,-1.261,-1.266,-1.271,-1.276,-1.281,     &
                    -1.286,-1.291,-1.296,-1.301,-1.306,-1.311,-1.315,     &
                    -1.320,-1.325,-1.330,-1.334,-1.339,-1.344,-1.348,     &
                    -1.353,-1.358,-1.362,-1.367,-1.371,-1.376,-1.380,     &
                    -1.385,-1.389,-1.394,-1.398,-1.403,-1.407,-1.412,     &
                    -1.416,-1.420,-1.425,-1.429,-1.434,-1.438,-1.442,     &
                    -1.447,-1.451,-1.455,-1.459,-1.464,-1.468,-1.472,     &
                    -1.476,-1.480,-1.485,-1.489,-1.493,-1.497,-1.501,     &
                    -1.505,-1.510,-1.514,-1.518,-1.522,-1.526,-1.530,     &
                    -1.534,-1.538,-1.542,-1.546,-1.550,-1.554,-1.558,     &
                    -1.562,-1.566,-1.570,-1.574,-1.578,-1.582,-1.586,     &
                    -1.590,-1.594,-1.598,-1.602,-1.606,-1.610,-1.614,     &
                    -1.618,-1.622,-1.625,-1.629,-1.633,-1.637,-1.641,     &
                    -1.645,-1.649,-1.652,-1.656,-1.660,-1.664,-1.668,     &
                    -1.672,-1.675,-1.679,-1.683,-1.687,-1.690,-1.694,     &
                    -1.698,-1.702,-1.706,-1.709,-1.713,-1.717,-1.721,     &
                    -1.724,-1.728,-1.732,-1.735,-1.739,-1.743,-1.747,     &
                    -1.750,-1.754,-1.758,-1.761,-1.765,-1.769,-1.772,     &
                    -1.776,-1.780,-1.783,-1.787,-1.791,-1.794,-1.798,     &
                    -1.801/)
!
! *** (h ,hso4)
!
    bnc08m(1:561) = (/-0.045,-0.089,-0.107,-0.117,-0.124,-0.128,-0.131,   &
                    -0.132,-0.133,-0.132,-0.132,-0.130,-0.128,-0.126,     &
                    -0.123,-0.120,-0.117,-0.113,-0.109,-0.105,-0.101,     &
                    -0.096,-0.091,-0.086,-0.081,-0.076,-0.070,-0.065,     &
                    -0.059,-0.053,-0.047,-0.040,-0.034,-0.027,-0.021,     &
                    -0.014,-0.007,0.000,0.007,0.014,0.021,0.028,0.036,    &
                    0.043,0.051,0.058,0.066,0.074,0.082,0.089,0.097,0.105,&
                    0.113,0.121,0.130,0.138,0.146,0.154,0.163,0.171,0.179,&
                    0.188,0.196,0.205,0.214,0.222,0.231,0.240,0.248,0.257,&
                    0.266,0.275,0.284,0.293,0.302,0.311,0.320,0.329,0.338,&
                    0.347,0.357,0.366,0.375,0.385,0.394,0.404,0.413,0.423,&
                    0.433,0.442,0.452,0.462,0.472,0.482,0.492,0.502,0.512,&
                    0.522,0.532,0.542,0.552,0.563,0.573,0.583,0.594,0.604,&
                    0.614,0.625,0.635,0.646,0.656,0.667,0.677,0.688,0.698,&
                    0.709,0.719,0.730,0.741,0.751,0.762,0.772,0.783,0.794,&
                    0.804,0.815,0.825,0.836,0.846,0.857,0.867,0.878,0.888,&
                    0.899,0.909,0.920,0.930,0.941,0.951,0.962,0.972,0.982,&
                    0.993,1.003,1.014,1.024,1.034,1.044,1.055,1.065,1.075,&
                    1.085,1.096,1.106,1.116,1.126,1.136,1.146,1.156,1.166,&
                    1.176,1.186,1.196,1.206,1.216,1.226,1.236,1.246,1.256,&
                    1.266,1.276,1.285,1.295,1.305,1.315,1.325,1.334,1.344,&
                    1.354,1.363,1.373,1.383,1.392,1.402,1.411,1.421,1.430,&
                    1.440,1.449,1.459,1.468,1.478,1.487,1.496,1.506,1.515,&
                    1.524,1.534,1.543,1.552,1.561,1.571,1.580,1.589,1.598,&
                    1.607,1.616,1.625,1.634,1.643,1.652,1.661,1.670,1.679,&
                    1.688,1.697,1.706,1.715,1.724,1.733,1.742,1.750,1.759,&
                    1.768,1.777,1.786,1.794,1.803,1.812,1.820,1.829,1.838,&
                    1.846,1.855,1.863,1.872,1.880,1.889,1.897,1.906,1.914,&
                    1.923,1.931,1.940,1.948,1.956,1.965,1.973,1.981,1.990,&
                    1.998,2.006,2.015,2.023,2.031,2.039,2.047,2.056,2.064,&
                    2.072,2.080,2.088,2.096,2.104,2.112,2.120,2.128,2.136,&
                    2.144,2.152,2.160,2.168,2.176,2.184,2.192,2.200,2.207,&
                    2.215,2.223,2.231,2.239,2.246,2.254,2.262,2.270,2.277,&
                    2.285,2.293,2.300,2.308,2.316,2.323,2.331,2.339,2.346,&
                    2.354,2.361,2.369,2.376,2.384,2.391,2.399,2.406,2.414,&
                    2.421,2.428,2.436,2.443,2.451,2.458,2.465,2.473,2.480,&
                    2.487,2.494,2.502,2.509,2.516,2.523,2.531,2.538,2.545,&
                    2.552,2.559,2.566,2.574,2.581,2.588,2.595,2.602,2.609,&
                    2.616,2.623,2.630,2.637,2.644,2.651,2.658,2.665,2.672,&
                    2.679,2.686,2.693,2.700,2.707,2.713,2.720,2.727,2.734,&
                    2.741,2.748,2.754,2.761,2.768,2.775,2.781,2.788,2.795,&
                    2.802,2.808,2.815,2.822,2.828,2.835,2.842,2.848,2.855,&
                    2.862,2.868,2.875,2.881,2.888,2.894,2.901,2.907,2.914,&
                    2.920,2.927,2.933,2.940,2.946,2.953,2.959,2.966,2.972,&
                    2.978,2.985,2.991,2.998,3.004,3.010,3.017,3.023,3.029,&
                    3.036,3.042,3.048,3.054,3.061,3.067,3.134,3.195,3.254,&
                    3.313,3.371,3.428,3.485,3.540,3.595,3.648,3.701,3.754,&
                    3.805,3.856,3.906,3.955,4.004,4.052,4.100,4.147,4.193,&
                    4.239,4.284,4.328,4.372,4.416,4.459,4.501,4.543,4.585,&
                    4.626,4.666,4.707,4.746,4.786,4.824,4.863,4.901,4.938,&
                    4.976,5.013,5.049,5.085,5.121,5.156,5.191,5.226,5.260,&
                    5.294,5.328,5.361,5.395,5.427,5.460,5.492,5.524,5.555,&
                    5.587,5.618,5.649,5.679,5.709,5.739,5.769,5.799,5.828,&
                    5.857,5.886,5.914,5.942,5.971,5.998,6.026,6.053,6.081,&
                    6.108,6.134,6.161,6.187,6.214,6.240,6.265,6.291,6.316,&
                    6.342,6.367,6.392,6.416,6.441,6.465,6.489,6.513,6.537,&
                    6.561,6.584,6.608,6.631,6.654,6.677,6.699,6.722,6.744,&
                    6.767,6.789,6.811,6.833,6.854,6.876,6.897,6.919,6.940,&
                    6.961,6.982,7.003,7.023,7.044,7.064,7.085,7.105,7.125,&
                    7.145,7.165,7.184,7.204,7.223,7.243,7.262,7.281,7.300,&
                    7.319,7.338,7.356,7.375,7.394,7.412,7.430,7.448,7.467,&
                    7.485,7.502,7.520,7.538,7.556,7.573,7.591,7.608,7.625,&
                    7.642,7.659,7.676,7.693,7.710,7.727,7.743,7.760,7.776,&
                    7.793,7.809,7.825,7.841,7.858/)
!
! *** nh4hso4
!
    bnc09m(1:561) = (/-0.047,-0.099,-0.125,-0.142,-0.155,-0.166,-0.175,   &
                    -0.182,-0.189,-0.195,-0.200,-0.204,-0.208,-0.212,     &
                    -0.215,-0.218,-0.221,-0.223,-0.225,-0.227,-0.228,     &
                    -0.230,-0.231,-0.232,-0.233,-0.234,-0.235,-0.235,     &
                    -0.235,-0.236,-0.236,-0.236,-0.236,-0.235,-0.235,     &
                    -0.235,-0.234,-0.233,-0.233,-0.232,-0.231,-0.230,     &
                    -0.229,-0.228,-0.227,-0.226,-0.224,-0.223,-0.222,     &
                    -0.220,-0.219,-0.217,-0.215,-0.214,-0.212,-0.210,     &
                    -0.208,-0.207,-0.205,-0.203,-0.201,-0.199,-0.197,     &
                    -0.195,-0.192,-0.190,-0.188,-0.186,-0.183,-0.181,     &
                    -0.179,-0.176,-0.174,-0.171,-0.169,-0.166,-0.164,     &
                    -0.161,-0.159,-0.156,-0.153,-0.151,-0.148,-0.145,     &
                    -0.143,-0.140,-0.137,-0.134,-0.131,-0.128,-0.125,     &
                    -0.122,-0.120,-0.117,-0.114,-0.110,-0.107,-0.104,     &
                    -0.101,-0.098,-0.095,-0.092,-0.089,-0.085,-0.082,     &
                    -0.079,-0.076,-0.073,-0.069,-0.066,-0.063,-0.060,     &
                    -0.056,-0.053,-0.050,-0.046,-0.043,-0.040,-0.036,     &
                    -0.033,-0.030,-0.026,-0.023,-0.020,-0.016,-0.013,     &
                    -0.010,-0.006,-0.003,0.000,0.004,0.007,0.010,0.014,   &
                    0.017,0.020,0.024,0.027,0.030,0.034,0.037,0.040,0.043,&
                    0.047,0.050,0.053,0.056,0.060,0.063,0.066,0.069,0.073,&
                    0.076,0.079,0.082,0.086,0.089,0.092,0.095,0.098,0.101,&
                    0.105,0.108,0.111,0.114,0.117,0.120,0.123,0.127,0.130,&
                    0.133,0.136,0.139,0.142,0.145,0.148,0.151,0.154,0.157,&
                    0.160,0.163,0.166,0.170,0.173,0.176,0.179,0.182,0.184,&
                    0.187,0.190,0.193,0.196,0.199,0.202,0.205,0.208,0.211,&
                    0.214,0.217,0.220,0.223,0.226,0.228,0.231,0.234,0.237,&
                    0.240,0.243,0.246,0.248,0.251,0.254,0.257,0.260,0.262,&
                    0.265,0.268,0.271,0.274,0.276,0.279,0.282,0.285,0.287,&
                    0.290,0.293,0.296,0.298,0.301,0.304,0.306,0.309,0.312,&
                    0.315,0.317,0.320,0.323,0.325,0.328,0.330,0.333,0.336,&
                    0.338,0.341,0.344,0.346,0.349,0.351,0.354,0.357,0.359,&
                    0.362,0.364,0.367,0.369,0.372,0.375,0.377,0.380,0.382,&
                    0.385,0.387,0.390,0.392,0.395,0.397,0.400,0.402,0.405,&
                    0.407,0.410,0.412,0.415,0.417,0.419,0.422,0.424,0.427,&
                    0.429,0.432,0.434,0.437,0.439,0.441,0.444,0.446,0.449,&
                    0.451,0.453,0.456,0.458,0.460,0.463,0.465,0.467,0.470,&
                    0.472,0.474,0.477,0.479,0.481,0.484,0.486,0.488,0.491,&
                    0.493,0.495,0.498,0.500,0.502,0.504,0.507,0.509,0.511,&
                    0.513,0.516,0.518,0.520,0.522,0.525,0.527,0.529,0.531,&
                    0.534,0.536,0.538,0.540,0.542,0.545,0.547,0.549,0.551,&
                    0.553,0.556,0.558,0.560,0.562,0.564,0.566,0.569,0.571,&
                    0.573,0.575,0.577,0.579,0.581,0.584,0.586,0.588,0.590,&
                    0.592,0.594,0.596,0.598,0.600,0.602,0.605,0.607,0.609,&
                    0.611,0.613,0.615,0.617,0.619,0.621,0.623,0.625,0.627,&
                    0.629,0.631,0.633,0.635,0.637,0.639,0.642,0.644,0.646,&
                    0.648,0.650,0.652,0.654,0.656,0.658,0.660,0.662,0.663,&
                    0.665,0.667,0.669,0.671,0.673,0.675,0.677,0.679,0.681,&
                    0.683,0.685,0.687,0.689,0.691,0.712,0.730,0.749,0.767,&
                    0.785,0.803,0.820,0.837,0.854,0.871,0.887,0.903,0.919,&
                    0.935,0.950,0.965,0.980,0.995,1.010,1.024,1.039,1.053,&
                    1.067,1.080,1.094,1.107,1.120,1.133,1.146,1.159,1.172,&
                    1.184,1.196,1.209,1.221,1.232,1.244,1.256,1.267,1.279,&
                    1.290,1.301,1.312,1.323,1.334,1.344,1.355,1.365,1.376,&
                    1.386,1.396,1.406,1.416,1.426,1.436,1.445,1.455,1.464,&
                    1.474,1.483,1.492,1.501,1.511,1.519,1.528,1.537,1.546,&
                    1.555,1.563,1.572,1.580,1.588,1.597,1.605,1.613,1.621,&
                    1.629,1.637,1.645,1.653,1.660,1.668,1.676,1.683,1.691,&
                    1.698,1.706,1.713,1.720,1.727,1.734,1.742,1.749,1.756,&
                    1.763,1.769,1.776,1.783,1.790,1.796,1.803,1.810,1.816,&
                    1.823,1.829,1.835,1.842,1.848,1.854,1.861,1.867,1.873,&
                    1.879,1.885,1.891,1.897,1.903,1.909,1.915,1.920,1.926,&
                    1.932,1.938,1.943,1.949,1.954,1.960,1.965,1.971,1.976,&
                    1.982,1.987,1.992,1.998,2.003,2.008,2.013,2.018,2.024,&
                    2.029,2.034,2.039,2.044,2.049,2.054,2.059,2.063,2.068,&
                    2.073,2.078,2.083,2.087,2.092,2.097,2.101,2.106,2.111,&
                    2.115,2.120,2.124,2.129/)
!
! *** (h ,no3)
!
    bnc10m(1:561) = (/-0.046,-0.096,-0.118,-0.132,-0.142,-0.150,-0.156,   &
                    -0.161,-0.164,-0.167,-0.170,-0.171,-0.173,-0.174,     &
                    -0.174,-0.175,-0.175,-0.175,-0.175,-0.174,-0.174,     &
                    -0.173,-0.172,-0.171,-0.170,-0.169,-0.168,-0.167,     &
                    -0.165,-0.164,-0.163,-0.161,-0.160,-0.158,-0.156,     &
                    -0.155,-0.153,-0.151,-0.150,-0.148,-0.146,-0.144,     &
                    -0.142,-0.141,-0.139,-0.137,-0.135,-0.133,-0.131,     &
                    -0.129,-0.127,-0.125,-0.124,-0.122,-0.120,-0.118,     &
                    -0.116,-0.114,-0.112,-0.110,-0.108,-0.106,-0.104,     &
                    -0.102,-0.100,-0.098,-0.096,-0.094,-0.092,-0.090,     &
                    -0.088,-0.086,-0.084,-0.082,-0.080,-0.078,-0.076,     &
                    -0.074,-0.071,-0.069,-0.067,-0.065,-0.063,-0.061,     &
                    -0.058,-0.056,-0.054,-0.052,-0.049,-0.047,-0.045,     &
                    -0.043,-0.040,-0.038,-0.035,-0.033,-0.031,-0.028,     &
                    -0.026,-0.023,-0.021,-0.019,-0.016,-0.014,-0.011,     &
                    -0.009,-0.006,-0.004,-0.001,0.001,0.004,0.007,0.009,  &
                    0.012,0.014,0.017,0.019,0.022,0.025,0.027,0.030,0.032,&
                    0.035,0.038,0.040,0.043,0.045,0.048,0.051,0.053,0.056,&
                    0.058,0.061,0.064,0.066,0.069,0.071,0.074,0.077,0.079,&
                    0.082,0.084,0.087,0.090,0.092,0.095,0.097,0.100,0.103,&
                    0.105,0.108,0.110,0.113,0.115,0.118,0.121,0.123,0.126,&
                    0.128,0.131,0.133,0.136,0.138,0.141,0.143,0.146,0.148,&
                    0.151,0.154,0.156,0.159,0.161,0.164,0.166,0.169,0.171,&
                    0.174,0.176,0.179,0.181,0.184,0.186,0.189,0.191,0.193,&
                    0.196,0.198,0.201,0.203,0.206,0.208,0.211,0.213,0.216,&
                    0.218,0.220,0.223,0.225,0.228,0.230,0.232,0.235,0.237,&
                    0.240,0.242,0.245,0.247,0.249,0.252,0.254,0.256,0.259,&
                    0.261,0.264,0.266,0.268,0.271,0.273,0.275,0.278,0.280,&
                    0.282,0.285,0.287,0.289,0.292,0.294,0.296,0.299,0.301,&
                    0.303,0.306,0.308,0.310,0.312,0.315,0.317,0.319,0.322,&
                    0.324,0.326,0.328,0.331,0.333,0.335,0.337,0.340,0.342,&
                    0.344,0.346,0.349,0.351,0.353,0.355,0.357,0.360,0.362,&
                    0.364,0.366,0.369,0.371,0.373,0.375,0.377,0.379,0.382,&
                    0.384,0.386,0.388,0.390,0.393,0.395,0.397,0.399,0.401,&
                    0.403,0.405,0.408,0.410,0.412,0.414,0.416,0.418,0.420,&
                    0.422,0.425,0.427,0.429,0.431,0.433,0.435,0.437,0.439,&
                    0.441,0.443,0.445,0.448,0.450,0.452,0.454,0.456,0.458,&
                    0.460,0.462,0.464,0.466,0.468,0.470,0.472,0.474,0.476,&
                    0.478,0.480,0.482,0.484,0.486,0.488,0.490,0.492,0.494,&
                    0.496,0.498,0.500,0.502,0.504,0.506,0.508,0.510,0.512,&
                    0.514,0.516,0.518,0.520,0.522,0.524,0.526,0.528,0.530,&
                    0.532,0.534,0.536,0.537,0.539,0.541,0.543,0.545,0.547,&
                    0.549,0.551,0.553,0.555,0.557,0.558,0.560,0.562,0.564,&
                    0.566,0.568,0.570,0.572,0.574,0.575,0.577,0.579,0.581,&
                    0.583,0.585,0.587,0.588,0.590,0.592,0.594,0.596,0.598,&
                    0.599,0.601,0.603,0.605,0.607,0.609,0.610,0.612,0.614,&
                    0.616,0.618,0.619,0.621,0.623,0.625,0.627,0.628,0.630,&
                    0.632,0.634,0.635,0.637,0.639,0.641,0.643,0.644,0.663,&
                    0.680,0.697,0.714,0.730,0.747,0.763,0.778,0.794,0.809,&
                    0.824,0.839,0.854,0.868,0.883,0.897,0.911,0.924,0.938,&
                    0.951,0.965,0.978,0.990,1.003,1.016,1.028,1.040,1.053,&
                    1.065,1.076,1.088,1.100,1.111,1.122,1.133,1.145,1.155,&
                    1.166,1.177,1.188,1.198,1.208,1.219,1.229,1.239,1.249,&
                    1.258,1.268,1.278,1.287,1.297,1.306,1.315,1.324,1.333,&
                    1.342,1.351,1.360,1.369,1.377,1.386,1.394,1.403,1.411,&
                    1.419,1.428,1.436,1.444,1.452,1.459,1.467,1.475,1.483,&
                    1.490,1.498,1.505,1.513,1.520,1.527,1.535,1.542,1.549,&
                    1.556,1.563,1.570,1.577,1.584,1.590,1.597,1.604,1.610,&
                    1.617,1.623,1.630,1.636,1.643,1.649,1.655,1.661,1.667,&
                    1.674,1.680,1.686,1.692,1.698,1.703,1.709,1.715,1.721,&
                    1.727,1.732,1.738,1.744,1.749,1.755,1.760,1.766,1.771,&
                    1.776,1.782,1.787,1.792,1.797,1.803,1.808,1.813,1.818,&
                    1.823,1.828,1.833,1.838,1.843,1.848,1.853,1.857,1.862,&
                    1.867,1.872,1.876,1.881,1.886,1.890,1.895,1.899,1.904,&
                    1.908,1.913,1.917,1.922,1.926,1.930,1.935,1.939,1.943,&
                    1.948,1.952,1.956,1.960,1.964,1.968,1.973/)
!
! *** (h ,cl)
!
    bnc11m(1:561) = (/-0.045,-0.090,-0.108,-0.119,-0.126,-0.130,-0.133,   &
                    -0.135,-0.135,-0.135,-0.134,-0.133,-0.131,-0.129,     &
                    -0.126,-0.124,-0.121,-0.117,-0.114,-0.110,-0.106,     &
                    -0.102,-0.098,-0.093,-0.089,-0.084,-0.079,-0.075,     &
                    -0.070,-0.065,-0.059,-0.054,-0.049,-0.044,-0.038,     &
                    -0.033,-0.027,-0.022,-0.016,-0.010,-0.004,0.001,0.007,&
                    0.013,0.019,0.025,0.031,0.037,0.043,0.049,0.055,0.061,&
                    0.067,0.073,0.079,0.085,0.091,0.098,0.104,0.110,0.116,&
                    0.122,0.129,0.135,0.141,0.147,0.154,0.160,0.166,0.173,&
                    0.179,0.185,0.192,0.198,0.205,0.211,0.218,0.224,0.231,&
                    0.238,0.244,0.251,0.258,0.264,0.271,0.278,0.285,0.292,&
                    0.298,0.305,0.312,0.319,0.326,0.333,0.341,0.348,0.355,&
                    0.362,0.369,0.377,0.384,0.391,0.398,0.406,0.413,0.421,&
                    0.428,0.435,0.443,0.450,0.458,0.465,0.473,0.480,0.488,&
                    0.495,0.503,0.510,0.518,0.526,0.533,0.541,0.548,0.556,&
                    0.563,0.571,0.578,0.586,0.594,0.601,0.609,0.616,0.624,&
                    0.631,0.639,0.646,0.654,0.661,0.669,0.676,0.684,0.691,&
                    0.699,0.706,0.713,0.721,0.728,0.736,0.743,0.750,0.758,&
                    0.765,0.772,0.780,0.787,0.794,0.802,0.809,0.816,0.823,&
                    0.831,0.838,0.845,0.852,0.859,0.867,0.874,0.881,0.888,&
                    0.895,0.902,0.909,0.916,0.924,0.931,0.938,0.945,0.952,&
                    0.959,0.966,0.973,0.980,0.987,0.994,1.000,1.007,1.014,&
                    1.021,1.028,1.035,1.042,1.049,1.055,1.062,1.069,1.076,&
                    1.082,1.089,1.096,1.103,1.109,1.116,1.123,1.129,1.136,&
                    1.143,1.149,1.156,1.163,1.169,1.176,1.182,1.189,1.195,&
                    1.202,1.208,1.215,1.221,1.228,1.234,1.241,1.247,1.254,&
                    1.260,1.266,1.273,1.279,1.286,1.292,1.298,1.305,1.311,&
                    1.317,1.323,1.330,1.336,1.342,1.348,1.355,1.361,1.367,&
                    1.373,1.379,1.386,1.392,1.398,1.404,1.410,1.416,1.422,&
                    1.428,1.434,1.440,1.446,1.452,1.458,1.464,1.470,1.476,&
                    1.482,1.488,1.494,1.500,1.506,1.512,1.518,1.524,1.530,&
                    1.535,1.541,1.547,1.553,1.559,1.564,1.570,1.576,1.582,&
                    1.588,1.593,1.599,1.605,1.610,1.616,1.622,1.628,1.633,&
                    1.639,1.644,1.650,1.656,1.661,1.667,1.672,1.678,1.684,&
                    1.689,1.695,1.700,1.706,1.711,1.717,1.722,1.728,1.733,&
                    1.739,1.744,1.750,1.755,1.760,1.766,1.771,1.777,1.782,&
                    1.787,1.793,1.798,1.803,1.809,1.814,1.819,1.825,1.830,&
                    1.835,1.840,1.846,1.851,1.856,1.861,1.867,1.872,1.877,&
                    1.882,1.887,1.893,1.898,1.903,1.908,1.913,1.918,1.923,&
                    1.929,1.934,1.939,1.944,1.949,1.954,1.959,1.964,1.969,&
                    1.974,1.979,1.984,1.989,1.994,1.999,2.004,2.009,2.014,&
                    2.019,2.024,2.029,2.034,2.039,2.044,2.048,2.053,2.058,&
                    2.063,2.068,2.073,2.078,2.082,2.087,2.092,2.097,2.102,&
                    2.107,2.111,2.116,2.121,2.126,2.130,2.135,2.140,2.145,&
                    2.149,2.154,2.159,2.163,2.168,2.173,2.177,2.182,2.187,&
                    2.191,2.196,2.201,2.205,2.210,2.215,2.264,2.309,2.353,&
                    2.396,2.439,2.481,2.523,2.564,2.604,2.643,2.683,2.721,&
                    2.759,2.797,2.834,2.870,2.906,2.941,2.976,3.011,3.045,&
                    3.079,3.112,3.145,3.177,3.210,3.241,3.273,3.304,3.334,&
                    3.364,3.394,3.424,3.453,3.482,3.511,3.539,3.567,3.595,&
                    3.622,3.649,3.676,3.702,3.729,3.755,3.781,3.806,3.831,&
                    3.856,3.881,3.906,3.930,3.954,3.978,4.002,4.025,4.048,&
                    4.071,4.094,4.117,4.139,4.161,4.183,4.205,4.227,4.248,&
                    4.270,4.291,4.312,4.332,4.353,4.373,4.394,4.414,4.434,&
                    4.453,4.473,4.493,4.512,4.531,4.550,4.569,4.588,4.606,&
                    4.625,4.643,4.661,4.679,4.697,4.715,4.733,4.750,4.768,&
                    4.785,4.802,4.819,4.836,4.853,4.870,4.886,4.903,4.919,&
                    4.936,4.952,4.968,4.984,5.000,5.015,5.031,5.046,5.062,&
                    5.077,5.093,5.108,5.123,5.138,5.153,5.167,5.182,5.197,&
                    5.211,5.226,5.240,5.254,5.268,5.282,5.296,5.310,5.324,&
                    5.338,5.352,5.365,5.379,5.392,5.406,5.419,5.432,5.445,&
                    5.458,5.471,5.484,5.497,5.510,5.523,5.535,5.548,5.560,&
                    5.573,5.585,5.598,5.610,5.622,5.634,5.646,5.658,5.670,&
                    5.682,5.694,5.706,5.717,5.729/)
!
! *** nahso4
!
    bnc12m(1:561) = (/-0.046,-0.096,-0.118,-0.132,-0.143,-0.151,-0.158,   &
                    -0.163,-0.167,-0.170,-0.173,-0.175,-0.177,-0.178,     &
                    -0.179,-0.180,-0.180,-0.180,-0.180,-0.180,-0.179,     &
                    -0.179,-0.178,-0.177,-0.176,-0.174,-0.173,-0.171,     &
                    -0.170,-0.168,-0.166,-0.164,-0.162,-0.159,-0.157,     &
                    -0.155,-0.152,-0.150,-0.147,-0.144,-0.142,-0.139,     &
                    -0.136,-0.133,-0.130,-0.127,-0.124,-0.120,-0.117,     &
                    -0.114,-0.110,-0.107,-0.104,-0.100,-0.097,-0.093,     &
                    -0.090,-0.086,-0.082,-0.079,-0.075,-0.071,-0.067,     &
                    -0.064,-0.060,-0.056,-0.052,-0.048,-0.044,-0.040,     &
                    -0.036,-0.032,-0.028,-0.024,-0.020,-0.015,-0.011,     &
                    -0.007,-0.003,0.002,0.006,0.010,0.015,0.019,0.024,    &
                    0.028,0.033,0.037,0.042,0.046,0.051,0.056,0.060,0.065,&
                    0.070,0.075,0.079,0.084,0.089,0.094,0.099,0.104,0.109,&
                    0.114,0.119,0.124,0.129,0.134,0.139,0.144,0.149,0.154,&
                    0.159,0.164,0.169,0.174,0.179,0.184,0.189,0.195,0.200,&
                    0.205,0.210,0.215,0.220,0.225,0.230,0.236,0.241,0.246,&
                    0.251,0.256,0.261,0.266,0.271,0.276,0.281,0.286,0.291,&
                    0.297,0.302,0.307,0.312,0.317,0.322,0.327,0.332,0.337,&
                    0.342,0.347,0.352,0.357,0.361,0.366,0.371,0.376,0.381,&
                    0.386,0.391,0.396,0.401,0.406,0.410,0.415,0.420,0.425,&
                    0.430,0.434,0.439,0.444,0.449,0.454,0.458,0.463,0.468,&
                    0.473,0.477,0.482,0.487,0.491,0.496,0.501,0.505,0.510,&
                    0.515,0.519,0.524,0.529,0.533,0.538,0.542,0.547,0.551,&
                    0.556,0.560,0.565,0.570,0.574,0.579,0.583,0.588,0.592,&
                    0.596,0.601,0.605,0.610,0.614,0.619,0.623,0.627,0.632,&
                    0.636,0.641,0.645,0.649,0.654,0.658,0.662,0.667,0.671,&
                    0.675,0.680,0.684,0.688,0.692,0.697,0.701,0.705,0.709,&
                    0.714,0.718,0.722,0.726,0.730,0.735,0.739,0.743,0.747,&
                    0.751,0.755,0.759,0.764,0.768,0.772,0.776,0.780,0.784,&
                    0.788,0.792,0.796,0.800,0.804,0.808,0.812,0.816,0.820,&
                    0.824,0.828,0.832,0.836,0.840,0.844,0.848,0.852,0.856,&
                    0.860,0.864,0.868,0.872,0.875,0.879,0.883,0.887,0.891,&
                    0.895,0.899,0.902,0.906,0.910,0.914,0.918,0.922,0.925,&
                    0.929,0.933,0.937,0.940,0.944,0.948,0.952,0.955,0.959,&
                    0.963,0.967,0.970,0.974,0.978,0.981,0.985,0.989,0.992,&
                    0.996,1.000,1.003,1.007,1.011,1.014,1.018,1.022,1.025,&
                    1.029,1.032,1.036,1.040,1.043,1.047,1.050,1.054,1.057,&
                    1.061,1.064,1.068,1.071,1.075,1.079,1.082,1.086,1.089,&
                    1.092,1.096,1.099,1.103,1.106,1.110,1.113,1.117,1.120,&
                    1.124,1.127,1.130,1.134,1.137,1.141,1.144,1.147,1.151,&
                    1.154,1.158,1.161,1.164,1.168,1.171,1.174,1.178,1.181,&
                    1.184,1.188,1.191,1.194,1.198,1.201,1.204,1.207,1.211,&
                    1.214,1.217,1.220,1.224,1.227,1.230,1.233,1.237,1.240,&
                    1.243,1.246,1.250,1.253,1.256,1.259,1.262,1.266,1.269,&
                    1.272,1.275,1.278,1.281,1.285,1.288,1.291,1.294,1.297,&
                    1.300,1.303,1.306,1.310,1.313,1.316,1.319,1.322,1.325,&
                    1.358,1.388,1.418,1.447,1.476,1.504,1.532,1.559,1.586,&
                    1.613,1.639,1.665,1.691,1.716,1.741,1.766,1.790,1.814,&
                    1.838,1.861,1.884,1.907,1.929,1.951,1.973,1.995,2.016,&
                    2.037,2.058,2.079,2.099,2.120,2.140,2.159,2.179,2.198,&
                    2.217,2.236,2.255,2.274,2.292,2.310,2.328,2.346,2.363,&
                    2.381,2.398,2.415,2.432,2.449,2.465,2.482,2.498,2.514,&
                    2.530,2.546,2.562,2.577,2.592,2.608,2.623,2.638,2.653,&
                    2.667,2.682,2.697,2.711,2.725,2.739,2.753,2.767,2.781,&
                    2.795,2.808,2.822,2.835,2.848,2.861,2.874,2.887,2.900,&
                    2.913,2.925,2.938,2.950,2.963,2.975,2.987,2.999,3.011,&
                    3.023,3.035,3.046,3.058,3.069,3.081,3.092,3.104,3.115,&
                    3.126,3.137,3.148,3.159,3.170,3.180,3.191,3.202,3.212,&
                    3.223,3.233,3.244,3.254,3.264,3.274,3.284,3.294,3.304,&
                    3.314,3.324,3.334,3.343,3.353,3.363,3.372,3.382,3.391,&
                    3.400,3.410,3.419,3.428,3.437,3.446,3.455,3.464,3.473,&
                    3.482,3.491,3.499,3.508,3.517,3.525,3.534,3.542,3.551,&
                    3.559,3.568,3.576,3.584,3.592,3.601,3.609,3.617,3.625,&
                    3.633,3.641,3.649,3.657,3.664,3.672,3.680,3.688/)
!
! *** (nh4)3h(so4)2
!
    bnc13m(1:561) = (/-0.076,-0.165,-0.209,-0.240,-0.264,-0.284,-0.301,   &
                    -0.316,-0.329,-0.341,-0.352,-0.362,-0.371,-0.379,     &
                    -0.387,-0.394,-0.401,-0.408,-0.414,-0.420,-0.425,     &
                    -0.430,-0.435,-0.440,-0.444,-0.449,-0.453,-0.457,     &
                    -0.460,-0.464,-0.467,-0.471,-0.474,-0.477,-0.480,     &
                    -0.483,-0.485,-0.488,-0.490,-0.493,-0.495,-0.497,     &
                    -0.499,-0.501,-0.503,-0.505,-0.507,-0.509,-0.510,     &
                    -0.512,-0.514,-0.515,-0.516,-0.518,-0.519,-0.520,     &
                    -0.522,-0.523,-0.524,-0.525,-0.526,-0.527,-0.528,     &
                    -0.529,-0.530,-0.531,-0.532,-0.532,-0.533,-0.534,     &
                    -0.534,-0.535,-0.536,-0.536,-0.537,-0.537,-0.538,     &
                    -0.538,-0.539,-0.539,-0.540,-0.540,-0.540,-0.541,     &
                    -0.541,-0.541,-0.542,-0.542,-0.542,-0.542,-0.542,     &
                    -0.543,-0.543,-0.543,-0.543,-0.543,-0.543,-0.543,     &
                    -0.543,-0.543,-0.543,-0.543,-0.543,-0.543,-0.543,     &
                    -0.543,-0.543,-0.543,-0.543,-0.542,-0.542,-0.542,     &
                    -0.542,-0.542,-0.542,-0.542,-0.541,-0.541,-0.541,     &
                    -0.541,-0.540,-0.540,-0.540,-0.540,-0.540,-0.539,     &
                    -0.539,-0.539,-0.538,-0.538,-0.538,-0.538,-0.537,     &
                    -0.537,-0.537,-0.536,-0.536,-0.536,-0.536,-0.535,     &
                    -0.535,-0.535,-0.534,-0.534,-0.534,-0.533,-0.533,     &
                    -0.533,-0.532,-0.532,-0.532,-0.531,-0.531,-0.530,     &
                    -0.530,-0.530,-0.529,-0.529,-0.529,-0.528,-0.528,     &
                    -0.528,-0.527,-0.527,-0.527,-0.526,-0.526,-0.525,     &
                    -0.525,-0.525,-0.524,-0.524,-0.524,-0.523,-0.523,     &
                    -0.522,-0.522,-0.522,-0.521,-0.521,-0.521,-0.520,     &
                    -0.520,-0.519,-0.519,-0.519,-0.518,-0.518,-0.518,     &
                    -0.517,-0.517,-0.516,-0.516,-0.516,-0.515,-0.515,     &
                    -0.515,-0.514,-0.514,-0.513,-0.513,-0.513,-0.512,     &
                    -0.512,-0.512,-0.511,-0.511,-0.510,-0.510,-0.510,     &
                    -0.509,-0.509,-0.509,-0.508,-0.508,-0.507,-0.507,     &
                    -0.507,-0.506,-0.506,-0.506,-0.505,-0.505,-0.504,     &
                    -0.504,-0.504,-0.503,-0.503,-0.503,-0.502,-0.502,     &
                    -0.501,-0.501,-0.501,-0.500,-0.500,-0.500,-0.499,     &
                    -0.499,-0.499,-0.498,-0.498,-0.497,-0.497,-0.497,     &
                    -0.496,-0.496,-0.496,-0.495,-0.495,-0.495,-0.494,     &
                    -0.494,-0.493,-0.493,-0.493,-0.492,-0.492,-0.492,     &
                    -0.491,-0.491,-0.491,-0.490,-0.490,-0.490,-0.489,     &
                    -0.489,-0.489,-0.488,-0.488,-0.487,-0.487,-0.487,     &
                    -0.486,-0.486,-0.486,-0.485,-0.485,-0.485,-0.484,     &
                    -0.484,-0.484,-0.483,-0.483,-0.483,-0.482,-0.482,     &
                    -0.482,-0.481,-0.481,-0.481,-0.480,-0.480,-0.480,     &
                    -0.479,-0.479,-0.479,-0.478,-0.478,-0.478,-0.477,     &
                    -0.477,-0.477,-0.476,-0.476,-0.476,-0.475,-0.475,     &
                    -0.475,-0.474,-0.474,-0.474,-0.473,-0.473,-0.473,     &
                    -0.472,-0.472,-0.472,-0.471,-0.471,-0.471,-0.471,     &
                    -0.470,-0.470,-0.470,-0.469,-0.469,-0.469,-0.468,     &
                    -0.468,-0.468,-0.467,-0.467,-0.467,-0.466,-0.466,     &
                    -0.466,-0.466,-0.465,-0.465,-0.465,-0.464,-0.464,     &
                    -0.464,-0.463,-0.463,-0.463,-0.463,-0.462,-0.462,     &
                    -0.462,-0.461,-0.461,-0.461,-0.460,-0.460,-0.460,     &
                    -0.460,-0.459,-0.459,-0.459,-0.458,-0.458,-0.458,     &
                    -0.458,-0.457,-0.457,-0.457,-0.456,-0.456,-0.456,     &
                    -0.456,-0.455,-0.455,-0.455,-0.454,-0.454,-0.454,     &
                    -0.454,-0.453,-0.453,-0.453,-0.452,-0.452,-0.452,     &
                    -0.452,-0.451,-0.451,-0.451,-0.450,-0.450,-0.450,     &
                    -0.450,-0.449,-0.449,-0.449,-0.449,-0.448,-0.448,     &
                    -0.448,-0.445,-0.442,-0.440,-0.437,-0.435,-0.432,     &
                    -0.430,-0.428,-0.426,-0.423,-0.421,-0.419,-0.417,     &
                    -0.415,-0.413,-0.411,-0.409,-0.407,-0.406,-0.404,     &
                    -0.402,-0.401,-0.399,-0.397,-0.396,-0.394,-0.393,     &
                    -0.391,-0.390,-0.389,-0.387,-0.386,-0.385,-0.383,     &
                    -0.382,-0.381,-0.380,-0.379,-0.378,-0.377,-0.376,     &
                    -0.375,-0.374,-0.373,-0.372,-0.371,-0.370,-0.369,     &
                    -0.368,-0.367,-0.367,-0.366,-0.365,-0.364,-0.364,     &
                    -0.363,-0.362,-0.362,-0.361,-0.361,-0.360,-0.360,     &
                    -0.359,-0.359,-0.358,-0.358,-0.357,-0.357,-0.356,     &
                    -0.356,-0.356,-0.355,-0.355,-0.355,-0.354,-0.354,     &
                    -0.354,-0.354,-0.353,-0.353,-0.353,-0.353,-0.353,     &
                    -0.352,-0.352,-0.352,-0.352,-0.352,-0.352,-0.352,     &
                    -0.352,-0.352,-0.352,-0.352,-0.352,-0.352,-0.352,     &
                    -0.352,-0.352,-0.352,-0.352,-0.352,-0.352,-0.352,     &
                    -0.352,-0.353,-0.353,-0.353,-0.353,-0.353,-0.353,     &
                    -0.354,-0.354,-0.354,-0.354,-0.355,-0.355,-0.355,     &
                    -0.355,-0.356,-0.356,-0.356,-0.357,-0.357,-0.357,     &
                    -0.358,-0.358,-0.358,-0.359,-0.359,-0.359,-0.360,     &
                    -0.360,-0.361,-0.361,-0.362,-0.362,-0.362,-0.363,     &
                    -0.363,-0.364,-0.364,-0.365,-0.365,-0.366,-0.366,     &
                    -0.367,-0.367,-0.368,-0.369,-0.369,-0.370,-0.370,     &
                    -0.371,-0.371,-0.372,-0.373,-0.373,-0.374,-0.374,     &
                    -0.375/)
!
! *** caso4
!
    bnc14m(1:561) = (/0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,    &
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000/)
!
! *** cano32
!
    bnc15m(1:561) = (/-0.094,-0.201,-0.251,-0.286,-0.312,-0.334,-0.351,   &
                    -0.366,-0.379,-0.390,-0.400,-0.409,-0.417,-0.424,     &
                    -0.430,-0.436,-0.442,-0.447,-0.451,-0.455,-0.459,     &
                    -0.463,-0.466,-0.469,-0.472,-0.475,-0.477,-0.479,     &
                    -0.481,-0.483,-0.485,-0.487,-0.489,-0.490,-0.491,     &
                    -0.493,-0.494,-0.495,-0.496,-0.497,-0.498,-0.499,     &
                    -0.500,-0.501,-0.501,-0.502,-0.503,-0.503,-0.504,     &
                    -0.504,-0.505,-0.505,-0.505,-0.506,-0.506,-0.506,     &
                    -0.507,-0.507,-0.507,-0.507,-0.507,-0.508,-0.508,     &
                    -0.508,-0.508,-0.508,-0.508,-0.508,-0.508,-0.508,     &
                    -0.508,-0.508,-0.508,-0.507,-0.507,-0.507,-0.507,     &
                    -0.507,-0.506,-0.506,-0.506,-0.506,-0.505,-0.505,     &
                    -0.505,-0.504,-0.504,-0.503,-0.503,-0.502,-0.502,     &
                    -0.501,-0.501,-0.500,-0.500,-0.499,-0.499,-0.498,     &
                    -0.497,-0.497,-0.496,-0.495,-0.495,-0.494,-0.493,     &
                    -0.493,-0.492,-0.491,-0.490,-0.489,-0.489,-0.488,     &
                    -0.487,-0.486,-0.485,-0.485,-0.484,-0.483,-0.482,     &
                    -0.481,-0.480,-0.479,-0.478,-0.477,-0.477,-0.476,     &
                    -0.475,-0.474,-0.473,-0.472,-0.471,-0.470,-0.469,     &
                    -0.468,-0.467,-0.466,-0.465,-0.464,-0.463,-0.462,     &
                    -0.461,-0.460,-0.459,-0.459,-0.458,-0.457,-0.456,     &
                    -0.455,-0.454,-0.453,-0.452,-0.451,-0.450,-0.449,     &
                    -0.448,-0.447,-0.446,-0.445,-0.444,-0.443,-0.442,     &
                    -0.441,-0.440,-0.439,-0.438,-0.437,-0.436,-0.435,     &
                    -0.434,-0.433,-0.432,-0.431,-0.430,-0.429,-0.428,     &
                    -0.427,-0.426,-0.425,-0.424,-0.423,-0.422,-0.421,     &
                    -0.420,-0.419,-0.418,-0.417,-0.416,-0.415,-0.414,     &
                    -0.413,-0.412,-0.411,-0.410,-0.409,-0.408,-0.407,     &
                    -0.406,-0.405,-0.404,-0.403,-0.402,-0.401,-0.400,     &
                    -0.399,-0.398,-0.397,-0.396,-0.395,-0.394,-0.393,     &
                    -0.392,-0.391,-0.390,-0.389,-0.388,-0.387,-0.386,     &
                    -0.385,-0.384,-0.383,-0.382,-0.382,-0.381,-0.380,     &
                    -0.379,-0.378,-0.377,-0.376,-0.375,-0.374,-0.373,     &
                    -0.372,-0.371,-0.370,-0.369,-0.368,-0.367,-0.366,     &
                    -0.365,-0.364,-0.363,-0.362,-0.361,-0.360,-0.359,     &
                    -0.359,-0.358,-0.357,-0.356,-0.355,-0.354,-0.353,     &
                    -0.352,-0.351,-0.350,-0.349,-0.348,-0.347,-0.346,     &
                    -0.345,-0.344,-0.344,-0.343,-0.342,-0.341,-0.340,     &
                    -0.339,-0.338,-0.337,-0.336,-0.335,-0.334,-0.333,     &
                    -0.332,-0.332,-0.331,-0.330,-0.329,-0.328,-0.327,     &
                    -0.326,-0.325,-0.324,-0.323,-0.323,-0.322,-0.321,     &
                    -0.320,-0.319,-0.318,-0.317,-0.316,-0.315,-0.314,     &
                    -0.314,-0.313,-0.312,-0.311,-0.310,-0.309,-0.308,     &
                    -0.307,-0.307,-0.306,-0.305,-0.304,-0.303,-0.302,     &
                    -0.301,-0.300,-0.300,-0.299,-0.298,-0.297,-0.296,     &
                    -0.295,-0.294,-0.294,-0.293,-0.292,-0.291,-0.290,     &
                    -0.289,-0.288,-0.288,-0.287,-0.286,-0.285,-0.284,     &
                    -0.283,-0.282,-0.282,-0.281,-0.280,-0.279,-0.278,     &
                    -0.277,-0.277,-0.276,-0.275,-0.274,-0.273,-0.273,     &
                    -0.272,-0.271,-0.270,-0.269,-0.268,-0.268,-0.267,     &
                    -0.266,-0.265,-0.264,-0.263,-0.263,-0.262,-0.261,     &
                    -0.260,-0.259,-0.259,-0.258,-0.257,-0.256,-0.255,     &
                    -0.255,-0.254,-0.253,-0.252,-0.251,-0.251,-0.250,     &
                    -0.249,-0.248,-0.248,-0.247,-0.246,-0.245,-0.244,     &
                    -0.244,-0.243,-0.242,-0.241,-0.240,-0.240,-0.239,     &
                    -0.238,-0.237,-0.237,-0.236,-0.235,-0.234,-0.234,     &
                    -0.233,-0.232,-0.231,-0.230,-0.230,-0.229,-0.228,     &
                    -0.227,-0.219,-0.212,-0.205,-0.198,-0.191,-0.184,     &
                    -0.177,-0.170,-0.163,-0.157,-0.151,-0.144,-0.138,     &
                    -0.132,-0.126,-0.120,-0.114,-0.108,-0.103,-0.097,     &
                    -0.091,-0.086,-0.081,-0.075,-0.070,-0.065,-0.060,     &
                    -0.055,-0.050,-0.045,-0.041,-0.036,-0.031,-0.027,     &
                    -0.022,-0.018,-0.014,-0.009,-0.005,-0.001,0.003,0.007,&
                    0.011,0.015,0.019,0.023,0.027,0.031,0.034,0.038,0.041,&
                    0.045,0.048,0.052,0.055,0.059,0.062,0.065,0.068,0.071,&
                    0.075,0.078,0.081,0.084,0.087,0.089,0.092,0.095,0.098,&
                    0.101,0.103,0.106,0.109,0.111,0.114,0.116,0.119,0.121,&
                    0.124,0.126,0.128,0.131,0.133,0.135,0.138,0.140,0.142,&
                    0.144,0.146,0.148,0.150,0.152,0.154,0.156,0.158,0.160,&
                    0.162,0.164,0.166,0.167,0.169,0.171,0.173,0.174,0.176,&
                    0.178,0.179,0.181,0.182,0.184,0.186,0.187,0.189,0.190,&
                    0.191,0.193,0.194,0.196,0.197,0.198,0.200,0.201,0.202,&
                    0.203,0.205,0.206,0.207,0.208,0.209,0.210,0.212,0.213,&
                    0.214,0.215,0.216,0.217,0.218,0.219,0.220,0.221,0.222,&
                    0.222,0.223,0.224,0.225,0.226,0.227,0.228,0.228,0.229,&
                    0.230,0.231,0.231,0.232,0.233,0.234,0.234,0.235,0.235,&
                    0.236,0.237/)
!
! *** cacl2
!
    bnc16m(1:561) = (/-0.093,-0.193,-0.238,-0.267,-0.288,-0.304,-0.316,   &
                    -0.326,-0.334,-0.340,-0.345,-0.350,-0.353,-0.355,     &
                    -0.357,-0.359,-0.360,-0.360,-0.360,-0.360,-0.359,     &
                    -0.358,-0.357,-0.356,-0.355,-0.353,-0.351,-0.349,     &
                    -0.347,-0.345,-0.342,-0.340,-0.337,-0.335,-0.332,     &
                    -0.329,-0.326,-0.323,-0.321,-0.317,-0.314,-0.311,     &
                    -0.308,-0.305,-0.302,-0.299,-0.295,-0.292,-0.289,     &
                    -0.286,-0.282,-0.279,-0.276,-0.272,-0.269,-0.266,     &
                    -0.262,-0.259,-0.255,-0.252,-0.249,-0.245,-0.242,     &
                    -0.238,-0.235,-0.231,-0.228,-0.224,-0.221,-0.217,     &
                    -0.213,-0.210,-0.206,-0.203,-0.199,-0.195,-0.192,     &
                    -0.188,-0.184,-0.180,-0.177,-0.173,-0.169,-0.165,     &
                    -0.161,-0.157,-0.153,-0.149,-0.145,-0.141,-0.137,     &
                    -0.133,-0.129,-0.125,-0.120,-0.116,-0.112,-0.108,     &
                    -0.103,-0.099,-0.095,-0.090,-0.086,-0.081,-0.077,     &
                    -0.073,-0.068,-0.064,-0.059,-0.054,-0.050,-0.045,     &
                    -0.041,-0.036,-0.032,-0.027,-0.022,-0.018,-0.013,     &
                    -0.008,-0.004,0.001,0.005,0.010,0.015,0.019,0.024,    &
                    0.029,0.033,0.038,0.043,0.048,0.052,0.057,0.062,0.066,&
                    0.071,0.076,0.080,0.085,0.090,0.094,0.099,0.103,0.108,&
                    0.113,0.117,0.122,0.127,0.131,0.136,0.141,0.145,0.150,&
                    0.154,0.159,0.164,0.168,0.173,0.177,0.182,0.186,0.191,&
                    0.196,0.200,0.205,0.209,0.214,0.218,0.223,0.227,0.232,&
                    0.236,0.241,0.245,0.250,0.254,0.259,0.263,0.268,0.272,&
                    0.277,0.281,0.286,0.290,0.294,0.299,0.303,0.308,0.312,&
                    0.317,0.321,0.325,0.330,0.334,0.338,0.343,0.347,0.352,&
                    0.356,0.360,0.365,0.369,0.373,0.378,0.382,0.386,0.390,&
                    0.395,0.399,0.403,0.408,0.412,0.416,0.420,0.425,0.429,&
                    0.433,0.437,0.442,0.446,0.450,0.454,0.458,0.463,0.467,&
                    0.471,0.475,0.479,0.483,0.488,0.492,0.496,0.500,0.504,&
                    0.508,0.512,0.516,0.521,0.525,0.529,0.533,0.537,0.541,&
                    0.545,0.549,0.553,0.557,0.561,0.565,0.569,0.573,0.577,&
                    0.581,0.585,0.589,0.593,0.597,0.601,0.605,0.609,0.613,&
                    0.617,0.621,0.625,0.629,0.633,0.637,0.641,0.644,0.648,&
                    0.652,0.656,0.660,0.664,0.668,0.672,0.675,0.679,0.683,&
                    0.687,0.691,0.695,0.698,0.702,0.706,0.710,0.714,0.717,&
                    0.721,0.725,0.729,0.733,0.736,0.740,0.744,0.748,0.751,&
                    0.755,0.759,0.762,0.766,0.770,0.774,0.777,0.781,0.785,&
                    0.788,0.792,0.796,0.799,0.803,0.807,0.810,0.814,0.818,&
                    0.821,0.825,0.828,0.832,0.836,0.839,0.843,0.846,0.850,&
                    0.853,0.857,0.861,0.864,0.868,0.871,0.875,0.878,0.882,&
                    0.885,0.889,0.892,0.896,0.899,0.903,0.906,0.910,0.913,&
                    0.917,0.920,0.924,0.927,0.931,0.934,0.938,0.941,0.944,&
                    0.948,0.951,0.955,0.958,0.961,0.965,0.968,0.972,0.975,&
                    0.978,0.982,0.985,0.988,0.992,0.995,0.999,1.002,1.005,&
                    1.009,1.012,1.015,1.018,1.022,1.025,1.028,1.032,1.035,&
                    1.038,1.042,1.045,1.048,1.051,1.055,1.058,1.061,1.064,&
                    1.068,1.071,1.074,1.077,1.080,1.084,1.087,1.090,1.093,&
                    1.096,1.100,1.103,1.137,1.168,1.199,1.229,1.259,1.288,&
                    1.317,1.345,1.373,1.401,1.428,1.455,1.482,1.508,1.534,&
                    1.559,1.584,1.609,1.634,1.658,1.682,1.706,1.729,1.752,&
                    1.775,1.797,1.819,1.841,1.863,1.884,1.905,1.926,1.947,&
                    1.967,1.988,2.008,2.027,2.047,2.066,2.085,2.104,2.123,&
                    2.141,2.160,2.178,2.196,2.214,2.231,2.249,2.266,2.283,&
                    2.300,2.316,2.333,2.349,2.365,2.381,2.397,2.413,2.429,&
                    2.444,2.459,2.475,2.490,2.504,2.519,2.534,2.548,2.563,&
                    2.577,2.591,2.605,2.619,2.632,2.646,2.659,2.673,2.686,&
                    2.699,2.712,2.725,2.738,2.751,2.763,2.776,2.788,2.800,&
                    2.813,2.825,2.837,2.849,2.860,2.872,2.884,2.895,2.907,&
                    2.918,2.929,2.940,2.952,2.963,2.973,2.984,2.995,3.006,&
                    3.016,3.027,3.037,3.048,3.058,3.068,3.078,3.088,3.098,&
                    3.108,3.118,3.128,3.137,3.147,3.157,3.166,3.176,3.185,&
                    3.194,3.203,3.213,3.222,3.231,3.240,3.249,3.258,3.266,&
                    3.275,3.284,3.292,3.301,3.309,3.318,3.326,3.335,3.343,&
                    3.351,3.359,3.368,3.376,3.384,3.392,3.399,3.407,3.415,&
                    3.423,3.431,3.438,3.446,3.454,3.461,3.469,3.476,3.483,&
                    3.491,3.498/)
!
! *** k2so4
!
    bnc17m(1:561) = (/-0.096,-0.208,-0.265,-0.305,-0.337,-0.363,-0.385,   &
                    -0.405,-0.423,-0.439,-0.453,-0.467,-0.479,-0.491,     &
                    -0.502,-0.512,-0.522,-0.531,-0.540,-0.548,-0.556,     &
                    -0.564,-0.571,-0.578,-0.585,-0.592,-0.598,-0.605,     &
                    -0.610,-0.616,-0.622,-0.627,-0.633,-0.638,-0.643,     &
                    -0.648,-0.653,-0.657,-0.662,-0.666,-0.671,-0.675,     &
                    -0.679,-0.683,-0.688,-0.691,-0.695,-0.699,-0.703,     &
                    -0.707,-0.710,-0.714,-0.717,-0.721,-0.724,-0.727,     &
                    -0.731,-0.734,-0.737,-0.740,-0.743,-0.746,-0.749,     &
                    -0.752,-0.755,-0.758,-0.761,-0.763,-0.766,-0.769,     &
                    -0.772,-0.774,-0.777,-0.780,-0.782,-0.785,-0.787,     &
                    -0.790,-0.792,-0.795,-0.797,-0.799,-0.802,-0.804,     &
                    -0.807,-0.809,-0.811,-0.814,-0.816,-0.818,-0.820,     &
                    -0.823,-0.825,-0.827,-0.829,-0.831,-0.833,-0.836,     &
                    -0.838,-0.840,-0.842,-0.844,-0.846,-0.848,-0.850,     &
                    -0.852,-0.854,-0.856,-0.858,-0.860,-0.862,-0.864,     &
                    -0.866,-0.868,-0.870,-0.872,-0.874,-0.875,-0.877,     &
                    -0.879,-0.881,-0.883,-0.885,-0.887,-0.888,-0.890,     &
                    -0.892,-0.894,-0.895,-0.897,-0.899,-0.901,-0.902,     &
                    -0.904,-0.906,-0.908,-0.909,-0.911,-0.913,-0.914,     &
                    -0.916,-0.918,-0.919,-0.921,-0.923,-0.924,-0.926,     &
                    -0.927,-0.929,-0.931,-0.932,-0.934,-0.935,-0.937,     &
                    -0.938,-0.940,-0.942,-0.943,-0.945,-0.946,-0.948,     &
                    -0.949,-0.951,-0.952,-0.954,-0.955,-0.957,-0.958,     &
                    -0.960,-0.961,-0.962,-0.964,-0.965,-0.967,-0.968,     &
                    -0.970,-0.971,-0.972,-0.974,-0.975,-0.977,-0.978,     &
                    -0.979,-0.981,-0.982,-0.984,-0.985,-0.986,-0.988,     &
                    -0.989,-0.990,-0.992,-0.993,-0.994,-0.996,-0.997,     &
                    -0.998,-1.000,-1.001,-1.002,-1.004,-1.005,-1.006,     &
                    -1.007,-1.009,-1.010,-1.011,-1.013,-1.014,-1.015,     &
                    -1.016,-1.018,-1.019,-1.020,-1.021,-1.023,-1.024,     &
                    -1.025,-1.026,-1.027,-1.029,-1.030,-1.031,-1.032,     &
                    -1.034,-1.035,-1.036,-1.037,-1.038,-1.039,-1.041,     &
                    -1.042,-1.043,-1.044,-1.045,-1.047,-1.048,-1.049,     &
                    -1.050,-1.051,-1.052,-1.053,-1.055,-1.056,-1.057,     &
                    -1.058,-1.059,-1.060,-1.061,-1.063,-1.064,-1.065,     &
                    -1.066,-1.067,-1.068,-1.069,-1.070,-1.071,-1.073,     &
                    -1.074,-1.075,-1.076,-1.077,-1.078,-1.079,-1.080,     &
                    -1.081,-1.082,-1.083,-1.084,-1.086,-1.087,-1.088,     &
                    -1.089,-1.090,-1.091,-1.092,-1.093,-1.094,-1.095,     &
                    -1.096,-1.097,-1.098,-1.099,-1.100,-1.101,-1.102,     &
                    -1.103,-1.104,-1.105,-1.106,-1.107,-1.108,-1.109,     &
                    -1.110,-1.111,-1.112,-1.114,-1.115,-1.116,-1.117,     &
                    -1.118,-1.119,-1.119,-1.120,-1.121,-1.122,-1.123,     &
                    -1.124,-1.125,-1.126,-1.127,-1.128,-1.129,-1.130,     &
                    -1.131,-1.132,-1.133,-1.134,-1.135,-1.136,-1.137,     &
                    -1.138,-1.139,-1.140,-1.141,-1.142,-1.143,-1.144,     &
                    -1.145,-1.146,-1.147,-1.147,-1.148,-1.149,-1.150,     &
                    -1.151,-1.152,-1.153,-1.154,-1.155,-1.156,-1.157,     &
                    -1.158,-1.159,-1.160,-1.160,-1.161,-1.162,-1.163,     &
                    -1.164,-1.165,-1.166,-1.167,-1.168,-1.169,-1.169,     &
                    -1.170,-1.171,-1.172,-1.173,-1.174,-1.175,-1.176,     &
                    -1.177,-1.178,-1.178,-1.179,-1.180,-1.181,-1.182,     &
                    -1.183,-1.184,-1.185,-1.185,-1.186,-1.187,-1.188,     &
                    -1.189,-1.190,-1.191,-1.191,-1.192,-1.193,-1.194,     &
                    -1.195,-1.196,-1.197,-1.198,-1.198,-1.199,-1.200,     &
                    -1.201,-1.202,-1.203,-1.203,-1.204,-1.205,-1.206,     &
                    -1.207,-1.216,-1.224,-1.232,-1.240,-1.248,-1.256,     &
                    -1.263,-1.271,-1.279,-1.286,-1.293,-1.301,-1.308,     &
                    -1.315,-1.322,-1.329,-1.336,-1.343,-1.349,-1.356,     &
                    -1.363,-1.369,-1.376,-1.382,-1.389,-1.395,-1.402,     &
                    -1.408,-1.414,-1.420,-1.427,-1.433,-1.439,-1.445,     &
                    -1.451,-1.457,-1.463,-1.468,-1.474,-1.480,-1.486,     &
                    -1.492,-1.497,-1.503,-1.509,-1.514,-1.520,-1.525,     &
                    -1.531,-1.536,-1.542,-1.547,-1.553,-1.558,-1.563,     &
                    -1.569,-1.574,-1.579,-1.585,-1.590,-1.595,-1.600,     &
                    -1.605,-1.611,-1.616,-1.621,-1.626,-1.631,-1.636,     &
                    -1.641,-1.646,-1.651,-1.656,-1.661,-1.666,-1.671,     &
                    -1.676,-1.681,-1.685,-1.690,-1.695,-1.700,-1.705,     &
                    -1.710,-1.714,-1.719,-1.724,-1.729,-1.733,-1.738,     &
                    -1.743,-1.747,-1.752,-1.757,-1.761,-1.766,-1.770,     &
                    -1.775,-1.780,-1.784,-1.789,-1.793,-1.798,-1.802,     &
                    -1.807,-1.811,-1.816,-1.820,-1.825,-1.829,-1.834,     &
                    -1.838,-1.842,-1.847,-1.851,-1.856,-1.860,-1.864,     &
                    -1.869,-1.873,-1.877,-1.882,-1.886,-1.890,-1.895,     &
                    -1.899,-1.903,-1.908,-1.912,-1.916,-1.920,-1.925,     &
                    -1.929,-1.933,-1.937,-1.941,-1.946,-1.950,-1.954,     &
                    -1.958,-1.962,-1.966,-1.971,-1.975,-1.979,-1.983,     &
                    -1.987,-1.991,-1.995,-2.000,-2.004,-2.008,-2.012,     &
                    -2.016,-2.020,-2.024,-2.028,-2.032,-2.036,-2.040,     &
                    -2.044/)
!
! *** khso4
!
    bnc18m(1:561) = (/-0.047,-0.099,-0.124,-0.141,-0.154,-0.165,-0.173,   &
                    -0.181,-0.187,-0.193,-0.198,-0.202,-0.206,-0.209,     &
                    -0.212,-0.215,-0.217,-0.220,-0.221,-0.223,-0.225,     &
                    -0.226,-0.227,-0.228,-0.229,-0.229,-0.230,-0.230,     &
                    -0.230,-0.230,-0.230,-0.230,-0.230,-0.229,-0.229,     &
                    -0.228,-0.228,-0.227,-0.226,-0.225,-0.224,-0.223,     &
                    -0.222,-0.221,-0.220,-0.218,-0.217,-0.215,-0.214,     &
                    -0.212,-0.210,-0.209,-0.207,-0.205,-0.203,-0.201,     &
                    -0.199,-0.197,-0.195,-0.193,-0.191,-0.189,-0.187,     &
                    -0.185,-0.182,-0.180,-0.178,-0.175,-0.173,-0.171,     &
                    -0.168,-0.166,-0.163,-0.160,-0.158,-0.155,-0.153,     &
                    -0.150,-0.147,-0.144,-0.142,-0.139,-0.136,-0.133,     &
                    -0.130,-0.127,-0.124,-0.121,-0.118,-0.115,-0.112,     &
                    -0.109,-0.106,-0.103,-0.100,-0.097,-0.094,-0.091,     &
                    -0.087,-0.084,-0.081,-0.078,-0.074,-0.071,-0.068,     &
                    -0.064,-0.061,-0.058,-0.054,-0.051,-0.047,-0.044,     &
                    -0.041,-0.037,-0.034,-0.030,-0.027,-0.023,-0.020,     &
                    -0.017,-0.013,-0.010,-0.006,-0.003,0.001,0.004,0.008, &
                    0.011,0.015,0.018,0.021,0.025,0.028,0.032,0.035,0.039,&
                    0.042,0.046,0.049,0.052,0.056,0.059,0.063,0.066,0.069,&
                    0.073,0.076,0.079,0.083,0.086,0.090,0.093,0.096,0.100,&
                    0.103,0.106,0.110,0.113,0.116,0.119,0.123,0.126,0.129,&
                    0.133,0.136,0.139,0.142,0.146,0.149,0.152,0.155,0.158,&
                    0.162,0.165,0.168,0.171,0.174,0.177,0.181,0.184,0.187,&
                    0.190,0.193,0.196,0.199,0.203,0.206,0.209,0.212,0.215,&
                    0.218,0.221,0.224,0.227,0.230,0.233,0.236,0.239,0.242,&
                    0.245,0.248,0.251,0.254,0.257,0.260,0.263,0.266,0.269,&
                    0.272,0.275,0.278,0.281,0.284,0.287,0.290,0.292,0.295,&
                    0.298,0.301,0.304,0.307,0.310,0.313,0.315,0.318,0.321,&
                    0.324,0.327,0.330,0.332,0.335,0.338,0.341,0.343,0.346,&
                    0.349,0.352,0.355,0.357,0.360,0.363,0.366,0.368,0.371,&
                    0.374,0.376,0.379,0.382,0.384,0.387,0.390,0.393,0.395,&
                    0.398,0.400,0.403,0.406,0.408,0.411,0.414,0.416,0.419,&
                    0.422,0.424,0.427,0.429,0.432,0.434,0.437,0.440,0.442,&
                    0.445,0.447,0.450,0.452,0.455,0.457,0.460,0.463,0.465,&
                    0.468,0.470,0.473,0.475,0.478,0.480,0.483,0.485,0.487,&
                    0.490,0.492,0.495,0.497,0.500,0.502,0.505,0.507,0.509,&
                    0.512,0.514,0.517,0.519,0.522,0.524,0.526,0.529,0.531,&
                    0.533,0.536,0.538,0.541,0.543,0.545,0.548,0.550,0.552,&
                    0.555,0.557,0.559,0.562,0.564,0.566,0.569,0.571,0.573,&
                    0.575,0.578,0.580,0.582,0.585,0.587,0.589,0.591,0.594,&
                    0.596,0.598,0.600,0.603,0.605,0.607,0.609,0.612,0.614,&
                    0.616,0.618,0.621,0.623,0.625,0.627,0.629,0.632,0.634,&
                    0.636,0.638,0.640,0.642,0.645,0.647,0.649,0.651,0.653,&
                    0.655,0.658,0.660,0.662,0.664,0.666,0.668,0.670,0.672,&
                    0.675,0.677,0.679,0.681,0.683,0.685,0.687,0.689,0.691,&
                    0.693,0.695,0.698,0.700,0.702,0.704,0.706,0.708,0.710,&
                    0.712,0.714,0.716,0.718,0.720,0.722,0.724,0.726,0.728,&
                    0.730,0.732,0.734,0.756,0.775,0.795,0.814,0.832,0.851,&
                    0.869,0.886,0.904,0.921,0.938,0.955,0.972,0.988,1.004,&
                    1.020,1.036,1.051,1.066,1.081,1.096,1.111,1.125,1.140,&
                    1.154,1.168,1.181,1.195,1.208,1.222,1.235,1.248,1.261,&
                    1.273,1.286,1.298,1.311,1.323,1.335,1.347,1.358,1.370,&
                    1.381,1.393,1.404,1.415,1.426,1.437,1.448,1.459,1.469,&
                    1.480,1.490,1.500,1.510,1.521,1.531,1.540,1.550,1.560,&
                    1.570,1.579,1.589,1.598,1.607,1.616,1.625,1.635,1.643,&
                    1.652,1.661,1.670,1.679,1.687,1.696,1.704,1.712,1.721,&
                    1.729,1.737,1.745,1.753,1.761,1.769,1.777,1.785,1.792,&
                    1.800,1.808,1.815,1.823,1.830,1.838,1.845,1.852,1.859,&
                    1.866,1.874,1.881,1.888,1.895,1.901,1.908,1.915,1.922,&
                    1.929,1.935,1.942,1.948,1.955,1.961,1.968,1.974,1.980,&
                    1.987,1.993,1.999,2.005,2.011,2.018,2.024,2.030,2.036,&
                    2.041,2.047,2.053,2.059,2.065,2.070,2.076,2.082,2.087,&
                    2.093,2.099,2.104,2.110,2.115,2.120,2.126,2.131,2.136,&
                    2.142,2.147,2.152,2.157,2.163,2.168,2.173,2.178,2.183,&
                    2.188,2.193,2.198,2.203,2.208,2.212,2.217,2.222,2.227,&
                    2.232,2.236/)
!
! *** kno3
!
    bnc19m(1:561) = (/-0.049,-0.112,-0.147,-0.173,-0.194,-0.213,-0.230,   &
                    -0.245,-0.259,-0.272,-0.285,-0.297,-0.308,-0.319,     &
                    -0.329,-0.339,-0.348,-0.358,-0.367,-0.376,-0.384,     &
                    -0.392,-0.401,-0.408,-0.416,-0.424,-0.431,-0.439,     &
                    -0.446,-0.453,-0.460,-0.466,-0.473,-0.479,-0.486,     &
                    -0.492,-0.498,-0.504,-0.510,-0.516,-0.522,-0.528,     &
                    -0.533,-0.539,-0.544,-0.550,-0.555,-0.560,-0.565,     &
                    -0.571,-0.576,-0.580,-0.585,-0.590,-0.595,-0.600,     &
                    -0.604,-0.609,-0.613,-0.618,-0.622,-0.627,-0.631,     &
                    -0.635,-0.640,-0.644,-0.648,-0.652,-0.656,-0.660,     &
                    -0.664,-0.668,-0.672,-0.676,-0.680,-0.684,-0.688,     &
                    -0.692,-0.696,-0.700,-0.704,-0.707,-0.711,-0.715,     &
                    -0.719,-0.722,-0.726,-0.730,-0.734,-0.737,-0.741,     &
                    -0.745,-0.748,-0.752,-0.755,-0.759,-0.763,-0.766,     &
                    -0.770,-0.773,-0.777,-0.780,-0.784,-0.788,-0.791,     &
                    -0.795,-0.798,-0.801,-0.805,-0.808,-0.812,-0.815,     &
                    -0.819,-0.822,-0.825,-0.829,-0.832,-0.835,-0.839,     &
                    -0.842,-0.845,-0.849,-0.852,-0.855,-0.858,-0.862,     &
                    -0.865,-0.868,-0.871,-0.874,-0.878,-0.881,-0.884,     &
                    -0.887,-0.890,-0.893,-0.896,-0.899,-0.902,-0.905,     &
                    -0.908,-0.911,-0.914,-0.917,-0.920,-0.923,-0.926,     &
                    -0.929,-0.932,-0.935,-0.937,-0.940,-0.943,-0.946,     &
                    -0.949,-0.951,-0.954,-0.957,-0.960,-0.963,-0.965,     &
                    -0.968,-0.971,-0.973,-0.976,-0.979,-0.981,-0.984,     &
                    -0.987,-0.989,-0.992,-0.994,-0.997,-0.999,-1.002,     &
                    -1.005,-1.007,-1.010,-1.012,-1.015,-1.017,-1.020,     &
                    -1.022,-1.024,-1.027,-1.029,-1.032,-1.034,-1.036,     &
                    -1.039,-1.041,-1.044,-1.046,-1.048,-1.051,-1.053,     &
                    -1.055,-1.057,-1.060,-1.062,-1.064,-1.067,-1.069,     &
                    -1.071,-1.073,-1.075,-1.078,-1.080,-1.082,-1.084,     &
                    -1.086,-1.089,-1.091,-1.093,-1.095,-1.097,-1.099,     &
                    -1.101,-1.103,-1.106,-1.108,-1.110,-1.112,-1.114,     &
                    -1.116,-1.118,-1.120,-1.122,-1.124,-1.126,-1.128,     &
                    -1.130,-1.132,-1.134,-1.136,-1.138,-1.140,-1.142,     &
                    -1.144,-1.145,-1.147,-1.149,-1.151,-1.153,-1.155,     &
                    -1.157,-1.159,-1.160,-1.162,-1.164,-1.166,-1.168,     &
                    -1.170,-1.171,-1.173,-1.175,-1.177,-1.179,-1.180,     &
                    -1.182,-1.184,-1.186,-1.187,-1.189,-1.191,-1.193,     &
                    -1.194,-1.196,-1.198,-1.199,-1.201,-1.203,-1.205,     &
                    -1.206,-1.208,-1.210,-1.211,-1.213,-1.214,-1.216,     &
                    -1.218,-1.219,-1.221,-1.223,-1.224,-1.226,-1.227,     &
                    -1.229,-1.231,-1.232,-1.234,-1.235,-1.237,-1.238,     &
                    -1.240,-1.241,-1.243,-1.244,-1.246,-1.248,-1.249,     &
                    -1.251,-1.252,-1.254,-1.255,-1.256,-1.258,-1.259,     &
                    -1.261,-1.262,-1.264,-1.265,-1.267,-1.268,-1.270,     &
                    -1.271,-1.272,-1.274,-1.275,-1.277,-1.278,-1.279,     &
                    -1.281,-1.282,-1.284,-1.285,-1.286,-1.288,-1.289,     &
                    -1.290,-1.292,-1.293,-1.294,-1.296,-1.297,-1.298,     &
                    -1.300,-1.301,-1.302,-1.304,-1.305,-1.306,-1.308,     &
                    -1.309,-1.310,-1.311,-1.313,-1.314,-1.315,-1.316,     &
                    -1.318,-1.319,-1.320,-1.321,-1.323,-1.324,-1.325,     &
                    -1.326,-1.328,-1.329,-1.330,-1.331,-1.332,-1.334,     &
                    -1.335,-1.336,-1.337,-1.338,-1.340,-1.341,-1.342,     &
                    -1.343,-1.344,-1.345,-1.347,-1.348,-1.349,-1.350,     &
                    -1.351,-1.352,-1.353,-1.355,-1.356,-1.357,-1.358,     &
                    -1.359,-1.360,-1.361,-1.362,-1.364,-1.365,-1.366,     &
                    -1.367,-1.368,-1.369,-1.370,-1.371,-1.372,-1.373,     &
                    -1.374,-1.386,-1.396,-1.405,-1.415,-1.424,-1.433,     &
                    -1.442,-1.450,-1.458,-1.466,-1.474,-1.482,-1.489,     &
                    -1.496,-1.503,-1.510,-1.516,-1.523,-1.529,-1.535,     &
                    -1.541,-1.547,-1.553,-1.558,-1.564,-1.569,-1.574,     &
                    -1.580,-1.585,-1.589,-1.594,-1.599,-1.604,-1.608,     &
                    -1.613,-1.617,-1.621,-1.626,-1.630,-1.634,-1.638,     &
                    -1.642,-1.646,-1.650,-1.654,-1.657,-1.661,-1.665,     &
                    -1.668,-1.672,-1.675,-1.679,-1.682,-1.685,-1.689,     &
                    -1.692,-1.695,-1.698,-1.701,-1.705,-1.708,-1.711,     &
                    -1.714,-1.717,-1.720,-1.723,-1.725,-1.728,-1.731,     &
                    -1.734,-1.737,-1.739,-1.742,-1.745,-1.748,-1.750,     &
                    -1.753,-1.755,-1.758,-1.761,-1.763,-1.766,-1.768,     &
                    -1.771,-1.773,-1.776,-1.778,-1.781,-1.783,-1.785,     &
                    -1.788,-1.790,-1.792,-1.795,-1.797,-1.799,-1.802,     &
                    -1.804,-1.806,-1.809,-1.811,-1.813,-1.815,-1.817,     &
                    -1.820,-1.822,-1.824,-1.826,-1.828,-1.830,-1.833,     &
                    -1.835,-1.837,-1.839,-1.841,-1.843,-1.845,-1.847,     &
                    -1.849,-1.851,-1.854,-1.856,-1.858,-1.860,-1.862,     &
                    -1.864,-1.866,-1.868,-1.870,-1.872,-1.874,-1.876,     &
                    -1.878,-1.880,-1.881,-1.883,-1.885,-1.887,-1.889,     &
                    -1.891,-1.893,-1.895,-1.897,-1.899,-1.901,-1.903,     &
                    -1.905,-1.906,-1.908,-1.910,-1.912,-1.914,-1.916,     &
                    -1.918,-1.919,-1.921,-1.923,-1.925,-1.927,-1.929,     &
                    -1.930/)
!
! *** kcl
!
    bnc20m(1:561) = (/-0.047,-0.100,-0.126,-0.143,-0.156,-0.167,-0.176,   &
                    -0.183,-0.190,-0.195,-0.200,-0.205,-0.209,-0.212,     &
                    -0.215,-0.218,-0.221,-0.224,-0.226,-0.228,-0.230,     &
                    -0.232,-0.233,-0.235,-0.236,-0.238,-0.239,-0.240,     &
                    -0.241,-0.242,-0.243,-0.244,-0.245,-0.246,-0.246,     &
                    -0.247,-0.248,-0.248,-0.249,-0.249,-0.250,-0.250,     &
                    -0.251,-0.251,-0.251,-0.252,-0.252,-0.252,-0.253,     &
                    -0.253,-0.253,-0.253,-0.254,-0.254,-0.254,-0.254,     &
                    -0.254,-0.254,-0.254,-0.255,-0.255,-0.255,-0.255,     &
                    -0.255,-0.255,-0.255,-0.255,-0.255,-0.255,-0.255,     &
                    -0.255,-0.255,-0.255,-0.255,-0.255,-0.255,-0.255,     &
                    -0.255,-0.254,-0.254,-0.254,-0.254,-0.254,-0.254,     &
                    -0.254,-0.253,-0.253,-0.253,-0.253,-0.253,-0.252,     &
                    -0.252,-0.252,-0.252,-0.251,-0.251,-0.251,-0.250,     &
                    -0.250,-0.250,-0.249,-0.249,-0.249,-0.248,-0.248,     &
                    -0.248,-0.247,-0.247,-0.247,-0.246,-0.246,-0.245,     &
                    -0.245,-0.245,-0.244,-0.244,-0.243,-0.243,-0.243,     &
                    -0.242,-0.242,-0.241,-0.241,-0.240,-0.240,-0.240,     &
                    -0.239,-0.239,-0.238,-0.238,-0.237,-0.237,-0.236,     &
                    -0.236,-0.235,-0.235,-0.234,-0.234,-0.234,-0.233,     &
                    -0.233,-0.232,-0.232,-0.231,-0.231,-0.230,-0.230,     &
                    -0.229,-0.229,-0.228,-0.228,-0.227,-0.227,-0.226,     &
                    -0.226,-0.225,-0.225,-0.224,-0.224,-0.224,-0.223,     &
                    -0.223,-0.222,-0.222,-0.221,-0.221,-0.220,-0.220,     &
                    -0.219,-0.219,-0.218,-0.218,-0.217,-0.217,-0.216,     &
                    -0.216,-0.215,-0.215,-0.214,-0.214,-0.213,-0.213,     &
                    -0.212,-0.212,-0.211,-0.211,-0.210,-0.210,-0.209,     &
                    -0.209,-0.208,-0.208,-0.207,-0.207,-0.207,-0.206,     &
                    -0.206,-0.205,-0.205,-0.204,-0.204,-0.203,-0.203,     &
                    -0.202,-0.202,-0.201,-0.201,-0.200,-0.200,-0.199,     &
                    -0.199,-0.198,-0.198,-0.197,-0.197,-0.196,-0.196,     &
                    -0.195,-0.195,-0.194,-0.194,-0.194,-0.193,-0.193,     &
                    -0.192,-0.192,-0.191,-0.191,-0.190,-0.190,-0.189,     &
                    -0.189,-0.188,-0.188,-0.187,-0.187,-0.186,-0.186,     &
                    -0.186,-0.185,-0.185,-0.184,-0.184,-0.183,-0.183,     &
                    -0.182,-0.182,-0.181,-0.181,-0.180,-0.180,-0.180,     &
                    -0.179,-0.179,-0.178,-0.178,-0.177,-0.177,-0.176,     &
                    -0.176,-0.175,-0.175,-0.174,-0.174,-0.174,-0.173,     &
                    -0.173,-0.172,-0.172,-0.171,-0.171,-0.170,-0.170,     &
                    -0.170,-0.169,-0.169,-0.168,-0.168,-0.167,-0.167,     &
                    -0.166,-0.166,-0.166,-0.165,-0.165,-0.164,-0.164,     &
                    -0.163,-0.163,-0.162,-0.162,-0.162,-0.161,-0.161,     &
                    -0.160,-0.160,-0.159,-0.159,-0.159,-0.158,-0.158,     &
                    -0.157,-0.157,-0.156,-0.156,-0.156,-0.155,-0.155,     &
                    -0.154,-0.154,-0.153,-0.153,-0.153,-0.152,-0.152,     &
                    -0.151,-0.151,-0.150,-0.150,-0.150,-0.149,-0.149,     &
                    -0.148,-0.148,-0.148,-0.147,-0.147,-0.146,-0.146,     &
                    -0.145,-0.145,-0.145,-0.144,-0.144,-0.143,-0.143,     &
                    -0.143,-0.142,-0.142,-0.141,-0.141,-0.141,-0.140,     &
                    -0.140,-0.139,-0.139,-0.139,-0.138,-0.138,-0.137,     &
                    -0.137,-0.137,-0.136,-0.136,-0.135,-0.135,-0.135,     &
                    -0.134,-0.134,-0.133,-0.133,-0.133,-0.132,-0.132,     &
                    -0.131,-0.131,-0.131,-0.130,-0.130,-0.129,-0.129,     &
                    -0.129,-0.128,-0.128,-0.128,-0.127,-0.127,-0.126,     &
                    -0.126,-0.126,-0.125,-0.125,-0.124,-0.124,-0.124,     &
                    -0.123,-0.123,-0.123,-0.122,-0.122,-0.121,-0.121,     &
                    -0.121,-0.120,-0.120,-0.120,-0.119,-0.119,-0.118,     &
                    -0.118,-0.114,-0.111,-0.107,-0.103,-0.100,-0.097,     &
                    -0.093,-0.090,-0.087,-0.084,-0.080,-0.077,-0.074,     &
                    -0.071,-0.068,-0.065,-0.063,-0.060,-0.057,-0.054,     &
                    -0.052,-0.049,-0.046,-0.044,-0.041,-0.039,-0.036,     &
                    -0.034,-0.031,-0.029,-0.027,-0.024,-0.022,-0.020,     &
                    -0.018,-0.016,-0.013,-0.011,-0.009,-0.007,-0.005,     &
                    -0.003,-0.001,0.001,0.003,0.004,0.006,0.008,0.010,    &
                    0.012,0.013,0.015,0.017,0.018,0.020,0.022,0.023,0.025,&
                    0.026,0.028,0.030,0.031,0.032,0.034,0.035,0.037,0.038,&
                    0.040,0.041,0.042,0.044,0.045,0.046,0.047,0.049,0.050,&
                    0.051,0.052,0.053,0.055,0.056,0.057,0.058,0.059,0.060,&
                    0.061,0.062,0.063,0.064,0.065,0.066,0.067,0.068,0.069,&
                    0.070,0.071,0.072,0.073,0.074,0.075,0.075,0.076,0.077,&
                    0.078,0.079,0.079,0.080,0.081,0.082,0.083,0.083,0.084,&
                    0.085,0.085,0.086,0.087,0.087,0.088,0.089,0.089,0.090,&
                    0.091,0.091,0.092,0.092,0.093,0.094,0.094,0.095,0.095,&
                    0.096,0.096,0.097,0.097,0.098,0.098,0.099,0.099,0.100,&
                    0.100,0.100,0.101,0.101,0.102,0.102,0.103,0.103,0.103,&
                    0.104,0.104,0.104,0.105,0.105,0.105,0.106,0.106,0.106,&
                    0.107,0.107,0.107,0.108/)
!
! *** mgso4
!
    bnc21m(1:561) = (/-0.190,-0.411,-0.520,-0.597,-0.656,-0.705,-0.746,   &
                    -0.782,-0.814,-0.843,-0.868,-0.892,-0.914,-0.934,     &
                    -0.953,-0.970,-0.986,-1.002,-1.016,-1.030,-1.043,     &
                    -1.056,-1.068,-1.079,-1.090,-1.100,-1.110,-1.120,     &
                    -1.129,-1.138,-1.147,-1.155,-1.163,-1.171,-1.178,     &
                    -1.186,-1.193,-1.200,-1.206,-1.213,-1.219,-1.225,     &
                    -1.231,-1.237,-1.243,-1.249,-1.254,-1.260,-1.265,     &
                    -1.270,-1.275,-1.280,-1.285,-1.289,-1.294,-1.299,     &
                    -1.303,-1.307,-1.312,-1.316,-1.320,-1.324,-1.328,     &
                    -1.332,-1.336,-1.340,-1.344,-1.347,-1.351,-1.354,     &
                    -1.358,-1.361,-1.365,-1.368,-1.372,-1.375,-1.378,     &
                    -1.381,-1.384,-1.387,-1.390,-1.393,-1.396,-1.399,     &
                    -1.402,-1.405,-1.408,-1.411,-1.413,-1.416,-1.419,     &
                    -1.421,-1.424,-1.426,-1.429,-1.431,-1.434,-1.436,     &
                    -1.439,-1.441,-1.443,-1.446,-1.448,-1.450,-1.452,     &
                    -1.455,-1.457,-1.459,-1.461,-1.463,-1.465,-1.467,     &
                    -1.469,-1.471,-1.473,-1.475,-1.477,-1.479,-1.481,     &
                    -1.483,-1.485,-1.487,-1.489,-1.491,-1.493,-1.494,     &
                    -1.496,-1.498,-1.500,-1.501,-1.503,-1.505,-1.507,     &
                    -1.508,-1.510,-1.512,-1.513,-1.515,-1.517,-1.518,     &
                    -1.520,-1.521,-1.523,-1.525,-1.526,-1.528,-1.529,     &
                    -1.531,-1.532,-1.534,-1.535,-1.537,-1.538,-1.540,     &
                    -1.541,-1.543,-1.544,-1.546,-1.547,-1.548,-1.550,     &
                    -1.551,-1.553,-1.554,-1.555,-1.557,-1.558,-1.559,     &
                    -1.561,-1.562,-1.563,-1.565,-1.566,-1.567,-1.569,     &
                    -1.570,-1.571,-1.573,-1.574,-1.575,-1.576,-1.578,     &
                    -1.579,-1.580,-1.581,-1.583,-1.584,-1.585,-1.586,     &
                    -1.587,-1.589,-1.590,-1.591,-1.592,-1.593,-1.594,     &
                    -1.596,-1.597,-1.598,-1.599,-1.600,-1.601,-1.603,     &
                    -1.604,-1.605,-1.606,-1.607,-1.608,-1.609,-1.610,     &
                    -1.611,-1.613,-1.614,-1.615,-1.616,-1.617,-1.618,     &
                    -1.619,-1.620,-1.621,-1.622,-1.623,-1.624,-1.625,     &
                    -1.626,-1.627,-1.628,-1.630,-1.631,-1.632,-1.633,     &
                    -1.634,-1.635,-1.636,-1.637,-1.638,-1.639,-1.640,     &
                    -1.641,-1.642,-1.643,-1.644,-1.645,-1.646,-1.647,     &
                    -1.647,-1.648,-1.649,-1.650,-1.651,-1.652,-1.653,     &
                    -1.654,-1.655,-1.656,-1.657,-1.658,-1.659,-1.660,     &
                    -1.661,-1.662,-1.663,-1.664,-1.664,-1.665,-1.666,     &
                    -1.667,-1.668,-1.669,-1.670,-1.671,-1.672,-1.673,     &
                    -1.674,-1.674,-1.675,-1.676,-1.677,-1.678,-1.679,     &
                    -1.680,-1.681,-1.682,-1.682,-1.683,-1.684,-1.685,     &
                    -1.686,-1.687,-1.688,-1.688,-1.689,-1.690,-1.691,     &
                    -1.692,-1.693,-1.694,-1.694,-1.695,-1.696,-1.697,     &
                    -1.698,-1.699,-1.700,-1.700,-1.701,-1.702,-1.703,     &
                    -1.704,-1.705,-1.705,-1.706,-1.707,-1.708,-1.709,     &
                    -1.709,-1.710,-1.711,-1.712,-1.713,-1.713,-1.714,     &
                    -1.715,-1.716,-1.717,-1.718,-1.718,-1.719,-1.720,     &
                    -1.721,-1.722,-1.722,-1.723,-1.724,-1.725,-1.725,     &
                    -1.726,-1.727,-1.728,-1.729,-1.729,-1.730,-1.731,     &
                    -1.732,-1.733,-1.733,-1.734,-1.735,-1.736,-1.736,     &
                    -1.737,-1.738,-1.739,-1.739,-1.740,-1.741,-1.742,     &
                    -1.743,-1.743,-1.744,-1.745,-1.746,-1.746,-1.747,     &
                    -1.748,-1.749,-1.749,-1.750,-1.751,-1.752,-1.752,     &
                    -1.753,-1.754,-1.755,-1.755,-1.756,-1.757,-1.758,     &
                    -1.758,-1.759,-1.760,-1.760,-1.761,-1.762,-1.763,     &
                    -1.763,-1.764,-1.765,-1.766,-1.766,-1.767,-1.768,     &
                    -1.769,-1.769,-1.770,-1.771,-1.771,-1.772,-1.773,     &
                    -1.774,-1.781,-1.788,-1.795,-1.802,-1.809,-1.816,     &
                    -1.823,-1.830,-1.836,-1.843,-1.849,-1.856,-1.862,     &
                    -1.869,-1.875,-1.882,-1.888,-1.894,-1.901,-1.907,     &
                    -1.913,-1.919,-1.926,-1.932,-1.938,-1.944,-1.950,     &
                    -1.956,-1.962,-1.968,-1.974,-1.981,-1.987,-1.993,     &
                    -1.999,-2.004,-2.010,-2.016,-2.022,-2.028,-2.034,     &
                    -2.040,-2.046,-2.052,-2.058,-2.064,-2.069,-2.075,     &
                    -2.081,-2.087,-2.093,-2.099,-2.104,-2.110,-2.116,     &
                    -2.122,-2.128,-2.133,-2.139,-2.145,-2.151,-2.157,     &
                    -2.162,-2.168,-2.174,-2.180,-2.185,-2.191,-2.197,     &
                    -2.202,-2.208,-2.214,-2.220,-2.225,-2.231,-2.237,     &
                    -2.242,-2.248,-2.254,-2.260,-2.265,-2.271,-2.277,     &
                    -2.282,-2.288,-2.294,-2.299,-2.305,-2.311,-2.316,     &
                    -2.322,-2.328,-2.333,-2.339,-2.345,-2.350,-2.356,     &
                    -2.361,-2.367,-2.373,-2.378,-2.384,-2.390,-2.395,     &
                    -2.401,-2.407,-2.412,-2.418,-2.423,-2.429,-2.435,     &
                    -2.440,-2.446,-2.452,-2.457,-2.463,-2.468,-2.474,     &
                    -2.480,-2.485,-2.491,-2.496,-2.502,-2.508,-2.513,     &
                    -2.519,-2.524,-2.530,-2.536,-2.541,-2.547,-2.552,     &
                    -2.558,-2.564,-2.569,-2.575,-2.580,-2.586,-2.592,     &
                    -2.597,-2.603,-2.608,-2.614,-2.620,-2.625,-2.631,     &
                    -2.636,-2.642,-2.647,-2.653,-2.659,-2.664,-2.670,     &
                    -2.675,-2.681,-2.686,-2.692,-2.698,-2.703,-2.709,     &
                    -2.714/)
!
! *** mgno32
!
    bnc22m(1:561) = (/-0.093,-0.193,-0.238,-0.268,-0.289,-0.305,-0.318,   &
                    -0.328,-0.336,-0.343,-0.348,-0.352,-0.356,-0.359,     &
                    -0.361,-0.362,-0.364,-0.364,-0.364,-0.364,-0.364,     &
                    -0.364,-0.363,-0.362,-0.360,-0.359,-0.357,-0.355,     &
                    -0.354,-0.352,-0.349,-0.347,-0.345,-0.342,-0.340,     &
                    -0.337,-0.335,-0.332,-0.329,-0.326,-0.324,-0.321,     &
                    -0.318,-0.315,-0.312,-0.309,-0.306,-0.303,-0.300,     &
                    -0.297,-0.294,-0.290,-0.287,-0.284,-0.281,-0.278,     &
                    -0.275,-0.271,-0.268,-0.265,-0.262,-0.258,-0.255,     &
                    -0.252,-0.249,-0.245,-0.242,-0.239,-0.235,-0.232,     &
                    -0.229,-0.225,-0.222,-0.218,-0.215,-0.211,-0.208,     &
                    -0.204,-0.201,-0.197,-0.194,-0.190,-0.186,-0.183,     &
                    -0.179,-0.175,-0.172,-0.168,-0.164,-0.160,-0.156,     &
                    -0.152,-0.148,-0.144,-0.140,-0.136,-0.132,-0.128,     &
                    -0.124,-0.120,-0.116,-0.112,-0.107,-0.103,-0.099,     &
                    -0.095,-0.090,-0.086,-0.082,-0.078,-0.073,-0.069,     &
                    -0.065,-0.060,-0.056,-0.051,-0.047,-0.043,-0.038,     &
                    -0.034,-0.029,-0.025,-0.020,-0.016,-0.011,-0.007,     &
                    -0.003,0.002,0.006,0.011,0.015,0.020,0.024,0.029,     &
                    0.033,0.038,0.042,0.046,0.051,0.055,0.060,0.064,0.069,&
                    0.073,0.078,0.082,0.086,0.091,0.095,0.100,0.104,0.109,&
                    0.113,0.117,0.122,0.126,0.130,0.135,0.139,0.144,0.148,&
                    0.152,0.157,0.161,0.165,0.170,0.174,0.178,0.183,0.187,&
                    0.191,0.196,0.200,0.204,0.209,0.213,0.217,0.221,0.226,&
                    0.230,0.234,0.239,0.243,0.247,0.251,0.256,0.260,0.264,&
                    0.268,0.272,0.277,0.281,0.285,0.289,0.293,0.298,0.302,&
                    0.306,0.310,0.314,0.318,0.323,0.327,0.331,0.335,0.339,&
                    0.343,0.347,0.351,0.355,0.360,0.364,0.368,0.372,0.376,&
                    0.380,0.384,0.388,0.392,0.396,0.400,0.404,0.408,0.412,&
                    0.416,0.420,0.424,0.428,0.432,0.436,0.440,0.444,0.448,&
                    0.452,0.456,0.460,0.464,0.468,0.472,0.475,0.479,0.483,&
                    0.487,0.491,0.495,0.499,0.503,0.507,0.510,0.514,0.518,&
                    0.522,0.526,0.530,0.533,0.537,0.541,0.545,0.549,0.552,&
                    0.556,0.560,0.564,0.568,0.571,0.575,0.579,0.583,0.586,&
                    0.590,0.594,0.598,0.601,0.605,0.609,0.612,0.616,0.620,&
                    0.623,0.627,0.631,0.634,0.638,0.642,0.645,0.649,0.653,&
                    0.656,0.660,0.664,0.667,0.671,0.674,0.678,0.682,0.685,&
                    0.689,0.692,0.696,0.699,0.703,0.707,0.710,0.714,0.717,&
                    0.721,0.724,0.728,0.731,0.735,0.738,0.742,0.745,0.749,&
                    0.752,0.756,0.759,0.763,0.766,0.769,0.773,0.776,0.780,&
                    0.783,0.787,0.790,0.793,0.797,0.800,0.804,0.807,0.810,&
                    0.814,0.817,0.821,0.824,0.827,0.831,0.834,0.837,0.841,&
                    0.844,0.847,0.851,0.854,0.857,0.861,0.864,0.867,0.870,&
                    0.874,0.877,0.880,0.884,0.887,0.890,0.893,0.897,0.900,&
                    0.903,0.906,0.910,0.913,0.916,0.919,0.922,0.926,0.929,&
                    0.932,0.935,0.938,0.942,0.945,0.948,0.951,0.954,0.957,&
                    0.961,0.964,0.967,0.970,0.973,0.976,0.979,0.983,0.986,&
                    0.989,0.992,0.995,0.998,1.001,1.004,1.007,1.010,1.013,&
                    1.016,1.020,1.023,1.026,1.029,1.061,1.091,1.120,1.149,&
                    1.178,1.206,1.233,1.261,1.288,1.314,1.340,1.366,1.391,&
                    1.417,1.441,1.466,1.490,1.514,1.537,1.560,1.583,1.606,&
                    1.628,1.650,1.672,1.694,1.715,1.736,1.757,1.777,1.797,&
                    1.817,1.837,1.857,1.876,1.895,1.914,1.933,1.951,1.970,&
                    1.988,2.006,2.023,2.041,2.058,2.075,2.092,2.109,2.126,&
                    2.142,2.159,2.175,2.191,2.207,2.222,2.238,2.253,2.268,&
                    2.283,2.298,2.313,2.328,2.342,2.357,2.371,2.385,2.399,&
                    2.413,2.426,2.440,2.453,2.467,2.480,2.493,2.506,2.519,&
                    2.532,2.545,2.557,2.570,2.582,2.594,2.606,2.618,2.630,&
                    2.642,2.654,2.666,2.677,2.689,2.700,2.711,2.722,2.733,&
                    2.744,2.755,2.766,2.777,2.788,2.798,2.809,2.819,2.829,&
                    2.840,2.850,2.860,2.870,2.880,2.890,2.900,2.909,2.919,&
                    2.929,2.938,2.948,2.957,2.966,2.976,2.985,2.994,3.003,&
                    3.012,3.021,3.030,3.039,3.047,3.056,3.065,3.073,3.082,&
                    3.090,3.099,3.107,3.115,3.123,3.132,3.140,3.148,3.156,&
                    3.164,3.172,3.179,3.187,3.195,3.203,3.210,3.218,3.225,&
                    3.233,3.240,3.248,3.255,3.262,3.270,3.277,3.284,3.291,&
                    3.298,3.305,3.312,3.319/)
!
! *** mgcl2
!
    bnc23m(1:561) = (/-0.092,-0.190,-0.233,-0.261,-0.280,-0.295,-0.306,   &
                    -0.314,-0.321,-0.326,-0.330,-0.333,-0.335,-0.336,     &
                    -0.336,-0.336,-0.336,-0.335,-0.334,-0.332,-0.330,     &
                    -0.328,-0.326,-0.323,-0.320,-0.317,-0.314,-0.311,     &
                    -0.308,-0.304,-0.300,-0.297,-0.293,-0.289,-0.285,     &
                    -0.281,-0.277,-0.272,-0.268,-0.264,-0.259,-0.255,     &
                    -0.251,-0.246,-0.242,-0.237,-0.233,-0.228,-0.224,     &
                    -0.219,-0.215,-0.210,-0.205,-0.201,-0.196,-0.192,     &
                    -0.187,-0.182,-0.178,-0.173,-0.168,-0.164,-0.159,     &
                    -0.154,-0.150,-0.145,-0.140,-0.135,-0.131,-0.126,     &
                    -0.121,-0.116,-0.111,-0.106,-0.102,-0.097,-0.092,     &
                    -0.087,-0.082,-0.077,-0.072,-0.067,-0.062,-0.056,     &
                    -0.051,-0.046,-0.041,-0.036,-0.030,-0.025,-0.019,     &
                    -0.014,-0.009,-0.003,0.002,0.008,0.013,0.019,0.025,   &
                    0.030,0.036,0.042,0.047,0.053,0.059,0.065,0.071,0.077,&
                    0.082,0.088,0.094,0.100,0.106,0.112,0.118,0.124,0.130,&
                    0.136,0.142,0.148,0.154,0.160,0.166,0.172,0.178,0.184,&
                    0.190,0.196,0.202,0.208,0.214,0.220,0.226,0.232,0.238,&
                    0.244,0.250,0.256,0.262,0.268,0.274,0.280,0.286,0.292,&
                    0.298,0.304,0.310,0.316,0.322,0.328,0.334,0.340,0.346,&
                    0.352,0.358,0.364,0.370,0.376,0.382,0.387,0.393,0.399,&
                    0.405,0.411,0.417,0.423,0.429,0.434,0.440,0.446,0.452,&
                    0.458,0.463,0.469,0.475,0.481,0.487,0.492,0.498,0.504,&
                    0.509,0.515,0.521,0.527,0.532,0.538,0.544,0.549,0.555,&
                    0.561,0.566,0.572,0.578,0.583,0.589,0.594,0.600,0.606,&
                    0.611,0.617,0.622,0.628,0.633,0.639,0.645,0.650,0.656,&
                    0.661,0.667,0.672,0.678,0.683,0.688,0.694,0.699,0.705,&
                    0.710,0.716,0.721,0.726,0.732,0.737,0.743,0.748,0.753,&
                    0.759,0.764,0.769,0.775,0.780,0.785,0.791,0.796,0.801,&
                    0.807,0.812,0.817,0.822,0.828,0.833,0.838,0.843,0.848,&
                    0.854,0.859,0.864,0.869,0.874,0.879,0.885,0.890,0.895,&
                    0.900,0.905,0.910,0.915,0.920,0.925,0.931,0.936,0.941,&
                    0.946,0.951,0.956,0.961,0.966,0.971,0.976,0.981,0.986,&
                    0.991,0.996,1.001,1.006,1.011,1.015,1.020,1.025,1.030,&
                    1.035,1.040,1.045,1.050,1.055,1.059,1.064,1.069,1.074,&
                    1.079,1.084,1.088,1.093,1.098,1.103,1.108,1.112,1.117,&
                    1.122,1.127,1.131,1.136,1.141,1.146,1.150,1.155,1.160,&
                    1.164,1.169,1.174,1.178,1.183,1.188,1.192,1.197,1.201,&
                    1.206,1.211,1.215,1.220,1.224,1.229,1.234,1.238,1.243,&
                    1.247,1.252,1.256,1.261,1.265,1.270,1.274,1.279,1.283,&
                    1.288,1.292,1.297,1.301,1.306,1.310,1.315,1.319,1.323,&
                    1.328,1.332,1.337,1.341,1.345,1.350,1.354,1.359,1.363,&
                    1.367,1.372,1.376,1.380,1.385,1.389,1.393,1.398,1.402,&
                    1.406,1.410,1.415,1.419,1.423,1.428,1.432,1.436,1.440,&
                    1.444,1.449,1.453,1.457,1.461,1.466,1.470,1.474,1.478,&
                    1.482,1.486,1.491,1.495,1.499,1.503,1.507,1.511,1.515,&
                    1.519,1.524,1.528,1.532,1.536,1.540,1.544,1.548,1.552,&
                    1.556,1.560,1.564,1.568,1.611,1.651,1.690,1.728,1.765,&
                    1.802,1.839,1.875,1.911,1.945,1.980,2.014,2.048,2.081,&
                    2.113,2.146,2.177,2.209,2.240,2.270,2.300,2.330,2.360,&
                    2.389,2.417,2.446,2.474,2.502,2.529,2.556,2.583,2.609,&
                    2.635,2.661,2.686,2.712,2.737,2.761,2.786,2.810,2.834,&
                    2.857,2.881,2.904,2.927,2.950,2.972,2.994,3.016,3.038,&
                    3.060,3.081,3.102,3.123,3.144,3.164,3.184,3.205,3.225,&
                    3.244,3.264,3.283,3.302,3.322,3.340,3.359,3.378,3.396,&
                    3.414,3.432,3.450,3.468,3.485,3.503,3.520,3.537,3.554,&
                    3.571,3.588,3.604,3.621,3.637,3.653,3.669,3.685,3.701,&
                    3.717,3.732,3.748,3.763,3.778,3.793,3.808,3.823,3.838,&
                    3.852,3.867,3.881,3.896,3.910,3.924,3.938,3.952,3.966,&
                    3.979,3.993,4.006,4.020,4.033,4.046,4.059,4.072,4.085,&
                    4.098,4.111,4.123,4.136,4.148,4.161,4.173,4.185,4.197,&
                    4.210,4.222,4.233,4.245,4.257,4.269,4.280,4.292,4.303,&
                    4.315,4.326,4.337,4.348,4.359,4.370,4.381,4.392,4.403,&
                    4.414,4.424,4.435,4.446,4.456,4.466,4.477,4.487,4.497,&
                    4.507,4.518,4.528,4.538,4.548,4.557,4.567,4.577,4.587,&
                    4.596,4.606,4.615/)
!
! *** find position in arrays for bincoef
!
    if ( ionic<=0.200000E+02 ) then
      ipos = min(nint(0.200000E+02*ionic)+1,400)
    else
      ipos = 400 + nint(0.200000E+01*ionic-0.400000E+02)
    end if
    ipos = min(ipos,561)
!
! *** assign values to return array
!
    binarr(01) = bnc01m(ipos)
    binarr(02) = bnc02m(ipos)
    binarr(03) = bnc03m(ipos)
    binarr(04) = bnc04m(ipos)
    binarr(05) = bnc05m(ipos)
    binarr(06) = bnc06m(ipos)
    binarr(07) = bnc07m(ipos)
    binarr(08) = bnc08m(ipos)
    binarr(09) = bnc09m(ipos)
    binarr(10) = bnc10m(ipos)
    binarr(11) = bnc11m(ipos)
    binarr(12) = bnc12m(ipos)
    binarr(13) = bnc13m(ipos)
    binarr(14) = bnc14m(ipos)
    binarr(15) = bnc15m(ipos)
    binarr(16) = bnc16m(ipos)
    binarr(17) = bnc17m(ipos)
    binarr(18) = bnc18m(ipos)
    binarr(19) = bnc19m(ipos)
    binarr(20) = bnc20m(ipos)
    binarr(21) = bnc21m(ipos)
    binarr(22) = bnc22m(ipos)
    binarr(23) = bnc23m(ipos)
  end subroutine km273
!
!======================================================================
!
! *** isorropia code
! *** subroutine km298
! *** calculates binary activity coefficients by kusik-meissner method.
!     the computations have been performed and the results are stored in
!     lookup tables. the ionic activity 'in' is input , and the array
!     'binarr' is returned with the binary coefficients.
!
!     temperature is 298k
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine km298(ionic,binarr)
    implicit none
    integer(ik4) :: ipos
    real(rkx) , dimension(23) :: binarr
    real(rkx) :: ionic
    real(rkx) , dimension(561) :: bnc01m , bnc02m , bnc03m , bnc04m ,     &
                                  bnc05m , bnc06m , bnc07m , bnc08m ,     &
                                  bnc09m , bnc10m , bnc11m , bnc12m ,     &
                                  bnc13m , bnc14m , bnc15m , bnc16m ,     &
                                  bnc17m , bnc18m , bnc19m , bnc20m ,     &
                                  bnc21m , bnc22m , bnc23m
!
! *** nacl
!
    bnc01m(1:561) = (/-0.045,-0.095,-0.117,-0.132,-0.142,-0.150,-0.157,   &
                    -0.162,-0.166,-0.170,-0.173,-0.175,-0.177,-0.179,     &
                    -0.180,-0.181,-0.182,-0.182,-0.183,-0.183,-0.183,     &
                    -0.183,-0.182,-0.182,-0.182,-0.181,-0.181,-0.180,     &
                    -0.179,-0.178,-0.178,-0.177,-0.176,-0.175,-0.174,     &
                    -0.173,-0.172,-0.170,-0.169,-0.168,-0.167,-0.166,     &
                    -0.164,-0.163,-0.162,-0.161,-0.159,-0.158,-0.157,     &
                    -0.155,-0.154,-0.153,-0.151,-0.150,-0.148,-0.147,     &
                    -0.146,-0.144,-0.143,-0.141,-0.140,-0.138,-0.137,     &
                    -0.136,-0.134,-0.133,-0.131,-0.130,-0.128,-0.127,     &
                    -0.125,-0.124,-0.122,-0.121,-0.119,-0.117,-0.116,     &
                    -0.114,-0.113,-0.111,-0.110,-0.108,-0.106,-0.105,     &
                    -0.103,-0.101,-0.100,-0.098,-0.096,-0.094,-0.093,     &
                    -0.091,-0.089,-0.087,-0.086,-0.084,-0.082,-0.080,     &
                    -0.078,-0.076,-0.075,-0.073,-0.071,-0.069,-0.067,     &
                    -0.065,-0.063,-0.061,-0.059,-0.057,-0.055,-0.053,     &
                    -0.051,-0.049,-0.047,-0.046,-0.044,-0.042,-0.040,     &
                    -0.038,-0.036,-0.034,-0.032,-0.030,-0.028,-0.026,     &
                    -0.024,-0.022,-0.019,-0.017,-0.015,-0.013,-0.011,     &
                    -0.009,-0.007,-0.005,-0.003,-0.001,0.001,0.003,0.005, &
                    0.007,0.009,0.011,0.013,0.015,0.017,0.019,0.021,0.023,&
                    0.025,0.027,0.029,0.031,0.033,0.035,0.037,0.039,0.041,&
                    0.043,0.045,0.047,0.049,0.050,0.052,0.054,0.056,0.058,&
                    0.060,0.062,0.064,0.066,0.068,0.070,0.072,0.074,0.076,&
                    0.078,0.080,0.082,0.084,0.086,0.088,0.090,0.091,0.093,&
                    0.095,0.097,0.099,0.101,0.103,0.105,0.107,0.109,0.111,&
                    0.113,0.114,0.116,0.118,0.120,0.122,0.124,0.126,0.128,&
                    0.130,0.131,0.133,0.135,0.137,0.139,0.141,0.143,0.145,&
                    0.146,0.148,0.150,0.152,0.154,0.156,0.158,0.159,0.161,&
                    0.163,0.165,0.167,0.169,0.170,0.172,0.174,0.176,0.178,&
                    0.180,0.181,0.183,0.185,0.187,0.189,0.190,0.192,0.194,&
                    0.196,0.198,0.199,0.201,0.203,0.205,0.206,0.208,0.210,&
                    0.212,0.214,0.215,0.217,0.219,0.221,0.222,0.224,0.226,&
                    0.228,0.229,0.231,0.233,0.235,0.236,0.238,0.240,0.242,&
                    0.243,0.245,0.247,0.248,0.250,0.252,0.254,0.255,0.257,&
                    0.259,0.260,0.262,0.264,0.265,0.267,0.269,0.271,0.272,&
                    0.274,0.276,0.277,0.279,0.281,0.282,0.284,0.286,0.287,&
                    0.289,0.291,0.292,0.294,0.296,0.297,0.299,0.301,0.302,&
                    0.304,0.305,0.307,0.309,0.310,0.312,0.314,0.315,0.317,&
                    0.318,0.320,0.322,0.323,0.325,0.327,0.328,0.330,0.331,&
                    0.333,0.335,0.336,0.338,0.339,0.341,0.343,0.344,0.346,&
                    0.347,0.349,0.350,0.352,0.354,0.355,0.357,0.358,0.360,&
                    0.361,0.363,0.364,0.366,0.368,0.369,0.371,0.372,0.374,&
                    0.375,0.377,0.378,0.380,0.381,0.383,0.384,0.386,0.388,&
                    0.389,0.391,0.392,0.394,0.395,0.397,0.398,0.400,0.401,&
                    0.403,0.404,0.406,0.407,0.409,0.410,0.412,0.413,0.415,&
                    0.416,0.418,0.419,0.421,0.422,0.423,0.425,0.426,0.428,&
                    0.429,0.431,0.432,0.434,0.435,0.437,0.438,0.440,0.441,&
                    0.442,0.444,0.445,0.447,0.448,0.450,0.451,0.466,0.480,&
                    0.494,0.508,0.522,0.535,0.548,0.561,0.574,0.586,0.599,&
                    0.611,0.624,0.636,0.648,0.659,0.671,0.683,0.694,0.705,&
                    0.716,0.727,0.738,0.749,0.760,0.770,0.781,0.791,0.801,&
                    0.811,0.821,0.831,0.841,0.851,0.861,0.870,0.879,0.889,&
                    0.898,0.907,0.916,0.925,0.934,0.943,0.952,0.961,0.969,&
                    0.978,0.986,0.995,1.003,1.011,1.019,1.028,1.036,1.044,&
                    1.051,1.059,1.067,1.075,1.082,1.090,1.098,1.105,1.113,&
                    1.120,1.127,1.135,1.142,1.149,1.156,1.163,1.170,1.177,&
                    1.184,1.191,1.198,1.204,1.211,1.218,1.224,1.231,1.237,&
                    1.244,1.250,1.257,1.263,1.269,1.276,1.282,1.288,1.294,&
                    1.300,1.307,1.313,1.319,1.325,1.330,1.336,1.342,1.348,&
                    1.354,1.360,1.365,1.371,1.377,1.382,1.388,1.393,1.399,&
                    1.404,1.410,1.415,1.421,1.426,1.431,1.437,1.442,1.447,&
                    1.453,1.458,1.463,1.468,1.473,1.478,1.483,1.488,1.493,&
                    1.498,1.503,1.508,1.513,1.518,1.523,1.528,1.532,1.537,&
                    1.542,1.547,1.551,1.556,1.561,1.565,1.570,1.575,1.579,&
                    1.584,1.588,1.593,1.597,1.602,1.606,1.611,1.615,1.620,&
                    1.624,1.628,1.633,1.637,1.641,1.645/)
!
! *** na2so4
!
    bnc02m(1:561) = (/-0.093,-0.202,-0.256,-0.295,-0.325,-0.349,-0.371,   &
                    -0.389,-0.405,-0.420,-0.434,-0.446,-0.457,-0.468,     &
                    -0.478,-0.487,-0.496,-0.504,-0.512,-0.520,-0.527,     &
                    -0.533,-0.540,-0.546,-0.552,-0.558,-0.563,-0.569,     &
                    -0.574,-0.579,-0.584,-0.588,-0.593,-0.597,-0.602,     &
                    -0.606,-0.610,-0.614,-0.618,-0.621,-0.625,-0.628,     &
                    -0.632,-0.635,-0.639,-0.642,-0.645,-0.648,-0.651,     &
                    -0.654,-0.657,-0.660,-0.663,-0.665,-0.668,-0.671,     &
                    -0.673,-0.676,-0.678,-0.681,-0.683,-0.686,-0.688,     &
                    -0.690,-0.692,-0.695,-0.697,-0.699,-0.701,-0.703,     &
                    -0.705,-0.707,-0.709,-0.711,-0.713,-0.715,-0.717,     &
                    -0.719,-0.721,-0.723,-0.724,-0.726,-0.728,-0.730,     &
                    -0.731,-0.733,-0.735,-0.737,-0.738,-0.740,-0.741,     &
                    -0.743,-0.745,-0.746,-0.748,-0.749,-0.751,-0.752,     &
                    -0.754,-0.755,-0.757,-0.758,-0.760,-0.761,-0.763,     &
                    -0.764,-0.766,-0.767,-0.768,-0.770,-0.771,-0.772,     &
                    -0.774,-0.775,-0.776,-0.778,-0.779,-0.780,-0.782,     &
                    -0.783,-0.784,-0.785,-0.787,-0.788,-0.789,-0.790,     &
                    -0.791,-0.793,-0.794,-0.795,-0.796,-0.797,-0.798,     &
                    -0.800,-0.801,-0.802,-0.803,-0.804,-0.805,-0.806,     &
                    -0.807,-0.808,-0.810,-0.811,-0.812,-0.813,-0.814,     &
                    -0.815,-0.816,-0.817,-0.818,-0.819,-0.820,-0.821,     &
                    -0.822,-0.823,-0.824,-0.825,-0.826,-0.827,-0.828,     &
                    -0.829,-0.830,-0.831,-0.832,-0.832,-0.833,-0.834,     &
                    -0.835,-0.836,-0.837,-0.838,-0.839,-0.840,-0.841,     &
                    -0.841,-0.842,-0.843,-0.844,-0.845,-0.846,-0.847,     &
                    -0.847,-0.848,-0.849,-0.850,-0.851,-0.852,-0.852,     &
                    -0.853,-0.854,-0.855,-0.856,-0.856,-0.857,-0.858,     &
                    -0.859,-0.860,-0.860,-0.861,-0.862,-0.863,-0.864,     &
                    -0.864,-0.865,-0.866,-0.867,-0.867,-0.868,-0.869,     &
                    -0.869,-0.870,-0.871,-0.872,-0.872,-0.873,-0.874,     &
                    -0.875,-0.875,-0.876,-0.877,-0.877,-0.878,-0.879,     &
                    -0.879,-0.880,-0.881,-0.882,-0.882,-0.883,-0.884,     &
                    -0.884,-0.885,-0.886,-0.886,-0.887,-0.888,-0.888,     &
                    -0.889,-0.889,-0.890,-0.891,-0.891,-0.892,-0.893,     &
                    -0.893,-0.894,-0.895,-0.895,-0.896,-0.896,-0.897,     &
                    -0.898,-0.898,-0.899,-0.899,-0.900,-0.901,-0.901,     &
                    -0.902,-0.903,-0.903,-0.904,-0.904,-0.905,-0.905,     &
                    -0.906,-0.907,-0.907,-0.908,-0.908,-0.909,-0.910,     &
                    -0.910,-0.911,-0.911,-0.912,-0.912,-0.913,-0.913,     &
                    -0.914,-0.915,-0.915,-0.916,-0.916,-0.917,-0.917,     &
                    -0.918,-0.918,-0.919,-0.919,-0.920,-0.921,-0.921,     &
                    -0.922,-0.922,-0.923,-0.923,-0.924,-0.924,-0.925,     &
                    -0.925,-0.926,-0.926,-0.927,-0.927,-0.928,-0.928,     &
                    -0.929,-0.929,-0.930,-0.930,-0.931,-0.931,-0.932,     &
                    -0.932,-0.933,-0.933,-0.934,-0.934,-0.935,-0.935,     &
                    -0.936,-0.936,-0.937,-0.937,-0.938,-0.938,-0.939,     &
                    -0.939,-0.940,-0.940,-0.940,-0.941,-0.941,-0.942,     &
                    -0.942,-0.943,-0.943,-0.944,-0.944,-0.945,-0.945,     &
                    -0.946,-0.946,-0.946,-0.947,-0.947,-0.948,-0.948,     &
                    -0.949,-0.949,-0.950,-0.950,-0.950,-0.951,-0.951,     &
                    -0.952,-0.952,-0.953,-0.953,-0.954,-0.954,-0.954,     &
                    -0.955,-0.955,-0.956,-0.956,-0.957,-0.957,-0.957,     &
                    -0.958,-0.958,-0.959,-0.959,-0.959,-0.960,-0.960,     &
                    -0.961,-0.961,-0.962,-0.962,-0.962,-0.963,-0.963,     &
                    -0.964,-0.964,-0.964,-0.965,-0.965,-0.966,-0.966,     &
                    -0.966,-0.967,-0.967,-0.968,-0.968,-0.968,-0.969,     &
                    -0.969,-0.973,-0.977,-0.981,-0.984,-0.988,-0.991,     &
                    -0.995,-0.998,-1.001,-1.004,-1.007,-1.010,-1.013,     &
                    -1.016,-1.019,-1.022,-1.024,-1.027,-1.030,-1.032,     &
                    -1.035,-1.037,-1.040,-1.042,-1.045,-1.047,-1.049,     &
                    -1.052,-1.054,-1.056,-1.058,-1.061,-1.063,-1.065,     &
                    -1.067,-1.069,-1.071,-1.073,-1.075,-1.077,-1.079,     &
                    -1.080,-1.082,-1.084,-1.086,-1.088,-1.089,-1.091,     &
                    -1.093,-1.095,-1.096,-1.098,-1.100,-1.101,-1.103,     &
                    -1.104,-1.106,-1.108,-1.109,-1.111,-1.112,-1.114,     &
                    -1.115,-1.117,-1.118,-1.119,-1.121,-1.122,-1.124,     &
                    -1.125,-1.126,-1.128,-1.129,-1.130,-1.132,-1.133,     &
                    -1.134,-1.135,-1.137,-1.138,-1.139,-1.140,-1.142,     &
                    -1.143,-1.144,-1.145,-1.146,-1.148,-1.149,-1.150,     &
                    -1.151,-1.152,-1.153,-1.154,-1.155,-1.157,-1.158,     &
                    -1.159,-1.160,-1.161,-1.162,-1.163,-1.164,-1.165,     &
                    -1.166,-1.167,-1.168,-1.169,-1.170,-1.171,-1.172,     &
                    -1.173,-1.174,-1.175,-1.176,-1.177,-1.178,-1.179,     &
                    -1.180,-1.180,-1.181,-1.182,-1.183,-1.184,-1.185,     &
                    -1.186,-1.187,-1.188,-1.188,-1.189,-1.190,-1.191,     &
                    -1.192,-1.193,-1.193,-1.194,-1.195,-1.196,-1.197,     &
                    -1.198,-1.198,-1.199,-1.200,-1.201,-1.201,-1.202,     &
                    -1.203,-1.204,-1.205,-1.205,-1.206,-1.207,-1.208,     &
                    -1.208,-1.209,-1.210,-1.210,-1.211,-1.212,-1.213,     &
                    -1.213/)
!
! *** nano3
!
    bnc03m(1:561) = (/-0.047,-0.102,-0.129,-0.149,-0.164,-0.177,-0.188,   &
                    -0.198,-0.206,-0.214,-0.221,-0.228,-0.234,-0.239,     &
                    -0.245,-0.250,-0.255,-0.259,-0.263,-0.267,-0.271,     &
                    -0.275,-0.279,-0.282,-0.285,-0.289,-0.292,-0.295,     &
                    -0.298,-0.300,-0.303,-0.306,-0.308,-0.311,-0.313,     &
                    -0.316,-0.318,-0.320,-0.322,-0.324,-0.326,-0.329,     &
                    -0.331,-0.332,-0.334,-0.336,-0.338,-0.340,-0.342,     &
                    -0.343,-0.345,-0.347,-0.348,-0.350,-0.352,-0.353,     &
                    -0.355,-0.356,-0.358,-0.359,-0.360,-0.362,-0.363,     &
                    -0.365,-0.366,-0.367,-0.369,-0.370,-0.371,-0.372,     &
                    -0.374,-0.375,-0.376,-0.377,-0.378,-0.380,-0.381,     &
                    -0.382,-0.383,-0.384,-0.385,-0.386,-0.388,-0.389,     &
                    -0.390,-0.391,-0.392,-0.393,-0.394,-0.395,-0.396,     &
                    -0.397,-0.398,-0.399,-0.400,-0.401,-0.402,-0.403,     &
                    -0.404,-0.405,-0.406,-0.407,-0.408,-0.408,-0.409,     &
                    -0.410,-0.411,-0.412,-0.413,-0.414,-0.415,-0.416,     &
                    -0.416,-0.417,-0.418,-0.419,-0.420,-0.421,-0.422,     &
                    -0.422,-0.423,-0.424,-0.425,-0.426,-0.427,-0.427,     &
                    -0.428,-0.429,-0.430,-0.430,-0.431,-0.432,-0.433,     &
                    -0.434,-0.434,-0.435,-0.436,-0.437,-0.437,-0.438,     &
                    -0.439,-0.439,-0.440,-0.441,-0.442,-0.442,-0.443,     &
                    -0.444,-0.444,-0.445,-0.446,-0.447,-0.447,-0.448,     &
                    -0.449,-0.449,-0.450,-0.451,-0.451,-0.452,-0.453,     &
                    -0.453,-0.454,-0.454,-0.455,-0.456,-0.456,-0.457,     &
                    -0.458,-0.458,-0.459,-0.460,-0.460,-0.461,-0.461,     &
                    -0.462,-0.463,-0.463,-0.464,-0.464,-0.465,-0.466,     &
                    -0.466,-0.467,-0.467,-0.468,-0.468,-0.469,-0.470,     &
                    -0.470,-0.471,-0.471,-0.472,-0.472,-0.473,-0.474,     &
                    -0.474,-0.475,-0.475,-0.476,-0.476,-0.477,-0.477,     &
                    -0.478,-0.478,-0.479,-0.479,-0.480,-0.480,-0.481,     &
                    -0.482,-0.482,-0.483,-0.483,-0.484,-0.484,-0.485,     &
                    -0.485,-0.486,-0.486,-0.487,-0.487,-0.488,-0.488,     &
                    -0.489,-0.489,-0.490,-0.490,-0.491,-0.491,-0.491,     &
                    -0.492,-0.492,-0.493,-0.493,-0.494,-0.494,-0.495,     &
                    -0.495,-0.496,-0.496,-0.497,-0.497,-0.498,-0.498,     &
                    -0.498,-0.499,-0.499,-0.500,-0.500,-0.501,-0.501,     &
                    -0.502,-0.502,-0.502,-0.503,-0.503,-0.504,-0.504,     &
                    -0.505,-0.505,-0.505,-0.506,-0.506,-0.507,-0.507,     &
                    -0.508,-0.508,-0.508,-0.509,-0.509,-0.510,-0.510,     &
                    -0.511,-0.511,-0.511,-0.512,-0.512,-0.513,-0.513,     &
                    -0.513,-0.514,-0.514,-0.515,-0.515,-0.515,-0.516,     &
                    -0.516,-0.517,-0.517,-0.517,-0.518,-0.518,-0.518,     &
                    -0.519,-0.519,-0.520,-0.520,-0.520,-0.521,-0.521,     &
                    -0.522,-0.522,-0.522,-0.523,-0.523,-0.523,-0.524,     &
                    -0.524,-0.525,-0.525,-0.525,-0.526,-0.526,-0.526,     &
                    -0.527,-0.527,-0.527,-0.528,-0.528,-0.529,-0.529,     &
                    -0.529,-0.530,-0.530,-0.530,-0.531,-0.531,-0.531,     &
                    -0.532,-0.532,-0.532,-0.533,-0.533,-0.533,-0.534,     &
                    -0.534,-0.534,-0.535,-0.535,-0.535,-0.536,-0.536,     &
                    -0.536,-0.537,-0.537,-0.537,-0.538,-0.538,-0.538,     &
                    -0.539,-0.539,-0.539,-0.540,-0.540,-0.540,-0.541,     &
                    -0.541,-0.541,-0.542,-0.542,-0.542,-0.543,-0.543,     &
                    -0.543,-0.544,-0.544,-0.544,-0.545,-0.545,-0.545,     &
                    -0.546,-0.546,-0.546,-0.546,-0.547,-0.547,-0.547,     &
                    -0.548,-0.548,-0.548,-0.549,-0.549,-0.549,-0.549,     &
                    -0.550,-0.550,-0.550,-0.551,-0.551,-0.551,-0.552,     &
                    -0.552,-0.552,-0.552,-0.553,-0.553,-0.553,-0.554,     &
                    -0.554,-0.557,-0.560,-0.563,-0.565,-0.568,-0.570,     &
                    -0.573,-0.575,-0.578,-0.580,-0.583,-0.585,-0.587,     &
                    -0.589,-0.592,-0.594,-0.596,-0.598,-0.600,-0.602,     &
                    -0.604,-0.606,-0.608,-0.609,-0.611,-0.613,-0.615,     &
                    -0.617,-0.618,-0.620,-0.622,-0.623,-0.625,-0.627,     &
                    -0.628,-0.630,-0.631,-0.633,-0.634,-0.636,-0.637,     &
                    -0.639,-0.640,-0.642,-0.643,-0.644,-0.646,-0.647,     &
                    -0.648,-0.650,-0.651,-0.652,-0.653,-0.655,-0.656,     &
                    -0.657,-0.658,-0.660,-0.661,-0.662,-0.663,-0.664,     &
                    -0.665,-0.667,-0.668,-0.669,-0.670,-0.671,-0.672,     &
                    -0.673,-0.674,-0.675,-0.676,-0.677,-0.678,-0.679,     &
                    -0.680,-0.681,-0.682,-0.683,-0.684,-0.685,-0.686,     &
                    -0.687,-0.688,-0.689,-0.690,-0.691,-0.692,-0.692,     &
                    -0.693,-0.694,-0.695,-0.696,-0.697,-0.698,-0.698,     &
                    -0.699,-0.700,-0.701,-0.702,-0.703,-0.703,-0.704,     &
                    -0.705,-0.706,-0.706,-0.707,-0.708,-0.709,-0.709,     &
                    -0.710,-0.711,-0.712,-0.712,-0.713,-0.714,-0.715,     &
                    -0.715,-0.716,-0.717,-0.717,-0.718,-0.719,-0.719,     &
                    -0.720,-0.721,-0.721,-0.722,-0.723,-0.723,-0.724,     &
                    -0.725,-0.725,-0.726,-0.727,-0.727,-0.728,-0.729,     &
                    -0.729,-0.730,-0.730,-0.731,-0.732,-0.732,-0.733,     &
                    -0.733,-0.734,-0.735,-0.735,-0.736,-0.736,-0.737,     &
                    -0.737,-0.738,-0.739,-0.739,-0.740,-0.740,-0.741,     &
                    -0.741/)
!
! *** (nh4)2so4
!
    bnc04m(1:561) = (/-0.093,-0.203,-0.257,-0.296,-0.326,-0.351,-0.372,   &
                    -0.391,-0.408,-0.423,-0.436,-0.449,-0.460,-0.471,     &
                    -0.481,-0.491,-0.500,-0.508,-0.516,-0.524,-0.531,     &
                    -0.538,-0.545,-0.552,-0.558,-0.564,-0.569,-0.575,     &
                    -0.580,-0.585,-0.590,-0.595,-0.600,-0.605,-0.609,     &
                    -0.613,-0.618,-0.622,-0.626,-0.629,-0.633,-0.637,     &
                    -0.641,-0.644,-0.648,-0.651,-0.654,-0.658,-0.661,     &
                    -0.664,-0.667,-0.670,-0.673,-0.676,-0.678,-0.681,     &
                    -0.684,-0.687,-0.689,-0.692,-0.694,-0.697,-0.699,     &
                    -0.702,-0.704,-0.707,-0.709,-0.711,-0.713,-0.716,     &
                    -0.718,-0.720,-0.722,-0.724,-0.726,-0.728,-0.730,     &
                    -0.732,-0.734,-0.736,-0.738,-0.740,-0.742,-0.744,     &
                    -0.746,-0.748,-0.749,-0.751,-0.753,-0.755,-0.756,     &
                    -0.758,-0.760,-0.762,-0.763,-0.765,-0.767,-0.768,     &
                    -0.770,-0.772,-0.773,-0.775,-0.776,-0.778,-0.779,     &
                    -0.781,-0.782,-0.784,-0.786,-0.787,-0.788,-0.790,     &
                    -0.791,-0.793,-0.794,-0.796,-0.797,-0.799,-0.800,     &
                    -0.801,-0.803,-0.804,-0.805,-0.807,-0.808,-0.809,     &
                    -0.811,-0.812,-0.813,-0.815,-0.816,-0.817,-0.819,     &
                    -0.820,-0.821,-0.822,-0.824,-0.825,-0.826,-0.827,     &
                    -0.828,-0.830,-0.831,-0.832,-0.833,-0.834,-0.835,     &
                    -0.837,-0.838,-0.839,-0.840,-0.841,-0.842,-0.843,     &
                    -0.844,-0.846,-0.847,-0.848,-0.849,-0.850,-0.851,     &
                    -0.852,-0.853,-0.854,-0.855,-0.856,-0.857,-0.858,     &
                    -0.859,-0.860,-0.861,-0.862,-0.863,-0.864,-0.865,     &
                    -0.866,-0.867,-0.868,-0.869,-0.870,-0.871,-0.872,     &
                    -0.873,-0.874,-0.875,-0.876,-0.877,-0.878,-0.878,     &
                    -0.879,-0.880,-0.881,-0.882,-0.883,-0.884,-0.885,     &
                    -0.886,-0.886,-0.887,-0.888,-0.889,-0.890,-0.891,     &
                    -0.892,-0.893,-0.893,-0.894,-0.895,-0.896,-0.897,     &
                    -0.898,-0.898,-0.899,-0.900,-0.901,-0.902,-0.902,     &
                    -0.903,-0.904,-0.905,-0.906,-0.906,-0.907,-0.908,     &
                    -0.909,-0.910,-0.910,-0.911,-0.912,-0.913,-0.913,     &
                    -0.914,-0.915,-0.916,-0.916,-0.917,-0.918,-0.919,     &
                    -0.919,-0.920,-0.921,-0.922,-0.922,-0.923,-0.924,     &
                    -0.924,-0.925,-0.926,-0.927,-0.927,-0.928,-0.929,     &
                    -0.929,-0.930,-0.931,-0.931,-0.932,-0.933,-0.933,     &
                    -0.934,-0.935,-0.936,-0.936,-0.937,-0.938,-0.938,     &
                    -0.939,-0.940,-0.940,-0.941,-0.941,-0.942,-0.943,     &
                    -0.943,-0.944,-0.945,-0.945,-0.946,-0.947,-0.947,     &
                    -0.948,-0.949,-0.949,-0.950,-0.950,-0.951,-0.952,     &
                    -0.952,-0.953,-0.954,-0.954,-0.955,-0.955,-0.956,     &
                    -0.957,-0.957,-0.958,-0.958,-0.959,-0.960,-0.960,     &
                    -0.961,-0.961,-0.962,-0.962,-0.963,-0.964,-0.964,     &
                    -0.965,-0.965,-0.966,-0.967,-0.967,-0.968,-0.968,     &
                    -0.969,-0.969,-0.970,-0.970,-0.971,-0.972,-0.972,     &
                    -0.973,-0.973,-0.974,-0.974,-0.975,-0.975,-0.976,     &
                    -0.977,-0.977,-0.978,-0.978,-0.979,-0.979,-0.980,     &
                    -0.980,-0.981,-0.981,-0.982,-0.982,-0.983,-0.983,     &
                    -0.984,-0.984,-0.985,-0.986,-0.986,-0.987,-0.987,     &
                    -0.988,-0.988,-0.989,-0.989,-0.990,-0.990,-0.991,     &
                    -0.991,-0.992,-0.992,-0.993,-0.993,-0.994,-0.994,     &
                    -0.995,-0.995,-0.996,-0.996,-0.997,-0.997,-0.998,     &
                    -0.998,-0.998,-0.999,-0.999,-1.000,-1.000,-1.001,     &
                    -1.001,-1.002,-1.002,-1.003,-1.003,-1.004,-1.004,     &
                    -1.005,-1.005,-1.006,-1.006,-1.006,-1.007,-1.007,     &
                    -1.008,-1.008,-1.009,-1.009,-1.010,-1.010,-1.011,     &
                    -1.011,-1.016,-1.020,-1.024,-1.029,-1.033,-1.037,     &
                    -1.040,-1.044,-1.048,-1.052,-1.055,-1.059,-1.062,     &
                    -1.065,-1.069,-1.072,-1.075,-1.078,-1.081,-1.084,     &
                    -1.087,-1.090,-1.093,-1.096,-1.099,-1.102,-1.104,     &
                    -1.107,-1.110,-1.112,-1.115,-1.117,-1.120,-1.122,     &
                    -1.125,-1.127,-1.129,-1.132,-1.134,-1.136,-1.139,     &
                    -1.141,-1.143,-1.145,-1.147,-1.149,-1.151,-1.153,     &
                    -1.155,-1.157,-1.159,-1.161,-1.163,-1.165,-1.167,     &
                    -1.169,-1.171,-1.173,-1.174,-1.176,-1.178,-1.180,     &
                    -1.181,-1.183,-1.185,-1.187,-1.188,-1.190,-1.191,     &
                    -1.193,-1.195,-1.196,-1.198,-1.199,-1.201,-1.202,     &
                    -1.204,-1.205,-1.207,-1.208,-1.210,-1.211,-1.213,     &
                    -1.214,-1.216,-1.217,-1.218,-1.220,-1.221,-1.223,     &
                    -1.224,-1.225,-1.226,-1.228,-1.229,-1.230,-1.232,     &
                    -1.233,-1.234,-1.235,-1.237,-1.238,-1.239,-1.240,     &
                    -1.242,-1.243,-1.244,-1.245,-1.246,-1.247,-1.249,     &
                    -1.250,-1.251,-1.252,-1.253,-1.254,-1.255,-1.256,     &
                    -1.258,-1.259,-1.260,-1.261,-1.262,-1.263,-1.264,     &
                    -1.265,-1.266,-1.267,-1.268,-1.269,-1.270,-1.271,     &
                    -1.272,-1.273,-1.274,-1.275,-1.276,-1.277,-1.278,     &
                    -1.279,-1.280,-1.281,-1.282,-1.283,-1.283,-1.284,     &
                    -1.285,-1.286,-1.287,-1.288,-1.289,-1.290,-1.291,     &
                    -1.292,-1.292,-1.293,-1.294,-1.295,-1.296,-1.297,     &
                    -1.298/)
!
! *** nh4no3
!
    bnc05m(1:561) = (/-0.047,-0.104,-0.134,-0.155,-0.172,-0.187,-0.199,   &
                    -0.211,-0.221,-0.230,-0.239,-0.247,-0.255,-0.262,     &
                    -0.268,-0.275,-0.281,-0.287,-0.293,-0.298,-0.303,     &
                    -0.308,-0.313,-0.318,-0.323,-0.327,-0.332,-0.336,     &
                    -0.340,-0.344,-0.348,-0.352,-0.356,-0.359,-0.363,     &
                    -0.366,-0.370,-0.373,-0.377,-0.380,-0.383,-0.386,     &
                    -0.389,-0.392,-0.395,-0.398,-0.401,-0.404,-0.406,     &
                    -0.409,-0.412,-0.414,-0.417,-0.420,-0.422,-0.425,     &
                    -0.427,-0.429,-0.432,-0.434,-0.436,-0.439,-0.441,     &
                    -0.443,-0.445,-0.447,-0.450,-0.452,-0.454,-0.456,     &
                    -0.458,-0.460,-0.462,-0.464,-0.466,-0.468,-0.470,     &
                    -0.472,-0.474,-0.476,-0.478,-0.480,-0.481,-0.483,     &
                    -0.485,-0.487,-0.489,-0.491,-0.492,-0.494,-0.496,     &
                    -0.498,-0.500,-0.501,-0.503,-0.505,-0.507,-0.508,     &
                    -0.510,-0.512,-0.514,-0.515,-0.517,-0.519,-0.520,     &
                    -0.522,-0.524,-0.525,-0.527,-0.529,-0.530,-0.532,     &
                    -0.534,-0.535,-0.537,-0.538,-0.540,-0.542,-0.543,     &
                    -0.545,-0.546,-0.548,-0.550,-0.551,-0.553,-0.554,     &
                    -0.556,-0.557,-0.559,-0.560,-0.562,-0.563,-0.565,     &
                    -0.566,-0.568,-0.569,-0.571,-0.572,-0.573,-0.575,     &
                    -0.576,-0.578,-0.579,-0.580,-0.582,-0.583,-0.585,     &
                    -0.586,-0.587,-0.589,-0.590,-0.591,-0.593,-0.594,     &
                    -0.595,-0.597,-0.598,-0.599,-0.601,-0.602,-0.603,     &
                    -0.604,-0.606,-0.607,-0.608,-0.610,-0.611,-0.612,     &
                    -0.613,-0.615,-0.616,-0.617,-0.618,-0.619,-0.621,     &
                    -0.622,-0.623,-0.624,-0.625,-0.627,-0.628,-0.629,     &
                    -0.630,-0.631,-0.632,-0.634,-0.635,-0.636,-0.637,     &
                    -0.638,-0.639,-0.640,-0.642,-0.643,-0.644,-0.645,     &
                    -0.646,-0.647,-0.648,-0.649,-0.650,-0.651,-0.652,     &
                    -0.654,-0.655,-0.656,-0.657,-0.658,-0.659,-0.660,     &
                    -0.661,-0.662,-0.663,-0.664,-0.665,-0.666,-0.667,     &
                    -0.668,-0.669,-0.670,-0.671,-0.672,-0.673,-0.674,     &
                    -0.675,-0.676,-0.677,-0.678,-0.679,-0.680,-0.681,     &
                    -0.682,-0.683,-0.684,-0.685,-0.686,-0.687,-0.688,     &
                    -0.688,-0.689,-0.690,-0.691,-0.692,-0.693,-0.694,     &
                    -0.695,-0.696,-0.697,-0.698,-0.699,-0.699,-0.700,     &
                    -0.701,-0.702,-0.703,-0.704,-0.705,-0.706,-0.707,     &
                    -0.707,-0.708,-0.709,-0.710,-0.711,-0.712,-0.713,     &
                    -0.713,-0.714,-0.715,-0.716,-0.717,-0.718,-0.718,     &
                    -0.719,-0.720,-0.721,-0.722,-0.723,-0.723,-0.724,     &
                    -0.725,-0.726,-0.727,-0.727,-0.728,-0.729,-0.730,     &
                    -0.731,-0.731,-0.732,-0.733,-0.734,-0.735,-0.735,     &
                    -0.736,-0.737,-0.738,-0.738,-0.739,-0.740,-0.741,     &
                    -0.741,-0.742,-0.743,-0.744,-0.744,-0.745,-0.746,     &
                    -0.747,-0.747,-0.748,-0.749,-0.750,-0.750,-0.751,     &
                    -0.752,-0.753,-0.753,-0.754,-0.755,-0.755,-0.756,     &
                    -0.757,-0.758,-0.758,-0.759,-0.760,-0.760,-0.761,     &
                    -0.762,-0.762,-0.763,-0.764,-0.764,-0.765,-0.766,     &
                    -0.767,-0.767,-0.768,-0.769,-0.769,-0.770,-0.771,     &
                    -0.771,-0.772,-0.773,-0.773,-0.774,-0.775,-0.775,     &
                    -0.776,-0.777,-0.777,-0.778,-0.778,-0.779,-0.780,     &
                    -0.780,-0.781,-0.782,-0.782,-0.783,-0.784,-0.784,     &
                    -0.785,-0.785,-0.786,-0.787,-0.787,-0.788,-0.789,     &
                    -0.789,-0.790,-0.790,-0.791,-0.792,-0.792,-0.793,     &
                    -0.793,-0.794,-0.795,-0.795,-0.796,-0.796,-0.797,     &
                    -0.798,-0.798,-0.799,-0.799,-0.800,-0.801,-0.801,     &
                    -0.802,-0.802,-0.803,-0.804,-0.804,-0.805,-0.805,     &
                    -0.806,-0.812,-0.817,-0.823,-0.828,-0.833,-0.838,     &
                    -0.843,-0.848,-0.852,-0.857,-0.861,-0.866,-0.870,     &
                    -0.874,-0.878,-0.882,-0.886,-0.890,-0.893,-0.897,     &
                    -0.900,-0.904,-0.907,-0.911,-0.914,-0.917,-0.920,     &
                    -0.924,-0.927,-0.930,-0.933,-0.936,-0.938,-0.941,     &
                    -0.944,-0.947,-0.949,-0.952,-0.954,-0.957,-0.959,     &
                    -0.962,-0.964,-0.967,-0.969,-0.971,-0.973,-0.976,     &
                    -0.978,-0.980,-0.982,-0.984,-0.986,-0.988,-0.990,     &
                    -0.992,-0.994,-0.996,-0.998,-1.000,-1.002,-1.003,     &
                    -1.005,-1.007,-1.009,-1.010,-1.012,-1.014,-1.015,     &
                    -1.017,-1.019,-1.020,-1.022,-1.023,-1.025,-1.026,     &
                    -1.028,-1.029,-1.031,-1.032,-1.033,-1.035,-1.036,     &
                    -1.037,-1.039,-1.040,-1.041,-1.043,-1.044,-1.045,     &
                    -1.046,-1.048,-1.049,-1.050,-1.051,-1.052,-1.054,     &
                    -1.055,-1.056,-1.057,-1.058,-1.059,-1.060,-1.061,     &
                    -1.062,-1.063,-1.064,-1.066,-1.067,-1.068,-1.069,     &
                    -1.070,-1.070,-1.071,-1.072,-1.073,-1.074,-1.075,     &
                    -1.076,-1.077,-1.078,-1.079,-1.080,-1.081,-1.081,     &
                    -1.082,-1.083,-1.084,-1.085,-1.086,-1.086,-1.087,     &
                    -1.088,-1.089,-1.090,-1.090,-1.091,-1.092,-1.093,     &
                    -1.093,-1.094,-1.095,-1.096,-1.096,-1.097,-1.098,     &
                    -1.099,-1.099,-1.100,-1.101,-1.101,-1.102,-1.103,     &
                    -1.103,-1.104,-1.105,-1.105,-1.106,-1.107,-1.107,     &
                    -1.108/)
!
! *** nh4cl
!
    bnc06m(1:561) = (/-0.046,-0.098,-0.123,-0.140,-0.153,-0.163,-0.172,   &
                    -0.180,-0.186,-0.191,-0.196,-0.201,-0.205,-0.208,     &
                    -0.212,-0.215,-0.217,-0.220,-0.222,-0.224,-0.226,     &
                    -0.228,-0.230,-0.231,-0.233,-0.234,-0.235,-0.237,     &
                    -0.238,-0.239,-0.240,-0.241,-0.241,-0.242,-0.243,     &
                    -0.244,-0.244,-0.245,-0.246,-0.246,-0.247,-0.247,     &
                    -0.247,-0.248,-0.248,-0.249,-0.249,-0.249,-0.250,     &
                    -0.250,-0.250,-0.250,-0.251,-0.251,-0.251,-0.251,     &
                    -0.251,-0.251,-0.252,-0.252,-0.252,-0.252,-0.252,     &
                    -0.252,-0.252,-0.252,-0.252,-0.252,-0.252,-0.252,     &
                    -0.252,-0.252,-0.252,-0.252,-0.252,-0.252,-0.252,     &
                    -0.252,-0.252,-0.251,-0.251,-0.251,-0.251,-0.251,     &
                    -0.251,-0.250,-0.250,-0.250,-0.250,-0.250,-0.249,     &
                    -0.249,-0.249,-0.249,-0.248,-0.248,-0.248,-0.248,     &
                    -0.247,-0.247,-0.247,-0.246,-0.246,-0.246,-0.245,     &
                    -0.245,-0.245,-0.244,-0.244,-0.244,-0.243,-0.243,     &
                    -0.243,-0.242,-0.242,-0.241,-0.241,-0.241,-0.240,     &
                    -0.240,-0.239,-0.239,-0.239,-0.238,-0.238,-0.237,     &
                    -0.237,-0.236,-0.236,-0.236,-0.235,-0.235,-0.234,     &
                    -0.234,-0.233,-0.233,-0.233,-0.232,-0.232,-0.231,     &
                    -0.231,-0.230,-0.230,-0.229,-0.229,-0.228,-0.228,     &
                    -0.228,-0.227,-0.227,-0.226,-0.226,-0.225,-0.225,     &
                    -0.224,-0.224,-0.223,-0.223,-0.222,-0.222,-0.221,     &
                    -0.221,-0.220,-0.220,-0.220,-0.219,-0.219,-0.218,     &
                    -0.218,-0.217,-0.217,-0.216,-0.216,-0.215,-0.215,     &
                    -0.214,-0.214,-0.213,-0.213,-0.212,-0.212,-0.211,     &
                    -0.211,-0.210,-0.210,-0.210,-0.209,-0.209,-0.208,     &
                    -0.208,-0.207,-0.207,-0.206,-0.206,-0.205,-0.205,     &
                    -0.204,-0.204,-0.203,-0.203,-0.202,-0.202,-0.201,     &
                    -0.201,-0.200,-0.200,-0.199,-0.199,-0.198,-0.198,     &
                    -0.197,-0.197,-0.197,-0.196,-0.196,-0.195,-0.195,     &
                    -0.194,-0.194,-0.193,-0.193,-0.192,-0.192,-0.191,     &
                    -0.191,-0.190,-0.190,-0.189,-0.189,-0.188,-0.188,     &
                    -0.187,-0.187,-0.186,-0.186,-0.186,-0.185,-0.185,     &
                    -0.184,-0.184,-0.183,-0.183,-0.182,-0.182,-0.181,     &
                    -0.181,-0.180,-0.180,-0.179,-0.179,-0.178,-0.178,     &
                    -0.177,-0.177,-0.177,-0.176,-0.176,-0.175,-0.175,     &
                    -0.174,-0.174,-0.173,-0.173,-0.172,-0.172,-0.171,     &
                    -0.171,-0.170,-0.170,-0.170,-0.169,-0.169,-0.168,     &
                    -0.168,-0.167,-0.167,-0.166,-0.166,-0.165,-0.165,     &
                    -0.164,-0.164,-0.164,-0.163,-0.163,-0.162,-0.162,     &
                    -0.161,-0.161,-0.160,-0.160,-0.159,-0.159,-0.159,     &
                    -0.158,-0.158,-0.157,-0.157,-0.156,-0.156,-0.155,     &
                    -0.155,-0.154,-0.154,-0.154,-0.153,-0.153,-0.152,     &
                    -0.152,-0.151,-0.151,-0.150,-0.150,-0.149,-0.149,     &
                    -0.149,-0.148,-0.148,-0.147,-0.147,-0.146,-0.146,     &
                    -0.145,-0.145,-0.145,-0.144,-0.144,-0.143,-0.143,     &
                    -0.142,-0.142,-0.142,-0.141,-0.141,-0.140,-0.140,     &
                    -0.139,-0.139,-0.138,-0.138,-0.138,-0.137,-0.137,     &
                    -0.136,-0.136,-0.135,-0.135,-0.135,-0.134,-0.134,     &
                    -0.133,-0.133,-0.132,-0.132,-0.132,-0.131,-0.131,     &
                    -0.130,-0.130,-0.129,-0.129,-0.129,-0.128,-0.128,     &
                    -0.127,-0.127,-0.126,-0.126,-0.126,-0.125,-0.125,     &
                    -0.124,-0.124,-0.123,-0.123,-0.123,-0.122,-0.122,     &
                    -0.121,-0.121,-0.121,-0.120,-0.120,-0.119,-0.119,     &
                    -0.118,-0.118,-0.118,-0.117,-0.117,-0.116,-0.116,     &
                    -0.116,-0.115,-0.115,-0.114,-0.114,-0.114,-0.113,     &
                    -0.113,-0.108,-0.104,-0.100,-0.096,-0.092,-0.089,     &
                    -0.085,-0.081,-0.077,-0.074,-0.070,-0.066,-0.063,     &
                    -0.059,-0.055,-0.052,-0.049,-0.045,-0.042,-0.038,     &
                    -0.035,-0.032,-0.028,-0.025,-0.022,-0.019,-0.016,     &
                    -0.012,-0.009,-0.006,-0.003,0.000,0.003,0.006,0.009,  &
                    0.012,0.014,0.017,0.020,0.023,0.026,0.029,0.031,0.034,&
                    0.037,0.040,0.042,0.045,0.047,0.050,0.053,0.055,0.058,&
                    0.060,0.063,0.065,0.068,0.070,0.073,0.075,0.078,0.080,&
                    0.082,0.085,0.087,0.089,0.092,0.094,0.096,0.099,0.101,&
                    0.103,0.105,0.107,0.110,0.112,0.114,0.116,0.118,0.120,&
                    0.123,0.125,0.127,0.129,0.131,0.133,0.135,0.137,0.139,&
                    0.141,0.143,0.145,0.147,0.149,0.151,0.153,0.155,0.157,&
                    0.159,0.160,0.162,0.164,0.166,0.168,0.170,0.172,0.173,&
                    0.175,0.177,0.179,0.181,0.182,0.184,0.186,0.188,0.189,&
                    0.191,0.193,0.195,0.196,0.198,0.200,0.201,0.203,0.205,&
                    0.206,0.208,0.210,0.211,0.213,0.215,0.216,0.218,0.219,&
                    0.221,0.223,0.224,0.226,0.227,0.229,0.230,0.232,0.233,&
                    0.235,0.236,0.238,0.239,0.241,0.242,0.244,0.245,0.247,&
                    0.248,0.250,0.251,0.253,0.254,0.256,0.257,0.258,      &
                    0.260/)
!
! *** (2h ,so4)
!
    bnc07m(1:561) = (/-0.093,-0.202,-0.255,-0.293,-0.323,-0.347,-0.368,   &
                    -0.386,-0.402,-0.417,-0.430,-0.442,-0.453,-0.463,     &
                    -0.473,-0.482,-0.490,-0.498,-0.506,-0.513,-0.520,     &
                    -0.526,-0.532,-0.538,-0.544,-0.549,-0.555,-0.560,     &
                    -0.565,-0.569,-0.574,-0.578,-0.583,-0.587,-0.591,     &
                    -0.595,-0.598,-0.602,-0.606,-0.609,-0.613,-0.616,     &
                    -0.619,-0.622,-0.625,-0.628,-0.631,-0.634,-0.637,     &
                    -0.640,-0.642,-0.645,-0.648,-0.650,-0.653,-0.655,     &
                    -0.657,-0.660,-0.662,-0.664,-0.666,-0.669,-0.671,     &
                    -0.673,-0.675,-0.677,-0.679,-0.681,-0.683,-0.685,     &
                    -0.687,-0.688,-0.690,-0.692,-0.694,-0.695,-0.697,     &
                    -0.699,-0.701,-0.702,-0.704,-0.705,-0.707,-0.709,     &
                    -0.710,-0.712,-0.713,-0.715,-0.716,-0.718,-0.719,     &
                    -0.721,-0.722,-0.723,-0.725,-0.726,-0.727,-0.729,     &
                    -0.730,-0.731,-0.733,-0.734,-0.735,-0.737,-0.738,     &
                    -0.739,-0.740,-0.741,-0.743,-0.744,-0.745,-0.746,     &
                    -0.747,-0.748,-0.750,-0.751,-0.752,-0.753,-0.754,     &
                    -0.755,-0.756,-0.757,-0.758,-0.759,-0.760,-0.762,     &
                    -0.763,-0.764,-0.765,-0.766,-0.767,-0.768,-0.769,     &
                    -0.769,-0.770,-0.771,-0.772,-0.773,-0.774,-0.775,     &
                    -0.776,-0.777,-0.778,-0.779,-0.780,-0.781,-0.781,     &
                    -0.782,-0.783,-0.784,-0.785,-0.786,-0.787,-0.787,     &
                    -0.788,-0.789,-0.790,-0.791,-0.792,-0.792,-0.793,     &
                    -0.794,-0.795,-0.795,-0.796,-0.797,-0.798,-0.799,     &
                    -0.799,-0.800,-0.801,-0.802,-0.802,-0.803,-0.804,     &
                    -0.804,-0.805,-0.806,-0.807,-0.807,-0.808,-0.809,     &
                    -0.809,-0.810,-0.811,-0.811,-0.812,-0.813,-0.814,     &
                    -0.814,-0.815,-0.816,-0.816,-0.817,-0.817,-0.818,     &
                    -0.819,-0.819,-0.820,-0.821,-0.821,-0.822,-0.823,     &
                    -0.823,-0.824,-0.824,-0.825,-0.826,-0.826,-0.827,     &
                    -0.827,-0.828,-0.829,-0.829,-0.830,-0.830,-0.831,     &
                    -0.831,-0.832,-0.833,-0.833,-0.834,-0.834,-0.835,     &
                    -0.835,-0.836,-0.837,-0.837,-0.838,-0.838,-0.839,     &
                    -0.839,-0.840,-0.840,-0.841,-0.841,-0.842,-0.842,     &
                    -0.843,-0.843,-0.844,-0.845,-0.845,-0.846,-0.846,     &
                    -0.847,-0.847,-0.848,-0.848,-0.849,-0.849,-0.850,     &
                    -0.850,-0.851,-0.851,-0.851,-0.852,-0.852,-0.853,     &
                    -0.853,-0.854,-0.854,-0.855,-0.855,-0.856,-0.856,     &
                    -0.857,-0.857,-0.858,-0.858,-0.859,-0.859,-0.859,     &
                    -0.860,-0.860,-0.861,-0.861,-0.862,-0.862,-0.863,     &
                    -0.863,-0.863,-0.864,-0.864,-0.865,-0.865,-0.866,     &
                    -0.866,-0.866,-0.867,-0.867,-0.868,-0.868,-0.869,     &
                    -0.869,-0.869,-0.870,-0.870,-0.871,-0.871,-0.871,     &
                    -0.872,-0.872,-0.873,-0.873,-0.873,-0.874,-0.874,     &
                    -0.875,-0.875,-0.875,-0.876,-0.876,-0.877,-0.877,     &
                    -0.877,-0.878,-0.878,-0.878,-0.879,-0.879,-0.880,     &
                    -0.880,-0.880,-0.881,-0.881,-0.881,-0.882,-0.882,     &
                    -0.883,-0.883,-0.883,-0.884,-0.884,-0.884,-0.885,     &
                    -0.885,-0.885,-0.886,-0.886,-0.887,-0.887,-0.887,     &
                    -0.888,-0.888,-0.888,-0.889,-0.889,-0.889,-0.890,     &
                    -0.890,-0.890,-0.891,-0.891,-0.891,-0.892,-0.892,     &
                    -0.892,-0.893,-0.893,-0.893,-0.894,-0.894,-0.894,     &
                    -0.895,-0.895,-0.895,-0.896,-0.896,-0.896,-0.897,     &
                    -0.897,-0.897,-0.898,-0.898,-0.898,-0.899,-0.899,     &
                    -0.899,-0.900,-0.900,-0.900,-0.901,-0.901,-0.901,     &
                    -0.901,-0.902,-0.902,-0.902,-0.903,-0.903,-0.903,     &
                    -0.904,-0.904,-0.904,-0.905,-0.905,-0.905,-0.905,     &
                    -0.906,-0.909,-0.912,-0.915,-0.917,-0.920,-0.922,     &
                    -0.925,-0.927,-0.930,-0.932,-0.935,-0.937,-0.939,     &
                    -0.941,-0.943,-0.945,-0.947,-0.949,-0.951,-0.953,     &
                    -0.955,-0.957,-0.959,-0.961,-0.962,-0.964,-0.966,     &
                    -0.967,-0.969,-0.971,-0.972,-0.974,-0.975,-0.977,     &
                    -0.978,-0.980,-0.981,-0.983,-0.984,-0.986,-0.987,     &
                    -0.988,-0.990,-0.991,-0.992,-0.994,-0.995,-0.996,     &
                    -0.997,-0.999,-1.000,-1.001,-1.002,-1.003,-1.004,     &
                    -1.006,-1.007,-1.008,-1.009,-1.010,-1.011,-1.012,     &
                    -1.013,-1.014,-1.015,-1.016,-1.017,-1.018,-1.019,     &
                    -1.020,-1.021,-1.022,-1.023,-1.024,-1.025,-1.026,     &
                    -1.027,-1.028,-1.028,-1.029,-1.030,-1.031,-1.032,     &
                    -1.033,-1.034,-1.034,-1.035,-1.036,-1.037,-1.038,     &
                    -1.039,-1.039,-1.040,-1.041,-1.042,-1.042,-1.043,     &
                    -1.044,-1.045,-1.045,-1.046,-1.047,-1.048,-1.048,     &
                    -1.049,-1.050,-1.050,-1.051,-1.052,-1.052,-1.053,     &
                    -1.054,-1.055,-1.055,-1.056,-1.056,-1.057,-1.058,     &
                    -1.058,-1.059,-1.060,-1.060,-1.061,-1.062,-1.062,     &
                    -1.063,-1.063,-1.064,-1.065,-1.065,-1.066,-1.066,     &
                    -1.067,-1.068,-1.068,-1.069,-1.069,-1.070,-1.070,     &
                    -1.071,-1.071,-1.072,-1.073,-1.073,-1.074,-1.074,     &
                    -1.075,-1.075,-1.076,-1.076,-1.077,-1.077,-1.078,     &
                    -1.078,-1.079,-1.079,-1.080,-1.080,-1.081,-1.081,     &
                    -1.082/)
!
! *** (h ,hso4)
!
    bnc08m(1:561) = (/-0.044,-0.088,-0.106,-0.116,-0.123,-0.128,-0.131,   &
                    -0.133,-0.134,-0.134,-0.134,-0.133,-0.131,-0.129,     &
                    -0.127,-0.125,-0.122,-0.119,-0.115,-0.112,-0.108,     &
                    -0.104,-0.100,-0.095,-0.091,-0.086,-0.081,-0.076,     &
                    -0.071,-0.066,-0.060,-0.055,-0.049,-0.043,-0.037,     &
                    -0.031,-0.025,-0.019,-0.013,-0.006,0.000,0.007,0.013, &
                    0.020,0.027,0.034,0.041,0.048,0.055,0.062,0.069,0.076,&
                    0.083,0.090,0.098,0.105,0.113,0.120,0.127,0.135,0.143,&
                    0.150,0.158,0.166,0.173,0.181,0.189,0.197,0.205,0.212,&
                    0.220,0.228,0.236,0.245,0.253,0.261,0.269,0.277,0.286,&
                    0.294,0.302,0.311,0.319,0.328,0.336,0.345,0.353,0.362,&
                    0.371,0.380,0.388,0.397,0.406,0.415,0.424,0.433,0.442,&
                    0.451,0.460,0.469,0.479,0.488,0.497,0.506,0.516,0.525,&
                    0.535,0.544,0.553,0.563,0.572,0.582,0.591,0.601,0.610,&
                    0.620,0.629,0.639,0.648,0.658,0.667,0.677,0.686,0.696,&
                    0.705,0.715,0.725,0.734,0.744,0.753,0.763,0.772,0.782,&
                    0.791,0.800,0.810,0.819,0.829,0.838,0.848,0.857,0.866,&
                    0.876,0.885,0.894,0.904,0.913,0.922,0.932,0.941,0.950,&
                    0.959,0.968,0.978,0.987,0.996,1.005,1.014,1.023,1.032,&
                    1.041,1.050,1.059,1.069,1.077,1.086,1.095,1.104,1.113,&
                    1.122,1.131,1.140,1.149,1.158,1.166,1.175,1.184,1.193,&
                    1.202,1.210,1.219,1.228,1.236,1.245,1.254,1.262,1.271,&
                    1.279,1.288,1.296,1.305,1.314,1.322,1.330,1.339,1.347,&
                    1.356,1.364,1.373,1.381,1.389,1.398,1.406,1.414,1.422,&
                    1.431,1.439,1.447,1.455,1.464,1.472,1.480,1.488,1.496,&
                    1.504,1.512,1.520,1.528,1.537,1.545,1.553,1.561,1.568,&
                    1.576,1.584,1.592,1.600,1.608,1.616,1.624,1.632,1.639,&
                    1.647,1.655,1.663,1.670,1.678,1.686,1.694,1.701,1.709,&
                    1.717,1.724,1.732,1.740,1.747,1.755,1.762,1.770,1.777,&
                    1.785,1.792,1.800,1.807,1.815,1.822,1.830,1.837,1.844,&
                    1.852,1.859,1.866,1.874,1.881,1.888,1.896,1.903,1.910,&
                    1.917,1.925,1.932,1.939,1.946,1.953,1.961,1.968,1.975,&
                    1.982,1.989,1.996,2.003,2.010,2.017,2.024,2.031,2.038,&
                    2.045,2.052,2.059,2.066,2.073,2.080,2.087,2.094,2.101,&
                    2.108,2.114,2.121,2.128,2.135,2.142,2.148,2.155,2.162,&
                    2.169,2.175,2.182,2.189,2.196,2.202,2.209,2.216,2.222,&
                    2.229,2.236,2.242,2.249,2.255,2.262,2.268,2.275,2.281,&
                    2.288,2.295,2.301,2.308,2.314,2.320,2.327,2.333,2.340,&
                    2.346,2.353,2.359,2.365,2.372,2.378,2.384,2.391,2.397,&
                    2.403,2.410,2.416,2.422,2.429,2.435,2.441,2.447,2.453,&
                    2.460,2.466,2.472,2.478,2.484,2.491,2.497,2.503,2.509,&
                    2.515,2.521,2.527,2.533,2.539,2.546,2.552,2.558,2.564,&
                    2.570,2.576,2.582,2.588,2.594,2.600,2.606,2.612,2.617,&
                    2.623,2.629,2.635,2.641,2.647,2.653,2.659,2.665,2.670,&
                    2.676,2.682,2.688,2.694,2.700,2.705,2.711,2.717,2.723,&
                    2.728,2.734,2.740,2.746,2.751,2.757,2.818,2.874,2.928,&
                    2.982,3.035,3.087,3.139,3.190,3.239,3.289,3.337,3.385,&
                    3.432,3.479,3.525,3.571,3.615,3.660,3.703,3.746,3.789,&
                    3.831,3.873,3.914,3.954,3.995,4.034,4.073,4.112,4.151,&
                    4.189,4.226,4.263,4.300,4.336,4.372,4.408,4.443,4.478,&
                    4.513,4.547,4.581,4.614,4.647,4.680,4.713,4.745,4.777,&
                    4.809,4.840,4.871,4.902,4.933,4.963,4.993,5.023,5.052,&
                    5.082,5.111,5.140,5.168,5.196,5.224,5.252,5.280,5.307,&
                    5.335,5.362,5.388,5.415,5.441,5.468,5.494,5.519,5.545,&
                    5.570,5.596,5.621,5.646,5.670,5.695,5.719,5.743,5.767,&
                    5.791,5.815,5.838,5.862,5.885,5.908,5.931,5.954,5.976,&
                    5.999,6.021,6.043,6.065,6.087,6.109,6.130,6.152,6.173,&
                    6.194,6.215,6.236,6.257,6.278,6.298,6.319,6.339,6.359,&
                    6.380,6.400,6.419,6.439,6.459,6.478,6.498,6.517,6.536,&
                    6.555,6.574,6.593,6.612,6.631,6.649,6.668,6.686,6.705,&
                    6.723,6.741,6.759,6.777,6.795,6.812,6.830,6.848,6.865,&
                    6.882,6.900,6.917,6.934,6.951,6.968,6.985,7.002,7.018,&
                    7.035,7.052,7.068,7.084,7.101,7.117,7.133,7.149,7.165,&
                    7.181,7.197,7.213,7.229,7.244/)
!
! *** nh4hso4
!
    bnc09m(1:561) = (/-0.046,-0.097,-0.122,-0.138,-0.151,-0.162,-0.170,   &
                    -0.177,-0.184,-0.189,-0.194,-0.198,-0.202,-0.206,     &
                    -0.209,-0.212,-0.214,-0.216,-0.218,-0.220,-0.222,     &
                    -0.223,-0.224,-0.225,-0.226,-0.227,-0.227,-0.228,     &
                    -0.228,-0.228,-0.228,-0.228,-0.228,-0.228,-0.228,     &
                    -0.227,-0.227,-0.226,-0.225,-0.225,-0.224,-0.223,     &
                    -0.222,-0.221,-0.220,-0.219,-0.218,-0.216,-0.215,     &
                    -0.214,-0.212,-0.211,-0.209,-0.208,-0.206,-0.204,     &
                    -0.203,-0.201,-0.199,-0.197,-0.195,-0.193,-0.192,     &
                    -0.190,-0.188,-0.185,-0.183,-0.181,-0.179,-0.177,     &
                    -0.175,-0.173,-0.170,-0.168,-0.166,-0.163,-0.161,     &
                    -0.159,-0.156,-0.154,-0.151,-0.149,-0.146,-0.144,     &
                    -0.141,-0.139,-0.136,-0.133,-0.131,-0.128,-0.125,     &
                    -0.122,-0.120,-0.117,-0.114,-0.111,-0.108,-0.106,     &
                    -0.103,-0.100,-0.097,-0.094,-0.091,-0.088,-0.085,     &
                    -0.082,-0.079,-0.076,-0.073,-0.070,-0.067,-0.064,     &
                    -0.061,-0.058,-0.055,-0.052,-0.049,-0.045,-0.042,     &
                    -0.039,-0.036,-0.033,-0.030,-0.027,-0.024,-0.021,     &
                    -0.018,-0.014,-0.011,-0.008,-0.005,-0.002,0.001,0.004,&
                    0.007,0.010,0.013,0.016,0.020,0.023,0.026,0.029,0.032,&
                    0.035,0.038,0.041,0.044,0.047,0.050,0.053,0.056,0.059,&
                    0.062,0.065,0.068,0.071,0.074,0.077,0.080,0.083,0.086,&
                    0.089,0.092,0.095,0.098,0.101,0.104,0.106,0.109,0.112,&
                    0.115,0.118,0.121,0.124,0.127,0.130,0.132,0.135,0.138,&
                    0.141,0.144,0.147,0.150,0.152,0.155,0.158,0.161,0.164,&
                    0.166,0.169,0.172,0.175,0.177,0.180,0.183,0.186,0.188,&
                    0.191,0.194,0.197,0.199,0.202,0.205,0.208,0.210,0.213,&
                    0.216,0.218,0.221,0.224,0.226,0.229,0.232,0.234,0.237,&
                    0.239,0.242,0.245,0.247,0.250,0.253,0.255,0.258,0.260,&
                    0.263,0.265,0.268,0.271,0.273,0.276,0.278,0.281,0.283,&
                    0.286,0.288,0.291,0.293,0.296,0.298,0.301,0.303,0.306,&
                    0.308,0.311,0.313,0.316,0.318,0.321,0.323,0.326,0.328,&
                    0.330,0.333,0.335,0.338,0.340,0.343,0.345,0.347,0.350,&
                    0.352,0.355,0.357,0.359,0.362,0.364,0.366,0.369,0.371,&
                    0.374,0.376,0.378,0.381,0.383,0.385,0.388,0.390,0.392,&
                    0.394,0.397,0.399,0.401,0.404,0.406,0.408,0.410,0.413,&
                    0.415,0.417,0.420,0.422,0.424,0.426,0.429,0.431,0.433,&
                    0.435,0.437,0.440,0.442,0.444,0.446,0.449,0.451,0.453,&
                    0.455,0.457,0.460,0.462,0.464,0.466,0.468,0.470,0.473,&
                    0.475,0.477,0.479,0.481,0.483,0.485,0.488,0.490,0.492,&
                    0.494,0.496,0.498,0.500,0.502,0.504,0.507,0.509,0.511,&
                    0.513,0.515,0.517,0.519,0.521,0.523,0.525,0.527,0.529,&
                    0.531,0.534,0.536,0.538,0.540,0.542,0.544,0.546,0.548,&
                    0.550,0.552,0.554,0.556,0.558,0.560,0.562,0.564,0.566,&
                    0.568,0.570,0.572,0.574,0.576,0.578,0.580,0.582,0.584,&
                    0.586,0.588,0.590,0.592,0.593,0.595,0.597,0.599,0.601,&
                    0.603,0.605,0.607,0.609,0.611,0.613,0.615,0.617,0.619,&
                    0.620,0.622,0.624,0.626,0.628,0.630,0.632,0.634,0.636,&
                    0.637,0.639,0.641,0.643,0.645,0.665,0.683,0.701,0.719,&
                    0.736,0.753,0.770,0.787,0.803,0.820,0.836,0.852,0.867,&
                    0.883,0.898,0.913,0.928,0.942,0.957,0.971,0.985,0.999,&
                    1.013,1.027,1.040,1.053,1.067,1.080,1.093,1.105,1.118,&
                    1.131,1.143,1.155,1.167,1.179,1.191,1.203,1.215,1.226,&
                    1.238,1.249,1.260,1.271,1.283,1.293,1.304,1.315,1.326,&
                    1.336,1.347,1.357,1.367,1.377,1.388,1.398,1.408,1.417,&
                    1.427,1.437,1.446,1.456,1.466,1.475,1.484,1.493,1.503,&
                    1.512,1.521,1.530,1.539,1.548,1.556,1.565,1.574,1.582,&
                    1.591,1.599,1.608,1.616,1.624,1.633,1.641,1.649,1.657,&
                    1.665,1.673,1.681,1.689,1.697,1.704,1.712,1.720,1.727,&
                    1.735,1.742,1.750,1.757,1.765,1.772,1.779,1.787,1.794,&
                    1.801,1.808,1.815,1.822,1.829,1.836,1.843,1.850,1.857,&
                    1.864,1.870,1.877,1.884,1.890,1.897,1.904,1.910,1.917,&
                    1.923,1.930,1.936,1.942,1.949,1.955,1.961,1.968,1.974,&
                    1.980,1.986,1.992,1.998,2.004,2.010,2.016,2.022,2.028,&
                    2.034,2.040,2.046,2.052,2.058,2.063,2.069,2.075,2.080,&
                    2.086,2.092,2.097,2.103,2.108,2.114,2.119,2.125,2.130,&
                    2.136,2.141,2.147,2.152/)
!
! *** (h ,no3)
!
    bnc10m(1:561) = (/-0.045,-0.094,-0.116,-0.130,-0.140,-0.147,-0.153,   &
                    -0.158,-0.162,-0.165,-0.167,-0.169,-0.171,-0.172,     &
                    -0.173,-0.173,-0.174,-0.174,-0.174,-0.173,-0.173,     &
                    -0.172,-0.172,-0.171,-0.170,-0.169,-0.168,-0.167,     &
                    -0.166,-0.165,-0.163,-0.162,-0.161,-0.159,-0.158,     &
                    -0.156,-0.155,-0.153,-0.152,-0.150,-0.148,-0.147,     &
                    -0.145,-0.143,-0.142,-0.140,-0.138,-0.136,-0.135,     &
                    -0.133,-0.131,-0.129,-0.127,-0.126,-0.124,-0.122,     &
                    -0.120,-0.118,-0.117,-0.115,-0.113,-0.111,-0.109,     &
                    -0.107,-0.106,-0.104,-0.102,-0.100,-0.098,-0.096,     &
                    -0.094,-0.092,-0.090,-0.088,-0.086,-0.084,-0.083,     &
                    -0.081,-0.079,-0.077,-0.074,-0.072,-0.070,-0.068,     &
                    -0.066,-0.064,-0.062,-0.060,-0.058,-0.056,-0.053,     &
                    -0.051,-0.049,-0.047,-0.045,-0.042,-0.040,-0.038,     &
                    -0.036,-0.033,-0.031,-0.029,-0.026,-0.024,-0.022,     &
                    -0.019,-0.017,-0.015,-0.012,-0.010,-0.008,-0.005,     &
                    -0.003,0.000,0.002,0.004,0.007,0.009,0.012,0.014,     &
                    0.017,0.019,0.022,0.024,0.026,0.029,0.031,0.034,0.036,&
                    0.039,0.041,0.044,0.046,0.049,0.051,0.053,0.056,0.058,&
                    0.061,0.063,0.066,0.068,0.071,0.073,0.075,0.078,0.080,&
                    0.083,0.085,0.088,0.090,0.092,0.095,0.097,0.100,0.102,&
                    0.105,0.107,0.109,0.112,0.114,0.117,0.119,0.121,0.124,&
                    0.126,0.129,0.131,0.133,0.136,0.138,0.140,0.143,0.145,&
                    0.148,0.150,0.152,0.155,0.157,0.159,0.162,0.164,0.166,&
                    0.169,0.171,0.173,0.176,0.178,0.180,0.183,0.185,0.187,&
                    0.190,0.192,0.194,0.197,0.199,0.201,0.204,0.206,0.208,&
                    0.210,0.213,0.215,0.217,0.220,0.222,0.224,0.226,0.229,&
                    0.231,0.233,0.235,0.238,0.240,0.242,0.244,0.247,0.249,&
                    0.251,0.253,0.256,0.258,0.260,0.262,0.264,0.267,0.269,&
                    0.271,0.273,0.275,0.278,0.280,0.282,0.284,0.286,0.288,&
                    0.291,0.293,0.295,0.297,0.299,0.301,0.304,0.306,0.308,&
                    0.310,0.312,0.314,0.317,0.319,0.321,0.323,0.325,0.327,&
                    0.329,0.331,0.333,0.336,0.338,0.340,0.342,0.344,0.346,&
                    0.348,0.350,0.352,0.354,0.356,0.359,0.361,0.363,0.365,&
                    0.367,0.369,0.371,0.373,0.375,0.377,0.379,0.381,0.383,&
                    0.385,0.387,0.389,0.391,0.393,0.395,0.397,0.399,0.401,&
                    0.403,0.405,0.407,0.409,0.411,0.413,0.415,0.417,0.419,&
                    0.421,0.423,0.425,0.427,0.429,0.431,0.433,0.435,0.437,&
                    0.439,0.441,0.443,0.445,0.447,0.449,0.451,0.453,0.455,&
                    0.456,0.458,0.460,0.462,0.464,0.466,0.468,0.470,0.472,&
                    0.474,0.476,0.477,0.479,0.481,0.483,0.485,0.487,0.489,&
                    0.491,0.493,0.494,0.496,0.498,0.500,0.502,0.504,0.506,&
                    0.507,0.509,0.511,0.513,0.515,0.517,0.518,0.520,0.522,&
                    0.524,0.526,0.528,0.529,0.531,0.533,0.535,0.537,0.539,&
                    0.540,0.542,0.544,0.546,0.547,0.549,0.551,0.553,0.555,&
                    0.556,0.558,0.560,0.562,0.564,0.565,0.567,0.569,0.571,&
                    0.572,0.574,0.576,0.578,0.579,0.581,0.583,0.585,0.586,&
                    0.588,0.590,0.592,0.593,0.595,0.597,0.598,0.600,0.602,&
                    0.604,0.622,0.639,0.655,0.672,0.688,0.704,0.719,0.735,&
                    0.750,0.765,0.780,0.795,0.809,0.824,0.838,0.852,0.866,&
                    0.879,0.893,0.906,0.919,0.932,0.945,0.958,0.971,0.983,&
                    0.996,1.008,1.020,1.032,1.044,1.055,1.067,1.079,1.090,&
                    1.101,1.112,1.123,1.134,1.145,1.156,1.167,1.177,1.188,&
                    1.198,1.208,1.218,1.228,1.238,1.248,1.258,1.268,1.278,&
                    1.287,1.297,1.306,1.315,1.325,1.334,1.343,1.352,1.361,&
                    1.370,1.379,1.387,1.396,1.405,1.413,1.422,1.430,1.438,&
                    1.447,1.455,1.463,1.471,1.479,1.487,1.495,1.503,1.511,&
                    1.519,1.527,1.534,1.542,1.550,1.557,1.565,1.572,1.579,&
                    1.587,1.594,1.601,1.608,1.616,1.623,1.630,1.637,1.644,&
                    1.651,1.657,1.664,1.671,1.678,1.685,1.691,1.698,1.704,&
                    1.711,1.718,1.724,1.730,1.737,1.743,1.750,1.756,1.762,&
                    1.768,1.775,1.781,1.787,1.793,1.799,1.805,1.811,1.817,&
                    1.823,1.829,1.835,1.841,1.846,1.852,1.858,1.864,1.869,&
                    1.875,1.881,1.886,1.892,1.897,1.903,1.908,1.914,1.919,&
                    1.925,1.930,1.936,1.941,1.946,1.952,1.957,1.962,1.967,&
                    1.972,1.978,1.983,1.988,1.993,1.998,2.003,2.008,      &
                    2.013/)
!
! *** (h ,cl)
!
    bnc11m(1:561) = (/-0.044,-0.089,-0.107,-0.118,-0.125,-0.130,-0.133,   &
                    -0.135,-0.136,-0.136,-0.136,-0.135,-0.134,-0.132,     &
                    -0.130,-0.128,-0.125,-0.122,-0.119,-0.116,-0.113,     &
                    -0.109,-0.105,-0.102,-0.098,-0.094,-0.089,-0.085,     &
                    -0.081,-0.076,-0.072,-0.067,-0.062,-0.057,-0.053,     &
                    -0.048,-0.043,-0.038,-0.033,-0.028,-0.022,-0.017,     &
                    -0.012,-0.007,-0.001,0.004,0.009,0.015,0.020,0.025,   &
                    0.031,0.036,0.042,0.047,0.053,0.058,0.064,0.069,0.075,&
                    0.081,0.086,0.092,0.097,0.103,0.109,0.114,0.120,0.126,&
                    0.132,0.137,0.143,0.149,0.155,0.161,0.167,0.172,0.178,&
                    0.184,0.190,0.196,0.202,0.208,0.214,0.220,0.227,0.233,&
                    0.239,0.245,0.251,0.258,0.264,0.270,0.277,0.283,0.290,&
                    0.296,0.303,0.309,0.316,0.322,0.329,0.335,0.342,0.349,&
                    0.355,0.362,0.369,0.376,0.382,0.389,0.396,0.403,0.409,&
                    0.416,0.423,0.430,0.437,0.444,0.450,0.457,0.464,0.471,&
                    0.478,0.485,0.491,0.498,0.505,0.512,0.519,0.526,0.533,&
                    0.539,0.546,0.553,0.560,0.567,0.573,0.580,0.587,0.594,&
                    0.601,0.607,0.614,0.621,0.628,0.634,0.641,0.648,0.654,&
                    0.661,0.668,0.675,0.681,0.688,0.694,0.701,0.708,0.714,&
                    0.721,0.728,0.734,0.741,0.747,0.754,0.760,0.767,0.773,&
                    0.780,0.786,0.793,0.799,0.806,0.812,0.819,0.825,0.831,&
                    0.838,0.844,0.850,0.857,0.863,0.870,0.876,0.882,0.888,&
                    0.895,0.901,0.907,0.914,0.920,0.926,0.932,0.938,0.945,&
                    0.951,0.957,0.963,0.969,0.975,0.981,0.988,0.994,1.000,&
                    1.006,1.012,1.018,1.024,1.030,1.036,1.042,1.048,1.054,&
                    1.060,1.066,1.072,1.078,1.084,1.090,1.096,1.101,1.107,&
                    1.113,1.119,1.125,1.131,1.137,1.142,1.148,1.154,1.160,&
                    1.165,1.171,1.177,1.183,1.188,1.194,1.200,1.205,1.211,&
                    1.217,1.222,1.228,1.234,1.239,1.245,1.251,1.256,1.262,&
                    1.267,1.273,1.278,1.284,1.289,1.295,1.300,1.306,1.311,&
                    1.317,1.322,1.328,1.333,1.339,1.344,1.349,1.355,1.360,&
                    1.366,1.371,1.376,1.382,1.387,1.392,1.398,1.403,1.408,&
                    1.413,1.419,1.424,1.429,1.434,1.440,1.445,1.450,1.455,&
                    1.460,1.466,1.471,1.476,1.481,1.486,1.491,1.497,1.502,&
                    1.507,1.512,1.517,1.522,1.527,1.532,1.537,1.542,1.547,&
                    1.552,1.557,1.562,1.567,1.572,1.577,1.582,1.587,1.592,&
                    1.597,1.602,1.607,1.612,1.617,1.622,1.626,1.631,1.636,&
                    1.641,1.646,1.651,1.656,1.660,1.665,1.670,1.675,1.680,&
                    1.684,1.689,1.694,1.699,1.703,1.708,1.713,1.718,1.722,&
                    1.727,1.732,1.736,1.741,1.746,1.750,1.755,1.760,1.764,&
                    1.769,1.774,1.778,1.783,1.787,1.792,1.797,1.801,1.806,&
                    1.810,1.815,1.819,1.824,1.828,1.833,1.838,1.842,1.847,&
                    1.851,1.855,1.860,1.864,1.869,1.873,1.878,1.882,1.887,&
                    1.891,1.895,1.900,1.904,1.909,1.913,1.917,1.922,1.926,&
                    1.931,1.935,1.939,1.944,1.948,1.952,1.957,1.961,1.965,&
                    1.969,1.974,1.978,1.982,1.987,1.991,1.995,1.999,2.045,&
                    2.086,2.127,2.167,2.206,2.245,2.284,2.322,2.359,2.396,&
                    2.432,2.468,2.503,2.538,2.572,2.606,2.639,2.672,2.705,&
                    2.737,2.769,2.800,2.831,2.862,2.892,2.922,2.952,2.981,&
                    3.010,3.039,3.067,3.095,3.123,3.150,3.178,3.204,3.231,&
                    3.257,3.283,3.309,3.335,3.360,3.385,3.410,3.435,3.459,&
                    3.483,3.507,3.531,3.554,3.577,3.600,3.623,3.646,3.668,&
                    3.691,3.713,3.735,3.756,3.778,3.799,3.820,3.841,3.862,&
                    3.883,3.903,3.924,3.944,3.964,3.984,4.003,4.023,4.043,&
                    4.062,4.081,4.100,4.119,4.138,4.156,4.175,4.193,4.211,&
                    4.229,4.247,4.265,4.283,4.300,4.318,4.335,4.352,4.369,&
                    4.386,4.403,4.420,4.437,4.453,4.470,4.486,4.503,4.519,&
                    4.535,4.551,4.567,4.582,4.598,4.614,4.629,4.644,4.660,&
                    4.675,4.690,4.705,4.720,4.735,4.750,4.764,4.779,4.794,&
                    4.808,4.822,4.837,4.851,4.865,4.879,4.893,4.907,4.921,&
                    4.935,4.948,4.962,4.975,4.989,5.002,5.016,5.029,5.042,&
                    5.055,5.068,5.081,5.094,5.107,5.120,5.133,5.145,5.158,&
                    5.171,5.183,5.196,5.208,5.220,5.233,5.245,5.257,5.269,&
                    5.281,5.293,5.305,5.317,5.329,5.340,5.352/)
!
! *** nahso4
!
    bnc12m(1:561) = (/-0.045,-0.094,-0.116,-0.130,-0.140,-0.148,-0.155,   &
                    -0.160,-0.164,-0.168,-0.170,-0.173,-0.174,-0.176,     &
                    -0.177,-0.178,-0.178,-0.179,-0.179,-0.178,-0.178,     &
                    -0.177,-0.177,-0.176,-0.175,-0.174,-0.172,-0.171,     &
                    -0.170,-0.168,-0.166,-0.164,-0.162,-0.160,-0.158,     &
                    -0.156,-0.154,-0.152,-0.149,-0.147,-0.144,-0.142,     &
                    -0.139,-0.136,-0.134,-0.131,-0.128,-0.125,-0.122,     &
                    -0.119,-0.116,-0.113,-0.110,-0.107,-0.103,-0.100,     &
                    -0.097,-0.094,-0.090,-0.087,-0.084,-0.080,-0.077,     &
                    -0.073,-0.070,-0.066,-0.063,-0.059,-0.055,-0.052,     &
                    -0.048,-0.044,-0.040,-0.037,-0.033,-0.029,-0.025,     &
                    -0.021,-0.017,-0.013,-0.010,-0.006,-0.002,0.003,0.007,&
                    0.011,0.015,0.019,0.023,0.027,0.032,0.036,0.040,0.045,&
                    0.049,0.053,0.058,0.062,0.066,0.071,0.075,0.080,0.084,&
                    0.089,0.094,0.098,0.103,0.107,0.112,0.116,0.121,0.126,&
                    0.130,0.135,0.140,0.144,0.149,0.154,0.158,0.163,0.168,&
                    0.172,0.177,0.182,0.186,0.191,0.196,0.200,0.205,0.210,&
                    0.215,0.219,0.224,0.229,0.233,0.238,0.242,0.247,0.252,&
                    0.256,0.261,0.266,0.270,0.275,0.279,0.284,0.289,0.293,&
                    0.298,0.302,0.307,0.311,0.316,0.320,0.325,0.329,0.334,&
                    0.338,0.343,0.347,0.352,0.356,0.361,0.365,0.370,0.374,&
                    0.379,0.383,0.387,0.392,0.396,0.400,0.405,0.409,0.414,&
                    0.418,0.422,0.427,0.431,0.435,0.440,0.444,0.448,0.452,&
                    0.457,0.461,0.465,0.469,0.474,0.478,0.482,0.486,0.490,&
                    0.495,0.499,0.503,0.507,0.511,0.515,0.520,0.524,0.528,&
                    0.532,0.536,0.540,0.544,0.548,0.552,0.556,0.561,0.565,&
                    0.569,0.573,0.577,0.581,0.585,0.589,0.593,0.597,0.601,&
                    0.605,0.609,0.613,0.616,0.620,0.624,0.628,0.632,0.636,&
                    0.640,0.644,0.648,0.652,0.655,0.659,0.663,0.667,0.671,&
                    0.675,0.679,0.682,0.686,0.690,0.694,0.697,0.701,0.705,&
                    0.709,0.713,0.716,0.720,0.724,0.728,0.731,0.735,0.739,&
                    0.742,0.746,0.750,0.753,0.757,0.761,0.764,0.768,0.772,&
                    0.775,0.779,0.783,0.786,0.790,0.793,0.797,0.801,0.804,&
                    0.808,0.811,0.815,0.818,0.822,0.825,0.829,0.833,0.836,&
                    0.840,0.843,0.847,0.850,0.854,0.857,0.861,0.864,0.867,&
                    0.871,0.874,0.878,0.881,0.885,0.888,0.892,0.895,0.898,&
                    0.902,0.905,0.909,0.912,0.915,0.919,0.922,0.925,0.929,&
                    0.932,0.935,0.939,0.942,0.945,0.949,0.952,0.955,0.959,&
                    0.962,0.965,0.969,0.972,0.975,0.978,0.982,0.985,0.988,&
                    0.991,0.995,0.998,1.001,1.004,1.008,1.011,1.014,1.017,&
                    1.020,1.024,1.027,1.030,1.033,1.036,1.039,1.043,1.046,&
                    1.049,1.052,1.055,1.058,1.061,1.065,1.068,1.071,1.074,&
                    1.077,1.080,1.083,1.086,1.089,1.092,1.096,1.099,1.102,&
                    1.105,1.108,1.111,1.114,1.117,1.120,1.123,1.126,1.129,&
                    1.132,1.135,1.138,1.141,1.144,1.147,1.150,1.153,1.156,&
                    1.159,1.162,1.165,1.168,1.171,1.174,1.177,1.180,1.183,&
                    1.185,1.188,1.191,1.194,1.197,1.200,1.203,1.206,1.209,&
                    1.240,1.268,1.296,1.323,1.350,1.377,1.403,1.429,1.454,&
                    1.480,1.505,1.529,1.553,1.577,1.601,1.624,1.647,1.670,&
                    1.692,1.715,1.737,1.758,1.780,1.801,1.822,1.843,1.863,&
                    1.883,1.903,1.923,1.943,1.962,1.981,2.000,2.019,2.038,&
                    2.056,2.075,2.093,2.111,2.128,2.146,2.163,2.181,2.198,&
                    2.215,2.231,2.248,2.264,2.281,2.297,2.313,2.329,2.345,&
                    2.360,2.376,2.391,2.406,2.422,2.437,2.451,2.466,2.481,&
                    2.495,2.510,2.524,2.538,2.552,2.566,2.580,2.594,2.608,&
                    2.621,2.635,2.648,2.661,2.674,2.687,2.700,2.713,2.726,&
                    2.739,2.751,2.764,2.776,2.789,2.801,2.813,2.825,2.838,&
                    2.850,2.861,2.873,2.885,2.897,2.908,2.920,2.931,2.943,&
                    2.954,2.965,2.976,2.987,2.998,3.009,3.020,3.031,3.042,&
                    3.053,3.063,3.074,3.084,3.095,3.105,3.116,3.126,3.136,&
                    3.146,3.156,3.166,3.176,3.186,3.196,3.206,3.216,3.226,&
                    3.235,3.245,3.255,3.264,3.274,3.283,3.292,3.302,3.311,&
                    3.320,3.330,3.339,3.348,3.357,3.366,3.375,3.384,3.393,&
                    3.402,3.410,3.419,3.428,3.437,3.445,3.454,3.462,3.471,&
                    3.479,3.488,3.496,3.505,3.513,3.521,3.529,3.538/)
!
! *** (nh4)3h(so4)2
!
    bnc13m(1:561) = (/-0.074,-0.160,-0.203,-0.233,-0.256,-0.275,-0.291,   &
                    -0.306,-0.318,-0.329,-0.339,-0.349,-0.357,-0.365,     &
                    -0.372,-0.379,-0.386,-0.392,-0.397,-0.402,-0.407,     &
                    -0.412,-0.417,-0.421,-0.425,-0.429,-0.433,-0.436,     &
                    -0.439,-0.443,-0.446,-0.448,-0.451,-0.454,-0.456,     &
                    -0.459,-0.461,-0.463,-0.466,-0.468,-0.470,-0.471,     &
                    -0.473,-0.475,-0.477,-0.478,-0.480,-0.481,-0.482,     &
                    -0.484,-0.485,-0.486,-0.487,-0.488,-0.489,-0.490,     &
                    -0.491,-0.492,-0.493,-0.494,-0.495,-0.496,-0.496,     &
                    -0.497,-0.498,-0.498,-0.499,-0.499,-0.500,-0.500,     &
                    -0.501,-0.501,-0.501,-0.502,-0.502,-0.502,-0.503,     &
                    -0.503,-0.503,-0.503,-0.503,-0.504,-0.504,-0.504,     &
                    -0.504,-0.504,-0.504,-0.504,-0.504,-0.504,-0.504,     &
                    -0.504,-0.504,-0.504,-0.504,-0.503,-0.503,-0.503,     &
                    -0.503,-0.503,-0.503,-0.502,-0.502,-0.502,-0.502,     &
                    -0.501,-0.501,-0.501,-0.501,-0.500,-0.500,-0.500,     &
                    -0.499,-0.499,-0.498,-0.498,-0.498,-0.497,-0.497,     &
                    -0.497,-0.496,-0.496,-0.495,-0.495,-0.494,-0.494,     &
                    -0.494,-0.493,-0.493,-0.492,-0.492,-0.491,-0.491,     &
                    -0.490,-0.490,-0.489,-0.489,-0.488,-0.488,-0.487,     &
                    -0.487,-0.486,-0.486,-0.485,-0.485,-0.484,-0.484,     &
                    -0.483,-0.483,-0.482,-0.482,-0.481,-0.481,-0.480,     &
                    -0.479,-0.479,-0.478,-0.478,-0.477,-0.477,-0.476,     &
                    -0.476,-0.475,-0.475,-0.474,-0.473,-0.473,-0.472,     &
                    -0.472,-0.471,-0.471,-0.470,-0.470,-0.469,-0.468,     &
                    -0.468,-0.467,-0.467,-0.466,-0.466,-0.465,-0.464,     &
                    -0.464,-0.463,-0.463,-0.462,-0.462,-0.461,-0.461,     &
                    -0.460,-0.459,-0.459,-0.458,-0.458,-0.457,-0.457,     &
                    -0.456,-0.455,-0.455,-0.454,-0.454,-0.453,-0.453,     &
                    -0.452,-0.451,-0.451,-0.450,-0.450,-0.449,-0.449,     &
                    -0.448,-0.447,-0.447,-0.446,-0.446,-0.445,-0.445,     &
                    -0.444,-0.443,-0.443,-0.442,-0.442,-0.441,-0.441,     &
                    -0.440,-0.440,-0.439,-0.438,-0.438,-0.437,-0.437,     &
                    -0.436,-0.436,-0.435,-0.434,-0.434,-0.433,-0.433,     &
                    -0.432,-0.432,-0.431,-0.431,-0.430,-0.429,-0.429,     &
                    -0.428,-0.428,-0.427,-0.427,-0.426,-0.426,-0.425,     &
                    -0.424,-0.424,-0.423,-0.423,-0.422,-0.422,-0.421,     &
                    -0.421,-0.420,-0.419,-0.419,-0.418,-0.418,-0.417,     &
                    -0.417,-0.416,-0.416,-0.415,-0.415,-0.414,-0.413,     &
                    -0.413,-0.412,-0.412,-0.411,-0.411,-0.410,-0.410,     &
                    -0.409,-0.409,-0.408,-0.408,-0.407,-0.406,-0.406,     &
                    -0.405,-0.405,-0.404,-0.404,-0.403,-0.403,-0.402,     &
                    -0.402,-0.401,-0.401,-0.400,-0.399,-0.399,-0.398,     &
                    -0.398,-0.397,-0.397,-0.396,-0.396,-0.395,-0.395,     &
                    -0.394,-0.394,-0.393,-0.393,-0.392,-0.392,-0.391,     &
                    -0.391,-0.390,-0.390,-0.389,-0.388,-0.388,-0.387,     &
                    -0.387,-0.386,-0.386,-0.385,-0.385,-0.384,-0.384,     &
                    -0.383,-0.383,-0.382,-0.382,-0.381,-0.381,-0.380,     &
                    -0.380,-0.379,-0.379,-0.378,-0.378,-0.377,-0.377,     &
                    -0.376,-0.376,-0.375,-0.375,-0.374,-0.374,-0.373,     &
                    -0.373,-0.372,-0.372,-0.371,-0.371,-0.370,-0.370,     &
                    -0.369,-0.369,-0.368,-0.368,-0.367,-0.367,-0.366,     &
                    -0.366,-0.365,-0.365,-0.364,-0.364,-0.363,-0.363,     &
                    -0.362,-0.362,-0.361,-0.361,-0.360,-0.360,-0.359,     &
                    -0.359,-0.358,-0.358,-0.357,-0.357,-0.356,-0.356,     &
                    -0.355,-0.355,-0.354,-0.354,-0.353,-0.353,-0.352,     &
                    -0.352,-0.352,-0.351,-0.351,-0.350,-0.350,-0.349,     &
                    -0.349,-0.344,-0.339,-0.334,-0.330,-0.325,-0.321,     &
                    -0.316,-0.312,-0.307,-0.303,-0.299,-0.295,-0.290,     &
                    -0.286,-0.282,-0.278,-0.274,-0.270,-0.266,-0.262,     &
                    -0.258,-0.255,-0.251,-0.247,-0.243,-0.240,-0.236,     &
                    -0.232,-0.229,-0.225,-0.222,-0.218,-0.215,-0.211,     &
                    -0.208,-0.204,-0.201,-0.198,-0.194,-0.191,-0.188,     &
                    -0.185,-0.182,-0.178,-0.175,-0.172,-0.169,-0.166,     &
                    -0.163,-0.160,-0.157,-0.154,-0.151,-0.148,-0.145,     &
                    -0.142,-0.139,-0.137,-0.134,-0.131,-0.128,-0.125,     &
                    -0.123,-0.120,-0.117,-0.115,-0.112,-0.109,-0.107,     &
                    -0.104,-0.101,-0.099,-0.096,-0.094,-0.091,-0.089,     &
                    -0.086,-0.084,-0.081,-0.079,-0.076,-0.074,-0.071,     &
                    -0.069,-0.067,-0.064,-0.062,-0.060,-0.057,-0.055,     &
                    -0.053,-0.050,-0.048,-0.046,-0.043,-0.041,-0.039,     &
                    -0.037,-0.035,-0.032,-0.030,-0.028,-0.026,-0.024,     &
                    -0.022,-0.020,-0.017,-0.015,-0.013,-0.011,-0.009,     &
                    -0.007,-0.005,-0.003,-0.001,0.001,0.003,0.005,0.007,  &
                    0.009,0.011,0.013,0.015,0.017,0.019,0.021,0.022,0.024,&
                    0.026,0.028,0.030,0.032,0.034,0.036,0.037,0.039,0.041,&
                    0.043,0.045,0.046,0.048,0.050,0.052,0.053,0.055,0.057,&
                    0.059,0.060,0.062,0.064,0.066,0.067,0.069,0.071,0.072,&
                    0.074,0.076,0.077,0.079,0.081,0.082/)
!
! *** caso4
!
    bnc14m(1:561) = (/0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,    &
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000/)
!
! *** cano32
!
    bnc15m(1:561) = (/-0.092,-0.196,-0.245,-0.279,-0.304,-0.325,-0.342,   &
                    -0.356,-0.368,-0.379,-0.389,-0.397,-0.405,-0.412,     &
                    -0.418,-0.424,-0.429,-0.433,-0.438,-0.442,-0.445,     &
                    -0.448,-0.452,-0.454,-0.457,-0.459,-0.462,-0.464,     &
                    -0.466,-0.467,-0.469,-0.470,-0.472,-0.473,-0.474,     &
                    -0.475,-0.476,-0.477,-0.478,-0.479,-0.480,-0.480,     &
                    -0.481,-0.482,-0.482,-0.483,-0.483,-0.483,-0.484,     &
                    -0.484,-0.484,-0.484,-0.485,-0.485,-0.485,-0.485,     &
                    -0.485,-0.485,-0.485,-0.485,-0.485,-0.485,-0.485,     &
                    -0.485,-0.485,-0.484,-0.484,-0.484,-0.484,-0.484,     &
                    -0.483,-0.483,-0.483,-0.482,-0.482,-0.482,-0.481,     &
                    -0.481,-0.480,-0.480,-0.479,-0.479,-0.478,-0.478,     &
                    -0.477,-0.477,-0.476,-0.475,-0.475,-0.474,-0.474,     &
                    -0.473,-0.472,-0.471,-0.471,-0.470,-0.469,-0.468,     &
                    -0.467,-0.467,-0.466,-0.465,-0.464,-0.463,-0.462,     &
                    -0.461,-0.460,-0.459,-0.459,-0.458,-0.457,-0.456,     &
                    -0.455,-0.454,-0.453,-0.452,-0.451,-0.450,-0.448,     &
                    -0.447,-0.446,-0.445,-0.444,-0.443,-0.442,-0.441,     &
                    -0.440,-0.439,-0.438,-0.437,-0.436,-0.434,-0.433,     &
                    -0.432,-0.431,-0.430,-0.429,-0.428,-0.427,-0.425,     &
                    -0.424,-0.423,-0.422,-0.421,-0.420,-0.419,-0.417,     &
                    -0.416,-0.415,-0.414,-0.413,-0.412,-0.411,-0.409,     &
                    -0.408,-0.407,-0.406,-0.405,-0.404,-0.402,-0.401,     &
                    -0.400,-0.399,-0.398,-0.397,-0.395,-0.394,-0.393,     &
                    -0.392,-0.391,-0.390,-0.388,-0.387,-0.386,-0.385,     &
                    -0.384,-0.383,-0.381,-0.380,-0.379,-0.378,-0.377,     &
                    -0.376,-0.374,-0.373,-0.372,-0.371,-0.370,-0.369,     &
                    -0.367,-0.366,-0.365,-0.364,-0.363,-0.361,-0.360,     &
                    -0.359,-0.358,-0.357,-0.356,-0.354,-0.353,-0.352,     &
                    -0.351,-0.350,-0.349,-0.347,-0.346,-0.345,-0.344,     &
                    -0.343,-0.342,-0.341,-0.339,-0.338,-0.337,-0.336,     &
                    -0.335,-0.334,-0.332,-0.331,-0.330,-0.329,-0.328,     &
                    -0.327,-0.325,-0.324,-0.323,-0.322,-0.321,-0.320,     &
                    -0.319,-0.317,-0.316,-0.315,-0.314,-0.313,-0.312,     &
                    -0.311,-0.309,-0.308,-0.307,-0.306,-0.305,-0.304,     &
                    -0.303,-0.301,-0.300,-0.299,-0.298,-0.297,-0.296,     &
                    -0.295,-0.294,-0.292,-0.291,-0.290,-0.289,-0.288,     &
                    -0.287,-0.286,-0.285,-0.283,-0.282,-0.281,-0.280,     &
                    -0.279,-0.278,-0.277,-0.276,-0.275,-0.273,-0.272,     &
                    -0.271,-0.270,-0.269,-0.268,-0.267,-0.266,-0.265,     &
                    -0.263,-0.262,-0.261,-0.260,-0.259,-0.258,-0.257,     &
                    -0.256,-0.255,-0.254,-0.253,-0.251,-0.250,-0.249,     &
                    -0.248,-0.247,-0.246,-0.245,-0.244,-0.243,-0.242,     &
                    -0.241,-0.240,-0.238,-0.237,-0.236,-0.235,-0.234,     &
                    -0.233,-0.232,-0.231,-0.230,-0.229,-0.228,-0.227,     &
                    -0.226,-0.225,-0.224,-0.222,-0.221,-0.220,-0.219,     &
                    -0.218,-0.217,-0.216,-0.215,-0.214,-0.213,-0.212,     &
                    -0.211,-0.210,-0.209,-0.208,-0.207,-0.206,-0.205,     &
                    -0.204,-0.203,-0.202,-0.201,-0.200,-0.198,-0.197,     &
                    -0.196,-0.195,-0.194,-0.193,-0.192,-0.191,-0.190,     &
                    -0.189,-0.188,-0.187,-0.186,-0.185,-0.184,-0.183,     &
                    -0.182,-0.181,-0.180,-0.179,-0.178,-0.177,-0.176,     &
                    -0.175,-0.174,-0.173,-0.172,-0.171,-0.170,-0.169,     &
                    -0.168,-0.167,-0.166,-0.165,-0.164,-0.163,-0.162,     &
                    -0.161,-0.160,-0.159,-0.158,-0.157,-0.156,-0.155,     &
                    -0.154,-0.153,-0.152,-0.151,-0.150,-0.149,-0.148,     &
                    -0.147,-0.146,-0.145,-0.144,-0.143,-0.142,-0.141,     &
                    -0.140,-0.130,-0.121,-0.111,-0.102,-0.093,-0.083,     &
                    -0.074,-0.066,-0.057,-0.048,-0.039,-0.031,-0.022,     &
                    -0.014,-0.006,0.003,0.011,0.019,0.027,0.035,0.043,    &
                    0.050,0.058,0.066,0.073,0.081,0.088,0.095,0.103,0.110,&
                    0.117,0.124,0.131,0.138,0.145,0.152,0.159,0.165,0.172,&
                    0.179,0.185,0.192,0.198,0.205,0.211,0.217,0.223,0.230,&
                    0.236,0.242,0.248,0.254,0.260,0.266,0.272,0.278,0.283,&
                    0.289,0.295,0.301,0.306,0.312,0.317,0.323,0.328,0.334,&
                    0.339,0.345,0.350,0.355,0.360,0.366,0.371,0.376,0.381,&
                    0.386,0.391,0.396,0.401,0.406,0.411,0.416,0.421,0.426,&
                    0.431,0.435,0.440,0.445,0.449,0.454,0.459,0.463,0.468,&
                    0.472,0.477,0.482,0.486,0.490,0.495,0.499,0.504,0.508,&
                    0.512,0.517,0.521,0.525,0.529,0.534,0.538,0.542,0.546,&
                    0.550,0.554,0.559,0.563,0.567,0.571,0.575,0.579,0.583,&
                    0.587,0.590,0.594,0.598,0.602,0.606,0.610,0.614,0.617,&
                    0.621,0.625,0.629,0.632,0.636,0.640,0.643,0.647,0.651,&
                    0.654,0.658,0.661,0.665,0.669,0.672,0.676,0.679,0.683,&
                    0.686,0.690,0.693,0.696,0.700,0.703,0.707,0.710,0.713,&
                    0.717,0.720,0.723,0.727,0.730/)
!
! *** cacl2
!
    bnc16m(1:561) = (/-0.091,-0.188,-0.233,-0.261,-0.282,-0.298,-0.310,   &
                    -0.320,-0.328,-0.335,-0.340,-0.345,-0.348,-0.351,     &
                    -0.353,-0.355,-0.356,-0.356,-0.357,-0.357,-0.356,     &
                    -0.356,-0.355,-0.354,-0.353,-0.351,-0.350,-0.348,     &
                    -0.346,-0.344,-0.342,-0.340,-0.337,-0.335,-0.333,     &
                    -0.330,-0.327,-0.325,-0.322,-0.319,-0.316,-0.314,     &
                    -0.311,-0.308,-0.305,-0.302,-0.299,-0.296,-0.293,     &
                    -0.290,-0.287,-0.283,-0.280,-0.277,-0.274,-0.271,     &
                    -0.268,-0.264,-0.261,-0.258,-0.255,-0.252,-0.248,     &
                    -0.245,-0.242,-0.238,-0.235,-0.232,-0.228,-0.225,     &
                    -0.222,-0.218,-0.215,-0.211,-0.208,-0.204,-0.201,     &
                    -0.197,-0.194,-0.190,-0.187,-0.183,-0.179,-0.176,     &
                    -0.172,-0.168,-0.164,-0.161,-0.157,-0.153,-0.149,     &
                    -0.145,-0.141,-0.137,-0.133,-0.129,-0.125,-0.121,     &
                    -0.117,-0.113,-0.109,-0.105,-0.101,-0.096,-0.092,     &
                    -0.088,-0.084,-0.080,-0.075,-0.071,-0.067,-0.062,     &
                    -0.058,-0.054,-0.049,-0.045,-0.041,-0.036,-0.032,     &
                    -0.027,-0.023,-0.019,-0.014,-0.010,-0.005,-0.001,     &
                    0.003,0.008,0.012,0.017,0.021,0.026,0.030,0.034,0.039,&
                    0.043,0.048,0.052,0.057,0.061,0.065,0.070,0.074,0.079,&
                    0.083,0.087,0.092,0.096,0.101,0.105,0.109,0.114,0.118,&
                    0.123,0.127,0.131,0.136,0.140,0.144,0.149,0.153,0.157,&
                    0.162,0.166,0.170,0.175,0.179,0.183,0.188,0.192,0.196,&
                    0.201,0.205,0.209,0.214,0.218,0.222,0.226,0.231,0.235,&
                    0.239,0.243,0.248,0.252,0.256,0.260,0.265,0.269,0.273,&
                    0.277,0.281,0.286,0.290,0.294,0.298,0.302,0.306,0.311,&
                    0.315,0.319,0.323,0.327,0.331,0.336,0.340,0.344,0.348,&
                    0.352,0.356,0.360,0.364,0.368,0.372,0.377,0.381,0.385,&
                    0.389,0.393,0.397,0.401,0.405,0.409,0.413,0.417,0.421,&
                    0.425,0.429,0.433,0.437,0.441,0.445,0.449,0.453,0.457,&
                    0.461,0.465,0.469,0.473,0.477,0.481,0.485,0.488,0.492,&
                    0.496,0.500,0.504,0.508,0.512,0.516,0.520,0.524,0.527,&
                    0.531,0.535,0.539,0.543,0.547,0.550,0.554,0.558,0.562,&
                    0.566,0.570,0.573,0.577,0.581,0.585,0.588,0.592,0.596,&
                    0.600,0.604,0.607,0.611,0.615,0.618,0.622,0.626,0.630,&
                    0.633,0.637,0.641,0.644,0.648,0.652,0.656,0.659,0.663,&
                    0.667,0.670,0.674,0.677,0.681,0.685,0.688,0.692,0.696,&
                    0.699,0.703,0.706,0.710,0.714,0.717,0.721,0.724,0.728,&
                    0.732,0.735,0.739,0.742,0.746,0.749,0.753,0.756,0.760,&
                    0.763,0.767,0.770,0.774,0.777,0.781,0.784,0.788,0.791,&
                    0.795,0.798,0.802,0.805,0.809,0.812,0.816,0.819,0.823,&
                    0.826,0.829,0.833,0.836,0.840,0.843,0.846,0.850,0.853,&
                    0.857,0.860,0.863,0.867,0.870,0.873,0.877,0.880,0.884,&
                    0.887,0.890,0.894,0.897,0.900,0.904,0.907,0.910,0.913,&
                    0.917,0.920,0.923,0.927,0.930,0.933,0.936,0.940,0.943,&
                    0.946,0.950,0.953,0.956,0.959,0.963,0.966,0.969,0.972,&
                    0.975,0.979,0.982,0.985,0.988,0.991,0.995,0.998,1.001,&
                    1.004,1.007,1.011,1.014,1.017,1.020,1.023,1.026,1.029,&
                    1.033,1.036,1.039,1.042,1.076,1.106,1.136,1.166,1.195,&
                    1.224,1.253,1.281,1.309,1.337,1.364,1.391,1.417,1.444,&
                    1.470,1.495,1.521,1.546,1.570,1.595,1.619,1.643,1.667,&
                    1.690,1.713,1.736,1.759,1.781,1.803,1.825,1.847,1.868,&
                    1.890,1.911,1.932,1.952,1.973,1.993,2.013,2.033,2.053,&
                    2.072,2.092,2.111,2.130,2.149,2.167,2.186,2.204,2.222,&
                    2.240,2.258,2.276,2.293,2.311,2.328,2.345,2.362,2.379,&
                    2.396,2.412,2.429,2.445,2.461,2.478,2.493,2.509,2.525,&
                    2.541,2.556,2.572,2.587,2.602,2.617,2.632,2.647,2.661,&
                    2.676,2.691,2.705,2.719,2.734,2.748,2.762,2.776,2.789,&
                    2.803,2.817,2.830,2.844,2.857,2.871,2.884,2.897,2.910,&
                    2.923,2.936,2.949,2.961,2.974,2.987,2.999,3.012,3.024,&
                    3.036,3.048,3.061,3.073,3.085,3.097,3.108,3.120,3.132,&
                    3.144,3.155,3.167,3.178,3.190,3.201,3.212,3.224,3.235,&
                    3.246,3.257,3.268,3.279,3.290,3.300,3.311,3.322,3.333,&
                    3.343,3.354,3.364,3.375,3.385,3.395,3.406,3.416,3.426,&
                    3.436,3.446,3.456,3.466,3.476,3.486,3.496,3.506,3.515,&
                    3.525,3.535,3.544,3.554,3.563,3.573,3.582,3.592,3.601,&
                    3.610,3.620,3.629/)
!
! *** k2so4
!
    bnc17m(1:561) = (/-0.093,-0.203,-0.257,-0.296,-0.326,-0.351,-0.372,   &
                    -0.391,-0.408,-0.423,-0.436,-0.449,-0.460,-0.471,     &
                    -0.481,-0.491,-0.500,-0.508,-0.516,-0.524,-0.531,     &
                    -0.538,-0.545,-0.552,-0.558,-0.564,-0.569,-0.575,     &
                    -0.580,-0.585,-0.590,-0.595,-0.600,-0.605,-0.609,     &
                    -0.613,-0.618,-0.622,-0.626,-0.629,-0.633,-0.637,     &
                    -0.641,-0.644,-0.648,-0.651,-0.654,-0.658,-0.661,     &
                    -0.664,-0.667,-0.670,-0.673,-0.676,-0.678,-0.681,     &
                    -0.684,-0.687,-0.689,-0.692,-0.694,-0.697,-0.699,     &
                    -0.702,-0.704,-0.707,-0.709,-0.711,-0.713,-0.716,     &
                    -0.718,-0.720,-0.722,-0.724,-0.726,-0.728,-0.730,     &
                    -0.732,-0.734,-0.736,-0.738,-0.740,-0.742,-0.744,     &
                    -0.746,-0.748,-0.749,-0.751,-0.753,-0.755,-0.756,     &
                    -0.758,-0.760,-0.762,-0.763,-0.765,-0.767,-0.768,     &
                    -0.770,-0.772,-0.773,-0.775,-0.776,-0.778,-0.779,     &
                    -0.781,-0.782,-0.784,-0.786,-0.787,-0.788,-0.790,     &
                    -0.791,-0.793,-0.794,-0.796,-0.797,-0.799,-0.800,     &
                    -0.801,-0.803,-0.804,-0.805,-0.807,-0.808,-0.809,     &
                    -0.811,-0.812,-0.813,-0.815,-0.816,-0.817,-0.819,     &
                    -0.820,-0.821,-0.822,-0.824,-0.825,-0.826,-0.827,     &
                    -0.828,-0.830,-0.831,-0.832,-0.833,-0.834,-0.835,     &
                    -0.837,-0.838,-0.839,-0.840,-0.841,-0.842,-0.843,     &
                    -0.844,-0.846,-0.847,-0.848,-0.849,-0.850,-0.851,     &
                    -0.852,-0.853,-0.854,-0.855,-0.856,-0.857,-0.858,     &
                    -0.859,-0.860,-0.861,-0.862,-0.863,-0.864,-0.865,     &
                    -0.866,-0.867,-0.868,-0.869,-0.870,-0.871,-0.872,     &
                    -0.873,-0.874,-0.875,-0.876,-0.877,-0.878,-0.878,     &
                    -0.879,-0.880,-0.881,-0.882,-0.883,-0.884,-0.885,     &
                    -0.886,-0.886,-0.887,-0.888,-0.889,-0.890,-0.891,     &
                    -0.892,-0.893,-0.893,-0.894,-0.895,-0.896,-0.897,     &
                    -0.898,-0.898,-0.899,-0.900,-0.901,-0.902,-0.902,     &
                    -0.903,-0.904,-0.905,-0.906,-0.906,-0.907,-0.908,     &
                    -0.909,-0.910,-0.910,-0.911,-0.912,-0.913,-0.913,     &
                    -0.914,-0.915,-0.916,-0.916,-0.917,-0.918,-0.919,     &
                    -0.919,-0.920,-0.921,-0.922,-0.922,-0.923,-0.924,     &
                    -0.924,-0.925,-0.926,-0.927,-0.927,-0.928,-0.929,     &
                    -0.929,-0.930,-0.931,-0.931,-0.932,-0.933,-0.933,     &
                    -0.934,-0.935,-0.936,-0.936,-0.937,-0.938,-0.938,     &
                    -0.939,-0.940,-0.940,-0.941,-0.941,-0.942,-0.943,     &
                    -0.943,-0.944,-0.945,-0.945,-0.946,-0.947,-0.947,     &
                    -0.948,-0.949,-0.949,-0.950,-0.950,-0.951,-0.952,     &
                    -0.952,-0.953,-0.954,-0.954,-0.955,-0.955,-0.956,     &
                    -0.957,-0.957,-0.958,-0.958,-0.959,-0.960,-0.960,     &
                    -0.961,-0.961,-0.962,-0.962,-0.963,-0.964,-0.964,     &
                    -0.965,-0.965,-0.966,-0.967,-0.967,-0.968,-0.968,     &
                    -0.969,-0.969,-0.970,-0.970,-0.971,-0.972,-0.972,     &
                    -0.973,-0.973,-0.974,-0.974,-0.975,-0.975,-0.976,     &
                    -0.977,-0.977,-0.978,-0.978,-0.979,-0.979,-0.980,     &
                    -0.980,-0.981,-0.981,-0.982,-0.982,-0.983,-0.983,     &
                    -0.984,-0.984,-0.985,-0.986,-0.986,-0.987,-0.987,     &
                    -0.988,-0.988,-0.989,-0.989,-0.990,-0.990,-0.991,     &
                    -0.991,-0.992,-0.992,-0.993,-0.993,-0.994,-0.994,     &
                    -0.995,-0.995,-0.996,-0.996,-0.997,-0.997,-0.998,     &
                    -0.998,-0.998,-0.999,-0.999,-1.000,-1.000,-1.001,     &
                    -1.001,-1.002,-1.002,-1.003,-1.003,-1.004,-1.004,     &
                    -1.005,-1.005,-1.006,-1.006,-1.006,-1.007,-1.007,     &
                    -1.008,-1.008,-1.009,-1.009,-1.010,-1.010,-1.011,     &
                    -1.011,-1.016,-1.020,-1.024,-1.029,-1.033,-1.037,     &
                    -1.040,-1.044,-1.048,-1.052,-1.055,-1.059,-1.062,     &
                    -1.065,-1.069,-1.072,-1.075,-1.078,-1.081,-1.084,     &
                    -1.087,-1.090,-1.093,-1.096,-1.099,-1.102,-1.104,     &
                    -1.107,-1.110,-1.112,-1.115,-1.117,-1.120,-1.122,     &
                    -1.125,-1.127,-1.129,-1.132,-1.134,-1.136,-1.139,     &
                    -1.141,-1.143,-1.145,-1.147,-1.149,-1.151,-1.153,     &
                    -1.155,-1.157,-1.159,-1.161,-1.163,-1.165,-1.167,     &
                    -1.169,-1.171,-1.173,-1.174,-1.176,-1.178,-1.180,     &
                    -1.181,-1.183,-1.185,-1.187,-1.188,-1.190,-1.191,     &
                    -1.193,-1.195,-1.196,-1.198,-1.199,-1.201,-1.202,     &
                    -1.204,-1.205,-1.207,-1.208,-1.210,-1.211,-1.213,     &
                    -1.214,-1.216,-1.217,-1.218,-1.220,-1.221,-1.223,     &
                    -1.224,-1.225,-1.226,-1.228,-1.229,-1.230,-1.232,     &
                    -1.233,-1.234,-1.235,-1.237,-1.238,-1.239,-1.240,     &
                    -1.242,-1.243,-1.244,-1.245,-1.246,-1.247,-1.249,     &
                    -1.250,-1.251,-1.252,-1.253,-1.254,-1.255,-1.256,     &
                    -1.258,-1.259,-1.260,-1.261,-1.262,-1.263,-1.264,     &
                    -1.265,-1.266,-1.267,-1.268,-1.269,-1.270,-1.271,     &
                    -1.272,-1.273,-1.274,-1.275,-1.276,-1.277,-1.278,     &
                    -1.279,-1.280,-1.281,-1.282,-1.283,-1.283,-1.284,     &
                    -1.285,-1.286,-1.287,-1.288,-1.289,-1.290,-1.291,     &
                    -1.292,-1.292,-1.293,-1.294,-1.295,-1.296,-1.297,     &
                    -1.298/)
!
! *** khso4
!
    bnc18m(1:561) = (/-0.046,-0.097,-0.121,-0.138,-0.150,-0.161,-0.169,   &
                    -0.176,-0.182,-0.188,-0.192,-0.196,-0.200,-0.203,     &
                    -0.206,-0.209,-0.211,-0.213,-0.215,-0.217,-0.218,     &
                    -0.219,-0.220,-0.221,-0.222,-0.223,-0.223,-0.223,     &
                    -0.223,-0.224,-0.223,-0.223,-0.223,-0.223,-0.222,     &
                    -0.222,-0.221,-0.220,-0.220,-0.219,-0.218,-0.217,     &
                    -0.216,-0.215,-0.213,-0.212,-0.211,-0.209,-0.208,     &
                    -0.206,-0.205,-0.203,-0.202,-0.200,-0.198,-0.196,     &
                    -0.195,-0.193,-0.191,-0.189,-0.187,-0.185,-0.183,     &
                    -0.181,-0.179,-0.177,-0.174,-0.172,-0.170,-0.168,     &
                    -0.165,-0.163,-0.161,-0.158,-0.156,-0.153,-0.151,     &
                    -0.148,-0.146,-0.143,-0.141,-0.138,-0.136,-0.133,     &
                    -0.130,-0.127,-0.125,-0.122,-0.119,-0.116,-0.114,     &
                    -0.111,-0.108,-0.105,-0.102,-0.099,-0.096,-0.093,     &
                    -0.090,-0.087,-0.084,-0.081,-0.078,-0.075,-0.072,     &
                    -0.069,-0.066,-0.063,-0.060,-0.056,-0.053,-0.050,     &
                    -0.047,-0.044,-0.041,-0.037,-0.034,-0.031,-0.028,     &
                    -0.025,-0.021,-0.018,-0.015,-0.012,-0.009,-0.005,     &
                    -0.002,0.001,0.004,0.007,0.011,0.014,0.017,0.020,     &
                    0.023,0.027,0.030,0.033,0.036,0.039,0.043,0.046,0.049,&
                    0.052,0.055,0.058,0.061,0.065,0.068,0.071,0.074,0.077,&
                    0.080,0.083,0.086,0.089,0.093,0.096,0.099,0.102,0.105,&
                    0.108,0.111,0.114,0.117,0.120,0.123,0.126,0.129,0.132,&
                    0.135,0.138,0.141,0.144,0.147,0.150,0.153,0.156,0.159,&
                    0.162,0.165,0.168,0.171,0.174,0.176,0.179,0.182,0.185,&
                    0.188,0.191,0.194,0.197,0.199,0.202,0.205,0.208,0.211,&
                    0.214,0.217,0.219,0.222,0.225,0.228,0.231,0.233,0.236,&
                    0.239,0.242,0.244,0.247,0.250,0.253,0.255,0.258,0.261,&
                    0.264,0.266,0.269,0.272,0.274,0.277,0.280,0.282,0.285,&
                    0.288,0.290,0.293,0.296,0.298,0.301,0.304,0.306,0.309,&
                    0.312,0.314,0.317,0.319,0.322,0.325,0.327,0.330,0.332,&
                    0.335,0.337,0.340,0.343,0.345,0.348,0.350,0.353,0.355,&
                    0.358,0.360,0.363,0.365,0.368,0.370,0.373,0.375,0.378,&
                    0.380,0.383,0.385,0.388,0.390,0.393,0.395,0.397,0.400,&
                    0.402,0.405,0.407,0.410,0.412,0.414,0.417,0.419,0.422,&
                    0.424,0.426,0.429,0.431,0.434,0.436,0.438,0.441,0.443,&
                    0.445,0.448,0.450,0.452,0.455,0.457,0.459,0.462,0.464,&
                    0.466,0.469,0.471,0.473,0.475,0.478,0.480,0.482,0.485,&
                    0.487,0.489,0.491,0.494,0.496,0.498,0.500,0.503,0.505,&
                    0.507,0.509,0.512,0.514,0.516,0.518,0.520,0.523,0.525,&
                    0.527,0.529,0.531,0.534,0.536,0.538,0.540,0.542,0.544,&
                    0.547,0.549,0.551,0.553,0.555,0.557,0.560,0.562,0.564,&
                    0.566,0.568,0.570,0.572,0.574,0.577,0.579,0.581,0.583,&
                    0.585,0.587,0.589,0.591,0.593,0.595,0.597,0.600,0.602,&
                    0.604,0.606,0.608,0.610,0.612,0.614,0.616,0.618,0.620,&
                    0.622,0.624,0.626,0.628,0.630,0.632,0.634,0.636,0.638,&
                    0.640,0.642,0.644,0.646,0.648,0.650,0.652,0.654,0.656,&
                    0.658,0.660,0.662,0.664,0.666,0.668,0.670,0.672,0.674,&
                    0.676,0.678,0.680,0.682,0.684,0.704,0.723,0.742,0.760,&
                    0.778,0.796,0.813,0.831,0.848,0.865,0.881,0.898,0.914,&
                    0.930,0.946,0.961,0.977,0.992,1.007,1.022,1.036,1.051,&
                    1.065,1.079,1.093,1.107,1.121,1.134,1.148,1.161,1.174,&
                    1.187,1.200,1.213,1.225,1.238,1.250,1.262,1.275,1.287,&
                    1.298,1.310,1.322,1.333,1.345,1.356,1.367,1.379,1.390,&
                    1.401,1.411,1.422,1.433,1.443,1.454,1.464,1.475,1.485,&
                    1.495,1.505,1.515,1.525,1.535,1.545,1.554,1.564,1.573,&
                    1.583,1.592,1.602,1.611,1.620,1.629,1.638,1.647,1.656,&
                    1.665,1.674,1.682,1.691,1.700,1.708,1.717,1.725,1.734,&
                    1.742,1.750,1.758,1.767,1.775,1.783,1.791,1.799,1.807,&
                    1.815,1.822,1.830,1.838,1.845,1.853,1.861,1.868,1.876,&
                    1.883,1.891,1.898,1.905,1.912,1.920,1.927,1.934,1.941,&
                    1.948,1.955,1.962,1.969,1.976,1.983,1.990,1.997,2.003,&
                    2.010,2.017,2.023,2.030,2.037,2.043,2.050,2.056,2.063,&
                    2.069,2.075,2.082,2.088,2.094,2.101,2.107,2.113,2.119,&
                    2.125,2.131,2.137,2.143,2.149,2.155,2.161,2.167,2.173,&
                    2.179,2.185,2.191,2.197,2.202,2.208,2.214,2.219,2.225,&
                    2.231,2.236,2.242,2.248/)
!
! *** kno3
!
    bnc19m(1:561) = (/-0.048,-0.109,-0.141,-0.166,-0.186,-0.204,-0.219,   &
                    -0.233,-0.246,-0.258,-0.270,-0.281,-0.291,-0.301,     &
                    -0.310,-0.319,-0.328,-0.336,-0.344,-0.352,-0.360,     &
                    -0.367,-0.375,-0.382,-0.389,-0.395,-0.402,-0.409,     &
                    -0.415,-0.421,-0.427,-0.433,-0.439,-0.445,-0.451,     &
                    -0.456,-0.462,-0.467,-0.472,-0.477,-0.483,-0.488,     &
                    -0.492,-0.497,-0.502,-0.507,-0.511,-0.516,-0.521,     &
                    -0.525,-0.529,-0.534,-0.538,-0.542,-0.546,-0.550,     &
                    -0.554,-0.558,-0.562,-0.566,-0.570,-0.574,-0.578,     &
                    -0.581,-0.585,-0.589,-0.592,-0.596,-0.600,-0.603,     &
                    -0.607,-0.610,-0.613,-0.617,-0.620,-0.624,-0.627,     &
                    -0.630,-0.634,-0.637,-0.640,-0.643,-0.647,-0.650,     &
                    -0.653,-0.656,-0.660,-0.663,-0.666,-0.669,-0.672,     &
                    -0.675,-0.678,-0.682,-0.685,-0.688,-0.691,-0.694,     &
                    -0.697,-0.700,-0.703,-0.706,-0.709,-0.712,-0.715,     &
                    -0.718,-0.721,-0.724,-0.727,-0.730,-0.733,-0.736,     &
                    -0.739,-0.741,-0.744,-0.747,-0.750,-0.753,-0.756,     &
                    -0.758,-0.761,-0.764,-0.767,-0.770,-0.772,-0.775,     &
                    -0.778,-0.780,-0.783,-0.786,-0.789,-0.791,-0.794,     &
                    -0.796,-0.799,-0.802,-0.804,-0.807,-0.809,-0.812,     &
                    -0.814,-0.817,-0.819,-0.822,-0.824,-0.827,-0.829,     &
                    -0.832,-0.834,-0.837,-0.839,-0.841,-0.844,-0.846,     &
                    -0.848,-0.851,-0.853,-0.855,-0.858,-0.860,-0.862,     &
                    -0.865,-0.867,-0.869,-0.871,-0.873,-0.876,-0.878,     &
                    -0.880,-0.882,-0.884,-0.887,-0.889,-0.891,-0.893,     &
                    -0.895,-0.897,-0.899,-0.901,-0.903,-0.905,-0.908,     &
                    -0.910,-0.912,-0.914,-0.916,-0.918,-0.920,-0.922,     &
                    -0.924,-0.925,-0.927,-0.929,-0.931,-0.933,-0.935,     &
                    -0.937,-0.939,-0.941,-0.943,-0.945,-0.946,-0.948,     &
                    -0.950,-0.952,-0.954,-0.956,-0.957,-0.959,-0.961,     &
                    -0.963,-0.965,-0.966,-0.968,-0.970,-0.972,-0.973,     &
                    -0.975,-0.977,-0.978,-0.980,-0.982,-0.984,-0.985,     &
                    -0.987,-0.989,-0.990,-0.992,-0.993,-0.995,-0.997,     &
                    -0.998,-1.000,-1.002,-1.003,-1.005,-1.006,-1.008,     &
                    -1.009,-1.011,-1.013,-1.014,-1.016,-1.017,-1.019,     &
                    -1.020,-1.022,-1.023,-1.025,-1.026,-1.028,-1.029,     &
                    -1.031,-1.032,-1.034,-1.035,-1.036,-1.038,-1.039,     &
                    -1.041,-1.042,-1.044,-1.045,-1.046,-1.048,-1.049,     &
                    -1.051,-1.052,-1.053,-1.055,-1.056,-1.057,-1.059,     &
                    -1.060,-1.061,-1.063,-1.064,-1.065,-1.067,-1.068,     &
                    -1.069,-1.071,-1.072,-1.073,-1.074,-1.076,-1.077,     &
                    -1.078,-1.079,-1.081,-1.082,-1.083,-1.084,-1.086,     &
                    -1.087,-1.088,-1.089,-1.090,-1.092,-1.093,-1.094,     &
                    -1.095,-1.096,-1.098,-1.099,-1.100,-1.101,-1.102,     &
                    -1.103,-1.105,-1.106,-1.107,-1.108,-1.109,-1.110,     &
                    -1.111,-1.112,-1.114,-1.115,-1.116,-1.117,-1.118,     &
                    -1.119,-1.120,-1.121,-1.122,-1.123,-1.124,-1.125,     &
                    -1.127,-1.128,-1.129,-1.130,-1.131,-1.132,-1.133,     &
                    -1.134,-1.135,-1.136,-1.137,-1.138,-1.139,-1.140,     &
                    -1.141,-1.142,-1.143,-1.144,-1.145,-1.146,-1.147,     &
                    -1.148,-1.149,-1.150,-1.151,-1.152,-1.153,-1.153,     &
                    -1.154,-1.155,-1.156,-1.157,-1.158,-1.159,-1.160,     &
                    -1.161,-1.162,-1.163,-1.164,-1.165,-1.165,-1.166,     &
                    -1.167,-1.168,-1.169,-1.170,-1.171,-1.172,-1.172,     &
                    -1.173,-1.174,-1.175,-1.176,-1.177,-1.178,-1.178,     &
                    -1.179,-1.180,-1.181,-1.182,-1.183,-1.183,-1.184,     &
                    -1.185,-1.186,-1.187,-1.188,-1.188,-1.189,-1.190,     &
                    -1.191,-1.199,-1.207,-1.214,-1.221,-1.228,-1.234,     &
                    -1.240,-1.246,-1.252,-1.258,-1.263,-1.268,-1.273,     &
                    -1.278,-1.283,-1.287,-1.292,-1.296,-1.300,-1.304,     &
                    -1.308,-1.312,-1.315,-1.319,-1.322,-1.325,-1.329,     &
                    -1.332,-1.335,-1.338,-1.341,-1.343,-1.346,-1.349,     &
                    -1.351,-1.354,-1.356,-1.358,-1.361,-1.363,-1.365,     &
                    -1.367,-1.369,-1.371,-1.373,-1.375,-1.377,-1.378,     &
                    -1.380,-1.382,-1.383,-1.385,-1.387,-1.388,-1.390,     &
                    -1.391,-1.393,-1.394,-1.395,-1.397,-1.398,-1.399,     &
                    -1.400,-1.402,-1.403,-1.404,-1.405,-1.406,-1.407,     &
                    -1.408,-1.409,-1.410,-1.411,-1.412,-1.413,-1.414,     &
                    -1.415,-1.416,-1.417,-1.418,-1.419,-1.420,-1.420,     &
                    -1.421,-1.422,-1.423,-1.424,-1.424,-1.425,-1.426,     &
                    -1.427,-1.427,-1.428,-1.429,-1.429,-1.430,-1.430,     &
                    -1.431,-1.432,-1.432,-1.433,-1.434,-1.434,-1.435,     &
                    -1.435,-1.436,-1.436,-1.437,-1.437,-1.438,-1.438,     &
                    -1.439,-1.439,-1.440,-1.440,-1.441,-1.441,-1.442,     &
                    -1.442,-1.443,-1.443,-1.443,-1.444,-1.444,-1.445,     &
                    -1.445,-1.445,-1.446,-1.446,-1.447,-1.447,-1.447,     &
                    -1.448,-1.448,-1.448,-1.449,-1.449,-1.449,-1.450,     &
                    -1.450,-1.450,-1.451,-1.451,-1.451,-1.452,-1.452,     &
                    -1.452,-1.453,-1.453,-1.453,-1.454,-1.454,-1.454,     &
                    -1.454,-1.455,-1.455,-1.455,-1.455,-1.456,-1.456,     &
                    -1.456/)
!
! *** kcl
!
    bnc20m(1:561) = (/-0.046,-0.098,-0.123,-0.139,-0.152,-0.162,-0.171,   &
                    -0.178,-0.184,-0.190,-0.195,-0.199,-0.203,-0.206,     &
                    -0.209,-0.212,-0.215,-0.217,-0.219,-0.221,-0.223,     &
                    -0.225,-0.226,-0.228,-0.229,-0.230,-0.231,-0.232,     &
                    -0.233,-0.234,-0.235,-0.236,-0.236,-0.237,-0.238,     &
                    -0.238,-0.239,-0.239,-0.240,-0.240,-0.240,-0.241,     &
                    -0.241,-0.241,-0.242,-0.242,-0.242,-0.242,-0.243,     &
                    -0.243,-0.243,-0.243,-0.243,-0.243,-0.243,-0.243,     &
                    -0.243,-0.243,-0.243,-0.243,-0.243,-0.243,-0.243,     &
                    -0.243,-0.243,-0.243,-0.243,-0.243,-0.243,-0.243,     &
                    -0.243,-0.242,-0.242,-0.242,-0.242,-0.242,-0.242,     &
                    -0.241,-0.241,-0.241,-0.241,-0.241,-0.240,-0.240,     &
                    -0.240,-0.239,-0.239,-0.239,-0.239,-0.238,-0.238,     &
                    -0.238,-0.237,-0.237,-0.237,-0.236,-0.236,-0.235,     &
                    -0.235,-0.235,-0.234,-0.234,-0.233,-0.233,-0.232,     &
                    -0.232,-0.232,-0.231,-0.231,-0.230,-0.230,-0.229,     &
                    -0.229,-0.228,-0.228,-0.227,-0.227,-0.226,-0.226,     &
                    -0.225,-0.225,-0.224,-0.224,-0.223,-0.223,-0.222,     &
                    -0.222,-0.221,-0.220,-0.220,-0.219,-0.219,-0.218,     &
                    -0.218,-0.217,-0.217,-0.216,-0.216,-0.215,-0.214,     &
                    -0.214,-0.213,-0.213,-0.212,-0.212,-0.211,-0.210,     &
                    -0.210,-0.209,-0.209,-0.208,-0.208,-0.207,-0.207,     &
                    -0.206,-0.205,-0.205,-0.204,-0.204,-0.203,-0.203,     &
                    -0.202,-0.201,-0.201,-0.200,-0.200,-0.199,-0.199,     &
                    -0.198,-0.197,-0.197,-0.196,-0.196,-0.195,-0.194,     &
                    -0.194,-0.193,-0.193,-0.192,-0.192,-0.191,-0.190,     &
                    -0.190,-0.189,-0.189,-0.188,-0.188,-0.187,-0.186,     &
                    -0.186,-0.185,-0.185,-0.184,-0.184,-0.183,-0.182,     &
                    -0.182,-0.181,-0.181,-0.180,-0.180,-0.179,-0.178,     &
                    -0.178,-0.177,-0.177,-0.176,-0.176,-0.175,-0.174,     &
                    -0.174,-0.173,-0.173,-0.172,-0.172,-0.171,-0.170,     &
                    -0.170,-0.169,-0.169,-0.168,-0.168,-0.167,-0.166,     &
                    -0.166,-0.165,-0.165,-0.164,-0.164,-0.163,-0.162,     &
                    -0.162,-0.161,-0.161,-0.160,-0.160,-0.159,-0.158,     &
                    -0.158,-0.157,-0.157,-0.156,-0.156,-0.155,-0.155,     &
                    -0.154,-0.153,-0.153,-0.152,-0.152,-0.151,-0.151,     &
                    -0.150,-0.150,-0.149,-0.148,-0.148,-0.147,-0.147,     &
                    -0.146,-0.146,-0.145,-0.145,-0.144,-0.143,-0.143,     &
                    -0.142,-0.142,-0.141,-0.141,-0.140,-0.140,-0.139,     &
                    -0.139,-0.138,-0.137,-0.137,-0.136,-0.136,-0.135,     &
                    -0.135,-0.134,-0.134,-0.133,-0.133,-0.132,-0.131,     &
                    -0.131,-0.130,-0.130,-0.129,-0.129,-0.128,-0.128,     &
                    -0.127,-0.127,-0.126,-0.126,-0.125,-0.125,-0.124,     &
                    -0.123,-0.123,-0.122,-0.122,-0.121,-0.121,-0.120,     &
                    -0.120,-0.119,-0.119,-0.118,-0.118,-0.117,-0.117,     &
                    -0.116,-0.116,-0.115,-0.115,-0.114,-0.113,-0.113,     &
                    -0.112,-0.112,-0.111,-0.111,-0.110,-0.110,-0.109,     &
                    -0.109,-0.108,-0.108,-0.107,-0.107,-0.106,-0.106,     &
                    -0.105,-0.105,-0.104,-0.104,-0.103,-0.103,-0.102,     &
                    -0.102,-0.101,-0.101,-0.100,-0.100,-0.099,-0.099,     &
                    -0.098,-0.098,-0.097,-0.097,-0.096,-0.096,-0.095,     &
                    -0.095,-0.094,-0.094,-0.093,-0.093,-0.092,-0.092,     &
                    -0.091,-0.091,-0.090,-0.090,-0.089,-0.089,-0.088,     &
                    -0.088,-0.087,-0.087,-0.086,-0.086,-0.085,-0.085,     &
                    -0.084,-0.084,-0.083,-0.083,-0.082,-0.082,-0.081,     &
                    -0.081,-0.080,-0.080,-0.079,-0.079,-0.078,-0.078,     &
                    -0.077,-0.077,-0.077,-0.076,-0.076,-0.075,-0.075,     &
                    -0.074,-0.069,-0.064,-0.060,-0.055,-0.051,-0.046,     &
                    -0.042,-0.037,-0.033,-0.028,-0.024,-0.020,-0.016,     &
                    -0.012,-0.008,-0.004,0.000,0.004,0.008,0.012,0.016,   &
                    0.020,0.024,0.028,0.031,0.035,0.039,0.042,0.046,0.049,&
                    0.053,0.056,0.060,0.063,0.067,0.070,0.073,0.077,0.080,&
                    0.083,0.087,0.090,0.093,0.096,0.099,0.102,0.105,0.108,&
                    0.111,0.114,0.117,0.120,0.123,0.126,0.129,0.132,0.135,&
                    0.138,0.141,0.143,0.146,0.149,0.152,0.154,0.157,0.160,&
                    0.162,0.165,0.168,0.170,0.173,0.176,0.178,0.181,0.183,&
                    0.186,0.188,0.191,0.193,0.196,0.198,0.200,0.203,0.205,&
                    0.208,0.210,0.212,0.215,0.217,0.219,0.221,0.224,0.226,&
                    0.228,0.231,0.233,0.235,0.237,0.239,0.242,0.244,0.246,&
                    0.248,0.250,0.252,0.254,0.256,0.258,0.261,0.263,0.265,&
                    0.267,0.269,0.271,0.273,0.275,0.277,0.279,0.281,0.283,&
                    0.285,0.286,0.288,0.290,0.292,0.294,0.296,0.298,0.300,&
                    0.302,0.304,0.305,0.307,0.309,0.311,0.313,0.314,0.316,&
                    0.318,0.320,0.322,0.323,0.325,0.327,0.329,0.330,0.332,&
                    0.334,0.335,0.337,0.339,0.341,0.342,0.344,0.346,0.347,&
                    0.349,0.350,0.352,0.354,0.355/)
!
! *** mgso4
!
    bnc21m(1:561) = (/-0.185,-0.400,-0.506,-0.579,-0.636,-0.683,-0.723,   &
                    -0.757,-0.787,-0.814,-0.839,-0.861,-0.881,-0.900,     &
                    -0.917,-0.934,-0.949,-0.963,-0.977,-0.989,-1.001,     &
                    -1.013,-1.023,-1.034,-1.044,-1.053,-1.062,-1.071,     &
                    -1.079,-1.087,-1.094,-1.102,-1.109,-1.116,-1.122,     &
                    -1.129,-1.135,-1.141,-1.147,-1.152,-1.158,-1.163,     &
                    -1.168,-1.173,-1.178,-1.183,-1.187,-1.192,-1.196,     &
                    -1.200,-1.205,-1.209,-1.213,-1.216,-1.220,-1.224,     &
                    -1.228,-1.231,-1.235,-1.238,-1.241,-1.244,-1.248,     &
                    -1.251,-1.254,-1.257,-1.260,-1.263,-1.265,-1.268,     &
                    -1.271,-1.274,-1.276,-1.279,-1.281,-1.284,-1.286,     &
                    -1.288,-1.291,-1.293,-1.295,-1.297,-1.300,-1.302,     &
                    -1.304,-1.306,-1.308,-1.310,-1.312,-1.314,-1.316,     &
                    -1.317,-1.319,-1.321,-1.323,-1.324,-1.326,-1.328,     &
                    -1.329,-1.331,-1.333,-1.334,-1.336,-1.337,-1.339,     &
                    -1.340,-1.341,-1.343,-1.344,-1.346,-1.347,-1.348,     &
                    -1.350,-1.351,-1.352,-1.353,-1.355,-1.356,-1.357,     &
                    -1.358,-1.359,-1.360,-1.362,-1.363,-1.364,-1.365,     &
                    -1.366,-1.367,-1.368,-1.369,-1.370,-1.371,-1.372,     &
                    -1.373,-1.374,-1.375,-1.376,-1.377,-1.377,-1.378,     &
                    -1.379,-1.380,-1.381,-1.382,-1.383,-1.383,-1.384,     &
                    -1.385,-1.386,-1.387,-1.387,-1.388,-1.389,-1.390,     &
                    -1.390,-1.391,-1.392,-1.393,-1.393,-1.394,-1.395,     &
                    -1.395,-1.396,-1.397,-1.397,-1.398,-1.399,-1.399,     &
                    -1.400,-1.401,-1.401,-1.402,-1.402,-1.403,-1.404,     &
                    -1.404,-1.405,-1.405,-1.406,-1.406,-1.407,-1.407,     &
                    -1.408,-1.408,-1.409,-1.410,-1.410,-1.411,-1.411,     &
                    -1.412,-1.412,-1.412,-1.413,-1.413,-1.414,-1.414,     &
                    -1.415,-1.415,-1.416,-1.416,-1.417,-1.417,-1.417,     &
                    -1.418,-1.418,-1.419,-1.419,-1.420,-1.420,-1.420,     &
                    -1.421,-1.421,-1.422,-1.422,-1.422,-1.423,-1.423,     &
                    -1.423,-1.424,-1.424,-1.424,-1.425,-1.425,-1.426,     &
                    -1.426,-1.426,-1.427,-1.427,-1.427,-1.428,-1.428,     &
                    -1.428,-1.428,-1.429,-1.429,-1.429,-1.430,-1.430,     &
                    -1.430,-1.431,-1.431,-1.431,-1.431,-1.432,-1.432,     &
                    -1.432,-1.433,-1.433,-1.433,-1.433,-1.434,-1.434,     &
                    -1.434,-1.434,-1.435,-1.435,-1.435,-1.435,-1.436,     &
                    -1.436,-1.436,-1.436,-1.437,-1.437,-1.437,-1.437,     &
                    -1.437,-1.438,-1.438,-1.438,-1.438,-1.439,-1.439,     &
                    -1.439,-1.439,-1.439,-1.440,-1.440,-1.440,-1.440,     &
                    -1.440,-1.441,-1.441,-1.441,-1.441,-1.441,-1.441,     &
                    -1.442,-1.442,-1.442,-1.442,-1.442,-1.442,-1.443,     &
                    -1.443,-1.443,-1.443,-1.443,-1.443,-1.444,-1.444,     &
                    -1.444,-1.444,-1.444,-1.444,-1.445,-1.445,-1.445,     &
                    -1.445,-1.445,-1.445,-1.445,-1.446,-1.446,-1.446,     &
                    -1.446,-1.446,-1.446,-1.446,-1.446,-1.447,-1.447,     &
                    -1.447,-1.447,-1.447,-1.447,-1.447,-1.447,-1.448,     &
                    -1.448,-1.448,-1.448,-1.448,-1.448,-1.448,-1.448,     &
                    -1.448,-1.449,-1.449,-1.449,-1.449,-1.449,-1.449,     &
                    -1.449,-1.449,-1.449,-1.449,-1.450,-1.450,-1.450,     &
                    -1.450,-1.450,-1.450,-1.450,-1.450,-1.450,-1.450,     &
                    -1.450,-1.451,-1.451,-1.451,-1.451,-1.451,-1.451,     &
                    -1.451,-1.451,-1.451,-1.451,-1.451,-1.451,-1.451,     &
                    -1.451,-1.452,-1.452,-1.452,-1.452,-1.452,-1.452,     &
                    -1.452,-1.452,-1.452,-1.452,-1.452,-1.452,-1.452,     &
                    -1.452,-1.452,-1.453,-1.453,-1.453,-1.453,-1.453,     &
                    -1.453,-1.453,-1.453,-1.453,-1.453,-1.453,-1.453,     &
                    -1.453,-1.454,-1.454,-1.454,-1.455,-1.455,-1.455,     &
                    -1.455,-1.455,-1.455,-1.455,-1.455,-1.455,-1.455,     &
                    -1.455,-1.454,-1.454,-1.454,-1.454,-1.453,-1.453,     &
                    -1.453,-1.452,-1.452,-1.452,-1.451,-1.451,-1.450,     &
                    -1.450,-1.450,-1.449,-1.449,-1.448,-1.448,-1.447,     &
                    -1.447,-1.446,-1.446,-1.445,-1.445,-1.444,-1.444,     &
                    -1.443,-1.442,-1.442,-1.441,-1.441,-1.440,-1.440,     &
                    -1.439,-1.438,-1.438,-1.437,-1.437,-1.436,-1.436,     &
                    -1.435,-1.434,-1.434,-1.433,-1.433,-1.432,-1.431,     &
                    -1.431,-1.430,-1.430,-1.429,-1.428,-1.428,-1.427,     &
                    -1.426,-1.426,-1.425,-1.425,-1.424,-1.423,-1.423,     &
                    -1.422,-1.422,-1.421,-1.420,-1.420,-1.419,-1.418,     &
                    -1.418,-1.417,-1.417,-1.416,-1.415,-1.415,-1.414,     &
                    -1.414,-1.413,-1.412,-1.412,-1.411,-1.411,-1.410,     &
                    -1.409,-1.409,-1.408,-1.408,-1.407,-1.406,-1.406,     &
                    -1.405,-1.405,-1.404,-1.403,-1.403,-1.402,-1.402,     &
                    -1.401,-1.400,-1.400,-1.399,-1.399,-1.398,-1.398,     &
                    -1.397,-1.396,-1.396,-1.395,-1.395,-1.394,-1.394,     &
                    -1.393,-1.392,-1.392,-1.391,-1.391,-1.390,-1.390,     &
                    -1.389,-1.388,-1.388,-1.387,-1.387,-1.386,-1.386,     &
                    -1.385,-1.385,-1.384,-1.383,-1.383,-1.382,-1.382,     &
                    -1.381,-1.381,-1.380,-1.380,-1.379,-1.378,-1.378,     &
                    -1.377,-1.377,-1.376,-1.376,-1.375,-1.375,-1.374,     &
                    -1.374/)
!
! *** mgno32
!
    bnc22m(1:561) = (/-0.091,-0.189,-0.233,-0.262,-0.283,-0.299,-0.312,   &
                    -0.322,-0.330,-0.337,-0.343,-0.347,-0.351,-0.354,     &
                    -0.356,-0.358,-0.359,-0.360,-0.361,-0.361,-0.361,     &
                    -0.360,-0.360,-0.359,-0.358,-0.356,-0.355,-0.354,     &
                    -0.352,-0.350,-0.348,-0.346,-0.344,-0.342,-0.340,     &
                    -0.337,-0.335,-0.332,-0.330,-0.327,-0.325,-0.322,     &
                    -0.319,-0.316,-0.314,-0.311,-0.308,-0.305,-0.302,     &
                    -0.299,-0.297,-0.294,-0.291,-0.288,-0.285,-0.282,     &
                    -0.279,-0.276,-0.273,-0.270,-0.267,-0.263,-0.260,     &
                    -0.257,-0.254,-0.251,-0.248,-0.245,-0.242,-0.238,     &
                    -0.235,-0.232,-0.229,-0.225,-0.222,-0.219,-0.215,     &
                    -0.212,-0.209,-0.205,-0.202,-0.198,-0.195,-0.191,     &
                    -0.188,-0.184,-0.181,-0.177,-0.173,-0.170,-0.166,     &
                    -0.162,-0.159,-0.155,-0.151,-0.147,-0.143,-0.140,     &
                    -0.136,-0.132,-0.128,-0.124,-0.120,-0.116,-0.112,     &
                    -0.108,-0.104,-0.100,-0.096,-0.091,-0.087,-0.083,     &
                    -0.079,-0.075,-0.071,-0.067,-0.062,-0.058,-0.054,     &
                    -0.050,-0.046,-0.041,-0.037,-0.033,-0.029,-0.024,     &
                    -0.020,-0.016,-0.012,-0.008,-0.003,0.001,0.005,0.009, &
                    0.014,0.018,0.022,0.026,0.031,0.035,0.039,0.043,0.047,&
                    0.052,0.056,0.060,0.064,0.069,0.073,0.077,0.081,0.085,&
                    0.090,0.094,0.098,0.102,0.106,0.110,0.115,0.119,0.123,&
                    0.127,0.131,0.135,0.140,0.144,0.148,0.152,0.156,0.160,&
                    0.164,0.169,0.173,0.177,0.181,0.185,0.189,0.193,0.197,&
                    0.201,0.205,0.209,0.214,0.218,0.222,0.226,0.230,0.234,&
                    0.238,0.242,0.246,0.250,0.254,0.258,0.262,0.266,0.270,&
                    0.274,0.278,0.282,0.286,0.290,0.294,0.298,0.302,0.306,&
                    0.310,0.314,0.318,0.321,0.325,0.329,0.333,0.337,0.341,&
                    0.345,0.349,0.353,0.357,0.361,0.364,0.368,0.372,0.376,&
                    0.380,0.384,0.388,0.391,0.395,0.399,0.403,0.407,0.410,&
                    0.414,0.418,0.422,0.426,0.429,0.433,0.437,0.441,0.445,&
                    0.448,0.452,0.456,0.459,0.463,0.467,0.471,0.474,0.478,&
                    0.482,0.486,0.489,0.493,0.497,0.500,0.504,0.508,0.511,&
                    0.515,0.519,0.522,0.526,0.530,0.533,0.537,0.540,0.544,&
                    0.548,0.551,0.555,0.559,0.562,0.566,0.569,0.573,0.576,&
                    0.580,0.584,0.587,0.591,0.594,0.598,0.601,0.605,0.608,&
                    0.612,0.615,0.619,0.622,0.626,0.629,0.633,0.636,0.640,&
                    0.643,0.647,0.650,0.654,0.657,0.661,0.664,0.668,0.671,&
                    0.674,0.678,0.681,0.685,0.688,0.691,0.695,0.698,0.702,&
                    0.705,0.708,0.712,0.715,0.719,0.722,0.725,0.729,0.732,&
                    0.735,0.739,0.742,0.745,0.749,0.752,0.755,0.759,0.762,&
                    0.765,0.768,0.772,0.775,0.778,0.782,0.785,0.788,0.791,&
                    0.795,0.798,0.801,0.804,0.808,0.811,0.814,0.817,0.821,&
                    0.824,0.827,0.830,0.833,0.837,0.840,0.843,0.846,0.849,&
                    0.853,0.856,0.859,0.862,0.865,0.868,0.872,0.875,0.878,&
                    0.881,0.884,0.887,0.890,0.893,0.897,0.900,0.903,0.906,&
                    0.909,0.912,0.915,0.918,0.921,0.924,0.928,0.931,0.934,&
                    0.937,0.940,0.943,0.946,0.949,0.952,0.955,0.958,0.961,&
                    0.964,0.967,0.970,0.973,0.976,1.008,1.038,1.067,1.095,&
                    1.124,1.152,1.179,1.206,1.233,1.260,1.286,1.312,1.337,&
                    1.363,1.387,1.412,1.437,1.461,1.484,1.508,1.531,1.554,&
                    1.577,1.600,1.622,1.644,1.666,1.687,1.709,1.730,1.751,&
                    1.772,1.792,1.812,1.832,1.852,1.872,1.892,1.911,1.930,&
                    1.949,1.968,1.987,2.005,2.023,2.042,2.060,2.077,2.095,&
                    2.113,2.130,2.147,2.164,2.181,2.198,2.215,2.231,2.248,&
                    2.264,2.280,2.296,2.312,2.328,2.343,2.359,2.374,2.389,&
                    2.405,2.420,2.435,2.449,2.464,2.479,2.493,2.508,2.522,&
                    2.536,2.550,2.564,2.578,2.592,2.606,2.619,2.633,2.646,&
                    2.660,2.673,2.686,2.699,2.712,2.725,2.738,2.751,2.763,&
                    2.776,2.788,2.801,2.813,2.826,2.838,2.850,2.862,2.874,&
                    2.886,2.898,2.910,2.921,2.933,2.945,2.956,2.967,2.979,&
                    2.990,3.001,3.013,3.024,3.035,3.046,3.057,3.068,3.079,&
                    3.089,3.100,3.111,3.121,3.132,3.142,3.153,3.163,3.173,&
                    3.184,3.194,3.204,3.214,3.224,3.234,3.244,3.254,3.264,&
                    3.274,3.284,3.293,3.303,3.313,3.322,3.332,3.341,3.351,&
                    3.360,3.370,3.379,3.388,3.397,3.407,3.416,3.425,3.434,&
                    3.443,3.452,3.461,3.470/)
!
! *** mgcl2
!
    bnc23m(1:561) = (/-0.090,-0.186,-0.229,-0.256,-0.276,-0.290,-0.302,   &
                    -0.310,-0.317,-0.322,-0.327,-0.330,-0.332,-0.333,     &
                    -0.334,-0.335,-0.335,-0.334,-0.333,-0.332,-0.331,     &
                    -0.329,-0.327,-0.325,-0.322,-0.320,-0.317,-0.314,     &
                    -0.311,-0.308,-0.305,-0.301,-0.298,-0.294,-0.291,     &
                    -0.287,-0.283,-0.279,-0.275,-0.272,-0.268,-0.264,     &
                    -0.260,-0.255,-0.251,-0.247,-0.243,-0.239,-0.235,     &
                    -0.231,-0.226,-0.222,-0.218,-0.214,-0.209,-0.205,     &
                    -0.201,-0.196,-0.192,-0.188,-0.184,-0.179,-0.175,     &
                    -0.170,-0.166,-0.162,-0.157,-0.153,-0.148,-0.144,     &
                    -0.139,-0.135,-0.130,-0.126,-0.121,-0.117,-0.112,     &
                    -0.107,-0.103,-0.098,-0.093,-0.089,-0.084,-0.079,     &
                    -0.074,-0.069,-0.065,-0.060,-0.055,-0.050,-0.045,     &
                    -0.040,-0.034,-0.029,-0.024,-0.019,-0.014,-0.009,     &
                    -0.003,0.002,0.007,0.013,0.018,0.023,0.029,0.034,     &
                    0.040,0.045,0.050,0.056,0.061,0.067,0.073,0.078,0.084,&
                    0.089,0.095,0.100,0.106,0.112,0.117,0.123,0.128,0.134,&
                    0.140,0.145,0.151,0.157,0.162,0.168,0.174,0.179,0.185,&
                    0.190,0.196,0.202,0.207,0.213,0.219,0.224,0.230,0.235,&
                    0.241,0.247,0.252,0.258,0.263,0.269,0.275,0.280,0.286,&
                    0.291,0.297,0.302,0.308,0.313,0.319,0.325,0.330,0.336,&
                    0.341,0.347,0.352,0.358,0.363,0.369,0.374,0.380,0.385,&
                    0.390,0.396,0.401,0.407,0.412,0.418,0.423,0.428,0.434,&
                    0.439,0.445,0.450,0.455,0.461,0.466,0.471,0.477,0.482,&
                    0.487,0.493,0.498,0.503,0.509,0.514,0.519,0.525,0.530,&
                    0.535,0.540,0.546,0.551,0.556,0.561,0.567,0.572,0.577,&
                    0.582,0.587,0.593,0.598,0.603,0.608,0.613,0.618,0.624,&
                    0.629,0.634,0.639,0.644,0.649,0.654,0.659,0.664,0.669,&
                    0.675,0.680,0.685,0.690,0.695,0.700,0.705,0.710,0.715,&
                    0.720,0.725,0.730,0.735,0.740,0.745,0.750,0.755,0.759,&
                    0.764,0.769,0.774,0.779,0.784,0.789,0.794,0.799,0.804,&
                    0.808,0.813,0.818,0.823,0.828,0.833,0.837,0.842,0.847,&
                    0.852,0.857,0.861,0.866,0.871,0.876,0.880,0.885,0.890,&
                    0.895,0.899,0.904,0.909,0.914,0.918,0.923,0.928,0.932,&
                    0.937,0.942,0.946,0.951,0.955,0.960,0.965,0.969,0.974,&
                    0.979,0.983,0.988,0.992,0.997,1.001,1.006,1.011,1.015,&
                    1.020,1.024,1.029,1.033,1.038,1.042,1.047,1.051,1.056,&
                    1.060,1.065,1.069,1.073,1.078,1.082,1.087,1.091,1.096,&
                    1.100,1.104,1.109,1.113,1.118,1.122,1.126,1.131,1.135,&
                    1.139,1.144,1.148,1.152,1.157,1.161,1.165,1.170,1.174,&
                    1.178,1.183,1.187,1.191,1.195,1.200,1.204,1.208,1.212,&
                    1.217,1.221,1.225,1.229,1.234,1.238,1.242,1.246,1.250,&
                    1.255,1.259,1.263,1.267,1.271,1.275,1.279,1.284,1.288,&
                    1.292,1.296,1.300,1.304,1.308,1.312,1.316,1.321,1.325,&
                    1.329,1.333,1.337,1.341,1.345,1.349,1.353,1.357,1.361,&
                    1.365,1.369,1.373,1.377,1.381,1.385,1.389,1.393,1.397,&
                    1.401,1.405,1.409,1.413,1.417,1.421,1.425,1.428,1.432,&
                    1.436,1.440,1.444,1.448,1.452,1.456,1.497,1.535,1.573,&
                    1.610,1.646,1.682,1.717,1.752,1.787,1.821,1.854,1.888,&
                    1.920,1.953,1.985,2.016,2.048,2.078,2.109,2.139,2.169,&
                    2.198,2.227,2.256,2.285,2.313,2.341,2.368,2.395,2.422,&
                    2.449,2.475,2.501,2.527,2.553,2.578,2.603,2.628,2.653,&
                    2.677,2.701,2.725,2.749,2.772,2.796,2.819,2.841,2.864,&
                    2.887,2.909,2.931,2.953,2.974,2.996,3.017,3.038,3.059,&
                    3.080,3.100,3.121,3.141,3.161,3.181,3.201,3.221,3.240,&
                    3.259,3.279,3.298,3.317,3.335,3.354,3.372,3.391,3.409,&
                    3.427,3.445,3.463,3.480,3.498,3.516,3.533,3.550,3.567,&
                    3.584,3.601,3.618,3.634,3.651,3.667,3.684,3.700,3.716,&
                    3.732,3.748,3.764,3.779,3.795,3.810,3.826,3.841,3.856,&
                    3.872,3.887,3.902,3.916,3.931,3.946,3.961,3.975,3.989,&
                    4.004,4.018,4.032,4.046,4.060,4.074,4.088,4.102,4.116,&
                    4.129,4.143,4.157,4.170,4.183,4.197,4.210,4.223,4.236,&
                    4.249,4.262,4.275,4.288,4.300,4.313,4.326,4.338,4.351,&
                    4.363,4.376,4.388,4.400,4.412,4.424,4.436,4.449,4.460,&
                    4.472,4.484,4.496,4.508,4.519,4.531,4.543,4.554,4.566,&
                    4.577,4.588,4.600,4.611,4.622/)
!
! *** find position in arrays for bincoef
!
    if ( ionic<=0.200000D+02 ) then
      ipos = min(nint(0.200000D+02*ionic)+1,400)
    else
      ipos = 400 + nint(0.200000D+01*ionic-0.400000D+02)
    end if
    ipos = min(ipos,561)
!
! *** assign values to return array
!
    binarr(01) = bnc01m(ipos)
    binarr(02) = bnc02m(ipos)
    binarr(03) = bnc03m(ipos)
    binarr(04) = bnc04m(ipos)
    binarr(05) = bnc05m(ipos)
    binarr(06) = bnc06m(ipos)
    binarr(07) = bnc07m(ipos)
    binarr(08) = bnc08m(ipos)
    binarr(09) = bnc09m(ipos)
    binarr(10) = bnc10m(ipos)
    binarr(11) = bnc11m(ipos)
    binarr(12) = bnc12m(ipos)
    binarr(13) = bnc13m(ipos)
    binarr(14) = bnc14m(ipos)
    binarr(15) = bnc15m(ipos)
    binarr(16) = bnc16m(ipos)
    binarr(17) = bnc17m(ipos)
    binarr(18) = bnc18m(ipos)
    binarr(19) = bnc19m(ipos)
    binarr(20) = bnc20m(ipos)
    binarr(21) = bnc21m(ipos)
    binarr(22) = bnc22m(ipos)
    binarr(23) = bnc23m(ipos)
  end subroutine km298
!
!======================================================================
!
! *** isorropia code
! *** subroutine km323
! *** calculates binary activity coefficients by kusik-meissner method.
!     the computations have been performed and the results are stored in
!     lookup tables. the ionic activity 'in' is input , and the array
!     'binarr' is returned with the binary coefficients.
!
!     temperature is 323k
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine km323(ionic,binarr)
    implicit none
    integer(ik4) :: ipos
    real(rkx) , dimension(23) :: binarr
    real(rkx) :: ionic
    real(rkx) , dimension(561) :: bnc01m , bnc02m , bnc03m , bnc04m ,     &
                                  bnc05m , bnc06m , bnc07m , bnc08m ,     &
                                  bnc09m , bnc10m , bnc11m , bnc12m ,     &
                                  bnc13m , bnc14m , bnc15m , bnc16m ,     &
                                  bnc17m , bnc18m , bnc19m , bnc20m ,     &
                                  bnc21m , bnc22m , bnc23m
!
! *** nacl
!
    bnc01m(1:561) = (/-0.044,-0.092,-0.114,-0.129,-0.139,-0.147,-0.154,   &
                    -0.159,-0.163,-0.167,-0.170,-0.172,-0.174,-0.176,     &
                    -0.177,-0.178,-0.179,-0.180,-0.180,-0.181,-0.181,     &
                    -0.181,-0.181,-0.180,-0.180,-0.180,-0.179,-0.179,     &
                    -0.178,-0.177,-0.177,-0.176,-0.175,-0.174,-0.173,     &
                    -0.172,-0.171,-0.170,-0.169,-0.168,-0.167,-0.166,     &
                    -0.165,-0.163,-0.162,-0.161,-0.160,-0.159,-0.157,     &
                    -0.156,-0.155,-0.153,-0.152,-0.151,-0.150,-0.148,     &
                    -0.147,-0.146,-0.144,-0.143,-0.142,-0.140,-0.139,     &
                    -0.137,-0.136,-0.135,-0.133,-0.132,-0.130,-0.129,     &
                    -0.127,-0.126,-0.125,-0.123,-0.122,-0.120,-0.119,     &
                    -0.117,-0.116,-0.114,-0.113,-0.111,-0.109,-0.108,     &
                    -0.106,-0.105,-0.103,-0.101,-0.100,-0.098,-0.096,     &
                    -0.095,-0.093,-0.091,-0.090,-0.088,-0.086,-0.084,     &
                    -0.083,-0.081,-0.079,-0.077,-0.076,-0.074,-0.072,     &
                    -0.070,-0.068,-0.066,-0.065,-0.063,-0.061,-0.059,     &
                    -0.057,-0.055,-0.053,-0.052,-0.050,-0.048,-0.046,     &
                    -0.044,-0.042,-0.040,-0.038,-0.036,-0.035,-0.033,     &
                    -0.031,-0.029,-0.027,-0.025,-0.023,-0.021,-0.019,     &
                    -0.017,-0.015,-0.013,-0.012,-0.010,-0.008,-0.006,     &
                    -0.004,-0.002,0.000,0.002,0.004,0.006,0.008,0.009,    &
                    0.011,0.013,0.015,0.017,0.019,0.021,0.023,0.025,0.027,&
                    0.028,0.030,0.032,0.034,0.036,0.038,0.040,0.042,0.044,&
                    0.045,0.047,0.049,0.051,0.053,0.055,0.057,0.059,0.060,&
                    0.062,0.064,0.066,0.068,0.070,0.072,0.074,0.075,0.077,&
                    0.079,0.081,0.083,0.085,0.086,0.088,0.090,0.092,0.094,&
                    0.096,0.097,0.099,0.101,0.103,0.105,0.107,0.108,0.110,&
                    0.112,0.114,0.116,0.117,0.119,0.121,0.123,0.125,0.127,&
                    0.128,0.130,0.132,0.134,0.135,0.137,0.139,0.141,0.143,&
                    0.144,0.146,0.148,0.150,0.151,0.153,0.155,0.157,0.159,&
                    0.160,0.162,0.164,0.166,0.167,0.169,0.171,0.173,0.174,&
                    0.176,0.178,0.180,0.181,0.183,0.185,0.186,0.188,0.190,&
                    0.192,0.193,0.195,0.197,0.198,0.200,0.202,0.204,0.205,&
                    0.207,0.209,0.210,0.212,0.214,0.215,0.217,0.219,0.221,&
                    0.222,0.224,0.226,0.227,0.229,0.231,0.232,0.234,0.236,&
                    0.237,0.239,0.241,0.242,0.244,0.246,0.247,0.249,0.250,&
                    0.252,0.254,0.255,0.257,0.259,0.260,0.262,0.264,0.265,&
                    0.267,0.268,0.270,0.272,0.273,0.275,0.276,0.278,0.280,&
                    0.281,0.283,0.285,0.286,0.288,0.289,0.291,0.293,0.294,&
                    0.296,0.297,0.299,0.300,0.302,0.304,0.305,0.307,0.308,&
                    0.310,0.311,0.313,0.315,0.316,0.318,0.319,0.321,0.322,&
                    0.324,0.325,0.327,0.329,0.330,0.332,0.333,0.335,0.336,&
                    0.338,0.339,0.341,0.342,0.344,0.345,0.347,0.349,0.350,&
                    0.352,0.353,0.355,0.356,0.358,0.359,0.361,0.362,0.364,&
                    0.365,0.367,0.368,0.370,0.371,0.373,0.374,0.376,0.377,&
                    0.379,0.380,0.382,0.383,0.384,0.386,0.387,0.389,0.390,&
                    0.392,0.393,0.395,0.396,0.398,0.399,0.401,0.402,0.404,&
                    0.405,0.406,0.408,0.409,0.411,0.412,0.414,0.415,0.417,&
                    0.418,0.419,0.421,0.422,0.424,0.425,0.427,0.428,0.429,&
                    0.445,0.459,0.472,0.486,0.499,0.513,0.526,0.539,0.552,&
                    0.565,0.577,0.590,0.602,0.614,0.626,0.638,0.650,0.662,&
                    0.674,0.685,0.697,0.708,0.719,0.730,0.741,0.752,0.763,&
                    0.773,0.784,0.795,0.805,0.815,0.825,0.836,0.846,0.856,&
                    0.866,0.875,0.885,0.895,0.904,0.914,0.923,0.933,0.942,&
                    0.951,0.960,0.969,0.979,0.987,0.996,1.005,1.014,1.023,&
                    1.031,1.040,1.049,1.057,1.065,1.074,1.082,1.090,1.099,&
                    1.107,1.115,1.123,1.131,1.139,1.147,1.155,1.163,1.170,&
                    1.178,1.186,1.193,1.201,1.209,1.216,1.224,1.231,1.238,&
                    1.246,1.253,1.260,1.267,1.275,1.282,1.289,1.296,1.303,&
                    1.310,1.317,1.324,1.331,1.338,1.345,1.351,1.358,1.365,&
                    1.372,1.378,1.385,1.392,1.398,1.405,1.411,1.418,1.424,&
                    1.431,1.437,1.443,1.450,1.456,1.462,1.469,1.475,1.481,&
                    1.487,1.493,1.500,1.506,1.512,1.518,1.524,1.530,1.536,&
                    1.542,1.548,1.554,1.560,1.565,1.571,1.577,1.583,1.589,&
                    1.594,1.600,1.606,1.612,1.617,1.623,1.629,1.634,1.640,&
                    1.645,1.651,1.656,1.662,1.667,1.673,1.678,1.684,1.689,&
                    1.695,1.700,1.705,1.711,1.716,1.721,1.727,1.732/)
!
! *** na2so4
!
    bnc02m(1:561) = (/-0.091,-0.196,-0.249,-0.285,-0.314,-0.337,-0.358,   &
                    -0.375,-0.390,-0.404,-0.417,-0.428,-0.439,-0.449,     &
                    -0.458,-0.466,-0.475,-0.482,-0.489,-0.496,-0.502,     &
                    -0.508,-0.514,-0.520,-0.525,-0.530,-0.535,-0.540,     &
                    -0.544,-0.549,-0.553,-0.557,-0.561,-0.565,-0.569,     &
                    -0.572,-0.576,-0.579,-0.582,-0.585,-0.588,-0.591,     &
                    -0.594,-0.597,-0.600,-0.603,-0.605,-0.608,-0.610,     &
                    -0.613,-0.615,-0.617,-0.620,-0.622,-0.624,-0.626,     &
                    -0.628,-0.630,-0.632,-0.634,-0.636,-0.638,-0.640,     &
                    -0.641,-0.643,-0.645,-0.647,-0.648,-0.650,-0.651,     &
                    -0.653,-0.654,-0.656,-0.657,-0.659,-0.660,-0.662,     &
                    -0.663,-0.665,-0.666,-0.667,-0.669,-0.670,-0.671,     &
                    -0.672,-0.674,-0.675,-0.676,-0.677,-0.678,-0.679,     &
                    -0.681,-0.682,-0.683,-0.684,-0.685,-0.686,-0.687,     &
                    -0.688,-0.689,-0.690,-0.691,-0.692,-0.693,-0.694,     &
                    -0.695,-0.696,-0.697,-0.698,-0.699,-0.700,-0.701,     &
                    -0.701,-0.702,-0.703,-0.704,-0.705,-0.706,-0.706,     &
                    -0.707,-0.708,-0.709,-0.710,-0.710,-0.711,-0.712,     &
                    -0.713,-0.713,-0.714,-0.715,-0.716,-0.716,-0.717,     &
                    -0.718,-0.718,-0.719,-0.720,-0.720,-0.721,-0.722,     &
                    -0.722,-0.723,-0.724,-0.724,-0.725,-0.725,-0.726,     &
                    -0.727,-0.727,-0.728,-0.728,-0.729,-0.730,-0.730,     &
                    -0.731,-0.731,-0.732,-0.732,-0.733,-0.733,-0.734,     &
                    -0.734,-0.735,-0.735,-0.736,-0.736,-0.737,-0.737,     &
                    -0.738,-0.738,-0.739,-0.739,-0.740,-0.740,-0.741,     &
                    -0.741,-0.742,-0.742,-0.742,-0.743,-0.743,-0.744,     &
                    -0.744,-0.745,-0.745,-0.745,-0.746,-0.746,-0.747,     &
                    -0.747,-0.747,-0.748,-0.748,-0.748,-0.749,-0.749,     &
                    -0.750,-0.750,-0.750,-0.751,-0.751,-0.751,-0.752,     &
                    -0.752,-0.752,-0.753,-0.753,-0.753,-0.754,-0.754,     &
                    -0.754,-0.755,-0.755,-0.755,-0.755,-0.756,-0.756,     &
                    -0.756,-0.757,-0.757,-0.757,-0.758,-0.758,-0.758,     &
                    -0.758,-0.759,-0.759,-0.759,-0.759,-0.760,-0.760,     &
                    -0.760,-0.760,-0.761,-0.761,-0.761,-0.761,-0.762,     &
                    -0.762,-0.762,-0.762,-0.763,-0.763,-0.763,-0.763,     &
                    -0.764,-0.764,-0.764,-0.764,-0.764,-0.765,-0.765,     &
                    -0.765,-0.765,-0.765,-0.766,-0.766,-0.766,-0.766,     &
                    -0.766,-0.767,-0.767,-0.767,-0.767,-0.767,-0.768,     &
                    -0.768,-0.768,-0.768,-0.768,-0.768,-0.769,-0.769,     &
                    -0.769,-0.769,-0.769,-0.769,-0.770,-0.770,-0.770,     &
                    -0.770,-0.770,-0.770,-0.771,-0.771,-0.771,-0.771,     &
                    -0.771,-0.771,-0.771,-0.771,-0.772,-0.772,-0.772,     &
                    -0.772,-0.772,-0.772,-0.772,-0.773,-0.773,-0.773,     &
                    -0.773,-0.773,-0.773,-0.773,-0.773,-0.773,-0.774,     &
                    -0.774,-0.774,-0.774,-0.774,-0.774,-0.774,-0.774,     &
                    -0.774,-0.775,-0.775,-0.775,-0.775,-0.775,-0.775,     &
                    -0.775,-0.775,-0.775,-0.775,-0.775,-0.775,-0.776,     &
                    -0.776,-0.776,-0.776,-0.776,-0.776,-0.776,-0.776,     &
                    -0.776,-0.776,-0.776,-0.776,-0.776,-0.777,-0.777,     &
                    -0.777,-0.777,-0.777,-0.777,-0.777,-0.777,-0.777,     &
                    -0.777,-0.777,-0.777,-0.777,-0.777,-0.777,-0.777,     &
                    -0.777,-0.777,-0.778,-0.778,-0.778,-0.778,-0.778,     &
                    -0.778,-0.778,-0.778,-0.778,-0.778,-0.778,-0.778,     &
                    -0.778,-0.778,-0.778,-0.778,-0.778,-0.778,-0.778,     &
                    -0.778,-0.778,-0.778,-0.778,-0.778,-0.778,-0.778,     &
                    -0.778,-0.778,-0.778,-0.778,-0.778,-0.778,-0.778,     &
                    -0.778,-0.778,-0.779,-0.779,-0.779,-0.779,-0.779,     &
                    -0.779,-0.779,-0.779,-0.778,-0.778,-0.778,-0.778,     &
                    -0.777,-0.777,-0.776,-0.776,-0.775,-0.774,-0.774,     &
                    -0.773,-0.772,-0.771,-0.770,-0.769,-0.768,-0.767,     &
                    -0.766,-0.765,-0.764,-0.763,-0.762,-0.760,-0.759,     &
                    -0.758,-0.756,-0.755,-0.754,-0.752,-0.751,-0.749,     &
                    -0.748,-0.746,-0.745,-0.743,-0.742,-0.740,-0.739,     &
                    -0.737,-0.735,-0.734,-0.732,-0.730,-0.729,-0.727,     &
                    -0.725,-0.723,-0.722,-0.720,-0.718,-0.716,-0.714,     &
                    -0.713,-0.711,-0.709,-0.707,-0.705,-0.703,-0.701,     &
                    -0.699,-0.697,-0.695,-0.693,-0.691,-0.690,-0.688,     &
                    -0.686,-0.683,-0.681,-0.679,-0.677,-0.675,-0.673,     &
                    -0.671,-0.669,-0.667,-0.665,-0.663,-0.661,-0.659,     &
                    -0.656,-0.654,-0.652,-0.650,-0.648,-0.646,-0.644,     &
                    -0.641,-0.639,-0.637,-0.635,-0.633,-0.630,-0.628,     &
                    -0.626,-0.624,-0.621,-0.619,-0.617,-0.615,-0.612,     &
                    -0.610,-0.608,-0.606,-0.603,-0.601,-0.599,-0.597,     &
                    -0.594,-0.592,-0.590,-0.587,-0.585,-0.583,-0.580,     &
                    -0.578,-0.576,-0.573,-0.571,-0.569,-0.566,-0.564,     &
                    -0.562,-0.559,-0.557,-0.555,-0.552,-0.550,-0.547,     &
                    -0.545,-0.543,-0.540,-0.538,-0.536,-0.533,-0.531,     &
                    -0.528,-0.526,-0.524,-0.521,-0.519,-0.516,-0.514,     &
                    -0.511,-0.509,-0.507,-0.504,-0.502,-0.499,-0.497,     &
                    -0.494,-0.492,-0.490,-0.487,-0.485,-0.482,-0.480,     &
                    -0.477/)
!
! *** nano3
!
    bnc03m(1:561) = (/-0.045,-0.099,-0.125,-0.144,-0.159,-0.171,-0.181,   &
                    -0.190,-0.198,-0.206,-0.212,-0.218,-0.224,-0.229,     &
                    -0.234,-0.239,-0.243,-0.247,-0.251,-0.255,-0.258,     &
                    -0.261,-0.265,-0.268,-0.271,-0.274,-0.276,-0.279,     &
                    -0.281,-0.284,-0.286,-0.289,-0.291,-0.293,-0.295,     &
                    -0.297,-0.299,-0.301,-0.303,-0.305,-0.306,-0.308,     &
                    -0.310,-0.312,-0.313,-0.315,-0.316,-0.318,-0.319,     &
                    -0.321,-0.322,-0.323,-0.325,-0.326,-0.327,-0.329,     &
                    -0.330,-0.331,-0.332,-0.333,-0.335,-0.336,-0.337,     &
                    -0.338,-0.339,-0.340,-0.341,-0.342,-0.343,-0.344,     &
                    -0.345,-0.346,-0.347,-0.348,-0.349,-0.350,-0.350,     &
                    -0.351,-0.352,-0.353,-0.354,-0.355,-0.356,-0.356,     &
                    -0.357,-0.358,-0.359,-0.359,-0.360,-0.361,-0.362,     &
                    -0.363,-0.363,-0.364,-0.365,-0.365,-0.366,-0.367,     &
                    -0.368,-0.368,-0.369,-0.370,-0.370,-0.371,-0.372,     &
                    -0.372,-0.373,-0.374,-0.374,-0.375,-0.375,-0.376,     &
                    -0.377,-0.377,-0.378,-0.378,-0.379,-0.380,-0.380,     &
                    -0.381,-0.381,-0.382,-0.383,-0.383,-0.384,-0.384,     &
                    -0.385,-0.385,-0.386,-0.386,-0.387,-0.387,-0.388,     &
                    -0.388,-0.389,-0.389,-0.390,-0.390,-0.391,-0.391,     &
                    -0.392,-0.392,-0.393,-0.393,-0.394,-0.394,-0.395,     &
                    -0.395,-0.396,-0.396,-0.397,-0.397,-0.397,-0.398,     &
                    -0.398,-0.399,-0.399,-0.400,-0.400,-0.400,-0.401,     &
                    -0.401,-0.402,-0.402,-0.402,-0.403,-0.403,-0.404,     &
                    -0.404,-0.404,-0.405,-0.405,-0.406,-0.406,-0.406,     &
                    -0.407,-0.407,-0.407,-0.408,-0.408,-0.408,-0.409,     &
                    -0.409,-0.409,-0.410,-0.410,-0.411,-0.411,-0.411,     &
                    -0.412,-0.412,-0.412,-0.413,-0.413,-0.413,-0.413,     &
                    -0.414,-0.414,-0.414,-0.415,-0.415,-0.415,-0.416,     &
                    -0.416,-0.416,-0.417,-0.417,-0.417,-0.417,-0.418,     &
                    -0.418,-0.418,-0.419,-0.419,-0.419,-0.419,-0.420,     &
                    -0.420,-0.420,-0.421,-0.421,-0.421,-0.421,-0.422,     &
                    -0.422,-0.422,-0.422,-0.423,-0.423,-0.423,-0.423,     &
                    -0.424,-0.424,-0.424,-0.424,-0.425,-0.425,-0.425,     &
                    -0.425,-0.426,-0.426,-0.426,-0.426,-0.427,-0.427,     &
                    -0.427,-0.427,-0.428,-0.428,-0.428,-0.428,-0.428,     &
                    -0.429,-0.429,-0.429,-0.429,-0.430,-0.430,-0.430,     &
                    -0.430,-0.430,-0.431,-0.431,-0.431,-0.431,-0.431,     &
                    -0.432,-0.432,-0.432,-0.432,-0.432,-0.433,-0.433,     &
                    -0.433,-0.433,-0.433,-0.434,-0.434,-0.434,-0.434,     &
                    -0.434,-0.435,-0.435,-0.435,-0.435,-0.435,-0.435,     &
                    -0.436,-0.436,-0.436,-0.436,-0.436,-0.436,-0.437,     &
                    -0.437,-0.437,-0.437,-0.437,-0.438,-0.438,-0.438,     &
                    -0.438,-0.438,-0.438,-0.438,-0.439,-0.439,-0.439,     &
                    -0.439,-0.439,-0.439,-0.440,-0.440,-0.440,-0.440,     &
                    -0.440,-0.440,-0.440,-0.441,-0.441,-0.441,-0.441,     &
                    -0.441,-0.441,-0.442,-0.442,-0.442,-0.442,-0.442,     &
                    -0.442,-0.442,-0.442,-0.443,-0.443,-0.443,-0.443,     &
                    -0.443,-0.443,-0.443,-0.444,-0.444,-0.444,-0.444,     &
                    -0.444,-0.444,-0.444,-0.444,-0.445,-0.445,-0.445,     &
                    -0.445,-0.445,-0.445,-0.445,-0.445,-0.446,-0.446,     &
                    -0.446,-0.446,-0.446,-0.446,-0.446,-0.446,-0.446,     &
                    -0.447,-0.447,-0.447,-0.447,-0.447,-0.447,-0.447,     &
                    -0.447,-0.447,-0.447,-0.448,-0.448,-0.448,-0.448,     &
                    -0.448,-0.448,-0.448,-0.448,-0.448,-0.449,-0.449,     &
                    -0.449,-0.449,-0.449,-0.449,-0.449,-0.449,-0.449,     &
                    -0.449,-0.449,-0.450,-0.450,-0.450,-0.450,-0.450,     &
                    -0.450,-0.451,-0.452,-0.452,-0.453,-0.454,-0.454,     &
                    -0.455,-0.455,-0.456,-0.456,-0.457,-0.457,-0.457,     &
                    -0.458,-0.458,-0.458,-0.458,-0.458,-0.458,-0.458,     &
                    -0.459,-0.459,-0.459,-0.459,-0.459,-0.458,-0.458,     &
                    -0.458,-0.458,-0.458,-0.458,-0.458,-0.457,-0.457,     &
                    -0.457,-0.457,-0.456,-0.456,-0.456,-0.455,-0.455,     &
                    -0.455,-0.454,-0.454,-0.453,-0.453,-0.453,-0.452,     &
                    -0.452,-0.451,-0.451,-0.450,-0.450,-0.449,-0.449,     &
                    -0.448,-0.448,-0.447,-0.446,-0.446,-0.445,-0.445,     &
                    -0.444,-0.443,-0.443,-0.442,-0.442,-0.441,-0.440,     &
                    -0.440,-0.439,-0.438,-0.437,-0.437,-0.436,-0.435,     &
                    -0.435,-0.434,-0.433,-0.432,-0.432,-0.431,-0.430,     &
                    -0.429,-0.429,-0.428,-0.427,-0.426,-0.425,-0.425,     &
                    -0.424,-0.423,-0.422,-0.421,-0.420,-0.420,-0.419,     &
                    -0.418,-0.417,-0.416,-0.415,-0.414,-0.414,-0.413,     &
                    -0.412,-0.411,-0.410,-0.409,-0.408,-0.407,-0.406,     &
                    -0.405,-0.404,-0.404,-0.403,-0.402,-0.401,-0.400,     &
                    -0.399,-0.398,-0.397,-0.396,-0.395,-0.394,-0.393,     &
                    -0.392,-0.391,-0.390,-0.389,-0.388,-0.387,-0.386,     &
                    -0.385,-0.384,-0.383,-0.382,-0.381,-0.380,-0.379,     &
                    -0.378,-0.377,-0.376,-0.375,-0.374,-0.373,-0.372,     &
                    -0.371,-0.370,-0.369,-0.368,-0.367,-0.366,-0.365,     &
                    -0.364,-0.363,-0.362,-0.361,-0.360,-0.359,-0.358,     &
                    -0.356/)
!
! *** (nh4)2so4
!
    bnc04m(1:561) = (/-0.091,-0.197,-0.249,-0.286,-0.315,-0.339,-0.359,   &
                    -0.377,-0.392,-0.406,-0.419,-0.431,-0.442,-0.452,     &
                    -0.461,-0.470,-0.478,-0.486,-0.493,-0.500,-0.507,     &
                    -0.513,-0.519,-0.525,-0.530,-0.535,-0.540,-0.545,     &
                    -0.550,-0.554,-0.559,-0.563,-0.567,-0.571,-0.575,     &
                    -0.579,-0.582,-0.586,-0.589,-0.592,-0.596,-0.599,     &
                    -0.602,-0.605,-0.608,-0.611,-0.613,-0.616,-0.619,     &
                    -0.621,-0.624,-0.626,-0.628,-0.631,-0.633,-0.635,     &
                    -0.637,-0.640,-0.642,-0.644,-0.646,-0.648,-0.650,     &
                    -0.652,-0.653,-0.655,-0.657,-0.659,-0.661,-0.662,     &
                    -0.664,-0.666,-0.667,-0.669,-0.670,-0.672,-0.673,     &
                    -0.675,-0.676,-0.678,-0.679,-0.681,-0.682,-0.683,     &
                    -0.685,-0.686,-0.687,-0.689,-0.690,-0.691,-0.693,     &
                    -0.694,-0.695,-0.696,-0.697,-0.699,-0.700,-0.701,     &
                    -0.702,-0.703,-0.704,-0.705,-0.707,-0.708,-0.709,     &
                    -0.710,-0.711,-0.712,-0.713,-0.714,-0.715,-0.716,     &
                    -0.717,-0.718,-0.719,-0.720,-0.721,-0.722,-0.723,     &
                    -0.724,-0.724,-0.725,-0.726,-0.727,-0.728,-0.729,     &
                    -0.730,-0.731,-0.731,-0.732,-0.733,-0.734,-0.735,     &
                    -0.735,-0.736,-0.737,-0.738,-0.738,-0.739,-0.740,     &
                    -0.741,-0.741,-0.742,-0.743,-0.744,-0.744,-0.745,     &
                    -0.746,-0.746,-0.747,-0.748,-0.748,-0.749,-0.750,     &
                    -0.750,-0.751,-0.752,-0.752,-0.753,-0.754,-0.754,     &
                    -0.755,-0.755,-0.756,-0.757,-0.757,-0.758,-0.758,     &
                    -0.759,-0.759,-0.760,-0.761,-0.761,-0.762,-0.762,     &
                    -0.763,-0.763,-0.764,-0.764,-0.765,-0.765,-0.766,     &
                    -0.766,-0.767,-0.767,-0.768,-0.768,-0.769,-0.769,     &
                    -0.770,-0.770,-0.771,-0.771,-0.772,-0.772,-0.772,     &
                    -0.773,-0.773,-0.774,-0.774,-0.775,-0.775,-0.776,     &
                    -0.776,-0.776,-0.777,-0.777,-0.778,-0.778,-0.778,     &
                    -0.779,-0.779,-0.780,-0.780,-0.780,-0.781,-0.781,     &
                    -0.781,-0.782,-0.782,-0.783,-0.783,-0.783,-0.784,     &
                    -0.784,-0.784,-0.785,-0.785,-0.785,-0.786,-0.786,     &
                    -0.786,-0.787,-0.787,-0.787,-0.788,-0.788,-0.788,     &
                    -0.789,-0.789,-0.789,-0.790,-0.790,-0.790,-0.790,     &
                    -0.791,-0.791,-0.791,-0.792,-0.792,-0.792,-0.793,     &
                    -0.793,-0.793,-0.793,-0.794,-0.794,-0.794,-0.794,     &
                    -0.795,-0.795,-0.795,-0.795,-0.796,-0.796,-0.796,     &
                    -0.796,-0.797,-0.797,-0.797,-0.797,-0.798,-0.798,     &
                    -0.798,-0.798,-0.799,-0.799,-0.799,-0.799,-0.800,     &
                    -0.800,-0.800,-0.800,-0.800,-0.801,-0.801,-0.801,     &
                    -0.801,-0.801,-0.802,-0.802,-0.802,-0.802,-0.802,     &
                    -0.803,-0.803,-0.803,-0.803,-0.803,-0.804,-0.804,     &
                    -0.804,-0.804,-0.804,-0.804,-0.805,-0.805,-0.805,     &
                    -0.805,-0.805,-0.806,-0.806,-0.806,-0.806,-0.806,     &
                    -0.806,-0.806,-0.807,-0.807,-0.807,-0.807,-0.807,     &
                    -0.807,-0.808,-0.808,-0.808,-0.808,-0.808,-0.808,     &
                    -0.808,-0.809,-0.809,-0.809,-0.809,-0.809,-0.809,     &
                    -0.809,-0.810,-0.810,-0.810,-0.810,-0.810,-0.810,     &
                    -0.810,-0.810,-0.810,-0.811,-0.811,-0.811,-0.811,     &
                    -0.811,-0.811,-0.811,-0.811,-0.812,-0.812,-0.812,     &
                    -0.812,-0.812,-0.812,-0.812,-0.812,-0.812,-0.812,     &
                    -0.813,-0.813,-0.813,-0.813,-0.813,-0.813,-0.813,     &
                    -0.813,-0.813,-0.813,-0.813,-0.814,-0.814,-0.814,     &
                    -0.814,-0.814,-0.814,-0.814,-0.814,-0.814,-0.814,     &
                    -0.814,-0.814,-0.814,-0.815,-0.815,-0.815,-0.815,     &
                    -0.815,-0.815,-0.815,-0.815,-0.815,-0.815,-0.815,     &
                    -0.815,-0.816,-0.816,-0.817,-0.817,-0.817,-0.817,     &
                    -0.817,-0.817,-0.817,-0.817,-0.817,-0.817,-0.816,     &
                    -0.816,-0.816,-0.815,-0.815,-0.814,-0.813,-0.813,     &
                    -0.812,-0.811,-0.811,-0.810,-0.809,-0.808,-0.807,     &
                    -0.806,-0.805,-0.804,-0.803,-0.802,-0.801,-0.800,     &
                    -0.799,-0.797,-0.796,-0.795,-0.794,-0.792,-0.791,     &
                    -0.790,-0.788,-0.787,-0.786,-0.784,-0.783,-0.781,     &
                    -0.780,-0.778,-0.777,-0.775,-0.774,-0.772,-0.771,     &
                    -0.769,-0.767,-0.766,-0.764,-0.762,-0.761,-0.759,     &
                    -0.757,-0.756,-0.754,-0.752,-0.750,-0.749,-0.747,     &
                    -0.745,-0.743,-0.741,-0.740,-0.738,-0.736,-0.734,     &
                    -0.732,-0.730,-0.728,-0.727,-0.725,-0.723,-0.721,     &
                    -0.719,-0.717,-0.715,-0.713,-0.711,-0.709,-0.707,     &
                    -0.705,-0.703,-0.701,-0.699,-0.697,-0.695,-0.693,     &
                    -0.691,-0.689,-0.687,-0.685,-0.683,-0.681,-0.678,     &
                    -0.676,-0.674,-0.672,-0.670,-0.668,-0.666,-0.664,     &
                    -0.661,-0.659,-0.657,-0.655,-0.653,-0.651,-0.648,     &
                    -0.646,-0.644,-0.642,-0.640,-0.638,-0.635,-0.633,     &
                    -0.631,-0.629,-0.626,-0.624,-0.622,-0.620,-0.618,     &
                    -0.615,-0.613,-0.611,-0.609,-0.606,-0.604,-0.602,     &
                    -0.599,-0.597,-0.595,-0.593,-0.590,-0.588,-0.586,     &
                    -0.583,-0.581,-0.579,-0.577,-0.574,-0.572,-0.570,     &
                    -0.567,-0.565,-0.563,-0.560,-0.558,-0.556,-0.553,     &
                    -0.551/)
!
! *** nh4no3
!
    bnc05m(1:561) = (/-0.046,-0.101,-0.129,-0.149,-0.166,-0.179,-0.191,   &
                    -0.202,-0.211,-0.220,-0.228,-0.235,-0.242,-0.249,     &
                    -0.255,-0.261,-0.266,-0.272,-0.277,-0.282,-0.286,     &
                    -0.291,-0.295,-0.299,-0.303,-0.307,-0.311,-0.315,     &
                    -0.319,-0.322,-0.326,-0.329,-0.332,-0.335,-0.339,     &
                    -0.342,-0.345,-0.348,-0.350,-0.353,-0.356,-0.359,     &
                    -0.361,-0.364,-0.366,-0.369,-0.371,-0.374,-0.376,     &
                    -0.378,-0.380,-0.383,-0.385,-0.387,-0.389,-0.391,     &
                    -0.393,-0.395,-0.397,-0.399,-0.401,-0.403,-0.405,     &
                    -0.406,-0.408,-0.410,-0.412,-0.414,-0.415,-0.417,     &
                    -0.419,-0.420,-0.422,-0.424,-0.425,-0.427,-0.428,     &
                    -0.430,-0.432,-0.433,-0.435,-0.436,-0.438,-0.439,     &
                    -0.441,-0.442,-0.444,-0.445,-0.447,-0.448,-0.449,     &
                    -0.451,-0.452,-0.454,-0.455,-0.456,-0.458,-0.459,     &
                    -0.461,-0.462,-0.463,-0.465,-0.466,-0.467,-0.469,     &
                    -0.470,-0.471,-0.473,-0.474,-0.475,-0.477,-0.478,     &
                    -0.479,-0.480,-0.482,-0.483,-0.484,-0.485,-0.487,     &
                    -0.488,-0.489,-0.490,-0.492,-0.493,-0.494,-0.495,     &
                    -0.496,-0.498,-0.499,-0.500,-0.501,-0.502,-0.503,     &
                    -0.504,-0.506,-0.507,-0.508,-0.509,-0.510,-0.511,     &
                    -0.512,-0.513,-0.514,-0.515,-0.516,-0.517,-0.519,     &
                    -0.520,-0.521,-0.522,-0.523,-0.524,-0.525,-0.526,     &
                    -0.527,-0.528,-0.529,-0.530,-0.531,-0.532,-0.533,     &
                    -0.534,-0.535,-0.535,-0.536,-0.537,-0.538,-0.539,     &
                    -0.540,-0.541,-0.542,-0.543,-0.544,-0.545,-0.546,     &
                    -0.547,-0.547,-0.548,-0.549,-0.550,-0.551,-0.552,     &
                    -0.553,-0.553,-0.554,-0.555,-0.556,-0.557,-0.558,     &
                    -0.559,-0.559,-0.560,-0.561,-0.562,-0.563,-0.563,     &
                    -0.564,-0.565,-0.566,-0.567,-0.567,-0.568,-0.569,     &
                    -0.570,-0.570,-0.571,-0.572,-0.573,-0.574,-0.574,     &
                    -0.575,-0.576,-0.576,-0.577,-0.578,-0.579,-0.579,     &
                    -0.580,-0.581,-0.582,-0.582,-0.583,-0.584,-0.584,     &
                    -0.585,-0.586,-0.586,-0.587,-0.588,-0.589,-0.589,     &
                    -0.590,-0.591,-0.591,-0.592,-0.593,-0.593,-0.594,     &
                    -0.595,-0.595,-0.596,-0.596,-0.597,-0.598,-0.598,     &
                    -0.599,-0.600,-0.600,-0.601,-0.602,-0.602,-0.603,     &
                    -0.603,-0.604,-0.605,-0.605,-0.606,-0.606,-0.607,     &
                    -0.608,-0.608,-0.609,-0.609,-0.610,-0.611,-0.611,     &
                    -0.612,-0.612,-0.613,-0.613,-0.614,-0.615,-0.615,     &
                    -0.616,-0.616,-0.617,-0.617,-0.618,-0.618,-0.619,     &
                    -0.620,-0.620,-0.621,-0.621,-0.622,-0.622,-0.623,     &
                    -0.623,-0.624,-0.624,-0.625,-0.625,-0.626,-0.626,     &
                    -0.627,-0.627,-0.628,-0.628,-0.629,-0.629,-0.630,     &
                    -0.630,-0.631,-0.631,-0.632,-0.632,-0.633,-0.633,     &
                    -0.634,-0.634,-0.635,-0.635,-0.636,-0.636,-0.637,     &
                    -0.637,-0.638,-0.638,-0.639,-0.639,-0.639,-0.640,     &
                    -0.640,-0.641,-0.641,-0.642,-0.642,-0.643,-0.643,     &
                    -0.643,-0.644,-0.644,-0.645,-0.645,-0.646,-0.646,     &
                    -0.647,-0.647,-0.647,-0.648,-0.648,-0.649,-0.649,     &
                    -0.649,-0.650,-0.650,-0.651,-0.651,-0.652,-0.652,     &
                    -0.652,-0.653,-0.653,-0.654,-0.654,-0.654,-0.655,     &
                    -0.655,-0.656,-0.656,-0.656,-0.657,-0.657,-0.658,     &
                    -0.658,-0.658,-0.659,-0.659,-0.659,-0.660,-0.660,     &
                    -0.661,-0.661,-0.661,-0.662,-0.662,-0.662,-0.663,     &
                    -0.663,-0.663,-0.664,-0.664,-0.665,-0.665,-0.665,     &
                    -0.666,-0.666,-0.666,-0.667,-0.667,-0.667,-0.668,     &
                    -0.668,-0.668,-0.669,-0.669,-0.669,-0.670,-0.670,     &
                    -0.670,-0.674,-0.677,-0.680,-0.683,-0.686,-0.688,     &
                    -0.691,-0.693,-0.696,-0.698,-0.700,-0.702,-0.705,     &
                    -0.706,-0.708,-0.710,-0.712,-0.714,-0.715,-0.717,     &
                    -0.718,-0.720,-0.721,-0.722,-0.723,-0.725,-0.726,     &
                    -0.727,-0.728,-0.729,-0.730,-0.731,-0.732,-0.732,     &
                    -0.733,-0.734,-0.735,-0.735,-0.736,-0.736,-0.737,     &
                    -0.737,-0.738,-0.738,-0.739,-0.739,-0.739,-0.740,     &
                    -0.740,-0.740,-0.741,-0.741,-0.741,-0.741,-0.741,     &
                    -0.741,-0.741,-0.741,-0.741,-0.741,-0.741,-0.741,     &
                    -0.741,-0.741,-0.741,-0.741,-0.741,-0.741,-0.741,     &
                    -0.740,-0.740,-0.740,-0.740,-0.739,-0.739,-0.739,     &
                    -0.739,-0.738,-0.738,-0.738,-0.737,-0.737,-0.736,     &
                    -0.736,-0.736,-0.735,-0.735,-0.734,-0.734,-0.733,     &
                    -0.733,-0.732,-0.732,-0.731,-0.731,-0.730,-0.729,     &
                    -0.729,-0.728,-0.728,-0.727,-0.726,-0.726,-0.725,     &
                    -0.725,-0.724,-0.723,-0.723,-0.722,-0.721,-0.721,     &
                    -0.720,-0.719,-0.718,-0.718,-0.717,-0.716,-0.715,     &
                    -0.715,-0.714,-0.713,-0.712,-0.711,-0.711,-0.710,     &
                    -0.709,-0.708,-0.707,-0.707,-0.706,-0.705,-0.704,     &
                    -0.703,-0.702,-0.701,-0.701,-0.700,-0.699,-0.698,     &
                    -0.697,-0.696,-0.695,-0.694,-0.693,-0.692,-0.692,     &
                    -0.691,-0.690,-0.689,-0.688,-0.687,-0.686,-0.685,     &
                    -0.684,-0.683,-0.682,-0.681,-0.680,-0.679,-0.678,     &
                    -0.677/)
!
! *** nh4cl
!
    bnc06m(1:561) = (/-0.045,-0.096,-0.120,-0.136,-0.149,-0.159,-0.167,   &
                    -0.174,-0.181,-0.186,-0.191,-0.195,-0.199,-0.202,     &
                    -0.205,-0.208,-0.210,-0.213,-0.215,-0.217,-0.219,     &
                    -0.220,-0.222,-0.223,-0.225,-0.226,-0.227,-0.228,     &
                    -0.229,-0.230,-0.231,-0.232,-0.232,-0.233,-0.234,     &
                    -0.234,-0.235,-0.235,-0.236,-0.236,-0.237,-0.237,     &
                    -0.237,-0.238,-0.238,-0.238,-0.238,-0.238,-0.239,     &
                    -0.239,-0.239,-0.239,-0.239,-0.239,-0.239,-0.239,     &
                    -0.239,-0.239,-0.239,-0.239,-0.239,-0.239,-0.239,     &
                    -0.239,-0.239,-0.239,-0.239,-0.239,-0.239,-0.239,     &
                    -0.238,-0.238,-0.238,-0.238,-0.238,-0.238,-0.237,     &
                    -0.237,-0.237,-0.237,-0.237,-0.236,-0.236,-0.236,     &
                    -0.235,-0.235,-0.235,-0.235,-0.234,-0.234,-0.234,     &
                    -0.233,-0.233,-0.233,-0.232,-0.232,-0.231,-0.231,     &
                    -0.231,-0.230,-0.230,-0.229,-0.229,-0.229,-0.228,     &
                    -0.228,-0.227,-0.227,-0.226,-0.226,-0.225,-0.225,     &
                    -0.224,-0.224,-0.224,-0.223,-0.223,-0.222,-0.222,     &
                    -0.221,-0.221,-0.220,-0.219,-0.219,-0.218,-0.218,     &
                    -0.217,-0.217,-0.216,-0.216,-0.215,-0.215,-0.214,     &
                    -0.214,-0.213,-0.213,-0.212,-0.211,-0.211,-0.210,     &
                    -0.210,-0.209,-0.209,-0.208,-0.208,-0.207,-0.206,     &
                    -0.206,-0.205,-0.205,-0.204,-0.204,-0.203,-0.203,     &
                    -0.202,-0.201,-0.201,-0.200,-0.200,-0.199,-0.199,     &
                    -0.198,-0.197,-0.197,-0.196,-0.196,-0.195,-0.195,     &
                    -0.194,-0.193,-0.193,-0.192,-0.192,-0.191,-0.191,     &
                    -0.190,-0.189,-0.189,-0.188,-0.188,-0.187,-0.186,     &
                    -0.186,-0.185,-0.185,-0.184,-0.184,-0.183,-0.182,     &
                    -0.182,-0.181,-0.181,-0.180,-0.179,-0.179,-0.178,     &
                    -0.178,-0.177,-0.177,-0.176,-0.175,-0.175,-0.174,     &
                    -0.174,-0.173,-0.172,-0.172,-0.171,-0.171,-0.170,     &
                    -0.170,-0.169,-0.168,-0.168,-0.167,-0.167,-0.166,     &
                    -0.165,-0.165,-0.164,-0.164,-0.163,-0.163,-0.162,     &
                    -0.161,-0.161,-0.160,-0.160,-0.159,-0.158,-0.158,     &
                    -0.157,-0.157,-0.156,-0.156,-0.155,-0.154,-0.154,     &
                    -0.153,-0.153,-0.152,-0.151,-0.151,-0.150,-0.150,     &
                    -0.149,-0.149,-0.148,-0.147,-0.147,-0.146,-0.146,     &
                    -0.145,-0.145,-0.144,-0.143,-0.143,-0.142,-0.142,     &
                    -0.141,-0.140,-0.140,-0.139,-0.139,-0.138,-0.138,     &
                    -0.137,-0.136,-0.136,-0.135,-0.135,-0.134,-0.134,     &
                    -0.133,-0.132,-0.132,-0.131,-0.131,-0.130,-0.130,     &
                    -0.129,-0.128,-0.128,-0.127,-0.127,-0.126,-0.126,     &
                    -0.125,-0.125,-0.124,-0.123,-0.123,-0.122,-0.122,     &
                    -0.121,-0.121,-0.120,-0.119,-0.119,-0.118,-0.118,     &
                    -0.117,-0.117,-0.116,-0.115,-0.115,-0.114,-0.114,     &
                    -0.113,-0.113,-0.112,-0.112,-0.111,-0.110,-0.110,     &
                    -0.109,-0.109,-0.108,-0.108,-0.107,-0.107,-0.106,     &
                    -0.105,-0.105,-0.104,-0.104,-0.103,-0.103,-0.102,     &
                    -0.102,-0.101,-0.100,-0.100,-0.099,-0.099,-0.098,     &
                    -0.098,-0.097,-0.097,-0.096,-0.096,-0.095,-0.094,     &
                    -0.094,-0.093,-0.093,-0.092,-0.092,-0.091,-0.091,     &
                    -0.090,-0.090,-0.089,-0.088,-0.088,-0.087,-0.087,     &
                    -0.086,-0.086,-0.085,-0.085,-0.084,-0.084,-0.083,     &
                    -0.083,-0.082,-0.081,-0.081,-0.080,-0.080,-0.079,     &
                    -0.079,-0.078,-0.078,-0.077,-0.077,-0.076,-0.076,     &
                    -0.075,-0.074,-0.074,-0.073,-0.073,-0.072,-0.072,     &
                    -0.071,-0.071,-0.070,-0.070,-0.069,-0.069,-0.068,     &
                    -0.068,-0.067,-0.067,-0.066,-0.065,-0.065,-0.064,     &
                    -0.064,-0.058,-0.053,-0.048,-0.043,-0.038,-0.033,     &
                    -0.028,-0.023,-0.018,-0.013,-0.008,-0.003,0.002,0.006,&
                    0.011,0.016,0.021,0.025,0.030,0.035,0.039,0.044,0.048,&
                    0.053,0.057,0.062,0.066,0.070,0.075,0.079,0.083,0.088,&
                    0.092,0.096,0.100,0.105,0.109,0.113,0.117,0.121,0.125,&
                    0.129,0.133,0.137,0.141,0.145,0.149,0.153,0.157,0.161,&
                    0.165,0.169,0.173,0.176,0.180,0.184,0.188,0.192,0.195,&
                    0.199,0.203,0.207,0.210,0.214,0.218,0.221,0.225,0.228,&
                    0.232,0.236,0.239,0.243,0.246,0.250,0.253,0.257,0.260,&
                    0.264,0.267,0.271,0.274,0.278,0.281,0.285,0.288,0.291,&
                    0.295,0.298,0.301,0.305,0.308,0.311,0.315,0.318,0.321,&
                    0.325,0.328,0.331,0.334,0.338,0.341,0.344,0.347,0.350,&
                    0.354,0.357,0.360,0.363,0.366,0.369,0.373,0.376,0.379,&
                    0.382,0.385,0.388,0.391,0.394,0.397,0.400,0.403,0.406,&
                    0.409,0.412,0.416,0.419,0.422,0.425,0.428,0.430,0.433,&
                    0.436,0.439,0.442,0.445,0.448,0.451,0.454,0.457,0.460,&
                    0.463,0.466,0.469,0.471,0.474,0.477,0.480,0.483,0.486,&
                    0.489,0.491,0.494,0.497,0.500,0.503,0.506,0.508,0.511,&
                    0.514,0.517,0.520/)
!
! *** (2h ,so4)
!
    bnc07m(1:561) = (/-0.091,-0.196,-0.248,-0.284,-0.312,-0.336,-0.355,   &
                    -0.372,-0.388,-0.401,-0.414,-0.425,-0.435,-0.445,     &
                    -0.453,-0.462,-0.469,-0.477,-0.484,-0.490,-0.496,     &
                    -0.502,-0.508,-0.513,-0.518,-0.523,-0.528,-0.532,     &
                    -0.536,-0.540,-0.544,-0.548,-0.552,-0.556,-0.559,     &
                    -0.562,-0.566,-0.569,-0.572,-0.575,-0.578,-0.580,     &
                    -0.583,-0.586,-0.588,-0.591,-0.593,-0.595,-0.598,     &
                    -0.600,-0.602,-0.604,-0.606,-0.608,-0.610,-0.612,     &
                    -0.614,-0.616,-0.618,-0.620,-0.621,-0.623,-0.625,     &
                    -0.626,-0.628,-0.629,-0.631,-0.632,-0.634,-0.635,     &
                    -0.637,-0.638,-0.639,-0.641,-0.642,-0.643,-0.644,     &
                    -0.646,-0.647,-0.648,-0.649,-0.650,-0.652,-0.653,     &
                    -0.654,-0.655,-0.656,-0.657,-0.658,-0.659,-0.660,     &
                    -0.661,-0.662,-0.663,-0.664,-0.665,-0.666,-0.666,     &
                    -0.667,-0.668,-0.669,-0.670,-0.671,-0.671,-0.672,     &
                    -0.673,-0.674,-0.675,-0.675,-0.676,-0.677,-0.678,     &
                    -0.678,-0.679,-0.680,-0.680,-0.681,-0.682,-0.682,     &
                    -0.683,-0.684,-0.684,-0.685,-0.686,-0.686,-0.687,     &
                    -0.687,-0.688,-0.689,-0.689,-0.690,-0.690,-0.691,     &
                    -0.691,-0.692,-0.692,-0.693,-0.693,-0.694,-0.694,     &
                    -0.695,-0.695,-0.696,-0.696,-0.697,-0.697,-0.698,     &
                    -0.698,-0.699,-0.699,-0.700,-0.700,-0.700,-0.701,     &
                    -0.701,-0.702,-0.702,-0.702,-0.703,-0.703,-0.704,     &
                    -0.704,-0.704,-0.705,-0.705,-0.705,-0.706,-0.706,     &
                    -0.706,-0.707,-0.707,-0.707,-0.708,-0.708,-0.708,     &
                    -0.709,-0.709,-0.709,-0.710,-0.710,-0.710,-0.711,     &
                    -0.711,-0.711,-0.711,-0.712,-0.712,-0.712,-0.712,     &
                    -0.713,-0.713,-0.713,-0.713,-0.714,-0.714,-0.714,     &
                    -0.714,-0.715,-0.715,-0.715,-0.715,-0.716,-0.716,     &
                    -0.716,-0.716,-0.716,-0.717,-0.717,-0.717,-0.717,     &
                    -0.717,-0.718,-0.718,-0.718,-0.718,-0.718,-0.719,     &
                    -0.719,-0.719,-0.719,-0.719,-0.719,-0.720,-0.720,     &
                    -0.720,-0.720,-0.720,-0.720,-0.720,-0.721,-0.721,     &
                    -0.721,-0.721,-0.721,-0.721,-0.721,-0.722,-0.722,     &
                    -0.722,-0.722,-0.722,-0.722,-0.722,-0.722,-0.723,     &
                    -0.723,-0.723,-0.723,-0.723,-0.723,-0.723,-0.723,     &
                    -0.723,-0.723,-0.724,-0.724,-0.724,-0.724,-0.724,     &
                    -0.724,-0.724,-0.724,-0.724,-0.724,-0.724,-0.724,     &
                    -0.725,-0.725,-0.725,-0.725,-0.725,-0.725,-0.725,     &
                    -0.725,-0.725,-0.725,-0.725,-0.725,-0.725,-0.725,     &
                    -0.725,-0.725,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.726,-0.726,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.726,-0.726,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.726,-0.726,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.726,-0.726,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.726,-0.726,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.726,-0.726,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.726,-0.726,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.726,-0.726,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.726,-0.726,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.726,-0.726,-0.726,-0.726,-0.726,-0.726,-0.726,     &
                    -0.725,-0.725,-0.725,-0.725,-0.725,-0.725,-0.725,     &
                    -0.725,-0.725,-0.725,-0.725,-0.725,-0.725,-0.725,     &
                    -0.725,-0.725,-0.725,-0.725,-0.725,-0.725,-0.725,     &
                    -0.724,-0.724,-0.724,-0.724,-0.724,-0.724,-0.724,     &
                    -0.724,-0.724,-0.724,-0.724,-0.724,-0.724,-0.724,     &
                    -0.724,-0.724,-0.723,-0.723,-0.723,-0.723,-0.723,     &
                    -0.723,-0.722,-0.721,-0.721,-0.720,-0.719,-0.717,     &
                    -0.716,-0.715,-0.714,-0.713,-0.711,-0.710,-0.709,     &
                    -0.707,-0.706,-0.704,-0.703,-0.701,-0.700,-0.698,     &
                    -0.696,-0.695,-0.693,-0.691,-0.689,-0.688,-0.686,     &
                    -0.684,-0.682,-0.680,-0.678,-0.676,-0.675,-0.673,     &
                    -0.671,-0.669,-0.667,-0.665,-0.663,-0.661,-0.658,     &
                    -0.656,-0.654,-0.652,-0.650,-0.648,-0.646,-0.644,     &
                    -0.642,-0.639,-0.637,-0.635,-0.633,-0.631,-0.628,     &
                    -0.626,-0.624,-0.622,-0.619,-0.617,-0.615,-0.612,     &
                    -0.610,-0.608,-0.605,-0.603,-0.601,-0.598,-0.596,     &
                    -0.594,-0.591,-0.589,-0.587,-0.584,-0.582,-0.579,     &
                    -0.577,-0.575,-0.572,-0.570,-0.567,-0.565,-0.563,     &
                    -0.560,-0.558,-0.555,-0.553,-0.550,-0.548,-0.545,     &
                    -0.543,-0.540,-0.538,-0.535,-0.533,-0.530,-0.528,     &
                    -0.525,-0.523,-0.520,-0.518,-0.515,-0.513,-0.510,     &
                    -0.508,-0.505,-0.503,-0.500,-0.498,-0.495,-0.493,     &
                    -0.490,-0.487,-0.485,-0.482,-0.480,-0.477,-0.475,     &
                    -0.472,-0.470,-0.467,-0.464,-0.462,-0.459,-0.457,     &
                    -0.454,-0.451,-0.449,-0.446,-0.444,-0.441,-0.438,     &
                    -0.436,-0.433,-0.431,-0.428,-0.425,-0.423,-0.420,     &
                    -0.418,-0.415,-0.412,-0.410,-0.407,-0.404,-0.402,     &
                    -0.399,-0.397,-0.394,-0.391,-0.389,-0.386,-0.383,     &
                    -0.381,-0.378,-0.375,-0.373,-0.370,-0.367,-0.365,     &
                    -0.362/)
!
! *** (h ,hso4)
!
    bnc08m(1:561) = (/-0.043,-0.086,-0.105,-0.116,-0.123,-0.128,-0.131,   &
                    -0.134,-0.135,-0.136,-0.136,-0.135,-0.134,-0.133,     &
                    -0.131,-0.129,-0.127,-0.124,-0.122,-0.118,-0.115,     &
                    -0.112,-0.108,-0.104,-0.100,-0.096,-0.092,-0.088,     &
                    -0.083,-0.079,-0.074,-0.069,-0.064,-0.059,-0.054,     &
                    -0.048,-0.043,-0.037,-0.032,-0.026,-0.021,-0.015,     &
                    -0.009,-0.003,0.003,0.009,0.015,0.021,0.027,0.034,    &
                    0.040,0.046,0.053,0.059,0.066,0.072,0.079,0.086,0.092,&
                    0.099,0.106,0.112,0.119,0.126,0.133,0.140,0.147,0.154,&
                    0.161,0.168,0.175,0.182,0.189,0.196,0.204,0.211,0.218,&
                    0.226,0.233,0.240,0.248,0.255,0.263,0.270,0.278,0.286,&
                    0.293,0.301,0.309,0.317,0.324,0.332,0.340,0.348,0.356,&
                    0.364,0.372,0.380,0.389,0.397,0.405,0.413,0.421,0.430,&
                    0.438,0.446,0.455,0.463,0.471,0.480,0.488,0.497,0.505,&
                    0.514,0.522,0.530,0.539,0.547,0.556,0.564,0.573,0.581,&
                    0.590,0.598,0.607,0.615,0.624,0.632,0.641,0.649,0.658,&
                    0.666,0.675,0.683,0.692,0.700,0.708,0.717,0.725,0.734,&
                    0.742,0.750,0.759,0.767,0.775,0.784,0.792,0.800,0.808,&
                    0.817,0.825,0.833,0.841,0.849,0.858,0.866,0.874,0.882,&
                    0.890,0.898,0.906,0.915,0.923,0.931,0.939,0.947,0.955,&
                    0.963,0.971,0.979,0.986,0.994,1.002,1.010,1.018,1.026,&
                    1.034,1.042,1.049,1.057,1.065,1.073,1.080,1.088,1.096,&
                    1.104,1.111,1.119,1.127,1.134,1.142,1.150,1.157,1.165,&
                    1.172,1.180,1.187,1.195,1.202,1.210,1.217,1.225,1.232,&
                    1.240,1.247,1.254,1.262,1.269,1.276,1.284,1.291,1.298,&
                    1.306,1.313,1.320,1.327,1.335,1.342,1.349,1.356,1.363,&
                    1.371,1.378,1.385,1.392,1.399,1.406,1.413,1.420,1.427,&
                    1.434,1.441,1.448,1.455,1.462,1.469,1.476,1.483,1.490,&
                    1.497,1.504,1.510,1.517,1.524,1.531,1.538,1.545,1.551,&
                    1.558,1.565,1.572,1.578,1.585,1.592,1.598,1.605,1.612,&
                    1.618,1.625,1.632,1.638,1.645,1.651,1.658,1.664,1.671,&
                    1.678,1.684,1.691,1.697,1.704,1.710,1.716,1.723,1.729,&
                    1.736,1.742,1.749,1.755,1.761,1.768,1.774,1.780,1.787,&
                    1.793,1.799,1.805,1.812,1.818,1.824,1.830,1.837,1.843,&
                    1.849,1.855,1.861,1.868,1.874,1.880,1.886,1.892,1.898,&
                    1.904,1.910,1.916,1.923,1.929,1.935,1.941,1.947,1.953,&
                    1.959,1.965,1.971,1.977,1.983,1.988,1.994,2.000,2.006,&
                    2.012,2.018,2.024,2.030,2.036,2.041,2.047,2.053,2.059,&
                    2.065,2.070,2.076,2.082,2.088,2.094,2.099,2.105,2.111,&
                    2.116,2.122,2.128,2.134,2.139,2.145,2.150,2.156,2.162,&
                    2.167,2.173,2.179,2.184,2.190,2.195,2.201,2.206,2.212,&
                    2.218,2.223,2.229,2.234,2.240,2.245,2.251,2.256,2.261,&
                    2.267,2.272,2.278,2.283,2.289,2.294,2.299,2.305,2.310,&
                    2.316,2.321,2.326,2.332,2.337,2.342,2.348,2.353,2.358,&
                    2.364,2.369,2.374,2.379,2.385,2.390,2.395,2.400,2.406,&
                    2.411,2.416,2.421,2.426,2.432,2.437,2.442,2.447,2.502,&
                    2.553,2.602,2.651,2.699,2.746,2.793,2.839,2.884,2.929,&
                    2.973,3.017,3.060,3.102,3.144,3.186,3.227,3.267,3.307,&
                    3.346,3.385,3.424,3.462,3.499,3.536,3.573,3.610,3.646,&
                    3.681,3.716,3.751,3.786,3.820,3.854,3.887,3.920,3.953,&
                    3.985,4.018,4.049,4.081,4.112,4.143,4.174,4.204,4.234,&
                    4.264,4.294,4.323,4.352,4.381,4.410,4.438,4.466,4.494,&
                    4.522,4.549,4.577,4.604,4.630,4.657,4.683,4.710,4.736,&
                    4.761,4.787,4.812,4.838,4.863,4.888,4.912,4.937,4.961,&
                    4.985,5.009,5.033,5.057,5.080,5.104,5.127,5.150,5.173,&
                    5.196,5.218,5.241,5.263,5.285,5.307,5.329,5.351,5.372,&
                    5.394,5.415,5.436,5.457,5.478,5.499,5.520,5.541,5.561,&
                    5.581,5.602,5.622,5.642,5.662,5.682,5.701,5.721,5.740,&
                    5.760,5.779,5.798,5.817,5.836,5.855,5.874,5.892,5.911,&
                    5.929,5.948,5.966,5.984,6.002,6.020,6.038,6.056,6.074,&
                    6.092,6.109,6.127,6.144,6.161,6.179,6.196,6.213,6.230,&
                    6.247,6.264,6.280,6.297,6.314,6.330,6.347,6.363,6.379,&
                    6.396,6.412,6.428,6.444,6.460,6.476,6.492,6.507,6.523,&
                    6.539,6.554,6.570,6.585,6.600,6.616,6.631/)
!
! *** nh4hso4
!
    bnc09m(1:561) = (/-0.045,-0.095,-0.119,-0.135,-0.147,-0.157,-0.166,   &
                    -0.173,-0.179,-0.184,-0.189,-0.193,-0.196,-0.200,     &
                    -0.203,-0.205,-0.208,-0.210,-0.212,-0.213,-0.215,     &
                    -0.216,-0.217,-0.218,-0.219,-0.219,-0.220,-0.220,     &
                    -0.221,-0.221,-0.221,-0.221,-0.221,-0.221,-0.220,     &
                    -0.220,-0.219,-0.219,-0.218,-0.217,-0.217,-0.216,     &
                    -0.215,-0.214,-0.213,-0.212,-0.211,-0.210,-0.208,     &
                    -0.207,-0.206,-0.204,-0.203,-0.201,-0.200,-0.198,     &
                    -0.197,-0.195,-0.193,-0.192,-0.190,-0.188,-0.186,     &
                    -0.185,-0.183,-0.181,-0.179,-0.177,-0.175,-0.173,     &
                    -0.171,-0.169,-0.167,-0.165,-0.162,-0.160,-0.158,     &
                    -0.156,-0.154,-0.151,-0.149,-0.147,-0.144,-0.142,     &
                    -0.140,-0.137,-0.135,-0.132,-0.130,-0.127,-0.125,     &
                    -0.122,-0.120,-0.117,-0.115,-0.112,-0.109,-0.107,     &
                    -0.104,-0.101,-0.099,-0.096,-0.093,-0.091,-0.088,     &
                    -0.085,-0.082,-0.079,-0.077,-0.074,-0.071,-0.068,     &
                    -0.065,-0.063,-0.060,-0.057,-0.054,-0.051,-0.048,     &
                    -0.046,-0.043,-0.040,-0.037,-0.034,-0.031,-0.028,     &
                    -0.025,-0.023,-0.020,-0.017,-0.014,-0.011,-0.008,     &
                    -0.005,-0.003,0.000,0.003,0.006,0.009,0.012,0.014,    &
                    0.017,0.020,0.023,0.026,0.029,0.031,0.034,0.037,0.040,&
                    0.043,0.045,0.048,0.051,0.054,0.057,0.059,0.062,0.065,&
                    0.068,0.070,0.073,0.076,0.079,0.081,0.084,0.087,0.089,&
                    0.092,0.095,0.098,0.100,0.103,0.106,0.108,0.111,0.114,&
                    0.116,0.119,0.122,0.124,0.127,0.130,0.132,0.135,0.137,&
                    0.140,0.143,0.145,0.148,0.150,0.153,0.156,0.158,0.161,&
                    0.163,0.166,0.168,0.171,0.174,0.176,0.179,0.181,0.184,&
                    0.186,0.189,0.191,0.194,0.196,0.199,0.201,0.204,0.206,&
                    0.209,0.211,0.214,0.216,0.219,0.221,0.223,0.226,0.228,&
                    0.231,0.233,0.236,0.238,0.240,0.243,0.245,0.248,0.250,&
                    0.252,0.255,0.257,0.260,0.262,0.264,0.267,0.269,0.271,&
                    0.274,0.276,0.278,0.281,0.283,0.285,0.288,0.290,0.292,&
                    0.295,0.297,0.299,0.302,0.304,0.306,0.308,0.311,0.313,&
                    0.315,0.317,0.320,0.322,0.324,0.326,0.329,0.331,0.333,&
                    0.335,0.338,0.340,0.342,0.344,0.346,0.349,0.351,0.353,&
                    0.355,0.357,0.360,0.362,0.364,0.366,0.368,0.371,0.373,&
                    0.375,0.377,0.379,0.381,0.383,0.386,0.388,0.390,0.392,&
                    0.394,0.396,0.398,0.400,0.403,0.405,0.407,0.409,0.411,&
                    0.413,0.415,0.417,0.419,0.421,0.423,0.426,0.428,0.430,&
                    0.432,0.434,0.436,0.438,0.440,0.442,0.444,0.446,0.448,&
                    0.450,0.452,0.454,0.456,0.458,0.460,0.462,0.464,0.466,&
                    0.468,0.470,0.472,0.474,0.476,0.478,0.480,0.482,0.484,&
                    0.486,0.488,0.490,0.492,0.494,0.496,0.498,0.500,0.502,&
                    0.504,0.506,0.508,0.510,0.511,0.513,0.515,0.517,0.519,&
                    0.521,0.523,0.525,0.527,0.529,0.531,0.533,0.534,0.536,&
                    0.538,0.540,0.542,0.544,0.546,0.548,0.549,0.551,0.553,&
                    0.555,0.557,0.559,0.561,0.563,0.564,0.566,0.568,0.570,&
                    0.572,0.574,0.575,0.577,0.579,0.581,0.583,0.585,0.586,&
                    0.588,0.590,0.592,0.594,0.595,0.597,0.599,0.618,0.636,&
                    0.653,0.670,0.687,0.704,0.721,0.737,0.753,0.769,0.784,&
                    0.800,0.815,0.830,0.845,0.860,0.875,0.889,0.904,0.918,&
                    0.932,0.946,0.959,0.973,0.986,1.000,1.013,1.026,1.039,&
                    1.052,1.065,1.077,1.090,1.102,1.114,1.126,1.138,1.150,&
                    1.162,1.174,1.186,1.197,1.209,1.220,1.231,1.242,1.254,&
                    1.265,1.276,1.286,1.297,1.308,1.318,1.329,1.339,1.350,&
                    1.360,1.370,1.381,1.391,1.401,1.411,1.421,1.430,1.440,&
                    1.450,1.459,1.469,1.479,1.488,1.497,1.507,1.516,1.525,&
                    1.534,1.544,1.553,1.562,1.571,1.579,1.588,1.597,1.606,&
                    1.615,1.623,1.632,1.640,1.649,1.657,1.666,1.674,1.683,&
                    1.691,1.699,1.707,1.715,1.724,1.732,1.740,1.748,1.756,&
                    1.764,1.771,1.779,1.787,1.795,1.803,1.810,1.818,1.826,&
                    1.833,1.841,1.848,1.856,1.863,1.871,1.878,1.885,1.893,&
                    1.900,1.907,1.915,1.922,1.929,1.936,1.943,1.950,1.957,&
                    1.964,1.971,1.978,1.985,1.992,1.999,2.006,2.013,2.019,&
                    2.026,2.033,2.040,2.046,2.053,2.060,2.066,2.073,2.079,&
                    2.086,2.093,2.099,2.105,2.112,2.118,2.125,2.131,2.138,&
                    2.144,2.150,2.156,2.163,2.169,2.175/)
!
! *** (h ,no3)
!
    bnc10m(1:561) = (/-0.044,-0.092,-0.113,-0.127,-0.137,-0.145,-0.151,   &
                    -0.156,-0.160,-0.163,-0.165,-0.167,-0.169,-0.170,     &
                    -0.171,-0.172,-0.172,-0.172,-0.172,-0.172,-0.172,     &
                    -0.172,-0.171,-0.171,-0.170,-0.169,-0.168,-0.167,     &
                    -0.166,-0.165,-0.164,-0.163,-0.162,-0.160,-0.159,     &
                    -0.158,-0.156,-0.155,-0.153,-0.152,-0.151,-0.149,     &
                    -0.147,-0.146,-0.144,-0.143,-0.141,-0.140,-0.138,     &
                    -0.136,-0.135,-0.133,-0.131,-0.130,-0.128,-0.126,     &
                    -0.125,-0.123,-0.121,-0.120,-0.118,-0.116,-0.114,     &
                    -0.113,-0.111,-0.109,-0.107,-0.106,-0.104,-0.102,     &
                    -0.100,-0.099,-0.097,-0.095,-0.093,-0.091,-0.089,     &
                    -0.088,-0.086,-0.084,-0.082,-0.080,-0.078,-0.076,     &
                    -0.074,-0.072,-0.070,-0.068,-0.066,-0.064,-0.062,     &
                    -0.060,-0.058,-0.056,-0.054,-0.052,-0.050,-0.048,     &
                    -0.045,-0.043,-0.041,-0.039,-0.037,-0.035,-0.032,     &
                    -0.030,-0.028,-0.026,-0.024,-0.021,-0.019,-0.017,     &
                    -0.015,-0.012,-0.010,-0.008,-0.006,-0.003,-0.001,     &
                    0.001,0.004,0.006,0.008,0.010,0.013,0.015,0.017,0.020,&
                    0.022,0.024,0.027,0.029,0.031,0.033,0.036,0.038,0.040,&
                    0.043,0.045,0.047,0.049,0.052,0.054,0.056,0.059,0.061,&
                    0.063,0.066,0.068,0.070,0.072,0.075,0.077,0.079,0.081,&
                    0.084,0.086,0.088,0.091,0.093,0.095,0.097,0.100,0.102,&
                    0.104,0.106,0.109,0.111,0.113,0.115,0.118,0.120,0.122,&
                    0.124,0.127,0.129,0.131,0.133,0.135,0.138,0.140,0.142,&
                    0.144,0.147,0.149,0.151,0.153,0.155,0.158,0.160,0.162,&
                    0.164,0.166,0.169,0.171,0.173,0.175,0.177,0.179,0.182,&
                    0.184,0.186,0.188,0.190,0.192,0.195,0.197,0.199,0.201,&
                    0.203,0.205,0.207,0.210,0.212,0.214,0.216,0.218,0.220,&
                    0.222,0.224,0.227,0.229,0.231,0.233,0.235,0.237,0.239,&
                    0.241,0.243,0.245,0.248,0.250,0.252,0.254,0.256,0.258,&
                    0.260,0.262,0.264,0.266,0.268,0.270,0.272,0.274,0.276,&
                    0.278,0.281,0.283,0.285,0.287,0.289,0.291,0.293,0.295,&
                    0.297,0.299,0.301,0.303,0.305,0.307,0.309,0.311,0.313,&
                    0.315,0.317,0.319,0.321,0.323,0.325,0.327,0.329,0.331,&
                    0.333,0.335,0.337,0.339,0.341,0.343,0.344,0.346,0.348,&
                    0.350,0.352,0.354,0.356,0.358,0.360,0.362,0.364,0.366,&
                    0.368,0.370,0.372,0.373,0.375,0.377,0.379,0.381,0.383,&
                    0.385,0.387,0.389,0.391,0.393,0.394,0.396,0.398,0.400,&
                    0.402,0.404,0.406,0.408,0.409,0.411,0.413,0.415,0.417,&
                    0.419,0.421,0.422,0.424,0.426,0.428,0.430,0.432,0.433,&
                    0.435,0.437,0.439,0.441,0.443,0.444,0.446,0.448,0.450,&
                    0.452,0.454,0.455,0.457,0.459,0.461,0.463,0.464,0.466,&
                    0.468,0.470,0.471,0.473,0.475,0.477,0.479,0.480,0.482,&
                    0.484,0.486,0.487,0.489,0.491,0.493,0.494,0.496,0.498,&
                    0.500,0.501,0.503,0.505,0.507,0.508,0.510,0.512,0.514,&
                    0.515,0.517,0.519,0.521,0.522,0.524,0.526,0.527,0.529,&
                    0.531,0.533,0.534,0.536,0.538,0.539,0.541,0.543,0.544,&
                    0.546,0.548,0.549,0.551,0.553,0.554,0.556,0.558,0.560,&
                    0.561,0.563,0.581,0.597,0.613,0.629,0.645,0.660,0.676,&
                    0.691,0.706,0.721,0.736,0.750,0.765,0.779,0.793,0.807,&
                    0.820,0.834,0.848,0.861,0.874,0.887,0.900,0.913,0.926,&
                    0.938,0.951,0.963,0.975,0.987,0.999,1.011,1.023,1.035,&
                    1.046,1.058,1.069,1.081,1.092,1.103,1.114,1.125,1.136,&
                    1.147,1.157,1.168,1.178,1.189,1.199,1.210,1.220,1.230,&
                    1.240,1.250,1.260,1.270,1.279,1.289,1.299,1.308,1.318,&
                    1.327,1.337,1.346,1.355,1.365,1.374,1.383,1.392,1.401,&
                    1.410,1.419,1.427,1.436,1.445,1.454,1.462,1.471,1.479,&
                    1.488,1.496,1.504,1.513,1.521,1.529,1.537,1.546,1.554,&
                    1.562,1.570,1.578,1.586,1.593,1.601,1.609,1.617,1.624,&
                    1.632,1.640,1.647,1.655,1.662,1.670,1.677,1.685,1.692,&
                    1.700,1.707,1.714,1.721,1.729,1.736,1.743,1.750,1.757,&
                    1.764,1.771,1.778,1.785,1.792,1.799,1.806,1.813,1.820,&
                    1.826,1.833,1.840,1.847,1.853,1.860,1.866,1.873,1.880,&
                    1.886,1.893,1.899,1.906,1.912,1.918,1.925,1.931,1.938,&
                    1.944,1.950,1.956,1.963,1.969,1.975,1.981,1.987,1.994,&
                    2.000,2.006,2.012,2.018,2.024,2.030,2.036,2.042,2.048,&
                    2.054/)
!
! *** (h ,cl)
!
    bnc11m(1:561) = (/-0.043,-0.087,-0.106,-0.117,-0.124,-0.130,-0.133,   &
                    -0.135,-0.137,-0.138,-0.138,-0.137,-0.136,-0.135,     &
                    -0.134,-0.132,-0.130,-0.127,-0.125,-0.122,-0.119,     &
                    -0.116,-0.113,-0.110,-0.107,-0.103,-0.099,-0.096,     &
                    -0.092,-0.088,-0.084,-0.080,-0.076,-0.071,-0.067,     &
                    -0.063,-0.058,-0.054,-0.049,-0.045,-0.040,-0.036,     &
                    -0.031,-0.027,-0.022,-0.017,-0.012,-0.008,-0.003,     &
                    0.002,0.007,0.012,0.017,0.022,0.027,0.031,0.036,0.041,&
                    0.046,0.051,0.056,0.061,0.066,0.071,0.077,0.082,0.087,&
                    0.092,0.097,0.102,0.107,0.113,0.118,0.123,0.128,0.134,&
                    0.139,0.144,0.149,0.155,0.160,0.166,0.171,0.177,0.182,&
                    0.188,0.193,0.199,0.204,0.210,0.216,0.221,0.227,0.233,&
                    0.239,0.244,0.250,0.256,0.262,0.268,0.274,0.280,0.286,&
                    0.292,0.298,0.304,0.310,0.316,0.322,0.328,0.334,0.340,&
                    0.346,0.352,0.358,0.364,0.370,0.377,0.383,0.389,0.395,&
                    0.401,0.407,0.413,0.420,0.426,0.432,0.438,0.444,0.450,&
                    0.457,0.463,0.469,0.475,0.481,0.487,0.493,0.499,0.505,&
                    0.512,0.518,0.524,0.530,0.536,0.542,0.548,0.554,0.560,&
                    0.566,0.572,0.578,0.584,0.590,0.596,0.602,0.608,0.614,&
                    0.620,0.626,0.632,0.638,0.643,0.649,0.655,0.661,0.667,&
                    0.673,0.679,0.684,0.690,0.696,0.702,0.708,0.713,0.719,&
                    0.725,0.731,0.737,0.742,0.748,0.754,0.759,0.765,0.771,&
                    0.776,0.782,0.788,0.793,0.799,0.805,0.810,0.816,0.821,&
                    0.827,0.833,0.838,0.844,0.849,0.855,0.860,0.866,0.871,&
                    0.877,0.882,0.888,0.893,0.899,0.904,0.909,0.915,0.920,&
                    0.926,0.931,0.936,0.942,0.947,0.953,0.958,0.963,0.969,&
                    0.974,0.979,0.984,0.990,0.995,1.000,1.005,1.011,1.016,&
                    1.021,1.026,1.032,1.037,1.042,1.047,1.052,1.057,1.062,&
                    1.068,1.073,1.078,1.083,1.088,1.093,1.098,1.103,1.108,&
                    1.113,1.118,1.123,1.128,1.133,1.138,1.143,1.148,1.153,&
                    1.158,1.163,1.168,1.173,1.178,1.183,1.188,1.193,1.198,&
                    1.202,1.207,1.212,1.217,1.222,1.227,1.232,1.236,1.241,&
                    1.246,1.251,1.256,1.260,1.265,1.270,1.275,1.279,1.284,&
                    1.289,1.293,1.298,1.303,1.307,1.312,1.317,1.321,1.326,&
                    1.331,1.335,1.340,1.345,1.349,1.354,1.358,1.363,1.368,&
                    1.372,1.377,1.381,1.386,1.390,1.395,1.399,1.404,1.408,&
                    1.413,1.417,1.422,1.426,1.431,1.435,1.440,1.444,1.449,&
                    1.453,1.457,1.462,1.466,1.471,1.475,1.479,1.484,1.488,&
                    1.492,1.497,1.501,1.506,1.510,1.514,1.518,1.523,1.527,&
                    1.531,1.536,1.540,1.544,1.548,1.553,1.557,1.561,1.565,&
                    1.570,1.574,1.578,1.582,1.587,1.591,1.595,1.599,1.603,&
                    1.607,1.612,1.616,1.620,1.624,1.628,1.632,1.636,1.640,&
                    1.645,1.649,1.653,1.657,1.661,1.665,1.669,1.673,1.677,&
                    1.681,1.685,1.689,1.693,1.697,1.701,1.705,1.709,1.713,&
                    1.717,1.721,1.725,1.729,1.733,1.737,1.741,1.745,1.749,&
                    1.753,1.757,1.761,1.765,1.769,1.772,1.776,1.780,1.784,&
                    1.826,1.864,1.901,1.938,1.974,2.010,2.045,2.080,2.114,&
                    2.148,2.181,2.214,2.246,2.278,2.310,2.341,2.372,2.403,&
                    2.433,2.463,2.492,2.522,2.550,2.579,2.607,2.635,2.663,&
                    2.690,2.717,2.744,2.770,2.796,2.822,2.848,2.873,2.898,&
                    2.923,2.948,2.972,2.996,3.020,3.044,3.068,3.091,3.114,&
                    3.137,3.160,3.182,3.205,3.227,3.249,3.271,3.292,3.314,&
                    3.335,3.356,3.377,3.398,3.418,3.439,3.459,3.479,3.499,&
                    3.519,3.539,3.558,3.578,3.597,3.616,3.635,3.654,3.673,&
                    3.691,3.710,3.728,3.746,3.765,3.783,3.800,3.818,3.836,&
                    3.853,3.871,3.888,3.905,3.922,3.939,3.956,3.973,3.990,&
                    4.006,4.023,4.039,4.055,4.071,4.088,4.104,4.119,4.135,&
                    4.151,4.167,4.182,4.198,4.213,4.228,4.243,4.259,4.274,&
                    4.289,4.304,4.318,4.333,4.348,4.362,4.377,4.391,4.406,&
                    4.420,4.434,4.448,4.462,4.476,4.490,4.504,4.518,4.532,&
                    4.545,4.559,4.572,4.586,4.599,4.613,4.626,4.639,4.652,&
                    4.665,4.678,4.691,4.704,4.717,4.730,4.743,4.756,4.768,&
                    4.781,4.793,4.806,4.818,4.831,4.843,4.855,4.867,4.880,&
                    4.892,4.904,4.916,4.928,4.940,4.952,4.964,4.975/)
!
! *** nahso4
!
    bnc12m(1:561) = (/-0.044,-0.092,-0.113,-0.127,-0.138,-0.146,-0.152,   &
                    -0.157,-0.162,-0.165,-0.168,-0.170,-0.172,-0.174,     &
                    -0.175,-0.176,-0.176,-0.177,-0.177,-0.177,-0.176,     &
                    -0.176,-0.176,-0.175,-0.174,-0.173,-0.172,-0.171,     &
                    -0.169,-0.168,-0.167,-0.165,-0.163,-0.161,-0.160,     &
                    -0.158,-0.156,-0.154,-0.151,-0.149,-0.147,-0.145,     &
                    -0.142,-0.140,-0.137,-0.135,-0.132,-0.130,-0.127,     &
                    -0.124,-0.122,-0.119,-0.116,-0.113,-0.110,-0.107,     &
                    -0.104,-0.101,-0.098,-0.095,-0.092,-0.089,-0.086,     &
                    -0.083,-0.080,-0.076,-0.073,-0.070,-0.067,-0.063,     &
                    -0.060,-0.057,-0.053,-0.050,-0.046,-0.043,-0.039,     &
                    -0.036,-0.032,-0.029,-0.025,-0.021,-0.018,-0.014,     &
                    -0.010,-0.007,-0.003,0.001,0.005,0.009,0.012,0.016,   &
                    0.020,0.024,0.028,0.032,0.036,0.040,0.044,0.048,0.052,&
                    0.056,0.060,0.064,0.068,0.073,0.077,0.081,0.085,0.089,&
                    0.093,0.098,0.102,0.106,0.110,0.115,0.119,0.123,0.127,&
                    0.131,0.136,0.140,0.144,0.148,0.153,0.157,0.161,0.165,&
                    0.170,0.174,0.178,0.182,0.187,0.191,0.195,0.199,0.204,&
                    0.208,0.212,0.216,0.220,0.225,0.229,0.233,0.237,0.241,&
                    0.245,0.250,0.254,0.258,0.262,0.266,0.270,0.274,0.279,&
                    0.283,0.287,0.291,0.295,0.299,0.303,0.307,0.311,0.315,&
                    0.319,0.323,0.327,0.331,0.335,0.339,0.343,0.347,0.351,&
                    0.355,0.359,0.363,0.367,0.371,0.375,0.379,0.383,0.387,&
                    0.391,0.395,0.399,0.402,0.406,0.410,0.414,0.418,0.422,&
                    0.426,0.429,0.433,0.437,0.441,0.445,0.449,0.452,0.456,&
                    0.460,0.464,0.467,0.471,0.475,0.479,0.482,0.486,0.490,&
                    0.494,0.497,0.501,0.505,0.508,0.512,0.516,0.519,0.523,&
                    0.527,0.530,0.534,0.538,0.541,0.545,0.548,0.552,0.556,&
                    0.559,0.563,0.566,0.570,0.573,0.577,0.581,0.584,0.588,&
                    0.591,0.595,0.598,0.602,0.605,0.609,0.612,0.616,0.619,&
                    0.623,0.626,0.630,0.633,0.636,0.640,0.643,0.647,0.650,&
                    0.654,0.657,0.660,0.664,0.667,0.671,0.674,0.677,0.681,&
                    0.684,0.687,0.691,0.694,0.697,0.701,0.704,0.707,0.711,&
                    0.714,0.717,0.721,0.724,0.727,0.730,0.734,0.737,0.740,&
                    0.743,0.747,0.750,0.753,0.756,0.760,0.763,0.766,0.769,&
                    0.773,0.776,0.779,0.782,0.785,0.788,0.792,0.795,0.798,&
                    0.801,0.804,0.807,0.811,0.814,0.817,0.820,0.823,0.826,&
                    0.829,0.832,0.835,0.839,0.842,0.845,0.848,0.851,0.854,&
                    0.857,0.860,0.863,0.866,0.869,0.872,0.875,0.878,0.881,&
                    0.884,0.887,0.890,0.893,0.896,0.899,0.902,0.905,0.908,&
                    0.911,0.914,0.917,0.920,0.923,0.926,0.929,0.932,0.935,&
                    0.938,0.941,0.944,0.947,0.949,0.952,0.955,0.958,0.961,&
                    0.964,0.967,0.970,0.973,0.975,0.978,0.981,0.984,0.987,&
                    0.990,0.993,0.995,0.998,1.001,1.004,1.007,1.010,1.012,&
                    1.015,1.018,1.021,1.024,1.026,1.029,1.032,1.035,1.038,&
                    1.040,1.043,1.046,1.049,1.051,1.054,1.057,1.060,1.062,&
                    1.065,1.068,1.071,1.073,1.076,1.079,1.082,1.084,1.087,&
                    1.090,1.092,1.121,1.148,1.174,1.199,1.225,1.250,1.274,&
                    1.299,1.323,1.346,1.370,1.393,1.416,1.438,1.461,1.483,&
                    1.504,1.526,1.547,1.568,1.589,1.610,1.630,1.650,1.670,&
                    1.690,1.710,1.729,1.748,1.767,1.786,1.805,1.823,1.841,&
                    1.860,1.878,1.895,1.913,1.930,1.948,1.965,1.982,1.999,&
                    2.015,2.032,2.048,2.065,2.081,2.097,2.113,2.129,2.144,&
                    2.160,2.175,2.191,2.206,2.221,2.236,2.251,2.265,2.280,&
                    2.294,2.309,2.323,2.337,2.352,2.366,2.379,2.393,2.407,&
                    2.421,2.434,2.448,2.461,2.474,2.488,2.501,2.514,2.527,&
                    2.540,2.552,2.565,2.578,2.590,2.603,2.615,2.628,2.640,&
                    2.652,2.664,2.676,2.688,2.700,2.712,2.724,2.735,2.747,&
                    2.759,2.770,2.782,2.793,2.805,2.816,2.827,2.838,2.849,&
                    2.860,2.871,2.882,2.893,2.904,2.915,2.926,2.936,2.947,&
                    2.957,2.968,2.978,2.989,2.999,3.010,3.020,3.030,3.040,&
                    3.050,3.060,3.070,3.080,3.090,3.100,3.110,3.120,3.130,&
                    3.139,3.149,3.159,3.168,3.178,3.188,3.197,3.206,3.216,&
                    3.225,3.235,3.244,3.253,3.262,3.272,3.281,3.290,3.299,&
                    3.308,3.317,3.326,3.335,3.344,3.353,3.361,3.370,3.379,&
                    3.388/)
!
! *** (nh4)3h(so4)2
!
    bnc13m(1:561) = (/-0.072,-0.156,-0.197,-0.226,-0.248,-0.266,-0.282,   &
                    -0.295,-0.307,-0.317,-0.327,-0.336,-0.344,-0.351,     &
                    -0.358,-0.364,-0.370,-0.375,-0.380,-0.385,-0.390,     &
                    -0.394,-0.398,-0.402,-0.406,-0.409,-0.412,-0.415,     &
                    -0.418,-0.421,-0.424,-0.426,-0.429,-0.431,-0.433,     &
                    -0.435,-0.437,-0.439,-0.441,-0.442,-0.444,-0.446,     &
                    -0.447,-0.449,-0.450,-0.451,-0.452,-0.453,-0.454,     &
                    -0.456,-0.456,-0.457,-0.458,-0.459,-0.460,-0.461,     &
                    -0.461,-0.462,-0.462,-0.463,-0.463,-0.464,-0.464,     &
                    -0.465,-0.465,-0.465,-0.466,-0.466,-0.466,-0.467,     &
                    -0.467,-0.467,-0.467,-0.467,-0.467,-0.467,-0.467,     &
                    -0.467,-0.467,-0.467,-0.467,-0.467,-0.467,-0.467,     &
                    -0.467,-0.467,-0.466,-0.466,-0.466,-0.466,-0.466,     &
                    -0.465,-0.465,-0.465,-0.464,-0.464,-0.464,-0.463,     &
                    -0.463,-0.463,-0.462,-0.462,-0.461,-0.461,-0.460,     &
                    -0.460,-0.459,-0.459,-0.458,-0.458,-0.457,-0.457,     &
                    -0.456,-0.456,-0.455,-0.455,-0.454,-0.454,-0.453,     &
                    -0.452,-0.452,-0.451,-0.451,-0.450,-0.449,-0.449,     &
                    -0.448,-0.447,-0.447,-0.446,-0.445,-0.445,-0.444,     &
                    -0.443,-0.443,-0.442,-0.441,-0.441,-0.440,-0.439,     &
                    -0.439,-0.438,-0.437,-0.437,-0.436,-0.435,-0.434,     &
                    -0.434,-0.433,-0.432,-0.432,-0.431,-0.430,-0.429,     &
                    -0.429,-0.428,-0.427,-0.427,-0.426,-0.425,-0.424,     &
                    -0.424,-0.423,-0.422,-0.421,-0.421,-0.420,-0.419,     &
                    -0.418,-0.418,-0.417,-0.416,-0.415,-0.415,-0.414,     &
                    -0.413,-0.412,-0.412,-0.411,-0.410,-0.409,-0.409,     &
                    -0.408,-0.407,-0.406,-0.406,-0.405,-0.404,-0.403,     &
                    -0.403,-0.402,-0.401,-0.400,-0.400,-0.399,-0.398,     &
                    -0.397,-0.397,-0.396,-0.395,-0.394,-0.394,-0.393,     &
                    -0.392,-0.391,-0.391,-0.390,-0.389,-0.388,-0.388,     &
                    -0.387,-0.386,-0.385,-0.385,-0.384,-0.383,-0.382,     &
                    -0.381,-0.381,-0.380,-0.379,-0.378,-0.378,-0.377,     &
                    -0.376,-0.375,-0.375,-0.374,-0.373,-0.372,-0.372,     &
                    -0.371,-0.370,-0.369,-0.369,-0.368,-0.367,-0.366,     &
                    -0.366,-0.365,-0.364,-0.363,-0.363,-0.362,-0.361,     &
                    -0.360,-0.360,-0.359,-0.358,-0.357,-0.357,-0.356,     &
                    -0.355,-0.354,-0.354,-0.353,-0.352,-0.351,-0.351,     &
                    -0.350,-0.349,-0.348,-0.348,-0.347,-0.346,-0.345,     &
                    -0.345,-0.344,-0.343,-0.342,-0.342,-0.341,-0.340,     &
                    -0.339,-0.339,-0.338,-0.337,-0.336,-0.336,-0.335,     &
                    -0.334,-0.333,-0.333,-0.332,-0.331,-0.331,-0.330,     &
                    -0.329,-0.328,-0.328,-0.327,-0.326,-0.325,-0.325,     &
                    -0.324,-0.323,-0.322,-0.322,-0.321,-0.320,-0.320,     &
                    -0.319,-0.318,-0.317,-0.317,-0.316,-0.315,-0.314,     &
                    -0.314,-0.313,-0.312,-0.312,-0.311,-0.310,-0.309,     &
                    -0.309,-0.308,-0.307,-0.306,-0.306,-0.305,-0.304,     &
                    -0.304,-0.303,-0.302,-0.301,-0.301,-0.300,-0.299,     &
                    -0.299,-0.298,-0.297,-0.296,-0.296,-0.295,-0.294,     &
                    -0.294,-0.293,-0.292,-0.291,-0.291,-0.290,-0.289,     &
                    -0.289,-0.288,-0.287,-0.286,-0.286,-0.285,-0.284,     &
                    -0.284,-0.283,-0.282,-0.281,-0.281,-0.280,-0.279,     &
                    -0.279,-0.278,-0.277,-0.277,-0.276,-0.275,-0.274,     &
                    -0.274,-0.273,-0.272,-0.272,-0.271,-0.270,-0.270,     &
                    -0.269,-0.268,-0.267,-0.267,-0.266,-0.265,-0.265,     &
                    -0.264,-0.263,-0.263,-0.262,-0.261,-0.261,-0.260,     &
                    -0.259,-0.258,-0.258,-0.257,-0.256,-0.256,-0.255,     &
                    -0.254,-0.254,-0.253,-0.252,-0.252,-0.251,-0.250,     &
                    -0.250,-0.242,-0.235,-0.229,-0.222,-0.215,-0.209,     &
                    -0.202,-0.196,-0.189,-0.183,-0.176,-0.170,-0.164,     &
                    -0.157,-0.151,-0.145,-0.139,-0.133,-0.127,-0.121,     &
                    -0.115,-0.109,-0.103,-0.097,-0.091,-0.085,-0.079,     &
                    -0.073,-0.067,-0.062,-0.056,-0.050,-0.045,-0.039,     &
                    -0.033,-0.028,-0.022,-0.017,-0.011,-0.006,0.000,0.005,&
                    0.010,0.016,0.021,0.026,0.032,0.037,0.042,0.048,0.053,&
                    0.058,0.063,0.068,0.073,0.079,0.084,0.089,0.094,0.099,&
                    0.104,0.109,0.114,0.119,0.124,0.129,0.134,0.138,0.143,&
                    0.148,0.153,0.158,0.163,0.167,0.172,0.177,0.182,0.186,&
                    0.191,0.196,0.201,0.205,0.210,0.215,0.219,0.224,0.228,&
                    0.233,0.238,0.242,0.247,0.251,0.256,0.260,0.265,0.269,&
                    0.274,0.278,0.283,0.287,0.291,0.296,0.300,0.305,0.309,&
                    0.313,0.318,0.322,0.326,0.331,0.335,0.339,0.344,0.348,&
                    0.352,0.357,0.361,0.365,0.369,0.374,0.378,0.382,0.386,&
                    0.390,0.395,0.399,0.403,0.407,0.411,0.415,0.419,0.424,&
                    0.428,0.432,0.436,0.440,0.444,0.448,0.452,0.456,0.460,&
                    0.464,0.468,0.472,0.476,0.480,0.484,0.488,0.492,0.496,&
                    0.500,0.504,0.508,0.512,0.516,0.520,0.524,0.528,0.532,&
                    0.536,0.540/)
!
! *** caso4
!
    bnc14m(1:561) = (/0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,    &
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,&
                    0.000,0.000,0.000,0.000/)
!
! *** cano32
!
    bnc15m(1:561) = (/-0.090,-0.191,-0.239,-0.271,-0.296,-0.316,-0.332,   &
                    -0.346,-0.358,-0.368,-0.378,-0.386,-0.393,-0.400,     &
                    -0.405,-0.411,-0.416,-0.420,-0.424,-0.428,-0.431,     &
                    -0.434,-0.437,-0.440,-0.442,-0.444,-0.446,-0.448,     &
                    -0.450,-0.451,-0.453,-0.454,-0.455,-0.456,-0.457,     &
                    -0.458,-0.459,-0.460,-0.460,-0.461,-0.461,-0.462,     &
                    -0.462,-0.463,-0.463,-0.463,-0.463,-0.463,-0.464,     &
                    -0.464,-0.464,-0.464,-0.464,-0.464,-0.464,-0.463,     &
                    -0.463,-0.463,-0.463,-0.463,-0.462,-0.462,-0.462,     &
                    -0.462,-0.461,-0.461,-0.460,-0.460,-0.460,-0.459,     &
                    -0.459,-0.458,-0.458,-0.457,-0.457,-0.456,-0.455,     &
                    -0.455,-0.454,-0.454,-0.453,-0.452,-0.451,-0.451,     &
                    -0.450,-0.449,-0.448,-0.448,-0.447,-0.446,-0.445,     &
                    -0.444,-0.443,-0.442,-0.441,-0.440,-0.439,-0.439,     &
                    -0.438,-0.437,-0.435,-0.434,-0.433,-0.432,-0.431,     &
                    -0.430,-0.429,-0.428,-0.427,-0.426,-0.425,-0.423,     &
                    -0.422,-0.421,-0.420,-0.419,-0.417,-0.416,-0.415,     &
                    -0.414,-0.413,-0.411,-0.410,-0.409,-0.408,-0.406,     &
                    -0.405,-0.404,-0.403,-0.401,-0.400,-0.399,-0.398,     &
                    -0.396,-0.395,-0.394,-0.392,-0.391,-0.390,-0.388,     &
                    -0.387,-0.386,-0.385,-0.383,-0.382,-0.381,-0.379,     &
                    -0.378,-0.377,-0.375,-0.374,-0.373,-0.371,-0.370,     &
                    -0.369,-0.367,-0.366,-0.365,-0.363,-0.362,-0.361,     &
                    -0.359,-0.358,-0.357,-0.355,-0.354,-0.353,-0.351,     &
                    -0.350,-0.349,-0.347,-0.346,-0.345,-0.343,-0.342,     &
                    -0.341,-0.339,-0.338,-0.337,-0.335,-0.334,-0.333,     &
                    -0.331,-0.330,-0.328,-0.327,-0.326,-0.324,-0.323,     &
                    -0.322,-0.320,-0.319,-0.318,-0.316,-0.315,-0.314,     &
                    -0.312,-0.311,-0.310,-0.308,-0.307,-0.306,-0.304,     &
                    -0.303,-0.301,-0.300,-0.299,-0.297,-0.296,-0.295,     &
                    -0.293,-0.292,-0.291,-0.289,-0.288,-0.287,-0.285,     &
                    -0.284,-0.283,-0.281,-0.280,-0.279,-0.277,-0.276,     &
                    -0.275,-0.273,-0.272,-0.271,-0.269,-0.268,-0.267,     &
                    -0.265,-0.264,-0.263,-0.261,-0.260,-0.259,-0.257,     &
                    -0.256,-0.255,-0.253,-0.252,-0.251,-0.249,-0.248,     &
                    -0.247,-0.245,-0.244,-0.243,-0.241,-0.240,-0.239,     &
                    -0.237,-0.236,-0.235,-0.233,-0.232,-0.231,-0.230,     &
                    -0.228,-0.227,-0.226,-0.224,-0.223,-0.222,-0.220,     &
                    -0.219,-0.218,-0.216,-0.215,-0.214,-0.213,-0.211,     &
                    -0.210,-0.209,-0.207,-0.206,-0.205,-0.203,-0.202,     &
                    -0.201,-0.200,-0.198,-0.197,-0.196,-0.194,-0.193,     &
                    -0.192,-0.191,-0.189,-0.188,-0.187,-0.185,-0.184,     &
                    -0.183,-0.182,-0.180,-0.179,-0.178,-0.176,-0.175,     &
                    -0.174,-0.173,-0.171,-0.170,-0.169,-0.167,-0.166,     &
                    -0.165,-0.164,-0.162,-0.161,-0.160,-0.159,-0.157,     &
                    -0.156,-0.155,-0.154,-0.152,-0.151,-0.150,-0.149,     &
                    -0.147,-0.146,-0.145,-0.144,-0.142,-0.141,-0.140,     &
                    -0.139,-0.137,-0.136,-0.135,-0.134,-0.132,-0.131,     &
                    -0.130,-0.129,-0.127,-0.126,-0.125,-0.124,-0.122,     &
                    -0.121,-0.120,-0.119,-0.117,-0.116,-0.115,-0.114,     &
                    -0.113,-0.111,-0.110,-0.109,-0.108,-0.106,-0.105,     &
                    -0.104,-0.103,-0.102,-0.100,-0.099,-0.098,-0.097,     &
                    -0.095,-0.094,-0.093,-0.092,-0.091,-0.089,-0.088,     &
                    -0.087,-0.086,-0.085,-0.083,-0.082,-0.081,-0.080,     &
                    -0.079,-0.077,-0.076,-0.075,-0.074,-0.073,-0.071,     &
                    -0.070,-0.069,-0.068,-0.067,-0.065,-0.064,-0.063,     &
                    -0.062,-0.061,-0.059,-0.058,-0.057,-0.056,-0.055,     &
                    -0.054,-0.041,-0.029,-0.018,-0.006,0.005,0.017,0.028, &
                    0.039,0.050,0.061,0.072,0.083,0.094,0.104,0.115,0.125,&
                    0.136,0.146,0.156,0.167,0.177,0.187,0.197,0.207,0.217,&
                    0.227,0.236,0.246,0.256,0.265,0.275,0.284,0.294,0.303,&
                    0.312,0.322,0.331,0.340,0.349,0.358,0.367,0.376,0.385,&
                    0.394,0.403,0.411,0.420,0.429,0.437,0.446,0.455,0.463,&
                    0.472,0.480,0.488,0.497,0.505,0.513,0.521,0.530,0.538,&
                    0.546,0.554,0.562,0.570,0.578,0.586,0.594,0.602,0.610,&
                    0.617,0.625,0.633,0.641,0.648,0.656,0.664,0.671,0.679,&
                    0.686,0.694,0.701,0.709,0.716,0.723,0.731,0.738,0.745,&
                    0.753,0.760,0.767,0.774,0.782,0.789,0.796,0.803,0.810,&
                    0.817,0.824,0.831,0.838,0.845,0.852,0.859,0.866,0.873,&
                    0.880,0.886,0.893,0.900,0.907,0.914,0.920,0.927,0.934,&
                    0.940,0.947,0.954,0.960,0.967,0.973,0.980,0.987,0.993,&
                    1.000,1.006,1.013,1.019,1.025,1.032,1.038,1.045,1.051,&
                    1.057,1.064,1.070,1.076,1.083,1.089,1.095,1.101,1.108,&
                    1.114,1.120,1.126,1.132,1.139,1.145,1.151,1.157,1.163,&
                    1.169,1.175,1.181,1.187,1.193,1.199,1.205,1.211,1.217,&
                    1.223/)
!
! *** cacl2
!
    bnc16m(1:561) = (/-0.088,-0.184,-0.228,-0.256,-0.277,-0.293,-0.305,   &
                    -0.315,-0.323,-0.330,-0.335,-0.340,-0.343,-0.346,     &
                    -0.349,-0.350,-0.352,-0.353,-0.353,-0.353,-0.353,     &
                    -0.353,-0.352,-0.352,-0.351,-0.349,-0.348,-0.347,     &
                    -0.345,-0.343,-0.341,-0.339,-0.337,-0.335,-0.333,     &
                    -0.331,-0.328,-0.326,-0.324,-0.321,-0.318,-0.316,     &
                    -0.313,-0.311,-0.308,-0.305,-0.302,-0.299,-0.297,     &
                    -0.294,-0.291,-0.288,-0.285,-0.282,-0.279,-0.276,     &
                    -0.273,-0.270,-0.267,-0.264,-0.261,-0.258,-0.255,     &
                    -0.252,-0.249,-0.246,-0.243,-0.239,-0.236,-0.233,     &
                    -0.230,-0.227,-0.223,-0.220,-0.217,-0.214,-0.210,     &
                    -0.207,-0.203,-0.200,-0.197,-0.193,-0.190,-0.186,     &
                    -0.183,-0.179,-0.176,-0.172,-0.168,-0.165,-0.161,     &
                    -0.158,-0.154,-0.150,-0.146,-0.143,-0.139,-0.135,     &
                    -0.131,-0.127,-0.123,-0.119,-0.115,-0.111,-0.107,     &
                    -0.103,-0.099,-0.095,-0.091,-0.087,-0.083,-0.079,     &
                    -0.075,-0.071,-0.067,-0.063,-0.059,-0.055,-0.050,     &
                    -0.046,-0.042,-0.038,-0.034,-0.030,-0.026,-0.021,     &
                    -0.017,-0.013,-0.009,-0.005,0.000,0.004,0.008,0.012,  &
                    0.016,0.020,0.025,0.029,0.033,0.037,0.041,0.046,0.050,&
                    0.054,0.058,0.062,0.066,0.071,0.075,0.079,0.083,0.087,&
                    0.091,0.095,0.100,0.104,0.108,0.112,0.116,0.120,0.124,&
                    0.128,0.133,0.137,0.141,0.145,0.149,0.153,0.157,0.161,&
                    0.165,0.169,0.174,0.178,0.182,0.186,0.190,0.194,0.198,&
                    0.202,0.206,0.210,0.214,0.218,0.222,0.226,0.230,0.234,&
                    0.238,0.242,0.246,0.250,0.254,0.258,0.262,0.266,0.270,&
                    0.274,0.278,0.282,0.286,0.290,0.294,0.298,0.302,0.306,&
                    0.310,0.314,0.318,0.321,0.325,0.329,0.333,0.337,0.341,&
                    0.345,0.349,0.353,0.356,0.360,0.364,0.368,0.372,0.376,&
                    0.380,0.383,0.387,0.391,0.395,0.399,0.403,0.406,0.410,&
                    0.414,0.418,0.422,0.425,0.429,0.433,0.437,0.440,0.444,&
                    0.448,0.452,0.455,0.459,0.463,0.467,0.470,0.474,0.478,&
                    0.482,0.485,0.489,0.493,0.496,0.500,0.504,0.507,0.511,&
                    0.515,0.518,0.522,0.526,0.529,0.533,0.537,0.540,0.544,&
                    0.548,0.551,0.555,0.558,0.562,0.566,0.569,0.573,0.576,&
                    0.580,0.584,0.587,0.591,0.594,0.598,0.601,0.605,0.609,&
                    0.612,0.616,0.619,0.623,0.626,0.630,0.633,0.637,0.640,&
                    0.644,0.647,0.651,0.654,0.658,0.661,0.665,0.668,0.672,&
                    0.675,0.678,0.682,0.685,0.689,0.692,0.696,0.699,0.703,&
                    0.706,0.709,0.713,0.716,0.720,0.723,0.726,0.730,0.733,&
                    0.737,0.740,0.743,0.747,0.750,0.753,0.757,0.760,0.763,&
                    0.767,0.770,0.773,0.777,0.780,0.783,0.787,0.790,0.793,&
                    0.797,0.800,0.803,0.807,0.810,0.813,0.816,0.820,0.823,&
                    0.826,0.829,0.833,0.836,0.839,0.842,0.846,0.849,0.852,&
                    0.855,0.859,0.862,0.865,0.868,0.871,0.875,0.878,0.881,&
                    0.884,0.887,0.891,0.894,0.897,0.900,0.903,0.906,0.910,&
                    0.913,0.916,0.919,0.922,0.925,0.928,0.932,0.935,0.938,&
                    0.941,0.944,0.947,0.950,0.953,0.957,0.960,0.963,0.966,&
                    0.969,0.972,0.975,0.978,0.981,1.014,1.044,1.074,1.103,&
                    1.132,1.161,1.189,1.218,1.245,1.273,1.300,1.327,1.353,&
                    1.380,1.406,1.431,1.457,1.482,1.507,1.532,1.556,1.580,&
                    1.604,1.628,1.652,1.675,1.698,1.721,1.744,1.766,1.788,&
                    1.811,1.832,1.854,1.876,1.897,1.918,1.939,1.960,1.981,&
                    2.001,2.022,2.042,2.062,2.082,2.101,2.121,2.140,2.160,&
                    2.179,2.198,2.217,2.235,2.254,2.273,2.291,2.309,2.327,&
                    2.345,2.363,2.381,2.398,2.416,2.433,2.451,2.468,2.485,&
                    2.502,2.519,2.535,2.552,2.569,2.585,2.601,2.618,2.634,&
                    2.650,2.666,2.682,2.698,2.713,2.729,2.745,2.760,2.775,&
                    2.791,2.806,2.821,2.836,2.851,2.866,2.881,2.895,2.910,&
                    2.925,2.939,2.954,2.968,2.982,2.997,3.011,3.025,3.039,&
                    3.053,3.067,3.081,3.094,3.108,3.122,3.135,3.149,3.162,&
                    3.176,3.189,3.202,3.216,3.229,3.242,3.255,3.268,3.281,&
                    3.294,3.307,3.319,3.332,3.345,3.357,3.370,3.383,3.395,&
                    3.407,3.420,3.432,3.444,3.457,3.469,3.481,3.493,3.505,&
                    3.517,3.529,3.541,3.553,3.565,3.577,3.588,3.600,3.612,&
                    3.623,3.635,3.646,3.658,3.669,3.681,3.692,3.704,3.715,&
                    3.726,3.737,3.748,3.760/)
!
! *** k2so4
!
    bnc17m(1:561) = (/-0.091,-0.197,-0.249,-0.286,-0.315,-0.339,-0.359,   &
                    -0.377,-0.392,-0.406,-0.419,-0.431,-0.442,-0.452,     &
                    -0.461,-0.470,-0.478,-0.486,-0.493,-0.500,-0.507,     &
                    -0.513,-0.519,-0.525,-0.530,-0.535,-0.540,-0.545,     &
                    -0.550,-0.554,-0.559,-0.563,-0.567,-0.571,-0.575,     &
                    -0.579,-0.582,-0.586,-0.589,-0.592,-0.596,-0.599,     &
                    -0.602,-0.605,-0.608,-0.611,-0.613,-0.616,-0.619,     &
                    -0.621,-0.624,-0.626,-0.628,-0.631,-0.633,-0.635,     &
                    -0.637,-0.640,-0.642,-0.644,-0.646,-0.648,-0.650,     &
                    -0.652,-0.653,-0.655,-0.657,-0.659,-0.661,-0.662,     &
                    -0.664,-0.666,-0.667,-0.669,-0.670,-0.672,-0.673,     &
                    -0.675,-0.676,-0.678,-0.679,-0.681,-0.682,-0.683,     &
                    -0.685,-0.686,-0.687,-0.689,-0.690,-0.691,-0.693,     &
                    -0.694,-0.695,-0.696,-0.697,-0.699,-0.700,-0.701,     &
                    -0.702,-0.703,-0.704,-0.705,-0.707,-0.708,-0.709,     &
                    -0.710,-0.711,-0.712,-0.713,-0.714,-0.715,-0.716,     &
                    -0.717,-0.718,-0.719,-0.720,-0.721,-0.722,-0.723,     &
                    -0.724,-0.724,-0.725,-0.726,-0.727,-0.728,-0.729,     &
                    -0.730,-0.731,-0.731,-0.732,-0.733,-0.734,-0.735,     &
                    -0.735,-0.736,-0.737,-0.738,-0.738,-0.739,-0.740,     &
                    -0.741,-0.741,-0.742,-0.743,-0.744,-0.744,-0.745,     &
                    -0.746,-0.746,-0.747,-0.748,-0.748,-0.749,-0.750,     &
                    -0.750,-0.751,-0.752,-0.752,-0.753,-0.754,-0.754,     &
                    -0.755,-0.755,-0.756,-0.757,-0.757,-0.758,-0.758,     &
                    -0.759,-0.759,-0.760,-0.761,-0.761,-0.762,-0.762,     &
                    -0.763,-0.763,-0.764,-0.764,-0.765,-0.765,-0.766,     &
                    -0.766,-0.767,-0.767,-0.768,-0.768,-0.769,-0.769,     &
                    -0.770,-0.770,-0.771,-0.771,-0.772,-0.772,-0.772,     &
                    -0.773,-0.773,-0.774,-0.774,-0.775,-0.775,-0.776,     &
                    -0.776,-0.776,-0.777,-0.777,-0.778,-0.778,-0.778,     &
                    -0.779,-0.779,-0.780,-0.780,-0.780,-0.781,-0.781,     &
                    -0.781,-0.782,-0.782,-0.783,-0.783,-0.783,-0.784,     &
                    -0.784,-0.784,-0.785,-0.785,-0.785,-0.786,-0.786,     &
                    -0.786,-0.787,-0.787,-0.787,-0.788,-0.788,-0.788,     &
                    -0.789,-0.789,-0.789,-0.790,-0.790,-0.790,-0.790,     &
                    -0.791,-0.791,-0.791,-0.792,-0.792,-0.792,-0.793,     &
                    -0.793,-0.793,-0.793,-0.794,-0.794,-0.794,-0.794,     &
                    -0.795,-0.795,-0.795,-0.795,-0.796,-0.796,-0.796,     &
                    -0.796,-0.797,-0.797,-0.797,-0.797,-0.798,-0.798,     &
                    -0.798,-0.798,-0.799,-0.799,-0.799,-0.799,-0.800,     &
                    -0.800,-0.800,-0.800,-0.800,-0.801,-0.801,-0.801,     &
                    -0.801,-0.801,-0.802,-0.802,-0.802,-0.802,-0.802,     &
                    -0.803,-0.803,-0.803,-0.803,-0.803,-0.804,-0.804,     &
                    -0.804,-0.804,-0.804,-0.804,-0.805,-0.805,-0.805,     &
                    -0.805,-0.805,-0.806,-0.806,-0.806,-0.806,-0.806,     &
                    -0.806,-0.806,-0.807,-0.807,-0.807,-0.807,-0.807,     &
                    -0.807,-0.808,-0.808,-0.808,-0.808,-0.808,-0.808,     &
                    -0.808,-0.809,-0.809,-0.809,-0.809,-0.809,-0.809,     &
                    -0.809,-0.810,-0.810,-0.810,-0.810,-0.810,-0.810,     &
                    -0.810,-0.810,-0.810,-0.811,-0.811,-0.811,-0.811,     &
                    -0.811,-0.811,-0.811,-0.811,-0.812,-0.812,-0.812,     &
                    -0.812,-0.812,-0.812,-0.812,-0.812,-0.812,-0.812,     &
                    -0.813,-0.813,-0.813,-0.813,-0.813,-0.813,-0.813,     &
                    -0.813,-0.813,-0.813,-0.813,-0.814,-0.814,-0.814,     &
                    -0.814,-0.814,-0.814,-0.814,-0.814,-0.814,-0.814,     &
                    -0.814,-0.814,-0.814,-0.815,-0.815,-0.815,-0.815,     &
                    -0.815,-0.815,-0.815,-0.815,-0.815,-0.815,-0.815,     &
                    -0.815,-0.816,-0.816,-0.817,-0.817,-0.817,-0.817,     &
                    -0.817,-0.817,-0.817,-0.817,-0.817,-0.817,-0.816,     &
                    -0.816,-0.816,-0.815,-0.815,-0.814,-0.813,-0.813,     &
                    -0.812,-0.811,-0.811,-0.810,-0.809,-0.808,-0.807,     &
                    -0.806,-0.805,-0.804,-0.803,-0.802,-0.801,-0.800,     &
                    -0.799,-0.797,-0.796,-0.795,-0.794,-0.792,-0.791,     &
                    -0.790,-0.788,-0.787,-0.786,-0.784,-0.783,-0.781,     &
                    -0.780,-0.778,-0.777,-0.775,-0.774,-0.772,-0.771,     &
                    -0.769,-0.767,-0.766,-0.764,-0.762,-0.761,-0.759,     &
                    -0.757,-0.756,-0.754,-0.752,-0.750,-0.749,-0.747,     &
                    -0.745,-0.743,-0.741,-0.740,-0.738,-0.736,-0.734,     &
                    -0.732,-0.730,-0.728,-0.727,-0.725,-0.723,-0.721,     &
                    -0.719,-0.717,-0.715,-0.713,-0.711,-0.709,-0.707,     &
                    -0.705,-0.703,-0.701,-0.699,-0.697,-0.695,-0.693,     &
                    -0.691,-0.689,-0.687,-0.685,-0.683,-0.681,-0.678,     &
                    -0.676,-0.674,-0.672,-0.670,-0.668,-0.666,-0.664,     &
                    -0.661,-0.659,-0.657,-0.655,-0.653,-0.651,-0.648,     &
                    -0.646,-0.644,-0.642,-0.640,-0.638,-0.635,-0.633,     &
                    -0.631,-0.629,-0.626,-0.624,-0.622,-0.620,-0.618,     &
                    -0.615,-0.613,-0.611,-0.609,-0.606,-0.604,-0.602,     &
                    -0.599,-0.597,-0.595,-0.593,-0.590,-0.588,-0.586,     &
                    -0.583,-0.581,-0.579,-0.577,-0.574,-0.572,-0.570,     &
                    -0.567,-0.565,-0.563,-0.560,-0.558,-0.556,-0.553,     &
                    -0.551/)
!
! *** khso4
!
    bnc18m(1:561) = (/-0.045,-0.094,-0.118,-0.134,-0.147,-0.156,-0.165,   &
                    -0.171,-0.177,-0.182,-0.187,-0.191,-0.195,-0.198,     &
                    -0.200,-0.203,-0.205,-0.207,-0.209,-0.210,-0.212,     &
                    -0.213,-0.214,-0.215,-0.215,-0.216,-0.216,-0.216,     &
                    -0.217,-0.217,-0.217,-0.217,-0.216,-0.216,-0.216,     &
                    -0.215,-0.214,-0.214,-0.213,-0.212,-0.211,-0.210,     &
                    -0.209,-0.208,-0.207,-0.206,-0.205,-0.204,-0.202,     &
                    -0.201,-0.199,-0.198,-0.196,-0.195,-0.193,-0.191,     &
                    -0.190,-0.188,-0.186,-0.184,-0.183,-0.181,-0.179,     &
                    -0.177,-0.175,-0.173,-0.171,-0.169,-0.167,-0.165,     &
                    -0.163,-0.160,-0.158,-0.156,-0.154,-0.152,-0.149,     &
                    -0.147,-0.145,-0.142,-0.140,-0.137,-0.135,-0.133,     &
                    -0.130,-0.128,-0.125,-0.123,-0.120,-0.117,-0.115,     &
                    -0.112,-0.109,-0.107,-0.104,-0.101,-0.099,-0.096,     &
                    -0.093,-0.090,-0.088,-0.085,-0.082,-0.079,-0.076,     &
                    -0.074,-0.071,-0.068,-0.065,-0.062,-0.059,-0.056,     &
                    -0.053,-0.050,-0.047,-0.044,-0.042,-0.039,-0.036,     &
                    -0.033,-0.030,-0.027,-0.024,-0.021,-0.018,-0.015,     &
                    -0.012,-0.009,-0.006,-0.003,0.000,0.003,0.006,0.009,  &
                    0.012,0.015,0.018,0.020,0.023,0.026,0.029,0.032,0.035,&
                    0.038,0.041,0.044,0.047,0.050,0.053,0.055,0.058,0.061,&
                    0.064,0.067,0.070,0.073,0.076,0.078,0.081,0.084,0.087,&
                    0.090,0.093,0.095,0.098,0.101,0.104,0.107,0.109,0.112,&
                    0.115,0.118,0.121,0.123,0.126,0.129,0.132,0.134,0.137,&
                    0.140,0.143,0.145,0.148,0.151,0.153,0.156,0.159,0.162,&
                    0.164,0.167,0.170,0.172,0.175,0.178,0.180,0.183,0.186,&
                    0.188,0.191,0.193,0.196,0.199,0.201,0.204,0.206,0.209,&
                    0.212,0.214,0.217,0.219,0.222,0.225,0.227,0.230,0.232,&
                    0.235,0.237,0.240,0.242,0.245,0.247,0.250,0.252,0.255,&
                    0.257,0.260,0.262,0.265,0.267,0.270,0.272,0.275,0.277,&
                    0.280,0.282,0.285,0.287,0.289,0.292,0.294,0.297,0.299,&
                    0.302,0.304,0.306,0.309,0.311,0.314,0.316,0.318,0.321,&
                    0.323,0.325,0.328,0.330,0.333,0.335,0.337,0.340,0.342,&
                    0.344,0.347,0.349,0.351,0.354,0.356,0.358,0.360,0.363,&
                    0.365,0.367,0.370,0.372,0.374,0.376,0.379,0.381,0.383,&
                    0.386,0.388,0.390,0.392,0.395,0.397,0.399,0.401,0.403,&
                    0.406,0.408,0.410,0.412,0.415,0.417,0.419,0.421,0.423,&
                    0.425,0.428,0.430,0.432,0.434,0.436,0.439,0.441,0.443,&
                    0.445,0.447,0.449,0.451,0.454,0.456,0.458,0.460,0.462,&
                    0.464,0.466,0.468,0.471,0.473,0.475,0.477,0.479,0.481,&
                    0.483,0.485,0.487,0.489,0.491,0.494,0.496,0.498,0.500,&
                    0.502,0.504,0.506,0.508,0.510,0.512,0.514,0.516,0.518,&
                    0.520,0.522,0.524,0.526,0.528,0.530,0.532,0.534,0.536,&
                    0.538,0.540,0.542,0.544,0.546,0.548,0.550,0.552,0.554,&
                    0.556,0.558,0.560,0.562,0.564,0.566,0.568,0.570,0.572,&
                    0.574,0.576,0.578,0.580,0.582,0.584,0.585,0.587,0.589,&
                    0.591,0.593,0.595,0.597,0.599,0.601,0.603,0.605,0.607,&
                    0.608,0.610,0.612,0.614,0.616,0.618,0.620,0.622,0.623,&
                    0.625,0.627,0.629,0.631,0.633,0.653,0.671,0.689,0.707,&
                    0.724,0.741,0.758,0.775,0.792,0.808,0.824,0.840,0.856,&
                    0.872,0.887,0.903,0.918,0.933,0.947,0.962,0.977,0.991,&
                    1.005,1.019,1.033,1.047,1.060,1.074,1.087,1.101,1.114,&
                    1.127,1.140,1.152,1.165,1.178,1.190,1.202,1.215,1.227,&
                    1.239,1.251,1.262,1.274,1.286,1.297,1.309,1.320,1.332,&
                    1.343,1.354,1.365,1.376,1.387,1.397,1.408,1.419,1.429,&
                    1.440,1.450,1.461,1.471,1.481,1.491,1.501,1.511,1.521,&
                    1.531,1.541,1.551,1.561,1.570,1.580,1.589,1.599,1.608,&
                    1.617,1.627,1.636,1.645,1.654,1.663,1.672,1.681,1.690,&
                    1.699,1.708,1.717,1.726,1.734,1.743,1.751,1.760,1.768,&
                    1.777,1.785,1.794,1.802,1.810,1.819,1.827,1.835,1.843,&
                    1.851,1.859,1.867,1.875,1.883,1.891,1.899,1.907,1.915,&
                    1.922,1.930,1.938,1.945,1.953,1.960,1.968,1.976,1.983,&
                    1.990,1.998,2.005,2.013,2.020,2.027,2.034,2.042,2.049,&
                    2.056,2.063,2.070,2.077,2.084,2.091,2.098,2.105,2.112,&
                    2.119,2.126,2.133,2.140,2.147,2.154,2.160,2.167,2.174,&
                    2.180,2.187,2.194,2.200,2.207,2.213,2.220,2.227,2.233,&
                    2.240,2.246,2.252,2.259/)
!
! *** kno3
!
    bnc19m(1:561) = (/-0.046,-0.105,-0.136,-0.159,-0.178,-0.194,-0.208,   &
                    -0.221,-0.233,-0.244,-0.255,-0.265,-0.274,-0.283,     &
                    -0.291,-0.299,-0.307,-0.315,-0.322,-0.329,-0.336,     &
                    -0.342,-0.349,-0.355,-0.361,-0.367,-0.373,-0.379,     &
                    -0.384,-0.390,-0.395,-0.400,-0.405,-0.410,-0.415,     &
                    -0.420,-0.425,-0.430,-0.434,-0.439,-0.443,-0.447,     &
                    -0.452,-0.456,-0.460,-0.464,-0.468,-0.472,-0.476,     &
                    -0.480,-0.483,-0.487,-0.491,-0.494,-0.498,-0.501,     &
                    -0.505,-0.508,-0.511,-0.515,-0.518,-0.521,-0.524,     &
                    -0.528,-0.531,-0.534,-0.537,-0.540,-0.543,-0.546,     &
                    -0.549,-0.552,-0.555,-0.557,-0.560,-0.563,-0.566,     &
                    -0.569,-0.571,-0.574,-0.577,-0.580,-0.582,-0.585,     &
                    -0.588,-0.590,-0.593,-0.596,-0.598,-0.601,-0.604,     &
                    -0.606,-0.609,-0.611,-0.614,-0.616,-0.619,-0.622,     &
                    -0.624,-0.627,-0.629,-0.632,-0.634,-0.637,-0.639,     &
                    -0.641,-0.644,-0.646,-0.649,-0.651,-0.654,-0.656,     &
                    -0.658,-0.661,-0.663,-0.666,-0.668,-0.670,-0.673,     &
                    -0.675,-0.677,-0.679,-0.682,-0.684,-0.686,-0.688,     &
                    -0.691,-0.693,-0.695,-0.697,-0.699,-0.702,-0.704,     &
                    -0.706,-0.708,-0.710,-0.712,-0.714,-0.716,-0.718,     &
                    -0.721,-0.723,-0.725,-0.727,-0.729,-0.731,-0.733,     &
                    -0.735,-0.737,-0.739,-0.740,-0.742,-0.744,-0.746,     &
                    -0.748,-0.750,-0.752,-0.754,-0.756,-0.757,-0.759,     &
                    -0.761,-0.763,-0.765,-0.767,-0.768,-0.770,-0.772,     &
                    -0.774,-0.775,-0.777,-0.779,-0.780,-0.782,-0.784,     &
                    -0.786,-0.787,-0.789,-0.791,-0.792,-0.794,-0.796,     &
                    -0.797,-0.799,-0.800,-0.802,-0.804,-0.805,-0.807,     &
                    -0.808,-0.810,-0.811,-0.813,-0.814,-0.816,-0.817,     &
                    -0.819,-0.820,-0.822,-0.823,-0.825,-0.826,-0.828,     &
                    -0.829,-0.831,-0.832,-0.834,-0.835,-0.836,-0.838,     &
                    -0.839,-0.841,-0.842,-0.843,-0.845,-0.846,-0.847,     &
                    -0.849,-0.850,-0.851,-0.853,-0.854,-0.855,-0.857,     &
                    -0.858,-0.859,-0.861,-0.862,-0.863,-0.864,-0.866,     &
                    -0.867,-0.868,-0.869,-0.871,-0.872,-0.873,-0.874,     &
                    -0.875,-0.877,-0.878,-0.879,-0.880,-0.881,-0.882,     &
                    -0.884,-0.885,-0.886,-0.887,-0.888,-0.889,-0.890,     &
                    -0.892,-0.893,-0.894,-0.895,-0.896,-0.897,-0.898,     &
                    -0.899,-0.900,-0.901,-0.902,-0.904,-0.905,-0.906,     &
                    -0.907,-0.908,-0.909,-0.910,-0.911,-0.912,-0.913,     &
                    -0.914,-0.915,-0.916,-0.917,-0.918,-0.919,-0.920,     &
                    -0.921,-0.922,-0.923,-0.924,-0.925,-0.926,-0.926,     &
                    -0.927,-0.928,-0.929,-0.930,-0.931,-0.932,-0.933,     &
                    -0.934,-0.935,-0.936,-0.936,-0.937,-0.938,-0.939,     &
                    -0.940,-0.941,-0.942,-0.943,-0.943,-0.944,-0.945,     &
                    -0.946,-0.947,-0.948,-0.948,-0.949,-0.950,-0.951,     &
                    -0.952,-0.953,-0.953,-0.954,-0.955,-0.956,-0.957,     &
                    -0.957,-0.958,-0.959,-0.960,-0.960,-0.961,-0.962,     &
                    -0.963,-0.963,-0.964,-0.965,-0.966,-0.966,-0.967,     &
                    -0.968,-0.969,-0.969,-0.970,-0.971,-0.971,-0.972,     &
                    -0.973,-0.974,-0.974,-0.975,-0.976,-0.976,-0.977,     &
                    -0.978,-0.978,-0.979,-0.980,-0.980,-0.981,-0.982,     &
                    -0.982,-0.983,-0.984,-0.984,-0.985,-0.986,-0.986,     &
                    -0.987,-0.988,-0.988,-0.989,-0.989,-0.990,-0.991,     &
                    -0.991,-0.992,-0.992,-0.993,-0.994,-0.994,-0.995,     &
                    -0.995,-0.996,-0.997,-0.997,-0.998,-0.998,-0.999,     &
                    -1.000,-1.000,-1.001,-1.001,-1.002,-1.002,-1.003,     &
                    -1.003,-1.004,-1.005,-1.005,-1.006,-1.006,-1.007,     &
                    -1.007,-1.013,-1.018,-1.022,-1.027,-1.031,-1.035,     &
                    -1.039,-1.042,-1.046,-1.049,-1.052,-1.055,-1.058,     &
                    -1.060,-1.063,-1.065,-1.067,-1.069,-1.071,-1.073,     &
                    -1.075,-1.076,-1.078,-1.079,-1.081,-1.082,-1.083,     &
                    -1.084,-1.085,-1.086,-1.087,-1.088,-1.088,-1.089,     &
                    -1.089,-1.090,-1.090,-1.091,-1.091,-1.091,-1.092,     &
                    -1.092,-1.092,-1.092,-1.092,-1.092,-1.092,-1.092,     &
                    -1.092,-1.092,-1.092,-1.091,-1.091,-1.091,-1.091,     &
                    -1.090,-1.090,-1.090,-1.089,-1.089,-1.088,-1.088,     &
                    -1.087,-1.087,-1.086,-1.085,-1.085,-1.084,-1.084,     &
                    -1.083,-1.082,-1.082,-1.081,-1.080,-1.079,-1.079,     &
                    -1.078,-1.077,-1.076,-1.075,-1.074,-1.074,-1.073,     &
                    -1.072,-1.071,-1.070,-1.069,-1.068,-1.067,-1.066,     &
                    -1.065,-1.064,-1.063,-1.062,-1.061,-1.060,-1.059,     &
                    -1.058,-1.057,-1.056,-1.055,-1.054,-1.053,-1.052,     &
                    -1.051,-1.050,-1.049,-1.047,-1.046,-1.045,-1.044,     &
                    -1.043,-1.042,-1.041,-1.040,-1.038,-1.037,-1.036,     &
                    -1.035,-1.034,-1.032,-1.031,-1.030,-1.029,-1.028,     &
                    -1.026,-1.025,-1.024,-1.023,-1.022,-1.020,-1.019,     &
                    -1.018,-1.017,-1.015,-1.014,-1.013,-1.012,-1.010,     &
                    -1.009,-1.008,-1.007,-1.005,-1.004,-1.003,-1.001,     &
                    -1.000,-0.999,-0.998,-0.996,-0.995,-0.994,-0.992,     &
                    -0.991,-0.990,-0.989,-0.987,-0.986,-0.985,-0.983,     &
                    -0.982/)
!
! *** kcl
!
    bnc20m(1:561) = (/-0.045,-0.095,-0.119,-0.136,-0.148,-0.158,-0.166,   &
                    -0.173,-0.179,-0.184,-0.189,-0.193,-0.197,-0.200,     &
                    -0.203,-0.206,-0.208,-0.210,-0.212,-0.214,-0.216,     &
                    -0.217,-0.219,-0.220,-0.221,-0.222,-0.223,-0.224,     &
                    -0.225,-0.226,-0.227,-0.227,-0.228,-0.229,-0.229,     &
                    -0.229,-0.230,-0.230,-0.231,-0.231,-0.231,-0.231,     &
                    -0.232,-0.232,-0.232,-0.232,-0.232,-0.232,-0.232,     &
                    -0.232,-0.233,-0.233,-0.233,-0.232,-0.232,-0.232,     &
                    -0.232,-0.232,-0.232,-0.232,-0.232,-0.232,-0.232,     &
                    -0.232,-0.231,-0.231,-0.231,-0.231,-0.231,-0.230,     &
                    -0.230,-0.230,-0.230,-0.229,-0.229,-0.229,-0.229,     &
                    -0.228,-0.228,-0.228,-0.227,-0.227,-0.227,-0.226,     &
                    -0.226,-0.226,-0.225,-0.225,-0.224,-0.224,-0.224,     &
                    -0.223,-0.223,-0.222,-0.222,-0.221,-0.221,-0.220,     &
                    -0.220,-0.219,-0.219,-0.218,-0.218,-0.217,-0.217,     &
                    -0.216,-0.216,-0.215,-0.215,-0.214,-0.213,-0.213,     &
                    -0.212,-0.212,-0.211,-0.211,-0.210,-0.209,-0.209,     &
                    -0.208,-0.208,-0.207,-0.206,-0.206,-0.205,-0.205,     &
                    -0.204,-0.203,-0.203,-0.202,-0.201,-0.201,-0.200,     &
                    -0.200,-0.199,-0.198,-0.198,-0.197,-0.196,-0.196,     &
                    -0.195,-0.194,-0.194,-0.193,-0.192,-0.192,-0.191,     &
                    -0.191,-0.190,-0.189,-0.189,-0.188,-0.187,-0.187,     &
                    -0.186,-0.185,-0.185,-0.184,-0.183,-0.183,-0.182,     &
                    -0.181,-0.181,-0.180,-0.179,-0.179,-0.178,-0.177,     &
                    -0.177,-0.176,-0.175,-0.175,-0.174,-0.173,-0.173,     &
                    -0.172,-0.171,-0.171,-0.170,-0.169,-0.169,-0.168,     &
                    -0.167,-0.167,-0.166,-0.165,-0.165,-0.164,-0.163,     &
                    -0.163,-0.162,-0.161,-0.161,-0.160,-0.159,-0.159,     &
                    -0.158,-0.157,-0.157,-0.156,-0.155,-0.155,-0.154,     &
                    -0.153,-0.153,-0.152,-0.151,-0.151,-0.150,-0.149,     &
                    -0.149,-0.148,-0.147,-0.147,-0.146,-0.145,-0.145,     &
                    -0.144,-0.143,-0.143,-0.142,-0.141,-0.141,-0.140,     &
                    -0.140,-0.139,-0.138,-0.138,-0.137,-0.136,-0.136,     &
                    -0.135,-0.134,-0.134,-0.133,-0.132,-0.132,-0.131,     &
                    -0.130,-0.130,-0.129,-0.128,-0.128,-0.127,-0.126,     &
                    -0.126,-0.125,-0.124,-0.124,-0.123,-0.122,-0.122,     &
                    -0.121,-0.120,-0.120,-0.119,-0.119,-0.118,-0.117,     &
                    -0.117,-0.116,-0.115,-0.115,-0.114,-0.113,-0.113,     &
                    -0.112,-0.111,-0.111,-0.110,-0.109,-0.109,-0.108,     &
                    -0.108,-0.107,-0.106,-0.106,-0.105,-0.104,-0.104,     &
                    -0.103,-0.102,-0.102,-0.101,-0.100,-0.100,-0.099,     &
                    -0.099,-0.098,-0.097,-0.097,-0.096,-0.095,-0.095,     &
                    -0.094,-0.093,-0.093,-0.092,-0.092,-0.091,-0.090,     &
                    -0.090,-0.089,-0.088,-0.088,-0.087,-0.087,-0.086,     &
                    -0.085,-0.085,-0.084,-0.083,-0.083,-0.082,-0.082,     &
                    -0.081,-0.080,-0.080,-0.079,-0.078,-0.078,-0.077,     &
                    -0.077,-0.076,-0.075,-0.075,-0.074,-0.073,-0.073,     &
                    -0.072,-0.072,-0.071,-0.070,-0.070,-0.069,-0.069,     &
                    -0.068,-0.067,-0.067,-0.066,-0.065,-0.065,-0.064,     &
                    -0.064,-0.063,-0.062,-0.062,-0.061,-0.061,-0.060,     &
                    -0.059,-0.059,-0.058,-0.058,-0.057,-0.056,-0.056,     &
                    -0.055,-0.055,-0.054,-0.053,-0.053,-0.052,-0.051,     &
                    -0.051,-0.050,-0.050,-0.049,-0.048,-0.048,-0.047,     &
                    -0.047,-0.046,-0.046,-0.045,-0.044,-0.044,-0.043,     &
                    -0.043,-0.042,-0.041,-0.041,-0.040,-0.040,-0.039,     &
                    -0.038,-0.038,-0.037,-0.037,-0.036,-0.035,-0.035,     &
                    -0.034,-0.034,-0.033,-0.032,-0.032,-0.031,-0.031,     &
                    -0.030,-0.024,-0.018,-0.012,-0.007,-0.001,0.005,0.010,&
                    0.016,0.021,0.027,0.032,0.037,0.043,0.048,0.053,0.058,&
                    0.064,0.069,0.074,0.079,0.084,0.089,0.094,0.099,0.104,&
                    0.109,0.113,0.118,0.123,0.128,0.132,0.137,0.142,0.146,&
                    0.151,0.156,0.160,0.165,0.169,0.174,0.178,0.183,0.187,&
                    0.191,0.196,0.200,0.205,0.209,0.213,0.217,0.222,0.226,&
                    0.230,0.234,0.238,0.242,0.247,0.251,0.255,0.259,0.263,&
                    0.267,0.271,0.275,0.279,0.283,0.287,0.291,0.295,0.298,&
                    0.302,0.306,0.310,0.314,0.318,0.321,0.325,0.329,0.333,&
                    0.337,0.340,0.344,0.348,0.351,0.355,0.359,0.362,0.366,&
                    0.370,0.373,0.377,0.380,0.384,0.387,0.391,0.394,0.398,&
                    0.401,0.405,0.408,0.412,0.415,0.419,0.422,0.426,0.429,&
                    0.433,0.436,0.439,0.443,0.446,0.449,0.453,0.456,0.459,&
                    0.463,0.466,0.469,0.473,0.476,0.479,0.482,0.486,0.489,&
                    0.492,0.495,0.499,0.502,0.505,0.508,0.511,0.514,0.518,&
                    0.521,0.524,0.527,0.530,0.533,0.536,0.540,0.543,0.546,&
                    0.549,0.552,0.555,0.558,0.561,0.564,0.567,0.570,0.573,&
                    0.576,0.579,0.582,0.585,0.588,0.591,0.594,0.597,0.600,&
                    0.603/)
!
! *** mgso4
!
    bnc21m(1:561) = (/-0.181,-0.389,-0.491,-0.562,-0.617,-0.661,-0.699,   &
                    -0.732,-0.760,-0.786,-0.809,-0.829,-0.849,-0.866,     &
                    -0.882,-0.897,-0.911,-0.924,-0.937,-0.948,-0.959,     &
                    -0.969,-0.979,-0.988,-0.997,-1.006,-1.014,-1.021,     &
                    -1.028,-1.035,-1.042,-1.048,-1.055,-1.061,-1.066,     &
                    -1.072,-1.077,-1.082,-1.087,-1.092,-1.096,-1.100,     &
                    -1.105,-1.109,-1.113,-1.117,-1.120,-1.124,-1.128,     &
                    -1.131,-1.134,-1.137,-1.141,-1.144,-1.146,-1.149,     &
                    -1.152,-1.155,-1.157,-1.160,-1.162,-1.165,-1.167,     &
                    -1.169,-1.172,-1.174,-1.176,-1.178,-1.180,-1.182,     &
                    -1.184,-1.186,-1.187,-1.189,-1.191,-1.192,-1.194,     &
                    -1.196,-1.197,-1.199,-1.200,-1.202,-1.203,-1.204,     &
                    -1.206,-1.207,-1.208,-1.209,-1.210,-1.211,-1.213,     &
                    -1.214,-1.215,-1.216,-1.217,-1.218,-1.218,-1.219,     &
                    -1.220,-1.221,-1.222,-1.223,-1.223,-1.224,-1.225,     &
                    -1.225,-1.226,-1.227,-1.227,-1.228,-1.229,-1.229,     &
                    -1.230,-1.230,-1.231,-1.231,-1.232,-1.232,-1.233,     &
                    -1.233,-1.233,-1.234,-1.234,-1.235,-1.235,-1.235,     &
                    -1.236,-1.236,-1.236,-1.236,-1.237,-1.237,-1.237,     &
                    -1.237,-1.238,-1.238,-1.238,-1.238,-1.238,-1.239,     &
                    -1.239,-1.239,-1.239,-1.239,-1.239,-1.239,-1.239,     &
                    -1.239,-1.239,-1.240,-1.240,-1.240,-1.240,-1.240,     &
                    -1.240,-1.240,-1.240,-1.240,-1.240,-1.240,-1.240,     &
                    -1.240,-1.239,-1.239,-1.239,-1.239,-1.239,-1.239,     &
                    -1.239,-1.239,-1.239,-1.239,-1.239,-1.238,-1.238,     &
                    -1.238,-1.238,-1.238,-1.238,-1.238,-1.237,-1.237,     &
                    -1.237,-1.237,-1.237,-1.237,-1.236,-1.236,-1.236,     &
                    -1.236,-1.235,-1.235,-1.235,-1.235,-1.235,-1.234,     &
                    -1.234,-1.234,-1.233,-1.233,-1.233,-1.233,-1.232,     &
                    -1.232,-1.232,-1.232,-1.231,-1.231,-1.231,-1.230,     &
                    -1.230,-1.230,-1.229,-1.229,-1.229,-1.228,-1.228,     &
                    -1.228,-1.227,-1.227,-1.227,-1.226,-1.226,-1.226,     &
                    -1.225,-1.225,-1.225,-1.224,-1.224,-1.223,-1.223,     &
                    -1.223,-1.222,-1.222,-1.222,-1.221,-1.221,-1.220,     &
                    -1.220,-1.220,-1.219,-1.219,-1.218,-1.218,-1.217,     &
                    -1.217,-1.217,-1.216,-1.216,-1.215,-1.215,-1.214,     &
                    -1.214,-1.214,-1.213,-1.213,-1.212,-1.212,-1.211,     &
                    -1.211,-1.210,-1.210,-1.210,-1.209,-1.209,-1.208,     &
                    -1.208,-1.207,-1.207,-1.206,-1.206,-1.205,-1.205,     &
                    -1.204,-1.204,-1.203,-1.203,-1.202,-1.202,-1.201,     &
                    -1.201,-1.200,-1.200,-1.199,-1.199,-1.198,-1.198,     &
                    -1.197,-1.197,-1.196,-1.196,-1.195,-1.195,-1.194,     &
                    -1.194,-1.193,-1.193,-1.192,-1.192,-1.191,-1.191,     &
                    -1.190,-1.190,-1.189,-1.188,-1.188,-1.187,-1.187,     &
                    -1.186,-1.186,-1.185,-1.185,-1.184,-1.184,-1.183,     &
                    -1.182,-1.182,-1.181,-1.181,-1.180,-1.180,-1.179,     &
                    -1.179,-1.178,-1.177,-1.177,-1.176,-1.176,-1.175,     &
                    -1.175,-1.174,-1.174,-1.173,-1.172,-1.172,-1.171,     &
                    -1.171,-1.170,-1.169,-1.169,-1.168,-1.168,-1.167,     &
                    -1.167,-1.166,-1.165,-1.165,-1.164,-1.164,-1.163,     &
                    -1.163,-1.162,-1.161,-1.161,-1.160,-1.160,-1.159,     &
                    -1.158,-1.158,-1.157,-1.157,-1.156,-1.155,-1.155,     &
                    -1.154,-1.154,-1.153,-1.152,-1.152,-1.151,-1.151,     &
                    -1.150,-1.149,-1.149,-1.148,-1.148,-1.147,-1.146,     &
                    -1.146,-1.145,-1.144,-1.144,-1.143,-1.143,-1.142,     &
                    -1.141,-1.141,-1.140,-1.140,-1.139,-1.138,-1.138,     &
                    -1.137,-1.136,-1.136,-1.135,-1.135,-1.134,-1.133,     &
                    -1.133,-1.126,-1.120,-1.113,-1.107,-1.100,-1.094,     &
                    -1.087,-1.080,-1.074,-1.067,-1.060,-1.054,-1.047,     &
                    -1.040,-1.033,-1.027,-1.020,-1.013,-1.006,-0.999,     &
                    -0.992,-0.985,-0.978,-0.971,-0.965,-0.958,-0.951,     &
                    -0.944,-0.937,-0.930,-0.923,-0.916,-0.909,-0.902,     &
                    -0.895,-0.888,-0.881,-0.874,-0.867,-0.860,-0.853,     &
                    -0.846,-0.839,-0.832,-0.825,-0.818,-0.811,-0.804,     &
                    -0.797,-0.790,-0.783,-0.776,-0.769,-0.762,-0.755,     &
                    -0.748,-0.741,-0.734,-0.727,-0.720,-0.713,-0.706,     &
                    -0.699,-0.692,-0.685,-0.678,-0.671,-0.664,-0.657,     &
                    -0.650,-0.644,-0.637,-0.630,-0.623,-0.616,-0.609,     &
                    -0.602,-0.595,-0.588,-0.581,-0.574,-0.567,-0.560,     &
                    -0.554,-0.547,-0.540,-0.533,-0.526,-0.519,-0.512,     &
                    -0.505,-0.498,-0.492,-0.485,-0.478,-0.471,-0.464,     &
                    -0.457,-0.450,-0.444,-0.437,-0.430,-0.423,-0.416,     &
                    -0.409,-0.403,-0.396,-0.389,-0.382,-0.375,-0.369,     &
                    -0.362,-0.355,-0.348,-0.341,-0.335,-0.328,-0.321,     &
                    -0.314,-0.308,-0.301,-0.294,-0.287,-0.280,-0.274,     &
                    -0.267,-0.260,-0.254,-0.247,-0.240,-0.233,-0.227,     &
                    -0.220,-0.213,-0.206,-0.200,-0.193,-0.186,-0.180,     &
                    -0.173,-0.166,-0.160,-0.153,-0.146,-0.140,-0.133,     &
                    -0.126,-0.119,-0.113,-0.106,-0.100,-0.093,-0.086,     &
                    -0.080,-0.073,-0.066,-0.060,-0.053,-0.046,-0.040,     &
                    -0.033/)
!
! *** mgno32
!
    bnc22m(1:561) = (/-0.088,-0.185,-0.228,-0.257,-0.278,-0.294,-0.306,   &
                    -0.317,-0.325,-0.332,-0.337,-0.342,-0.346,-0.349,     &
                    -0.351,-0.353,-0.355,-0.356,-0.357,-0.357,-0.357,     &
                    -0.357,-0.356,-0.356,-0.355,-0.354,-0.353,-0.352,     &
                    -0.350,-0.349,-0.347,-0.345,-0.343,-0.341,-0.339,     &
                    -0.337,-0.335,-0.333,-0.330,-0.328,-0.326,-0.323,     &
                    -0.321,-0.318,-0.316,-0.313,-0.310,-0.308,-0.305,     &
                    -0.302,-0.300,-0.297,-0.294,-0.291,-0.288,-0.286,     &
                    -0.283,-0.280,-0.277,-0.274,-0.271,-0.268,-0.266,     &
                    -0.263,-0.260,-0.257,-0.254,-0.251,-0.248,-0.245,     &
                    -0.242,-0.239,-0.235,-0.232,-0.229,-0.226,-0.223,     &
                    -0.220,-0.216,-0.213,-0.210,-0.207,-0.203,-0.200,     &
                    -0.197,-0.193,-0.190,-0.186,-0.183,-0.180,-0.176,     &
                    -0.173,-0.169,-0.165,-0.162,-0.158,-0.155,-0.151,     &
                    -0.147,-0.143,-0.140,-0.136,-0.132,-0.128,-0.125,     &
                    -0.121,-0.117,-0.113,-0.109,-0.105,-0.101,-0.098,     &
                    -0.094,-0.090,-0.086,-0.082,-0.078,-0.074,-0.070,     &
                    -0.066,-0.062,-0.058,-0.054,-0.050,-0.046,-0.042,     &
                    -0.038,-0.034,-0.030,-0.026,-0.022,-0.018,-0.014,     &
                    -0.010,-0.006,-0.002,0.002,0.006,0.010,0.014,0.018,   &
                    0.022,0.026,0.030,0.034,0.038,0.042,0.046,0.050,0.054,&
                    0.058,0.062,0.066,0.070,0.074,0.078,0.082,0.086,0.090,&
                    0.094,0.098,0.102,0.106,0.110,0.114,0.118,0.122,0.126,&
                    0.130,0.134,0.137,0.141,0.145,0.149,0.153,0.157,0.161,&
                    0.165,0.169,0.173,0.177,0.180,0.184,0.188,0.192,0.196,&
                    0.200,0.204,0.207,0.211,0.215,0.219,0.223,0.227,0.231,&
                    0.234,0.238,0.242,0.246,0.250,0.253,0.257,0.261,0.265,&
                    0.269,0.272,0.276,0.280,0.284,0.288,0.291,0.295,0.299,&
                    0.303,0.306,0.310,0.314,0.318,0.321,0.325,0.329,0.332,&
                    0.336,0.340,0.343,0.347,0.351,0.355,0.358,0.362,0.366,&
                    0.369,0.373,0.377,0.380,0.384,0.388,0.391,0.395,0.398,&
                    0.402,0.406,0.409,0.413,0.417,0.420,0.424,0.427,0.431,&
                    0.435,0.438,0.442,0.445,0.449,0.452,0.456,0.459,0.463,&
                    0.467,0.470,0.474,0.477,0.481,0.484,0.488,0.491,0.495,&
                    0.498,0.502,0.505,0.509,0.512,0.516,0.519,0.523,0.526,&
                    0.530,0.533,0.537,0.540,0.543,0.547,0.550,0.554,0.557,&
                    0.561,0.564,0.567,0.571,0.574,0.578,0.581,0.584,0.588,&
                    0.591,0.595,0.598,0.601,0.605,0.608,0.611,0.615,0.618,&
                    0.621,0.625,0.628,0.631,0.635,0.638,0.641,0.645,0.648,&
                    0.651,0.655,0.658,0.661,0.665,0.668,0.671,0.674,0.678,&
                    0.681,0.684,0.687,0.691,0.694,0.697,0.700,0.704,0.707,&
                    0.710,0.713,0.717,0.720,0.723,0.726,0.729,0.733,0.736,&
                    0.739,0.742,0.745,0.749,0.752,0.755,0.758,0.761,0.764,&
                    0.768,0.771,0.774,0.777,0.780,0.783,0.786,0.790,0.793,&
                    0.796,0.799,0.802,0.805,0.808,0.811,0.814,0.818,0.821,&
                    0.824,0.827,0.830,0.833,0.836,0.839,0.842,0.845,0.848,&
                    0.851,0.854,0.857,0.860,0.864,0.867,0.870,0.873,0.876,&
                    0.879,0.882,0.885,0.888,0.891,0.894,0.897,0.900,0.903,&
                    0.906,0.909,0.912,0.915,0.918,0.921,0.924,0.955,0.984,&
                    1.013,1.041,1.070,1.097,1.125,1.152,1.179,1.205,1.231,&
                    1.257,1.283,1.308,1.334,1.359,1.383,1.408,1.432,1.456,&
                    1.479,1.503,1.526,1.549,1.572,1.594,1.617,1.639,1.661,&
                    1.683,1.704,1.726,1.747,1.768,1.789,1.810,1.830,1.851,&
                    1.871,1.891,1.911,1.930,1.950,1.969,1.989,2.008,2.027,&
                    2.046,2.064,2.083,2.101,2.120,2.138,2.156,2.174,2.192,&
                    2.209,2.227,2.244,2.262,2.279,2.296,2.313,2.330,2.347,&
                    2.363,2.380,2.396,2.413,2.429,2.445,2.461,2.477,2.493,&
                    2.509,2.525,2.540,2.556,2.571,2.587,2.602,2.617,2.632,&
                    2.647,2.662,2.677,2.692,2.707,2.721,2.736,2.750,2.765,&
                    2.779,2.793,2.807,2.822,2.836,2.850,2.863,2.877,2.891,&
                    2.905,2.918,2.932,2.946,2.959,2.972,2.986,2.999,3.012,&
                    3.025,3.039,3.052,3.065,3.078,3.090,3.103,3.116,3.129,&
                    3.141,3.154,3.167,3.179,3.191,3.204,3.216,3.229,3.241,&
                    3.253,3.265,3.277,3.289,3.301,3.313,3.325,3.337,3.349,&
                    3.361,3.372,3.384,3.396,3.407,3.419,3.431,3.442,3.453,&
                    3.465,3.476,3.488,3.499,3.510,3.521,3.532,3.544,3.555,&
                    3.566,3.577,3.588,3.599,3.610,3.621/)
!
! *** mgcl2
!
    bnc23m(1:561) = (/-0.088,-0.182,-0.225,-0.252,-0.271,-0.286,-0.297,   &
                    -0.306,-0.313,-0.319,-0.323,-0.327,-0.329,-0.331,     &
                    -0.332,-0.333,-0.333,-0.333,-0.333,-0.332,-0.331,     &
                    -0.329,-0.328,-0.326,-0.324,-0.322,-0.319,-0.317,     &
                    -0.314,-0.312,-0.309,-0.306,-0.303,-0.300,-0.296,     &
                    -0.293,-0.290,-0.286,-0.283,-0.279,-0.276,-0.272,     &
                    -0.268,-0.265,-0.261,-0.257,-0.253,-0.250,-0.246,     &
                    -0.242,-0.238,-0.234,-0.230,-0.226,-0.223,-0.219,     &
                    -0.215,-0.211,-0.207,-0.203,-0.199,-0.195,-0.191,     &
                    -0.187,-0.183,-0.179,-0.174,-0.170,-0.166,-0.162,     &
                    -0.158,-0.154,-0.150,-0.145,-0.141,-0.137,-0.133,     &
                    -0.128,-0.124,-0.120,-0.115,-0.111,-0.106,-0.102,     &
                    -0.097,-0.093,-0.088,-0.084,-0.079,-0.074,-0.070,     &
                    -0.065,-0.060,-0.056,-0.051,-0.046,-0.041,-0.036,     &
                    -0.031,-0.027,-0.022,-0.017,-0.012,-0.007,-0.002,     &
                    0.003,0.008,0.013,0.019,0.024,0.029,0.034,0.039,0.044,&
                    0.049,0.055,0.060,0.065,0.070,0.075,0.081,0.086,0.091,&
                    0.096,0.101,0.107,0.112,0.117,0.122,0.128,0.133,0.138,&
                    0.143,0.149,0.154,0.159,0.164,0.170,0.175,0.180,0.185,&
                    0.190,0.196,0.201,0.206,0.211,0.216,0.222,0.227,0.232,&
                    0.237,0.242,0.248,0.253,0.258,0.263,0.268,0.273,0.279,&
                    0.284,0.289,0.294,0.299,0.304,0.309,0.314,0.320,0.325,&
                    0.330,0.335,0.340,0.345,0.350,0.355,0.360,0.365,0.370,&
                    0.375,0.380,0.386,0.391,0.396,0.401,0.406,0.411,0.416,&
                    0.421,0.426,0.431,0.436,0.441,0.446,0.450,0.455,0.460,&
                    0.465,0.470,0.475,0.480,0.485,0.490,0.495,0.500,0.505,&
                    0.509,0.514,0.519,0.524,0.529,0.534,0.539,0.543,0.548,&
                    0.553,0.558,0.563,0.568,0.572,0.577,0.582,0.587,0.592,&
                    0.596,0.601,0.606,0.611,0.615,0.620,0.625,0.629,0.634,&
                    0.639,0.644,0.648,0.653,0.658,0.662,0.667,0.672,0.676,&
                    0.681,0.686,0.690,0.695,0.700,0.704,0.709,0.713,0.718,&
                    0.723,0.727,0.732,0.736,0.741,0.745,0.750,0.754,0.759,&
                    0.764,0.768,0.773,0.777,0.782,0.786,0.791,0.795,0.800,&
                    0.804,0.809,0.813,0.817,0.822,0.826,0.831,0.835,0.840,&
                    0.844,0.848,0.853,0.857,0.862,0.866,0.870,0.875,0.879,&
                    0.884,0.888,0.892,0.897,0.901,0.905,0.910,0.914,0.918,&
                    0.923,0.927,0.931,0.935,0.940,0.944,0.948,0.953,0.957,&
                    0.961,0.965,0.970,0.974,0.978,0.982,0.986,0.991,0.995,&
                    0.999,1.003,1.007,1.012,1.016,1.020,1.024,1.028,1.032,&
                    1.037,1.041,1.045,1.049,1.053,1.057,1.061,1.065,1.070,&
                    1.074,1.078,1.082,1.086,1.090,1.094,1.098,1.102,1.106,&
                    1.110,1.114,1.118,1.122,1.126,1.130,1.134,1.138,1.142,&
                    1.146,1.150,1.154,1.158,1.162,1.166,1.170,1.174,1.178,&
                    1.182,1.186,1.190,1.194,1.198,1.202,1.206,1.210,1.214,&
                    1.217,1.221,1.225,1.229,1.233,1.237,1.241,1.245,1.248,&
                    1.252,1.256,1.260,1.264,1.268,1.271,1.275,1.279,1.283,&
                    1.287,1.290,1.294,1.298,1.302,1.306,1.309,1.313,1.317,&
                    1.321,1.324,1.328,1.332,1.336,1.339,1.343,1.383,1.420,&
                    1.456,1.491,1.527,1.561,1.596,1.630,1.663,1.696,1.729,&
                    1.761,1.793,1.825,1.856,1.887,1.918,1.948,1.978,2.008,&
                    2.037,2.066,2.095,2.123,2.152,2.180,2.207,2.235,2.262,&
                    2.289,2.315,2.342,2.368,2.394,2.419,2.445,2.470,2.495,&
                    2.520,2.544,2.569,2.593,2.617,2.641,2.664,2.688,2.711,&
                    2.734,2.757,2.780,2.802,2.824,2.847,2.869,2.890,2.912,&
                    2.934,2.955,2.976,2.997,3.018,3.039,3.060,3.080,3.101,&
                    3.121,3.141,3.161,3.181,3.201,3.220,3.240,3.259,3.279,&
                    3.298,3.317,3.336,3.354,3.373,3.392,3.410,3.428,3.447,&
                    3.465,3.483,3.501,3.519,3.536,3.554,3.572,3.589,3.606,&
                    3.624,3.641,3.658,3.675,3.692,3.709,3.725,3.742,3.759,&
                    3.775,3.791,3.808,3.824,3.840,3.856,3.872,3.888,3.904,&
                    3.920,3.935,3.951,3.967,3.982,3.997,4.013,4.028,4.043,&
                    4.058,4.074,4.089,4.103,4.118,4.133,4.148,4.163,4.177,&
                    4.192,4.206,4.221,4.235,4.249,4.264,4.278,4.292,4.306,&
                    4.320,4.334,4.348,4.362,4.376,4.390,4.403,4.417,4.431,&
                    4.444,4.458,4.471,4.484,4.498,4.511,4.524,4.538,4.551,&
                    4.564,4.577,4.590,4.603,4.616,4.629/)
!
! *** find position in arrays for bincoef
!
    if ( ionic<=0.200000D+02 ) then
      ipos = min(nint(0.200000D+02*ionic)+1,400)
    else
      ipos = 400 + nint(0.200000D+01*ionic-0.400000D+02)
    end if
    ipos = min(ipos,561)
!
! *** assign values to return array
!
    binarr(01) = bnc01m(ipos)
    binarr(02) = bnc02m(ipos)
    binarr(03) = bnc03m(ipos)
    binarr(04) = bnc04m(ipos)
    binarr(05) = bnc05m(ipos)
    binarr(06) = bnc06m(ipos)
    binarr(07) = bnc07m(ipos)
    binarr(08) = bnc08m(ipos)
    binarr(09) = bnc09m(ipos)
    binarr(10) = bnc10m(ipos)
    binarr(11) = bnc11m(ipos)
    binarr(12) = bnc12m(ipos)
    binarr(13) = bnc13m(ipos)
    binarr(14) = bnc14m(ipos)
    binarr(15) = bnc15m(ipos)
    binarr(16) = bnc16m(ipos)
    binarr(17) = bnc17m(ipos)
    binarr(18) = bnc18m(ipos)
    binarr(19) = bnc19m(ipos)
    binarr(20) = bnc20m(ipos)
    binarr(21) = bnc21m(ipos)
    binarr(22) = bnc22m(ipos)
    binarr(23) = bnc23m(ipos)
  end subroutine km323

!************************************************************************
!
!  tolbox library v.1.0 (may 1995)
!
!  pogram unit   : subroutine chrbln &
!  prpose        : position of last non-blank character in a string &
!  athor         : athanasios nenes
!
!  ====================== arguments / usage =============================
!
!  sr        is the character variable containing the string examined &
!  ilk       is a integer(ik4) :: variable containing the position of last non
!             blank character. if string is all spaces (ie '   '), then
!             the value returned is 1.
!
!  eample:
!             str = 'test1.dat     '
!             call chrbln (str , iblk)
!
!  ater execution of this code segment , "iblk" has the value "9", which &
!  i the position of the last non-blank character of "str".
!
!**********************************************************************
!
  subroutine chrbln(str,iblk)
    implicit none
    integer(ik4) :: i , ilen
    character(len=*) str
    integer(ik4) :: iblk
    iblk = 1                         ! substring pointer (default= 1)
    ilen = len(str)                  ! length of string
    do i = ilen , 1 , -1
      if ( str(i:i)/=' ' .and. str(i:i)/=char(0) ) then
        iblk = i
        return
      end if
    end do
  end subroutine chrbln

!************************************************************************
!
!  tolbox library v.1.0 (may 1995)
!
!  pogram unit   : subroutine shftrght &
!  prpose        : right-justification function on a string &
!  athor         : athanasios nenes
!
!  ====================== arguments / usage =============================
!
!  sring     is the character variable with the string to be justified
!
!  eample:
!             string    = 'aaaa    '
!             call shftrght (string)
!
!  ater execution of this code segment , string contains the value
!  '    aaaa'.
!
!************************************************************************
!
  subroutine shftrght(chr)
    implicit none
    integer(ik4) :: i , i1 , i2
    character(len=*) :: chr
    i1 = len(chr)                ! total length of string
    call chrbln(chr,i2)           ! position of last non-blank character
    if ( i2==i1 ) return
    do i = i2 , 1 , -1            ! shift characters
      chr(i1+i-i2:i1+i-i2) = chr(i:i)
      chr(i:i) = ' '
    end do
  end subroutine shftrght

!************************************************************************
!
!  tolbox library v.1.0 (may 1995)
!
!  pogram unit   : subroutine rplstr &
!  prpose        : replace characters occuring in a string &
!  athor         : athanasios nenes
!
!  ====================== arguments / usage =============================
!
!  sring     is the character variable with the string to be edited &
!  od        is the old character which is to be replaced &
!  nw        is the new character which old is to be replaced with &
!  irr       is 0 if everything went well , is 1 if 'new' contains 'old'.
!             in this case , this is invalid , and no change is done.
!
!  eample:
!             string    = 'aaaa'
!             old       = 'a'
!             new       = 'b'
!             call rplstr (string , old , new)
!
!  ater execution of this code segment , string contains the value &
!  'bbb'.
!
!************************************************************************
!
  subroutine rplstr(string,old,new,ierr)
    implicit none
    integer(ik4) :: ilo , ip
    character(len=*) :: string , old , new
    integer(ik4) :: ierr
!
! *** initialize
! ********************************************************
    ilo = len(old)
!
! *** check and see if 'new' contains 'old', which cannot
! ***************
    ip = index(new,old)
    if ( ip/=0 ) then
      ierr = 1
      return
    else
      ierr = 0
    end if
!
! *** proceed with replacing *******************************************
!
    do
      ip = index(string,old)         ! see if 'old' exists in 'string'
      if ( ip==0 ) return           ! 'old' does not exist ; return
      string(ip:ip+ilo-1) = new     ! replace substring 'old' with 'new'
    end do
  end subroutine rplstr

!************************************************************************
!
!  tolbox library v.1.0 (may 1995)
!
!  pogram unit   : subroutine inptd &
!  prpose        : prompts user for a value (double). a default value
!                   is provided , so if user presses <enter>, the default
!                   is used. &
!  athor         : athanasios nenes
!
!  ====================== arguments / usage =============================
!
!  vr        is the real(rkx) variable which value is to be saved &
!  df        is a real(rkx) variable , with the default value of var. &
!  pompt     is a character varible containing the prompt string. &
!  pfmt      is a character variable containing the format specifier
!             for the default value def. &
!  irr       is an integer(ik4) :: error flag , and has the values:
!             0 - no error detected.
!             1 - invalid format and/or invalid default value.
!             2 - bad value specified by user
!
!  eample:
!             call inptd (var , 1.0d0 , 'give value for a ', '*', ierr)
!
!  ater execution of this code segment , the user is prompted for the &
!  vlue of variable var. if <enter> is pressed (ie no value is specified) &
!  ten 1.0 is assigned to var. the default value is displayed in free- &
!  frmat. the error status is specified by variable ierr
!
!**********************************************************************
!
  subroutine inptd(var,def,prompt,prfmt,ierr)
    implicit none
    integer(ik4) :: iend
    character(len=*) :: prompt , prfmt
    character(len=128) :: buffer
    real(rkx) :: def , var
    integer(ik4) :: ierr , ires
!
    ierr = 0
!
! *** write default value to work buffer *******************************
!
    write (buffer,fmt = prfmt,iostat=ires) def
    if ( ires /= 0 ) then
      ierr = 1       ! bad format and/or bad default value
      return
    end if
    call chrbln(buffer,iend)
!
! *** prompt user for input and read it ********************************
!
    write (*,*) prompt , ' [' , buffer(1:iend) , ']: '
    read (*,'(a)',iostat=ires) buffer
    if ( ires /= 0 ) then
      ierr = 2       ! bad number given by user
      return
    end if
    call chrbln(buffer,iend)
!
! *** read data or set default ?
! ****************************************
    if ( iend==1 .and. buffer(1:1)==' ' ) then
      var = def
    else
      read (buffer,*,iostat=ires) var
      if ( ires /= 0 ) then
        ierr = 2       ! bad number given by user
        return
      end if
    end if
  end subroutine inptd
!
!************************************************************************
!
!  tolbox library v.1.0 (may 1995)
!
!  pogram unit   : subroutine pushend &
!  prpose        : positions the pointer of a sequential file at its end
!                   simulates the access ='append' clause of a f77l open
!                   statement with standard fortran commands.
!
!  ====================== arguments / usage =============================
!
!  init      is a integer(ik4) :: variable , the file unit which the file is
!             connected to.
!
!  eample:
!             call pushend (10)
!
!  ater execution of this code segment , the pointer of unit 10 is &
!  pshed to its end.
!
!**********************************************************************
!
  subroutine pushend(iunit)
    implicit none
    integer(ik4) :: iunit , ierr
!
!**********************************************************************
!
    logical opned
!
! *** inquire if iunit connected to file
! ********************************
    inquire (unit = iunit,opened = opned)
    if ( .not.opned ) return
!
! *** iunit connected , push pointer to end
! ******************************
    do
      read (iunit,'()',iostat=ierr)
      if ( ierr /= 0 ) exit
    end do
    backspace(iunit)
  end subroutine pushend


!************************************************************************
!
!  tolbox library v.1.0 (may 1995)
!
!  pogram unit   : subroutine appendext &
!  prpose        : fix extension in file name string
!
!  ====================== arguments / usage =============================
!
!  flename   is the character variable with the file name &
!  dfext     is the character variable with extension (including '.',
!             ex. '.dat') &
!  oerwrite  is a logical value , .true. overwrites any existing extension
!             in "filename" with "defext", .false. puts "defext" only if
!             there is no extension in "filename".
!
!  eample:
!             filename1 = 'test.dat'
!             filename2 = 'test.dat'
!             call appendext (filename1 , '.txt', .false.)
!             call appendext (filename2 , '.txt', .true. )
!
!  ater execution of this code segment , "filename1" has the value &
!  'est.dat', while "filename2" has the value 'test.txt'
!
!**********************************************************************
!
  subroutine appendext(filename,defext,overwrite)
    implicit none
    integer(ik4) :: idot , iend
    character(len=*) :: filename , defext
    logical :: overwrite
    call chrbln(filename,iend)
    if ( filename(1:1)==' ' .and. iend==1 ) return      ! filename empty
    idot = index(filename,'.')                           ! append extension ?
    if ( idot==0 ) filename = filename(1:iend)//defext
    if ( overwrite .and. idot/=0 ) filename = filename(:idot-1)//defext
  end subroutine appendext
!
!======================================================================
!
! *** isorropia code
! *** subroutine poly3
! *** finds the real(rkx) :: roots of the third order algebraic equation:
!     x**3 + a1*x**2 + a2*x + a3 = 0.0
!     the equation is solved analytically.
!
!     parameters a1 , a2 , a3 are specified by the user. the minimum
!     nonegative root is returned in variable 'root'. if no root is
!     found (which is greater than zero), root has the value 1d30.
!     and the flag islv has a value greater than zero.
!
!     solution formula is found in page 32 of:
!     mathematical handbook of formulas and tables
!     schaum's outline series
!     murray spieger , mcgraw-hill , new york , 1968
!     (greek translation: by sotirios persides , espi , athens , 1976)
!
!     a special case is considered seperately ; when a3 = 0 , then
!     one root is x = 0.0 , and the other two from the solution of the
!     quadratic equation x**2 + a1*x + a2 = 0.0
!     this special case is considered because the analytical formula
!     does not yield accurate results (due to numerical roundoff errors)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine poly3(a1,a2,a3,root,islv)
    implicit none
    real(rkx) :: a1 , a2 , a3 , coef , d , eps , expon , pi , q , r , &
         root , s , sqd , ssig , t , thet , thet1 , thet2 , tsig , zero
    integer(ik4) :: i , islv , ix
    parameter (expon = 1._rkx/3._rkx,zero = 0._rkx, &
               thet1 = 120._rkx/180._rkx,       &
               thet2 = 240._rkx/180._rkx, &
               pi = 3.14159265358932, &
               eps = epsilon(1.0_rkx))
    real(rkx) x(3)
!
! *** special case : quadratic*x equation *****************************
!
    if ( abs(a3)<=eps ) then
      islv = 1
      ix = 1
      x(1) = zero
      d = a1*a1 - 4._rkx*a2
      if ( d>=zero ) then
        ix = 3
        sqd = sqrt(d)
        x(2) = 0.5_rkx*(-a1+sqd)
        x(3) = 0.5_rkx*(-a1-sqd)
      end if
    else
!
!   *** normal case : cubic equation
!   ************************************
!   defne parameters q , r , s , t , d
!
      islv = 1
      q = (3._rkx*a2-a1*a1)/9._rkx
      r = (9._rkx*a1*a2-27._rkx*a3-2._rkx*a1*a1*a1)/54._rkx
      d = q*q*q + r*r
!
!   *** calculate roots
!   *************************************************
!   d  0 , three real(rkx) :: roots
!
      if ( d<-eps ) then           ! d < -eps  : d < zero
        ix = 3
        thet = expon*acos(r/sqrt(-q*q*q))
        coef = 2._rkx*sqrt(-q)
        x(1) = coef*cos(thet) - expon*a1
        x(2) = coef*cos(thet+thet1*pi) - expon*a1
        x(3) = coef*cos(thet+thet2*pi) - expon*a1
!
!     d  0 , three real(rkx) :: (one double) roots
!
      else if ( d<=eps ) then       ! -eps <= d <= eps  : d = zero
        ix = 2
        ssig = sign(1._rkx,r)
        s = ssig*(abs(r))**expon
        x(1) = 2._rkx*s - expon*a1
        x(2) = -s - expon*a1
!
!     d  0 , one real(rkx) :: root
!
      else                          ! d > eps  : d > zero
        ix = 1
        sqd = sqrt(d)
        ssig = sign(1._rkx,r+sqd)              ! transfer sign to ssig
        tsig = sign(1._rkx,r-sqd)
        s = ssig*(abs(r+sqd))**expon        ! exponentiate abs()
        t = tsig*(abs(r-sqd))**expon
        x(1) = s + t - expon*a1
      end if
    end if
!
! *** select appropriate root *****************************************
!
    root = 1.e30_rkx
    do i = 1 , ix
      if ( x(i)>zero ) then
        root = min(root,x(i))
        islv = 0
      end if
    end do
  end subroutine poly3
!
!======================================================================
!
! *** isorropia code
! *** subroutine poly3b
! *** finds a real(rkx) :: root of the third order algebraic equation:
!     x**3 + a1*x**2 + a2*x + a3 = 0.0
!     the equation is solved numerically (bisection).
!
!     parameters a1 , a2 , a3 are specified by the user. the minimum
!     nonegative root is returned in variable 'root'. if no root is
!     found (which is greater than zero), root has the value 1d30.
!     and the flag islv has a value greater than zero.
!
!     rtlw , rthi define the interval which the root is looked for.
!
!======================================================================
!
  subroutine poly3b(a1,a2,a3,rtlw,rthi,root,islv)
!
    implicit none
    real(rkx) :: a1 , a2 , a3 , dx , root , rthi , &
         rtlw , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i , islv
    real(rkx) , parameter :: zero = 0._rkx
    real(rkx) , parameter :: eps = 1e-15_rkx
    integer(ik4) , parameter :: maxit = 100
    integer(ik4) , parameter :: ndiv = 5
!
! *** initial values for bisection *************************************
!
    x1 = rtlw
    y1 = func(x1)
    if ( abs(y1)<=eps ) then       ! is low a root?
      root = rtlw
      return
    end if
!
! *** root tracking ; for the range of hi and lo ***********************
!
    dx = (rthi-rtlw)/float(ndiv)
    do i = 1 , ndiv
      x2 = x1 + dx
      y2 = func(x2)
      if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100      ! (y1*y2 < zero)
      x1 = x2
      y1 = y2
    end do
!
! *** no subdivision with solution found
!
    if ( abs(y2)<eps ) then       ! x2 is a root
      root = x2
    else
      root = 1.e30_rkx
      islv = 1
    end if
    return
!
! *** bisection *******************************************************
!
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = func(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) exit
    end do
!
! *** converged ; return ***********************************************
!
    x3 = 0.5_rkx*(x1+x2)
    y3 = func(x3)
    root = x3
    islv = 0
    contains
      real(rkx) function func(x)
        implicit none
        real(rkx) , intent(in) :: x
        func = x**30 + a1*x**2 + a2*x + a3
      end function func
  end subroutine poly3b
!
!      program driver
!      real(rkx) root
!
!      call poly3 (-1.d0 , 1.d0 , -1.d0 , root , islv)
!      if (islv.ne.0) stop 'error in poly3'
!      write (*,*) 'root =', root
!
!      call poly3b (-1.d0 , 1.d0 , -1.d0 , -10.d0 , 10.d0 , root , islv)
!      if (islv.ne.0) stop 'error in poly3b'
!      write (*,*) 'root =', root
!
!    end subroutine
!======================================================================
!
! *** isorropia code
! *** function ex10
! *** 10^x function ; alternate of library routine ; used because it is
!     much faster but without great loss in accuracy. ,
!     maximum error is 2%, execution time is 42% of the library routine
!     (on a 80286/80287 machine , using lahey fortran 77 v.3.0).
!
!     exponent range is between -k and k (k is the real(rkx) :: argument 'k')
!     max value for k: 9.999
!     if x < -k , x is set to -k , if x > k , x is set to k
!
!     the exponent is calculated by the product adec*aint , where adec
!     is the mantissa and aint is the magnitude (exponent). both
!     mantissa and magnitude are pre-calculated and stored in lookup
!     tables ; this leads to the increased speed.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
    function ex10(x,k)
    implicit none
    real(rkx) :: x , ex10 , y , k
    real(rkx) , dimension(20) :: aint10
    real(rkx) , dimension(200) :: adec10
    integer(ik4) :: k1 , k2
!
! *** common block definition
!
! *** integer(ik4) :: part
!
    aint10 = (/0.1000E-08,0.1000E-07,0.1000E-06,0.1000E-05,0.1000E-04,    &
             0.1000E-03,0.1000E-02,0.1000E-01,0.1000E+00,0.1000E+01,      &
             0.1000E+02,0.1000E+03,0.1000E+04,0.1000E+05,0.1000E+06,      &
             0.1000E+07,0.1000E+08,0.1000E+09,0.1000E+10,0.1000E+11/)
!
! *** decimal part
!
    adec10(1:100) = (/0.1023E+00,0.1047E+00,0.1072E+00,0.1096E+00,        &
                    0.1122E+00,0.1148E+00,0.1175E+00,0.1202E+00,          &
                    0.1230E+00,0.1259E+00,0.1288E+00,0.1318E+00,          &
                    0.1349E+00,0.1380E+00,0.1413E+00,0.1445E+00,          &
                    0.1479E+00,0.1514E+00,0.1549E+00,0.1585E+00,          &
                    0.1622E+00,0.1660E+00,0.1698E+00,0.1738E+00,          &
                    0.1778E+00,0.1820E+00,0.1862E+00,0.1905E+00,          &
                    0.1950E+00,0.1995E+00,0.2042E+00,0.2089E+00,          &
                    0.2138E+00,0.2188E+00,0.2239E+00,0.2291E+00,          &
                    0.2344E+00,0.2399E+00,0.2455E+00,0.2512E+00,          &
                    0.2570E+00,0.2630E+00,0.2692E+00,0.2754E+00,          &
                    0.2818E+00,0.2884E+00,0.2951E+00,0.3020E+00,          &
                    0.3090E+00,0.3162E+00,0.3236E+00,0.3311E+00,          &
                    0.3388E+00,0.3467E+00,0.3548E+00,0.3631E+00,          &
                    0.3715E+00,0.3802E+00,0.3890E+00,0.3981E+00,          &
                    0.4074E+00,0.4169E+00,0.4266E+00,0.4365E+00,          &
                    0.4467E+00,0.4571E+00,0.4677E+00,0.4786E+00,          &
                    0.4898E+00,0.5012E+00,0.5129E+00,0.5248E+00,          &
                    0.5370E+00,0.5495E+00,0.5623E+00,0.5754E+00,          &
                    0.5888E+00,0.6026E+00,0.6166E+00,0.6310E+00,          &
                    0.6457E+00,0.6607E+00,0.6761E+00,0.6918E+00,          &
                    0.7079E+00,0.7244E+00,0.7413E+00,0.7586E+00,          &
                    0.7762E+00,0.7943E+00,0.8128E+00,0.8318E+00,          &
                    0.8511E+00,0.8710E+00,0.8913E+00,0.9120E+00,          &
                    0.9333E+00,0.9550E+00,0.9772E+00,0.1000E+01/)

    adec10(101:200) = (/0.1023E+01,0.1047E+01,0.1072E+01,0.1096E+01,      &
                      0.1122E+01,0.1148E+01,0.1175E+01,0.1202E+01,        &
                      0.1230E+01,0.1259E+01,0.1288E+01,0.1318E+01,        &
                      0.1349E+01,0.1380E+01,0.1413E+01,0.1445E+01,        &
                      0.1479E+01,0.1514E+01,0.1549E+01,0.1585E+01,        &
                      0.1622E+01,0.1660E+01,0.1698E+01,0.1738E+01,        &
                      0.1778E+01,0.1820E+01,0.1862E+01,0.1905E+01,        &
                      0.1950E+01,0.1995E+01,0.2042E+01,0.2089E+01,        &
                      0.2138E+01,0.2188E+01,0.2239E+01,0.2291E+01,        &
                      0.2344E+01,0.2399E+01,0.2455E+01,0.2512E+01,        &
                      0.2570E+01,0.2630E+01,0.2692E+01,0.2754E+01,        &
                      0.2818E+01,0.2884E+01,0.2951E+01,0.3020E+01,        &
                      0.3090E+01,0.3162E+01,0.3236E+01,0.3311E+01,        &
                      0.3388E+01,0.3467E+01,0.3548E+01,0.3631E+01,        &
                      0.3715E+01,0.3802E+01,0.3890E+01,0.3981E+01,        &
                      0.4074E+01,0.4169E+01,0.4266E+01,0.4365E+01,        &
                      0.4467E+01,0.4571E+01,0.4677E+01,0.4786E+01,        &
                      0.4898E+01,0.5012E+01,0.5129E+01,0.5248E+01,        &
                      0.5370E+01,0.5495E+01,0.5623E+01,0.5754E+01,        &
                      0.5888E+01,0.6026E+01,0.6166E+01,0.6310E+01,        &
                      0.6457E+01,0.6607E+01,0.6761E+01,0.6918E+01,        &
                      0.7079E+01,0.7244E+01,0.7413E+01,0.7586E+01,        &
                      0.7762E+01,0.7943E+01,0.8128E+01,0.8318E+01,        &
                      0.8511E+01,0.8710E+01,0.8913E+01,0.9120E+01,        &
                      0.9333E+01,0.9550E+01,0.9772E+01,0.1000E+02/)
!
! *** limit x to [-k , k] range
! *****************************************
    y = max(-k,min(x,k))           ! min: -9.999 , max: 9.999
!
! *** get integer(ik4) :: and decimal part *************************************
!
    k1 = int(y)
    k2 = int(100*(y-k1))
!
! *** calculate exp function *******************************************
!
    ex10 = aint10(k1+10)*adec10(k2+100)
  end function ex10
!
!======================================================================
!
! *** isorropia code
! *** subroutine pusherr
! *** this subroutine saves an error message in the error stack
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine pusherr(ierr,errinf)
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: ierr
    character(len=*) :: errinf
!
! *** save error code if there is any space ***************************
!
    if ( nofer<nerrmx ) then
      nofer = nofer + 1
      errstk(nofer) = ierr
      errmsg(nofer) = errinf
      stkofl = .false.
    else
      stkofl = .true.               ! stack overflow
    end if
  end subroutine pusherr
!
!======================================================================
!
! *** isorropia code
! *** subroutine iserrinf
! *** this subroutine obtains a copy of the error stack (& messages)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine iserrinf(errstki,errmsgi,noferi,stkofli)
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i , noferi
    character(len=40) , dimension(nerrmx) :: errmsgi
    integer(ik4) , dimension(nerrmx) :: errstki
    logical :: stkofli
!
! *** obtain whole error stack ****************************************
!
    do i = 1 , nofer               ! error messages & codes
      errstki(i) = errstk(i)
      errmsgi(i) = errmsg(i)
    end do
    stkofli = stkofl
    noferi = nofer
  end subroutine iserrinf
!
!======================================================================
!
! *** isorropia code
! *** subroutine errstat
! *** this subroutine reports error messages to unit 'io'
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine errstat(io,ierr,errinf)
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: iend , ierr , io , iok
    character(len=4) :: cer
    character(len=29) :: ncis
    character(len=27) :: ncif
    character(len=26) :: nsis
    character(len=24) :: nsif
    character(len=*) :: errinf
    data ncis/'no convergence in subroutine '/ ,                          &
         ncif/'no convergence in function '/ ,                            &
         nsis/'no solution in subroutine '/ ,                             &
         nsif/'no solution in function '/
!
! *** write error in character *****************************************
!
    write (cer,'(i4)') ierr
    call rplstr(cer,' ','0',iok)         ! replace blanks with zeros
    call chrbln(errinf,iend)             ! last position of errinf char
!
! *** write error type (fatal , warning )
! *******************************
    if ( ierr==0 ) then
      write (io,99001) 'no errors detected '
      go to 100
!
    else if ( ierr<0 ) then
      write (io,99001) 'error stack exhausted '
      go to 100
!
    else if ( ierr>1000 ) then
      write (io,99002) 'fatal' , cer
!
    else
      write (io,99002) 'warning' , cer
    end if
!
! *** write error message **********************************************
!
! fatl messages
!
    if ( ierr==1001 ) then
      call chrbln(scase,iend)
      write (io,99001) 'case not supported in calcmr ['//scase(1:iend)    &
                       //']'
!
    else if ( ierr==1002 ) then
      call chrbln(scase,iend)
      write (io,99001) 'case not supported ['//scase(1:iend)//']'
!
!   waring messages
!
    else if ( ierr==0001 ) then
      write (io,99001) nsis , errinf
!
    else if ( ierr==0002 ) then
      write (io,99001) ncis , errinf
!
    else if ( ierr==0003 ) then
      write (io,99001) nsif , errinf
!
    else if ( ierr==0004 ) then
      write (io,99001) ncif , errinf
!
    else if ( ierr==0019 ) then
      write (io,99001) 'hno3(aq) affects h+, which '//                    &
                       'might affect so4/hso4 ratio'
      write (io,99001) 'direct increase in h+ [' , errinf(1:iend) , '] %'
!
    else if ( ierr==0020 ) then
      if ( w(4)>tiny1 .and. w(5)>tiny1 ) then
        write (io,99001) 'hso4-so4 equilibrium might affect hno3 ,'//     &
                         'hcl dissolution'
      else
        write (io,99001) 'hso4-so4 equilibrium might affect nh3 '//       &
                         'dissolution'
      end if
      write (io,99001) 'direct decrease in h+ [' , errinf(1:iend) , '] %'
!
    else if ( ierr==0021 ) then
      write (io,99001) 'hno3(aq),hcl(aq) affect h+, which '//             &
                       'might affect so4/hso4 ratio'
      write (io,99001) 'direct increase in h+ [' , errinf(1:iend) , '] %'
!
    else if ( ierr==0022 ) then
      write (io,99001) 'hcl(g) equilibrium yields nonphysical '//         &
                       'dissolution'
      write (io,99001) 'a tiny1 amount [' , errinf(1:iend) ,              &
                       '] is '//'assumed to be dissolved'
!
    else if ( ierr==0033 ) then
      write (io,99001) 'hcl(aq) affects h+, which '//                     &
                       'might affect so4/hso4 ratio'
      write (io,99001) 'direct increase in h+ [' , errinf(1:iend) , '] %'
!
    else if ( ierr==0050 ) then
      write (io,99001) 'too much sodium given as input.'
      write (io,99001) 'reduced to completely neutralize so4 ,cl ,no3.'
      write (io,99001) 'excess sodium is ignored.'
!
    else if ( ierr==0051 ) then
      write (io,99001) 'too much calcium given as input.'
      write (io,99001) 'reduced to completely neutralize so4 ,cl ,no3.'
      write (io,99001) 'excess calcium is ignored.'
!
    else if ( ierr==0052 ) then
      write (io,99001) 'too much sodium (+ca) given as input.'
      write (io,99001) 'reduced to completely neutralize so4 ,cl ,no3.'
      write (io,99001) 'excess sodium is ignored.'
!
    else if ( ierr==0053 ) then
      write (io,99001) 'too much magnesium (+ca ,na) given as input.'
      write (io,99001) 'reduced to completely neutralize so4 ,cl ,no3.'
      write (io,99001) 'excess magnesium is ignored.'
!
    else if ( ierr==0054 ) then
      write (io,99001) 'too much potassium(+ca ,na ,mg) given as input.'
      write (io,99001) 'reduced to completely neutralize so4 ,cl ,no3.'
      write (io,99001) 'excess potassium is ignored.'
!
    else
      write (io,99001) 'no diagnostic message available'
    end if
!
100 continue
!
! *** format statements *************************************
!
99001 format (1x,a:a:a:a:a)
99002 format (1x,a,' error [',a4,']:')
  end subroutine errstat
!
!======================================================================
!
! *** isorropia code
! *** subroutine isorinf
! *** this subroutine provides information about isorropia
!
! ======================= arguments / usage ===========================
!
!  ouput:
!  1. [versi]
!     character*15 variable.
!     contains version-date information of isorropia
!
!  2. [ncmp]
!     integer(ik4) :: variable.
!     the number of components needed in input array wi
!     (or , the number of major species accounted for by isorropia)
!
!  3. [nion]
!     integer(ik4) :: variable
!     the number of ions considered in the aqueous phase
!
!  4. [naqgas]
!     integer(ik4) :: variable
!     the number of undissociated species found in aqueous aerosol
!     phase
!
!  5. [nsol]
!     integer(ik4) :: variable
!     the number of solids considered in the solid aerosol phase
!
!  6. [nerr]
!     integer(ik4) :: variable
!     the size of the error stack (maximum number of errors that can
!     be stored before the stack exhausts).
!
!  7. [tin]
!     real(rkx) variable
!     the value used for a very small number.
!
!  8. [grt]
!     real(rkx) variable
!     the value used for a very large number.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine isorinf(versi,ncmp,nion,naqgas,nsol,nerr,tin,grt)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: grt , tin
    integer(ik4) :: naqgas , ncmp , nerr , nion , nsol
    character(len=*) :: versi
!
! *** assign info
! *******************************************************
    versi = version
    ncmp = ncomp
    nion = nions
    naqgas = ngasaq
    nsol = nslds
    nerr = nerrmx
    tin = tiny1
    grt = great
  end subroutine isorinf
!
!======================================================================
!
! *** isorropia code
! *** subroutine isrp1f
! *** this subroutine is the driver routine for the foreward problem of
!     an ammonium-sulfate aerosol system.
!     the composition regime is determined by the sulfate ratio and by
!     the ambient relative humidity.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine isrp1f(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: dc , rhi , tempi
    real(rkx) , dimension(ncomp) :: wi
!
! *** initialize all variables in common block
! **************************
    call init1(wi,rhi,tempi)
!
! *** calculate sulfate ratio
! *******************************************
    sulrat = w(3)/w(2)
!
! *** find calculation regime from (sulrat ,rh)
! **************************
! *** sulfate poor
!
    if ( 2.0<=sulrat ) then
      dc = w(3) - 2.001_rkx*w(2)    ! for numerical stability
      w(3) = w(3) + max(-dc,zero)
!
      if ( metstbl==1 ) then
        scase = 'a2'
        call calca2                  ! only liquid (metastable)
!
      else if ( rh<drnh42s4 ) then
        scase = 'a1'
        call calca1                  ! nh42so4              ; case a1
!
      else if ( drnh42s4<=rh ) then
        scase = 'a2'
        call calca2                  ! only liquid          ; case a2
      end if
!
!   *** sulfate rich (no acid)
!
    else if ( 1.0<=sulrat .and. sulrat<2.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'b4'
        call calcb4                  ! only liquid (metastable)
!
      else if ( rh<drnh4hs4 ) then
        scase = 'b1'
        call calcb1                  ! nh4hso4 ,lc ,nh42so4   ; case b1
!
      else if ( drnh4hs4<=rh .and. rh<drlc ) then
        scase = 'b2'
        call calcb2                  ! lc ,nh42s4            ; case b2
!
      else if ( drlc<=rh .and. rh<drnh42s4 ) then
        scase = 'b3'
        call calcb3                  ! nh42s4               ; case b3
!
      else if ( drnh42s4<=rh ) then
        scase = 'b4'
        call calcb4                  ! only liquid          ; case b4
      end if
      call calcnh3
!
!   *** sulfate rich (free acid)
!
    else if ( sulrat<1.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'c2'
        call calcc2                  ! only liquid (metastable)
!
      else if ( rh<drnh4hs4 ) then
        scase = 'c1'
        call calcc1                  ! nh4hso4              ; case c1
!
      else if ( drnh4hs4<=rh ) then
        scase = 'c2'
        call calcc2                  ! only liquid          ; case c2
!
      end if
      call calcnh3
    end if
  end subroutine isrp1f
!
!======================================================================
!
! *** isorropia code
! *** subroutine isrp2f
! *** this subroutine is the driver routine for the foreward problem of
!     an ammonium-sulfate-nitrate aerosol system.
!     the composition regime is determined by the sulfate ratio and by
!     the ambient relative humidity.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine isrp2f(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: rhi , tempi
    real(rkx) , dimension(ncomp) :: wi
!
! *** initialize all variables in common block
! **************************
    call init2(wi,rhi,tempi)
!
! *** calculate sulfate ratio
! *******************************************
    sulrat = w(3)/w(2)
!
! *** find calculation regime from (sulrat ,rh)
! **************************
! *** sulfate poor
!
    if ( 2.0<=sulrat ) then
!
      if ( metstbl==1 ) then
        scase = 'd3'
        call calcd3                  ! only liquid (metastable)
!
      else if ( rh<drnh4no3 ) then
        scase = 'd1'
        call calcd1                  ! nh42so4 ,nh4no3       ; case d1
!
      else if ( drnh4no3<=rh .and. rh<drnh42s4 ) then
        scase = 'd2'
        call calcd2                  ! nh42s4               ; case d2
!
      else if ( drnh42s4<=rh ) then
        scase = 'd3'
        call calcd3                  ! only liquid          ; case d3
      end if
!
!   *** sulfate rich (no acid)
!   for solving this case , nitric acid is assumed a minor species ,
!   that does not significantly perturb the hso4-so4 equilibrium.
!   subroutines calcb? are called , and then the nitric acid is
!   dissolved from the hno3(g) -> (h+) + (no3-) equilibrium.
!
    else if ( 1.0<=sulrat .and. sulrat<2.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'b4'
        call calcb4                  ! only liquid (metastable)
        scase = 'e4'
!
      else if ( rh<drnh4hs4 ) then
        scase = 'b1'
        call calcb1                  ! nh4hso4 ,lc ,nh42so4   ; case e1
        scase = 'e1'
!
      else if ( drnh4hs4<=rh .and. rh<drlc ) then
        scase = 'b2'
        call calcb2                  ! lc ,nh42s4            ; case e2
        scase = 'e2'
!
      else if ( drlc<=rh .and. rh<drnh42s4 ) then
        scase = 'b3'
        call calcb3                  ! nh42s4               ; case e3
        scase = 'e3'
!
      else if ( drnh42s4<=rh ) then
        scase = 'b4'
        call calcb4                  ! only liquid          ; case e4
        scase = 'e4'
      end if
!
      call calcna                 ! hno3(g) dissolution
!
!   *** sulfate rich (free acid)
!   for solving this case , nitric acid is assumed a minor species ,
!   that does not significantly perturb the hso4-so4 equilibrium
!   subroutine calcc? is called , and then the nitric acid is dissolved
!   from the hno3(g) -> (h+) + (no3-) equilibrium.
!
    else if ( sulrat<1.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'c2'
        call calcc2                  ! only liquid (metastable)
        scase = 'f2'
!
      else if ( rh<drnh4hs4 ) then
        scase = 'c1'
        call calcc1                  ! nh4hso4              ; case f1
        scase = 'f1'
!
      else if ( drnh4hs4<=rh ) then
        scase = 'c2'
        call calcc2                  ! only liquid          ; case f2
        scase = 'f2'
      end if
!
      call calcna                 ! hno3(g) dissolution
    end if
  end subroutine isrp2f
!
!======================================================================
!
! *** isorropia code
! *** subroutine isrp3f
! *** this subroutine is the driver routine for the forward problem of
!     an ammonium-sulfate-nitrate-chloride-sodium aerosol system.
!     the composition regime is determined by the sulfate & sodium
!     ratios and by the ambient relative humidity.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine isrp3f(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: rest , rhi , tempi
    real(rkx) , dimension(ncomp) :: wi
!
! *** adjust for too little ammonium and chloride
! ***********************
    wi(3) = max(wi(3),1.e-10_rkx)      ! nh4+ : 1e-4 umoles/m3
    wi(5) = max(wi(5),1.e-10_rkx)      ! cl-  : 1e-4 umoles/m3
!
! *** adjust for too little sodium , sulfate and nitrate combined
! ********
    if ( wi(1)+wi(2)+wi(4)<=1e-10_rkx ) then
      wi(1) = 1.e-10_rkx     ! na+  : 1e-4 umoles/m3
      wi(2) = 1.e-10_rkx     ! so4- : 1e-4 umoles/m3
    end if
!
! *** initialize all variables in common block
! **************************
    call isoinit3(wi,rhi,tempi)
!
! *** check if too much sodium ; adjust and issue error message
! *********
    rest = 2._rkx*w(2) + w(4) + w(5)
    if ( w(1)>rest ) then              ! na > 2*so4+cl+no3 ?
      w(1) = (one-1e-6_rkx)*rest            ! adjust na amount
      call pusherr(0050,'isrp3f')        ! warning error: na adjusted
    end if
!
! *** calculate sulfate & sodium ratios
! *********************************
    sulrat = (w(1)+w(3))/w(2)
    sodrat = w(1)/w(2)
!
! *** find calculation regime from (sulrat ,rh)

! ************************** *** sulfate poor ; sodium poor
!
    if ( 2.0<=sulrat .and. sodrat<2.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'g5'
        call calcg5                  ! only liquid (metastable)
!
      else if ( rh<drnh4no3 ) then
        scase = 'g1'
        call calcg1                  ! nh42so4 ,nh4no3 ,nh4cl ,na2so4
!
      else if ( drnh4no3<=rh .and. rh<drnh4cl ) then
        scase = 'g2'
        call calcg2                  ! nh42so4 ,nh4cl ,na2so4
!
      else if ( drnh4cl<=rh .and. rh<drnh42s4 ) then
        scase = 'g3'
        call calcg3                  ! nh42so4 ,na2so4
!
      else if ( drnh42s4<=rh .and. rh<drna2so4 ) then
        scase = 'g4'
        call calcg4                  ! na2so4
!
      else if ( drna2so4<=rh ) then
        scase = 'g5'
        call calcg5                  ! only liquid
      end if
!
!   *** sulfate poor ; sodium rich
!
    else if ( sulrat>=2.0 .and. sodrat>=2.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'h6'
        call calch6                  ! only liquid (metastable)
!
      else if ( rh<drnh4no3 ) then
        scase = 'h1'
        call calch1                  ! nh4no3 ,nh4cl ,na2so4 ,nacl ,nano3
!
      else if ( drnh4no3<=rh .and. rh<drnano3 ) then
        scase = 'h2'
        call calch2                  ! nh4cl ,na2so4 ,nacl ,nano3
!
      else if ( drnano3<=rh .and. rh<drnacl ) then
        scase = 'h3'
        call calch3                  ! nh4cl ,na2so4 ,nacl
!
      else if ( drnacl<=rh .and. rh<drnh4cl ) then
        scase = 'h4'
        call calch4                  ! nh4cl ,na2so4
!
      else if ( drnh4cl<=rh .and. rh<drna2so4 ) then
        scase = 'h5'
        call calch5                  ! na2so4
!
      else if ( drna2so4<=rh ) then
        scase = 'h6'
        call calch6                  ! no solid
      end if
!
!   *** sulfate rich (no acid)
!
    else if ( 1.0<=sulrat .and. sulrat<2.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'i6'
        call calci6                  ! only liquid (metastable)
!
      else if ( rh<drnh4hs4 ) then
        scase = 'i1'
        call calci1                  ! na2so4 ,(nh4)2so4 ,nahso4 ,nh4hso4 ,lc
!
      else if ( drnh4hs4<=rh .and. rh<drnahso4 ) then
        scase = 'i2'
        call calci2                  ! na2so4 ,(nh4)2so4 ,nahso4 ,lc
!
      else if ( drnahso4<=rh .and. rh<drlc ) then
        scase = 'i3'
        call calci3                  ! na2so4 ,(nh4)2so4 ,lc
!
      else if ( drlc<=rh .and. rh<drnh42s4 ) then
        scase = 'i4'
        call calci4                  ! na2so4 ,(nh4)2so4
!
      else if ( drnh42s4<=rh .and. rh<drna2so4 ) then
        scase = 'i5'
        call calci5                  ! na2so4
!
      else if ( drna2so4<=rh ) then
        scase = 'i6'
        call calci6                  ! no solids
      end if
!
      call calcnha                ! minor species: hno3 , hcl
      call calcnh3                !                nh3
!
!   *** sulfate rich (free acid)
!
    else if ( sulrat<1.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'j3'
        call calcj3                  ! only liquid (metastable)
!
      else if ( rh<drnh4hs4 ) then
        scase = 'j1'
        call calcj1                  ! nh4hso4 ,nahso4
!
      else if ( drnh4hs4<=rh .and. rh<drnahso4 ) then
        scase = 'j2'
        call calcj2                  ! nahso4
!
      else if ( drnahso4<=rh ) then
        scase = 'j3'
        call calcj3
      end if
!
      call calcnha                ! minor species: hno3 , hcl
      call calcnh3                !                nh3
    end if
  end subroutine isrp3f
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine isrp4f
! *** this subroutine is the driver routine for the forward problem of
!     an ammonium-sulfate-nitrate-chloride-sodium-calcium-potassium-magnesium
!     aerosol system.
!     the composition regime is determined by the sulfate & sodium
!     ratios and by the ambient relative humidity.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine isrp4f(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: cafri , ccacl2i , ccano32i , ccaso4i , clfri , cmgcl2i , &
         cmgno32i , cmgso4i , cna2so4i , cnacli , cnano3i , frmgi ,       &
         frso4i , rest , rest1 , rest2 , rest3 , rhi , tempi
    real(rkx) , dimension(ncomp) :: wi
    real(rkx) :: nafri , no3fri , no3fr
!
! *** adjust for too little ammonium and chloride
! ***********************
! wi(3) = max (wi(3), 1.d-10)  ! nh4+ : 1e-4 umoles/m3
! wi(5) = max (wi(5), 1.d-10)  ! cl-  : 1e-4 umoles/m3
!
! *** adjust for too little sodium , sulfate and nitrate combined
! ********
! if (wi(1)+wi(2)+wi(4)  <=  1d-10) then
! wi(1) = 1.d-10  ! na+  : 1e-4 umoles/m3
! wi(2) = 1.d-10  ! so4- : 1e-4 umoles/m3
! end if
!
! *** initialize all variables in common block
! **************************
    call init4(wi,rhi,tempi)
!
! *** check if too much sodium+crustals ; adjust and issue error message
!
    rest = 2._rkx*w(2) + w(4) + w(5)
!
    if ( w(1)+w(6)+w(7)+w(8)>rest ) then
!
      ccaso4i = min(w(2),w(6))
      frso4i = max(w(2)-ccaso4i,zero)
      cafri = max(w(6)-ccaso4i,zero)
      ccano32i = min(cafri,0.5_rkx*w(4))
      cafri = max(cafri-ccano32i,zero)
      no3fri = max(w(4)-2._rkx*ccano32i,zero)
      ccacl2i = min(cafri,0.5_rkx*w(5))
      clfri = max(w(5)-2._rkx*ccacl2i,zero)
      rest1 = 2._rkx*frso4i + no3fri + clfri
!
      cna2so4i = min(frso4i,0.5_rkx*w(1))
      frso4i = max(frso4i-cna2so4i,zero)
      nafri = max(w(1)-2._rkx*cna2so4i,zero)
      cnacli = min(nafri,clfri)
      nafri = max(nafri-cnacli,zero)
      clfri = max(clfri-cnacli,zero)
      cnano3i = min(nafri,no3fri)
      no3fr = max(no3fri-cnano3i,zero)
      rest2 = 2._rkx*frso4i + no3fri + clfri
!
      cmgso4i = min(frso4i,w(8))
      frmgi = max(w(8)-cmgso4i,zero)
      frso4i = max(frso4i-cmgso4i,zero)
      cmgno32i = min(frmgi,0.5_rkx*no3fri)
      frmgi = max(frmgi-cmgno32i,zero)
      no3fri = max(no3fri-2._rkx*cmgno32i,zero)
      cmgcl2i = min(frmgi,0.5_rkx*clfri)
      clfri = max(clfri-2._rkx*cmgcl2i,zero)
      rest3 = 2._rkx*frso4i + no3fri + clfri
!
      if ( w(6)>rest ) then                          ! ca > 2*so4+cl+no3 ?
        w(6) = (one-1e-6_rkx)*rest                   ! adjust ca amount
        w(1) = zero                               ! adjust na amount
        w(7) = zero                               ! adjust k amount
        w(8) = zero                               ! adjust mg amount
        call pusherr(0051,'isrp4f')               ! warning error: ca , na , k , mg in excess
!
      else if ( w(1)>rest1 ) then                    ! na > 2*frso4+frcl+frno3 ?
        w(1) = (one-1e-6_rkx)*rest1                  ! adjust na amount
        w(7) = zero                               ! adjust k amount
        w(8) = zero                               ! adjust mg amount
        call pusherr(0052,'isrp4f')               ! warning error: na , k , mg in excess
!
      else if ( w(8)>rest2 ) then                    ! mg > 2*frso4+frcl+frno3 ?
        w(8) = (one-1e-6_rkx)*rest2                  ! adjust mg amount
        w(7) = zero                               ! adjust k amount
        call pusherr(0053,'isrp4f')               ! warning error: k , mg in excess
!
      else if ( w(7)>rest3 ) then                    ! k > 2*frso4+frcl+frno3 ?
        w(7) = (one-1e-6_rkx)*rest3                  ! adjust k amount
        call pusherr(0054,'isrp4f')               ! warning error: k in excess
      end if
    end if
!
! *** calculate ratios *************************************************
!
    so4rat = (w(1)+w(3)+w(6)+w(7)+w(8))/w(2)
    crnarat = (w(1)+w(6)+w(7)+w(8))/w(2)
    crrat = (w(6)+w(7)+w(8))/w(2)
!
! *** find calculation regime from (so4rat , crnarat , crrat , rrh)
! ********
! *** sulfate poor: rso4>2; (dust + sodium) poor: r(cr+na)<2
!
    if ( 2.0<=so4rat .and. crnarat<2.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'o7'
        call calco7                  ! only liquid (metastable)
!
      else if ( rh<drnh4no3 ) then
        scase = 'o1'
        call calco1                  ! caso4 , nh4no3 , nh4cl , (nh4)2so4 , mgso4 , na2so4 , k2so4
!
      else if ( drnh4no3<=rh .and. rh<drnh4cl ) then
        scase = 'o2'
        call calco2                  ! caso4 , nh4cl , (nh4)2so4 , mgso4 , na2so4 , k2so4
!
      else if ( drnh4cl<=rh .and. rh<drnh42s4 ) then
        scase = 'o3'
        call calco3                  ! caso4 , (nh4)2so4 , mgso4 , na2so4 , k2so4
!
      else if ( drnh42s4<=rh .and. rh<drmgso4 ) then
        scase = 'o4'
        call calco4                  ! caso4 , mgso4 , na2so4 , k2so4
!
      else if ( drmgso4<=rh .and. rh<drna2so4 ) then
        scase = 'o5'
        call calco5                  ! caso4 , na2so4 , k2so4
!
      else if ( drna2so4<=rh .and. rh<drk2so4 ) then
        scase = 'o6'
        call calco6                  ! caso4 , k2so4
!
      else if ( drk2so4<=rh ) then
        scase = 'o7'
        call calco7                  ! caso4
      end if
!
!   *** sulfate poor: rso4>2; (dust + sodium) rich: r(cr+na)>2; dust
!   poor: rcr<2.
    else if ( so4rat>=2.0 .and. crnarat>=2.0 ) then
!
      if ( crrat<=2.0 ) then
!
        if ( metstbl==1 ) then
          scase = 'm8'
          call calcm8                ! only liquid (metastable)
!
        else if ( rh<drnh4no3 ) then
          scase = 'm1'
          call calcm1               ! caso4 , nh4no3 , nh4cl , mgso4 , na2so4 , k2so4 , nacl , nano3
!
        else if ( drnh4no3<=rh .and. rh<drnano3 ) then
          scase = 'm2'
          call calcm2               ! caso4 , nh4cl , mgso4 , na2so4 , k2so4 , nacl , nano3
!
        else if ( drnano3<=rh .and. rh<drnacl ) then
          scase = 'm3'
          call calcm3               ! caso4 , nh4cl , mgso4 , na2so4 , k2so4 , nacl
!
        else if ( drnacl<=rh .and. rh<drnh4cl ) then
          scase = 'm4'
          call calcm4               ! caso4 , nh4cl , mgso4 , na2so4 , k2so4
!
        else if ( drnh4cl<=rh .and. rh<drmgso4 ) then
          scase = 'm5'
          call calcm5               ! caso4 , mgso4 , na2so4 , k2so4
!
        else if ( drmgso4<=rh .and. rh<drna2so4 ) then
          scase = 'm6'
          call calcm6               ! caso4 , na2so4 , k2so4
!
        else if ( drna2so4<=rh .and. rh<drk2so4 ) then
          scase = 'm7'
          call calcm7               ! caso4 , k2so4
!
        else if ( drk2so4<=rh ) then
          scase = 'm8'
          call calcm8               ! caso4
        end if
!     call calchco3
!
!     *** sulfate poor: rso4>2; (dust + sodium) rich: r(cr+na)>2; dust
!     poor: rcr<2.
      else if ( crrat>2.0 ) then
!
        if ( metstbl==1 ) then
          scase = 'p13'
          call calcp13                ! only liquid (metastable)
!
        else if ( rh<drcacl2 ) then
          scase = 'p1'
          call calcp1                ! caso4 , ca(no3)2 , cacl2 , k2so4 , kno3 , kcl , mgso4 ,
!       ! mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
        else if ( drcacl2<=rh .and. rh<drmgcl2 ) then
          scase = 'p2'
          call calcp2               ! caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 ,
!       ! mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
        else if ( drmgcl2<=rh .and. rh<drcano32 ) then
          scase = 'p3'
          call calcp3               ! caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 ,
!       ! mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl
!
        else if ( drcano32<=rh .and. rh<drmgno32 ) then
          scase = 'p4'
          call calcp4               ! caso4 , k2so4 , kno3 , kcl , mgso4 ,
!       ! mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl
!
        else if ( drmgno32<=rh .and. rh<drnh4no3 ) then
          scase = 'p5'
          call calcp5               ! caso4 , k2so4 , kno3 , kcl , mgso4 ,
!       ! nano3 , nacl , nh4no3 , nh4cl
!
        else if ( drnh4no3<=rh .and. rh<drnano3 ) then
          scase = 'p6'
          call calcp6               ! caso4 , k2so4 , kno3 , kcl , mgso4 , nano3 , nacl , nh4cl
!
        else if ( drnano3<=rh .and. rh<drnacl ) then
          scase = 'p7'
          call calcp7               ! caso4 , k2so4 , kno3 , kcl , mgso4 , nacl , nh4cl
!
        else if ( drnacl<=rh .and. rh<drnh4cl ) then
          scase = 'p8'
          call calcp8               ! caso4 , k2so4 , kno3 , kcl , mgso4 , nh4cl
!
        else if ( drnh4cl<=rh .and. rh<drkcl ) then
          scase = 'p9'
          call calcp9               ! caso4 , k2so4 , kno3 , kcl , mgso4
!
        else if ( drkcl<=rh .and. rh<drmgso4 ) then
          scase = 'p10'
          call calcp10               ! caso4 , k2so4 , kno3 , mgso4
!
        else if ( drmgso4<=rh .and. rh<drkno3 ) then
          scase = 'p11'
          call calcp11               ! caso4 , k2so4 , kno3
!
        else if ( drkno3<=rh .and. rh<drk2so4 ) then
          scase = 'p12'
          call calcp12               ! caso4 , k2so4
!
        else if ( drk2so4<=rh ) then
          scase = 'p13'
          call calcp13               ! caso4
        end if
!     call calchco3
      end if
!
!   *** sulfate rich (no acid): 1<rso4<2;
!
    else if ( 1.0<=so4rat .and. so4rat<2.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'l9'
        call calcl9                 ! only liquid (metastable)
!
      else if ( rh<drnh4hs4 ) then
        scase = 'l1'
        call calcl1                ! caso4 ,k2so4 ,mgso4 ,khso4 ,na2so4 ,(nh4)2so4 ,nahso4 ,nh4hso4 ,lc
!
      else if ( drnh4hs4<=rh .and. rh<drnahso4 ) then
        scase = 'l2'
        call calcl2                ! caso4 ,k2so4 ,mgso4 ,khso4 ,na2so4 ,(nh4)2so4 ,nahso4 ,lc
!
      else if ( drnahso4<=rh .and. rh<drlc ) then
        scase = 'l3'
        call calcl3                ! caso4 ,k2so4 ,mgso4 ,khso4 ,na2so4 ,(nh4)2so4 ,lc
!
      else if ( drlc<=rh .and. rh<drnh42s4 ) then
        scase = 'l4'
        call calcl4                ! caso4 ,k2so4 ,mgso4 ,khso4 ,na2so4 ,(nh4)2so4
!
      else if ( drnh42s4<=rh .and. rh<drkhso4 ) then
        scase = 'l5'
        call calcl5                ! caso4 ,k2so4 ,mgso4 ,khso4 ,na2so4
!
      else if ( drkhso4<=rh .and. rh<drmgso4 ) then
        scase = 'l6'
        call calcl6                ! caso4 ,k2so4 ,mgso4 ,na2so4
!
      else if ( drmgso4<=rh .and. rh<drna2so4 ) then
        scase = 'l7'
        call calcl7                ! caso4 ,k2so4 ,na2so4
!
      else if ( drna2so4<=rh .and. rh<drk2so4 ) then
        scase = 'l8'
        call calcl8                ! caso4 ,k2so4
!
      else if ( drk2so4<=rh ) then
        scase = 'l9'
        call calcl9                ! caso4 &
      end if
!
      call calcnha                ! minor species: hno3 , hcl
      call calcnh3                !                nh3
!
!   *** sulfate super rich (free acid): rso4<1;
!
    else if ( so4rat<1.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'k4'
        call calck4                  ! only liquid (metastable)
!
      else if ( rh<drnh4hs4 ) then                 ! rh < 0.4
        scase = 'k1'
        call calck1               ! nh4hso4 ,nahso4 ,khso4 ,caso4
!
      else if ( drnh4hs4<=rh .and. rh<drnahso4 ) then
        scase = 'k2'
        call calck2               ! nahso4 ,khso4 ,caso4
!
      else if ( drnahso4<=rh .and. rh<drkhso4 ) then
        scase = 'k3'
        call calck3               ! khso4 ,caso4    0.52 < rh < 0.86
!
      else if ( drkhso4<=rh ) then
        scase = 'k4'
        call calck4               ! caso4
      end if
!
      call calcnha                  ! minor species: hno3 , hcl
      call calcnh3                  !                nh3
!
    end if
  end subroutine isrp4f
!
!======================================================================
!
! *** isorropia code
! *** subroutine calca2
! *** case a2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat >= 2.0)
!     2. liquid aerosol phase only possible
!
!     for calculations , a bisection is performed towards x , the
!     amount of hydrogen ions (h+) found in the liquid phase.
!     for each estimation of h+, function funcb2a calculates the
!     concentration of ions from the nh3(gas) - nh4+(liq) equilibrium.
!     electroneutrality is used as the objective function.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calca2
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: dx , omehi , omelo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.           ! outer loop activity calculation flag
    omelo = tiny1              ! low  limit: solution is very basic
    omehi = 2.0_rkx*w(2)        ! high limit: from nh4+ -> nh3(g) + h+(aq)
!
! *** calculate water content *****************************************
!
    molal(5) = w(2)
    molal(6) = zero
    call calcmr
!
! *** initial values for bisection ************************************
!
    x1 = omehi
    y1 = funca2(x1)
    y2 = y1
    if ( abs(y1)<=eps ) return
!
! *** root tracking ; for the range of hi and lo **********************
!
    dx = (omehi-omelo)/float(ndiv)
    do i = 1 , ndiv
      x2 = max(x1-dx,omelo)
      y2 = funca2(x2)
      if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100     ! (y1*y2 < zero)
      x1 = x2
      y1 = y2
    end do
    if ( abs(y2)<=eps ) then
      return
    else
      call pusherr(0001,'calca2')          ! warning error: no solution
      return
    end if
!
! *** perform bisection ***********************************************
!
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funca2(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calca2')         ! warning error: no convergence
!
! *** converged ; return **********************************************
200 continue
    x3 = 0.5_rkx*(x1+x2)
    y3 = funca2(x3)
  end subroutine calca2
!
!======================================================================
!
! *** isorropia code
! *** function funca2
! *** case a2
!     function that solves the system of equations for case a2 ;
!     and returns the value of the zeroed function in funca2.
!
!======================================================================
!
  real(rkx) function funca2(omegi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a1 , a2 , a3 , denom , omegi , psi , zeta
    integer(ik4) :: i
    real(rkx) :: lamda
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    psi = w(2)              ! initial amount of (nh4)2so4 in solution
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a1 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
      a2 = xk2*r*temp/xkw*(gama(8)/gama(9))**2.
      a3 = xkw*rh*water*water
!
      lamda = psi/(a1/omegi+one)
      zeta = a3/omegi
!
!   *** speciation & water content
!   ***************************************
      molal(1) = omegi                                            ! hi
      molal(5) = max(psi-lamda,tiny1)                               ! so4i
      molal(3) = max(w(3)/(one/a2/omegi+one),2.*molal(5))         ! nh4i
      molal(6) = lamda                                            ! hso4i
      gnh3 = max(w(3)-molal(3),tiny1)                              ! nh3gi
      coh = zeta                                                  ! ohi
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    denom = (2.0*molal(5)+molal(6))
    funca2 = (molal(3)/denom-one) + molal(1)/denom
  end function funca2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calca1
! *** case a1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4
!
!     a simple material balance is performed , and the solid (nh4)2so4
!     is calculated from the sulfates. the excess ammonia remains in
!     the gas phase.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calca1
    use mod_che_common_isorropia
    implicit none
    cnh42s4 = w(2)
    gnh3 = max(w(3)-2.0*cnh42s4,zero)
  end subroutine calca1
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb4
! *** case b4
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. liquid aerosol phase only possible
!
!     for calculations , a bisection is performed with respect to h+.
!     the objective function is the difference between the estimated h+
!     and that calculated from electroneutrality.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb4
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: ak1 , bb , bet , cc , dd , gam
    integer(ik4) :: i
!
! *** solve equations **************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
! *** calculate water content ******************************************
!
    call calcb1a           ! get dry salt content , and use for water.
    molalr(13) = clc
    molalr(9) = cnh4hs4
    molalr(4) = cnh42s4
    clc = zero
    cnh4hs4 = zero
    cnh42s4 = zero
    water = molalr(13)/m0(13) + molalr(9)/m0(9) + molalr(4)/m0(4)
!
    molal(3) = w(3)       ! nh4i
!
    do i = 1 , nsweep
      ak1 = xk1*((gama(8)/gama(7))**2)*(water/gama(7))
      bet = w(2)
      gam = molal(3)
!
      bb = bet + ak1 - gam
      cc = -ak1*bet
      dd = bb*bb - 4._rkx*cc
!
!   *** speciation & water content
!   ***************************************
      molal(5) = max(tiny1,min(0.5_rkx*(-bb+sqrt(dd)),w(2)))         ! so4i
      molal(6) = max(tiny1,min(w(2)-molal(5),w(2)))              ! hso4i
      molal(1) = max(tiny1,min(ak1*molal(6)/molal(5),w(2)))      ! hi
      call calcmr                                              ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.calain ) return
      call calcact
    end do
  end subroutine calcb4
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb3
! *** case b3
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. both liquid & solid phase is possible
!     3. solids possible: (nh4)2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb3
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: tlc , tnh42s4 , tnh4hs4 , x , y
!
! *** calculate equivalent amount of hso4 and so4
! ***********************
    x = max(2*w(2)-w(3),zero)      ! equivalent nh4hso4
    y = max(w(3)-w(2),zero)        ! equivalent nh42so4
!
! *** calculate species according to relative abundance of hso4
! *********
    if ( x<y ) then               ! lc is the min (x ,y)
      scase = 'b3 ; subcase 1'
      tlc = x
      tnh42s4 = y - x
      call calcb3a(tlc,tnh42s4)           ! lc + (nh4)2so4
    else
      scase = 'b3 ; subcase 2'
      tlc = y
      tnh4hs4 = x - y
      call calcb3b(tlc,tnh4hs4)           ! lc + nh4hso4
    end if
  end subroutine calcb3
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb3a
! *** case b3 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate rich (1.0 < sulrat < 2.0)
!     2. both liquid & solid phase is possible
!     3. solids possible: (nh4)2so4
!
!     for calculations , a bisection is performed towards zeta , the
!     amount of solid (nh4)2so4 dissolved in the liquid phase.
!     for each estimation of zeta , function funcb3a calculates the
!     amount of h+ produced (based on the so4 released into the
!     solution). the solubility product of (nh4)2so4 is used as the
!     objective function.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb3a(tlc,tnh42s4)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: dz , tlc , tnh42s4 , y1 , y2 , y3 , &
         yhi , ylo , z1 , z2 , z3 , zhi , zk , zlo
    integer(ik4) :: i
!
    calaou = .true.           ! outer loop activity calculation flag
    zlo = zero                ! min dissolved (nh4)2so4
    zhi = tnh42s4             ! max dissolved (nh4)2so4
!
! *** initial values for bisection (dissolved (nh4)2so4 ****************
!
    z1 = zlo
    y1 = funcb3a(z1,tlc,tnh42s4)
    if ( abs(y1)<=eps ) return
    ylo = y1
    y2 = y1
!
! *** root tracking ; for the range of hi and lo ***********************
!
    dz = (zhi-zlo)/float(ndiv)
    do i = 1 , ndiv
      z2 = z1 + dz
      y2 = funcb3a(z2,tlc,tnh42s4)
      if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100     ! (y1*y2 < zero)
      z1 = z2
      y1 = y2
    end do
!
! *** no subdivision with solution found
!
    yhi = y1                        ! save y-value at hi position
    if ( abs(y2)<eps ) then       ! x2 is a solution
      return
!
!   *** { ylo , yhi } < 0.0 the solution is always undersaturated with
!   lc
    else if ( ylo<zero .and. yhi<zero ) then
      z1 = zhi
      z2 = zhi
      go to 200
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   lc
    else if ( ylo>zero .and. yhi>zero ) then
      z1 = zlo
      z2 = zlo
      go to 200
    else
      call pusherr(0001,'calcb3a')          ! warning error: no solution
      return
    end if
!
! *** perform bisection ***********************************************
!
100 continue
    do i = 1 , maxit
      z3 = 0.5_rkx*(z1+z2)
      y3 = funcb3a(z3,tlc,tnh42s4)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        z2 = z3
      else
        y1 = y3
        z1 = z3
      end if
      if ( abs(z2-z1)<=eps*z1 ) go to 200
    end do
    call pusherr(0002,'calcb3a')         ! warning error: no convergence
!
! *** converged ; return
! ************************************************
200 continue
    zk = 0.5_rkx*(z1+z2)
    y3 = funcb3a(zk,tlc,tnh42s4)
  end subroutine calcb3a
!
!======================================================================
!
! *** isorropia code
! *** function funcb3a
! *** case b3 ; subcase 1
!     function that solves the system of equations for case b3
!     and returns the value of the zeroed function in funca3.
!
!======================================================================
!
  real(rkx) function funcb3a(zk,y,x)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: dd , grat1 , x , y , zk
    integer(ik4) :: i
    real(rkx) :: kk
!
! *** solve equations ; with iterations for activity coef. ************
!
    frst = .true.
    calain = .true.
    do i = 1 , nsweep
      grat1 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
      dd = sqrt((zk+grat1+y)**24.0*y*grat1)
      kk = 0.5_rkx*(-(zk+grat1+y)+dd)
!
!   *** speciation & water content
!   ***************************************
      molal(1) = kk                    ! hi
      molal(5) = kk + zk + y           ! so4i
      molal(6) = max(y-kk,tiny1)         ! hso4i
      molal(3) = 3.0*y + 2*zk          ! nh4i
      cnh42s4 = x - zk                 ! solid (nh4)2so4
      call calcmr                      ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
! funcb3a = ( so4i*nh4i**2 )/( xk7*(water/gama(4))**3 )
    funcb3a = molal(5)*molal(3)**2
    funcb3a = funcb3a/(xk7*(water/gama(4))**3) - one
  end function funcb3a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb3b
! *** case b3 ; subcase 2
!
!     the main characteristics of this regime are:
!     1. sulfate rich (1.0 < sulrat < 2.0)
!     2. liquid phase only is possible
!
!     speciation calculations is based on the hso4 <--> so4 equilibrium.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb3b(y,x)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: dd , grat1 , x , y
    integer(ik4) :: i
    real(rkx) :: kk
!
    calaou = .false.          ! outer loop activity calculation flag
    frst = .false.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      grat1 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
      dd = sqrt((grat1+y)**24.0*(x+y)*grat1)
      kk = 0.5_rkx*(-(grat1+y)+dd)
!
!   *** speciation & water content
!   ***************************************
      molal(1) = kk                       ! hi
      molal(5) = y + kk                   ! so4i
      molal(6) = max(x+y-kk,tiny1)          ! hso4i
      molal(3) = 3.0*y + x                ! nh4i
      call calcmr                         ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.calain ) return
      call calcact
    end do
  end subroutine calcb3b
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb2
! *** case b2
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : lc , (nh4)2so4
!
!     there are two possible regimes here , depending on the sulfate ratio:
!     1. when both lc and (nh4)2so4 are possible (subroutine calcb2a)
!     2. when only lc is possible (subroutine calcb2b).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb2
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: x , y
!
! *** calculate equivalent amount of hso4 and so4
! ***********************
    x = max(2*w(2)-w(3),tiny1)      ! equivalent nh4hso4
    y = max(w(3)-w(2),tiny1)        ! equivalent nh42so4
!
! *** calculate species according to relative abundance of hso4
! *********
    if ( x<=y ) then               ! lc is the min (x ,y)
      scase = 'b2 ; subcase 1'
      call calcb2a(x,y-x)           ! lc + (nh4)2so4 possible
    else
      scase = 'b2 ; subcase 2'
      call calcb2b(y,x-y)           ! lc only possible
    end if
  end subroutine calcb2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb2
! *** case b2 ; subcase a.
!
!     the main characteristics of this regime are:
!     1. sulfate rich (1.0 < sulrat < 2.0)
!     2. solid phase only possible
!     3. solids possible: lc , (nh4)2so4
!
!     there are two possible regimes here , depending on relative humidity:
!     1. when rh >= mdrh ; liquid phase possible (mdrh region)
!     2. when rh < mdrh  ; only solid phase possible
!
!     for solid calculations , a material balance based on the stoichimetric
!     proportion of ammonium and sulfate is done to calculate the amount
!     of lc and (nh4)2so4 in the solid phase.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb2a(tlc,tnh42s4)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: tlc , tnh42s4
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmlcas ) then
      scase = 'b2 ; subcase a1'         ! solids possible only
      clc = tlc
      cnh42s4 = tnh42s4
      scase = 'b2 ; subcase a1'
    else
      scase = 'b2 ; subcase a2'
      call calcb2a2(tlc,tnh42s4)         ! liquid & solid phase possible
      scase = 'b2 ; subcase a2'
    end if
  end subroutine calcb2a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb2a2
! *** case b2 ; subcase a2.
!
!     the main characteristics of this regime are:
!     1. sulfate rich (1.0 < sulrat < 2.0)
!     2. solid phase only possible
!     3. solids possible: lc , (nh4)2so4
!
!     this is the case where the relative humidity is in the mutual
!     drh region. the solution is assumed to be the sum of two weighted
!     solutions ; the solid phase only (subroutine calcb2a1) and the
!     the solid with liquid phase (subroutine calcb3).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb2a2(tlc,tnh42s4)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: clco , cnh42so , onemwf , tlc , tnh42s4 , wf
!
! *** find weight factor **********************************************
!
    if ( wftyp==0 ) then
      wf = zero
    else if ( wftyp==1 ) then
      wf = 0.5_rkx
    else
      wf = (drlc-rh)/(drlc-drmlcas)
    end if
    onemwf = one - wf
!
! *** find first section ; dry one ************************************
!
    clco = tlc                           ! first (dry) solution
    cnh42so = tnh42s4
!
! *** find second section ; dry & liquid ******************************
!
    clc = zero
    cnh42s4 = zero
    call calcb3                          ! second (liquid) solution
!
! *** find solution at mdrh by weighting dry & liquid solutions.
!
    molal(1) = onemwf*molal(1)                                     ! h+
    molal(3) = onemwf*(2._rkx*(cnh42so-cnh42s4)+3._rkx*(clco-clc))     ! nh4+
    molal(5) = onemwf*(cnh42so-cnh42s4+clco-clc)                   ! so4--
    molal(6) = onemwf*(clco-clc)                                   ! hso4-
!
    water = onemwf*water
!
    clc = wf*clco + onemwf*clc
    cnh42s4 = wf*cnh42so + onemwf*cnh42s4
  end subroutine calcb2a2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb2
! *** case b2 ; subcase b
!
!     the main characteristics of this regime are:
!     1. sulfate rich (1.0 < sulrat < 2.0)
!     2. both liquid & solid phase is possible
!     3. solids possible: lc
!
!     for calculations , a bisection is performed towards zeta , the
!     amount of solid lc dissolved in the liquid phase.
!     for each estimation of zeta , function funcb2a calculates the
!     amount of h+ produced (based on the hso4 , so4 released into the
!     solution). the solubility product of lc is used as the objective
!     function.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb2b(tlc,tnh4hs4)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: dx , tlc , tnh4hs4 , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo , zhi , zlo
    integer(ik4) :: i
!
    calaou = .true.         ! outer loop activity calculation flag
    zlo = zero
    zhi = tlc               ! high limit: all of it in liquid phase
!
! *** initial values for bisection
! **************************************
    x1 = zhi
    y1 = funcb2b(x1,tnh4hs4,tlc)
    if ( abs(y1)<=eps ) return
    yhi = y1                          ! save y-value at hi position
    y2 = y1
!
! *** root tracking ; for the range of hi and lo
! ************************
    dx = (zhi-zlo)/ndiv
    do i = 1 , ndiv
      x2 = x1 - dx
      y2 = funcb2b(x2,tnh4hs4,tlc)
      if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100     ! (y1*y2 < zero)
      x1 = x2
      y1 = y2
    end do
!
! *** no subdivision with solution found
!
    ylo = y1                        ! save y-value at lo position
    if ( abs(y2)<eps ) then       ! x2 is a solution
      return
!
!   *** { ylo , yhi } < 0.0 the solution is always undersaturated with
!   lc
    else if ( ylo<zero .and. yhi<zero ) then
      x1 = zhi
      x2 = zhi
      go to 200
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   lc
    else if ( ylo>zero .and. yhi>zero ) then
      x1 = zlo
      x2 = zlo
      go to 200
    else
      call pusherr(0001,'calcb2b')          ! warning error: no solution
      return
    end if
!
! *** perform bisection
! *************************************************
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcb2b(x3,tnh4hs4,tlc)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcb2b')         ! warning error: no convergence
!
! *** converged ; return
! ************************************************
200 continue
    x3 = 0.5_rkx*(x1+x2)
    y3 = funcb2b(x3,tnh4hs4,tlc)
  end subroutine calcb2b
!
!======================================================================
!
! *** isorropia code
! *** function funcb2b
! *** case b2 ;
!     function that solves the system of equations for case b2 ; subcase 2
!     and returns the value of the zeroed function in funcb2b.
!
!======================================================================
!
  real(rkx) function funcb2b(x,tnh4hs4,tlc)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: delta , grat2 , omega , parm , tlc , tnh4hs4 , x
    integer(ik4) :: i
!
! *** solve equations **************************************************
!
    frst = .true.
    calain = .true.
    do i = 1 , nsweep
      grat2 = xk1*water*(gama(8)/gama(7))**2/gama(7)
      parm = x + grat2
      delta = parm*parm + 4.0*(x+tnh4hs4)*grat2    ! diakrinousa
      omega = 0.5_rkx*(-parm+sqrt(delta))              ! thetiki riza (ie:h+>0)
!
!   *** speciation & water content
!   ***************************************
      molal(1) = omega                             ! hi
      molal(3) = 3.0*x + tnh4hs4                   ! nh4i
      molal(5) = x + omega                         ! so4i
      molal(6) = max(x+tnh4hs4-omega,tiny1)          ! hso4i
      clc = max(tlc-x,zero)                         ! solid lc
      call calcmr                                  ! water content
!
!   *** calculate activities or terminate internal loop
!   ******************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function
! **************************************
! funcb2b = ( nh4i**3so4i*hso4i )/( xk13*(water/gama(13))**5. )
    funcb2b = (molal(3)**3)*molal(5)*molal(6)
    funcb2b = funcb2b/(xk13*(water/gama(13))**5) - one
  end function funcb2b
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb1
! *** case b1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : lc , (nh4)2so4 , nh4hso4
!
!     there are two possible regimes here , depending on relative humidity:
!     1. when rh >= mdrh ; liquid phase possible (mdrh region)
!     2. when rh < mdrh  ; only solid phase possible (subroutine calcb1a)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmlcab ) then
      scase = 'b1 ; subcase 1'
      call calcb1a                 ! solid phase only possible
      scase = 'b1 ; subcase 1'
    else
      scase = 'b1 ; subcase 2'
      call calcb1b                 ! liquid & solid phase possible
      scase = 'b1 ; subcase 2'
    end if
  end subroutine calcb1
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb1a
! *** case b1 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate rich
!     2. there is no liquid phase
!     3. solids possible: lc , { (nh4)2so4  xor  nh4hso4 } (one of two
!                         but not both)
!
!     a simple material balance is performed , and the amount of lc
!     is calculated from the (nh4)2so4 and nh4hso4 which is least
!     abundant (stoichimetrically). the remaining excess of salt
!     is mixed with the lc.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: x , y
!
! *** setup parameters ************************************************
!
    x = 2*w(2) - w(3)       ! equivalent nh4hso4
    y = w(3) - w(2)         ! equivalent (nh4)2so4
!
! *** calculate composition *******************************************
!
    if ( x<=y ) then        ! lc is the min (x ,y)
      clc = x               ! nh4hso4 >= (nh4)2s04
      cnh4hs4 = zero
      cnh42s4 = y - x
    else
      clc = y               ! nh4hso4 <  (nh4)2s04
      cnh4hs4 = x - y
      cnh42s4 = zero
    end if
  end subroutine calcb1a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcb1b
! *** case b1 ; subcase 2
!
!     the main characteristics of this regime are:
!     1. sulfate rich
!     2. there is both a liquid & solid phase
!     3. solids possible: lc , { (nh4)2so4  xor  nh4hso4 } (one of two
!                         but not both)
!
!     this is the case where the relative humidity is in the mutual
!     drh region. the solution is assumed to be the sum of two weighted
!     solutions ; the solid phase only (subroutine calcb1a) and the
!     the solid with liquid phase (subroutine calcb2).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcb1b
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: clco , cnh42so , cnh4hso , onemwf , wf
!
! *** find weight factor **********************************************
!
    if ( wftyp==0 ) then
      wf = zero
    else if ( wftyp==1 ) then
      wf = 0.5_rkx
    else
      wf = (drnh4hs4-rh)/(drnh4hs4-drmlcab)
    end if
    onemwf = one - wf
!
! *** find first section ; dry one ************************************
!
    call calcb1a
    clco = clc                     ! first (dry) solution
    cnh42so = cnh42s4
    cnh4hso = cnh4hs4
!
! *** find second section ; dry & liquid ******************************
!
    clc = zero
    cnh42s4 = zero
    cnh4hs4 = zero
    call calcb2                    ! second (liquid) solution
!
! *** find solution at mdrh by weighting dry & liquid solutions.
!
    molal(1) = onemwf*molal(1)                                     ! h+
    molal(3) = onemwf*(2._rkx*(cnh42so-cnh42s4)+(cnh4hso-cnh4hs4)           &
               +3._rkx*(clco-clc))                                 ! nh4+
    molal(5) = onemwf*(cnh42so-cnh42s4+clco-clc)                   ! so4--
    molal(6) = onemwf*(cnh4hso-cnh4hs4+clco-clc)                   ! hso4-
!
    water = onemwf*water
!
    clc = wf*clco + onemwf*clc
    cnh42s4 = wf*cnh42so + onemwf*cnh42s4
    cnh4hs4 = wf*cnh4hso + onemwf*cnh4hs4
  end subroutine calcb1b

!======================================================================
!
! *** isorropia code
! *** subroutine calcc2
! *** case c2
!
!     the main characteristics of this regime are:
!     1. sulfate rich , free acid (sulrat < 1.0)
!     2. there is only a liquid phase
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcc2
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: bb , cc , parm , psi
    integer(ik4) :: i
    real(rkx) :: lamda , kapa
!
    calaou = .true.          ! outer loop activity calculation flag
    frst = .true.
    calain = .true.
!
! *** solve equations **************************************************
!
    lamda = w(3)              ! nh4hso4 initially in solution
    psi = w(2) - w(3)         ! h2so4 in solution
    do i = 1 , nsweep
      parm = water*xk1/gama(7)*(gama(8)/gama(7))**2.
      bb = psi + parm
      cc = -parm*(lamda+psi)
      kapa = 0.5_rkx*(-bb+sqrt(bb*bb-4.0*cc))
!
!   *** speciation & water content
!   ***************************************
      molal(1) = psi + kapa                                ! hi
      molal(3) = lamda                                     ! nh4i
      molal(5) = kapa                                      ! so4i
      molal(6) = max(lamda+psi-kapa,tiny1)                   ! hso4i
      ch2so4 = max(molal(5)+molal(6)-molal(3),zero)        ! free h2so4
      call calcmr                                          ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.calain ) return
      call calcact
    end do
  end subroutine calcc2


!======================================================================
!
! *** isorropia code
! *** subroutine calcc1
! *** case c1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , free acid (sulrat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible: nh4hso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcc1
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: dx , x1 , x2 , x3 , y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
    real(rkx) :: klo , khi
!
    calaou = .true.      ! outer loop activity calculation flag
    klo = tiny1
    khi = w(3)
!
! *** initial values for bisection *************************************
!
    x1 = klo
    y1 = funcc1(x1)
    if ( abs(y1)<=eps ) return
    ylo = y1
    y2 = y1
!
! *** root tracking ; for the range of hi and lo ***********************
!
    dx = (khi-klo)/float(ndiv)
    do i = 1 , ndiv
      x2 = x1 + dx
      y2 = funcc1(x2)
      if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100      ! (y1*y2  <  zero)
      x1 = x2
      y1 = y2
    end do
!
! *** no subdivision with solution found
!
    yhi = y2                   ! save y-value at hi position
    if ( abs(y2)<eps ) then       ! x2 is a solution
      return
!
!   *** { ylo , yhi } < 0.0  solution is always undersaturated with
!   nh4hs04
    else if ( ylo<zero .and. yhi<zero ) then
      return
!
!   *** { ylo , yhi } > 0.0 solution is always supersaturated with
!   nh4hs04
    else if ( ylo>zero .and. yhi>zero ) then
      x1 = klo
      x2 = klo
      go to 200
    else
      call pusherr(0001,'calcc1')          ! warning error: no solution
      return
    end if
!
! *** perform bisection of dissolved nh4hso4 **************************
!
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcc1(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcc1')         ! warning error: no convergence
!
! *** converged ; return ***********************************************
!
200 continue
    x3 = 0.5_rkx*(x1+x2)
    y3 = funcc1(x3)
!
  end subroutine calcc1
!
!======================================================================
!
! *** isorropia code
! *** function funcc1
! *** case c1 ;
!     function that solves the system of equations for case c1
!     and returns the value of the zeroed function in funcc1.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funcc1(kapa)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: bb , cc , par1 , par2 , psi
    integer(ik4) :: i
    real(rkx) :: kapa , lamda
!
! *** solve equations **************************************************
!
    frst = .true.
    calain = .true.
    par2 = one
!
    psi = w(2) - w(3)
    do i = 1 , nsweep
      par1 = xk1*water/gama(7)*(gama(8)/gama(7))**2
      par2 = xk12*(water/gama(9))**2
      bb = psi + par1
      cc = -par1*(psi+kapa)
      lamda = 0.5_rkx*(-bb+sqrt(bb*bb-4*cc))
!
!   *** save concentrations in molal array
!   *******************************
      molal(1) = psi + lamda                     ! hi
      molal(3) = kapa                            ! nh4i
      molal(5) = lamda                           ! so4i
      molal(6) = max(zero,psi+kapa-lamda)         ! hso4i
      cnh4hs4 = max(w(3)-molal(3),zero)          ! solid nh4hso4
      ch2so4 = max(psi,zero)                      ! free h2so4
      call calcmr                                ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate zero function
! *******************************************
! funcc1 = (nh4i*hso4i/par2) - one
    funcc1 = (molal(3)*molal(6)/par2) - one
  end function funcc1
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcd3
! *** case d3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. there is olny a liquid phase
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcd3
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , p4 , psi4hi , psi4lo , &
         x1 , x2 , x3 , y1 , y2 , y3 , yhi , ylo , yy
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcd1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4no3                 ! save from calcd1 run
    chi2 = cnh42s4
    chi3 = ghno3
    chi4 = gnh3
!
    psi1 = cnh4no3                 ! assign initial psi's
    psi2 = chi2
    psi3 = zero
    psi4 = zero
!
    molal(5) = zero
    molal(6) = zero
    molal(3) = psi1
    molal(7) = psi1
    call calcmr                    ! initial water
!
    calaou = .true.                ! outer loop activity calculation flag
    psi4lo = tiny1                  ! low  limit
    psi4hi = chi4                  ! high limit
!
! *** initial values for bisection ************************************
!
 100  x1 = psi4lo
    y1 = funcd3(x1)
    if ( abs(y1)<=eps ) return
    ylo = y1                   ! save y-value at hi position
    y2 = y1
!
! *** root tracking ; for the range of hi and lo **********************
!
    dx = (psi4hi-psi4lo)/float(ndiv)
    do i = 1 , ndiv
      x2 = x1 + dx
      y2 = funcd3(x2)
      if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 200     ! (y1*y2 < zero)
      x1 = x2
      y1 = y2
    end do
!
! *** no subdivision with solution found
!
    yhi = y1                        ! save y-value at hi position
    if ( abs(y2)<eps ) then       ! x2 is a solution
      return
!
!   *** { ylo , yhi } < 0.0 the solution is always undersaturated with
!   nh3 & phyically i dont know when this might happen , but i have put
!   this & brach in for completeness. i assume there is no solution;
!   all no3 goes to the gas phase.
!
    else if ( ylo<zero .and. yhi<zero ) then
      p4 = tiny1    ! psi4lo ! chi4
      yy = funcd3(p4)
      go to 400
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   nh3 & thi happens when sul.rat. = 2.0 , so some nh4+ from sulfate
!   evaporates and goes to the gas phase ; so i redefine the lo and hi
!   limits of psi4 and proceed again with root tracking.
!
    else if ( ylo>zero .and. yhi>zero ) then
      psi4hi = psi4lo
      psi4lo = psi4lo - 0.1*(psi1+psi2)    ! no solution; some nh3 evaporates
      if ( psi4lo<-(psi1+psi2) ) then
        call pusherr(0001,'calcd3')         ! warning error: no solution
        return
      else
        molal(5) = zero
        molal(6) = zero
        molal(3) = psi1
        molal(7) = psi1
        call calcmr                      ! initial water
        go to 100                          ! redo root tracking
      end if
    end if
!
! *** perform bisection ***********************************************
!
200 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcd3(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*abs(x1) ) go to 300
    end do
    call pusherr(0002,'calcd3')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
300 continue
    x3 = 0.5_rkx*(x1+x2)
    y3 = funcd3(x3)
!
! *** calculate hso4 speciation and return
! *******************************
400 continue
    if ( molal(1)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcd3
!
!======================================================================
!
! *** isorropia code
! *** function funcd3
! *** case d3
!     function that solves the system of equations for case d3 ;
!     and returns the value of the zeroed function in funcd3.
!
!======================================================================
!
  real(rkx) function funcd3(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: abb , ahi , bb , denm , p4
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    psi4 = p4
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a2 = water/gama(4)
      if ( a2 > 1.0e-3_rkx ) then
        a2 = a2*a2*a2
        a2 = xk7*a2
      else
        a2 = 0.0_rkx
      end if
      a3 = water/gama(10)
      if ( a3 > 1.0e-5_rkx ) then
        a3 = a3*a3
        a3 = xk4*r*temp*a3
      else
        a3 = 0.0_rkx
      end if
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a7 = water
      if ( a7 > 1.0e-5_rkx ) then
        a7 = a7*a7
        a7 = xkw*rh*a7
      else
        a7 = 0.0_rkx
      end if
      if ( a2 < epsilon(1.0_rkx) ) a2 = 0.0_rkx
      if ( a3 < epsilon(1.0_rkx) ) a3 = 0.0_rkx
      if ( a4 < epsilon(1.0_rkx) ) a4 = 0.0_rkx
      if ( a7 < epsilon(1.0_rkx) ) a7 = 0.0_rkx
!
      psi3 = a3*a4*chi3*(chi4-psi4) - psi1*(2._rkx*psi2+psi1+psi4)
      psi3 = psi3/(a3*a4*(chi4-psi4)+2._rkx*psi2+psi1+psi4)
      psi3 = min(max(psi3,zero),chi3)
!
      bb = psi4 - psi3
      if ( bb < epsilon(1.0_rkx) ) bb = 0.0_rkx

!ol ahi  = 0.5d0*(-bb + sqrt(bb*bb + 4.d0*a7)) ! this is correct also
!   ahi  = 2.0*a7/(bb+sqrt(bb*bb + 4.d0*a7)) ! avoid overflow when hi->0
      if ( bb > 1.0e-5_rkx ) then
        denm = bb + sqrt(bb*bb+4._rkx*a7)
      else
        denm = bb + sqrt(4._rkx*a7)
      end if
      if ( denm<=tiny1 ) then          ! avoid overflow when hi->0
        abb = abs(bb)
        denm = (bb+abb) + 2.0*a7/abb     ! taylor expansion of sqrt
      end if
      ahi = 2.0*a7/denm
!
!   *** speciation & water content
!   ***************************************
      molal(1) = ahi                                 ! hi
      molal(3) = psi1 + psi4 + 2._rkx*psi2             ! nh4i
      molal(5) = psi2                                ! so4i
      molal(6) = zero                                ! hso4i
      molal(7) = psi3 + psi1                         ! no3i
      cnh42s4 = chi2 - psi2                          ! solid (nh4)2so4
      cnh4no3 = zero                                 ! solid nh4no3
      ghno3 = chi3 - psi3                            ! gas hno3
      gnh3 = chi4 - psi4                             ! gas nh3
      call calcmr                                    ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
! funcd3 = nh4i/hi/max(gnh3 ,tiny1)/a4 - one
    funcd3 = molal(3)/molal(1)/max(gnh3,tiny1)/a4 - one
!
  end function funcd3
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcd2
! *** case d2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcd2
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , p4 , psi4hi , psi4lo , &
         x1 , x2 , x3 , y1 , y2 , y3 , yhi , ylo , yy
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcd1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4no3                 ! save from calcd1 run
    chi2 = cnh42s4
    chi3 = ghno3
    chi4 = gnh3
!
    psi1 = cnh4no3                 ! assign initial psi's
    psi2 = cnh42s4
    psi3 = zero
    psi4 = zero
!
    molal(5) = zero
    molal(6) = zero
    molal(3) = psi1
    molal(7) = psi1
    call calcmr                    ! initial water
!
    calaou = .true.                ! outer loop activity calculation flag
    psi4lo = tiny1                  ! low  limit
    psi4hi = chi4                  ! high limit
!
! *** initial values for bisection ************************************
!
 100  x1 = psi4lo
    y1 = funcd2(x1)
    y2 = y1
    if ( abs(y1)<=eps ) return
    ylo = y1                   ! save y-value at hi position
!
! *** root tracking ; for the range of hi and lo **********************
!
    dx = (psi4hi-psi4lo)/float(ndiv)
    do i = 1 , ndiv
      x2 = x1 + dx
      y2 = funcd2(x2)
      if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) then
!
!     thi is done , in case if y(psi4lo)>0 , but y(psi4lo+dx) < 0
!     (i.e.undersat)
        if ( y1<=y2 ) go to 200       ! (y1*y2 < zero)
      end if
      x1 = x2
      y1 = y2
    end do
!
! *** no subdivision with solution found
!
    yhi = y1                        ! save y-value at hi position
    if ( abs(y2)<eps ) then       ! x2 is a solution
      return
!
!   *** { ylo , yhi } < 0.0 the solution is always undersaturated with
!   nh3 & phyically i dont know when this might happen , but i have put
!   this & brach in for completeness. i assume there is no solution;
!   all no3 goes to the gas phase.
!
    else if ( ylo<zero .and. yhi<zero ) then
      p4 = tiny1    ! psi4lo ! chi4
      yy = funcd2(p4)
      go to 400
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   nh3 & thi happens when sul.rat. = 2.0 , so some nh4+ from sulfate
!   evaporates and goes to the gas phase ; so i redefine the lo and hi
!   limits of psi4 and proceed again with root tracking.
!
    else if ( ylo>zero .and. yhi>zero ) then
      psi4hi = psi4lo
      psi4lo = psi4lo - 0.1*(psi1+psi2)    ! no solution; some nh3 evaporates
      if ( psi4lo<-(psi1+psi2) ) then
        call pusherr(0001,'calcd2')         ! warning error: no solution
        return
      else
        molal(5) = zero
        molal(6) = zero
        molal(3) = psi1
        molal(7) = psi1
        call calcmr                      ! initial water
        go to 100                          ! redo root tracking
      end if
    end if
!
! *** perform bisection ***********************************************
!
200 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcd2(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*abs(x1) ) go to 300
    end do
    call pusherr(0002,'calcd2')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
300 continue
    x3 = min(x1,x2)  ! 0.5d0*(x1+x2)  ! get "low" side , it's acidic soln.
    y3 = funcd2(x3)
!
! *** calculate hso4 speciation and return
! *******************************
400 continue
    if ( molal(1)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcd2
!
!======================================================================
!
! *** isorropia code
! *** function funcd2
! *** case d2
!     function that solves the system of equations for case d2 ;
!     and returns the value of the zeroed function in funcd2.
!
!======================================================================
!
  real(rkx) function funcd2(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: abb , ahi , bb , denm , p4
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    call rstgam         ! reset activity coefficients to 0.1
    frst = .true.
    calain = .true.
    psi4 = p4
    psi2 = chi2
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a2 = xk7*(water/gama(4))**3
      a3 = xk4*r*temp*(water/gama(10))**2
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a7 = xkw*rh*water*water
!
      if ( chi2>tiny1 .and. water>tiny1 ) then
        psi14 = psi1 + psi4
        call poly3(psi14,0.25_rkx*psi14**2,-a2/4._rkx,psi2,islv)            ! psi2
        if ( islv==0 ) then
          psi2 = min(psi2,chi2)
        else
          psi2 = tiny1
        end if
      end if
!
      psi3 = a3*a4*chi3*(chi4-psi4) - psi1*(2._rkx*psi2+psi1+psi4)
      psi3 = psi3/(a3*a4*(chi4-psi4)+2._rkx*psi2+psi1+psi4)
!   psi3  = min(max(psi3 , zero), chi3)
!
      bb = psi4 - psi3    ! (bb > 0 , acidic solution , <0 alkaline)
!
!   do ot change computation scheme for h+, all others did not work
!   well.
      denm = bb + sqrt(bb*bb+4._rkx*a7)
      if ( denm<=tiny1 ) then          ! avoid overflow when hi->0
        abb = abs(bb)
        denm = (bb+abb) + 2._rkx*a7/abb     ! taylor expansion of sqrt
      end if
      ahi = 2._rkx*a7/denm
!
!   *** speciation & water content
!   ***************************************
      molal(1) = ahi                                  ! hi
      molal(3) = psi1 + psi4 + 2._rkx*psi2              ! nh4
      molal(5) = psi2                                 ! so4
      molal(6) = zero                                 ! hso4
      molal(7) = psi3 + psi1                          ! no3
      cnh42s4 = chi2 - psi2                           ! solid (nh4)2so4
      cnh4no3 = zero                                  ! solid nh4no3
      ghno3 = chi3 - psi3                             ! gas hno3
      gnh3 = chi4 - psi4                              ! gas nh3
      call calcmr                                     ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
! funcd2 = nh4i/hi/max(gnh3 ,tiny1)/a4 - one
    funcd2 = molal(3)/molal(1)/max(gnh3,tiny1)/a4 - one
  end function funcd2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcd1
! *** case d1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4 , nh4no3
!
!     there are two regimes defined by relative humidity:
!     1. rh < mdrh ; only solid phase possible (subroutine calcd1a)
!     2. rh >= mdrh ; liquid phase possible (mdrh region)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcd1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmasan ) then
      scase = 'd1 ; subcase 1'      ! solid phase only possible
      call calcd1a
      scase = 'd1 ; subcase 1'
    else
      scase = 'd1 ; subcase 2'      ! liquid & solid phase possible
      call calcmdrh(rh,drmasan,drnh4no3,calcd1a,calcd2)
      scase = 'd1 ; subcase 2'
    end if
  end subroutine calcd1

!======================================================================
!
! *** isorropia code
! *** subroutine calcd1a
! *** case d1 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4 , nh4no3
!
!     the solid (nh4)2so4 is calculated from the sulfates , while nh4no3
!     is calculated from nh3-hno3 equilibrium. 'ze' is the amount of
!     nh4no3 that volatizes when all possilbe nh4no3 is initially in
!     the solid phase.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcd1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: diak , om , omps , parm , ps , x , ze
!
! *** setup parameters ************************************************
!
    parm = xk10/(r*temp)/(r*temp)
!
! *** calculate nh4no3 that volatizes *********************************
!
    cnh42s4 = w(2)
    x = max(zero,min(w(3)-2.0*cnh42s4,w(4)))              ! max nh4no3
    ps = max(w(3)-x-2.0*cnh42s4,zero)
    om = max(w(4)-x,zero)
!
    omps = om + ps
    diak = sqrt(omps*omps+4.0*parm)                     ! diakrinousa
    ze = min(x,0.5_rkx*(-omps+diak))                         ! thetiki riza
!
! *** speciation *******************************************************
!
    cnh4no3 = x - ze       ! solid nh4no3
    gnh3 = ps + ze         ! gas nh3
    ghno3 = om + ze        ! gas hno3
!
  end subroutine calcd1a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcg5
! *** case g5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcg5
    use mod_che_common_isorropia
    use mod_che_common_caseg
    implicit none
    real(rkx) :: delta , dx , psi6hi , psi6lo , &
         x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi1 = 0.5_rkx*w(1)
    chi2 = max(w(2)-chi1,zero)
    chi3 = zero
    chi4 = max(w(3)-2._rkx*chi2,zero)
    chi5 = w(4)
    chi6 = w(5)
!
    psi1 = chi1
    psi2 = chi2
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
    water = chi2/m0(4) + chi1/m0(2)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcg5a(x1)
    y2 = y1
    if ( chi6>tiny1 ) then
!   if (abs(y1) <= eps .or. chi6 <= tiny1) goto 50
!   if (water  <=  tiny1) return                    ! no water
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcg5a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcg5a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcg5a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcg5')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
200 continue
    x3 = 0.5_rkx*(x1+x2)
    y3 = funcg5a(x3)
!
! *** calculate hso4 speciation and return
! *******************************
300 continue
    if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then  ! if quadrat.called
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                       ! h+   effect
      molal(5) = molal(5) - delta                       ! so4  effect
      molal(6) = delta                                  ! hso4 effect
    end if
  end subroutine calcg5
!
!======================================================================
!
! *** isorropia code
! *** subroutine funcg5a
! *** case g5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funcg5a(x)
    use mod_che_common_isorropia
    use mod_che_common_caseg
    implicit none
    real(rkx) :: akk , bb , cc , dd , hi , ohi , smin , x
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    psi6 = x
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a2 = xk7*(water/gama(4))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      akk = a4*a6
!
!   caculate dissociation quantities
!
      if ( chi5>=tiny1 ) then
        psi5 = psi6*chi5/(a6/a5*(chi6-psi6)+psi6)
      else
        psi5 = tiny1
      end if
!
!   if(chi4 > tiny1) then
      if ( w(2)>tiny1 ) then         ! accounts for nh3 evaporation
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6) - 2._rkx*psi2/a4
        dd = max(bb*bb-4._rkx*cc,zero)                  ! patch proposed by uma shankar , 19/11/01
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
      else
        psi4 = tiny1
      end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = 2.0_rkx*psi1                              ! nai
      molal(3) = 2.0*psi2 + psi4                         ! nh4i
      molal(4) = psi6                                    ! cli
      molal(5) = psi2 + psi1                             ! so4i
      molal(6) = zero
      molal(7) = psi5                                    ! no3i
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)                          ! gas nh3
      ghno3 = max(chi5-psi5,tiny1)                         ! gas hno3
      ghcl = max(chi6-psi6,tiny1)                          ! gas hcl
!
      cnh42s4 = zero                                     ! solid (nh4)2so4
      cnh4no3 = zero                                     ! solid nh4no3
      cnh4cl = zero                                      ! solid nh4cl
!
      call calcmr                                        ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcg5a = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcg5a = molal(1)*molal(4)/ghcl/a6 - one
  end function funcg5a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcg4
! *** case g4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcg4
    use mod_che_common_isorropia
    use mod_che_common_caseg
    implicit none
    real(rkx) :: delta , dx , psi6hi , psi6lo , &
         x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi1 = 0.5_rkx*w(1)
    chi2 = max(w(2)-chi1,zero)
    chi3 = zero
    chi4 = max(w(3)-2._rkx*chi2,zero)
    chi5 = w(4)
    chi6 = w(5)
!
    psi2 = chi2
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
    water = chi2/m0(4) + chi1/m0(2)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcg4a(x1)
    y2 = y1
    if ( chi6>tiny1 ) then
!   if (abs(y1) <= eps .or. chi6 <= tiny1 .or. water  <=  tiny1) goto 50
!   if (water  <=  tiny1) return                    ! no water
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcg4a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcg4a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcg4a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcg4')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
200 continue
    x3 = 0.5_rkx*(x1+x2)
    y3 = funcg4a(x3)
!
! *** calculate hso4 speciation and return
! *******************************
300 continue
    if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcg4
!
!======================================================================
!
! *** isorropia code
! *** subroutine funcg4a
! *** case g4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funcg4a(x)
    use mod_che_common_isorropia
    use mod_che_common_caseg
    implicit none
    real(rkx) :: bb , cc , cli , dd , hi , ohi , so4i , x
    integer(ik4) :: i , islv
    real(rkx) :: nai , nh4i , no3i
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a2 = xk7*(water/gama(4))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
!
!   caculate dissociation quantities
!
      if ( chi5>=tiny1 ) then
        psi5 = psi6*chi5/(a6/a5*(chi6-psi6)+psi6)
      else
        psi5 = tiny1
      end if
!
!   if(chi4 > tiny1) then
      if ( w(2)>tiny1 ) then         ! accounts for nh3 evaporation
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6) - 2._rkx*psi2/a4
        dd = max(bb*bb-4._rkx*cc,zero)        ! patch proposed by uma shankar , 19/11/2001
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
      else
        psi4 = tiny1
      end if
!
!   caculate concentrations
!
      nh4i = 2.0*psi2 + psi4
      cli = psi6
      so4i = psi2 + psi1
      no3i = psi5
      nai = 2.0_rkx*psi1
!
      call calcph(2._rkx*so4i+no3i+cli-nai-nh4i,hi,ohi)
!
!   *** na2so4 dissolution
!
      if ( chi1>tiny1 .and. water>tiny1 ) then             ! psi1
        call poly3(psi2,zero,-a1/4._rkx,psi1,islv)
        if ( islv==0 ) then
          psi1 = min(psi1,chi1)
        else
          psi1 = zero
        end if
      else
        psi1 = zero
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = zero
      molal(7) = no3i
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnh4cl = zero
      cna2so4 = max(chi1-psi1,zero)
!
!   *** calculate molalr array , water and activities
!   **********************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcg4a = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcg4a = molal(1)*molal(4)/ghcl/a6 - one
  end function funcg4a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcg3
! *** case g3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. liquid & solid phase are both possible
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcg3
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** regime depends on the existance of water and of the rh
! ************
    if ( w(4)>tiny1 .and. w(5)>tiny1 ) then     ! no3 ,cl exist , water possible
      scase = 'g3 ; subcase 1'
      call calcg3a
      scase = 'g3 ; subcase 1'
    else                                        ! no3 , cl non existant
      scase = 'g1 ; subcase 1'
      call calcg1a
      scase = 'g1 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drmg3 ) then           ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcg1a
        scase = 'g3 ; subcase 2'
        return
      else
        scase = 'g3 ; subcase 3'      ! mdrh region (na2so4 , nh42s4)
        call calcmdrh(rh,drmg3,drnh42s4,calcg1a,calcg4)
        scase = 'g3 ; subcase 3'
      end if
    end if
  end subroutine calcg3
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcg3a
! *** case g3 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcg3a
    use mod_che_common_isorropia
    use mod_che_common_caseg
    implicit none
    real(rkx) :: delta , dx , psi6hi , psi6lo , &
         x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi1 = 0.5_rkx*w(1)
    chi2 = max(w(2)-chi1,zero)
    chi3 = zero
    chi4 = max(w(3)-2._rkx*chi2,zero)
    chi5 = w(4)
    chi6 = w(5)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
    water = tiny1
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcg3a(x1)
    y2 = y1
    if ( chi6>tiny1 ) then
!   if (abs(y1) <= eps .or. chi6 <= tiny1 .or. water  <=  tiny1) goto 50
!   if (water  <=  tiny1) return                    ! no water
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcg3a(x2)
!
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcg3a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcg3a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcg3a')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
200 continue
    x3 = 0.5_rkx*(x1+x2)
    y3 = funcg3a(x3)
!
! *** final calculations
! *************************************************
!
! *** na2so4 dissolution
!
300 continue
    if ( chi1>tiny1 .and. water>tiny1 ) then        ! psi1
      call poly3(psi2,zero,-a1/4._rkx,psi1,islv)
      if ( islv==0 ) then
        psi1 = min(psi1,chi1)
      else
        psi1 = zero
      end if
    else
      psi1 = zero
    end if
    molal(2) = 2.0_rkx*psi1                 ! na+  effect
    molal(5) = molal(5) + psi1            ! so4  effect
    cna2so4 = max(chi1-psi1,zero)          ! na2so4(s) depletion
!
! *** hso4 equilibrium
!
    if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcg3a
!
!======================================================================
!
! *** isorropia code
! *** subroutine funcg3a
! *** case g3 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funcg3a(x)
    use mod_che_common_isorropia
    use mod_che_common_caseg
    implicit none
    real(rkx) :: bb , cc , dd , hi , ohi , psi20 , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi2 = chi2
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a2 = xk7*(water/gama(4))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
!
!   caculate dissociation quantities
!
      if ( chi5>=tiny1 ) then
        psi5 = psi6*chi5/(a6/a5*(chi6-psi6)+psi6)
      else
        psi5 = tiny1
      end if
!
!   if(chi4 > tiny1) then
      if ( w(2)>tiny1 ) then         ! accounts for nh3 evaporation
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6) - 2._rkx*psi2/a4
        dd = max(bb*bb-4._rkx*cc,zero)         ! patch proposed by uma shankar , 19/11/01
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
      else
        psi4 = tiny1
      end if
!
      if ( chi2>tiny1 .and. water>tiny1 ) then
        call poly3(psi4,psi4*psi4/4._rkx,-a2/4._rkx,psi20,islv)
        if ( islv==0 ) psi2 = min(psi20,chi2)
      end if
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      molal(2) = zero                                    ! na
      molal(3) = 2.0*psi2 + psi4                         ! nh4i
      molal(4) = psi6                                    ! cli
      molal(5) = psi2                                    ! so4i
      molal(6) = zero                                    ! hso4
      molal(7) = psi5                                    ! no3i
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)                          ! gas nh3
      ghno3 = max(chi5-psi5,tiny1)                         ! gas hno3
      ghcl = max(chi6-psi6,tiny1)                          ! gas hcl
!
      cnh42s4 = chi2 - psi2                              ! solid (nh4)2so4
      cnh4no3 = zero                                     ! solid nh4no3
      cnh4cl = zero                                      ! solid nh4cl
!
      call calcmr                                        ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcg3a = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcg3a = molal(1)*molal(4)/ghcl/a6 - one
  end function funcg3a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcg2
! *** case g2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. liquid & solid phase are both possible
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcg2
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** regime depends on the existance of nitrates
! ***********************
    if ( w(4)>tiny1 ) then          ! no3 exists , water possible
      scase = 'g2 ; subcase 1'
      call calcg2a
      scase = 'g2 ; subcase 1'
    else                            ! no3 non existant , water not possible
      scase = 'g1 ; subcase 1'
      call calcg1a
      scase = 'g1 ; subcase 1'
    end if
!
! *** regime depends on the existance of water and of the rh
! ************
    if ( water<=tiny1 ) then
      if ( rh<drmg2 ) then                ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcg1a
        scase = 'g2 ; subcase 2'
      else
        if ( w(5)>tiny1 ) then
          scase = 'g2 ; subcase 3'         ! mdrh (nh4cl , na2so4 , nh42s4)
          call calcmdrh(rh,drmg2,drnh4cl,calcg1a,calcg3a)
          scase = 'g2 ; subcase 3'
        end if
        if ( water<=tiny1 .and. rh>=drmg3 ) then
          scase = 'g2 ; subcase 4'         ! mdrh (na2so4 , nh42s4)
          call calcmdrh(rh,drmg3,drnh42s4,calcg1a,calcg4)
          scase = 'g2 ; subcase 4'
        else
          water = tiny1
          do i = 1 , nions
            molal(i) = zero
          end do
          call calcg1a
          scase = 'g2 ; subcase 2'
        end if
      end if
    end if
  end subroutine calcg2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcg2a
! *** case g2 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcg2a
    use mod_che_common_isorropia
    use mod_che_common_caseg
    implicit none
    real(rkx) :: delta , dx , psi6hi , psi6lo , &
         x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi1 = 0.5_rkx*w(1)
    chi2 = max(w(2)-chi1,zero)
    chi3 = zero
    chi4 = max(w(3)-2._rkx*chi2,zero)
    chi5 = w(4)
    chi6 = w(5)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1
!
    water = tiny1
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcg2a(x1)
    y2 = y1
    if ( chi6>tiny1 ) then
!   if (abs(y1) <= eps .or. chi6 <= tiny1) goto 50
!   if (water  <=  tiny1) goto 50               ! no water
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcg2a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) water = tiny1
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcg2a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcg2a')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
200 continue
    x3 = 0.5_rkx*(x1+x2)
    if ( x3<=tiny2 ) then     ! practically no nitrates , so dry solution
      water = tiny1
    else
      y3 = funcg2a(x3)
    end if
!
! *** calculate hso4 speciation and return
! *******************************
!
! *** na2so4 dissolution
!
300 continue
    if ( chi1>tiny1 .and. water>tiny1 ) then        ! psi1
      call poly3(psi2,zero,-a1/4._rkx,psi1,islv)
      if ( islv==0 ) then
        psi1 = min(psi1,chi1)
      else
        psi1 = zero
      end if
    else
      psi1 = zero
    end if
    molal(2) = 2.0_rkx*psi1                 ! na+  effect
    molal(5) = molal(5) + psi1            ! so4  effect
    cna2so4 = max(chi1-psi1,zero)          ! na2so4(s) depletion
!
! *** hso4 equilibrium
!
    if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta        ! h+   affect
      molal(5) = molal(5) - delta        ! so4  affect
      molal(6) = delta                   ! hso4 affect
    end if
  end subroutine calcg2a
!
!======================================================================
!
! *** isorropia code
! *** subroutine funcg2a
! *** case g2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funcg2a(x)
    use mod_che_common_isorropia
    use mod_che_common_caseg
    implicit none
    real(rkx) :: bb , cc , dd , delt , deno , hi , ohi , &
         psi20 , psi31 , psi32 , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi2 = chi2
    psi3 = zero
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a2 = xk7*(water/gama(4))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
!
      deno = max(chi6-psi6-psi3,zero)
      psi5 = chi5/((a6/a5)*(deno/psi6)+one)
!
      psi4 = min(psi5+psi6,chi4)
!
      if ( chi2>tiny1 .and. water>tiny1 ) then
        call poly3(psi4,psi4*psi4/4._rkx,-a2/4._rkx,psi20,islv)
        if ( islv==0 ) psi2 = min(psi20,chi2)
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(2) = zero                                 ! na
      molal(3) = 2.0*psi2 + psi4                      ! nh4i
      molal(4) = psi6                                 ! cli
      molal(5) = psi2                                 ! so4i
      molal(6) = zero                                 ! hso4
      molal(7) = psi5                                 ! no3i
!
!   molal (1) = max(chi5 - psi5 , tiny1)*a5/psi5   ! hi
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = max(chi2-psi2,zero)
      cnh4no3 = zero
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
!   *** calculate molalr array , water and activities
!   **********************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    if ( chi4<=tiny1 ) then
      funcg2a = molal(1)*molal(4)/ghcl/a6 - one
    else
      funcg2a = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    end if
  end function funcg2a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcg1
! *** case g1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4 , nh4no3 , nh4cl , na2so4
!
!     there are two possible regimes here , depending on relative humidity:
!     1. when rh >= mdrh ; liquid phase possible (mdrh region)
!     2. when rh < mdrh  ; only solid phase possible (subroutine calcg1a)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcg1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmg1 ) then
      scase = 'g1 ; subcase 1'
      call calcg1a                 ! solid phase only possible
      scase = 'g1 ; subcase 1'
    else
      scase = 'g1 ; subcase 2'     ! liquid & solid phase possible
      call calcmdrh(rh,drmg1,drnh4no3,calcg1a,calcg2a)
      scase = 'g1 ; subcase 2'
    end if
  end subroutine calcg1

!======================================================================
!
! *** isorropia code
! *** subroutine calcg1a
! *** case g1 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4 , nh4no3
!
!     solid (nh4)2so4 is calculated from the sulfates , while nh4no3
!     is calculated from nh3-hno3 equilibrium. 'ze' is the amount of
!     nh4no3 that volatizes when all possilbe nh4no3 is initially in
!     the solid phase.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcg1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a1 , a2 , alf , bb , bet , cc , dd , &
         dd1 , dd2 , frna , gam , rtsq , so4fr , sqdd , &
         sqdd1 , sqdd2 , theta1 , theta2
    real(rkx) :: lamda , lamda1 , lamda2 , kapa , kapa1 , kapa2
!
! *** calculate non volatile solids ***********************************
!
    cna2so4 = min(0.5_rkx*w(1),w(2))
    frna = max(w(1)-2._rkx*cna2so4,zero)
    so4fr = max(w(2)-cna2so4,zero)
! cnh42s4 = w(2) - cna2so4
    cnh42s4 = max(so4fr,zero)                       ! cnh42s4
!
! *** calculate volatile species **************************************
!
    alf = w(3) - 2.0*cnh42s4
    bet = w(5)
    gam = w(4)
!
    rtsq = r*temp*r*temp
    a1 = xk6/rtsq
    a2 = xk10/rtsq
!
    theta1 = gam - bet*(a2/a1)
    theta2 = a2/a1
!
! quaratic equation solution
!
    bb = (theta1-alf-bet*(one+theta2))/(one+theta2)
    cc = (alf*bet-a1-bet*theta1)/(one+theta2)
    dd = bb*bb - 4.0_rkx*cc
    if ( dd>=zero ) then        ! solve each reaction seperately
!
!   two roots for kapa , check and see if any valid
!
      sqdd = sqrt(dd)
      kapa1 = 0.5_rkx*(-bb+sqdd)
      kapa2 = 0.5_rkx*(-bb-sqdd)
      lamda1 = theta1 + theta2*kapa1
      lamda2 = theta1 + theta2*kapa2
!
      if ( kapa1>=zero .and. lamda1>=zero ) then
        if ( alf-kapa1-lamda1>=zero .and. bet-kapa1>=zero .and.           &
             gam-lamda1>=zero ) then
          kapa = kapa1
          lamda = lamda1
          go to 100
        end if
      end if
!
      if ( kapa2>=zero .and. lamda2>=zero ) then
        if ( alf-kapa2-lamda2>=zero .and. bet-kapa2>=zero .and.           &
             gam-lamda2>=zero ) then
          kapa = kapa2
          lamda = lamda2
          go to 100
        end if
      end if
    end if
!
! seprate solution of nh4cl & nh4no3 equilibria
!
    kapa = zero
    lamda = zero
    dd1 = (alf+bet)*(alf+bet) - 4.0_rkx*(alf*bet-a1)
    dd2 = (alf+gam)*(alf+gam) - 4.0_rkx*(alf*gam-a2)
!
! nh4l equilibrium
!
    if ( dd1>=zero ) then
      sqdd1 = sqrt(dd1)
      kapa1 = 0.5_rkx*(alf+bet+sqdd1)
      kapa2 = 0.5_rkx*(alf+bet-sqdd1)
!
      if ( kapa1>=zero .and. kapa1<=min(alf,bet) ) then
        kapa = kapa1
      else if ( kapa2>=zero .and. kapa2<=min(alf,bet) ) then
        kapa = kapa2
      else
        kapa = zero
      end if
    end if
!
! nh4o3 equilibrium
!
    if ( dd2>=zero ) then
      sqdd2 = sqrt(dd2)
      lamda1 = 0.5_rkx*(alf+gam+sqdd2)
      lamda2 = 0.5_rkx*(alf+gam-sqdd2)
!
      if ( lamda1>=zero .and. lamda1<=min(alf,gam) ) then
        lamda = lamda1
      else if ( lamda2>=zero .and. lamda2<=min(alf,gam) ) then
        lamda = lamda2
      else
        lamda = zero
      end if
    end if
!
! if oth kapa , lamda are > 0 , then apply existance criterion
!
    if ( kapa>zero .and. lamda>zero ) then
      if ( bet<lamda/theta1 ) then
        kapa = zero
      else
        lamda = zero
      end if
    end if
!
! *** calculate composition of volatile species ***********************
!
100 continue
    cnh4no3 = lamda
    cnh4cl = kapa
!
    gnh3 = max(alf-kapa-lamda,zero)
    ghno3 = max(gam-lamda,zero)
    ghcl = max(bet-kapa,zero)
  end subroutine calcg1a
!======================================================================
!
! *** isorropia code
! *** subroutine calch6
! *** case h6
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calch6
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frna , psi6hi , psi6lo , &
         x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi1 = w(2)                                    ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funch6a(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funch6a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funch6a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funch6a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calch6')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
200 continue
    x3 = 0.5_rkx*(x1+x2)
    y3 = funch6a(x3)
!
! *** calculate hso4 speciation and return
! *******************************
300 continue
    if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calch6
!
!======================================================================
!
! *** isorropia code
! *** subroutine funch6a
! *** case h6
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funch6a(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , cc , dd , hi , ohi , smin , x
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
      a9 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = max(psi5,tiny1)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = bb*bb - 4._rkx*cc
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(psi4,chi4)
      else
        psi4 = tiny1
      end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                   ! nai
      molal(3) = psi4                                      ! nh4i
      molal(4) = psi6 + psi7                               ! cli
      molal(5) = psi2 + psi1                               ! so4i
      molal(6) = zero                                      ! hso4i
      molal(7) = psi5 + psi8                               ! no3i
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      cna2so4 = max(chi1-psi1,zero)
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    funch6a = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
  end function funch6a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calch5
! *** case h5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calch5
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frna , psi6hi , psi6lo , &
         x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** regime depends on the existance of nitrates
! ***********************
    if ( w(4)<=tiny1 .and. w(5)<=tiny1 ) then
      scase = 'h5'
      call calch1a
      scase = 'h5'
      return
    end if
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi1 = w(2)                                    ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funch5a(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funch5a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funch5a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funch5a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calch5')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
200 continue
    x3 = 0.5_rkx*(x1+x2)
    y3 = funch5a(x3)
!
! *** calculate hso4 speciation and return
! *******************************
300 continue
    if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   efect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calch5
!
!======================================================================
!
! *** isorropia code
! *** subroutine funch5a
! *** case h5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : none
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funch5a(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , cc , dd , hi , ohi , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
      a9 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = max(psi5,tiny1)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = bb*bb - 4._rkx*cc
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(psi4,chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi1>tiny1 .and. water>tiny1 ) then          ! na2so4 dissolution
        aa = psi7 + psi8
        bb = aa*aa
        cc = -a1/4._rkx
        call poly3(aa,bb,cc,psi1,islv)
        if ( islv==0 ) then
          psi1 = min(psi1,chi1)
        else
          psi1 = zero
        end if
      end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                    ! nai
      molal(3) = psi4                                       ! nh4i
      molal(4) = psi6 + psi7                                ! cli
      molal(5) = psi2 + psi1                                ! so4i
      molal(6) = zero
      molal(7) = psi5 + psi8                                ! no3i
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      cna2so4 = max(chi1-psi1,zero)
!
      call calcmr                                  ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    funch5a = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
  end function funch5a

!======================================================================
!
! *** isorropia code
! *** subroutine calch4
! *** case h4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calch4
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frna , psi6hi , psi6lo , &
         x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** regime depends on the existance of nitrates
! ***********************
    if ( w(4)<=tiny1 .and. w(5)<=tiny1 ) then
      scase = 'h4'
      call calch1a
      scase = 'h4'
      return
    end if
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi1 = w(2)                                    ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funch4a(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funch4a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funch4a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
100 continue
    do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funch4a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calch4')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
200 continue
    x3 = 0.5_rkx*(x1+x2)
    y3 = funch4a(x3)
!
! *** calculate hso4 speciation and return
! *******************************
300 continue
    if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                         ! h+   effect
      molal(5) = molal(5) - delta                         ! so4  effect
      molal(6) = delta                                    ! hso4 effect
    end if
  end subroutine calch4
!
!======================================================================
!
! *** isorropia code
! *** subroutine funch4a
! *** case h4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funch4a(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , cc , dd , delt , hi , ohi , psi31 , psi32 , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
      a9 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = max(psi5,tiny1)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = bb*bb - 4._rkx*cc
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(psi4,chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi1>tiny1 .and. water>tiny1 ) then          ! na2so4 dissolution
        aa = psi7 + psi8
        bb = aa*aa
        cc = -a1/4._rkx
        call poly3(aa,bb,cc,psi1,islv)
        if ( islv==0 ) then
          psi1 = min(psi1,chi1)
        else
          psi1 = zero
        end if
      end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                    ! nai
      molal(3) = psi4                                       ! nh4i
      molal(4) = psi6 + psi7                                ! cli
      molal(5) = psi2 + psi1                                ! so4i
      molal(6) = zero
      molal(7) = psi5 + psi8                                ! no3i
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      cna2so4 = max(chi1-psi1,zero)
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      delt = min(gnh3,ghcl)
      bb = -(gnh3+ghcl)
      cc = gnh3*ghcl - a3
      dd = bb*bb - 4._rkx*cc
      psi31 = 0.5_rkx*(-bb+sqrt(dd))
      psi32 = 0.5_rkx*(-bb-sqrt(dd))
      if ( delt-psi31>zero .and. psi31>zero ) then
        psi3 = psi31
      else if ( delt-psi32>zero .and. psi32>zero ) then
        psi3 = psi32
      else
        psi3 = zero
      end if
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
      call calcmr                              ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    funch4a = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
  end function funch4a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calch3
! *** case h3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calch3
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frna , psi6hi , psi6lo , &
         x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** regime depends on the existance of nitrates
! ***********************
    if ( w(4)<=tiny1 ) then          ! no3 not exist , water not possible
      scase = 'h3'
      call calch1a
      scase = 'h3'
      return
    end if
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi1 = w(2)                                    ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funch3a(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funch3a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funch3a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funch3a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calch3')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funch3a(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calch3
!
!======================================================================
!
! *** isorropia code
! *** subroutine funch3a
! *** case h3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funch3a(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , cc , dd , delt , diak , hi , ohi , &
         psi31 , psi32 , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
      a9 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = max(psi5,tiny1)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = bb*bb - 4._rkx*cc
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(psi4,chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then          ! nacl dissolution
        diak = (psi8-psi6)**20 + 4._rkx*a7
        psi7 = 0.5_rkx*(-(psi8+psi6)+sqrt(diak))
        psi7 = max(min(psi7,chi7),zero)
      end if
!
      if ( chi1>tiny1 .and. water>tiny1 ) then          ! na2so4 dissolution
        aa = psi7 + psi8
        bb = aa*aa
        cc = -a1/4._rkx
        call poly3(aa,bb,cc,psi1,islv)
        if ( islv==0 ) then
          psi1 = min(psi1,chi1)
        else
          psi1 = zero
        end if
      end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                 ! nai
      molal(3) = psi4                                    ! nh4i
      molal(4) = psi6 + psi7                             ! cli
      molal(5) = psi2 + psi1                             ! so4i
      molal(6) = zero
      molal(7) = psi5 + psi8                             ! no3i
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      cna2so4 = max(chi1-psi1,zero)
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      delt = min(gnh3,ghcl)
      bb = -(gnh3+ghcl)
      cc = gnh3*ghcl - a3
      dd = bb*bb - 4._rkx*cc
      psi31 = 0.5_rkx*(-bb+sqrt(dd))
      psi32 = 0.5_rkx*(-bb-sqrt(dd))
      if ( delt-psi31>zero .and. psi31>zero ) then
        psi3 = psi31
      else if ( delt-psi32>zero .and. psi32>zero ) then
        psi3 = psi32
      else
        psi3 = zero
      end if
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
      call calcmr                                    ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    funch3a = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
  end function funch3a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calch2
! *** case h2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : nh4cl , na2so4 , nano3 , nacl
!
!     there are three regimes in this case:
!     1. nh4no3(s) possible. liquid & solid aerosol (subroutine calch2a)
!     2. nh4no3(s) not possible , and rh < mdrh. solid aerosol only
!     3. nh4no3(s) not possible , and rh >= mdrh. (mdrh region)
!
!     regimes 2. and 3. are considered to be the same as cases h1a , h2b
!     respectively (because mdrh points coincide).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calch2
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends on the existance of nitrates
! ***********************
    if ( w(4)>tiny1 ) then          ! no3 exists , water possible
      scase = 'h2 ; subcase 1'
      call calch2a
      scase = 'h2 ; subcase 1'
    else                            ! no3 non existant , water not possible
      scase = 'h2 ; subcase 1'
      call calch1a
      scase = 'h2 ; subcase 1'
    end if
    if ( water<=tiny1 .and. rh<drmh2 ) then          ! dry aerosol
      scase = 'h2 ; subcase 2'
    else if ( water<=tiny1 .and. rh>=drmh2 ) then      ! mdrh of h2
      scase = 'h2 ; subcase 3'
      call calcmdrh(rh,drmh2,drnano3,calch1a,calch3)
      scase = 'h2 ; subcase 3'
    end if
  end subroutine calch2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calch2a
! *** case h2 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calch2a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frna , psi6hi , psi6lo , &
         x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi1 = w(2)                                    ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funch2a(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funch2a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( y2>eps ) y2 = funch2a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funch2a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calch2a')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funch2a(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                       ! h+   effect
      molal(5) = molal(5) - delta                       ! so4  effect
      molal(6) = delta                                  ! hso4 effect
    end if
  end subroutine calch2a
!
!======================================================================
!
! *** isorropia code
! *** subroutine funch2a
! *** case h2 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funch2a(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: a64 , aa , bb , cc , dd , delt , diak , hi , ohi , psi31 , &
         psi32 , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    frst = .true.
    calain = .true.
    a64 = tiny1
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
      a64 = (xk3*xk2/xkw)*(gama(10)/gama(5)/gama(11))**2
      a64 = a64*(r*temp*water)**2
      a9 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = max(psi5,tiny1)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = bb*bb - 4._rkx*cc
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(psi4,chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then          ! nacl dissolution
        diak = (psi8-psi6)**20 + 4._rkx*a7
        psi7 = 0.5_rkx*(-(psi8+psi6)+sqrt(diak))
        psi7 = max(min(psi7,chi7),zero)
      end if
!
      if ( chi8>tiny1 .and. water>tiny1 ) then          ! nano3 dissolution
        diak = (psi7-psi5)**20 + 4._rkx*a8
        psi8 = 0.5_rkx*(-(psi7+psi5)+sqrt(diak))
        psi8 = max(min(psi8,chi8),zero)
      end if
!
      if ( chi1>tiny1 .and. water>tiny1 ) then          ! na2so4 dissolution
        aa = psi7 + psi8
        bb = aa*aa
        cc = -a1/4._rkx
        call poly3(aa,bb,cc,psi1,islv)
        if ( islv==0 ) then
          psi1 = min(psi1,chi1)
        else
          psi1 = zero
        end if
      end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                     ! nai
      molal(3) = psi4                                        ! nh4i
      molal(4) = psi6 + psi7                                 ! cli
      molal(5) = psi2 + psi1                                 ! so4i
      molal(6) = zero                                        ! hso4i
      molal(7) = psi5 + psi8                                 ! no3i
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      cna2so4 = max(chi1-psi1,zero)
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      delt = min(gnh3,ghcl)
      bb = -(gnh3+ghcl)
      cc = gnh3*ghcl - a3
      dd = bb*bb - 4._rkx*cc
      psi31 = 0.5_rkx*(-bb+sqrt(dd))
      psi32 = 0.5_rkx*(-bb-sqrt(dd))
      if ( delt-psi31>zero .and. psi31>zero ) then
        psi3 = psi31
      else if ( delt-psi32>zero .and. psi32>zero ) then
        psi3 = psi32
      else
        psi3 = zero
      end if
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
      call calcmr                           ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    funch2a = molal(3)*molal(4)/ghcl/gnh3/a64 - one
  end function funch2a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calch1
! *** case h1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4no3 , nh4cl , na2so4
!
!     there are two possible regimes here , depending on relative humidity:
!     1. when rh >= mdrh ; liquid phase possible (mdrh region)
!     2. when rh < mdrh  ; only solid phase possible (subroutine calch1a)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calch1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmh1 ) then
      scase = 'h1 ; subcase 1'
      call calch1a                 ! solid phase only possible
      scase = 'h1 ; subcase 1'
    else
      scase = 'h1 ; subcase 2'     ! liquid & solid phase possible
      call calcmdrh(rh,drmh1,drnh4no3,calch1a,calch2a)
      scase = 'h1 ; subcase 2'
    end if
  end subroutine calch1

!======================================================================
!
! *** isorropia code
! *** subroutine calch1a
! *** case h1 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4no3 , nh4cl , nano3 , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calch1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a1 , a2 , alf , bb , bet , cc , clfr , dd , dd1 , dd2 , &
         gam , rtsq , sqdd , sqdd1 , sqdd2 , theta1 , theta2
    real(rkx) :: lamda , lamda1 , lamda2 , kapa , kapa1 , kapa2 , nafr , &
         no3fr
!
! *** calculate non volatile solids ***********************************
!
    cna2so4 = w(2)
    cnh42s4 = zero
    nafr = max(w(1)-2*cna2so4,zero)
    cnano3 = min(nafr,w(4))
    no3fr = max(w(4)-cnano3,zero)
    cnacl = min(max(nafr-cnano3,zero),w(5))
    clfr = max(w(5)-cnacl,zero)
!
! *** calculate volatile species **************************************
!
    alf = w(3)                           ! free nh3
    bet = clfr                           ! free cl
    gam = no3fr                          ! free no3
!
    rtsq = r*temp*r*temp
    a1 = xk6/rtsq
    a2 = xk10/rtsq
!
    theta1 = gam - bet*(a2/a1)
    theta2 = a2/a1
!
! quaratic equation solution
!
    bb = (theta1-alf-bet*(one+theta2))/(one+theta2)
    cc = (alf*bet-a1-bet*theta1)/(one+theta2)
    dd = bb*bb - 4.0_rkx*cc
    if ( dd>=zero ) then        ! solve each reaction seperately
!
!   two roots for kapa , check and see if any valid
!
      sqdd = sqrt(dd)
      kapa1 = 0.5_rkx*(-bb+sqdd)
      kapa2 = 0.5_rkx*(-bb-sqdd)
      lamda1 = theta1 + theta2*kapa1
      lamda2 = theta1 + theta2*kapa2
!
      if ( kapa1>=zero .and. lamda1>=zero ) then
        if ( alf-kapa1-lamda1>=zero .and. bet-kapa1>=zero .and.           &
             gam-lamda1>=zero ) then
          kapa = kapa1
          lamda = lamda1
          go to 100
        end if
      end if
!
      if ( kapa2>=zero .and. lamda2>=zero ) then
        if ( alf-kapa2-lamda2>=zero .and. bet-kapa2>=zero .and.           &
             gam-lamda2>=zero ) then
          kapa = kapa2
          lamda = lamda2
          go to 100
        end if
      end if
    end if
!
! seprate solution of nh4cl & nh4no3 equilibria
!
    kapa = zero
    lamda = zero
    dd1 = (alf+bet)*(alf+bet) - 4.0_rkx*(alf*bet-a1)
    dd2 = (alf+gam)*(alf+gam) - 4.0_rkx*(alf*gam-a2)
!
! nh4l equilibrium
!
    if ( dd1>=zero ) then
      sqdd1 = sqrt(dd1)
      kapa1 = 0.5_rkx*(alf+bet+sqdd1)
      kapa2 = 0.5_rkx*(alf+bet-sqdd1)
!
      if ( kapa1>=zero .and. kapa1<=min(alf,bet) ) then
        kapa = kapa1
      else if ( kapa2>=zero .and. kapa2<=min(alf,bet) ) then
        kapa = kapa2
      else
        kapa = zero
      end if
    end if
!
! nh4o3 equilibrium
!
    if ( dd2>=zero ) then
      sqdd2 = sqrt(dd2)
      lamda1 = 0.5_rkx*(alf+gam+sqdd2)
      lamda2 = 0.5_rkx*(alf+gam-sqdd2)
!
      if ( lamda1>=zero .and. lamda1<=min(alf,gam) ) then
        lamda = lamda1
      else if ( lamda2>=zero .and. lamda2<=min(alf,gam) ) then
        lamda = lamda2
      else
        lamda = zero
      end if
    end if
!
! if oth kapa , lamda are > 0 , then apply existance criterion
!
    if ( kapa>zero .and. lamda>zero ) then
      if ( bet<lamda/theta1 ) then
        kapa = zero
      else
        lamda = zero
      end if
    end if
!
! *** calculate composition of volatile species ***********************
!
 100  cnh4no3 = lamda
    cnh4cl = kapa
!
    gnh3 = alf - kapa - lamda
    ghno3 = gam - lamda
    ghcl = bet - kapa
  end subroutine calch1a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calci6
! *** case i6
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calci6
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , cc , dd
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calci1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calci1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = clc
    psi3 = cnahso4
    psi4 = cna2so4
    psi5 = cnh42s4
!
    calaou = .true.                ! outer loop activity calculation flag
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a6 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      bb = psi2 + psi4 + psi5 + a6                         ! psi6
      cc = -a6*(psi2+psi3+psi1)
      dd = bb*bb - 4._rkx*cc
      psi6 = 0.5_rkx*(-bb+sqrt(dd))
!
!   *** calculate speciation
!   ********************************************
      molal(1) = psi6                                        ! hi
      molal(2) = 2._rkx*psi4 + psi3                            ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1                ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6                   ! so4i
      molal(6) = psi2 + psi3 + psi1 - psi6                   ! hso4i
      clc = zero
      cnahso4 = zero
      cna2so4 = chi4 - psi4
      cnh42s4 = zero
      cnh4hs4 = zero
      call calcmr                                            ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
  end subroutine calci6
!
!======================================================================
!
! *** isorropia code
! *** subroutine calci5
! *** case i5
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calci5
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi4hi , psi4lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calci1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calci1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = clc
    psi3 = cnahso4
    psi4 = zero
    psi5 = cnh42s4
!
    calaou = .true.                ! outer loop activity calculation flag
    psi4lo = zero                  ! low  limit
    psi4hi = chi4                  ! high limit
!
! *** if na2so4(s) = 0 , call funci5b for y4=0
! ***************************
    if ( chi4<=tiny1 ) then
      y1 = funci5a(zero)
      return
    end if
!
! *** initial values for bisection ************************************
!
    x1 = psi4hi
    y1 = funci5a(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with na2so4 **
!
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi4hi-psi4lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funci5a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   nh4cl
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funci5a(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calci5')        ! warning error: no solution
      end if
      return
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funci5a(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calci5')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funci5a(x3)
    end if
!
  end subroutine calci5
!
!======================================================================
!
! *** isorropia code
! *** subroutine funci5a
! *** case i5
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funci5a(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , cc , dd , p4
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    psi4 = p4         ! psi3 already assigned in funci5a
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = xk5*(water/gama(2))**3
      a5 = xk7*(water/gama(4))**3
      a6 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      bb = psi2 + psi4 + psi5 + a6                         ! psi6
      cc = -a6*(psi2+psi3+psi1)
      dd = bb*bb - 4._rkx*cc
      psi6 = 0.5_rkx*(-bb+sqrt(dd))
!
!   *** calculate speciation
!   ********************************************
      molal(1) = psi6                                ! hi
      molal(2) = 2._rkx*psi4 + psi3                    ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1        ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6           ! so4i
      molal(6) = psi2 + psi3 + psi1 - psi6           ! hso4i
      clc = zero
      cnahso4 = zero
      cna2so4 = chi4 - psi4
      cnh42s4 = zero
      cnh4hs4 = zero
      call calcmr                                    ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    a4 = xk5*(water/gama(2))**3
    funci5a = molal(5)*molal(2)*molal(2)/a4 - one
  end function funci5a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calci4
! *** case i4
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calci4
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi4hi , psi4lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calci1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calci1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = clc
    psi3 = cnahso4
    psi4 = zero
    psi5 = zero
!
    calaou = .true.                ! outer loop activity calculation flag
    psi4lo = zero                  ! low  limit
    psi4hi = chi4                  ! high limit
!
! *** if na2so4(s) = 0 , call funci4b for y4=0
! ***************************
    if ( chi4<=tiny1 ) then
      y1 = funci4a(zero)
      return
    end if
!
! *** initial values for bisection ************************************
!
    x1 = psi4hi
    y1 = funci4a(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with na2so4 **
!
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi4hi-psi4lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funci4a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   nh4cl
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funci4a(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calci4')        ! warning error: no solution
      end if
      return
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funci4a(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calci4')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funci4a(x3)
    end if
  end subroutine calci4
!
!======================================================================
!
! *** isorropia code
! *** subroutine funci4a
! *** case i4
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funci4a(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , cc , dd , p4
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    psi4 = p4         ! psi3 already assigned in funci4a
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = xk5*(water/gama(2))**3
      a5 = xk7*(water/gama(4))**3
      a6 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
      a7 = sqrt(a4/a5)
!
!   caculate dissociation quantities
!
      bb = psi2 + psi4 + psi5 + a6                         ! psi6
      cc = -a6*(psi2+psi3+psi1)
      dd = bb*bb - 4._rkx*cc
      psi6 = 0.5_rkx*(-bb+sqrt(dd))
!
      psi5 = (psi3+2._rkx*psi4-a7*(3._rkx*psi2+psi1))/2._rkx/a7
      psi5 = max(min(psi5,chi5),zero)
!
!   *** calculate speciation
!   ********************************************
      molal(1) = psi6                                ! hi
      molal(2) = 2._rkx*psi4 + psi3                    ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1        ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6           ! so4i
      molal(6) = psi2 + psi3 + psi1 - psi6           ! hso4i
      clc = zero
      cnahso4 = zero
      cna2so4 = chi4 - psi4
      cnh42s4 = chi5 - psi5
      cnh4hs4 = zero
      call calcmr                                    ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    a4 = xk5*(water/gama(2))**3
    funci4a = molal(5)*molal(2)*molal(2)/a4 - one
  end function funci4a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calci3
! *** case i3
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4 , nahso4 , lc
!
!     there are three regimes in this case:
!     1.(na ,nh4)hso4(s) possible. liquid & solid aerosol (subroutine calci3a)
!     2.(na ,nh4)hso4(s) not possible , and rh < mdrh. solid aerosol only
!     3.(na ,nh4)hso4(s) not possible , and rh >= mdrh. solid & liquid aerosol
!
!     regimes 2. and 3. are considered to be the same as cases i1a , i2b
!     respectively
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calci3
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calci1a
!
! *** regime depends upon the possible solids & rh
! **********************
    if ( cnh4hs4>tiny1 .or. cnahso4>tiny1 ) then
      scase = 'i3 ; subcase 1'
      call calci3a                        ! full solution
      scase = 'i3 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drmi3 ) then            ! solid solution
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calci1a
        scase = 'i3 ; subcase 2'
      else if ( rh>=drmi3 ) then        ! mdrh of i3
        scase = 'i3 ; subcase 3'
        call calcmdrh(rh,drmi3,drlc,calci1a,calci4)
        scase = 'i3 ; subcase 3'
      end if
    end if
  end subroutine calci3
!
!======================================================================
!
! *** isorropia code
! *** subroutine calci3a
! *** case i3 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4 , lc
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calci3a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi2hi , psi2lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calci1a           ! needed when called from calcmdrh
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calci1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = zero
    psi3 = cnahso4
    psi4 = zero
    psi5 = zero
!
    calaou = .true.                ! outer loop activity calculation flag
    psi2lo = zero                  ! low  limit
    psi2hi = chi2                  ! high limit
!
! *** initial values for bisection ************************************
!
    x1 = psi2hi
    y1 = funci3a(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with lc *********
!
    if ( yhi>=eps ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi2hi-psi2lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = max(x1-dx,psi2lo)
        y2 = funci3a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   lc
      if ( y2>eps ) y2 = funci3a(zero)
    end if
    return
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funci3a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calci3a')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funci3a(x3)
  end subroutine calci3a
!
!======================================================================
!
! *** isorropia code
! *** subroutine funci3a
! *** case i3 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4 , lc
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funci3a(p2)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , p2 , psi4hi , psi4lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    psi2 = p2                      ! save psi2 in common block
    psi4lo = zero                  ! low  limit for psi4
    psi4hi = chi4                  ! high limit for psi4
!
! *** if nh3 = 0 , call funci3b for y4=0
! ********************************
    if ( chi4<=tiny1 ) then
      funci3a = funci3b(zero)
      go to 200
    end if
!
! *** initial values for bisection ************************************
!
    x1 = psi4hi
    y1 = funci3b(x1)
    y2 = y1
    if ( abs(y1)>eps ) then
      yhi = y1                      ! save y-value at hi position
!
!   *** yhi < 0.0 the solution is always undersaturated with na2so4
!   *****
      if ( yhi>=zero ) then
!
!     *** root tracking ; for the range of hi and lo
!     **********************
        dx = (psi4hi-psi4lo)/float(ndiv)
        do i = 1 , ndiv
          x2 = max(x1-dx,psi4lo)
          y2 = funci3b(x2)
          if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50  ! (y1*y2 < zero)
          x1 = x2
          y1 = y2
        end do
!
!     *** { ylo , yhi } > 0.0 the solution is always supersaturated
!     with na2so4
        if ( y2>eps ) y2 = funci3b(psi4lo)
      end if
      go to 200
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funci3b(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0004,'funci3a')       ! warning error: no convergence
!
!   *** inner loop converged
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funci3b(x3)
    end if
!
! *** calculate function value for outer loop
! ***************************
 200  a2 = xk13*(water/gama(13))**5.0
    funci3a = molal(5)*molal(6)*molal(3)**30/a2 - one
  end function funci3a
!
!======================================================================
!
! *** isorropia code
! *** function funci3b
! *** case i3 ; subcase 2
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4 , lc
!
!     solution is saved in common block /case/
!
!======================================================================
!
  real(rkx) function funci3b(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , cc , dd , p4
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    psi4 = p4
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = xk5*(water/gama(2))**3
      a5 = xk7*(water/gama(4))**3
      a6 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
      a7 = sqrt(a4/a5)
!
!   caculate dissociation quantities
!
      bb = psi2 + psi4 + psi5 + a6                         ! psi6
      cc = -a6*(psi2+psi3+psi1)
      dd = bb*bb - 4._rkx*cc
      psi6 = 0.5_rkx*(-bb+sqrt(dd))
!
      psi5 = (psi3+2._rkx*psi4-a7*(3._rkx*psi2+psi1))/2._rkx/a7
      psi5 = max(min(psi5,chi5),zero)
!
!   *** calculate speciation
!   ********************************************
      molal(1) = psi6                                     ! hi
      molal(2) = 2._rkx*psi4 + psi3                         ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1             ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6                ! so4i
      molal(6) = max(psi2+psi3+psi1-psi6,tiny1)             ! hso4i
      clc = max(chi2-psi2,zero)
      cnahso4 = zero
      cna2so4 = max(chi4-psi4,zero)
      cnh42s4 = max(chi5-psi5,zero)
      cnh4hs4 = zero
      call calcmr                                          ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    a4 = xk5*(water/gama(2))**3
    funci3b = molal(5)*molal(2)*molal(2)/a4 - one
  end function funci3b
!
!======================================================================
!
! *** isorropia code
! *** subroutine calci2
! *** case i2
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4 , nahso4 , lc
!
!     there are three regimes in this case:
!     1. nh4hso4(s) possible. liquid & solid aerosol (subroutine calci2a)
!     2. nh4hso4(s) not possible , and rh < mdrh. solid aerosol only
!     3. nh4hso4(s) not possible , and rh >= mdrh. solid & liquid aerosol
!
!     regimes 2. and 3. are considered to be the same as cases i1a , i2b
!     respectively
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calci2
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calci1a
!
! *** regime depends upon the possible solids & rh
! **********************
    if ( cnh4hs4>tiny1 ) then
      scase = 'i2 ; subcase 1'
      call calci2a
      scase = 'i2 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drmi2 ) then            ! solid solution only
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calci1a
        scase = 'i2 ; subcase 2'
      else if ( rh>=drmi2 ) then        ! mdrh of i2
        scase = 'i2 ; subcase 3'
        call calcmdrh(rh,drmi2,drnahso4,calci1a,calci3a)
        scase = 'i2 ; subcase 3'
      end if
    end if
  end subroutine calci2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calci2a
! *** case i2 ; subcase a
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4 , nahso4 , lc
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calci2a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi2hi , psi2lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calci1a      ! needed when called from calcmdrh
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calci1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = zero
    psi3 = zero
    psi4 = zero
    psi5 = zero
!
    calaou = .true.                ! outer loop activity calculation flag
    psi2lo = zero                  ! low  limit
    psi2hi = chi2                  ! high limit
!
! *** initial values for bisection ************************************
!
    x1 = psi2hi
    y1 = funci2a(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with lc *********
!
    if ( yhi>=eps ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi2hi-psi2lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = max(x1-dx,psi2lo)
        y2 = funci2a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   lc
      if ( y2>eps ) y2 = funci2a(zero)
    end if
    return
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funci2a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calci2a')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funci2a(x3)
  end subroutine calci2a
!
!======================================================================
!
! *** isorropia code
! *** subroutine funci2a
! *** case i2 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , na2so4 , nahso4 , lc
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funci2a(p2)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , cc , dd , p2
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    psi2 = p2                      ! save psi2 in common block
    psi3 = chi3
    psi4 = chi4
    psi5 = chi5
    psi6 = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a3 = xk11*(water/gama(12))**2
      a4 = xk5*(water/gama(2))**3
      a5 = xk7*(water/gama(4))**3
      a6 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
      a7 = sqrt(a4/a5)
!
!   caculate dissociation quantities
!
      if ( chi5>tiny1 .and. water>tiny1 ) then
        psi5 = (psi3+2._rkx*psi4-a7*(3._rkx*psi2+psi1))/2._rkx/a7
        psi5 = max(min(psi5,chi5),tiny1)
      end if
!
      if ( chi4>tiny1 .and. water>tiny1 ) then
        aa = psi2 + psi5 + psi6 + psi3
        bb = psi3*aa
        cc = 0.25_rkx*(psi3*psi3*(psi2+psi5+psi6)-a4)
        call poly3(aa,bb,cc,psi4,islv)
        if ( islv==0 ) then
          psi4 = min(psi4,chi4)
        else
          psi4 = zero
        end if
      end if
!
      if ( chi3>tiny1 .and. water>tiny1 ) then
        aa = 2._rkx*psi4 + psi2 + psi1 - psi6
        bb = 2._rkx*psi4*(psi2+psi1-psi6) - a3
        cc = zero
        call poly3(aa,bb,cc,psi3,islv)
        if ( islv==0 ) then
          psi3 = min(psi3,chi3)
        else
          psi3 = zero
        end if
      end if
!
      bb = psi2 + psi4 + psi5 + a6                         ! psi6
      cc = -a6*(psi2+psi3+psi1)
      dd = bb*bb - 4._rkx*cc
      psi6 = 0.5_rkx*(-bb+sqrt(dd))
!
!   *** calculate speciation
!   ********************************************
      molal(1) = psi6                               ! hi
      molal(2) = 2._rkx*psi4 + psi3                   ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1       ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6          ! so4i
      molal(6) = psi2 + psi3 + psi1 - psi6          ! hso4i
      clc = chi2 - psi2
      cnahso4 = chi3 - psi3
      cna2so4 = chi4 - psi4
      cnh42s4 = chi5 - psi5
      cnh4hs4 = zero
      call calcmr                                   ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    a2 = xk13*(water/gama(13))**5.0
    funci2a = molal(5)*molal(6)*molal(3)**30/a2 - one
  end function funci2a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calci1
! *** case i1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4no3 , nh4cl , na2so4
!
!     there are two possible regimes here , depending on relative humidity:
!     1. when rh >= mdrh ; liquid phase possible (mdrh region)
!     2. when rh < mdrh  ; only solid phase possible (subroutine calci1a)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calci1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmi1 ) then
      scase = 'i1 ; subcase 1'
      call calci1a                 ! solid phase only possible
      scase = 'i1 ; subcase 1'
    else
      scase = 'i1 ; subcase 2'     ! liquid & solid phase possible
      call calcmdrh(rh,drmi1,drnh4hs4,calci1a,calci2a)
      scase = 'i1 ; subcase 2'
    end if
!
! *** ammonia in gas phase
! **********************************************
!   call calcnh3
  end subroutine calci1

!======================================================================
!
! *** isorropia code
! *** subroutine calci1a
! *** case i1 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4hso4 , nahso4 , (nh4)2so4 , na2so4 , lc
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calci1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: frnh4 , frso4
!
! *** calculate non volatile solids ***********************************
!
    cna2so4 = 0.5_rkx*w(1)
    cnh4hs4 = zero
    cnahso4 = zero
    cnh42s4 = zero
    frso4 = max(w(2)-cna2so4,zero)
!
    clc = min(w(3)/3._rkx,frso4/2._rkx)
    frso4 = max(frso4-2._rkx*clc,zero)
    frnh4 = max(w(3)-3._rkx*clc,zero)
!
    if ( frso4<=tiny1 ) then
      clc = max(clc-frnh4,zero)
      cnh42s4 = 2._rkx*frnh4

    else if ( frnh4<=tiny1 ) then
      cnh4hs4 = 3._rkx*min(frso4,clc)
      clc = max(clc-frso4,zero)
      if ( cna2so4>tiny1 ) then
        frso4 = max(frso4-cnh4hs4/3._rkx,zero)
        cnahso4 = 2._rkx*frso4
        cna2so4 = max(cna2so4-frso4,zero)
      end if
    end if
!
! *** calculate gas species
! *********************************************
    ghno3 = w(4)
    ghcl = w(5)
    gnh3 = zero
  end subroutine calci1a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcj3
! *** case j3
!
!     the main characteristics of this regime are:
!     1. sulfate rich , free acid (sulrat < 1.0)
!     2. there is only a liquid phase
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcj3
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a3 , bb , cc , chi1 , chi2 , dd , psi1 , psi2
    integer(ik4) :: i
    real(rkx) :: lamda , kapa
!
! *** setup parameters ************************************************
!
    calaou = .true.                ! outer loop activity calculation flag
    frst = .true.
    calain = .true.
!
    lamda = max(w(2)-w(3)-w(1),tiny1)          ! free h2so4
    chi1 = w(1)                               ! na total as nahso4
    chi2 = w(3)                               ! nh4 total as nh4hso4
    psi1 = chi1
    psi2 = chi2                               ! all nh4hso4 deliquesced
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a3 = xk1*water/gama(7)*(gama(8)/gama(7))**2
!
!   caculate dissociation quantities
!
      bb = a3 + lamda                           ! kapa
      cc = -a3*(lamda+psi1+psi2)
      dd = bb*bb - 4._rkx*cc
      kapa = 0.5_rkx*(-bb+sqrt(dd))
!
!   *** calculate speciation
!   ********************************************
      molal(1) = lamda + kapa                     ! hi
      molal(2) = psi1                             ! nai
      molal(3) = psi2                             ! nh4i
      molal(4) = zero                             ! cli
      molal(5) = kapa                             ! so4i
      molal(6) = lamda + psi1 + psi2 - kapa       ! hso4i
      molal(7) = zero                             ! no3i
!
      cnahso4 = zero
      cnh4hs4 = zero
!
      call calcmr                                 ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
  end subroutine calcj3
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcj2
! *** case j2
!
!     the main characteristics of this regime are:
!     1. sulfate rich , free acid (sulrat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : nahso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcj2
    use mod_che_common_isorropia
    use mod_che_common_casej
    implicit none
    real(rkx) :: dx , psi1hi , psi1lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.                ! outer loop activity calculation flag
    chi1 = w(1)                    ! na total
    chi2 = w(3)                    ! nh4 total
    psi1lo = tiny1                  ! low  limit
    psi1hi = chi1                  ! high limit
!
! *** initial values for bisection ************************************
!
    x1 = psi1hi
    y1 = funcj2(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with nh42so4 ****
!
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi1hi-psi1lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funcj2(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   nh42so4
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funcj2(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calcj2')        ! warning error: no solution
      end if
      return
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funcj2(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calcj2')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funcj2(x3)
    end if
  end subroutine calcj2
!
!======================================================================
!
! *** isorropia code
! *** subroutine funcj2
! *** case j2
!
!     the main characteristics of this regime are:
!     1. sulfate rich , free acid (sulrat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funcj2(p1)
    use mod_che_common_isorropia
    use mod_che_common_casej
    implicit none
    real(rkx) :: bb , cc , dd , p1
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
!
    lamda = max(w(2)-w(3)-w(1),tiny1)          ! free h2so4
    psi1 = p1
    psi2 = chi2                               ! all nh4hso4 deliquesced
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk11*(water/gama(12))**2
      a3 = xk1*water/gama(7)*(gama(8)/gama(7))**2
!
!   caculate dissociation quantities
!
      bb = a3 + lamda                           ! kapa
      cc = -a3*(lamda+psi1+psi2)
      dd = bb*bb - 4._rkx*cc
      kapa = 0.5_rkx*(-bb+sqrt(dd))
!
!   *** calculate speciation
!   ********************************************
      molal(1) = lamda + kapa                      ! hi
      molal(2) = psi1                              ! nai
      molal(3) = psi2                              ! nh4i
      molal(4) = zero                              ! cli
      molal(5) = kapa                              ! so4i
      molal(6) = lamda + psi1 + psi2 - kapa        ! hso4i
      molal(7) = zero                              ! no3i
!
      cnahso4 = max(chi1-psi1,zero)
      cnh4hs4 = zero
!
      call calcmr                                  ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    funcj2 = molal(2)*molal(6)/a1 - one
  end function funcj2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcj1
! *** case j1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , free acid (sulrat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : nh4hso4 , nahso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  subroutine calcj1
    use mod_che_common_isorropia
    use mod_che_common_casej
    implicit none
    real(rkx) :: dx , psi1hi , psi1lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.                ! outer loop activity calculation flag
    chi1 = w(1)                    ! total na initially as nahso4
    chi2 = w(3)                    ! total nh4 initially as nh4hso4
!
    psi1lo = tiny1                  ! low  limit
    psi1hi = chi1                  ! high limit
!
! *** initial values for bisection ************************************
!
    x1 = psi1hi
    y1 = funcj1(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with nh42so4 ****
!
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi1hi-psi1lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funcj1(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   nh42so4
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funcj1(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calcj1')        ! warning error: no solution
      end if
      return
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funcj1(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calcj1')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funcj1(x3)
    end if
  end subroutine calcj1
!======================================================================
!
! *** isorropia code
! *** subroutine funcj1
! *** case j1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , free acid (sulrat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
! *** updated by christos fountoukis
!
!======================================================================
!
  real(rkx) function funcj1(p1)
    use mod_che_common_isorropia
    use mod_che_common_casej
    implicit none
    real(rkx) :: bb , cc , dd , p1
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
!
    lamda = max(w(2)-w(3)-w(1),tiny1)          ! free h2so4
    psi1 = p1
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk11*(water/gama(12))**2
      a2 = xk12*(water/gama(09))**2
      a3 = xk1*water/gama(7)*(gama(8)/gama(7))**2
!
      psi2 = 0.5_rkx*(-(lamda+psi1)+sqrt((lamda+psi1)**20+4._rkx*a2))       ! psi2
      psi2 = min(psi2,chi2)
!
      bb = a3 + lamda                           ! kapa
      cc = -a3*(lamda+psi2+psi1)
      dd = bb*bb - 4._rkx*cc
      kapa = 0.5_rkx*(-bb+sqrt(dd))
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = lamda + kapa                      ! hi
      molal(2) = psi1                              ! nai
      molal(3) = psi2                              ! nh4i
      molal(4) = zero
      molal(5) = kapa                              ! so4i
      molal(6) = lamda + psi1 + psi2 - kapa        ! hso4i
      molal(7) = zero
!
      cnahso4 = max(chi1-psi1,zero)
      cnh4hs4 = max(chi2-psi2,zero)
!
      call calcmr                                  ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    funcj1 = molal(2)*molal(6)/a1 - one
  end function funcj1
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calco7
! *** case o7
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4
!     4. completely dissolved: nh4no3 , nh4cl , (nh4)2so4 , mgso4 , na2so4 , k2so4
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calco7
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3 , nafr
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi9 = min(w(6),w(2))                           ! ccaso4
    so4fr = max(w(2)-chi9,zero)
    cafr = max(w(6)-chi9,zero)
    chi7 = min(0.5_rkx*w(7),so4fr)                    ! ck2so4
    frk = max(w(7)-2._rkx*chi7,zero)
    so4fr = max(so4fr-chi7,zero)
    chi1 = min(0.5_rkx*w(1),so4fr)                    ! na2so4
    nafr = max(w(1)-2._rkx*chi1,zero)
    so4fr = max(so4fr-chi1,zero)
    chi8 = min(w(8),so4fr)                          ! cmgso4
    frmg = max(w(8)-chi8,zero)
    so4fr = max(so4fr-chi8,zero)
    chi3 = zero
    chi5 = w(4)
    chi6 = w(5)
    chi2 = max(so4fr,zero)
    chi4 = max(w(3)-2._rkx*chi2,zero)
!
    psi1 = chi1
    psi2 = chi2
    psi3 = zero
    psi4 = zero
    psi5 = zero
    psi6 = zero
    psi7 = chi7
    psi8 = chi8
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
    water = chi2/m0(4) + chi1/m0(2) + chi7/m0(17) + chi8/m0(21)
    water = max(water,tiny1)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funco7(x1)
    y2 = y1
    if ( chi6>tiny1 ) then
!   if (abs(y1) <= eps .or. chi6 <= tiny1) goto 50
!   if (water  <=  tiny1) return                    ! no water
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funco7(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funco7(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funco7(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calco7')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funco7(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then  ! if quadrat.called
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                       ! h+   effect
      molal(5) = molal(5) - delta                       ! so4  effect
      molal(6) = delta                                  ! hso4 effect
    end if
  end subroutine calco7
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funco7
! *** case o7
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4
!     4. completely dissolved: nh4no3 , nh4cl , (nh4)2so4 , mgso4 , na2so4 , k2so4
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funco7(x)
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: bb , cc , dd , hi , ohi , smin , x
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    psi6 = x
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
!
!
      if ( chi5>=tiny1 ) then
        psi5 = psi6*chi5/(a6/a5*(chi6-psi6)+psi6)
        psi5 = min(psi5,chi5)
      else
        psi5 = tiny1
      end if
!
!   if(chi4 > tiny1) then
      if ( w(2)>tiny1 ) then         ! accounts for nh3 evaporation
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6) - 2._rkx*psi2/a4
        dd = max(bb*bb-4._rkx*cc,zero)                  ! patch proposed by uma shankar , 19/11/01
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = max(min(psi4,chi4),zero)
      else
        psi4 = tiny1
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(2) = 2.0_rkx*psi1                           ! na+
      molal(3) = 2.0_rkx*psi2 + psi4                    ! nh4i
      molal(4) = psi6                                 ! cli
      molal(5) = psi1 + psi2 + psi7 + psi8            ! so4i
      molal(6) = zero                                 ! hso4
      molal(7) = psi5                                 ! no3i
      molal(8) = zero                                 ! cai
      molal(9) = 2.0_rkx*psi7                           ! ki
      molal(10) = psi8                                 ! mg
!
!   *** calculate hso4 speciation and return
!   *******************************
!   molal (1) = max(chi5 - psi5 , tiny1)*a5/psi5   ! hi
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cna2so4 = zero
      cnh42s4 = zero
      cnh4no3 = zero
      cnh4cl = zero
      ck2so4 = zero
      cmgso4 = zero
      ccaso4 = chi9
!
!   *** calculate molalr array , water and activities
!   **********************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    funco7 = molal(1)*molal(4)/ghcl/a6 - one
  end function funco7
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calco6
! *** case o6
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4
!     4. completely dissolved: nh4no3 , nh4cl , (nh4)2so4 , mgso4 , na2so4
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calco6
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3 , nafr
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi9 = min(w(6),w(2))                           ! ccaso4
    so4fr = max(w(2)-chi9,zero)
    cafr = max(w(6)-chi9,zero)
    chi7 = min(0.5_rkx*w(7),so4fr)                    ! ck2so4
    frk = max(w(7)-2._rkx*chi7,zero)
    so4fr = max(so4fr-chi7,zero)
    chi1 = min(0.5_rkx*w(1),so4fr)                    ! na2so4
    nafr = max(w(1)-2._rkx*chi1,zero)
    so4fr = max(so4fr-chi1,zero)
    chi8 = min(w(8),so4fr)                          ! cmgso4
    frmg = max(w(8)-chi8,zero)
    so4fr = max(so4fr-chi8,zero)
    chi3 = zero
    chi5 = w(4)
    chi6 = w(5)
    chi2 = max(so4fr,zero)
    chi4 = max(w(3)-2._rkx*chi2,zero)
!
!
    psi1 = chi1
    psi2 = chi2
    psi3 = zero
    psi7 = zero
    psi8 = chi8
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
    water = chi2/m0(4) + chi1/m0(2) + chi7/m0(17) + chi8/m0(21)
    water = max(water,tiny1)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funco6(x1)
    y2 = y1
    if ( chi6>tiny1 ) then
!   if (abs(y1) <= eps .or. chi6 <= tiny1) goto 50
!   if (water  <=  tiny1) return                    ! no water
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funco6(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funco6(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funco6(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calco6')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funco6(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then  ! if quadrat.called
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                       ! h+   effect
      molal(5) = molal(5) - delta                       ! so4  effect
      molal(6) = delta                                  ! hso4 effect
    end if
  end subroutine calco6
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funco6
! *** case o6
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4
!     4. completely dissolved: nh4no3 , nh4cl , (nh4)2so4 , mgso4 , na2so4
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funco6(x)
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: bb , cc , dd , hi , ohi , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk17*(water/gama(17))**3
!
!
      if ( chi5>=tiny1 ) then
        psi5 = psi6*chi5/(a6/a5*(chi6-psi6)+psi6)
        psi5 = min(psi5,chi5)
      else
        psi5 = tiny1
      end if
!
!   if(chi4 > tiny1) then
      if ( w(2)>tiny1 ) then         ! accounts for nh3 evaporation
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6) - 2._rkx*psi2/a4
        dd = max(bb*bb-4._rkx*cc,zero)                  ! patch proposed by uma shankar , 19/11/01
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = max(min(psi4,chi4),zero)
      else
        psi4 = tiny1
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then             ! psi7
        call poly3(psi1+psi2+psi8,zero,-a7/4._rkx,psi7,islv)
        if ( islv==0 ) then
          psi7 = max(min(psi7,chi7),zero)
        else
          psi7 = zero
        end if
      else
        psi7 = zero
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(2) = 2.0_rkx*psi1                           ! na+
      molal(3) = 2.0_rkx*psi2 + psi4                    ! nh4i
      molal(4) = psi6                                 ! cli
      molal(5) = psi1 + psi2 + psi7 + psi8            ! so4i
      molal(6) = zero                                 ! hso4
      molal(7) = psi5                                 ! no3i
      molal(8) = zero                                 ! cai
      molal(9) = 2.0_rkx*psi7                           ! ki
      molal(10) = psi8                                 ! mg
!
!   *** calculate hso4 speciation and return
!   *******************************
!   molal (1) = max(chi5 - psi5 , tiny1)*a5/psi5   ! hi
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cna2so4 = zero
      cnh42s4 = zero
      cnh4no3 = zero
      cnh4cl = zero
      ck2so4 = max(chi7-psi7,tiny1)
      cmgso4 = zero
      ccaso4 = chi9
!
!   *** calculate molalr array , water and activities
!   **********************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    funco6 = molal(1)*molal(4)/ghcl/a6 - one
  end function funco6
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calco5
! *** case o5
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4
!     4. completely dissolved: nh4no3 , nh4cl , (nh4)2so4 , mgso4
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calco5
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3 , nafr
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi9 = min(w(6),w(2))                           ! ccaso4
    so4fr = max(w(2)-chi9,zero)
    cafr = max(w(6)-chi9,zero)
    chi7 = min(0.5_rkx*w(7),so4fr)                    ! ck2so4
    frk = max(w(7)-2._rkx*chi7,zero)
    so4fr = max(so4fr-chi7,zero)
    chi1 = min(0.5_rkx*w(1),so4fr)                    ! na2so4
    nafr = max(w(1)-2._rkx*chi1,zero)
    so4fr = max(so4fr-chi1,zero)
    chi8 = min(w(8),so4fr)                          ! cmgso4
    frmg = max(w(8)-chi8,zero)
    so4fr = max(so4fr-chi8,zero)
    chi3 = zero
    chi5 = w(4)
    chi6 = w(5)
    chi2 = max(so4fr,zero)
    chi4 = max(w(3)-2._rkx*chi2,zero)
!
    psi1 = zero
    psi2 = chi2
    psi3 = zero
    psi7 = zero
    psi8 = chi8
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
    water = chi2/m0(4) + chi1/m0(2) + chi7/m0(17) + chi8/m0(21)
    water = max(water,tiny1)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funco5(x1)
    y2 = y1
    if ( chi6>tiny1 ) then
!   if (abs(y1) <= eps .or. chi6 <= tiny1) goto 50
!   if (water  <=  tiny1) return                    ! no water
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funco5(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funco5(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funco5(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calco5')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funco5(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then  ! if quadrat.called
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                       ! h+   effect
      molal(5) = molal(5) - delta                       ! so4  effect
      molal(6) = delta                                  ! hso4 effect
    end if
  end subroutine calco5
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funco5
! *** case o5
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4
!     4. completely dissolved: nh4no3 , nh4cl , (nh4)2so4 , mgso4
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funco5(x)
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: bb , cc , dd , hi , ohi , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk17*(water/gama(17))**3
!
      if ( chi5>=tiny1 ) then
        psi5 = psi6*chi5/(a6/a5*(chi6-psi6)+psi6)
        psi5 = min(psi5,chi5)
      else
        psi5 = tiny1
      end if
!
!   if(chi4 > tiny1) then
      if ( w(2)>tiny1 ) then         ! accounts for nh3 evaporation
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6) - 2._rkx*psi2/a4
        dd = max(bb*bb-4._rkx*cc,zero)                  ! patch proposed by uma shankar , 19/11/01
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = max(min(psi4,chi4),zero)
      else
        psi4 = tiny1
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then             ! psi7
        call poly3((psi2+psi8)/(sqrt(a1/a7)+1._rkx),zero,                   &
                   -a7/4._rkx/(sqrt(a1/a7)+1._rkx),psi7,islv)
        if ( islv==0 ) then
          psi7 = max(min(psi7,chi7),zero)
        else
          psi7 = zero
        end if
      else
        psi7 = zero
      end if
!
      if ( chi1>=tiny1 ) then                                 ! psi1
        psi1 = sqrt(a1/a7)*psi7
        psi1 = min(psi1,chi1)
      else
        psi1 = zero
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(2) = 2.0_rkx*psi1                           ! nai
      molal(3) = 2.0_rkx*psi2 + psi4                    ! nh4i
      molal(4) = psi6                                 ! cli
      molal(5) = psi1 + psi2 + psi7 + psi8            ! so4i
      molal(6) = zero                                 ! hso4
      molal(7) = psi5                                 ! no3i
      molal(8) = zero                                 ! cai
      molal(9) = 2.0_rkx*psi7                           ! ki
      molal(10) = psi8                                 ! mg
!
!   *** calculate hso4 speciation and return
!   *******************************
!   molal (1) = max(chi5 - psi5 , tiny1)*a5/psi5   ! hi
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cna2so4 = max(chi1-psi1,tiny1)
      cnh42s4 = zero
      cnh4no3 = zero
      cnh4cl = zero
      ck2so4 = max(chi7-psi7,tiny1)
      cmgso4 = zero
      ccaso4 = chi9
!
!   *** calculate molalr array , water and activities
!   **********************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    funco5 = molal(1)*molal(4)/ghcl/a6 - one
  end function funco5
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calco4
! *** case o4
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : na2so4 , k2so4 , mgso4 , caso4
!     4. completely dissolved: nh4no3 , nh4cl , (nh4)2so4
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calco4
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3 , nafr
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi9 = min(w(6),w(2))                           ! ccaso4
    so4fr = max(w(2)-chi9,zero)
    cafr = max(w(6)-chi9,zero)
    chi7 = min(0.5_rkx*w(7),so4fr)                    ! ck2so4
    frk = max(w(7)-2._rkx*chi7,zero)
    so4fr = max(so4fr-chi7,zero)
    chi1 = min(0.5_rkx*w(1),so4fr)                    ! na2so4
    nafr = max(w(1)-2._rkx*chi1,zero)
    so4fr = max(so4fr-chi1,zero)
    chi8 = min(w(8),so4fr)                          ! cmgso4
    frmg = max(w(8)-chi8,zero)
    so4fr = max(so4fr-chi8,zero)
    chi3 = zero
    chi5 = w(4)
    chi6 = w(5)
    chi2 = max(so4fr,zero)
    chi4 = max(w(3)-2._rkx*chi2,zero)
!
    psi2 = chi2
    psi3 = zero
    psi7 = zero
    psi8 = chi8
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
    water = chi2/m0(4) + chi1/m0(2) + chi7/m0(17) + chi8/m0(21)
    water = max(water,tiny1)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funco4(x1)
    y2 = y1
    if ( chi6>tiny1 ) then
!   if (abs(y1) <= eps .or. chi6 <= tiny1) goto 50
!   if (water  <=  tiny1) goto 50               ! no water
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funco4(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funco4(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funco4(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calco4')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funco4(x3)
!
! *** final calculations **********************************************
!
!
! *** na2so4 dissolution
!
 300  if ( chi1>tiny1 .and. water>tiny1 ) then        ! psi1
      call poly3(psi2+psi7+psi8,zero,-a1/4._rkx,psi1,islv)
      if ( islv==0 ) then
        psi1 = min(psi1,chi1)
      else
        psi1 = zero
      end if
    else
      psi1 = zero
    end if
    molal(2) = 2.0_rkx*psi1                 ! na+  effect
    molal(5) = molal(5) + psi1            ! so4  effect
    cna2so4 = max(chi1-psi1,zero)          ! na2so4(s) depletion
!
! *** hso4 equilibrium
!
    if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta        ! h+   affect
      molal(5) = molal(5) - delta        ! so4  affect
      molal(6) = delta                   ! hso4 affect
    end if
  end subroutine calco4
!
!======================================================================
!
! *** isorropia code
! *** subroutine funco4
! *** case o4 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funco4(x)
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: bb , cc , dd , hi , ohi , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi2 = chi2
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk17*(water/gama(17))**3
!   a8  = xk23 *(water/gama(21))**2
!
!   caculate dissociation quantities
!
      if ( chi5>=tiny1 ) then
        psi5 = psi6*chi5/(a6/a5*(chi6-psi6)+psi6)
        psi5 = min(psi5,chi5)
      else
        psi5 = tiny1
      end if
!
!   if(chi4 > tiny1) then
      if ( w(2)>tiny1 ) then         ! accounts for nh3 evaporation
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6) - 2._rkx*psi2/a4
        dd = max(bb*bb-4._rkx*cc,zero)                  ! patch proposed by uma shankar , 19/11/01
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = max(min(psi4,chi4),zero)
      else
        psi4 = tiny1
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then             ! psi7
        call poly3(psi2+psi8,zero,-a7/4._rkx,psi7,islv)
        if ( islv==0 ) then
          psi7 = max(min(psi7,chi7),zero)
        else
          psi7 = zero
        end if
      else
        psi7 = zero
      end if
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      molal(2) = zero                                 ! nai
      molal(3) = 2.0_rkx*psi2 + psi4                    ! nh4i
      molal(4) = psi6                                 ! cli
      molal(5) = psi2 + psi7 + psi8                   ! so4i
      molal(6) = zero                                 ! hso4
      molal(7) = psi5                                 ! no3i
      molal(8) = zero                                 ! cai
      molal(9) = 2.0_rkx*psi7                           ! ki
      molal(10) = psi8                                 ! mgi
!
!   *** calculate hso4 speciation and return
!   *******************************
!   molal (1) = max(chi5 - psi5 , tiny1)*a5/psi5   ! hi
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnh4cl = zero
      ck2so4 = max(chi7-psi7,tiny1)
      cmgso4 = zero
      ccaso4 = chi9
!
      call calcmr                                        ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    funco4 = molal(1)*molal(4)/ghcl/a6 - one
! funco4 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
  end function funco4
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calco3
! *** case o3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , nh4no3 , nh4cl , na2so4 , k2so4 , mgso4 , caso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calco3
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** regime depends on the existance of water and of the rh
! ************
    if ( w(4)>tiny1 .and. w(5)>tiny1 ) then     ! no3 ,cl exist , water possible
      scase = 'o3 ; subcase 1'
      call calco3a
      scase = 'o3 ; subcase 1'
    else                                        ! no3 , cl non existant
      scase = 'o1 ; subcase 1'
      call calco1a
      scase = 'o1 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drmo3 ) then           ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calco1a
        scase = 'o3 ; subcase 2'
        return
      else
        scase = 'o3 ; subcase 3'      ! mdrh region (na2so4 , nh42s4 , k2so4 , mgso4 , caso4)
        call calcmdrh2(rh,drmo3,drnh42s4,calco1a,calco4)
        scase = 'o3 ; subcase 3'
      end if
    end if
  end subroutine calco3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calco3a
! *** case o3 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , na2so4 , k2so4 , mgso4 , caso4
!     4. completely dissolved: nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calco3a
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3 , nafr
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi9 = min(w(6),w(2))                           ! ccaso4
    so4fr = max(w(2)-chi9,zero)
    cafr = max(w(6)-chi9,zero)
    chi7 = min(0.5_rkx*w(7),so4fr)                    ! ck2so4
    frk = max(w(7)-2._rkx*chi7,zero)
    so4fr = max(so4fr-chi7,zero)
    chi1 = min(0.5_rkx*w(1),so4fr)                    ! na2so4
    nafr = max(w(1)-2._rkx*chi1,zero)
    so4fr = max(so4fr-chi1,zero)
    chi8 = min(w(8),so4fr)                          ! cmgso4
    frmg = max(w(8)-chi8,zero)
    so4fr = max(so4fr-chi8,zero)
    chi3 = zero
    chi5 = w(4)
    chi6 = w(5)
    chi2 = max(so4fr,zero)
    chi4 = max(w(3)-2._rkx*chi2,zero)
!
    psi8 = chi8
    psi6lo = tiny1
    psi6hi = chi6 - tiny1
!
    water = tiny1
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funco3a(x1)
    y2 = y1
    if ( chi6>tiny1 ) then
!   if (abs(y1) <= eps .or. chi7 <= tiny1) goto 50
!   if (water  <=  tiny1) goto 50               ! no water
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funco3a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funco3a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funco3a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calco3a')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funco3a(x3)
!
! *** final calculations
! *************************************************
!
! *** na2so4 dissolution
!
 300  if ( chi1>tiny1 .and. water>tiny1 ) then        ! psi1
      call poly3(psi2+psi7+psi8,zero,-a1/4._rkx,psi1,islv)
      if ( islv==0 ) then
        psi1 = min(max(psi1,zero),chi1)
      else
        psi1 = zero
      end if
    else
      psi1 = zero
    end if
    molal(2) = 2.0_rkx*psi1                 ! na+  effect
    molal(5) = molal(5) + psi1            ! so4  effect
    cna2so4 = max(chi1-psi1,zero)          ! na2so4(s) depletion
!
! *** hso4 equilibrium
!
    if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta        ! h+   affect
      molal(5) = molal(5) - delta        ! so4  affect
      molal(6) = delta                   ! hso4 affect
    end if
  end subroutine calco3a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funco3a
! *** case o3; subcase 1
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , na2so4 , k2so4 , mgso4 , caso4
!     4. completely dissolved: nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funco3a(x)
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: a65 , bb , cc , dd , deno , hi , ohi , psi20 , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi2 = chi2
    psi8 = chi8
    psi3 = zero
    psi6 = x
!
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a2 = xk7*(water/gama(4))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk17*(water/gama(17))**3
!   a8  = xk23 *(water/gama(21))**2
      a65 = a6/a5
!
!   caculate dissociation quantities
!
      deno = max(chi6-psi6-psi3,zero)
      psi5 = psi6*chi5/(a6/a5*deno+psi6)
      psi5 = min(max(psi5,zero),chi5)
!
!   if(chi4 > tiny1) then                             ! psi4
      if ( w(2)>tiny1 ) then         ! accounts for nh3 evaporation
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6) - 2._rkx*psi2/a4
        dd = max(bb*bb-4._rkx*cc,zero)         ! patch proposed by uma shankar , 19/11/01
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
      else
        psi4 = tiny1
      end if
      psi4 = min(max(psi4,zero),chi4)
!
      if ( chi7>tiny1 .and. water>tiny1 ) then             ! psi7
        call poly3(psi2+psi8,zero,-a7/4._rkx,psi7,islv)
        if ( islv==0 ) then
          psi7 = max(min(psi7,chi7),zero)
        else
          psi7 = zero
        end if
      else
        psi7 = zero
      end if
!
      if ( chi2>tiny1 .and. water>tiny1 ) then
        call poly3(psi7+psi8+psi4,psi4*(psi7+psi8)+psi4*psi4/4._rkx,        &
                   (psi4*psi4*(psi7+psi8)-a2)/4._rkx,psi20,islv)
        if ( islv==0 ) psi2 = min(max(psi20,zero),chi2)
      end if
!   psi2 = 0.5d0*(2.0d0*sqrt(a2/a7)*psi7 - psi4)
!   psi2 = min (max(psi2 , zero), chi2)
!
!   *** save concentrations in molal array
!   ******************************
      molal(2) = zero                                 ! nai
      molal(3) = 2.0_rkx*psi2 + psi4                    ! nh4i
      molal(4) = psi6                                 ! cli
      molal(5) = psi2 + psi7 + psi8                   ! so4i
      molal(6) = zero                                 ! hso4
      molal(7) = psi5                                 ! no3i
      molal(8) = zero                                 ! cai
      molal(9) = 2.0_rkx*psi7                           ! ki
      molal(10) = psi8                                 ! mgi
!
!   molal (1) = max(chi5 - psi5 , tiny1)*a5/psi5   ! hi
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
!   cna2so4  = max(chi1 - psi1 , zero)
      cnh42s4 = max(chi2-psi2,zero)
      cnh4no3 = zero
      cnh4cl = zero
      ck2so4 = max(chi7-psi7,zero)
      cmgso4 = zero
      ccaso4 = chi9
!
!   *** calculate molalr array , water and activities
!   **********************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
    funco3a = molal(1)*molal(4)/ghcl/a6 - one
  end function funco3a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calco2
! *** case o2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , nh4no3 , nh4cl , na2so4 , k2so4 , mgso4 , caso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calco2
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** regime depends on the existance of nitrates
! ***********************
    if ( w(4)>tiny1 ) then          ! no3 exists , water possible
      scase = 'o2 ; subcase 1'
      call calco2a
      scase = 'o2 ; subcase 1'
    else                            ! no3 non existant , water not possible
      scase = 'o1 ; subcase 1'
      call calco1a
      scase = 'o1 ; subcase 1'
    end if
!
! *** regime depends on the existance of water and of the rh
! ************
    if ( water<=tiny1 ) then
      if ( rh<drmo2 ) then                ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calco1a
        scase = 'o2 ; subcase 2'
      else
        if ( w(5)>tiny1 ) then
          scase = 'o2 ; subcase 3'         ! mdrh (nh4cl , na2so4 , nh42s4 , k2so4 , mgso4 , caso4)
          call calcmdrh2(rh,drmo2,drnh4cl,calco1a,calco3a)
          scase = 'o2 ; subcase 3'
        end if
        if ( water<=tiny1 .and. rh>=drmo3 ) then
          scase = 'o2 ; subcase 4'         ! mdrh (na2so4 , nh42s4 , k2so4 , mgso4 , caso4)
          call calcmdrh2(rh,drmo3,drnh42s4,calco1a,calco4)
          scase = 'o2 ; subcase 4'
        else
          water = tiny1
          do i = 1 , nions
            molal(i) = zero
          end do
          call calco1a
          scase = 'o2 ; subcase 2'
        end if
      end if
    end if
  end subroutine calco2
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calco2a
! *** case o2 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4 , k2so4 , mgso4 , caso4
!     4. completely dissolved: nh4no3
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calco2a
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: cafr , nafr , delta , dx , frk , frmg , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i , islv
!
! *** setup parameters *************************************************
!
    calaou = .true.
    chi9 = min(w(6),w(2))                           ! ccaso4
    so4fr = max(w(2)-chi9,zero)
    cafr = max(w(6)-chi9,zero)
    chi7 = min(0.5_rkx*w(7),so4fr)                    ! ck2so4
    frk = max(w(7)-2._rkx*chi7,zero)
    so4fr = max(so4fr-chi7,zero)
    chi1 = min(0.5_rkx*w(1),so4fr)                    ! na2so4
    nafr = max(w(1)-2._rkx*chi1,zero)
    so4fr = max(so4fr-chi1,zero)
    chi8 = min(w(8),so4fr)                          ! cmgso4
    frmg = max(w(8)-chi8,zero)
    so4fr = max(so4fr-chi8,zero)
    chi3 = zero
    chi5 = w(4)
    chi6 = w(5)
    chi2 = max(so4fr,zero)
    chi4 = max(w(3)-2._rkx*chi2,zero)
!
    psi8 = chi8
    psi6lo = tiny1
    psi6hi = chi6 - tiny1
!
    water = tiny1
!
! *** initial values for bisection *************************************
!
    x1 = psi6lo
    y1 = funco2a(x1)
    y2 = y1
    if ( chi6>tiny1 ) then
!   if (abs(y1) <= eps .or. chi6 <= tiny1) goto 50
!   if (water  <=  tiny1) goto 50               ! no water
!
!   *** root tracking ; for the range of hi and lo
!   ***********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funco2a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) water = tiny1
    end if
    go to 300
!
! *** perform bisection ************************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funco2a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calco2a')         ! warning error: no convergence
!
! *** converged ; return ***********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    if ( x3<=tiny2 ) then     ! practically no nitrates , so dry solution
      water = tiny1
    else
      y3 = funco2a(x3)
    end if
!
! *** final calculations
! *************************************************
!
! *** na2so4 dissolution
!
 300  if ( chi1>tiny1 .and. water>tiny1 ) then        ! psi1
      call poly3(psi2+psi7+psi8,zero,-a1/4._rkx,psi1,islv)
      if ( islv==0 ) then
        psi1 = min(psi1,chi1)
      else
        psi1 = zero
      end if
    else
      psi1 = zero
    end if
    molal(2) = 2.0_rkx*psi1                 ! na+  effect
    molal(5) = molal(5) + psi1            ! so4  effect
    cna2so4 = max(chi1-psi1,zero)          ! na2so4(s) depletion
!
! *** hso4 equilibrium
!
    if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta        ! h+   affect
      molal(5) = molal(5) - delta        ! so4  affect
      molal(6) = delta                   ! hso4 affect
    end if
  end subroutine calco2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funco2a
! *** case o2; subcase 1
!
!     the main characteristics of this regime are:
!     1. (rsulfate > 2.0 ; r(cr+na) < 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4 , nh4cl , na2so4 , k2so4 , mgso4 , caso4
!     4. completely dissolved: nh4no3
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funco2a(x)
    use mod_che_common_isorropia
    use mod_che_common_caseo
    implicit none
    real(rkx) :: a65 , bb , cc , dd , delt , deno , hi , ohi , psi20 , &
         psi31 , psi32 , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi2 = chi2
    psi3 = zero
!
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a2 = xk7*(water/gama(4))**3
      a3 = xk6/(r*temp*r*temp)
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a65 = a6/a5
      a7 = xk17*(water/gama(17))**3
!   a8  = xk23 *(water/gama(21))**2
!
      deno = max(chi6-psi6-psi3,zero)
      psi5 = psi6*chi5/(a6/a5*deno+psi6)
      psi5 = min(psi5,chi5)
!
      psi4 = min(psi5+psi6,chi4)
!
!
      if ( chi7>tiny1 .and. water>tiny1 ) then             ! psi7
        call poly3(psi2+psi8,zero,-a7/4._rkx,psi7,islv)
        if ( islv==0 ) then
          psi7 = max(min(psi7,chi7),zero)
        else
          psi7 = zero
        end if
      else
        psi7 = zero
      end if
!
      if ( chi2>tiny1 .and. water>tiny1 ) then
        call poly3(psi7+psi8+psi4,psi4*(psi7+psi8)+psi4*psi4/4._rkx,        &
                   (psi4*psi4*(psi7+psi8)-a2)/4._rkx,psi20,islv)
        if ( islv==0 ) psi2 = min(max(psi20,zero),chi2)
      end if
!   psi2 = 0.5d0*(2.0d0*sqrt(a2/a7)*psi7 - psi4)
!   psi2 = min (psi2 , chi2)
!
!   *** save concentrations in molal array
!   ******************************
      molal(2) = zero                                 ! nai
      molal(3) = 2.0_rkx*psi2 + psi4                    ! nh4i
      molal(4) = psi6                                 ! cli
      molal(5) = psi2 + psi7 + psi8                   ! so4i
      molal(6) = zero                                 ! hso4
      molal(7) = psi5                                 ! no3i
      molal(8) = zero                                 ! cai
      molal(9) = 2.0_rkx*psi7                           ! ki
      molal(10) = psi8                                 ! mgi
!
!   molal (1) = max(chi5 - psi5 , tiny1)*a5/psi5   ! hi
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
!   cna2so4  = max(chi1 - psi1 , zero)
      cnh42s4 = max(chi2-psi2,zero)
      cnh4no3 = zero
      ck2so4 = max(chi7-psi7,zero)
      cmgso4 = zero
      ccaso4 = chi9
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = min(min(psi3,chi4-psi4),chi6-psi6)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
!   *** calculate molalr array , water and activities
!   *********************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop **************************
!
! if (chi4 <= tiny1) then
! funco2a = molal(1)*molal(4)/ghcl/a6 - one
! else
    funco2a = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
! end if
  end function funco2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calco1
! *** case o1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. solid & liquid aerosol possible
!     3. solids possible : (nh4)2so4 , nh4no3 , nh4cl , na2so4 , k2so4 , mgso4 , caso4
!
!     there are two possible regimes here , depending on relative humidity:
!     1. when rh >= mdrh ; liquid phase possible (mdrh region)
!     2. when rh < mdrh  ; only solid phase possible (subroutine calco1a)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calco1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmo1 ) then
      scase = 'o1 ; subcase 1'
      call calco1a                 ! solid phase only possible
      scase = 'o1 ; subcase 1'
    else
      scase = 'o1 ; subcase 2'     ! liquid & solid phase possible
      call calcmdrh2(rh,drmo1,drnh4no3,calco1a,calco2a)
      scase = 'o1 ; subcase 2'
    end if
  end subroutine calco1
!======================================================================
!
! *** isorropia code ii
! *** subroutine calco1a
! *** case o1a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4 , nh4no3 , nh4cl , na2so4 , k2so4 , mgso4 , caso4
!
!     solid (nh4)2so4 is calculated from the sulfates , while nh4no3
!     is calculated from nh3-hno3 equilibrium. 'ze' is the amount of
!     nh4no3 that volatizes when all possilbe nh4no3 is initially in
!     the solid phase.
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calco1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a1 , a2 , alf , bb , bet , cafr , cc , dd , dd1 , dd2 , &
         frk , frmg , frna , gam , rtsq , so4fr , sqdd , sqdd1 , sqdd2 , &
         theta1 , theta2
    real(rkx) :: lamda , lamda1 , lamda2 , kapa , kapa1 , kapa2
!
! *** calculate non volatile solids ***********************************
!
    ccaso4 = min(w(6),w(2))                         ! ccaso4
    so4fr = max(w(2)-ccaso4,zero)
    cafr = max(w(6)-ccaso4,zero)
    ck2so4 = min(0.5_rkx*w(7),so4fr)                  ! ck2s04
    frk = max(w(7)-2._rkx*ck2so4,zero)
    so4fr = max(so4fr-ck2so4,zero)
    cna2so4 = min(0.5_rkx*w(1),so4fr)                 ! cna2so4
    frna = max(w(1)-2._rkx*cna2so4,zero)
    so4fr = max(so4fr-cna2so4,zero)
    cmgso4 = min(w(8),so4fr)                        ! cmgso4
    frmg = max(w(8)-cmgso4,zero)
    so4fr = max(so4fr-cmgso4,zero)
!
    cnh42s4 = max(so4fr,zero)                       ! cnh42s4
!
! *** calculate volatile species **************************************
!
    alf = w(3) - 2.0_rkx*cnh42s4
    bet = w(5)
    gam = w(4)
!
    rtsq = r*temp*r*temp
    a1 = xk6/rtsq
    a2 = xk10/rtsq
    !print * , a2
!
    theta1 = gam - bet*(a2/a1)
    theta2 = a2/a1
!
! quaratic equation solution
!
    bb = (theta1-alf-bet*(one+theta2))/(one+theta2)
    cc = (alf*bet-a1-bet*theta1)/(one+theta2)
    dd = bb*bb - 4.0_rkx*cc
    if ( dd>=zero ) then        ! solve each reaction seperately
!
!   two roots for kapa , check and see if any valid
!
      sqdd = sqrt(dd)
      kapa1 = 0.5_rkx*(-bb+sqdd)
      kapa2 = 0.5_rkx*(-bb-sqdd)
      lamda1 = theta1 + theta2*kapa1
      lamda2 = theta1 + theta2*kapa2
!
      if ( kapa1>=zero .and. lamda1>=zero ) then
        if ( alf-kapa1-lamda1>=zero .and. bet-kapa1>=zero .and.           &
             gam-lamda1>=zero ) then
          kapa = kapa1
          lamda = lamda1
          go to 100
        end if
      end if
!
      if ( kapa2>=zero .and. lamda2>=zero ) then
        if ( alf-kapa2-lamda2>=zero .and. bet-kapa2>=zero .and.           &
             gam-lamda2>=zero ) then
          kapa = kapa2
          lamda = lamda2
          go to 100
        end if
      end if
    end if
!
! seprate solution of nh4cl & nh4no3 equilibria
!
    kapa = zero
    lamda = zero
    dd1 = (alf+bet)*(alf+bet) - 4.0_rkx*(alf*bet-a1)
    dd2 = (alf+gam)*(alf+gam) - 4.0_rkx*(alf*gam-a2)
!
! nh4l equilibrium
!
    if ( dd1>=zero ) then
      sqdd1 = sqrt(dd1)
      kapa1 = 0.5_rkx*(alf+bet+sqdd1)
      kapa2 = 0.5_rkx*(alf+bet-sqdd1)
!
      if ( kapa1>=zero .and. kapa1<=min(alf,bet) ) then
        kapa = kapa1
      else if ( kapa2>=zero .and. kapa2<=min(alf,bet) ) then
        kapa = kapa2
      else
        kapa = zero
      end if
    end if
!
! nh4o3 equilibrium
!
    if ( dd2>=zero ) then
      sqdd2 = sqrt(dd2)
      lamda1 = 0.5_rkx*(alf+gam+sqdd2)
      lamda2 = 0.5_rkx*(alf+gam-sqdd2)
!
      if ( lamda1>=zero .and. lamda1<=min(alf,gam) ) then
        lamda = lamda1
      else if ( lamda2>=zero .and. lamda2<=min(alf,gam) ) then
        lamda = lamda2
      else
        lamda = zero
      end if
    end if
!
! if oth kapa , lamda are > 0 , then apply existance criterion
!
    if ( kapa>zero .and. lamda>zero ) then
      if ( bet<lamda/theta1 ) then
        kapa = zero
      else
        lamda = zero
      end if
    end if
!
! *** calculate composition of volatile species ************************
!
 100  cnh4no3 = lamda
    cnh4cl = kapa
!
    gnh3 = max(alf-kapa-lamda,zero)
    ghno3 = max(gam-lamda,zero)
    ghcl = max(bet-kapa,zero)
  end subroutine calco1a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcm8
! *** case m8
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl , mgso4 , na2so4 , k2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcm8
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , frna , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(6),w(2))                         ! ccaso4
    so4fr = max(w(2)-chi11,zero)
    cafr = max(w(6)-chi11,zero)
    chi9 = min(0.5_rkx*w(7),so4fr)                   ! ck2s04
    frk = max(w(7)-2._rkx*chi9,zero)
    so4fr = max(so4fr-chi9,zero)
    chi10 = min(w(8),so4fr)                       ! cmgso4
    frmg = max(w(8)-chi10,zero)
    so4fr = max(so4fr-chi10,zero)
    chi1 = max(so4fr,zero)                          ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcm8(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcm8(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcm8(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcm8(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcm8')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcm8(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcm8
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcm8
! *** case m8
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl , mgso4 , na2so4 , k2so4
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  real(rkx) function funcm8(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , cc , dd , hi , ohi , smin , x
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = chi9
    psi10 = chi10
    psi11 = zero
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
!   a1  = xk5 *(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
!   a7  = xk8 *(water/gama(1))**2
!   a8  = xk9 *(water/gama(3))**2
!   a11 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                   ! nai
      molal(3) = psi4                                      ! nh4i
      molal(4) = psi6 + psi7                               ! cli
      molal(5) = psi2 + psi1 + psi9 + psi10                ! so4i
      molal(6) = zero                                      ! hso4i
      molal(7) = psi5 + psi8                               ! no3i
      molal(8) = psi11                                     ! cai
      molal(9) = 2._rkx*psi9                                 ! ki
      molal(10) = psi10                                     ! mgi
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = zero
      cnano3 = zero
      cna2so4 = zero
      ck2so4 = zero
      cmgso4 = zero
      ccaso4 = chi11
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcm8 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcm8 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcm8
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcm7
! *** case m7
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl , mgso4 , na2so4
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcm7
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , frna , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(6),w(2))                         ! ccaso4
    so4fr = max(w(2)-chi11,zero)
    cafr = max(w(6)-chi11,zero)
    chi9 = min(0.5_rkx*w(7),so4fr)                   ! ck2s04
    frk = max(w(7)-2._rkx*chi9,zero)
    so4fr = max(so4fr-chi9,zero)
    chi10 = min(w(8),so4fr)                       ! cmgso4
    frmg = max(w(8)-chi10,zero)
    so4fr = max(so4fr-chi10,zero)
    chi1 = max(so4fr,zero)                          ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcm7(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcm7(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcm7(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcm7(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcm7')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcm7(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcm7
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcm7
! *** case m7
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl , mgso4 , na2so4
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  real(rkx) function funcm7(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , cc , dd , hi , ohi , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
!   a1  = xk5 *(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
!   a7  = xk8 *(water/gama(1))**2
!   a8  = xk9 *(water/gama(3))**2
!   a11 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        call poly3(psi1+psi10,zero,-a9/4._rkx,psi9,islv)
        if ( islv==0 ) then
          psi9 = max(min(psi9,chi9),zero)
        else
          psi9 = zero
        end if
      end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                   ! nai
      molal(3) = psi4                                      ! nh4i
      molal(4) = psi6 + psi7                               ! cli
      molal(5) = psi2 + psi1 + psi9 + psi10                ! so4i
      molal(6) = zero                                      ! hso4i
      molal(7) = psi5 + psi8                               ! no3i
      molal(8) = psi11                                     ! cai
      molal(9) = 2._rkx*psi9                                 ! ki
      molal(10) = psi10                                     ! mgi
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = zero
      cnano3 = zero
      cna2so4 = zero
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcm7 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcm7 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcm7
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcm6
! *** case m6
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , na2so4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl , mgso4
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcm6
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , frna , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(6),w(2))                         ! ccaso4
    so4fr = max(w(2)-chi11,zero)
    cafr = max(w(6)-chi11,zero)
    chi9 = min(0.5_rkx*w(7),so4fr)                   ! ck2s04
    frk = max(w(7)-2._rkx*chi9,zero)
    so4fr = max(so4fr-chi9,zero)
    chi10 = min(w(8),so4fr)                       ! cmgso4
    frmg = max(w(8)-chi10,zero)
    so4fr = max(so4fr-chi10,zero)
    chi1 = max(so4fr,zero)                          ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcm6(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcm6(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcm6(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcm6(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcm6')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcm6(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcm6
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcm6
! *** case m6
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , na2so4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl , mgso4
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  real(rkx) function funcm6(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , cc , dd , hi , ohi , riz , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
!   a7  = xk8 *(water/gama(1))**2
!   a8  = xk9 *(water/gama(3))**2
!   a11 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi1>tiny1 .and. water>tiny1 ) then        !na2so4
        riz = sqrt(a9/a1)
        aa = (0.5_rkx*riz*(psi7+psi8)+psi10+(1._rkx+riz)*(psi7+psi8))         &
             /(1._rkx+riz)
        bb = ((psi7+psi8)*(0.5_rkx*riz*(psi7+psi8)+psi10)+0.25_rkx*(psi7+psi8)&
             **2*(1._rkx+riz))/(1._rkx+riz)
        cc = (0.25_rkx*(psi7+psi8)**2*(0.5_rkx*riz*(psi7+psi8)+psi10)       &
             -a1/4._rkx)/(1._rkx+riz)
!     aa  = psi7+psi8+psi9+psi10
!     bb  = (psi7+psi8)*(psi9+psi10)+0.25d0*(psi7+psi8)**2.
!     cc  = ((psi7+psi8)**2(psi9+psi10)-a1)/4.0d0
!
        call poly3(aa,bb,cc,psi1,islv)
        if ( islv==0 ) then
          psi1 = min(psi1,chi1)
        else
          psi1 = zero
        end if
      end if
!
!   if (chi9 >= tiny1 .and. water > tiny1) then
!   psi9  = 0.5d0*sqrt(a9/a1)*(2.0d0*psi1+psi7+psi8)
!   psi9  = max (min (psi9 ,chi9), zero)
!   else
!   psi9  = zero
!   end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then        !k2so4
        call poly3(psi1+psi10,zero,-a9/4._rkx,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(psi9,chi9)
        else
          psi9 = zero
        end if
      end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                   ! nai
      molal(3) = psi4                                      ! nh4i
      molal(4) = psi6 + psi7                               ! cli
      molal(5) = psi2 + psi1 + psi9 + psi10                ! so4i
      molal(6) = zero                                      ! hso4i
      molal(7) = psi5 + psi8                               ! no3i
      molal(8) = psi11                                     ! cai
      molal(9) = 2._rkx*psi9                                 ! ki
      molal(10) = psi10                                     ! mgi
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = zero
      cnano3 = zero
      cna2so4 = max(chi1-psi1,zero)
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcm6 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcm6 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcm6
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcm5
! *** case m5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcm5
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , frna , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(6),w(2))                         ! ccaso4
    so4fr = max(w(2)-chi11,zero)
    cafr = max(w(6)-chi11,zero)
    chi9 = min(0.5_rkx*w(7),so4fr)                   ! ck2s04
    frk = max(w(7)-2._rkx*chi9,zero)
    so4fr = max(so4fr-chi9,zero)
    chi10 = min(w(8),so4fr)                       ! cmgso4
    frmg = max(w(8)-chi10,zero)
    so4fr = max(so4fr-chi10,zero)
    chi1 = max(so4fr,zero)                          ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcm5(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcm5(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcm5(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcm5(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcm5')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcm5(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcm5
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcm5
! *** case m5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  real(rkx) function funcm5(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , cc , dd , hi , ohi , riz , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
!   a7  = xk8 *(water/gama(1))**2
!   a8  = xk9 *(water/gama(3))**2
!   a11 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi1>tiny1 .and. water>tiny1 ) then        !na2so4
        riz = sqrt(a9/a1)
        aa = (0.5_rkx*riz*(psi7+psi8)+psi10+(1._rkx+riz)*(psi7+psi8))         &
             /(1._rkx+riz)
        bb = ((psi7+psi8)*(0.5_rkx*riz*(psi7+psi8)+psi10)+0.25_rkx*(psi7+psi8)&
             **2*(1._rkx+riz))/(1._rkx+riz)
        cc = (0.25_rkx*(psi7+psi8)**2*(0.5_rkx*riz*(psi7+psi8)+psi10)       &
             -a1/4._rkx)/(1._rkx+riz)
!     aa  = psi7+psi8+psi9+psi10
!     bb  = (psi7+psi8)*(psi9+psi10)+0.25d0*(psi7+psi8)**2.
!     cc  = ((psi7+psi8)**2(psi9+psi10)-a1)/4.0d0
!
        call poly3(aa,bb,cc,psi1,islv)
        if ( islv==0 ) then
          psi1 = min(psi1,chi1)
        else
          psi1 = zero
        end if
      end if
!
      if ( chi9>=tiny1 .and. water>tiny1 ) then
        psi9 = 0.5_rkx*sqrt(a9/a1)*(2.0_rkx*psi1+psi7+psi8)
        psi9 = max(min(psi9,chi9),zero)
      else
        psi9 = zero
      end if
!
!   if (chi9 > tiny1 .and. water > tiny1) then   !k2so4
!   call poly3 (psi1+psi10 ,zero ,-a9/4.d0 , psi9 , islv)
!   if (islv == 0) then
!   psi9 = min (psi9 ,chi9)
!   else
!   psi9 = zero
!   end if
!   end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                   ! nai
      molal(3) = psi4                                      ! nh4i
      molal(4) = psi6 + psi7                               ! cli
      molal(5) = psi2 + psi1 + psi9 + psi10                ! so4i
      molal(6) = zero                                      ! hso4i
      molal(7) = psi5 + psi8                               ! no3i
      molal(8) = psi11                                     ! cai
      molal(9) = 2._rkx*psi9                                 ! ki
      molal(10) = psi10                                     ! mgi
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = zero
      cnano3 = zero
      cna2so4 = max(chi1-psi1,zero)
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcm5 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcm5 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcm5
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcm4
! *** case m4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4 , nh4cl
!     4. completely dissolved: nh4no3 , nano3 , nacl
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcm4
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , frna , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** regime depends on the existance of nitrates
! ***********************
    if ( w(4)<=tiny1 .and. w(5)<=tiny1 ) then
      scase = 'm4 ; subcase 1'
      call calcm1a
      scase = 'm4 ; subcase 1'
      return
    end if
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(6),w(2))                         ! ccaso4
    so4fr = max(w(2)-chi11,zero)
    cafr = max(w(6)-chi11,zero)
    chi9 = min(0.5_rkx*w(7),so4fr)                   ! ck2s04
    frk = max(w(7)-2._rkx*chi9,zero)
    so4fr = max(so4fr-chi9,zero)
    chi10 = min(w(8),so4fr)                       ! cmgso4
    frmg = max(w(8)-chi10,zero)
    so4fr = max(so4fr-chi10,zero)
    chi1 = max(so4fr,zero)                          ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcm4(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcm4(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcm4(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcm4(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcm4')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcm4(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcm4
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcm4
! *** case m4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4 , nh4cl
!     4. completely dissolved: nh4no3 , nano3 , nacl
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  real(rkx) function funcm4(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , cc , dd , delt , hi , ohi , psi31 , psi32 , riz , &
         smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a3 = xk6/(r*temp*r*temp)
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
!   a7  = xk8 *(water/gama(1))**2
!   a8  = xk9 *(water/gama(3))**2
!   a11 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,tiny1),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi1>tiny1 .and. water>tiny1 ) then        !na2so4
        riz = sqrt(a9/a1)
        aa = (0.5_rkx*riz*(psi7+psi8)+psi10+(1._rkx+riz)*(psi7+psi8))         &
             /(1._rkx+riz)
        bb = ((psi7+psi8)*(0.5_rkx*riz*(psi7+psi8)+psi10)+0.25_rkx*(psi7+psi8)&
             **2*(1._rkx+riz))/(1._rkx+riz)
        cc = (0.25_rkx*(psi7+psi8)**2*(0.5_rkx*riz*(psi7+psi8)+psi10)       &
             -a1/4._rkx)/(1._rkx+riz)
!     aa  = psi7+psi8+psi9+psi10
!     bb  = (psi7+psi8)*(psi9+psi10)+0.25d0*(psi7+psi8)**2.
!     cc  = ((psi7+psi8)**2(psi9+psi10)-a1)/4.0d0
!
        call poly3(aa,bb,cc,psi1,islv)
        if ( islv==0 ) then
          psi1 = min(psi1,chi1)
        else
          psi1 = zero
        end if
      end if
!
      if ( chi9>=tiny1 .and. water>tiny1 ) then
        psi9 = 0.5_rkx*sqrt(a9/a1)*(2.0_rkx*psi1+psi7+psi8)
        psi9 = max(min(psi9,chi9),zero)
      else
        psi9 = zero
      end if
!
!   if (chi9 > tiny1 .and. water > tiny1) then   !k2so4
!   call poly3 (psi1+psi10 ,zero ,-a9/4.d0 , psi9 , islv)
!   if (islv == 0) then
!   psi9 = min (psi9 ,chi9)
!   else
!   psi9 = zero
!   end if
!   end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                   ! nai
      molal(3) = psi4                                      ! nh4i
      molal(4) = psi6 + psi7                               ! cli
      molal(5) = psi2 + psi1 + psi9 + psi10                ! so4i
      molal(6) = zero                                      ! hso4i
      molal(7) = psi5 + psi8                               ! no3i
      molal(8) = psi11                                     ! cai
      molal(9) = 2._rkx*psi9                                 ! ki
      molal(10) = psi10                                     ! mgi
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = zero
      cnano3 = zero
      cna2so4 = max(chi1-psi1,zero)
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = max(min(min(psi3,chi4-psi4),chi6-psi6),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcm4 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcm4 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcm4
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcm3
! *** case m3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4 , nh4cl , nacl
!     4. completely dissolved: nh4no3 , nano3
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcm3
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , frna , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** regime depends on the existance of nitrates
! ***********************
    if ( w(4)<=tiny1 ) then          ! no3 not exist , water not possible
      scase = 'm3 ; subcase 1'
      call calcm1a
      scase = 'm3 ; subcase 1'
      return
    end if
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(6),w(2))                         ! ccaso4
    so4fr = max(w(2)-chi11,zero)
    cafr = max(w(6)-chi11,zero)
    chi9 = min(0.5_rkx*w(7),so4fr)                   ! ck2s04
    frk = max(w(7)-2._rkx*chi9,zero)
    so4fr = max(so4fr-chi9,zero)
    chi10 = min(w(8),so4fr)                       ! cmgso4
    frmg = max(w(8)-chi10,zero)
    so4fr = max(so4fr-chi10,zero)
    chi1 = max(so4fr,zero)                          ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcm3(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcm3(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcm3(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcm3(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcm3')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcm3(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcm3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcm3
! *** case m3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4 , nh4cl , nacl
!     4. completely dissolved: nh4no3 , nano3
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  real(rkx) function funcm3(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , cc , dd , delt , diak , hi , ohi , psi31 , psi32 , &
         riz , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a3 = xk6/(r*temp*r*temp)
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk8*(water/gama(1))**2
      a9 = xk17*(water/gama(17))**3
      a10 = xk23*(water/gama(21))**2
!   a8  = xk9 *(water/gama(3))**2
!   a11 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,tiny1),chi4)
      else
        psi4 = tiny1
      end if
!
!   if (chi7 > tiny1 .and. water > tiny1) then     ! nacl dissolution
!   vita = 2.d0*psi1+psi8+psi6  ! an de doulepsei kala vgale psi1 apo dw
!   gkama = psi6*(2.d0*psi1+psi8)-a7
!   diak = max(vita**2 - 4.0d0*gkama ,zero)
!   psi7 = 0.5d0*( -vita + sqrt(diak) )
!   psi7 = max(min(psi7 , chi7), zero)
!   end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then          ! nacl dissolution
        diak = (psi8-psi6)**20 + 4._rkx*a7
        psi7 = 0.5_rkx*(-(psi8+psi6)+sqrt(diak))
        psi7 = max(min(psi7,chi7),zero)
      end if
!
      if ( chi1>tiny1 .and. water>tiny1 ) then        !na2so4
        riz = sqrt(a9/a1)
        aa = (0.5_rkx*riz*(psi7+psi8)+psi10+(1._rkx+riz)*(psi7+psi8))         &
             /(1._rkx+riz)
        bb = ((psi7+psi8)*(0.5_rkx*riz*(psi7+psi8)+psi10)+0.25_rkx*(psi7+psi8)&
             **2*(1._rkx+riz))/(1._rkx+riz)
        cc = (0.25_rkx*(psi7+psi8)**2*(0.5_rkx*riz*(psi7+psi8)+psi10)       &
             -a1/4._rkx)/(1._rkx+riz)
!     aa  = psi7+psi8+psi9+psi10
!     bb  = (psi7+psi8)*(psi9+psi10)+0.25d0*(psi7+psi8)**2.
!     cc  = ((psi7+psi8)**2(psi9+psi10)-a1)/4.0d0
!
        call poly3(aa,bb,cc,psi1,islv)
        if ( islv==0 ) then
          psi1 = min(psi1,chi1)
        else
          psi1 = zero
        end if
      end if
!
      if ( chi9>=tiny1 ) then
        psi9 = 0.5_rkx*sqrt(a9/a1)*(2.0_rkx*psi1+psi7+psi8)
        psi9 = max(min(psi9,chi9),zero)
      else
        psi9 = zero
      end if
!
!   if (chi9 > tiny1 .and. water > tiny1) then   !k2so4
!   call poly3 (psi1+psi10 ,zero ,-a9/4.d0 , psi9 , islv)
!   if (islv == 0) then
!   psi9 = min (psi9 ,chi9)
!   else
!   psi9 = zero
!   end if
!   end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                   ! nai
      molal(3) = psi4                                      ! nh4i
      molal(4) = psi6 + psi7                               ! cli
      molal(5) = psi2 + psi1 + psi9 + psi10                ! so4i
      molal(6) = zero                                      ! hso4i
      molal(7) = psi5 + psi8                               ! no3i
      molal(8) = psi11                                     ! cai
      molal(9) = 2._rkx*psi9                                 ! ki
      molal(10) = psi10                                     ! mgi
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = zero
      cna2so4 = max(chi1-psi1,zero)
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = max(min(min(psi3,chi4-psi4),chi6-psi6),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcm3 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcm3 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcm3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcm2
! *** case m2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. solid & liquid aerosol possible
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4 , nh4cl , nacl , nano3
!
!     there are three regimes in this case:
!     1. nh4no3(s) possible. liquid & solid aerosol (subroutine calch2a)
!     2. nh4no3(s) not possible , and rh < mdrh. solid aerosol only
!     3. nh4no3(s) not possible , and rh >= mdrh. (mdrh region)
!
!     regimes 2. and 3. are considered to be the same as cases m1a , m2b
!     respectively (because mdrh points coincide).
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcm2
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends on the existance of nitrates
! ***********************
    call calcm1a
    if ( cnh4no3>tiny1 ) then          ! no3 exists , water possible
      scase = 'm2 ; subcase 1'
      call calcm2a
      scase = 'm2 ; subcase 1'
    else                            ! no3 non existant , water not possible
      scase = 'm2 ; subcase 1'
      call calcm1a
      scase = 'm2 ; subcase 1'
    end if
    if ( water<=tiny1 .and. rh<drmm2 ) then          ! dry aerosol
      scase = 'm2 ; subcase 2'
    else if ( water<=tiny1 .and. rh>=drmm2 ) then      ! mdrh of m2
      scase = 'm2 ; subcase 3'
      call calcmdrh2(rh,drmm2,drnano3,calcm1a,calcm3)
      scase = 'm2 ; subcase 3'
    end if
  end subroutine calcm2
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcm2a
! *** case m2a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4 , nh4cl , nacl , nano3
!     4. completely dissolved: nh4no3
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcm2a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: cafr , delta , dx , frk , frmg , frna , psi6hi , &
         psi6lo , so4fr , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(6),w(2))                         ! ccaso4
    so4fr = max(w(2)-chi11,zero)
    cafr = max(w(6)-chi11,zero)
    chi9 = min(0.5_rkx*w(7),so4fr)                   ! ck2s04
    frk = max(w(7)-2._rkx*chi9,zero)
    so4fr = max(so4fr-chi9,zero)
    chi10 = min(w(8),so4fr)                       ! cmgso4
    frmg = max(w(8)-chi10,zero)
    so4fr = max(so4fr-chi10,zero)
    chi1 = max(so4fr,zero)                          ! cna2so4
    chi2 = zero                                    ! cnh42s4
    chi3 = zero                                    ! cnh4cl
    frna = max(w(1)-2._rkx*chi1,zero)
    chi8 = min(frna,w(4))                           ! cnano3
    chi4 = w(3)                                    ! nh3(g)
    chi5 = max(w(4)-chi8,zero)                      ! hno3(g)
    chi7 = min(max(frna-chi8,zero),w(5))            ! cnacl
    chi6 = max(w(5)-chi7,zero)                      ! hcl(g)
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcm2a(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcm2a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcm2a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcm2a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcm2a')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcm2a(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcm2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcm2a
! *** case m2a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4 , nh4cl , nacl , nano3
!     4. completely dissolved: nh4no3
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by  christos fountoukis & athanasios nenes
!
!======================================================================
!
  real(rkx) function funcm2a(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: a64 , aa , bb , cc , dd , delt , diak , hi , ohi , psi31 , &
         psi32 , riz , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = chi1
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk5*(water/gama(2))**3
      a3 = xk6/(r*temp*r*temp)
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
      a9 = xk17*(water/gama(17))**3
      a64 = (xk3*xk2/xkw)*(gama(10)/gama(5)/gama(11))**2
      a64 = a64*(r*temp*water)**2
!   a11 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7) - a6/a5*psi8*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,tiny1),chi4)
      else
        psi4 = tiny1
      end if
!
!   if (chi7 > tiny1 .and. water > tiny1) then     ! nacl dissolution
!   vita = 2.d0*psi1+psi8+psi6
!   gkama = psi6*(2.d0*psi1+psi8)-a7
!   diak = max(vita**2 - 4.0d0*gkama ,zero)
!   psi7 = 0.5d0*( -vita + sqrt(diak) )
!   psi7 = max(min(psi7 , chi7), zero)
!   end if
!
!   if (chi8 > tiny1 .and. water > tiny1) then     ! nano3 dissolution
!   bit  = 2.d0*psi1+psi7+psi5
!   gkam = psi5*(2.d0*psi1+psi8)-a8
!   dia  = bit**2 - 4.0d0*gkam
!   psi8 = 0.5d0*( -bit + sqrt(dia) )
!   psi8 = max(min(psi8 , chi8), zero)
!   end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then          ! nacl dissolution
        diak = (psi8-psi6)**20 + 4._rkx*a7
        psi7 = 0.5_rkx*(-(psi8+psi6)+sqrt(diak))
        psi7 = max(min(psi7,chi7),zero)
      end if
!
      if ( chi8>tiny1 .and. water>tiny1 ) then          ! nano3 dissolution
        diak = (psi7-psi5)**20 + 4._rkx*a8
        psi8 = 0.5_rkx*(-(psi7+psi5)+sqrt(diak))
        psi8 = max(min(psi8,chi8),zero)
      end if
!
      if ( chi1>tiny1 .and. water>tiny1 ) then        !na2so4
        riz = sqrt(a9/a1)
        aa = (0.5_rkx*riz*(psi7+psi8)+psi10+(1._rkx+riz)*(psi7+psi8))         &
             /(1._rkx+riz)
        bb = ((psi7+psi8)*(0.5_rkx*riz*(psi7+psi8)+psi10)+0.25_rkx*(psi7+psi8)&
             **2*(1._rkx+riz))/(1._rkx+riz)
        cc = (0.25_rkx*(psi7+psi8)**2*(0.5_rkx*riz*(psi7+psi8)+psi10)       &
             -a1/4._rkx)/(1._rkx+riz)
!
!     aa  = psi7+psi8+psi9+psi10
!     bb  = (psi7+psi8)*(psi9+psi10)+0.25d0*(psi7+psi8)**2.
!     cc  = ((psi7+psi8)**2(psi9+psi10)-a1)/4.0d0
!
        call poly3(aa,bb,cc,psi1,islv)
        if ( islv==0 ) then
          psi1 = min(psi1,chi1)
        else
          psi1 = zero
        end if
      end if
!
      if ( chi9>=tiny1 .and. water>tiny1 ) then
!     psi9  = 0.5d0*sqrt(a9/a1)*(2.0d0*psi1+psi7+psi8)
        psi9 = 0.5_rkx*sqrt(a9/a1)*(2.0_rkx*psi1+psi7+psi8)
        psi9 = max(min(psi9,chi9),zero)
      else
        psi9 = zero
      end if
!
!   if (chi9 > tiny1 .and. water > tiny1) then   !k2so4
!   call poly3 (psi1+psi10 ,zero ,-a9/4.d0 , psi9 , islv)
!   if (islv == 0) then
!   psi9 = max (min (psi9 ,chi9), zero)
!   else
!   psi9 = zero
!   end if
!   end if
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7 + 2._rkx*psi1                   ! nai
      molal(3) = psi4                                      ! nh4i
      molal(4) = psi6 + psi7                               ! cli
      molal(5) = psi2 + psi1 + psi9 + psi10                ! so4i
      molal(6) = zero                                      ! hso4i
      molal(7) = psi5 + psi8                               ! no3i
      molal(8) = psi11                                     ! cai
      molal(9) = 2._rkx*psi9                                 ! ki
      molal(10) = psi10                                     ! mgi
!
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      cna2so4 = max(chi1-psi1,zero)
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = max(min(min(psi3,chi4-psi4),chi6-psi6),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcm2a = molal(3)*molal(4)/ghcl/gnh3/a64 - one
    funcm2a = molal(1)*molal(4)/ghcl/a6 - one
  end function funcm2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcm1
! *** case m1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4 , nh4cl , nacl , nano3 , nh4no3
!
!     there are two possible regimes here , depending on relative humidity:
!     1. when rh >= mdrh ; liquid phase possible (mdrh region)
!     2. when rh < mdrh  ; only solid phase possible (subroutine calch1a)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcm1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmm1 ) then
      scase = 'm1 ; subcase 1'
      call calcm1a                 ! solid phase only possible
      scase = 'm1 ; subcase 1'
    else
      scase = 'm1 ; subcase 2'     ! liquid & solid phase possible
      call calcmdrh2(rh,drmm1,drnh4no3,calcm1a,calcm2a)
      scase = 'm1 ; subcase 2'
    end if
  end subroutine calcm1
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcm1a
! *** case m1a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr < 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , k2so4 , na2so4 , mgso4 , nh4cl , nacl , nano3 , nh4no3
!
! *** copyright 1996-2000 , university of miami , carnegie mellon university
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================

  subroutine calcm1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a1 , a2 , alf , bb , bet , cafr , cc , clfr , dd , &
         dd1 , dd2 , frk , frmg , gam , rtsq , so4fr , sqdd , sqdd1 , &
         sqdd2 , theta1 , theta2
    real(rkx) :: lamda , lamda1 , lamda2 , kapa , kapa1 , kapa2 , nafr , &
              no3fr
!
! *** calculate non volatile solids ***********************************
!
    ccaso4 = min(w(6),w(2))                         ! ccaso4
    so4fr = max(w(2)-ccaso4,zero)
    cafr = max(w(6)-ccaso4,zero)
    ck2so4 = min(0.5_rkx*w(7),so4fr)                  ! ck2s04
    frk = max(w(7)-2._rkx*ck2so4,zero)
    so4fr = max(so4fr-ck2so4,zero)
    cmgso4 = min(w(8),so4fr)                        ! cmgso4
    frmg = max(w(8)-cmgso4,zero)
    so4fr = max(so4fr-cmgso4,zero)
    cna2so4 = max(so4fr,zero)                        ! cna2so4
    nafr = max(w(1)-2._rkx*cna2so4,zero)
    cnano3 = min(nafr,w(4))                          ! cnano3
    no3fr = max(w(4)-cnano3,zero)
    cnacl = min(max(nafr-cnano3,zero),w(5))          ! cnacl
    clfr = max(w(5)-cnacl,zero)
!
! *** calculate volatile species **************************************
!
    alf = w(3)                           ! free nh3
    bet = clfr                           ! free cl
    gam = no3fr                          ! free no3
!
    rtsq = r*temp*r*temp
    a1 = xk6/rtsq
    a2 = xk10/rtsq
!
    theta1 = gam - bet*(a2/a1)
    theta2 = a2/a1
!
! quaratic equation solution
!
    bb = (theta1-alf-bet*(one+theta2))/(one+theta2)
    cc = (alf*bet-a1-bet*theta1)/(one+theta2)
    dd = bb*bb - 4.0_rkx*cc
    if ( dd>=zero ) then        ! solve each reaction seperately
!
!   two roots for kapa , check and see if any valid
!
      sqdd = sqrt(dd)
      kapa1 = 0.5_rkx*(-bb+sqdd)
      kapa2 = 0.5_rkx*(-bb-sqdd)
      lamda1 = theta1 + theta2*kapa1
      lamda2 = theta1 + theta2*kapa2
!
      if ( kapa1>=zero .and. lamda1>=zero ) then
        if ( alf-kapa1-lamda1>=zero .and. bet-kapa1>=zero .and.           &
             gam-lamda1>=zero ) then
          kapa = kapa1
          lamda = lamda1
          go to 100
        end if
      end if
!
      if ( kapa2>=zero .and. lamda2>=zero ) then
        if ( alf-kapa2-lamda2>=zero .and. bet-kapa2>=zero .and.           &
             gam-lamda2>=zero ) then
          kapa = kapa2
          lamda = lamda2
          go to 100
        end if
      end if
    end if
!
! seprate solution of nh4cl & nh4no3 equilibria
!
    kapa = zero
    lamda = zero
    dd1 = (alf+bet)*(alf+bet) - 4.0_rkx*(alf*bet-a1)
    dd2 = (alf+gam)*(alf+gam) - 4.0_rkx*(alf*gam-a2)
!
! nh4l equilibrium
!
    if ( dd1>=zero ) then
      sqdd1 = sqrt(dd1)
      kapa1 = 0.5_rkx*(alf+bet+sqdd1)
      kapa2 = 0.5_rkx*(alf+bet-sqdd1)
!
      if ( kapa1>=zero .and. kapa1<=min(alf,bet) ) then
        kapa = kapa1
      else if ( kapa2>=zero .and. kapa2<=min(alf,bet) ) then
        kapa = kapa2
      else
        kapa = zero
      end if
    end if
!
! nh4o3 equilibrium
!
    if ( dd2>=zero ) then
      sqdd2 = sqrt(dd2)
      lamda1 = 0.5_rkx*(alf+gam+sqdd2)
      lamda2 = 0.5_rkx*(alf+gam-sqdd2)
!
      if ( lamda1>=zero .and. lamda1<=min(alf,gam) ) then
        lamda = lamda1
      else if ( lamda2>=zero .and. lamda2<=min(alf,gam) ) then
        lamda = lamda2
      else
        lamda = zero
      end if
    end if
!
! if oth kapa , lamda are > 0 , then apply existance criterion
!
    if ( kapa>zero .and. lamda>zero ) then
      if ( bet<lamda/theta1 ) then
        kapa = zero
      else
        lamda = zero
      end if
    end if
!
! *** calculate composition of volatile species ***********************
!
 100  cnh4no3 = lamda
    cnh4cl = kapa
!
    gnh3 = alf - kapa - lamda
    ghno3 = gam - lamda
    ghcl = bet - kapa
  end subroutine calcm1a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp13
! *** case p13
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4
!     4. completely dissolved: ca(no3)2 , cacl2 , k2so4 , kno3 , kcl , mgso4 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp13
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp13(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp13(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp13(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp13(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp13')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp13(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp13

!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp13
! *** case p13
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4
!     4. completely dissolved: ca(no3)2 , cacl2 , k2so4 , kno3 , kcl , mgso4 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp13(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , cc , dd , hi , ohi , smin , x
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi4 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = chi9
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = chi13
    psi14 = chi14
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)                 &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6)
      psi5 = psi5/(a6/a5*(chi6-psi6)+psi6+psi7+psi14+2._rkx*psi16+          &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
!   *** calculate speciation
!   *********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   *** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh4no3 = zero
      cnh4cl = zero
      cnacl = zero
      cnano3 = zero
      ck2so4 = zero
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = zero
      ckcl = zero
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp13 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp13 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp13
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp12
! *** case p12
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4
!     4. completely dissolved: ca(no3)2 , cacl2 , kno3 , kcl , mgso4 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp12
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp12(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp12(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp12(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp12(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp12')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp12(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp12
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp12
! *** case p12
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4
!     4. completely dissolved: ca(no3)2 , cacl2 , kno3 , kcl , mgso4 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp12(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , hi , ohi , smin , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi4 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = chi13
    psi14 = chi14
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)                 &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh4no3 = zero
      cnh4cl = zero
      cnacl = zero
      cnano3 = zero
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = zero
      ckcl = zero
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp12 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp12 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp12
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp11
! *** case p11
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3
!     4. completely dissolved: ca(no3)2 , cacl2 , kcl , mgso4 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp11
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp11(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp11(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp11(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp11(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp11')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp11(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp11
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp11
! *** case p11
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3
!     4. completely dissolved: ca(no3)2 , cacl2 , kcl , mgso4 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp11(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , delta , gkama , hi , &
         ohi , smin , vhta , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = zero
    psi14 = chi14
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
      a13 = xk19*(water/gama(19))**2
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)                 &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi13>tiny1 .and. water>tiny1 ) then               !kno3
        vhta = psi5 + psi8 + 2._rkx*psi12 + 2._rkx*psi15 + psi14 + 2._rkx*psi9
        gkama = (psi5+psi8+2._rkx*psi12+2._rkx*psi15)*(2._rkx*psi9+psi14) - a13
        delta = max(vhta*vhta-4._rkx*gkama,zero)
        psi13 = 0.5_rkx*(-vhta+sqrt(delta))
        psi13 = min(max(psi13,zero),chi13)
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh4no3 = zero
      cnh4cl = zero
      cnacl = zero
      cnano3 = zero
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = max(chi13-psi13,zero)
      ckcl = zero
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp11 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp11 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp11
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp10
! *** case p10
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4
!     4. completely dissolved: ca(no3)2 , cacl2 , kcl ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp10
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp10(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp10(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp10(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp10(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp10')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp10(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp10
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp10
! *** case p10
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4
!     4. completely dissolved: ca(no3)2 , cacl2 , kcl ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp10(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , delta , gkama , hi , &
         ohi , smin , vhta , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = zero
    psi14 = chi14
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
      a13 = xk19*(water/gama(19))**2
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)                 &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi13>tiny1 .and. water>tiny1 ) then               !kno3
        vhta = psi5 + psi8 + 2._rkx*psi12 + 2._rkx*psi15 + psi14 + 2._rkx*psi9
        gkama = (psi5+psi8+2._rkx*psi12+2._rkx*psi15)*(2._rkx*psi9+psi14) - a13
        delta = max(vhta*vhta-4._rkx*gkama,zero)
        psi13 = 0.5_rkx*(-vhta+sqrt(delta))
        psi13 = min(max(psi13,zero),chi13)
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh4no3 = zero
      cnh4cl = zero
      cnacl = zero
      cnano3 = zero
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = max(chi13-psi13,zero)
      ckcl = zero
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp10 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp10 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp10
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp9
! *** case p9
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp9
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp9(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp9(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp9(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp9(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp9')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp9(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp9
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp9
! *** case p9
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp9(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , delta , gkama , hi , &
         ohi , smin , vhta , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = zero
    psi14 = zero
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
      a13 = xk19*(water/gama(19))**2
      a14 = xk20*(water/gama(20))**2
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)                 &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi13>tiny1 .and. water>tiny1 ) then               !kno3
        vhta = psi5 + psi8 + 2._rkx*psi12 + 2._rkx*psi15 + psi14 + 2._rkx*psi9
        gkama = (psi5+psi8+2._rkx*psi12+2._rkx*psi15)*(2._rkx*psi9+psi14) - a13
        delta = max(vhta*vhta-4._rkx*gkama,zero)
        psi13 = 0.5_rkx*(-vhta+sqrt(delta))
        psi13 = min(max(psi13,zero),chi13)
      end if
!
      if ( chi14>tiny1 .and. water>tiny1 ) then               !kcl
        psi14 = a14/a13*(psi5+psi8+2._rkx*psi12+psi13+2._rkx*psi15)           &
                - psi6 - psi7 - 2._rkx*psi16 - 2._rkx*psi17
        psi14 = min(max(psi14,zero),chi14)
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh4no3 = zero
      cnh4cl = zero
      cnacl = zero
      cnano3 = zero
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = max(chi13-psi13,zero)
      ckcl = max(chi14-psi14,zero)
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp9 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp9 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp9
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp8
! *** case p8
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp8
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp8(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp8(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp8(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp8(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp8')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp8(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp8
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp8
! *** case p8
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp8(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , delt , delta , gkama , &
         hi , ohi , psi31 , psi32 , smin , vhta , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi7 = chi7
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = zero
    psi14 = zero
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
      a13 = xk19*(water/gama(19))**2
      a14 = xk20*(water/gama(20))**2
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)                 &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi13>tiny1 .and. water>tiny1 ) then               !kno3
        vhta = psi5 + psi8 + 2._rkx*psi12 + 2._rkx*psi15 + psi14 + 2._rkx*psi9
        gkama = (psi5+psi8+2._rkx*psi12+2._rkx*psi15)*(2._rkx*psi9+psi14) - a13
        delta = max(vhta*vhta-4._rkx*gkama,zero)
        psi13 = 0.5_rkx*(-vhta+sqrt(delta))
        psi13 = min(max(psi13,zero),chi13)
      end if
!
      if ( chi14>tiny1 .and. water>tiny1 ) then               !kcl
        psi14 = a14/a13*(psi5+psi8+2._rkx*psi12+psi13+2._rkx*psi15)           &
                - psi6 - psi7 - 2._rkx*psi16 - 2._rkx*psi17
        psi14 = min(max(psi14,zero),chi14)
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh4no3 = zero
!   cnh4cl    = zero
      cnacl = zero
      cnano3 = zero
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = max(chi13-psi13,zero)
      ckcl = max(chi14-psi14,zero)
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = max(min(min(psi3,chi4-psi4),chi6-psi6),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp8 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp8 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp8
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp7
! *** case p7
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nano3 , nh4no3
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp7
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp7(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp7(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp7(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp7(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp7')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp7(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp7
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp7
! *** case p7
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nano3 , nh4no3
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp7(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , delt , delta , diak , &
         gkama , hi , ohi , psi31 , psi32 , smin , vhta , vita , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi7 = zero
    psi8 = chi8
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = zero
    psi14 = zero
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
      a13 = xk19*(water/gama(19))**2
      a14 = xk20*(water/gama(20))**2
      a7 = xk8*(water/gama(1))**2
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)                 &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi13>tiny1 .and. water>tiny1 ) then               !kno3
        vhta = psi5 + psi8 + 2._rkx*psi12 + 2._rkx*psi15 + psi14 + 2._rkx*psi9
        gkama = (psi5+psi8+2._rkx*psi12+2._rkx*psi15)*(2._rkx*psi9+psi14) - a13
        delta = max(vhta*vhta-4._rkx*gkama,zero)
        psi13 = 0.5_rkx*(-vhta+sqrt(delta))
        psi13 = min(max(psi13,zero),chi13)
      end if
!
      if ( chi14>tiny1 .and. water>tiny1 ) then               !kcl
        psi14 = a14/a13*(psi5+psi8+2._rkx*psi12+psi13+2._rkx*psi15)           &
                - psi6 - psi7 - 2._rkx*psi16 - 2._rkx*psi17
        psi14 = min(max(psi14,zero),chi14)
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then          ! nacl dissolution
        vita = psi6 + psi14 + psi8 + 2._rkx*psi16 + 2._rkx*psi17
        gkama = psi8*(2._rkx*psi16+psi6+psi14+2._rkx*psi17) - a7
        diak = max(vita*vita-4.0_rkx*gkama,zero)
        psi7 = 0.5_rkx*(-vita+sqrt(diak))
        psi7 = max(min(psi7,chi7),zero)
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh4no3 = zero
!   cnh4cl    = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = zero
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = max(chi13-psi13,zero)
      ckcl = max(chi14-psi14,zero)
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = max(min(min(psi3,chi4-psi4),chi6-psi6),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp7 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp7 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp7
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp6
! *** case p6
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl , nano3
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nh4no3
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp6
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp6(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp6(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp6(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp6(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp6')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp6(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp6
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp6
! *** case p6
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl , nano3
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nh4no3
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp6(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , delt , delta , diak , &
         gkama , hi , ohi , psi31 , psi32 , smin , vhta , vita , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi7 = zero
    psi8 = zero
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = zero
    psi14 = zero
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
      a13 = xk19*(water/gama(19))**2
      a14 = xk20*(water/gama(20))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
!
!   caculate dissociation quantities
!
      psi5 = chi5*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)                 &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi13>tiny1 .and. water>tiny1 ) then               !kno3
        vhta = psi5 + psi8 + 2._rkx*psi12 + 2._rkx*psi15 + psi14 + 2._rkx*psi9
        gkama = (psi5+psi8+2._rkx*psi12+2._rkx*psi15)*(2._rkx*psi9+psi14) - a13
        delta = max(vhta*vhta-4._rkx*gkama,zero)
        psi13 = 0.5_rkx*(-vhta+sqrt(delta))
        psi13 = min(max(psi13,zero),chi13)
      end if
!
      if ( chi14>tiny1 .and. water>tiny1 ) then               !kcl
        psi14 = a14/a13*(psi5+psi8+2._rkx*psi12+psi13+2._rkx*psi15)           &
                - psi6 - psi7 - 2._rkx*psi16 - 2._rkx*psi17
        psi14 = min(max(psi14,zero),chi14)
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then          ! nacl dissolution
        vita = psi6 + psi14 + psi8 + 2._rkx*psi16 + 2._rkx*psi17
        gkama = psi8*(2._rkx*psi16+psi6+psi14+2._rkx*psi17) - a7
        diak = max(vita*vita-4.0_rkx*gkama,zero)
        psi7 = 0.5_rkx*(-vita+sqrt(diak))
        psi7 = max(min(psi7,chi7),zero)
      end if
!
      if ( chi8>tiny1 .and. water>tiny1 ) then          ! nano3 dissolution
!     vit  = psi5+psi13+psi7+2.d0*psi12+2.d0*psi15
!     gkam = psi7*(2.d0*psi12+psi5+psi13+2.d0*psi15)-a8
!     dia  = max(vit*vit - 4.0d0*gkam ,zero)
!     psi8 = 0.5d0*( -vit + sqrt(dia) )
        psi8 = a8/a7*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)              &
               - psi5 - 2._rkx*psi12 - psi13 - 2._rkx*psi15
        psi8 = max(min(psi8,chi8),zero)
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
      cnh4no3 = zero
!   cnh4cl    = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = max(chi13-psi13,zero)
      ckcl = max(chi14-psi14,zero)
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = max(min(min(psi3,chi4-psi4),chi6-psi6),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp6 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp6 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp6
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp5
! *** case p5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , k2so4 , kno3 , kcl , mgso4 ,
!                          nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcp5
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** regime depends on the existance of water and of the rh
! ************
    if ( w(4)>tiny1 ) then     ! no3 exist , water possible
      scase = 'p5 ; subcase 1'
      call calcp5a
      scase = 'p5 ; subcase 1'
    else                                        ! no3 , cl non existant
      scase = 'p1 ; subcase 1'
      call calcp1a
      scase = 'p1 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drmp5 ) then           ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcp1a
        scase = 'p5 ; subcase 2'
        return
      else
        scase = 'p5 ; subcase 3'      ! mdrh region (caso4 , k2so4 , kno3 , kcl , mgso4 ,
!     nano3 , nacl , nh4no3 , nh4cl)
        call calcmdrh2(rh,drmp5,drnh4no3,calcp1a,calcp6)
        scase = 'p5 ; subcase 3'
      end if
    end if
  end subroutine calcp5
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp5a
! *** case p5a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl , nano3 , nh4no3
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp5a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp5(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp5(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp5(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp5(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp5')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp5(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp5a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp5
! *** case p5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl , nano3 , nh4no3
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp5(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , delt , delta , diak , &
         gkama , hi , ohi , psi21 , psi22 , psi31 , psi32 , smin , &
         vhta , vita , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi7 = zero
    psi8 = zero
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = zero
    psi14 = zero
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
      a13 = xk19*(water/gama(19))**2
      a14 = xk20*(water/gama(20))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
!
!   caculate dissociation quantities
!
      psi5 = (chi5-psi2)*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)          &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi13>tiny1 .and. water>tiny1 ) then               !kno3
        vhta = psi5 + psi8 + 2._rkx*psi12 + 2._rkx*psi15 + psi14 + 2._rkx*psi9
        gkama = (psi5+psi8+2._rkx*psi12+2._rkx*psi15)*(2._rkx*psi9+psi14) - a13
        delta = max(vhta*vhta-4._rkx*gkama,zero)
        psi13 = 0.5_rkx*(-vhta+sqrt(delta))
        psi13 = min(max(psi13,zero),chi13)
      end if
!
      if ( chi14>tiny1 .and. water>tiny1 ) then               !kcl
        psi14 = a14/a13*(psi5+psi8+2._rkx*psi12+psi13+2._rkx*psi15)           &
                - psi6 - psi7 - 2._rkx*psi16 - 2._rkx*psi17
        psi14 = min(max(psi14,zero),chi14)
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then          ! nacl dissolution
        vita = psi6 + psi14 + psi8 + 2._rkx*psi16 + 2._rkx*psi17
        gkama = psi8*(2._rkx*psi16+psi6+psi14+2._rkx*psi17) - a7
        diak = max(vita*vita-4.0_rkx*gkama,zero)
        psi7 = 0.5_rkx*(-vita+sqrt(diak))
        psi7 = max(min(psi7,chi7),zero)
      end if
!
      if ( chi8>tiny1 .and. water>tiny1 ) then          ! nano3 dissolution
!     vit  = psi5+psi13+psi7+2.d0*psi12+2.d0*psi15
!     gkam = psi7*(2.d0*psi12+psi5+psi13+2.d0*psi15)-a8
!     dia  = max(vit*vit - 4.0d0*gkam ,zero)
!     psi8 = 0.5d0*( -vit + sqrt(dia) )
        psi8 = a8/a7*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)              &
               - psi5 - 2._rkx*psi12 - psi13 - 2._rkx*psi15
        psi8 = max(min(psi8,chi8),zero)
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   ** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
!   cnh4no3   = zero
!   cnh4cl    = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = max(chi13-psi13,zero)
      ckcl = max(chi14-psi14,zero)
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = max(min(min(psi3,chi4-psi4),chi6-psi6),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
!   *** nh4no3(s) calculations
!
      a2 = xk10/(r*temp*r*temp)
      if ( gnh3*ghno3>a2 ) then
        delt = min(gnh3,ghno3)
        bb = -(gnh3+ghno3)
        cc = gnh3*ghno3 - a2
        dd = bb*bb - 4._rkx*cc
        psi21 = 0.5_rkx*(-bb+sqrt(dd))
        psi22 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi21>zero .and. psi21>zero ) then
          psi2 = psi21
        else if ( delt-psi22>zero .and. psi22>zero ) then
          psi2 = psi22
        else
          psi2 = zero
        end if
      else
        psi2 = zero
      end if
      psi2 = max(min(min(psi2,chi4-psi4-psi3),chi5-psi5),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi2,tiny1)
      ghcl = max(ghno3-psi2,tiny1)
      cnh4no3 = psi2
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp5 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp5 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp5
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp4
! *** case p4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , k2so4 , kno3 , kcl , mgso4 ,
!                          mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcp4
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** regime depends on the existance of water and of the rh
! ************
    if ( w(4)>tiny1 ) then     ! no3 exist , water possible
      scase = 'p4 ; subcase 1'
      call calcp4a
      scase = 'p4 ; subcase 1'
    else                                        ! no3 , cl non existant
      scase = 'p1 ; subcase 1'
      call calcp1a
      scase = 'p1 ; subcase 1'
    end if
!
    if ( water<=tiny1 ) then
      if ( rh<drmp4 ) then           ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcp1a
        scase = 'p4 ; subcase 2'
        return
      else
        scase = 'p4 ; subcase 3'      ! mdrh region (caso4 , k2so4 , kno3 , kcl , mgso4 ,
!     mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl)
        call calcmdrh2(rh,drmp4,drmgno32,calcp1a,calcp5a)
        scase = 'p4 ; subcase 3'
      end if
    end if
  end subroutine calcp4
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp4a
! *** case p4a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl , nano3 , nh4no3 , mg(no3)2
!     4. completely dissolved: ca(no3)2 , cacl2 , mgcl2
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp4a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp4(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp4(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp4(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp4(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp4')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp4(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp4a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp4
! *** case p4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl , nano3 , nh4no3 , mg(no3)2
!     4. completely dissolved: ca(no3)2 , cacl2 , mgcl2
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp4(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , delt , delta , diak , &
         gkama , hi , ohi , psi21 , psi22 , psi31 , psi32 , smin , vhta , &
         vita , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi7 = zero
    psi8 = zero
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = zero
    psi14 = zero
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
      a13 = xk19*(water/gama(19))**2
      a14 = xk20*(water/gama(20))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
!
!   caculate dissociation quantities
!
      psi5 = (chi5-psi2)*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)          &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi13>tiny1 .and. water>tiny1 ) then               !kno3
        vhta = psi5 + psi8 + 2._rkx*psi12 + 2._rkx*psi15 + psi14 + 2._rkx*psi9
        gkama = (psi5+psi8+2._rkx*psi12+2._rkx*psi15)*(2._rkx*psi9+psi14) - a13
        delta = max(vhta*vhta-4._rkx*gkama,zero)
        psi13 = 0.5_rkx*(-vhta+sqrt(delta))
        psi13 = min(max(psi13,zero),chi13)
      end if
!
      if ( chi14>tiny1 .and. water>tiny1 ) then               !kcl
        psi14 = a14/a13*(psi5+psi8+2._rkx*psi12+psi13+2._rkx*psi15)           &
                - psi6 - psi7 - 2._rkx*psi16 - 2._rkx*psi17
        psi14 = min(max(psi14,zero),chi14)
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then          ! nacl dissolution
        vita = psi6 + psi14 + psi8 + 2._rkx*psi16 + 2._rkx*psi17
        gkama = psi8*(2._rkx*psi16+psi6+psi14+2._rkx*psi17) - a7
        diak = max(vita*vita-4.0_rkx*gkama,zero)
        psi7 = 0.5_rkx*(-vita+sqrt(diak))
        psi7 = max(min(psi7,chi7),zero)
      end if
!
      if ( chi8>tiny1 .and. water>tiny1 ) then          ! nano3 dissolution
!     vit  = psi5+psi13+psi7+2.d0*psi12+2.d0*psi15
!     gkam = psi7*(2.d0*psi12+psi5+psi13+2.d0*psi15)-a8
!     dia  = max(vit*vit - 4.0d0*gkam ,zero)
!     psi8 = 0.5d0*( -vit + sqrt(dia) )
        psi8 = a8/a7*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)              &
               - psi5 - 2._rkx*psi12 - psi13 - 2._rkx*psi15
        psi8 = max(min(psi8,chi8),zero)
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
!   cnh4cl    = zero
!   cnh4no3   = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = max(chi13-psi13,zero)
      ckcl = max(chi14-psi14,zero)
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = max(min(min(psi3,chi4-psi4),chi6-psi6),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
!   *** nh4no3(s) calculations
!
      a2 = xk10/(r*temp*r*temp)
      if ( gnh3*ghno3>a2 ) then
        delt = min(gnh3,ghno3)
        bb = -(gnh3+ghno3)
        cc = gnh3*ghno3 - a2
        dd = bb*bb - 4._rkx*cc
        psi21 = 0.5_rkx*(-bb+sqrt(dd))
        psi22 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi21>zero .and. psi21>zero ) then
          psi2 = psi21
        else if ( delt-psi22>zero .and. psi22>zero ) then
          psi2 = psi22
        else
          psi2 = zero
        end if
      else
        psi2 = zero
      end if
      psi2 = max(min(min(psi2,chi4-psi4-psi3),chi5-psi5),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi2,tiny1)
      ghcl = max(ghno3-psi2,tiny1)
      cnh4no3 = psi2
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp4 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp4 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp4
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp3
! *** case p3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 ,
!                          mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcp3
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** regime depends on the existance of water and of the rh
! ************
    if ( w(4)>tiny1 .and. w(5)>tiny1 ) then     ! no3 ,cl exist , water possible
      scase = 'p3 ; subcase 1'
      call calcp3a
      scase = 'p3 ; subcase 1'
    else                                        ! no3 , cl non existant
      scase = 'p1 ; subcase 1'
      call calcp1a
      scase = 'p1 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drmp3 ) then           ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcp1a
        scase = 'p3 ; subcase 2'
        return
      else
        scase = 'p3 ; subcase 3'      ! mdrh region (caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 ,
!     mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl)
        call calcmdrh2(rh,drmp3,drcano32,calcp1a,calcp4a)
        scase = 'p3 ; subcase 3'
      end if
    end if
  end subroutine calcp3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp3a
! *** case p3a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl ,
!                          nano3 , nh4no3 , mg(no3)2 , ca(no3)2
!     4. completely dissolved: cacl2 , mgcl2
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp3a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , frno3 , &
         frso4 , psi6hi , psi6lo , x1 , x2 , x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp3(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp3(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp3(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp3(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp3')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp3(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp3a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp3
! *** case p3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl ,
!                          nano3 , nh4no3 , mg(no3)2 , ca(no3)2
!     4. completely dissolved: cacl2 , mgcl2
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp3(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , delt , delta , diak , &
         gkama , hi , ohi , psi21 , psi22 , psi31 , psi32 , smin , vhta , &
         vita , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi7 = zero
    psi8 = zero
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = zero
    psi14 = zero
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
      a13 = xk19*(water/gama(19))**2
      a14 = xk20*(water/gama(20))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
!
!   caculate dissociation quantities
!
      psi5 = (chi5-psi2)*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)          &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi13>tiny1 .and. water>tiny1 ) then               !kno3
        vhta = psi5 + psi8 + 2._rkx*psi12 + 2._rkx*psi15 + psi14 + 2._rkx*psi9
        gkama = (psi5+psi8+2._rkx*psi12+2._rkx*psi15)*(2._rkx*psi9+psi14) - a13
        delta = max(vhta*vhta-4._rkx*gkama,zero)
        psi13 = 0.5_rkx*(-vhta+sqrt(delta))
        psi13 = min(max(psi13,zero),chi13)
      end if
!
      if ( chi14>tiny1 .and. water>tiny1 ) then               !kcl
        psi14 = a14/a13*(psi5+psi8+2._rkx*psi12+psi13+2._rkx*psi15)           &
                - psi6 - psi7 - 2._rkx*psi16 - 2._rkx*psi17
        psi14 = min(max(psi14,zero),chi14)
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then          ! nacl dissolution
        vita = psi6 + psi14 + psi8 + 2._rkx*psi16 + 2._rkx*psi17
        gkama = psi8*(2._rkx*psi16+psi6+psi14+2._rkx*psi17) - a7
        diak = max(vita*vita-4.0_rkx*gkama,zero)
        psi7 = 0.5_rkx*(-vita+sqrt(diak))
        psi7 = max(min(psi7,chi7),zero)
      end if
!
      if ( chi8>tiny1 .and. water>tiny1 ) then          ! nano3 dissolution
!     vit  = psi5+psi13+psi7+2.d0*psi12+2.d0*psi15
!     gkam = psi7*(2.d0*psi12+psi5+psi13+2.d0*psi15)-a8
!     dia  = max(vit*vit - 4.0d0*gkam ,zero)
!     psi8 = 0.5d0*( -vit + sqrt(dia) )
        psi8 = a8/a7*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)              &
               - psi5 - 2._rkx*psi12 - psi13 - 2._rkx*psi15
        psi8 = max(min(psi8,chi8),zero)
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
!   cnh4cl    = zero
!   cnh4no3   = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = max(chi13-psi13,zero)
      ckcl = max(chi14-psi14,zero)
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = max(min(min(psi3,chi4-psi4),chi6-psi6),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
!   *** nh4no3(s) calculations
!
      a2 = xk10/(r*temp*r*temp)
      if ( gnh3*ghno3>a2 ) then
        delt = min(gnh3,ghno3)
        bb = -(gnh3+ghno3)
        cc = gnh3*ghno3 - a2
        dd = bb*bb - 4._rkx*cc
        psi21 = 0.5_rkx*(-bb+sqrt(dd))
        psi22 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi21>zero .and. psi21>zero ) then
          psi2 = psi21
        else if ( delt-psi22>zero .and. psi22>zero ) then
          psi2 = psi22
        else
          psi2 = zero
        end if
      else
        psi2 = zero
      end if
      psi2 = max(min(min(psi2,chi4-psi4-psi3),chi5-psi5),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi2,tiny1)
      ghcl = max(ghno3-psi2,tiny1)
      cnh4no3 = psi2
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp3 = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp3 = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp2
! *** case p2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 ,
!                          mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
!     there are three regimes in this case:
!     1. cacl2(s) possible. liquid & solid aerosol (subroutine calcl2a)
!     2. cacl2(s) not possible , and rh < mdrh. solid aerosol only
!     3. cacl2(s) not possible , and rh >= mdrh. solid & liquid aerosol
!
!     regimes 2. and 3. are considered to be the same as cases p1a , p2b
!     respectively
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
!
  subroutine calcp2
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcp1a
!
! *** regime depends upon the possible solids & rh
! **********************
    if ( ccacl2>tiny1 ) then
      scase = 'p2 ; subcase 1'
      call calcp2a
      scase = 'p2 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drmp2 ) then                ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcp1a
        scase = 'p2 ; subcase 2'
      else
        if ( cmgcl2>tiny1 ) then
          scase = 'p2 ; subcase 3'         ! mdrh (caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 , mgcl2 ,
!       mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl)
          call calcmdrh2(rh,drmp2,drmgcl2,calcp1a,calcp3a)
          scase = 'p2 ; subcase 3'
        end if
        if ( water<=tiny1 .and. rh>=drmp3 .and. rh<drmp4 ) then
          scase = 'p2 ; subcase 4'         ! mdrh (caso4 , k2so4 , kno3 , kcl , mgso4 , cano32 ,
!       mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl)
          call calcmdrh2(rh,drmp3,drcano32,calcp1a,calcp4a)
          scase = 'p2 ; subcase 4'
        end if
        if ( water<=tiny1 .and. rh>=drmp4 .and. rh<drmp5 ) then
          scase = 'p2 ; subcase 5'         ! mdrh (caso4 , k2so4 , kno3 , kcl , mgso4 ,
!       mgno32 , nano3 , nacl , nh4no3 , nh4cl)
          call calcmdrh2(rh,drmp4,drmgno32,calcp1a,calcp5a)
          scase = 'p2 ; subcase 5'
        end if
        if ( water<=tiny1 .and. rh>=drmp5 ) then
          scase = 'p2 ; subcase 6'         ! mdrh (caso4 , k2so4 , kno3 , kcl , mgso4 ,
!       nano3 , nacl , nh4no3 , nh4cl)
          call calcmdrh2(rh,drmp5,drnh4no3,calcp1a,calcp6)
          scase = 'p2 ; subcase 6'
        else
          water = tiny1
          do i = 1 , nions
            molal(i) = zero
          end do
          call calcp1a
          scase = 'p2 ; subcase 2'
        end if
      end if
    end if
  end subroutine calcp2
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp2a
! *** case p2a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl ,
!                          nano3 , nh4no3 , mg(no3)2 , ca(no3)2
!     4. completely dissolved: cacl2
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcp2a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: delta , dx , frca , frcl , frk , frmg , frna , &
         frno3 , frso4 , psi6hi , psi6lo , x1 , x2 ,  &
         x3 , y1 , y2 , y3
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.
    chi11 = min(w(2),w(6))                          ! ccaso4
    frca = max(w(6)-chi11,zero)
    frso4 = max(w(2)-chi11,zero)
    chi9 = min(frso4,0.5_rkx*w(7))                     ! ck2so4
    frk = max(w(7)-2._rkx*chi9,zero)
    frso4 = max(frso4-chi9,zero)
    chi10 = frso4                                   ! cmgso4
    frmg = max(w(8)-chi10,zero)
    chi7 = min(w(1),w(5))                           ! cnacl
    frna = max(w(1)-chi7,zero)
    frcl = max(w(5)-chi7,zero)
    chi12 = min(frca,0.5_rkx*w(4))                     ! ccano32
    frca = max(frca-chi12,zero)
    frno3 = max(w(4)-2._rkx*chi12,zero)
    chi17 = min(frca,0.5_rkx*frcl)                     ! ccacl2
    frca = max(frca-chi17,zero)
    frcl = max(frcl-2._rkx*chi17,zero)
    chi15 = min(frmg,0.5_rkx*frno3)                    ! cmgno32
    frmg = max(frmg-chi15,zero)
    frno3 = max(frno3-2._rkx*chi15,zero)
    chi16 = min(frmg,0.5_rkx*frcl)                     ! cmgcl2
    frmg = max(frmg-chi16,zero)
    frcl = max(frcl-2._rkx*chi16,zero)
    chi8 = min(frna,frno3)                           ! cnano3
    frna = max(frna-chi8,zero)
    frno3 = max(frno3-chi8,zero)
    chi14 = min(frk,frcl)                            ! ckcl
    frk = max(frk-chi14,zero)
    frcl = max(frcl-chi14,zero)
    chi13 = min(frk,frno3)                           ! ckno3
    frk = max(frk-chi13,zero)
    frno3 = max(frno3-chi13,zero)
!
    chi5 = frno3                                    ! hno3(g)
    chi6 = frcl                                     ! hcl(g)
    chi4 = w(3)                                     ! nh3(g)
!
    chi3 = zero                                     ! cnh4cl
    chi1 = zero
    chi2 = zero
!
    psi6lo = tiny1
    psi6hi = chi6 - tiny1    ! min(chi6-tiny1 , chi4)
!
! *** initial values for bisection ************************************
!
    x1 = psi6lo
    y1 = funcp2a(x1)
    y2 = y1
    if ( abs(y1)>eps .and. chi6>tiny1 ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 + dx
        y2 = funcp2a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** no subdivision with solution; if abs(y2)<eps solution is assumed
!
      if ( abs(y2)>eps ) y2 = funcp2a(psi6lo)
    end if
    go to 300
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcp2a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcp2a')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcp2a(x3)
!
! *** calculate hso4 speciation and return
! *******************************
 300  if ( molal(1)>tiny1 .and. molal(5)>tiny1 ) then
      call calchs4(molal(1),molal(5),zero,delta)
      molal(1) = molal(1) - delta                        ! h+   effect
      molal(5) = molal(5) - delta                        ! so4  effect
      molal(6) = delta                                   ! hso4 effect
    end if
  end subroutine calcp2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcp2a
! *** case p2a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl ,
!                          nano3 , nh4no3 , mg(no3)2 , ca(no3)2 , mgcl2
!     4. completely dissolved: cacl2
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcp2a(x)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , bbp , cc , ccp , dd , ddp , delt , delta , diak , &
         gkama , hi , ohi , psi21 , psi22 , psi31 , psi32 , smin , vhta , &
         vita , x
    integer(ik4) :: i , islv
!
! *** setup parameters ************************************************
!
    psi6 = x
    psi1 = zero
    psi2 = zero
    psi3 = zero
    psi7 = zero
    psi8 = zero
    psi9 = zero
    psi10 = chi10
    psi11 = zero
    psi12 = chi12
    psi13 = zero
    psi14 = zero
    psi15 = chi15
    psi16 = chi16
    psi17 = chi17
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2
      a5 = xk4*r*temp*(water/gama(10))**2
      a6 = xk3*r*temp*(water/gama(11))**2
      a9 = xk17*(water/gama(17))**3
      a13 = xk19*(water/gama(19))**2
      a14 = xk20*(water/gama(20))**2
      a7 = xk8*(water/gama(1))**2
      a8 = xk9*(water/gama(3))**2
!
!   caculate dissociation quantities
!
      psi5 = (chi5-psi2)*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)          &
             - a6/a5*(psi8+2._rkx*psi12+psi13+2._rkx*psi15)*(chi6-psi6-psi3)
      psi5 = psi5/(a6/a5*(chi6-psi6-psi3)+psi6+psi7+psi14+2._rkx*psi16+     &
             2._rkx*psi17)
      psi5 = min(max(psi5,tiny1),chi5)
!
      if ( w(3)>tiny1 .and. water>tiny1 ) then       ! first try 3rd order soln
        bb = -(chi4+psi6+psi5+1._rkx/a4)
        cc = chi4*(psi5+psi6)
        dd = max(bb*bb-4._rkx*cc,zero)
        psi4 = 0.5_rkx*(-bb-sqrt(dd))
        psi4 = min(max(psi4,zero),chi4)
      else
        psi4 = tiny1
      end if
!
      if ( chi13>tiny1 .and. water>tiny1 ) then               !kno3
        vhta = psi5 + psi8 + 2._rkx*psi12 + 2._rkx*psi15 + psi14 + 2._rkx*psi9
        gkama = (psi5+psi8+2._rkx*psi12+2._rkx*psi15)*(2._rkx*psi9+psi14) - a13
        delta = max(vhta*vhta-4._rkx*gkama,zero)
        psi13 = 0.5_rkx*(-vhta+sqrt(delta))
        psi13 = min(max(psi13,zero),chi13)
      end if
!
      if ( chi14>tiny1 .and. water>tiny1 ) then               !kcl
        psi14 = a14/a13*(psi5+psi8+2._rkx*psi12+psi13+2._rkx*psi15)           &
                - psi6 - psi7 - 2._rkx*psi16 - 2._rkx*psi17
        psi14 = min(max(psi14,zero),chi14)
      end if
!
      if ( chi9>tiny1 .and. water>tiny1 ) then               !k2so4
        bbp = psi10 + psi13 + psi14
        ccp = (psi13+psi14)*(0.25_rkx*(psi13+psi14)+psi10)
        ddp = 0.25_rkx*(psi13+psi14)**2*psi10 - a9/4._rkx
        call poly3(bbp,ccp,ddp,psi9,islv)
        if ( islv==0 ) then
          psi9 = min(max(psi9,zero),chi9)
        else
          psi9 = zero
        end if
      end if
!
      if ( chi7>tiny1 .and. water>tiny1 ) then          ! nacl dissolution
        vita = psi6 + psi14 + psi8 + 2._rkx*psi16 + 2._rkx*psi17
        gkama = psi8*(2._rkx*psi16+psi6+psi14+2._rkx*psi17) - a7
        diak = max(vita*vita-4.0_rkx*gkama,zero)
        psi7 = 0.5_rkx*(-vita+sqrt(diak))
        psi7 = max(min(psi7,chi7),zero)
      end if
!
      if ( chi8>tiny1 .and. water>tiny1 ) then          ! nano3 dissolution
!     vit  = psi5+psi13+psi7+2.d0*psi12+2.d0*psi15
!     gkam = psi7*(2.d0*psi12+psi5+psi13+2.d0*psi15)-a8
!     dia  = max(vit*vit - 4.0d0*gkam ,zero)
!     psi8 = 0.5d0*( -vit + sqrt(dia) )
        psi8 = a8/a7*(psi6+psi7+psi14+2._rkx*psi16+2._rkx*psi17)              &
               - psi5 - 2._rkx*psi12 - psi13 - 2._rkx*psi15
        psi8 = max(min(psi8,chi8),zero)
      end if
!
!
!   *** calculate speciation
!   ********************************************
      molal(2) = psi8 + psi7                                         ! nai
      molal(3) = psi4                                                ! nh4i
      molal(4) = psi6 + psi7 + psi14 + 2._rkx*psi16 + 2._rkx*psi17       ! cli
      molal(5) = psi9 + psi10                                        ! so4i
      molal(6) = zero                                                ! hso4i
      molal(7) = psi5 + psi8 + 2._rkx*psi12 + psi13 + 2._rkx*psi15       ! no3i
      molal(8) = psi11 + psi12 + psi17                               ! cai
      molal(9) = 2._rkx*psi9 + psi13 + psi14                           ! ki
      molal(10) = psi10 + psi15 + psi16                               ! mgi
!
!   *** calculate h+
!   *****************************************************
!   rest  = 2.d0*w(2) + w(4) + w(5)
!
!   delt1 = 0.0d0
!   delt2 = 0.0d0
!   if (w(1)+w(6)+w(7)+w(8) > rest) then
!
!   ** calculate equilibrium constants
!   **********************************
!   alfa1 = xk26*rh*(water/1.0)                   ! co2(aq) + h2o
!   alfa2 = xk27*(water/1.0)                      ! hco3-
!
!   x     = w(1)+w(6)+w(7)+w(8) - rest            ! excess of crustals
!   equals co2(aq)
!   diak  = sqrt( (alfa1)**2 + 4.0d0*alfa1*x)
!   delt1 = 0.5d0*(-alfa1 + diak)
!   delt1 = min ( max (delt1 , zero), x)
!   delt2 = alfa2
!   delt2 = min ( delt2 , delt1)
!   molal(1) = delt1 + delt2                      ! h+
!   else
!
!   ** no excess of crustals calculate h+
!   *******************************
      smin = 2._rkx*molal(5) + molal(7) + molal(4) - molal(2) - molal(3)    &
             - molal(9) - 2._rkx*molal(10) - 2._rkx*molal(8)
      call calcph(smin,hi,ohi)
      molal(1) = hi
!   end if
!
      gnh3 = max(chi4-psi4,tiny1)
      ghno3 = max(chi5-psi5,tiny1)
      ghcl = max(chi6-psi6,tiny1)
!
!   cnh4cl    = zero
!   cnh4no3   = zero
      cnacl = max(chi7-psi7,zero)
      cnano3 = max(chi8-psi8,zero)
      ck2so4 = max(chi9-psi9,zero)
      cmgso4 = zero
      ccaso4 = chi11
      ccano32 = zero
      ckno3 = max(chi13-psi13,zero)
      ckcl = max(chi14-psi14,zero)
      cmgno32 = zero
      cmgcl2 = zero
      ccacl2 = zero
!
!   *** nh4cl(s) calculations
!
      a3 = xk6/(r*temp*r*temp)
      if ( gnh3*ghcl>a3 ) then
        delt = min(gnh3,ghcl)
        bb = -(gnh3+ghcl)
        cc = gnh3*ghcl - a3
        dd = bb*bb - 4._rkx*cc
        psi31 = 0.5_rkx*(-bb+sqrt(dd))
        psi32 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi31>zero .and. psi31>zero ) then
          psi3 = psi31
        else if ( delt-psi32>zero .and. psi32>zero ) then
          psi3 = psi32
        else
          psi3 = zero
        end if
      else
        psi3 = zero
      end if
      psi3 = max(min(min(psi3,chi4-psi4),chi6-psi6),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi3,tiny1)
      ghcl = max(ghcl-psi3,tiny1)
      cnh4cl = psi3
!
!   *** nh4no3(s) calculations
!
      a2 = xk10/(r*temp*r*temp)
      if ( gnh3*ghno3>a2 ) then
        delt = min(gnh3,ghno3)
        bb = -(gnh3+ghno3)
        cc = gnh3*ghno3 - a2
        dd = bb*bb - 4._rkx*cc
        psi21 = 0.5_rkx*(-bb+sqrt(dd))
        psi22 = 0.5_rkx*(-bb-sqrt(dd))
        if ( delt-psi21>zero .and. psi21>zero ) then
          psi2 = psi21
        else if ( delt-psi22>zero .and. psi22>zero ) then
          psi2 = psi22
        else
          psi2 = zero
        end if
      else
        psi2 = zero
      end if
      psi2 = max(min(min(psi2,chi4-psi4-psi3),chi5-psi5),zero)
!
!   *** calculate gas / solid species (liquid in molal already)
!   *********
      gnh3 = max(gnh3-psi2,tiny1)
      ghcl = max(ghno3-psi2,tiny1)
      cnh4no3 = psi2
!
      call calcmr                                       ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate function value for outer loop
! ***************************
! funcp2a = molal(3)*molal(4)/ghcl/gnh3/a6/a4 - one
    funcp2a = molal(1)*molal(4)/ghcl/a6 - one
  end function funcp2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp1
! *** case p1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , ca(no3)2 , cacl2 , k2so4 , kno3 , kcl , mgso4 ,
!                          mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
!     there are two possible regimes here , depending on relative humidity:
!     1. when rh >= mdrh ; liquid phase possible (mdrh region)
!     2. when rh < mdrh  ; only solid phase possible (subroutine calcp1a)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcp1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmp1 ) then
      scase = 'p1 ; subcase 1'
      call calcp1a                 ! solid phase only possible
      scase = 'p1 ; subcase 1'
    else
      scase = 'p1 ; subcase 2'     ! liquid & solid phase possible
      call calcmdrh2(rh,drmp1,drcacl2,calcp1a,calcp2a)
      scase = 'p1 ; subcase 2'
    end if
  end subroutine calcp1
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcp1a
! *** case p1a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , ca(no3)2 , cacl2 , k2so4 , kno3 , kcl , mgso4 ,
!                          mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================

  subroutine calcp1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a1 , a2 , alf , bb , bet , cafr , cc , clfr , dd , &
         dd1 , dd2 , frk , frmg , gam , rtsq , so4fr , sqdd , sqdd1 , &
         sqdd2 , theta1 , theta2
    real(rkx) :: lamda , lamda1 , lamda2 , kapa , kapa1 , kapa2 , nafr , &
              no3fr
!
! *** calculate non volatile solids ***********************************
!
    ccaso4 = min(w(2),w(6))                         !solid caso4
    cafr = max(w(6)-ccaso4,zero)
    so4fr = max(w(2)-ccaso4,zero)
    ck2so4 = min(so4fr,0.5_rkx*w(7))                   !solid k2so4
    frk = max(w(7)-2._rkx*ck2so4,zero)
    so4fr = max(so4fr-ck2so4,zero)
    cmgso4 = so4fr                                  !solid mgso4
    frmg = max(w(8)-cmgso4,zero)
    cnacl = min(w(1),w(5))                          !solid nacl
    nafr = max(w(1)-cnacl,zero)
    clfr = max(w(5)-cnacl,zero)
    ccano32 = min(cafr,0.5_rkx*w(4))                   !solid ca(no3)2
    cafr = max(cafr-ccano32,zero)
    no3fr = max(w(4)-2._rkx*ccano32,zero)
    ccacl2 = min(cafr,0.5_rkx*clfr)                    !solid cacl2
    cafr = max(cafr-ccacl2,zero)
    clfr = max(clfr-2._rkx*ccacl2,zero)
    cmgno32 = min(frmg,0.5_rkx*no3fr)                  !solid mg(no3)2
    frmg = max(frmg-cmgno32,zero)
    no3fr = max(no3fr-2._rkx*cmgno32,zero)
    cmgcl2 = min(frmg,0.5_rkx*clfr)                    !solid mgcl2
    frmg = max(frmg-cmgcl2,zero)
    clfr = max(clfr-2._rkx*cmgcl2,zero)
    cnano3 = min(nafr,no3fr)                         !solid nano3
    nafr = max(nafr-cnano3,zero)
    no3fr = max(no3fr-cnano3,zero)
    ckcl = min(frk,clfr)                             !solid kcl
    frk = max(frk-ckcl,zero)
    clfr = max(clfr-ckcl,zero)
    ckno3 = min(frk,no3fr)                           !solid kno3
    frk = max(frk-ckno3,zero)
    no3fr = max(no3fr-ckno3,zero)
!
! *** calculate volatile species **************************************
!
    alf = w(3)                           ! free nh3
    bet = clfr                           ! free cl
    gam = no3fr                          ! free no3
!
    rtsq = r*temp*r*temp
    a1 = xk6/rtsq
    a2 = xk10/rtsq
!
    theta1 = gam - bet*(a2/a1)
    theta2 = a2/a1
!
! quaratic equation solution
!
    bb = (theta1-alf-bet*(one+theta2))/(one+theta2)
    cc = (alf*bet-a1-bet*theta1)/(one+theta2)
    dd = bb*bb - 4.0_rkx*cc
    if ( dd>=zero ) then        ! solve each reaction seperately
!
!   two roots for kapa , check and see if any valid
!
      sqdd = sqrt(dd)
      kapa1 = 0.5_rkx*(-bb+sqdd)
      kapa2 = 0.5_rkx*(-bb-sqdd)
      lamda1 = theta1 + theta2*kapa1
      lamda2 = theta1 + theta2*kapa2
!
      if ( kapa1>=zero .and. lamda1>=zero ) then
        if ( alf-kapa1-lamda1>=zero .and. bet-kapa1>=zero .and.           &
             gam-lamda1>=zero ) then
          kapa = kapa1
          lamda = lamda1
          go to 100
        end if
      end if
!
      if ( kapa2>=zero .and. lamda2>=zero ) then
        if ( alf-kapa2-lamda2>=zero .and. bet-kapa2>=zero .and.           &
             gam-lamda2>=zero ) then
          kapa = kapa2
          lamda = lamda2
          go to 100
        end if
      end if
    end if
!
! seprate solution of nh4cl & nh4no3 equilibria
!
    kapa = zero
    lamda = zero
    dd1 = (alf+bet)*(alf+bet) - 4.0_rkx*(alf*bet-a1)
    dd2 = (alf+gam)*(alf+gam) - 4.0_rkx*(alf*gam-a2)
!
! nh4l equilibrium
!
    if ( dd1>=zero ) then
      sqdd1 = sqrt(dd1)
      kapa1 = 0.5_rkx*(alf+bet+sqdd1)
      kapa2 = 0.5_rkx*(alf+bet-sqdd1)
!
      if ( kapa1>=zero .and. kapa1<=min(alf,bet) ) then
        kapa = kapa1
      else if ( kapa2>=zero .and. kapa2<=min(alf,bet) ) then
        kapa = kapa2
      else
        kapa = zero
      end if
    end if
!
! nh4o3 equilibrium
!
    if ( dd2>=zero ) then
      sqdd2 = sqrt(dd2)
      lamda1 = 0.5_rkx*(alf+gam+sqdd2)
      lamda2 = 0.5_rkx*(alf+gam-sqdd2)
!
      if ( lamda1>=zero .and. lamda1<=min(alf,gam) ) then
        lamda = lamda1
      else if ( lamda2>=zero .and. lamda2<=min(alf,gam) ) then
        lamda = lamda2
      else
        lamda = zero
      end if
    end if
!
! if oth kapa , lamda are > 0 , then apply existance criterion
!
    if ( kapa>zero .and. lamda>zero ) then
      if ( bet<lamda/theta1 ) then
        kapa = zero
      else
        lamda = zero
      end if
    end if
!
! *** calculate composition of volatile species ***********************
!
 100  cnh4no3 = lamda
    cnh4cl = kapa
!
    gnh3 = alf - kapa - lamda
    ghno3 = gam - lamda
    ghcl = bet - kapa
  end subroutine calcp1a
!
!=====================================================================
!
! *** isorropia code
! *** subroutine calcl9
! *** case l9
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : caso4
!     4. completely dissolved: nh4hso4 , nahso4 , lc , (nh4)2so4 , khso4 , mgso4 , na2so4 , k2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcl9
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , cc , dd
    integer(ik4) :: i
    real(rkx) :: lamda
!
! *** find dry composition
! **********************************************
    call calcl1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calcl1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
    chi6 = ck2so4
    chi7 = cmgso4
    chi8 = ckhso4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = clc
    psi3 = cnahso4
    psi4 = cna2so4
    psi5 = cnh42s4
    psi6 = ck2so4
    psi7 = cmgso4
    psi8 = ckhso4
!
    calaou = .true.                ! outer loop activity calculation flag
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk1*water/gama(7)*(gama(8)/gama(7))**2.
!
!   caculate dissociation quantities
!
      bb = psi7 + psi6 + psi5 + psi4 + psi2 + a9                   ! lamda
      cc = -a9*(psi8+psi1+psi2+psi3)
      dd = max(bb*bb-4._rkx*cc,zero)
      lamda = 0.5_rkx*(-bb+sqrt(dd))
      lamda = min(max(lamda,tiny1),psi8+psi3+psi2+psi1)
!
!   *** calculate speciation
!   ********************************************
      molal(1) = lamda                                               ! hi
      molal(2) = 2._rkx*psi4 + psi3                                    ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1                        ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6 + psi7 + lamda            ! so4i
      molal(6) = psi2 + psi3 + psi1 + psi8 - lamda                   ! hso4i
      molal(9) = psi8 + 2.0_rkx*psi6                                   ! ki
      molal(10) = psi7                                                ! mgi
!
      clc = zero
      cnahso4 = zero
      cna2so4 = zero
      cnh42s4 = zero
      cnh4hs4 = zero
      ck2so4 = zero
      cmgso4 = zero
      ckhso4 = zero
!
      call calcmr                                            ! water content

!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
  end subroutine calcl9
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcl8
! *** case l8
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4
!     4. completely dissolved: nh4hso4 , nahso4 , lc , (nh4)2so4 , khso4 , mgso4 , na2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcl8
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi6hi , psi6lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcl1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calcl1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
    chi6 = ck2so4
    chi7 = cmgso4
    chi8 = ckhso4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = clc
    psi3 = cnahso4
    psi4 = cna2so4
    psi5 = cnh42s4
    psi6 = zero
    psi7 = cmgso4
    psi8 = ckhso4
!
    calaou = .true.                ! outer loop activity calculation flag
    psi6lo = zero                  ! low  limit
    psi6hi = chi6                  ! high limit
!
! *** initial values for bisection ************************************
!
    if ( chi6<=tiny1 ) then
      y1 = funcl8(zero)
      return
    end if
!
    x1 = psi6hi
    y1 = funcl8(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with k2so4
! *********
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi6hi-psi6lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funcl8(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   k2so4
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funcl8(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calcl8')        ! warning error: no solution
      end if
      return
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funcl8(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calcl8')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funcl8(x3)
    end if
  end subroutine calcl8
!
!======================================================================
!
! *** isorropia code ii
! *** function funcl8
! *** case l8
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4
!     4. completely dissolved: nh4hso4 , nahso4 , lc , (nh4)2so4 , khso4 , mgso4 , na2so4
!
!     solution is saved in common block /case/
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcl8(p6)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: bb , cc , dd , p6
    integer(ik4) :: i
    real(rkx) :: lamda
!
! *** setup parameters ************************************************
!
    psi6 = p6
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk1*(water)*(gama(8)**2)/(gama(7)**3)
!
!   caculate dissociation quantities
!
      bb = psi7 + psi6 + psi5 + psi4 + psi2 + a9                   ! lamda
      cc = -a9*(psi8+psi1+psi2+psi3)
      dd = bb*bb - 4._rkx*cc
      lamda = 0.5_rkx*(-bb+sqrt(dd))
      lamda = min(max(lamda,tiny1),psi8+psi3+psi2+psi1)
!
!   *** calculate speciation
!   ********************************************
      molal(1) = lamda                                               ! hi
      molal(2) = 2._rkx*psi4 + psi3                                    ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1                        ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6 + psi7 + lamda            ! so4i
      molal(6) = max(psi2+psi3+psi1+psi8-lamda,tiny1)                  ! hso4i
      molal(9) = psi8 + 2.0*psi6                                     ! ki
      molal(10) = psi7                                                ! mgi
!
      clc = zero
      cnahso4 = zero
      cna2so4 = zero
      cnh42s4 = zero
      cnh4hs4 = zero
      ck2so4 = max(chi6-psi6,zero)
      cmgso4 = zero
      ckhso4 = zero
      call calcmr                                          ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    a6 = xk17*(water/gama(17))**3
    funcl8 = molal(9)*molal(9)*molal(5)/a6 - one
  end function funcl8
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcl7
! *** case l7
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , na2so4
!     4. completely dissolved: nh4hso4 , nahso4 , lc , (nh4)2so4 , khso4 , mgso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcl7
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi4hi , psi4lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcl1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calcl1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
    chi6 = ck2so4
    chi7 = cmgso4
    chi8 = ckhso4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = clc
    psi3 = cnahso4
    psi4 = zero
    psi5 = cnh42s4
    psi6 = zero
    psi7 = cmgso4
    psi8 = ckhso4
!
    calaou = .true.                ! outer loop activity calculation flag
    psi4lo = zero                  ! low  limit
    psi4hi = chi4                  ! high limit
!
! *** initial values for bisection ************************************
!
    if ( chi4<=tiny1 ) then
      y1 = funcl7(zero)
      return
    end if
!
    x1 = psi4hi
    y1 = funcl7(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with k2so4
! *********
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi4hi-psi4lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funcl7(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   k2so4
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funcl7(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calcl7')        ! warning error: no solution
      end if
      return
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funcl7(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calcl7')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funcl7(x3)
    end if
  end subroutine calcl7
!
!======================================================================
!
! *** isorropia code ii
! *** function funcl7
! *** case l7
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , na2so4
!     4. completely dissolved: nh4hso4 , nahso4 , lc , (nh4)2so4 , khso4 , mgso4
!
!     solution is saved in common block /case/
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcl7(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , cc , dd , p4
    integer(ik4) :: i , islv
    real(rkx) :: lamda
!
! *** setup parameters ************************************************
!
    psi4 = p4
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    lamda = tiny1
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = xk5*(water/gama(2))**3
      a6 = xk17*(water/gama(17))**3
      a9 = xk1*(water)*(gama(8)**2)/(gama(7)**3)
!
!   caculate dissociation quantities
!
!   psi6 = 0.5d0*(sqrt(a6/a4)*(2.d0*psi4+psi3)-psi8)             ! psi6
!   psi6 = min (max (psi6 , zero), chi6)
!
      if ( chi6>tiny1 .and. water>tiny1 ) then
        aa = psi5 + psi4 + psi2 + psi7 + psi8 + lamda
        bb = psi8*(psi5+psi4+psi2+psi7+0.25_rkx*psi8+lamda)
        cc = 0.25_rkx*(psi8*psi8*(psi5+psi4+psi2+psi7+lamda)-a6)
        call poly3(aa,bb,cc,psi6,islv)
        if ( islv==0 ) then
          psi6 = min(psi6,chi6)
        else
          psi6 = zero
        end if
      end if
!
      bb = psi7 + psi6 + psi5 + psi4 + psi2 + a9                   ! lamda
      cc = -a9*(psi8+psi1+psi2+psi3)
      dd = bb*bb - 4._rkx*cc
      lamda = 0.5_rkx*(-bb+sqrt(dd))
      lamda = min(max(lamda,tiny1),psi8+psi3+psi2+psi1)
!
!
!   *** calculate speciation
!   ********************************************
      molal(1) = lamda                                               ! hi
      molal(2) = 2._rkx*psi4 + psi3                                    ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1                        ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6 + psi7 + lamda            ! so4i
      molal(6) = max(psi2+psi3+psi1+psi8-lamda,tiny1)                  ! hso4i
      molal(9) = psi8 + 2.0*psi6                                     ! ki
      molal(10) = psi7                                                ! mgi
!
      clc = zero
      cnahso4 = zero
      cna2so4 = max(chi4-psi4,zero)
      cnh42s4 = zero
      cnh4hs4 = zero
      ck2so4 = max(chi6-psi6,zero)
      cmgso4 = zero
      ckhso4 = zero
      call calcmr                                          ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    a4 = xk5*(water/gama(2))**3
    funcl7 = molal(5)*molal(2)*molal(2)/a4 - one
  end function funcl7
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcl6
! *** case l6
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , na2so4
!     4. completely dissolved: nh4hso4 , nahso4 , lc , (nh4)2so4 , khso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcl6
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi4hi , psi4lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcl1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calcl1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
    chi6 = ck2so4
    chi7 = cmgso4
    chi8 = ckhso4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = clc
    psi3 = cnahso4
    psi4 = zero
    psi5 = cnh42s4
    psi6 = zero
    psi7 = zero
    psi8 = ckhso4
!
    calaou = .true.                ! outer loop activity calculation flag
    psi4lo = zero                  ! low  limit
    psi4hi = chi4                  ! high limit
!
! *** initial values for bisection ************************************
!
    if ( chi4<=tiny1 ) then
      y1 = funcl6(zero)
      return
    end if
!
    x1 = psi4hi
    y1 = funcl6(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with k2so4
! *********
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi4hi-psi4lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funcl6(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   k2so4
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funcl6(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calcl6')        ! warning error: no solution
      end if
      return
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funcl6(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calcl6')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funcl6(x3)
    end if
  end subroutine calcl6
!
!======================================================================
!
! *** isorropia code ii
! *** function funcl6
! *** case l6
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , na2so4
!
!     solution is saved in common block /case/
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcl6(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , cc , dd , p4
    integer(ik4) :: i , islv
    real(rkx) :: lamda
!
! *** setup parameters ************************************************
!
    psi4 = p4
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    lamda = tiny1
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = xk5*(water/gama(2))**3
      a6 = xk17*(water/gama(17))**3
      a9 = xk1*(water)*(gama(8)**2)/(gama(7)**3)
!
!   caculate dissociation quantities
!
!   psi6 = 0.5d0*(sqrt(a6/a4)*(2.d0*psi4+psi3)-psi8)             ! psi6
!   psi6 = min (max (psi6 , zero), chi6)
!
      if ( chi6>tiny1 .and. water>tiny1 ) then
        aa = psi5 + psi4 + psi2 + psi7 + psi8 + lamda
        bb = psi8*(psi5+psi4+psi2+psi7+0.25_rkx*psi8+lamda)
        cc = 0.25_rkx*(psi8*psi8*(psi5+psi4+psi2+psi7+lamda)-a6)
        call poly3(aa,bb,cc,psi6,islv)
        if ( islv==0 ) then
          psi6 = min(psi6,chi6)
        else
          psi6 = zero
        end if
      end if
!
      psi7 = chi7
!
      bb = psi7 + psi6 + psi5 + psi4 + psi2 + a9                    ! lamda
      cc = -a9*(psi8+psi1+psi2+psi3)
      dd = bb*bb - 4._rkx*cc
      lamda = 0.5_rkx*(-bb+sqrt(dd))
      lamda = min(max(lamda,tiny1),psi8+psi3+psi2+psi1)
!
!   *** calculate speciation
!   ********************************************
      molal(1) = lamda                                               ! hi
      molal(2) = 2._rkx*psi4 + psi3                                    ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1                        ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6 + psi7 + lamda            ! so4i
      molal(6) = max(psi2+psi3+psi1+psi8-lamda,tiny1)                  ! hso4i
      molal(9) = psi8 + 2.0*psi6                                     ! ki
      molal(10) = psi7                                                ! mgi
!
      clc = zero
      cnahso4 = zero
      cna2so4 = max(chi4-psi4,zero)
      cnh42s4 = zero
      cnh4hs4 = zero
      ck2so4 = max(chi6-psi6,zero)
      cmgso4 = zero
      ckhso4 = zero
      call calcmr                                          ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    a4 = xk5*(water/gama(2))**3
    funcl6 = molal(5)*molal(2)*molal(2)/a4 - one
  end function funcl6
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcl5
! *** case l5
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , na2so4
!     4. completely dissolved: nh4hso4 , nahso4 , lc , (nh4)2so4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcl5
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi4hi , psi4lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcl1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calcl1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
    chi6 = ck2so4
    chi7 = cmgso4
    chi8 = ckhso4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = clc
    psi3 = cnahso4
    psi4 = zero
    psi5 = cnh42s4
    psi6 = zero
    psi7 = zero
    psi8 = zero
!
    calaou = .true.                ! outer loop activity calculation flag
    psi4lo = zero                  ! low  limit
    psi4hi = chi4                  ! high limit

!
! *** initial values for bisection ************************************
!
    if ( chi4<=tiny1 ) then
      y1 = funcl5(zero)
      return
    end if
!
    x1 = psi4hi
    y1 = funcl5(x1)
    y2 = y1
    yhi = y1                        ! save y-value at hi position
!
! *** yhi < 0.0 the solution is always undersaturated with na2so4
! *********
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************

      dx = (psi4hi-psi4lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = max(x1-dx,psi4lo)
        y2 = funcl5(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   na2so4
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funcl5(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calcl5')        ! warning error: no solution
      end if
      return
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funcl5(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calcl5')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funcl5(x3)
    end if
  end subroutine calcl5
!
!======================================================================
!
! *** isorropia code ii
! *** function funcl5
! *** case l5
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , na2so4
!     4. completely dissolved: nh4hso4 , nahso4 , lc , (nh4)2so4
!
!     solution is saved in common block /case/
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcl5(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , bita , cama , cc , dd , delt , p4
    integer(ik4) :: i , islv
    real(rkx) :: lamda
!
! *** setup parameters ************************************************
!
    psi4 = p4
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    lamda = tiny1
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = xk5*(water/gama(2))**3
      a6 = xk17*(water/gama(17))**3
      a8 = xk18*(water/gama(18))**2
      a9 = xk1*(water)*(gama(8)**2)/(gama(7)**3)
!
!   caculate dissociation quantities
!
!   psi6 = 0.5d0*(sqrt(a6/a4)*(2.d0*psi4+psi3)-psi8)             ! psi6
!   psi6 = min (max (psi6 , zero), chi6)
!
      if ( chi6>tiny1 .and. water>tiny1 ) then
        aa = psi5 + psi4 + psi2 + psi7 + psi8 + lamda
        bb = psi8*(psi5+psi4+psi2+psi7+0.25_rkx*psi8+lamda)
        cc = 0.25_rkx*(psi8*psi8*(psi5+psi4+psi2+psi7+lamda)-a6)
        call poly3(aa,bb,cc,psi6,islv)
        if ( islv==0 ) then
          psi6 = min(psi6,chi6)
        else
          psi6 = zero
        end if
      end if
!
      psi7 = chi7
!
      bb = psi7 + psi6 + psi5 + psi4 + psi2 + a9                    ! lamda
      cc = -a9*(psi8+psi1+psi2+psi3)
      dd = max(bb*bb-4._rkx*cc,zero)
      lamda = 0.5_rkx*(-bb+sqrt(dd))
      lamda = min(max(lamda,tiny1),psi8+psi3+psi2+psi1)
!
      bita = psi3 + psi2 + psi1 + 2._rkx*psi6 - lamda
      cama = 2._rkx*psi6*(psi3+psi2+psi1-lamda) - a8
      delt = max(bita*bita-4._rkx*cama,zero)
      psi8 = 0.5_rkx*(-bita+sqrt(delt))
      psi8 = min(max(psi8,zero),chi8)
!
!   *** calculate speciation
!   ********************************************
      molal(1) = lamda                                               ! hi
      molal(2) = 2._rkx*psi4 + psi3                                    ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1                        ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6 + psi7 + lamda            ! so4i
      molal(6) = max(psi2+psi3+psi1+psi8-lamda,tiny1)                  ! hso4i
      molal(9) = psi8 + 2.0_rkx*psi6                                   ! ki
      molal(10) = psi7                                                ! mgi
!
      clc = zero
      cnahso4 = zero
      cna2so4 = max(chi4-psi4,zero)
      cnh42s4 = zero
      cnh4hs4 = zero
      ck2so4 = max(chi6-psi6,zero)
      cmgso4 = zero
      ckhso4 = max(chi8-psi8,zero)
!
      call calcmr                                          ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************

      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    a4 = xk5*(water/gama(2))**3
    funcl5 = molal(5)*molal(2)*molal(2)/a4 - one
  end function funcl5
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcl4
! *** case l4
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , (nh4)2so4 , na2so4
!     4. completely dissolved: nh4hso4 , nahso4 , lc
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcl4
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi4hi , psi4lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcl1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calcl1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
    chi6 = ck2so4
    chi7 = cmgso4
    chi8 = ckhso4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = clc
    psi3 = cnahso4
    psi4 = zero
    psi5 = zero
    psi6 = zero
    psi7 = zero
    psi8 = zero
!
    calaou = .true.                ! outer loop activity calculation flag
    psi4lo = zero                  ! low  limit
    psi4hi = chi4                  ! high limit
!
    if ( chi4<=tiny1 ) then
      y1 = funcl4(zero)
      return
    end if
!
! *** initial values for bisection ************************************
!
    x1 = psi4hi
    y1 = funcl4(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with na2so4
! *********
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi4hi-psi4lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funcl4(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   na2so4 **
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funcl4(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calcl4')        ! warning error: no solution
      end if
      return
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funcl4(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calcl4')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funcl4(x3)
    end if
  end subroutine calcl4
!
!======================================================================
!
! *** isorropia code ii
! *** function funcl4
! *** case l4
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , (nh4)2so4 , na2so4
!     4. completely dissolved: nh4hso4 , nahso4 , lc
!
!     solution is saved in common block /case/
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcl4(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , bita , cama , cc , dd , delt , p4
    integer(ik4) :: i , islv
    real(rkx) :: lamda
!
! *** setup parameters ************************************************
!
    psi4 = p4
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = xk5*(water/gama(2))**3
      a5 = xk7*(water/gama(4))**3
      a6 = xk17*(water/gama(17))**3
      a8 = xk18*(water/gama(18))**2
      a9 = xk1*water/gama(7)*(gama(8)/gama(7))**2
!
!   caculate dissociation quantities
!
      psi5 = (psi3+2._rkx*psi4-sqrt(a4/a5)*(3._rkx*psi2+psi1))                &
             /2._rkx/sqrt(a4/a5)                                      ! ! psi5
      psi5 = max(min(psi5,chi5),zero)
!
      psi7 = chi7
!
      bb = psi7 + psi6 + psi5 + psi4 + psi2 + a9                    ! lamda
      cc = -a9*(psi8+psi1+psi2+psi3)
      dd = max(bb*bb-4._rkx*cc,zero)
      lamda = 0.5_rkx*(-bb+sqrt(dd))
      lamda = min(max(lamda,tiny1),psi8+psi3+psi2+psi1)
!
!   psi6 = 0.5d0*(sqrt(a6/a4)*(2.d0*psi4+psi3)-psi8)             ! psi6
!   psi6 = min (max (psi6 , zero), chi6)
!
      if ( chi6>tiny1 .and. water>tiny1 ) then
        aa = psi5 + psi4 + psi2 + psi7 + psi8 + lamda
        bb = psi8*(psi5+psi4+psi2+psi7+0.25_rkx*psi8+lamda)
        cc = 0.25_rkx*(psi8*psi8*(psi5+psi4+psi2+psi7+lamda)-a6)
        call poly3(aa,bb,cc,psi6,islv)
        if ( islv==0 ) then
          psi6 = min(psi6,chi6)
        else
          psi6 = zero
        end if
      end if
!
      bita = psi3 + psi2 + psi1 + 2._rkx*psi6 - lamda
      cama = 2._rkx*psi6*(psi3+psi2+psi1-lamda) - a8
      delt = max(bita*bita-4._rkx*cama,zero)
      psi8 = 0.5_rkx*(-bita+sqrt(delt))
      psi8 = min(max(psi8,zero),chi8)
!
!   *** calculate speciation
!   ********************************************
      molal(1) = lamda                                               ! hi
      molal(2) = 2._rkx*psi4 + psi3                                    ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1                        ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6 + psi7 + lamda            ! so4i
      molal(6) = max(psi2+psi3+psi1+psi8-lamda,tiny1)                  ! hso4i
      molal(9) = psi8 + 2.0_rkx*psi6                                   ! ki
      molal(10) = psi7                                                ! mgi
!
      clc = zero
      cnahso4 = zero
      cna2so4 = max(chi4-psi4,zero)
      cnh42s4 = max(chi5-psi5,zero)
      cnh4hs4 = zero
      ck2so4 = max(chi6-psi6,zero)
      cmgso4 = zero
      ckhso4 = max(chi8-psi8,zero)
      call calcmr                                          ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    a4 = xk5*(water/gama(2))**3
    funcl4 = molal(5)*molal(2)*molal(2)/a4 - one
  end function funcl4
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcl3
! *** case l3
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , nh4hso4 , nahso4 , (nh4)2so4 , na2so4 , lc
!
!     there are three regimes in this case:
!     1.(na ,nh4)hso4(s) possible. liquid & solid aerosol (subroutine calci3a)
!     2.(na ,nh4)hso4(s) not possible , and rh < mdrh. solid aerosol only
!     3.(na ,nh4)hso4(s) not possible , and rh >= mdrh. solid & liquid aerosol
!
!     regimes 2. and 3. are considered to be the same as cases i1a , i2b
!     respectively
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcl3
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** find dry composition *********************************************
!
    call calcl1a
!
! *** regime depends upon the possible solids & rh *********************
!
    if ( cnh4hs4>tiny1 .or. cnahso4>tiny1 ) then
      scase = 'l3 ; subcase 1'
      call calcl3a                        ! full solution
      scase = 'l3 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drml3 ) then            ! solid solution
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcl1a
        scase = 'l3 ; subcase 2'
      else if ( rh>=drml3 ) then        ! mdrh of l3
        scase = 'l3 ; subcase 3'
        call calcmdrh2(rh,drml3,drlc,calcl1a,calcl4)
        scase = 'l3 ; subcase 3'
      end if
    end if
  end subroutine calcl3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcl3a
! *** case l3 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , (nh4)2so4 , na2so4 , lc
!     4. completely dissolved: nh4hso4 , nahso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcl3a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi2hi , psi2lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcl1a
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calcl1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
    chi6 = ck2so4
    chi7 = cmgso4
    chi8 = ckhso4
!
    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = zero
    psi3 = cnahso4
    psi4 = zero
    psi5 = zero
    psi6 = zero
    psi7 = zero
    psi8 = zero
!
    calaou = .true.                ! outer loop activity calculation flag
    psi2lo = zero                  ! low  limit
    psi2hi = chi2                  ! high limit
!
! *** initial values for bisection ************************************
!
    x1 = psi2hi
    y1 = funcl3a(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with lc *********
!
    if ( yhi>=eps ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi2hi-psi2lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = max(x1-dx,psi2lo)
        y2 = funcl3a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   lc
      if ( y2>eps ) y2 = funcl3a(zero)
    end if
    return
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcl3a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcl3a')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcl3a(x3)
  end subroutine calcl3a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcl3a
! *** case l3 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , (nh4)2so4 , na2so4 , lc
!     4. completely dissolved: nh4hso4 , nahso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcl3a(p2)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , p2 , psi4hi , psi4lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!

    psi2 = p2                      ! save psi2 in common block
    psi4lo = zero                  ! low  limit for psi4
    psi4hi = chi4                  ! high limit for psi4
!
! *** if nh3 = 0 , call funcl3b for y4=0
! ********************************
    if ( chi4<=tiny1 ) then
      funcl3a = funcl3b(zero)
      go to 200
    end if
!
! *** initial values for bisection ************************************
!
    x1 = psi4hi
    y1 = funcl3b(x1)
    y2 = y1
    if ( abs(y1)>eps ) then
      yhi = y1                      ! save y-value at hi position
!
!   *** yhi < 0.0 the solution is always undersaturated with na2so4
!   *********
      if ( yhi>=zero ) then
!
!     *** root tracking ; for the range of hi and lo
!     **********************
        dx = (psi4hi-psi4lo)/float(ndiv)
        do i = 1 , ndiv
          x2 = max(x1-dx,psi4lo)
          y2 = funcl3b(x2)
          if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50  ! (y1*y2 < zero)
          x1 = x2
          y1 = y2
        end do
!
!     *** { ylo , yhi } > 0.0 the solution is always supersaturated
!     with na2so4
        if ( y2>eps ) y2 = funcl3b(psi4lo)
      end if
      go to 200
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funcl3b(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0004,'funcl3a')       ! warning error: no convergence
!
!   *** inner loop converged
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funcl3b(x3)
    end if
!
! *** calculate function value for internal loop
! ***************************
 200  a2 = xk13*(water/gama(13))**5.0
    funcl3a = molal(5)*molal(6)*molal(3)**3/a2 - one
  end function funcl3a
!
!======================================================================
!
! *** isorropia code ii
! *** function funcl3b
! *** case l3 ; subcase 2
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= sulrat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , (nh4)2so4 , na2so4 , lc
!     4. completely dissolved: nh4hso4 , nahso4
!
!     solution is saved in common block /case/
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcl3b(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , bita , cama , cc , dd , delt , p4
    integer(ik4) :: i , islv
    real(rkx) :: lamda
!
! *** setup parameters ************************************************
!
    psi4 = p4
!
    frst = .true.
    calain = .true.
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = xk5*(water/gama(2))**3
      a5 = xk7*(water/gama(4))**3
      a6 = xk17*(water/gama(17))**3
      a8 = xk18*(water/gama(18))**2
      a9 = xk1*(water)*(gama(8)**2)/(gama(7)**3)
!
!   caculate dissociation quantities
!
      psi5 = (psi3+2._rkx*psi4-sqrt(a4/a5)*(3._rkx*psi2+psi1))                &
             /2._rkx/sqrt(a4/a5)                                      ! ! psi5
      psi5 = max(min(psi5,chi5),zero)
!
      psi7 = chi7
!
      bb = psi7 + psi6 + psi5 + psi4 + psi2 + a9                    ! lamda
      cc = -a9*(psi8+psi1+psi2+psi3)
      dd = max(bb*bb-4._rkx*cc,zero)
      lamda = 0.5_rkx*(-bb+sqrt(dd))
      lamda = min(max(lamda,tiny1),psi8+psi3+psi2+psi1)
!
!   psi6 = 0.5d0*(sqrt(a6/a4)*(2.d0*psi4+psi3)-psi8)             ! psi6
!   psi6 = min (max (psi6 , zero), chi6)
!
      if ( chi6>tiny1 .and. water>tiny1 ) then
        aa = psi5 + psi4 + psi2 + psi7 + psi8 + lamda
        bb = psi8*(psi5+psi4+psi2+psi7+0.25_rkx*psi8+lamda)
        cc = 0.25_rkx*(psi8*psi8*(psi5+psi4+psi2+psi7+lamda)-a6)
        call poly3(aa,bb,cc,psi6,islv)
        if ( islv==0 ) then
          psi6 = min(psi6,chi6)
        else
          psi6 = zero
        end if
      end if
!
      bita = psi3 + psi2 + psi1 + 2._rkx*psi6 - lamda
      cama = 2._rkx*psi6*(psi3+psi2+psi1-lamda) - a8
      delt = max(bita*bita-4._rkx*cama,zero)
      psi8 = 0.5_rkx*(-bita+sqrt(delt))
      psi8 = min(max(psi8,zero),chi8)
!
!   *** calculate speciation
!   ********************************************
      molal(1) = lamda                                               ! hi
      molal(2) = 2._rkx*psi4 + psi3                                    ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1                        ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6 + psi7 + lamda            ! so4i
      molal(6) = max(psi2+psi3+psi1+psi8-lamda,tiny1)                  ! hso4i
      molal(9) = psi8 + 2.0_rkx*psi6                                   ! ki
      molal(10) = psi7                                                ! mgi
!
      clc = max(chi2-psi2,zero)
      cnahso4 = zero
      cna2so4 = max(chi4-psi4,zero)
      cnh42s4 = max(chi5-psi5,zero)
      cnh4hs4 = zero
      ck2so4 = max(chi6-psi6,zero)
      cmgso4 = max(chi7-psi7,zero)
      ckhso4 = max(chi8-psi8,zero)
      call calcmr                                          ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    a4 = xk5*(water/gama(2))**3
    funcl3b = molal(5)*molal(2)*molal(2)/a4 - one
  end function funcl3b
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcl2
! *** case l2
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , nh4hso4 , nahso4 , (nh4)2so4 , na2so4 , lc
!
!     there are three regimes in this case:
!     1. nh4hso4(s) possible. liquid & solid aerosol (subroutine calcl2a)
!     2. nh4hso4(s) not possible , and rh < mdrh. solid aerosol only
!     3. nh4hso4(s) not possible , and rh >= mdrh. solid & liquid aerosol
!
!     regimes 2. and 3. are considered to be the same as cases l1a , l2b
!     respectively
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcl2
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcl1a
!
! *** regime depends upon the possible solids & rh
! **********************
    if ( cnh4hs4>tiny1 ) then
      scase = 'l2 ; subcase 1'
      call calcl2a
      scase = 'l2 ; subcase 1'
    end if
!
    if ( water<=tiny1 ) then
      if ( rh<drml2 ) then            ! solid solution only
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcl1a
        scase = 'l2 ; subcase 2'
!
      else if ( rh>=drml2 ) then        ! mdrh of l2
        scase = 'l2 ; subcase 3'
        call calcmdrh2(rh,drml2,drnahso4,calcl1a,calcl3a)
        scase = 'l2 ; subcase 3'
      end if
    end if
  end subroutine calcl2
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcl2a
! *** case l2 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , nahso4 , (nh4)2so4 , na2so4 , lc
!     4. completely dissolved: nh4hso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcl2a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , psi2hi , psi2lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    chi1 = cnh4hs4                 ! save from calcl1 run
    chi2 = clc
    chi3 = cnahso4
    chi4 = cna2so4
    chi5 = cnh42s4
    chi6 = ck2so4
    chi7 = cmgso4
    chi8 = ckhso4
!

    psi1 = cnh4hs4                 ! assign initial psi's
    psi2 = zero
    psi3 = zero
    psi4 = zero
    psi5 = zero
    psi6 = zero
    psi7 = zero
    psi8 = zero
!
    calaou = .true.                ! outer loop activity calculation flag
    psi2lo = zero                  ! low  limit
    psi2hi = chi2                  ! high limit
!
! *** initial values for bisection ************************************
!
    x1 = psi2hi
    y1 = funcl2a(x1)
    y2 = y1
    yhi = y1                        ! save y-value at hi position
!
! *** yhi < 0.0 the solution is always undersaturated with na2so4
! *********
    if ( yhi>=eps ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi2hi-psi2lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = max(x1-dx,psi2lo)
        y2 = funcl2a(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100   ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   na2so4
      if ( y2>eps ) y2 = funcl2a(zero)
    end if
    return
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcl2a(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcl2a')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcl2a(x3)
  end subroutine calcl2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcl2a
! *** case l2 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , nahso4 , (nh4)2so4 , na2so4 , lc
!     4. completely dissolved: nh4hso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcl2a(p2)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , p2 , psi4hi , psi4lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!

    psi2 = p2                      ! save psi3 in common block
    psi4lo = zero                  ! low  limit for psi4
    psi4hi = chi4                  ! high limit for psi4
!
! *** if nh3 = 0 , call funcl3b for y4=0
! ********************************

    if ( chi4<=tiny1 ) then
      funcl2a = funcl2b(zero)
      go to 200
    end if
!
! *** initial values for bisection ************************************
!

    x1 = psi4hi
    y1 = funcl2b(x1)
    y2 = y1

    if ( abs(y1)>eps ) then
      yhi = y1                      ! save y-value at hi position
!
!   *** yhi < 0.0 the solution is always undersaturated with lc
!   *********
      if ( yhi>=zero ) then
!
!     *** root tracking ; for the range of hi and lo
!     **********************
        dx = (psi4hi-psi4lo)/float(ndiv)
        do i = 1 , ndiv
          x2 = max(x1-dx,psi4lo)
          y2 = funcl2b(x2)
          if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50  ! (y1*y2 < zero)
          x1 = x2
          y1 = y2
        end do
!
!     *** { ylo , yhi } > 0.0 the solution is always supersaturated
!     with lc
        if ( y2>eps ) y2 = funcl2b(psi4lo)
      end if
      go to 200
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funcl2b(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0004,'funcl2a')       ! warning error: no convergence
!
!   *** inner loop converged
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funcl2b(x3)
    end if
!
! *** calculate function value for outer loop
! ***************************
 200  a2 = xk13*(water/gama(13))**5.0
    funcl2a = molal(5)*molal(6)*molal(3)**3/a2 - one
  end function funcl2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funcl2b
! *** case l2 ; subcase 2
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , nahso4 , (nh4)2so4 , na2so4 , lc
!     4. completely dissolved: nh4hso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funcl2b(p4)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: aa , bb , bita , cama , cc , dd , delt , p4
    integer(ik4) :: i , islv
    real(rkx) :: lamda
!
! *** setup parameters ************************************************
!
    psi4 = p4                      ! save psi4 in common block
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    psi3 = chi3
    psi5 = chi5
    lamda = zero
    psi6 = chi6
    psi8 = chi8
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a3 = xk11*(water/gama(12))**2
      a4 = xk5*(water/gama(2))**3
      a5 = xk7*(water/gama(4))**3
      a6 = xk17*(water/gama(17))**3
      a8 = xk18*(water/gama(18))**2
      a9 = xk1*(water)*(gama(8)**2)/(gama(7)**3)
!
!   caculate dissociation quantities
!
      psi5 = (psi3+2._rkx*psi4-sqrt(a4/a5)*(3._rkx*psi2+psi1))                &
             /2._rkx/sqrt(a4/a5)                                      ! ! psi5
      psi5 = max(min(psi5,chi5),zero)
!
      if ( chi3>tiny1 .and. water>tiny1 ) then
        aa = 2._rkx*psi4 + psi2 + psi1 + psi8 - lamda
        bb = 2._rkx*psi4*(psi2+psi1+psi8-lamda) - a3
        cc = zero
        call poly3(aa,bb,cc,psi3,islv)
        if ( islv==0 ) then
          psi3 = min(psi3,chi3)
        else
          psi3 = zero
        end if
      end if
!
      psi7 = chi7
!
      bb = psi7 + psi6 + psi5 + psi4 + psi2 + a9                    ! lamda
      cc = -a9*(psi8+psi1+psi2+psi3)
      dd = max(bb*bb-4._rkx*cc,zero)
      lamda = 0.5_rkx*(-bb+sqrt(dd))
      lamda = min(max(lamda,tiny1),psi8+psi3+psi2+psi1)
!
!   psi6 = 0.5d0*(sqrt(a6/a4)*(2.d0*psi4+psi3)-psi8)             ! psi6
!   psi6 = min (max (psi6 , zero), chi6)
!
      if ( chi6>tiny1 .and. water>tiny1 ) then
        aa = psi5 + psi4 + psi2 + psi7 + psi8 + lamda
        bb = psi8*(psi5+psi4+psi2+psi7+0.25_rkx*psi8+lamda)
        cc = 0.25_rkx*(psi8*psi8*(psi5+psi4+psi2+psi7+lamda)-a6)
        call poly3(aa,bb,cc,psi6,islv)
        if ( islv==0 ) then
          psi6 = min(psi6,chi6)
        else
          psi6 = zero
        end if
      end if
!
      bita = psi3 + psi2 + psi1 + 2._rkx*psi6 - lamda                 ! psi8
      cama = 2._rkx*psi6*(psi3+psi2+psi1-lamda) - a8
      delt = max(bita*bita-4._rkx*cama,zero)
      psi8 = 0.5_rkx*(-bita+sqrt(delt))
      psi8 = min(max(psi8,zero),chi8)
!
!   *** calculate speciation
!   ********************************************
      molal(1) = lamda                                               ! hi
      molal(2) = 2._rkx*psi4 + psi3                                    ! nai
      molal(3) = 3._rkx*psi2 + 2._rkx*psi5 + psi1                        ! nh4i
      molal(5) = psi2 + psi4 + psi5 + psi6 + psi7 + lamda            ! so4i
      molal(6) = max(psi2+psi3+psi1+psi8-lamda,tiny1)                  ! hso4i
      molal(9) = psi8 + 2.0_rkx*psi6                                   ! ki
      molal(10) = psi7                                                ! mgi
!
      clc = max(chi2-psi2,zero)
      cnahso4 = max(chi3-psi3,zero)
      cna2so4 = max(chi4-psi4,zero)
      cnh42s4 = max(chi5-psi5,zero)
      cnh4hs4 = zero
      ck2so4 = max(chi6-psi6,zero)
      cmgso4 = max(chi7-psi7,zero)
      ckhso4 = max(chi8-psi8,zero)
      call calcmr                                          ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    a4 = xk5*(water/gama(2))**3
    funcl2b = molal(5)*molal(2)*molal(2)/a4 - one
  end function funcl2b
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcl1
! *** case l1
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , nh4hso4 , nahso4 , (nh4)2so4 , na2so4 , lc
!
!     there are two possible regimes here , depending on relative humidity:
!     1. when rh >= mdrh ; liquid phase possible (mdrh region)
!     2. when rh < mdrh  ; only solid phase possible (subroutine calci1a)
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcl1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drml1 ) then
      scase = 'l1 ; subcase 1'
      call calcl1a                 ! solid phase only possible
      scase = 'l1 ; subcase 1'
    else
      scase = 'l1 ; subcase 2'     ! liquid & solid phase possible
      call calcmdrh2(rh,drml1,drnh4hs4,calcl1a,calcl2a)
      scase = 'l1 ; subcase 2'
    end if
!
! *** ammonia in gas phase
! **********************************************
! call calcnh3
!
  end subroutine calcl1
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcl1a
! *** case l1a
!
!     the main characteristics of this regime are:
!     1. sulfate rich , no free acid (1.0 <= so4rat < 2.0)
!     2. solid aerosol only
!     3. solids possible : k2so4 , caso4 , mgso4 , khso4 , nh4hso4 , nahso4 , (nh4)2so4 , na2so4 , lc
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcl1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: cafr , frk , frmg , frna , frnh4 , frso4
!
! *** calculate non volatile solids ***********************************
!
    ccaso4 = min(w(6),w(2))                         ! ccaso4
    frso4 = max(w(2)-ccaso4,zero)
    cafr = max(w(6)-ccaso4,zero)
    ck2so4 = min(0.5_rkx*w(7),frso4)                  ! ck2so4
    frk = max(w(7)-2._rkx*ck2so4,zero)
    frso4 = max(frso4-ck2so4,zero)
    cna2so4 = min(0.5_rkx*w(1),frso4)                 ! cna2so4
    frna = max(w(1)-2._rkx*cna2so4,zero)
    frso4 = max(frso4-cna2so4,zero)
    cmgso4 = min(w(8),frso4)                        ! cmgso4
    frmg = max(w(8)-cmgso4,zero)
    frso4 = max(frso4-cmgso4,zero)
!
    cnh4hs4 = zero
    cnahso4 = zero
    cnh42s4 = zero
    ckhso4 = zero
!
    clc = min(w(3)/3._rkx,frso4/2._rkx)
    frso4 = max(frso4-2._rkx*clc,zero)
    frnh4 = max(w(3)-3._rkx*clc,zero)
!
    if ( frso4<=tiny1 ) then
      clc = max(clc-frnh4,zero)
      cnh42s4 = 2._rkx*frnh4

    else if ( frnh4<=tiny1 ) then
      cnh4hs4 = 3._rkx*min(frso4,clc)
      clc = max(clc-frso4,zero)
!   if (ck2so4 > tiny1) then
!   frso4  = max(frso4-cnh4hs4/3.d0 , zero)
!   ckhso4 = 2.d0*frso4
!   ck2so4 = max(ck2so4-frso4 , zero)
!   end if
!   if (cna2so4 > tiny1) then
!   frso4   = max(frso4-ckhso4/2.d0 , zero)
!   cnahso4 = 2.d0*frso4
!   cna2so4 = max(cna2so4-frso4 , zero)
!   end if
!
      if ( cna2so4>tiny1 ) then
        frso4 = max(frso4-cnh4hs4/3._rkx,zero)
        cnahso4 = 2._rkx*frso4
        cna2so4 = max(cna2so4-frso4,zero)
      end if
      if ( ck2so4>tiny1 ) then
        frso4 = max(frso4-cnh4hs4/3._rkx,zero)
        ckhso4 = 2._rkx*frso4
        ck2so4 = max(ck2so4-frso4,zero)
      end if
    end if
!
! *** calculate gas species ********************************************
!
    ghno3 = w(4)
    ghcl = w(5)
    gnh3 = zero
  end subroutine calcl1a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calck4
! *** case k4
!
!     the main characteristics of this regime are:
!     1. sulfate super rich , free acid (so4rat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calck4
    use mod_che_common_isorropia
    use mod_che_common_casek
    implicit none
    real(rkx) :: bb , cc , dd , psi4
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.                ! outer loop activity calculation flag
    frst = .true.
    calain = .true.
!
    chi1 = w(3)                    !  total nh4 initially as nh4hso4
    chi2 = w(1)                    !  total na initially as nahso4
    chi3 = w(7)                    !  total k initially as khso4
    chi4 = w(8)                    !  total mg initially as mgso4
!
    lamda = max(w(2)-w(3)-w(1)-w(6)-w(7)-w(8),tiny1)                ! free h2so4
    psi1 = chi1                                ! all nh4hso4 deliquesced
    psi2 = chi2                                ! all nahso4 deliquesced
    psi3 = chi3                                ! all khso4 deliquesced
    psi4 = chi4                                ! all mgso4 deliquesced
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a4 = xk1*water/gama(7)*(gama(8)/gama(7))**2
!
      bb = a4 + lamda + psi4                                ! kapa
      cc = -a4*(lamda+psi3+psi2+psi1) + lamda*psi4
      dd = max(bb*bb-4._rkx*cc,zero)
      kapa = 0.5_rkx*(-bb+sqrt(dd))
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = max(lamda+kapa,tiny1)                                 ! hi
      molal(2) = psi2                                                ! nai
      molal(3) = psi1                                                ! nh4i
      molal(5) = max(kapa+psi4,zero)                                  ! so4i
      molal(6) = max(lamda+psi1+psi2+psi3-kapa,zero)                  ! hso4i
      molal(9) = psi3                                                ! ki
      molal(10) = psi4                                                ! mgi
!
      cnh4hs4 = zero
      cnahso4 = zero
      ckhso4 = zero
      ccaso4 = w(6)
      cmgso4 = zero
!
      call calcmr                                         ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
  end subroutine calck4
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calck3
! *** case k3
!
!     the main characteristics of this regime are:
!     1. sulfate super rich , free acid (so4rat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : khso4 , caso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calck3
    use mod_che_common_isorropia
    use mod_che_common_casek
    implicit none
    real(rkx) :: dx , psi3hi , psi3lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.                ! outer loop activity calculation flag
    chi1 = w(3)                    !  total nh4 initially as nh4hso4
    chi2 = w(1)                    !  total na initially as nahso4
    chi3 = w(7)                    !  total k initially as khso4
    chi4 = w(8)                    !  total mg initially as mgso4
!
    psi3lo = tiny1                  ! low  limit
    psi3hi = chi3                  ! high limit
!
! *** initial values for bisection ************************************
!
    x1 = psi3hi
    y1 = funck3(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with khso4 ****
!
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi3hi-psi3lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funck3(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   khso4
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funck3(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calck3')        ! warning error: no solution
      end if
      return
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funck3(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calck3')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funck3(x3)
    end if
  end subroutine calck3
!
!======================================================================
!
! *** isorropia code
! *** subroutine funck3
! *** case k3
!
!     the main characteristics of this regime are:
!     1. sulfate super rich , free acid (so4rat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : khso4 , caso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funck3(p1)
    use mod_che_common_isorropia
    use mod_che_common_casek
    implicit none
    real(rkx) :: bb , cc , dd , p1 , psi4
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
!
    lamda = max(w(2)-w(3)-w(1)-w(6)-w(7)-w(8),tiny1)                ! free h2so4
    psi3 = p1
    psi1 = chi1                                 ! all nh4hso4 deliquesced
    psi2 = chi2                                 ! all nahso4 deliquesced
    psi4 = chi4                                 ! all mgso4 deliquesced

!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a3 = xk18*(water/gama(18))**2
      a4 = xk1*water/gama(7)*(gama(8)/gama(7))**2
!
!
      bb = a4 + lamda + psi4                              ! kapa
      cc = -a4*(lamda+psi3+psi2+psi1) + lamda*psi4
      dd = max(bb*bb-4._rkx*cc,zero)
      kapa = 0.5_rkx*(-bb+sqrt(dd))
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = max(lamda+kapa,zero)                        ! hi
      molal(2) = psi2                                       ! nai
      molal(3) = psi1                                       ! nh4i
      molal(4) = zero
      molal(5) = max(kapa+psi4,zero)                         ! so4i
      molal(6) = max(lamda+psi1+psi2+psi3-kapa,zero)         ! hso4i
      molal(7) = zero
      molal(8) = zero
      molal(9) = psi3                                       ! ki
      molal(10) = psi4
!
      cnh4hs4 = zero
      cnahso4 = zero
      ckhso4 = chi3 - psi3
      ccaso4 = w(6)
      cmgso4 = zero
!
      call calcmr                                         ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    funck3 = molal(9)*molal(6)/a3 - one
  end function funck3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calck2
! *** case k2
!
!     the main characteristics of this regime are:
!     1. sulfate super rich , free acid (so4rat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : nahso4 , khso4 , caso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calck2
    use mod_che_common_isorropia
    use mod_che_common_casek
    implicit none
    real(rkx) :: dx , psi3hi , psi3lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.                ! outer loop activity calculation flag
    chi1 = w(3)                    !  total nh4 initially as nh4hso4
    chi2 = w(1)                    !  total na initially as nahso4
    chi3 = w(7)                    !  total k initially as khso4
    chi4 = w(8)                    !  total mg initially as mgso4
!
    psi3lo = tiny1                  ! low  limit
    psi3hi = chi3                  ! high limit
!
! *** initial values for bisection ************************************
!
    x1 = psi3hi
    y1 = funck2(x1)
    yhi = y1                        ! save y-value at hi position
    y2 = y1
!
! *** yhi < 0.0 the solution is always undersaturated with khso4 ****
!
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi3hi-psi3lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funck2(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   khso4
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funck2(zero)
      else if ( abs(y2)>=eps ) then    ! x2 is a solution
        call pusherr(0001,'calck2')        ! warning error: no solution
      end if
      return
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funck2(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calck2')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funck2(x3)
    end if
  end subroutine calck2
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funck2
! *** case k2
!
!     the main characteristics of this regime are:
!     1. sulfate super rich , free acid (so4rat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : nahso4 , khso4 , caso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funck2(p1)
    use mod_che_common_isorropia
    use mod_che_common_casek
    implicit none
    real(rkx) :: bb , cc , dd , p1 , psi4
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
!
    lamda = max(w(2)-w(3)-w(1)-w(6)-w(7)-w(8),tiny1)                ! free h2so4
    psi3 = p1
    psi1 = chi1                                  ! all nh4hso4 deliquesced
    psi4 = chi4                                  ! all mgso4 deliquesced
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a2 = xk11*(water/gama(12))**2
      a3 = xk18*(water/gama(18))**2
      a4 = xk1*water/gama(7)*(gama(8)/gama(7))**2
!
      psi2 = a2/a3*psi3                                      ! psi2
      psi2 = min(max(psi2,zero),chi2)
!
      bb = a4 + lamda + psi4                                 ! kapa
      cc = -a4*(lamda+psi3+psi2+psi1) + lamda*psi4
      dd = max(bb*bb-4._rkx*cc,zero)
      kapa = 0.5_rkx*(-bb+sqrt(dd))
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = max(lamda+kapa,zero)                        ! hi
      molal(2) = psi2                                       ! nai
      molal(3) = psi1                                       ! nh4i
      molal(4) = zero
      molal(5) = max(kapa+psi4,zero)                         ! so4i
      molal(6) = max(lamda+psi1+psi2+psi3-kapa,zero)         ! hso4i
      molal(7) = zero
      molal(8) = zero
      molal(9) = psi3                                       ! ki
      molal(10) = psi4
!
      cnh4hs4 = zero
      cnahso4 = chi2 - psi2
      ckhso4 = chi3 - psi3
      ccaso4 = w(6)
      cmgso4 = zero
!
      call calcmr                                         ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    funck2 = molal(9)*molal(6)/a3 - one
  end function funck2
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calck1
! *** case k1
!
!     the main characteristics of this regime are:
!     1. sulfate super rich , free acid (so4rat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : nh4hso4 , nahso4 , khso4 , caso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calck1
    use mod_che_common_isorropia
    use mod_che_common_casek
    implicit none
    real(rkx) :: dx , psi3hi , psi3lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    calaou = .true.                ! outer loop activity calculation flag
    chi1 = w(3)                    !  total nh4 initially as nh4hso4
    chi2 = w(1)                    !  total na initially as nahso4
    chi3 = w(7)                    !  total k initially as khso4
    chi4 = w(8)                    !  total mg initially as mgso4
!
    psi3lo = tiny1                  ! low  limit
    psi3hi = chi3                  ! high limit
!
! *** initial values for bisection ************************************
!
    x1 = psi3hi
    y1 = funck1(x1)
    y2 = y1
    yhi = y1                        ! save y-value at hi position
!
! *** yhi < 0.0 the solution is always undersaturated with khso4 ****
!
    if ( abs(y1)>eps .and. yhi>=zero ) then
!
!   *** root tracking ; for the range of hi and lo
!   **********************
      dx = (psi3hi-psi3lo)/float(ndiv)
      do i = 1 , ndiv
        x2 = x1 - dx
        y2 = funck1(x2)
        if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 50    ! (y1*y2 < zero)
        x1 = x2
        y1 = y2
      end do
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   khso4
      ylo = y1                      ! save y-value at hi position
      if ( ylo>zero .and. yhi>zero ) then
        y3 = funck1(zero)
      else if ( abs(y2)>=eps ) then        ! x2 is a solution
        call pusherr(0001,'calck1')       ! warning error: no solution
      end if
      return
!
!   *** perform bisection
!   ***********************************************
 50   do i = 1 , maxit
        x3 = 0.5_rkx*(x1+x2)
        y3 = funck1(x3)
        if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then       ! (y1*y3  <=  zero)
          y2 = y3
          x2 = x3
        else
          y1 = y3
          x1 = x3
        end if
        if ( abs(x2-x1)<=eps*x1 ) go to 100
      end do
      call pusherr(0002,'calck1')       ! warning error: no convergence
!
!   *** converged ; return
!   **********************************************
 100  x3 = 0.5_rkx*(x1+x2)
      y3 = funck1(x3)
    end if
  end subroutine calck1
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine funck1
! *** case k1
!
!     the main characteristics of this regime are:
!     1. sulfate super rich , free acid (so4rat < 1.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : nh4hso4 , nahso4 , khso4 , caso4
!
! *** copyright 1996-2006 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  real(rkx) function funck1(p1)
    use mod_che_common_isorropia
    use mod_che_common_casek
    implicit none
    real(rkx) :: bb , cc , dd , p1 , psi4
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
!
    lamda = max(w(2)-w(3)-w(1)-w(6)-w(7)-w(8),tiny1)                ! free h2so4
    psi3 = p1
    psi4 = chi4                                        ! all mgso4 deliquesced
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a1 = xk12*(water/gama(09))**2
      a2 = xk11*(water/gama(12))**2
      a3 = xk18*(water/gama(18))**2
      a4 = xk1*water/gama(7)*(gama(8)/gama(7))**2
!
      psi1 = a1/a3*psi3                                      ! psi1
      psi1 = min(max(psi1,zero),chi1)
!
      psi2 = a2/a3*psi3                                      ! psi2
      psi2 = min(max(psi2,zero),chi2)
!
      bb = a4 + lamda + psi4                                 ! kapa
      cc = -a4*(lamda+psi3+psi2+psi1) + lamda*psi4
      dd = max(bb*bb-4._rkx*cc,zero)
      kapa = 0.5_rkx*(-bb+sqrt(dd))
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = max(lamda+kapa,zero)                      ! hi
      molal(2) = psi2                                     ! nai
      molal(3) = psi1                                     ! nh4i
      molal(4) = zero                                     ! cli
      molal(5) = max(kapa+psi4,zero)                       ! so4i
      molal(6) = max(lamda+psi1+psi2+psi3-kapa,zero)       ! hso4i
      molal(7) = zero                                     ! no3i
      molal(8) = zero                                     ! cai
      molal(9) = psi3                                     ! ki
      molal(10) = psi4                                     ! mgi
!
      cnh4hs4 = chi1 - psi1
      cnahso4 = chi2 - psi2
      ckhso4 = chi3 - psi3
      ccaso4 = w(6)
      cmgso4 = zero
!
      call calcmr                                         ! water content
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    funck1 = molal(9)*molal(6)/a3 - one
  end function funck1
!
!======================================================================
!
! *** isorropia code
! *** subroutine isrp1r
! *** this subroutine is the driver routine for the reverse problem of
!     an ammonium-sulfate aerosol system.
!     the composition regime is determined by the sulfate ratio and by
!     the ambient relative humidity.
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine isrp1r(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: rhi , tempi
    real(rkx) , dimension(ncomp) :: wi
!
! *** initialize common block variables
! *********************************
    call init1(wi,rhi,tempi)
!
! *** calculate sulfate ratio
! *******************************************
    if ( rh>=drnh42s4 ) then           ! wet aerosol , need nh4 at sratio = 2.0
      sulratw = getasr(waer(2),rhi)         ! aerosol sulfate ratio
    else
      sulratw = 2.0_rkx                       ! dry aerosol sulfate ratio
    end if
    sulrat = waer(3)/waer(2)            ! sulfate ratio
!
! *** find calculation regime from (sulrat ,rh)
! **************************
! *** sulfate poor
!
    if ( sulratw<=sulrat ) then
!
      if ( metstbl==1 ) then
        scase = 's2'
        call calcs2                  ! only liquid (metastable)
      else if ( rh<drnh42s4 ) then
        scase = 's1'
        call calcs1                  ! nh42so4              ; case k1
      else if ( drnh42s4<=rh ) then
        scase = 's2'
        call calcs2                  ! only liquid          ; case k2
      end if
!
!   *** sulfate rich (no acid)
!
    else if ( 1.0<=sulrat .and. sulrat<sulratw ) then
      w(2) = waer(2)
      w(3) = waer(3)
!
      if ( metstbl==1 ) then
        scase = 'b4'
        call calcb4                  ! only liquid (metastable)
        scase = 'b4'
!
      else if ( rh<drnh4hs4 ) then
        scase = 'b1'
        call calcb1                  ! nh4hso4 ,lc ,nh42so4   ; case b1
        scase = 'b1'
!
      else if ( drnh4hs4<=rh .and. rh<drlc ) then
        scase = 'b2'
        call calcb2                  ! lc ,nh42s4            ; case b2
        scase = 'b2'
!
      else if ( drlc<=rh .and. rh<drnh42s4 ) then
        scase = 'b3'
        call calcb3                  ! nh42s4               ; case b3
        scase = 'b3'
!
      else if ( drnh42s4<=rh ) then
        scase = 'b4'
        call calcb4                  ! only liquid          ; case b4
        scase = 'b4'
      end if
!
      call calcnh3p          ! compute nh3(g)
!
!   *** sulfate rich (free acid)
!
    else if ( sulrat<1.0 ) then
      w(2) = waer(2)
      w(3) = waer(3)
!
      if ( metstbl==1 ) then
        scase = 'c2'
        call calcc2                  ! only liquid (metastable)
        scase = 'c2'
!
      else if ( rh<drnh4hs4 ) then
        scase = 'c1'
        call calcc1                  ! nh4hso4              ; case c1
        scase = 'c1'
!
      else if ( drnh4hs4<=rh ) then
        scase = 'c2'
        call calcc2                  ! only liquid          ; case c2
        scase = 'c2'
      end if
      call calcnh3p
    end if
  end subroutine isrp1r
!
!======================================================================
!
! *** isorropia code
! *** subroutine isrp2r
! *** this subroutine is the driver routine for the reverse problem of
!     an ammonium-sulfate-nitrate aerosol system.
!     the composition regime is determined by the sulfate ratio and by
!     the ambient relative humidity.
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine isrp2r(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: rhi , tempi
    real(rkx) , dimension(ncomp) :: wi
    logical :: tryliq
!
! *** initialize all variables in common block
! **************************
    tryliq = .true.               ! assume liquid phase , sulfate poor limit
!
 100  call init2(wi,rhi,tempi)
!
! *** calculate sulfate ratio
! *******************************************
    if ( tryliq .and. rh>=drnh4no3 ) then   ! *** wet aerosol
      sulratw = getasr(waer(2),rhi)         ! limiting sulfate ratio
    else
      sulratw = 2.0_rkx                       ! *** dry aerosol
    end if
    sulrat = waer(3)/waer(2)
!
! *** find calculation regime from (sulrat ,rh)
! **************************
! *** sulfate poor
!
    if ( sulratw<=sulrat ) then
!
      if ( metstbl==1 ) then
        scase = 'n3'
        call calcn3                  ! only liquid (metastable)
!
      else if ( rh<drnh4no3 ) then
        scase = 'n1'
        call calcn1                  ! nh42so4 ,nh4no3       ; case n1
!
      else if ( drnh4no3<=rh .and. rh<drnh42s4 ) then
        scase = 'n2'
        call calcn2                  ! nh42s4               ; case n2
!
      else if ( drnh42s4<=rh ) then
        scase = 'n3'
        call calcn3                  ! only liquid          ; case n3
      end if
!
!   *** sulfate rich (no acid)
!
!   for solving this case , nitric acid and ammonia in the gas phase are
!   assumed a minor species , that do not significantly affect the
!   aerosol equilibrium.
!
    else if ( 1.0<=sulrat .and. sulrat<sulratw ) then
      w(2) = waer(2)
      w(3) = waer(3)
      w(4) = waer(4)
!
      if ( metstbl==1 ) then
        scase = 'b4'
        call calcb4                  ! only liquid (metastable)
        scase = 'b4'
!
      else if ( rh<drnh4hs4 ) then
        scase = 'b1'
        call calcb1                  ! nh4hso4 ,lc ,nh42so4   ; case o1
        scase = 'b1'
!
      else if ( drnh4hs4<=rh .and. rh<drlc ) then
        scase = 'b2'
        call calcb2                  ! lc ,nh42s4            ; case o2
        scase = 'b2'
!
      else if ( drlc<=rh .and. rh<drnh42s4 ) then
        scase = 'b3'
        call calcb3                  ! nh42s4               ; case o3
        scase = 'b3'
!
      else if ( drnh42s4<=rh ) then
        scase = 'b4'
        call calcb4                  ! only liquid          ; case o4
        scase = 'b4'
      end if
!
!   *** add the no3 to the solution now and calculate partitioning.
!
      molal(7) = waer(4)             ! there is always water , so no3(aer) is no3-
      molal(1) = molal(1) + waer(4)  ! add h+ to balance out
      call calcnap            ! hno3 , nh3 dissolved
      call calcnh3p
!
!   *** sulfate rich (free acid)
!
!   for solving this case , nitric acid and ammonia in the gas phase are
!   assumed a minor species , that do not significantly affect the
!   aerosol equilibrium.
!
    else if ( sulrat<1.0 ) then
      w(2) = waer(2)
      w(3) = waer(3)
      w(4) = waer(4)
!
      if ( metstbl==1 ) then
        scase = 'c2'
        call calcc2                  ! only liquid (metastable)
        scase = 'c2'
!
      else if ( rh<drnh4hs4 ) then
        scase = 'c1'
        call calcc1                  ! nh4hso4              ; case p1
        scase = 'c1'
!
      else if ( drnh4hs4<=rh ) then
        scase = 'c2'
        call calcc2                  ! only liquid          ; case p2
        scase = 'c2'
      end if
!
!   *** add the no3 to the solution now and calculate partitioning.
!
      molal(7) = waer(4)             ! there is always water , so no3(aer) is no3-
      molal(1) = molal(1) + waer(4)  ! add h+ to balance out
!
      call calcnap                   ! hno3 , nh3 dissolved
      call calcnh3p
    end if
!
! *** if sulratw < sulrat < 2.0 and water = 0 => sulfate rich case.
!
    if ( sulratw<=sulrat .and. sulrat<2.0 .and. water<=tiny1 ) then
      tryliq = .false.
      go to 100
    end if
  end subroutine isrp2r
!
!======================================================================
!
! *** isorropia code
! *** subroutine isrp3r
! *** this subroutine is the driver routine for the reverse problem of
!     an ammonium-sulfate-nitrate-chloride-sodium aerosol system.
!     the composition regime is determined by the sulfate & sodium
!     ratios and by the ambient relative humidity.
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine isrp3r(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: frso4 , rhi , sri , tempi
    integer(ik4) :: i
    real(rkx) , dimension(ncomp) :: wi
    logical :: tryliq
! &
! ** adjust for too little ammonium and chloride ***********************
!
! wi(3) = max (wi(3), 1.d-10)  ! nh4+ : 1e-4 umoles/m3
! wi(5) = max (wi(5), 1.d-10)  ! cl-  : 1e-4 umoles/m3
!
! *** initialize all variables
! ******************************************
    tryliq = .true.               ! use liquid phase sulfate poor limit
!
 100  call isoinit3(wi,rhi,tempi)      ! common block variables
! &
! ** check if too much sodium ; adjust and issue error message *********
!
! rest = 2.d0*waer(2) + waer(4) + waer(5)
! if (waer(1) > rest) then            ! na > 2*so4+cl+no3 ?
! waer(1) = (one-1d-6)*rest         ! adjust na amount
! call pusherr (0050 , 'isrp3r')     ! warning error: na adjusted
! end if
!
! *** calculate sulfate & sodium ratios
! *********************************
    if ( tryliq .and. rh>=drnh4no3 ) then    ! ** wet aerosol
      frso4 = waer(2) - waer(1)/2.0_rkx          ! sulfate unbound by sodium
      frso4 = max(frso4,tiny1)
      sri = getasr(frso4,rhi)                   ! sulfate ratio for nh4+
      sulratw = (waer(1)+frso4*sri)/waer(2)    ! limiting sulfate ratio
      sulratw = min(sulratw,2.0_rkx)
    else
      sulratw = 2.0_rkx                        ! ** dry aerosol
    end if
    sulrat = (waer(1)+waer(3))/waer(2)
    sodrat = waer(1)/waer(2)
!
! *** find calculation regime from (sulrat ,rh)
! **************************
! *** sulfate poor ; sodium poor
!
    if ( sulratw<=sulrat .and. sodrat<2.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'q5'
        call calcq5                  ! only liquid (metastable)
        scase = 'q5'
!
      else if ( rh<drnh4no3 ) then
        scase = 'q1'
        call calcq1                  ! nh42so4 ,nh4no3 ,nh4cl ,na2so4
!
      else if ( drnh4no3<=rh .and. rh<drnh4cl ) then
        scase = 'q2'
        call calcq2                  ! nh42so4 ,nh4cl ,na2so4
!
      else if ( drnh4cl<=rh .and. rh<drnh42s4 ) then
        scase = 'q3'
        call calcq3                  ! nh42so4 ,na2so4
!
      else if ( drnh42s4<=rh .and. rh<drna2so4 ) then
        scase = 'q4'
        call calcq4                  ! na2so4
        scase = 'q4'
!
      else if ( drna2so4<=rh ) then
        scase = 'q5'
        call calcq5                  ! only liquid
        scase = 'q5'
      end if
!
!   *** sulfate poor ; sodium rich
!
    else if ( sulrat>=sulratw .and. sodrat>=2.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'r6'
        call calcr6                  ! only liquid (metastable)
        scase = 'r6'
!
      else if ( rh<drnh4no3 ) then
        scase = 'r1'
        call calcr1                  ! nh4no3 ,nh4cl ,na2so4 ,nacl ,nano3
!
      else if ( drnh4no3<=rh .and. rh<drnano3 ) then
        scase = 'r2'
        call calcr2                  ! nh4cl ,na2so4 ,nacl ,nano3
!
      else if ( drnano3<=rh .and. rh<drnacl ) then
        scase = 'r3'
        call calcr3                  ! nh4cl ,na2so4 ,nacl
!
      else if ( drnacl<=rh .and. rh<drnh4cl ) then
        scase = 'r4'
        call calcr4                  ! nh4cl ,na2so4
!
      else if ( drnh4cl<=rh .and. rh<drna2so4 ) then
        scase = 'r5'
        call calcr5                  ! na2so4
        scase = 'r5'
!
      else if ( drna2so4<=rh ) then
        scase = 'r6'
        call calcr6                  ! no solid
        scase = 'r6'
      end if
!
!   *** sulfate rich (no acid)
!
    else if ( 1.0<=sulrat .and. sulrat<sulratw ) then
      do i = 1 , ncomp
        w(i) = waer(i)
      end do
!
      if ( metstbl==1 ) then
        scase = 'i6'
        call calci6                  ! only liquid (metastable)
        scase = 'i6'
!
      else if ( rh<drnh4hs4 ) then
        scase = 'i1'
        call calci1                  ! na2so4 ,(nh4)2so4 ,nahso4 ,nh4hso4 ,lc
        scase = 'i1'
!
      else if ( drnh4hs4<=rh .and. rh<drnahso4 ) then
        scase = 'i2'
        call calci2                  ! na2so4 ,(nh4)2so4 ,nahso4 ,lc
        scase = 'i2'
!
      else if ( drnahso4<=rh .and. rh<drlc ) then
        scase = 'i3'
        call calci3                  ! na2so4 ,(nh4)2so4 ,lc
        scase = 'i3'
!
      else if ( drlc<=rh .and. rh<drnh42s4 ) then
        scase = 'i4'
        call calci4                  ! na2so4 ,(nh4)2so4
        scase = 'i4'
!
      else if ( drnh42s4<=rh .and. rh<drna2so4 ) then
        scase = 'i5'
        call calci5                  ! na2so4
        scase = 'i5'
!
      else if ( drna2so4<=rh ) then
        scase = 'i6'
        call calci6                  ! no solids
        scase = 'i6'
      end if
!
      call calcnhp                ! hno3 , nh3 , hcl in gas phase
      call calcnh3p
!
!   *** sulfate rich (free acid)
!
    else if ( sulrat<1.0 ) then
      do i = 1 , ncomp
        w(i) = waer(i)
      end do
!
      if ( metstbl==1 ) then
        scase = 'j3'
        call calcj3                  ! only liquid (metastable)
        scase = 'j3'
!
      else if ( rh<drnh4hs4 ) then
        scase = 'j1'
        call calcj1                  ! nh4hso4 ,nahso4
        scase = 'j1'
!
      else if ( drnh4hs4<=rh .and. rh<drnahso4 ) then
        scase = 'j2'
        call calcj2                  ! nahso4
        scase = 'j2'
!
      else if ( drnahso4<=rh ) then
        scase = 'j3'
        call calcj3
        scase = 'j3'
      end if
!
      call calcnhp                ! hno3 , nh3 , hcl in gas phase
      call calcnh3p
!
    end if
!
! *** if after calculations , sulratw < sulrat < 2.0
! and water = 0          => sulfate rich case.
!
    if ( sulratw<=sulrat .and. sulrat<2.0 .and. water<=tiny1 ) then
      tryliq = .false.
      go to 100
    end if
  end subroutine isrp3r
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine isrp4r
! *** this subroutine is the driver routine for the reverse problem of
!     an ammonium-sulfate-nitrate-chloride-sodium-calcium-pottasium-magnesium aerosol system.
!     the composition regime is determined by the sulfate & sodium
!     ratios and by the ambient relative humidity.
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine isrp4r(wi,rhi,tempi)
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: frso4 , rhi , sri , tempi
    integer(ik4) :: i
    real(rkx) , dimension(ncomp) :: wi
    logical :: tryliq
!
! ** adjust for too little ammonium and chloride ***********************
!
! wi(3) = max (wi(3), 1.d-10)  ! nh4+ : 1e-4 umoles/m3
! wi(5) = max (wi(5), 1.d-10)  ! cl-  : 1e-4 umoles/m3
!
! *** initialize all variables
! ******************************************
    tryliq = .true.                ! use liquid phase sulfate poor limit
    iprob = 1                ! solve reverse problem
! metstbl = 1
!
 100  call init4(wi,rhi,tempi)      ! common block variables
! &
! ** check if too much sodium ; adjust and issue error message *********
!
! rest = 2.d0*waer(2) + waer(4) + waer(5)
! if (waer(1) > rest) then            ! na > 2*so4+cl+no3 ?
! waer(1) = (one-1d-6)*rest         ! adjust na amount
! call pusherr (0050 , 'isrp3r')     ! warning error: na adjusted
! end if
!
! *** calculate sulfate , crustal & sodium ratios
! ***********************
    if ( tryliq ) then                               ! ** wet aerosol
      frso4 = waer(2) - waer(1)/2.0_rkx - waer(6) - waer(7)/2.0_rkx - waer(8)
                                                    ! sulfate unbound by sodium ,calcium ,pottasium ,magnesium
      frso4 = max(frso4,tiny1)
      sri = getasr(frso4,rhi)                         ! sulfate ratio for nh4+
      sulratw = (waer(1)+frso4*sri+waer(6)+waer(7)+waer(8))/waer(2)
                                                   ! limiting sulfate ratio
      sulratw = min(sulratw,2.0_rkx)
    else
      sulratw = 2.0_rkx                        ! ** dry aerosol
    end if
    so4rat = (waer(1)+waer(3)+waer(6)+waer(7)+waer(8))/waer(2)
    crnarat = (waer(1)+waer(6)+waer(7)+waer(8))/waer(2)
    crrat = (waer(6)+waer(7)+waer(8))/waer(2)
!
! *** find calculation regime from (sulrat ,rh)
! **************************
! *** sulfate poor ; sodium+crustals poor
!
    if ( sulratw<=so4rat .and. crnarat<2.0 ) then
!
      if ( metstbl==1 ) then
        scase = 'v7'
        call calcv7                  ! only liquid (metastable)
!
      else if ( rh<drnh4no3 ) then
        scase = 'v1'
        call calcv1                  ! caso4 , nh4no3 , nh4cl , (nh4)2so4 , mgso4 , na2so4 , k2so4
!
      else if ( drnh4no3<=rh .and. rh<drnh4cl ) then
        scase = 'v2'
        call calcv2                  ! caso4 , nh4cl , (nh4)2so4 , mgso4 , na2so4 , k2so4
!
      else if ( drnh4cl<=rh .and. rh<drnh42s4 ) then
        scase = 'v3'
        call calcv3                  ! caso4 , (nh4)2so4 , mgso4 , na2so4 , k2so4
!
      else if ( drnh42s4<=rh .and. rh<drmgso4 ) then
        scase = 'v4'
        call calcv4                  ! caso4 , mgso4 , na2so4 , k2so4
!
      else if ( drmgso4<=rh .and. rh<drna2so4 ) then
        scase = 'v5'
        call calcv5                  ! caso4 , na2so4 , k2so4
!
      else if ( drna2so4<=rh .and. rh<drk2so4 ) then
        scase = 'v6'
        call calcv6                  ! caso4 , k2so4
!
      else if ( drk2so4<=rh ) then
        scase = 'v7'
        call calcv7                  ! caso4
      end if
!
!   *** sulfate poor: rso4>2; (dust + sodium) rich: r(cr+na)>2; dust
!   poor: rcr<2.
    else if ( so4rat>=sulratw .and. crnarat>=2.0 ) then
!
      if ( crrat<=2.0 ) then
!
        if ( metstbl==1 ) then
          scase = 'u8'
          call calcu8                ! only liquid (metastable)
!
        else if ( rh<drnh4no3 ) then
          scase = 'u1'
          call calcu1                ! caso4 , nh4no3 , nh4cl , mgso4 , na2so4 , k2so4 , nacl , nano3
!
        else if ( drnh4no3<=rh .and. rh<drnano3 ) then
          scase = 'u2'
          call calcu2               ! caso4 , nh4cl , mgso4 , na2so4 , k2so4 , nacl , nano3
!
        else if ( drnano3<=rh .and. rh<drnacl ) then
          scase = 'u3'
          call calcu3               ! caso4 , nh4cl , mgso4 , na2so4 , k2so4 , nacl
!
        else if ( drnacl<=rh .and. rh<drnh4cl ) then
          scase = 'u4'
          call calcu4               ! caso4 , nh4cl , mgso4 , na2so4 , k2so4
!
        else if ( drnh4cl<=rh .and. rh<drmgso4 ) then
          scase = 'u5'
          call calcu5               ! caso4 , mgso4 , na2so4 , k2so4
!
        else if ( drmgso4<=rh .and. rh<drna2so4 ) then
          scase = 'u6'
          call calcu6               ! caso4 , na2so4 , k2so4
!
        else if ( drna2so4<=rh .and. rh<drk2so4 ) then
          scase = 'u7'
          call calcu7               ! caso4 , k2so4
!
        else if ( drk2so4<=rh ) then
          scase = 'u8'
          call calcu8               ! caso4
        end if
!
!     *** sulfate poor: rso4>2; (dust + sodium) rich: r(cr+na)>2; dust
!     poor: rcr<2.
      else if ( crrat>2.0 ) then
!
        if ( metstbl==1 ) then
          scase = 'w13'
          call calcw13                ! only liquid (metastable)
!
        else if ( rh<drcacl2 ) then
          scase = 'w1'
          call calcw1                ! caso4 , ca(no3)2 , cacl2 , k2so4 , kno3 , kcl , mgso4 ,
!       ! mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
        else if ( drcacl2<=rh .and. rh<drmgcl2 ) then
          scase = 'w2'
          call calcw2               ! caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 ,
!       ! mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
        else if ( drmgcl2<=rh .and. rh<drcano32 ) then
          scase = 'w3'
          call calcw3               ! caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 ,
!       ! mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl
!
        else if ( drcano32<=rh .and. rh<drmgno32 ) then
          scase = 'w4'
          call calcw4               ! caso4 , k2so4 , kno3 , kcl , mgso4 ,
!       ! mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl
!
        else if ( drmgno32<=rh .and. rh<drnh4no3 ) then
          scase = 'w5'
          call calcw5               ! caso4 , k2so4 , kno3 , kcl , mgso4 ,
!       ! nano3 , nacl , nh4no3 , nh4cl
!
        else if ( drnh4no3<=rh .and. rh<drnano3 ) then
          scase = 'w6'
          call calcw6               ! caso4 , k2so4 , kno3 , kcl , mgso4 , nano3 , nacl , nh4cl
!
        else if ( drnano3<=rh .and. rh<drnacl ) then
          scase = 'w7'
          call calcw7               ! caso4 , k2so4 , kno3 , kcl , mgso4 , nacl , nh4cl
!
        else if ( drnacl<=rh .and. rh<drnh4cl ) then
          scase = 'w8'
          call calcw8               ! caso4 , k2so4 , kno3 , kcl , mgso4 , nh4cl
!
        else if ( drnh4cl<=rh .and. rh<drkcl ) then
          scase = 'w9'
          call calcw9               ! caso4 , k2so4 , kno3 , kcl , mgso4
!
        else if ( drkcl<=rh .and. rh<drmgso4 ) then
          scase = 'w10'
          call calcw10               ! caso4 , k2so4 , kno3 , mgso4
!
        else if ( drmgso4<=rh .and. rh<drkno3 ) then
          scase = 'w11'
          call calcw11               ! caso4 , k2so4 , kno3
!
        else if ( drkno3<=rh .and. rh<drk2so4 ) then
          scase = 'w12'
          call calcw12               ! caso4 , k2so4
!
        else if ( drk2so4<=rh ) then
          scase = 'w13'
          call calcw13               ! caso4
        end if
!     call calcnh3
      end if
!
!   *** sulfate rich (no acid): 1<rso4<2;
!
    else if ( 1.0<=so4rat .and. so4rat<sulratw ) then
      do i = 1 , ncomp
        w(i) = waer(i)
      end do
!
      if ( metstbl==1 ) then
        scase = 'l9'
        call calcl9                   ! only liquid (metastable)
!
      else if ( rh<drnh4hs4 ) then
        scase = 'l1'
        call calcl1                ! caso4 ,k2so4 ,mgso4 ,khso4 ,na2so4 ,(nh4)2so4 ,nahso4 ,nh4hso4 ,lc
!
      else if ( drnh4hs4<=rh .and. rh<drnahso4 ) then
        scase = 'l2'
        call calcl2                ! caso4 ,k2so4 ,mgso4 ,khso4 ,na2so4 ,(nh4)2so4 ,nahso4 ,lc
!
      else if ( drnahso4<=rh .and. rh<drlc ) then
        scase = 'l3'
        call calcl3                ! caso4 ,k2so4 ,mgso4 ,khso4 ,na2so4 ,(nh4)2so4 ,lc
!
      else if ( drlc<=rh .and. rh<drnh42s4 ) then
        scase = 'l4'
        call calcl4                ! caso4 ,k2so4 ,mgso4 ,khso4 ,na2so4 ,(nh4)2so4
!
      else if ( drnh42s4<=rh .and. rh<drkhso4 ) then
        scase = 'l5'
        call calcl5                ! caso4 ,k2so4 ,mgso4 ,khso4 ,na2so4
!
      else if ( drkhso4<=rh .and. rh<drmgso4 ) then
        scase = 'l6'
        call calcl6                ! caso4 ,k2so4 ,mgso4 ,na2so4
!
      else if ( drmgso4<=rh .and. rh<drna2so4 ) then
        scase = 'l7'
        call calcl7                ! caso4 ,k2so4 ,na2so4
!
      else if ( drna2so4<=rh .and. rh<drk2so4 ) then
        scase = 'l8'
        call calcl8                ! caso4 ,k2so4
!
      else if ( drk2so4<=rh ) then
        scase = 'l9'
        call calcl9                ! caso4
      end if
!
      call calcnhp                  ! minor species: hno3 , hcl
      call calcnh3p                 !                nh3
!
!   *** sulfate super rich (free acid): rso4<1;
!
    else if ( so4rat<1.0 ) then
      do i = 1 , ncomp
        w(i) = waer(i)
      end do
!
      if ( metstbl==1 ) then
        scase = 'k4'
        call calck4                   ! only liquid (metastable)
!
      else if ( rh<drnh4hs4 ) then                  ! rh < 0.4
        scase = 'k1'
        call calck1               ! nh4hso4 ,nahso4 ,khso4 ,caso4
!
      else if ( drnh4hs4<=rh .and. rh<drnahso4 ) then
        scase = 'k2'
        call calck2               ! nahso4 ,khso4 ,caso4
!
      else if ( drnahso4<=rh .and. rh<drkhso4 ) then
        scase = 'k3'
        call calck3               ! khso4 ,caso4    0.52 < rh < 0.86
!
      else if ( drkhso4<=rh ) then
        scase = 'k4'
        call calck4               ! caso4
      end if
!
      call calcnhp                    ! minor species: hno3 , hcl
      call calcnh3p                   !                nh3
!
    end if
!
! *** if after calculations , sulratw < so4rat < 2.0
! and water = 0          => sulfate rich case.
!
    if ( sulratw<=so4rat .and. so4rat<2.0 .and. water<=tiny1 ) then
      tryliq = .false.
      go to 100
    end if
  end subroutine isrp4r
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcs2
! *** case s2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. liquid aerosol phase only possible
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcs2
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: a2 , akw , del , hi , hso4i , ohi , so4i
    integer(ik4) :: i
    real(rkx) nh4i , nh3gi , nh3aq
!
! *** setup parameters ************************************************
!
    calaou = .true.        ! outer loop activity calculation flag
    frst = .true.
    calain = .true.
!
! *** calculate water content *****************************************
!
    molalr(4) = min(waer(2),0.5_rkx*waer(3))
    water = molalr(4)/m0(4)       ! zsr correlation
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!   a21  = xk21*water*r*temp
      a2 = xk2*r*temp/xkw/rh*(gama(8)/gama(9))**2.
      akw = xkw*rh*water*water
!
      nh4i = waer(3)
      so4i = waer(2)
      hso4i = zero
!
      call calcph(2._rkx*so4i-nh4i,hi,ohi)              ! get ph
!
      nh3aq = zero                                  ! ammonia equilibrium
      if ( hi<ohi ) then
        call calcamaq(nh4i,ohi,del)
        nh4i = max(nh4i-del,zero)
        ohi = max(ohi-del,tiny1)
        nh3aq = del
        hi = akw/ohi
      end if
!
      call calchs4(hi,so4i,zero,del)                   ! sulfate equilibrium
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!
      nh3gi = nh4i/hi/a2      !    nh3aq/a21
!
!   *** speciation & water content
!   ***************************************
      molal(1) = hi
      molal(3) = nh4i
      molal(5) = so4i
      molal(6) = hso4i
      coh = ohi
      gasaq(1) = nh3aq
      gnh3 = nh3gi
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
  end subroutine calcs2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcs1
! *** case s1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4
!
!     a simple material balance is performed , and the solid (nh4)2so4
!     is calculated from the sulfates. the excess ammonia remains in
!     the gas phase.
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcs1
    use mod_che_common_isorropia
    implicit none
!
    cnh42s4 = min(waer(2),0.5_rkx*waer(3))    ! for bad input problems
    gnh3 = zero
!
    w(2) = cnh42s4
    w(3) = 2._rkx*cnh42s4 + gnh3
  end subroutine calcs1

!======================================================================
!
! *** isorropia code
! *** subroutine calcn3
! *** case n3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. there is only a liquid phase
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcn3
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , aml5 , del , gg , hi , hso4i , ohi , so4i
    integer(ik4) :: i
    real(rkx) :: nh4i , no3i , nh3aq , no3aq
!
! *** setup parameters ************************************************
!
    calaou = .true.               ! outer loop activity calculation flag
    frst = .true.
    calain = .true.
!
! *** aerosol water content
!
    molalr(4) = min(waer(2),0.5_rkx*waer(3))         ! (nh4)2so4
    aml5 = max(waer(3)-2._rkx*molalr(4),zero)        ! "free" nh4
    molalr(5) = max(min(aml5,waer(4)),zero)         ! nh4no3= min("free",no3)
    water = molalr(4)/m0(4) + molalr(5)/m0(5)
    water = max(water,tiny1)
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a2 = xk2*r*temp/xkw/rh*(gama(8)/gama(9))**2.
!   a21   = xk21*water*r*temp
      a3 = xk4*r*temp*(water/gama(10))**2
      a4 = xk7*(water/gama(4))**3
      akw = xkw*rh*water*water
!
!   ion concentrations
!
      nh4i = waer(3)
      no3i = waer(4)
      so4i = waer(2)
      hso4i = zero
!
      call calcph(2._rkx*so4i+no3i-nh4i,hi,ohi)
!
!   ammnia association equilibrium
!
      nh3aq = zero
      no3aq = zero
      gg = 2._rkx*so4i + no3i - nh4i
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
      else
        hi = zero
        call calcniaq2(gg,no3i,hi,no3aq)            ! hno3
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(3) = nh4i
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      coh = ohi
!
      cnh42s4 = zero
      cnh4no3 = zero
!
      gasaq(1) = nh3aq
      gasaq(3) = no3aq
!
      ghno3 = hi*no3i/a3
      gnh3 = nh4i/hi/a2           !   nh3aq/a21
!
!   *** calculate activities or terminate internal loop
!   ******************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
  end subroutine calcn3
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcn2
! *** case n2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. there is both a liquid & solid phase
!     3. solids possible : (nh4)2so4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcn2
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: dx , p4 , psi1hi , psi1lo , x1 , x2 , x3 , &
         y1 , y2 , y3 , yhi , ylo , yy
    integer(ik4) :: i
!
! *** setup parameters ************************************************
!
    chi1 = min(waer(2),0.5_rkx*waer(3))         ! (nh4)2so4
    chi2 = max(waer(3)-2._rkx*chi1,zero)         ! "free" nh4+
    chi3 = max(waer(4)-chi2,zero)              ! "free" no3
!
    psi2 = chi2
    psi3 = chi3
!
    calaou = .true.                ! outer loop activity calculation flag
    psi1lo = tiny1                  ! low  limit
    psi1hi = chi1                  ! high limit
!
! *** initial values for bisection ************************************
!
    x1 = psi1hi
    y1 = funcn2(x1)
    if ( y1<=eps ) return     ! if (abs(y1) <= eps .or. y1 <= zero) return
    yhi = y1                   ! save y-value at hi position
    y2 = y1
!
! *** root tracking ; for the range of hi and lo **********************
!
    dx = (psi1hi-psi1lo)/float(ndiv)
    do i = 1 , ndiv
      x2 = max(x1-dx,zero)
      y2 = funcn2(x2)
      if ( sign(1._rkx,y1)*sign(1._rkx,y2)<zero ) go to 100     ! (y1*y2 < zero)
      x1 = x2
      y1 = y2
    end do
!
! *** no subdivision with solution found
!
    ylo = y1                        ! save y-value at hi position
    if ( abs(y2)<eps ) then       ! x2 is a solution
      return
!
!   *** { ylo , yhi } < 0.0 the solution is always undersaturated with
!   nh3
    else if ( ylo<zero .and. yhi<zero ) then
      p4 = chi4
      yy = funcn2(p4)
      return
!
!   *** { ylo , yhi } > 0.0 the solution is always supersaturated with
!   nh3
    else if ( ylo>zero .and. yhi>zero ) then
      p4 = tiny1
      yy = funcn2(p4)
      return
    else
      call pusherr(0001,'calcn2')          ! warning error: no solution
      return
    end if
!
! *** perform bisection ***********************************************
!
 100  do i = 1 , maxit
      x3 = 0.5_rkx*(x1+x2)
      y3 = funcn2(x3)
      if ( sign(1._rkx,y1)*sign(1._rkx,y3)<=zero ) then         ! (y1*y3  <=  zero)
        y2 = y3
        x2 = x3
      else
        y1 = y3
        x1 = x3
      end if
      if ( abs(x2-x1)<=eps*x1 ) go to 200
    end do
    call pusherr(0002,'calcn2')         ! warning error: no convergence
!
! *** converged ; return **********************************************
!
 200  x3 = 0.5_rkx*(x1+x2)
    y3 = funcn2(x3)
  end subroutine calcn2
!
!=====================================================================
!
! *** isorropia code
! *** function funcn2
! *** case d2
!     function that solves the system of equations for case d2 ;
!     and returns the value of the zeroed function in funcn2.
!
!======================================================================
!
  real(rkx) function funcn2(p1)
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , del , gg , hi , hso4i , ohi , p1 , so4i
    integer(ik4) :: i
    real(rkx) :: nh4i , no3i , nh3aq , no3aq
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    psi1 = p1
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a2 = xk2*r*temp/xkw/rh*(gama(8)/gama(9))**2.
!   a21   = xk21*water*r*temp
      a3 = xk4*r*temp*(water/gama(10))**2
      a4 = xk7*(water/gama(4))**3
      akw = xkw*rh*water*water
!
!   ion concentrations
!
      nh4i = 2._rkx*psi1 + psi2
      no3i = psi2 + psi3
      so4i = psi1
      hso4i = zero
!
      call calcph(2._rkx*so4i+no3i-nh4i,hi,ohi)
!
!   ammnia association equilibrium
!
      nh3aq = zero
      no3aq = zero
      gg = 2._rkx*so4i + no3i - nh4i
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
      else
        hi = zero
        call calcniaq2(gg,no3i,hi,no3aq)            ! hno3
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(3) = nh4i
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      coh = ohi
!
      cnh42s4 = chi1 - psi1
      cnh4no3 = zero
!
      gasaq(1) = nh3aq
      gasaq(3) = no3aq
!
      ghno3 = hi*no3i/a3
      gnh3 = nh4i/hi/a2           !   nh3aq/a21
!
!   *** calculate molalr array , water and activities
!   **********************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
!
! *** calculate objective function ************************************
!
    funcn2 = nh4i*nh4i*so4i/a4 - one
  end function funcn2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcn1
! *** case n1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4 , nh4no3
!
!     there are two regimes defined by relative humidity:
!     1. rh < mdrh  ; only solid phase possible (subroutine calcn1a)
!     2. rh >= mdrh ; liquid phase possible (mdrh region)
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcn1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmasan ) then
      scase = 'n1 ; subcase 1'
      call calcn1a                 ! solid phase only possible
      scase = 'n1 ; subcase 1'
    else
      scase = 'n1 ; subcase 2'
      call calcmdrp(rh,drmasan,drnh4no3,calcn1a,calcn2)
      scase = 'n1 ; subcase 2'
    end if
  end subroutine calcn1
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcn1a
! *** case n1 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4 , nh4no3
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcn1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: psi1 , psi2
!
! *** setup parameters *************************************************
!
! a1      = xk10/r/temp/r/temp
!
! *** calculate aerosol composition ************************************
!
! chi1    = 2.d0*waer(4)        ! free parameter ; arbitrary value.
    psi1 = waer(4)
!
! *** the following statment is here to avoid negative nh4+ values in
! calcn? routines that call calcn1a
!
    psi2 = max(min(waer(2),0.5_rkx*(waer(3)-psi1)),tiny1)
!
    cnh4no3 = psi1
    cnh42s4 = psi2
!
! gnh3    = chi1 + psi1 + 2.0*psi2
! ghno3   = a1/(chi1-psi1) + psi1
    gnh3 = zero
    ghno3 = zero
!
    w(2) = psi2
    w(3) = gnh3 + psi1 + 2.0*psi2
    w(4) = ghno3 + psi1
  end subroutine calcn1a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcq5
! *** case q5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); sodium poor (sodrat < 2.0)
!     2. liquid and solid phases are possible
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcq5
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , ggcl , ggno3 , &
         hi , hso4i , ohi , so4i
    integer(ik4) :: i
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
! *** calculate initial solution ***************************************
!
    call calcq1a
!
    psi1 = cna2so4          ! salts dissolved
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi6 = cnh42s4
!
    call calcmr             ! water
!
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      akw = xkw*rh*water*water                  ! h2o       <==> h+
!
!   ion concentrations
!
      nai = waer(1)
      so4i = waer(2)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i
      if ( gg>tiny1 ) then                           ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                           ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
        hso4i = zero
      else
        ggno3 = max(2._rkx*so4i+no3i-nai-nh4i,zero)
        ggcl = max(gg-ggno3,zero)
        if ( ggcl>tiny1 ) call calcclaq2(ggcl,cli,hi,claq)             ! hcl
        if ( ggno3>tiny1 ) then
          if ( ggcl<=tiny1 ) hi = zero
          call calcniaq2(ggno3,no3i,hi,no3aq)                          ! hno3
        end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
! call pusherr (0002 , 'calcq5')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cna2so4 = zero
  end subroutine calcq5
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcq4
! *** case q4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); sodium poor (sodrat < 2.0)
!     2. liquid and solid phases are possible
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcq4
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , ggcl , ggno3 , &
         hi , hso4i , ohi , psi1o , root3 , so4i
    integer(ik4) :: i , islv
    logical :: psconv1
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv1 = .true.
    psi1o = -great
    root3 = zero
!
! *** calculate initial solution ***************************************
!
    call calcq1a
!
    chi1 = cna2so4          ! salts
!
    psi1 = cna2so4          ! amount dissolved
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi6 = cnh42s4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = waer(2)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a5 = xk5*(water/gama(2))**3             ! na2so4    <==> na+
      akw = xkw*rh*water*water                  ! h2o       <==> h+
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a5 ) then
        bb = -(waer(2)+waer(1))
        cc = waer(1)*waer(2) + 0.25_rkx*waer(1)*waer(1)
        dd = -0.25_rkx*(waer(1)*waer(1)*waer(2)-a5)
        call poly3(bb,cc,dd,root3,islv)
        if ( islv/=0 ) root3 = tiny1
        root3 = min(root3,waer(1)/2.0,waer(2),chi1)
        root3 = max(root3,zero)
        psi1 = chi1 - root3
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ion concentrations ; corrections
!
      nai = waer(1) - 2._rkx*root3
      so4i = waer(2) - root3
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i
      if ( gg>tiny1 ) then                           ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                           ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
        hso4i = zero
      else
        ggno3 = max(2._rkx*so4i+no3i-nai-nh4i,zero)
        ggcl = max(gg-ggno3,zero)
        if ( ggcl>tiny1 ) call calcclaq2(ggcl,cli,hi,claq)             ! hcl
        if ( ggno3>tiny1 ) then
          if ( ggcl<=tiny1 ) hi = zero
          call calcniaq2(ggno3,no3i,hi,no3aq)                          ! hno3
        end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv1 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcq4')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cna2so4 = chi1 - psi1
  end subroutine calcq4
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcq3
! *** case q3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : nh4cl , na2so4 , nano3 , nacl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcq3
    use mod_che_common_isorropia
    implicit none
    logical :: exno , excl
!
! *** regime depends on ambient relative humidity & possible species ***
!
    exno = waer(4)>tiny1
    excl = waer(5)>tiny1
!
    if ( exno .or. excl ) then             ! *** nitrate or chloride exists
      scase = 'q3 ; subcase 1'
      call calcq3a
      scase = 'q3 ; subcase 1'
!
    else if ( rh<drmg3 ) then
      scase = 'q3 ; subcase 2'
      call calcq1a                   ! solid
      scase = 'q3 ; subcase 2'
    else
      scase = 'q3 ; subcase 3'       ! mdrh (nh4)2so4 , na2so4
      call calcmdrp(rh,drmg3,drnh42s4,calcq1a,calcq4)
      scase = 'q3 ; subcase 3'
    end if
  end subroutine calcq3
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcq3a
! *** case q3 ; subcase a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); sodium poor (sodrat < 2.0)
!     2. liquid and solid phases are possible
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcq3a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , ggcl , ggno3 , &
         hi , hso4i , ohi , psi1o , psi6o , root1 , root3 , so4i
    integer(ik4) :: i , islv
    logical :: psconv1 , psconv6
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv1 = .true.
    psconv6 = .true.
!
    psi1o = -great
    psi6o = -great
!
    root1 = zero
    root3 = zero
!
! *** calculate initial solution ***************************************
!
    call calcq1a
!
    chi1 = cna2so4          ! salts
    chi4 = cnh4cl
    chi6 = cnh42s4
!
    psi1 = cna2so4          ! amount dissolved
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi6 = cnh42s4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = waer(2)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a5 = xk5*(water/gama(2))**3          ! na2so4    <==> na+
      a7 = xk7*(water/gama(4))**3          ! (nh4)2so4 <==> nh4+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a5 ) then
        bb = -(waer(2)+waer(1)-root1)
        cc = waer(1)*(waer(2)-root1) + 0.25_rkx*waer(1)*waer(1)
        dd = -0.25_rkx*(waer(1)*waer(1)*(waer(2)-root1)-a5)
        call poly3(bb,cc,dd,root3,islv)
        if ( islv/=0 ) root3 = tiny1
        root3 = min(root3,waer(1)/2.0,waer(2)-root1,chi1)
        root3 = max(root3,zero)
        psi1 = chi1 - root3
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ammnium sulfate
!
      if ( nh4i*nh4i*so4i>a7 ) then
        bb = -(waer(2)+waer(3)-root3)
        cc = waer(3)*(waer(2)-root3+0.5_rkx*waer(3))
        dd = -((waer(2)-root3)*waer(3)**20+a7)/4._rkx
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = min(root1,waer(3),waer(2)-root3,chi6)
        root1 = max(root1,zero)
        psi6 = chi6 - root1
      end if
      psconv6 = abs(psi6-psi6o)<=eps*psi6o
      psi6o = psi6
!
!   ion concentrations
!
      nai = waer(1) - 2._rkx*root3
      so4i = waer(2) - root1 - root3
      nh4i = waer(3) - 2._rkx*root1
      no3i = waer(4)
      cli = waer(5)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
        hso4i = zero
      else
        ggno3 = max(2._rkx*so4i+no3i-nai-nh4i,zero)
        ggcl = max(gg-ggno3,zero)
        if ( ggcl>tiny1 ) call calcclaq2(ggcl,cli,hi,claq)          ! hcl
        if ( ggno3>tiny1 ) then
          if ( ggcl<=tiny1 ) hi = zero
          call calcniaq2(ggno3,no3i,hi,no3aq)                       ! hno3
        end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv1 .and. psconv6 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcq3a')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = chi6 - psi6
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cna2so4 = chi1 - psi1
  end subroutine calcq3a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcq2
! *** case q2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. solid & liquid aerosol possible
!     3. solids possible : nh4cl , na2so4 , nano3 , nacl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcq2
    use mod_che_common_isorropia
    implicit none
    logical :: exno , excl
!
! *** regime depends on ambient relative humidity & possible species ***
!
    exno = waer(4)>tiny1
    excl = waer(5)>tiny1
!
    if ( exno ) then                       ! *** nitrate exists
      scase = 'q2 ; subcase 1'
      call calcq2a
      scase = 'q2 ; subcase 1'
!
    else if ( .not.exno .and. excl ) then   ! *** only chloride exists
      if ( rh<drmg2 ) then
        scase = 'q2 ; subcase 2'
        call calcq1a                 ! solid
        scase = 'q2 ; subcase 2'
      else
        scase = 'q2 ; subcase 3'     ! mdrh (nh4)2so4 , na2so4 , nh4cl
        call calcmdrp(rh,drmg2,drnh4cl,calcq1a,calcq3a)
        scase = 'q2 ; subcase 3'
      end if
!
    else if ( rh<drmg3 ) then
      scase = 'q2 ; subcase 2'
      call calcq1a                   ! solid
      scase = 'q2 ; subcase 2'
    else
      scase = 'q2 ; subcase 4'       ! mdrh (nh4)2so4 , na2so4
      call calcmdrp(rh,drmg3,drnh42s4,calcq1a,calcq4)
      scase = 'q2 ; subcase 4'
    end if
  end subroutine calcq2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcq2a
! *** case q2 ; subcase a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); sodium poor (sodrat < 2.0)
!     2. liquid and solid phases are possible
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcq2a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , ggcl , ggno3 , &
         hi , hso4i , ohi , psi1o , psi4o , psi6o , root1 , root2 ,   &
         root2a , root2b , root3
    real(rkx) :: so4i
    integer(ik4) :: i , islv
    logical :: psconv1 , psconv4 , psconv6
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv1 = .true.
    psconv4 = .true.
    psconv6 = .true.
!
    psi1o = -great
    psi4o = -great
    psi6o = -great
!
    root1 = zero
    root2 = zero
    root3 = zero
!
! *** calculate initial solution ***************************************
!
    call calcq1a
!
    chi1 = cna2so4          ! salts
    chi4 = cnh4cl
    chi6 = cnh42s4
!
    psi1 = cna2so4          ! amount dissolved
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi6 = cnh42s4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = waer(2)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a5 = xk5*(water/gama(2))**3          ! na2so4    <==> na+
      a14 = xk14*(water/gama(6))**2        ! nh4cl     <==> nh4+
      a7 = xk7*(water/gama(4))**3          ! (nh4)2so4 <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   ammnium chloride
!
      if ( nh4i*cli>a14 ) then
        bb = -(waer(3)+waer(5)-2._rkx*root1)
        cc = waer(5)*(waer(3)-2._rkx*root1) - a14
        dd = bb*bb - 4._rkx*cc
        if ( dd<zero ) then
          root2 = zero
        else
          dd = sqrt(dd)
          root2a = 0.5_rkx*(-bb+dd)
          root2b = 0.5_rkx*(-bb-dd)
          if ( zero<=root2a ) then
            root2 = root2a
          else
            root2 = root2b
          end if
          root2 = min(root2,waer(5),waer(3)-2._rkx*root1,chi4)
          root2 = max(root2,zero)
          psi4 = chi4 - root2
        end if
      end if
      psconv4 = abs(psi4-psi4o)<=eps*psi4o
      psi4o = psi4
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a5 ) then
        bb = -(waer(2)+waer(1)-root1)
        cc = waer(1)*(waer(2)-root1) + 0.25_rkx*waer(1)*waer(1)
        dd = -0.25_rkx*(waer(1)*waer(1)*(waer(2)-root1)-a5)
        call poly3(bb,cc,dd,root3,islv)
        if ( islv/=0 ) root3 = tiny1
        root3 = min(root3,waer(1)/2.0,waer(2)-root1,chi1)
        root3 = max(root3,zero)
        psi1 = chi1 - root3
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ammnium sulfate
!
      if ( nh4i*nh4i*so4i>a7 ) then
        bb = -(waer(2)+waer(3)-root2-root3)
        cc = (waer(3)-root2)*(waer(2)-root3+0.5_rkx*(waer(3)-root2))
        dd = -((waer(2)-root3)*(waer(3)-root2)**20+a7)/4._rkx
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = min(root1,waer(3)-root2,waer(2)-root3,chi6)
        root1 = max(root1,zero)
        psi6 = chi6 - root1
      end if
      psconv6 = abs(psi6-psi6o)<=eps*psi6o
      psi6o = psi6
!
!   ion concentrations
!
      nai = waer(1) - 2._rkx*root3
      so4i = waer(2) - root1 - root3
      nh4i = waer(3) - root2 - 2._rkx*root1
      no3i = waer(4)
      cli = waer(5) - root2
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
        hso4i = zero
      else
        ggno3 = max(2._rkx*so4i+no3i-nai-nh4i,zero)
        ggcl = max(gg-ggno3,zero)
        if ( ggcl>tiny1 ) call calcclaq2(ggcl,cli,hi,claq)          ! hcl
        if ( ggno3>tiny1 ) then
          if ( ggcl<=tiny1 ) hi = zero
          call calcniaq2(ggno3,no3i,hi,no3aq)                       ! hno3
        end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv1 .and. psconv4 .and. psconv6 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcq2a')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = chi6 - psi6
    cnh4no3 = zero
    cnh4cl = chi4 - psi4
    cnacl = zero
    cnano3 = zero
    cna2so4 = chi1 - psi1
  end subroutine calcq2a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcq1
! *** case q1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4no3 , nh4cl , (nh4)2so4 , na2so4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcq1
    use mod_che_common_isorropia
    implicit none
    logical :: exno , excl
!
! *** regime depends on ambient relative humidity & possible species ***
!
    exno = waer(4)>tiny1
    excl = waer(5)>tiny1
!
    if ( exno .and. excl ) then           ! *** nitrate & chloride exist
      if ( rh<drmg1 ) then
        scase = 'q1 ; subcase 1'
        call calcq1a                 ! solid
        scase = 'q1 ; subcase 1'
      else
        scase = 'q1 ; subcase 2'     ! mdrh (nh4)2so4 , na2so4 , nh4cl , nh4no3
        call calcmdrp(rh,drmg1,drnh4no3,calcq1a,calcq2a)
        scase = 'q1 ; subcase 2'
      end if
!
    else if ( exno .and. .not.excl ) then ! *** only nitrate exists
      if ( rh<drmq1 ) then
        scase = 'q1 ; subcase 1'
        call calcq1a                 ! solid
        scase = 'q1 ; subcase 1'
      else
        scase = 'q1 ; subcase 3'     ! mdrh (nh4)2so4 , na2so4 , nh4no3
        call calcmdrp(rh,drmq1,drnh4no3,calcq1a,calcq2a)
        scase = 'q1 ; subcase 3'
      end if
!
    else if ( .not.exno .and. excl ) then ! *** only chloride exists
      if ( rh<drmg2 ) then
        scase = 'q1 ; subcase 1'
        call calcq1a                 ! solid
        scase = 'q1 ; subcase 1'
      else
        scase = 'q1 ; subcase 4'     ! mdrh (nh4)2so4 , na2so4 , nh4cl
        call calcmdrp(rh,drmg2,drnh4cl,calcq1a,calcq3a)
        scase = 'q1 ; subcase 4'
      end if
!
    else if ( rh<drmg3 ) then
      scase = 'q1 ; subcase 1'
      call calcq1a                   ! solid
      scase = 'q1 ; subcase 1'
    else
      scase = 'q1 ; subcase 5'       ! mdrh (nh4)2so4 , na2so4
      call calcmdrp(rh,drmg3,drnh42s4,calcq1a,calcq4)
      scase = 'q1 ; subcase 5'
    end if
  end subroutine calcq1
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcq1a
! *** case q1 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium poor (sodrat < 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4no3 , nh4cl , (nh4)2so4 , na2so4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcq1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: frnh3 , frso4
!
! *** calculate solids
! **************************************************
    cna2so4 = 0.5_rkx*waer(1)
    frso4 = max(waer(2)-cna2so4,zero)
!
    cnh42s4 = max(min(frso4,0.5_rkx*waer(3)),tiny1)
    frnh3 = max(waer(3)-2._rkx*cnh42s4,zero)
!
    cnh4no3 = min(frnh3,waer(4))
! frno3   = max (waer(4)-cnh4no3 , zero)
    frnh3 = max(frnh3-cnh4no3,zero)
!
    cnh4cl = min(frnh3,waer(5))
! frcl    = max (waer(5)-cnh4cl , zero)
    frnh3 = max(frnh3-cnh4cl,zero)
!
! *** other phases
! ******************************************************
    water = zero
!
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
  end subroutine calcq1a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcr6
! *** case r6
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); sodium rich (sodrat >= 2.0)
!     2. there is only a liquid phase
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcr6
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , ggcl , ggno3 , &
         hi , hso4i , ohi , so4i
    integer(ik4) :: i
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq
!
! *** setup parameters ************************************************
!
    call calcr1a
!
    psi1 = cna2so4
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
! *** calculate water **************************************************
!
    call calcmr
!
! *** setup liquid concentrations **************************************
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      akw = xkw*rh*water*water                        ! h2o    <==> h+
!
      nai = waer(1)
      so4i = waer(2)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*waer(2) + no3i + cli - nai - nh4i
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
      else
        ggno3 = max(2._rkx*so4i+no3i-nai-nh4i,zero)
        ggcl = max(gg-ggno3,zero)
        if ( ggcl>tiny1 ) call calcclaq2(ggcl,cli,hi,claq)          ! hcl
        if ( ggno3>tiny1 ) then
          if ( ggcl<=tiny1 ) hi = zero
          call calcniaq2(ggno3,no3i,hi,no3aq)                       ! hno3
        end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
! call pusherr (0002 , 'calcr6')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2        ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2                ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2                ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cna2so4 = zero
  end subroutine calcr6
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcr5
! *** case r5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); sodium rich (sodrat >= 2.0)
!     2. liquid and solid phases are possible
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcr5
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , ggcl , ggno3 , &
         hi , hso4i , ohi , psio , root , so4i
    integer(ik4) :: i , islv
    logical :: psconv
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq
    logical :: nean , neac , nesn , nesc
!
! *** setup parameters ************************************************
!
    call calcr1a                               ! dry solution
!
    nean = cnh4no3<=tiny1        ! nh4no3       ! water exists?
    neac = cnh4cl<=tiny1         ! nh4cl
    nesn = cnano3<=tiny1         ! nano3
    nesc = cnacl<=tiny1          ! nacl
    if ( nean .and. neac .and. nesn .and. nesc ) return
!
    chi1 = cna2so4
!
    psi1 = cna2so4
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
!
    psio = -great
!
! *** calculate water **************************************************
!
    call calcmr
!
    frst = .true.
    calain = .true.
    calaou = .true.
    psconv = .false.
!
! *** setup liquid concentrations **************************************
!
    nai = waer(1)
    so4i = waer(2)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a5 = xk5*(water/gama(2))**3                   ! na2so4 <==> na+
      akw = xkw*rh*water*water                        ! h2o    <==> h+
!
!   sodum sulfate
!
      root = zero
      if ( nai*nai*so4i>a5 ) then
        bb = -3._rkx*chi1
        cc = 3._rkx*chi1**2
        dd = -chi1**3 + 0.25_rkx*a5
        call poly3(bb,cc,dd,root,islv)
        if ( islv/=0 ) root = tiny1
        root = min(max(root,zero),chi1)
        psi1 = chi1 - root
      end if
      psconv = abs(psi1-psio)<=eps*psio
      psio = psi1
!
!   ion concentrations
!
      nai = waer(1) - 2._rkx*root
      so4i = waer(2) - root
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
      else
        ggno3 = max(2._rkx*so4i+no3i-nai-nh4i,zero)
        ggcl = max(gg-ggno3,zero)
        if ( ggcl>tiny1 ) call calcclaq2(ggcl,cli,hi,claq)          ! hcl
        if ( ggno3>tiny1 ) then
          if ( ggcl<=tiny1 ) hi = zero
          call calcniaq2(ggno3,no3i,hi,no3aq)                       ! hno3
        end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcr5')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2        ! nh3  <==> nh4+
! a21      = xk21*water*r*temp
    a3 = xk4*r*temp*(water/gama(10))**2                ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2                ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2        ! nh4i*ohi/a2/akw
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cna2so4 = chi1 - psi1
  end subroutine calcr5
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcr4
! *** case r4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4no3 , nh4cl , na2so4 , nano3 , nacl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcr4
    use mod_che_common_isorropia
    implicit none
    logical :: exan , exac , exsn , exsc
!
! *** solve for dry case and see which solids are possible
! **************
    scase = 'r4 ; subcase 2'
    call calcr1a                ! solid
    scase = 'r4 ; subcase 2'
!
    exan = cnh4no3>tiny1        ! nh4no3
    exac = cnh4cl>tiny1         ! nh4cl
    exsn = cnano3>tiny1         ! nano3
    exsc = cnacl>tiny1          ! nacl
!
! *** regime depends on relative humidity and possible species
! **********
    if ( exan .or. exsn .or. exsc ) then   ! *** nh4no3 ,nano3 exist
      if ( rh>=drmh1 ) then
        scase = 'r4 ; subcase 1'
        call calcr4a
        scase = 'r4 ; subcase 1'
      end if
!
    else if ( exac ) then                  ! *** nh4cl exists only
      if ( rh>=drmr5 ) then
        scase = 'r4 ; subcase 3'
        call calcmdrp(rh,drmr5,drnh4cl,calcr1a,calcr5)
        scase = 'r4 ; subcase 3'
      end if
    end if
  end subroutine calcr4
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcr4a
! *** case r4a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); sodium rich (sodrat >= 2.0)
!     2. liquid and solid phases are possible
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcr4a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , ggcl , ggno3 , &
         hi , hso4i , ohi , psio1 , psio4 , root , so4i
    integer(ik4) :: i , islv
    logical :: psconv1 , psconv4
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
    psconv1 = .false.
    psconv4 = .false.
    psio1 = -great
    psio4 = -great
!
! *** calculate initial solution ***************************************
!
    call calcr1a
!
    chi1 = cna2so4          ! salts
    chi4 = cnh4cl
!
    psi1 = cna2so4
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = waer(2)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a5 = xk5*(water/gama(2))**3                   ! na2so4 <==> na+
      a14 = xk14*(water/gama(6))**2                 ! nh4cl  <==> nh4+
      akw = xkw*rh*water*water                        ! h2o    <==> h+
!
!   sodum sulfate
!
      root = zero
      if ( nai*nai*so4i>a5 ) then
        bb = -3._rkx*chi1
        cc = 3._rkx*chi1**2
        dd = -chi1**3 + 0.25_rkx*a5
        call poly3(bb,cc,dd,root,islv)
        if ( islv/=0 ) root = tiny1
        root = min(max(root,zero),chi1)
        psi1 = chi1 - root
        nai = waer(1) - 2._rkx*root
        so4i = waer(2) - root
      end if
      psconv1 = abs(psi1-psio1)<=eps*psio1
      psio1 = psi1
!
!   ammnium chloride
!
      root = zero
      if ( nh4i*cli>a14 ) then
        bb = -(nh4i+cli)
        cc = -a14 + nh4i*cli
        dd = bb*bb - 4._rkx*cc
        root = 0.5_rkx*(-bb-sqrt(dd))
        if ( root>tiny1 ) then
          root = min(root,chi4)
          psi4 = chi4 - root
          nh4i = waer(3) - root
          cli = waer(5) - root
        end if
      end if
      psconv4 = abs(psi4-psio4)<=eps*psio4
      psio4 = psi4
!
      no3i = waer(4)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
      else
        ggno3 = max(2._rkx*so4i+no3i-nai-nh4i,zero)
        ggcl = max(gg-ggno3,zero)
        if ( ggcl>tiny1 ) call calcclaq2(ggcl,cli,hi,claq)          ! hcl
        if ( ggno3>tiny1 ) then
          if ( ggcl<=tiny1 ) hi = zero
          call calcniaq2(ggno3,no3i,hi,no3aq)                       ! hno3
        end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv1 .and. psconv4 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcr4a')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = chi4 - psi4
    cnacl = zero
    cnano3 = zero
    cna2so4 = chi1 - psi1
  end subroutine calcr4a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcr3
! *** case r3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4no3 , nh4cl , na2so4 , nano3 , nacl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcr3
    use mod_che_common_isorropia
    implicit none
    logical :: exan , exac , exsn , exsc
!
! *** solve for dry case and see which solids are possible
! **************
    scase = 'r3 ; subcase 2'
    call calcr1a                ! solid
    scase = 'r3 ; subcase 2'
!
    exan = cnh4no3>tiny1        ! nh4no3
    exac = cnh4cl>tiny1         ! nh4cl
    exsn = cnano3>tiny1         ! nano3
    exsc = cnacl>tiny1          ! nacl
!
! *** regime depends on relative humidity and possible species
! **********
    if ( exan .or. exsn ) then                   ! *** nh4no3 ,nano3 exist
      if ( rh>=drmh1 ) then
        scase = 'r3 ; subcase 1'
        call calcr3a
        scase = 'r3 ; subcase 1'
      end if
!
    else if ( .not.exan .and. .not.exsn ) then   ! *** nh4no3 ,nano3 = 0
      if ( exac .and. exsc ) then
        if ( rh>=drmr4 ) then
          scase = 'r3 ; subcase 3'
          call calcmdrp(rh,drmr4,drnacl,calcr1a,calcr4a)
          scase = 'r3 ; subcase 3'
        end if

      else if ( .not.exac .and. exsc ) then
        if ( rh>=drmr2 ) then
          scase = 'r3 ; subcase 4'
          call calcmdrp(rh,drmr2,drnacl,calcr1a,calcr4a)
          scase = 'r3 ; subcase 4'
        end if

      else if ( exac .and. .not.exsc ) then
        if ( rh>=drmr5 ) then
          scase = 'r3 ; subcase 5'
          call calcmdrp(rh,drmr5,drnacl,calcr1a,calcr5)
          scase = 'r3 ; subcase 5'
        end if
      end if
    end if
  end subroutine calcr3
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcr3a
! *** case r3a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); sodium rich (sodrat >= 2.0)
!     2. liquid and solid phases are possible
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcr3a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , ggcl , ggno3 , &
         hi , hso4i , ohi , psi1o , psi3o , psi4o , root1 , root2 ,   &
         root2a , root2b , root3
    real(rkx) :: root3a , root3b , so4i
    integer(ik4) :: i , islv
    logical :: psconv1 , psconv3 , psconv4
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
    psconv1 = .true.
    psconv3 = .true.
    psconv4 = .true.
    psi1o = -great
    psi3o = -great
    psi4o = -great
    root1 = zero
    root2 = zero
    root3 = zero
!
! *** calculate initial solution ***************************************
!
    call calcr1a
!
    chi1 = cna2so4          ! salts
    chi4 = cnh4cl
    chi3 = cnacl
!
    psi1 = cna2so4
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = waer(2)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
    molal(1) = zero
    molal(2) = nai
    molal(3) = nh4i
    molal(4) = cli
    molal(5) = so4i
    molal(6) = hso4i
    molal(7) = no3i
!
    call calcact            ! calculate activity coefficients
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a5 = xk5*(water/gama(2))**3                   ! na2so4 <==> na+
      a8 = xk8*(water/gama(1))**2                   ! nacl   <==> na+
      a14 = xk14*(water/gama(6))**2                 ! nh4cl  <==> nh4+
      akw = xkw*rh*water*water                        ! h2o    <==> h+
!
!   ammnium chloride
!
      if ( nh4i*cli>a14 ) then
        bb = -(waer(3)+waer(5)-root3)
        cc = -a14 + nh4i*(waer(5)-root3)
        dd = max(bb*bb-4._rkx*cc,zero)
        root2a = 0.5_rkx*(-bb+sqrt(dd))
        root2b = 0.5_rkx*(-bb-sqrt(dd))
        if ( zero<=root2a ) then
          root2 = root2a
        else
          root2 = root2b
        end if
        root2 = min(max(zero,root2),max(waer(5)-root3,zero),chi4,waer(3))
        psi4 = chi4 - root2
      end if
      psconv4 = abs(psi4-psi4o)<=eps*psi4o
      psi4o = psi4
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a5 ) then
        bb = -(chi1+waer(1)-root3)
        cc = 0.25_rkx*(waer(1)-root3)*(4._rkx*chi1+waer(1)-root3)
        dd = -0.25_rkx*(chi1*(waer(1)-root3)**20-a5)
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = min(max(root1,zero),max(waer(1)-root3,zero),chi1,waer(2))
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ion concentrations
!
      nai = waer(1) - (2._rkx*root1+root3)
      so4i = waer(2) - root1
      nh4i = waer(3) - root2
      cli = waer(5) - (root3+root2)
      no3i = waer(4)
!
!   sodum chloride  ; to obtain new value for root3
!
      if ( nai*cli>a8 ) then
        bb = -((chi1-2._rkx*root1)+(waer(5)-root2))
        cc = (chi1-2._rkx*root1)*(waer(5)-root2) - a8
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root3a = 0.5_rkx*(-bb-sqrt(dd))
        root3b = 0.5_rkx*(-bb+sqrt(dd))
        if ( zero<=root3a ) then
          root3 = root3a
        else
          root3 = root3b
        end if
        root3 = min(max(root3,zero),chi3)
        psi3 = chi3 - root3
      end if
      psconv3 = abs(psi3-psi3o)<=eps*psi3o
      psi3o = psi3
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
      else
        ggno3 = max(2._rkx*so4i+no3i-nai-nh4i,zero)
        ggcl = max(gg-ggno3,zero)
        if ( ggcl>tiny1 ) call calcclaq2(ggcl,cli,hi,claq)          ! hcl
        if ( ggno3>tiny1 ) then
          if ( ggcl<=tiny1 ) hi = zero
          call calcniaq2(ggno3,no3i,hi,no3aq)                       ! hno3
        end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv1 .and. psconv3 .and. psconv4 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcr3a')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    if ( cli<=tiny1 .and. waer(5)>tiny1 ) then     !no disslv cl-;solid only
      do i = 1 , nions
        molal(i) = zero
      end do
      do i = 1 , ngasaq
        gasaq(i) = zero
      end do
      call calcr1a
    else
      a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2        ! nh3  <==> nh4+
      a3 = xk4*r*temp*(water/gama(10))**2                ! hno3 <==> no3-
      a4 = xk3*r*temp*(water/gama(11))**2                ! hcl  <==> cl-
!
      gnh3 = nh4i/hi/a2
      ghno3 = hi*no3i/a3
      ghcl = hi*cli/a4
!
      gasaq(1) = nh3aq
      gasaq(2) = claq
      gasaq(3) = no3aq
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnh4cl = chi4 - psi4
      cnacl = chi3 - psi3
      cnano3 = zero
      cna2so4 = chi1 - psi1
    end if
  end subroutine calcr3a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcr2
! *** case r2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4no3 , nh4cl , na2so4 , nano3 , nacl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcr2
    use mod_che_common_isorropia
    implicit none
    logical :: exan , exac , exsn , exsc
!
! *** solve for dry case and see which solids are possible
! **************
    scase = 'r2 ; subcase 2'
    call calcr1a                ! solid
    scase = 'r2 ; subcase 2'
!
    exan = cnh4no3>tiny1        ! nh4no3
    exac = cnh4cl>tiny1         ! nh4cl
    exsn = cnano3>tiny1         ! nano3
    exsc = cnacl>tiny1          ! nacl
!
! *** regime depends on relative humidity and possible species
! **********
    if ( exan ) then                             ! *** nh4no3 exists
      if ( rh>=drmh1 ) then
        scase = 'r2 ; subcase 1'
        call calcr2a
        scase = 'r2 ; subcase 1'
      end if
!
    else if ( .not.exan ) then                   ! *** nh4no3 = 0
      if ( exac .and. exsn .and. exsc ) then
        if ( rh>=drmh2 ) then
          scase = 'r2 ; subcase 3'
          call calcmdrp(rh,drmh2,drnano3,calcr1a,calcr3a)
          scase = 'r2 ; subcase 3'
        end if

      else if ( .not.exac .and. exsn .and. exsc ) then
        if ( rh>=drmr1 ) then
          scase = 'r2 ; subcase 4'
          call calcmdrp(rh,drmr1,drnano3,calcr1a,calcr3a)
          scase = 'r2 ; subcase 4'
        end if

      else if ( .not.exac .and. .not.exsn .and. exsc ) then
        if ( rh>=drmr2 ) then
          scase = 'r2 ; subcase 5'
          call calcmdrp(rh,drmr2,drnacl,calcr1a,calcr4a)
          scase = 'r2 ; subcase 5'
        end if

      else if ( .not.exac .and. exsn .and. .not.exsc ) then
        if ( rh>=drmr3 ) then
          scase = 'r2 ; subcase 6'
          call calcmdrp(rh,drmr3,drnano3,calcr1a,calcr3a)
          scase = 'r2 ; subcase 6'
        end if

      else if ( exac .and. .not.exsn .and. exsc ) then
        if ( rh>=drmr4 ) then
          scase = 'r2 ; subcase 7'
          call calcmdrp(rh,drmr4,drnacl,calcr1a,calcr4a)
          scase = 'r2 ; subcase 7'
        end if

      else if ( exac .and. .not.exsn .and. .not.exsc ) then
        if ( rh>=drmr5 ) then
          scase = 'r2 ; subcase 8'
          call calcmdrp(rh,drmr5,drnh4cl,calcr1a,calcr5)
          scase = 'r2 ; subcase 8'
        end if

      else if ( exac .and. exsn .and. .not.exsc ) then
        if ( rh>=drmr6 ) then
          scase = 'r2 ; subcase 9'
          call calcmdrp(rh,drmr6,drnano3,calcr1a,calcr3a)
          scase = 'r2 ; subcase 9'
        end if
      end if
    end if
  end subroutine calcr2
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcr2a
! *** case r2a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); sodium rich (sodrat >= 2.0)
!     2. liquid and solid phases are possible
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcr2a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , ggcl , ggno3 , &
         hi , hso4i , ohi , psi1o , psi2o , psi3o , psi4o , root1 ,   &
         root2 , root2a , root2b
    real(rkx) :: root3 , root3a , root3b , root4 , root4a , root4b , so4i
    integer(ik4) :: i , islv
    logical :: psconv1 , psconv2 , psconv3 , psconv4
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv1 = .true.
    psconv2 = .true.
    psconv3 = .true.
    psconv4 = .true.
!
    psi1o = -great
    psi2o = -great
    psi3o = -great
    psi4o = -great
!
    root1 = zero
    root2 = zero
    root3 = zero
    root4 = zero
!
! *** calculate initial solution ***************************************
!
    call calcr1a
!
    chi1 = cna2so4          ! salts
    chi2 = cnano3
    chi3 = cnacl
    chi4 = cnh4cl
!
    psi1 = cna2so4
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = waer(2)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
    molal(1) = zero
    molal(2) = nai
    molal(3) = nh4i
    molal(4) = cli
    molal(5) = so4i
    molal(6) = hso4i
    molal(7) = no3i
!
    call calcact            ! calculate activity coefficients
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a5 = xk5*(water/gama(2))**3                   ! na2so4 <==> na+
      a8 = xk8*(water/gama(1))**2                   ! nacl   <==> na+
      a9 = xk9*(water/gama(3))**2                   ! nano3  <==> na+
      a14 = xk14*(water/gama(6))**2                 ! nh4cl  <==> nh4+
      akw = xkw*rh*water*water                        ! h2o    <==> h+
!
!   ammnium chloride
!
      if ( nh4i*cli>a14 ) then
        bb = -(waer(3)+waer(5)-root3)
        cc = nh4i*(waer(5)-root3) - a14
        dd = max(bb*bb-4._rkx*cc,zero)
        dd = sqrt(dd)
        root2a = 0.5_rkx*(-bb+dd)
        root2b = 0.5_rkx*(-bb-dd)
        if ( zero<=root2a ) then
          root2 = root2a
        else
          root2 = root2b
        end if
        root2 = min(max(root2,zero),chi4)
        psi4 = chi4 - root2
      end if
      psconv4 = abs(psi4-psi4o)<=eps*psi4o
      psi4o = psi4
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a5 ) then
        bb = -(waer(2)+waer(1)-root3-root4)
        cc = waer(1)*(2._rkx*root3+2._rkx*root4-4._rkx*waer(2)-one)             &
             - (root3+root4)**2 + 4._rkx*waer(2)*(root3+root4)
        cc = -0.25_rkx*cc
        dd = waer(1)*waer(2)*(one-2._rkx*root3-2._rkx*root4) + waer(2)        &
             *(root3+root4)**2 - a5
        dd = -0.25_rkx*dd
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = min(max(root1,zero),chi1)
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   sodum nitrate
!
      if ( nai*no3i>a9 ) then
        bb = -(waer(4)+waer(1)-2._rkx*root1-root3)
        cc = waer(4)*(waer(1)-2._rkx*root1-root3) - a9
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root4a = 0.5_rkx*(-bb-dd)
        root4b = 0.5_rkx*(-bb+dd)
        if ( zero<=root4a ) then
          root4 = root4a
        else
          root4 = root4b
        end if
        root4 = min(max(root4,zero),chi2)
        psi2 = chi2 - root4
      end if
      psconv2 = abs(psi2-psi2o)<=eps*psi2o
      psi2o = psi2
!
!   ion concentrations
!
      nai = waer(1) - (2._rkx*root1+root3+root4)
      so4i = waer(2) - root1
      nh4i = waer(3) - root2
      no3i = waer(4) - root4
      cli = waer(5) - (root3+root2)
!
!   sodum chloride  ; to obtain new value for root3
!
      if ( nai*cli>a8 ) then
        bb = -(waer(1)-2._rkx*root1+waer(5)-root2-root4)
        cc = (waer(5)+root2)*(waer(1)-2._rkx*root1-root4) - a8
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root3a = 0.5_rkx*(-bb-dd)
        root3b = 0.5_rkx*(-bb+dd)
        if ( zero<=root3a ) then
          root3 = root3a
        else
          root3 = root3b
        end if
        root3 = min(max(root3,zero),chi3)
        psi3 = chi3 - root3
      end if
      psconv3 = abs(psi3-psi3o)<=eps*psi3o
      psi3o = psi3
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi<ohi ) then
        call calcamaq2(-gg,nh4i,ohi,nh3aq)
        hi = akw/ohi
      else
        ggno3 = max(2._rkx*so4i+no3i-nai-nh4i,zero)
        ggcl = max(gg-ggno3,zero)
        if ( ggcl>tiny1 ) call calcclaq2(ggcl,cli,hi,claq)          ! hcl
        if ( ggno3>tiny1 ) then
          if ( ggcl<=tiny1 ) hi = zero
          call calcniaq2(ggno3,no3i,hi,no3aq)                       ! hno3
        end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
        so4i = so4i - del
        hi = hi - del
        hso4i = del
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv1 .and. psconv2 .and. psconv3 .and. psconv4 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcr2a')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    if ( cli<=tiny1 .and. waer(5)>tiny1 ) then     !no disslv cl-;solid only
      do i = 1 , nions
        molal(i) = zero
      end do
      do i = 1 , ngasaq
        gasaq(i) = zero
      end do
      call calcr1a
    else                                       ! ok , aqueous phase present
      a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2        ! nh3  <==> nh4+
      a3 = xk4*r*temp*(water/gama(10))**2                ! hno3 <==> no3-
      a4 = xk3*r*temp*(water/gama(11))**2                ! hcl  <==> cl-
!
      gnh3 = nh4i/hi/a2
      ghno3 = hi*no3i/a3
      ghcl = hi*cli/a4
!
      gasaq(1) = nh3aq
      gasaq(2) = claq
      gasaq(3) = no3aq
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnh4cl = chi4 - psi4
      cnacl = chi3 - psi3
      cnano3 = chi2 - psi2
      cna2so4 = chi1 - psi1
    end if
  end subroutine calcr2a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcr1
! *** case r1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4no3 , nh4cl , na2so4 , nano3 , nacl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcr1
    use mod_che_common_isorropia
    implicit none
    logical :: exan , exac , exsn , exsc
!
! *** solve for dry case and see which solids are possible
! **************
    scase = 'r1 ; subcase 1'
    call calcr1a                ! solid
    scase = 'r1 ; subcase 1'
!
    exan = cnh4no3>tiny1        ! nh4no3
    exac = cnh4cl>tiny1         ! nh4cl
    exsn = cnano3>tiny1         ! nano3
    exsc = cnacl>tiny1          ! nacl
!
! *** regime depends on relative humidity and possible species
! **********
    if ( exan .and. exac .and. exsc .and. exsn ) then
                                                 ! *** all exist
      if ( rh>=drmh1 ) then
        scase = 'r1 ; subcase 2'      ! mdrh
        call calcmdrp(rh,drmh1,drnh4no3,calcr1a,calcr2a)
        scase = 'r1 ; subcase 2'
      end if
!
    else if ( .not.exan ) then                   ! *** nh4no3 = 0
      if ( exac .and. exsn .and. exsc ) then
        if ( rh>=drmh2 ) then
          scase = 'r1 ; subcase 3'
          call calcmdrp(rh,drmh2,drnano3,calcr1a,calcr3a)
          scase = 'r1 ; subcase 3'
        end if

      else if ( .not.exac .and. exsn .and. exsc ) then
        if ( rh>=drmr1 ) then
          scase = 'r1 ; subcase 4'
          call calcmdrp(rh,drmr1,drnano3,calcr1a,calcr3a)
          scase = 'r1 ; subcase 4'
        end if

      else if ( .not.exac .and. .not.exsn .and. exsc ) then
        if ( rh>=drmr2 ) then
          scase = 'r1 ; subcase 5'
          call calcmdrp(rh,drmr2,drnacl,calcr1a,calcr3a)               !, calcr4a)
          scase = 'r1 ; subcase 5'
        end if

      else if ( .not.exac .and. exsn .and. .not.exsc ) then
        if ( rh>=drmr3 ) then
          scase = 'r1 ; subcase 6'
          call calcmdrp(rh,drmr3,drnano3,calcr1a,calcr3a)
          scase = 'r1 ; subcase 6'
        end if

      else if ( exac .and. .not.exsn .and. exsc ) then
        if ( rh>=drmr4 ) then
          scase = 'r1 ; subcase 7'
          call calcmdrp(rh,drmr4,drnacl,calcr1a,calcr3a)               !, calcr4a)
          scase = 'r1 ; subcase 7'
        end if

      else if ( exac .and. .not.exsn .and. .not.exsc ) then
        if ( rh>=drmr5 ) then
          scase = 'r1 ; subcase 8'
          call calcmdrp(rh,drmr5,drnh4cl,calcr1a,calcr3a)               !, calcr5)
          scase = 'r1 ; subcase 8'
        end if

      else if ( exac .and. exsn .and. .not.exsc ) then
        if ( rh>=drmr6 ) then
          scase = 'r1 ; subcase 9'
          call calcmdrp(rh,drmr6,drnano3,calcr1a,calcr3a)
          scase = 'r1 ; subcase 9'
        end if
      end if
!
    else if ( .not.exac ) then                   ! *** nh4cl  = 0
      if ( exan .and. exsn .and. exsc ) then
        if ( rh>=drmr7 ) then
          scase = 'r1 ; subcase 10'
          call calcmdrp(rh,drmr7,drnh4no3,calcr1a,calcr2a)
          scase = 'r1 ; subcase 10'
        end if

      else if ( exan .and. .not.exsn .and. exsc ) then
        if ( rh>=drmr8 ) then
          scase = 'r1 ; subcase 11'
          call calcmdrp(rh,drmr8,drnh4no3,calcr1a,calcr2a)
          scase = 'r1 ; subcase 11'
        end if

      else if ( exan .and. .not.exsn .and. .not.exsc ) then
        if ( rh>=drmr9 ) then
          scase = 'r1 ; subcase 12'
          call calcmdrp(rh,drmr9,drnh4no3,calcr1a,calcr2a)
          scase = 'r1 ; subcase 12'
        end if

      else if ( exan .and. exsn .and. .not.exsc ) then
        if ( rh>=drmr10 ) then
          scase = 'r1 ; subcase 13'
          call calcmdrp(rh,drmr10,drnh4no3,calcr1a,calcr2a)
          scase = 'r1 ; subcase 13'
        end if
      end if
!
    else if ( .not.exsn ) then                  ! *** nano3  = 0
      if ( exan .and. exac .and. exsc ) then
        if ( rh>=drmr11 ) then
          scase = 'r1 ; subcase 14'
          call calcmdrp(rh,drmr11,drnh4no3,calcr1a,calcr2a)
          scase = 'r1 ; subcase 14'
        end if

      else if ( exan .and. exac .and. .not.exsc ) then
        if ( rh>=drmr12 ) then
          scase = 'r1 ; subcase 15'
          call calcmdrp(rh,drmr12,drnh4no3,calcr1a,calcr2a)
          scase = 'r1 ; subcase 15'
        end if
      end if
!
    else if ( .not.exsc ) then                  ! *** nacl   = 0
      if ( exan .and. exac .and. exsn ) then
        if ( rh>=drmr13 ) then
          scase = 'r1 ; subcase 16'
          call calcmdrp(rh,drmr13,drnh4no3,calcr1a,calcr2a)
          scase = 'r1 ; subcase 16'
        end if
      end if
    end if
  end subroutine calcr1
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcr1a
! *** case r1 ; subcase 1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; sodium rich (sodrat >= 2.0)
!     2. solid aerosol only
!     3. solids possible : nh4no3 , nh4cl , nano3 , na2so4 , nano3 , nacl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by athanasios nenes
!
!======================================================================
!
  subroutine calcr1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: frcl , frna , frnh3 , frno3
!
! *** calculate solids
! **************************************************
    cna2so4 = waer(2)
    frna = max(waer(1)-2*cna2so4,zero)
!
    cnh42s4 = zero
!
    cnano3 = min(frna,waer(4))
    frno3 = max(waer(4)-cnano3,zero)
    frna = max(frna-cnano3,zero)
!
    cnacl = min(frna,waer(5))
    frcl = max(waer(5)-cnacl,zero)
    frna = max(frna-cnacl,zero)
!
    cnh4no3 = min(frno3,waer(3))
    frno3 = max(frno3-cnh4no3,zero)
    frnh3 = max(waer(3)-cnh4no3,zero)
!
    cnh4cl = min(frcl,frnh3)
    frcl = max(frcl-cnh4cl,zero)
    frnh3 = max(frnh3-cnh4cl,zero)
!
! *** other phases
! ******************************************************
    water = zero
!
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
  end subroutine calcr1a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcv7
! *** case v7
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcv7
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , ohi , so4i
    integer(ik4) :: i
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
! *** calculate initial solution ***************************************
!
    call calcv1a
!
    chi9 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi6 = cnh42s4
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nh3aq = zero
    no3aq = zero
    claq = zero
    no3i = zero
    nh4i = zero
    cli = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   ion concentrations
!
      nai = waer(1)
      so4i = max(waer(2)-waer(6),zero)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      ki = waer(7)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                    ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                     ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if

!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
! call pusherr (0002 , 'calcv7')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cna2so4 = zero
    cmgso4 = zero
    ck2so4 = zero
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcv7
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcv6
! *** case v6
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4
!     4. completely dissolved: nh4no3 , nh4cl , (nh4)2so4 , mgso4 , na2so4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcv6
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi70 , root7 , so4i
    integer(ik4) :: i , islv
    logical :: psconv7
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .true.
    psi70 = -great                                    ! great = 1.d10
    root7 = zero
!
! *** calculate initial solution ***************************************
!
    call calcv1a
!
    chi9 = ccaso4
    chi7 = ck2so4           ! salts
!
    psi1 = cna2so4          ! amount dissolved
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi6 = cnh42s4
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7))
        cc = waer(7)*(waer(2)-waer(6)) + 0.25_rkx*waer(7)*waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*waer(2)-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = min(root7,waer(7)/2.0,max(waer(2)-waer(6),zero),chi7)
        root7 = max(root7,zero)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root7,zero)
      so4i = max(waer(2)-waer(6)-root7,zero)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      nai = waer(1)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcv6')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cna2so4 = zero
    cmgso4 = zero
    ck2so4 = chi7 - psi7
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcv6
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcv5
! *** case v5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4
!     4. completely dissolved: nh4no3 , nh4cl , (nh4)2so4 , mgso4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcv5
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi1o , psi70 , root1 , root7 , so4i
    integer(ik4) :: i , islv
    logical :: psconv7 , psconv1
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .true.
    psconv1 = .true.
!
    psi70 = -great                                    ! great = 1.d10
    psi1o = -great
!
    root7 = zero
    root1 = zero
!
! *** calculate initial solution ***************************************
!
    call calcv1a
!
    chi9 = ccaso4
    chi7 = ck2so4           ! salts
    chi1 = cna2so4
!
    psi1 = cna2so4          ! amount dissolved
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi6 = cnh42s4
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a1 = xk5*(water/gama(2))**3          ! na2s04    <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root1)
        cc = waer(7)*((waer(2)-waer(6))-root1) + 0.25_rkx*waer(7)*waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*((waer(2)-waer(6))-root1)-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = max(root7,zero)
        root7 = min(root7,waer(7)/2.0,max(waer(2)-waer(6)-root1,zero),    &
                chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a1 ) then
        bb = -((waer(2)-waer(6))+waer(1)-root7)
        cc = waer(1)*((waer(2)-waer(6))-root7) + 0.25_rkx*waer(1)*waer(1)
        dd = -0.25_rkx*(waer(1)*waer(1)*((waer(2)-waer(6))-root7)-a1)
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = max(root1,zero)
        root1 = min(root1,waer(1)/2.0,max((waer(2)-waer(6))-root7,zero),  &
                chi1)
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root7,zero)
      nai = max(waer(1)-2._rkx*root1,zero)
      so4i = max((waer(2)-waer(6))-root7-root1,zero)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 .and. psconv1 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcv5')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cna2so4 = chi1 - psi1
    cmgso4 = zero
    ck2so4 = chi7 - psi7
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcv5
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcv4
! *** case v4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4
!     4. completely dissolved: nh4no3 , nh4cl , (nh4)2so4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcv4
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi1o , psi70 , root1 , root7 , so4i
    integer(ik4) :: i , islv
    logical :: psconv7 , psconv1
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .true.
    psconv1 = .true.
!
    psi70 = -great                                    ! great = 1.d10
    psi1o = -great
!
    root7 = zero
    root1 = zero
!
! *** calculate initial solution ***************************************
!
    call calcv1a
!
    chi9 = ccaso4
    chi7 = ck2so4           ! salts
    chi1 = cna2so4
    chi8 = cmgso4
!
    psi1 = cna2so4          ! amount dissolved
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi6 = cnh42s4
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = waer(2)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a1 = xk5*(water/gama(2))**3          ! na2s04    <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root1)
        cc = waer(7)*((waer(2)-waer(6))-root1) + 0.25_rkx*waer(7)*waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*((waer(2)-waer(6))-root1)-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = max(root7,zero)
        root7 = min(root7,waer(7)/2.0,max((waer(2)-waer(6))-root1,zero),  &
                chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a1 ) then
        bb = -((waer(2)-waer(6))+waer(1)-root7)
        cc = waer(1)*((waer(2)-waer(6))-root7) + 0.25_rkx*waer(1)*waer(1)
        dd = -0.25_rkx*(waer(1)*waer(1)*((waer(2)-waer(6))-root7)-a1)
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = max(root1,zero)
        root1 = min(root1,waer(1)/2.0,max((waer(2)-waer(6))-root7,zero),  &
                chi1)
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root7,zero)
      nai = max(waer(1)-2._rkx*root1,zero)
      so4i = max((waer(2)-waer(6))-root7-root1,zero)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 .and. psconv1 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcv4')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cna2so4 = chi1 - psi1
    cmgso4 = zero
    ck2so4 = chi7 - psi7
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcv4
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcv3
! *** case v3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , (nh4)2so4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcv3
    use mod_che_common_isorropia
    implicit none
    logical :: exno , excl
!
! *** regime depends on ambient relative humidity & possible species ***
!
    exno = waer(4)>tiny1
    excl = waer(5)>tiny1
    if ( exno .or. excl ) then             ! *** nitrate or chloride exists
      scase = 'v3 ; subcase 1'
      call calcv3a
      scase = 'v3 ; subcase 1'
    else if ( rh<drmo3 ) then
      scase = 'v3 ; subcase 2'
      call calcv1a                   ! solid
      scase = 'v3 ; subcase 2'
    else
      scase = 'v3 ; subcase 3'       ! mdrh (caso4 , (nh4)2so4 , mgso4 , na2so4 , k2so4)
      call calcmdrpii(rh,drmo3,drnh42s4,calcv1a,calcv4)
      scase = 'v3 ; subcase 3'
    end if
  end subroutine calcv3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcv3a
! *** case v3a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , (nh4)2so4
!     4. completely dissolved: nh4no3 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcv3a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi1o , psi60 , psi70 , root1 , root6 , root7 , so4i
    integer(ik4) :: i , islv
    logical :: psconv7 , psconv1 , psconv6
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .true.
    psconv1 = .true.
    psconv6 = .true.
!
    psi70 = -great                                    ! great = 1.d10
    psi1o = -great
    psi60 = -great
!
    root7 = zero
    root1 = zero
    root6 = zero
!
! *** calculate initial solution ***************************************
!
    call calcv1a
!
    chi9 = ccaso4
    chi7 = ck2so4           ! salts
    chi1 = cna2so4
    chi8 = cmgso4
    chi6 = cnh42s4
!
    psi1 = cna2so4          ! amount dissolved
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi6 = cnh42s4
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = waer(2)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a1 = xk5*(water/gama(2))**3          ! na2s04    <==> na+
      a6 = xk7*(water/gama(4))**3          !(nh4)2so4  <==> nh4+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root1-root6)
        cc = waer(7)*((waer(2)-waer(6))-root1-root6) + 0.25_rkx*waer(7)       &
             *waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*((waer(2)-waer(6))-root1-root6)-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = max(root7,zero)
        root7 = min(root7,waer(7)/2.0,                                    &
                max(waer(2)-waer(6)-root1-root6,zero),chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a1 ) then
        bb = -((waer(2)-waer(6))+waer(1)-root7-root6)
        cc = waer(1)*((waer(2)-waer(6))-root7-root6) + 0.25_rkx*waer(1)       &
             *waer(1)
        dd = -0.25_rkx*(waer(1)*waer(1)*((waer(2)-waer(6))-root7-root6)-a1)
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = max(root1,zero)
        root1 = min(root1,waer(1)/2.0,                                    &
                max(waer(2)-waer(6)-root7-root6,zero),chi1)
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ammnium sulfate
!
      if ( nh4i*nh4i*so4i>a6 ) then
        bb = -((waer(2)-waer(6))+waer(3)-root7-root1)
        cc = waer(3)*((waer(2)-waer(6))-root7-root1) + 0.25_rkx*waer(3)       &
             *waer(3)
        dd = -0.25_rkx*(waer(3)*waer(3)*((waer(2)-waer(6))-root7-root1)-a6)
        call poly3(bb,cc,dd,root6,islv)
        if ( islv/=0 ) root6 = tiny1
        root6 = max(root6,zero)
        root6 = min(root6,waer(3)/2.0,                                    &
                max(waer(2)-waer(6)-root7-root1,zero),chi6)
        psi6 = chi6 - root6
      end if
      psconv6 = abs(psi6-psi60)<=eps*psi60
      psi60 = psi6
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root7,zero)
      nai = max(waer(1)-2._rkx*root1,zero)
      so4i = max(waer(2)-waer(6)-root7-root1-root6,zero)
      nh4i = max(waer(3)-2._rkx*root6,zero)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 .and. psconv1 .and. psconv6 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcv3')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = chi6 - psi6
    cnh4no3 = zero
    cnh4cl = zero
    cna2so4 = chi1 - psi1
    cmgso4 = zero
    ck2so4 = chi7 - psi7
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcv3a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcv2
! *** case v2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , (nh4)2so4 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcv2
    use mod_che_common_isorropia
    implicit none
    logical :: exno , excl
!
! *** regime depends on ambient relative humidity & possible species ***
!
    exno = waer(4)>tiny1
    excl = waer(5)>tiny1
!
    if ( exno ) then                       ! *** nitrate exists
      scase = 'v2 ; subcase 1'
      call calcv2a
      scase = 'v2 ; subcase 1'
!
    else if ( .not.exno .and. excl ) then   ! *** only chloride exists
      if ( rh<drmo2 ) then
        scase = 'v2 ; subcase 2'
        call calcv1a                 ! solid
        scase = 'v2 ; subcase 2'
      else
        scase = 'v2 ; subcase 3'     ! mdrh caso4 , nh4cl , (nh4)2so4 , mgso4 , na2so4 , k2so4
        call calcmdrpii(rh,drmo2,drnh4cl,calcv1a,calcv3a)
        scase = 'v2 ; subcase 3'
      end if
!
    else if ( rh<drmo3 ) then
      scase = 'v2 ; subcase 2'
      call calcv1a                   ! solid
      scase = 'v2 ; subcase 2'
    else
      scase = 'v2 ; subcase 4'       ! mdrh caso4 , (nh4)2so4 , mgso4 , na2so4 , k2so4
      call calcmdrpii(rh,drmo3,drnh42s4,calcv1a,calcv4)
      scase = 'v2 ; subcase 4'
    end if
  end subroutine calcv2
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcv2a
! *** case v2a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , (nh4)2so4 , nh4cl
!     4. completely dissolved: nh4no3
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcv2a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi1o , psi40 , psi60 , psi70 , root1 , root4 , root4a , &
         root4b , root6 , root7
    real(rkx) :: so4i
    integer(ik4) :: i , islv
    logical :: psconv7 , psconv1 , psconv6 , psconv4
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .true.
    psconv1 = .true.
    psconv6 = .true.
    psconv4 = .true.
!
    psi70 = -great                                    ! great = 1.d10
    psi1o = -great
    psi60 = -great
    psi40 = -great
!
    root7 = zero
    root1 = zero
    root6 = zero
    root4 = zero
!
! *** calculate initial solution ***************************************
!
    call calcv1a
!
    chi9 = ccaso4
    chi7 = ck2so4           ! salts
    chi1 = cna2so4
    chi8 = cmgso4
    chi6 = cnh42s4
    chi4 = cnh4cl
!
    psi1 = cna2so4          ! amount dissolved
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi6 = cnh42s4
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a1 = xk5*(water/gama(2))**3          ! na2s04    <==> na+
      a6 = xk7*(water/gama(4))**3          ! (nh4)2so4 <==> nh4+
      a14 = xk14*(water/gama(6))**2        ! nh4cl     <==> nh4+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   ammnium chloride
!
      if ( nh4i*cli>a14 ) then
        bb = -(waer(3)+waer(5)-2._rkx*root6)
        cc = waer(5)*(waer(3)-2._rkx*root6) - a14
        dd = bb*bb - 4._rkx*cc
        if ( dd<zero ) then
          root4 = zero
        else
          dd = sqrt(dd)
          root4a = 0.5_rkx*(-bb+dd)
          root4b = 0.5_rkx*(-bb-dd)
          if ( zero<=root4a ) then
            root4 = root4a
          else
            root4 = root4b
          end if
          root4 = max(root4,zero)
          root4 = min(root4,waer(5),max(waer(3)-2._rkx*root6,zero),chi4)
          psi4 = chi4 - root4
        end if
      end if
      psconv4 = abs(psi4-psi40)<=eps*psi40
      psi40 = psi4
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root1-root6)
        cc = waer(7)*((waer(2)-waer(6))-root1-root6) + 0.25_rkx*waer(7)       &
             *waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*((waer(2)-waer(6))-root1-root6)-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = max(root7,zero)
        root7 = min(root7,waer(7)/2.0,                                    &
                max(waer(2)-waer(6)-root1-root6,zero),chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a1 ) then
        bb = -((waer(2)-waer(6))+waer(1)-root7-root6)
        cc = waer(1)*((waer(2)-waer(6))-root7-root6) + 0.25_rkx*waer(1)       &
             *waer(1)
        dd = -0.25_rkx*(waer(1)*waer(1)*((waer(2)-waer(6))-root7-root6)-a1)
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = max(root1,zero)
        root1 = min(root1,waer(1)/2.0,                                    &
                max(waer(2)-waer(6)-root7-root6,zero),chi1)
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ammnium sulfate
!
      if ( nh4i*nh4i*so4i>a6 ) then
        bb = -((waer(2)-waer(6))+waer(3)-root7-root1-root4)
        cc = waer(3)*((waer(2)-waer(6))-root7-root1)                      &
             + 0.25_rkx*(waer(3)-root4)                                       &
             **2 + root4*(root1+root7-(waer(2)-waer(6)))
        dd = -0.25_rkx*((waer(3)-root4)**2*((waer(2)-waer(6))-root7-root1)  &
             -a6)
        call poly3(bb,cc,dd,root6,islv)
        if ( islv/=0 ) root6 = tiny1
        root6 = max(root6,zero)
        root6 = min(root6,waer(3)/2.0,                                    &
                max(waer(2)-waer(6)-root7-root1,zero),chi6)
        psi6 = chi6 - root6
      end if
      psconv6 = abs(psi6-psi60)<=eps*psi60
      psi60 = psi6
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root7,zero)
      nai = max(waer(1)-2._rkx*root1,zero)
      so4i = max(waer(2)-waer(6)-root7-root1-root6,zero)
      nh4i = max(waer(3)-2._rkx*root6,zero)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 .and. psconv1 .and. psconv6 .and. psconv4 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcv2')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = chi6 - psi6
    cnh4no3 = zero
    cnh4cl = chi4 - psi4
    cna2so4 = chi1 - psi1
    cmgso4 = zero
    ck2so4 = chi7 - psi7
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcv2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcv1
! *** case v1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4 , nh4no3 , nh4cl , na2so4 , k2so4 , mgso4 , caso4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcv1
    use mod_che_common_isorropia
    implicit none
    logical :: exno , excl
!
! *** regime depends on ambient relative humidity & possible species ***
!
    exno = waer(4)>tiny1
    excl = waer(5)>tiny1
!
    if ( exno .and. excl ) then           ! *** nitrate & chloride exist
      if ( rh<drmo1 ) then
        scase = 'v1 ; subcase 1'
        call calcv1a                 ! solid
        scase = 'v1 ; subcase 1'
      else
        scase = 'v1 ; subcase 2'     ! mdrh (nh4)2so4 , nh4no3 , nh4cl , na2so4 , k2so4 , mgso4 , caso4
        call calcmdrpii(rh,drmo1,drnh4no3,calcv1a,calcv2a)
        scase = 'v1 ; subcase 2'
      end if
!
    else if ( exno .and. .not.excl ) then ! *** only nitrate exists
      if ( rh<drmv1 ) then
        scase = 'v1 ; subcase 1'
        call calcv1a                 ! solid
        scase = 'v1 ; subcase 1'
      else
        scase = 'v1 ; subcase 3'     ! mdrh (nh4)2so4 , nh4no3 , na2so4 , k2so4 , mgso4 , caso4
        call calcmdrpii(rh,drmv1,drnh4no3,calcv1a,calcv2a)
        scase = 'v1 ; subcase 3'
      end if
!
    else if ( .not.exno .and. excl ) then ! *** only chloride exists
      if ( rh<drmo2 ) then
        scase = 'v1 ; subcase 1'
        call calcv1a                 ! solid
        scase = 'v1 ; subcase 1'
      else
        scase = 'v1 ; subcase 4'     ! mdrh (nh4)2so4 , nh4cl , na2so4 , k2so4 , mgso4 , caso4
        call calcmdrpii(rh,drmo2,drnh4cl,calcv1a,calcv3a)
        scase = 'v1 ; subcase 4'
      end if
!
    else if ( rh<drmo3 ) then
      scase = 'v1 ; subcase 1'
      call calcv1a                   ! solid
      scase = 'v1 ; subcase 1'
    else
      scase = 'v1 ; subcase 5'       ! mdrh (nh4)2so4 , na2so4 , k2so4 , mgso4 , caso4
      call calcmdrpii(rh,drmo3,drnh42s4,calcv1a,calcv4)
      scase = 'v1 ; subcase 5'
    end if
!
! if (rh < drmo1) then
! scase = 'v1 ; subcase 1'
! call calcv1a              ! solid phase only possible
! scase = 'v1 ; subcase 1'
! else
! scase = 'v1 ; subcase 2'  ! liquid & solid phase possible
! call calcmdrpii (rh , drmo1 , drnh4no3 , calcv1a , calcv2a)
! scase = 'v1 ; subcase 2'
! end if
!
  end subroutine calcv1
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcv1a
! *** case v1a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. solid aerosol only
!     3. solids possible : (nh4)2so4 , nh4no3 , nh4cl , na2so4 , k2so4 , mgso4 , caso4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcv1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: cafr , frk , frmg , frnh3 , so4fr , nafr
!
! *** calculate solids
! **************************************************
    ccaso4 = min(waer(6),waer(2))                          ! ccaso4
    so4fr = max(waer(2)-ccaso4,zero)
    cafr = max(waer(6)-ccaso4,zero)
    ck2so4 = min(0.5_rkx*waer(7),so4fr)                      ! ck2so4
    frk = max(waer(7)-2._rkx*ck2so4,zero)
    so4fr = max(so4fr-ck2so4,zero)
    cna2so4 = min(0.5_rkx*waer(1),so4fr)                     ! cna2so4
    nafr = max(waer(1)-2._rkx*cna2so4,zero)
    so4fr = max(so4fr-cna2so4,zero)
    cmgso4 = min(waer(8),so4fr)                            ! cmgso4
    frmg = max(waer(8)-cmgso4,zero)
    so4fr = max(so4fr-cmgso4,zero)
    cnh42s4 = max(min(so4fr,0.5_rkx*waer(3)),tiny1)
    frnh3 = max(waer(3)-2._rkx*cnh42s4,zero)
!
    cnh4no3 = min(frnh3,waer(4))
! frno3   = max (waer(4) - cnh4no3 , zero)
    frnh3 = max(frnh3-cnh4no3,zero)
!
    cnh4cl = min(frnh3,waer(5))
! frcl    = max (waer(5) - cnh4cl , zero)
    frnh3 = max(frnh3-cnh4cl,zero)
!
! *** other phases
! ******************************************************
    water = zero
!
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
  end subroutine calcv1a
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcu8
! *** case u8
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); crustal+sodium rich (crnarat >= 2.0); crustal poor (crrat<2)
!     2. there is only a liquid phase
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu8
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , ohi , so4i
    integer(ik4) :: i
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    call calcu1a
!
    chi9 = ccaso4            ! salts
!
    psi1 = cna2so4
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
! *** calculate water **************************************************
!
    call calcmr
!
! *** setup liquid concentrations **************************************
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
    no3i = zero
    nh4i = zero
    cli = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      akw = xkw*rh*water*water                        ! h2o    <==> h+
!
      nai = waer(1)
      so4i = max(waer(2)-waer(6),zero)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      ki = waer(7)
      mgi = waer(8)

!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
      if ( hi<=tiny1 ) hi = sqrt(akw)
!   ohi   = akw/hi
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
! call pusherr (0002 , 'calcu8')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2        ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2                ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2                ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cna2so4 = zero
    cmgso4 = zero
    ck2so4 = zero
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcu8
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcu7
! *** case u7
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), crustal+sodium rich (crnarat >= 2.0); crustal poor (crrat<2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl , mgso4 , na2so4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu7
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi70 , root7 , so4i
    integer(ik4) :: i , islv
    logical :: psconv7
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .true.
    psi70 = -great                                    ! great = 1.d10
    root7 = zero
!
! *** calculate initial solution ***************************************
!
    call calcu1a
!
    chi7 = ck2so4           ! salts
    chi9 = ccaso4
!
    psi1 = cna2so4
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
    molal(1) = zero
    molal(2) = nai
    molal(3) = nh4i
    molal(4) = cli
    molal(5) = so4i
    molal(6) = hso4i
    molal(7) = no3i
    molal(8) = cai
    molal(9) = ki
    molal(10) = mgi
!
    call calcact            ! calculate activity coefficients
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7))
        cc = waer(7)*(waer(2)-waer(6)) + 0.25_rkx*waer(7)*waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*(waer(2)-waer(6))-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = max(root7,zero)
        root7 = min(root7,waer(7)/2.0,max(waer(2)-waer(6),zero),chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root7,zero)
      so4i = max(waer(2)-waer(6)-root7,zero)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      nai = waer(1)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!   if (hi <= tiny1) hi = sqrt(akw)
!   ohi   = akw/hi
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcu7')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cna2so4 = zero
    cmgso4 = zero
    ck2so4 = chi7 - psi7
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcu7
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcu6
! *** case u6
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl , mgso4
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu6
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi1o , psi70 , root1 , root7 , so4i
    integer(ik4) :: i , islv
    logical :: psconv7 , psconv1
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .true.
    psconv1 = .true.
!
    psi70 = -great                                    ! great = 1.d10
    psi1o = -great
!
    root7 = zero
    root1 = zero
!
! *** calculate initial solution ***************************************
!
    call calcu1a
!
    chi1 = cna2so4                ! salts
    chi7 = ck2so4
    chi9 = ccaso4
!
    psi1 = cna2so4
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
    molal(1) = zero
    molal(2) = nai
    molal(3) = nh4i
    molal(4) = cli
    molal(5) = so4i
    molal(6) = hso4i
    molal(7) = no3i
    molal(8) = cai
    molal(9) = ki
    molal(10) = mgi
!
    call calcact            ! calculate activity coefficients
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a1 = xk5*(water/gama(2))**3          ! na2s04    <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root1)
        cc = waer(7)*((waer(2)-waer(6))-root1) + 0.25_rkx*waer(7)*waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*((waer(2)-waer(6))-root1)-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = max(root7,zero)
        root7 = min(root7,waer(7)/2.0,max((waer(2)-waer(6))-root1,zero),  &
                chi7)
        psi7 = chi7 - root7

      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a1 ) then
        bb = -((waer(2)-waer(6))+waer(1)-root7)
        cc = waer(1)*((waer(2)-waer(6))-root7) + 0.25_rkx*waer(1)*waer(1)
        dd = -0.25_rkx*(waer(1)*waer(1)*((waer(2)-waer(6))-root7)-a1)
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = max(root1,zero)
        root1 = min(root1,waer(1)/2.0,max((waer(2)-waer(6))-root7,zero),  &
                chi1)
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root7,zero)
      nai = max(waer(1)-2._rkx*root1,zero)
      so4i = max(waer(2)-waer(6)-root7-root1,zero)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!   if (hi <= tiny1) hi = sqrt(akw)
!   ohi   = akw/hi
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 .and. psconv1 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcu6')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cna2so4 = chi1 - psi1
    cmgso4 = zero
    ck2so4 = chi7 - psi7
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcu6
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcu5
! *** case u5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4
!     4. completely dissolved: nh4no3 , nh4cl , nano3 , nacl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu5
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi1o , psi70 , root1 , root7 , so4i
    integer(ik4) :: i , islv
    logical :: psconv7 , psconv1
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .true.
    psconv1 = .true.
!
    psi70 = -great                                    ! great = 1.d10
    psi1o = -great
!
    root7 = zero
    root1 = zero
!
! *** calculate initial solution ***************************************
!
    call calcu1a
!
    chi1 = cna2so4                ! salts
    chi7 = ck2so4
    chi8 = cmgso4
    chi9 = ccaso4
!
    psi1 = cna2so4
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
    molal(1) = zero
    molal(2) = nai
    molal(3) = nh4i
    molal(4) = cli
    molal(5) = so4i
    molal(6) = hso4i
    molal(7) = no3i
    molal(8) = cai
    molal(9) = ki
    molal(10) = mgi
!
    call calcact            ! calculate activity coefficients
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a1 = xk5*(water/gama(2))**3          ! na2s04    <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root1)
        cc = waer(7)*((waer(2)-waer(6))-root1) + 0.25_rkx*waer(7)*waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*((waer(2)-waer(6))-root1)-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = max(root7,zero)
        root7 = min(root7,waer(7)/2.0,max(waer(2)-waer(6)-root1,zero),    &
                chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a1 ) then
        bb = -((waer(2)-waer(6))+waer(1)-root7)
        cc = waer(1)*((waer(2)-waer(6))-root7) + 0.25_rkx*waer(1)*waer(1)
        dd = -0.25_rkx*(waer(1)*waer(1)*((waer(2)-waer(6))-root7)-a1)
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = max(root1,zero)
        root1 = min(root1,waer(1)/2.0,max(waer(2)-waer(6)-root7,zero),    &
                chi1)
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root7,zero)
      nai = max(waer(1)-2._rkx*root1,zero)
      so4i = max(waer(2)-waer(6)-root7-root1,zero)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!   if (hi <= tiny1) hi = sqrt(akw)
!   ohi   = akw/hi
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 .and. psconv1 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcu5')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cna2so4 = chi1 - psi1
    cmgso4 = zero
    ck2so4 = chi7 - psi7
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcu5
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcu4
! *** case u4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), (dust + sodium) rich: r(cr+na)>2; dust poor: rcr<2.
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu4
    use mod_che_common_isorropia
    implicit none
    logical :: exan , exac , exsn , exsc
!
! *** solve for dry case and see which solids are possible
! **************
    scase = 'u4 ; subcase 2'
    call calcu1a                ! solid
    scase = 'u4 ; subcase 2'
    exan = cnh4no3>tiny1        ! nh4no3
    exac = cnh4cl>tiny1         ! nh4cl
    exsn = cnano3>tiny1         ! nano3
    exsc = cnacl>tiny1          ! nacl
!
! *** regime depends on relative humidity and possible species
! **********
    if ( exan .or. exsn .or. exsc ) then   ! *** nh4no3 ,nano3 exist
      if ( rh>=drmm1 ) then
        scase = 'u4 ; subcase 1'
        call calcu4a
        scase = 'u4 ; subcase 1'
      end if
    else if ( exac ) then                  ! *** nh4cl exists only
      if ( rh>=drmr5 ) then
        scase = 'u4 ; subcase 3'
        call calcmdrpii(rh,drmr5,drnh4cl,calcu1a,calcu5)
        scase = 'u4 ; subcase 3'
      end if
    end if
  end subroutine calcu4
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcu4a
! *** case u4a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , nh4cl
!     4. completely dissolved: nh4no3 , nano3 , nacl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu4a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi1o , psi40 , psi70 , root1 , root4 , root7 , so4i
    integer(ik4) :: i , islv
    logical :: psconv7 , psconv1 , psconv4
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .false.
    psconv1 = .false.
    psconv4 = .false.
!
    psi70 = -great                                    ! great = 1.d10
    psi1o = -great
    psi40 = -great
!
    root7 = zero
    root1 = zero
    root4 = zero
!
! *** calculate initial solution ***************************************
!
    call calcu1a
!
    chi1 = cna2so4                ! salts
    chi4 = cnh4cl
    chi7 = ck2so4
    chi8 = cmgso4
    chi9 = ccaso4
!
    psi1 = cna2so4
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
    molal(1) = zero
    molal(2) = nai
    molal(3) = nh4i
    molal(4) = cli
    molal(5) = so4i
    molal(6) = hso4i
    molal(7) = no3i
    molal(8) = cai
    molal(9) = ki
    molal(10) = mgi
!
    call calcact            ! calculate activity coefficients
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a1 = xk5*(water/gama(2))**3          ! na2s04    <==> na+
      a14 = xk14*(water/gama(6))**2        ! nh4cl     <==> nh4+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root1)
        cc = waer(7)*((waer(2)-waer(6))-root1) + 0.25_rkx*waer(7)*waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*((waer(2)-waer(6))-root1)-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = max(root7,zero)
        root7 = min(root7,waer(7)/2.0,max(waer(2)-waer(6)-root1,zero),    &
                chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a1 ) then
        bb = -((waer(2)-waer(6))+waer(1)-root7)
        cc = waer(1)*((waer(2)-waer(6))-root7) + 0.25_rkx*waer(1)*waer(1)
        dd = -0.25_rkx*(waer(1)*waer(1)*((waer(2)-waer(6))-root7)-a1)
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = max(root1,zero)
        root1 = min(root1,waer(1)/2.0,max(waer(2)-waer(6)-root7,zero),    &
                chi1)
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ammnium chloride
!
      if ( nh4i*cli>a14 ) then
        bb = -(nh4i+cli)
        cc = -a14 + nh4i*cli
        dd = bb*bb - 4._rkx*cc
        root4 = 0.5_rkx*(-bb-sqrt(dd))
        if ( root4>tiny1 ) then
          root4 = min(max(root4,zero),chi4)
          psi4 = chi4 - root4
        end if
      end if
      psconv4 = abs(psi4-psi40)<=eps*psi40
      psi40 = psi4
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root7,zero)
      nai = max(waer(1)-2._rkx*root1,zero)
      so4i = max(waer(2)-waer(6)-root7-root1,zero)
      nh4i = max(waer(3)-root4,zero)
      no3i = waer(4)
      cli = max(waer(5)-root4,zero)
      cai = zero
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!   if (hi <= tiny1) hi = sqrt(akw)
!   ohi   = akw/hi
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 .and. psconv1 .and. psconv4 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcu4')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = chi4 - psi4
    cnacl = zero
    cnano3 = zero
    cna2so4 = chi1 - psi1
    cmgso4 = zero
    ck2so4 = chi7 - psi7
    ccaso4 = min(waer(6),waer(2))
  end subroutine calcu4a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcu3
! *** case u3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), (dust + sodium) rich: r(cr+na)>2; dust poor: rcr<2.
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , nh4cl , nano3
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu3
    use mod_che_common_isorropia
    implicit none
    logical :: exan , exac , exsn , exsc
!
! *** solve for dry case and see which solids are possible
! **************
    scase = 'u3 ; subcase 2'
    call calcu1a                ! solid
    scase = 'u3 ; subcase 2'
!
    exan = cnh4no3>tiny1        ! nh4no3
    exac = cnh4cl>tiny1         ! nh4cl
    exsn = cnano3>tiny1         ! nano3
    exsc = cnacl>tiny1          ! nacl
!
! *** regime depends on relative humidity and possible species
! **********
    if ( exan .or. exsn ) then                   ! *** nh4no3 ,nano3 exist
      if ( rh>=drmm1 ) then
        scase = 'u3 ; subcase 1'
        call calcu3a
        scase = 'u3 ; subcase 1'
      end if
!
    else if ( .not.exan .and. .not.exsn ) then   ! *** nh4no3 ,nano3 = 0
      if ( exac .and. exsc ) then
        if ( rh>=drmr4 ) then
          scase = 'u3 ; subcase 3'
          call calcmdrpii(rh,drmr4,drnacl,calcu1a,calcu4a)
          scase = 'u3 ; subcase 3'
        end if

      else if ( .not.exac .and. exsc ) then
        if ( rh>=drmr2 ) then
          scase = 'u3 ; subcase 4'
          call calcmdrpii(rh,drmr2,drnacl,calcu1a,calcu4a)
          scase = 'u3 ; subcase 4'
        end if

      else if ( exac .and. .not.exsc ) then
        if ( rh>=drmr5 ) then
          scase = 'u3 ; subcase 5'
          call calcmdrpii(rh,drmr5,drnacl,calcu1a,calcu5)
          scase = 'u3 ; subcase 5'
        end if
      end if
    end if
  end subroutine calcu3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcu3a
! *** case u3a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , nh4cl , nacl
!     4. completely dissolved: nh4no3 , nano3
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu3a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi1o , psi30 , psi40 , psi70 , root1 , root3 , root3a , &
         root3b , root4 , root4a
    real(rkx) :: root4b , root7 , so4i
    integer(ik4) :: i , islv
    logical :: psconv7 , psconv1 , psconv4 , psconv3
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .false.
    psconv1 = .false.
    psconv4 = .false.
    psconv3 = .false.
!
    psi70 = -great                                    ! great = 1.d10
    psi1o = -great
    psi40 = -great
    psi30 = -great
!
    root7 = zero
    root1 = zero
    root4 = zero
    root3 = zero
!
! *** calculate initial solution ***************************************
!
    call calcu1a
!
    chi1 = cna2so4                ! salts
    chi3 = cnacl
    chi4 = cnh4cl
    chi7 = ck2so4
    chi8 = cmgso4
    chi9 = ccaso4
!
    psi1 = cna2so4          ! amount dissolved
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
    molal(1) = zero
    molal(2) = nai
    molal(3) = nh4i
    molal(4) = cli
    molal(5) = so4i
    molal(6) = hso4i
    molal(7) = no3i
    molal(8) = cai
    molal(9) = ki
    molal(10) = mgi
!
    call calcact            ! calculate activity coefficients
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a1 = xk5*(water/gama(2))**3          ! na2s04    <==> na+
      a14 = xk14*(water/gama(6))**2        ! nh4cl     <==> nh4+
      akw = xkw*rh*water*water               ! h2o       <==> h+
      a8 = xk8*(water/gama(1))**2          ! nacl      <==> na+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root1)
        cc = waer(7)*((waer(2)-waer(6))-root1) + 0.25_rkx*waer(7)*waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*((waer(2)-waer(6))-root1)-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = max(root7,zero)
        root7 = min(root7,waer(7)/2.0,max(waer(2)-waer(6)-root1,zero),    &
                chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a1 ) then
        bb = -(((waer(2)-waer(6))-root7)*(waer(1)-root3))
        cc = ((waer(2)-waer(6))-root7)*(waer(1)-root3)                    &
             + 0.25_rkx*(waer(1)-root3)**2.
        dd = -0.25_rkx*(((waer(2)-waer(6))-root7)*(waer(1)-root3)**20-a1)
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = min(max(root1,zero),max(waer(1)-root3,zero),chi1,         &
                max(waer(2)-waer(6),zero))
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ammnium chloride
!
      if ( nh4i*cli>a14 ) then
        bb = -(waer(3)+waer(5)-root4)
        cc = -a14 + nh4i*(waer(5)-root4)
        dd = max(bb*bb-4._rkx*cc,zero)
        root4a = 0.5_rkx*(-bb+sqrt(dd))
        root4b = 0.5_rkx*(-bb-sqrt(dd))
        if ( zero<=root4a ) then
          root4 = root4a
        else
          root4 = root4b
        end if
        root4 = min(max(zero,root4),max(waer(5)-root3,zero),chi4,waer(3))
        psi4 = chi4 - root4
      end if
      psconv4 = abs(psi4-psi40)<=eps*psi40
      psi40 = psi4
!
!   sodum chloride  ; to obtain new value for root3
!
      if ( nai*cli>a8 ) then
        bb = -((chi1-2._rkx*root1)+(waer(5)-root4))
        cc = (chi1-2._rkx*root1)*(waer(5)-root4) - a8
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root3a = 0.5_rkx*(-bb-sqrt(dd))
        root3b = 0.5_rkx*(-bb+sqrt(dd))
        if ( zero<=root3a ) then
          root3 = root3a
        else
          root3 = root3b
        end if
        root3 = min(max(root3,zero),chi3)
        psi3 = chi3 - root3
      end if
      psconv3 = abs(psi3-psi30)<=eps*psi30
      psi30 = psi3
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root7,zero)
      nai = max(waer(1)-2._rkx*root1-root3,zero)
      so4i = max(waer(2)-waer(6)-root7-root1,zero)
      nh4i = max(waer(3)-root4,zero)
      no3i = waer(4)
      cli = max(waer(5)-root4-root3,zero)
      cai = zero
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!   if (hi <= tiny1) hi = sqrt(akw)
!   ohi   = akw/hi
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 .and. psconv1 .and. psconv4 .and. psconv3 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcu3a')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    if ( cli<=tiny1 .and. waer(5)>tiny1 ) then     !no disslv cl-;solid only
      do i = 1 , nions
        molal(i) = zero
      end do
      do i = 1 , ngasaq
        gasaq(i) = zero
      end do
      call calcu1a
    else
      a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2     ! nh3  <==> nh4+
      a3 = xk4*r*temp*(water/gama(10))**2             ! hno3 <==> no3-
      a4 = xk3*r*temp*(water/gama(11))**2             ! hcl  <==> cl-
!
      gnh3 = nh4i/hi/a2
      ghno3 = hi*no3i/a3
      ghcl = hi*cli/a4
!
      gasaq(1) = nh3aq
      gasaq(2) = claq
      gasaq(3) = no3aq
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnh4cl = chi4 - psi4
      cnacl = chi3 - psi3
      cnano3 = zero
      cna2so4 = chi1 - psi1
      cmgso4 = zero
      ck2so4 = chi7 - psi7
      ccaso4 = min(waer(6),waer(2))
    end if
  end subroutine calcu3a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcu2
! *** case u2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), (dust + sodium) rich: r(cr+na)>2; dust poor: rcr<2.
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , nh4cl , nano3 , nacl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu2
    use mod_che_common_isorropia
    implicit none
    logical :: exan , exac , exsn , exsc
!
! *** solve for dry case and see which solids are possible
! **************
    scase = 'u2 ; subcase 2'
    call calcu1a                ! solid
    scase = 'u2 ; subcase 2'
!
    exan = cnh4no3>tiny1        ! nh4no3
    exac = cnh4cl>tiny1         ! nh4cl
    exsn = cnano3>tiny1         ! nano3
    exsc = cnacl>tiny1          ! nacl
!
! *** regime depends on relative humidity and possible species
! **********
    if ( exan ) then                             ! *** nh4no3 exists
      if ( rh>=drmm1 ) then
        scase = 'u2 ; subcase 1'
        call calcu2a
        scase = 'u2 ; subcase 1'
      end if
!
    else if ( .not.exan ) then                   ! *** nh4no3 = 0
      if ( exac .and. exsn .and. exsc ) then
        if ( rh>=drmm2 ) then
          scase = 'u2 ; subcase 3'
          call calcmdrpii(rh,drmm2,drnano3,calcu1a,calcu3a)
          scase = 'u2 ; subcase 3'
        end if

      else if ( .not.exac .and. exsn .and. exsc ) then
        if ( rh>=drmr1 ) then
          scase = 'u2 ; subcase 4'
          call calcmdrpii(rh,drmr1,drnano3,calcu1a,calcu3a)
          scase = 'u2 ; subcase 4'
        end if

      else if ( .not.exac .and. .not.exsn .and. exsc ) then
        if ( rh>=drmr2 ) then
          scase = 'u2 ; subcase 5'
          call calcmdrpii(rh,drmr2,drnacl,calcu1a,calcu4a)
          scase = 'u2 ; subcase 5'
        end if

      else if ( .not.exac .and. exsn .and. .not.exsc ) then
        if ( rh>=drmr3 ) then
          scase = 'u2 ; subcase 6'
          call calcmdrpii(rh,drmr3,drnano3,calcu1a,calcu3a)
          scase = 'u2 ; subcase 6'
        end if

      else if ( exac .and. .not.exsn .and. exsc ) then
        if ( rh>=drmr4 ) then
          scase = 'u2 ; subcase 7'
          call calcmdrpii(rh,drmr4,drnacl,calcu1a,calcu4a)
          scase = 'u2 ; subcase 7'
        end if

      else if ( exac .and. .not.exsn .and. .not.exsc ) then
        if ( rh>=drmr5 ) then
          scase = 'u2 ; subcase 8'
          call calcmdrpii(rh,drmr5,drnh4cl,calcu1a,calcu5)
          scase = 'u2 ; subcase 8'
        end if

      else if ( exac .and. exsn .and. .not.exsc ) then
        if ( rh>=drmr6 ) then
          scase = 'u2 ; subcase 9'
          call calcmdrpii(rh,drmr6,drnano3,calcu1a,calcu3a)
          scase = 'u2 ; subcase 9'
        end if
      end if
!
    end if
!

! if (w(4) > tiny1) then        ! no3 exists , water possible
! scase = 'u2 ; subcase 1'
! call calcu2a
! scase = 'u2 ; subcase 1'
! else                          ! no3 non existant , water not possible
! scase = 'u2 ; subcase 1'
! call calcu1a
! scase = 'u2 ; subcase 1'
! end if
!
! if (water <= tiny1 .and. rh < drmm2) then      ! dry aerosol
! scase = 'u2 ; subcase 2'
! call calcu2a
! scase = 'u2 ; subcase 1'
!
! else if (water <= tiny1 .and. rh >= drmm2) then  ! mdrh of m2
! scase = 'u2 ; subcase 3'
! call calcmdrpii (rh , drmm2 , drnano3 , calcu1a , calcu3a)
! scase = 'u2 ; subcase 3'
! end if
!
  end subroutine calcu2
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcu2a
! *** case u2a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), cr+na poor (crnarat < 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , nh4cl , nacl , nano3
!     4. completely dissolved: nh4no3
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu2a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi1o , psi20 , psi30 , psi40 , psi50 , psi70 , root1 , &
         root3 , root3a , root3b
    real(rkx) :: root4 , root4a , root4b , root5 , root5a , root5b , &
         root7 , so4i
    integer(ik4) :: i , islv
    logical :: psconv7 , psconv1 , psconv4 , psconv3 , psconv5
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv7 = .false.
    psconv1 = .false.
    psconv4 = .false.
    psconv3 = .false.
    psconv5 = .false.
!
    psi70 = -great                                    ! great = 1.d10
    psi1o = -great
    psi40 = -great
    psi30 = -great
    psi50 = -great
!
    root7 = zero
    root1 = zero
    root4 = zero
    root3 = zero
    root5 = zero
!
! *** calculate initial solution ***************************************
!
    call calcu1a
!
    chi1 = cna2so4                ! salts
    chi2 = cnano3
    chi3 = cnacl
    chi4 = cnh4cl
    chi7 = ck2so4
    chi8 = cmgso4
    chi9 = ccaso4
!
    psi1 = cna2so4          ! amount dissolved
    psi2 = cnano3
    psi3 = cnacl
    psi4 = cnh4cl
    psi5 = cnh4no3
    psi7 = ck2so4
    psi8 = cmgso4
    psi9 = ccaso4
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
    psi20 = zero
!
    molal(1) = zero
    molal(2) = nai
    molal(3) = nh4i
    molal(4) = cli
    molal(5) = so4i
    molal(6) = hso4i
    molal(7) = no3i
    molal(8) = cai
    molal(9) = ki
    molal(10) = mgi
!
    call calcact            ! calculate activity coefficients
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a7 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a1 = xk5*(water/gama(2))**3          ! na2s04    <==> na+
      a14 = xk14*(water/gama(6))**2        ! nh4cl     <==> nh4+
      a8 = xk8*(water/gama(1))**2          ! nacl      <==> na+
      a9 = xk9*(water/gama(3))**2          ! nano3     <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a7 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root1)
        cc = waer(7)*((waer(2)-waer(6))-root1) + 0.25_rkx*waer(7)*waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*((waer(2)-waer(6))-root1)-a7)
        call poly3(bb,cc,dd,root7,islv)
        if ( islv/=0 ) root7 = tiny1
        root7 = max(root7,zero)
        root7 = min(root7,waer(7)/2.0,max(waer(2)-waer(6)-root1,zero),    &
                chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi70)<=eps*psi70
      psi70 = psi7
!
!   sodum sulfate
!
      if ( nai*nai*so4i>a1 ) then
        bb = -(((waer(2)-waer(6))-root7)*(waer(1)-root3-root5))
        cc = ((waer(2)-waer(6))-root7)*(waer(1)-root3-root5)              &
             + 0.25_rkx*(waer(1)-root3-root5)**2
        dd = -0.25_rkx*(((waer(2)-waer(6))-root7)*(waer(1)-root3-root5)     &
             **20-a1)
        call poly3(bb,cc,dd,root1,islv)
        if ( islv/=0 ) root1 = tiny1
        root1 = min(max(root1,zero),max(waer(1)-root3-root5,zero),chi1,   &
                max(waer(2)-waer(6),zero))
        psi1 = chi1 - root1
      end if
      psconv1 = abs(psi1-psi1o)<=eps*psi1o
      psi1o = psi1
!
!   ammnium chloride
!
      if ( nh4i*cli>a14 ) then
        bb = -(waer(3)+waer(5)-root4)
        cc = -a14 + nh4i*(waer(5)-root4)
        dd = max(bb*bb-4._rkx*cc,zero)
        root4a = 0.5_rkx*(-bb+sqrt(dd))
        root4b = 0.5_rkx*(-bb-sqrt(dd))
        if ( zero<=root4a ) then
          root4 = root4a
        else
          root4 = root4b
        end if
        root4 = min(max(zero,root4),max(waer(5)-root3,zero),chi4,waer(3))
        psi4 = chi4 - root4
      end if
      psconv4 = abs(psi4-psi40)<=eps*psi40
      psi40 = psi4
!
!   sodum chloride  ; to obtain new value for root3
!
      if ( nai*cli>a8 ) then
        bb = -((chi1-2._rkx*root1-root5)+(waer(5)-root4))
        cc = (chi1-2._rkx*root1-root5)*(waer(5)-root4) - a8
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root3a = 0.5_rkx*(-bb-sqrt(dd))
        root3b = 0.5_rkx*(-bb+sqrt(dd))
        if ( zero<=root3a ) then
          root3 = root3a
        else
          root3 = root3b
        end if
        root3 = min(max(root3,zero),chi3)
        psi3 = chi3 - root3
      end if
      psconv3 = abs(psi3-psi30)<=eps*psi30
      psi30 = psi3
!
!   sodum nitrate
!
      if ( nai*no3i>a9 ) then
        bb = -(waer(4)+waer(1)-2._rkx*root1-root3)
        cc = waer(4)*(waer(1)-2._rkx*root1-root3) - a9
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root5a = 0.5_rkx*(-bb-dd)
        root5b = 0.5_rkx*(-bb+dd)
        if ( zero<=root5a ) then
          root5 = root5a
        else
          root5 = root5b
        end if
        root5 = min(max(root5,zero),chi2)
        psi2 = chi2 - root5
      end if
!
      psconv5 = abs(psi2-psi20)<=eps*psi20
      psi20 = psi2
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2.0_rkx*root7,zero)
      nai = max(waer(1)-2.0_rkx*root1-root3-root5,zero)
      so4i = max(waer(2)-waer(6)-root7-root1,zero)
      nh4i = max(waer(3)-root4,zero)
      no3i = max(waer(4)-root5,zero)
      cli = max(waer(5)-root4-root3,zero)
      cai = zero
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!   if (hi <= tiny1) hi = sqrt(akw)
!   ohi   = akw/hi
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv7 .and. psconv1 .and. psconv4 .and. psconv3 .and.   &
                psconv5 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcu2a')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    if ( cli<=tiny1 .and. waer(5)>tiny1 ) then     !no disslv cl-;solid only
      do i = 1 , nions
        molal(i) = zero
      end do
      do i = 1 , ngasaq
        gasaq(i) = zero
      end do
      call calcu1a
    else                                       ! ok , aqueous phase present
      a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2     ! nh3  <==> nh4+
      a3 = xk4*r*temp*(water/gama(10))**2             ! hno3 <==> no3-
      a4 = xk3*r*temp*(water/gama(11))**2             ! hcl  <==> cl-
!
      gnh3 = nh4i/hi/a2
      ghno3 = hi*no3i/a3
      ghcl = hi*cli/a4
!
      gasaq(1) = nh3aq
      gasaq(2) = claq
      gasaq(3) = no3aq
!
      cnh42s4 = zero
      cnh4no3 = zero
      cnh4cl = chi4 - psi4
      cnacl = chi3 - psi3
      cnano3 = chi2 - psi2
      cna2so4 = chi1 - psi1
      cmgso4 = zero
      ck2so4 = chi7 - psi7
      ccaso4 = min(waer(6),waer(2))
    end if
  end subroutine calcu2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcu1
! *** case u1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (so4rat > 2.0), (dust + sodium) rich: r(cr+na)>2; dust poor: rcr<2.
!     2. there is both a liquid & solid phase
!     3. solids possible : k2so4 , caso4 , na2so4 , mgso4 , nh4cl , nano3 , nacl , nh4no3
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu1
    use mod_che_common_isorropia
    implicit none
    logical :: exan , exac , exsn , exsc
!
! *** solve for dry case and see which solids are possible
! **************
    scase = 'u1 ; subcase 1'
    call calcu1a                ! solid
    scase = 'u1 ; subcase 1'
    exan = cnh4no3>tiny1        ! nh4no3
    exac = cnh4cl>tiny1         ! nh4cl
    exsn = cnano3>tiny1         ! nano3
    exsc = cnacl>tiny1          ! nacl
!
! *** regime depends on relative humidity and possible species
! **********
    if ( exan .or. exac .or. exsc .or. exsn ) then
                                              ! *** water possible
      if ( rh>=drmm1 ) then
        scase = 'u1 ; subcase 2'      ! mdrh
        call calcmdrpii(rh,drmm1,drnh4no3,calcu1a,calcu2a)
        scase = 'u1 ; subcase 2'
      end if
    else if ( .not.exan ) then                   ! *** nh4no3 = 0
      if ( exac .and. exsn .and. exsc ) then
        if ( rh>=drmm2 ) then
          scase = 'u1 ; subcase 3'
          call calcmdrpii(rh,drmm2,drnano3,calcu1a,calcu3a)
          scase = 'u1 ; subcase 3'
        end if
      else if ( .not.exac .and. exsn .and. exsc ) then
        if ( rh>=drmr1 ) then
          scase = 'u1 ; subcase 4'
          call calcmdrpii(rh,drmr1,drnano3,calcu1a,calcu3a)
          scase = 'u1 ; subcase 4'
        end if
      else if ( .not.exac .and. .not.exsn .and. exsc ) then
        if ( rh>=drmr2 ) then
          scase = 'u1 ; subcase 5'
          call calcmdrpii(rh,drmr2,drnacl,calcu1a,calcu3a)               !, calcr4a)
          scase = 'u1 ; subcase 5'
        end if
      else if ( .not.exac .and. exsn .and. .not.exsc ) then
        if ( rh>=drmr3 ) then
          scase = 'u1 ; subcase 6'
          call calcmdrpii(rh,drmr3,drnano3,calcu1a,calcu3a)
          scase = 'u1 ; subcase 6'
        end if
      else if ( exac .and. .not.exsn .and. exsc ) then
        if ( rh>=drmr4 ) then
          scase = 'u1 ; subcase 7'
          call calcmdrpii(rh,drmr4,drnacl,calcu1a,calcu3a)               !, calcr4a)
          scase = 'u1 ; subcase 7'
        end if
      else if ( exac .and. .not.exsn .and. .not.exsc ) then
        if ( rh>=drmr5 ) then
          scase = 'u1 ; subcase 8'
          call calcmdrpii(rh,drmr5,drnh4cl,calcu1a,calcu3a)               !, calcr5)
          scase = 'u1 ; subcase 8'
        end if
      else if ( exac .and. exsn .and. .not.exsc ) then
        if ( rh>=drmr6 ) then
          scase = 'u1 ; subcase 9'
          call calcmdrpii(rh,drmr6,drnano3,calcu1a,calcu3a)
          scase = 'u1 ; subcase 9'
        end if
      end if
    else if ( .not.exac ) then                   ! *** nh4cl  = 0
      if ( exan .and. exsn .and. exsc ) then
        if ( rh>=drmr7 ) then
          scase = 'u1 ; subcase 10'
          call calcmdrpii(rh,drmr7,drnh4no3,calcu1a,calcu2a)
          scase = 'u1 ; subcase 10'
        end if
      else if ( exan .and. .not.exsn .and. exsc ) then
        if ( rh>=drmr8 ) then
          scase = 'u1 ; subcase 11'
          call calcmdrpii(rh,drmr8,drnh4no3,calcu1a,calcu2a)
          scase = 'u1 ; subcase 11'
        end if
      else if ( exan .and. .not.exsn .and. .not.exsc ) then
        if ( rh>=drmr9 ) then
          scase = 'u1 ; subcase 12'
          call calcmdrpii(rh,drmr9,drnh4no3,calcu1a,calcu2a)
          scase = 'u1 ; subcase 12'
        end if
      else if ( exan .and. exsn .and. .not.exsc ) then
        if ( rh>=drmr10 ) then
          scase = 'u1 ; subcase 13'
          call calcmdrpii(rh,drmr10,drnh4no3,calcu1a,calcu2a)
          scase = 'u1 ; subcase 13'
        end if
      end if
    else if ( .not.exsn ) then                  ! *** nano3  = 0
      if ( exan .and. exac .and. exsc ) then
        if ( rh>=drmr11 ) then
          scase = 'u1 ; subcase 14'
          call calcmdrpii(rh,drmr11,drnh4no3,calcu1a,calcu2a)
          scase = 'u1 ; subcase 14'
        end if
      else if ( exan .and. exac .and. .not.exsc ) then
        if ( rh>=drmr12 ) then
          scase = 'u1 ; subcase 15'
          call calcmdrpii(rh,drmr12,drnh4no3,calcu1a,calcu2a)
          scase = 'u1 ; subcase 15'
        end if
      end if
    else if ( .not.exsc ) then                  ! *** nacl   = 0
      if ( exan .and. exac .and. exsn ) then
        if ( rh>=drmr13 ) then
          scase = 'u1 ; subcase 16'
          call calcmdrpii(rh,drmr13,drnh4no3,calcu1a,calcu2a)
          scase = 'u1 ; subcase 16'
        end if
      end if
    end if
! if (rh < drmm1) then
! scase = 'u1 ; subcase 1'
! call calcu1a              ! solid phase only possible
! scase = 'u1 ; subcase 1'
! else
! scase = 'u1 ; subcase 2'  ! liquid & solid phase possible
! call calcmdrpii (rh , drmm1 , drnh4no3 , calcu1a , calcu2a)
! scase = 'u1 ; subcase 2'
! end if
  end subroutine calcu1
!
!======================================================================
!
! *** isorropia code
! *** subroutine calcu1a
! *** case u1a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0); crustal+sodium rich (crnarat >= 2.0); crustal poor (crrat<2)
!     2. there is only a solid phase
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcu1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: cafr , frcl , frk , frmg , frna , frnh3 , frno3 , so4fr
!
! *** calculate solids *************************************************
!
    ccaso4 = min(waer(6),waer(2))                      ! ccaso4
    so4fr = max(waer(2)-ccaso4,zero)
    cafr = max(waer(6)-ccaso4,zero)
    ck2so4 = min(0.5_rkx*waer(7),so4fr)                  ! ck2so4
    frk = max(waer(7)-2._rkx*ck2so4,zero)
    so4fr = max(so4fr-ck2so4,zero)
    cmgso4 = min(waer(8),so4fr)                        ! cmgso4
    frmg = max(waer(8)-cmgso4,zero)
    so4fr = max(so4fr-cmgso4,zero)
    cna2so4 = max(so4fr,zero)                           ! cna2so4
    frna = max(waer(1)-2._rkx*cna2so4,zero)
!
    cnh42s4 = zero
!
    cnano3 = min(frna,waer(4))
    frno3 = max(waer(4)-cnano3,zero)
    frna = max(frna-cnano3,zero)
!
    cnacl = min(frna,waer(5))
    frcl = max(waer(5)-cnacl,zero)
    frna = max(frna-cnacl,zero)
!
    cnh4no3 = min(frno3,waer(3))
    frno3 = max(frno3-cnh4no3,zero)
    frnh3 = max(waer(3)-cnh4no3,zero)
!
    cnh4cl = min(frcl,frnh3)
    frcl = max(frcl-cnh4cl,zero)
    frnh3 = max(frnh3-cnh4cl,zero)
!
! *** other phases
! ******************************************************
    water = zero
!
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
  end subroutine calcu1a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw13
! *** case w13
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4
!     4. completely dissolved: ca(no3)2 , cacl2 , k2so4 , kno3 , kcl , mgso4 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcw13
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , so4i , kcl
    integer(ik4) :: i
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nh3aq = zero
    no3aq = zero
    claq = zero
    no3i = zero
    nh4i = zero
    cli = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep

      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   ion concentrations
!
      nai = waer(1)
      so4i = max(waer(2)-waer(6),zero)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      ki = waer(7)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( .not.(frst .and. calaou .or. .not.frst .and. calain) ) exit
      call calcact
    end do
! call pusherr (0002 , 'calcw13')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cmgso4 = zero
    ck2so4 = zero
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = zero
    kcl = zero
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw13
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw12
! *** case w12
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4
!     4. completely dissolved: ca(no3)2 , cacl2 , kno3 , kcl , mgso4 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw12
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi9o , root9 , so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psi9o = -great                                    ! great = 1.d10
    root9 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi9 = ck2so4           ! salts
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7))
        cc = waer(7)*(waer(2)-waer(6)) + 0.25_rkx*waer(7)*waer(7)
        dd = -0.25_rkx*(waer(7)*waer(7)*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0,(waer(2)-waer(6)),chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = waer(3)
      no3i = waer(4)
      cli = waer(5)
      cai = zero
      nai = waer(1)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw12')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = zero
    kcl = zero
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw12
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw11
! *** case w11
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3
!     4. completely dissolved: ca(no3)2 , cacl2 , kcl , mgso4 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw11
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi13o , psi9o , root13 , root13a , root13b , root9 , &
         so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9 , psconv13
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psconv13 = .true.
!
    psi9o = -great
    psi13o = -great                                  ! great = 1.d10
!
    root9 = zero
    root13 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi9 = ck2so4           ! salts
    chi13 = ckno3
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a13 = xk19*(water/gama(19))**2       ! kno3      <==> k+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root13)
        cc = (waer(7)-root13)*(waer(2)-waer(6)) + 0.25_rkx*(waer(7)-root13) &
             **2
        dd = -0.25_rkx*((waer(7)-root13)**2*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0-root13,(waer(2)-waer(6)),chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   potssium nitrate
!
      if ( ki*no3i>a13 ) then
        bb = -(waer(4)+waer(7)-2._rkx*root9)
        cc = waer(4)*(waer(7)-2._rkx*root9) - a13
        dd = sqrt(max(bb*bb-4._rkx*cc,zero))
        root13a = 0.5_rkx*(-bb-dd)
        root13b = 0.5_rkx*(-bb+dd)
        if ( zero<=root13a ) then
          root13 = root13a
        else
          root13 = root13b
        end if
        root13 = min(max(root13,zero),chi13)
        psi13 = chi13 - root13
      end if
      psconv13 = abs(psi13-psi13o)<=eps*psi13o
      psi13o = psi13
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9-root13,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = waer(3)
      no3i = max(waer(4)-root13,zero)
      cli = waer(5)
      cai = zero
      nai = waer(1)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 .and. psconv13 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw11')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = chi13 - psi13
    kcl = zero
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw11
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw10
! *** case w10
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4
!     4. completely dissolved: ca(no3)2 , cacl2 , kcl ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw10
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi13o , psi9o , root13 , root13a , root13b , root9 , &
         so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9 , psconv13
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psconv13 = .true.
!
    psi9o = -great
    psi13o = -great                                  ! great = 1.d10
!
    root9 = zero
    root13 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a

!
    chi9 = ck2so4           ! salts
    chi13 = ckno3
    chi10 = cmgso4
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a13 = xk19*(water/gama(19))**2       ! kno3      <==> k+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root13)
        cc = (waer(7)-root13)*(waer(2)-waer(6)) + 0.25_rkx*(waer(7)-root13) &
             **2
        dd = -0.25_rkx*((waer(7)-root13)**2*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0-root13,(waer(2)-waer(6)),chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   potssium nitrate
!
      if ( ki*no3i>a13 ) then
        bb = -(waer(4)+waer(7)-2._rkx*root9)
        cc = waer(4)*(waer(7)-2._rkx*root9) - a13
        dd = sqrt(max(bb*bb-4._rkx*cc,zero))
        root13a = 0.5_rkx*(-bb-dd)
        root13b = 0.5_rkx*(-bb+dd)
        if ( zero<=root13a ) then
          root13 = root13a
        else
          root13 = root13b
        end if
        root13 = min(max(root13,zero),chi13)
        psi13 = chi13 - root13
      end if
      psconv13 = abs(psi13-psi13o)<=eps*psi13o
      psi13o = psi13
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9-root13,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = waer(3)
      no3i = max(waer(4)-root13,zero)
      cli = waer(5)
      cai = zero
      nai = waer(1)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 .and. psconv13 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw10')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = chi13 - psi13
    kcl = zero
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw10
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw9
! *** case w9
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw9
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi13o , psi14o , psi9o , root13 , root13a , root13b , &
         root14 , root14a , root14b , root9 , so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9 , psconv13 , psconv14
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psconv13 = .true.
    psconv14 = .true.
!
    psi9o = -great
    psi13o = -great
    psi14o = -great                                ! great = 1.d10
!
    root9 = zero
    root13 = zero
    root14 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi9 = ck2so4           ! salts
    chi13 = ckno3
    chi10 = cmgso4
    chi14 = ckcl
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a13 = xk19*(water/gama(19))**2       ! kno3      <==> k+
      a14 = xk20*(water/gama(20))**2       ! kcl       <==> k+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root13-root14)
        cc = (waer(7)-root13-root14)*(waer(2)-waer(6))                    &
             + 0.25_rkx*(waer(7)-root13-root14)**2
        dd = -0.25_rkx*((waer(7)-root13-root14)**2*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0-root13-root14,(waer(2)-waer(6)),    &
                chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   potssium nitrate
!
      if ( ki*no3i>a13 ) then
        bb = -(waer(4)+waer(7)-2._rkx*root9-root14)
        cc = waer(4)*(waer(7)-2._rkx*root9-root14) - a13
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root13a = 0.5_rkx*(-bb-dd)
        root13b = 0.5_rkx*(-bb+dd)
        if ( zero<=root13a ) then
          root13 = root13a
        else
          root13 = root13b
        end if
        root13 = min(max(root13,zero),chi13)
        psi13 = chi13 - root13
      end if
      psconv13 = abs(psi13-psi13o)<=eps*psi13o
      psi13o = psi13
!
!   potssium cloride
!
      if ( ki*cli>a14 ) then
        bb = -(waer(5)+waer(7)-2._rkx*root9-root13)
        cc = waer(5)*(waer(7)-2._rkx*root9-root13) - a14
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root14a = 0.5_rkx*(-bb-dd)
        root14b = 0.5_rkx*(-bb+dd)
        if ( zero<=root14a ) then
          root14 = root14a
        else
          root14 = root14b
        end if
        root14 = min(max(root14,zero),chi14)
        psi14 = chi14 - root14
      end if
      psconv14 = abs(psi14-psi14o)<=eps*psi14o
      psi14o = psi14
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9-root13-root14,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = waer(3)
      no3i = max(waer(4)-root13,zero)
      cli = max(waer(5)-root14,zero)
      cai = zero
      nai = waer(1)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 .and. psconv13 .and. psconv14 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw9')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = zero
    cnacl = zero
    cnano3 = zero
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = chi13 - psi13
    kcl = chi14 - psi14
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw9
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw8
! *** case w8
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw8
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi13o , psi14o , psi5o , psi9o , root13 , root13a , &
         root13b , root14 , root14a , root14b
    real(rkx) :: root5 , root5a , root5b , root9 , so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9 , psconv13 , psconv14 , psconv5
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psconv13 = .true.
    psconv14 = .true.
    psconv5 = .true.
!
    psi9o = -great
    psi13o = -great
    psi14o = -great
    psi5o = -great                                 ! great = 1.d10
!
    root9 = zero
    root13 = zero
    root14 = zero
    root5 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi9 = ck2so4           ! salts
    chi13 = ckno3
    chi10 = cmgso4
    chi14 = ckcl
    chi5 = cnh4cl
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a13 = xk19*(water/gama(19))**2       ! kno3      <==> k+
      a14 = xk20*(water/gama(20))**2       ! kcl       <==> k+
      a5 = xk14*(water/gama(6))**2         ! nh4cl     <==> nh4+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root13-root14)
        cc = (waer(7)-root13-root14)*(waer(2)-waer(6))                    &
             + 0.25_rkx*(waer(7)-root13-root14)**2
        dd = -0.25_rkx*((waer(7)-root13-root14)**2*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0-root13-root14,(waer(2)-waer(6)),    &
                chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   potssium nitrate
!
      if ( ki*no3i>a13 ) then
        bb = -(waer(4)+waer(7)-2._rkx*root9-root14)
        cc = waer(4)*(waer(7)-2._rkx*root9-root14) - a13
        dd = sqrt(max(bb*bb-4._rkx*cc,zero))
        root13a = 0.5_rkx*(-bb-dd)
        root13b = 0.5_rkx*(-bb+dd)
        if ( zero<=root13a ) then
          root13 = root13a
        else
          root13 = root13b
        end if
        root13 = min(max(root13,zero),chi13)
        psi13 = chi13 - root13
      end if
      psconv13 = abs(psi13-psi13o)<=eps*psi13o
      psi13o = psi13
!
!   potssium cloride
!
      if ( ki*cli>a14 ) then
        bb = -(waer(5)-root5+waer(7)-2._rkx*root9-root13)
        cc = (waer(5)-root5)*(waer(7)-2._rkx*root9-root13) - a14
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root14a = 0.5_rkx*(-bb-dd)
        root14b = 0.5_rkx*(-bb+dd)
        if ( zero<=root14a ) then
          root14 = root14a
        else
          root14 = root14b
        end if
        root14 = min(max(root14,zero),chi14)
        psi14 = chi14 - root14
      end if
      psconv14 = abs(psi14-psi14o)<=eps*psi14o
      psi14o = psi14
!
!   ammnium cloride
!
      if ( nh4i*cli>a5 ) then
        bb = -(waer(5)+waer(3)-root14)
        cc = (waer(5)-root14)*waer(3) - a5
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root5a = 0.5_rkx*(-bb-dd)
        root5b = 0.5_rkx*(-bb+dd)
        if ( zero<=root5a ) then
          root5 = root5a
        else
          root5 = root5b
        end if
        root5 = min(max(root5,zero),chi5)
        psi5 = chi5 - root5
      end if
      psconv5 = abs(psi5-psi5o)<=eps*psi5o
      psi5o = psi5
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9-root13-root14,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = max(waer(3)-root5,zero)
      no3i = max(waer(4)-root13,zero)
      cli = max(waer(5)-root14-root5,zero)
      cai = zero
      nai = waer(1)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 .and. psconv13 .and. psconv14 .and. psconv5 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw8')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = chi5 - psi5
    cnacl = zero
    cnano3 = zero
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = chi13 - psi13
    kcl = chi14 - psi14
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw8
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw7
! *** case w7
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nano3 , nh4no3
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw7
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi13o , psi14o , psi5o , psi7o , psi9o , root13 , &
         root13a , root13b , root14 , root14a
    real(rkx) :: root14b , root5 , root5a , root5b , root7 , root7a , &
         root7b , root9 , so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9 , psconv13 , psconv14 , psconv5 , psconv7
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psconv13 = .true.
    psconv14 = .true.
    psconv5 = .true.
    psconv7 = .true.
!
    psi9o = -great
    psi13o = -great
    psi14o = -great
    psi5o = -great
    psi7o = -great                              ! great = 1.d10
!
    root9 = zero
    root13 = zero
    root14 = zero
    root5 = zero
    root7 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi9 = ck2so4           ! salts
    chi13 = ckno3
    chi10 = cmgso4
    chi14 = ckcl
    chi5 = cnh4cl
    chi7 = cnacl
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a13 = xk19*(water/gama(19))**2       ! kno3      <==> k+
      a14 = xk20*(water/gama(20))**2       ! kcl       <==> k+
      a5 = xk14*(water/gama(6))**2         ! nh4cl     <==> nh4+
      a7 = xk8*(water/gama(1))**2          ! nacl      <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potassium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root13-root14)
        cc = (waer(7)-root13-root14)*(waer(2)-waer(6))                    &
             + 0.25_rkx*(waer(7)-root13-root14)**2
        dd = -0.25_rkx*((waer(7)-root13-root14)**2*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0-root13-root14,(waer(2)-waer(6)),    &
                chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   potssium nitrate
!
      if ( ki*no3i>a13 ) then
        bb = -(waer(4)+waer(7)-2._rkx*root9-root14)
        cc = waer(4)*(waer(7)-2._rkx*root9-root14) - a13
        dd = sqrt(max(bb*bb-4._rkx*cc,zero))
        root13a = 0.5_rkx*(-bb-dd)
        root13b = 0.5_rkx*(-bb+dd)
        if ( zero<=root13a ) then
          root13 = root13a
        else
          root13 = root13b
        end if
        root13 = min(max(root13,zero),chi13)
        psi13 = chi13 - root13
      end if
      psconv13 = abs(psi13-psi13o)<=eps*psi13o
      psi13o = psi13
!
!   potssium cloride
!
      if ( ki*cli>a14 ) then
        bb = -(waer(5)-root5-root7+waer(7)-2._rkx*root9-root13)
        cc = (waer(5)-root5-root7)*(waer(7)-2._rkx*root9-root13) - a14
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root14a = 0.5_rkx*(-bb-dd)
        root14b = 0.5_rkx*(-bb+dd)
        if ( zero<=root14a ) then
          root14 = root14a
        else
          root14 = root14b
        end if
        root14 = min(max(root14,zero),chi14)
        psi14 = chi14 - root14
      end if
      psconv14 = abs(psi14-psi14o)<=eps*psi14o
      psi14o = psi14
!
!   ammnium cloride
!
      if ( nh4i*cli>a5 ) then
        bb = -(waer(5)+waer(3)-root14-root7)
        cc = (waer(5)-root14-root7)*waer(3) - a5
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root5a = 0.5_rkx*(-bb-dd)
        root5b = 0.5_rkx*(-bb+dd)
        if ( zero<=root5a ) then
          root5 = root5a
        else
          root5 = root5b
        end if
        root5 = min(max(root5,zero),chi5)
        psi5 = chi5 - root5
      end if
      psconv5 = abs(psi5-psi5o)<=eps*psi5o
      psi5o = psi5
!
!   sodum cloride
!
      if ( nai*cli>a7 ) then
        bb = -(waer(5)+waer(1)-root14-root5)
        cc = (waer(5)-root14-root5)*waer(1) - a7
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root7a = 0.5_rkx*(-bb-dd)
        root7b = 0.5_rkx*(-bb+dd)
        if ( zero<=root7a ) then
          root7 = root7a
        else
          root7 = root7b
        end if
        root7 = min(max(root7,zero),chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi7o)<=eps*psi7o
      psi7o = psi7
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9-root13-root14,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = max(waer(3)-root5,zero)
      no3i = max(waer(4)-root13,zero)
      cli = max(waer(5)-root14-root5-root7,zero)
      cai = zero
      nai = max(waer(1)-root7,zero)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 .and. psconv13 .and. psconv14 .and. psconv5 .and. &
                psconv7 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw7')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = chi5 - psi5
    cnacl = chi7 - psi7
    cnano3 = zero
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = chi13 - psi13
    kcl = chi14 - psi14
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw7
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw6
! *** case w6
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl , nano3
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2 , nh4no3
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw6
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi13o , psi14o , psi5o , psi7o , psi8o , psi9o , &
         root13 , root13a , root13b , root14
    real(rkx) :: root14a , root14b , root5 , root5a , root5b , root7 , &
         root7a , root7b , root8 , root8a , root8b , root9 , so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9 , psconv13 , psconv14 , psconv5 , psconv7 , psconv8
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psconv13 = .true.
    psconv14 = .true.
    psconv5 = .true.
    psconv7 = .true.
    psconv8 = .true.
!
    psi9o = -great
    psi13o = -great
    psi14o = -great
    psi5o = -great
    psi7o = -great
    psi8o = -great                        ! great = 1.d10
!
    root9 = zero
    root13 = zero
    root14 = zero
    root5 = zero
    root7 = zero
    root8 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi9 = ck2so4           ! salts
    chi13 = ckno3
    chi10 = cmgso4
    chi14 = ckcl
    chi5 = cnh4cl
    chi7 = cnacl
    chi8 = cnano3
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a13 = xk19*(water/gama(19))**2       ! kno3      <==> k+
      a14 = xk20*(water/gama(20))**2       ! kcl       <==> k+
      a5 = xk14*(water/gama(6))**2         ! nh4cl     <==> nh4+
      a7 = xk8*(water/gama(1))**2          ! nacl      <==> na+
      a8 = xk9*(water/gama(3))**2          ! nano3     <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root13-root14)
        cc = (waer(7)-root13-root14)*(waer(2)-waer(6))                    &
             + 0.25_rkx*(waer(7)-root13-root14)**2
        dd = -0.25_rkx*((waer(7)-root13-root14)**2*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0-root13-root14,(waer(2)-waer(6)),    &
                chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   potssium nitrate
!
      if ( ki*no3i>a13 ) then
        bb = -(waer(4)-root8+waer(7)-2._rkx*root9-root14)
        cc = (waer(4)-root8)*(waer(7)-2._rkx*root9-root14) - a13
        dd = sqrt(max(bb*bb-4._rkx*cc,zero))
        root13a = 0.5_rkx*(-bb-dd)
        root13b = 0.5_rkx*(-bb+dd)
        if ( zero<=root13a ) then
          root13 = root13a
        else
          root13 = root13b
        end if
        root13 = min(max(root13,zero),chi13)
        psi13 = chi13 - root13
      end if
      psconv13 = abs(psi13-psi13o)<=eps*psi13o
      psi13o = psi13
!
!   potssium cloride
!
      if ( ki*cli>a14 ) then
        bb = -(waer(5)-root5-root7+waer(7)-2._rkx*root9-root13)
        cc = (waer(5)-root5-root7)*(waer(7)-2._rkx*root9-root13) - a14
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root14a = 0.5_rkx*(-bb-dd)
        root14b = 0.5_rkx*(-bb+dd)
        if ( zero<=root14a ) then
          root14 = root14a
        else
          root14 = root14b
        end if
        root14 = min(max(root14,zero),chi14)
        psi14 = chi14 - root14
      end if
      psconv14 = abs(psi14-psi14o)<=eps*psi14o
      psi14o = psi14
!
!   ammnium cloride
!
      if ( nh4i*cli>a5 ) then
        bb = -(waer(5)+waer(3)-root14-root7)
        cc = (waer(5)-root14-root7)*waer(3) - a5
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root5a = 0.5_rkx*(-bb-dd)
        root5b = 0.5_rkx*(-bb+dd)
        if ( zero<=root5a ) then
          root5 = root5a
        else
          root5 = root5b
        end if
        root5 = min(max(root5,zero),chi5)
        psi5 = chi5 - root5
      end if
      psconv5 = abs(psi5-psi5o)<=eps*psi5o
      psi5o = psi5
!
!   sodum cloride
!
      if ( nai*cli>a7 ) then
        bb = -(waer(5)+waer(1)-root8-root14-root5)
        cc = (waer(5)-root14-root5)*(waer(1)-root8) - a7
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root7a = 0.5_rkx*(-bb-dd)
        root7b = 0.5_rkx*(-bb+dd)
        if ( zero<=root7a ) then
          root7 = root7a
        else
          root7 = root7b
        end if
        root7 = min(max(root7,zero),chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi7o)<=eps*psi7o
      psi7o = psi7
!
!   sodum nitrate
!
      if ( nai*no3i>a8 ) then
        bb = -(waer(4)-root13+waer(1)-root7)
        cc = (waer(4)-root13)*(waer(1)-root7) - a8
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root8a = 0.5_rkx*(-bb-dd)
        root8b = 0.5_rkx*(-bb+dd)
        if ( zero<=root8a ) then
          root8 = root8a
        else
          root8 = root8b
        end if
        root8 = min(max(root8,zero),chi8)
        psi8 = chi8 - root8
      end if
      psconv8 = abs(psi8-psi8o)<=eps*psi8o
      psi8o = psi8
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9-root13-root14,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = max(waer(3)-root5,zero)
      no3i = max(waer(4)-root13-root8,zero)
      cli = max(waer(5)-root14-root5-root7,zero)
      cai = zero
      nai = max(waer(1)-root7-root8,zero)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 .and. psconv13 .and. psconv14 .and. psconv5 .and. &
                psconv7 .and. psconv8 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw6')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = chi5 - psi5
    cnacl = chi7 - psi7
    cnano3 = chi8 - psi8
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = chi13 - psi13
    kcl = chi14 - psi14
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw6
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw5
! *** case w5
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl , nano3 , nh4no3
!     4. completely dissolved: ca(no3)2 , cacl2 ,
!                              mg(no3)2 , mgcl2
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw5
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** regime depends on the existance of water and of the rh
! ************
    if ( waer(4)>tiny1 ) then     ! no3 exist , water possible
      scase = 'w5 ; subcase 1'
      call calcw5a
      scase = 'w5 ; subcase 1'
    else                                        ! no3 , cl non existant
      scase = 'w1 ; subcase 1'
      call calcw1a
      scase = 'w1 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drmp5 ) then           ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcw1a
        scase = 'w5 ; subcase 2'
        return
      else
        scase = 'w5 ; subcase 3'      ! mdrh region (caso4 , k2so4 , kno3 , kcl , mgso4 ,
!     nano3 , nacl , nh4no3 , nh4cl)
        call calcmdrpii(rh,drmp5,drnh4no3,calcw1a,calcw6)
        scase = 'w5 ; subcase 3'
      end if
    end if
  end subroutine calcw5
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw5a
! *** case w5a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl ,
!                          nano3 , nh4no3
!     4. completely dissolved: ca(no3)2 , cacl2 , mg(no3)2 , mgcl2
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw5a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi13o , psi14o , psi5o , psi7o , psi8o , psi9o , root13 , &
         root13a , root13b , root14
    real(rkx) :: root14a , root14b , root5 , root5a , root5b , root7 , &
         root7a , root7b , root8 , root8a , root8b , root9 , so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9 , psconv13 , psconv14 , psconv5 , psconv7 , psconv8
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psconv13 = .true.
    psconv14 = .true.
    psconv5 = .true.
    psconv7 = .true.
    psconv8 = .true.
!
    psi9o = -great
    psi13o = -great
    psi14o = -great
    psi5o = -great
    psi7o = -great
    psi8o = -great                   ! great = 1.d10
!
    root9 = zero
    root13 = zero
    root14 = zero
    root5 = zero
    root7 = zero
    root8 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi9 = ck2so4           ! salts
    chi13 = ckno3
    chi10 = cmgso4
    chi14 = ckcl
    chi5 = cnh4cl
    chi7 = cnacl
    chi8 = cnano3
    chi6 = cnh4no3
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a13 = xk19*(water/gama(19))**2       ! kno3      <==> k+
      a14 = xk20*(water/gama(20))**2       ! kcl       <==> k+
      a5 = xk14*(water/gama(6))**2         ! nh4cl     <==> nh4+
      a7 = xk8*(water/gama(1))**2          ! nacl      <==> na+
      a8 = xk9*(water/gama(3))**2          ! nano3     <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root13-root14)
        cc = (waer(7)-root13-root14)*(waer(2)-waer(6))                    &
             + 0.25_rkx*(waer(7)-root13-root14)**2
        dd = -0.25_rkx*((waer(7)-root13-root14)**2*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0-root13-root14,(waer(2)-waer(6)),    &
                chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   potssium nitrate
!
      if ( ki*no3i>a13 ) then
        bb = -(waer(4)-root8+waer(7)-2._rkx*root9-root14)
        cc = (waer(4)-root8)*(waer(7)-2._rkx*root9-root14) - a13
        dd = sqrt(max(bb*bb-4._rkx*cc,zero))
        root13a = 0.5_rkx*(-bb-dd)
        root13b = 0.5_rkx*(-bb+dd)
        if ( zero<=root13a ) then
          root13 = root13a
        else
          root13 = root13b
        end if
        root13 = min(max(root13,zero),chi13)
        psi13 = chi13 - root13
      end if
      psconv13 = abs(psi13-psi13o)<=eps*psi13o
      psi13o = psi13
!
!   potssium cloride
!
      if ( ki*cli>a14 ) then
        bb = -(waer(5)-root5-root7+waer(7)-2._rkx*root9-root13)
        cc = (waer(5)-root5-root7)*(waer(7)-2._rkx*root9-root13) - a14
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root14a = 0.5_rkx*(-bb-dd)
        root14b = 0.5_rkx*(-bb+dd)
        if ( zero<=root14a ) then
          root14 = root14a
        else
          root14 = root14b
        end if
        root14 = min(max(root14,zero),chi14)
        psi14 = chi14 - root14
      end if
      psconv14 = abs(psi14-psi14o)<=eps*psi14o
      psi14o = psi14
!
!   ammnium cloride
!
      if ( nh4i*cli>a5 ) then
        bb = -(waer(5)+waer(3)-root14-root7)
        cc = (waer(5)-root14-root7)*waer(3) - a5
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root5a = 0.5_rkx*(-bb-dd)
        root5b = 0.5_rkx*(-bb+dd)
        if ( zero<=root5a ) then
          root5 = root5a
        else
          root5 = root5b
        end if
        root5 = min(max(root5,zero),chi5)
        psi5 = chi5 - root5
      end if
      psconv5 = abs(psi5-psi5o)<=eps*psi5o
      psi5o = psi5
!
!   sodum cloride
!
      if ( nai*cli>a7 ) then
        bb = -(waer(5)+waer(1)-root8-root14-root5)
        cc = (waer(5)-root14-root5)*(waer(1)-root8) - a7
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root7a = 0.5_rkx*(-bb-dd)
        root7b = 0.5_rkx*(-bb+dd)
        if ( zero<=root7a ) then
          root7 = root7a
        else
          root7 = root7b
        end if
        root7 = min(max(root7,zero),chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi7o)<=eps*psi7o
      psi7o = psi7
!
!   sodum nitrate
!
      if ( nai*no3i>a8 ) then
        bb = -(waer(4)-root13+waer(1)-root7)
        cc = (waer(4)-root13)*(waer(1)-root7) - a8
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root8a = 0.5_rkx*(-bb-dd)
        root8b = 0.5_rkx*(-bb+dd)
        if ( zero<=root8a ) then
          root8 = root8a
        else
          root8 = root8b
        end if
        root8 = min(max(root8,zero),chi8)
        psi8 = chi8 - root8
      end if
      psconv8 = abs(psi8-psi8o)<=eps*psi8o
      psi8o = psi8
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9-root13-root14,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = max(waer(3)-root5,zero)
      no3i = max(waer(4)-root13-root8,zero)
      cli = max(waer(5)-root14-root5-root7,zero)
      cai = zero
      nai = max(waer(1)-root7-root8,zero)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 .and. psconv13 .and. psconv14 .and. psconv5 .and. &
                psconv7 .and. psconv8 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw5')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = chi5 - psi5
    cnacl = chi7 - psi7
    cnano3 = chi8 - psi8
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = chi13 - psi13
    kcl = chi14 - psi14
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw5a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw4
! *** case w4
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , k2so4 , kno3 , kcl , mgso4 ,
!                          mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw4
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** regime depends on the existance of water and of the rh
! ************
    if ( waer(4)>tiny1 ) then     ! no3 exist , water possible
      scase = 'w4 ; subcase 1'
      call calcw4a
      scase = 'w4 ; subcase 1'
    else                                        ! no3 , cl non existant
      scase = 'w1 ; subcase 1'
      call calcw1a
      scase = 'w1 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drmp4 ) then           ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcw1a
        scase = 'w4 ; subcase 2'
        return
      else
        scase = 'w4 ; subcase 3'      ! mdrh region (caso4 , k2so4 , kno3 , kcl , mgso4 ,
!     mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl)
        call calcmdrpii(rh,drmp4,drmgno32,calcw1a,calcw5a)
        scase = 'w4 ; subcase 3'
      end if
    end if
  end subroutine calcw4
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw4a
! *** case w4a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl ,
!                          nano3 , nh4no3 , mg(no3)2
!     4. completely dissolved: ca(no3)2 , cacl2 , mgcl2
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw4a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi13o , psi14o , psi5o , psi7o , psi8o , psi9o , root13 , &
         root13a , root13b , root14
    real(rkx) :: root14a , root14b , root5 , root5a , root5b , root7 , &
         root7a , root7b , root8 , root8a , root8b , root9 , so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9 , psconv13 , psconv14 , psconv5 , psconv7 , psconv8
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psconv13 = .true.
    psconv14 = .true.
    psconv5 = .true.
    psconv7 = .true.
    psconv8 = .true.
!
    psi9o = -great
    psi13o = -great
    psi14o = -great
    psi5o = -great
    psi7o = -great
    psi8o = -great                   ! great = 1.d10
!
    root9 = zero
    root13 = zero
    root14 = zero
    root5 = zero
    root7 = zero
    root8 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi9 = ck2so4           ! salts
    chi13 = ckno3
    chi10 = cmgso4
    chi14 = ckcl
    chi5 = cnh4cl
    chi7 = cnacl
    chi8 = cnano3
    chi6 = cnh4no3
    chi15 = cmgno32
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a13 = xk19*(water/gama(19))**2       ! kno3      <==> k+
      a14 = xk20*(water/gama(20))**2       ! kcl       <==> k+
      a5 = xk14*(water/gama(6))**2         ! nh4cl     <==> nh4+
      a7 = xk8*(water/gama(1))**2          ! nacl      <==> na+
      a8 = xk9*(water/gama(3))**2          ! nano3     <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root13-root14)
        cc = (waer(7)-root13-root14)*(waer(2)-waer(6))                    &
             + 0.25_rkx*(waer(7)-root13-root14)**2
        dd = -0.25_rkx*((waer(7)-root13-root14)**2*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0-root13-root14,(waer(2)-waer(6)),    &
                chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   potssium nitrate
!
      if ( ki*no3i>a13 ) then
        bb = -(waer(4)-root8+waer(7)-2._rkx*root9-root14)
        cc = (waer(4)-root8)*(waer(7)-2._rkx*root9-root14) - a13
        dd = sqrt(max(bb*bb-4._rkx*cc,zero))
        root13a = 0.5_rkx*(-bb-dd)
        root13b = 0.5_rkx*(-bb+dd)
        if ( zero<=root13a ) then
          root13 = root13a
        else
          root13 = root13b
        end if
        root13 = min(max(root13,zero),chi13)
        psi13 = chi13 - root13
      end if
      psconv13 = abs(psi13-psi13o)<=eps*psi13o
      psi13o = psi13
!
!   potssium cloride
!
      if ( ki*cli>a14 ) then
        bb = -(waer(5)-root5-root7+waer(7)-2._rkx*root9-root13)
        cc = (waer(5)-root5-root7)*(waer(7)-2._rkx*root9-root13) - a14
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root14a = 0.5_rkx*(-bb-dd)
        root14b = 0.5_rkx*(-bb+dd)
        if ( zero<=root14a ) then
          root14 = root14a
        else
          root14 = root14b
        end if
        root14 = min(max(root14,zero),chi14)
        psi14 = chi14 - root14
      end if
      psconv14 = abs(psi14-psi14o)<=eps*psi14o
      psi14o = psi14
!
!   ammnium cloride
!
      if ( nh4i*cli>a5 ) then
        bb = -(waer(5)+waer(3)-root14-root7)
        cc = (waer(5)-root14-root7)*waer(3) - a5
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root5a = 0.5_rkx*(-bb-dd)
        root5b = 0.5_rkx*(-bb+dd)
        if ( zero<=root5a ) then
          root5 = root5a
        else
          root5 = root5b
        end if
        root5 = min(max(root5,zero),chi5)
        psi5 = chi5 - root5
      end if
      psconv5 = abs(psi5-psi5o)<=eps*psi5o
      psi5o = psi5
!
!   sodum cloride
!
      if ( nai*cli>a7 ) then
        bb = -(waer(5)+waer(1)-root8-root14-root5)
        cc = (waer(5)-root14-root5)*(waer(1)-root8) - a7
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root7a = 0.5_rkx*(-bb-dd)
        root7b = 0.5_rkx*(-bb+dd)
        if ( zero<=root7a ) then
          root7 = root7a
        else
          root7 = root7b
        end if
        root7 = min(max(root7,zero),chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi7o)<=eps*psi7o
      psi7o = psi7
!
!   sodum nitrate
!
      if ( nai*no3i>a8 ) then
        bb = -(waer(4)-root13+waer(1)-root7)
        cc = (waer(4)-root13)*(waer(1)-root7) - a8
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root8a = 0.5_rkx*(-bb-dd)
        root8b = 0.5_rkx*(-bb+dd)
        if ( zero<=root8a ) then
          root8 = root8a
        else
          root8 = root8b
        end if
        root8 = min(max(root8,zero),chi8)
        psi8 = chi8 - root8
      end if
      psconv8 = abs(psi8-psi8o)<=eps*psi8o
      psi8o = psi8
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9-root13-root14,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = max(waer(3)-root5,zero)
      no3i = max(waer(4)-root13-root8,zero)
      cli = max(waer(5)-root14-root5-root7,zero)
      cai = zero
      nai = max(waer(1)-root7-root8,zero)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 .and. psconv13 .and. psconv14 .and. psconv5 .and. &
                psconv7 .and. psconv8 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw4')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = chi5 - psi5
    cnacl = chi7 - psi7
    cnano3 = chi8 - psi8
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = chi13 - psi13
    kcl = chi14 - psi14
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw4a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw3
! *** case w3
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 ,
!                          mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw3
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** regime depends on the existance of water and of the rh
! ************
! if (waer(4) > tiny1 .and. waer(5) > tiny1) then ! no3 ,cl exist ,
! water possible scase = 'w3 ; subcase 1'
! call calcw3a
! scase = 'w3 ; subcase 1'
! else                                      ! no3 , cl non existant
! scase = 'w1 ; subcase 1'
! call calcw1a
! scase = 'w1 ; subcase 1'
! end if
!
    call calcw1a
    if ( water>tiny1 ) then
      scase = 'w3 ; subcase 1'
      call calcw3a
      scase = 'w3 ; subcase 1'
    else if ( rh<drmp3 ) then        ! only solids
      water = tiny1
      do i = 1 , nions
        molal(i) = zero
      end do
      call calcw1a
      scase = 'w3 ; subcase 2'
      return
    else
      scase = 'w3 ; subcase 3'        ! mdrh region (caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 ,
!   mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl)
      call calcmdrpii(rh,drmp3,drcano32,calcw1a,calcw4a)
      scase = 'w3 ; subcase 3'
    end if
  end subroutine calcw3
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw3a
! *** case w3a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl ,
!                          nano3 , nh4no3 , ca(no3)2 , mg(no3)2
!     4. completely dissolved: cacl2 , mgcl2
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw3a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi13o , psi14o , psi5o , psi7o , psi8o , psi9o , &
         root13 , root13a , root13b , root14
    real(rkx) :: root14a , root14b , root5 , root5a , root5b , root7 , &
         root7a , root7b , root8 , root8a , root8b , root9 , so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9 , psconv13 , psconv14 , psconv5 , psconv7 , psconv8
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psconv13 = .true.
    psconv14 = .true.
    psconv5 = .true.
    psconv7 = .true.
    psconv8 = .true.
!
    psi9o = -great
    psi13o = -great
    psi14o = -great
    psi5o = -great
    psi7o = -great
    psi8o = -great                   ! great = 1.d10
!
    root9 = zero
    root13 = zero
    root14 = zero
    root5 = zero
    root7 = zero
    root8 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi9 = ck2so4           ! salts
    chi13 = ckno3
    chi10 = cmgso4
    chi14 = ckcl
    chi5 = cnh4cl
    chi7 = cnacl
    chi8 = cnano3
    chi6 = cnh4no3
    chi15 = cmgno32
    chi12 = ccano32
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a13 = xk19*(water/gama(19))**2       ! kno3      <==> k+
      a14 = xk20*(water/gama(20))**2       ! kcl       <==> k+
      a5 = xk14*(water/gama(6))**2         ! nh4cl     <==> nh4+
      a7 = xk8*(water/gama(1))**2          ! nacl      <==> na+
      a8 = xk9*(water/gama(3))**2          ! nano3     <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root13-root14)
        cc = (waer(7)-root13-root14)*(waer(2)-waer(6))                    &
             + 0.25_rkx*(waer(7)-root13-root14)**2
        dd = -0.25_rkx*((waer(7)-root13-root14)**2*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0-root13-root14,(waer(2)-waer(6)),    &
                chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   potssium nitrate
!
      if ( ki*no3i>a13 ) then
        bb = -(waer(4)-root8+waer(7)-2._rkx*root9-root14)
        cc = (waer(4)-root8)*(waer(7)-2._rkx*root9-root14) - a13
        dd = sqrt(max(bb*bb-4._rkx*cc,zero))
        root13a = 0.5_rkx*(-bb-dd)
        root13b = 0.5_rkx*(-bb+dd)
        if ( zero<=root13a ) then
          root13 = root13a
        else
          root13 = root13b
        end if
        root13 = min(max(root13,zero),chi13)
        psi13 = chi13 - root13
      end if
      psconv13 = abs(psi13-psi13o)<=eps*psi13o
      psi13o = psi13
!
!   potssium cloride
!
      if ( ki*cli>a14 ) then
        bb = -(waer(5)-root5-root7+waer(7)-2._rkx*root9-root13)
        cc = (waer(5)-root5-root7)*(waer(7)-2._rkx*root9-root13) - a14
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root14a = 0.5_rkx*(-bb-dd)
        root14b = 0.5_rkx*(-bb+dd)
        if ( zero<=root14a ) then
          root14 = root14a
        else
          root14 = root14b
        end if
        root14 = min(max(root14,zero),chi14)
        psi14 = chi14 - root14
      end if
      psconv14 = abs(psi14-psi14o)<=eps*psi14o
      psi14o = psi14
!
!   ammnium cloride
!
      if ( nh4i*cli>a5 ) then
        bb = -(waer(5)+waer(3)-root14-root7)
        cc = (waer(5)-root14-root7)*waer(3) - a5
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root5a = 0.5_rkx*(-bb-dd)
        root5b = 0.5_rkx*(-bb+dd)
        if ( zero<=root5a ) then
          root5 = root5a
        else
          root5 = root5b
        end if
        root5 = min(max(root5,zero),chi5)
        psi5 = chi5 - root5
      end if
      psconv5 = abs(psi5-psi5o)<=eps*psi5o
      psi5o = psi5
!
!   sodum cloride
!
      if ( nai*cli>a7 ) then
        bb = -(waer(5)+waer(1)-root8-root14-root5)
        cc = (waer(5)-root14-root5)*(waer(1)-root8) - a7
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root7a = 0.5_rkx*(-bb-dd)
        root7b = 0.5_rkx*(-bb+dd)
        if ( zero<=root7a ) then
          root7 = root7a
        else
          root7 = root7b
        end if
        root7 = min(max(root7,zero),chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi7o)<=eps*psi7o
      psi7o = psi7
!
!   sodum nitrate
!
      if ( nai*no3i>a8 ) then
        bb = -(waer(4)-root13+waer(1)-root7)
        cc = (waer(4)-root13)*(waer(1)-root7) - a8
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root8a = 0.5_rkx*(-bb-dd)
        root8b = 0.5_rkx*(-bb+dd)
        if ( zero<=root8a ) then
          root8 = root8a
        else
          root8 = root8b
        end if
        root8 = min(max(root8,zero),chi8)
        psi8 = chi8 - root8
      end if
      psconv8 = abs(psi8-psi8o)<=eps*psi8o
      psi8o = psi8
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9-root13-root14,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = max(waer(3)-root5,zero)
      no3i = max(waer(4)-root13-root8,zero)
      cli = max(waer(5)-root14-root5-root7,zero)
      cai = zero
      nai = max(waer(1)-root7-root8,zero)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 .and. psconv13 .and. psconv14 .and. psconv5 .and. &
                psconv7 .and. psconv8 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw3')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = chi5 - psi5
    cnacl = chi7 - psi7
    cnano3 = chi8 - psi8
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = chi13 - psi13
    kcl = chi14 - psi14
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw3a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw2
! *** case w2
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 ,
!                          mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
!     there are three regimes in this case:
!     1. cacl2(s) possible. liquid & solid aerosol (subroutine calcl2a)
!     2. cacl2(s) not possible , and rh < mdrh. solid aerosol only
!     3. cacl2(s) not possible , and rh >= mdrh. solid & liquid aerosol
!
!     regimes 2. and 3. are considered to be the same as cases w1a , w2b
!     respectively
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
!
  subroutine calcw2
    use mod_che_common_isorropia
    implicit none
    integer(ik4) :: i
!
! *** find dry composition
! **********************************************
    call calcw1a
!
! *** regime depends upon the possible solids & rh
! **********************
    if ( ccacl2>tiny1 ) then
      scase = 'w2 ; subcase 1'
      call calcw2a
      scase = 'w2 ; subcase 1'
    end if
    if ( water<=tiny1 ) then
      if ( rh<drmp2 ) then                ! only solids
        water = tiny1
        do i = 1 , nions
          molal(i) = zero
        end do
        call calcw1a
        scase = 'w2 ; subcase 2'
      else
        if ( cmgcl2>tiny1 ) then
          scase = 'w2 ; subcase 3'         ! mdrh (caso4 , ca(no3)2 , k2so4 , kno3 , kcl , mgso4 , mgcl2 ,
!       mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl)
          call calcmdrpii(rh,drmp2,drmgcl2,calcw1a,calcw3a)
          scase = 'w2 ; subcase 3'
        end if
        if ( water<=tiny1 .and. rh>=drmp3 .and. rh<drmp4 ) then
          scase = 'w2 ; subcase 4'         ! mdrh (caso4 , k2so4 , kno3 , kcl , mgso4 , cano32 ,
!       mg(no3)2 , nano3 , nacl , nh4no3 , nh4cl)
          call calcmdrpii(rh,drmp3,drcano32,calcw1a,calcw4a)
          scase = 'w2 ; subcase 4'
        end if
        if ( water<=tiny1 .and. rh>=drmp4 .and. rh<drmp5 ) then
          scase = 'w2 ; subcase 5'         ! mdrh (caso4 , k2so4 , kno3 , kcl , mgso4 ,
!       mgno32 , nano3 , nacl , nh4no3 , nh4cl)
          call calcmdrpii(rh,drmp4,drmgno32,calcw1a,calcw5a)
          scase = 'w2 ; subcase 5'
        end if
        if ( water<=tiny1 .and. rh>=drmp5 ) then
          scase = 'w2 ; subcase 6'         ! mdrh (caso4 , k2so4 , kno3 , kcl , mgso4 ,
!       nano3 , nacl , nh4no3 , nh4cl)
          call calcmdrpii(rh,drmp5,drnh4no3,calcw1a,calcw6)
          scase = 'w2 ; subcase 6'
        else
          water = tiny1
          do i = 1 , nions
            molal(i) = zero
          end do
          call calcw1a
          scase = 'w2 ; subcase 2'
        end if
      end if
    end if
  end subroutine calcw2
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw2a
! *** case w2a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. there is both a liquid & solid phase
!     3. solids possible : caso4 , k2so4 , kno3 , mgso4 , kcl , nh4cl , nacl ,
!                          nano3 , nh4no3 , ca(no3)2 , mg(no3)2 , mgcl2
!     4. completely dissolved: cacl2
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw2a
    use mod_che_common_isorropia
    use mod_che_common_solut
    implicit none
    real(rkx) :: akw , bb , cc , cli , dd , del , gg , hi , hso4i , &
         ohi , psi13o , psi14o , psi5o , psi7o , psi8o , psi9o , root13 , &
         root13a , root13b , root14
    real(rkx) :: root14a , root14b , root5 , root5a , root5b , root7 , &
         root7a , root7b , root8 , root8a , root8b , root9 , so4i , kcl
    integer(ik4) :: i , islv
    logical :: psconv9 , psconv13 , psconv14 , psconv5 , psconv7 , psconv8
    real(rkx) :: nh4i , nai , no3i , nh3aq , no3aq , claq , cai , ki , mgi
!
! *** setup parameters ************************************************
!
    frst = .true.
    calain = .true.
    calaou = .true.
!
    psconv9 = .true.
    psconv13 = .true.
    psconv14 = .true.
    psconv5 = .true.
    psconv7 = .true.
    psconv8 = .true.
!
    psi9o = -great
    psi13o = -great
    psi14o = -great
    psi5o = -great
    psi7o = -great
    psi8o = -great                   ! great = 1.d10
!
    root9 = zero
    root13 = zero
    root14 = zero
    root5 = zero
    root7 = zero
    root8 = zero
!
! *** calculate initial solution ***************************************
!
    call calcw1a
!
    chi9 = ck2so4           ! salts
    chi13 = ckno3
    chi10 = cmgso4
    chi14 = ckcl
    chi5 = cnh4cl
    chi7 = cnacl
    chi8 = cnano3
    chi6 = cnh4no3
    chi15 = cmgno32
    chi12 = ccano32
    chi16 = cmgcl2
    chi11 = ccaso4
!
    psi1 = cna2so4          ! salts dissolved
    psi5 = cnh4cl
    psi6 = cnh4no3
    psi7 = cnacl
    psi8 = cnano3
    psi9 = ck2so4
    psi10 = cmgso4
    psi11 = ccaso4
    psi12 = ccano32
    psi13 = ckno3
    psi14 = ckcl
    psi15 = cmgno32
    psi16 = cmgcl2
    psi17 = ccacl2
!
    call calcmr             ! water
!
    nai = waer(1)           ! liquid concentrations
    so4i = max(waer(2)-waer(6),zero)
    nh4i = waer(3)
    no3i = waer(4)
    cli = waer(5)
    cai = waer(6)
    ki = waer(7)
    mgi = waer(8)
!
    hso4i = zero
    nh3aq = zero
    no3aq = zero
    claq = zero
!
! *** solve equations ; with iterations for activity coef. ************
!
    do i = 1 , nsweep
!
      a9 = xk17*(water/gama(17))**3        ! k2so4     <==> k+
      a13 = xk19*(water/gama(19))**2       ! kno3      <==> k+
      a14 = xk20*(water/gama(20))**2       ! kcl       <==> k+
      a5 = xk14*(water/gama(6))**2         ! nh4cl     <==> nh4+
      a7 = xk8*(water/gama(1))**2          ! nacl      <==> na+
      a8 = xk9*(water/gama(3))**2          ! nano3     <==> na+
      akw = xkw*rh*water*water               ! h2o       <==> h+
!
!   potssium sulfate
!
      if ( ki*ki*so4i>a9 ) then
        bb = -((waer(2)-waer(6))+waer(7)-root13-root14)
        cc = (waer(7)-root13-root14)*(waer(2)-waer(6))                    &
             + 0.25_rkx*(waer(7)-root13-root14)**2
        dd = -0.25_rkx*((waer(7)-root13-root14)**2*(waer(2)-waer(6))-a9)
        call poly3(bb,cc,dd,root9,islv)
        if ( islv/=0 ) root9 = tiny1
        root9 = min(root9,waer(7)/2.0-root13-root14,(waer(2)-waer(6)),    &
                chi9)
        root9 = max(root9,zero)
        psi9 = chi9 - root9
      end if
      psconv9 = abs(psi9-psi9o)<=eps*psi9o
      psi9o = psi9
!
!   potssium nitrate
!
      if ( ki*no3i>a13 ) then
        bb = -(waer(4)-root8+waer(7)-2._rkx*root9-root14)
        cc = (waer(4)-root8)*(waer(7)-2._rkx*root9-root14) - a13
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root13a = 0.5_rkx*(-bb-dd)
        root13b = 0.5_rkx*(-bb+dd)
        if ( zero<=root13a ) then
          root13 = root13a
        else
          root13 = root13b
        end if
        root13 = min(max(root13,zero),chi13)
        psi13 = chi13 - root13
      end if
      psconv13 = abs(psi13-psi13o)<=eps*psi13o
      psi13o = psi13
!
!   potssium cloride
!
      if ( ki*cli>a14 ) then
        bb = -(waer(5)-root5-root7+waer(7)-2._rkx*root9-root13)
        cc = (waer(5)-root5-root7)*(waer(7)-2._rkx*root9-root13) - a14
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root14a = 0.5_rkx*(-bb-dd)
        root14b = 0.5_rkx*(-bb+dd)
        if ( zero<=root14a ) then
          root14 = root14a
        else
          root14 = root14b
        end if
        root14 = min(max(root14,zero),chi14)
        psi14 = chi14 - root14
      end if
      psconv14 = abs(psi14-psi14o)<=eps*psi14o
      psi14o = psi14
!
!   ammnium cloride
!
      if ( nh4i*cli>a5 ) then
        bb = -(waer(5)+waer(3)-root14-root7)
        cc = (waer(5)-root14-root7)*waer(3) - a5
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root5a = 0.5_rkx*(-bb-dd)
        root5b = 0.5_rkx*(-bb+dd)
        if ( zero<=root5a ) then
          root5 = root5a
        else
          root5 = root5b
        end if
        root5 = min(max(root5,zero),chi5)
        psi5 = chi5 - root5
      end if
      psconv5 = abs(psi5-psi5o)<=eps*psi5o
      psi5o = psi5
!
!   sodum cloride
!
      if ( nai*cli>a7 ) then
        bb = -(waer(5)+waer(1)-root8-root14-root5)
        cc = (waer(5)-root14-root5)*(waer(1)-root8) - a7
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root7a = 0.5_rkx*(-bb-dd)
        root7b = 0.5_rkx*(-bb+dd)
        if ( zero<=root7a ) then
          root7 = root7a
        else
          root7 = root7b
        end if
        root7 = min(max(root7,zero),chi7)
        psi7 = chi7 - root7
      end if
      psconv7 = abs(psi7-psi7o)<=eps*psi7o
      psi7o = psi7
!
!   sodum nitrate
!
      if ( nai*no3i>a8 ) then
        bb = -(waer(4)-root13+waer(1)-root7)
        cc = (waer(4)-root13)*(waer(1)-root7) - a8
        dd = sqrt(max(bb*bb-4._rkx*cc,tiny1))
        root8a = 0.5_rkx*(-bb-dd)
        root8b = 0.5_rkx*(-bb+dd)
        if ( zero<=root8a ) then
          root8 = root8a
        else
          root8 = root8b
        end if
        root8 = min(max(root8,zero),chi8)
        psi8 = chi8 - root8
      end if
      psconv8 = abs(psi8-psi8o)<=eps*psi8o
      psi8o = psi8
!
!   ion concentrations ; corrections
!
      ki = max(waer(7)-2._rkx*root9-root13-root14,zero)
      so4i = max(waer(2)-waer(6)-root9,zero)
      nh4i = max(waer(3)-root5,zero)
      no3i = max(waer(4)-root13-root8,zero)
      cli = max(waer(5)-root14-root5-root7,zero)
      cai = zero
      nai = max(waer(1)-root7-root8,zero)
      mgi = waer(8)
!
!   soltion acidic or basic?
!
      gg = 2._rkx*so4i + no3i + cli - nai - nh4i - 2._rkx*cai - ki - 2._rkx*mgi
      if ( gg>tiny1 ) then                        ! h+ in excess
        bb = -gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        hi = 0.5_rkx*(-bb+sqrt(dd))
        ohi = akw/hi
      else                                        ! oh- in excess
        bb = gg
        cc = -akw
        dd = bb*bb - 4._rkx*cc
        ohi = 0.5_rkx*(-bb+sqrt(dd))
        hi = akw/ohi
      end if
!
!   undssociated species equilibria
!
      if ( hi>ohi ) then
!     call calcamaq2 (-gg , nh4i , ohi , nh3aq)
!     hi    = akw/ohi
!     hso4i = zero
!     else
!     ggno3 = max(2.d0*so4i + no3i - nai - nh4i - 2.d0*cai
!     &           - ki - 2.d0*mgi , zero)
!     ggcl  = max(gg-ggno3 , zero)
!     if (ggcl  > tiny1) call calcclaq2 (ggcl , cli , hi , claq) ! hcl
!     if (ggno3 > tiny1) then
!     if (ggcl <= tiny1) hi = zero
!     call calcniaq2 (ggno3 , no3i , hi , no3aq)              ! hno3
!     end if
!
!     conentration adjustments ; hso4 minor species.
!
        call calchs4(hi,so4i,zero,del)
      else
        del = zero
      end if
      so4i = so4i - del
      hi = hi - del
      hso4i = del
!   if (hi <= tiny1) hi = sqrt(akw)
      ohi = akw/hi
!
      if ( hi<=tiny1 ) then
        hi = sqrt(akw)
        ohi = akw/hi
      end if
!
!   *** save concentrations in molal array
!   ******************************
      molal(1) = hi
      molal(2) = nai
      molal(3) = nh4i
      molal(4) = cli
      molal(5) = so4i
      molal(6) = hso4i
      molal(7) = no3i
      molal(8) = cai
      molal(9) = ki
      molal(10) = mgi
!
!   *** calculate water
!   **************************************************
      call calcmr
!
!   *** calculate activities or terminate internal loop
!   *****************
      if ( frst .and. calaou .or. .not.frst .and. calain ) then
        call calcact
      else if ( psconv9 .and. psconv13 .and. psconv14 .and. psconv5 .and. &
                psconv7 .and. psconv8 ) then
        exit
      end if
    end do
! call pusherr (0002 , 'calcw2')    ! warning error: no convergence
!
! *** calculate gas / solid species (liquid in molal already) *********
!
    a2 = (xk2/xkw)*r*temp*(gama(10)/gama(5))**2       ! nh3  <==> nh4+
    a3 = xk4*r*temp*(water/gama(10))**2               ! hno3 <==> no3-
    a4 = xk3*r*temp*(water/gama(11))**2               ! hcl  <==> cl-
!
    gnh3 = nh4i/hi/a2
    ghno3 = hi*no3i/a3
    ghcl = hi*cli/a4
!
    gasaq(1) = nh3aq
    gasaq(2) = claq
    gasaq(3) = no3aq
!
    cnh42s4 = zero
    cnh4no3 = zero
    cnh4cl = chi5 - psi5
    cnacl = chi7 - psi7
    cnano3 = chi8 - psi8
    cmgso4 = zero
    ck2so4 = chi9 - psi9
    ccaso4 = min(waer(6),waer(2))
    ccano32 = zero
    ckno3 = chi13 - psi13
    kcl = chi14 - psi14
    cmgno32 = zero
    cmgcl2 = zero
    ccacl2 = zero
  end subroutine calcw2a
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw1
! *** case w1
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , ca(no3)2 , cacl2 , k2so4 , kno3 , kcl , mgso4 ,
!                          mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
!     there are two possible regimes here , depending on relative humidity:
!     1. when rh >= mdrh ; liquid phase possible (mdrh region)
!     2. when rh < mdrh  ; only solid phase possible (subroutine calcp1a)
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university ,
! *** georgia institute of technology
! *** written by christos fountoukis & athanasios nenes
!
!======================================================================
!
  subroutine calcw1
    use mod_che_common_isorropia
    implicit none
!
! *** regime depends upon the ambient relative humidity
! *****************
    if ( rh<drmp1 ) then
      scase = 'w1 ; subcase 1'
      call calcw1a                 ! solid phase only possible
      scase = 'w1 ; subcase 1'
    else
      scase = 'w1 ; subcase 2'     ! liquid & solid phase possible
      call calcmdrpii(rh,drmp1,drcacl2,calcw1a,calcw2a)
      scase = 'w1 ; subcase 2'
    end if
  end subroutine calcw1
!
!======================================================================
!
! *** isorropia code ii
! *** subroutine calcw1a
! *** case w1a
!
!     the main characteristics of this regime are:
!     1. sulfate poor (sulrat > 2.0) ; rcr+na >= 2.0 ; rcr > 2)
!     2. solid aerosol only
!     3. solids possible : caso4 , ca(no3)2 , cacl2 , k2so4 , kno3 , kcl , mgso4 ,
!                          mg(no3)2 , mgcl2 , nano3 , nacl , nh4no3 , nh4cl
!
! *** copyright 1996-2008 , university of miami , carnegie mellon university
! *** georgia institute of technology
! *** written by christos fountoukis and athanasios nenes
!
!======================================================================
!
  subroutine calcw1a
    use mod_che_common_isorropia
    implicit none
    real(rkx) :: cafr , clfr , frk , frmg , frna , frno3 , so4fr
!
! *** calculate solids
! **************************************************
    ccaso4 = min(waer(2),waer(6))                   !solid caso4
    cafr = max(waer(6)-ccaso4,zero)
    so4fr = max(waer(2)-ccaso4,zero)
    ck2so4 = min(so4fr,0.5_rkx*waer(7))                !solid k2so4
    frk = max(waer(7)-2._rkx*ck2so4,zero)
    so4fr = max(so4fr-ck2so4,zero)
    cmgso4 = so4fr                                  !solid mgso4
    frmg = max(waer(8)-cmgso4,zero)
    cnacl = min(waer(1),waer(5))                    !solid nacl
    frna = max(waer(1)-cnacl,zero)
    clfr = max(waer(5)-cnacl,zero)
    ccacl2 = min(cafr,0.5_rkx*clfr)                    !solid cacl2
    cafr = max(cafr-ccacl2,zero)
    clfr = max(waer(5)-2._rkx*ccacl2,zero)
    ccano32 = min(cafr,0.5_rkx*waer(4))                !solid ca(no3)2
    cafr = max(cafr-ccano32,zero)
    frno3 = max(waer(4)-2._rkx*ccano32,zero)
    cmgcl2 = min(frmg,0.5_rkx*clfr)                    !solid mgcl2
    frmg = max(frmg-cmgcl2,zero)
    clfr = max(clfr-2._rkx*cmgcl2,zero)
    cmgno32 = min(frmg,0.5_rkx*frno3)                  !solid mg(no3)2
    frmg = max(frmg-cmgno32,zero)
    frno3 = max(frno3-2._rkx*cmgno32,zero)
    cnano3 = min(frna,frno3)                         !solid nano3
    frna = max(frna-cnano3,zero)
    frno3 = max(frno3-cnano3,zero)
    ckcl = min(frk,clfr)                             !solid kcl
    frk = max(frk-ckcl,zero)
    clfr = max(clfr-ckcl,zero)
    ckno3 = min(frk,frno3)                           !solid kno3
    frk = max(frk-ckno3,zero)
    frno3 = max(frno3-ckno3,zero)
!
! *** other phases
! ******************************************************
    water = zero
!
    gnh3 = zero
    ghno3 = zero
    ghcl = zero
  end subroutine calcw1a

end module mod_che_isorropia
! vim: tabstop=8 expandtab shiftwidth=2 softtabstop=2
