!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
!
!    This file is part of ICTP RegCM.
!
!    ICTP RegCM is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    ICTP RegCM is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with ICTP RegCM.  If not, see <http://www.gnu.org/licenses/>.
!
!::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

module mod_ipcc_scenario
  !
  ! IPCC Scenarios obtained from:
  !
  !     ftp://crgd.atmos.uiuc.edu/pub/post-sres
  !
  use mod_intkinds
  use mod_realkinds
  use mod_dynparam
  use mod_mpmessage
  use mod_runparams

  implicit none

  private

  public :: set_scenario , cgas

  integer(ik4) , parameter :: nsc = 10
  integer(ik4) , parameter :: n_greenhouse_gases = 5
  character(len=8) , dimension(nsc) :: scenarios

  real(rkx) , dimension(1+n_greenhouse_gases,1850:2100) :: cgas

  integer(ik4) :: i , j

  integer(ik4) , public :: igh_co2 = 2
  integer(ik4) , public :: igh_ch4 = 3
  integer(ik4) , public :: igh_n2o = 4
  integer(ik4) , public :: igh_cfc11 = 5
  integer(ik4) , public :: igh_cfc12 = 6

  ! SRES and RCP Scenarios

  data scenarios /'CONST   ','A1B     ','A2      ','B1      ','B2      ', &
                  'RF      ','RCP2.6  ','RCP4.5  ','RCP6.0  ','RCP8.5  '/
!
!-----------------------------------------------------------------------
!
  data ((cgas(i,j),i=1,6),j=1850,1899) / &
    1850.0_rkx , 284.70_rkx , 791.60_rkx , 275.70_rkx , 0.00_rkx , 0.00_rkx , &
    1851.0_rkx , 284.90_rkx , 792.90_rkx , 275.80_rkx , 0.00_rkx , 0.00_rkx , &
    1852.0_rkx , 285.00_rkx , 794.30_rkx , 275.90_rkx , 0.00_rkx , 0.00_rkx , &
    1853.0_rkx , 285.10_rkx , 795.60_rkx , 276.00_rkx , 0.00_rkx , 0.00_rkx , &
    1854.0_rkx , 285.30_rkx , 797.00_rkx , 276.10_rkx , 0.00_rkx , 0.00_rkx , &
    1855.0_rkx , 285.40_rkx , 798.40_rkx , 276.20_rkx , 0.00_rkx , 0.00_rkx , &
    1856.0_rkx , 285.60_rkx , 799.80_rkx , 276.30_rkx , 0.00_rkx , 0.00_rkx , &
    1857.0_rkx , 285.70_rkx , 801.20_rkx , 276.40_rkx , 0.00_rkx , 0.00_rkx , &
    1858.0_rkx , 285.90_rkx , 802.70_rkx , 276.50_rkx , 0.00_rkx , 0.00_rkx , &
    1859.0_rkx , 286.10_rkx , 804.10_rkx , 276.60_rkx , 0.00_rkx , 0.00_rkx , &
    1860.0_rkx , 286.20_rkx , 805.60_rkx , 276.70_rkx , 0.00_rkx , 0.00_rkx , &
    1861.0_rkx , 286.40_rkx , 807.10_rkx , 276.80_rkx , 0.00_rkx , 0.00_rkx , &
    1862.0_rkx , 286.50_rkx , 808.60_rkx , 276.90_rkx , 0.00_rkx , 0.00_rkx , &
    1863.0_rkx , 286.60_rkx , 810.20_rkx , 276.90_rkx , 0.00_rkx , 0.00_rkx , &
    1864.0_rkx , 286.80_rkx , 811.70_rkx , 277.00_rkx , 0.00_rkx , 0.00_rkx , &
    1865.0_rkx , 286.90_rkx , 813.30_rkx , 277.10_rkx , 0.00_rkx , 0.00_rkx , &
    1866.0_rkx , 287.00_rkx , 814.80_rkx , 277.20_rkx , 0.00_rkx , 0.00_rkx , &
    1867.0_rkx , 287.10_rkx , 816.40_rkx , 277.20_rkx , 0.00_rkx , 0.00_rkx , &
    1868.0_rkx , 287.20_rkx , 817.90_rkx , 277.30_rkx , 0.00_rkx , 0.00_rkx , &
    1869.0_rkx , 287.40_rkx , 819.50_rkx , 277.40_rkx , 0.00_rkx , 0.00_rkx , &
    1870.0_rkx , 287.50_rkx , 821.00_rkx , 277.40_rkx , 0.00_rkx , 0.00_rkx , &
    1871.0_rkx , 287.70_rkx , 822.50_rkx , 277.50_rkx , 0.00_rkx , 0.00_rkx , &
    1872.0_rkx , 287.90_rkx , 824.00_rkx , 277.60_rkx , 0.00_rkx , 0.00_rkx , &
    1873.0_rkx , 288.10_rkx , 825.60_rkx , 277.60_rkx , 0.00_rkx , 0.00_rkx , &
    1874.0_rkx , 288.40_rkx , 827.10_rkx , 277.70_rkx , 0.00_rkx , 0.00_rkx , &
    1875.0_rkx , 288.70_rkx , 828.60_rkx , 277.80_rkx , 0.00_rkx , 0.00_rkx , &
    1876.0_rkx , 289.00_rkx , 830.20_rkx , 277.90_rkx , 0.00_rkx , 0.00_rkx , &
    1877.0_rkx , 289.40_rkx , 831.90_rkx , 277.90_rkx , 0.00_rkx , 0.00_rkx , &
    1878.0_rkx , 289.80_rkx , 833.50_rkx , 278.00_rkx , 0.00_rkx , 0.00_rkx , &
    1879.0_rkx , 290.20_rkx , 835.30_rkx , 278.10_rkx , 0.00_rkx , 0.00_rkx , &
    1880.0_rkx , 290.70_rkx , 837.10_rkx , 278.10_rkx , 0.00_rkx , 0.00_rkx , &
    1881.0_rkx , 291.20_rkx , 839.00_rkx , 278.20_rkx , 0.00_rkx , 0.00_rkx , &
    1882.0_rkx , 291.70_rkx , 841.00_rkx , 278.30_rkx , 0.00_rkx , 0.00_rkx , &
    1883.0_rkx , 292.10_rkx , 843.00_rkx , 278.40_rkx , 0.00_rkx , 0.00_rkx , &
    1884.0_rkx , 292.60_rkx , 845.00_rkx , 278.40_rkx , 0.00_rkx , 0.00_rkx , &
    1885.0_rkx , 293.00_rkx , 847.00_rkx , 278.50_rkx , 0.00_rkx , 0.00_rkx , &
    1886.0_rkx , 293.30_rkx , 849.00_rkx , 278.60_rkx , 0.00_rkx , 0.00_rkx , &
    1887.0_rkx , 293.60_rkx , 851.00_rkx , 278.60_rkx , 0.00_rkx , 0.00_rkx , &
    1888.0_rkx , 293.80_rkx , 853.00_rkx , 278.70_rkx , 0.00_rkx , 0.00_rkx , &
    1889.0_rkx , 294.00_rkx , 854.80_rkx , 278.80_rkx , 0.00_rkx , 0.00_rkx , &
    1890.0_rkx , 294.20_rkx , 856.60_rkx , 278.80_rkx , 0.00_rkx , 0.00_rkx , &
    1891.0_rkx , 294.30_rkx , 858.30_rkx , 278.90_rkx , 0.00_rkx , 0.00_rkx , &
    1892.0_rkx , 294.50_rkx , 859.90_rkx , 279.00_rkx , 0.00_rkx , 0.00_rkx , &
    1893.0_rkx , 294.60_rkx , 861.60_rkx , 279.00_rkx , 0.00_rkx , 0.00_rkx , &
    1894.0_rkx , 294.70_rkx , 863.30_rkx , 279.10_rkx , 0.00_rkx , 0.00_rkx , &
    1895.0_rkx , 294.80_rkx , 865.30_rkx , 279.20_rkx , 0.00_rkx , 0.00_rkx , &
    1896.0_rkx , 294.90_rkx , 867.40_rkx , 279.30_rkx , 0.00_rkx , 0.00_rkx , &
    1897.0_rkx , 295.00_rkx , 869.80_rkx , 279.40_rkx , 0.00_rkx , 0.00_rkx , &
    1898.0_rkx , 295.20_rkx , 872.60_rkx , 279.50_rkx , 0.00_rkx , 0.00_rkx , &
    1899.0_rkx , 295.50_rkx , 875.70_rkx , 279.60_rkx , 0.00_rkx , 0.00_rkx /
  data ((cgas(i,j),i=1,6),j=1900,1949) / &
    1900.0_rkx , 295.80_rkx , 879.40_rkx , 279.60_rkx , 0.00_rkx , 0.00_rkx , &
    1901.0_rkx , 296.10_rkx , 883.50_rkx , 279.80_rkx , 0.00_rkx , 0.00_rkx , &
    1902.0_rkx , 296.50_rkx , 887.90_rkx , 279.90_rkx , 0.00_rkx , 0.00_rkx , &
    1903.0_rkx , 296.80_rkx , 892.20_rkx , 280.00_rkx , 0.00_rkx , 0.00_rkx , &
    1904.0_rkx , 297.20_rkx , 896.50_rkx , 280.10_rkx , 0.00_rkx , 0.00_rkx , &
    1905.0_rkx , 297.60_rkx , 900.90_rkx , 280.30_rkx , 0.00_rkx , 0.00_rkx , &
    1906.0_rkx , 298.10_rkx , 905.30_rkx , 280.40_rkx , 0.00_rkx , 0.00_rkx , &
    1907.0_rkx , 298.50_rkx , 909.80_rkx , 280.60_rkx , 0.00_rkx , 0.00_rkx , &
    1908.0_rkx , 298.90_rkx , 914.40_rkx , 280.80_rkx , 0.00_rkx , 0.00_rkx , &
    1909.0_rkx , 299.30_rkx , 919.20_rkx , 281.00_rkx , 0.00_rkx , 0.00_rkx , &
    1910.0_rkx , 299.70_rkx , 924.00_rkx , 281.20_rkx , 0.00_rkx , 0.00_rkx , &
    1911.0_rkx , 300.10_rkx , 928.90_rkx , 281.40_rkx , 0.00_rkx , 0.00_rkx , &
    1912.0_rkx , 300.40_rkx , 933.90_rkx , 281.60_rkx , 0.00_rkx , 0.00_rkx , &
    1913.0_rkx , 300.80_rkx , 939.00_rkx , 281.80_rkx , 0.00_rkx , 0.00_rkx , &
    1914.0_rkx , 301.10_rkx , 944.30_rkx , 282.10_rkx , 0.00_rkx , 0.00_rkx , &
    1915.0_rkx , 301.40_rkx , 949.70_rkx , 282.30_rkx , 0.00_rkx , 0.00_rkx , &
    1916.0_rkx , 301.70_rkx , 955.10_rkx , 282.50_rkx , 0.00_rkx , 0.00_rkx , &
    1917.0_rkx , 302.10_rkx , 960.70_rkx , 282.70_rkx , 0.00_rkx , 0.00_rkx , &
    1918.0_rkx , 302.40_rkx , 966.40_rkx , 283.00_rkx , 0.00_rkx , 0.00_rkx , &
    1919.0_rkx , 302.70_rkx , 972.20_rkx , 283.20_rkx , 0.00_rkx , 0.00_rkx , &
    1920.0_rkx , 303.00_rkx , 978.10_rkx , 283.40_rkx , 0.00_rkx , 0.00_rkx , &
    1921.0_rkx , 303.40_rkx , 984.00_rkx , 283.70_rkx , 0.00_rkx , 0.00_rkx , &
    1922.0_rkx , 303.80_rkx , 989.90_rkx , 283.90_rkx , 0.00_rkx , 0.00_rkx , &
    1923.0_rkx , 304.10_rkx , 995.80_rkx , 284.10_rkx , 0.00_rkx , 0.00_rkx , &
    1924.0_rkx , 304.50_rkx , 1001.80_rkx , 284.40_rkx , 0.00_rkx , 0.00_rkx , &
    1925.0_rkx , 305.00_rkx , 1007.70_rkx , 284.60_rkx , 0.00_rkx , 0.00_rkx , &
    1926.0_rkx , 305.40_rkx , 1013.60_rkx , 284.80_rkx , 0.00_rkx , 0.00_rkx , &
    1927.0_rkx , 305.80_rkx , 1019.30_rkx , 285.10_rkx , 0.00_rkx , 0.00_rkx , &
    1928.0_rkx , 306.30_rkx , 1025.00_rkx , 285.30_rkx , 0.00_rkx , 0.00_rkx , &
    1929.0_rkx , 306.80_rkx , 1030.60_rkx , 285.50_rkx , 0.00_rkx , 0.00_rkx , &
    1930.0_rkx , 307.20_rkx , 1036.20_rkx , 285.70_rkx , 0.00_rkx , 0.00_rkx , &
    1931.0_rkx , 307.70_rkx , 1041.70_rkx , 285.90_rkx , 0.00_rkx , 0.00_rkx , &
    1932.0_rkx , 308.20_rkx , 1047.20_rkx , 286.00_rkx , 0.00_rkx , 0.00_rkx , &
    1933.0_rkx , 308.60_rkx , 1052.60_rkx , 286.20_rkx , 0.00_rkx , 0.00_rkx , &
    1934.0_rkx , 309.00_rkx , 1058.00_rkx , 286.40_rkx , 0.00_rkx , 0.00_rkx , &
    1935.0_rkx , 309.40_rkx , 1063.40_rkx , 286.60_rkx , 0.00_rkx , 0.00_rkx , &
    1936.0_rkx , 309.80_rkx , 1068.70_rkx , 286.80_rkx , 0.00_rkx , 0.00_rkx , &
    1937.0_rkx , 310.00_rkx , 1073.90_rkx , 287.00_rkx , 0.00_rkx , 0.00_rkx , &
    1938.0_rkx , 310.20_rkx , 1078.90_rkx , 287.30_rkx , 0.00_rkx , 0.00_rkx , &
    1939.0_rkx , 310.30_rkx , 1083.90_rkx , 287.50_rkx , 0.00_rkx , 0.00_rkx , &
    1940.0_rkx , 310.40_rkx , 1088.90_rkx , 287.70_rkx , 0.00_rkx , 0.00_rkx , &
    1941.0_rkx , 310.40_rkx , 1093.90_rkx , 288.00_rkx , 0.00_rkx , 0.00_rkx , &
    1942.0_rkx , 310.30_rkx , 1099.00_rkx , 288.20_rkx , 0.00_rkx , 0.00_rkx , &
    1943.0_rkx , 310.20_rkx , 1104.10_rkx , 288.40_rkx , 0.00_rkx , 0.00_rkx , &
    1944.0_rkx , 310.10_rkx , 1109.40_rkx , 288.60_rkx , 0.00_rkx , 0.00_rkx , &
    1945.0_rkx , 310.10_rkx , 1114.90_rkx , 288.80_rkx , 0.00_rkx , 0.00_rkx , &
    1946.0_rkx , 310.10_rkx , 1120.60_rkx , 289.10_rkx , 0.00_rkx , 0.00_rkx , &
    1947.0_rkx , 310.20_rkx , 1126.70_rkx , 289.30_rkx , 0.00_rkx , 0.00_rkx , &
    1948.0_rkx , 310.30_rkx , 1133.20_rkx , 289.50_rkx , 0.00_rkx , 0.00_rkx , &
    1949.0_rkx , 310.50_rkx , 1140.10_rkx , 289.80_rkx , 0.00_rkx , 0.00_rkx /
  data ((cgas(i,j),i=1,6),j=1950,2000) / &
    1950.0_rkx , 310.70_rkx , 1147.50_rkx , 290.00_rkx , 0.00_rkx , 0.00_rkx , &
    1951.0_rkx , 311.10_rkx , 1155.40_rkx , 290.20_rkx , 1.75_rkx , 3.03_rkx , &
    1952.0_rkx , 311.50_rkx , 1163.80_rkx , 290.50_rkx , 3.50_rkx , 6.06_rkx , &
    1953.0_rkx , 311.90_rkx , 1172.70_rkx , 290.80_rkx , 5.25_rkx , 9.09_rkx , &
    1954.0_rkx , 312.40_rkx , 1182.10_rkx , 291.00_rkx , 7.00_rkx , 12.12_rkx , &
    1955.0_rkx , 313.00_rkx , 1192.00_rkx , 291.30_rkx , 8.75_rkx , 15.15_rkx , &
    1956.0_rkx , 313.60_rkx , 1202.40_rkx , 291.50_rkx , 10.50_rkx , 18.18_rkx , &
    1957.0_rkx , 314.20_rkx , 1213.10_rkx , 291.80_rkx , 12.25_rkx , 21.21_rkx , &
    1958.0_rkx , 314.90_rkx , 1224.20_rkx , 292.10_rkx , 14.00_rkx , 24.24_rkx , &
    1959.0_rkx , 315.60_rkx , 1235.70_rkx , 292.30_rkx , 15.75_rkx , 27.27_rkx , &
    1960.0_rkx , 316.30_rkx , 1247.50_rkx , 292.60_rkx , 17.50_rkx , 30.30_rkx , &
    1961.0_rkx , 317.00_rkx , 1259.70_rkx , 292.90_rkx , 21.75_rkx , 39.27_rkx , &
    1962.0_rkx , 317.70_rkx , 1272.20_rkx , 293.20_rkx , 26.00_rkx , 48.24_rkx , &
    1963.0_rkx , 318.40_rkx , 1285.10_rkx , 293.50_rkx , 30.25_rkx , 57.21_rkx , &
    1964.0_rkx , 319.20_rkx , 1298.40_rkx , 293.90_rkx , 34.50_rkx , 66.18_rkx , &
    1965.0_rkx , 320.00_rkx , 1312.00_rkx , 294.20_rkx , 38.75_rkx , 75.15_rkx , &
    1966.0_rkx , 320.80_rkx , 1326.00_rkx , 294.60_rkx , 43.00_rkx , 84.12_rkx , &
    1967.0_rkx , 321.80_rkx , 1340.40_rkx , 295.00_rkx , 47.25_rkx , 93.09_rkx , &
    1968.0_rkx , 322.80_rkx , 1355.30_rkx , 295.40_rkx , 51.50_rkx , 102.06_rkx , &
    1969.0_rkx , 323.80_rkx , 1370.50_rkx , 295.80_rkx , 55.75_rkx , 111.03_rkx , &
    1970.0_rkx , 324.80_rkx , 1386.00_rkx , 296.30_rkx , 60.00_rkx , 120.00_rkx , &
    1971.0_rkx , 325.80_rkx , 1401.70_rkx , 296.70_rkx , 71.00_rkx , 137.00_rkx , &
    1972.0_rkx , 326.90_rkx , 1417.50_rkx , 297.20_rkx , 82.00_rkx , 154.00_rkx , &
    1973.0_rkx , 328.00_rkx , 1433.40_rkx , 297.70_rkx , 93.00_rkx , 171.00_rkx , &
    1974.0_rkx , 329.20_rkx , 1449.30_rkx , 298.20_rkx , 104.00_rkx , 188.00_rkx , &
    1975.0_rkx , 330.40_rkx , 1465.40_rkx , 298.70_rkx , 115.00_rkx , 205.00_rkx , &
    1976.0_rkx , 331.70_rkx , 1481.50_rkx , 299.30_rkx , 126.60_rkx , 223.00_rkx , &
    1977.0_rkx , 333.00_rkx , 1497.70_rkx , 299.80_rkx , 138.20_rkx , 241.00_rkx , &
    1978.0_rkx , 334.30_rkx , 1514.00_rkx , 300.40_rkx , 149.80_rkx , 259.00_rkx , &
    1979.0_rkx , 335.60_rkx , 1530.50_rkx , 301.00_rkx , 161.40_rkx , 277.00_rkx , &
    1980.0_rkx , 337.00_rkx , 1547.10_rkx , 301.60_rkx , 173.00_rkx , 295.00_rkx , &
    1981.0_rkx , 338.40_rkx , 1564.00_rkx , 302.30_rkx , 182.80_rkx , 312.40_rkx , &
    1982.0_rkx , 339.90_rkx , 1580.90_rkx , 302.90_rkx , 192.60_rkx , 329.80_rkx , &
    1983.0_rkx , 341.30_rkx , 1597.70_rkx , 303.60_rkx , 202.40_rkx , 347.20_rkx , &
    1984.0_rkx , 342.80_rkx , 1614.20_rkx , 304.20_rkx , 212.20_rkx , 364.60_rkx , &
    1985.0_rkx , 344.30_rkx , 1630.00_rkx , 304.90_rkx , 222.00_rkx , 382.00_rkx , &
    1986.0_rkx , 345.80_rkx , 1644.90_rkx , 305.60_rkx , 230.20_rkx , 401.00_rkx , &
    1987.0_rkx , 347.40_rkx , 1658.80_rkx , 306.30_rkx , 238.40_rkx , 420.00_rkx , &
    1988.0_rkx , 348.90_rkx , 1671.60_rkx , 307.00_rkx , 246.60_rkx , 439.00_rkx , &
    1989.0_rkx , 350.50_rkx , 1683.40_rkx , 307.70_rkx , 254.80_rkx , 458.00_rkx , &
    1990.0_rkx , 352.00_rkx , 1694.30_rkx , 308.50_rkx , 260.71_rkx , 471.30_rkx , &
    1991.0_rkx , 353.60_rkx , 1704.30_rkx , 309.20_rkx , 265.83_rkx , 483.55_rkx , &
    1992.0_rkx , 355.10_rkx , 1713.40_rkx , 309.90_rkx , 270.17_rkx , 494.77_rkx , &
    1993.0_rkx , 356.60_rkx , 1721.60_rkx , 310.70_rkx , 273.76_rkx , 504.98_rkx , &
    1994.0_rkx , 358.20_rkx , 1729.10_rkx , 311.40_rkx , 276.61_rkx , 514.17_rkx , &
    1995.0_rkx , 359.70_rkx , 1735.80_rkx , 312.20_rkx , 278.73_rkx , 522.37_rkx , &
    1996.0_rkx , 361.20_rkx , 1741.90_rkx , 313.00_rkx , 280.14_rkx , 529.57_rkx , &
    1997.0_rkx , 362.70_rkx , 1747.30_rkx , 313.70_rkx , 280.84_rkx , 535.79_rkx , &
    1998.0_rkx , 364.10_rkx , 1752.10_rkx , 314.50_rkx , 280.87_rkx , 541.05_rkx , &
    1999.0_rkx , 365.60_rkx , 1756.30_rkx , 315.20_rkx , 280.22_rkx , 545.34_rkx , &
    2000.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx /
!
  contains
!
  subroutine set_scenario(csc)
    implicit none
    character(len=8) , intent(in) :: csc
    integer(ik4) :: ii , jj
    real(rkx) , dimension(1+n_greenhouse_gases) :: ctemp

    select case (csc)
      case ( 'A1B' )
        cgas(1:6,2001:2050) = reshape((/   &
    2001.0_rkx , 369.10_rkx , 1771.10_rkx , 316.80_rkx , 277.52_rkx , 551.75_rkx , &
    2002.0_rkx , 371.20_rkx , 1782.20_rkx , 317.60_rkx , 276.05_rkx , 554.56_rkx , &
    2003.0_rkx , 373.30_rkx , 1793.30_rkx , 318.40_rkx , 274.50_rkx , 557.12_rkx , &
    2004.0_rkx , 375.40_rkx , 1804.40_rkx , 319.20_rkx , 272.88_rkx , 559.41_rkx , &
    2005.0_rkx , 377.50_rkx , 1815.50_rkx , 320.00_rkx , 271.17_rkx , 561.46_rkx , &
    2006.0_rkx , 379.60_rkx , 1826.60_rkx , 320.80_rkx , 269.39_rkx , 563.25_rkx , &
    2007.0_rkx , 381.70_rkx , 1837.70_rkx , 321.60_rkx , 267.54_rkx , 564.80_rkx , &
    2008.0_rkx , 383.80_rkx , 1848.80_rkx , 322.40_rkx , 265.62_rkx , 566.10_rkx , &
    2009.0_rkx , 385.90_rkx , 1859.90_rkx , 323.20_rkx , 263.62_rkx , 567.15_rkx , &
    2010.0_rkx , 388.00_rkx , 1871.00_rkx , 324.00_rkx , 261.56_rkx , 567.97_rkx , &
    2011.0_rkx , 391.00_rkx , 1886.50_rkx , 324.70_rkx , 259.35_rkx , 568.17_rkx , &
    2012.0_rkx , 394.00_rkx , 1902.00_rkx , 325.40_rkx , 256.98_rkx , 567.78_rkx , &
    2013.0_rkx , 397.00_rkx , 1917.50_rkx , 326.10_rkx , 254.47_rkx , 566.79_rkx , &
    2014.0_rkx , 400.00_rkx , 1933.00_rkx , 326.80_rkx , 251.82_rkx , 565.21_rkx , &
    2015.0_rkx , 403.00_rkx , 1948.50_rkx , 327.50_rkx , 249.03_rkx , 563.05_rkx , &
    2016.0_rkx , 406.00_rkx , 1964.00_rkx , 328.20_rkx , 246.10_rkx , 560.31_rkx , &
    2017.0_rkx , 409.00_rkx , 1979.50_rkx , 328.90_rkx , 243.03_rkx , 557.00_rkx , &
    2018.0_rkx , 412.00_rkx , 1995.00_rkx , 329.60_rkx , 239.83_rkx , 553.12_rkx , &
    2019.0_rkx , 415.00_rkx , 2010.50_rkx , 330.30_rkx , 236.51_rkx , 548.68_rkx , &
    2020.0_rkx , 418.00_rkx , 2026.00_rkx , 331.00_rkx , 233.05_rkx , 543.68_rkx , &
    2021.0_rkx , 420.90_rkx , 2043.60_rkx , 331.70_rkx , 229.59_rkx , 538.70_rkx , &
    2022.0_rkx , 423.80_rkx , 2061.20_rkx , 332.40_rkx , 226.12_rkx , 533.74_rkx , &
    2023.0_rkx , 426.70_rkx , 2078.80_rkx , 333.10_rkx , 222.65_rkx , 528.79_rkx , &
    2024.0_rkx , 429.60_rkx , 2096.40_rkx , 333.80_rkx , 219.17_rkx , 523.85_rkx , &
    2025.0_rkx , 432.50_rkx , 2114.00_rkx , 334.50_rkx , 215.68_rkx , 518.93_rkx , &
    2026.0_rkx , 435.40_rkx , 2131.60_rkx , 335.20_rkx , 212.19_rkx , 514.02_rkx , &
    2027.0_rkx , 438.30_rkx , 2149.20_rkx , 335.90_rkx , 208.70_rkx , 509.13_rkx , &
    2028.0_rkx , 441.20_rkx , 2166.80_rkx , 336.60_rkx , 205.19_rkx , 504.25_rkx , &
    2029.0_rkx , 444.10_rkx , 2184.40_rkx , 337.30_rkx , 201.68_rkx , 499.38_rkx , &
    2030.0_rkx , 447.00_rkx , 2202.00_rkx , 338.00_rkx , 198.17_rkx , 494.53_rkx , &
    2031.0_rkx , 450.60_rkx , 2215.50_rkx , 338.60_rkx , 194.69_rkx , 489.73_rkx , &
    2032.0_rkx , 454.20_rkx , 2229.00_rkx , 339.20_rkx , 191.26_rkx , 484.96_rkx , &
    2033.0_rkx , 457.80_rkx , 2242.50_rkx , 339.80_rkx , 187.86_rkx , 480.25_rkx , &
    2034.0_rkx , 461.40_rkx , 2256.00_rkx , 340.40_rkx , 184.50_rkx , 475.57_rkx , &
    2035.0_rkx , 465.00_rkx , 2269.50_rkx , 341.00_rkx , 181.18_rkx , 470.94_rkx , &
    2036.0_rkx , 468.60_rkx , 2283.00_rkx , 341.60_rkx , 177.90_rkx , 466.36_rkx , &
    2037.0_rkx , 472.20_rkx , 2296.50_rkx , 342.20_rkx , 174.65_rkx , 461.82_rkx , &
    2038.0_rkx , 475.80_rkx , 2310.00_rkx , 342.80_rkx , 171.44_rkx , 457.31_rkx , &
    2039.0_rkx , 479.40_rkx , 2323.50_rkx , 343.40_rkx , 168.26_rkx , 452.86_rkx , &
    2040.0_rkx , 483.00_rkx , 2337.00_rkx , 344.00_rkx , 165.11_rkx , 448.44_rkx , &
    2041.0_rkx , 486.90_rkx , 2343.30_rkx , 344.60_rkx , 162.02_rkx , 444.06_rkx , &
    2042.0_rkx , 490.80_rkx , 2349.60_rkx , 345.20_rkx , 158.98_rkx , 439.73_rkx , &
    2043.0_rkx , 494.70_rkx , 2355.90_rkx , 345.80_rkx , 155.98_rkx , 435.44_rkx , &
    2044.0_rkx , 498.60_rkx , 2362.20_rkx , 346.40_rkx , 153.03_rkx , 431.20_rkx , &
    2045.0_rkx , 502.50_rkx , 2368.50_rkx , 347.00_rkx , 150.13_rkx , 426.99_rkx , &
    2046.0_rkx , 506.40_rkx , 2374.80_rkx , 347.60_rkx , 147.28_rkx , 422.82_rkx , &
    2047.0_rkx , 510.30_rkx , 2381.10_rkx , 348.20_rkx , 144.47_rkx , 418.70_rkx , &
    2048.0_rkx , 514.20_rkx , 2387.40_rkx , 348.80_rkx , 141.70_rkx , 414.61_rkx , &
    2049.0_rkx , 518.10_rkx , 2393.70_rkx , 349.40_rkx , 138.98_rkx , 410.57_rkx , &
    2050.0_rkx , 522.00_rkx , 2400.00_rkx , 350.00_rkx , 136.30_rkx , 406.56_rkx /), &
        (/6,50/))
        cgas(1:6,2051:2100) = reshape((/   &
    2051.0_rkx , 526.10_rkx , 2398.60_rkx , 350.60_rkx , 133.67_rkx , 402.60_rkx , &
    2052.0_rkx , 530.20_rkx , 2397.20_rkx , 351.20_rkx , 131.09_rkx , 398.67_rkx , &
    2053.0_rkx , 534.30_rkx , 2395.80_rkx , 351.80_rkx , 128.55_rkx , 394.78_rkx , &
    2054.0_rkx , 538.40_rkx , 2394.40_rkx , 352.40_rkx , 126.06_rkx , 390.93_rkx , &
    2055.0_rkx , 542.50_rkx , 2393.00_rkx , 353.00_rkx , 123.62_rkx , 387.11_rkx , &
    2056.0_rkx , 546.60_rkx , 2391.60_rkx , 353.60_rkx , 121.21_rkx , 383.34_rkx , &
    2057.0_rkx , 550.70_rkx , 2390.20_rkx , 354.20_rkx , 118.86_rkx , 379.60_rkx , &
    2058.0_rkx , 554.80_rkx , 2388.80_rkx , 354.80_rkx , 116.54_rkx , 375.89_rkx , &
    2059.0_rkx , 558.90_rkx , 2387.40_rkx , 355.40_rkx , 114.26_rkx , 372.23_rkx , &
    2060.0_rkx , 563.00_rkx , 2386.00_rkx , 356.00_rkx , 112.03_rkx , 368.60_rkx , &
    2061.0_rkx , 566.80_rkx , 2377.50_rkx , 356.40_rkx , 109.84_rkx , 365.00_rkx , &
    2062.0_rkx , 570.60_rkx , 2369.00_rkx , 356.80_rkx , 107.69_rkx , 361.44_rkx , &
    2063.0_rkx , 574.40_rkx , 2360.50_rkx , 357.20_rkx , 105.58_rkx , 357.91_rkx , &
    2064.0_rkx , 578.20_rkx , 2352.00_rkx , 357.60_rkx , 103.51_rkx , 354.42_rkx , &
    2065.0_rkx , 582.00_rkx , 2343.50_rkx , 358.00_rkx , 101.48_rkx , 350.96_rkx , &
    2066.0_rkx , 585.80_rkx , 2335.00_rkx , 358.40_rkx , 99.489_rkx , 347.54_rkx , &
    2067.0_rkx , 589.60_rkx , 2326.50_rkx , 358.80_rkx , 97.535_rkx , 344.15_rkx , &
    2068.0_rkx , 593.40_rkx , 2318.00_rkx , 359.20_rkx , 95.618_rkx , 340.79_rkx , &
    2069.0_rkx , 597.20_rkx , 2309.50_rkx , 359.60_rkx , 93.738_rkx , 337.47_rkx , &
    2070.0_rkx , 601.00_rkx , 2301.00_rkx , 360.00_rkx , 91.893_rkx , 334.17_rkx , &
    2071.0_rkx , 604.80_rkx , 2290.00_rkx , 360.50_rkx , 90.083_rkx , 330.91_rkx , &
    2072.0_rkx , 608.60_rkx , 2279.00_rkx , 361.00_rkx , 88.309_rkx , 327.69_rkx , &
    2073.0_rkx , 612.40_rkx , 2268.00_rkx , 361.50_rkx , 86.570_rkx , 324.49_rkx , &
    2074.0_rkx , 616.20_rkx , 2257.00_rkx , 362.00_rkx , 84.864_rkx , 321.32_rkx , &
    2075.0_rkx , 620.00_rkx , 2246.00_rkx , 362.50_rkx , 83.191_rkx , 318.19_rkx , &
    2076.0_rkx , 623.80_rkx , 2235.00_rkx , 363.00_rkx , 81.551_rkx , 315.08_rkx , &
    2077.0_rkx , 627.60_rkx , 2224.00_rkx , 363.50_rkx , 79.942_rkx , 312.01_rkx , &
    2078.0_rkx , 631.40_rkx , 2213.00_rkx , 364.00_rkx , 78.365_rkx , 308.97_rkx , &
    2079.0_rkx , 635.20_rkx , 2202.00_rkx , 364.50_rkx , 76.818_rkx , 305.95_rkx , &
    2080.0_rkx , 639.00_rkx , 2191.00_rkx , 365.00_rkx , 75.301_rkx , 302.97_rkx , &
    2081.0_rkx , 642.50_rkx , 2179.70_rkx , 365.30_rkx , 73.815_rkx , 300.01_rkx , &
    2082.0_rkx , 646.00_rkx , 2168.40_rkx , 365.60_rkx , 72.357_rkx , 297.08_rkx , &
    2083.0_rkx , 649.50_rkx , 2157.10_rkx , 365.90_rkx , 70.928_rkx , 294.19_rkx , &
    2084.0_rkx , 653.00_rkx , 2145.80_rkx , 366.20_rkx , 69.526_rkx , 291.32_rkx , &
    2085.0_rkx , 656.50_rkx , 2134.50_rkx , 366.50_rkx , 68.153_rkx , 288.47_rkx , &
    2086.0_rkx , 660.00_rkx , 2123.20_rkx , 366.80_rkx , 66.806_rkx , 285.66_rkx , &
    2087.0_rkx , 663.50_rkx , 2111.90_rkx , 367.10_rkx , 65.486_rkx , 282.87_rkx , &
    2088.0_rkx , 667.00_rkx , 2100.60_rkx , 367.40_rkx , 64.191_rkx , 280.11_rkx , &
    2089.0_rkx , 670.50_rkx , 2089.30_rkx , 367.70_rkx , 62.922_rkx , 277.38_rkx , &
    2090.0_rkx , 674.00_rkx , 2078.00_rkx , 368.00_rkx , 61.678_rkx , 274.67_rkx , &
    2091.0_rkx , 676.90_rkx , 2067.60_rkx , 368.40_rkx , 60.458_rkx , 271.99_rkx , &
    2092.0_rkx , 679.80_rkx , 2057.20_rkx , 368.80_rkx , 59.263_rkx , 269.34_rkx , &
    2093.0_rkx , 682.70_rkx , 2046.80_rkx , 369.20_rkx , 58.091_rkx , 266.71_rkx , &
    2094.0_rkx , 685.60_rkx , 2036.40_rkx , 369.60_rkx , 56.942_rkx , 264.11_rkx , &
    2095.0_rkx , 688.50_rkx , 2026.00_rkx , 370.00_rkx , 55.816_rkx , 261.53_rkx , &
    2096.0_rkx , 691.40_rkx , 2015.60_rkx , 370.40_rkx , 54.712_rkx , 258.98_rkx , &
    2097.0_rkx , 694.30_rkx , 2005.20_rkx , 370.80_rkx , 53.630_rkx , 256.46_rkx , &
    2098.0_rkx , 697.20_rkx , 1994.80_rkx , 371.20_rkx , 52.569_rkx , 253.95_rkx , &
    2099.0_rkx , 700.10_rkx , 1984.40_rkx , 371.60_rkx , 51.529_rkx , 251.48_rkx , &
    2100.0_rkx , 703.00_rkx , 1974.00_rkx , 372.00_rkx , 50.509_rkx , 249.02_rkx /), &
        (/6,50/))
      case( 'RF' , '20C' )
        cgas(1:6,2001:2050) = reshape((/   &
    2001.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2002.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2003.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2004.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2005.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2006.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2007.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2008.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2009.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2010.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2011.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2012.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2013.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2014.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2015.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2016.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2017.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2018.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2019.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2020.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2021.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2022.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2023.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2024.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2025.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2026.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2027.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2028.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2029.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2030.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2031.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2032.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2033.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2034.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2035.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2036.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2037.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2038.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2039.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2040.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2041.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2042.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2043.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2044.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2045.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2046.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2047.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2048.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2049.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2059.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx /), &
        (/6,50/))
        cgas(1:6,2051:2100) = reshape((/  &
    2051.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2052.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2053.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2054.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2055.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2056.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2057.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2058.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2059.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2060.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2061.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2062.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2063.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2064.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2065.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2066.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2067.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2068.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2069.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2070.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2071.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2072.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2073.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2074.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2075.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2076.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2077.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2078.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2079.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2080.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2081.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2082.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2083.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2084.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2085.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2086.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2087.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2088.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2089.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2090.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2091.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2092.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2093.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2094.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2095.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2096.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2097.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2098.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2099.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx , &
    2100.0_rkx , 367.00_rkx , 1760.00_rkx , 316.00_rkx , 278.91_rkx , 548.67_rkx /), &
        (/6,50/))
      case( 'A2' )
        cgas(1:6,2001:2050) = reshape((/   &
    2001.0_rkx , 368.90_rkx , 1770.10_rkx , 316.90_rkx , 277.52_rkx , 551.75_rkx , &
    2002.0_rkx , 370.80_rkx , 1780.20_rkx , 317.80_rkx , 276.05_rkx , 554.56_rkx , &
    2003.0_rkx , 372.70_rkx , 1790.30_rkx , 318.70_rkx , 274.50_rkx , 557.12_rkx , &
    2004.0_rkx , 374.60_rkx , 1800.40_rkx , 319.60_rkx , 272.88_rkx , 559.41_rkx , &
    2005.0_rkx , 376.50_rkx , 1810.50_rkx , 320.50_rkx , 271.17_rkx , 561.46_rkx , &
    2006.0_rkx , 378.40_rkx , 1820.60_rkx , 321.40_rkx , 269.39_rkx , 563.25_rkx , &
    2007.0_rkx , 380.30_rkx , 1830.70_rkx , 322.30_rkx , 267.54_rkx , 564.80_rkx , &
    2008.0_rkx , 382.20_rkx , 1840.80_rkx , 323.20_rkx , 265.62_rkx , 566.10_rkx , &
    2009.0_rkx , 384.10_rkx , 1850.90_rkx , 324.10_rkx , 263.62_rkx , 567.15_rkx , &
    2010.0_rkx , 386.00_rkx , 1861.00_rkx , 325.00_rkx , 261.56_rkx , 567.97_rkx , &
    2011.0_rkx , 388.80_rkx , 1874.60_rkx , 326.00_rkx , 259.35_rkx , 568.17_rkx , &
    2012.0_rkx , 391.60_rkx , 1888.20_rkx , 327.00_rkx , 256.98_rkx , 567.78_rkx , &
    2013.0_rkx , 394.40_rkx , 1901.80_rkx , 328.00_rkx , 254.47_rkx , 566.79_rkx , &
    2014.0_rkx , 397.20_rkx , 1915.40_rkx , 329.00_rkx , 251.82_rkx , 565.21_rkx , &
    2015.0_rkx , 400.00_rkx , 1929.00_rkx , 330.00_rkx , 249.03_rkx , 563.05_rkx , &
    2016.0_rkx , 402.80_rkx , 1942.60_rkx , 331.00_rkx , 246.10_rkx , 560.31_rkx , &
    2017.0_rkx , 405.60_rkx , 1956.20_rkx , 332.00_rkx , 243.03_rkx , 557.00_rkx , &
    2018.0_rkx , 408.40_rkx , 1969.80_rkx , 333.00_rkx , 239.83_rkx , 553.12_rkx , &
    2019.0_rkx , 411.20_rkx , 1983.40_rkx , 334.00_rkx , 236.51_rkx , 548.68_rkx , &
    2020.0_rkx , 414.00_rkx , 1997.00_rkx , 335.00_rkx , 233.05_rkx , 543.68_rkx , &
    2021.0_rkx , 417.00_rkx , 2013.60_rkx , 336.20_rkx , 229.59_rkx , 538.70_rkx , &
    2022.0_rkx , 420.00_rkx , 2030.20_rkx , 337.40_rkx , 226.12_rkx , 533.74_rkx , &
    2023.0_rkx , 423.00_rkx , 2046.80_rkx , 338.60_rkx , 222.65_rkx , 528.79_rkx , &
    2024.0_rkx , 426.00_rkx , 2063.40_rkx , 339.80_rkx , 219.17_rkx , 523.85_rkx , &
    2025.0_rkx , 429.00_rkx , 2080.00_rkx , 341.00_rkx , 215.68_rkx , 518.93_rkx , &
    2026.0_rkx , 432.00_rkx , 2096.60_rkx , 342.20_rkx , 212.19_rkx , 514.02_rkx , &
    2027.0_rkx , 435.00_rkx , 2113.20_rkx , 343.40_rkx , 208.70_rkx , 509.13_rkx , &
    2028.0_rkx , 438.00_rkx , 2129.80_rkx , 344.60_rkx , 205.19_rkx , 504.25_rkx , &
    2029.0_rkx , 441.00_rkx , 2146.40_rkx , 345.80_rkx , 201.68_rkx , 499.38_rkx , &
    2030.0_rkx , 444.00_rkx , 2163.00_rkx , 347.00_rkx , 198.17_rkx , 494.53_rkx , &
    2031.0_rkx , 447.70_rkx , 2182.40_rkx , 348.30_rkx , 194.69_rkx , 489.73_rkx , &
    2032.0_rkx , 451.40_rkx , 2201.80_rkx , 349.60_rkx , 191.26_rkx , 484.96_rkx , &
    2033.0_rkx , 455.10_rkx , 2221.20_rkx , 350.90_rkx , 187.86_rkx , 480.25_rkx , &
    2034.0_rkx , 458.80_rkx , 2240.60_rkx , 352.20_rkx , 184.50_rkx , 475.57_rkx , &
    2035.0_rkx , 462.50_rkx , 2260.00_rkx , 353.50_rkx , 181.18_rkx , 470.94_rkx , &
    2036.0_rkx , 466.20_rkx , 2279.40_rkx , 354.80_rkx , 177.90_rkx , 466.36_rkx , &
    2037.0_rkx , 469.90_rkx , 2298.80_rkx , 356.10_rkx , 174.65_rkx , 461.82_rkx , &
    2038.0_rkx , 473.60_rkx , 2318.20_rkx , 357.40_rkx , 171.44_rkx , 457.31_rkx , &
    2039.0_rkx , 477.30_rkx , 2337.60_rkx , 358.70_rkx , 168.26_rkx , 452.86_rkx , &
    2040.0_rkx , 481.00_rkx , 2357.00_rkx , 360.00_rkx , 165.11_rkx , 448.44_rkx , &
    2041.0_rkx , 485.10_rkx , 2377.50_rkx , 361.30_rkx , 162.02_rkx , 444.06_rkx , &
    2042.0_rkx , 489.20_rkx , 2398.00_rkx , 362.60_rkx , 158.98_rkx , 439.73_rkx , &
    2043.0_rkx , 493.30_rkx , 2418.50_rkx , 363.90_rkx , 155.98_rkx , 435.44_rkx , &
    2044.0_rkx , 497.40_rkx , 2439.00_rkx , 365.20_rkx , 153.03_rkx , 431.20_rkx , &
    2045.0_rkx , 501.50_rkx , 2459.50_rkx , 366.50_rkx , 150.13_rkx , 426.99_rkx , &
    2046.0_rkx , 505.60_rkx , 2480.00_rkx , 367.80_rkx , 147.28_rkx , 422.82_rkx , &
    2047.0_rkx , 509.70_rkx , 2500.50_rkx , 369.10_rkx , 144.47_rkx , 418.70_rkx , &
    2048.0_rkx , 513.80_rkx , 2521.00_rkx , 370.40_rkx , 141.70_rkx , 414.61_rkx , &
    2049.0_rkx , 517.90_rkx , 2541.50_rkx , 371.70_rkx , 138.98_rkx , 410.57_rkx , &
    2050.0_rkx , 522.00_rkx , 2562.00_rkx , 373.00_rkx , 136.30_rkx , 406.56_rkx /), &
        (/6,50/))
        cgas(1:6,2051:2100) = reshape((/   &
    2051.0_rkx , 526.60_rkx , 2583.70_rkx , 374.40_rkx , 133.67_rkx , 402.60_rkx , &
    2052.0_rkx , 531.20_rkx , 2605.40_rkx , 375.80_rkx , 131.09_rkx , 398.67_rkx , &
    2053.0_rkx , 535.80_rkx , 2627.10_rkx , 377.20_rkx , 128.55_rkx , 394.78_rkx , &
    2054.0_rkx , 540.40_rkx , 2648.80_rkx , 378.60_rkx , 126.06_rkx , 390.93_rkx , &
    2055.0_rkx , 545.00_rkx , 2670.50_rkx , 380.00_rkx , 123.62_rkx , 387.11_rkx , &
    2056.0_rkx , 549.60_rkx , 2692.20_rkx , 381.40_rkx , 121.21_rkx , 383.34_rkx , &
    2057.0_rkx , 554.20_rkx , 2713.90_rkx , 382.80_rkx , 118.86_rkx , 379.60_rkx , &
    2058.0_rkx , 558.80_rkx , 2735.60_rkx , 384.20_rkx , 116.54_rkx , 375.89_rkx , &
    2059.0_rkx , 563.40_rkx , 2757.30_rkx , 385.60_rkx , 114.26_rkx , 372.23_rkx , &
    2060.0_rkx , 568.00_rkx , 2779.00_rkx , 387.00_rkx , 112.03_rkx , 368.60_rkx , &
    2061.0_rkx , 573.20_rkx , 2802.20_rkx , 388.40_rkx , 109.84_rkx , 365.00_rkx , &
    2062.0_rkx , 578.40_rkx , 2825.40_rkx , 389.80_rkx , 107.69_rkx , 361.44_rkx , &
    2063.0_rkx , 583.60_rkx , 2848.60_rkx , 391.20_rkx , 105.58_rkx , 357.91_rkx , &
    2064.0_rkx , 588.80_rkx , 2871.80_rkx , 392.60_rkx , 103.51_rkx , 354.42_rkx , &
    2065.0_rkx , 594.00_rkx , 2895.00_rkx , 394.00_rkx , 101.48_rkx , 350.96_rkx , &
    2066.0_rkx , 599.20_rkx , 2918.20_rkx , 395.40_rkx , 99.489_rkx , 347.54_rkx , &
    2067.0_rkx , 604.40_rkx , 2941.40_rkx , 396.80_rkx , 97.535_rkx , 344.15_rkx , &
    2068.0_rkx , 609.60_rkx , 2964.60_rkx , 398.20_rkx , 95.618_rkx , 340.79_rkx , &
    2069.0_rkx , 614.80_rkx , 2987.80_rkx , 399.60_rkx , 93.738_rkx , 337.47_rkx , &
    2070.0_rkx , 620.00_rkx , 3011.00_rkx , 401.00_rkx , 91.893_rkx , 334.17_rkx , &
    2071.0_rkx , 626.20_rkx , 3035.10_rkx , 402.50_rkx , 90.083_rkx , 330.91_rkx , &
    2072.0_rkx , 632.40_rkx , 3059.20_rkx , 404.00_rkx , 88.309_rkx , 327.69_rkx , &
    2073.0_rkx , 638.60_rkx , 3083.30_rkx , 405.50_rkx , 86.570_rkx , 324.49_rkx , &
    2074.0_rkx , 644.80_rkx , 3107.40_rkx , 407.00_rkx , 84.864_rkx , 321.32_rkx , &
    2075.0_rkx , 651.00_rkx , 3131.50_rkx , 408.50_rkx , 83.191_rkx , 318.19_rkx , &
    2076.0_rkx , 657.20_rkx , 3155.60_rkx , 410.00_rkx , 81.551_rkx , 315.08_rkx , &
    2077.0_rkx , 663.40_rkx , 3179.70_rkx , 411.50_rkx , 79.942_rkx , 312.01_rkx , &
    2078.0_rkx , 669.60_rkx , 3203.80_rkx , 413.00_rkx , 78.365_rkx , 308.97_rkx , &
    2079.0_rkx , 675.80_rkx , 3227.90_rkx , 414.50_rkx , 76.818_rkx , 305.95_rkx , &
    2080.0_rkx , 682.00_rkx , 3252.00_rkx , 416.00_rkx , 75.301_rkx , 302.97_rkx , &
    2081.0_rkx , 689.20_rkx , 3276.10_rkx , 417.60_rkx , 73.815_rkx , 300.01_rkx , &
    2082.0_rkx , 696.40_rkx , 3300.20_rkx , 419.20_rkx , 72.357_rkx , 297.08_rkx , &
    2083.0_rkx , 703.60_rkx , 3324.30_rkx , 420.80_rkx , 70.928_rkx , 294.19_rkx , &
    2084.0_rkx , 710.80_rkx , 3348.40_rkx , 422.40_rkx , 69.526_rkx , 291.32_rkx , &
    2085.0_rkx , 718.00_rkx , 3372.50_rkx , 424.00_rkx , 68.153_rkx , 288.47_rkx , &
    2086.0_rkx , 725.20_rkx , 3396.60_rkx , 425.60_rkx , 66.806_rkx , 285.66_rkx , &
    2087.0_rkx , 732.40_rkx , 3420.70_rkx , 427.20_rkx , 65.486_rkx , 282.87_rkx , &
    2088.0_rkx , 739.60_rkx , 3444.80_rkx , 428.80_rkx , 64.191_rkx , 280.11_rkx , &
    2089.0_rkx , 746.80_rkx , 3468.90_rkx , 430.40_rkx , 62.922_rkx , 277.38_rkx , &
    2090.0_rkx , 754.00_rkx , 3493.00_rkx , 432.00_rkx , 61.678_rkx , 274.67_rkx , &
    2091.0_rkx , 762.20_rkx , 3516.80_rkx , 433.50_rkx , 60.458_rkx , 271.99_rkx , &
    2092.0_rkx , 770.40_rkx , 3540.60_rkx , 435.00_rkx , 59.263_rkx , 269.34_rkx , &
    2093.0_rkx , 778.60_rkx , 3564.40_rkx , 436.50_rkx , 58.091_rkx , 266.71_rkx , &
    2094.0_rkx , 786.80_rkx , 3588.20_rkx , 438.00_rkx , 56.942_rkx , 264.11_rkx , &
    2095.0_rkx , 795.00_rkx , 3612.00_rkx , 439.50_rkx , 55.816_rkx , 261.53_rkx , &
    2096.0_rkx , 803.20_rkx , 3635.80_rkx , 441.00_rkx , 54.712_rkx , 258.98_rkx , &
    2097.0_rkx , 811.40_rkx , 3659.60_rkx , 442.50_rkx , 53.630_rkx , 256.46_rkx , &
    2098.0_rkx , 819.60_rkx , 3683.40_rkx , 444.00_rkx , 52.569_rkx , 253.95_rkx , &
    2099.0_rkx , 827.80_rkx , 3707.20_rkx , 445.50_rkx , 51.529_rkx , 251.48_rkx , &
    2100.0_rkx , 836.00_rkx , 3731.00_rkx , 447.00_rkx , 50.509_rkx , 249.02_rkx /), &
        (/6,50/))
      case( 'B1' )
        cgas(1:6,2001:2050) = reshape((/   &
    2001.0_rkx , 368.90_rkx , 1766.70_rkx , 316.80_rkx , 277.52_rkx , 551.75_rkx , &
    2002.0_rkx , 370.80_rkx , 1773.40_rkx , 317.60_rkx , 276.05_rkx , 554.56_rkx , &
    2003.0_rkx , 372.70_rkx , 1780.10_rkx , 318.40_rkx , 274.50_rkx , 557.12_rkx , &
    2004.0_rkx , 374.60_rkx , 1786.80_rkx , 319.20_rkx , 272.88_rkx , 559.41_rkx , &
    2005.0_rkx , 376.50_rkx , 1793.50_rkx , 320.00_rkx , 271.17_rkx , 561.46_rkx , &
    2006.0_rkx , 378.40_rkx , 1800.20_rkx , 320.80_rkx , 269.39_rkx , 563.25_rkx , &
    2007.0_rkx , 380.30_rkx , 1806.90_rkx , 321.60_rkx , 267.54_rkx , 564.80_rkx , &
    2008.0_rkx , 382.20_rkx , 1813.60_rkx , 322.40_rkx , 265.62_rkx , 566.10_rkx , &
    2009.0_rkx , 384.10_rkx , 1820.30_rkx , 323.20_rkx , 263.62_rkx , 567.15_rkx , &
    2010.0_rkx , 386.00_rkx , 1827.00_rkx , 324.00_rkx , 261.56_rkx , 567.97_rkx , &
    2011.0_rkx , 388.40_rkx , 1833.40_rkx , 324.90_rkx , 259.35_rkx , 568.17_rkx , &
    2012.0_rkx , 390.80_rkx , 1839.80_rkx , 325.80_rkx , 256.98_rkx , 567.78_rkx , &
    2013.0_rkx , 393.20_rkx , 1846.20_rkx , 326.70_rkx , 254.47_rkx , 566.79_rkx , &
    2014.0_rkx , 395.60_rkx , 1852.60_rkx , 327.60_rkx , 251.82_rkx , 565.21_rkx , &
    2015.0_rkx , 398.00_rkx , 1859.00_rkx , 328.50_rkx , 249.03_rkx , 563.05_rkx , &
    2016.0_rkx , 400.40_rkx , 1865.40_rkx , 329.40_rkx , 246.10_rkx , 560.31_rkx , &
    2017.0_rkx , 402.80_rkx , 1871.80_rkx , 330.30_rkx , 243.03_rkx , 557.00_rkx , &
    2018.0_rkx , 405.20_rkx , 1878.20_rkx , 331.20_rkx , 239.83_rkx , 553.12_rkx , &
    2019.0_rkx , 407.60_rkx , 1884.60_rkx , 332.10_rkx , 236.51_rkx , 548.68_rkx , &
    2020.0_rkx , 410.00_rkx , 1891.00_rkx , 333.00_rkx , 233.05_rkx , 543.68_rkx , &
    2021.0_rkx , 412.20_rkx , 1894.60_rkx , 333.80_rkx , 229.59_rkx , 538.70_rkx , &
    2022.0_rkx , 414.40_rkx , 1898.20_rkx , 334.60_rkx , 226.12_rkx , 533.74_rkx , &
    2023.0_rkx , 416.60_rkx , 1901.80_rkx , 335.40_rkx , 222.65_rkx , 528.79_rkx , &
    2024.0_rkx , 418.80_rkx , 1905.40_rkx , 336.20_rkx , 219.17_rkx , 523.85_rkx , &
    2025.0_rkx , 421.00_rkx , 1909.00_rkx , 337.00_rkx , 215.68_rkx , 518.93_rkx , &
    2026.0_rkx , 423.20_rkx , 1912.60_rkx , 337.80_rkx , 212.19_rkx , 514.02_rkx , &
    2027.0_rkx , 425.40_rkx , 1916.20_rkx , 338.60_rkx , 208.70_rkx , 509.13_rkx , &
    2028.0_rkx , 427.60_rkx , 1919.80_rkx , 339.40_rkx , 205.19_rkx , 504.25_rkx , &
    2029.0_rkx , 429.80_rkx , 1923.40_rkx , 340.20_rkx , 201.68_rkx , 499.38_rkx , &
    2030.0_rkx , 432.00_rkx , 1927.00_rkx , 341.00_rkx , 198.17_rkx , 494.53_rkx , &
    2031.0_rkx , 434.50_rkx , 1926.20_rkx , 341.80_rkx , 194.69_rkx , 489.73_rkx , &
    2032.0_rkx , 437.00_rkx , 1925.40_rkx , 342.60_rkx , 191.26_rkx , 484.96_rkx , &
    2033.0_rkx , 439.50_rkx , 1924.60_rkx , 343.40_rkx , 187.86_rkx , 480.25_rkx , &
    2034.0_rkx , 442.00_rkx , 1923.80_rkx , 344.20_rkx , 184.50_rkx , 475.57_rkx , &
    2035.0_rkx , 444.50_rkx , 1923.00_rkx , 345.00_rkx , 181.18_rkx , 470.94_rkx , &
    2036.0_rkx , 447.00_rkx , 1922.20_rkx , 345.80_rkx , 177.90_rkx , 466.36_rkx , &
    2037.0_rkx , 449.50_rkx , 1921.40_rkx , 346.60_rkx , 174.65_rkx , 461.82_rkx , &
    2038.0_rkx , 452.00_rkx , 1920.60_rkx , 347.40_rkx , 171.44_rkx , 457.31_rkx , &
    2039.0_rkx , 454.50_rkx , 1919.80_rkx , 348.20_rkx , 168.26_rkx , 452.86_rkx , &
    2040.0_rkx , 457.00_rkx , 1919.00_rkx , 349.00_rkx , 165.11_rkx , 448.44_rkx , &
    2041.0_rkx , 459.50_rkx , 1915.20_rkx , 349.80_rkx , 162.02_rkx , 444.06_rkx , &
    2042.0_rkx , 462.00_rkx , 1911.40_rkx , 350.60_rkx , 158.98_rkx , 439.73_rkx , &
    2043.0_rkx , 464.50_rkx , 1907.60_rkx , 351.40_rkx , 155.98_rkx , 435.44_rkx , &
    2044.0_rkx , 467.00_rkx , 1903.80_rkx , 352.20_rkx , 153.03_rkx , 431.20_rkx , &
    2045.0_rkx , 469.50_rkx , 1900.00_rkx , 353.00_rkx , 150.13_rkx , 426.99_rkx , &
    2046.0_rkx , 472.00_rkx , 1896.20_rkx , 353.80_rkx , 147.28_rkx , 422.82_rkx , &
    2047.0_rkx , 474.50_rkx , 1892.40_rkx , 354.60_rkx , 144.47_rkx , 418.70_rkx , &
    2048.0_rkx , 477.00_rkx , 1888.60_rkx , 355.40_rkx , 141.70_rkx , 414.61_rkx , &
    2049.0_rkx , 479.50_rkx , 1884.80_rkx , 356.20_rkx , 138.98_rkx , 410.57_rkx , &
    2050.0_rkx , 482.00_rkx , 1881.00_rkx , 357.00_rkx , 136.30_rkx , 406.56_rkx /), &
        (/6,50/))
        cgas(1:6,2051:2100) = reshape((/   &
    2051.0_rkx , 484.10_rkx , 1876.50_rkx , 357.60_rkx , 133.67_rkx , 402.60_rkx , &
    2052.0_rkx , 486.20_rkx , 1872.00_rkx , 358.20_rkx , 131.09_rkx , 398.67_rkx , &
    2053.0_rkx , 488.30_rkx , 1867.50_rkx , 358.80_rkx , 128.55_rkx , 394.78_rkx , &
    2054.0_rkx , 490.40_rkx , 1863.00_rkx , 359.40_rkx , 126.06_rkx , 390.93_rkx , &
    2055.0_rkx , 492.50_rkx , 1858.50_rkx , 360.00_rkx , 123.62_rkx , 387.11_rkx , &
    2056.0_rkx , 494.60_rkx , 1854.00_rkx , 360.60_rkx , 121.21_rkx , 383.34_rkx , &
    2057.0_rkx , 496.70_rkx , 1849.50_rkx , 361.20_rkx , 118.86_rkx , 379.60_rkx , &
    2058.0_rkx , 498.80_rkx , 1845.00_rkx , 361.80_rkx , 116.54_rkx , 375.89_rkx , &
    2059.0_rkx , 500.90_rkx , 1840.50_rkx , 362.40_rkx , 114.26_rkx , 372.23_rkx , &
    2060.0_rkx , 503.00_rkx , 1836.00_rkx , 363.00_rkx , 112.03_rkx , 368.60_rkx , &
    2061.0_rkx , 504.50_rkx , 1832.10_rkx , 363.50_rkx , 109.84_rkx , 365.00_rkx , &
    2062.0_rkx , 506.00_rkx , 1828.20_rkx , 364.00_rkx , 107.69_rkx , 361.44_rkx , &
    2063.0_rkx , 507.50_rkx , 1824.30_rkx , 364.50_rkx , 105.58_rkx , 357.91_rkx , &
    2064.0_rkx , 509.00_rkx , 1820.40_rkx , 365.00_rkx , 103.51_rkx , 354.42_rkx , &
    2065.0_rkx , 510.50_rkx , 1816.50_rkx , 365.50_rkx , 101.48_rkx , 350.96_rkx , &
    2066.0_rkx , 512.00_rkx , 1812.60_rkx , 366.00_rkx , 99.489_rkx , 347.54_rkx , &
    2067.0_rkx , 513.50_rkx , 1808.70_rkx , 366.50_rkx , 97.535_rkx , 344.15_rkx , &
    2068.0_rkx , 515.00_rkx , 1804.80_rkx , 367.00_rkx , 95.618_rkx , 340.79_rkx , &
    2069.0_rkx , 516.50_rkx , 1800.90_rkx , 367.50_rkx , 93.738_rkx , 337.47_rkx , &
    2070.0_rkx , 518.00_rkx , 1797.00_rkx , 368.00_rkx , 91.893_rkx , 334.17_rkx , &
    2071.0_rkx , 519.20_rkx , 1791.40_rkx , 368.30_rkx , 90.083_rkx , 330.91_rkx , &
    2072.0_rkx , 520.40_rkx , 1785.80_rkx , 368.60_rkx , 88.309_rkx , 327.69_rkx , &
    2073.0_rkx , 521.60_rkx , 1780.20_rkx , 368.90_rkx , 86.570_rkx , 324.49_rkx , &
    2074.0_rkx , 522.80_rkx , 1774.60_rkx , 369.20_rkx , 84.864_rkx , 321.32_rkx , &
    2075.0_rkx , 524.00_rkx , 1769.00_rkx , 369.50_rkx , 83.191_rkx , 318.19_rkx , &
    2076.0_rkx , 525.20_rkx , 1763.40_rkx , 369.80_rkx , 81.551_rkx , 315.08_rkx , &
    2077.0_rkx , 526.40_rkx , 1757.80_rkx , 370.10_rkx , 79.942_rkx , 312.01_rkx , &
    2078.0_rkx , 527.60_rkx , 1752.20_rkx , 370.40_rkx , 78.365_rkx , 308.97_rkx , &
    2079.0_rkx , 528.80_rkx , 1746.60_rkx , 370.70_rkx , 76.818_rkx , 305.95_rkx , &
    2080.0_rkx , 530.00_rkx , 1741.00_rkx , 371.00_rkx , 75.301_rkx , 302.97_rkx , &
    2081.0_rkx , 530.80_rkx , 1733.20_rkx , 371.30_rkx , 73.815_rkx , 300.01_rkx , &
    2082.0_rkx , 531.60_rkx , 1725.40_rkx , 371.60_rkx , 72.357_rkx , 297.08_rkx , &
    2083.0_rkx , 532.40_rkx , 1717.60_rkx , 371.90_rkx , 70.928_rkx , 294.19_rkx , &
    2084.0_rkx , 533.20_rkx , 1709.80_rkx , 372.20_rkx , 69.526_rkx , 291.32_rkx , &
    2085.0_rkx , 534.00_rkx , 1702.00_rkx , 372.50_rkx , 68.153_rkx , 288.47_rkx , &
    2086.0_rkx , 534.80_rkx , 1694.20_rkx , 372.80_rkx , 66.806_rkx , 285.66_rkx , &
    2087.0_rkx , 535.60_rkx , 1686.40_rkx , 373.10_rkx , 65.486_rkx , 282.87_rkx , &
    2088.0_rkx , 536.40_rkx , 1678.60_rkx , 373.40_rkx , 64.191_rkx , 280.11_rkx , &
    2089.0_rkx , 537.20_rkx , 1670.80_rkx , 373.70_rkx , 62.922_rkx , 277.38_rkx , &
    2090.0_rkx , 538.00_rkx , 1663.00_rkx , 374.00_rkx , 61.678_rkx , 274.67_rkx , &
    2091.0_rkx , 538.20_rkx , 1654.10_rkx , 374.10_rkx , 60.458_rkx , 271.99_rkx , &
    2092.0_rkx , 538.40_rkx , 1645.20_rkx , 374.20_rkx , 59.263_rkx , 269.34_rkx , &
    2093.0_rkx , 538.60_rkx , 1636.30_rkx , 374.30_rkx , 58.091_rkx , 266.71_rkx , &
    2094.0_rkx , 538.80_rkx , 1627.40_rkx , 374.40_rkx , 56.942_rkx , 264.11_rkx , &
    2095.0_rkx , 539.00_rkx , 1618.50_rkx , 374.50_rkx , 55.816_rkx , 261.53_rkx , &
    2096.0_rkx , 539.20_rkx , 1609.60_rkx , 374.60_rkx , 54.712_rkx , 258.98_rkx , &
    2097.0_rkx , 539.40_rkx , 1600.70_rkx , 374.70_rkx , 53.630_rkx , 256.46_rkx , &
    2098.0_rkx , 539.60_rkx , 1591.80_rkx , 374.80_rkx , 52.569_rkx , 253.95_rkx , &
    2099.0_rkx , 539.80_rkx , 1582.90_rkx , 374.90_rkx , 51.529_rkx , 251.48_rkx , &
    2100.0_rkx , 540.00_rkx , 1574.00_rkx , 375.00_rkx , 50.509_rkx , 249.02_rkx /), &
        (/6,50/))
      case( 'B2 ' )
        cgas(1:6,2001:2050) = reshape((/   &
    2001.0_rkx , 373.28_rkx , 1810.00_rkx , 318.99_rkx , 277.52_rkx , 551.75_rkx , &
    2002.0_rkx , 375.11_rkx , 1820.50_rkx , 319.85_rkx , 276.05_rkx , 554.56_rkx , &
    2003.0_rkx , 376.98_rkx , 1831.50_rkx , 320.71_rkx , 274.50_rkx , 557.12_rkx , &
    2004.0_rkx , 378.87_rkx , 1843.20_rkx , 321.56_rkx , 272.88_rkx , 559.41_rkx , &
    2005.0_rkx , 380.80_rkx , 1855.30_rkx , 322.42_rkx , 271.17_rkx , 561.46_rkx , &
    2006.0_rkx , 382.76_rkx , 1868.00_rkx , 323.26_rkx , 269.39_rkx , 563.25_rkx , &
    2007.0_rkx , 384.75_rkx , 1881.10_rkx , 324.11_rkx , 267.54_rkx , 564.80_rkx , &
    2008.0_rkx , 386.77_rkx , 1894.60_rkx , 324.95_rkx , 265.62_rkx , 566.10_rkx , &
    2009.0_rkx , 388.82_rkx , 1908.50_rkx , 325.79_rkx , 263.62_rkx , 567.15_rkx , &
    2010.0_rkx , 390.90_rkx , 1922.80_rkx , 326.63_rkx , 261.56_rkx , 567.97_rkx , &
    2011.0_rkx , 393.01_rkx , 1938.00_rkx , 327.46_rkx , 259.35_rkx , 568.17_rkx , &
    2012.0_rkx , 395.12_rkx , 1954.00_rkx , 328.28_rkx , 256.98_rkx , 567.78_rkx , &
    2013.0_rkx , 397.25_rkx , 1970.80_rkx , 329.10_rkx , 254.47_rkx , 566.79_rkx , &
    2014.0_rkx , 399.38_rkx , 1988.30_rkx , 329.90_rkx , 251.82_rkx , 565.21_rkx , &
    2015.0_rkx , 401.53_rkx , 2006.50_rkx , 330.71_rkx , 249.03_rkx , 563.05_rkx , &
    2016.0_rkx , 403.68_rkx , 2025.30_rkx , 331.50_rkx , 246.10_rkx , 560.31_rkx , &
    2017.0_rkx , 405.85_rkx , 2044.70_rkx , 332.29_rkx , 243.03_rkx , 557.00_rkx , &
    2018.0_rkx , 408.04_rkx , 2064.60_rkx , 333.08_rkx , 239.83_rkx , 553.12_rkx , &
    2019.0_rkx , 410.23_rkx , 2085.10_rkx , 333.85_rkx , 236.51_rkx , 548.68_rkx , &
    2020.0_rkx , 412.45_rkx , 2106.00_rkx , 334.62_rkx , 233.05_rkx , 543.68_rkx , &
    2021.0_rkx , 414.66_rkx , 2127.10_rkx , 335.38_rkx , 229.59_rkx , 538.70_rkx , &
    2022.0_rkx , 416.84_rkx , 2148.60_rkx , 336.12_rkx , 226.12_rkx , 533.74_rkx , &
    2023.0_rkx , 419.00_rkx , 2170.20_rkx , 336.84_rkx , 222.65_rkx , 528.79_rkx , &
    2024.0_rkx , 421.14_rkx , 2192.10_rkx , 337.55_rkx , 219.17_rkx , 523.85_rkx , &
    2025.0_rkx , 423.25_rkx , 2214.10_rkx , 338.25_rkx , 215.68_rkx , 518.93_rkx , &
    2026.0_rkx , 425.34_rkx , 2236.40_rkx , 338.94_rkx , 212.19_rkx , 514.02_rkx , &
    2027.0_rkx , 427.41_rkx , 2258.80_rkx , 339.61_rkx , 208.70_rkx , 509.13_rkx , &
    2028.0_rkx , 429.47_rkx , 2281.40_rkx , 340.26_rkx , 205.19_rkx , 504.25_rkx , &
    2029.0_rkx , 431.50_rkx , 2304.20_rkx , 340.90_rkx , 201.68_rkx , 499.38_rkx , &
    2030.0_rkx , 433.52_rkx , 2327.10_rkx , 341.53_rkx , 198.17_rkx , 494.53_rkx , &
    2031.0_rkx , 435.54_rkx , 2349.30_rkx , 342.15_rkx , 194.69_rkx , 489.73_rkx , &
    2032.0_rkx , 437.57_rkx , 2370.90_rkx , 342.76_rkx , 191.26_rkx , 484.96_rkx , &
    2033.0_rkx , 439.61_rkx , 2391.70_rkx , 343.35_rkx , 187.86_rkx , 480.25_rkx , &
    2034.0_rkx , 441.67_rkx , 2412.10_rkx , 343.93_rkx , 184.50_rkx , 475.57_rkx , &
    2035.0_rkx , 443.74_rkx , 2431.90_rkx , 344.51_rkx , 181.18_rkx , 470.94_rkx , &
    2036.0_rkx , 445.82_rkx , 2451.20_rkx , 345.07_rkx , 177.90_rkx , 466.36_rkx , &
    2037.0_rkx , 447.91_rkx , 2470.00_rkx , 345.62_rkx , 174.65_rkx , 461.82_rkx , &
    2038.0_rkx , 450.02_rkx , 2488.40_rkx , 346.16_rkx , 171.44_rkx , 457.31_rkx , &
    2039.0_rkx , 452.14_rkx , 2506.50_rkx , 346.68_rkx , 168.26_rkx , 452.86_rkx , &
    2040.0_rkx , 454.27_rkx , 2524.20_rkx , 347.20_rkx , 165.11_rkx , 448.44_rkx , &
    2041.0_rkx , 456.40_rkx , 2540.90_rkx , 347.71_rkx , 162.02_rkx , 444.06_rkx , &
    2042.0_rkx , 458.53_rkx , 2556.90_rkx , 348.20_rkx , 158.98_rkx , 439.73_rkx , &
    2043.0_rkx , 460.64_rkx , 2571.90_rkx , 348.68_rkx , 155.98_rkx , 435.44_rkx , &
    2044.0_rkx , 462.75_rkx , 2586.30_rkx , 349.15_rkx , 153.03_rkx , 431.20_rkx , &
    2045.0_rkx , 464.85_rkx , 2600.00_rkx , 349.61_rkx , 150.13_rkx , 426.99_rkx , &
    2046.0_rkx , 466.95_rkx , 2613.00_rkx , 350.05_rkx , 147.28_rkx , 422.82_rkx , &
    2047.0_rkx , 469.04_rkx , 2625.40_rkx , 350.49_rkx , 144.47_rkx , 418.70_rkx , &
    2048.0_rkx , 471.12_rkx , 2637.30_rkx , 350.91_rkx , 141.70_rkx , 414.61_rkx , &
    2049.0_rkx , 473.19_rkx , 2648.70_rkx , 351.32_rkx , 138.98_rkx , 410.57_rkx , &
    2050.0_rkx , 475.27_rkx , 2659.60_rkx , 351.71_rkx , 136.30_rkx , 406.56_rkx /), &
        (/6,50/))
        cgas(1:6,2051:2100) = reshape((/   &
    2051.0_rkx , 477.34_rkx , 2669.90_rkx , 352.10_rkx , 133.67_rkx , 402.60_rkx , &
    2052.0_rkx , 479.44_rkx , 2679.80_rkx , 352.48_rkx , 131.09_rkx , 398.67_rkx , &
    2053.0_rkx , 481.56_rkx , 2689.10_rkx , 352.86_rkx , 128.55_rkx , 394.78_rkx , &
    2054.0_rkx , 483.69_rkx , 2698.10_rkx , 353.23_rkx , 126.06_rkx , 390.93_rkx , &
    2055.0_rkx , 485.85_rkx , 2706.60_rkx , 353.59_rkx , 123.62_rkx , 387.11_rkx , &
    2056.0_rkx , 488.03_rkx , 2714.80_rkx , 353.94_rkx , 121.21_rkx , 383.34_rkx , &
    2057.0_rkx , 490.22_rkx , 2722.60_rkx , 354.29_rkx , 118.86_rkx , 379.60_rkx , &
    2058.0_rkx , 492.43_rkx , 2730.10_rkx , 354.63_rkx , 116.54_rkx , 375.89_rkx , &
    2059.0_rkx , 494.66_rkx , 2737.30_rkx , 354.96_rkx , 114.26_rkx , 372.23_rkx , &
    2060.0_rkx , 496.91_rkx , 2744.20_rkx , 355.28_rkx , 112.03_rkx , 368.60_rkx , &
    2061.0_rkx , 499.17_rkx , 2750.50_rkx , 355.60_rkx , 109.84_rkx , 365.00_rkx , &
    2062.0_rkx , 501.44_rkx , 2756.30_rkx , 355.91_rkx , 107.69_rkx , 361.44_rkx , &
    2063.0_rkx , 503.70_rkx , 2761.60_rkx , 356.21_rkx , 105.58_rkx , 357.91_rkx , &
    2064.0_rkx , 505.96_rkx , 2766.40_rkx , 356.50_rkx , 103.51_rkx , 354.42_rkx , &
    2065.0_rkx , 508.23_rkx , 2770.80_rkx , 356.78_rkx , 101.48_rkx , 350.96_rkx , &
    2066.0_rkx , 510.49_rkx , 2774.70_rkx , 357.05_rkx , 99.489_rkx , 347.54_rkx , &
    2067.0_rkx , 512.77_rkx , 2778.40_rkx , 357.32_rkx , 97.535_rkx , 344.15_rkx , &
    2068.0_rkx , 515.04_rkx , 2781.60_rkx , 357.57_rkx , 95.618_rkx , 340.79_rkx , &
    2069.0_rkx , 517.32_rkx , 2784.50_rkx , 357.82_rkx , 93.738_rkx , 337.47_rkx , &
    2070.0_rkx , 519.60_rkx , 2787.10_rkx , 358.06_rkx , 91.893_rkx , 334.17_rkx , &
    2071.0_rkx , 521.89_rkx , 2789.20_rkx , 358.30_rkx , 90.083_rkx , 330.91_rkx , &
    2072.0_rkx , 524.20_rkx , 2790.60_rkx , 358.53_rkx , 88.309_rkx , 327.69_rkx , &
    2073.0_rkx , 526.53_rkx , 2791.60_rkx , 358.76_rkx , 86.570_rkx , 324.49_rkx , &
    2074.0_rkx , 528.88_rkx , 2792.10_rkx , 358.98_rkx , 84.864_rkx , 321.32_rkx , &
    2075.0_rkx , 531.25_rkx , 2792.10_rkx , 359.20_rkx , 83.191_rkx , 318.19_rkx , &
    2076.0_rkx , 533.64_rkx , 2791.70_rkx , 359.42_rkx , 81.551_rkx , 315.08_rkx , &
    2077.0_rkx , 536.05_rkx , 2790.90_rkx , 359.63_rkx , 79.942_rkx , 312.01_rkx , &
    2078.0_rkx , 538.47_rkx , 2789.80_rkx , 359.83_rkx , 78.365_rkx , 308.97_rkx , &
    2079.0_rkx , 540.92_rkx , 2788.40_rkx , 360.04_rkx , 76.818_rkx , 305.95_rkx , &
    2080.0_rkx , 543.39_rkx , 2786.60_rkx , 360.24_rkx , 75.301_rkx , 302.97_rkx , &
    2081.0_rkx , 545.87_rkx , 2784.20_rkx , 360.43_rkx , 73.815_rkx , 300.01_rkx , &
    2082.0_rkx , 548.38_rkx , 2781.30_rkx , 360.62_rkx , 72.357_rkx , 297.08_rkx , &
    2083.0_rkx , 550.91_rkx , 2777.80_rkx , 360.81_rkx , 70.928_rkx , 294.19_rkx , &
    2084.0_rkx , 553.47_rkx , 2773.90_rkx , 360.98_rkx , 69.526_rkx , 291.32_rkx , &
    2085.0_rkx , 556.05_rkx , 2769.60_rkx , 361.16_rkx , 68.153_rkx , 288.47_rkx , &
    2086.0_rkx , 558.65_rkx , 2764.80_rkx , 361.32_rkx , 66.806_rkx , 285.66_rkx , &
    2087.0_rkx , 561.27_rkx , 2759.70_rkx , 361.48_rkx , 65.486_rkx , 282.87_rkx , &
    2088.0_rkx , 563.91_rkx , 2754.20_rkx , 361.64_rkx , 64.191_rkx , 280.11_rkx , &
    2089.0_rkx , 566.58_rkx , 2748.30_rkx , 361.79_rkx , 62.922_rkx , 277.38_rkx , &
    2090.0_rkx , 569.27_rkx , 2742.20_rkx , 361.93_rkx , 61.678_rkx , 274.67_rkx , &
    2091.0_rkx , 571.97_rkx , 2736.50_rkx , 362.07_rkx , 60.458_rkx , 271.99_rkx , &
    2092.0_rkx , 574.70_rkx , 2731.20_rkx , 362.21_rkx , 59.263_rkx , 269.34_rkx , &
    2093.0_rkx , 577.45_rkx , 2726.20_rkx , 362.35_rkx , 58.091_rkx , 266.71_rkx , &
    2094.0_rkx , 580.21_rkx , 2721.60_rkx , 362.48_rkx , 56.942_rkx , 264.11_rkx , &
    2095.0_rkx , 582.99_rkx , 2717.30_rkx , 362.61_rkx , 55.816_rkx , 261.53_rkx , &
    2096.0_rkx , 585.79_rkx , 2713.30_rkx , 362.73_rkx , 54.712_rkx , 258.98_rkx , &
    2097.0_rkx , 588.61_rkx , 2709.60_rkx , 362.86_rkx , 53.630_rkx , 256.46_rkx , &
    2098.0_rkx , 591.44_rkx , 2706.10_rkx , 362.97_rkx , 52.569_rkx , 253.95_rkx , &
    2099.0_rkx , 594.30_rkx , 2702.90_rkx , 363.09_rkx , 51.529_rkx , 251.48_rkx , &
    2100.0_rkx , 597.17_rkx , 2699.90_rkx , 363.20_rkx , 50.509_rkx , 249.02_rkx /), &
        (/6,50/))
      case( 'RCP3PD' , 'RCP2.6' , 'RCP26' )
        cgas(1:6,1850:2100) = reshape((/    &
    1850.0_rkx , 284.725_rkx , 790.979_rkx , 275.425_rkx , 0.00_rkx , 0.00_rkx , &
    1851.0_rkx , 284.875_rkx , 792.250_rkx , 275.500_rkx , 0.00_rkx , 0.00_rkx , &
    1852.0_rkx , 285.000_rkx , 793.750_rkx , 275.600_rkx , 0.00_rkx , 0.00_rkx , &
    1853.0_rkx , 285.125_rkx , 795.250_rkx , 275.700_rkx , 0.00_rkx , 0.00_rkx , &
    1854.0_rkx , 285.275_rkx , 796.750_rkx , 275.800_rkx , 0.00_rkx , 0.00_rkx , &
    1855.0_rkx , 285.425_rkx , 798.000_rkx , 275.900_rkx , 0.00_rkx , 0.00_rkx , &
    1856.0_rkx , 285.575_rkx , 799.250_rkx , 276.000_rkx , 0.00_rkx , 0.00_rkx , &
    1857.0_rkx , 285.725_rkx , 800.750_rkx , 276.100_rkx , 0.00_rkx , 0.00_rkx , &
    1858.0_rkx , 285.900_rkx , 802.250_rkx , 276.200_rkx , 0.00_rkx , 0.00_rkx , &
    1859.0_rkx , 286.075_rkx , 803.750_rkx , 276.300_rkx , 0.00_rkx , 0.00_rkx , &
    1860.0_rkx , 286.225_rkx , 805.250_rkx , 276.400_rkx , 0.00_rkx , 0.00_rkx , &
    1861.0_rkx , 286.375_rkx , 806.750_rkx , 276.500_rkx , 0.00_rkx , 0.00_rkx , &
    1862.0_rkx , 286.500_rkx , 808.250_rkx , 276.600_rkx , 0.00_rkx , 0.00_rkx , &
    1863.0_rkx , 286.625_rkx , 809.750_rkx , 276.700_rkx , 0.00_rkx , 0.00_rkx , &
    1864.0_rkx , 286.775_rkx , 811.250_rkx , 276.800_rkx , 0.00_rkx , 0.00_rkx , &
    1865.0_rkx , 286.900_rkx , 813.000_rkx , 276.900_rkx , 0.00_rkx , 0.00_rkx , &
    1866.0_rkx , 287.000_rkx , 814.750_rkx , 277.000_rkx , 0.00_rkx , 0.00_rkx , &
    1867.0_rkx , 287.100_rkx , 816.250_rkx , 277.100_rkx , 0.00_rkx , 0.00_rkx , &
    1868.0_rkx , 287.225_rkx , 817.750_rkx , 277.200_rkx , 0.00_rkx , 0.00_rkx , &
    1869.0_rkx , 287.375_rkx , 819.250_rkx , 277.300_rkx , 0.00_rkx , 0.00_rkx , &
    1870.0_rkx , 287.525_rkx , 821.000_rkx , 277.375_rkx , 0.00_rkx , 0.00_rkx , &
    1871.0_rkx , 287.700_rkx , 822.750_rkx , 277.425_rkx , 0.00_rkx , 0.00_rkx , &
    1872.0_rkx , 287.900_rkx , 824.250_rkx , 277.500_rkx , 0.00_rkx , 0.00_rkx , &
    1873.0_rkx , 288.125_rkx , 825.750_rkx , 277.600_rkx , 0.00_rkx , 0.00_rkx , &
    1874.0_rkx , 288.400_rkx , 827.250_rkx , 277.700_rkx , 0.00_rkx , 0.00_rkx , &
    1875.0_rkx , 288.700_rkx , 829.000_rkx , 277.775_rkx , 0.00_rkx , 0.00_rkx , &
    1876.0_rkx , 289.025_rkx , 830.750_rkx , 277.825_rkx , 0.00_rkx , 0.00_rkx , &
    1877.0_rkx , 289.400_rkx , 832.250_rkx , 277.900_rkx , 0.00_rkx , 0.00_rkx , &
    1878.0_rkx , 289.800_rkx , 833.750_rkx , 278.000_rkx , 0.00_rkx , 0.00_rkx , &
    1879.0_rkx , 290.225_rkx , 835.250_rkx , 278.100_rkx , 0.00_rkx , 0.00_rkx , &
    1880.0_rkx , 290.700_rkx , 837.000_rkx , 278.200_rkx , 0.00_rkx , 0.00_rkx , &
    1881.0_rkx , 291.200_rkx , 839.000_rkx , 278.300_rkx , 0.00_rkx , 0.00_rkx , &
    1882.0_rkx , 291.675_rkx , 841.000_rkx , 278.400_rkx , 0.00_rkx , 0.00_rkx , &
    1883.0_rkx , 292.125_rkx , 843.000_rkx , 278.500_rkx , 0.00_rkx , 0.00_rkx , &
    1884.0_rkx , 292.575_rkx , 844.750_rkx , 278.600_rkx , 0.00_rkx , 0.00_rkx , &
    1885.0_rkx , 292.975_rkx , 846.250_rkx , 278.700_rkx , 0.00_rkx , 0.00_rkx , &
    1886.0_rkx , 293.300_rkx , 848.000_rkx , 278.800_rkx , 0.00_rkx , 0.00_rkx , &
    1887.0_rkx , 293.575_rkx , 850.000_rkx , 278.875_rkx , 0.00_rkx , 0.00_rkx , &
    1888.0_rkx , 293.800_rkx , 852.000_rkx , 278.925_rkx , 0.00_rkx , 0.00_rkx , &
    1889.0_rkx , 294.000_rkx , 854.000_rkx , 279.000_rkx , 0.00_rkx , 0.00_rkx , &
    1890.0_rkx , 294.175_rkx , 856.000_rkx , 279.100_rkx , 0.00_rkx , 0.00_rkx , &
    1891.0_rkx , 294.325_rkx , 858.250_rkx , 279.200_rkx , 0.00_rkx , 0.00_rkx , &
    1892.0_rkx , 294.475_rkx , 860.750_rkx , 279.300_rkx , 0.00_rkx , 0.00_rkx , &
    1893.0_rkx , 294.600_rkx , 863.000_rkx , 279.375_rkx , 0.00_rkx , 0.00_rkx , &
    1894.0_rkx , 294.700_rkx , 865.000_rkx , 279.425_rkx , 0.00_rkx , 0.00_rkx , &
    1895.0_rkx , 294.800_rkx , 867.250_rkx , 279.475_rkx , 0.00_rkx , 0.00_rkx , &
    1896.0_rkx , 294.900_rkx , 869.750_rkx , 279.525_rkx , 0.00_rkx , 0.00_rkx , &
    1897.0_rkx , 295.025_rkx , 872.000_rkx , 279.600_rkx , 0.00_rkx , 0.00_rkx , &
    1898.0_rkx , 295.225_rkx , 874.250_rkx , 279.675_rkx , 0.00_rkx , 0.00_rkx , &
    1899.0_rkx , 295.500_rkx , 876.750_rkx , 279.725_rkx , 0.00_rkx , 0.00_rkx , &
    1900.0_rkx , 295.800_rkx , 879.500_rkx , 279.800_rkx , 0.00_rkx , 0.00_rkx , &
    1901.0_rkx , 296.125_rkx , 883.000_rkx , 279.900_rkx , 0.00_rkx , 0.00_rkx , &
    1902.0_rkx , 296.475_rkx , 887.000_rkx , 280.000_rkx , 0.00_rkx , 0.00_rkx , &
    1903.0_rkx , 296.825_rkx , 891.250_rkx , 280.100_rkx , 0.00_rkx , 0.00_rkx , &
    1904.0_rkx , 297.200_rkx , 895.750_rkx , 280.200_rkx , 0.00_rkx , 0.00_rkx , &
    1905.0_rkx , 297.625_rkx , 900.250_rkx , 280.300_rkx , 0.00_rkx , 0.00_rkx , &
    1906.0_rkx , 298.075_rkx , 904.750_rkx , 280.400_rkx , 0.00_rkx , 0.00_rkx , &
    1907.0_rkx , 298.500_rkx , 909.250_rkx , 280.525_rkx , 0.00_rkx , 0.00_rkx , &
    1908.0_rkx , 298.900_rkx , 914.000_rkx , 280.675_rkx , 0.00_rkx , 0.00_rkx , &
    1909.0_rkx , 299.300_rkx , 919.000_rkx , 280.825_rkx , 0.00_rkx , 0.00_rkx , &
    1910.0_rkx , 299.700_rkx , 923.750_rkx , 280.975_rkx , 0.00_rkx , 0.00_rkx , &
    1911.0_rkx , 300.075_rkx , 928.250_rkx , 281.125_rkx , 0.00_rkx , 0.00_rkx , &
    1912.0_rkx , 300.425_rkx , 933.000_rkx , 281.300_rkx , 0.00_rkx , 0.00_rkx , &
    1913.0_rkx , 300.775_rkx , 938.250_rkx , 281.475_rkx , 0.00_rkx , 0.00_rkx , &
    1914.0_rkx , 301.100_rkx , 943.750_rkx , 281.625_rkx , 0.00_rkx , 0.00_rkx , &
    1915.0_rkx , 301.400_rkx , 949.250_rkx , 281.800_rkx , 0.00_rkx , 0.00_rkx , &
    1916.0_rkx , 301.725_rkx , 954.750_rkx , 282.000_rkx , 0.00_rkx , 0.00_rkx , &
    1917.0_rkx , 302.075_rkx , 960.250_rkx , 282.225_rkx , 0.00_rkx , 0.00_rkx , &
    1918.0_rkx , 302.400_rkx , 966.000_rkx , 282.475_rkx , 0.00_rkx , 0.00_rkx , &
    1919.0_rkx , 302.700_rkx , 972.000_rkx , 282.700_rkx , 0.00_rkx , 0.00_rkx , &
    1920.0_rkx , 303.025_rkx , 977.750_rkx , 282.925_rkx , 0.00_rkx , 0.00_rkx , &
    1921.0_rkx , 303.400_rkx , 983.250_rkx , 283.175_rkx , 0.00_rkx , 0.00_rkx , &
    1922.0_rkx , 303.775_rkx , 989.000_rkx , 283.400_rkx , 0.00_rkx , 0.00_rkx , &
    1923.0_rkx , 304.125_rkx , 995.000_rkx , 283.600_rkx , 0.00_rkx , 0.00_rkx , &
    1924.0_rkx , 304.525_rkx , 1001.000_rkx , 283.800_rkx , 0.00_rkx , 0.00_rkx , &
    1925.0_rkx , 304.975_rkx , 1007.000_rkx , 284.000_rkx , 0.00_rkx , 0.00_rkx , &
    1926.0_rkx , 305.400_rkx , 1013.000_rkx , 284.225_rkx , 0.00_rkx , 0.00_rkx , &
    1927.0_rkx , 305.825_rkx , 1019.000_rkx , 284.450_rkx , 0.00_rkx , 0.00_rkx , &
    1928.0_rkx , 306.300_rkx , 1025.000_rkx , 284.625_rkx , 0.00_rkx , 0.00_rkx , &
    1929.0_rkx , 306.775_rkx , 1030.750_rkx , 284.800_rkx , 0.00_rkx , 0.00_rkx , &
    1930.0_rkx , 307.225_rkx , 1036.250_rkx , 284.975_rkx , 0.00_rkx , 0.00_rkx , &
    1931.0_rkx , 307.700_rkx , 1041.750_rkx , 285.125_rkx , 0.00_rkx , 0.00_rkx , &
    1932.0_rkx , 308.175_rkx , 1047.250_rkx , 285.300_rkx , 0.00_rkx , 0.00_rkx , &
    1933.0_rkx , 308.600_rkx , 1052.750_rkx , 285.500_rkx , 0.00_rkx , 0.01_rkx , &
    1934.0_rkx , 309.000_rkx , 1058.000_rkx , 285.700_rkx , 0.00_rkx , 0.01_rkx , &
    1935.0_rkx , 309.400_rkx , 1063.000_rkx , 285.900_rkx , 0.00_rkx , 0.02_rkx , &
    1936.0_rkx , 309.750_rkx , 1068.000_rkx , 286.100_rkx , 0.00_rkx , 0.03_rkx , &
    1937.0_rkx , 310.000_rkx , 1073.000_rkx , 286.275_rkx , 0.00_rkx , 0.05_rkx , &
    1938.0_rkx , 310.175_rkx , 1078.000_rkx , 286.425_rkx , 0.00_rkx , 0.08_rkx , &
    1939.0_rkx , 310.300_rkx , 1083.000_rkx , 286.575_rkx , 0.00_rkx , 0.13_rkx , &
    1940.0_rkx , 310.375_rkx , 1088.250_rkx , 286.725_rkx , 0.01_rkx , 0.20_rkx , &
    1941.0_rkx , 310.375_rkx , 1093.750_rkx , 286.900_rkx , 0.02_rkx , 0.29_rkx , &
    1942.0_rkx , 310.300_rkx , 1099.000_rkx , 287.100_rkx , 0.02_rkx , 0.42_rkx , &
    1943.0_rkx , 310.200_rkx , 1104.000_rkx , 287.300_rkx , 0.03_rkx , 0.58_rkx , &
    1944.0_rkx , 310.125_rkx , 1109.250_rkx , 287.525_rkx , 0.04_rkx , 0.77_rkx , &
    1945.0_rkx , 310.100_rkx , 1114.750_rkx , 287.775_rkx , 0.05_rkx , 1.02_rkx , &
    1946.0_rkx , 310.125_rkx , 1120.500_rkx , 288.025_rkx , 0.07_rkx , 1.35_rkx , &
    1947.0_rkx , 310.200_rkx , 1126.750_rkx , 288.275_rkx , 0.10_rkx , 1.87_rkx , &
    1948.0_rkx , 310.325_rkx , 1133.250_rkx , 288.525_rkx , 0.16_rkx , 2.70_rkx , &
    1949.0_rkx , 310.500_rkx , 1140.000_rkx , 288.775_rkx , 0.29_rkx , 3.80_rkx , &
    1950.0_rkx , 310.750_rkx , 1147.250_rkx , 289.000_rkx , 0.50_rkx , 5.00_rkx , &
    1951.0_rkx , 311.100_rkx , 1155.000_rkx , 289.200_rkx , 0.81_rkx , 6.31_rkx , &
    1952.0_rkx , 311.500_rkx , 1163.500_rkx , 289.425_rkx , 1.17_rkx , 7.81_rkx , &
    1953.0_rkx , 311.925_rkx , 1172.750_rkx , 289.675_rkx , 1.58_rkx , 9.46_rkx , &
    1954.0_rkx , 312.425_rkx , 1182.250_rkx , 289.900_rkx , 2.15_rkx , 11.24_rkx , &
    1955.0_rkx , 313.000_rkx , 1192.000_rkx , 290.125_rkx , 2.89_rkx , 13.24_rkx , &
    1956.0_rkx , 313.600_rkx , 1202.250_rkx , 290.375_rkx , 3.79_rkx , 15.48_rkx , &
    1957.0_rkx , 314.225_rkx , 1213.000_rkx , 290.600_rkx , 4.92_rkx , 18.05_rkx , &
    1958.0_rkx , 314.848_rkx , 1224.000_rkx , 290.825_rkx , 6.24_rkx , 21.01_rkx , &
    1959.0_rkx , 315.500_rkx , 1235.250_rkx , 291.100_rkx , 7.56_rkx , 24.21_rkx , &
    1960.0_rkx , 316.272_rkx , 1247.000_rkx , 291.400_rkx , 8.82_rkx , 27.68_rkx , &
    1961.0_rkx , 317.075_rkx , 1259.250_rkx , 291.675_rkx , 10.30_rkx , 31.68_rkx , &
    1962.0_rkx , 317.795_rkx , 1272.000_rkx , 291.925_rkx , 12.23_rkx , 36.31_rkx , &
    1963.0_rkx , 318.397_rkx , 1285.000_rkx , 292.225_rkx , 14.71_rkx , 41.59_rkx , &
    1964.0_rkx , 318.925_rkx , 1298.250_rkx , 292.600_rkx , 17.80_rkx , 47.73_rkx , &
    1965.0_rkx , 319.647_rkx , 1312.000_rkx , 293.000_rkx , 21.50_rkx , 54.93_rkx , &
    1966.0_rkx , 320.647_rkx , 1326.000_rkx , 293.400_rkx , 25.77_rkx , 63.16_rkx , &
    1967.0_rkx , 321.605_rkx , 1340.250_rkx , 293.800_rkx , 30.57_rkx , 72.39_rkx , &
    1968.0_rkx , 322.635_rkx , 1355.000_rkx , 294.225_rkx , 35.95_rkx , 82.77_rkx , &
    1969.0_rkx , 323.902_rkx , 1370.250_rkx , 294.700_rkx , 42.05_rkx , 94.44_rkx , &
    1970.0_rkx , 324.985_rkx , 1385.750_rkx , 295.200_rkx , 49.05_rkx , 107.45_rkx , &
    1971.0_rkx , 325.855_rkx , 1401.250_rkx , 295.700_rkx , 57.06_rkx , 121.80_rkx , &
    1972.0_rkx , 327.140_rkx , 1417.000_rkx , 296.225_rkx , 65.97_rkx , 137.30_rkx , &
    1973.0_rkx , 328.678_rkx , 1433.000_rkx , 296.800_rkx , 75.83_rkx , 154.05_rkx , &
    1974.0_rkx , 329.743_rkx , 1449.000_rkx , 297.400_rkx , 87.04_rkx , 172.45_rkx , &
    1975.0_rkx , 330.585_rkx , 1465.000_rkx , 298.000_rkx , 99.56_rkx , 192.60_rkx , &
    1976.0_rkx , 331.748_rkx , 1481.000_rkx , 298.650_rkx , 112.30_rkx , 213.15_rkx , &
    1977.0_rkx , 333.272_rkx , 1497.250_rkx , 299.360_rkx , 124.70_rkx , 232.90_rkx , &
    1978.0_rkx , 334.848_rkx , 1513.750_rkx , 300.082_rkx , 136.70_rkx , 251.90_rkx , &
    1979.0_rkx , 336.525_rkx , 1530.250_rkx , 300.740_rkx , 148.80_rkx , 270.50_rkx , &
    1980.0_rkx , 338.360_rkx , 1547.750_rkx , 301.382_rkx , 159.40_rkx , 287.35_rkx , &
    1981.0_rkx , 339.728_rkx , 1566.750_rkx , 302.220_rkx , 168.55_rkx , 304.25_rkx , &
    1982.0_rkx , 340.793_rkx , 1586.250_rkx , 303.098_rkx , 177.85_rkx , 321.70_rkx , &
    1983.0_rkx , 342.197_rkx , 1605.845_rkx , 303.805_rkx , 185.80_rkx , 338.15_rkx , &
    1984.0_rkx , 343.783_rkx , 1623.705_rkx , 304.397_rkx , 193.75_rkx , 355.00_rkx , &
    1985.0_rkx , 345.283_rkx , 1637.993_rkx , 305.087_rkx , 202.85_rkx , 372.50_rkx , &
    1986.0_rkx , 346.798_rkx , 1650.435_rkx , 305.772_rkx , 212.10_rkx , 389.80_rkx , &
    1987.0_rkx , 348.645_rkx , 1662.200_rkx , 306.262_rkx , 222.80_rkx , 408.65_rkx , &
    1988.0_rkx , 350.738_rkx , 1673.070_rkx , 307.195_rkx , 234.90_rkx , 428.45_rkx , &
    1989.0_rkx , 352.488_rkx , 1683.470_rkx , 308.507_rkx , 244.90_rkx , 448.25_rkx , &
    1990.0_rkx , 353.855_rkx , 1693.630_rkx , 309.485_rkx , 252.75_rkx , 466.15_rkx , &
    1991.0_rkx , 355.017_rkx , 1703.818_rkx , 310.113_rkx , 260.15_rkx , 481.30_rkx , &
    1992.0_rkx , 355.885_rkx , 1711.800_rkx , 310.603_rkx , 264.95_rkx , 494.40_rkx , &
    1993.0_rkx , 356.777_rkx , 1716.290_rkx , 311.077_rkx , 267.75_rkx , 504.80_rkx , &
    1994.0_rkx , 358.127_rkx , 1721.013_rkx , 311.603_rkx , 269.50_rkx , 513.10_rkx , &
    1995.0_rkx , 359.837_rkx , 1726.350_rkx , 312.295_rkx , 269.50_rkx , 519.55_rkx , &
    1996.0_rkx , 361.462_rkx , 1730.010_rkx , 313.048_rkx , 268.95_rkx , 525.05_rkx , &
    1997.0_rkx , 363.155_rkx , 1734.818_rkx , 313.760_rkx , 268.20_rkx , 529.15_rkx , &
    1998.0_rkx , 365.322_rkx , 1742.710_rkx , 314.533_rkx , 266.95_rkx , 532.40_rkx , &
    1999.0_rkx , 367.348_rkx , 1749.243_rkx , 315.257_rkx , 265.20_rkx , 534.95_rkx , &
    2000.0_rkx , 368.865_rkx , 1751.023_rkx , 315.850_rkx , 263.45_rkx , 537.05_rkx , &
    2001.0_rkx , 370.467_rkx , 1750.707_rkx , 316.488_rkx , 261.90_rkx , 538.85_rkx , &
    2002.0_rkx , 372.522_rkx , 1752.188_rkx , 317.245_rkx , 260.10_rkx , 539.95_rkx , &
    2003.0_rkx , 374.760_rkx , 1754.760_rkx , 318.017_rkx , 257.95_rkx , 540.05_rkx , &
    2004.0_rkx , 376.812_rkx , 1755.065_rkx , 318.710_rkx , 256.05_rkx , 539.90_rkx , &
    2005.0_rkx , 378.812_rkx , 1753.735_rkx , 319.440_rkx , 254.10_rkx , 539.40_rkx , &
    2006.0_rkx , 380.827_rkx , 1754.514_rkx , 320.200_rkx , 251.30_rkx , 537.50_rkx , &
    2007.0_rkx , 382.777_rkx , 1757.684_rkx , 320.889_rkx , 247.95_rkx , 534.70_rkx , &
    2008.0_rkx , 384.800_rkx , 1761.980_rkx , 321.578_rkx , 244.60_rkx , 531.80_rkx , &
    2009.0_rkx , 387.001_rkx , 1767.259_rkx , 322.267_rkx , 241.03_rkx , 528.54_rkx , &
    2010.0_rkx , 389.285_rkx , 1773.128_rkx , 322.957_rkx , 237.28_rkx , 524.94_rkx , &
    2011.0_rkx , 391.563_rkx , 1779.540_rkx , 323.646_rkx , 233.54_rkx , 521.21_rkx , &
    2012.0_rkx , 393.843_rkx , 1784.552_rkx , 324.327_rkx , 229.79_rkx , 517.32_rkx , &
    2013.0_rkx , 396.117_rkx , 1786.369_rkx , 324.993_rkx , 226.06_rkx , 513.32_rkx , &
    2014.0_rkx , 398.396_rkx , 1785.286_rkx , 325.642_rkx , 222.34_rkx , 509.20_rkx , &
    2015.0_rkx , 400.681_rkx , 1781.582_rkx , 326.276_rkx , 218.64_rkx , 505.00_rkx , &
    2016.0_rkx , 402.968_rkx , 1775.478_rkx , 326.894_rkx , 214.97_rkx , 500.73_rkx , &
    2017.0_rkx , 405.252_rkx , 1767.151_rkx , 327.496_rkx , 211.32_rkx , 496.39_rkx , &
    2018.0_rkx , 407.529_rkx , 1756.774_rkx , 328.082_rkx , 207.70_rkx , 491.98_rkx , &
    2019.0_rkx , 409.800_rkx , 1744.511_rkx , 328.653_rkx , 204.10_rkx , 487.53_rkx , &
    2020.0_rkx , 412.068_rkx , 1730.518_rkx , 329.208_rkx , 200.53_rkx , 483.03_rkx , &
    2021.0_rkx , 414.326_rkx , 1714.871_rkx , 329.749_rkx , 196.98_rkx , 478.49_rkx , &
    2022.0_rkx , 416.517_rkx , 1698.724_rkx , 330.278_rkx , 193.45_rkx , 473.91_rkx , &
    2023.0_rkx , 418.603_rkx , 1683.306_rkx , 330.801_rkx , 189.94_rkx , 469.29_rkx , &
    2024.0_rkx , 420.601_rkx , 1668.825_rkx , 331.318_rkx , 186.46_rkx , 464.65_rkx , &
    2025.0_rkx , 422.516_rkx , 1655.367_rkx , 331.829_rkx , 183.02_rkx , 460.01_rkx , &
    2026.0_rkx , 424.349_rkx , 1642.855_rkx , 332.335_rkx , 179.61_rkx , 455.36_rkx , &
    2027.0_rkx , 426.097_rkx , 1631.187_rkx , 332.834_rkx , 176.25_rkx , 450.72_rkx , &
    2028.0_rkx , 427.752_rkx , 1620.251_rkx , 333.327_rkx , 172.94_rkx , 446.08_rkx , &
    2029.0_rkx , 429.314_rkx , 1609.952_rkx , 333.815_rkx , 169.66_rkx , 441.46_rkx , &
    2030.0_rkx , 430.783_rkx , 1600.215_rkx , 334.297_rkx , 166.42_rkx , 436.84_rkx , &
    2031.0_rkx , 432.163_rkx , 1590.976_rkx , 334.774_rkx , 163.23_rkx , 432.24_rkx , &
    2032.0_rkx , 433.436_rkx , 1582.200_rkx , 335.244_rkx , 160.07_rkx , 427.65_rkx , &
    2033.0_rkx , 434.593_rkx , 1573.879_rkx , 335.707_rkx , 156.94_rkx , 423.06_rkx , &
    2034.0_rkx , 435.653_rkx , 1566.005_rkx , 336.163_rkx , 153.84_rkx , 418.49_rkx , &
    2035.0_rkx , 436.628_rkx , 1558.564_rkx , 336.613_rkx , 150.79_rkx , 413.94_rkx , &
    2036.0_rkx , 437.522_rkx , 1551.541_rkx , 337.055_rkx , 147.78_rkx , 409.42_rkx , &
    2037.0_rkx , 438.334_rkx , 1544.927_rkx , 337.491_rkx , 144.83_rkx , 404.93_rkx , &
    2038.0_rkx , 439.060_rkx , 1538.684_rkx , 337.920_rkx , 141.92_rkx , 400.47_rkx , &
    2039.0_rkx , 439.691_rkx , 1532.755_rkx , 338.342_rkx , 139.05_rkx , 396.05_rkx , &
    2040.0_rkx , 440.222_rkx , 1527.098_rkx , 338.758_rkx , 136.24_rkx , 391.66_rkx , &
    2041.0_rkx , 440.657_rkx , 1521.680_rkx , 339.168_rkx , 133.46_rkx , 387.31_rkx , &
    2042.0_rkx , 441.025_rkx , 1516.026_rkx , 339.562_rkx , 130.74_rkx , 382.99_rkx , &
    2043.0_rkx , 441.347_rkx , 1509.650_rkx , 339.934_rkx , 128.05_rkx , 378.70_rkx , &
    2044.0_rkx , 441.621_rkx , 1502.599_rkx , 340.282_rkx , 125.40_rkx , 374.44_rkx , &
    2045.0_rkx , 441.864_rkx , 1494.992_rkx , 340.607_rkx , 122.80_rkx , 370.21_rkx , &
    2046.0_rkx , 442.085_rkx , 1486.926_rkx , 340.910_rkx , 120.23_rkx , 366.01_rkx , &
    2047.0_rkx , 442.283_rkx , 1478.487_rkx , 341.190_rkx , 117.71_rkx , 361.86_rkx , &
    2048.0_rkx , 442.458_rkx , 1469.753_rkx , 341.447_rkx , 115.24_rkx , 357.75_rkx , &
    2049.0_rkx , 442.601_rkx , 1460.765_rkx , 341.683_rkx , 112.81_rkx , 353.68_rkx , &
    2050.0_rkx , 442.700_rkx , 1451.540_rkx , 341.896_rkx , 110.44_rkx , 349.66_rkx , &
    2051.0_rkx , 442.752_rkx , 1442.096_rkx , 342.087_rkx , 108.10_rkx , 345.68_rkx , &
    2052.0_rkx , 442.761_rkx , 1432.710_rkx , 342.260_rkx , 105.82_rkx , 341.74_rkx , &
    2053.0_rkx , 442.734_rkx , 1423.622_rkx , 342.421_rkx , 103.57_rkx , 337.85_rkx , &
    2054.0_rkx , 442.663_rkx , 1414.743_rkx , 342.569_rkx , 101.37_rkx , 333.99_rkx , &
    2055.0_rkx , 442.548_rkx , 1406.012_rkx , 342.704_rkx , 99.20_rkx , 330.16_rkx , &
    2056.0_rkx , 442.406_rkx , 1397.454_rkx , 342.826_rkx , 97.06_rkx , 326.37_rkx , &
    2057.0_rkx , 442.248_rkx , 1389.078_rkx , 342.936_rkx , 94.97_rkx , 322.61_rkx , &
    2058.0_rkx , 442.075_rkx , 1380.892_rkx , 343.033_rkx , 92.92_rkx , 318.90_rkx , &
    2059.0_rkx , 441.886_rkx , 1372.907_rkx , 343.118_rkx , 90.91_rkx , 315.24_rkx , &
    2060.0_rkx , 441.673_rkx , 1365.106_rkx , 343.192_rkx , 88.95_rkx , 311.62_rkx , &
    2061.0_rkx , 441.424_rkx , 1357.451_rkx , 343.253_rkx , 87.02_rkx , 308.04_rkx , &
    2062.0_rkx , 441.135_rkx , 1350.226_rkx , 343.308_rkx , 85.14_rkx , 304.51_rkx , &
    2063.0_rkx , 440.803_rkx , 1343.686_rkx , 343.363_rkx , 83.29_rkx , 301.01_rkx , &
    2064.0_rkx , 440.430_rkx , 1337.750_rkx , 343.417_rkx , 81.48_rkx , 297.56_rkx , &
    2065.0_rkx , 440.010_rkx , 1332.296_rkx , 343.472_rkx , 79.71_rkx , 294.14_rkx , &
    2066.0_rkx , 439.545_rkx , 1327.233_rkx , 343.526_rkx , 77.96_rkx , 290.75_rkx , &
    2067.0_rkx , 439.052_rkx , 1322.552_rkx , 343.581_rkx , 76.25_rkx , 287.40_rkx , &
    2068.0_rkx , 438.543_rkx , 1318.238_rkx , 343.635_rkx , 74.57_rkx , 284.08_rkx , &
    2069.0_rkx , 438.019_rkx , 1314.274_rkx , 343.689_rkx , 72.93_rkx , 280.81_rkx , &
    2070.0_rkx , 437.481_rkx , 1310.651_rkx , 343.744_rkx , 71.32_rkx , 277.57_rkx , &
    2071.0_rkx , 436.919_rkx , 1307.333_rkx , 343.798_rkx , 69.75_rkx , 274.37_rkx , &
    2072.0_rkx , 436.343_rkx , 1304.279_rkx , 343.851_rkx , 68.21_rkx , 271.22_rkx , &
    2073.0_rkx , 435.764_rkx , 1301.455_rkx , 343.901_rkx , 66.70_rkx , 268.10_rkx , &
    2074.0_rkx , 435.182_rkx , 1298.831_rkx , 343.947_rkx , 65.23_rkx , 265.02_rkx , &
    2075.0_rkx , 434.595_rkx , 1296.381_rkx , 343.991_rkx , 63.79_rkx , 261.98_rkx , &
    2076.0_rkx , 433.995_rkx , 1294.031_rkx , 344.031_rkx , 62.37_rkx , 258.97_rkx , &
    2077.0_rkx , 433.385_rkx , 1291.735_rkx , 344.068_rkx , 60.98_rkx , 255.98_rkx , &
    2078.0_rkx , 432.780_rkx , 1289.522_rkx , 344.102_rkx , 59.62_rkx , 253.03_rkx , &
    2079.0_rkx , 432.190_rkx , 1287.407_rkx , 344.133_rkx , 58.29_rkx , 250.10_rkx , &
    2080.0_rkx , 431.617_rkx , 1285.405_rkx , 344.161_rkx , 56.98_rkx , 247.22_rkx , &
    2081.0_rkx , 431.058_rkx , 1283.531_rkx , 344.186_rkx , 55.70_rkx , 244.37_rkx , &
    2082.0_rkx , 430.510_rkx , 1281.757_rkx , 344.208_rkx , 54.45_rkx , 241.55_rkx , &
    2083.0_rkx , 429.964_rkx , 1280.041_rkx , 344.227_rkx , 53.23_rkx , 238.77_rkx , &
    2084.0_rkx , 429.414_rkx , 1278.363_rkx , 344.242_rkx , 52.04_rkx , 236.03_rkx , &
    2085.0_rkx , 428.859_rkx , 1276.718_rkx , 344.253_rkx , 50.87_rkx , 233.32_rkx , &
    2086.0_rkx , 428.299_rkx , 1275.098_rkx , 344.261_rkx , 49.73_rkx , 230.64_rkx , &
    2087.0_rkx , 427.727_rkx , 1273.447_rkx , 344.266_rkx , 48.61_rkx , 227.99_rkx , &
    2088.0_rkx , 427.143_rkx , 1271.736_rkx , 344.268_rkx , 47.51_rkx , 225.36_rkx , &
    2089.0_rkx , 426.566_rkx , 1270.005_rkx , 344.266_rkx , 46.44_rkx , 222.76_rkx , &
    2090.0_rkx , 426.005_rkx , 1268.282_rkx , 344.261_rkx , 45.38_rkx , 220.19_rkx , &
    2091.0_rkx , 425.461_rkx , 1266.590_rkx , 344.253_rkx , 44.35_rkx , 217.65_rkx , &
    2092.0_rkx , 424.937_rkx , 1264.974_rkx , 344.241_rkx , 43.34_rkx , 215.15_rkx , &
    2093.0_rkx , 424.431_rkx , 1263.447_rkx , 344.225_rkx , 42.36_rkx , 212.67_rkx , &
    2094.0_rkx , 423.931_rkx , 1261.986_rkx , 344.206_rkx , 41.40_rkx , 210.23_rkx , &
    2095.0_rkx , 423.431_rkx , 1260.573_rkx , 344.184_rkx , 40.46_rkx , 207.82_rkx , &
    2096.0_rkx , 422.929_rkx , 1259.199_rkx , 344.157_rkx , 39.54_rkx , 205.43_rkx , &
    2097.0_rkx , 422.428_rkx , 1257.857_rkx , 344.127_rkx , 38.64_rkx , 203.08_rkx , &
    2098.0_rkx , 421.918_rkx , 1256.489_rkx , 344.094_rkx , 37.76_rkx , 200.75_rkx , &
    2099.0_rkx , 421.401_rkx , 1255.067_rkx , 344.057_rkx , 36.90_rkx , 198.44_rkx , &
    2100.0_rkx , 420.895_rkx , 1253.628_rkx , 344.016_rkx , 36.05_rkx , 196.15_rkx /) , &
      (/6,251/))
      case( 'RCP4.5' , 'RCP45' )
        cgas(1:6,1850:2100) = reshape((/    &
    1850.0_rkx , 284.725_rkx , 790.979_rkx , 275.425_rkx , 0.00_rkx , 0.00_rkx , &
    1851.0_rkx , 284.875_rkx , 792.250_rkx , 275.500_rkx , 0.00_rkx , 0.00_rkx , &
    1852.0_rkx , 285.000_rkx , 793.750_rkx , 275.600_rkx , 0.00_rkx , 0.00_rkx , &
    1853.0_rkx , 285.125_rkx , 795.250_rkx , 275.700_rkx , 0.00_rkx , 0.00_rkx , &
    1854.0_rkx , 285.275_rkx , 796.750_rkx , 275.800_rkx , 0.00_rkx , 0.00_rkx , &
    1855.0_rkx , 285.425_rkx , 798.000_rkx , 275.900_rkx , 0.00_rkx , 0.00_rkx , &
    1856.0_rkx , 285.575_rkx , 799.250_rkx , 276.000_rkx , 0.00_rkx , 0.00_rkx , &
    1857.0_rkx , 285.725_rkx , 800.750_rkx , 276.100_rkx , 0.00_rkx , 0.00_rkx , &
    1858.0_rkx , 285.900_rkx , 802.250_rkx , 276.200_rkx , 0.00_rkx , 0.00_rkx , &
    1859.0_rkx , 286.075_rkx , 803.750_rkx , 276.300_rkx , 0.00_rkx , 0.00_rkx , &
    1860.0_rkx , 286.225_rkx , 805.250_rkx , 276.400_rkx , 0.00_rkx , 0.00_rkx , &
    1861.0_rkx , 286.375_rkx , 806.750_rkx , 276.500_rkx , 0.00_rkx , 0.00_rkx , &
    1862.0_rkx , 286.500_rkx , 808.250_rkx , 276.600_rkx , 0.00_rkx , 0.00_rkx , &
    1863.0_rkx , 286.625_rkx , 809.750_rkx , 276.700_rkx , 0.00_rkx , 0.00_rkx , &
    1864.0_rkx , 286.775_rkx , 811.250_rkx , 276.800_rkx , 0.00_rkx , 0.00_rkx , &
    1865.0_rkx , 286.900_rkx , 813.000_rkx , 276.900_rkx , 0.00_rkx , 0.00_rkx , &
    1866.0_rkx , 287.000_rkx , 814.750_rkx , 277.000_rkx , 0.00_rkx , 0.00_rkx , &
    1867.0_rkx , 287.100_rkx , 816.250_rkx , 277.100_rkx , 0.00_rkx , 0.00_rkx , &
    1868.0_rkx , 287.225_rkx , 817.750_rkx , 277.200_rkx , 0.00_rkx , 0.00_rkx , &
    1869.0_rkx , 287.375_rkx , 819.250_rkx , 277.300_rkx , 0.00_rkx , 0.00_rkx , &
    1870.0_rkx , 287.525_rkx , 821.000_rkx , 277.375_rkx , 0.00_rkx , 0.00_rkx , &
    1871.0_rkx , 287.700_rkx , 822.750_rkx , 277.425_rkx , 0.00_rkx , 0.00_rkx , &
    1872.0_rkx , 287.900_rkx , 824.250_rkx , 277.500_rkx , 0.00_rkx , 0.00_rkx , &
    1873.0_rkx , 288.125_rkx , 825.750_rkx , 277.600_rkx , 0.00_rkx , 0.00_rkx , &
    1874.0_rkx , 288.400_rkx , 827.250_rkx , 277.700_rkx , 0.00_rkx , 0.00_rkx , &
    1875.0_rkx , 288.700_rkx , 829.000_rkx , 277.775_rkx , 0.00_rkx , 0.00_rkx , &
    1876.0_rkx , 289.025_rkx , 830.750_rkx , 277.825_rkx , 0.00_rkx , 0.00_rkx , &
    1877.0_rkx , 289.400_rkx , 832.250_rkx , 277.900_rkx , 0.00_rkx , 0.00_rkx , &
    1878.0_rkx , 289.800_rkx , 833.750_rkx , 278.000_rkx , 0.00_rkx , 0.00_rkx , &
    1879.0_rkx , 290.225_rkx , 835.250_rkx , 278.100_rkx , 0.00_rkx , 0.00_rkx , &
    1880.0_rkx , 290.700_rkx , 837.000_rkx , 278.200_rkx , 0.00_rkx , 0.00_rkx , &
    1881.0_rkx , 291.200_rkx , 839.000_rkx , 278.300_rkx , 0.00_rkx , 0.00_rkx , &
    1882.0_rkx , 291.675_rkx , 841.000_rkx , 278.400_rkx , 0.00_rkx , 0.00_rkx , &
    1883.0_rkx , 292.125_rkx , 843.000_rkx , 278.500_rkx , 0.00_rkx , 0.00_rkx , &
    1884.0_rkx , 292.575_rkx , 844.750_rkx , 278.600_rkx , 0.00_rkx , 0.00_rkx , &
    1885.0_rkx , 292.975_rkx , 846.250_rkx , 278.700_rkx , 0.00_rkx , 0.00_rkx , &
    1886.0_rkx , 293.300_rkx , 848.000_rkx , 278.800_rkx , 0.00_rkx , 0.00_rkx , &
    1887.0_rkx , 293.575_rkx , 850.000_rkx , 278.875_rkx , 0.00_rkx , 0.00_rkx , &
    1888.0_rkx , 293.800_rkx , 852.000_rkx , 278.925_rkx , 0.00_rkx , 0.00_rkx , &
    1889.0_rkx , 294.000_rkx , 854.000_rkx , 279.000_rkx , 0.00_rkx , 0.00_rkx , &
    1890.0_rkx , 294.175_rkx , 856.000_rkx , 279.100_rkx , 0.00_rkx , 0.00_rkx , &
    1891.0_rkx , 294.325_rkx , 858.250_rkx , 279.200_rkx , 0.00_rkx , 0.00_rkx , &
    1892.0_rkx , 294.475_rkx , 860.750_rkx , 279.300_rkx , 0.00_rkx , 0.00_rkx , &
    1893.0_rkx , 294.600_rkx , 863.000_rkx , 279.375_rkx , 0.00_rkx , 0.00_rkx , &
    1894.0_rkx , 294.700_rkx , 865.000_rkx , 279.425_rkx , 0.00_rkx , 0.00_rkx , &
    1895.0_rkx , 294.800_rkx , 867.250_rkx , 279.475_rkx , 0.00_rkx , 0.00_rkx , &
    1896.0_rkx , 294.900_rkx , 869.750_rkx , 279.525_rkx , 0.00_rkx , 0.00_rkx , &
    1897.0_rkx , 295.025_rkx , 872.000_rkx , 279.600_rkx , 0.00_rkx , 0.00_rkx , &
    1898.0_rkx , 295.225_rkx , 874.250_rkx , 279.675_rkx , 0.00_rkx , 0.00_rkx , &
    1899.0_rkx , 295.500_rkx , 876.750_rkx , 279.725_rkx , 0.00_rkx , 0.00_rkx , &
    1900.0_rkx , 295.800_rkx , 879.500_rkx , 279.800_rkx , 0.00_rkx , 0.00_rkx , &
    1901.0_rkx , 296.125_rkx , 883.000_rkx , 279.900_rkx , 0.00_rkx , 0.00_rkx , &
    1902.0_rkx , 296.475_rkx , 887.000_rkx , 280.000_rkx , 0.00_rkx , 0.00_rkx , &
    1903.0_rkx , 296.825_rkx , 891.250_rkx , 280.100_rkx , 0.00_rkx , 0.00_rkx , &
    1904.0_rkx , 297.200_rkx , 895.750_rkx , 280.200_rkx , 0.00_rkx , 0.00_rkx , &
    1905.0_rkx , 297.625_rkx , 900.250_rkx , 280.300_rkx , 0.00_rkx , 0.00_rkx , &
    1906.0_rkx , 298.075_rkx , 904.750_rkx , 280.400_rkx , 0.00_rkx , 0.00_rkx , &
    1907.0_rkx , 298.500_rkx , 909.250_rkx , 280.525_rkx , 0.00_rkx , 0.00_rkx , &
    1908.0_rkx , 298.900_rkx , 914.000_rkx , 280.675_rkx , 0.00_rkx , 0.00_rkx , &
    1909.0_rkx , 299.300_rkx , 919.000_rkx , 280.825_rkx , 0.00_rkx , 0.00_rkx , &
    1910.0_rkx , 299.700_rkx , 923.750_rkx , 280.975_rkx , 0.00_rkx , 0.00_rkx , &
    1911.0_rkx , 300.075_rkx , 928.250_rkx , 281.125_rkx , 0.00_rkx , 0.00_rkx , &
    1912.0_rkx , 300.425_rkx , 933.000_rkx , 281.300_rkx , 0.00_rkx , 0.00_rkx , &
    1913.0_rkx , 300.775_rkx , 938.250_rkx , 281.475_rkx , 0.00_rkx , 0.00_rkx , &
    1914.0_rkx , 301.100_rkx , 943.750_rkx , 281.625_rkx , 0.00_rkx , 0.00_rkx , &
    1915.0_rkx , 301.400_rkx , 949.250_rkx , 281.800_rkx , 0.00_rkx , 0.00_rkx , &
    1916.0_rkx , 301.725_rkx , 954.750_rkx , 282.000_rkx , 0.00_rkx , 0.00_rkx , &
    1917.0_rkx , 302.075_rkx , 960.250_rkx , 282.225_rkx , 0.00_rkx , 0.00_rkx , &
    1918.0_rkx , 302.400_rkx , 966.000_rkx , 282.475_rkx , 0.00_rkx , 0.00_rkx , &
    1919.0_rkx , 302.700_rkx , 972.000_rkx , 282.700_rkx , 0.00_rkx , 0.00_rkx , &
    1920.0_rkx , 303.025_rkx , 977.750_rkx , 282.925_rkx , 0.00_rkx , 0.00_rkx , &
    1921.0_rkx , 303.400_rkx , 983.250_rkx , 283.175_rkx , 0.00_rkx , 0.00_rkx , &
    1922.0_rkx , 303.775_rkx , 989.000_rkx , 283.400_rkx , 0.00_rkx , 0.00_rkx , &
    1923.0_rkx , 304.125_rkx , 995.000_rkx , 283.600_rkx , 0.00_rkx , 0.00_rkx , &
    1924.0_rkx , 304.525_rkx , 1001.000_rkx , 283.800_rkx , 0.00_rkx , 0.00_rkx , &
    1925.0_rkx , 304.975_rkx , 1007.000_rkx , 284.000_rkx , 0.00_rkx , 0.00_rkx , &
    1926.0_rkx , 305.400_rkx , 1013.000_rkx , 284.225_rkx , 0.00_rkx , 0.00_rkx , &
    1927.0_rkx , 305.825_rkx , 1019.000_rkx , 284.450_rkx , 0.00_rkx , 0.00_rkx , &
    1928.0_rkx , 306.300_rkx , 1025.000_rkx , 284.625_rkx , 0.00_rkx , 0.00_rkx , &
    1929.0_rkx , 306.775_rkx , 1030.750_rkx , 284.800_rkx , 0.00_rkx , 0.00_rkx , &
    1930.0_rkx , 307.225_rkx , 1036.250_rkx , 284.975_rkx , 0.00_rkx , 0.00_rkx , &
    1931.0_rkx , 307.700_rkx , 1041.750_rkx , 285.125_rkx , 0.00_rkx , 0.00_rkx , &
    1932.0_rkx , 308.175_rkx , 1047.250_rkx , 285.300_rkx , 0.00_rkx , 0.00_rkx , &
    1933.0_rkx , 308.600_rkx , 1052.750_rkx , 285.500_rkx , 0.00_rkx , 0.01_rkx , &
    1934.0_rkx , 309.000_rkx , 1058.000_rkx , 285.700_rkx , 0.00_rkx , 0.01_rkx , &
    1935.0_rkx , 309.400_rkx , 1063.000_rkx , 285.900_rkx , 0.00_rkx , 0.02_rkx , &
    1936.0_rkx , 309.750_rkx , 1068.000_rkx , 286.100_rkx , 0.00_rkx , 0.03_rkx , &
    1937.0_rkx , 310.000_rkx , 1073.000_rkx , 286.275_rkx , 0.00_rkx , 0.05_rkx , &
    1938.0_rkx , 310.175_rkx , 1078.000_rkx , 286.425_rkx , 0.00_rkx , 0.08_rkx , &
    1939.0_rkx , 310.300_rkx , 1083.000_rkx , 286.575_rkx , 0.00_rkx , 0.13_rkx , &
    1940.0_rkx , 310.375_rkx , 1088.250_rkx , 286.725_rkx , 0.01_rkx , 0.20_rkx , &
    1941.0_rkx , 310.375_rkx , 1093.750_rkx , 286.900_rkx , 0.02_rkx , 0.29_rkx , &
    1942.0_rkx , 310.300_rkx , 1099.000_rkx , 287.100_rkx , 0.02_rkx , 0.42_rkx , &
    1943.0_rkx , 310.200_rkx , 1104.000_rkx , 287.300_rkx , 0.03_rkx , 0.58_rkx , &
    1944.0_rkx , 310.125_rkx , 1109.250_rkx , 287.525_rkx , 0.04_rkx , 0.77_rkx , &
    1945.0_rkx , 310.100_rkx , 1114.750_rkx , 287.775_rkx , 0.05_rkx , 1.02_rkx , &
    1946.0_rkx , 310.125_rkx , 1120.500_rkx , 288.025_rkx , 0.07_rkx , 1.35_rkx , &
    1947.0_rkx , 310.200_rkx , 1126.750_rkx , 288.275_rkx , 0.10_rkx , 1.87_rkx , &
    1948.0_rkx , 310.325_rkx , 1133.250_rkx , 288.525_rkx , 0.16_rkx , 2.70_rkx , &
    1949.0_rkx , 310.500_rkx , 1140.000_rkx , 288.775_rkx , 0.29_rkx , 3.80_rkx , &
    1950.0_rkx , 310.750_rkx , 1147.250_rkx , 289.000_rkx , 0.50_rkx , 5.00_rkx , &
    1951.0_rkx , 311.100_rkx , 1155.000_rkx , 289.200_rkx , 0.81_rkx , 6.31_rkx , &
    1952.0_rkx , 311.500_rkx , 1163.500_rkx , 289.425_rkx , 1.17_rkx , 7.81_rkx , &
    1953.0_rkx , 311.925_rkx , 1172.750_rkx , 289.675_rkx , 1.58_rkx , 9.46_rkx , &
    1954.0_rkx , 312.425_rkx , 1182.250_rkx , 289.900_rkx , 2.15_rkx , 11.24_rkx , &
    1955.0_rkx , 313.000_rkx , 1192.000_rkx , 290.125_rkx , 2.89_rkx , 13.24_rkx , &
    1956.0_rkx , 313.600_rkx , 1202.250_rkx , 290.375_rkx , 3.79_rkx , 15.48_rkx , &
    1957.0_rkx , 314.225_rkx , 1213.000_rkx , 290.600_rkx , 4.92_rkx , 18.05_rkx , &
    1958.0_rkx , 314.848_rkx , 1224.000_rkx , 290.825_rkx , 6.24_rkx , 21.01_rkx , &
    1959.0_rkx , 315.500_rkx , 1235.250_rkx , 291.100_rkx , 7.56_rkx , 24.21_rkx , &
    1960.0_rkx , 316.272_rkx , 1247.000_rkx , 291.400_rkx , 8.82_rkx , 27.68_rkx , &
    1961.0_rkx , 317.075_rkx , 1259.250_rkx , 291.675_rkx , 10.30_rkx , 31.68_rkx , &
    1962.0_rkx , 317.795_rkx , 1272.000_rkx , 291.925_rkx , 12.23_rkx , 36.31_rkx , &
    1963.0_rkx , 318.397_rkx , 1285.000_rkx , 292.225_rkx , 14.71_rkx , 41.59_rkx , &
    1964.0_rkx , 318.925_rkx , 1298.250_rkx , 292.600_rkx , 17.80_rkx , 47.73_rkx , &
    1965.0_rkx , 319.647_rkx , 1312.000_rkx , 293.000_rkx , 21.50_rkx , 54.93_rkx , &
    1966.0_rkx , 320.647_rkx , 1326.000_rkx , 293.400_rkx , 25.77_rkx , 63.16_rkx , &
    1967.0_rkx , 321.605_rkx , 1340.250_rkx , 293.800_rkx , 30.57_rkx , 72.39_rkx , &
    1968.0_rkx , 322.635_rkx , 1355.000_rkx , 294.225_rkx , 35.95_rkx , 82.77_rkx , &
    1969.0_rkx , 323.902_rkx , 1370.250_rkx , 294.700_rkx , 42.05_rkx , 94.44_rkx , &
    1970.0_rkx , 324.985_rkx , 1385.750_rkx , 295.200_rkx , 49.05_rkx , 107.45_rkx , &
    1971.0_rkx , 325.855_rkx , 1401.250_rkx , 295.700_rkx , 57.06_rkx , 121.80_rkx , &
    1972.0_rkx , 327.140_rkx , 1417.000_rkx , 296.225_rkx , 65.97_rkx , 137.30_rkx , &
    1973.0_rkx , 328.678_rkx , 1433.000_rkx , 296.800_rkx , 75.83_rkx , 154.05_rkx , &
    1974.0_rkx , 329.743_rkx , 1449.000_rkx , 297.400_rkx , 87.04_rkx , 172.45_rkx , &
    1975.0_rkx , 330.585_rkx , 1465.000_rkx , 298.000_rkx , 99.56_rkx , 192.60_rkx , &
    1976.0_rkx , 331.748_rkx , 1481.000_rkx , 298.650_rkx , 112.30_rkx , 213.15_rkx , &
    1977.0_rkx , 333.272_rkx , 1497.250_rkx , 299.360_rkx , 124.70_rkx , 232.90_rkx , &
    1978.0_rkx , 334.848_rkx , 1513.750_rkx , 300.082_rkx , 136.70_rkx , 251.90_rkx , &
    1979.0_rkx , 336.525_rkx , 1530.250_rkx , 300.740_rkx , 148.80_rkx , 270.50_rkx , &
    1980.0_rkx , 338.360_rkx , 1547.750_rkx , 301.382_rkx , 159.40_rkx , 287.35_rkx , &
    1981.0_rkx , 339.728_rkx , 1566.750_rkx , 302.220_rkx , 168.55_rkx , 304.25_rkx , &
    1982.0_rkx , 340.793_rkx , 1586.250_rkx , 303.098_rkx , 177.85_rkx , 321.70_rkx , &
    1983.0_rkx , 342.197_rkx , 1605.845_rkx , 303.805_rkx , 185.80_rkx , 338.15_rkx , &
    1984.0_rkx , 343.783_rkx , 1623.705_rkx , 304.397_rkx , 193.75_rkx , 355.00_rkx , &
    1985.0_rkx , 345.283_rkx , 1637.993_rkx , 305.087_rkx , 202.85_rkx , 372.50_rkx , &
    1986.0_rkx , 346.798_rkx , 1650.435_rkx , 305.772_rkx , 212.10_rkx , 389.80_rkx , &
    1987.0_rkx , 348.645_rkx , 1662.200_rkx , 306.262_rkx , 222.80_rkx , 408.65_rkx , &
    1988.0_rkx , 350.738_rkx , 1673.070_rkx , 307.195_rkx , 234.90_rkx , 428.45_rkx , &
    1989.0_rkx , 352.488_rkx , 1683.470_rkx , 308.507_rkx , 244.90_rkx , 448.25_rkx , &
    1990.0_rkx , 353.855_rkx , 1693.630_rkx , 309.485_rkx , 252.75_rkx , 466.15_rkx , &
    1991.0_rkx , 355.017_rkx , 1703.818_rkx , 310.113_rkx , 260.15_rkx , 481.30_rkx , &
    1992.0_rkx , 355.885_rkx , 1711.800_rkx , 310.603_rkx , 264.95_rkx , 494.40_rkx , &
    1993.0_rkx , 356.777_rkx , 1716.290_rkx , 311.077_rkx , 267.75_rkx , 504.80_rkx , &
    1994.0_rkx , 358.127_rkx , 1721.013_rkx , 311.603_rkx , 269.50_rkx , 513.10_rkx , &
    1995.0_rkx , 359.837_rkx , 1726.350_rkx , 312.295_rkx , 269.50_rkx , 519.55_rkx , &
    1996.0_rkx , 361.462_rkx , 1730.010_rkx , 313.048_rkx , 268.95_rkx , 525.05_rkx , &
    1997.0_rkx , 363.155_rkx , 1734.818_rkx , 313.760_rkx , 268.20_rkx , 529.15_rkx , &
    1998.0_rkx , 365.322_rkx , 1742.710_rkx , 314.533_rkx , 266.95_rkx , 532.40_rkx , &
    1999.0_rkx , 367.348_rkx , 1749.243_rkx , 315.257_rkx , 265.20_rkx , 534.95_rkx , &
    2000.0_rkx , 368.865_rkx , 1751.023_rkx , 315.850_rkx , 263.45_rkx , 537.05_rkx , &
    2001.0_rkx , 370.467_rkx , 1750.707_rkx , 316.488_rkx , 261.90_rkx , 538.85_rkx , &
    2002.0_rkx , 372.522_rkx , 1752.188_rkx , 317.245_rkx , 260.10_rkx , 539.95_rkx , &
    2003.0_rkx , 374.760_rkx , 1754.760_rkx , 318.017_rkx , 257.95_rkx , 540.05_rkx , &
    2004.0_rkx , 376.812_rkx , 1755.065_rkx , 318.710_rkx , 256.05_rkx , 539.90_rkx , &
    2005.0_rkx , 378.812_rkx , 1753.735_rkx , 319.440_rkx , 254.10_rkx , 539.40_rkx , &
    2006.0_rkx , 380.827_rkx , 1754.514_rkx , 320.200_rkx , 251.30_rkx , 537.50_rkx , &
    2007.0_rkx , 382.777_rkx , 1757.278_rkx , 320.889_rkx , 247.95_rkx , 534.70_rkx , &
    2008.0_rkx , 384.800_rkx , 1760.392_rkx , 321.581_rkx , 244.60_rkx , 531.80_rkx , &
    2009.0_rkx , 386.952_rkx , 1763.776_rkx , 322.273_rkx , 241.03_rkx , 528.54_rkx , &
    2010.0_rkx , 389.128_rkx , 1767.098_rkx , 322.967_rkx , 237.28_rkx , 524.94_rkx , &
    2011.0_rkx , 391.274_rkx , 1770.370_rkx , 323.661_rkx , 233.54_rkx , 521.21_rkx , &
    2012.0_rkx , 393.421_rkx , 1773.634_rkx , 324.358_rkx , 229.80_rkx , 517.33_rkx , &
    2013.0_rkx , 395.583_rkx , 1776.916_rkx , 325.056_rkx , 226.06_rkx , 513.32_rkx , &
    2014.0_rkx , 397.764_rkx , 1780.244_rkx , 325.755_rkx , 222.35_rkx , 509.22_rkx , &
    2015.0_rkx , 399.966_rkx , 1783.653_rkx , 326.456_rkx , 218.66_rkx , 505.02_rkx , &
    2016.0_rkx , 402.184_rkx , 1787.140_rkx , 327.159_rkx , 215.00_rkx , 500.76_rkx , &
    2017.0_rkx , 404.411_rkx , 1790.677_rkx , 327.863_rkx , 211.36_rkx , 496.42_rkx , &
    2018.0_rkx , 406.643_rkx , 1794.245_rkx , 328.568_rkx , 207.74_rkx , 492.03_rkx , &
    2019.0_rkx , 408.882_rkx , 1797.833_rkx , 329.275_rkx , 204.15_rkx , 487.59_rkx , &
    2020.0_rkx , 411.129_rkx , 1801.434_rkx , 329.983_rkx , 200.59_rkx , 483.10_rkx , &
    2021.0_rkx , 413.378_rkx , 1804.969_rkx , 330.693_rkx , 197.04_rkx , 478.56_rkx , &
    2022.0_rkx , 415.639_rkx , 1808.303_rkx , 331.404_rkx , 193.51_rkx , 473.98_rkx , &
    2023.0_rkx , 417.936_rkx , 1811.403_rkx , 332.116_rkx , 190.00_rkx , 469.36_rkx , &
    2024.0_rkx , 420.274_rkx , 1814.324_rkx , 332.828_rkx , 186.52_rkx , 464.73_rkx , &
    2025.0_rkx , 422.656_rkx , 1817.116_rkx , 333.541_rkx , 183.07_rkx , 460.07_rkx , &
    2026.0_rkx , 425.080_rkx , 1819.828_rkx , 334.255_rkx , 179.66_rkx , 455.42_rkx , &
    2027.0_rkx , 427.538_rkx , 1822.476_rkx , 334.970_rkx , 176.29_rkx , 450.77_rkx , &
    2028.0_rkx , 430.021_rkx , 1825.042_rkx , 335.685_rkx , 172.96_rkx , 446.12_rkx , &
    2029.0_rkx , 432.523_rkx , 1827.519_rkx , 336.401_rkx , 169.67_rkx , 441.47_rkx , &
    2030.0_rkx , 435.046_rkx , 1829.908_rkx , 337.118_rkx , 166.41_rkx , 436.83_rkx , &
    2031.0_rkx , 437.589_rkx , 1832.212_rkx , 337.836_rkx , 163.19_rkx , 432.20_rkx , &
    2032.0_rkx , 440.131_rkx , 1834.257_rkx , 338.551_rkx , 160.00_rkx , 427.57_rkx , &
    2033.0_rkx , 442.664_rkx , 1835.917_rkx , 339.263_rkx , 156.83_rkx , 422.94_rkx , &
    2034.0_rkx , 445.207_rkx , 1837.270_rkx , 339.971_rkx , 153.70_rkx , 418.32_rkx , &
    2035.0_rkx , 447.770_rkx , 1838.381_rkx , 340.676_rkx , 150.61_rkx , 413.72_rkx , &
    2036.0_rkx , 450.355_rkx , 1839.306_rkx , 341.376_rkx , 147.56_rkx , 409.13_rkx , &
    2037.0_rkx , 452.963_rkx , 1840.103_rkx , 342.072_rkx , 144.55_rkx , 404.57_rkx , &
    2038.0_rkx , 455.586_rkx , 1840.792_rkx , 342.765_rkx , 141.58_rkx , 400.04_rkx , &
    2039.0_rkx , 458.215_rkx , 1841.359_rkx , 343.454_rkx , 138.66_rkx , 395.54_rkx , &
    2040.0_rkx , 460.845_rkx , 1841.803_rkx , 344.139_rkx , 135.79_rkx , 391.07_rkx , &
    2041.0_rkx , 463.475_rkx , 1842.128_rkx , 344.820_rkx , 132.95_rkx , 386.62_rkx , &
    2042.0_rkx , 466.093_rkx , 1842.241_rkx , 345.495_rkx , 130.16_rkx , 382.21_rkx , &
    2043.0_rkx , 468.678_rkx , 1841.988_rkx , 346.163_rkx , 127.40_rkx , 377.81_rkx , &
    2044.0_rkx , 471.234_rkx , 1841.350_rkx , 346.822_rkx , 124.68_rkx , 373.44_rkx , &
    2045.0_rkx , 473.780_rkx , 1840.408_rkx , 347.473_rkx , 121.99_rkx , 369.09_rkx , &
    2046.0_rkx , 476.328_rkx , 1839.227_rkx , 348.115_rkx , 119.34_rkx , 364.76_rkx , &
    2047.0_rkx , 478.881_rkx , 1837.866_rkx , 348.750_rkx , 116.74_rkx , 360.47_rkx , &
    2048.0_rkx , 481.438_rkx , 1836.383_rkx , 349.377_rkx , 114.18_rkx , 356.22_rkx , &
    2049.0_rkx , 483.993_rkx , 1834.796_rkx , 349.996_rkx , 111.67_rkx , 352.01_rkx , &
    2050.0_rkx , 486.535_rkx , 1833.094_rkx , 350.608_rkx , 109.20_rkx , 347.83_rkx , &
    2051.0_rkx , 489.060_rkx , 1831.276_rkx , 351.211_rkx , 106.77_rkx , 343.69_rkx , &
    2052.0_rkx , 491.536_rkx , 1829.210_rkx , 351.807_rkx , 104.39_rkx , 339.59_rkx , &
    2053.0_rkx , 493.932_rkx , 1826.776_rkx , 352.396_rkx , 102.05_rkx , 335.53_rkx , &
    2054.0_rkx , 496.244_rkx , 1823.931_rkx , 352.978_rkx , 99.75_rkx , 331.49_rkx , &
    2055.0_rkx , 498.474_rkx , 1820.661_rkx , 353.553_rkx , 97.48_rkx , 327.48_rkx , &
    2056.0_rkx , 500.645_rkx , 1817.052_rkx , 354.120_rkx , 95.25_rkx , 323.50_rkx , &
    2057.0_rkx , 502.768_rkx , 1813.175_rkx , 354.681_rkx , 93.06_rkx , 319.56_rkx , &
    2058.0_rkx , 504.847_rkx , 1809.091_rkx , 355.235_rkx , 90.91_rkx , 315.65_rkx , &
    2059.0_rkx , 506.884_rkx , 1804.863_rkx , 355.782_rkx , 88.81_rkx , 311.79_rkx , &
    2060.0_rkx , 508.871_rkx , 1800.511_rkx , 356.322_rkx , 86.74_rkx , 307.97_rkx , &
    2061.0_rkx , 510.799_rkx , 1796.029_rkx , 356.856_rkx , 84.72_rkx , 304.19_rkx , &
    2062.0_rkx , 512.647_rkx , 1791.340_rkx , 357.382_rkx , 82.74_rkx , 300.45_rkx , &
    2063.0_rkx , 514.402_rkx , 1786.382_rkx , 357.901_rkx , 80.80_rkx , 296.74_rkx , &
    2064.0_rkx , 516.065_rkx , 1781.178_rkx , 358.411_rkx , 78.90_rkx , 293.08_rkx , &
    2065.0_rkx , 517.629_rkx , 1775.674_rkx , 358.914_rkx , 77.03_rkx , 289.45_rkx , &
    2066.0_rkx , 519.096_rkx , 1769.852_rkx , 359.409_rkx , 75.20_rkx , 285.85_rkx , &
    2067.0_rkx , 520.488_rkx , 1763.787_rkx , 359.897_rkx , 73.40_rkx , 282.28_rkx , &
    2068.0_rkx , 521.818_rkx , 1757.544_rkx , 360.377_rkx , 71.64_rkx , 278.75_rkx , &
    2069.0_rkx , 523.089_rkx , 1751.177_rkx , 360.849_rkx , 69.91_rkx , 275.25_rkx , &
    2070.0_rkx , 524.302_rkx , 1744.739_rkx , 361.314_rkx , 68.23_rkx , 271.80_rkx , &
    2071.0_rkx , 525.451_rkx , 1738.248_rkx , 361.771_rkx , 66.58_rkx , 268.39_rkx , &
    2072.0_rkx , 526.509_rkx , 1731.615_rkx , 362.220_rkx , 64.97_rkx , 265.02_rkx , &
    2073.0_rkx , 527.457_rkx , 1724.767_rkx , 362.661_rkx , 63.39_rkx , 261.69_rkx , &
    2074.0_rkx , 528.296_rkx , 1717.722_rkx , 363.093_rkx , 61.85_rkx , 258.40_rkx , &
    2075.0_rkx , 529.027_rkx , 1710.494_rkx , 363.517_rkx , 60.34_rkx , 255.14_rkx , &
    2076.0_rkx , 529.643_rkx , 1703.032_rkx , 363.932_rkx , 58.87_rkx , 251.92_rkx , &
    2077.0_rkx , 530.144_rkx , 1695.312_rkx , 364.339_rkx , 57.42_rkx , 248.73_rkx , &
    2078.0_rkx , 530.553_rkx , 1687.426_rkx , 364.738_rkx , 56.00_rkx , 245.57_rkx , &
    2079.0_rkx , 530.883_rkx , 1679.552_rkx , 365.129_rkx , 54.61_rkx , 242.45_rkx , &
    2080.0_rkx , 531.138_rkx , 1671.829_rkx , 365.511_rkx , 53.26_rkx , 239.36_rkx , &
    2081.0_rkx , 531.319_rkx , 1664.284_rkx , 365.885_rkx , 51.94_rkx , 236.32_rkx , &
    2082.0_rkx , 531.490_rkx , 1657.122_rkx , 366.254_rkx , 50.65_rkx , 233.31_rkx , &
    2083.0_rkx , 531.702_rkx , 1650.494_rkx , 366.618_rkx , 49.39_rkx , 230.34_rkx , &
    2084.0_rkx , 531.942_rkx , 1644.324_rkx , 366.979_rkx , 48.16_rkx , 227.41_rkx , &
    2085.0_rkx , 532.205_rkx , 1638.556_rkx , 367.336_rkx , 46.96_rkx , 224.52_rkx , &
    2086.0_rkx , 532.487_rkx , 1633.140_rkx , 367.690_rkx , 45.79_rkx , 221.67_rkx , &
    2087.0_rkx , 532.776_rkx , 1627.966_rkx , 368.040_rkx , 44.65_rkx , 218.84_rkx , &
    2088.0_rkx , 533.070_rkx , 1622.963_rkx , 368.386_rkx , 43.52_rkx , 216.04_rkx , &
    2089.0_rkx , 533.388_rkx , 1618.155_rkx , 368.729_rkx , 42.43_rkx , 213.27_rkx , &
    2090.0_rkx , 533.741_rkx , 1613.554_rkx , 369.068_rkx , 41.36_rkx , 210.54_rkx , &
    2091.0_rkx , 534.131_rkx , 1609.170_rkx , 369.404_rkx , 40.31_rkx , 207.84_rkx , &
    2092.0_rkx , 534.558_rkx , 1605.014_rkx , 369.736_rkx , 39.29_rkx , 205.17_rkx , &
    2093.0_rkx , 535.011_rkx , 1601.065_rkx , 370.065_rkx , 38.30_rkx , 202.55_rkx , &
    2094.0_rkx , 535.480_rkx , 1597.281_rkx , 370.390_rkx , 37.33_rkx , 199.95_rkx , &
    2095.0_rkx , 535.955_rkx , 1593.631_rkx , 370.712_rkx , 36.38_rkx , 197.39_rkx , &
    2096.0_rkx , 536.435_rkx , 1590.093_rkx , 371.031_rkx , 35.46_rkx , 194.86_rkx , &
    2097.0_rkx , 536.920_rkx , 1586.651_rkx , 371.347_rkx , 34.56_rkx , 192.36_rkx , &
    2098.0_rkx , 537.399_rkx , 1583.228_rkx , 371.659_rkx , 33.68_rkx , 189.89_rkx , &
    2099.0_rkx , 537.871_rkx , 1579.778_rkx , 371.968_rkx , 32.82_rkx , 187.45_rkx , &
    2100.0_rkx , 538.358_rkx , 1576.346_rkx , 372.274_rkx , 31.98_rkx , 185.03_rkx /), &
      (/6,251/))
      case( 'RCP6' , 'RCP60' , 'RCP6.0' )
        cgas(1:6,1850:2100) = reshape((/    &
    1850.0_rkx , 284.725_rkx , 790.979_rkx , 275.425_rkx , 0.00_rkx , 0.00_rkx , &
    1851.0_rkx , 284.875_rkx , 792.250_rkx , 275.500_rkx , 0.00_rkx , 0.00_rkx , &
    1852.0_rkx , 285.000_rkx , 793.750_rkx , 275.600_rkx , 0.00_rkx , 0.00_rkx , &
    1853.0_rkx , 285.125_rkx , 795.250_rkx , 275.700_rkx , 0.00_rkx , 0.00_rkx , &
    1854.0_rkx , 285.275_rkx , 796.750_rkx , 275.800_rkx , 0.00_rkx , 0.00_rkx , &
    1855.0_rkx , 285.425_rkx , 798.000_rkx , 275.900_rkx , 0.00_rkx , 0.00_rkx , &
    1856.0_rkx , 285.575_rkx , 799.250_rkx , 276.000_rkx , 0.00_rkx , 0.00_rkx , &
    1857.0_rkx , 285.725_rkx , 800.750_rkx , 276.100_rkx , 0.00_rkx , 0.00_rkx , &
    1858.0_rkx , 285.900_rkx , 802.250_rkx , 276.200_rkx , 0.00_rkx , 0.00_rkx , &
    1859.0_rkx , 286.075_rkx , 803.750_rkx , 276.300_rkx , 0.00_rkx , 0.00_rkx , &
    1860.0_rkx , 286.225_rkx , 805.250_rkx , 276.400_rkx , 0.00_rkx , 0.00_rkx , &
    1861.0_rkx , 286.375_rkx , 806.750_rkx , 276.500_rkx , 0.00_rkx , 0.00_rkx , &
    1862.0_rkx , 286.500_rkx , 808.250_rkx , 276.600_rkx , 0.00_rkx , 0.00_rkx , &
    1863.0_rkx , 286.625_rkx , 809.750_rkx , 276.700_rkx , 0.00_rkx , 0.00_rkx , &
    1864.0_rkx , 286.775_rkx , 811.250_rkx , 276.800_rkx , 0.00_rkx , 0.00_rkx , &
    1865.0_rkx , 286.900_rkx , 813.000_rkx , 276.900_rkx , 0.00_rkx , 0.00_rkx , &
    1866.0_rkx , 287.000_rkx , 814.750_rkx , 277.000_rkx , 0.00_rkx , 0.00_rkx , &
    1867.0_rkx , 287.100_rkx , 816.250_rkx , 277.100_rkx , 0.00_rkx , 0.00_rkx , &
    1868.0_rkx , 287.225_rkx , 817.750_rkx , 277.200_rkx , 0.00_rkx , 0.00_rkx , &
    1869.0_rkx , 287.375_rkx , 819.250_rkx , 277.300_rkx , 0.00_rkx , 0.00_rkx , &
    1870.0_rkx , 287.525_rkx , 821.000_rkx , 277.375_rkx , 0.00_rkx , 0.00_rkx , &
    1871.0_rkx , 287.700_rkx , 822.750_rkx , 277.425_rkx , 0.00_rkx , 0.00_rkx , &
    1872.0_rkx , 287.900_rkx , 824.250_rkx , 277.500_rkx , 0.00_rkx , 0.00_rkx , &
    1873.0_rkx , 288.125_rkx , 825.750_rkx , 277.600_rkx , 0.00_rkx , 0.00_rkx , &
    1874.0_rkx , 288.400_rkx , 827.250_rkx , 277.700_rkx , 0.00_rkx , 0.00_rkx , &
    1875.0_rkx , 288.700_rkx , 829.000_rkx , 277.775_rkx , 0.00_rkx , 0.00_rkx , &
    1876.0_rkx , 289.025_rkx , 830.750_rkx , 277.825_rkx , 0.00_rkx , 0.00_rkx , &
    1877.0_rkx , 289.400_rkx , 832.250_rkx , 277.900_rkx , 0.00_rkx , 0.00_rkx , &
    1878.0_rkx , 289.800_rkx , 833.750_rkx , 278.000_rkx , 0.00_rkx , 0.00_rkx , &
    1879.0_rkx , 290.225_rkx , 835.250_rkx , 278.100_rkx , 0.00_rkx , 0.00_rkx , &
    1880.0_rkx , 290.700_rkx , 837.000_rkx , 278.200_rkx , 0.00_rkx , 0.00_rkx , &
    1881.0_rkx , 291.200_rkx , 839.000_rkx , 278.300_rkx , 0.00_rkx , 0.00_rkx , &
    1882.0_rkx , 291.675_rkx , 841.000_rkx , 278.400_rkx , 0.00_rkx , 0.00_rkx , &
    1883.0_rkx , 292.125_rkx , 843.000_rkx , 278.500_rkx , 0.00_rkx , 0.00_rkx , &
    1884.0_rkx , 292.575_rkx , 844.750_rkx , 278.600_rkx , 0.00_rkx , 0.00_rkx , &
    1885.0_rkx , 292.975_rkx , 846.250_rkx , 278.700_rkx , 0.00_rkx , 0.00_rkx , &
    1886.0_rkx , 293.300_rkx , 848.000_rkx , 278.800_rkx , 0.00_rkx , 0.00_rkx , &
    1887.0_rkx , 293.575_rkx , 850.000_rkx , 278.875_rkx , 0.00_rkx , 0.00_rkx , &
    1888.0_rkx , 293.800_rkx , 852.000_rkx , 278.925_rkx , 0.00_rkx , 0.00_rkx , &
    1889.0_rkx , 294.000_rkx , 854.000_rkx , 279.000_rkx , 0.00_rkx , 0.00_rkx , &
    1890.0_rkx , 294.175_rkx , 856.000_rkx , 279.100_rkx , 0.00_rkx , 0.00_rkx , &
    1891.0_rkx , 294.325_rkx , 858.250_rkx , 279.200_rkx , 0.00_rkx , 0.00_rkx , &
    1892.0_rkx , 294.475_rkx , 860.750_rkx , 279.300_rkx , 0.00_rkx , 0.00_rkx , &
    1893.0_rkx , 294.600_rkx , 863.000_rkx , 279.375_rkx , 0.00_rkx , 0.00_rkx , &
    1894.0_rkx , 294.700_rkx , 865.000_rkx , 279.425_rkx , 0.00_rkx , 0.00_rkx , &
    1895.0_rkx , 294.800_rkx , 867.250_rkx , 279.475_rkx , 0.00_rkx , 0.00_rkx , &
    1896.0_rkx , 294.900_rkx , 869.750_rkx , 279.525_rkx , 0.00_rkx , 0.00_rkx , &
    1897.0_rkx , 295.025_rkx , 872.000_rkx , 279.600_rkx , 0.00_rkx , 0.00_rkx , &
    1898.0_rkx , 295.225_rkx , 874.250_rkx , 279.675_rkx , 0.00_rkx , 0.00_rkx , &
    1899.0_rkx , 295.500_rkx , 876.750_rkx , 279.725_rkx , 0.00_rkx , 0.00_rkx , &
    1900.0_rkx , 295.800_rkx , 879.500_rkx , 279.800_rkx , 0.00_rkx , 0.00_rkx , &
    1901.0_rkx , 296.125_rkx , 883.000_rkx , 279.900_rkx , 0.00_rkx , 0.00_rkx , &
    1902.0_rkx , 296.475_rkx , 887.000_rkx , 280.000_rkx , 0.00_rkx , 0.00_rkx , &
    1903.0_rkx , 296.825_rkx , 891.250_rkx , 280.100_rkx , 0.00_rkx , 0.00_rkx , &
    1904.0_rkx , 297.200_rkx , 895.750_rkx , 280.200_rkx , 0.00_rkx , 0.00_rkx , &
    1905.0_rkx , 297.625_rkx , 900.250_rkx , 280.300_rkx , 0.00_rkx , 0.00_rkx , &
    1906.0_rkx , 298.075_rkx , 904.750_rkx , 280.400_rkx , 0.00_rkx , 0.00_rkx , &
    1907.0_rkx , 298.500_rkx , 909.250_rkx , 280.525_rkx , 0.00_rkx , 0.00_rkx , &
    1908.0_rkx , 298.900_rkx , 914.000_rkx , 280.675_rkx , 0.00_rkx , 0.00_rkx , &
    1909.0_rkx , 299.300_rkx , 919.000_rkx , 280.825_rkx , 0.00_rkx , 0.00_rkx , &
    1910.0_rkx , 299.700_rkx , 923.750_rkx , 280.975_rkx , 0.00_rkx , 0.00_rkx , &
    1911.0_rkx , 300.075_rkx , 928.250_rkx , 281.125_rkx , 0.00_rkx , 0.00_rkx , &
    1912.0_rkx , 300.425_rkx , 933.000_rkx , 281.300_rkx , 0.00_rkx , 0.00_rkx , &
    1913.0_rkx , 300.775_rkx , 938.250_rkx , 281.475_rkx , 0.00_rkx , 0.00_rkx , &
    1914.0_rkx , 301.100_rkx , 943.750_rkx , 281.625_rkx , 0.00_rkx , 0.00_rkx , &
    1915.0_rkx , 301.400_rkx , 949.250_rkx , 281.800_rkx , 0.00_rkx , 0.00_rkx , &
    1916.0_rkx , 301.725_rkx , 954.750_rkx , 282.000_rkx , 0.00_rkx , 0.00_rkx , &
    1917.0_rkx , 302.075_rkx , 960.250_rkx , 282.225_rkx , 0.00_rkx , 0.00_rkx , &
    1918.0_rkx , 302.400_rkx , 966.000_rkx , 282.475_rkx , 0.00_rkx , 0.00_rkx , &
    1919.0_rkx , 302.700_rkx , 972.000_rkx , 282.700_rkx , 0.00_rkx , 0.00_rkx , &
    1920.0_rkx , 303.025_rkx , 977.750_rkx , 282.925_rkx , 0.00_rkx , 0.00_rkx , &
    1921.0_rkx , 303.400_rkx , 983.250_rkx , 283.175_rkx , 0.00_rkx , 0.00_rkx , &
    1922.0_rkx , 303.775_rkx , 989.000_rkx , 283.400_rkx , 0.00_rkx , 0.00_rkx , &
    1923.0_rkx , 304.125_rkx , 995.000_rkx , 283.600_rkx , 0.00_rkx , 0.00_rkx , &
    1924.0_rkx , 304.525_rkx , 1001.000_rkx , 283.800_rkx , 0.00_rkx , 0.00_rkx , &
    1925.0_rkx , 304.975_rkx , 1007.000_rkx , 284.000_rkx , 0.00_rkx , 0.00_rkx , &
    1926.0_rkx , 305.400_rkx , 1013.000_rkx , 284.225_rkx , 0.00_rkx , 0.00_rkx , &
    1927.0_rkx , 305.825_rkx , 1019.000_rkx , 284.450_rkx , 0.00_rkx , 0.00_rkx , &
    1928.0_rkx , 306.300_rkx , 1025.000_rkx , 284.625_rkx , 0.00_rkx , 0.00_rkx , &
    1929.0_rkx , 306.775_rkx , 1030.750_rkx , 284.800_rkx , 0.00_rkx , 0.00_rkx , &
    1930.0_rkx , 307.225_rkx , 1036.250_rkx , 284.975_rkx , 0.00_rkx , 0.00_rkx , &
    1931.0_rkx , 307.700_rkx , 1041.750_rkx , 285.125_rkx , 0.00_rkx , 0.00_rkx , &
    1932.0_rkx , 308.175_rkx , 1047.250_rkx , 285.300_rkx , 0.00_rkx , 0.00_rkx , &
    1933.0_rkx , 308.600_rkx , 1052.750_rkx , 285.500_rkx , 0.00_rkx , 0.01_rkx , &
    1934.0_rkx , 309.000_rkx , 1058.000_rkx , 285.700_rkx , 0.00_rkx , 0.01_rkx , &
    1935.0_rkx , 309.400_rkx , 1063.000_rkx , 285.900_rkx , 0.00_rkx , 0.02_rkx , &
    1936.0_rkx , 309.750_rkx , 1068.000_rkx , 286.100_rkx , 0.00_rkx , 0.03_rkx , &
    1937.0_rkx , 310.000_rkx , 1073.000_rkx , 286.275_rkx , 0.00_rkx , 0.05_rkx , &
    1938.0_rkx , 310.175_rkx , 1078.000_rkx , 286.425_rkx , 0.00_rkx , 0.08_rkx , &
    1939.0_rkx , 310.300_rkx , 1083.000_rkx , 286.575_rkx , 0.00_rkx , 0.13_rkx , &
    1940.0_rkx , 310.375_rkx , 1088.250_rkx , 286.725_rkx , 0.01_rkx , 0.20_rkx , &
    1941.0_rkx , 310.375_rkx , 1093.750_rkx , 286.900_rkx , 0.02_rkx , 0.29_rkx , &
    1942.0_rkx , 310.300_rkx , 1099.000_rkx , 287.100_rkx , 0.02_rkx , 0.42_rkx , &
    1943.0_rkx , 310.200_rkx , 1104.000_rkx , 287.300_rkx , 0.03_rkx , 0.58_rkx , &
    1944.0_rkx , 310.125_rkx , 1109.250_rkx , 287.525_rkx , 0.04_rkx , 0.77_rkx , &
    1945.0_rkx , 310.100_rkx , 1114.750_rkx , 287.775_rkx , 0.05_rkx , 1.02_rkx , &
    1946.0_rkx , 310.125_rkx , 1120.500_rkx , 288.025_rkx , 0.07_rkx , 1.35_rkx , &
    1947.0_rkx , 310.200_rkx , 1126.750_rkx , 288.275_rkx , 0.10_rkx , 1.87_rkx , &
    1948.0_rkx , 310.325_rkx , 1133.250_rkx , 288.525_rkx , 0.16_rkx , 2.70_rkx , &
    1949.0_rkx , 310.500_rkx , 1140.000_rkx , 288.775_rkx , 0.29_rkx , 3.80_rkx , &
    1950.0_rkx , 310.750_rkx , 1147.250_rkx , 289.000_rkx , 0.50_rkx , 5.00_rkx , &
    1951.0_rkx , 311.100_rkx , 1155.000_rkx , 289.200_rkx , 0.81_rkx , 6.31_rkx , &
    1952.0_rkx , 311.500_rkx , 1163.500_rkx , 289.425_rkx , 1.17_rkx , 7.81_rkx , &
    1953.0_rkx , 311.925_rkx , 1172.750_rkx , 289.675_rkx , 1.58_rkx , 9.46_rkx , &
    1954.0_rkx , 312.425_rkx , 1182.250_rkx , 289.900_rkx , 2.15_rkx , 11.24_rkx , &
    1955.0_rkx , 313.000_rkx , 1192.000_rkx , 290.125_rkx , 2.89_rkx , 13.24_rkx , &
    1956.0_rkx , 313.600_rkx , 1202.250_rkx , 290.375_rkx , 3.79_rkx , 15.48_rkx , &
    1957.0_rkx , 314.225_rkx , 1213.000_rkx , 290.600_rkx , 4.92_rkx , 18.05_rkx , &
    1958.0_rkx , 314.848_rkx , 1224.000_rkx , 290.825_rkx , 6.24_rkx , 21.01_rkx , &
    1959.0_rkx , 315.500_rkx , 1235.250_rkx , 291.100_rkx , 7.56_rkx , 24.21_rkx , &
    1960.0_rkx , 316.272_rkx , 1247.000_rkx , 291.400_rkx , 8.82_rkx , 27.68_rkx , &
    1961.0_rkx , 317.075_rkx , 1259.250_rkx , 291.675_rkx , 10.30_rkx , 31.68_rkx , &
    1962.0_rkx , 317.795_rkx , 1272.000_rkx , 291.925_rkx , 12.23_rkx , 36.31_rkx , &
    1963.0_rkx , 318.397_rkx , 1285.000_rkx , 292.225_rkx , 14.71_rkx , 41.59_rkx , &
    1964.0_rkx , 318.925_rkx , 1298.250_rkx , 292.600_rkx , 17.80_rkx , 47.73_rkx , &
    1965.0_rkx , 319.647_rkx , 1312.000_rkx , 293.000_rkx , 21.50_rkx , 54.93_rkx , &
    1966.0_rkx , 320.647_rkx , 1326.000_rkx , 293.400_rkx , 25.77_rkx , 63.16_rkx , &
    1967.0_rkx , 321.605_rkx , 1340.250_rkx , 293.800_rkx , 30.57_rkx , 72.39_rkx , &
    1968.0_rkx , 322.635_rkx , 1355.000_rkx , 294.225_rkx , 35.95_rkx , 82.77_rkx , &
    1969.0_rkx , 323.902_rkx , 1370.250_rkx , 294.700_rkx , 42.05_rkx , 94.44_rkx , &
    1970.0_rkx , 324.985_rkx , 1385.750_rkx , 295.200_rkx , 49.05_rkx , 107.45_rkx , &
    1971.0_rkx , 325.855_rkx , 1401.250_rkx , 295.700_rkx , 57.06_rkx , 121.80_rkx , &
    1972.0_rkx , 327.140_rkx , 1417.000_rkx , 296.225_rkx , 65.97_rkx , 137.30_rkx , &
    1973.0_rkx , 328.678_rkx , 1433.000_rkx , 296.800_rkx , 75.83_rkx , 154.05_rkx , &
    1974.0_rkx , 329.743_rkx , 1449.000_rkx , 297.400_rkx , 87.04_rkx , 172.45_rkx , &
    1975.0_rkx , 330.585_rkx , 1465.000_rkx , 298.000_rkx , 99.56_rkx , 192.60_rkx , &
    1976.0_rkx , 331.748_rkx , 1481.000_rkx , 298.650_rkx , 112.30_rkx , 213.15_rkx , &
    1977.0_rkx , 333.272_rkx , 1497.250_rkx , 299.360_rkx , 124.70_rkx , 232.90_rkx , &
    1978.0_rkx , 334.848_rkx , 1513.750_rkx , 300.082_rkx , 136.70_rkx , 251.90_rkx , &
    1979.0_rkx , 336.525_rkx , 1530.250_rkx , 300.740_rkx , 148.80_rkx , 270.50_rkx , &
    1980.0_rkx , 338.360_rkx , 1547.750_rkx , 301.382_rkx , 159.40_rkx , 287.35_rkx , &
    1981.0_rkx , 339.728_rkx , 1566.750_rkx , 302.220_rkx , 168.55_rkx , 304.25_rkx , &
    1982.0_rkx , 340.793_rkx , 1586.250_rkx , 303.098_rkx , 177.85_rkx , 321.70_rkx , &
    1983.0_rkx , 342.197_rkx , 1605.845_rkx , 303.805_rkx , 185.80_rkx , 338.15_rkx , &
    1984.0_rkx , 343.783_rkx , 1623.705_rkx , 304.397_rkx , 193.75_rkx , 355.00_rkx , &
    1985.0_rkx , 345.283_rkx , 1637.993_rkx , 305.087_rkx , 202.85_rkx , 372.50_rkx , &
    1986.0_rkx , 346.798_rkx , 1650.435_rkx , 305.772_rkx , 212.10_rkx , 389.80_rkx , &
    1987.0_rkx , 348.645_rkx , 1662.200_rkx , 306.262_rkx , 222.80_rkx , 408.65_rkx , &
    1988.0_rkx , 350.738_rkx , 1673.070_rkx , 307.195_rkx , 234.90_rkx , 428.45_rkx , &
    1989.0_rkx , 352.488_rkx , 1683.470_rkx , 308.507_rkx , 244.90_rkx , 448.25_rkx , &
    1990.0_rkx , 353.855_rkx , 1693.630_rkx , 309.485_rkx , 252.75_rkx , 466.15_rkx , &
    1991.0_rkx , 355.017_rkx , 1703.818_rkx , 310.113_rkx , 260.15_rkx , 481.30_rkx , &
    1992.0_rkx , 355.885_rkx , 1711.800_rkx , 310.603_rkx , 264.95_rkx , 494.40_rkx , &
    1993.0_rkx , 356.777_rkx , 1716.290_rkx , 311.077_rkx , 267.75_rkx , 504.80_rkx , &
    1994.0_rkx , 358.127_rkx , 1721.013_rkx , 311.603_rkx , 269.50_rkx , 513.10_rkx , &
    1995.0_rkx , 359.837_rkx , 1726.350_rkx , 312.295_rkx , 269.50_rkx , 519.55_rkx , &
    1996.0_rkx , 361.462_rkx , 1730.010_rkx , 313.048_rkx , 268.95_rkx , 525.05_rkx , &
    1997.0_rkx , 363.155_rkx , 1734.818_rkx , 313.760_rkx , 268.20_rkx , 529.15_rkx , &
    1998.0_rkx , 365.322_rkx , 1742.710_rkx , 314.533_rkx , 266.95_rkx , 532.40_rkx , &
    1999.0_rkx , 367.348_rkx , 1749.243_rkx , 315.257_rkx , 265.20_rkx , 534.95_rkx , &
    2000.0_rkx , 368.865_rkx , 1751.023_rkx , 315.850_rkx , 263.45_rkx , 537.05_rkx , &
    2001.0_rkx , 370.467_rkx , 1750.707_rkx , 316.488_rkx , 261.90_rkx , 538.85_rkx , &
    2002.0_rkx , 372.522_rkx , 1752.188_rkx , 317.245_rkx , 260.10_rkx , 539.95_rkx , &
    2003.0_rkx , 374.760_rkx , 1754.760_rkx , 318.017_rkx , 257.95_rkx , 540.05_rkx , &
    2004.0_rkx , 376.812_rkx , 1755.065_rkx , 318.710_rkx , 256.05_rkx , 539.90_rkx , &
    2005.0_rkx , 378.812_rkx , 1753.735_rkx , 319.440_rkx , 254.10_rkx , 539.40_rkx , &
    2006.0_rkx , 380.827_rkx , 1754.518_rkx , 320.200_rkx , 251.30_rkx , 537.50_rkx , &
    2007.0_rkx , 382.777_rkx , 1757.392_rkx , 320.896_rkx , 247.95_rkx , 534.70_rkx , &
    2008.0_rkx , 384.800_rkx , 1760.816_rkx , 321.607_rkx , 244.60_rkx , 531.80_rkx , &
    2009.0_rkx , 386.935_rkx , 1764.696_rkx , 322.332_rkx , 241.03_rkx , 528.54_rkx , &
    2010.0_rkx , 389.072_rkx , 1768.688_rkx , 323.071_rkx , 237.30_rkx , 524.96_rkx , &
    2011.0_rkx , 391.166_rkx , 1772.789_rkx , 323.826_rkx , 233.56_rkx , 521.23_rkx , &
    2012.0_rkx , 393.241_rkx , 1776.560_rkx , 324.581_rkx , 229.83_rkx , 517.36_rkx , &
    2013.0_rkx , 395.298_rkx , 1779.589_rkx , 325.325_rkx , 226.11_rkx , 513.37_rkx , &
    2014.0_rkx , 397.346_rkx , 1781.967_rkx , 326.057_rkx , 222.41_rkx , 509.28_rkx , &
    2015.0_rkx , 399.387_rkx , 1783.789_rkx , 326.778_rkx , 218.73_rkx , 505.10_rkx , &
    2016.0_rkx , 401.418_rkx , 1785.104_rkx , 327.486_rkx , 215.09_rkx , 500.85_rkx , &
    2017.0_rkx , 403.431_rkx , 1785.934_rkx , 328.183_rkx , 211.46_rkx , 496.54_rkx , &
    2018.0_rkx , 405.425_rkx , 1786.304_rkx , 328.867_rkx , 207.87_rkx , 492.17_rkx , &
    2019.0_rkx , 407.401_rkx , 1786.247_rkx , 329.540_rkx , 204.30_rkx , 487.74_rkx , &
    2020.0_rkx , 409.360_rkx , 1785.791_rkx , 330.202_rkx , 200.75_rkx , 483.28_rkx , &
    2021.0_rkx , 411.298_rkx , 1784.894_rkx , 330.852_rkx , 197.23_rkx , 478.77_rkx , &
    2022.0_rkx , 413.219_rkx , 1784.030_rkx , 331.502_rkx , 193.72_rkx , 474.22_rkx , &
    2023.0_rkx , 415.144_rkx , 1783.731_rkx , 332.165_rkx , 190.24_rkx , 469.63_rkx , &
    2024.0_rkx , 417.083_rkx , 1783.987_rkx , 332.841_rkx , 186.78_rkx , 465.03_rkx , &
    2025.0_rkx , 419.036_rkx , 1784.786_rkx , 333.529_rkx , 183.36_rkx , 460.41_rkx , &
    2026.0_rkx , 421.004_rkx , 1786.123_rkx , 334.230_rkx , 179.97_rkx , 455.79_rkx , &
    2027.0_rkx , 422.978_rkx , 1787.962_rkx , 334.943_rkx , 176.63_rkx , 451.17_rkx , &
    2028.0_rkx , 424.950_rkx , 1790.239_rkx , 335.669_rkx , 173.32_rkx , 446.55_rkx , &
    2029.0_rkx , 426.916_rkx , 1792.906_rkx , 336.408_rkx , 170.05_rkx , 441.94_rkx , &
    2030.0_rkx , 428.876_rkx , 1795.924_rkx , 337.159_rkx , 166.82_rkx , 437.33_rkx , &
    2031.0_rkx , 430.832_rkx , 1799.265_rkx , 337.922_rkx , 163.62_rkx , 432.74_rkx , &
    2032.0_rkx , 432.807_rkx , 1802.863_rkx , 338.697_rkx , 160.46_rkx , 428.15_rkx , &
    2033.0_rkx , 434.831_rkx , 1806.674_rkx , 339.485_rkx , 157.32_rkx , 423.56_rkx , &
    2034.0_rkx , 436.916_rkx , 1810.728_rkx , 340.285_rkx , 154.22_rkx , 418.99_rkx , &
    2035.0_rkx , 439.068_rkx , 1815.041_rkx , 341.097_rkx , 151.16_rkx , 414.43_rkx , &
    2036.0_rkx , 441.286_rkx , 1819.628_rkx , 341.922_rkx , 148.14_rkx , 409.90_rkx , &
    2037.0_rkx , 443.567_rkx , 1824.509_rkx , 342.759_rkx , 145.17_rkx , 405.39_rkx , &
    2038.0_rkx , 445.903_rkx , 1829.667_rkx , 343.607_rkx , 142.24_rkx , 400.92_rkx , &
    2039.0_rkx , 448.282_rkx , 1835.058_rkx , 344.467_rkx , 139.35_rkx , 396.47_rkx , &
    2040.0_rkx , 450.698_rkx , 1840.651_rkx , 345.339_rkx , 136.51_rkx , 392.06_rkx , &
    2041.0_rkx , 453.150_rkx , 1846.420_rkx , 346.223_rkx , 133.71_rkx , 387.67_rkx , &
    2042.0_rkx , 455.645_rkx , 1852.229_rkx , 347.116_rkx , 130.95_rkx , 383.32_rkx , &
    2043.0_rkx , 458.182_rkx , 1857.886_rkx , 348.020_rkx , 128.23_rkx , 378.98_rkx , &
    2044.0_rkx , 460.762_rkx , 1863.352_rkx , 348.932_rkx , 125.54_rkx , 374.66_rkx , &
    2045.0_rkx , 463.405_rkx , 1868.689_rkx , 349.853_rkx , 122.88_rkx , 370.37_rkx , &
    2046.0_rkx , 466.120_rkx , 1873.946_rkx , 350.783_rkx , 120.26_rkx , 366.10_rkx , &
    2047.0_rkx , 468.908_rkx , 1879.164_rkx , 351.723_rkx , 117.69_rkx , 361.87_rkx , &
    2048.0_rkx , 471.768_rkx , 1884.388_rkx , 352.670_rkx , 115.16_rkx , 357.67_rkx , &
    2049.0_rkx , 474.692_rkx , 1889.625_rkx , 353.627_rkx , 112.67_rkx , 353.51_rkx , &
    2050.0_rkx , 477.670_rkx , 1894.850_rkx , 354.592_rkx , 110.22_rkx , 349.39_rkx , &
    2051.0_rkx , 480.697_rkx , 1900.052_rkx , 355.566_rkx , 107.81_rkx , 345.30_rkx , &
    2052.0_rkx , 483.777_rkx , 1905.156_rkx , 356.548_rkx , 105.45_rkx , 341.24_rkx , &
    2053.0_rkx , 486.916_rkx , 1910.095_rkx , 357.539_rkx , 103.12_rkx , 337.22_rkx , &
    2054.0_rkx , 490.103_rkx , 1914.798_rkx , 358.539_rkx , 100.83_rkx , 333.23_rkx , &
    2055.0_rkx , 493.338_rkx , 1919.230_rkx , 359.547_rkx , 98.58_rkx , 329.25_rkx , &
    2056.0_rkx , 496.642_rkx , 1923.461_rkx , 360.564_rkx , 96.35_rkx , 325.31_rkx , &
    2057.0_rkx , 500.022_rkx , 1927.548_rkx , 361.589_rkx , 94.17_rkx , 321.40_rkx , &
    2058.0_rkx , 503.483_rkx , 1931.537_rkx , 362.623_rkx , 92.02_rkx , 317.52_rkx , &
    2059.0_rkx , 507.023_rkx , 1935.482_rkx , 363.664_rkx , 89.92_rkx , 313.69_rkx , &
    2060.0_rkx , 510.634_rkx , 1939.391_rkx , 364.714_rkx , 87.85_rkx , 309.89_rkx , &
    2061.0_rkx , 514.305_rkx , 1943.236_rkx , 365.772_rkx , 85.83_rkx , 306.12_rkx , &
    2062.0_rkx , 518.027_rkx , 1946.831_rkx , 366.836_rkx , 83.84_rkx , 302.39_rkx , &
    2063.0_rkx , 521.797_rkx , 1950.024_rkx , 367.905_rkx , 81.89_rkx , 298.69_rkx , &
    2064.0_rkx , 525.619_rkx , 1952.841_rkx , 368.979_rkx , 79.97_rkx , 295.03_rkx , &
    2065.0_rkx , 529.486_rkx , 1955.225_rkx , 370.057_rkx , 78.08_rkx , 291.39_rkx , &
    2066.0_rkx , 533.400_rkx , 1957.153_rkx , 371.140_rkx , 76.23_rkx , 287.77_rkx , &
    2067.0_rkx , 537.381_rkx , 1958.711_rkx , 372.227_rkx , 74.40_rkx , 284.18_rkx , &
    2068.0_rkx , 541.443_rkx , 1959.968_rkx , 373.318_rkx , 72.61_rkx , 280.62_rkx , &
    2069.0_rkx , 545.589_rkx , 1960.985_rkx , 374.414_rkx , 70.85_rkx , 277.09_rkx , &
    2070.0_rkx , 549.820_rkx , 1961.826_rkx , 375.515_rkx , 69.12_rkx , 273.59_rkx , &
    2071.0_rkx , 554.129_rkx , 1962.513_rkx , 376.619_rkx , 67.43_rkx , 270.13_rkx , &
    2072.0_rkx , 558.486_rkx , 1962.727_rkx , 377.725_rkx , 65.78_rkx , 266.70_rkx , &
    2073.0_rkx , 562.867_rkx , 1962.178_rkx , 378.828_rkx , 64.15_rkx , 263.30_rkx , &
    2074.0_rkx , 567.272_rkx , 1960.921_rkx , 379.927_rkx , 62.56_rkx , 259.94_rkx , &
    2075.0_rkx , 571.701_rkx , 1959.009_rkx , 381.024_rkx , 61.01_rkx , 256.61_rkx , &
    2076.0_rkx , 576.146_rkx , 1956.411_rkx , 382.118_rkx , 59.48_rkx , 253.30_rkx , &
    2077.0_rkx , 580.606_rkx , 1953.127_rkx , 383.209_rkx , 57.97_rkx , 250.02_rkx , &
    2078.0_rkx , 585.105_rkx , 1949.266_rkx , 384.297_rkx , 56.50_rkx , 246.76_rkx , &
    2079.0_rkx , 589.653_rkx , 1944.919_rkx , 385.383_rkx , 55.06_rkx , 243.54_rkx , &
    2080.0_rkx , 594.257_rkx , 1940.166_rkx , 386.465_rkx , 53.64_rkx , 240.34_rkx , &
    2081.0_rkx , 598.918_rkx , 1935.091_rkx , 387.545_rkx , 52.26_rkx , 237.18_rkx , &
    2082.0_rkx , 603.538_rkx , 1928.852_rkx , 388.618_rkx , 50.91_rkx , 234.06_rkx , &
    2083.0_rkx , 608.020_rkx , 1920.640_rkx , 389.682_rkx , 49.59_rkx , 230.97_rkx , &
    2084.0_rkx , 612.364_rkx , 1910.607_rkx , 390.736_rkx , 48.30_rkx , 227.91_rkx , &
    2085.0_rkx , 616.572_rkx , 1898.904_rkx , 391.780_rkx , 47.04_rkx , 224.88_rkx , &
    2086.0_rkx , 620.648_rkx , 1885.672_rkx , 392.815_rkx , 45.80_rkx , 221.89_rkx , &
    2087.0_rkx , 624.583_rkx , 1870.964_rkx , 393.840_rkx , 44.59_rkx , 218.92_rkx , &
    2088.0_rkx , 628.381_rkx , 1854.859_rkx , 394.856_rkx , 43.41_rkx , 215.99_rkx , &
    2089.0_rkx , 632.065_rkx , 1837.534_rkx , 395.862_rkx , 42.26_rkx , 213.07_rkx , &
    2090.0_rkx , 635.649_rkx , 1819.142_rkx , 396.859_rkx , 41.12_rkx , 210.19_rkx , &
    2091.0_rkx , 639.141_rkx , 1799.822_rkx , 397.847_rkx , 40.02_rkx , 207.35_rkx , &
    2092.0_rkx , 642.597_rkx , 1780.482_rkx , 398.825_rkx , 38.95_rkx , 204.54_rkx , &
    2093.0_rkx , 646.061_rkx , 1761.916_rkx , 399.792_rkx , 37.90_rkx , 201.76_rkx , &
    2094.0_rkx , 649.515_rkx , 1744.044_rkx , 400.749_rkx , 36.87_rkx , 199.02_rkx , &
    2095.0_rkx , 652.951_rkx , 1726.798_rkx , 401.694_rkx , 35.88_rkx , 196.31_rkx , &
    2096.0_rkx , 656.364_rkx , 1710.127_rkx , 402.629_rkx , 34.90_rkx , 193.63_rkx , &
    2097.0_rkx , 659.754_rkx , 1693.982_rkx , 403.554_rkx , 33.95_rkx , 190.99_rkx , &
    2098.0_rkx , 663.107_rkx , 1678.406_rkx , 404.468_rkx , 33.03_rkx , 188.37_rkx , &
    2099.0_rkx , 666.423_rkx , 1663.528_rkx , 405.372_rkx , 32.12_rkx , 185.78_rkx , &
    2100.0_rkx , 669.723_rkx , 1649.396_rkx , 406.265_rkx , 31.24_rkx , 183.22_rkx /), &
      (/6,251/))
      case( 'RCP8.5' , 'RCP85' )
        cgas(1:6,1850:2100) = reshape((/    &
    1850.0_rkx , 284.725_rkx , 790.979_rkx , 275.425_rkx , 0.00_rkx , 0.00_rkx , &
    1851.0_rkx , 284.875_rkx , 792.250_rkx , 275.500_rkx , 0.00_rkx , 0.00_rkx , &
    1852.0_rkx , 285.000_rkx , 793.750_rkx , 275.600_rkx , 0.00_rkx , 0.00_rkx , &
    1853.0_rkx , 285.125_rkx , 795.250_rkx , 275.700_rkx , 0.00_rkx , 0.00_rkx , &
    1854.0_rkx , 285.275_rkx , 796.750_rkx , 275.800_rkx , 0.00_rkx , 0.00_rkx , &
    1855.0_rkx , 285.425_rkx , 798.000_rkx , 275.900_rkx , 0.00_rkx , 0.00_rkx , &
    1856.0_rkx , 285.575_rkx , 799.250_rkx , 276.000_rkx , 0.00_rkx , 0.00_rkx , &
    1857.0_rkx , 285.725_rkx , 800.750_rkx , 276.100_rkx , 0.00_rkx , 0.00_rkx , &
    1858.0_rkx , 285.900_rkx , 802.250_rkx , 276.200_rkx , 0.00_rkx , 0.00_rkx , &
    1859.0_rkx , 286.075_rkx , 803.750_rkx , 276.300_rkx , 0.00_rkx , 0.00_rkx , &
    1860.0_rkx , 286.225_rkx , 805.250_rkx , 276.400_rkx , 0.00_rkx , 0.00_rkx , &
    1861.0_rkx , 286.375_rkx , 806.750_rkx , 276.500_rkx , 0.00_rkx , 0.00_rkx , &
    1862.0_rkx , 286.500_rkx , 808.250_rkx , 276.600_rkx , 0.00_rkx , 0.00_rkx , &
    1863.0_rkx , 286.625_rkx , 809.750_rkx , 276.700_rkx , 0.00_rkx , 0.00_rkx , &
    1864.0_rkx , 286.775_rkx , 811.250_rkx , 276.800_rkx , 0.00_rkx , 0.00_rkx , &
    1865.0_rkx , 286.900_rkx , 813.000_rkx , 276.900_rkx , 0.00_rkx , 0.00_rkx , &
    1866.0_rkx , 287.000_rkx , 814.750_rkx , 277.000_rkx , 0.00_rkx , 0.00_rkx , &
    1867.0_rkx , 287.100_rkx , 816.250_rkx , 277.100_rkx , 0.00_rkx , 0.00_rkx , &
    1868.0_rkx , 287.225_rkx , 817.750_rkx , 277.200_rkx , 0.00_rkx , 0.00_rkx , &
    1869.0_rkx , 287.375_rkx , 819.250_rkx , 277.300_rkx , 0.00_rkx , 0.00_rkx , &
    1870.0_rkx , 287.525_rkx , 821.000_rkx , 277.375_rkx , 0.00_rkx , 0.00_rkx , &
    1871.0_rkx , 287.700_rkx , 822.750_rkx , 277.425_rkx , 0.00_rkx , 0.00_rkx , &
    1872.0_rkx , 287.900_rkx , 824.250_rkx , 277.500_rkx , 0.00_rkx , 0.00_rkx , &
    1873.0_rkx , 288.125_rkx , 825.750_rkx , 277.600_rkx , 0.00_rkx , 0.00_rkx , &
    1874.0_rkx , 288.400_rkx , 827.250_rkx , 277.700_rkx , 0.00_rkx , 0.00_rkx , &
    1875.0_rkx , 288.700_rkx , 829.000_rkx , 277.775_rkx , 0.00_rkx , 0.00_rkx , &
    1876.0_rkx , 289.025_rkx , 830.750_rkx , 277.825_rkx , 0.00_rkx , 0.00_rkx , &
    1877.0_rkx , 289.400_rkx , 832.250_rkx , 277.900_rkx , 0.00_rkx , 0.00_rkx , &
    1878.0_rkx , 289.800_rkx , 833.750_rkx , 278.000_rkx , 0.00_rkx , 0.00_rkx , &
    1879.0_rkx , 290.225_rkx , 835.250_rkx , 278.100_rkx , 0.00_rkx , 0.00_rkx , &
    1880.0_rkx , 290.700_rkx , 837.000_rkx , 278.200_rkx , 0.00_rkx , 0.00_rkx , &
    1881.0_rkx , 291.200_rkx , 839.000_rkx , 278.300_rkx , 0.00_rkx , 0.00_rkx , &
    1882.0_rkx , 291.675_rkx , 841.000_rkx , 278.400_rkx , 0.00_rkx , 0.00_rkx , &
    1883.0_rkx , 292.125_rkx , 843.000_rkx , 278.500_rkx , 0.00_rkx , 0.00_rkx , &
    1884.0_rkx , 292.575_rkx , 844.750_rkx , 278.600_rkx , 0.00_rkx , 0.00_rkx , &
    1885.0_rkx , 292.975_rkx , 846.250_rkx , 278.700_rkx , 0.00_rkx , 0.00_rkx , &
    1886.0_rkx , 293.300_rkx , 848.000_rkx , 278.800_rkx , 0.00_rkx , 0.00_rkx , &
    1887.0_rkx , 293.575_rkx , 850.000_rkx , 278.875_rkx , 0.00_rkx , 0.00_rkx , &
    1888.0_rkx , 293.800_rkx , 852.000_rkx , 278.925_rkx , 0.00_rkx , 0.00_rkx , &
    1889.0_rkx , 294.000_rkx , 854.000_rkx , 279.000_rkx , 0.00_rkx , 0.00_rkx , &
    1890.0_rkx , 294.175_rkx , 856.000_rkx , 279.100_rkx , 0.00_rkx , 0.00_rkx , &
    1891.0_rkx , 294.325_rkx , 858.250_rkx , 279.200_rkx , 0.00_rkx , 0.00_rkx , &
    1892.0_rkx , 294.475_rkx , 860.750_rkx , 279.300_rkx , 0.00_rkx , 0.00_rkx , &
    1893.0_rkx , 294.600_rkx , 863.000_rkx , 279.375_rkx , 0.00_rkx , 0.00_rkx , &
    1894.0_rkx , 294.700_rkx , 865.000_rkx , 279.425_rkx , 0.00_rkx , 0.00_rkx , &
    1895.0_rkx , 294.800_rkx , 867.250_rkx , 279.475_rkx , 0.00_rkx , 0.00_rkx , &
    1896.0_rkx , 294.900_rkx , 869.750_rkx , 279.525_rkx , 0.00_rkx , 0.00_rkx , &
    1897.0_rkx , 295.025_rkx , 872.000_rkx , 279.600_rkx , 0.00_rkx , 0.00_rkx , &
    1898.0_rkx , 295.225_rkx , 874.250_rkx , 279.675_rkx , 0.00_rkx , 0.00_rkx , &
    1899.0_rkx , 295.500_rkx , 876.750_rkx , 279.725_rkx , 0.00_rkx , 0.00_rkx , &
    1900.0_rkx , 295.800_rkx , 879.500_rkx , 279.800_rkx , 0.00_rkx , 0.00_rkx , &
    1901.0_rkx , 296.125_rkx , 883.000_rkx , 279.900_rkx , 0.00_rkx , 0.00_rkx , &
    1902.0_rkx , 296.475_rkx , 887.000_rkx , 280.000_rkx , 0.00_rkx , 0.00_rkx , &
    1903.0_rkx , 296.825_rkx , 891.250_rkx , 280.100_rkx , 0.00_rkx , 0.00_rkx , &
    1904.0_rkx , 297.200_rkx , 895.750_rkx , 280.200_rkx , 0.00_rkx , 0.00_rkx , &
    1905.0_rkx , 297.625_rkx , 900.250_rkx , 280.300_rkx , 0.00_rkx , 0.00_rkx , &
    1906.0_rkx , 298.075_rkx , 904.750_rkx , 280.400_rkx , 0.00_rkx , 0.00_rkx , &
    1907.0_rkx , 298.500_rkx , 909.250_rkx , 280.525_rkx , 0.00_rkx , 0.00_rkx , &
    1908.0_rkx , 298.900_rkx , 914.000_rkx , 280.675_rkx , 0.00_rkx , 0.00_rkx , &
    1909.0_rkx , 299.300_rkx , 919.000_rkx , 280.825_rkx , 0.00_rkx , 0.00_rkx , &
    1910.0_rkx , 299.700_rkx , 923.750_rkx , 280.975_rkx , 0.00_rkx , 0.00_rkx , &
    1911.0_rkx , 300.075_rkx , 928.250_rkx , 281.125_rkx , 0.00_rkx , 0.00_rkx , &
    1912.0_rkx , 300.425_rkx , 933.000_rkx , 281.300_rkx , 0.00_rkx , 0.00_rkx , &
    1913.0_rkx , 300.775_rkx , 938.250_rkx , 281.475_rkx , 0.00_rkx , 0.00_rkx , &
    1914.0_rkx , 301.100_rkx , 943.750_rkx , 281.625_rkx , 0.00_rkx , 0.00_rkx , &
    1915.0_rkx , 301.400_rkx , 949.250_rkx , 281.800_rkx , 0.00_rkx , 0.00_rkx , &
    1916.0_rkx , 301.725_rkx , 954.750_rkx , 282.000_rkx , 0.00_rkx , 0.00_rkx , &
    1917.0_rkx , 302.075_rkx , 960.250_rkx , 282.225_rkx , 0.00_rkx , 0.00_rkx , &
    1918.0_rkx , 302.400_rkx , 966.000_rkx , 282.475_rkx , 0.00_rkx , 0.00_rkx , &
    1919.0_rkx , 302.700_rkx , 972.000_rkx , 282.700_rkx , 0.00_rkx , 0.00_rkx , &
    1920.0_rkx , 303.025_rkx , 977.750_rkx , 282.925_rkx , 0.00_rkx , 0.00_rkx , &
    1921.0_rkx , 303.400_rkx , 983.250_rkx , 283.175_rkx , 0.00_rkx , 0.00_rkx , &
    1922.0_rkx , 303.775_rkx , 989.000_rkx , 283.400_rkx , 0.00_rkx , 0.00_rkx , &
    1923.0_rkx , 304.125_rkx , 995.000_rkx , 283.600_rkx , 0.00_rkx , 0.00_rkx , &
    1924.0_rkx , 304.525_rkx , 1001.000_rkx , 283.800_rkx , 0.00_rkx , 0.00_rkx , &
    1925.0_rkx , 304.975_rkx , 1007.000_rkx , 284.000_rkx , 0.00_rkx , 0.00_rkx , &
    1926.0_rkx , 305.400_rkx , 1013.000_rkx , 284.225_rkx , 0.00_rkx , 0.00_rkx , &
    1927.0_rkx , 305.825_rkx , 1019.000_rkx , 284.450_rkx , 0.00_rkx , 0.00_rkx , &
    1928.0_rkx , 306.300_rkx , 1025.000_rkx , 284.625_rkx , 0.00_rkx , 0.00_rkx , &
    1929.0_rkx , 306.775_rkx , 1030.750_rkx , 284.800_rkx , 0.00_rkx , 0.00_rkx , &
    1930.0_rkx , 307.225_rkx , 1036.250_rkx , 284.975_rkx , 0.00_rkx , 0.00_rkx , &
    1931.0_rkx , 307.700_rkx , 1041.750_rkx , 285.125_rkx , 0.00_rkx , 0.00_rkx , &
    1932.0_rkx , 308.175_rkx , 1047.250_rkx , 285.300_rkx , 0.00_rkx , 0.00_rkx , &
    1933.0_rkx , 308.600_rkx , 1052.750_rkx , 285.500_rkx , 0.00_rkx , 0.01_rkx , &
    1934.0_rkx , 309.000_rkx , 1058.000_rkx , 285.700_rkx , 0.00_rkx , 0.01_rkx , &
    1935.0_rkx , 309.400_rkx , 1063.000_rkx , 285.900_rkx , 0.00_rkx , 0.02_rkx , &
    1936.0_rkx , 309.750_rkx , 1068.000_rkx , 286.100_rkx , 0.00_rkx , 0.03_rkx , &
    1937.0_rkx , 310.000_rkx , 1073.000_rkx , 286.275_rkx , 0.00_rkx , 0.05_rkx , &
    1938.0_rkx , 310.175_rkx , 1078.000_rkx , 286.425_rkx , 0.00_rkx , 0.08_rkx , &
    1939.0_rkx , 310.300_rkx , 1083.000_rkx , 286.575_rkx , 0.00_rkx , 0.13_rkx , &
    1940.0_rkx , 310.375_rkx , 1088.250_rkx , 286.725_rkx , 0.01_rkx , 0.20_rkx , &
    1941.0_rkx , 310.375_rkx , 1093.750_rkx , 286.900_rkx , 0.02_rkx , 0.29_rkx , &
    1942.0_rkx , 310.300_rkx , 1099.000_rkx , 287.100_rkx , 0.02_rkx , 0.42_rkx , &
    1943.0_rkx , 310.200_rkx , 1104.000_rkx , 287.300_rkx , 0.03_rkx , 0.58_rkx , &
    1944.0_rkx , 310.125_rkx , 1109.250_rkx , 287.525_rkx , 0.04_rkx , 0.77_rkx , &
    1945.0_rkx , 310.100_rkx , 1114.750_rkx , 287.775_rkx , 0.05_rkx , 1.02_rkx , &
    1946.0_rkx , 310.125_rkx , 1120.500_rkx , 288.025_rkx , 0.07_rkx , 1.35_rkx , &
    1947.0_rkx , 310.200_rkx , 1126.750_rkx , 288.275_rkx , 0.10_rkx , 1.87_rkx , &
    1948.0_rkx , 310.325_rkx , 1133.250_rkx , 288.525_rkx , 0.16_rkx , 2.70_rkx , &
    1949.0_rkx , 310.500_rkx , 1140.000_rkx , 288.775_rkx , 0.29_rkx , 3.80_rkx , &
    1950.0_rkx , 310.750_rkx , 1147.250_rkx , 289.000_rkx , 0.50_rkx , 5.00_rkx , &
    1951.0_rkx , 311.100_rkx , 1155.000_rkx , 289.200_rkx , 0.81_rkx , 6.31_rkx , &
    1952.0_rkx , 311.500_rkx , 1163.500_rkx , 289.425_rkx , 1.17_rkx , 7.81_rkx , &
    1953.0_rkx , 311.925_rkx , 1172.750_rkx , 289.675_rkx , 1.58_rkx , 9.46_rkx , &
    1954.0_rkx , 312.425_rkx , 1182.250_rkx , 289.900_rkx , 2.15_rkx , 11.24_rkx , &
    1955.0_rkx , 313.000_rkx , 1192.000_rkx , 290.125_rkx , 2.89_rkx , 13.24_rkx , &
    1956.0_rkx , 313.600_rkx , 1202.250_rkx , 290.375_rkx , 3.79_rkx , 15.48_rkx , &
    1957.0_rkx , 314.225_rkx , 1213.000_rkx , 290.600_rkx , 4.92_rkx , 18.05_rkx , &
    1958.0_rkx , 314.848_rkx , 1224.000_rkx , 290.825_rkx , 6.24_rkx , 21.01_rkx , &
    1959.0_rkx , 315.500_rkx , 1235.250_rkx , 291.100_rkx , 7.56_rkx , 24.21_rkx , &
    1960.0_rkx , 316.272_rkx , 1247.000_rkx , 291.400_rkx , 8.82_rkx , 27.68_rkx , &
    1961.0_rkx , 317.075_rkx , 1259.250_rkx , 291.675_rkx , 10.30_rkx , 31.68_rkx , &
    1962.0_rkx , 317.795_rkx , 1272.000_rkx , 291.925_rkx , 12.23_rkx , 36.31_rkx , &
    1963.0_rkx , 318.397_rkx , 1285.000_rkx , 292.225_rkx , 14.71_rkx , 41.59_rkx , &
    1964.0_rkx , 318.925_rkx , 1298.250_rkx , 292.600_rkx , 17.80_rkx , 47.73_rkx , &
    1965.0_rkx , 319.647_rkx , 1312.000_rkx , 293.000_rkx , 21.50_rkx , 54.93_rkx , &
    1966.0_rkx , 320.647_rkx , 1326.000_rkx , 293.400_rkx , 25.77_rkx , 63.16_rkx , &
    1967.0_rkx , 321.605_rkx , 1340.250_rkx , 293.800_rkx , 30.57_rkx , 72.39_rkx , &
    1968.0_rkx , 322.635_rkx , 1355.000_rkx , 294.225_rkx , 35.95_rkx , 82.77_rkx , &
    1969.0_rkx , 323.902_rkx , 1370.250_rkx , 294.700_rkx , 42.05_rkx , 94.44_rkx , &
    1970.0_rkx , 324.985_rkx , 1385.750_rkx , 295.200_rkx , 49.05_rkx , 107.45_rkx , &
    1971.0_rkx , 325.855_rkx , 1401.250_rkx , 295.700_rkx , 57.06_rkx , 121.80_rkx , &
    1972.0_rkx , 327.140_rkx , 1417.000_rkx , 296.225_rkx , 65.97_rkx , 137.30_rkx , &
    1973.0_rkx , 328.678_rkx , 1433.000_rkx , 296.800_rkx , 75.83_rkx , 154.05_rkx , &
    1974.0_rkx , 329.743_rkx , 1449.000_rkx , 297.400_rkx , 87.04_rkx , 172.45_rkx , &
    1975.0_rkx , 330.585_rkx , 1465.000_rkx , 298.000_rkx , 99.56_rkx , 192.60_rkx , &
    1976.0_rkx , 331.748_rkx , 1481.000_rkx , 298.650_rkx , 112.30_rkx , 213.15_rkx , &
    1977.0_rkx , 333.272_rkx , 1497.250_rkx , 299.360_rkx , 124.70_rkx , 232.90_rkx , &
    1978.0_rkx , 334.848_rkx , 1513.750_rkx , 300.082_rkx , 136.70_rkx , 251.90_rkx , &
    1979.0_rkx , 336.525_rkx , 1530.250_rkx , 300.740_rkx , 148.80_rkx , 270.50_rkx , &
    1980.0_rkx , 338.360_rkx , 1547.750_rkx , 301.382_rkx , 159.40_rkx , 287.35_rkx , &
    1981.0_rkx , 339.728_rkx , 1566.750_rkx , 302.220_rkx , 168.55_rkx , 304.25_rkx , &
    1982.0_rkx , 340.793_rkx , 1586.250_rkx , 303.098_rkx , 177.85_rkx , 321.70_rkx , &
    1983.0_rkx , 342.197_rkx , 1605.845_rkx , 303.805_rkx , 185.80_rkx , 338.15_rkx , &
    1984.0_rkx , 343.783_rkx , 1623.705_rkx , 304.397_rkx , 193.75_rkx , 355.00_rkx , &
    1985.0_rkx , 345.283_rkx , 1637.993_rkx , 305.087_rkx , 202.85_rkx , 372.50_rkx , &
    1986.0_rkx , 346.798_rkx , 1650.435_rkx , 305.772_rkx , 212.10_rkx , 389.80_rkx , &
    1987.0_rkx , 348.645_rkx , 1662.200_rkx , 306.262_rkx , 222.80_rkx , 408.65_rkx , &
    1988.0_rkx , 350.738_rkx , 1673.070_rkx , 307.195_rkx , 234.90_rkx , 428.45_rkx , &
    1989.0_rkx , 352.488_rkx , 1683.470_rkx , 308.507_rkx , 244.90_rkx , 448.25_rkx , &
    1990.0_rkx , 353.855_rkx , 1693.630_rkx , 309.485_rkx , 252.75_rkx , 466.15_rkx , &
    1991.0_rkx , 355.017_rkx , 1703.818_rkx , 310.113_rkx , 260.15_rkx , 481.30_rkx , &
    1992.0_rkx , 355.885_rkx , 1711.800_rkx , 310.603_rkx , 264.95_rkx , 494.40_rkx , &
    1993.0_rkx , 356.777_rkx , 1716.290_rkx , 311.077_rkx , 267.75_rkx , 504.80_rkx , &
    1994.0_rkx , 358.127_rkx , 1721.013_rkx , 311.603_rkx , 269.50_rkx , 513.10_rkx , &
    1995.0_rkx , 359.837_rkx , 1726.350_rkx , 312.295_rkx , 269.50_rkx , 519.55_rkx , &
    1996.0_rkx , 361.462_rkx , 1730.010_rkx , 313.048_rkx , 268.95_rkx , 525.05_rkx , &
    1997.0_rkx , 363.155_rkx , 1734.818_rkx , 313.760_rkx , 268.20_rkx , 529.15_rkx , &
    1998.0_rkx , 365.322_rkx , 1742.710_rkx , 314.533_rkx , 266.95_rkx , 532.40_rkx , &
    1999.0_rkx , 367.348_rkx , 1749.243_rkx , 315.257_rkx , 265.20_rkx , 534.95_rkx , &
    2000.0_rkx , 368.865_rkx , 1751.023_rkx , 315.850_rkx , 263.45_rkx , 537.05_rkx , &
    2001.0_rkx , 370.467_rkx , 1750.707_rkx , 316.488_rkx , 261.90_rkx , 538.85_rkx , &
    2002.0_rkx , 372.522_rkx , 1752.188_rkx , 317.245_rkx , 260.10_rkx , 539.95_rkx , &
    2003.0_rkx , 374.760_rkx , 1754.760_rkx , 318.017_rkx , 257.95_rkx , 540.05_rkx , &
    2004.0_rkx , 376.812_rkx , 1755.065_rkx , 318.710_rkx , 256.05_rkx , 539.90_rkx , &
    2005.0_rkx , 378.812_rkx , 1753.735_rkx , 319.440_rkx , 254.10_rkx , 539.40_rkx , &
    2006.0_rkx , 380.827_rkx , 1754.512_rkx , 320.200_rkx , 251.30_rkx , 537.50_rkx , &
    2007.0_rkx , 382.777_rkx , 1758.046_rkx , 320.895_rkx , 247.95_rkx , 534.70_rkx , &
    2008.0_rkx , 384.800_rkx , 1763.419_rkx , 321.604_rkx , 244.60_rkx , 531.80_rkx , &
    2009.0_rkx , 387.012_rkx , 1770.443_rkx , 322.326_rkx , 241.03_rkx , 528.53_rkx , &
    2010.0_rkx , 389.324_rkx , 1778.675_rkx , 323.061_rkx , 237.27_rkx , 524.93_rkx , &
    2011.0_rkx , 391.638_rkx , 1788.027_rkx , 323.809_rkx , 233.52_rkx , 521.19_rkx , &
    2012.0_rkx , 394.009_rkx , 1798.614_rkx , 324.575_rkx , 229.77_rkx , 517.30_rkx , &
    2013.0_rkx , 396.464_rkx , 1810.517_rkx , 325.364_rkx , 226.03_rkx , 513.29_rkx , &
    2014.0_rkx , 399.004_rkx , 1823.655_rkx , 326.176_rkx , 222.31_rkx , 509.17_rkx , &
    2015.0_rkx , 401.628_rkx , 1837.966_rkx , 327.010_rkx , 218.61_rkx , 504.97_rkx , &
    2016.0_rkx , 404.328_rkx , 1853.354_rkx , 327.867_rkx , 214.93_rkx , 500.69_rkx , &
    2017.0_rkx , 407.096_rkx , 1869.709_rkx , 328.746_rkx , 211.28_rkx , 496.35_rkx , &
    2018.0_rkx , 409.927_rkx , 1886.932_rkx , 329.647_rkx , 207.66_rkx , 491.94_rkx , &
    2019.0_rkx , 412.822_rkx , 1904.944_rkx , 330.570_rkx , 204.06_rkx , 487.48_rkx , &
    2020.0_rkx , 415.780_rkx , 1923.671_rkx , 331.514_rkx , 200.48_rkx , 482.98_rkx , &
    2021.0_rkx , 418.796_rkx , 1942.967_rkx , 332.481_rkx , 196.92_rkx , 478.42_rkx , &
    2022.0_rkx , 421.864_rkx , 1962.652_rkx , 333.466_rkx , 193.37_rkx , 473.82_rkx , &
    2023.0_rkx , 424.995_rkx , 1982.667_rkx , 334.469_rkx , 189.84_rkx , 469.17_rkx , &
    2024.0_rkx , 428.197_rkx , 2003.023_rkx , 335.489_rkx , 186.33_rkx , 464.51_rkx , &
    2025.0_rkx , 431.475_rkx , 2023.727_rkx , 336.526_rkx , 182.86_rkx , 459.82_rkx , &
    2026.0_rkx , 434.826_rkx , 2044.796_rkx , 337.580_rkx , 179.41_rkx , 455.13_rkx , &
    2027.0_rkx , 438.245_rkx , 2066.206_rkx , 338.650_rkx , 176.01_rkx , 450.43_rkx , &
    2028.0_rkx , 441.721_rkx , 2087.904_rkx , 339.737_rkx , 172.64_rkx , 445.74_rkx , &
    2029.0_rkx , 445.251_rkx , 2109.849_rkx , 340.841_rkx , 169.31_rkx , 441.04_rkx , &
    2030.0_rkx , 448.835_rkx , 2132.014_rkx , 341.960_rkx , 166.01_rkx , 436.34_rkx , &
    2031.0_rkx , 452.474_rkx , 2154.367_rkx , 343.096_rkx , 162.75_rkx , 431.65_rkx , &
    2032.0_rkx , 456.177_rkx , 2177.392_rkx , 344.247_rkx , 159.51_rkx , 426.96_rkx , &
    2033.0_rkx , 459.964_rkx , 2201.564_rkx , 345.415_rkx , 156.29_rkx , 422.26_rkx , &
    2034.0_rkx , 463.852_rkx , 2226.848_rkx , 346.599_rkx , 153.11_rkx , 417.57_rkx , &
    2035.0_rkx , 467.850_rkx , 2253.198_rkx , 347.800_rkx , 149.96_rkx , 412.88_rkx , &
    2036.0_rkx , 471.960_rkx , 2280.573_rkx , 349.015_rkx , 146.84_rkx , 408.21_rkx , &
    2037.0_rkx , 476.182_rkx , 2308.945_rkx , 350.247_rkx , 143.77_rkx , 403.55_rkx , &
    2038.0_rkx , 480.508_rkx , 2338.244_rkx , 351.494_rkx , 140.74_rkx , 398.92_rkx , &
    2039.0_rkx , 484.927_rkx , 2368.371_rkx , 352.757_rkx , 137.75_rkx , 394.32_rkx , &
    2040.0_rkx , 489.435_rkx , 2399.244_rkx , 354.035_rkx , 134.79_rkx , 389.73_rkx , &
    2041.0_rkx , 494.032_rkx , 2430.799_rkx , 355.328_rkx , 131.88_rkx , 385.16_rkx , &
    2042.0_rkx , 498.730_rkx , 2463.046_rkx , 356.631_rkx , 129.01_rkx , 380.62_rkx , &
    2043.0_rkx , 503.530_rkx , 2495.894_rkx , 357.939_rkx , 126.16_rkx , 376.09_rkx , &
    2044.0_rkx , 508.433_rkx , 2529.221_rkx , 359.251_rkx , 123.35_rkx , 371.57_rkx , &
    2045.0_rkx , 513.456_rkx , 2563.051_rkx , 360.568_rkx , 120.58_rkx , 367.07_rkx , &
    2046.0_rkx , 518.611_rkx , 2597.397_rkx , 361.890_rkx , 117.84_rkx , 362.60_rkx , &
    2047.0_rkx , 523.900_rkx , 2632.267_rkx , 363.216_rkx , 115.14_rkx , 358.15_rkx , &
    2048.0_rkx , 529.324_rkx , 2667.683_rkx , 364.546_rkx , 112.49_rkx , 353.73_rkx , &
    2049.0_rkx , 534.875_rkx , 2703.613_rkx , 365.881_rkx , 109.88_rkx , 349.34_rkx , &
    2050.0_rkx , 540.543_rkx , 2739.985_rkx , 367.220_rkx , 107.31_rkx , 344.98_rkx , &
    2051.0_rkx , 546.322_rkx , 2776.745_rkx , 368.563_rkx , 104.78_rkx , 340.65_rkx , &
    2052.0_rkx , 552.212_rkx , 2813.242_rkx , 369.909_rkx , 102.29_rkx , 336.35_rkx , &
    2053.0_rkx , 558.212_rkx , 2848.870_rkx , 371.256_rkx , 99.85_rkx , 332.09_rkx , &
    2054.0_rkx , 564.313_rkx , 2883.567_rkx , 372.604_rkx , 97.44_rkx , 327.84_rkx , &
    2055.0_rkx , 570.517_rkx , 2917.314_rkx , 373.953_rkx , 95.06_rkx , 323.61_rkx , &
    2056.0_rkx , 576.843_rkx , 2950.251_rkx , 375.304_rkx , 92.73_rkx , 319.41_rkx , &
    2057.0_rkx , 583.305_rkx , 2982.498_rkx , 376.655_rkx , 90.43_rkx , 315.24_rkx , &
    2058.0_rkx , 589.905_rkx , 3014.164_rkx , 378.008_rkx , 88.17_rkx , 311.10_rkx , &
    2059.0_rkx , 596.647_rkx , 3045.361_rkx , 379.361_rkx , 85.95_rkx , 306.99_rkx , &
    2060.0_rkx , 603.520_rkx , 3076.135_rkx , 380.716_rkx , 83.78_rkx , 302.92_rkx , &
    2061.0_rkx , 610.516_rkx , 3106.476_rkx , 382.072_rkx , 81.65_rkx , 298.89_rkx , &
    2062.0_rkx , 617.605_rkx , 3135.825_rkx , 383.427_rkx , 79.55_rkx , 294.89_rkx , &
    2063.0_rkx , 624.764_rkx , 3163.675_rkx , 384.782_rkx , 77.50_rkx , 290.92_rkx , &
    2064.0_rkx , 631.995_rkx , 3190.129_rkx , 386.135_rkx , 75.49_rkx , 286.99_rkx , &
    2065.0_rkx , 639.291_rkx , 3215.149_rkx , 387.487_rkx , 73.52_rkx , 283.08_rkx , &
    2066.0_rkx , 646.653_rkx , 3238.745_rkx , 388.837_rkx , 71.57_rkx , 279.19_rkx , &
    2067.0_rkx , 654.098_rkx , 3261.104_rkx , 390.187_rkx , 69.67_rkx , 275.34_rkx , &
    2068.0_rkx , 661.645_rkx , 3282.390_rkx , 391.535_rkx , 67.80_rkx , 271.51_rkx , &
    2069.0_rkx , 669.305_rkx , 3302.753_rkx , 392.882_rkx , 65.97_rkx , 267.72_rkx , &
    2070.0_rkx , 677.078_rkx , 3322.341_rkx , 394.227_rkx , 64.18_rkx , 263.97_rkx , &
    2071.0_rkx , 684.954_rkx , 3341.227_rkx , 395.572_rkx , 62.43_rkx , 260.26_rkx , &
    2072.0_rkx , 692.902_rkx , 3359.541_rkx , 396.916_rkx , 60.72_rkx , 256.58_rkx , &
    2073.0_rkx , 700.894_rkx , 3377.413_rkx , 398.261_rkx , 59.04_rkx , 252.94_rkx , &
    2074.0_rkx , 708.932_rkx , 3394.863_rkx , 399.607_rkx , 57.40_rkx , 249.33_rkx , &
    2075.0_rkx , 717.015_rkx , 3411.910_rkx , 400.954_rkx , 55.80_rkx , 245.76_rkx , &
    2076.0_rkx , 725.136_rkx , 3428.434_rkx , 402.302_rkx , 54.23_rkx , 242.22_rkx , &
    2077.0_rkx , 733.307_rkx , 3444.364_rkx , 403.651_rkx , 52.70_rkx , 238.71_rkx , &
    2078.0_rkx , 741.524_rkx , 3459.834_rkx , 405.000_rkx , 51.19_rkx , 235.22_rkx , &
    2079.0_rkx , 749.805_rkx , 3474.959_rkx , 406.351_rkx , 49.72_rkx , 231.77_rkx , &
    2080.0_rkx , 758.182_rkx , 3489.839_rkx , 407.702_rkx , 48.29_rkx , 228.35_rkx , &
    2081.0_rkx , 766.645_rkx , 3504.576_rkx , 409.054_rkx , 46.89_rkx , 224.97_rkx , &
    2082.0_rkx , 775.174_rkx , 3519.347_rkx , 410.408_rkx , 45.52_rkx , 221.63_rkx , &
    2083.0_rkx , 783.751_rkx , 3534.256_rkx , 411.766_rkx , 44.19_rkx , 218.33_rkx , &
    2084.0_rkx , 792.366_rkx , 3549.260_rkx , 413.126_rkx , 42.89_rkx , 215.06_rkx , &
    2085.0_rkx , 801.019_rkx , 3564.335_rkx , 414.490_rkx , 41.62_rkx , 211.83_rkx , &
    2086.0_rkx , 809.715_rkx , 3579.462_rkx , 415.857_rkx , 40.38_rkx , 208.63_rkx , &
    2087.0_rkx , 818.422_rkx , 3594.475_rkx , 417.228_rkx , 39.17_rkx , 205.46_rkx , &
    2088.0_rkx , 827.157_rkx , 3609.264_rkx , 418.601_rkx , 37.99_rkx , 202.32_rkx , &
    2089.0_rkx , 835.956_rkx , 3623.938_rkx , 419.978_rkx , 36.84_rkx , 199.21_rkx , &
    2090.0_rkx , 844.805_rkx , 3638.592_rkx , 421.357_rkx , 35.71_rkx , 196.13_rkx , &
    2091.0_rkx , 853.725_rkx , 3653.310_rkx , 422.740_rkx , 34.61_rkx , 193.09_rkx , &
    2092.0_rkx , 862.726_rkx , 3667.615_rkx , 424.123_rkx , 33.55_rkx , 190.08_rkx , &
    2093.0_rkx , 871.777_rkx , 3680.986_rkx , 425.504_rkx , 32.51_rkx , 187.11_rkx , &
    2094.0_rkx , 880.864_rkx , 3693.450_rkx , 426.883_rkx , 31.50_rkx , 184.18_rkx , &
    2095.0_rkx , 889.982_rkx , 3705.044_rkx , 428.259_rkx , 30.52_rkx , 181.28_rkx , &
    2096.0_rkx , 899.124_rkx , 3715.824_rkx , 429.633_rkx , 29.56_rkx , 178.42_rkx , &
    2097.0_rkx , 908.289_rkx , 3725.841_rkx , 431.005_rkx , 28.63_rkx , 175.59_rkx , &
    2098.0_rkx , 917.471_rkx , 3734.989_rkx , 432.374_rkx , 27.73_rkx , 172.79_rkx , &
    2099.0_rkx , 926.665_rkx , 3743.213_rkx , 433.741_rkx , 26.84_rkx , 170.02_rkx , &
    2100.0_rkx , 935.874_rkx , 3750.685_rkx , 435.106_rkx , 25.98_rkx , 167.28_rkx /), &
      (/6,251/))
      case( 'CONST' )
        ctemp = cgas(:,ghg_year_const)
        do jj = 1850 , 2100
          cgas(1,jj) = jj
          cgas(2,jj) = ctemp(2)
          cgas(3,jj) = ctemp(3)
          cgas(4,jj) = ctemp(4)
          cgas(5,jj) = ctemp(5)
          cgas(6,jj) = ctemp(6)
        end do
      case default
        write (stderr,*) 'Unsupported emission scenario: ', csc
        write (stderr,*) 'Use one in SRES/RCP ', scenarios
        call fatal(__FILE__,__LINE__, &
                   'UNSUPPORTED EMISSION SCENARIO')
    end select
    if ( itweak == 1 ) then
      if ( itweak_greenhouse_gases == 1 ) then
        do ii = 1 , n_greenhouse_gases
          do jj = 1850 , 2100
            cgas(ii+1,jj) = cgas(ii+1,jj) * gas_tweak_factors(ii)
          end do
        end do
      end if
    end if
  end subroutine set_scenario
!
end module mod_ipcc_scenario

! vim: tabstop=8 expandtab shiftwidth=2 softtabstop=2
